#include "includes.h"
#include "TalComponent.h"


TalComponent::TalComponent (TalCore* const ownerFilter) : AudioProcessorEditor (ownerFilter)
{
    TalCore* const filter = getFilter();

    const Image image = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);

    int width = 620;
    int height = 220;
    envelopeEditorView = new EnvelopeEditorView(filter, width, height);
	addAndMakeVisible(envelopeEditorView);

    envelopeEditorView->setBackgroundColor(Colour((const uint8)10, (const uint8)10, (const uint8)10, (const uint8)120));
    envelopeEditorView->setSplineColor(Colour((const uint8)200, (const uint8)200, (const uint8)200, (const uint8)200));

    envelopeEditorView->setBounds(10, 10, width, height);

	speedFactorComboBox = addComboBox(10, height + 20, 80, ownerFilter, SPEEDFACTOR);
	speedFactorComboBox->addItem(T("x1"),1);
	speedFactorComboBox->addItem(T("x2"),2);
	speedFactorComboBox->addItem(T("x4"),3);
	speedFactorComboBox->addItem(T("x8"),4);
	speedFactorComboBox->addItem(T("x16"),5);
	speedFactorComboBox->addItem(T("x32"),6);
	speedFactorComboBox->addItem(T("-"),7);

    filtertypeComboBox = addComboBox(110, height + 20, 80, ownerFilter, FILTERTYPE);
	filtertypeComboBox->addItem(T("LP 24dB"),1);
	filtertypeComboBox->addItem(T("LP 18dB"),2);
	filtertypeComboBox->addItem(T("LP 12dB"),3);
	filtertypeComboBox->addItem(T("LP 6dB"),4);
	filtertypeComboBox->addItem(T("HP 12dB"),5);
	filtertypeComboBox->addItem(T("BP 12dB"),6);
	filtertypeComboBox->addItem(T("Notch"),7);
	filtertypeComboBox->addItem(T("Volume"),8);
	filtertypeComboBox->addItem(T("Pan"),9);
	filtertypeComboBox->addItem(T("-"),10);

	const Image knobImage = ImageCache::getFromMemory(bmp00135_png, bmp00135_pngSize);
	int numberOfImages = knobImage.getHeight() / knobImage.getWidth();
    resonanceKnob = addNormalKnob(324, height + 39, ownerFilter, knobImage, numberOfImages, RESONANCE);
    depthKnob = addNormalKnob(390, height + 39, ownerFilter, knobImage, numberOfImages, DEPTH);
    volumeInKnob = addNormalKnob(514, height + 39, ownerFilter, knobImage, numberOfImages, VOLUMEIN);
    volumeOutKnob = addNormalKnob(576, height + 39, ownerFilter, knobImage, numberOfImages, VOLUMEOUT);

	// Version info
	versionLabel = new Label("Version Info", "V 1.20");
	versionLabel->setBounds(image.getWidth() - 68, image.getHeight() - 88, 100, 20);
	versionLabel->setColour(Label::textColourId, Colour((juce::uint8)100, (juce::uint8)100, (juce::uint8)100, 0.9f));
    addAndMakeVisible(versionLabel);

    updateParametersFromFilter();

    setSize (image.getWidth(), image.getHeight());
    ownerFilter->addChangeListener (this);
}

TalComponent::~TalComponent()
{
    getFilter()->removeChangeListener (this);
    deleteAllChildren();
}

ComboBox* TalComponent::addComboBox(int x, int y, int width, TalCore* const ownerFilter, int parameter)
{
	ComboBox *comboBox;
	addAndMakeVisible(comboBox = new ComboBox(T("ComboBox")));
	comboBox->setBounds(x, y, width, 20);
	comboBox->setColour(ComboBox::backgroundColourId, Colour((juce::uint8)40, (juce::uint8)40, (juce::uint8)40, 120.0f));
	comboBox->setColour(ComboBox::textColourId, Colour::greyLevel(1.0f));
	comboBox->setColour(ComboBox::buttonColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
	comboBox->setColour(ComboBox::arrowColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
        comboBox->addListener (this);
	return comboBox;
}

FilmStripKnob* TalComponent::addNormalKnob(int x, int y, TalCore* const ownerFilter, const Image knobImage, int numOfFrames, int parameter)
{
	FilmStripKnob *filmStripKnob;
	addAndMakeVisible(filmStripKnob = new FilmStripKnob(knobImage,
									 numOfFrames,
									 false,
                                     parameter));
    filmStripKnob->setBounds(x, y, knobImage.getWidth(), knobImage.getHeight() / numOfFrames);
	filmStripKnob->addListener (this);
	return filmStripKnob;
}

void TalComponent::sliderDragStarted (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getFilter();
        filter->beginParameterChangeGesture(values["index"]);
    }
}

void TalComponent::sliderDragEnded (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getFilter();
        filter->endParameterChangeGesture(values["index"]);
    }
}

void TalComponent::paint (Graphics& g)
{
    g.fillAll (Colour::greyLevel (0.9f));

    const Image image = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);
    g.drawImage (image,
                 0, 0, image.getWidth(), image.getHeight(),
                 0, 0, image.getWidth(), image.getHeight());
}

void TalComponent::changeListenerCallback (ChangeBroadcaster* source)
{
    updateParametersFromFilter();
}

void TalComponent::sliderValueChanged (Slider* caller)
{
    TalCore* const filter = getFilter();
	if (caller == resonanceKnob) filter->setParameterNotifyingHost(RESONANCE, (float)resonanceKnob->getValue());
	if (caller == volumeInKnob) filter->setParameterNotifyingHost(VOLUMEIN, (float)volumeInKnob->getValue());
	if (caller == volumeOutKnob) filter->setParameterNotifyingHost(VOLUMEOUT, (float)volumeOutKnob->getValue());
	if (caller == depthKnob) filter->setParameterNotifyingHost(DEPTH, (float)depthKnob->getValue());
}

void TalComponent::comboBoxChanged (ComboBox* caller)
{
    TalCore* const filter = getFilter();
	if (caller == speedFactorComboBox) filter->setParameterNotifyingHost(SPEEDFACTOR, float(speedFactorComboBox->getSelectedId()-1)/6.0f);
	if (caller == filtertypeComboBox) filter->setParameterNotifyingHost(FILTERTYPE, float(filtertypeComboBox->getSelectedId()-1)/9.0f);
}

//==============================================================================
void TalComponent::updateParametersFromFilter()
{
    TalCore* const filter = getFilter();

    filter->getCallbackLock().enter();
	float speedFactor = filter->getParameter(SPEEDFACTOR)*6.0f+1.0f;
	float filterType = filter->getParameter(FILTERTYPE)*9.0f+1.0f;
	float resonance = filter->getParameter(RESONANCE);
	float volumeIn = filter->getParameter(VOLUMEIN);
	float volumeOut = filter->getParameter(VOLUMEOUT);
	float depth = filter->getParameter(DEPTH);
    filter->getCallbackLock().exit();

    speedFactorComboBox->setSelectedId((int)speedFactor, dontSendNotification);
    filtertypeComboBox->setSelectedId((int)filterType, dontSendNotification);

    resonanceKnob->setValue(resonance, dontSendNotification);
    volumeInKnob->setValue(volumeIn, dontSendNotification);
    volumeOutKnob->setValue(volumeOut, dontSendNotification);
    depthKnob->setValue(depth, dontSendNotification);
}

//==============================================================================
// Binary resources - be careful not to edit any of these sections!

// JUCER_RESOURCE: bmp00128_png, 22611, "E:/Code/tal/tal-filter-2-juce/resources/bmp00128.png"
static const unsigned char resource_TalComponent_bmp00128_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,2,128,0,0,1,64,8,6,0,0,0,44,205,205,10,0,0,0,9,112,72,89,115,0,0,12,78,0,0,12,
78,1,127,119,140,35,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,77,126,73,68,65,84,120,218,236,221,121,124,20,245,253,63,240,215,204,238,6,216,13,144,64,54,92,74,2,138,146,13,158,36,214,170,28,
133,86,4,212,98,129,111,15,229,240,215,90,57,170,246,208,182,90,175,94,86,123,104,173,138,32,10,130,226,5,40,90,37,160,128,16,20,42,57,56,148,36,114,72,54,144,3,54,1,18,146,77,72,178,59,191,63,102,103,
118,102,118,246,202,1,98,94,207,62,210,96,142,205,204,231,243,153,253,188,230,61,151,48,117,202,20,9,68,68,68,68,212,101,136,108,2,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,
0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,6,64,34,34,34,34,98,
0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,162,115,138,85,146,36,182,2,17,17,17,81,23,194,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,
17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,55,145,85,
146,36,182,2,17,17,17,81,23,194,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,
32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,69,102,149,36,137,173,64,68,68,68,212,133,176,2,72,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,
68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,29,129,143,130,35,34,34,34,234,98,88,1,36,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,
6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,234,36,124,18,8,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,
0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,157,57,188,17,52,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,
32,17,17,17,17,49,0,18,17,17,17,209,89,197,251,0,18,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,157,
85,188,15,32,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,89,197,251,0,18,17,17,17,117,49,172,0,
18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,157,3,120,21,48,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,
17,17,17,17,3,32,17,17,17,17,125,221,240,54,48,68,68,68,68,93,12,43,128,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,244,117,195,219,192,
16,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,157,3,120,21,48,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,
3,32,17,17,17,17,49,0,18,17,17,17,209,57,128,87,1,19,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,157,3,120,21,48,17,17,17,81,23,195,10,32,
17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,57,128,87,1,19,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,
17,17,17,157,3,120,21,48,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,157,155,120,17,8,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,
17,17,3,32,17,17,17,17,157,155,120,17,8,17,17,17,81,23,195,10,32,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,57,128,87,1,19,17,17,17,117,49,172,0,18,17,17,17,
49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,185,137,23,129,16,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,185,137,23,129,16,17,17,17,117,49,
172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,185,137,23,129,16,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,49,0,18,17,17,17,209,185,137,23,129,
16,17,17,17,117,49,172,0,18,17,17,17,49,0,18,17,17,17,17,3,32,17,17,17,17,125,99,240,28,64,34,34,34,162,46,134,21,64,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,58,55,241,34,16,34,
34,34,162,46,134,21,64,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,58,55,241,34,16,34,34,34,162,46,134,21,64,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,58,
55,241,34,16,34,34,34,162,46,134,21,64,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,162,179,138,247,1,36,34,34,34,234,98,88,1,
36,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,64,34,34,34,34,138,
27,159,5,76,68,68,68,212,197,176,2,72,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,196,0,72,68,68,68,68,12,128,68,68,
68,68,196,0,72,68,68,68,68,12,128,68,68,68,68,20,55,62,11,152,136,136,136,168,139,97,5,144,136,136,136,136,1,144,136,136,136,136,24,0,137,136,136,136,136,1,144,136,136,136,136,24,0,137,136,136,136,136,
1,144,136,136,136,136,24,0,137,136,136,136,136,1,144,136,136,136,136,24,0,137,136,136,136,136,1,144,136,136,136,136,24,0,137,136,136,136,136,1,144,136,136,136,136,34,179,74,146,196,86,32,34,34,34,234,
74,1,144,77,64,68,20,63,135,195,1,151,203,5,87,102,38,210,210,211,225,76,117,194,233,116,170,223,87,118,174,27,234,235,113,168,180,20,238,210,82,184,75,221,40,200,207,135,215,235,253,70,181,67,86,118,
54,210,211,211,145,150,158,142,244,244,52,216,29,142,144,118,56,118,236,24,142,29,59,6,119,105,41,138,139,138,80,92,84,252,141,106,135,174,194,233,116,34,251,170,108,121,204,59,83,225,202,116,233,190,
47,73,18,36,73,66,105,105,41,60,199,142,193,93,234,70,126,126,62,202,220,238,115,126,221,179,179,179,145,54,36,29,46,87,102,132,237,189,1,135,74,15,169,219,123,113,113,17,170,61,213,95,203,245,17,70,94,
126,57,75,128,68,68,113,76,2,99,190,51,22,89,217,217,33,223,83,38,63,191,223,175,254,91,251,161,40,200,207,71,97,65,33,62,253,228,19,243,55,102,65,248,218,183,195,152,177,99,145,125,85,118,187,219,225,
147,173,159,96,103,97,225,57,219,14,93,101,103,103,204,216,49,152,56,121,178,46,244,40,180,253,172,253,183,46,24,53,52,160,176,160,0,239,190,179,6,213,213,213,231,76,127,187,50,51,49,102,236,24,100,103,
103,235,118,108,226,25,231,110,183,27,31,174,91,143,157,133,133,166,59,61,103,107,189,25,0,137,136,98,12,60,211,254,111,122,200,4,168,157,4,148,137,32,236,132,32,1,18,130,111,185,53,213,213,120,239,221,
247,176,125,219,182,200,111,212,95,163,137,49,214,118,8,215,6,230,237,80,131,247,222,125,247,156,106,135,174,18,252,166,77,159,142,49,99,199,152,134,31,179,49,175,11,128,82,160,151,37,109,111,3,251,74,
190,196,27,111,188,129,35,135,15,127,109,251,218,149,153,137,105,211,167,135,84,56,149,192,27,235,246,174,29,233,141,94,47,54,124,180,1,27,62,250,8,141,141,141,103,125,221,25,0,137,136,34,72,79,79,199,
204,217,179,195,30,234,242,249,124,225,39,4,191,102,66,128,190,42,160,117,188,230,56,150,191,252,50,246,237,219,23,113,18,56,155,147,162,43,51,19,115,231,207,51,13,126,218,245,55,111,7,101,130,68,196,
118,216,191,111,31,86,190,181,82,23,12,190,110,237,208,85,76,154,60,9,211,166,79,143,24,252,124,62,95,196,157,31,68,232,111,65,16,176,105,195,70,188,255,254,251,104,212,84,197,206,118,127,59,157,78,204,
157,63,63,98,240,83,182,121,227,186,171,219,60,66,171,128,198,237,125,229,91,111,97,247,174,93,103,117,189,25,0,137,136,34,76,130,51,103,207,54,157,8,140,193,207,239,11,124,150,252,134,0,24,12,63,128,
242,89,251,38,15,0,2,4,1,248,248,227,143,177,122,229,170,144,55,254,104,255,221,217,102,205,158,141,137,147,39,69,108,7,109,123,72,126,63,252,126,9,126,201,175,182,129,95,169,8,41,213,191,8,237,176,122,
213,106,108,254,248,227,168,19,34,131,96,199,115,56,28,248,205,125,247,153,238,240,152,142,251,40,213,176,96,21,208,164,191,5,1,199,143,31,199,226,133,139,80,94,94,126,214,251,123,204,216,177,152,53,123,
86,72,232,53,6,191,112,235,30,203,246,14,1,16,52,219,123,206,7,107,67,170,129,103,106,189,25,0,137,136,76,204,157,63,15,99,198,142,13,153,4,141,129,39,220,164,160,157,12,252,17,170,33,130,32,64,8,204,
136,162,32,224,192,129,3,120,105,241,139,234,164,160,125,243,63,211,19,98,164,48,224,243,249,66,219,192,231,135,207,239,11,9,128,126,201,31,241,16,112,176,29,228,137,81,16,4,236,248,108,7,94,91,177,194,
116,93,25,4,59,71,122,122,58,30,126,244,17,211,0,100,218,223,38,99,62,24,250,131,149,111,99,127,43,227,93,238,111,249,227,181,21,43,176,227,179,207,162,246,111,103,245,117,184,237,61,150,117,151,252,242,
14,142,49,0,154,110,239,114,2,84,215,187,162,188,28,207,254,231,25,243,237,61,240,179,157,181,238,12,128,68,68,134,208,51,119,254,188,144,139,27,180,19,129,110,50,8,132,158,136,147,33,12,193,71,210,87,
3,212,9,1,2,4,81,64,101,69,5,22,60,187,0,77,77,141,81,39,193,206,154,16,29,14,7,30,126,244,17,164,165,167,71,108,135,144,54,240,105,218,65,50,76,142,177,182,67,32,12,31,60,248,21,150,190,180,228,172,182,
67,87,14,127,198,0,164,235,107,147,113,31,62,0,234,43,97,218,106,175,220,223,34,4,65,192,27,175,191,142,252,188,188,152,194,95,71,246,183,89,248,51,11,126,193,29,29,31,124,198,0,168,169,116,135,236,232,
124,77,183,119,203,192,254,253,31,229,208,39,34,146,253,236,231,119,224,154,107,175,213,125,205,24,120,124,62,31,124,173,173,240,181,250,208,234,147,63,251,124,173,104,53,251,57,191,47,240,125,31,252,
126,31,124,62,253,97,99,101,50,213,134,166,196,196,158,184,120,248,112,236,222,181,11,45,45,45,49,77,6,29,57,49,132,11,127,166,237,224,11,182,65,171,175,53,208,54,62,245,115,184,54,241,251,140,135,207,
3,95,147,130,95,79,78,74,70,106,106,42,118,5,206,149,58,211,237,208,213,195,95,248,190,14,140,117,109,159,251,124,186,190,111,245,181,202,65,201,100,140,251,12,159,149,240,152,57,34,19,39,142,31,71,69,
69,133,26,16,141,253,218,209,125,29,46,252,181,182,6,215,171,181,181,53,184,94,173,225,183,247,86,67,59,196,190,189,39,226,226,225,23,159,241,237,157,21,64,34,162,8,147,129,250,166,111,50,25,250,125,62,
84,84,84,98,231,174,93,168,170,170,196,158,221,123,228,201,19,128,195,110,15,220,27,47,13,25,25,25,176,247,176,135,61,36,36,87,65,32,87,66,68,185,250,37,136,34,74,15,29,194,11,139,94,80,143,2,153,77,0,
157,17,126,30,126,244,81,221,97,223,144,48,16,104,143,86,165,34,228,243,97,255,254,253,216,185,107,23,142,86,85,225,224,193,131,234,239,165,166,166,194,233,116,194,229,114,33,35,99,184,238,240,88,176,
13,160,63,36,40,42,85,64,17,162,40,162,176,160,0,171,86,174,140,56,17,50,4,182,141,211,233,196,19,255,248,123,72,248,11,25,243,173,154,29,26,159,15,59,119,238,196,238,93,187,81,117,180,10,71,171,142,6,
126,15,72,31,146,14,167,51,5,174,12,23,6,167,13,54,61,228,111,172,246,42,125,45,136,2,68,81,196,171,203,150,163,168,168,40,248,211,66,108,253,30,175,105,255,55,29,211,166,79,55,221,201,49,219,230,67,182,
247,61,123,212,117,115,216,29,113,108,239,154,117,23,229,234,167,40,138,168,170,172,194,226,69,139,208,212,212,116,70,198,57,3,32,17,17,228,19,192,231,206,159,23,62,252,181,234,247,248,115,214,230,96,
213,234,213,56,120,240,32,12,103,44,1,38,239,203,163,71,143,194,168,235,174,195,197,195,135,27,14,9,5,194,79,224,176,167,32,136,16,69,121,98,16,69,17,31,111,220,132,77,27,55,6,2,146,249,132,216,145,147,
130,113,82,212,134,191,96,91,200,21,144,218,218,90,228,228,172,195,170,213,171,113,244,232,209,168,237,224,176,219,49,50,107,36,110,153,50,5,125,83,82,66,14,141,25,15,9,138,162,0,81,180,64,20,69,188,189,
122,21,10,11,10,53,235,215,185,237,208,85,60,241,143,191,235,42,189,218,240,167,11,65,173,62,148,87,148,227,229,165,203,240,233,182,79,81,95,95,31,181,191,157,78,39,70,141,26,133,9,215,127,15,61,236,118,
185,191,5,99,16,210,239,244,136,162,136,230,211,205,120,246,153,103,112,242,196,137,136,125,219,158,16,232,202,204,196,195,143,62,18,57,252,105,182,249,156,181,57,88,183,126,61,118,239,222,29,211,246,
158,53,114,36,38,76,152,128,139,135,95,28,101,123,15,174,183,40,138,216,190,109,59,214,190,255,126,196,245,212,158,27,216,158,113,206,0,72,68,172,130,152,84,65,180,19,129,246,115,97,65,33,30,127,252,113,
84,29,61,26,172,94,69,152,8,140,95,186,248,226,139,113,251,255,187,29,41,125,251,170,191,163,173,124,137,129,0,40,170,147,130,5,75,95,90,130,210,210,67,157,62,41,164,167,167,227,241,127,252,93,247,53,
179,54,240,249,124,88,251,193,90,60,251,236,115,168,111,168,111,83,59,92,123,237,181,248,225,15,127,8,187,189,71,176,29,66,130,176,168,134,192,230,230,102,44,120,238,57,212,158,60,25,185,29,24,2,219,21,
246,205,250,187,182,182,22,75,151,44,197,170,213,171,67,251,218,172,115,13,95,178,219,237,184,249,230,155,241,221,239,142,215,133,247,96,5,80,191,211,35,138,34,202,220,110,44,121,113,137,33,228,135,15,
253,241,244,181,195,225,192,51,207,61,27,178,189,107,15,241,42,167,53,20,20,20,116,242,246,46,111,235,202,186,91,68,49,176,189,151,118,250,246,110,25,208,175,223,163,220,12,136,168,43,155,59,127,62,210,
135,4,171,32,218,147,190,181,19,225,127,158,254,15,158,124,242,41,212,55,52,64,80,223,236,5,249,172,118,65,63,9,8,230,243,34,106,106,106,176,109,219,54,56,157,169,112,58,83,244,23,143,72,126,195,149,132,
242,73,244,253,7,12,64,222,142,29,234,36,109,250,89,115,136,173,173,207,120,191,255,193,63,32,41,41,41,100,82,212,182,69,237,201,90,60,250,232,163,120,237,245,215,209,220,210,220,230,118,56,124,248,48,
118,236,216,129,11,47,188,0,137,137,142,144,91,233,232,46,36,144,36,88,173,22,36,37,39,225,243,61,159,71,110,7,169,253,237,208,37,118,122,82,157,184,247,183,191,213,181,149,217,152,255,178,228,75,252,
225,15,127,192,39,159,126,106,30,128,98,232,239,150,150,22,124,241,197,23,40,41,249,18,35,70,92,2,139,197,162,158,255,103,118,1,5,36,9,189,147,146,112,242,228,73,84,86,86,198,212,223,241,244,245,143,111,
253,9,92,153,153,33,219,187,186,222,190,86,180,182,182,98,201,75,75,240,248,227,79,160,190,161,65,179,238,109,219,222,19,19,19,49,112,224,0,253,249,127,202,173,115,2,219,187,114,177,76,250,144,33,234,
77,209,181,183,148,233,232,237,221,50,128,23,129,16,81,23,230,202,204,196,79,110,187,213,116,34,212,78,136,143,253,245,49,188,247,222,127,13,59,254,230,19,65,52,45,45,45,200,207,207,71,82,114,18,6,244,
31,32,95,0,97,118,5,101,96,82,112,36,38,162,182,182,22,85,85,85,33,181,134,142,170,128,141,249,206,88,124,239,250,235,195,135,224,86,31,106,107,107,113,247,221,119,99,215,238,221,17,171,33,177,182,67,
99,99,35,242,243,243,113,209,197,23,35,209,145,104,18,132,181,55,209,6,82,82,82,224,118,187,113,242,228,73,205,95,16,98,174,8,82,208,236,219,111,71,122,186,249,78,143,54,252,221,115,207,61,161,213,47,
161,109,253,93,83,83,131,189,69,123,113,217,229,151,193,34,90,130,247,204,52,185,125,10,36,9,233,67,134,160,32,191,0,173,173,173,81,251,55,214,254,118,166,58,113,207,175,126,21,118,123,87,198,250,99,127,
125,12,171,86,173,54,228,220,182,111,239,187,119,239,70,82,82,18,6,12,232,15,159,46,248,134,174,123,247,110,221,1,65,128,91,173,2,118,252,246,222,229,2,96,138,211,169,158,235,208,150,135,144,219,237,118,
92,56,108,24,236,118,59,106,107,107,207,233,182,200,112,185,224,116,58,81,237,241,124,35,250,246,155,182,62,116,230,204,251,197,124,56,83,83,195,134,63,159,79,158,12,214,173,91,31,53,252,197,235,243,61,
159,163,119,239,222,232,63,160,191,102,34,12,13,62,144,128,254,253,251,99,199,103,159,105,222,248,59,54,252,220,247,187,223,194,17,56,36,102,214,14,39,107,79,226,238,187,238,198,129,192,5,30,102,97,64,
104,67,59,180,182,182,162,176,176,16,195,46,26,6,135,195,17,241,246,57,18,128,228,164,36,249,60,44,1,157,210,14,93,165,250,55,111,254,252,176,225,207,231,243,225,203,47,229,240,103,90,253,66,219,251,251,
84,221,41,20,21,23,227,210,75,47,149,67,160,174,10,6,221,205,163,173,22,43,106,106,106,112,52,176,227,99,214,175,130,230,176,106,44,135,68,141,193,215,108,123,255,235,99,127,237,156,237,253,243,207,209,
187,119,146,97,123,87,42,159,218,91,198,72,232,223,191,63,182,125,250,169,218,222,157,49,206,173,241,174,192,242,215,95,211,253,119,181,199,131,119,86,175,198,214,45,185,0,128,7,30,122,8,195,93,25,186,
159,217,186,37,23,43,150,47,135,215,235,197,45,211,166,226,150,169,83,177,120,225,66,245,119,148,112,246,228,127,158,70,73,81,49,30,251,243,159,163,254,93,0,186,159,181,219,237,120,242,153,255,224,233,
127,61,137,98,245,234,161,160,91,103,206,192,168,49,99,80,86,234,198,224,244,52,20,228,229,99,241,194,133,97,95,91,49,243,199,63,9,249,125,187,195,14,187,221,142,127,255,235,73,148,185,221,0,160,174,87,
164,182,233,44,74,155,255,250,238,123,116,1,104,212,152,209,184,99,206,28,188,179,122,53,222,89,181,90,253,218,173,51,103,162,172,84,94,238,123,126,243,107,172,88,190,92,93,198,12,151,11,247,63,244,96,
72,59,191,186,124,185,186,174,25,46,23,110,153,58,53,164,159,110,153,38,175,255,59,171,86,235,254,29,206,45,211,166,34,35,195,165,190,142,217,223,54,246,245,3,15,61,132,119,86,175,86,251,56,218,250,40,
253,187,98,249,43,88,159,147,163,11,140,102,235,16,105,44,172,88,254,10,70,102,101,133,252,78,134,203,133,59,230,220,137,7,127,127,63,38,76,154,24,50,14,180,237,103,54,78,140,99,153,206,236,68,104,118,
40,72,251,241,214,155,111,70,156,12,226,157,8,140,7,107,214,188,251,46,250,245,235,135,129,131,6,170,87,189,138,162,8,209,34,194,226,183,64,178,248,33,5,110,21,49,236,162,139,176,127,223,62,121,54,144,
164,192,39,1,128,4,65,16,32,73,242,103,229,100,123,245,191,163,200,190,42,91,247,136,55,179,118,88,242,210,18,28,56,112,80,223,14,130,208,230,9,81,219,14,222,198,70,44,88,240,60,126,255,187,223,193,110,
183,7,207,133,244,139,114,72,176,248,229,182,240,75,24,116,222,121,232,213,187,55,234,106,107,53,235,44,47,129,113,253,227,109,135,174,98,210,228,201,17,43,222,181,181,181,248,219,99,127,51,9,127,104,
87,127,43,93,82,94,94,142,215,94,127,29,179,102,206,84,175,128,21,69,1,126,209,15,209,34,194,239,183,192,226,151,32,89,36,92,59,234,58,236,222,189,27,202,141,4,181,227,92,146,36,53,252,73,144,32,68,89,
42,135,195,161,187,202,95,251,132,15,229,99,237,7,107,177,46,103,125,167,140,115,0,120,227,141,55,228,237,125,224,64,221,57,174,126,209,15,209,47,194,111,241,195,47,89,96,177,90,113,201,165,151,226,243,
61,123,32,69,217,222,141,159,59,45,0,106,67,145,50,249,221,243,155,95,163,184,168,88,13,31,127,251,243,95,212,9,218,110,183,227,142,185,115,48,97,210,68,53,12,84,123,60,184,50,43,75,55,73,143,204,202,
138,90,189,209,254,93,163,145,217,89,40,200,203,199,117,163,71,135,4,192,91,103,206,128,221,238,192,156,159,254,76,93,166,7,30,126,8,163,198,140,198,214,45,185,186,215,93,254,250,107,33,127,231,150,105,
83,145,225,114,225,215,119,221,173,86,14,71,141,25,141,7,30,126,72,247,53,109,208,10,215,54,157,165,218,227,193,200,172,44,93,200,185,210,208,166,25,46,23,110,157,57,19,143,253,233,207,106,152,27,156,
150,134,7,30,126,8,213,158,106,181,221,140,97,100,194,196,137,120,224,225,135,240,224,239,239,239,208,245,24,153,149,5,143,199,131,148,64,229,174,184,168,72,109,251,112,1,205,24,188,98,89,31,121,29,110,
64,65,126,126,92,99,204,108,44,12,119,101,96,194,196,137,106,59,219,237,118,220,49,231,78,252,251,95,79,134,29,7,183,76,155,138,219,102,206,84,215,197,248,125,250,250,76,132,198,9,161,188,188,28,207,60,
243,172,110,2,136,55,252,69,59,59,167,177,177,17,239,189,247,30,126,250,179,159,234,2,160,69,51,25,40,231,70,141,184,100,4,246,125,249,165,60,145,106,38,5,99,248,145,132,232,147,161,62,0,94,21,18,254,
180,247,46,43,44,40,212,29,14,107,203,164,24,75,59,188,251,222,123,152,54,245,7,242,9,241,66,160,13,196,64,248,179,4,171,162,89,217,217,216,248,209,6,205,68,8,221,164,24,79,40,232,138,204,66,144,122,159,
58,159,15,111,189,245,150,73,216,71,187,251,91,251,181,189,123,247,98,207,158,61,112,185,92,154,11,158,130,125,173,156,23,151,152,152,136,212,212,84,28,59,118,204,52,4,42,219,110,44,65,72,187,222,198,
167,217,248,125,126,212,213,214,226,153,103,158,49,172,96,199,142,115,9,192,178,229,203,113,239,189,191,209,108,239,126,88,44,34,196,192,78,142,82,5,29,49,98,4,246,4,174,58,142,180,189,107,23,44,158,16,
40,182,119,32,21,23,21,161,172,212,29,242,128,112,117,207,206,235,197,135,57,235,144,145,17,188,167,84,65,126,62,70,102,101,133,132,129,130,252,252,54,47,199,132,137,19,177,98,249,114,164,165,167,193,
110,183,155,126,79,187,76,47,60,191,48,174,215,126,236,79,127,214,29,54,222,186,37,23,5,121,249,24,53,102,76,155,219,166,35,197,210,166,215,79,188,1,139,159,95,168,134,37,0,40,115,187,177,98,249,114,92,
63,241,134,176,175,189,62,39,7,91,183,108,9,121,253,246,134,191,226,162,34,124,178,37,23,19,34,252,237,72,226,89,159,245,57,235,240,243,57,115,218,189,220,139,159,95,136,91,166,77,197,224,180,52,0,192,
29,115,231,96,107,110,174,110,25,140,222,89,181,58,164,42,78,95,15,102,193,71,27,126,150,46,89,26,60,4,19,103,248,147,98,152,12,20,95,29,58,132,188,188,124,180,180,180,200,31,173,45,104,105,109,69,107,
75,224,163,181,21,173,173,62,92,112,193,5,129,195,100,80,39,67,245,176,153,250,119,227,63,57,62,92,32,80,238,251,246,183,191,253,173,205,147,98,60,237,80,88,88,136,253,251,15,160,165,165,5,173,173,173,
104,105,105,69,75,107,11,90,91,91,2,159,229,182,184,224,130,161,80,158,179,42,73,193,191,96,118,1,72,91,46,18,248,102,143,249,108,221,161,126,99,95,87,84,84,224,229,165,47,183,57,236,199,211,223,239,127,
240,65,112,204,43,125,30,232,111,185,175,91,208,218,210,138,204,17,35,212,195,194,234,185,177,198,177,111,248,163,102,253,157,125,85,118,196,113,254,159,255,60,19,182,234,217,145,235,125,226,196,137,208,
237,93,221,206,229,109,190,165,181,21,3,7,13,66,183,110,221,212,64,104,92,103,237,253,52,219,50,190,219,20,0,51,92,46,245,227,150,105,83,145,226,76,49,61,236,170,173,154,120,52,149,23,175,215,171,11,44,
118,187,29,131,211,211,80,152,95,16,243,223,205,112,5,3,229,224,180,52,120,60,30,120,189,94,108,221,146,171,11,101,25,46,23,74,138,138,67,206,249,43,115,187,99,58,52,155,225,114,161,58,240,218,33,111,
86,134,208,149,146,226,12,105,155,193,233,105,234,137,156,157,169,48,191,0,131,53,225,87,9,127,218,229,86,66,87,72,120,204,203,143,26,238,10,243,11,58,52,0,94,23,168,190,154,5,215,120,67,100,44,235,163,
84,236,218,187,14,94,175,23,139,159,95,136,159,207,157,131,145,89,89,112,58,157,81,171,121,202,24,10,55,78,50,92,46,164,156,129,157,4,210,115,166,58,213,157,51,237,68,168,84,3,42,202,43,228,67,191,48,
175,130,196,53,17,72,234,59,183,225,35,248,245,237,219,183,7,130,95,139,33,248,105,63,124,56,127,240,249,97,194,79,248,201,48,18,237,33,112,93,59,4,130,112,97,97,33,170,170,142,198,61,41,154,183,131,249,
186,107,63,10,11,11,229,9,176,37,24,250,90,148,91,115,4,62,59,18,19,209,179,87,47,53,238,198,26,10,8,33,167,60,24,63,86,190,181,50,180,226,141,54,244,183,20,105,220,203,31,39,79,156,64,65,65,129,58,238,
91,90,90,2,125,31,120,218,72,224,118,44,131,6,13,210,135,32,132,6,255,104,65,200,225,112,168,235,110,12,190,126,191,31,117,181,117,109,218,222,209,198,237,125,243,199,31,7,199,120,96,91,87,119,114,52,
79,214,185,224,194,11,53,47,161,223,222,17,102,157,99,13,131,109,58,4,172,61,135,73,14,127,197,186,239,223,58,115,6,188,13,114,240,80,206,151,123,240,247,247,135,132,167,43,3,33,69,57,124,27,207,223,85,
42,108,74,133,238,147,64,152,43,200,207,199,3,15,61,168,59,20,218,94,202,186,152,133,1,253,68,159,1,167,211,9,187,195,142,193,105,105,216,186,37,23,15,254,254,254,54,93,112,210,166,42,96,158,220,150,91,
183,228,226,202,172,44,20,230,231,135,4,11,179,101,57,83,203,23,28,51,242,196,171,84,205,138,139,138,213,195,241,109,9,100,177,174,207,11,11,23,226,129,135,30,140,184,179,18,107,181,245,202,172,44,220,
49,119,14,30,251,83,232,33,234,81,163,71,235,42,222,131,211,211,240,244,191,158,12,25,39,90,91,115,183,96,235,22,94,192,114,38,25,175,128,52,126,108,221,186,53,238,170,151,233,68,96,146,66,130,247,133,
13,126,239,104,85,21,202,203,203,49,160,255,0,221,33,48,237,237,30,32,0,131,206,27,132,195,101,101,242,249,64,202,146,24,14,13,169,135,64,99,56,55,40,51,76,32,80,110,211,177,46,103,93,216,67,129,177,79,
136,102,109,32,153,182,231,238,221,187,49,110,252,56,56,236,14,136,146,168,191,40,38,176,110,128,124,239,198,186,218,90,245,198,186,146,161,29,148,191,17,107,59,116,181,113,111,22,246,253,62,31,214,173,
91,215,166,176,31,235,152,55,190,86,73,73,9,70,140,24,1,81,20,117,135,249,181,250,166,244,69,183,110,221,112,250,116,51,212,81,111,56,252,107,252,183,145,241,102,215,198,237,61,39,39,167,211,183,119,205,
22,139,147,39,79,162,168,168,8,195,135,15,15,156,234,225,135,85,178,134,188,134,60,135,75,250,223,54,59,20,12,156,153,115,0,141,231,101,221,243,155,95,235,38,240,173,91,130,135,196,26,26,26,76,15,143,
21,228,229,227,214,153,51,177,120,161,124,174,218,39,49,76,254,102,231,131,217,237,118,140,204,206,194,168,49,163,67,170,46,197,69,69,104,104,104,128,221,97,15,27,68,162,157,19,22,249,247,83,208,224,109,
208,76,226,185,106,53,232,142,57,115,224,245,54,156,209,171,82,11,243,243,213,202,218,200,236,44,172,88,190,28,19,38,77,212,5,35,187,221,30,18,144,82,156,206,168,33,208,184,174,237,49,106,204,104,12,78,
75,211,93,112,225,116,58,227,14,128,241,174,79,181,199,131,245,57,235,112,199,220,57,248,48,103,93,187,214,225,147,220,92,93,136,213,239,152,20,227,147,220,220,144,29,21,179,113,66,103,115,34,28,18,50,
33,4,39,6,227,68,136,168,19,97,164,73,80,138,113,2,41,41,41,65,74,74,74,112,50,48,16,4,1,41,125,83,212,60,165,6,72,117,2,208,20,111,164,216,194,90,154,89,32,208,124,124,242,201,167,109,159,20,219,216,
14,135,14,29,194,197,23,93,172,94,248,161,92,9,172,222,64,23,2,82,82,82,112,224,192,1,245,151,5,72,186,118,80,206,9,36,243,10,160,18,168,253,129,219,145,248,253,126,236,220,185,83,61,4,26,171,246,246,
247,190,125,251,80,95,95,15,187,221,174,187,250,61,248,67,114,159,167,56,157,40,63,114,68,191,219,20,38,252,152,5,254,112,59,58,82,224,243,78,229,121,211,49,6,223,72,59,57,82,140,237,85,90,90,138,11,46,
188,0,22,201,34,159,187,171,219,201,145,199,121,223,190,125,131,227,57,248,70,16,87,248,13,71,236,136,1,85,29,56,145,95,81,230,118,163,184,168,72,62,7,46,204,185,81,94,175,23,197,69,69,24,53,102,52,50,
92,174,54,159,255,167,84,15,103,254,248,39,234,199,226,133,11,113,221,232,209,234,178,216,237,246,144,74,88,134,203,133,7,194,92,113,170,165,252,190,246,144,115,176,210,51,6,133,97,150,123,197,242,229,
24,153,149,213,161,135,77,99,169,76,101,184,92,24,53,70,190,16,198,24,130,10,242,242,117,129,80,27,200,162,85,96,181,235,42,95,184,145,18,26,18,83,98,187,13,203,168,209,163,241,235,187,239,209,245,89,
138,51,37,238,195,160,109,89,159,245,57,57,112,58,157,184,50,107,100,167,245,67,117,181,71,29,255,237,173,54,82,231,137,28,124,36,245,118,39,49,151,189,76,38,193,240,135,131,205,191,233,118,151,201,135,
189,2,135,124,181,79,36,144,31,34,239,131,213,102,211,253,114,240,72,155,100,152,124,99,59,7,206,17,216,193,213,86,217,148,91,84,236,223,183,31,245,13,245,113,31,18,107,111,59,84,85,30,85,15,133,169,235,
111,184,82,181,111,223,190,80,239,145,163,28,10,54,107,7,30,14,214,244,181,195,124,167,71,146,131,208,206,157,187,218,177,211,35,133,116,109,216,190,150,244,223,170,172,172,12,246,119,171,121,127,167,
164,164,232,110,19,163,251,123,82,164,100,22,40,22,153,140,115,201,47,223,130,197,239,247,99,247,174,221,109,107,84,41,74,248,139,52,206,143,30,213,29,254,85,215,187,213,48,206,17,218,182,82,152,245,143,
231,156,215,118,159,3,56,106,204,104,140,26,51,166,77,135,239,10,243,243,113,203,212,169,49,29,254,13,71,123,69,166,90,97,9,84,192,148,243,225,214,231,172,195,47,127,243,107,53,196,201,87,143,206,192,
59,171,99,171,194,172,88,254,10,238,209,252,190,124,229,231,28,216,29,246,176,235,237,245,122,177,120,225,162,192,21,200,246,51,23,2,243,228,54,53,11,166,239,172,94,141,9,19,39,98,194,196,137,186,246,
155,48,113,162,174,45,236,14,187,174,143,239,249,205,175,117,235,170,156,19,57,97,226,68,221,57,135,25,174,12,221,233,0,198,115,221,6,167,165,97,100,86,22,220,110,119,72,80,92,159,179,46,238,139,65,98,
93,31,163,23,158,95,168,251,157,51,205,236,28,64,179,29,12,234,236,201,208,124,66,144,36,9,251,15,236,111,251,68,168,9,61,198,73,66,189,183,159,246,127,154,243,131,78,55,53,161,53,48,17,232,38,67,191,
50,33,248,49,112,224,64,221,105,69,102,147,97,60,39,132,27,207,139,82,67,160,228,199,169,83,167,218,86,253,147,218,215,14,101,135,203,130,235,110,18,6,124,126,31,108,9,54,67,59,32,98,59,180,231,100,249,
111,234,78,143,54,236,251,37,9,245,245,245,237,216,233,17,0,227,193,91,245,158,126,17,250,27,64,213,209,42,245,28,215,224,88,15,246,181,63,208,223,225,131,80,104,63,27,195,86,186,217,186,7,254,125,170,
238,148,233,142,78,188,219,187,20,231,56,63,114,228,72,216,29,28,117,156,219,172,250,115,0,77,195,111,219,198,116,220,135,128,75,138,138,117,231,226,121,60,30,60,253,175,39,213,9,221,237,46,69,67,132,
18,178,118,226,47,200,203,199,168,209,99,212,195,101,13,13,13,112,187,75,195,254,221,144,137,212,233,132,199,227,49,173,50,174,207,201,81,43,139,74,64,188,99,206,157,234,97,223,245,57,235,76,195,155,217,
223,41,200,207,7,158,95,136,91,103,206,192,224,180,52,249,34,150,188,124,221,249,95,102,149,175,226,162,34,108,205,205,109,87,133,51,22,218,54,87,14,77,42,161,90,187,92,213,30,15,30,251,211,159,113,203,
180,169,184,117,230,12,117,221,30,251,211,159,213,159,107,104,104,128,183,193,171,235,227,130,252,124,44,222,178,48,36,68,105,95,167,204,237,198,226,133,139,212,215,169,246,120,144,145,225,210,189,142,
219,93,10,175,215,107,122,184,191,32,63,95,125,173,72,99,65,187,174,177,172,143,89,159,202,87,10,191,98,90,197,140,54,22,162,45,95,180,10,168,89,187,104,199,11,157,121,250,208,35,65,242,75,168,63,21,251,
68,104,86,5,49,126,45,252,97,50,249,80,147,242,91,30,143,7,62,159,47,248,151,5,1,130,95,128,224,147,239,139,231,19,125,16,253,34,140,103,84,105,39,30,65,208,28,251,141,241,48,176,54,8,107,111,72,173,60,
241,35,230,12,16,49,4,199,222,14,146,36,161,213,215,170,111,7,229,217,169,126,49,240,56,45,179,186,34,15,249,198,220,95,218,138,111,224,198,219,186,170,183,208,89,253,45,232,135,167,36,161,177,177,73,
237,239,192,19,127,131,253,237,19,225,19,149,49,105,50,234,195,28,246,55,158,255,25,105,156,31,56,112,160,141,193,183,125,227,92,144,228,27,161,7,79,99,208,142,115,1,126,159,124,27,36,73,61,252,43,24,
118,180,12,231,188,198,121,24,88,184,226,210,75,89,24,39,162,46,231,209,63,253,17,174,204,76,205,45,71,148,171,16,91,80,80,80,136,223,220,123,111,76,231,3,73,38,255,144,204,38,3,227,196,96,156,114,2,111,
220,115,231,206,133,213,106,129,213,106,133,213,106,131,205,106,133,213,102,133,205,106,83,63,47,120,238,57,53,20,137,74,56,18,228,167,52,8,154,192,36,159,63,21,249,73,1,111,173,94,37,7,46,93,27,200,255,
94,186,116,41,150,191,242,138,110,249,98,107,7,41,100,162,138,173,29,228,182,30,56,112,32,166,76,249,190,220,6,22,205,250,43,109,97,179,225,88,213,81,172,89,179,70,183,254,250,118,8,174,59,159,12,34,203,
204,204,196,35,127,250,35,252,126,191,201,184,111,197,175,126,253,43,236,217,189,71,23,172,162,87,123,163,132,160,48,253,173,29,83,89,89,89,248,214,183,174,10,140,121,249,67,59,222,173,54,43,10,11,10,
145,191,99,71,72,95,235,250,29,225,251,91,217,222,67,198,121,107,11,10,242,11,226,223,222,205,214,189,77,219,251,28,221,122,27,215,221,102,211,111,239,202,54,15,200,65,81,48,89,231,88,199,185,149,211,
0,17,117,229,42,136,177,26,162,189,183,150,118,22,136,30,25,164,144,32,20,203,100,160,173,132,64,16,228,43,50,5,1,126,193,15,191,24,124,86,106,240,80,157,31,225,10,12,218,11,33,226,45,1,106,159,61,172,
252,219,225,112,132,252,118,212,73,177,93,237,32,87,69,228,155,244,250,33,8,126,136,66,224,6,197,129,234,167,228,15,158,187,5,132,94,93,26,188,16,70,210,84,70,12,205,194,113,31,242,188,105,229,66,155,
88,70,187,20,165,42,38,25,182,175,176,175,33,5,170,116,126,191,190,191,77,198,189,20,120,93,33,66,49,46,90,31,235,215,25,234,33,89,179,237,189,45,213,191,182,108,239,186,245,54,174,123,224,89,201,186,
149,11,116,147,32,68,95,233,104,109,34,114,83,32,34,78,134,193,243,134,194,5,159,184,39,66,205,185,72,186,137,193,100,242,85,126,186,127,255,254,193,11,81,2,39,230,43,147,128,242,208,248,211,77,77,161,
247,23,139,182,52,82,248,9,185,161,161,65,127,31,61,64,61,95,105,232,208,161,113,206,138,17,255,120,148,118,8,254,116,130,205,166,191,82,51,112,104,94,57,92,233,247,75,166,235,18,108,10,41,174,158,234,
210,59,64,82,176,207,131,253,29,99,175,155,156,110,39,33,180,15,36,179,254,214,132,126,135,195,97,24,247,146,97,220,27,238,1,216,134,222,85,78,29,10,142,243,224,235,94,122,201,165,109,216,209,9,55,248,
12,59,63,17,182,119,221,56,151,130,235,174,61,44,31,124,111,210,158,236,26,219,246,30,173,101,24,0,137,168,75,58,118,204,19,242,228,8,37,162,12,29,50,180,35,167,217,136,19,136,246,107,9,54,91,96,34,144,
76,39,77,73,2,60,158,234,184,163,104,36,218,27,213,235,194,152,4,164,246,75,109,235,170,134,124,201,180,34,168,251,185,224,87,147,147,147,3,23,228,4,174,210,148,252,33,149,155,83,167,234,66,111,178,75,
17,237,221,187,215,180,221,149,96,146,218,175,95,156,163,74,10,251,159,102,65,48,36,124,6,56,28,14,245,2,172,112,31,158,234,234,40,185,43,204,85,193,1,165,134,113,174,140,113,165,226,175,189,66,186,157,
169,218,116,76,235,150,50,240,229,62,125,250,200,235,237,15,238,212,40,225,79,25,231,117,117,117,49,109,104,186,43,129,99,220,24,24,0,137,168,75,242,120,142,133,153,210,228,255,93,114,201,37,0,132,216,
42,33,38,63,37,197,80,17,49,190,123,247,235,223,47,228,10,66,104,31,247,4,9,167,76,39,132,246,5,225,112,19,76,191,212,126,33,161,160,141,57,208,60,16,27,174,104,212,182,131,4,41,120,133,163,166,74,165,
76,222,161,19,99,236,9,176,43,63,21,78,169,248,154,245,202,165,151,92,210,49,59,59,146,121,175,132,59,36,220,79,25,99,38,21,97,229,247,154,79,159,110,215,46,143,199,48,206,37,109,45,81,2,46,185,244,210,
56,183,240,56,6,153,113,123,135,97,156,155,236,25,41,109,112,170,174,174,211,246,109,24,0,137,168,75,42,250,98,111,196,239,95,125,245,213,157,246,183,195,77,36,231,159,127,126,212,223,173,174,169,238,
208,101,41,45,61,20,241,251,151,140,24,209,201,61,161,111,141,132,132,4,244,239,215,63,234,111,85,148,87,112,16,183,101,220,239,13,63,238,135,14,29,138,212,212,212,51,186,60,177,140,121,0,168,168,104,
95,127,151,30,138,60,206,191,221,137,219,123,184,45,62,150,117,111,239,122,71,18,211,69,32,205,205,205,104,110,105,233,148,5,176,136,34,122,244,232,1,0,113,223,129,92,247,166,97,179,33,33,33,193,244,123,
241,188,110,143,238,221,97,177,88,58,124,61,59,170,13,149,246,106,108,108,132,207,239,143,187,109,226,89,142,142,234,155,88,218,220,231,243,117,72,251,104,215,53,158,54,138,165,13,58,67,91,251,209,231,
243,161,177,169,233,107,177,14,231,170,189,97,38,194,192,245,163,24,63,126,60,94,123,237,53,52,120,189,49,94,74,33,133,188,229,75,154,207,242,179,219,130,183,174,208,63,117,74,192,249,231,159,143,158,
61,123,6,238,131,17,124,226,133,242,88,55,101,217,42,59,120,66,48,15,194,193,43,95,110,190,249,102,108,220,184,41,174,201,78,136,120,16,74,109,145,208,171,20,5,1,23,92,112,129,186,238,234,45,46,228,11,
123,213,219,100,64,0,106,106,106,212,223,137,183,62,211,149,47,8,217,251,197,94,92,57,114,100,216,136,50,126,252,120,188,254,250,27,237,8,58,146,182,139,181,255,52,189,42,213,149,145,17,252,186,238,74,
238,224,66,85,86,86,134,93,222,88,251,178,180,84,190,133,152,242,30,42,104,255,128,0,140,27,63,14,47,188,240,2,26,34,60,21,43,174,74,156,110,123,15,110,195,202,23,146,147,147,209,183,79,223,224,152,214,
93,125,28,108,131,138,138,138,78,187,185,81,76,1,240,248,137,19,16,173,86,12,28,52,176,67,255,248,169,186,83,56,114,228,8,134,13,29,138,218,186,58,120,106,106,48,60,99,120,155,94,231,43,183,27,195,135,
13,51,253,254,254,131,7,145,125,85,118,212,215,41,41,46,65,255,212,84,36,118,212,185,0,29,220,134,218,246,58,82,89,137,243,206,59,15,61,123,245,140,186,151,92,95,95,143,254,129,18,123,172,203,161,253,
91,241,180,97,188,148,54,175,175,175,111,119,251,24,215,53,214,54,138,246,154,37,251,247,35,33,33,1,137,14,7,82,83,82,58,52,76,181,181,31,27,155,154,80,117,236,88,212,237,197,216,143,164,151,159,151,135,
203,175,184,66,157,156,180,19,143,35,209,129,113,227,198,225,191,239,191,223,1,123,255,82,216,104,164,188,185,187,92,25,16,5,49,240,97,126,155,139,250,250,122,212,84,215,152,76,164,109,159,34,74,75,75,
225,241,120,144,148,148,100,58,249,14,29,58,20,35,70,100,226,139,189,123,99,92,213,104,65,24,242,149,190,38,177,81,109,7,81,132,32,138,16,2,109,33,10,162,110,217,220,165,110,52,159,62,13,65,20,131,147,
173,192,43,125,219,82,1,12,134,13,185,13,111,186,249,38,188,247,222,127,209,224,109,136,97,84,9,250,88,100,22,252,4,1,130,201,85,177,130,32,160,95,191,126,24,48,96,0,68,81,148,63,140,99,62,176,108,218,
243,247,194,143,118,33,106,192,207,219,145,135,235,70,93,167,27,227,234,142,5,4,220,116,211,77,120,227,205,55,99,93,219,168,193,47,236,246,46,0,151,95,126,153,186,222,130,32,64,20,3,227,92,12,142,243,
150,230,22,84,86,84,196,180,189,107,127,68,136,241,253,32,230,219,192,124,127,202,20,204,155,63,175,67,7,98,94,94,30,238,154,255,11,0,192,201,218,90,204,155,63,31,51,52,55,3,142,57,24,157,58,133,235,191,
251,61,52,55,55,135,173,2,46,125,249,229,168,175,115,251,236,217,56,89,115,188,211,54,188,246,182,161,182,189,0,224,119,247,255,30,217,217,145,67,217,130,231,22,224,157,85,171,226,94,14,227,223,138,181,
13,227,165,109,243,246,182,143,217,186,198,210,70,49,5,213,146,18,188,187,102,13,214,188,179,6,221,187,117,195,121,3,59,110,103,168,173,253,56,60,99,120,212,62,49,235,71,210,79,8,87,92,121,165,238,13,
86,153,12,4,65,192,15,127,244,67,108,218,244,113,216,231,96,71,156,16,148,137,64,27,120,4,243,55,231,193,131,207,199,192,1,3,33,90,2,19,161,242,89,12,4,66,81,158,24,212,11,54,212,9,59,112,207,59,33,74,
8,20,194,87,96,228,118,216,129,235,39,76,208,175,191,166,10,113,215,93,119,225,206,57,115,218,89,21,210,4,1,193,124,138,186,252,242,203,208,171,87,47,136,162,8,139,24,108,3,101,82,20,2,147,164,187,180,
52,100,150,23,34,150,248,152,12,181,129,223,93,90,138,65,231,157,167,246,175,160,233,247,68,71,34,126,248,195,255,195,146,165,75,163,87,190,67,11,126,177,85,188,3,174,184,226,114,253,88,15,140,119,165,
159,67,198,125,152,229,137,181,119,243,243,118,224,186,81,215,153,6,95,65,0,110,186,233,38,108,218,180,9,199,60,158,120,238,161,30,92,71,205,173,106,34,109,239,253,251,247,71,90,90,90,200,118,110,28,231,
95,185,15,234,199,180,32,104,107,159,81,183,247,104,107,16,115,0,172,40,47,71,94,94,94,200,215,7,13,28,136,129,131,6,69,253,221,114,147,195,22,37,197,37,234,191,107,235,234,34,86,152,78,157,58,133,77,
27,55,226,251,83,166,132,124,175,103,207,158,24,63,126,60,62,219,190,93,173,144,156,171,78,157,58,133,146,146,146,176,213,178,179,58,89,154,244,63,128,152,2,86,73,73,137,225,177,82,193,10,85,71,181,79,
69,121,121,155,214,235,221,53,107,80,30,56,159,104,248,240,139,49,110,252,248,144,159,25,62,124,56,134,255,254,247,152,55,127,62,110,159,53,27,199,79,156,64,159,228,100,206,38,223,128,10,224,109,51,103,
32,33,33,33,180,226,6,1,137,137,137,152,255,139,249,248,251,223,255,30,195,132,32,191,237,135,155,12,5,201,252,234,188,132,132,4,140,26,53,10,22,171,5,22,139,21,22,139,37,16,128,44,250,48,40,136,248,226,
139,47,194,6,31,65,104,123,240,201,249,96,45,38,220,112,67,48,16,4,38,33,165,45,250,245,75,197,237,179,103,99,233,203,47,155,182,131,62,8,43,135,171,165,208,138,95,152,106,16,0,244,237,211,7,87,94,113,
37,44,22,139,220,6,22,11,44,150,224,250,91,68,11,68,65,68,75,75,75,72,32,16,76,179,95,244,138,80,87,181,101,243,22,220,58,227,54,253,141,179,53,31,55,222,116,35,62,219,177,35,194,105,18,145,119,124,212,
32,20,8,129,102,63,155,233,114,97,208,192,65,186,254,22,45,162,252,89,19,252,171,42,171,80,127,234,84,96,155,140,92,120,139,101,135,175,218,83,141,164,228,36,53,156,105,215,59,49,49,17,191,253,221,111,
113,239,189,247,197,20,248,228,251,24,26,199,190,20,195,246,126,29,172,86,171,60,166,195,140,115,65,20,176,127,223,126,77,240,51,228,192,24,86,58,90,155,196,20,0,19,18,18,176,97,195,6,108,216,176,65,247,
245,230,230,102,252,236,142,59,162,86,109,214,172,121,23,47,46,94,108,90,157,235,209,189,59,106,235,234,208,191,127,127,12,31,30,254,112,214,166,141,27,145,183,35,207,52,0,2,242,241,251,181,107,215,158,
243,1,176,164,164,4,63,251,127,63,13,123,152,177,71,247,238,103,101,185,18,19,19,77,43,73,245,245,245,248,162,40,250,161,161,39,30,127,28,95,124,254,133,233,249,149,231,13,24,208,97,237,211,183,13,161,
108,205,154,53,56,184,255,0,186,37,36,224,88,117,53,158,250,247,83,166,33,80,217,217,152,55,127,30,30,184,255,1,6,192,111,128,134,134,6,172,91,155,131,239,223,50,69,255,84,13,49,184,23,254,173,171,174,
194,172,89,51,177,108,217,242,152,10,93,225,38,67,93,53,32,240,115,9,221,18,48,121,210,36,216,237,14,88,68,81,158,12,173,22,88,45,86,117,66,84,130,96,85,149,126,34,20,194,4,177,182,4,31,143,199,131,226,
162,34,12,187,232,162,144,32,44,31,126,21,49,249,198,201,248,234,208,33,108,217,178,37,134,118,144,34,180,67,160,21,52,63,146,208,45,1,147,39,79,134,213,106,85,39,68,75,96,221,149,15,37,12,127,254,249,
231,104,110,110,142,24,8,4,134,190,200,1,112,203,22,220,50,245,7,230,59,62,129,113,127,223,125,247,226,209,71,30,69,169,219,29,165,10,104,18,132,162,244,247,176,139,46,196,183,175,249,54,68,81,233,235,
208,207,74,32,42,44,44,8,169,120,7,43,97,134,157,33,67,37,223,76,238,150,45,234,246,174,127,154,142,124,40,118,200,144,33,152,55,111,46,22,44,120,62,134,205,220,176,211,23,174,218,175,89,247,201,147,39,
33,41,41,73,13,127,86,171,28,128,213,113,30,8,131,71,171,142,170,135,127,133,48,59,58,161,219,123,124,131,62,166,0,216,39,57,217,116,178,171,58,122,52,230,63,212,55,57,57,108,56,115,31,62,140,201,55,221,
24,241,247,55,110,220,20,177,194,51,110,252,120,116,235,214,13,141,141,141,231,252,9,239,61,122,244,248,218,157,179,21,110,121,118,238,217,19,243,107,156,55,112,96,216,243,43,107,227,184,181,69,103,180,
79,223,62,125,212,101,43,41,249,50,108,0,4,128,158,189,122,113,6,249,6,201,201,201,193,245,19,174,135,45,33,65,158,4,68,77,240,9,4,193,241,227,198,97,195,135,27,80,81,89,17,121,66,8,55,25,2,250,74,72,
160,202,54,236,194,11,145,154,154,170,169,244,89,96,13,84,68,116,19,161,69,196,206,194,194,152,38,194,168,135,131,195,120,101,249,114,252,249,175,127,213,29,114,213,182,129,40,8,248,193,45,83,176,179,
176,208,180,154,31,113,98,212,84,65,161,169,12,2,128,197,98,193,176,11,47,132,221,225,144,207,131,82,130,176,40,63,14,207,162,76,144,22,11,124,173,173,216,251,249,231,166,237,32,8,145,3,1,15,3,7,121,27,
26,176,46,39,7,223,159,50,197,164,159,229,127,39,38,38,226,166,27,111,196,11,139,95,68,115,243,233,14,235,239,110,221,186,225,252,243,206,135,205,106,83,199,189,26,128,2,21,112,37,12,29,173,58,138,170,
138,202,144,16,36,104,43,97,225,66,79,152,211,30,114,114,114,112,253,13,19,228,240,171,86,185,3,135,157,3,149,246,171,191,117,53,54,109,216,136,47,247,237,107,219,78,95,152,237,125,64,255,254,232,147,
220,39,100,221,173,22,205,56,15,172,187,233,246,110,26,122,219,62,14,196,72,55,94,140,250,17,199,31,146,111,109,101,254,58,245,13,13,97,43,123,218,10,96,184,67,127,138,41,183,76,65,205,137,19,97,239,56,
30,211,114,182,167,61,58,168,173,16,195,235,197,115,35,43,93,219,119,244,114,116,84,219,118,240,114,197,221,70,129,103,161,158,170,175,151,175,196,140,224,84,93,93,204,203,208,105,253,24,79,219,119,214,
152,254,134,124,52,212,215,99,221,186,117,250,224,35,138,234,73,217,205,167,79,227,165,23,95,130,189,71,119,164,58,157,16,69,49,220,92,19,82,135,51,158,151,166,28,90,182,89,109,232,155,156,140,154,234,
106,236,218,181,11,18,164,192,51,64,173,106,232,177,90,228,231,224,90,44,22,28,216,127,192,252,234,95,245,174,178,134,123,235,65,210,252,59,220,35,239,244,31,165,135,74,177,53,119,171,90,17,81,218,67,
57,23,171,170,170,10,47,45,126,9,253,156,78,36,71,171,126,11,102,245,72,132,92,225,220,45,33,1,41,125,250,160,166,186,26,187,119,238,148,159,125,170,121,230,175,218,14,129,54,217,251,197,94,185,250,103,
18,231,36,9,48,222,12,88,191,254,82,76,237,208,85,62,86,175,92,133,154,234,106,77,63,43,125,45,127,222,89,184,19,235,215,173,195,160,129,3,224,176,219,99,232,111,68,237,111,135,221,142,228,222,189,81,
180,119,47,202,203,203,131,207,250,181,40,207,193,213,135,161,255,109,223,30,49,184,75,18,66,238,175,169,237,251,112,219,251,171,203,151,155,142,115,65,20,112,250,244,105,188,244,226,139,240,181,182,162,
127,191,126,209,183,119,33,250,246,110,181,88,208,55,57,25,126,159,15,219,62,253,20,146,36,5,159,245,107,181,133,140,243,195,101,101,97,110,255,18,250,28,20,201,116,253,99,27,231,103,253,62,128,141,77,
77,112,58,157,81,15,255,246,232,222,29,61,186,119,15,123,30,26,32,95,68,112,252,196,137,115,126,239,172,190,161,1,187,62,255,220,244,131,58,167,125,14,124,245,21,62,47,42,194,85,87,95,141,41,183,132,223,
25,41,41,41,193,130,231,22,32,49,49,145,29,241,13,242,246,170,213,40,43,43,51,156,144,45,226,244,233,211,88,180,112,81,32,124,9,232,213,179,39,206,27,52,8,61,35,245,191,161,42,161,157,20,68,65,64,98,162,
3,41,125,251,194,102,147,43,142,135,203,202,144,187,37,23,146,4,88,109,154,135,193,7,38,3,95,171,15,255,219,182,45,80,89,48,28,14,18,96,82,249,106,123,73,224,213,229,203,209,216,216,8,65,115,241,137,40,
10,168,172,168,196,162,133,139,208,212,212,40,31,21,74,74,198,121,131,6,133,156,146,34,68,8,129,198,118,232,221,187,23,250,36,247,129,40,90,32,8,34,202,202,202,80,144,95,0,171,77,19,2,149,73,209,98,193,
201,19,39,81,80,80,160,171,134,24,43,66,172,242,197,231,133,133,139,212,29,30,65,61,239,78,68,65,126,62,86,189,245,150,92,161,21,69,244,239,215,15,253,83,83,97,181,90,163,4,33,243,254,78,176,37,160,79,
159,100,244,234,217,43,80,109,147,255,198,225,178,195,106,216,183,218,172,106,248,179,90,172,40,218,187,23,199,107,106,130,85,62,33,244,50,239,208,234,87,108,253,159,187,37,23,37,37,37,134,113,46,226,
116,211,105,44,90,184,80,221,222,19,29,14,12,62,255,252,200,219,59,226,219,222,235,234,234,144,155,155,11,201,47,5,119,114,52,59,126,190,86,31,182,111,223,174,185,20,75,187,189,155,30,12,110,115,255,91,
207,246,0,60,126,226,4,110,152,52,49,74,245,111,19,122,247,238,141,230,230,102,148,20,151,132,189,232,96,248,240,225,232,223,191,63,106,235,234,208,187,131,15,211,197,115,223,53,45,139,197,18,215,121,
123,217,217,217,97,207,169,27,225,202,236,242,111,88,157,209,62,145,174,164,125,119,205,26,172,89,179,70,14,127,197,37,240,122,189,112,166,164,160,127,224,102,169,109,29,23,145,238,91,73,103,199,226,133,
139,240,187,7,238,71,66,66,2,68,139,136,211,245,167,241,220,179,207,162,162,188,60,16,178,228,39,83,216,172,86,56,157,78,36,37,37,225,196,201,147,240,122,189,240,251,253,161,71,131,4,65,173,208,37,216,
108,232,209,163,7,122,116,239,46,87,20,36,253,123,119,237,201,147,248,120,227,70,220,48,113,34,172,221,172,129,195,97,242,199,186,181,57,104,110,110,209,204,129,130,174,234,96,156,8,245,65,40,190,201,
193,235,245,226,233,167,158,194,239,238,191,95,61,28,219,116,250,52,150,190,244,18,154,26,27,53,135,212,36,116,75,72,192,128,254,253,209,216,212,132,83,245,245,230,237,160,44,75,160,34,145,96,179,193,
97,183,163,91,183,110,166,237,80,90,122,8,86,171,5,215,141,30,165,158,15,37,31,250,245,97,203,230,205,193,91,131,0,250,96,160,89,119,229,144,159,182,29,4,158,16,104,170,184,184,24,31,174,95,143,239,126,
239,123,106,8,202,219,177,3,175,191,246,186,46,96,8,130,252,184,54,187,221,142,83,245,245,104,240,122,225,13,220,47,47,82,127,247,232,222,29,14,187,61,248,94,103,232,239,29,159,125,6,155,205,134,139,46,
186,40,120,238,167,104,193,201,147,39,177,61,176,211,19,178,227,99,56,220,175,255,55,12,159,195,247,251,211,79,62,133,127,62,245,36,18,108,54,117,221,205,182,119,139,40,170,219,123,109,93,29,188,94,47,
90,91,91,35,110,239,86,139,5,246,30,61,96,183,219,195,111,239,155,54,225,134,137,19,97,75,176,169,167,122,88,45,86,124,188,113,19,234,79,213,235,15,113,107,171,140,66,199,109,239,103,61,0,214,214,213,
69,60,223,10,144,175,62,237,221,171,23,26,27,27,145,151,151,23,241,86,49,183,205,152,129,197,139,22,117,120,0,108,108,106,194,81,143,39,174,251,20,158,170,59,133,35,135,15,227,66,222,131,237,107,77,123,
21,240,184,241,227,116,213,232,139,135,15,71,118,246,85,168,40,47,199,169,186,83,216,191,127,191,26,252,44,22,75,155,198,69,69,121,5,234,27,26,212,16,73,95,15,101,101,101,120,241,133,197,248,197,221,119,
161,169,169,9,79,255,251,223,40,63,114,4,198,155,92,8,129,139,28,108,86,43,156,41,41,129,199,84,53,163,165,181,21,205,205,205,186,43,255,18,18,18,212,139,59,140,231,4,25,235,22,39,79,158,196,71,31,126,
136,201,55,221,136,110,221,186,193,34,90,176,125,251,54,84,86,86,234,131,159,114,62,84,7,77,132,33,85,238,226,18,188,182,98,5,126,244,163,31,225,116,83,19,254,243,212,211,56,126,252,184,186,238,242,225,
37,65,125,237,30,221,187,163,123,247,238,234,227,186,78,55,183,160,181,181,69,247,236,223,132,110,9,176,89,173,161,87,133,154,180,195,87,95,125,5,139,213,138,177,223,25,27,8,194,34,54,109,216,136,154,
154,154,224,205,161,181,237,160,107,11,253,250,134,222,103,154,65,208,104,197,43,175,226,252,243,7,227,194,97,23,162,188,162,28,111,175,90,29,188,131,145,38,8,41,247,139,233,153,152,136,68,135,3,62,191,
31,205,167,155,113,186,185,25,126,191,79,237,83,249,60,54,81,19,250,204,251,91,169,104,109,223,182,13,54,155,13,195,135,15,135,197,98,65,107,107,43,62,92,183,78,115,8,53,124,216,111,79,8,242,122,189,248,
219,95,31,195,31,255,244,71,136,162,136,87,95,89,142,242,35,71,32,8,130,122,8,85,29,47,129,237,189,111,159,62,232,147,156,140,230,230,102,180,180,200,219,187,122,142,163,4,216,108,193,138,117,92,219,187,
181,27,44,22,11,190,248,226,11,148,30,42,213,109,239,66,39,110,239,103,53,0,54,54,53,161,91,183,110,17,111,35,82,81,94,142,170,170,42,92,226,114,193,18,216,59,137,100,252,248,113,120,226,241,199,49,104,
192,128,14,127,162,71,44,247,93,51,6,215,187,230,205,143,235,111,228,229,229,225,246,89,179,249,174,116,6,219,71,185,10,24,0,22,45,92,136,55,87,190,165,134,192,225,195,135,235,2,97,69,121,57,238,190,235,
110,148,87,86,98,240,121,231,181,105,92,44,120,110,1,222,54,220,211,143,190,30,10,11,10,176,228,197,23,225,241,84,227,200,225,195,129,160,35,25,38,195,96,248,9,124,3,9,221,18,144,144,96,131,189,71,143,
192,243,123,149,73,65,82,207,221,212,158,28,14,99,229,32,224,196,137,19,234,85,201,135,190,250,10,159,239,249,92,23,122,66,39,131,206,169,122,125,184,110,61,250,246,77,193,39,185,185,56,114,36,92,59,232,
3,177,220,14,221,144,144,144,32,159,131,213,198,118,16,0,28,60,112,0,86,139,5,227,191,247,93,124,188,233,99,148,150,150,234,214,61,150,16,204,234,95,108,4,65,192,127,254,253,111,252,248,214,159,224,181,
87,87,160,81,83,233,213,7,161,192,51,153,3,109,111,17,69,116,239,209,29,221,187,119,83,207,227,86,158,95,171,123,134,117,12,253,253,201,214,173,72,176,217,48,244,130,11,240,223,247,222,67,125,125,189,
254,105,48,97,194,126,123,67,80,153,219,141,151,94,124,17,146,36,97,251,182,237,48,62,165,198,184,179,163,142,243,132,4,36,216,194,111,239,144,36,181,157,162,110,239,31,4,183,247,109,159,126,26,88,191,
208,237,93,187,158,102,219,123,91,14,5,159,213,0,120,252,196,9,140,143,82,253,43,41,41,129,207,231,211,157,223,85,82,82,18,246,156,193,129,131,6,97,248,240,225,168,173,171,235,240,219,116,84,148,87,96,
193,115,11,226,248,249,182,221,151,46,209,225,96,213,240,12,183,79,159,228,100,36,58,28,40,59,114,4,121,59,242,34,142,175,223,221,255,123,93,176,143,119,92,228,229,237,96,39,126,141,39,195,79,182,126,
162,123,175,86,66,143,36,73,17,195,143,90,238,50,187,75,191,230,187,102,151,237,9,154,63,120,188,166,6,171,223,90,137,83,218,91,190,24,66,143,89,184,233,200,170,151,32,8,120,125,197,138,176,237,160,13,
5,146,20,184,119,96,160,29,228,22,104,127,59,236,223,183,15,71,171,170,52,237,96,8,3,17,67,48,171,127,241,240,122,189,120,105,241,139,186,182,210,134,159,224,216,15,13,55,146,230,54,47,234,81,80,165,96,
40,25,174,144,55,237,111,121,252,108,254,248,99,20,22,20,170,253,173,6,199,136,97,191,125,59,61,241,108,239,218,245,143,229,210,91,33,214,237,253,120,13,86,173,92,169,191,215,161,38,252,169,231,187,10,
145,183,247,182,84,251,207,106,0,172,175,175,199,184,241,227,34,254,204,184,241,227,99,186,207,156,214,140,153,51,240,247,199,159,232,208,0,152,96,179,161,190,161,33,238,202,13,239,21,119,110,73,72,72,
48,189,197,69,71,143,139,206,120,220,32,117,124,24,84,38,1,101,50,212,63,195,86,83,13,212,78,132,17,166,3,211,187,70,104,255,21,184,183,215,169,64,5,196,108,18,52,78,6,230,147,162,208,169,237,160,15,5,
18,16,8,129,209,39,199,142,107,7,179,117,53,126,77,253,204,139,67,98,234,99,125,116,209,87,125,117,213,176,40,227,221,244,102,232,17,66,144,210,75,245,245,167,66,170,95,209,199,126,251,194,190,242,184,
54,227,56,55,238,236,152,135,224,112,183,135,143,111,156,215,159,165,237,253,172,5,192,230,230,102,88,109,182,168,231,255,181,197,184,241,227,241,135,7,254,16,241,209,112,145,170,146,245,13,13,166,19,
118,87,59,103,171,185,165,5,85,199,142,153,134,158,179,25,108,195,45,151,26,186,147,146,206,216,5,22,9,9,9,60,151,239,27,60,25,234,39,190,96,224,211,133,159,88,158,15,18,252,191,240,207,49,21,52,19,132,
97,34,208,79,8,161,147,65,184,67,65,237,9,130,145,67,65,252,237,96,54,57,134,107,135,144,112,16,97,98,140,105,82,100,254,139,35,232,7,115,188,89,53,12,218,155,27,11,209,194,80,164,190,54,246,183,230,223,
166,227,61,92,248,107,231,78,143,246,249,197,81,118,118,98,41,254,157,75,219,251,89,11,128,181,117,117,81,15,255,2,230,231,124,205,155,63,63,226,211,71,122,246,236,137,113,227,199,227,203,162,34,56,83,
82,98,94,166,41,83,166,168,23,3,232,151,97,7,14,187,203,186,84,213,102,208,192,129,248,217,29,119,132,124,189,162,188,28,27,62,250,232,172,5,192,112,203,165,120,119,205,26,52,183,180,240,10,91,234,208,
138,72,48,244,132,134,31,117,82,208,76,136,241,223,157,53,204,132,16,246,205,63,116,50,104,207,161,160,248,66,65,180,118,208,165,135,240,165,161,182,4,64,195,250,6,215,177,115,66,48,195,96,248,207,102,
161,95,130,102,91,136,165,191,13,87,179,199,22,128,34,135,191,182,158,242,16,109,103,71,89,195,96,213,255,220,223,222,207,90,0,60,126,226,68,212,195,191,128,124,69,90,159,228,100,245,132,251,227,39,78,
68,189,33,180,28,230,190,143,223,125,250,105,92,1,48,220,205,168,23,60,7,28,118,151,117,169,55,128,129,131,6,153,134,236,188,188,60,108,248,232,163,175,221,114,105,195,250,137,234,26,190,131,211,25,9,
63,134,82,8,162,62,160,93,138,113,34,140,105,178,51,155,12,58,239,144,167,190,61,34,181,131,129,113,130,140,167,29,194,78,140,241,181,7,197,31,132,204,195,31,76,67,191,174,75,67,250,91,8,127,195,251,176,
253,29,97,108,119,112,248,139,101,123,7,204,130,111,251,183,119,221,46,203,89,24,223,237,14,128,21,229,229,17,111,206,172,252,140,86,115,115,51,26,155,154,208,179,87,175,168,191,187,105,211,70,221,125,
244,122,244,232,129,47,75,74,162,254,94,207,94,189,208,216,212,164,222,163,45,218,207,199,179,252,157,217,86,37,197,37,49,191,94,44,63,107,182,236,177,44,71,60,127,51,150,215,58,85,119,170,67,218,39,222,
191,19,75,27,25,127,39,218,114,196,211,71,157,217,143,167,234,78,117,232,120,162,216,38,7,237,155,127,104,85,196,88,14,137,16,132,76,38,5,221,147,76,133,208,127,135,159,20,58,239,144,167,121,40,48,159,
28,35,29,42,83,231,195,54,182,67,44,231,250,233,126,158,199,125,59,56,4,34,108,232,55,13,130,177,140,251,144,29,159,72,33,39,210,118,208,65,21,111,221,249,128,136,48,190,59,122,123,143,117,61,35,111,15,
113,175,239,229,151,92,34,181,181,177,142,159,56,17,243,147,55,180,207,19,174,173,171,131,167,186,58,230,191,51,104,224,64,93,8,60,240,213,87,49,255,110,159,228,228,14,121,58,72,184,231,33,119,70,91,245,
232,209,3,131,6,12,136,248,51,229,149,149,234,229,250,241,44,123,60,203,17,203,242,197,211,23,198,126,108,107,251,196,250,119,226,105,35,229,119,98,93,142,88,250,40,22,109,237,199,198,166,38,148,155,62,
42,168,243,150,181,171,210,63,106,9,8,125,204,90,232,215,140,95,55,251,239,144,169,39,194,137,236,145,39,0,193,80,112,105,219,100,144,225,114,233,254,187,218,227,129,199,227,209,181,195,197,195,135,155,
62,82,170,252,200,17,120,189,141,24,122,193,80,221,163,10,27,189,141,168,12,60,63,185,119,82,18,146,146,146,76,31,99,120,242,228,73,156,110,106,66,82,82,18,142,234,158,49,47,200,143,157,19,128,147,39,
78,196,220,14,130,238,56,50,197,210,247,197,69,69,234,191,221,165,165,104,208,156,11,223,189,71,15,244,237,211,7,135,15,31,1,32,225,130,11,47,212,245,127,99,99,35,42,43,42,212,49,158,150,62,4,238,210,
67,154,194,159,132,238,221,187,163,119,82,18,142,86,85,33,45,61,61,76,152,23,224,46,61,20,54,232,183,53,0,217,29,14,100,101,101,33,37,213,137,234,99,30,228,231,231,195,171,89,191,180,244,116,120,60,30,
249,107,146,122,103,63,12,187,232,34,236,251,242,75,12,58,239,60,245,94,151,198,143,202,138,74,116,239,222,13,189,147,146,116,239,1,85,149,149,104,10,20,160,186,117,239,46,143,237,170,42,221,122,244,78,
74,130,0,160,169,233,52,6,12,28,160,91,95,229,163,169,169,73,115,115,234,142,11,127,237,14,128,68,68,223,196,201,48,35,211,165,11,62,199,142,30,67,97,126,62,188,94,47,36,9,184,250,219,87,35,185,79,31,
248,37,63,36,191,60,17,248,253,126,28,60,112,0,95,29,60,8,73,146,48,96,192,0,92,125,237,53,232,221,59,9,141,141,141,216,85,88,136,146,226,98,221,223,74,31,50,4,151,93,113,57,146,146,147,81,123,178,22,
187,119,238,68,233,161,67,97,222,220,163,79,126,237,153,12,94,125,243,13,53,4,0,64,90,90,26,114,183,228,226,213,101,203,212,175,45,89,190,28,123,247,126,1,191,207,7,191,223,15,159,207,15,191,223,143,183,
87,173,66,249,145,35,248,251,147,79,98,255,190,125,144,36,249,235,189,147,146,112,226,248,9,44,89,188,24,151,93,113,5,46,187,252,114,0,146,60,89,74,192,201,147,242,206,214,238,157,187,80,91,91,139,49,
223,249,14,150,45,89,170,45,14,97,252,247,190,11,64,192,166,13,27,34,4,223,142,9,193,93,213,171,111,190,129,219,126,248,35,245,223,235,214,174,197,171,203,150,171,97,102,216,197,23,97,242,141,55,226,169,
127,254,11,146,4,60,249,204,211,216,87,242,101,96,252,203,253,124,188,230,56,94,92,180,8,0,240,232,95,255,130,71,30,120,80,183,195,147,150,158,142,49,223,249,14,150,47,93,138,153,183,223,174,246,241,144,
161,23,160,244,208,33,245,121,188,75,22,47,142,218,199,186,49,31,37,236,167,165,167,225,15,15,63,130,252,188,60,84,123,60,72,113,58,145,149,157,141,191,254,233,143,112,151,186,1,0,127,120,228,97,188,189,
114,149,58,254,149,245,94,180,228,37,252,252,246,159,98,234,244,105,24,48,104,32,252,126,63,6,12,24,128,234,234,106,120,27,26,224,247,251,241,254,187,239,97,184,203,133,203,175,188,2,199,107,142,171,193,
176,255,128,254,120,247,237,119,80,82,92,172,91,119,173,239,140,31,7,64,64,73,113,49,38,223,116,35,4,65,128,221,238,64,223,148,190,168,170,172,146,31,195,88,89,137,119,223,126,167,195,183,119,0,176,198,
245,80,121,34,162,174,16,0,93,46,20,239,13,134,161,209,99,198,224,250,27,38,224,161,251,31,0,0,92,115,221,117,240,120,142,201,21,50,205,164,164,188,33,39,247,233,131,159,207,155,139,255,174,121,23,39,
78,156,64,82,82,18,38,223,124,19,4,81,12,188,174,132,43,70,142,196,196,27,39,35,231,191,31,96,207,174,221,72,78,78,198,109,179,102,97,237,251,239,163,48,63,31,198,89,77,251,94,175,61,4,101,252,119,123,
252,229,209,63,234,170,38,15,62,242,48,38,76,156,136,117,107,215,2,0,44,86,11,254,249,248,19,166,15,160,7,228,39,33,188,240,252,243,186,239,221,49,103,14,174,204,202,66,97,126,62,118,237,220,9,72,192,
184,239,142,135,4,9,31,111,216,168,78,227,67,134,14,145,159,53,107,152,207,229,103,5,155,214,102,53,135,35,245,247,106,227,188,22,63,109,155,141,204,206,70,126,94,158,60,86,37,57,100,5,31,172,32,63,21,
99,225,115,207,233,126,231,231,115,231,98,100,118,54,10,242,242,96,179,217,2,247,4,12,222,24,80,20,69,245,89,194,203,151,190,172,254,222,95,159,248,27,150,45,89,162,219,209,209,246,177,254,238,145,250,
254,21,4,65,189,233,116,56,191,188,247,94,188,178,236,101,228,110,222,162,126,173,120,236,24,252,242,222,123,241,203,249,191,208,142,166,144,113,35,47,175,132,213,43,87,170,213,205,187,126,249,75,228,
124,240,1,246,239,219,167,254,92,230,37,35,176,103,215,110,108,248,240,67,117,73,135,12,25,130,31,76,159,134,146,226,18,205,186,235,79,211,80,198,118,85,101,37,94,122,65,14,190,23,92,120,33,38,76,156,
136,69,11,22,24,182,255,142,223,222,227,58,7,240,193,71,31,9,57,76,80,92,84,132,191,60,250,71,56,157,78,252,251,185,103,145,187,121,139,110,193,87,188,245,38,110,253,191,31,182,233,53,59,210,157,243,230,
97,244,216,49,248,229,252,95,232,14,107,196,178,206,171,87,174,196,212,233,211,177,122,229,74,221,164,16,142,217,58,43,175,19,203,239,155,113,58,157,184,115,254,60,100,184,92,240,120,60,120,245,229,101,
200,207,203,67,90,122,58,126,117,223,189,112,58,157,88,183,118,45,94,121,57,184,183,126,195,164,73,112,56,28,88,189,114,165,186,7,22,238,103,137,72,243,30,180,183,72,221,110,212,137,234,241,199,213,67,
101,22,139,5,59,254,247,25,190,44,41,49,125,19,190,234,91,223,194,255,182,109,199,206,194,66,245,48,88,109,109,45,92,153,153,40,41,42,66,143,30,118,92,63,97,2,94,126,241,165,192,131,231,129,210,67,165,
56,122,244,40,230,204,159,143,146,162,34,221,19,25,204,38,130,206,174,118,121,27,26,176,104,193,243,184,115,222,220,96,0,12,132,0,179,199,101,73,146,4,171,205,22,50,81,185,221,165,72,73,73,129,122,86,
158,32,193,98,17,67,150,93,153,36,5,67,53,79,254,89,225,172,181,67,87,180,104,193,2,220,57,111,158,28,144,4,200,207,133,214,244,189,213,102,11,233,123,119,169,220,207,130,32,192,102,179,65,57,163,78,201,
60,106,255,26,250,201,102,75,48,237,187,112,213,239,88,170,126,138,172,236,108,84,123,60,186,240,7,0,185,155,183,224,134,73,147,144,21,8,186,97,3,146,213,106,216,1,19,96,177,90,116,59,123,146,36,65,20,
45,250,101,150,128,210,67,135,144,154,154,10,65,208,174,187,110,13,97,177,88,66,110,240,44,138,242,223,56,19,227,60,174,0,168,132,178,169,211,167,3,128,238,13,114,100,118,54,94,121,121,25,70,143,29,3,
187,195,161,59,190,222,214,215,236,208,189,250,76,23,22,45,88,128,209,99,199,182,233,111,180,55,144,182,247,247,127,48,125,58,138,247,202,193,56,35,211,133,95,223,123,47,242,111,207,195,15,166,79,195,
186,15,214,34,119,203,22,60,246,247,39,144,187,121,11,220,165,165,152,58,125,58,110,152,52,17,235,214,230,104,94,195,252,103,137,40,50,187,195,1,71,207,68,52,54,54,66,16,130,147,128,113,18,84,254,187,
182,246,36,70,143,29,139,202,202,10,124,190,123,15,0,224,208,87,95,225,208,87,95,65,16,4,140,184,244,82,28,59,118,12,85,149,149,186,55,244,202,138,10,184,75,75,113,201,101,151,33,239,179,207,206,122,224,
113,151,150,234,206,215,82,222,171,149,106,137,228,151,224,151,252,120,247,237,119,228,137,95,55,97,10,232,211,183,47,190,125,205,53,120,237,149,87,117,135,242,180,65,82,27,0,7,167,165,97,254,221,119,
169,143,195,18,4,1,125,83,82,176,227,127,159,49,248,157,225,29,160,130,188,60,181,240,33,8,34,172,22,171,46,24,105,199,125,74,74,10,190,125,237,181,88,177,124,57,0,65,173,0,106,83,90,248,0,104,139,208,
151,66,187,110,242,156,150,158,30,182,232,82,144,151,143,180,244,244,136,1,80,14,104,250,237,219,106,177,66,20,69,221,242,200,227,89,210,133,197,235,111,152,8,183,219,13,65,16,194,174,187,197,98,145,43,
222,186,195,186,114,91,135,140,243,78,56,175,181,195,110,3,51,122,236,24,60,245,143,127,6,82,119,86,72,226,142,215,140,217,179,212,202,224,162,5,207,171,111,68,191,186,239,94,84,123,60,176,219,237,40,
200,203,71,81,209,94,76,157,62,93,13,88,102,213,183,172,236,108,20,239,45,66,238,102,57,248,40,1,80,251,179,218,10,221,232,177,99,112,195,164,73,240,122,189,72,113,58,67,190,111,182,108,177,86,18,93,174,
76,140,204,206,82,95,91,91,201,187,115,222,92,181,2,106,172,206,105,171,170,202,121,11,202,186,41,237,190,117,243,22,100,101,103,195,93,90,10,143,231,152,46,252,69,250,89,34,210,27,53,118,140,238,60,64,
249,124,184,224,14,147,197,98,193,109,179,102,162,190,190,30,146,223,15,191,36,65,242,251,241,230,107,175,163,172,236,48,118,252,239,51,156,127,254,96,220,57,119,30,36,201,143,61,187,247,96,207,238,221,
216,241,191,255,1,0,82,82,82,112,180,170,202,116,50,171,170,170,132,51,240,190,99,58,1,126,13,46,112,80,151,65,0,68,136,234,213,147,86,155,13,47,44,93,2,191,95,62,55,204,227,169,198,166,13,27,112,240,
192,1,221,186,90,44,150,192,61,227,244,1,225,212,169,58,124,180,126,189,110,114,189,250,154,107,116,19,49,131,223,153,177,122,229,42,60,246,247,39,228,128,36,0,162,69,84,219,220,102,181,225,165,101,47,
195,239,151,207,245,244,120,60,216,248,209,71,56,176,127,63,4,1,129,0,24,218,95,166,1,48,193,22,83,223,158,141,254,54,59,223,206,88,1,84,198,243,196,27,39,99,242,77,55,171,231,0,238,217,189,11,43,95,127,
67,253,217,176,1,208,176,101,201,97,209,114,70,182,247,14,9,128,202,30,162,199,227,65,65,94,30,126,117,223,189,237,10,128,163,199,142,65,90,122,58,30,248,237,239,144,145,233,194,175,238,147,143,213,223,
57,111,46,22,45,88,128,226,189,69,120,240,209,71,226,122,51,223,26,88,30,119,169,59,98,217,215,238,112,96,198,172,89,184,231,23,119,1,0,158,126,246,153,152,150,45,190,189,106,55,22,45,88,128,172,236,108,
220,48,121,18,242,243,242,112,231,188,185,120,234,31,255,132,199,227,81,15,87,155,181,161,114,94,206,43,203,34,31,190,205,221,188,69,173,170,18,81,252,21,144,220,45,155,225,176,59,2,239,59,207,235,222,
51,68,81,196,167,91,63,65,105,233,33,249,170,193,192,155,126,77,77,141,90,249,120,123,213,42,188,189,106,21,134,93,116,17,46,189,252,50,220,120,243,77,24,156,54,24,171,222,124,203,228,141,95,83,29,19,
67,39,152,179,21,252,148,171,35,245,193,96,165,217,98,67,128,0,155,213,134,217,183,221,102,58,145,106,175,130,22,45,150,144,243,182,4,65,64,75,115,75,32,68,4,87,52,195,229,130,104,17,25,252,206,48,249,
20,128,5,184,115,222,92,188,178,108,153,174,221,173,54,43,102,254,120,102,200,233,15,106,64,12,28,2,214,118,178,246,28,64,99,5,48,82,159,182,167,191,61,158,99,200,112,101,154,126,111,112,122,26,10,218,
112,219,49,171,197,162,171,0,202,65,78,196,250,181,57,120,255,189,247,116,213,194,104,235,110,118,122,131,124,132,193,122,70,182,247,14,9,128,74,40,90,241,214,155,186,55,142,112,213,37,37,188,185,75,75,
77,207,67,115,58,83,213,178,109,241,222,34,117,111,88,91,206,141,245,92,58,187,195,129,172,236,108,100,101,103,107,190,102,15,27,0,211,210,211,224,118,187,213,67,216,110,183,59,166,101,139,71,117,224,
13,181,193,171,191,12,253,78,205,13,142,139,77,86,207,233,116,226,87,247,221,139,117,107,215,182,187,194,74,68,145,183,81,101,59,111,248,103,3,238,156,55,15,69,69,69,186,83,91,220,238,82,148,20,21,155,
86,199,126,116,219,79,240,233,214,173,40,115,151,97,255,190,125,216,191,111,31,214,190,255,1,254,253,204,127,176,250,173,149,168,168,168,192,248,239,125,23,57,31,124,16,242,251,89,217,217,120,111,205,
154,175,69,224,249,193,244,105,234,206,115,76,19,138,205,26,246,4,117,237,33,193,112,135,128,205,170,36,162,69,12,156,23,200,224,119,54,118,132,60,30,79,72,49,193,172,255,180,253,93,85,89,137,193,105,
131,113,184,172,76,55,183,26,79,121,8,134,69,116,202,206,78,126,94,62,102,204,154,133,117,134,60,146,150,158,14,151,203,133,69,11,158,87,179,72,90,154,254,112,113,184,12,35,90,12,59,104,18,2,231,0,74,
97,183,217,242,242,114,12,25,58,36,228,235,105,67,134,96,219,39,159,154,135,197,51,48,220,59,36,0,142,204,206,214,93,92,161,28,66,53,94,197,162,136,118,62,92,67,67,131,90,85,116,58,157,234,155,174,199,
227,129,211,233,132,39,112,41,119,181,97,207,212,110,242,168,54,229,112,180,118,89,254,253,220,179,33,193,205,110,183,203,111,252,199,60,234,97,95,0,186,127,71,90,182,14,217,219,122,110,129,186,142,102,
225,239,15,143,62,162,30,50,86,39,161,210,82,100,100,202,87,44,102,100,186,176,238,131,181,97,255,70,60,63,75,68,193,73,176,32,47,15,51,102,205,10,251,158,102,172,134,157,110,106,194,15,166,77,195,211,
79,62,165,222,87,204,233,116,162,182,182,22,0,176,123,231,78,252,100,198,109,184,118,212,40,124,186,117,171,250,171,215,141,30,141,158,189,122,97,215,206,157,103,37,240,104,39,250,140,76,23,236,118,187,
58,73,154,253,140,162,168,104,47,138,247,22,133,156,27,165,163,185,191,154,69,12,173,128,106,171,36,218,0,16,190,90,74,103,194,162,5,207,227,233,103,159,9,41,134,68,170,212,253,111,219,118,220,113,231,
157,248,112,253,58,84,123,170,49,56,109,48,110,158,50,5,79,60,246,183,144,223,139,86,1,108,239,188,250,202,178,101,234,81,51,101,126,157,49,107,22,94,89,182,76,157,191,115,55,111,193,131,143,60,172,238,
216,57,157,78,220,48,105,146,105,161,69,16,12,225,84,80,78,105,8,92,157,171,25,231,138,70,175,23,7,15,28,192,93,191,188,7,159,110,253,4,94,175,23,87,142,28,137,161,67,135,98,217,75,75,66,43,128,226,153,
25,243,150,254,169,169,143,198,251,75,174,204,204,64,149,170,8,89,217,217,184,112,216,48,252,247,221,119,213,239,123,60,213,248,233,29,63,195,198,13,27,112,243,148,239,195,149,153,137,209,99,199,98,244,
216,177,129,32,226,142,248,154,21,21,21,184,121,202,247,49,112,208,32,220,52,229,251,120,123,213,42,184,75,221,240,122,27,48,99,246,44,12,28,52,8,223,190,230,26,28,216,127,0,249,121,249,248,209,79,126,
140,164,228,36,184,50,93,184,112,216,48,188,189,114,149,250,186,119,206,155,135,77,31,109,64,133,230,166,185,206,84,39,6,14,26,4,8,114,120,29,118,209,48,164,165,167,163,32,47,15,238,82,55,210,210,211,
49,122,236,24,124,251,218,107,48,104,224,64,108,223,182,13,174,204,76,20,23,21,97,207,238,221,166,203,102,124,131,52,174,115,90,122,58,138,139,138,224,116,166,170,235,233,76,117,194,149,153,137,220,205,
91,80,91,123,18,51,102,207,66,82,82,50,110,155,61,11,91,55,111,129,215,235,85,95,115,198,236,217,234,114,43,175,155,187,121,11,106,79,214,226,174,123,238,193,183,175,189,6,54,155,77,119,104,88,219,166,
0,34,254,44,17,153,111,55,0,112,96,255,1,252,232,214,159,192,237,46,69,181,199,131,209,99,199,34,43,59,27,223,190,246,90,117,123,28,61,118,44,210,210,211,177,103,215,110,148,149,149,97,210,141,147,49,
254,123,223,67,239,228,36,100,101,103,227,182,89,51,177,234,205,183,112,184,172,12,130,32,96,223,151,95,98,254,61,119,99,208,121,131,144,158,62,4,163,199,142,197,196,201,147,240,207,39,158,80,131,226,
153,100,124,114,70,65,94,30,94,89,182,12,45,45,45,97,127,38,248,158,239,65,181,199,3,1,130,174,221,140,193,88,123,131,219,154,154,26,212,84,87,235,190,214,216,216,40,87,141,12,247,244,171,174,174,14,217,
225,167,142,237,123,165,223,140,125,216,210,210,130,3,7,246,163,218,227,81,231,186,136,253,12,224,171,131,7,225,107,109,197,200,236,108,92,122,249,101,176,217,18,240,202,203,203,212,177,175,253,16,69,
49,228,254,152,29,201,93,234,198,129,3,251,145,149,125,21,178,174,202,134,221,238,192,27,175,191,134,130,188,124,245,103,106,79,158,196,129,3,251,113,217,229,151,35,235,170,108,164,56,157,88,191,54,7,
185,91,204,171,223,101,165,110,221,252,44,8,2,106,148,49,106,24,231,202,199,222,207,191,128,35,49,17,163,199,200,231,23,215,213,213,225,249,103,159,211,189,78,96,51,129,215,219,96,154,147,58,188,223,47,
27,49,34,238,27,201,40,21,42,109,181,202,120,158,136,114,238,72,90,122,154,238,235,213,199,60,166,183,97,49,123,157,140,76,151,238,231,167,78,159,142,220,205,155,97,119,56,48,122,236,24,120,142,121,176,
110,237,90,216,29,14,249,208,109,169,27,105,233,105,186,50,174,82,241,50,86,10,157,78,167,90,17,171,62,230,129,221,225,8,222,9,60,176,252,222,134,6,245,235,74,229,81,249,190,113,217,180,180,39,143,43,
235,172,188,142,35,80,105,244,120,60,186,229,80,218,32,37,213,41,135,93,67,101,49,45,61,29,118,135,61,164,50,161,253,61,227,122,154,181,105,184,159,37,162,240,219,141,241,235,102,219,163,92,113,240,234,
14,27,101,101,103,171,71,12,114,55,111,14,121,77,249,20,149,44,56,157,169,240,120,142,33,63,47,191,195,142,42,16,17,117,120,0,60,91,126,117,223,189,114,160,242,120,144,225,114,225,47,127,252,19,223,44,
137,136,136,136,190,201,1,16,8,86,194,88,193,34,34,34,34,106,27,235,185,182,192,188,111,29,17,17,17,81,59,3,32,159,153,72,68,68,68,212,181,136,108,2,34,34,34,34,6,64,34,34,34,34,98,0,36,34,34,34,34,6,
64,34,34,34,34,98,0,36,34,34,34,162,175,191,255,63,0,254,190,5,13,235,69,177,169,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00128_png = (const char*) resource_TalComponent_bmp00128_png;
const int TalComponent::bmp00128_pngSize = 22611;

// JUCER_RESOURCE: bmp00135_png, 53534, "E:/Code/tal/tal-filter-2-juce/resources/bmp00135.png"
static const unsigned char resource_TalComponent_bmp00135_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,37,0,0,7,58,8,6,0,0,0,242,185,174,195,0,0,0,9,112,72,89,115,0,0,14,195,0,0,
14,195,1,199,111,168,100,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,
82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,
188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,
34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,
0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,
0,8,12,0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,
46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,
179,26,128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,
95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,
142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,
225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,
100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,
8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,
168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,
24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,
82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,
103,40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,
243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,
188,198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,
40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,
206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,
12,88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,
38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,
166,89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,
125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,
51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,198,73,73,68,65,84,120,218,236,221,251,87,91,231,157,47,254,189,181,165,173,11,18,66,8,132,16,2,33,4,24,99,108,16,66,32,132,16,8,
113,7,115,7,113,191,139,139,140,1,27,99,130,141,49,33,177,137,19,55,113,210,201,164,105,166,105,154,38,105,156,139,227,118,205,156,63,98,206,89,107,126,152,94,214,153,181,206,233,180,171,147,52,205,197,
109,218,243,93,77,115,194,251,251,67,216,79,246,150,192,230,34,201,233,89,249,129,149,222,226,60,149,196,214,179,95,251,253,188,63,212,79,127,250,83,234,32,63,20,69,209,20,69,169,41,138,50,81,20,117,146,
162,168,18,138,162,202,69,34,145,151,162,168,242,157,127,127,114,231,191,87,83,20,69,31,248,159,113,128,197,200,41,138,178,80,20,85,77,81,212,50,69,81,175,81,20,245,175,20,69,189,175,86,171,255,22,23,
23,7,133,66,241,55,138,162,222,223,249,207,95,219,249,223,85,239,252,125,242,176,45,138,162,40,150,162,168,44,138,162,70,41,138,122,131,162,168,15,41,138,250,146,162,40,196,199,199,163,176,176,16,19,19,
19,24,24,24,64,93,93,29,140,70,35,20,10,5,40,138,194,206,255,238,195,157,191,111,116,231,207,97,143,180,40,138,162,180,20,69,181,81,20,245,99,138,162,62,162,40,10,52,77,67,161,80,160,172,172,12,131,131,
131,184,124,249,50,166,167,167,209,211,211,131,230,230,102,212,215,215,195,110,183,195,108,54,131,101,89,208,52,205,45,240,163,157,63,167,141,162,40,237,161,22,69,81,148,145,162,168,243,20,69,253,130,
166,233,109,133,66,129,184,184,56,120,60,30,140,142,142,98,101,101,5,129,64,0,131,131,131,104,107,107,67,107,107,43,90,90,90,208,216,216,136,186,186,58,212,212,212,160,184,184,24,199,143,31,135,66,161,
128,84,42,5,77,211,219,20,69,253,98,231,207,53,30,104,81,59,31,210,77,177,88,252,59,163,209,8,155,205,134,193,193,65,204,205,205,97,113,113,17,211,211,211,24,24,24,64,123,123,187,96,65,45,45,45,56,125,
250,52,154,155,155,81,87,87,135,186,186,58,212,214,214,162,166,166,6,229,229,229,72,79,79,135,70,163,129,72,36,250,29,69,81,155,20,69,153,246,181,168,157,87,104,211,98,177,252,159,238,238,110,44,44,44,
96,121,121,25,129,64,0,67,67,67,232,233,233,65,123,123,251,125,23,196,253,52,53,53,161,177,177,17,13,13,13,168,171,171,67,99,99,35,60,30,15,114,114,114,160,80,40,254,207,206,194,140,247,93,212,206,103,
232,188,68,34,249,93,103,103,39,46,93,186,132,137,137,9,248,124,62,116,116,116,144,197,180,183,183,163,181,181,21,61,61,62,204,204,4,112,102,110,30,51,179,115,24,29,157,128,207,215,143,214,214,118,178,
32,110,81,245,245,245,168,169,169,129,215,235,133,195,225,128,66,161,0,77,211,191,219,121,43,181,187,46,106,231,183,172,141,162,168,95,164,165,165,225,218,181,107,152,154,154,66,103,103,103,200,130,198,
198,198,241,189,23,95,194,139,223,255,33,158,186,121,11,87,55,174,225,226,202,26,230,231,151,224,159,58,131,225,225,9,116,117,247,161,189,189,19,141,141,141,168,175,175,39,111,103,93,93,29,220,110,55,
148,74,37,247,75,240,11,154,166,219,196,98,49,187,219,162,178,40,138,250,49,77,211,219,167,78,157,194,245,235,215,67,94,161,190,190,62,188,243,206,59,184,123,247,46,238,220,185,139,31,190,242,58,158,126,
250,31,176,241,232,117,92,92,89,195,220,217,243,152,152,156,197,224,224,24,186,187,7,208,222,222,131,230,211,29,104,104,104,20,44,170,186,186,26,38,147,137,251,173,220,166,105,250,199,52,77,103,9,22,181,
115,97,28,165,40,234,35,165,82,137,169,169,41,44,45,45,9,22,52,59,59,139,215,95,127,29,119,239,222,221,249,249,41,94,127,227,109,60,251,220,11,216,220,220,194,197,149,53,156,153,91,196,216,248,52,250,
250,135,209,217,217,135,150,150,78,52,55,183,163,177,177,5,117,117,13,130,15,254,201,147,39,193,93,98,40,138,250,72,44,22,143,42,20,10,57,127,81,150,157,11,28,180,90,45,214,214,214,48,58,58,42,120,133,
132,11,250,234,231,206,123,63,197,11,223,251,1,30,191,118,3,23,87,214,16,8,44,96,120,100,18,221,61,131,104,107,239,70,243,233,14,52,54,181,161,177,177,21,141,141,45,104,104,104,68,77,77,13,106,106,106,
80,86,86,134,152,152,24,238,213,2,77,211,111,48,12,99,217,89,15,69,239,124,21,124,72,211,52,202,203,203,177,190,190,14,159,207,71,22,197,189,101,187,253,188,252,195,215,176,181,117,19,23,87,214,48,51,
59,143,193,193,49,116,118,246,225,116,75,231,215,11,106,106,67,115,115,59,154,79,119,144,183,207,227,241,192,104,52,146,69,81,20,245,97,108,108,108,181,193,96,160,169,157,47,205,101,138,162,190,84,42,
149,56,123,246,44,230,230,230,200,130,166,167,103,246,92,208,221,187,119,241,230,155,111,227,230,205,103,113,113,101,13,126,127,0,62,223,16,90,219,186,209,212,220,254,245,130,78,119,160,165,165,19,237,
237,61,232,232,236,132,215,235,133,215,235,133,205,102,227,46,170,160,40,234,75,133,66,177,172,86,171,213,220,133,242,53,138,162,80,88,88,136,107,215,174,97,112,112,144,252,218,127,239,197,151,112,231,
206,87,159,161,221,23,246,83,60,247,220,247,112,113,101,13,99,227,211,232,236,236,219,249,28,181,162,169,185,29,167,91,58,209,214,222,141,206,206,62,116,119,15,160,179,179,23,94,175,23,85,85,85,40,47,
47,135,193,96,128,72,36,226,222,194,215,104,154,54,81,59,219,140,127,77,76,76,196,220,220,28,230,231,231,201,133,177,167,199,135,23,191,255,67,252,240,149,215,241,250,27,111,227,206,123,63,221,243,45,
188,184,178,134,193,193,49,180,180,116,162,177,177,21,205,205,237,104,109,219,89,76,207,32,250,250,135,49,56,56,134,225,225,9,156,62,221,130,170,170,42,120,60,30,20,20,20,240,191,35,255,117,103,61,84,
9,69,81,239,231,231,231,99,107,107,75,240,93,54,51,19,192,83,55,111,225,233,167,255,1,207,62,247,2,94,248,222,15,240,242,15,95,195,155,111,190,45,120,229,110,223,126,27,23,87,214,208,217,213,143,198,166,
54,180,180,116,162,179,179,15,62,223,208,87,11,25,153,196,216,248,52,38,38,103,225,159,58,131,142,142,46,120,60,30,120,60,30,148,149,149,33,37,37,133,91,212,251,59,235,161,202,53,26,205,223,206,156,57,
131,133,133,5,193,87,199,153,185,121,92,221,184,134,141,71,175,99,115,115,11,143,95,187,129,173,173,155,184,121,243,89,60,247,220,247,240,242,15,95,195,237,219,111,227,238,221,187,184,184,178,134,211,
45,157,232,236,234,199,224,224,24,198,198,167,225,247,7,48,51,59,143,64,96,1,103,230,22,49,119,246,60,230,231,151,48,56,56,2,143,199,131,202,202,74,84,84,84,32,39,39,135,123,251,254,70,81,84,57,37,18,
137,188,137,137,137,152,156,156,196,240,240,176,224,187,108,102,118,14,23,87,214,246,253,179,178,207,31,191,127,134,44,200,237,118,227,212,169,83,252,75,131,151,162,40,170,92,173,86,255,45,47,47,15,19,
19,19,130,47,215,209,209,9,204,207,47,97,238,236,121,156,153,91,68,32,176,128,153,217,121,248,253,1,140,141,79,127,245,235,223,213,143,150,150,78,172,172,172,145,183,242,229,31,190,134,231,158,251,30,
110,222,124,22,91,91,55,241,248,181,27,216,220,220,194,198,163,215,113,117,227,26,252,83,179,100,65,110,183,155,127,105,248,219,206,150,250,171,207,148,201,100,66,123,123,59,58,59,59,201,183,189,207,215,
15,255,212,25,76,76,206,98,108,124,26,195,35,147,24,28,28,131,207,55,68,174,216,141,77,109,232,236,234,199,202,202,26,121,43,185,223,202,55,223,252,106,129,47,124,239,7,120,246,185,23,240,244,211,255,
128,167,110,222,194,200,232,56,89,80,97,97,33,196,98,49,183,40,242,153,58,73,81,212,191,106,52,26,228,230,230,162,191,191,159,108,61,90,91,219,49,60,60,129,193,193,49,244,245,15,163,187,103,16,157,157,
125,104,109,235,254,234,215,126,231,67,61,56,56,134,149,149,53,188,252,195,215,118,253,237,188,243,222,87,95,73,63,124,229,117,188,248,253,31,162,189,189,3,110,183,27,46,151,139,255,29,40,248,237,35,215,
169,244,244,116,180,181,181,161,189,189,157,236,135,186,186,185,235,75,31,218,218,187,113,186,165,243,171,11,227,206,85,186,179,179,15,99,227,211,88,89,89,195,115,207,125,239,190,215,179,59,119,238,226,
249,231,191,7,183,219,141,242,242,114,20,22,22,242,183,203,96,24,230,53,177,88,108,18,92,209,21,10,5,172,86,43,124,62,31,217,15,181,182,181,163,189,189,231,171,47,87,238,187,172,169,13,77,59,215,33,159,
111,8,126,127,0,43,43,107,184,121,243,217,157,203,197,222,223,0,51,51,51,40,47,47,223,237,85,250,50,57,57,121,217,98,177,168,67,190,251,50,50,50,48,56,56,136,211,167,79,147,77,90,111,223,16,230,231,207,
97,110,110,145,44,234,244,206,181,104,112,112,12,51,179,243,88,89,89,195,214,214,205,61,223,194,187,119,239,226,237,183,223,134,199,227,65,121,121,57,138,139,139,33,149,74,5,223,125,153,153,153,213,181,
181,181,116,200,46,65,163,209,160,181,181,21,157,157,157,104,104,104,64,111,111,47,54,54,54,112,247,238,93,188,250,234,143,49,55,183,136,230,211,29,104,107,239,70,119,207,32,134,71,38,17,8,44,96,101,101,
13,143,95,187,129,23,190,247,131,93,175,252,63,254,241,143,209,212,212,68,94,37,110,235,194,253,200,100,178,55,84,42,149,101,215,253,148,66,161,192,241,227,199,49,50,50,34,88,16,247,51,63,127,142,92,177,
251,250,135,49,54,62,141,51,115,139,88,89,89,195,230,230,22,158,125,238,5,188,254,134,240,138,255,227,31,255,24,3,3,3,100,65,46,151,11,201,201,201,252,69,125,148,144,144,48,122,242,228,73,249,158,59,79,
179,217,140,241,241,113,172,174,174,10,22,180,177,177,129,222,190,33,180,183,247,160,187,123,0,131,131,99,152,152,156,197,220,217,243,88,89,89,195,198,163,215,241,244,211,255,128,31,190,242,250,206,151,
248,87,111,25,255,21,114,185,92,40,46,46,6,203,178,100,231,25,27,27,251,99,157,78,151,117,223,61,186,94,175,71,125,125,61,158,121,230,25,178,160,171,87,175,162,167,167,7,117,117,117,240,249,122,209,217,
217,139,225,225,9,248,167,206,96,126,126,9,43,43,107,184,186,113,13,79,221,188,133,23,191,255,67,60,255,252,247,48,51,51,67,62,67,220,130,202,202,202,200,215,10,69,81,16,139,197,191,72,77,77,109,115,185,
92,236,125,239,102,196,98,241,239,82,83,83,49,58,58,138,39,159,124,18,171,171,171,232,233,233,65,77,77,13,170,171,171,201,126,232,244,233,22,116,116,116,97,112,112,4,126,255,12,252,83,179,24,25,29,39,
215,161,242,242,242,144,87,40,43,43,11,74,165,146,91,212,239,36,18,201,121,141,70,163,221,215,125,95,82,82,210,255,209,104,52,208,235,245,48,155,205,176,90,173,112,185,92,240,120,60,100,63,196,109,63,
248,95,174,220,149,154,91,140,221,110,71,110,110,46,146,147,147,161,80,40,200,222,137,162,168,253,221,247,5,223,33,139,68,162,223,197,196,196,64,165,82,65,161,80,64,46,151,195,108,54,195,102,179,193,235,
245,238,185,32,151,203,5,171,213,10,163,209,136,152,152,24,208,52,45,248,161,40,234,96,119,200,123,89,2,203,178,144,74,165,136,141,141,133,82,169,132,201,100,66,97,97,33,170,171,171,5,175,78,97,97,33,
146,147,147,201,54,55,104,49,219,98,177,248,23,10,133,226,224,150,192,255,140,137,68,162,54,154,166,5,234,34,145,72,160,86,171,161,82,169,144,146,146,2,155,205,134,252,252,124,164,166,166,242,247,220,
130,175,16,138,162,62,138,141,141,253,113,78,78,78,91,240,103,232,192,62,37,22,139,89,154,166,179,88,150,29,165,105,90,224,83,114,185,28,177,177,177,144,201,100,100,75,203,123,85,136,79,201,100,178,55,
210,210,210,70,117,58,93,22,255,183,236,200,146,167,80,40,228,12,195,88,226,227,227,171,21,10,197,50,77,211,68,242,164,82,233,223,36,18,9,68,34,17,145,60,134,97,94,75,78,78,94,182,90,173,213,42,149,202,
194,93,24,195,202,139,220,143,193,96,160,213,106,181,122,231,174,131,152,231,206,142,145,152,167,88,44,54,89,44,22,117,109,109,109,228,204,243,65,16,75,81,84,197,55,10,98,105,154,254,124,231,203,252,11,
165,82,249,233,67,133,88,14,99,211,211,211,225,243,249,208,223,223,143,154,154,26,100,100,100,112,223,109,17,135,216,143,41,138,2,203,178,144,203,229,72,77,77,133,215,235,197,196,196,4,22,22,22,48,55,
55,135,254,254,126,180,182,182,194,235,245,34,47,47,15,58,157,78,112,73,8,27,196,74,36,146,255,169,82,169,182,181,90,45,74,74,74,80,93,93,141,233,233,105,76,79,79,99,113,113,17,147,147,147,232,237,237,
69,123,123,59,185,19,170,175,175,71,117,117,53,92,46,23,44,22,11,255,187,238,203,35,67,172,201,100,250,99,115,115,51,206,156,57,131,153,153,25,156,59,119,14,129,64,0,227,227,227,240,249,124,232,238,238,
222,211,61,155,154,154,4,156,152,153,153,201,95,220,111,14,5,177,12,195,124,208,214,214,134,141,141,13,76,78,78,162,175,175,79,128,176,251,133,88,142,126,106,107,107,81,90,90,138,227,199,143,67,163,209,
128,166,233,15,14,4,177,20,69,253,38,61,61,29,155,155,155,152,158,158,222,213,61,15,10,177,220,171,198,129,153,197,98,129,92,46,255,227,190,33,150,162,40,212,215,215,99,99,99,99,87,25,62,10,196,114,139,
227,217,212,255,124,32,196,82,20,245,165,193,96,192,218,218,26,2,129,64,68,32,182,182,182,22,197,197,197,220,247,228,131,33,150,162,40,184,221,110,60,246,216,99,232,235,235,139,24,196,186,221,110,254,
131,165,143,239,11,177,41,41,41,120,228,145,71,16,8,4,34,10,177,85,85,85,130,235,216,158,16,75,81,20,26,27,27,177,177,177,129,254,254,254,136,66,172,215,235,229,187,212,222,16,107,177,88,112,237,218,53,
248,253,254,168,64,172,213,106,229,47,42,20,98,149,74,37,198,198,198,176,188,188,140,206,206,206,168,64,172,203,229,226,127,174,66,33,214,233,116,98,107,107,11,35,35,35,81,131,216,138,138,10,164,167,167,
239,74,65,37,9,9,9,159,14,13,13,9,100,56,90,16,155,159,159,207,223,66,127,13,177,58,157,238,139,182,182,54,76,78,78,70,29,98,157,78,39,226,227,227,185,87,234,115,142,23,43,40,138,130,201,100,194,248,248,
56,218,218,218,162,14,177,102,179,153,255,22,86,80,59,251,235,207,141,70,35,122,123,123,209,217,217,25,117,136,229,217,194,23,2,136,213,106,181,40,45,45,197,240,240,112,84,33,182,188,188,156,47,122,159,
10,32,150,162,40,162,120,45,45,45,81,129,216,242,242,114,242,65,223,19,98,141,70,35,6,7,7,209,209,209,17,21,136,117,58,157,130,175,154,61,33,214,227,241,96,104,104,136,236,135,218,219,59,49,60,50,129,
177,241,41,180,183,119,135,13,98,203,203,203,113,236,216,177,7,67,44,203,178,200,204,204,196,228,228,36,26,27,27,209,214,214,134,217,217,89,188,244,210,75,120,245,199,63,198,218,218,122,216,32,182,168,
168,72,112,53,191,47,196,90,44,22,76,77,77,161,167,167,7,231,206,157,195,27,111,188,65,254,208,167,159,190,133,211,59,116,125,84,136,205,200,200,216,55,196,126,156,152,152,136,241,241,113,92,188,120,81,
240,135,190,244,210,75,24,27,243,227,116,75,231,145,33,214,110,183,67,38,147,237,31,98,99,99,99,183,235,234,234,240,196,19,79,144,63,244,214,173,91,24,27,27,67,99,99,19,154,79,119,28,25,98,51,51,51,5,
159,165,7,66,172,68,34,249,159,38,147,9,253,253,253,184,113,227,6,54,54,54,48,56,56,72,118,140,71,133,216,226,226,98,254,173,214,254,33,54,33,33,225,143,90,173,22,122,189,30,249,249,249,112,185,92,36,
22,114,20,136,45,43,43,11,254,141,251,205,129,32,150,166,233,15,98,99,99,17,23,23,7,149,74,133,204,204,76,148,150,150,162,182,182,118,223,16,203,45,168,184,184,24,249,249,249,176,88,44,252,207,210,7,135,
130,216,157,59,89,196,196,196,144,197,229,228,228,16,33,222,107,65,14,135,3,167,78,157,66,86,86,22,180,90,45,20,10,5,24,134,225,95,185,255,120,100,136,229,148,69,169,84,130,99,236,130,130,2,84,87,87,147,
5,113,34,108,54,155,17,27,27,11,137,68,2,154,166,193,48,12,24,134,129,72,36,2,77,211,219,98,177,248,127,134,29,98,41,138,66,92,92,28,226,226,226,96,177,88,144,159,159,143,227,199,143,67,167,211,145,133,
112,139,9,242,207,143,35,14,177,44,203,66,163,209,64,38,147,65,44,22,131,166,105,136,68,34,62,222,63,60,136,149,74,165,159,138,197,226,47,120,59,199,111,14,196,6,155,231,183,16,187,27,196,82,20,245,215,
157,183,239,175,34,145,232,247,10,133,226,23,98,177,248,221,135,6,177,12,195,64,161,80,32,45,45,13,54,155,13,221,221,221,240,122,189,220,205,64,228,32,150,97,152,55,105,154,254,56,38,38,6,82,169,20,73,
73,73,72,79,79,135,205,102,67,109,109,45,206,158,61,139,241,241,113,44,44,44,96,116,116,20,157,157,157,240,120,60,200,204,204,228,239,153,194,7,177,9,9,9,191,117,58,157,219,29,29,29,232,238,238,70,32,
16,192,200,200,8,22,23,23,49,49,49,129,64,32,128,225,225,97,116,119,119,11,238,132,234,234,234,80,94,94,142,188,188,60,168,213,106,238,82,113,116,136,165,40,234,55,165,165,165,184,126,253,58,102,102,102,
48,54,54,134,129,129,1,244,244,244,160,187,187,27,237,237,237,232,232,232,216,211,61,235,234,234,80,81,81,129,226,226,98,36,37,37,241,115,82,135,131,88,138,162,222,55,26,141,88,95,95,199,217,179,103,119,
117,207,253,38,98,185,133,229,231,231,67,173,86,243,115,45,7,131,88,138,162,208,209,209,129,141,141,13,244,245,245,29,9,98,185,109,79,69,69,5,138,138,138,112,252,248,113,110,79,245,155,125,67,172,72,36,
66,122,122,58,214,215,215,49,53,53,21,54,136,229,16,214,227,241,192,102,179,65,175,215,131,101,217,223,238,11,98,149,74,37,166,167,167,177,186,186,42,136,86,134,51,17,91,81,81,129,99,199,142,113,223,155,
111,222,23,98,37,18,9,220,110,55,46,95,190,140,225,225,225,136,65,108,77,77,13,178,179,179,247,7,177,233,233,233,184,126,253,58,206,156,57,131,174,174,174,136,65,108,117,117,53,10,11,11,9,47,238,9,177,
6,131,1,11,11,11,88,90,90,34,11,138,36,196,218,237,246,7,39,98,251,251,251,177,190,190,78,194,167,145,134,88,151,203,5,134,97,200,222,75,0,177,50,153,236,221,162,162,34,60,249,228,147,24,31,31,23,252,
218,71,58,17,203,191,221,18,64,172,90,173,254,197,233,211,167,113,238,220,57,193,133,49,26,137,88,173,86,187,59,196,50,12,243,251,130,130,2,18,100,142,22,196,86,86,86,10,18,66,2,136,165,40,234,175,22,
139,5,19,19,19,130,239,178,104,64,108,126,126,62,127,81,127,21,64,108,90,90,26,252,126,63,186,186,186,162,10,177,78,167,147,31,240,250,26,98,41,138,250,171,66,161,64,125,125,61,134,134,134,162,10,177,
46,151,11,122,189,62,228,149,42,161,40,234,125,134,97,144,157,157,141,225,225,97,98,158,15,1,98,127,31,2,177,89,89,89,88,92,92,36,91,144,104,64,172,203,229,130,193,96,224,22,245,139,93,33,150,123,206,
199,237,135,6,6,71,48,56,56,142,193,193,177,176,67,44,183,51,229,237,247,223,13,129,88,131,193,128,233,233,105,116,116,116,160,169,169,9,195,195,195,216,120,116,19,239,188,243,14,174,95,127,2,29,157,190,
176,66,172,213,106,229,159,8,217,29,98,101,50,25,250,251,251,49,52,52,132,217,217,89,60,255,252,243,228,15,123,245,213,87,49,50,58,17,54,136,13,230,234,61,33,86,165,82,193,235,245,98,121,121,25,111,191,
45,252,127,252,248,227,215,209,222,222,67,190,203,142,2,177,78,167,51,56,55,188,55,196,210,52,253,241,137,19,39,4,41,216,151,95,126,25,11,11,11,232,233,241,161,177,177,5,205,205,237,71,134,216,252,252,
252,96,8,217,27,98,25,134,121,211,100,50,161,166,166,6,107,107,107,120,236,177,199,48,58,58,74,146,24,13,13,141,71,134,216,146,146,18,104,52,154,131,65,172,74,165,250,109,98,98,34,180,90,45,236,118,59,
42,42,42,208,208,208,16,54,136,13,250,174,219,63,196,82,20,245,155,216,216,88,104,52,26,112,79,183,60,30,207,145,33,214,110,183,11,206,95,29,24,98,119,190,177,161,86,171,161,86,171,97,50,153,208,208,208,
64,22,116,16,136,229,100,56,232,97,227,251,135,134,88,154,166,193,178,44,56,194,46,41,41,65,109,109,237,158,11,114,185,92,40,45,45,37,24,123,234,212,41,152,76,38,36,39,39,243,119,153,191,9,11,196,50,12,
131,184,184,56,168,213,106,88,173,86,114,255,198,125,211,23,20,20,224,248,241,227,72,72,72,128,90,173,134,76,38,11,193,216,157,171,246,111,195,14,177,42,149,10,90,173,22,22,139,5,57,57,57,48,153,76,80,
169,84,196,9,24,134,129,68,34,129,88,44,38,24,187,243,235,255,177,74,165,122,51,162,16,203,33,44,247,42,4,57,249,195,129,88,150,101,223,149,72,36,191,160,105,250,247,124,201,251,22,98,255,174,33,150,95,
77,32,22,139,215,35,14,177,52,77,223,166,40,234,67,145,72,180,205,33,236,206,55,60,184,147,185,122,189,30,39,78,156,224,168,103,155,97,152,143,35,2,177,44,203,190,101,177,88,254,79,102,102,38,108,54,27,
78,157,58,5,135,195,1,187,221,142,234,234,106,212,212,212,96,104,104,8,195,195,195,152,156,156,196,204,204,12,250,250,250,80,85,85,5,147,201,196,45,62,188,213,4,122,189,126,123,97,97,1,139,139,139,24,
31,31,135,223,239,199,200,200,8,198,199,199,49,50,50,130,209,209,81,12,13,13,9,2,169,156,121,218,237,118,226,157,97,131,88,173,86,139,197,197,69,172,173,173,161,191,191,31,29,29,29,2,217,123,144,123,86,
85,85,193,102,179,241,95,177,35,65,236,127,209,52,141,134,134,6,92,191,126,29,67,67,67,135,134,216,154,154,26,20,21,21,193,100,50,145,211,33,20,69,253,215,129,33,150,97,24,100,102,102,98,109,109,13,51,
51,51,71,78,196,86,85,85,193,233,116,226,228,201,147,193,25,226,253,39,98,83,82,82,112,249,242,101,44,47,47,11,220,243,168,16,235,116,58,145,159,159,15,179,217,204,29,165,251,197,190,32,86,161,80,96,120,
120,24,43,43,43,36,214,29,46,136,173,169,169,129,199,227,33,177,112,169,84,186,205,48,204,91,247,133,88,141,70,131,225,225,97,60,250,232,163,24,27,27,35,111,91,184,33,214,227,241,192,110,183,35,49,49,
17,44,203,254,127,123,66,172,66,161,64,75,75,11,86,86,86,34,94,77,192,157,76,226,110,34,104,154,190,29,2,177,114,185,252,227,150,150,22,18,62,141,6,196,86,85,85,33,46,46,110,119,136,101,89,118,221,98,
177,96,109,109,13,211,211,211,130,15,117,36,33,182,178,178,146,220,37,239,116,199,8,19,177,199,142,29,195,218,218,154,224,55,45,210,16,235,241,120,144,144,144,176,119,34,54,51,51,19,139,139,139,36,162,
27,45,136,229,49,80,104,53,129,193,96,128,223,239,135,207,231,139,106,34,54,168,242,66,8,177,122,189,30,35,35,35,24,24,24,136,42,196,114,207,103,118,190,122,132,16,75,81,20,234,234,234,48,54,54,22,85,
136,61,113,226,68,240,205,134,16,98,29,14,7,252,126,127,84,33,214,102,179,237,74,214,37,220,45,122,94,94,30,230,231,231,163,154,136,45,41,41,225,111,105,132,213,4,156,14,243,23,213,218,218,142,169,169,
51,24,26,158,136,24,196,58,157,78,254,174,33,52,17,155,157,157,141,165,165,37,52,55,55,163,183,183,23,179,179,179,184,117,235,89,188,240,194,247,48,119,118,49,34,16,91,92,92,76,246,248,33,137,88,134,97,
182,51,51,51,113,238,220,57,248,253,126,220,184,113,67,240,135,174,175,111,124,253,213,17,38,136,117,185,92,252,167,163,72,74,74,90,15,129,216,172,172,44,92,185,114,5,111,190,249,166,224,15,122,237,181,
215,17,56,51,79,22,20,46,136,205,202,202,226,51,227,199,33,16,75,211,244,109,147,201,132,115,231,206,225,7,63,248,1,238,222,189,139,183,222,122,11,155,155,155,152,152,152,64,107,219,215,175,82,56,18,177,
121,121,121,130,103,50,123,66,172,193,96,248,255,18,19,19,49,54,54,134,205,205,77,248,253,126,116,116,116,236,108,208,26,194,6,177,185,185,185,193,15,137,246,134,88,177,88,252,86,108,108,236,118,124,124,
60,10,10,10,224,241,120,72,31,89,184,32,214,110,183,35,54,54,246,96,16,75,81,212,47,98,98,98,160,213,106,161,211,233,80,91,91,75,126,142,10,177,37,37,37,72,76,76,60,60,196,114,213,23,90,173,22,86,171,
149,220,42,29,5,98,179,178,178,130,191,82,14,12,177,255,69,211,52,212,106,53,18,18,18,224,116,58,81,85,85,37,216,53,30,4,98,173,86,107,240,231,232,191,142,148,136,101,89,22,9,9,9,56,126,252,56,154,154,
154,238,187,32,110,17,78,167,19,46,151,11,14,135,3,69,69,69,193,159,163,240,64,172,66,161,128,78,167,67,97,97,33,202,203,203,73,10,214,102,179,33,47,47,15,217,217,217,72,73,73,129,193,96,64,98,98,34,18,
19,19,161,209,104,160,209,104,248,105,179,240,87,19,176,44,139,196,196,68,36,36,36,64,175,215,67,169,84,130,101,89,130,174,18,137,4,44,203,134,252,236,120,232,255,167,84,42,223,138,8,196,74,36,146,143,
85,42,21,33,105,177,88,76,10,136,120,53,96,228,102,96,7,98,111,71,28,98,149,74,229,58,191,154,224,91,136,253,127,18,98,163,146,136,85,169,84,255,77,165,82,221,99,89,118,91,169,84,130,97,24,242,212,129,
101,89,72,36,18,80,20,133,216,216,88,72,165,82,168,213,234,109,165,82,249,39,177,88,124,55,34,213,4,12,195,124,226,116,58,137,113,182,180,180,144,86,156,230,230,102,212,212,212,160,177,177,145,132,45,
134,134,134,48,48,48,128,202,202,74,36,37,37,133,31,98,165,82,233,182,199,227,193,141,27,55,16,8,4,8,184,250,124,62,82,222,208,219,219,75,48,182,171,171,11,62,159,143,116,198,158,60,121,18,114,185,28,
59,212,125,116,136,165,40,10,39,78,156,192,250,250,122,200,177,242,253,186,103,89,89,89,120,170,9,184,47,100,163,209,136,229,229,101,44,45,45,29,170,154,128,107,215,117,187,221,72,79,79,39,71,15,14,5,
177,52,77,195,100,50,225,242,229,203,88,89,89,17,156,115,63,12,196,214,213,213,161,176,176,16,169,169,169,252,231,200,7,131,216,132,132,4,204,204,204,96,105,105,73,224,158,71,129,216,170,170,42,148,150,
150,34,59,59,155,223,103,246,96,136,101,89,118,251,196,137,19,216,220,220,196,202,202,10,134,134,134,194,14,177,37,37,37,176,88,44,80,171,213,96,24,230,193,213,4,121,121,121,56,119,238,28,206,159,63,47,
248,12,133,19,98,189,94,47,156,78,39,78,157,58,5,173,86,11,154,166,63,217,21,98,229,114,249,221,162,162,34,60,241,196,19,88,92,92,20,124,134,34,85,77,192,177,245,206,22,231,191,133,64,172,78,167,251,19,
87,139,194,71,216,72,66,172,215,235,37,65,65,154,166,239,133,36,98,19,19,19,183,207,157,59,135,177,177,177,168,65,108,85,85,21,169,39,160,40,42,20,98,21,10,5,6,6,6,4,201,234,72,67,108,208,162,66,33,86,
171,213,162,191,191,159,160,126,180,32,54,55,55,119,111,136,165,40,10,93,93,93,240,251,253,81,133,216,251,38,98,89,150,69,85,85,21,252,126,127,84,33,214,98,177,236,157,136,149,201,100,168,173,173,197,
153,51,103,162,10,177,65,139,18,66,172,88,44,70,109,109,45,2,129,64,84,33,54,232,237,19,66,108,76,76,12,218,219,219,225,247,251,201,183,125,127,255,32,230,230,22,35,10,177,123,125,166,78,82,20,245,175,
18,137,4,189,189,189,152,156,156,68,119,119,55,38,38,38,112,233,210,101,188,248,253,151,176,249,216,117,244,15,140,70,4,98,249,245,190,33,16,27,23,23,135,177,177,49,204,204,204,224,241,199,31,199,143,
127,252,99,242,135,190,242,202,43,152,158,153,139,8,196,242,163,223,33,16,43,147,201,182,103,103,103,241,220,115,207,133,252,129,47,126,255,37,244,248,6,194,218,17,203,103,70,238,138,30,146,136,85,171,
213,247,242,243,243,5,103,218,159,121,230,25,44,46,46,98,112,112,84,176,160,112,64,44,39,51,41,41,41,220,162,254,20,2,177,49,49,49,255,205,104,52,162,170,170,10,103,207,158,37,79,180,234,235,235,209,208,
208,136,198,198,150,176,66,44,151,236,224,65,236,221,93,33,150,97,152,79,244,122,61,140,70,35,188,94,47,9,157,134,27,98,75,74,74,248,193,248,7,87,19,72,36,146,109,173,86,139,248,248,120,1,194,134,11,98,
131,142,211,237,191,35,150,243,206,228,228,100,56,157,78,52,53,53,133,5,98,237,118,59,191,237,230,224,16,75,211,52,180,90,45,18,18,18,208,222,222,78,22,117,88,136,221,109,65,135,134,88,149,74,5,139,197,
2,183,219,141,186,186,186,67,37,98,29,14,71,240,103,232,232,16,155,148,148,4,189,94,143,150,150,150,67,201,240,46,95,186,71,135,88,134,97,182,117,58,29,142,31,63,14,155,205,6,135,195,65,78,255,231,229,
229,33,61,61,29,22,139,5,153,153,153,200,204,204,68,94,94,30,142,31,63,78,202,107,249,31,234,176,67,108,98,98,34,20,10,5,193,85,254,95,165,82,41,249,43,7,178,65,173,0,159,68,164,154,128,101,217,187,50,
153,236,79,18,137,100,91,36,18,17,128,229,162,184,252,158,132,157,191,110,139,68,162,123,50,153,236,191,69,189,35,246,91,136,253,187,131,88,145,72,244,147,111,4,196,82,20,245,134,92,46,255,216,237,118,
111,31,59,118,12,22,139,5,42,149,10,74,165,146,208,14,135,177,59,155,182,109,174,154,64,36,18,77,70,100,88,151,86,171,133,207,231,195,149,43,87,48,50,50,2,159,207,71,224,149,235,236,44,41,41,129,211,233,
36,129,192,162,162,34,110,186,211,103,97,133,88,238,20,119,111,111,47,86,86,86,66,188,161,181,181,21,93,93,93,4,96,251,251,251,201,95,251,251,251,209,212,212,132,172,172,44,176,44,27,62,136,141,139,139,
195,196,196,4,121,242,206,101,172,14,82,201,106,179,217,160,213,106,185,107,219,225,33,86,36,18,129,59,201,118,245,234,85,1,126,28,4,98,27,27,27,81,85,85,133,188,188,60,254,179,191,255,58,84,53,129,201,
100,194,212,212,20,150,151,151,5,204,120,216,97,93,220,112,18,238,148,200,129,32,86,173,86,163,166,166,6,27,27,27,184,112,225,66,72,110,248,176,16,91,91,91,11,171,213,10,189,94,127,48,136,213,233,116,
219,253,253,253,36,201,200,213,90,132,11,98,171,170,170,80,84,84,4,131,193,192,69,148,190,220,19,98,25,134,153,212,235,245,159,157,61,123,22,171,171,171,24,31,31,23,48,99,184,33,182,176,176,16,9,9,9,36,
64,191,103,34,54,39,39,7,155,155,155,24,30,30,14,105,209,13,55,196,86,84,84,32,55,55,23,90,173,150,59,8,180,123,53,65,78,78,14,86,87,87,5,191,242,145,132,88,167,211,137,188,188,60,104,181,90,136,68,162,
143,67,32,86,44,22,111,31,63,126,28,43,43,43,130,87,41,146,16,203,221,233,112,135,238,119,29,214,101,179,217,176,180,180,20,85,136,229,198,230,237,108,22,127,34,128,88,138,162,144,159,159,79,22,21,45,
136,117,187,221,252,45,179,16,98,89,150,133,211,233,196,210,210,82,84,33,54,168,242,66,8,177,20,69,161,165,165,5,243,243,243,223,156,106,130,152,152,24,244,245,245,97,106,106,42,170,16,27,84,230,39,132,
88,173,86,139,139,23,47,98,100,100,132,124,185,250,253,51,56,191,180,18,81,136,77,75,75,219,27,98,227,227,227,177,177,177,129,225,225,97,140,142,142,98,110,110,14,79,220,184,129,151,254,233,7,184,124,
121,61,98,16,27,52,54,72,8,177,153,153,153,184,114,229,10,30,123,236,49,188,242,202,43,130,223,174,231,158,123,30,253,253,35,17,25,214,21,116,190,253,107,136,21,137,68,63,201,204,204,196,147,79,62,25,
146,241,188,123,247,46,54,55,175,193,231,27,140,8,196,242,147,251,33,16,107,177,88,48,60,60,140,103,158,121,6,119,238,220,193,243,207,63,191,211,38,63,135,254,254,225,176,15,235,226,0,132,87,161,18,10,
177,82,169,244,227,228,228,100,184,221,110,76,76,76,160,191,191,159,132,29,154,79,119,144,183,43,156,16,91,86,86,198,87,189,189,171,9,146,147,147,81,85,85,69,238,80,34,5,177,28,53,114,1,176,251,14,235,
146,201,100,208,235,245,240,122,189,2,239,12,39,196,58,157,78,50,146,120,231,85,250,76,171,213,78,238,9,177,220,237,84,66,66,2,202,202,202,4,99,132,195,1,177,165,165,165,176,88,44,252,153,125,251,135,
216,248,248,120,232,116,58,116,117,117,9,206,80,29,5,98,243,243,243,131,187,18,14,7,177,69,69,69,100,206,199,81,32,214,106,181,66,165,82,29,29,98,185,90,149,210,210,82,52,54,54,10,22,116,16,247,204,207,
207,15,110,99,62,26,196,42,149,74,36,39,39,163,178,178,146,236,22,185,197,84,86,86,146,38,128,221,22,228,116,58,145,147,147,19,28,207,13,15,196,170,213,234,109,141,70,131,180,180,52,36,39,39,131,123,84,
98,48,24,144,154,154,138,236,236,108,2,175,220,99,14,174,72,38,104,112,224,151,97,133,88,165,82,249,153,82,169,132,76,38,219,243,135,131,88,181,90,141,248,248,248,144,106,232,176,67,172,76,38,155,228,
170,9,118,146,174,130,20,44,23,211,149,72,36,144,201,100,144,74,165,164,39,225,161,12,235,218,3,98,127,242,45,196,126,99,33,54,38,38,230,103,74,165,242,151,59,179,136,30,46,196,82,20,245,161,92,46,223,
118,187,221,104,110,110,38,125,231,113,113,113,164,179,58,184,131,35,146,195,186,62,226,58,17,166,166,166,176,190,190,142,137,137,9,116,119,119,147,152,156,199,227,33,19,157,116,58,29,18,18,18,16,23,23,
199,137,241,71,98,177,248,109,154,166,59,194,6,177,82,169,20,249,249,249,88,95,95,199,35,143,60,34,168,66,228,170,8,90,90,90,200,164,96,110,122,70,115,115,51,218,219,219,81,85,85,5,189,94,191,205,178,
236,175,143,12,177,52,77,67,163,209,192,227,241,96,101,101,5,243,243,243,2,72,187,159,123,54,53,53,145,255,174,174,174,142,255,29,120,120,136,229,30,64,250,124,62,44,45,45,97,118,118,86,144,104,60,8,196,
214,213,213,161,172,172,12,58,157,142,251,220,29,28,98,181,90,45,90,90,90,176,182,182,134,139,23,47,98,108,108,76,32,123,135,129,216,154,154,26,156,60,121,146,255,5,253,96,136,165,105,186,67,173,86,255,
218,233,116,226,220,185,115,88,91,91,35,129,156,112,13,235,114,187,221,72,76,76,228,255,150,222,31,98,101,50,217,219,86,171,21,151,47,95,38,163,239,194,61,172,171,182,182,22,121,121,121,80,42,149,220,
150,248,203,7,38,98,253,126,63,41,110,224,47,40,156,16,91,94,94,142,172,172,44,196,196,196,112,95,218,123,67,172,76,38,195,204,204,12,105,171,140,20,196,86,87,87,195,233,116,10,166,21,236,9,177,201,201,
201,152,153,153,17,116,37,68,10,98,61,30,15,201,188,236,124,190,118,239,136,53,26,141,184,114,229,74,212,58,98,185,105,79,74,165,114,247,106,2,238,38,97,107,107,139,252,182,69,3,98,203,203,203,97,48,24,
32,18,137,32,145,72,126,22,2,177,121,121,121,88,91,91,139,42,196,122,60,30,210,46,72,211,244,47,5,16,43,22,139,81,93,93,77,116,56,90,16,203,59,194,194,141,230,248,26,98,53,26,13,198,199,199,49,59,59,27,
85,136,117,185,92,252,43,188,16,98,245,122,61,174,95,191,78,166,61,181,181,181,97,126,254,28,86,87,215,35,10,177,252,148,117,8,196,102,101,101,225,218,181,107,152,152,152,192,220,220,28,46,95,190,140,
231,158,251,46,190,247,189,239,227,210,165,245,136,65,236,94,217,225,18,145,72,244,65,117,117,53,174,93,187,134,159,252,228,39,130,15,241,123,239,189,135,39,159,250,78,68,32,182,188,188,60,216,60,191,
134,88,181,90,253,75,171,213,138,167,158,122,42,244,170,125,231,14,150,47,174,70,164,154,192,229,114,65,46,151,239,14,177,10,133,226,103,102,179,25,19,19,19,248,254,247,191,143,91,183,110,97,99,99,3,11,
11,11,152,158,62,131,222,190,225,136,12,235,114,56,28,130,207,83,8,196,74,165,210,109,131,193,128,190,190,62,210,88,210,216,216,136,214,182,118,180,183,247,68,4,98,131,62,228,187,119,196,170,213,106,28,
59,118,236,43,124,109,110,38,155,180,72,65,236,201,147,39,247,55,172,43,37,37,5,121,121,121,100,81,145,132,216,227,199,143,239,107,88,215,71,44,203,194,104,52,194,225,112,160,189,189,61,34,16,203,13,91,
226,253,230,221,63,17,203,65,108,74,74,10,250,250,250,194,158,136,117,185,92,36,2,192,189,74,74,165,242,237,125,39,98,203,203,203,209,214,214,22,54,136,229,78,172,241,169,145,97,152,95,27,141,198,142,
125,87,19,164,167,167,163,181,181,149,156,54,59,44,196,114,204,24,116,232,254,112,16,27,23,23,135,19,39,78,8,134,66,28,20,98,29,14,7,210,211,211,119,115,207,195,67,108,74,74,10,204,102,51,236,118,59,108,
54,27,172,86,43,156,78,167,0,96,239,183,160,160,42,167,240,64,172,66,161,248,53,87,47,192,178,44,100,50,25,20,10,5,148,74,37,244,122,61,44,22,11,242,242,242,80,86,86,70,190,58,184,11,99,80,196,59,124,
16,203,48,76,135,88,44,126,155,162,168,143,68,34,17,164,82,41,49,77,133,66,1,133,66,1,185,92,14,174,79,193,108,54,35,53,53,53,88,133,35,3,177,193,29,177,188,147,102,68,133,185,5,242,134,221,108,63,148,
97,93,193,16,75,211,244,7,98,177,248,151,98,177,248,103,223,66,236,55,186,154,128,101,217,127,51,24,12,127,98,89,246,111,15,29,98,25,134,217,78,76,76,68,117,117,53,105,4,40,46,46,70,92,92,28,201,168,71,
21,98,21,10,5,114,115,115,49,58,58,138,243,231,207,147,220,39,231,157,220,188,15,157,78,23,92,59,254,81,216,19,177,12,195,64,171,213,162,177,177,17,231,206,157,219,57,14,53,136,206,206,78,194,140,156,
218,113,198,233,245,122,225,118,187,73,154,76,42,149,110,179,44,251,191,24,134,89,61,114,34,86,36,18,193,233,116,226,204,153,51,184,116,233,18,252,126,63,241,134,221,220,147,219,177,242,255,51,78,138,
179,178,178,32,149,74,63,57,52,196,106,181,90,212,212,212,96,101,101,5,87,174,92,193,153,51,103,4,135,239,15,2,177,92,37,107,97,97,33,183,117,217,63,196,74,36,146,85,189,94,255,73,123,123,59,86,87,87,
177,186,186,74,102,217,30,53,17,203,153,103,124,124,252,193,18,177,42,149,234,127,117,116,116,8,16,54,92,195,186,184,205,34,127,111,190,175,68,108,108,108,236,246,205,155,55,49,61,61,29,50,95,45,92,195,
186,202,202,202,248,79,77,31,12,177,41,41,41,184,117,235,86,68,33,214,235,245,34,45,45,141,28,107,185,47,196,82,20,5,135,195,129,245,245,117,65,0,53,18,137,88,187,221,14,149,74,69,46,180,123,66,44,119,
148,124,113,113,81,240,42,69,10,98,119,46,15,247,135,88,173,86,139,137,137,9,76,77,77,69,5,98,57,29,222,201,167,239,14,177,233,233,233,130,193,111,145,134,88,46,82,201,189,141,33,213,4,98,177,24,118,187,
29,27,27,27,164,78,60,26,16,235,116,58,145,158,158,206,189,90,255,38,128,216,152,152,24,44,45,45,97,110,110,46,170,137,216,202,202,74,114,22,153,166,233,207,66,58,98,159,123,238,57,242,144,168,187,187,
27,231,151,150,241,232,14,11,70,10,98,43,42,42,248,77,149,127,19,64,108,123,123,59,214,215,215,177,180,180,132,141,141,13,60,241,196,19,120,241,197,23,241,189,239,125,31,235,87,31,139,24,196,242,171,163,
5,16,171,80,40,254,22,8,4,240,157,239,124,39,228,243,242,214,91,111,227,137,39,158,138,126,71,172,209,104,252,204,237,118,99,107,107,43,20,186,222,121,7,203,23,87,35,150,136,221,101,220,203,87,16,43,145,
72,254,205,100,50,161,167,167,7,235,235,235,88,93,93,197,194,194,2,206,156,153,195,220,220,98,196,134,117,57,28,142,224,221,169,176,154,32,62,62,30,38,147,137,12,128,227,246,67,93,221,220,245,37,252,16,
91,80,80,112,127,136,21,137,68,219,6,131,1,69,69,69,240,249,124,100,63,20,73,136,13,138,125,239,61,172,203,100,50,161,179,179,83,176,173,141,20,196,6,137,204,222,16,171,209,104,144,155,155,11,159,207,
23,209,68,108,89,89,153,0,246,239,11,177,220,136,216,177,177,49,18,114,136,4,196,150,148,148,236,191,154,128,162,168,47,69,34,17,92,46,23,58,59,59,35,2,177,92,189,239,129,19,177,25,25,25,24,24,24,16,236,
173,195,49,172,203,225,112,112,167,34,185,223,184,255,181,239,68,172,70,163,65,85,85,21,154,155,155,5,243,102,15,155,136,229,42,195,130,66,168,159,176,44,187,122,32,136,53,26,141,40,44,44,132,215,235,
21,116,195,30,4,98,237,118,59,82,82,82,194,7,177,26,141,230,19,141,70,3,150,101,17,27,27,139,180,180,52,228,230,230,162,160,160,0,78,167,147,60,198,223,109,65,14,135,99,183,87,230,232,16,203,48,204,42,
203,178,255,75,44,22,111,203,100,50,196,196,196,16,66,84,171,213,72,75,75,35,175,36,183,160,178,178,50,228,231,231,35,49,49,49,120,28,94,248,19,177,156,186,112,237,186,156,16,43,20,10,168,84,42,100,101,
101,193,106,181,34,35,35,67,240,234,240,102,101,69,7,98,185,122,2,238,85,228,215,103,126,99,32,150,97,152,191,177,44,251,25,77,211,255,246,109,53,193,223,69,71,44,77,211,31,196,198,198,254,109,39,114,
244,249,67,131,88,238,131,30,23,23,135,170,170,42,76,79,79,163,175,175,15,46,151,139,156,200,142,42,196,114,197,180,149,149,149,56,127,254,60,22,23,23,49,57,57,137,206,206,78,114,184,158,91,28,239,24,
74,100,32,150,162,40,36,38,38,194,235,245,98,113,113,17,75,75,75,152,154,154,34,100,196,49,35,119,206,166,190,190,158,220,211,113,153,226,157,112,253,209,171,9,148,74,37,108,54,27,134,134,134,176,180,
180,132,243,231,207,99,114,114,146,204,165,217,203,61,185,146,247,134,134,6,156,62,125,154,132,3,119,234,127,127,47,18,137,158,56,16,196,74,36,146,39,12,6,195,31,251,250,250,176,188,188,140,229,229,101,
4,2,1,12,14,14,238,59,162,27,220,17,203,85,0,55,54,54,34,39,39,7,82,169,244,143,7,74,196,170,213,234,223,251,253,126,92,185,114,5,227,227,227,232,238,238,14,57,197,125,88,136,229,74,32,119,120,113,255,
213,4,122,189,30,183,110,221,194,248,248,120,136,123,30,21,98,57,33,150,203,229,220,87,209,131,33,150,97,152,109,155,205,134,173,173,173,136,65,108,117,117,53,191,96,237,193,16,43,149,74,193,141,197,139,
20,196,214,212,212,4,159,63,190,63,196,198,199,199,227,185,231,158,19,244,109,68,2,98,203,202,202,248,141,205,247,135,88,183,219,141,235,215,175,71,5,98,51,50,50,4,55,163,187,66,172,76,38,195,230,230,
38,206,156,57,19,21,136,45,42,42,130,66,161,224,62,240,95,238,10,177,121,121,121,184,121,243,38,185,48,70,26,98,43,42,42,144,158,158,206,31,28,32,132,88,185,92,142,165,165,37,44,44,44,68,53,17,107,179,
217,248,175,150,176,154,32,55,55,151,52,220,12,14,14,98,96,96,0,231,206,45,225,59,79,63,23,81,136,45,47,47,71,106,106,106,104,34,150,101,217,207,167,167,167,241,248,227,143,227,214,173,91,248,238,119,
191,139,231,159,127,30,63,248,193,203,120,233,165,151,177,181,117,51,162,16,203,117,122,210,52,253,53,196,166,166,166,226,252,249,243,187,214,102,222,190,253,22,110,60,249,116,68,33,150,63,229,158,64,
172,90,173,254,91,85,85,21,174,95,191,30,178,168,55,222,248,9,46,93,190,26,49,136,117,185,92,252,98,191,207,9,196,210,52,253,65,70,70,6,6,7,7,225,247,251,49,51,51,131,233,233,105,204,207,47,98,97,241,
66,68,59,98,109,54,27,255,222,80,88,77,160,86,171,81,82,82,130,238,238,110,178,245,104,109,109,199,240,240,68,196,32,54,232,67,190,251,176,174,140,140,12,244,244,244,68,13,98,139,139,139,5,211,166,67,
32,150,162,168,47,85,42,21,28,14,7,186,187,187,163,2,177,65,9,235,221,33,150,162,40,88,44,22,12,14,14,70,37,17,27,20,252,218,27,98,181,90,45,249,222,139,36,196,6,109,93,238,15,177,20,69,225,248,241,227,
232,235,235,11,218,49,134,23,98,131,62,224,15,134,88,179,217,140,241,241,241,32,29,14,47,196,242,100,111,127,16,171,84,42,191,250,144,239,4,79,195,9,177,101,101,101,130,145,139,7,170,38,72,77,77,133,203,
229,18,44,232,40,16,235,116,58,81,92,92,140,156,156,28,62,234,255,254,64,137,216,248,248,248,63,106,52,26,40,149,74,164,165,165,161,168,168,136,44,102,191,16,203,165,169,243,242,242,72,185,26,239,186,
244,71,154,166,159,56,16,196,50,12,243,132,76,38,251,189,76,38,35,140,40,147,201,144,158,158,14,167,211,73,182,180,187,45,136,75,82,91,44,22,50,41,154,255,19,150,106,2,154,166,183,185,218,85,149,74,5,
185,92,14,131,193,0,187,221,78,22,194,45,198,102,179,193,108,54,147,183,136,183,144,200,66,44,195,48,80,169,84,136,137,137,129,94,175,39,197,143,25,25,25,130,135,138,252,175,144,168,65,172,88,44,134,74,
165,34,113,221,160,183,136,248,212,67,129,88,145,72,244,185,76,38,131,72,36,250,219,206,150,246,219,142,216,191,175,97,93,18,137,4,58,157,238,11,133,66,241,9,77,211,255,227,161,64,44,77,211,16,139,197,
80,40,20,176,219,237,24,30,30,198,240,240,48,220,110,55,255,169,85,116,32,150,97,24,168,213,106,152,76,38,120,189,94,204,207,207,99,117,117,21,126,191,31,61,61,61,104,108,108,36,245,41,113,113,113,252,
200,110,120,33,150,97,152,109,238,122,84,85,85,5,159,207,135,185,185,57,226,159,221,221,221,132,25,79,159,62,141,134,134,6,120,189,94,216,237,118,88,44,22,112,35,208,118,94,185,163,65,108,98,98,226,31,
90,90,90,48,53,53,133,169,169,41,76,79,79,99,102,102,6,126,191,31,67,67,67,130,136,238,94,16,235,241,120,80,92,92,140,140,140,12,40,149,74,206,216,127,123,168,68,172,92,46,255,208,239,247,227,234,213,
171,24,27,27,35,163,203,248,77,0,251,45,173,229,118,24,229,229,229,200,205,205,229,222,214,15,15,4,177,52,77,255,214,104,52,226,214,173,91,24,29,29,221,213,61,15,10,177,92,148,178,166,166,6,94,175,23,
153,153,153,96,89,246,15,251,134,88,133,66,129,166,166,38,108,108,108,68,4,98,171,170,170,144,155,155,203,61,145,120,48,196,82,20,245,165,193,96,192,229,203,151,49,51,51,19,17,136,173,173,173,229,231,
166,182,247,5,177,78,167,19,143,62,250,168,96,54,86,184,33,214,233,116,242,163,186,247,135,88,131,193,128,179,103,207,98,126,126,62,226,29,177,252,225,186,123,66,172,76,38,131,199,227,193,218,218,154,
160,9,32,210,16,187,179,213,217,29,98,77,38,19,150,151,151,49,59,59,27,21,136,205,207,207,39,29,253,187,66,172,74,165,66,127,127,63,86,86,86,162,6,177,65,141,130,66,136,165,105,250,127,88,173,86,92,186,
116,9,243,243,243,240,251,253,24,27,27,195,228,164,31,47,189,244,74,68,33,150,67,142,16,136,141,143,143,255,100,114,114,18,79,61,245,20,222,125,247,93,252,228,39,63,193,143,126,244,35,188,242,202,43,120,
245,213,215,240,220,115,47,68,180,35,150,247,120,247,235,142,88,131,193,240,197,196,196,4,158,125,246,217,93,164,228,29,60,251,236,243,17,131,216,160,243,237,95,87,19,72,36,18,88,173,86,172,175,175,227,
123,223,251,30,94,121,229,21,252,228,39,63,193,157,59,119,240,250,235,63,193,230,99,91,17,133,216,160,49,84,95,87,19,88,44,22,76,79,79,99,104,104,8,163,163,163,24,31,31,199,249,243,203,88,94,94,141,104,
71,108,144,14,127,17,50,172,203,229,114,161,175,175,143,124,219,251,124,253,240,79,157,137,40,196,150,150,150,242,83,140,159,8,32,86,44,22,195,98,177,96,116,116,148,44,42,26,16,155,151,151,199,191,79,
252,31,2,136,165,105,26,41,41,41,104,105,105,33,71,148,162,1,177,252,235,212,174,16,27,23,23,135,162,162,34,12,14,14,70,28,98,119,25,142,179,59,196,50,12,3,174,192,54,210,16,91,88,88,24,92,131,184,55,
196,102,100,100,144,210,218,72,66,44,87,48,179,47,136,213,235,245,232,235,235,67,103,103,103,196,32,214,110,183,7,167,25,239,15,177,12,195,108,123,60,30,244,246,246,6,21,215,134,15,98,121,105,88,80,20,
245,229,190,32,54,61,61,29,30,143,39,228,180,89,56,32,214,233,116,10,106,89,247,13,177,106,181,250,15,90,173,22,26,141,6,199,143,31,71,121,121,249,145,33,150,91,80,80,239,198,111,15,4,177,18,137,228,67,
165,82,9,181,90,13,149,74,5,179,217,140,146,146,146,67,85,19,88,173,86,228,229,229,33,35,35,131,255,27,247,225,97,171,9,126,43,145,72,16,19,19,3,181,90,13,165,82,137,204,204,76,65,26,118,183,5,113,163,
16,140,70,35,226,227,227,249,249,41,110,65,127,56,50,196,114,167,107,149,74,37,52,26,13,180,90,45,242,243,243,81,94,94,78,122,19,184,239,49,110,206,177,68,34,1,195,48,130,159,136,12,235,226,156,83,173,
86,67,173,86,35,37,37,5,54,155,13,249,249,249,48,26,141,144,201,100,100,70,22,247,19,213,68,44,203,178,164,46,147,171,34,248,70,64,44,77,211,255,67,42,149,126,194,178,236,23,59,11,249,118,88,215,223,39,
196,114,27,127,150,101,63,140,139,139,251,181,72,36,186,253,80,18,177,20,69,145,70,230,148,148,20,212,214,214,162,167,167,7,30,143,7,90,173,86,48,172,43,98,16,43,151,203,161,82,169,192,178,44,82,83,83,
97,183,219,209,216,216,136,225,225,97,172,172,172,32,16,8,144,225,93,46,151,11,102,179,153,159,138,13,31,196,170,213,234,255,93,92,92,188,205,117,82,249,124,62,44,44,44,96,97,97,129,56,232,216,216,24,
124,62,31,97,70,238,4,9,215,238,166,86,171,137,23,132,37,17,235,243,249,176,190,190,142,169,169,41,140,141,141,97,108,108,140,136,12,167,124,123,185,103,93,93,29,42,43,43,225,112,56,200,197,245,80,7,47,
120,71,84,222,215,235,245,248,238,119,191,11,191,223,191,43,51,238,55,17,91,89,89,9,187,221,46,152,89,180,243,11,114,176,142,88,165,82,137,206,206,206,61,221,243,32,16,91,95,95,143,234,234,106,84,84,84,
160,180,180,20,57,57,57,220,115,193,253,39,98,185,36,199,149,43,87,48,53,53,21,54,136,229,132,216,235,245,162,178,178,146,139,135,255,239,125,65,172,86,171,197,232,232,40,169,101,141,4,196,114,35,206,
118,190,180,239,15,177,34,145,8,141,141,141,88,93,93,21,52,160,134,27,98,171,171,171,249,57,207,251,67,108,106,106,42,158,122,234,41,76,79,79,147,167,11,145,128,216,234,234,106,50,221,126,103,103,177,
59,196,42,20,10,172,172,172,224,226,197,139,130,168,119,164,134,117,113,150,190,39,196,74,165,210,237,166,166,38,92,191,126,29,253,253,253,81,129,216,178,178,50,193,173,150,0,98,197,98,241,237,172,172,
44,92,191,126,29,27,27,27,56,119,238,28,102,103,103,49,62,62,142,239,191,244,114,196,32,214,237,118,11,110,221,5,16,155,156,156,252,235,174,174,46,60,253,244,211,130,127,224,171,175,190,138,55,111,191,
133,31,189,250,70,68,32,182,162,162,98,207,113,65,37,50,153,236,195,202,202,202,93,143,145,191,247,222,123,120,233,159,94,137,8,196,86,86,86,146,187,229,157,207,149,112,88,87,86,86,22,150,151,151,113,
233,210,37,220,188,121,19,223,255,254,247,241,214,91,111,225,246,237,219,120,250,233,103,35,6,177,123,29,184,39,29,177,220,163,87,159,207,135,161,161,33,172,172,172,98,237,202,163,88,137,96,71,172,211,
233,20,132,229,5,16,171,209,104,224,245,122,49,48,48,64,190,92,71,71,39,48,63,191,20,113,136,229,153,167,112,88,151,76,38,67,110,110,46,198,199,199,201,158,40,26,16,91,94,94,142,204,204,76,254,173,124,
137,160,161,57,43,43,11,51,51,51,100,81,209,128,88,167,211,73,198,80,209,52,253,235,144,68,108,74,74,10,86,86,86,200,180,129,72,67,108,121,121,185,0,246,25,134,185,29,2,177,82,169,148,60,200,142,70,34,
214,102,179,9,82,140,123,118,196,86,87,87,163,191,191,159,108,208,34,5,177,78,167,51,248,194,185,59,196,42,20,10,148,150,150,146,140,103,164,32,150,107,145,223,55,196,102,103,103,195,239,247,239,242,232,
62,124,16,155,151,151,39,24,230,252,64,136,53,153,76,112,187,221,36,125,17,110,136,45,41,41,225,15,125,219,31,196,42,20,138,255,173,211,233,160,86,171,145,151,151,7,143,199,131,218,218,218,176,65,108,
208,99,180,131,37,98,21,10,5,52,26,13,146,146,146,72,240,52,28,16,27,52,176,235,192,29,177,239,75,165,82,196,197,197,65,163,209,32,59,59,91,176,107,60,200,176,46,174,73,55,232,85,122,255,176,16,251,27,
174,218,80,171,213,34,33,33,1,78,167,19,181,181,181,123,46,136,251,220,112,131,191,205,102,51,140,70,35,116,58,29,223,21,126,19,22,136,165,105,26,42,149,10,177,177,177,200,205,205,21,44,200,237,118,147,
140,148,94,175,71,108,108,44,33,71,134,97,32,22,139,137,14,51,12,243,191,195,14,177,44,203,66,171,213,146,36,172,197,98,65,124,124,188,64,131,131,22,65,70,109,68,20,98,197,98,241,182,70,163,129,92,46,
39,11,216,69,131,163,15,177,12,195,220,86,40,20,191,166,105,250,195,61,134,117,125,11,177,127,127,16,75,81,212,251,52,77,255,119,137,68,242,78,52,33,246,15,252,14,14,238,209,72,108,108,44,52,26,13,44,
22,11,233,189,214,233,116,219,52,77,127,20,17,136,149,201,100,239,36,37,37,253,57,43,43,11,102,179,25,121,121,121,176,217,108,40,44,44,132,199,227,65,123,123,59,186,187,187,201,105,55,238,52,101,109,109,
45,244,122,61,119,209,12,111,34,214,106,181,110,79,77,77,97,102,102,6,51,51,51,152,154,154,194,200,200,8,38,38,38,224,247,251,49,60,60,140,193,193,65,146,225,107,109,109,197,233,211,167,225,241,120,144,
151,151,199,191,154,135,7,98,99,99,99,177,181,181,133,133,133,133,144,22,128,253,12,235,226,116,37,37,37,133,159,35,62,252,176,46,153,76,134,198,198,70,92,191,126,61,164,223,243,32,16,203,205,104,63,118,
236,24,191,136,230,224,195,186,104,154,38,67,113,130,243,158,135,73,196,86,87,87,195,225,112,32,39,39,7,26,141,134,123,213,14,6,177,70,163,17,203,203,203,88,92,92,20,184,231,81,32,150,155,242,196,29,99,
217,217,18,239,47,17,171,80,40,176,184,184,136,75,151,46,9,0,45,92,195,186,184,221,40,55,84,137,97,152,119,238,11,177,49,49,49,104,110,110,198,214,214,150,32,235,25,110,136,245,122,189,40,44,44,228,22,
245,231,61,33,54,38,38,6,53,53,53,88,91,91,131,223,239,143,104,34,182,186,186,26,229,229,229,228,131,191,43,196,50,12,243,81,75,75,11,86,87,87,49,55,55,39,200,157,71,10,98,61,30,15,159,129,254,16,2,177,
233,233,233,219,47,190,248,34,110,222,188,137,39,159,124,18,43,43,43,152,153,153,193,249,243,75,17,131,216,202,202,74,196,197,197,237,14,177,82,169,244,157,156,156,28,60,253,244,211,184,125,251,54,249,
7,190,243,206,59,184,125,251,173,136,65,44,247,176,114,175,68,236,127,207,206,206,198,218,218,26,222,125,247,221,93,95,141,72,37,98,121,71,200,67,171,9,204,102,51,206,157,59,135,169,169,41,172,172,172,
224,230,205,155,120,253,245,215,113,231,206,29,188,244,210,203,17,75,196,166,165,165,237,218,231,89,78,81,212,95,181,90,45,6,7,7,209,219,219,139,214,214,86,244,246,246,98,229,145,75,184,118,237,6,54,31,
123,34,98,16,27,148,165,18,66,172,70,163,33,207,131,185,239,178,153,217,185,125,15,234,58,44,196,6,159,71,22,64,172,68,34,65,89,89,25,166,167,167,163,10,177,118,187,125,247,142,216,157,247,18,217,217,
217,152,159,159,143,122,34,150,119,218,86,88,77,32,18,137,144,149,149,37,88,84,52,32,214,229,114,241,175,85,255,61,4,98,179,179,179,177,184,184,40,216,15,69,26,98,93,46,23,57,103,202,48,204,59,33,16,155,
153,153,137,139,23,47,162,165,165,37,42,16,235,116,58,249,134,190,43,196,254,193,100,50,97,105,105,73,208,15,27,73,136,181,88,44,130,163,118,187,66,108,122,122,58,6,6,6,200,72,179,72,38,98,115,114,114,
4,238,176,39,196,38,38,38,254,57,43,43,139,188,125,145,130,88,171,213,26,210,12,183,39,196,178,44,251,142,78,167,131,66,161,192,137,19,39,224,245,122,195,14,177,54,155,45,120,182,209,254,18,177,82,169,
20,90,173,22,41,41,41,168,168,168,64,99,99,99,88,32,182,164,164,132,63,148,242,224,16,43,145,72,160,213,106,17,31,31,143,242,242,114,114,178,241,40,16,27,244,61,119,184,97,93,20,69,65,173,86,67,163,209,
224,228,201,147,33,93,177,7,129,216,93,146,213,255,117,36,136,221,57,99,140,156,156,28,65,76,247,126,213,4,165,165,165,112,58,157,112,56,28,56,121,242,100,240,115,152,240,64,172,76,38,67,82,82,18,114,
115,115,209,220,220,12,143,199,67,108,188,160,160,0,22,139,5,233,233,233,208,233,116,208,106,181,208,106,181,136,141,141,37,61,11,252,207,80,88,33,86,42,149,66,167,211,65,167,211,193,96,48,32,62,62,30,
114,185,156,224,171,88,44,134,68,34,1,203,178,164,154,149,251,207,105,154,254,179,82,169,124,39,34,16,43,18,137,62,82,42,149,219,50,153,12,44,203,18,136,229,7,150,131,18,177,127,136,26,196,138,197,226,
119,118,190,213,223,255,22,98,255,159,130,216,168,118,196,202,229,242,159,42,149,202,123,156,204,112,23,67,238,87,94,38,147,145,177,121,90,173,22,10,133,98,91,163,209,252,153,97,152,119,35,146,136,85,
42,149,247,202,203,203,209,219,219,139,161,161,33,52,52,52,160,169,169,9,77,77,77,168,170,170,66,107,107,43,154,155,155,209,213,213,133,161,161,33,244,247,247,99,122,122,154,191,179,12,47,196,202,100,
178,237,230,230,102,172,174,174,34,16,8,192,231,243,161,187,187,27,3,3,3,240,249,124,24,24,24,192,192,192,0,250,250,250,208,223,223,79,74,73,91,90,90,224,118,187,113,226,196,9,50,220,59,44,16,75,81,20,
184,126,207,233,233,233,3,65,108,115,115,51,26,26,26,224,118,187,145,153,153,73,142,27,28,9,98,69,34,17,204,102,51,57,117,27,92,201,122,144,142,216,226,226,226,224,242,246,131,67,44,69,81,48,155,205,88,
90,90,194,133,11,23,4,147,195,142,2,177,153,153,153,252,174,197,253,67,44,77,211,208,233,116,184,120,241,34,46,94,188,40,112,207,163,66,172,195,225,128,217,108,230,211,245,254,58,98,143,29,59,134,169,
169,41,172,173,173,133,29,98,189,94,47,28,14,7,78,156,56,65,130,244,15,76,196,30,59,118,12,87,175,94,197,185,115,231,48,52,52,20,177,97,93,46,151,11,5,5,5,136,143,143,7,195,48,247,118,133,88,150,101,223,
181,88,44,88,95,95,199,35,143,60,34,136,232,70,10,98,157,78,39,76,38,19,183,219,248,105,8,196,166,165,165,253,101,109,109,13,55,111,222,196,91,111,189,133,103,158,121,6,75,75,75,152,152,152,136,24,196,
86,85,85,145,83,108,52,77,223,11,129,88,149,74,181,189,177,177,129,23,94,120,33,228,38,50,82,16,91,85,85,197,63,24,29,90,77,96,48,24,48,59,59,187,199,236,246,187,247,89,216,225,33,150,43,8,217,21,98,41,
138,250,87,149,74,133,145,145,17,248,253,126,76,79,79,227,242,229,203,120,241,197,23,191,122,53,110,71,14,98,131,6,225,8,33,86,169,84,162,165,165,5,227,227,227,104,107,107,67,71,71,7,150,150,150,113,235,
217,231,113,235,217,127,140,24,196,6,133,187,132,16,171,80,40,208,220,220,12,191,223,79,190,58,206,204,205,227,234,198,181,136,14,235,10,90,148,16,98,41,138,34,253,249,209,132,88,158,79,133,66,172,66,
161,128,199,227,33,57,207,104,65,108,80,124,41,20,98,155,155,155,49,51,51,19,85,136,13,106,210,21,66,172,84,42,69,67,67,3,166,166,166,162,218,17,27,228,232,194,142,88,165,82,9,159,207,135,137,137,137,
168,66,44,223,209,119,173,38,24,28,28,196,232,232,168,96,131,22,73,136,229,84,111,79,136,141,141,141,189,103,179,217,208,219,219,43,168,37,136,100,71,172,221,110,231,247,5,253,37,4,98,101,50,217,79,211,
210,210,96,179,217,4,19,123,35,5,177,197,197,197,130,199,106,50,153,236,221,93,33,86,38,147,221,75,78,78,134,197,98,129,215,235,69,67,67,67,196,32,54,104,144,224,253,19,177,44,203,110,39,38,38,66,167,
211,193,227,241,8,146,177,225,130,216,160,172,231,254,135,117,105,181,90,232,116,58,120,189,222,176,65,172,205,102,67,124,124,252,225,33,150,166,105,104,181,90,168,213,106,184,221,110,212,215,215,31,9,
98,237,118,123,112,102,248,240,16,27,27,27,11,189,94,79,94,173,131,86,19,112,13,186,65,143,101,143,6,177,220,237,150,209,104,36,249,225,221,22,20,188,24,110,65,92,51,64,216,134,117,113,150,32,18,137,182,
181,90,45,146,146,146,224,118,187,81,81,81,129,130,130,2,228,231,231,227,216,177,99,200,204,204,132,197,98,129,201,100,34,255,58,59,59,27,39,79,158,132,193,96,224,63,20,10,127,53,1,247,25,227,176,149,
97,24,1,188,6,255,240,122,18,64,81,212,189,136,36,98,89,150,125,87,169,84,254,133,97,152,109,46,150,203,37,97,185,6,75,46,64,200,21,64,50,12,115,111,231,194,252,112,134,117,125,11,177,127,23,16,75,211,
244,35,223,24,136,165,40,234,141,196,196,196,63,230,228,228,108,115,129,63,238,88,184,68,34,129,68,34,33,31,242,157,44,250,54,69,81,127,96,24,230,237,136,64,44,77,211,31,75,36,18,156,57,115,6,75,75,75,
24,25,25,193,200,200,8,218,219,219,225,243,249,72,164,141,155,6,93,89,89,137,210,210,82,148,151,151,115,179,177,254,178,147,103,15,223,176,174,228,228,100,140,142,142,98,99,99,3,131,131,131,232,232,232,
64,91,91,27,218,219,219,5,231,149,57,140,237,239,239,39,255,190,169,169,9,233,233,233,16,139,197,225,129,88,154,166,33,151,203,49,55,55,135,203,151,47,147,87,231,32,238,201,205,44,10,202,119,30,14,98,
185,188,231,196,196,4,214,215,215,5,105,198,131,64,44,151,138,205,205,205,229,143,158,58,56,196,74,36,18,24,12,6,92,184,112,1,143,60,242,8,134,135,135,5,169,216,195,12,235,114,58,157,48,24,12,124,33,222,
63,196,202,229,114,120,189,94,44,44,44,224,145,71,30,9,59,196,166,165,165,241,23,118,127,136,21,137,68,111,104,52,154,237,211,167,79,227,250,245,235,8,4,2,130,74,214,112,64,108,117,117,53,23,160,231,190,
51,239,63,172,43,37,37,229,47,243,243,243,228,40,121,164,32,150,43,12,73,77,77,229,94,177,143,119,133,88,177,88,252,118,94,94,30,190,251,221,239,226,159,254,233,159,240,238,187,239,226,169,167,158,194,
153,51,103,48,49,49,17,118,136,245,120,60,176,90,173,72,72,72,184,111,53,193,31,138,138,138,240,228,147,79,10,254,193,239,190,251,46,238,220,185,19,17,136,117,185,92,56,121,242,36,180,90,45,36,18,201,
31,67,32,86,34,145,108,23,21,21,97,117,117,21,215,175,95,199,219,111,223,255,246,59,92,16,235,114,185,72,84,87,161,80,60,34,128,88,154,166,97,183,219,177,188,188,140,129,129,1,4,2,1,108,109,109,225,206,
157,59,17,133,88,143,199,67,18,66,33,16,43,145,72,96,183,219,177,180,180,68,46,140,163,163,163,120,249,229,87,35,86,77,224,241,120,224,112,56,246,134,88,134,97,80,94,94,142,165,165,37,114,165,158,153,
9,224,169,155,183,34,58,172,43,40,82,41,132,88,150,101,81,93,93,141,249,249,249,168,66,108,144,228,9,33,86,42,149,98,104,104,136,152,103,180,32,118,151,248,210,215,16,43,151,203,17,8,4,48,54,54,22,85,
136,77,79,79,223,27,98,13,6,3,150,151,151,209,219,219,27,85,136,229,205,134,12,133,216,196,196,68,92,187,118,45,234,195,186,130,22,37,132,216,172,172,44,172,175,175,135,28,91,138,52,196,242,53,79,0,177,
52,77,63,98,50,153,208,223,223,47,232,74,136,6,196,242,170,46,66,33,54,46,46,238,143,58,157,14,46,151,11,237,237,237,81,129,88,151,203,197,55,171,63,236,154,136,101,24,6,122,189,30,213,213,213,17,135,
88,167,211,137,83,167,78,17,127,144,201,100,111,239,10,177,52,77,127,28,31,31,15,173,86,11,151,203,133,150,150,150,136,13,235,202,201,201,225,239,62,255,242,192,97,93,44,203,66,175,215,163,174,174,78,
48,176,43,28,16,107,183,219,145,154,154,42,48,43,149,74,245,198,3,33,150,243,206,132,132,4,50,50,56,92,16,27,68,213,7,131,88,78,136,143,31,63,142,206,206,206,35,119,196,158,58,117,42,56,51,124,56,136,
101,24,6,70,163,17,14,135,67,176,99,60,40,196,90,173,214,224,177,120,71,131,88,150,101,97,48,24,80,81,81,129,250,250,250,3,47,232,228,201,147,193,93,64,225,129,88,165,82,185,29,31,31,15,147,201,132,244,
244,116,24,141,70,242,87,179,217,12,179,217,140,227,199,143,163,176,176,16,86,171,149,180,232,22,20,20,4,231,133,195,55,172,75,36,18,189,161,86,171,255,162,84,42,33,147,201,32,147,201,32,149,74,5,127,
149,201,100,96,24,134,252,107,181,90,29,92,160,253,113,68,32,118,199,153,254,176,19,99,35,125,176,28,198,114,163,207,36,18,9,164,82,41,247,107,255,199,135,50,172,107,47,136,213,235,245,143,124,11,177,
223,200,68,172,92,46,127,79,46,151,255,251,206,219,245,249,67,133,88,138,162,62,212,233,116,219,220,117,203,102,179,65,175,215,147,231,120,92,7,16,175,233,52,178,29,177,52,77,35,41,41,9,43,43,43,88,88,
88,192,232,232,40,186,186,186,208,213,213,69,118,20,69,69,69,100,220,162,193,96,64,76,76,12,98,99,99,33,22,139,63,101,89,246,189,176,66,44,69,81,112,58,157,152,155,155,195,202,202,138,32,98,217,218,218,
74,238,21,59,58,58,208,213,213,69,146,177,92,88,213,235,245,34,53,53,117,91,36,18,253,71,88,18,177,26,141,6,46,151,11,143,61,246,24,230,230,230,4,178,247,32,247,228,15,239,58,113,226,4,247,246,30,30,98,
197,98,49,180,90,45,166,167,167,113,241,226,69,4,2,1,65,42,246,32,16,219,208,208,0,187,221,142,216,216,88,238,162,122,112,136,85,169,84,104,108,108,196,217,179,103,113,233,210,37,50,198,236,40,16,235,
245,122,201,142,243,64,137,88,153,76,246,31,54,155,141,64,254,212,212,148,192,61,143,58,172,139,235,50,227,237,60,239,15,177,49,49,49,239,57,157,78,220,186,117,11,143,61,246,24,230,231,231,195,158,136,
173,174,174,38,97,9,174,151,227,190,16,43,147,201,62,245,251,253,164,66,243,173,183,222,194,163,143,62,138,201,201,73,4,2,129,176,66,108,80,245,197,222,29,177,58,157,14,203,203,203,248,135,127,248,7,193,
237,209,75,47,189,132,55,223,124,51,108,16,91,93,93,141,178,178,50,152,205,102,178,189,217,179,35,86,163,209,96,101,101,5,103,206,156,193,205,155,55,247,101,158,71,129,216,162,162,34,36,37,37,113,151,
137,221,135,117,153,205,102,108,108,108,160,183,183,23,195,195,195,184,112,225,2,94,125,245,213,136,65,44,231,233,92,218,58,36,17,75,211,52,210,210,210,240,232,163,143,10,142,12,252,211,15,126,72,238,
225,34,1,177,229,229,229,196,19,24,134,121,79,0,177,20,69,33,39,39,7,87,174,92,33,23,198,158,30,31,94,252,254,15,35,58,172,171,178,178,146,140,182,166,105,250,223,5,16,43,149,74,81,89,89,137,139,23,47,
146,139,99,52,32,214,237,118,243,239,7,133,16,75,81,20,38,38,38,16,8,4,162,10,177,37,37,37,36,108,177,179,47,251,26,98,185,46,207,104,87,19,20,20,20,236,14,177,52,77,127,158,150,150,134,141,141,13,82,
83,31,45,136,229,149,108,135,66,172,211,233,196,229,203,151,209,214,214,22,53,136,13,154,179,38,132,216,152,152,152,127,183,219,237,8,4,2,164,31,54,26,16,235,114,185,248,93,9,66,136,149,201,100,239,89,
44,22,242,229,27,45,136,117,56,28,252,221,66,104,34,86,163,209,108,235,116,58,212,213,213,69,173,154,32,232,185,204,238,195,186,52,26,13,178,178,178,208,221,221,29,21,136,229,69,116,247,238,136,165,105,
26,70,163,17,39,79,158,36,157,9,145,130,88,174,150,117,95,29,177,92,64,194,235,245,70,20,98,157,78,39,249,122,161,40,234,211,7,66,108,108,108,44,146,146,146,4,71,225,194,57,172,171,160,160,0,9,9,9,228,
85,82,42,149,239,237,43,17,171,86,171,81,80,80,128,206,206,206,176,13,235,178,219,237,72,79,79,23,80,35,195,48,255,177,111,136,101,24,6,38,147,137,176,245,81,32,150,27,210,21,27,27,123,116,136,85,40,20,
48,153,76,36,31,117,24,136,117,58,157,200,200,200,8,102,198,163,65,108,74,74,10,172,86,43,108,54,27,138,139,139,225,116,58,81,86,86,70,182,29,247,91,80,81,81,17,255,129,80,248,32,86,34,145,252,135,86,
171,133,92,46,135,84,42,5,203,178,80,169,84,80,42,149,208,235,245,72,79,79,71,126,126,62,108,54,27,202,202,202,200,135,121,183,68,117,88,33,86,42,149,190,199,48,204,167,156,103,114,0,171,80,40,32,151,
203,33,147,201,160,80,40,16,27,27,139,180,180,52,164,165,165,237,198,212,145,31,214,197,251,237,33,167,181,185,197,177,44,203,109,220,182,163,62,172,107,23,136,253,124,39,227,240,239,98,177,248,189,111,
19,177,223,232,106,2,165,82,249,243,216,216,216,79,100,50,217,223,30,58,196,138,197,226,109,133,66,65,6,81,156,62,125,26,57,57,57,72,76,76,20,108,218,162,2,177,220,111,155,213,106,197,194,194,2,46,94,
188,72,82,141,220,142,162,182,182,22,89,89,89,208,235,245,36,44,31,246,97,93,124,136,229,106,235,150,150,150,112,246,236,89,210,243,201,221,108,112,251,123,110,47,214,220,220,76,78,166,165,167,167,67,
173,86,111,75,165,210,95,135,5,98,99,99,99,81,83,83,131,233,233,105,114,112,154,243,134,189,220,179,169,169,137,236,245,219,219,219,209,210,210,130,174,174,46,88,44,22,136,197,226,223,31,26,98,213,106,
53,170,170,170,48,62,62,142,43,87,174,96,122,122,250,64,50,204,237,199,26,26,26,200,190,44,43,43,139,19,188,131,65,108,92,92,220,239,171,171,171,225,247,251,113,229,202,21,4,2,129,144,136,238,97,32,182,
174,174,14,14,135,3,74,165,146,187,226,239,15,98,53,26,205,175,207,158,61,139,27,55,110,96,107,107,11,115,115,115,232,233,233,9,27,196,122,189,222,224,46,179,7,87,19,36,37,37,109,63,243,204,51,120,238,
185,231,200,200,160,11,23,46,144,142,141,112,12,235,42,41,41,225,191,90,95,62,176,154,32,59,59,27,27,27,27,33,130,247,157,239,124,39,232,120,249,225,33,150,59,20,205,111,126,219,19,98,105,154,70,113,113,
49,174,94,189,138,51,103,206,224,31,255,241,31,239,123,99,121,148,68,172,221,110,135,90,173,38,12,116,223,97,93,125,125,125,88,92,92,68,123,123,59,252,126,63,158,120,226,137,136,64,108,69,69,5,142,31,
63,206,239,227,216,29,98,229,114,57,206,157,59,39,104,210,61,119,238,124,196,32,150,203,167,239,28,212,8,133,88,110,148,245,250,250,58,185,22,181,182,182,226,123,47,190,20,177,142,88,110,218,147,70,163,
225,158,220,191,22,2,177,197,197,197,88,95,95,71,119,119,119,212,32,214,237,118,195,100,50,113,135,201,126,46,128,88,150,101,17,8,4,112,246,236,217,168,38,98,43,43,43,113,226,196,137,144,9,247,229,20,
69,253,213,100,50,97,117,117,21,131,131,131,81,133,216,138,138,10,126,214,243,111,2,136,117,58,157,228,173,139,38,196,114,95,59,33,16,171,84,42,255,54,62,62,142,185,185,57,193,151,107,52,32,54,200,168,
190,134,216,248,248,248,79,26,26,26,48,58,58,42,248,182,143,70,34,54,104,168,252,215,16,171,80,40,126,110,50,153,200,111,93,180,32,118,151,183,78,152,136,149,201,100,56,118,236,24,218,218,218,200,14,50,
210,16,91,84,84,36,184,139,14,129,88,134,97,182,211,210,210,80,81,81,17,181,97,93,199,143,31,127,48,196,138,68,34,24,141,70,52,53,53,161,165,165,37,226,16,107,52,26,31,12,177,92,48,34,63,63,31,189,189,
189,17,77,196,150,150,150,10,16,237,129,16,155,158,158,78,62,87,145,130,216,210,210,82,193,248,244,7,66,172,82,169,68,94,94,30,124,62,95,216,59,98,185,179,238,92,155,210,129,170,9,210,211,211,209,214,
214,70,250,18,194,53,172,171,176,176,16,41,41,41,4,99,25,134,249,245,190,33,86,165,82,33,63,63,31,221,221,221,100,65,71,73,196,90,173,86,24,12,134,224,68,227,239,15,12,177,156,14,115,83,121,185,195,170,
7,129,88,155,205,134,212,212,212,224,197,28,30,98,149,74,229,239,181,90,45,148,74,37,180,90,45,210,211,211,145,155,155,75,134,113,113,27,181,221,22,196,77,12,219,133,25,143,14,177,114,185,252,215,50,153,
108,91,161,80,32,38,38,134,240,161,90,173,134,209,104,132,205,102,131,221,110,39,159,29,238,233,84,80,37,65,248,33,150,63,172,139,171,27,80,40,20,224,18,178,42,149,10,105,105,105,200,207,207,71,86,86,
214,94,175,78,244,32,86,36,18,65,46,151,67,169,84,146,206,88,94,29,216,195,135,88,177,88,252,55,177,88,252,9,77,211,63,255,182,154,224,239,166,35,86,171,213,126,177,83,230,255,240,32,118,167,91,3,26,141,
6,109,109,109,36,245,81,92,92,28,156,67,143,14,196,114,207,103,234,234,234,72,212,114,100,100,4,93,93,93,130,201,78,73,73,73,252,120,91,100,32,86,38,147,65,167,211,161,180,180,20,243,243,243,184,120,241,
34,252,126,63,250,250,250,4,204,216,212,212,132,134,134,6,52,52,52,192,233,116,226,196,137,19,208,104,52,92,165,74,120,170,9,212,106,53,78,158,60,73,6,204,159,61,123,22,51,51,51,2,25,222,205,61,185,237,
14,87,77,80,87,87,135,188,188,60,24,141,70,176,44,251,193,161,32,86,175,215,223,171,171,171,195,248,248,56,46,95,190,140,185,185,57,76,76,76,160,179,179,147,132,80,247,3,177,95,237,92,191,54,207,198,198,
70,88,44,22,176,44,123,239,64,16,171,82,169,62,88,88,88,192,19,79,60,129,173,173,45,4,2,129,35,15,235,226,22,229,245,122,113,236,216,49,238,202,191,255,106,130,172,172,44,252,227,63,254,35,41,132,124,
241,197,23,5,11,59,10,196,114,66,204,171,251,125,48,196,178,44,187,93,91,91,139,171,87,175,10,194,166,63,248,193,15,112,238,220,185,176,64,108,85,85,21,146,146,146,72,144,254,129,16,27,19,19,131,165,165,
37,204,207,207,99,113,113,81,80,50,42,60,120,127,180,97,93,121,121,121,228,136,203,125,33,150,139,148,112,125,27,221,221,221,152,155,155,195,143,126,244,163,176,67,44,55,26,157,215,62,191,59,196,82,20,
5,151,203,133,205,205,77,2,250,157,157,157,187,78,128,58,42,196,242,135,42,221,23,98,181,90,45,230,231,231,113,230,204,25,242,91,54,61,61,19,17,136,173,170,170,130,205,102,35,179,102,118,237,136,229,32,
246,137,39,158,32,207,97,34,13,177,110,183,27,105,105,105,100,146,235,174,29,177,147,147,147,130,106,130,104,64,172,205,102,227,127,182,132,213,4,6,131,129,52,220,68,19,98,93,46,23,204,102,51,247,228,
65,152,136,245,249,124,184,116,233,18,186,187,187,163,14,177,156,14,211,52,253,5,129,88,189,94,143,177,177,49,65,105,109,180,32,118,151,74,214,175,32,86,167,211,125,81,83,83,67,106,9,162,9,177,65,36,36,
76,196,166,167,167,163,179,179,19,29,29,29,81,133,216,83,167,78,237,93,77,160,209,104,96,183,219,209,211,211,19,213,106,130,189,70,229,153,40,138,122,141,97,24,152,205,102,129,16,71,26,98,11,11,11,239,
15,177,59,247,249,176,90,173,232,234,234,34,231,167,34,9,177,65,157,140,187,67,172,68,34,129,197,98,193,200,200,136,160,39,54,18,16,235,112,56,130,195,204,123,67,172,86,171,69,77,77,13,58,58,58,34,10,
177,65,143,63,238,15,177,82,169,20,102,179,25,195,195,195,17,27,214,197,93,197,247,221,17,75,81,212,151,105,105,105,240,249,124,65,58,28,94,136,229,181,1,236,15,98,213,106,53,108,54,27,186,187,187,5,51,
103,195,1,177,101,101,101,130,115,13,7,170,38,72,79,79,71,99,99,35,185,111,59,42,196,58,157,78,20,22,22,194,104,52,242,63,224,31,28,8,98,53,26,205,189,248,248,120,112,61,196,92,232,148,195,216,253,64,
108,89,89,25,138,139,139,201,12,100,222,97,67,80,20,117,239,80,16,43,149,74,63,224,130,166,82,169,20,42,149,10,22,139,5,133,133,133,168,169,169,65,101,101,229,158,195,186,108,54,27,76,38,19,201,29,240,
127,194,82,77,192,48,204,182,88,44,134,92,46,39,105,88,157,78,135,252,252,124,56,157,78,193,130,138,138,138,96,50,153,200,91,20,180,152,200,64,44,23,52,85,42,149,136,137,137,129,74,165,66,94,94,158,0,
97,249,195,2,249,207,95,162,2,177,44,203,18,132,229,250,18,118,25,214,245,240,32,118,167,31,225,139,111,59,98,247,3,177,34,145,200,243,141,26,214,165,82,169,16,27,27,251,133,92,46,255,228,161,65,44,247,
219,165,215,235,225,116,58,49,49,49,1,159,207,7,183,219,77,46,148,81,131,88,145,72,4,181,90,141,228,228,100,116,117,117,97,98,98,2,23,47,94,36,202,215,208,208,0,143,199,131,220,220,92,168,213,106,193,
37,33,108,16,75,211,244,47,21,10,197,182,66,161,192,241,227,199,81,85,85,133,145,145,17,204,207,207,99,126,126,30,99,99,99,240,249,124,130,91,51,78,236,60,30,15,210,211,211,17,27,27,75,66,129,71,134,88,
163,209,120,207,233,116,194,231,243,225,204,153,51,8,4,2,152,159,159,199,228,228,36,250,251,251,209,217,217,121,95,247,228,190,138,42,42,42,144,157,157,141,132,132,4,62,43,30,28,98,21,10,197,135,211,211,
211,216,218,218,194,230,230,38,102,103,103,225,243,249,4,243,220,247,11,177,220,150,167,170,170,10,249,249,249,48,26,141,96,24,230,195,3,87,19,228,230,230,226,185,231,158,35,238,121,235,214,45,76,78,78,
30,26,98,185,183,148,219,101,236,156,52,186,119,160,142,216,193,193,65,172,173,173,225,173,183,222,34,219,218,27,55,110,96,116,116,244,200,16,91,85,85,133,140,140,12,238,115,246,203,7,66,44,119,156,252,
137,39,158,192,244,244,52,86,87,87,133,55,150,59,238,121,20,136,173,173,173,37,119,198,52,77,63,120,88,151,68,34,65,77,77,13,73,157,13,14,14,226,234,213,171,187,222,92,30,5,98,157,78,39,169,91,121,32,
196,26,12,6,172,175,175,35,16,8,144,207,143,223,239,23,188,141,225,128,216,202,202,74,65,149,216,158,16,43,18,137,80,83,83,131,181,181,53,242,236,165,189,189,61,98,195,186,130,238,144,119,135,216,148,
148,20,60,249,228,147,152,153,153,137,56,196,122,189,94,114,87,195,237,74,67,32,150,27,130,179,178,178,66,30,8,69,26,98,93,46,87,240,91,40,132,216,212,212,84,92,187,118,13,19,19,19,81,133,216,160,183,
240,107,136,213,104,52,159,76,76,76,96,121,121,89,240,213,17,173,142,88,169,84,202,189,133,95,161,25,195,48,158,132,132,132,47,38,39,39,49,54,54,38,248,234,136,6,196,186,92,46,254,220,144,175,59,98,213,
106,53,236,118,59,38,38,38,30,42,196,238,188,90,95,119,196,102,100,100,144,203,64,52,33,150,155,67,186,243,74,125,65,211,180,135,64,172,94,175,135,215,235,37,103,171,162,5,177,101,101,101,252,225,74,159,
8,32,150,139,188,13,15,15,147,69,69,163,154,32,200,208,133,16,203,237,183,187,187,187,209,209,209,17,21,136,117,58,157,216,73,126,236,13,177,113,113,113,112,56,28,24,28,28,140,120,34,214,229,114,5,79,
164,219,59,17,107,177,88,48,58,58,42,216,164,69,2,98,139,139,139,131,19,30,123,67,172,217,108,38,17,240,72,66,108,240,220,190,251,66,172,82,169,36,143,105,35,5,177,193,17,221,7,66,172,66,161,216,206,203,
203,195,240,240,176,96,183,24,78,136,229,79,204,220,169,66,184,63,196,210,52,253,75,206,59,249,93,9,225,132,88,254,243,152,125,67,108,66,66,194,189,132,132,4,152,76,38,156,58,117,10,213,213,213,100,81,
135,133,88,126,215,240,161,171,9,36,18,201,135,113,113,113,80,171,213,208,106,181,200,204,204,132,219,237,62,84,53,129,213,106,69,110,110,46,244,122,61,63,240,245,225,161,171,9,196,98,49,148,74,37,212,
106,53,98,99,99,97,54,155,81,83,83,131,138,138,138,61,23,84,90,90,138,252,252,124,152,76,38,104,52,26,178,53,225,158,170,243,100,216,116,164,80,151,72,36,66,76,76,12,226,226,226,160,209,104,144,147,147,
3,175,215,75,134,119,185,221,110,148,148,148,192,98,177,112,227,132,5,141,168,59,24,187,45,22,139,127,25,118,136,165,105,154,188,114,241,241,241,176,90,173,176,90,173,72,79,79,135,66,161,224,103,12,248,
21,78,209,129,88,150,101,161,84,42,161,80,40,32,145,72,130,105,250,225,66,172,84,42,253,68,34,145,124,177,243,170,124,115,134,117,237,108,204,190,77,196,238,11,98,119,62,75,127,141,141,141,253,84,38,147,
253,252,161,65,44,247,219,165,211,233,96,177,88,200,28,101,151,203,5,157,78,71,162,223,17,129,88,154,166,95,167,40,234,35,150,101,201,115,25,179,217,140,252,252,124,116,118,118,98,98,98,2,243,243,243,
8,4,2,232,239,239,71,107,107,43,41,229,227,93,193,195,7,177,6,131,225,131,180,180,180,109,143,199,131,218,218,90,12,15,15,99,120,120,24,115,115,115,8,4,2,152,157,157,197,240,240,48,122,122,122,200,237,
217,233,211,167,81,85,85,69,234,120,120,21,210,71,135,88,150,101,127,215,222,222,142,107,215,174,97,117,117,21,115,115,115,24,29,29,69,127,127,63,124,62,159,192,28,118,171,38,104,104,104,32,79,214,185,
195,170,135,122,222,199,63,162,146,147,147,131,91,183,110,225,187,223,253,46,238,222,189,139,173,173,45,12,15,15,31,24,98,185,144,51,119,118,230,208,29,177,50,153,12,227,227,227,88,91,91,19,68,41,87,86,
86,200,91,181,95,136,229,132,152,155,91,116,234,212,41,232,245,122,136,68,162,223,237,27,98,105,154,198,169,83,167,112,237,218,53,76,79,79,99,115,115,147,44,234,245,215,95,199,202,202,202,161,32,150,63,
167,148,251,13,149,74,165,31,236,11,98,101,50,25,166,167,167,177,180,180,68,30,117,112,85,5,95,131,236,209,32,214,237,118,67,175,215,115,16,251,250,125,33,150,101,89,84,84,84,96,99,99,67,240,249,9,126,
27,239,222,189,123,36,136,173,169,169,65,114,114,242,254,32,54,62,62,30,43,43,43,152,155,155,35,191,93,125,125,125,97,31,214,85,93,93,45,208,150,61,33,86,171,213,226,236,217,179,88,93,93,21,116,85,71,
10,98,185,161,223,123,38,98,25,134,217,246,122,189,228,40,121,52,32,182,164,164,68,176,247,10,129,88,139,197,130,155,55,111,98,102,102,38,170,16,203,207,80,9,32,54,54,54,246,231,45,45,45,88,89,89,17,92,
24,35,13,177,21,21,21,123,102,242,74,226,227,227,63,109,104,104,192,236,236,172,224,74,29,13,136,13,26,65,37,76,196,102,100,100,144,71,31,209,132,216,162,162,34,254,190,254,107,136,229,58,135,71,71,71,
209,217,217,25,85,136,117,58,157,193,227,94,190,134,88,189,94,143,230,230,102,244,247,247,71,21,98,203,203,203,249,197,144,194,68,44,87,134,60,49,49,17,85,136,229,71,43,105,154,254,84,0,177,156,226,77,
79,79,147,211,218,209,72,196,6,77,124,250,121,8,196,166,166,166,98,124,124,60,106,16,91,86,86,38,120,54,179,43,196,170,213,106,244,245,245,193,231,243,69,165,154,32,104,30,205,246,174,16,43,147,201,80,
93,93,141,225,225,225,136,119,196,218,237,118,168,84,170,7,67,44,203,178,56,117,234,20,198,199,199,35,58,172,203,233,116,6,119,111,220,31,98,179,178,178,176,176,176,32,56,212,21,110,136,205,207,207,15,
134,144,189,33,150,166,233,215,245,122,253,182,215,235,13,73,195,134,11,98,157,78,39,127,115,183,63,136,213,106,181,31,36,39,39,35,45,45,141,68,39,195,9,177,65,51,32,247,15,177,12,195,252,78,163,209,64,
173,86,35,45,45,13,118,187,157,68,66,142,10,177,73,73,73,71,235,136,21,139,197,224,48,54,51,51,19,229,229,229,100,81,7,153,141,85,86,86,134,210,210,210,224,118,146,195,119,196,238,76,21,135,70,163,129,
82,169,36,67,189,185,124,84,240,130,184,0,124,97,97,33,114,115,115,145,153,153,137,164,164,36,196,197,197,241,207,189,255,46,44,16,203,48,12,228,114,57,17,226,134,134,6,210,149,224,118,187,97,183,219,
145,157,157,13,189,94,79,14,223,115,141,1,220,15,77,211,219,12,195,124,16,118,136,85,40,20,136,143,143,135,193,96,64,78,78,14,178,179,179,185,155,202,144,69,240,81,118,7,98,95,143,24,196,210,52,189,173,
86,171,9,194,242,23,242,208,19,177,44,203,254,92,34,145,124,186,215,176,174,111,19,177,127,55,16,27,252,246,177,44,123,135,166,233,213,168,67,44,247,155,37,22,139,161,80,40,144,146,146,130,236,236,108,
84,85,85,161,176,176,16,90,173,54,114,16,171,84,42,127,22,31,31,255,167,148,148,20,152,76,38,228,228,228,224,196,137,19,40,46,46,70,85,85,21,78,159,62,77,236,115,126,126,30,19,19,19,232,234,234,130,195,
225,224,95,52,195,7,177,34,145,232,87,57,57,57,219,92,29,193,228,228,36,206,156,57,131,241,241,113,76,79,79,99,108,108,12,19,19,19,228,220,114,103,103,39,185,241,168,172,172,68,97,97,33,127,128,196,151,
97,169,38,72,75,75,195,141,27,55,112,243,230,77,124,255,251,223,199,163,143,62,42,0,144,251,185,103,83,83,19,121,43,141,70,35,191,65,254,240,16,203,178,44,122,123,123,113,245,234,85,188,247,222,123,100,
142,251,220,220,220,129,58,98,57,136,53,155,205,144,203,229,220,67,129,131,67,172,88,44,134,195,225,192,230,230,38,2,129,0,110,220,184,65,118,146,207,61,247,220,142,157,239,63,17,91,85,85,5,135,195,1,
171,213,138,164,164,164,131,117,196,114,253,83,122,189,30,43,43,43,152,159,159,71,87,87,23,102,103,103,241,218,107,95,223,20,188,242,202,171,7,134,88,46,74,201,125,121,171,84,42,208,52,253,171,125,65,
172,86,171,197,228,228,36,30,121,228,17,50,61,204,231,243,225,209,71,31,37,139,122,239,189,195,65,44,87,199,202,157,23,149,201,100,219,18,137,228,103,247,133,88,138,162,208,220,220,140,107,215,174,97,
100,100,132,20,55,204,206,206,6,117,196,30,13,98,171,171,171,97,183,219,33,151,203,193,48,204,159,246,132,88,133,66,1,175,215,139,171,87,175,10,218,79,35,5,177,46,151,139,63,25,113,119,136,205,207,207,
199,218,218,26,2,129,128,32,119,30,41,136,173,172,172,20,228,20,4,16,43,18,137,86,205,102,51,174,93,187,134,249,249,121,193,92,172,72,66,108,69,69,5,191,171,74,8,177,114,185,252,142,213,106,37,201,106,
126,16,62,146,16,235,241,120,4,73,179,144,68,108,86,86,86,212,33,214,227,241,236,13,177,20,69,189,111,54,155,49,59,59,27,245,68,44,79,241,66,33,54,53,53,21,211,211,211,130,39,156,209,128,216,160,44,213,
95,5,29,177,74,165,18,157,157,157,223,156,97,93,20,69,253,85,34,145,192,225,112,144,122,130,104,65,172,195,225,216,149,172,75,40,138,122,95,36,18,33,55,55,23,103,207,158,141,42,196,6,101,61,133,213,4,
52,77,35,43,43,11,243,243,243,81,75,196,114,222,192,131,216,208,68,108,86,86,22,150,151,151,163,54,172,139,115,7,174,219,133,97,152,59,33,16,155,145,145,129,165,165,165,168,13,235,114,56,28,130,33,57,
122,189,126,53,4,98,77,38,19,89,84,164,33,150,219,137,62,16,98,13,6,3,121,104,20,73,136,221,37,93,189,55,196,106,52,154,63,153,205,102,180,181,181,69,20,98,243,242,242,130,7,45,237,13,177,114,185,252,
103,58,157,14,122,189,30,249,249,249,168,173,173,13,59,196,218,237,246,224,22,203,125,37,98,127,197,133,0,51,50,50,80,89,89,137,218,218,218,176,64,172,211,233,12,238,26,62,88,53,1,195,48,208,106,181,80,
171,213,40,42,42,34,39,27,143,2,177,65,230,121,56,136,101,24,134,84,177,158,58,117,10,94,175,247,80,16,235,114,185,80,82,82,18,156,172,62,60,196,114,164,168,211,233,112,236,216,177,7,46,136,207,211,78,
167,19,69,69,69,40,42,42,10,158,253,248,155,176,64,44,23,115,227,198,232,113,79,203,185,250,10,174,45,144,27,151,30,27,27,11,165,82,9,185,92,46,248,77,19,139,197,191,10,43,196,178,44,11,173,86,11,189,
94,15,147,201,4,189,94,47,152,131,37,22,139,73,97,36,55,149,71,44,22,115,50,252,39,165,82,249,179,136,65,44,135,176,28,75,139,197,226,221,210,176,209,133,216,152,152,152,85,137,68,114,231,126,146,247,
45,196,254,93,67,44,151,136,165,105,186,54,162,16,171,84,42,255,89,44,22,127,196,178,236,54,215,209,194,253,186,115,183,223,177,177,177,80,171,213,208,104,52,72,73,73,217,150,203,229,247,34,86,77,160,
80,40,62,229,223,57,215,215,215,163,185,185,25,85,85,85,228,43,168,183,183,23,62,159,15,125,125,125,152,158,158,70,103,103,39,255,1,118,120,59,98,99,99,99,183,125,62,31,158,122,234,41,108,110,110,98,101,
101,5,35,35,35,232,235,235,195,192,192,0,250,251,251,49,56,56,136,254,254,126,244,244,244,144,222,198,166,166,38,120,189,94,100,101,101,113,222,25,30,136,165,105,26,78,167,19,87,174,92,33,135,239,223,
122,235,45,178,25,124,144,123,54,52,52,192,225,112,32,45,45,141,31,224,58,60,196,138,197,98,156,58,117,10,143,62,250,40,22,23,23,241,228,147,79,146,141,219,181,107,215,66,202,27,246,130,216,154,154,26,
56,28,142,224,41,154,7,135,88,174,219,115,101,101,5,139,139,139,232,233,233,193,217,179,103,5,33,212,185,185,185,125,67,108,77,77,13,138,139,139,5,253,65,135,130,216,64,32,128,229,229,101,226,158,195,
195,195,184,117,235,22,89,216,171,63,126,3,55,111,62,187,111,136,229,132,56,47,47,143,63,129,252,23,251,130,88,163,209,136,115,231,206,225,210,165,75,24,26,26,18,48,35,87,56,250,222,123,119,241,218,235,
111,30,24,98,107,106,106,200,33,177,248,248,120,72,36,146,7,87,19,24,141,70,204,206,206,98,121,121,89,48,236,109,118,118,54,196,61,143,90,77,96,181,90,185,234,139,79,247,132,88,179,217,140,203,151,47,
99,101,101,5,67,67,67,68,134,35,1,177,220,233,164,172,172,44,110,167,241,207,2,136,165,105,186,54,62,62,254,158,223,239,199,197,139,23,5,105,216,72,66,172,215,235,229,223,255,125,20,146,136,53,155,205,
184,114,229,10,70,71,71,5,11,138,36,196,86,85,85,241,23,21,58,172,43,33,33,1,139,139,139,130,207,81,164,33,54,104,81,161,16,155,152,152,136,193,193,65,12,12,12,124,51,170,9,40,138,122,63,62,62,30,253,
253,253,24,29,29,141,42,196,6,45,74,8,177,20,69,161,177,177,81,208,4,16,13,136,13,146,23,33,196,114,207,101,252,126,127,84,33,54,59,59,123,111,136,165,40,10,149,149,149,8,4,2,81,133,216,32,18,18,66,172,
76,38,131,199,227,65,32,16,136,42,196,6,37,207,66,33,182,174,174,14,129,64,32,170,29,177,54,155,237,254,195,186,234,235,235,49,53,53,21,85,136,229,15,236,10,73,196,74,165,210,237,230,230,102,248,253,126,
193,6,45,146,16,27,28,28,12,73,196,178,44,251,145,219,237,134,223,239,23,108,208,34,57,172,203,110,183,243,179,232,247,44,22,75,173,0,98,21,10,197,63,155,76,38,120,60,30,50,218,32,146,16,155,159,159,47,
120,214,183,39,196,74,165,210,79,117,58,29,204,102,51,57,10,23,9,136,181,217,108,252,157,231,131,171,9,100,50,217,54,183,48,175,215,27,118,136,205,207,207,15,22,189,47,247,213,17,75,211,52,116,58,29,18,
18,18,224,116,58,209,220,220,28,22,136,181,219,237,130,183,236,80,16,171,209,104,72,15,241,81,171,9,130,31,121,28,26,98,41,138,130,90,173,70,98,98,34,156,78,39,26,26,26,14,92,77,192,213,91,240,250,128,
194,3,177,220,43,214,208,208,128,234,234,234,144,5,85,84,84,160,162,162,2,229,229,229,187,202,112,208,35,217,240,65,172,70,163,129,193,96,128,221,110,39,242,123,242,228,73,156,56,113,2,217,217,217,48,
153,76,48,155,205,48,153,76,200,204,204,68,70,70,6,178,178,178,96,50,153,248,185,169,237,176,119,196,198,199,199,67,169,84,34,54,54,150,128,171,88,44,38,0,203,178,44,164,82,41,249,215,220,255,102,103,
65,159,70,172,154,64,38,147,221,219,9,205,16,249,229,38,139,241,79,113,239,68,116,183,105,154,254,72,46,151,255,115,164,33,182,118,175,97,93,223,66,236,255,115,16,27,241,68,44,195,48,183,147,146,146,62,
51,153,76,219,70,163,17,137,137,137,144,201,100,136,137,137,9,249,224,243,70,78,125,40,145,72,238,210,52,61,25,145,142,88,131,193,128,201,201,73,172,173,173,145,92,76,127,127,63,26,26,26,208,212,212,68,
178,118,13,13,13,164,152,166,182,182,22,89,89,89,144,201,100,127,14,251,176,46,141,70,131,64,32,128,173,173,45,188,248,226,139,120,243,205,55,73,209,95,71,71,7,122,122,122,208,221,221,77,26,2,124,62,31,
169,108,173,173,173,229,158,136,134,7,98,69,34,17,12,6,3,150,151,151,177,190,190,142,173,173,45,178,79,218,216,216,16,12,56,217,203,61,107,107,107,201,124,200,35,117,196,114,95,200,122,189,158,148,141,
246,247,247,99,126,126,158,44,234,197,23,95,196,200,200,200,3,33,182,177,177,17,46,151,43,120,52,227,225,33,118,102,102,6,171,171,171,132,25,199,199,199,241,131,31,252,128,44,236,202,149,245,125,65,44,
119,200,222,100,50,241,191,122,246,15,177,10,133,2,118,187,29,143,62,250,40,46,94,188,40,80,61,159,207,135,155,55,111,146,69,125,255,165,151,177,113,128,106,130,194,194,66,232,245,122,126,223,226,254,
32,214,235,245,98,105,105,9,103,207,158,21,4,7,251,250,250,136,234,189,247,222,93,220,126,235,109,252,195,243,47,30,8,98,185,129,74,59,109,186,247,31,214,197,48,204,164,94,175,255,115,32,16,32,33,102,
126,251,105,56,33,214,227,241,160,164,164,4,70,163,145,4,232,119,133,88,153,76,118,183,164,164,4,155,155,155,24,30,30,22,12,124,139,4,196,122,60,30,20,20,20,32,57,57,25,82,169,20,34,145,232,246,174,137,
216,218,218,90,44,45,45,145,3,247,145,134,88,183,219,141,130,130,2,168,213,106,136,197,226,207,66,32,86,44,22,111,59,157,78,204,207,207,71,21,98,93,46,23,247,172,48,20,98,101,50,25,220,110,55,22,22,22,
162,10,177,78,167,147,63,111,77,8,177,20,69,161,162,162,130,188,82,209,130,88,135,195,177,55,196,114,145,202,249,249,249,168,66,108,208,220,25,33,196,178,44,139,186,186,58,44,44,44,68,21,98,131,36,79,
8,177,44,203,162,189,189,157,152,103,180,32,54,136,172,133,16,27,23,23,135,137,137,9,140,143,143,71,21,98,131,14,73,11,33,86,169,84,98,105,105,9,67,67,67,81,133,216,160,69,9,33,54,37,37,5,23,46,92,64,
119,119,119,84,33,54,40,250,45,132,88,147,201,132,205,205,77,116,118,118,70,21,98,249,139,10,129,216,148,148,148,109,238,177,90,52,33,150,167,49,161,213,4,9,9,9,159,233,245,122,120,60,30,180,181,181,69,
5,98,131,186,61,67,19,177,52,77,223,214,104,52,72,76,76,252,106,33,59,65,135,72,65,44,71,141,220,45,153,76,38,187,123,223,142,88,157,78,135,242,242,114,65,243,105,56,33,182,164,164,132,116,156,237,188,
74,127,214,106,181,147,247,29,214,21,31,31,15,173,86,11,183,219,45,168,40,8,7,196,150,150,150,194,104,52,10,206,42,239,27,98,213,106,53,116,58,29,58,59,59,195,86,77,144,159,159,31,220,28,127,48,136,229,
10,178,114,115,115,9,93,31,5,98,11,10,10,248,131,78,143,6,177,6,131,1,217,217,217,104,107,107,19,44,232,32,16,187,11,230,31,13,98,165,82,41,12,6,3,121,251,184,197,84,85,85,237,57,172,43,248,21,10,74,85,
135,15,98,19,18,18,96,54,155,145,147,147,67,208,213,98,177,32,45,45,13,22,139,5,57,57,57,200,205,205,69,113,113,49,108,54,27,74,75,75,97,181,90,67,98,222,97,133,88,181,90,253,103,110,38,187,84,42,133,
76,38,35,240,202,253,103,92,61,1,199,67,188,99,114,145,233,136,149,201,100,147,44,203,222,229,170,9,56,112,229,119,195,114,53,5,156,12,51,12,179,77,211,244,103,50,153,236,246,67,25,214,245,45,196,254,
93,65,172,74,165,250,23,177,88,252,111,15,29,98,185,106,2,165,82,185,221,218,218,74,106,48,53,26,13,84,42,21,73,38,6,119,112,68,116,88,23,203,178,56,121,242,36,97,198,229,229,101,12,12,12,96,96,96,0,205,
205,205,228,12,251,177,99,199,144,158,158,142,244,244,116,104,181,90,40,20,10,40,149,202,207,196,98,241,157,176,66,172,66,161,32,33,212,43,87,174,224,205,55,223,20,184,231,233,211,167,201,148,96,46,170,
219,217,217,137,190,190,62,244,247,247,115,227,206,182,195,2,177,20,69,65,46,151,195,235,245,98,121,121,25,115,115,115,152,155,155,35,123,165,23,94,120,65,112,122,146,223,17,251,213,205,71,43,26,27,27,
201,52,177,157,222,235,223,30,9,98,117,58,29,25,71,53,59,59,139,158,158,30,76,76,76,224,213,87,95,37,145,221,169,169,169,7,66,108,93,93,29,242,243,243,57,246,57,56,196,210,52,253,219,248,248,120,84,86,
86,98,125,125,29,43,43,43,164,216,175,189,189,29,3,3,3,120,230,153,103,200,171,181,185,249,248,3,33,182,190,190,30,21,21,21,200,201,201,9,206,16,239,15,98,13,6,3,250,250,250,112,225,194,5,204,206,206,
10,220,179,189,189,125,231,149,122,109,199,62,223,195,173,103,255,1,43,251,132,88,174,36,153,215,157,126,127,136,229,159,220,190,120,241,34,198,198,198,246,132,216,59,119,238,226,39,63,185,141,151,95,
254,17,174,111,61,117,160,106,130,252,252,124,40,149,74,50,169,238,190,137,88,157,78,247,217,210,210,18,150,150,150,224,243,249,4,238,25,10,177,135,31,214,229,118,187,145,153,153,9,165,82,201,221,60,236,
221,17,171,86,171,113,227,198,141,168,64,108,105,105,41,44,22,11,153,75,186,103,53,1,55,229,158,79,213,145,130,88,78,241,120,25,245,221,135,117,101,102,102,146,6,230,104,64,44,55,237,41,62,62,30,52,77,
135,66,44,69,81,176,88,44,216,220,220,36,31,238,104,64,108,69,69,5,169,168,147,72,36,255,34,128,88,154,166,113,236,216,49,172,175,175,71,21,98,43,43,43,201,162,104,154,254,55,1,196,82,20,69,190,74,162,
9,177,21,21,21,252,97,56,66,136,85,40,20,232,233,233,193,153,51,103,162,10,177,65,221,195,66,136,77,76,76,196,234,234,42,153,202,19,45,136,61,121,242,228,222,16,171,215,235,177,181,181,5,159,207,23,85,
136,205,200,200,216,27,98,185,250,20,254,183,125,52,32,150,55,168,75,8,177,44,203,254,155,195,225,32,231,218,163,5,177,46,151,43,248,14,250,107,136,85,42,149,255,146,145,145,65,18,214,209,130,88,135,195,
193,175,62,12,133,88,133,66,177,157,157,157,141,214,214,214,168,65,108,208,195,162,221,135,117,25,12,6,228,230,230,18,243,140,52,196,6,93,14,118,175,38,16,139,197,224,22,198,111,2,136,20,196,242,155,74,
238,219,17,43,147,201,144,146,146,2,183,219,141,214,214,214,136,64,172,203,229,66,113,113,49,159,171,63,187,111,34,150,162,168,47,165,82,41,82,82,82,224,243,249,194,158,136,45,45,45,69,78,78,142,160,222,
87,169,84,222,121,32,196,114,49,202,162,162,162,176,14,235,42,45,45,133,217,108,230,39,56,14,14,177,169,169,169,112,56,28,104,105,105,17,148,89,29,20,98,157,78,39,25,111,205,151,97,138,162,126,123,96,
136,141,141,141,133,193,96,64,123,123,123,200,25,170,253,66,172,195,225,64,122,122,186,224,122,116,36,136,229,238,255,142,29,59,6,167,211,9,143,199,131,178,178,50,210,19,203,109,61,238,183,32,147,201,
180,219,153,247,163,67,172,78,167,35,7,236,21,10,5,212,106,53,226,226,226,160,211,233,144,145,145,129,227,199,143,195,102,179,145,154,157,221,58,91,194,14,177,44,203,222,145,203,229,159,113,201,87,153,
76,6,153,76,6,185,92,14,133,66,65,32,54,38,38,6,201,201,201,48,153,76,187,185,121,228,135,117,241,255,129,124,124,229,22,185,115,163,185,253,80,134,117,237,38,121,34,145,232,223,88,150,253,151,111,33,
246,27,11,177,52,77,191,161,86,171,127,21,23,23,119,79,36,18,125,254,208,33,86,34,145,108,167,165,165,161,189,189,29,61,61,61,168,174,174,38,201,68,94,214,46,58,16,203,245,34,20,23,23,99,110,110,14,143,
61,246,24,158,124,242,73,44,46,46,162,179,179,19,77,77,77,112,56,28,200,201,201,33,97,64,238,183,147,166,233,143,195,158,136,229,190,164,123,123,123,177,190,190,142,165,165,37,210,90,249,202,43,175,144,
225,1,167,79,159,198,233,211,167,209,212,212,68,182,62,86,171,21,22,139,5,58,157,110,91,38,147,253,39,77,211,23,143,12,177,74,165,18,30,143,7,171,171,171,228,222,112,116,116,20,255,244,79,255,68,42,90,
185,161,20,124,247,228,22,215,210,210,130,142,142,14,120,60,30,24,12,6,72,36,146,15,15,5,177,52,77,191,31,31,31,143,210,210,82,204,207,207,99,125,125,29,211,211,211,4,63,6,6,6,4,135,239,207,156,57,115,
95,136,173,175,175,71,85,85,21,178,178,178,184,173,203,251,251,134,88,134,97,46,106,52,154,15,109,54,27,70,71,71,113,238,220,57,4,2,1,210,4,192,253,116,119,119,227,137,39,190,238,141,125,236,241,235,15,
132,88,238,173,84,169,84,156,224,237,15,98,53,26,205,127,158,62,125,26,171,171,171,152,159,159,199,240,240,112,8,196,142,141,141,227,123,47,190,132,31,189,250,58,222,123,239,61,188,253,246,219,184,113,
227,38,230,23,46,220,23,98,235,234,234,80,89,89,137,140,140,12,254,70,239,193,137,88,179,217,140,245,245,117,204,204,204,144,129,205,123,65,236,15,95,121,29,47,190,248,3,220,122,246,31,112,233,242,198,
190,33,182,180,180,20,106,181,154,76,182,191,47,196,210,52,253,113,118,118,54,30,127,252,241,136,66,44,87,22,194,75,112,236,13,177,18,137,4,149,149,149,184,124,249,114,84,32,86,163,209,128,247,216,119,
119,136,165,40,10,129,64,0,103,207,158,141,10,196,158,56,113,2,50,153,108,239,97,93,20,69,125,169,84,42,177,188,188,140,145,145,145,168,64,108,89,89,25,82,83,83,177,211,166,179,59,196,154,205,102,172,
172,172,144,122,130,104,64,172,205,102,67,124,124,60,215,97,245,134,0,98,25,134,65,126,126,62,46,93,186,4,159,207,23,53,136,229,158,62,236,76,55,248,149,0,98,149,74,37,122,123,123,113,254,252,121,242,
42,69,3,98,43,43,43,249,211,233,238,9,32,86,167,211,225,218,181,107,24,27,27,139,42,196,86,84,84,240,11,253,62,23,64,108,126,126,62,174,93,187,246,80,134,117,5,245,48,126,5,177,98,177,248,243,254,254,
126,44,46,46,10,30,187,70,3,98,247,202,14,151,36,38,38,222,107,108,108,36,188,24,45,136,229,30,72,238,10,177,106,181,250,87,38,147,9,221,221,221,104,107,107,139,26,196,150,148,148,240,159,54,8,33,150,
166,233,55,146,146,146,144,157,157,13,159,207,23,53,136,229,215,18,236,10,177,98,177,120,219,104,52,162,170,170,138,236,183,35,13,177,65,79,27,118,135,88,150,101,145,150,150,134,158,158,158,168,64,236,
190,58,98,185,19,218,54,155,13,221,221,221,17,135,216,248,248,248,253,65,44,69,81,72,79,79,71,87,87,87,196,18,177,28,196,242,32,237,227,7,66,108,76,76,12,242,243,243,209,212,212,20,118,136,229,236,147,
255,160,104,95,137,88,138,162,96,50,153,80,85,85,133,150,150,150,176,65,172,195,225,192,137,19,39,160,211,233,248,135,170,255,115,223,16,171,86,171,97,177,88,208,213,213,69,210,176,71,73,196,218,237,118,
24,12,6,193,87,10,77,211,31,74,36,146,139,251,134,88,154,166,223,55,26,141,40,41,41,65,85,85,21,170,171,171,15,13,177,102,179,153,127,232,158,219,105,190,127,40,136,85,42,149,31,234,116,58,104,181,90,
112,21,24,54,155,13,37,37,37,40,45,45,133,219,237,222,115,65,92,61,202,46,105,216,163,65,44,77,211,23,21,10,197,127,42,149,74,40,20,10,66,135,18,137,4,106,181,26,41,41,41,56,126,252,56,92,46,23,233,75,
40,47,47,71,113,113,49,87,160,22,57,136,221,185,47,251,152,187,243,96,24,134,139,180,33,38,38,6,114,185,156,60,104,202,201,201,9,233,217,136,42,196,114,41,88,78,134,229,114,57,151,36,227,62,59,15,31,98,
105,154,254,156,101,217,123,12,195,252,138,97,152,55,190,133,216,111,124,53,1,203,178,31,26,141,198,47,118,98,222,159,63,52,136,165,40,234,75,150,101,97,54,155,209,221,221,77,170,87,11,11,11,131,103,135,
70,7,98,25,134,129,94,175,71,99,99,35,22,22,22,176,182,182,134,167,159,126,26,143,63,254,56,218,219,219,209,210,210,2,187,221,142,132,132,4,210,224,204,93,18,194,14,177,34,145,8,122,189,30,85,85,85,8,
4,2,152,159,159,199,244,244,52,158,120,226,9,178,69,153,157,157,37,77,167,205,205,205,164,91,95,167,211,65,169,84,66,34,145,132,39,17,27,27,27,139,236,236,108,156,62,125,26,83,83,83,100,49,62,159,15,189,
189,189,130,170,130,243,231,207,147,59,33,110,187,195,237,245,203,203,203,97,181,90,185,230,211,79,24,134,121,242,64,16,203,178,236,147,122,189,254,94,89,89,25,250,251,251,201,43,51,49,49,65,138,25,218,
219,219,209,213,213,133,205,205,77,178,40,254,96,10,62,196,126,181,23,107,64,99,99,35,42,43,43,97,52,26,193,178,236,189,3,37,98,147,147,147,63,25,24,24,192,242,242,50,102,103,103,49,58,58,26,18,30,108,
109,109,69,111,111,159,160,17,96,243,177,107,15,132,88,174,218,144,87,230,176,191,68,108,78,78,14,214,215,215,49,49,49,17,226,158,124,136,125,241,251,63,196,11,47,188,132,183,222,122,27,111,190,121,27,
155,155,215,31,8,177,181,181,181,176,90,173,252,103,130,15,134,88,153,76,182,93,83,83,131,149,149,149,125,65,236,211,207,60,143,167,159,126,14,215,183,158,196,226,226,242,3,33,182,174,174,14,229,229,229,
72,76,76,220,31,196,82,20,245,145,78,167,195,204,204,12,230,230,230,34,10,177,57,57,57,251,131,88,110,111,254,216,99,143,97,120,120,56,162,16,107,183,219,17,19,19,243,96,136,101,89,22,37,37,37,216,216,
216,136,248,176,46,46,250,198,85,227,223,23,98,47,92,184,32,208,225,72,65,108,85,85,21,236,118,59,212,106,53,247,54,126,185,43,196,166,165,165,225,153,103,158,33,149,172,145,134,88,183,219,13,139,197,
194,233,240,238,213,4,103,206,156,33,3,187,162,5,177,92,213,225,206,103,75,88,77,160,211,233,240,221,239,126,23,195,195,195,81,77,196,150,151,151,35,59,59,155,123,122,250,33,31,98,63,239,232,232,192,218,
218,154,160,239,60,90,16,203,75,156,253,95,2,177,106,181,26,231,207,159,135,223,239,23,244,254,68,3,98,119,169,100,253,10,98,141,70,227,255,237,232,232,192,192,192,64,212,33,214,237,118,243,187,24,9,89,
151,176,44,251,97,90,90,26,185,54,69,51,17,27,52,214,76,88,77,96,48,24,80,86,86,38,104,1,136,70,53,65,208,200,23,97,53,129,68,34,129,201,100,66,75,75,11,17,226,72,67,108,80,105,109,40,196,82,20,245,165,
78,167,67,81,81,17,121,181,34,13,177,233,233,233,15,134,88,134,97,144,158,158,142,150,150,22,114,176,43,146,195,186,130,12,125,111,136,213,233,116,112,185,92,232,233,233,137,120,71,236,190,134,117,81,
20,245,145,72,36,34,16,43,52,207,240,66,108,16,87,127,244,64,136,77,77,77,69,77,77,13,218,218,218,34,2,177,14,135,131,223,6,176,189,47,136,85,42,149,200,205,205,69,119,119,119,88,19,177,220,130,114,114,
114,8,23,29,40,17,107,50,153,80,89,89,41,88,208,81,33,246,216,177,99,208,235,245,124,140,253,228,64,137,216,248,248,248,123,137,137,137,72,77,77,197,177,99,199,96,179,217,224,241,120,14,4,177,101,101,
101,176,90,173,200,200,200,128,70,163,17,12,96,162,105,250,30,77,211,79,30,8,98,197,98,241,147,177,177,177,159,168,213,106,168,84,42,40,20,10,196,197,197,193,100,50,161,168,168,136,84,128,237,182,160,
178,178,50,216,108,54,152,76,38,196,196,196,8,206,191,243,102,130,28,45,17,203,48,204,182,76,38,35,206,169,80,40,160,209,104,80,80,80,64,106,192,184,87,167,184,184,24,153,153,153,220,0,230,224,209,84,
95,134,189,35,150,83,23,137,68,2,133,66,1,149,74,5,165,82,137,172,172,44,216,237,118,50,239,131,223,16,240,80,18,177,50,153,12,74,165,146,36,98,131,94,157,47,31,38,196,126,190,51,202,236,255,238,108,105,
191,173,38,248,187,234,136,253,92,169,84,34,57,57,249,11,134,97,62,124,104,16,203,125,208,117,58,29,154,155,155,49,55,55,135,190,190,62,20,22,22,66,171,213,146,187,222,168,64,44,119,57,72,79,79,71,111,
111,47,54,54,54,176,180,180,132,167,158,122,138,28,23,46,47,47,39,199,80,120,39,179,195,11,177,50,153,108,59,62,62,30,38,147,9,78,167,19,195,195,195,88,91,91,195,226,226,34,134,135,135,17,8,4,200,54,229,
241,199,31,71,75,75,11,60,30,15,74,75,75,5,23,210,35,119,196,178,44,251,100,82,82,210,39,69,69,69,104,104,104,192,192,192,0,230,231,231,177,184,184,136,217,217,89,193,8,152,165,165,37,193,188,172,142,
142,14,98,157,30,143,7,78,167,19,121,121,121,72,74,74,226,186,94,14,215,17,155,156,156,124,111,96,96,0,231,207,159,71,32,16,192,228,228,36,6,7,7,5,68,196,221,77,95,188,120,145,44,234,209,71,31,37,55,30,
124,90,172,169,169,33,55,11,59,66,124,111,71,136,247,223,17,91,84,84,116,95,247,108,109,109,69,79,143,15,51,51,1,220,184,241,20,15,99,31,71,123,123,231,174,29,177,220,246,199,229,114,113,207,147,63,217,
55,196,198,197,197,97,104,104,8,231,207,159,127,32,196,126,5,32,207,225,205,55,111,227,246,237,219,88,95,223,188,47,196,114,111,105,106,106,234,254,33,150,162,168,47,45,22,11,86,86,86,48,62,62,254,96,
136,125,250,31,240,216,99,79,224,250,214,147,184,186,241,24,166,166,231,30,8,177,53,53,53,56,118,236,24,247,29,249,224,97,93,34,145,8,205,205,205,88,89,89,17,140,162,10,39,196,214,212,212,160,168,168,
136,79,139,247,135,216,148,148,20,44,46,46,98,122,122,58,162,16,235,118,187,131,15,140,237,14,177,12,195,192,235,245,226,145,71,30,17,140,162,138,84,71,44,215,187,120,95,136,53,24,12,228,18,192,61,127,
137,100,34,214,102,179,9,246,94,33,16,203,178,44,78,159,62,141,149,149,21,18,136,143,6,196,242,198,163,135,66,108,122,122,58,86,86,86,224,247,251,209,217,217,25,53,136,61,113,226,196,238,29,177,44,203,
126,216,215,215,135,229,229,101,18,209,141,22,196,58,157,78,126,85,221,215,213,4,38,147,233,255,46,44,44,96,114,114,146,188,109,209,132,88,30,9,125,157,136,85,169,84,168,175,175,199,208,208,208,67,129,
216,32,129,33,213,4,159,91,44,22,65,105,109,180,32,150,219,123,5,147,117,9,69,81,239,107,181,90,116,117,117,161,171,171,43,170,16,91,82,82,194,23,152,15,5,16,203,133,228,199,199,199,163,6,177,101,101,
101,193,179,214,132,16,203,245,44,246,245,245,69,13,98,237,118,187,128,24,119,133,88,133,66,129,252,252,124,244,247,247,71,60,17,91,86,86,22,172,121,187,67,172,88,44,134,197,98,193,228,228,164,96,147,
22,9,136,181,217,108,193,53,136,123,67,108,74,74,10,186,186,186,34,94,77,144,149,149,181,127,136,213,106,181,112,58,157,232,237,237,13,218,49,134,15,98,185,39,162,251,134,88,169,84,186,109,177,88,48,56,
56,24,178,141,13,23,196,6,125,223,237,175,35,214,100,50,193,235,245,162,190,190,30,181,181,181,97,133,216,210,210,82,126,121,195,254,33,54,62,62,254,19,157,78,7,139,197,2,171,213,10,183,219,125,100,136,
229,203,48,47,143,176,255,142,216,29,239,188,199,213,23,234,116,58,228,230,230,194,225,112,16,140,221,111,34,182,180,180,20,39,79,158,132,201,100,130,86,171,229,39,99,239,81,71,233,136,85,169,84,224,64,
214,98,177,160,180,180,148,44,102,175,5,21,22,22,34,37,37,5,42,149,138,148,143,238,60,39,6,77,211,159,236,200,240,209,58,98,105,154,70,76,76,12,226,226,226,16,27,27,139,156,156,28,148,149,149,9,22,196,
61,115,209,106,181,228,78,133,97,24,136,68,34,110,65,219,17,129,88,145,72,132,152,152,24,168,213,106,104,181,90,228,229,229,161,184,184,24,57,57,57,80,171,213,216,173,166,53,106,16,203,48,12,84,42,21,
98,99,99,67,172,60,56,40,24,117,136,101,24,230,67,153,76,246,127,119,94,161,207,191,237,136,253,187,29,214,197,48,204,231,113,113,113,247,20,10,197,175,118,62,127,209,135,88,145,72,4,133,66,1,173,86,11,
155,205,134,190,190,62,116,119,119,147,100,44,55,172,43,98,16,75,211,244,199,220,215,131,66,161,128,217,108,70,113,113,49,90,90,90,112,246,236,89,92,188,120,17,87,174,92,193,214,214,22,252,126,63,236,
118,59,244,122,125,248,19,177,59,231,220,255,211,98,177,108,159,58,117,10,149,149,149,24,26,26,194,204,204,12,252,126,63,166,167,167,49,51,51,131,145,145,17,76,76,76,224,149,87,94,249,106,140,208,235,
175,163,173,173,141,228,88,120,155,185,163,15,235,138,143,143,255,176,181,181,21,103,207,158,37,221,176,126,191,159,188,77,252,58,196,233,233,105,188,251,238,187,184,123,247,46,94,122,233,37,12,14,14,
146,179,236,220,214,151,247,68,244,55,135,174,38,56,121,242,36,30,125,244,81,76,77,77,133,28,43,231,67,108,75,75,11,206,156,57,67,54,117,207,60,243,12,124,62,31,154,154,154,80,95,95,15,175,215,139,202,
202,74,20,23,23,35,53,53,149,171,99,125,159,58,232,176,174,196,196,68,76,76,76,144,209,28,247,131,216,51,115,243,120,236,241,235,60,182,190,142,246,246,118,1,196,114,80,86,90,90,138,130,130,2,174,14,227,
195,157,150,137,7,67,44,69,81,112,185,92,164,69,226,65,16,123,117,227,26,110,220,184,137,219,183,111,227,206,157,59,88,91,187,138,238,61,58,98,185,145,160,118,187,157,251,226,254,207,125,65,108,108,108,
44,174,94,189,138,185,185,185,251,118,196,114,16,187,241,232,117,92,186,188,129,199,30,223,194,149,245,141,125,65,108,89,89,25,52,26,13,247,125,121,127,136,229,218,42,183,182,182,4,109,18,225,134,88,175,
215,139,164,164,36,114,148,238,190,16,171,213,106,177,181,181,133,64,32,16,81,136,173,174,174,38,83,121,238,219,17,43,147,201,48,50,50,130,245,245,117,193,219,22,41,136,13,34,160,80,136,101,89,118,219,
106,181,98,107,107,11,220,136,151,72,67,172,221,110,23,236,189,4,16,75,211,244,27,169,169,169,88,89,89,17,76,16,139,52,196,150,149,149,241,43,164,133,16,171,209,104,126,117,250,244,105,92,185,114,133,
36,244,163,1,177,110,183,155,244,195,134,64,172,78,167,187,215,221,221,141,153,153,25,193,149,58,26,16,123,252,248,241,224,131,136,95,65,44,195,48,159,231,229,229,97,118,118,54,234,137,216,226,226,98,
126,199,139,176,35,150,51,79,238,64,78,180,32,214,229,114,5,199,42,191,238,136,213,106,181,59,206,233,139,122,34,150,55,69,236,115,1,196,198,196,196,224,196,137,19,24,29,29,141,42,196,150,151,151,35,51,
51,147,127,43,255,53,196,138,68,34,88,44,22,12,15,15,147,246,211,104,36,98,139,139,139,201,161,50,154,166,127,21,2,177,105,105,105,164,198,62,26,16,27,52,23,11,12,195,188,17,2,177,49,49,49,24,28,28,36,
27,180,40,87,19,108,239,10,177,20,69,145,48,68,164,135,117,57,28,142,224,179,128,123,119,196,102,101,101,97,102,102,70,112,134,42,18,16,203,251,112,223,31,98,105,154,254,216,108,54,195,239,247,147,253,
117,36,32,182,168,168,72,208,126,250,64,136,53,26,141,100,72,87,36,32,150,27,214,117,32,136,85,169,84,255,153,156,156,140,244,244,116,216,237,118,50,19,43,92,16,123,234,212,41,193,241,242,125,65,172,72,
36,186,168,80,40,62,140,143,143,135,90,173,134,217,108,134,219,237,70,125,125,125,88,32,150,183,5,62,212,176,174,247,185,60,103,76,76,12,178,178,178,224,241,120,200,254,250,32,195,186,56,208,15,170,29,
127,255,208,16,203,77,214,212,104,52,80,40,20,40,40,40,32,183,73,187,45,136,123,58,101,181,90,145,151,151,7,147,201,68,254,143,241,222,182,15,169,112,12,235,226,50,157,26,141,6,38,147,9,101,101,101,228,
206,151,235,134,61,126,252,56,244,122,61,98,99,99,73,230,147,83,97,30,200,254,167,66,161,184,24,86,136,21,139,197,136,143,143,135,70,163,65,110,110,46,242,242,242,96,52,26,161,80,40,136,2,243,121,154,
183,129,251,56,162,16,203,48,204,182,90,173,70,108,108,44,49,114,145,72,4,154,166,201,95,31,10,196,138,68,162,55,100,50,217,175,88,150,189,71,211,244,231,223,14,235,250,187,135,88,238,237,19,139,197,255,
38,151,203,255,229,161,64,44,247,129,150,201,100,164,70,218,225,112,192,235,245,162,164,164,4,44,203,70,14,98,229,114,249,29,173,86,251,153,82,169,132,94,175,71,102,102,38,50,51,51,145,159,159,143,186,
186,58,116,116,116,96,120,120,24,75,75,75,152,155,155,195,234,234,42,230,230,230,224,114,185,160,213,106,185,231,53,225,77,196,150,148,148,108,143,143,143,99,126,126,30,129,64,0,195,195,195,152,154,154,
66,32,16,128,223,239,199,200,200,8,6,7,7,209,223,223,143,171,87,175,226,238,221,187,248,201,79,126,130,190,190,62,193,83,173,176,64,44,77,211,191,77,78,78,198,230,230,38,22,22,22,48,48,48,128,142,142,
14,116,118,118,18,149,225,223,188,14,13,13,225,133,23,94,32,251,168,201,201,73,84,86,86,226,212,169,83,193,124,125,56,136,165,40,234,191,180,90,45,250,250,250,176,182,182,246,64,136,109,105,105,129,223,
239,199,91,111,189,133,187,119,239,226,71,63,250,17,70,70,70,80,95,95,79,70,3,29,59,118,140,159,216,248,175,3,67,44,69,81,168,175,175,199,234,234,42,38,38,38,30,8,177,51,179,115,184,116,249,10,121,149,
158,126,250,105,116,119,119,147,109,53,119,106,164,164,164,4,41,41,41,92,101,230,111,247,157,136,165,40,10,73,73,73,184,121,243,230,174,195,186,118,131,216,149,149,53,92,191,126,3,239,189,247,30,238,222,
189,139,203,107,235,232,236,236,222,117,88,151,205,102,67,86,86,22,247,170,237,47,17,171,80,40,48,58,58,138,43,87,174,236,27,98,47,174,172,97,121,121,21,79,220,120,10,155,155,215,48,48,56,182,103,71,44,
55,217,62,47,47,143,219,210,220,185,47,196,42,20,10,120,60,30,108,110,110,98,108,108,44,98,16,91,93,93,13,155,205,6,177,88,12,145,72,244,217,158,16,203,178,44,220,110,55,86,86,86,4,89,207,72,65,172,203,
229,122,112,34,182,160,160,0,87,175,94,197,153,51,103,4,179,177,34,5,177,46,151,75,176,27,13,129,88,149,74,181,189,181,181,133,133,133,5,193,231,40,146,16,91,94,94,206,55,79,33,196,170,213,234,127,41,
43,43,195,214,214,150,96,65,145,134,216,202,202,74,254,105,109,33,196,178,44,251,111,39,79,158,20,200,112,180,18,177,123,29,142,46,161,40,234,253,236,236,108,92,184,112,33,234,16,187,215,132,123,210,17,
27,8,4,208,221,221,29,85,136,13,106,0,16,66,172,86,171,69,75,75,11,250,251,251,163,10,177,193,157,158,2,136,85,40,20,112,187,221,152,152,152,136,42,196,150,150,150,242,159,49,11,135,117,81,20,133,172,
172,44,65,35,124,52,32,214,229,114,241,19,177,194,106,2,154,166,145,153,153,137,165,165,37,178,168,104,64,44,191,188,150,63,42,79,208,101,61,55,55,71,6,118,69,26,98,57,102,228,158,194,139,197,226,127,
9,129,216,140,140,12,204,205,205,161,189,189,61,106,195,186,82,82,82,238,15,177,122,189,30,171,171,171,130,39,229,145,132,88,62,87,239,9,177,177,177,177,240,251,253,100,81,145,130,88,167,211,25,108,85,
123,67,172,86,171,253,44,47,47,15,173,173,173,130,118,173,112,67,108,94,94,94,200,25,229,61,33,86,42,149,222,73,78,78,38,41,159,234,234,234,136,64,108,80,68,119,127,137,216,216,216,88,112,83,158,248,251,
235,112,12,235,10,234,221,216,127,34,150,166,233,223,198,196,196,64,171,213,146,30,98,254,54,246,176,16,155,155,155,27,92,60,122,96,136,253,47,46,43,21,31,31,143,236,236,108,84,85,85,29,26,98,249,79,171,
142,52,172,139,15,177,92,150,152,191,168,221,22,196,45,162,164,164,4,54,155,13,14,135,3,5,5,5,193,84,253,219,176,64,172,84,42,133,78,167,67,106,106,42,121,251,42,43,43,137,143,115,19,200,117,58,29,212,
106,53,98,98,98,16,19,19,67,250,23,249,245,22,97,77,196,50,12,3,141,70,3,153,76,134,99,199,142,193,96,48,32,46,46,142,252,131,197,98,49,249,217,57,128,207,87,225,207,148,74,229,157,136,65,44,203,178,219,
220,171,32,22,139,5,36,29,220,158,27,85,136,149,201,100,255,178,243,173,30,34,121,223,66,236,223,53,196,70,173,154,128,97,152,73,153,76,118,151,123,226,192,109,204,248,191,117,74,165,18,114,185,28,58,
157,14,105,105,105,72,78,78,222,86,42,149,159,209,52,125,59,34,137,88,189,94,255,231,242,242,114,180,183,183,147,121,49,92,105,104,93,93,29,57,210,57,57,57,137,225,225,97,204,205,205,33,16,8,224,228,201,
147,220,141,65,248,59,98,231,231,231,113,241,226,69,210,126,58,60,60,140,129,129,1,12,12,12,8,32,214,231,243,97,124,124,28,111,190,249,38,158,127,254,121,248,124,62,126,240,244,203,176,116,196,42,20,10,
148,151,151,227,169,167,158,194,240,240,48,225,161,189,220,179,163,163,3,235,235,235,100,47,181,182,182,70,198,183,240,166,240,28,30,98,185,188,231,197,139,23,67,250,61,119,91,208,233,211,167,49,52,52,
132,23,95,124,145,44,42,16,8,160,174,174,14,118,187,29,70,163,145,127,68,229,112,16,107,48,24,176,181,181,133,185,185,185,93,59,98,131,33,118,116,116,2,139,231,190,46,116,224,50,196,28,196,218,237,118,
100,102,102,242,133,120,255,29,177,92,199,112,32,16,192,165,75,151,246,5,177,23,87,214,48,191,112,1,215,174,61,193,171,190,184,134,246,246,118,178,81,228,160,172,160,160,0,241,241,241,220,222,106,127,
16,107,52,26,49,61,61,141,213,213,85,12,13,13,237,27,98,231,206,158,199,210,210,10,158,251,238,243,248,199,127,124,1,99,227,83,123,14,235,202,205,205,229,182,51,15,238,136,213,104,52,88,89,89,193,202,
202,10,134,134,134,200,231,232,32,16,59,48,56,138,193,161,241,251,118,196,186,221,110,82,131,207,178,236,159,37,18,201,100,8,196,50,12,115,59,57,57,25,179,179,179,88,95,95,23,180,0,68,10,98,221,110,55,
153,216,202,48,204,221,16,136,53,24,12,159,45,46,46,134,188,101,145,132,88,175,215,203,191,255,11,133,216,180,180,180,237,39,158,120,34,100,65,145,132,216,170,170,42,254,162,66,135,117,153,76,38,44,46,
46,70,21,98,171,170,170,130,67,205,194,106,2,131,193,128,249,249,249,168,38,98,61,30,79,240,177,223,127,13,25,214,53,60,60,140,161,161,161,168,66,108,208,205,169,16,98,227,226,226,208,210,210,130,209,
209,209,168,66,44,47,227,25,10,177,18,137,4,53,53,53,152,153,153,137,42,196,6,159,71,22,64,44,203,178,112,56,28,164,158,32,90,16,27,148,96,20,66,172,88,44,70,121,121,121,212,33,214,106,181,238,221,17,
75,81,20,220,110,55,2,129,64,84,59,98,139,139,139,247,174,38,144,74,165,40,41,41,17,44,42,90,16,203,221,210,239,58,172,203,235,245,146,69,69,3,98,185,109,204,125,33,214,102,179,97,122,122,90,80,75,16,
201,97,93,86,171,149,63,185,231,179,16,136,149,74,165,119,51,50,50,208,220,220,140,150,150,150,136,39,98,109,54,155,96,148,144,76,38,187,29,2,177,34,145,104,82,163,209,252,89,171,213,34,39,39,7,94,175,
55,98,195,186,242,243,243,15,86,77,192,245,15,115,175,90,184,33,214,106,181,242,27,111,246,15,177,28,144,113,133,14,252,78,178,163,66,44,239,225,208,193,59,98,119,198,141,33,62,62,30,46,151,139,12,34,
57,44,196,150,148,148,4,103,134,15,15,177,113,113,113,72,72,72,32,166,126,208,97,93,123,244,12,31,13,98,105,154,134,70,163,129,90,173,134,211,233,68,77,77,205,3,23,228,114,185,200,130,156,78,103,72,27,
115,88,58,98,185,252,139,86,171,69,89,89,25,170,171,171,81,82,82,130,162,162,34,88,173,86,100,101,101,33,51,51,19,169,169,169,72,77,77,69,122,122,58,140,70,35,76,38,19,140,70,163,32,123,16,118,136,141,
139,139,3,23,58,85,40,20,4,90,37,18,9,129,215,224,31,94,195,219,159,35,146,136,101,24,230,182,82,169,252,76,38,147,109,115,11,225,21,52,8,170,9,118,146,177,220,176,174,187,70,163,113,242,161,12,235,250,
22,98,191,241,16,75,211,116,237,55,6,98,185,68,108,92,92,220,61,179,217,188,205,181,150,112,223,244,92,224,129,11,199,179,44,11,137,68,178,45,22,139,63,146,74,165,255,28,177,142,216,172,172,44,76,76,76,
96,114,114,146,156,217,106,109,109,69,91,91,27,188,94,47,26,26,26,224,241,120,200,241,22,238,175,39,78,156,0,195,48,159,134,29,98,13,6,3,214,215,215,113,238,220,57,12,14,14,18,251,236,238,238,70,103,103,
39,122,122,122,224,243,249,208,215,215,135,190,190,62,12,14,14,98,121,121,153,160,91,66,66,2,196,98,113,120,134,117,81,20,133,212,212,84,76,79,79,227,210,165,75,130,58,205,189,220,179,185,185,25,83,83,
83,184,125,251,54,201,124,30,63,126,156,191,101,57,26,196,234,245,122,44,47,47,99,121,121,57,68,100,246,90,80,240,16,175,107,215,174,161,166,166,6,217,217,217,80,40,20,220,5,246,224,16,43,22,139,145,150,
150,134,233,233,105,92,190,124,25,253,253,253,2,29,222,11,98,125,190,126,76,77,77,11,144,109,122,122,26,117,117,117,228,200,83,80,153,195,254,32,150,166,105,88,173,86,44,45,45,97,105,105,41,228,21,218,
19,98,231,151,48,61,61,135,141,71,31,23,196,117,125,62,31,193,178,162,162,162,96,186,126,48,196,170,84,170,109,187,221,142,245,245,117,204,207,207,11,154,0,246,3,177,19,147,179,88,89,185,132,87,94,249,
209,87,147,234,2,103,5,16,203,5,232,77,38,19,183,176,251,67,172,66,161,248,148,251,173,153,155,155,19,100,61,15,154,136,157,59,123,14,19,147,211,104,111,239,9,129,88,143,199,3,171,213,138,180,180,52,242,
220,102,215,68,172,82,169,252,103,175,215,139,27,55,110,96,124,124,60,36,51,28,110,136,173,172,172,68,73,73,9,25,33,187,107,34,150,101,217,143,218,219,219,49,63,63,31,242,184,35,82,16,235,241,120,144,
151,151,7,153,76,6,134,97,238,197,198,198,214,10,32,86,42,149,110,55,53,53,225,204,153,51,81,133,216,210,210,82,240,74,108,132,16,43,147,201,80,95,95,47,104,148,136,6,196,58,28,142,189,59,98,105,154,134,
215,235,37,73,179,104,65,108,97,97,225,222,16,43,147,201,80,85,85,133,51,103,206,68,21,98,79,157,58,181,55,196,114,45,37,115,115,115,81,133,216,99,199,142,237,13,177,49,49,49,104,110,110,142,58,196,6,
159,71,22,64,44,69,81,24,30,30,134,223,239,143,42,196,6,5,154,255,26,210,17,59,58,58,138,145,145,145,168,66,108,208,237,188,16,98,19,19,19,113,225,194,5,12,14,14,70,21,98,247,122,12,98,162,40,234,181,
132,132,4,50,49,51,154,16,203,127,226,16,2,177,38,147,9,125,125,125,232,234,234,138,42,196,242,22,37,132,88,154,166,107,213,106,245,189,180,180,52,50,0,46,26,16,235,114,185,248,31,244,143,118,77,196,234,
116,58,36,38,38,146,3,247,145,132,216,210,210,82,28,59,118,140,4,193,100,50,217,63,239,217,17,171,209,104,144,144,144,64,198,7,71,2,98,185,220,48,47,94,249,233,3,33,86,171,213,34,62,62,30,110,183,27,13,
13,13,97,133,88,155,205,70,242,231,7,26,214,69,81,20,212,106,53,18,18,18,80,91,91,139,198,198,198,176,64,172,213,106,13,166,234,131,67,108,66,66,2,204,102,51,154,155,155,143,156,136,45,44,44,12,89,208,
161,32,150,155,167,157,149,149,69,186,200,14,3,177,187,189,66,71,134,88,131,193,0,187,221,142,230,230,230,93,23,228,118,187,247,92,144,221,110,71,92,92,92,248,33,86,46,151,111,235,245,122,20,20,20,32,
55,55,23,199,142,29,35,199,126,211,211,211,97,177,88,96,177,88,144,151,151,135,188,188,60,20,22,22,194,225,112,224,196,137,19,130,233,3,97,135,88,150,101,63,141,139,139,131,84,42,133,84,42,37,208,202,
253,107,153,76,22,130,176,65,5,218,145,233,136,149,201,100,255,76,211,244,71,34,145,104,155,187,232,113,127,229,174,57,28,208,114,81,93,154,166,239,61,172,97,93,123,66,108,126,126,126,237,183,16,251,141,
74,196,138,68,162,85,153,76,118,231,27,3,177,20,69,125,152,147,147,179,205,61,24,50,153,76,100,138,19,63,12,207,143,126,71,116,88,151,66,161,192,201,147,39,177,186,186,138,179,103,207,146,176,105,91,91,
27,186,187,187,73,149,74,126,126,62,185,150,165,164,164,64,167,211,65,161,80,252,137,101,217,159,133,21,98,41,138,130,199,227,193,165,75,151,48,63,63,47,104,1,224,126,90,91,91,209,221,221,141,222,222,
94,248,124,62,12,13,13,193,231,243,97,114,114,18,229,229,229,80,169,84,219,59,21,97,71,135,88,141,70,131,214,214,86,172,173,173,133,156,117,223,203,61,155,155,155,209,221,221,141,181,181,53,188,240,194,
11,152,159,159,71,78,78,14,87,241,116,120,136,21,139,197,208,233,116,24,24,24,192,198,198,6,38,39,39,119,29,214,181,219,130,154,155,155,49,59,59,75,202,108,223,123,239,61,56,28,142,163,65,172,68,34,65,
105,105,41,22,23,23,177,188,188,44,104,174,124,16,196,182,182,182,99,96,96,0,183,110,221,18,8,113,67,67,3,50,50,50,130,51,196,15,134,88,145,72,244,43,179,217,140,206,206,78,92,190,124,153,140,192,227,
67,218,253,32,214,63,117,6,35,163,126,92,88,94,33,11,122,245,213,87,49,54,54,134,218,218,90,56,28,14,232,116,58,174,254,226,193,16,171,84,42,127,150,149,149,181,189,177,177,129,11,23,46,96,98,98,66,240,
118,237,23,98,7,7,199,240,248,181,45,178,168,213,213,85,180,180,180,16,197,203,207,207,231,199,116,239,15,177,137,137,137,127,186,114,229,10,25,27,196,127,117,14,10,177,211,211,115,216,218,186,129,107,
215,182,208,219,55,24,2,177,121,121,121,252,88,201,222,195,186,140,70,35,54,55,55,49,56,56,24,50,206,236,48,16,219,210,218,245,53,51,6,65,44,119,240,112,167,136,123,239,106,2,179,217,140,141,141,141,144,
84,117,164,32,214,225,112,192,108,54,115,251,174,221,135,117,101,102,102,134,44,42,146,16,235,241,120,96,183,219,145,144,144,0,134,97,160,80,40,86,5,16,75,81,20,178,179,179,113,249,242,229,168,66,108,
69,69,5,185,15,220,233,226,16,66,236,137,19,39,112,249,242,229,168,14,235,242,120,60,252,125,188,16,98,69,34,17,202,202,202,176,180,180,20,245,97,93,188,135,71,66,136,101,89,22,3,3,3,81,135,216,146,146,
146,189,33,86,165,82,97,105,105,137,180,0,68,11,98,131,102,207,8,33,214,100,50,97,117,117,53,234,29,177,65,188,40,132,216,140,140,12,108,110,110,70,125,88,87,208,89,82,33,196,90,173,86,172,172,172,8,182,
31,209,24,214,21,100,12,95,67,172,76,38,187,147,149,149,133,169,169,41,82,75,16,173,142,216,61,135,117,137,68,162,85,131,193,128,172,172,44,65,87,66,52,170,9,120,101,163,187,15,235,74,78,78,198,169,83,
167,112,250,244,233,136,15,235,226,220,243,129,213,4,50,153,12,41,41,41,176,217,108,232,232,232,136,40,196,114,144,182,175,97,93,20,69,193,104,52,162,178,178,50,98,137,88,238,88,57,175,253,230,79,15,132,
88,149,74,133,228,228,100,52,52,52,8,22,22,14,136,117,58,157,56,118,236,152,160,35,72,169,84,254,108,95,16,171,209,104,144,147,147,67,182,177,225,128,88,155,205,6,163,209,40,56,68,45,22,139,127,181,111,
136,101,24,6,70,163,17,78,167,19,45,45,45,71,30,214,117,242,228,73,65,170,250,208,16,43,22,139,145,154,154,138,174,174,174,144,5,237,23,98,157,78,39,178,178,178,248,59,129,163,67,44,119,124,183,178,178,
18,30,143,7,46,151,139,244,229,123,60,30,84,84,84,236,185,32,110,187,27,132,30,71,135,88,154,166,127,165,213,106,161,80,40,16,19,19,3,149,74,133,248,248,120,104,181,90,24,12,6,152,76,38,228,228,228,144,
132,44,167,49,14,135,3,6,131,33,68,97,194,6,177,50,153,236,103,114,185,252,79,82,169,20,50,153,140,252,200,229,114,242,35,145,72,16,19,19,131,248,248,120,24,141,70,65,196,155,155,121,28,241,97,93,252,
127,32,77,211,16,139,197,100,161,50,153,140,127,7,252,225,55,6,98,37,18,201,29,189,94,191,250,109,34,246,27,91,77,160,84,42,127,174,82,169,62,165,40,234,175,59,31,230,135,7,177,12,195,108,159,58,117,10,
157,157,157,104,105,105,65,89,89,25,41,9,225,119,199,70,5,98,41,138,66,124,124,60,156,78,39,30,123,236,49,4,2,1,244,247,247,19,243,172,175,175,135,211,233,68,102,102,38,244,122,61,9,210,239,252,38,126,
68,211,244,235,97,133,88,174,249,59,16,8,96,121,121,25,179,179,179,2,102,60,125,250,52,185,11,106,107,107,35,117,81,205,205,205,40,41,41,129,213,106,69,124,124,252,118,76,76,204,7,71,134,88,46,156,236,
118,187,177,184,184,136,149,149,21,65,76,247,65,238,217,210,210,2,159,207,135,137,137,9,244,245,245,33,41,41,9,52,77,255,238,208,16,43,147,201,96,179,217,48,54,54,70,234,52,249,49,221,7,45,168,169,169,
9,189,189,189,120,252,241,175,114,159,215,175,95,199,201,147,39,57,193,59,24,196,178,44,251,59,155,205,134,254,254,126,60,242,200,35,152,159,159,199,208,208,144,160,192,225,65,16,219,212,212,132,182,182,
54,92,184,112,225,235,91,173,59,119,80,89,89,121,112,136,53,26,141,31,116,119,119,227,234,213,171,152,159,159,199,232,232,168,192,61,247,3,177,195,195,19,232,234,238,67,224,204,28,94,123,237,235,27,137,
43,87,174,160,161,161,129,100,18,118,22,118,127,136,165,105,250,117,163,209,184,253,157,239,124,135,204,111,15,102,198,253,66,108,111,223,48,158,121,230,107,178,190,117,235,22,250,251,251,81,91,91,11,
155,205,198,247,244,7,87,19,20,21,21,97,99,99,35,100,65,7,133,216,30,223,32,214,214,174,226,245,215,223,192,171,175,190,138,201,73,63,217,86,87,84,84,144,178,16,174,82,122,79,136,149,74,165,104,106,106,
194,133,11,23,4,159,159,195,66,108,103,87,31,2,129,121,12,14,142,226,116,75,7,129,88,238,132,146,78,167,35,143,121,247,132,88,133,66,1,191,223,31,18,64,141,4,196,114,108,173,86,171,185,139,235,238,16,
171,209,104,112,238,220,185,168,13,235,226,66,93,59,191,141,161,213,4,20,69,33,61,61,29,143,62,250,40,6,6,6,162,2,177,92,199,117,114,114,50,247,219,40,76,196,50,12,131,220,220,92,92,185,114,37,234,16,
203,21,57,136,68,162,159,11,32,150,162,40,244,244,244,96,105,105,137,188,74,209,128,216,202,202,74,254,161,251,79,5,16,107,48,24,176,178,178,130,137,137,137,168,66,172,219,237,230,147,245,215,16,75,211,
52,184,71,32,156,121,70,11,98,139,139,139,131,11,253,190,134,216,230,230,102,172,172,172,144,87,41,90,16,27,164,195,95,67,108,124,124,252,167,205,205,205,152,158,158,22,124,219,71,26,98,203,203,203,5,
147,201,5,16,171,84,42,127,110,177,88,224,247,251,163,10,177,78,167,83,48,191,47,36,17,107,52,26,97,181,90,201,244,176,104,64,172,205,102,19,24,67,72,34,150,97,152,237,212,212,84,212,213,213,161,181,181,
53,42,16,27,20,144,223,29,98,213,106,53,116,58,29,154,155,155,5,169,216,72,65,108,80,83,201,238,16,203,61,163,57,117,234,20,217,248,71,10,98,203,202,202,4,31,242,251,66,172,68,34,129,201,100,66,109,109,
45,161,197,72,64,108,208,144,202,7,87,19,176,44,139,172,172,44,116,117,117,133,189,154,128,143,177,92,34,86,165,82,189,254,64,136,165,105,26,105,105,105,240,122,189,97,133,88,167,211,137,220,220,92,36,
38,38,130,151,233,251,96,223,16,203,178,44,204,102,51,122,123,123,201,130,14,11,177,78,167,19,39,79,158,68,82,82,82,240,87,202,239,14,12,177,169,169,169,40,43,43,35,161,211,195,64,108,81,81,17,82,83,83,
131,7,190,29,30,98,197,98,241,239,244,122,61,226,227,227,97,50,153,144,149,149,5,155,205,134,162,162,34,148,151,151,19,140,221,109,65,101,101,101,56,117,234,148,0,241,195,6,177,113,113,113,31,40,20,138,
237,152,152,24,40,20,10,18,62,85,171,213,72,74,74,66,110,110,46,1,216,224,99,2,188,99,152,228,118,42,156,137,216,215,185,142,88,174,138,64,46,151,19,49,230,102,178,101,103,103,35,59,59,27,58,157,110,55,
166,142,14,196,242,103,24,113,139,228,58,20,184,95,247,111,4,196,74,36,146,79,69,34,209,207,191,173,38,216,13,98,69,34,145,231,27,213,17,171,86,171,63,81,171,213,95,236,60,160,126,120,16,203,13,74,53,
155,205,24,25,25,193,192,192,0,26,27,27,97,177,88,66,186,207,163,2,177,34,145,8,169,169,169,104,108,108,196,218,218,26,9,18,114,27,195,138,138,10,216,108,54,50,25,145,119,105,8,127,71,172,88,44,70,114,
114,50,92,46,23,57,205,61,62,62,78,238,166,185,109,116,83,83,19,57,107,83,85,85,133,147,39,79,194,104,52,114,201,254,109,138,162,126,121,100,136,149,201,100,176,88,44,240,122,189,88,88,88,192,133,11,23,
48,61,61,253,64,136,37,187,214,157,227,83,77,77,77,40,43,43,131,217,108,134,66,161,184,119,40,136,149,203,229,31,58,28,14,116,118,118,98,121,121,25,75,75,75,240,251,253,130,163,229,251,129,88,110,113,
62,159,15,243,243,243,24,30,30,134,94,175,7,77,211,31,30,8,98,83,83,83,239,77,79,79,99,125,125,29,103,207,158,13,57,86,190,95,136,229,22,52,52,52,132,239,124,231,59,132,25,173,86,43,183,159,218,31,196,
210,52,253,75,254,80,249,96,102,60,8,196,182,183,119,98,96,96,0,215,174,93,227,97,200,109,84,86,86,238,31,98,41,138,250,177,82,169,220,158,152,152,192,249,243,231,143,4,177,221,221,3,240,249,6,113,101,
125,67,112,3,177,180,180,132,154,154,26,104,181,218,253,67,172,74,165,218,117,190,218,97,134,117,117,117,247,225,229,151,95,33,127,207,165,75,151,208,209,209,129,154,154,26,28,59,118,140,223,45,180,55,
196,114,51,103,54,54,54,66,202,27,14,155,136,93,95,223,192,139,47,190,132,171,87,31,69,111,111,47,217,86,59,28,14,40,149,74,178,87,223,19,98,69,34,17,42,43,43,177,182,182,38,0,253,163,66,172,207,55,136,
246,14,95,8,196,102,101,101,221,127,88,23,119,216,240,236,217,179,130,38,128,72,66,108,73,73,9,52,26,13,73,91,239,10,177,41,41,41,184,122,245,42,6,7,7,163,6,177,217,217,217,252,137,174,66,136,165,40,10,
29,29,29,88,93,93,141,42,196,58,28,14,232,245,122,238,179,37,76,196,166,164,164,224,226,197,139,152,156,156,140,106,34,150,223,126,42,18,137,62,17,64,172,195,225,192,230,230,38,122,123,123,163,10,177,
21,21,21,252,118,184,47,104,154,246,80,20,69,85,104,52,26,156,57,115,70,16,209,141,22,196,186,221,238,224,70,221,10,138,97,24,143,86,171,253,98,112,112,16,67,67,67,130,47,215,104,64,172,221,110,223,125,
88,87,108,108,236,39,70,163,17,163,163,163,81,133,88,183,219,141,227,199,143,239,157,136,213,235,245,168,172,172,68,71,71,71,84,33,118,207,68,44,69,81,175,41,20,10,152,205,102,65,95,66,164,33,54,168,169,
36,20,98,185,30,79,187,221,142,174,174,174,168,64,108,208,24,170,221,33,150,123,28,210,221,221,29,113,136,221,229,173,219,27,98,117,58,29,138,139,139,209,211,211,19,209,68,172,221,110,223,223,176,46,78,
86,204,102,51,134,135,135,5,13,0,225,134,216,220,220,220,131,117,196,26,12,6,212,212,212,160,173,173,45,34,16,203,165,136,14,84,77,32,149,74,97,177,88,48,56,56,40,24,117,112,84,136,229,22,148,151,151,
71,246,81,98,177,248,151,251,134,88,147,201,68,46,13,220,110,241,40,137,216,210,210,82,228,228,228,32,33,33,129,239,159,247,14,4,177,82,169,244,67,157,78,7,179,217,140,252,252,124,82,177,202,45,106,63,
16,203,125,160,51,50,50,16,27,27,27,124,250,246,195,67,65,172,70,163,185,199,205,184,98,89,22,9,9,9,176,88,44,176,217,108,4,97,247,234,136,45,45,45,69,102,102,38,228,114,185,160,128,244,80,83,84,118,177,
132,95,238,20,59,66,169,84,66,165,82,65,161,80,64,165,82,33,55,55,23,118,187,157,196,118,185,175,142,227,199,143,35,54,54,150,188,42,188,197,132,191,35,150,251,7,72,165,82,200,229,114,168,84,42,40,149,
74,164,165,165,193,106,181,34,47,47,143,32,236,67,131,88,110,64,92,108,108,44,164,82,41,191,164,150,188,50,15,21,98,119,2,129,95,236,108,105,191,57,29,177,59,91,217,111,19,177,247,29,214,181,51,21,241,
139,135,10,177,59,167,56,160,215,235,73,211,105,99,99,35,78,157,58,5,133,66,193,221,199,69,7,98,185,38,240,156,156,28,12,14,14,98,110,110,14,231,206,157,195,192,192,0,58,58,190,178,2,110,200,32,215,222,
28,17,136,101,89,118,91,46,151,195,96,48,160,186,186,26,3,3,3,88,93,93,69,32,16,192,208,208,144,96,176,115,83,83,19,234,235,235,225,245,122,201,161,11,149,74,197,165,201,142,62,172,75,169,84,126,112,234,
212,41,120,60,30,204,206,206,98,118,118,22,75,75,75,152,154,154,34,222,112,63,136,229,190,196,235,235,235,113,242,228,73,164,164,164,112,183,83,135,171,38,48,24,12,247,2,129,0,113,79,191,223,143,190,190,
190,3,65,44,183,157,230,118,23,77,77,77,112,56,28,200,202,202,130,76,38,187,119,224,106,2,183,219,141,173,173,45,50,202,44,216,61,15,2,177,220,182,218,231,243,145,122,215,157,231,201,31,236,187,35,54,
62,62,158,140,50,59,42,196,114,11,155,152,152,192,11,47,188,128,187,119,239,226,217,103,159,229,207,175,221,223,176,174,204,204,76,220,188,121,83,48,23,235,48,16,219,222,222,131,150,214,46,248,253,83,
120,233,165,151,200,158,253,181,215,94,35,55,14,52,77,111,63,16,98,101,50,25,122,123,123,177,188,188,124,100,136,109,105,233,132,207,55,136,151,95,126,89,80,232,48,61,61,77,6,149,236,11,98,117,58,29,174,
93,187,70,230,34,31,181,35,118,108,220,143,151,95,254,225,87,184,246,242,203,240,251,253,104,108,108,132,219,237,22,28,24,219,19,98,185,48,124,240,208,183,163,66,236,242,242,35,184,113,227,41,76,78,206,
160,177,177,145,64,108,80,25,228,238,16,203,178,44,30,121,228,17,242,1,143,52,196,218,108,54,254,190,61,20,98,197,98,49,106,106,106,176,185,185,73,12,61,210,16,203,101,242,120,27,67,33,196,38,36,36,96,
105,105,9,51,51,51,81,237,136,205,201,201,225,239,84,133,16,219,220,220,140,171,87,175,162,175,175,47,170,16,235,112,56,248,208,241,117,53,129,94,175,255,98,116,116,20,51,51,51,130,175,142,104,64,108,
16,9,125,157,136,77,74,74,66,107,107,43,198,198,198,30,10,196,6,5,80,133,213,4,220,55,127,52,33,214,237,118,243,3,205,95,8,170,9,146,146,146,72,19,82,52,33,214,110,183,243,179,121,66,136,141,137,137,65,
122,122,58,198,199,199,163,6,177,187,196,42,133,16,203,29,31,231,190,239,162,1,177,118,187,93,16,135,219,21,98,227,226,226,96,179,217,48,48,48,16,113,136,45,43,43,11,254,154,217,29,98,41,138,66,70,70,
6,70,70,70,132,217,130,8,64,172,195,225,8,174,65,220,27,98,245,122,61,57,179,23,73,136,229,174,228,251,130,88,185,92,14,151,203,133,190,190,62,50,222,53,18,16,155,152,152,184,127,136,149,72,36,219,92,
54,157,143,176,225,132,216,160,153,125,251,27,214,149,150,150,134,238,238,110,52,53,53,69,4,98,249,93,46,251,238,136,85,40,20,31,232,116,58,228,228,228,192,225,112,160,161,161,1,213,213,213,71,130,88,
46,166,91,80,80,192,135,216,131,85,19,168,213,234,123,26,141,134,120,65,78,78,14,153,55,123,144,68,172,195,225,192,201,147,39,97,54,155,161,86,171,249,217,207,123,135,174,38,144,201,100,80,169,84,100,
224,77,90,90,26,156,78,39,89,204,94,16,107,181,90,97,52,26,5,42,188,243,68,29,20,69,125,16,150,97,93,12,195,16,231,84,40,20,200,202,202,66,113,113,49,121,56,237,118,187,225,112,56,200,156,99,110,66,15,
183,144,157,159,237,136,64,44,151,33,80,171,213,80,42,149,56,126,252,56,242,243,243,145,147,147,67,130,15,193,51,178,162,6,177,34,145,8,50,153,12,106,181,154,84,17,236,178,152,135,7,177,50,153,236,139,
29,234,249,118,88,215,223,39,196,238,64,255,223,148,74,229,39,82,169,244,231,15,13,98,185,242,99,174,148,134,203,53,88,173,86,136,197,98,136,68,162,237,136,67,44,77,211,144,201,100,144,74,165,48,155,205,
176,219,237,240,251,253,224,50,124,220,1,216,154,154,26,20,21,21,9,238,126,195,10,177,106,181,250,215,58,157,110,59,55,55,23,21,21,21,104,107,107,131,223,239,199,212,212,20,22,23,23,9,59,118,119,119,147,
91,179,198,198,70,210,173,152,155,155,43,152,116,120,100,136,213,104,52,191,159,154,154,194,165,75,151,48,61,61,141,185,185,57,248,253,126,12,15,15,195,231,243,161,187,187,251,190,16,91,87,87,135,202,
202,74,216,108,54,88,44,22,126,53,207,225,59,98,93,46,23,158,122,234,41,226,158,193,1,212,253,36,98,249,101,239,65,139,59,120,71,172,70,163,193,197,139,23,113,246,236,217,176,64,44,127,203,83,84,84,196,
157,109,255,253,190,33,150,162,40,56,157,78,220,184,113,35,164,112,244,48,16,203,109,171,155,155,155,209,215,215,135,115,231,206,161,167,167,135,171,100,253,245,190,32,54,46,46,14,75,75,75,33,147,159,
14,3,177,205,167,191,58,104,223,217,217,137,179,103,207,226,71,63,250,17,105,215,221,201,38,60,24,98,185,177,214,220,171,116,84,136,109,110,110,135,207,55,128,43,87,214,5,119,55,175,188,242,10,63,48,241,
96,136,93,90,90,34,3,150,194,49,172,139,95,247,123,247,238,93,60,247,220,115,24,30,30,22,220,178,239,9,177,12,195,160,191,191,31,235,235,235,232,239,239,15,27,196,46,45,93,196,235,175,191,65,222,182,254,
254,126,120,189,94,66,64,59,23,216,80,136,21,139,197,219,133,133,133,88,91,91,195,248,248,120,88,19,177,29,157,62,44,46,94,192,185,115,203,24,25,25,21,64,44,127,239,21,2,177,10,133,2,43,43,43,88,88,88,
32,215,163,176,67,108,207,160,0,98,93,46,151,224,28,178,0,98,149,74,229,207,91,91,91,5,233,234,104,64,108,69,69,69,48,114,124,13,177,90,173,246,147,145,145,17,76,79,79,11,174,212,209,128,216,160,28,213,
215,16,27,19,19,243,55,135,195,129,169,169,169,168,67,108,144,190,8,59,98,45,22,11,25,179,24,77,136,117,185,92,72,77,77,221,29,98,141,70,35,249,230,127,88,16,75,211,244,223,4,16,43,145,72,96,181,90,49,
54,54,22,85,136,45,47,47,231,235,240,39,2,136,229,222,66,110,75,27,173,97,93,118,187,157,236,179,104,154,254,121,8,196,154,76,38,140,140,140,144,168,110,164,33,182,180,180,84,48,207,104,87,136,149,201,
100,104,110,110,70,127,127,127,84,18,177,65,79,26,182,119,133,88,185,92,142,194,194,66,140,141,141,69,124,88,23,215,172,244,64,136,165,105,26,153,153,153,152,156,156,20,60,186,143,4,196,6,229,134,239,
15,177,25,25,25,88,88,88,32,241,144,72,64,172,213,106,13,62,227,126,127,136,213,106,181,219,220,182,37,82,16,27,116,124,224,193,16,171,84,42,127,205,113,98,105,105,41,185,35,9,23,196,22,22,22,10,94,165,
131,64,236,239,57,239,204,202,202,66,73,73,137,32,179,114,88,136,13,150,225,67,117,196,202,100,50,104,52,26,168,84,42,100,103,103,195,225,112,144,123,184,131,12,235,42,45,45,133,221,110,71,65,65,1,127,
83,119,248,142,88,138,162,32,149,74,193,189,114,86,171,149,196,117,119,91,16,119,81,228,200,209,96,48,144,147,182,188,183,237,247,97,129,88,46,211,169,209,104,144,146,146,66,10,108,185,102,36,135,195,
129,188,188,60,152,76,38,40,149,74,82,162,198,53,7,236,88,232,54,195,48,191,142,72,34,54,46,46,14,74,165,18,121,121,121,200,207,207,71,70,70,6,148,74,37,55,7,139,44,38,200,63,35,11,177,34,145,104,91,34,
145,32,54,54,150,188,34,252,133,4,159,216,142,42,196,178,44,251,115,150,101,63,17,137,68,127,219,107,88,215,183,16,251,119,3,177,52,77,127,190,227,234,255,206,178,236,123,81,135,88,238,3,205,61,149,74,
74,74,66,90,90,26,60,30,15,106,106,106,144,155,155,11,133,66,17,57,136,85,40,20,239,177,44,251,169,70,163,65,108,108,44,76,38,19,140,70,35,138,139,139,225,118,187,49,56,56,136,241,241,113,156,57,115,134,
244,198,54,52,52,192,106,181,242,187,59,195,7,177,50,153,236,63,234,234,234,182,135,134,134,48,58,58,10,191,223,15,191,223,143,64,32,128,233,233,105,76,79,79,99,116,116,20,253,253,253,130,59,161,198,198,
70,184,92,46,228,229,229,241,7,72,28,29,98,41,138,250,77,78,78,14,158,121,230,25,76,79,79,99,112,112,16,189,189,189,232,233,233,9,105,65,221,205,61,57,86,228,248,154,119,211,121,120,136,85,171,213,152,
159,159,199,210,210,210,174,238,185,223,106,130,138,138,10,194,215,50,153,140,75,96,31,110,88,87,117,117,53,158,120,226,9,12,15,15,31,9,98,235,235,235,81,83,83,131,178,178,50,88,173,86,100,100,100,112,
191,32,191,57,16,196,38,38,38,146,36,117,184,32,182,186,186,154,28,103,225,78,142,48,12,243,31,251,130,88,173,86,139,153,153,25,92,185,114,37,228,224,253,97,33,150,219,82,183,181,181,145,30,98,143,199,
195,61,192,124,239,129,16,91,85,85,133,181,181,53,12,15,15,135,5,98,27,27,91,80,95,223,136,193,193,65,172,173,173,225,246,237,219,184,123,247,46,230,230,230,184,157,196,167,123,66,44,77,211,176,217,108,
184,113,227,6,102,103,103,201,111,216,81,33,182,249,116,59,252,254,25,18,104,230,126,30,127,252,113,146,81,216,19,98,147,147,147,113,254,252,121,44,44,44,8,126,229,143,10,177,221,221,253,36,170,203,253,
108,109,109,225,244,233,211,247,175,38,72,74,74,218,126,228,145,71,240,200,35,143,8,100,56,28,16,219,222,222,131,103,158,121,150,228,135,231,230,230,208,209,209,129,242,242,114,126,55,191,16,98,21,10,
197,123,197,197,197,88,95,95,23,116,85,135,19,98,199,199,167,177,176,184,132,137,201,25,212,212,212,16,243,84,169,84,187,67,108,76,76,204,191,59,28,14,114,145,140,102,34,54,104,224,174,48,17,155,149,149,
133,249,249,249,168,67,44,103,10,59,95,218,95,67,44,77,211,159,27,141,70,248,253,126,116,117,117,69,21,98,131,116,88,8,177,201,201,201,232,235,235,67,127,127,127,84,33,182,168,168,104,239,97,93,20,69,
161,172,172,140,76,16,139,22,196,58,157,78,126,130,255,115,1,196,178,44,139,236,236,108,76,79,79,71,21,98,93,46,23,127,191,245,126,8,196,102,101,101,225,252,249,243,100,81,209,128,88,126,175,39,77,211,
255,46,128,88,154,166,97,52,26,177,184,184,40,152,141,21,73,136,229,116,152,219,231,51,12,243,94,8,196,154,76,38,242,240,49,26,16,91,84,84,196,159,84,190,59,196,198,197,197,145,203,66,164,33,214,233,116,
194,104,52,62,24,98,19,18,18,208,215,215,7,159,207,23,81,136,229,198,121,238,11,98,165,82,233,167,118,187,29,195,195,195,168,175,175,143,24,196,230,231,231,7,247,123,238,13,177,114,185,252,189,212,212,
84,88,44,22,20,23,23,163,185,185,57,236,16,107,183,219,5,95,194,251,130,88,137,68,242,31,26,141,6,113,113,113,200,205,205,69,121,121,57,26,27,27,195,6,177,65,161,248,253,67,44,151,239,140,143,143,135,
82,169,68,89,89,25,188,94,239,145,33,54,63,63,63,216,207,15,14,177,59,83,82,17,23,23,135,140,140,12,18,65,58,40,196,238,86,126,124,100,136,85,40,20,164,150,128,19,226,189,22,196,253,118,149,150,150,162,
176,176,16,249,249,249,56,126,252,120,240,211,207,223,132,13,98,119,14,64,195,235,245,146,137,98,110,183,27,197,197,197,56,113,226,4,204,102,51,18,18,18,72,246,147,27,155,206,5,86,119,174,218,255,17,86,
136,165,105,26,42,149,10,106,181,26,233,233,233,200,200,200,64,124,124,60,105,0,96,24,6,98,177,24,18,137,132,116,201,242,100,248,83,149,74,245,94,196,32,86,169,84,110,171,84,42,193,98,120,227,52,30,30,
196,178,44,251,30,195,48,255,190,179,221,248,252,91,136,253,187,135,216,224,142,88,154,166,31,137,56,196,138,197,226,183,41,138,250,131,88,44,222,166,105,154,252,170,75,36,18,176,44,11,169,84,138,132,
132,4,104,181,90,228,230,230,194,100,50,109,203,229,242,63,70,4,98,69,34,209,27,6,131,225,47,78,167,19,53,53,53,112,185,92,100,52,11,247,125,216,210,210,130,137,137,9,244,246,246,98,126,126,30,19,19,19,
232,236,236,68,118,118,54,247,27,249,113,88,171,9,226,227,227,183,167,167,167,177,178,178,130,201,201,73,140,140,140,96,116,116,20,195,195,195,24,29,29,197,208,208,16,134,135,135,73,50,182,171,171,139,
4,80,237,118,59,210,211,211,57,239,12,15,196,42,20,10,244,244,244,16,175,58,168,123,214,214,214,162,172,172,12,70,163,145,255,64,233,240,16,43,145,72,80,86,86,134,155,55,111,98,114,114,50,100,20,213,126,
33,182,166,166,6,133,133,133,92,105,237,193,190,144,249,91,23,174,32,114,101,101,133,208,205,81,19,177,101,101,101,200,202,202,130,92,46,63,88,71,44,55,8,192,104,52,226,226,197,139,120,228,145,71,4,238,
121,84,136,117,56,28,56,117,234,20,12,6,3,247,213,180,191,106,2,185,92,142,137,137,9,92,190,124,89,0,104,225,128,216,234,234,106,84,86,86,162,184,184,24,201,201,201,96,89,118,155,166,233,55,246,132,88,
154,166,63,54,24,12,152,159,159,199,149,43,87,48,50,50,18,54,136,173,171,107,32,249,134,142,142,14,180,182,182,162,186,186,26,70,163,17,44,203,254,101,79,136,149,72,36,24,29,29,197,218,218,154,32,200,
124,84,136,109,108,108,69,99,99,11,186,187,187,49,49,49,129,235,215,175,227,157,119,222,193,243,207,63,143,186,186,58,174,54,243,237,16,136,213,233,116,127,92,92,92,196,213,171,87,5,175,80,184,134,117,
245,248,6,176,181,117,35,228,207,106,104,104,224,126,27,255,32,128,88,154,166,31,177,88,44,184,118,237,26,9,11,134,187,35,118,105,233,162,224,239,187,125,251,54,174,94,189,138,188,188,60,82,123,17,146,
136,229,170,125,131,159,78,133,11,98,207,157,95,198,75,47,253,0,111,191,253,54,54,55,55,73,204,46,45,45,109,119,136,229,170,9,206,158,61,75,34,186,145,128,216,217,192,60,206,206,159,131,207,215,75,110,
60,246,76,196,82,20,245,190,193,96,128,223,239,135,207,231,139,42,196,6,197,150,222,23,116,196,202,229,114,244,246,246,98,116,116,52,170,16,27,4,29,66,136,149,74,165,240,120,60,196,60,163,5,177,65,97,
65,33,196,178,44,11,151,203,133,185,185,185,168,66,236,201,147,39,119,31,214,69,81,212,251,12,195,160,168,168,8,243,243,243,81,133,216,226,226,226,189,59,98,37,18,9,108,54,27,22,22,22,162,86,77,192,157,
190,229,161,199,191,134,64,172,205,102,195,226,226,98,212,58,98,57,119,224,22,21,146,136,149,72,36,219,86,171,21,243,243,243,130,13,90,36,33,150,227,33,238,237,211,235,245,143,4,67,236,31,114,115,115,
113,238,220,57,193,6,45,146,195,186,242,243,243,249,135,165,255,24,2,177,12,195,188,157,145,145,129,206,206,78,52,52,52,68,60,17,91,80,80,32,40,48,218,19,98,19,18,18,254,146,144,144,128,252,252,124,180,
183,183,71,20,98,131,10,28,62,222,19,98,105,154,126,67,161,80,108,235,116,58,228,230,230,194,237,118,147,115,122,225,130,88,155,205,22,50,212,100,95,213,4,92,161,168,70,163,65,117,117,117,216,32,214,110,
183,11,134,204,31,24,98,25,134,129,90,173,70,66,66,2,28,14,7,57,60,120,20,136,13,74,195,30,14,98,41,138,130,70,163,65,66,66,2,42,43,43,209,208,208,112,40,136,229,102,171,5,129,218,225,33,150,27,103,173,
213,106,97,183,219,9,194,30,68,134,11,10,10,4,51,178,194,6,177,220,91,201,149,56,184,221,110,148,150,150,226,228,201,147,56,117,234,20,210,211,211,73,2,205,96,48,192,104,52,34,57,57,25,201,201,201,252,
187,226,176,87,19,124,204,48,12,52,26,13,180,90,45,9,158,114,225,83,142,131,118,251,217,185,27,254,139,74,165,122,35,34,16,171,84,42,255,200,31,202,21,92,73,192,143,230,210,52,189,77,81,212,31,100,50,
217,219,209,128,216,71,246,26,214,245,45,196,254,63,3,177,81,73,196,178,44,251,174,209,104,252,179,66,161,216,214,106,181,144,203,229,164,21,128,187,249,224,82,178,59,151,129,47,21,10,197,61,137,68,242,
211,136,85,19,100,102,102,34,16,8,96,110,110,14,62,159,143,140,41,110,104,104,64,115,115,51,60,30,143,96,112,234,192,192,0,188,94,47,140,70,35,196,98,241,189,176,66,44,195,48,219,199,143,31,199,234,234,
42,150,151,151,225,243,249,208,222,222,142,238,238,110,116,118,118,146,5,114,89,153,158,158,30,244,245,245,145,41,7,14,135,3,9,9,9,96,24,102,59,44,16,203,31,224,124,254,252,121,65,232,107,191,149,172,
94,175,55,184,55,225,240,16,203,229,61,231,230,230,176,180,180,36,232,135,61,232,176,46,167,211,137,212,212,84,126,192,249,224,16,43,18,137,144,158,158,142,165,165,37,92,190,124,89,112,206,253,48,16,91,
83,83,131,146,146,18,190,169,31,60,17,107,54,155,177,184,184,136,149,149,21,129,123,134,3,98,211,210,210,72,151,208,190,32,86,38,147,109,59,28,14,50,95,45,18,16,107,179,217,96,54,155,161,84,42,31,220,
17,171,84,42,239,213,215,215,99,115,115,19,115,115,115,130,15,117,56,33,150,107,167,60,126,252,56,247,86,238,94,77,160,80,40,126,90,92,92,140,141,141,13,193,40,188,112,66,108,67,195,215,173,205,109,109,
109,168,170,170,130,197,98,225,30,128,191,27,2,177,113,113,113,247,198,199,199,113,254,252,249,144,2,144,112,64,108,115,115,59,218,59,124,24,28,28,196,153,51,103,240,228,147,79,226,205,55,223,196,196,
196,4,183,168,191,236,218,17,219,215,215,39,104,209,13,119,71,236,218,218,85,252,240,135,175,8,254,254,155,55,111,114,89,207,80,136,85,169,84,104,108,108,20,44,42,156,16,59,49,49,141,151,94,250,39,193,
223,251,198,27,111,96,121,121,121,111,136,149,74,165,104,110,110,142,88,71,236,192,224,40,158,186,249,52,238,222,189,139,127,252,199,23,176,180,180,132,193,193,65,56,157,206,189,33,150,162,40,82,57,30,
41,136,13,156,89,196,226,185,101,140,142,78,144,243,129,65,188,40,132,88,133,66,1,175,215,139,64,32,16,85,136,229,205,218,14,133,88,138,162,112,250,244,105,4,2,129,168,66,108,112,12,78,0,177,10,133,2,
237,237,237,152,154,154,138,42,196,238,69,214,37,20,69,189,175,84,42,49,62,62,142,137,137,137,168,66,108,80,242,76,8,177,42,149,10,51,51,51,130,185,88,209,128,216,204,204,204,189,59,98,147,147,147,113,
233,210,37,248,124,190,168,66,44,127,172,89,8,196,42,149,202,237,197,197,69,116,119,119,71,21,98,121,13,168,161,29,177,201,201,201,127,73,79,79,23,108,252,35,13,177,165,165,165,72,78,78,38,173,149,33,
16,43,145,72,222,77,76,76,132,209,104,36,119,40,145,132,216,178,178,50,193,231,73,38,147,253,116,207,106,2,157,78,135,148,148,20,148,149,149,225,244,233,211,17,129,88,187,221,142,148,148,20,62,162,221,
187,111,53,1,195,48,219,90,173,22,241,241,241,168,171,171,139,8,196,6,157,0,217,223,176,46,154,166,17,27,27,11,189,94,15,175,215,139,250,250,250,176,64,172,205,102,67,124,124,252,225,33,150,162,40,36,
36,36,32,41,41,137,204,247,56,10,196,22,20,20,4,103,134,15,7,177,52,77,35,33,33,1,153,153,153,36,220,112,24,136,229,238,144,131,22,116,180,68,172,193,96,128,213,106,221,117,65,149,149,149,164,9,96,183,
5,217,237,118,254,193,138,240,65,44,203,178,219,122,189,30,22,139,5,86,171,21,5,5,5,176,88,44,56,118,236,24,44,22,11,210,211,211,145,157,157,141,172,172,44,228,229,229,161,160,160,0,133,133,133,40,41,
41,9,249,80,135,21,98,165,82,233,189,248,248,120,18,189,229,67,171,84,42,21,252,123,177,88,28,50,39,43,98,213,4,44,203,254,148,101,217,123,92,187,46,47,15,76,254,202,253,236,168,240,182,72,36,250,139,
76,38,123,247,161,12,235,250,22,98,255,174,32,150,101,217,119,104,154,254,239,15,29,98,41,138,122,131,97,152,143,178,179,179,183,185,6,239,212,212,84,210,115,205,125,224,185,95,128,157,92,231,31,34,218,
17,27,19,19,131,234,234,106,108,109,109,97,118,118,150,28,252,233,233,233,33,207,3,185,150,239,252,252,124,28,59,118,12,169,169,169,72,75,75,131,74,165,250,51,195,48,239,132,13,98,41,138,250,50,38,38,
6,181,181,181,184,124,249,50,233,211,227,223,222,183,181,181,161,171,171,11,221,221,221,232,235,235,67,111,111,47,122,123,123,137,0,238,140,68,15,15,196,82,20,5,185,92,142,214,214,86,172,174,174,98,122,
122,90,16,66,221,143,123,214,215,215,163,170,170,10,134,255,159,189,123,125,106,235,188,247,5,174,165,133,150,164,37,201,146,184,72,66,136,139,12,194,220,177,16,23,33,196,69,200,220,49,6,203,88,54,6,140,
13,50,88,96,44,95,48,23,3,193,78,92,183,141,227,221,118,39,219,237,238,110,178,211,115,98,215,185,184,157,190,232,31,113,118,103,250,170,157,233,139,54,51,153,76,115,169,211,166,217,211,61,39,206,132,
239,121,81,158,39,107,73,2,115,209,146,211,51,121,193,100,146,76,198,79,36,177,244,172,207,250,62,223,159,213,42,204,119,238,14,98,25,134,65,86,86,22,166,166,166,112,227,198,13,122,120,108,55,16,219,209,
209,65,171,9,54,62,123,59,135,88,149,74,133,242,242,114,204,206,206,98,113,113,113,207,16,219,217,217,9,175,215,139,156,156,156,221,65,172,86,171,69,79,79,15,230,230,230,112,241,226,197,132,65,44,249,
69,200,202,202,18,30,16,219,26,98,85,42,213,219,37,37,37,88,94,94,198,202,202,10,66,161,144,200,206,19,5,177,53,53,53,72,75,75,163,51,254,182,132,88,171,213,250,223,4,242,131,193,96,66,134,117,109,6,177,
100,22,233,198,231,107,243,142,216,252,252,124,220,184,113,35,38,119,46,5,196,146,196,181,160,18,63,110,53,193,199,118,187,29,55,111,222,140,121,220,145,40,136,237,61,124,148,190,133,126,191,31,62,159,
15,101,101,101,208,233,116,96,24,230,207,113,33,182,176,176,16,75,75,75,34,174,78,36,196,6,2,39,232,49,132,217,217,89,220,188,121,19,11,11,11,112,56,28,241,19,177,50,153,12,69,69,69,88,88,88,160,139,74,
116,71,236,133,217,203,248,183,123,247,240,214,91,95,121,195,131,7,15,208,214,214,70,54,137,111,139,32,150,101,89,20,21,21,97,113,113,17,129,64,64,146,68,236,139,47,190,20,243,223,190,253,246,219,66,247,
252,175,24,136,109,106,106,194,181,107,215,36,131,216,27,55,111,225,209,163,71,120,253,245,159,226,246,237,219,180,10,65,80,81,46,134,88,142,227,208,223,223,143,153,153,25,201,32,246,98,100,14,115,215,
150,48,61,243,143,220,95,75,75,75,244,64,121,49,196,166,166,166,34,20,10,97,98,98,34,169,16,43,68,179,24,136,205,200,200,192,234,234,42,134,135,135,147,10,177,81,41,107,49,196,238,223,191,31,203,203,203,
8,4,2,73,133,88,147,201,20,31,98,25,134,249,47,242,155,215,223,223,159,52,136,109,104,104,136,142,195,125,5,177,28,199,189,93,88,88,136,235,215,175,139,246,68,82,67,108,67,67,131,40,86,25,3,177,86,171,
117,157,164,21,147,5,177,30,143,71,184,183,138,133,88,150,101,255,108,179,217,80,86,86,22,179,107,148,10,98,163,194,204,31,199,173,38,80,169,84,148,125,72,50,86,42,136,141,206,13,111,217,17,171,82,169,
144,149,149,133,206,206,206,168,22,128,196,14,235,170,173,173,21,14,236,250,239,45,33,86,38,147,125,169,84,42,41,43,118,117,117,37,20,98,235,234,234,232,195,108,242,42,105,181,218,183,159,10,177,196,59,
115,115,115,233,198,63,81,16,75,134,205,239,10,98,25,134,161,159,47,50,25,101,47,16,235,116,58,163,51,195,187,131,88,142,227,144,147,147,131,227,199,143,163,189,189,125,87,16,75,70,90,71,133,79,247,6,
177,100,6,86,125,125,61,154,155,155,69,115,176,200,130,154,155,155,55,93,208,254,253,251,163,139,27,18,3,177,102,179,25,90,173,22,164,123,49,61,61,29,38,147,9,102,179,25,121,121,121,56,112,224,0,202,202,
202,224,118,187,225,241,120,208,208,208,0,175,215,43,60,238,75,47,144,9,131,88,142,227,222,214,104,52,255,205,243,60,148,74,37,237,66,80,169,84,52,151,71,126,244,122,61,204,102,179,240,190,46,41,29,177,
31,147,82,7,65,242,149,202,48,89,220,198,103,104,125,227,218,119,255,235,0,177,255,149,146,146,242,246,55,16,251,181,133,88,150,101,31,102,100,100,188,203,113,220,71,207,28,98,101,50,217,71,74,165,114,
189,180,180,20,39,79,158,68,87,87,23,85,20,158,231,163,59,128,190,148,124,88,23,233,120,233,237,237,197,226,226,34,166,167,167,105,0,108,112,112,16,237,237,237,162,193,130,10,133,2,228,50,146,208,142,
88,2,177,114,185,28,118,187,29,211,211,211,88,91,91,195,196,196,132,168,232,143,108,163,251,251,251,209,215,247,143,51,199,193,96,16,237,237,237,180,9,60,39,39,103,157,227,184,63,38,4,98,141,70,35,124,
62,31,110,220,184,17,147,104,140,118,207,232,52,108,79,79,15,157,55,226,243,249,72,51,215,174,33,246,79,90,173,22,69,69,69,24,25,25,193,220,220,92,76,128,112,39,16,219,213,213,5,159,207,7,147,201,68,174,
244,127,218,49,196,90,44,22,12,12,12,32,18,137,32,18,137,224,244,233,211,34,25,222,13,196,18,33,86,42,149,228,75,122,123,16,171,215,235,255,24,8,4,112,227,198,13,204,207,207,99,124,124,60,6,209,118,11,
177,29,29,29,240,122,189,176,90,173,219,135,88,153,76,246,191,138,139,139,113,251,246,109,132,66,161,152,90,223,68,64,108,91,91,27,14,30,60,40,244,206,47,159,58,172,171,165,165,133,194,153,84,16,219,212,
212,132,3,7,14,8,163,224,155,67,172,70,163,65,119,119,55,34,145,136,164,16,75,170,47,178,179,179,133,197,71,241,59,98,141,70,35,102,102,102,98,30,8,73,5,177,78,167,147,132,154,227,119,196,146,209,46,223,
253,238,119,69,213,190,137,132,216,129,129,227,56,26,8,208,125,62,121,27,55,130,243,241,33,54,39,39,7,55,111,222,148,112,88,215,41,12,143,156,65,48,24,196,232,232,40,38,39,39,113,246,236,89,20,20,20,144,
193,150,15,69,16,43,147,201,80,89,89,41,41,196,78,157,191,128,87,94,185,135,215,95,255,169,232,191,31,25,25,33,234,242,174,8,98,85,42,21,186,187,187,113,245,234,85,90,220,145,104,136,125,225,133,219,248,
249,207,127,30,243,63,117,230,204,25,250,185,18,65,108,106,106,42,230,231,231,17,10,133,36,131,216,231,214,158,199,131,7,15,240,240,225,67,220,187,119,15,183,111,223,198,165,75,151,132,196,40,134,216,
172,172,44,220,185,115,135,30,208,145,4,98,23,86,176,178,114,19,87,175,206,99,96,96,0,221,221,221,168,175,175,223,28,98,219,218,218,176,180,180,36,250,114,77,6,196,70,233,240,87,16,171,82,169,62,10,4,
2,52,55,156,44,136,245,122,189,209,45,131,95,65,172,217,108,126,215,225,112,96,116,116,84,180,39,146,26,98,61,30,79,244,196,167,175,32,150,101,217,135,86,171,21,117,117,117,73,77,196,70,127,158,98,32,
150,227,184,245,220,220,92,244,245,245,161,191,191,63,41,16,123,240,224,65,209,77,70,220,142,88,149,74,133,204,204,76,28,63,126,60,41,16,91,88,88,248,244,142,88,50,210,179,186,186,90,52,89,64,42,136,21,
54,87,110,9,177,12,195,32,39,39,7,77,77,77,232,235,235,147,12,98,61,30,143,240,105,195,159,159,10,177,251,246,237,131,205,102,195,145,35,71,18,14,177,194,179,238,228,85,218,86,34,150,196,40,235,235,235,
233,36,177,68,64,108,93,93,29,28,14,135,40,132,202,178,236,31,183,13,177,106,181,154,206,243,35,11,218,45,196,122,60,30,148,148,148,144,169,190,123,130,216,63,165,166,166,162,181,181,149,30,177,220,45,
196,230,231,231,139,32,86,208,26,191,59,136,205,202,202,66,110,110,46,28,14,7,10,11,11,225,116,58,69,125,176,91,45,40,58,81,45,240,207,189,65,44,207,243,127,212,235,245,224,121,30,60,207,211,168,145,78,
167,131,213,106,69,97,97,33,92,46,23,124,62,31,188,94,175,136,169,5,15,172,19,15,177,164,35,54,37,37,133,86,13,168,84,42,240,60,15,141,70,3,149,74,133,125,251,246,193,110,183,163,164,164,4,153,153,153,
209,42,44,253,176,46,225,48,121,146,126,37,139,20,132,182,8,196,126,244,204,33,150,101,217,143,20,10,197,187,44,203,62,252,6,98,191,214,213,4,12,195,252,90,175,215,127,98,50,153,190,216,248,28,61,59,136,
149,201,100,95,106,181,90,184,92,46,156,62,125,154,206,204,38,179,137,4,101,162,201,129,88,165,82,137,188,188,60,156,60,121,18,145,72,4,225,112,88,116,7,212,214,214,134,210,210,82,164,167,167,131,231,
121,225,243,153,196,67,44,199,113,48,155,205,244,180,228,229,203,151,113,250,244,105,12,14,14,82,102,36,207,156,73,174,147,60,188,222,191,127,63,140,70,35,228,114,249,222,19,177,12,195,188,167,86,171,
97,177,88,208,212,212,132,241,241,113,92,190,124,25,147,147,147,180,51,97,51,247,36,197,14,135,15,31,166,219,159,182,182,54,20,21,21,65,169,84,126,188,43,136,85,169,84,31,229,228,228,160,187,187,27,211,
211,211,184,114,229,10,206,159,63,191,165,12,111,5,177,100,23,235,245,122,97,48,24,32,151,203,63,218,17,196,154,76,166,143,73,175,231,149,43,87,16,10,133,98,202,27,118,11,177,62,159,15,25,25,25,228,23,
225,189,109,39,98,123,122,122,112,235,214,45,140,141,141,197,29,103,182,23,136,237,232,232,64,113,113,177,176,200,253,233,16,203,178,236,58,249,173,146,2,98,59,58,58,224,241,120,160,211,233,182,151,136,
149,201,100,127,182,217,108,184,122,245,42,157,32,45,5,196,250,253,126,216,237,118,225,245,108,115,136,149,201,100,40,44,44,196,226,226,98,76,121,67,162,19,177,46,151,11,58,157,238,233,16,155,146,146,
130,198,198,70,204,207,207,139,234,9,164,128,88,114,220,87,80,123,17,31,98,211,210,210,112,246,236,89,58,222,69,106,136,173,169,169,33,125,47,177,195,186,72,247,98,110,110,46,214,214,214,104,155,110,194,
33,118,240,20,2,129,19,244,198,163,185,185,153,246,189,200,229,242,216,106,2,133,66,129,142,142,14,172,172,172,72,6,177,195,195,103,16,10,133,49,60,60,66,91,114,122,122,122,144,149,149,69,94,173,95,139,
32,86,171,213,226,214,173,91,152,156,156,148,44,17,123,97,246,50,94,121,229,30,126,252,227,31,227,181,215,94,195,253,251,247,241,206,59,239,96,124,124,156,252,38,126,34,130,88,167,211,137,219,183,111,
99,112,112,80,50,136,125,225,214,109,188,249,102,44,21,173,172,172,16,60,251,130,66,44,199,113,152,156,156,196,236,236,172,232,187,44,209,16,251,252,11,183,241,198,253,7,120,244,232,17,222,120,227,13,
188,250,234,171,120,249,229,151,209,223,223,47,156,78,247,15,136,181,90,173,95,140,141,141,209,130,98,41,33,246,31,215,180,69,4,131,65,4,2,1,116,118,118,10,159,102,125,5,177,6,131,225,147,178,178,50,156,
56,113,34,169,16,219,212,212,20,13,103,162,68,236,175,115,115,115,225,243,249,232,129,251,100,65,108,84,250,76,92,77,192,243,60,53,169,100,13,235,18,70,42,227,66,236,6,132,194,233,116,210,134,45,169,33,
54,170,181,50,62,196,114,28,135,220,220,220,141,183,238,136,164,16,235,118,187,163,195,204,155,67,108,70,70,6,202,203,203,113,226,196,9,73,19,177,46,151,75,4,104,91,66,44,199,113,200,203,203,67,48,24,
68,79,79,143,100,16,91,81,81,177,189,97,93,4,98,51,50,50,80,95,95,143,99,199,142,37,124,88,23,41,112,16,84,167,108,175,154,64,165,82,193,110,183,99,108,108,140,46,40,81,195,186,60,30,15,74,75,75,105,88,
98,219,137,88,134,97,222,51,153,76,232,233,233,193,225,195,135,233,110,113,47,137,216,234,234,106,216,237,118,232,245,122,225,93,243,199,59,130,88,142,227,62,202,204,204,68,101,101,37,234,234,234,224,
241,120,104,235,233,118,33,150,196,115,109,54,27,120,158,143,238,251,252,104,87,16,171,211,233,62,54,26,141,80,169,84,208,106,181,48,155,205,176,219,237,168,172,172,68,83,83,147,168,158,32,122,65,53,53,
53,200,205,205,141,89,140,224,126,111,111,137,88,134,97,214,213,106,53,180,90,45,180,90,45,52,26,13,12,6,3,242,243,243,225,245,122,233,208,46,210,27,91,92,92,12,131,193,32,202,129,18,133,73,56,196,146,
63,64,169,84,130,231,121,144,217,88,86,171,21,21,21,21,168,172,172,140,78,252,72,95,77,32,132,88,134,97,104,97,173,86,171,165,8,43,124,117,146,62,172,43,26,98,229,114,57,56,142,251,66,46,151,127,34,147,
201,126,253,77,53,193,63,77,71,44,199,113,159,103,101,101,129,231,249,39,50,153,236,131,103,6,177,27,223,236,168,168,168,192,200,200,8,198,199,199,209,217,217,137,162,162,34,225,158,59,57,16,75,22,227,
116,58,49,57,57,137,185,185,57,132,195,97,156,60,121,146,126,37,181,180,180,196,28,67,73,56,196,178,44,187,206,113,28,12,6,3,157,138,177,178,178,130,112,56,140,161,161,33,154,34,34,55,29,157,157,157,244,
236,67,81,81,145,240,73,232,222,134,117,41,20,138,111,235,116,186,15,205,102,51,220,110,55,134,135,135,177,176,176,128,249,249,121,76,77,77,61,21,98,73,251,252,161,67,135,208,214,214,6,143,199,35,108,
61,221,93,34,214,106,181,126,58,58,58,138,249,249,121,250,22,141,140,140,136,32,109,187,16,75,100,184,189,189,29,237,237,237,164,162,224,83,185,92,254,237,29,37,98,71,71,71,241,220,115,207,109,234,158,
187,133,216,67,135,14,161,174,174,142,128,217,135,219,134,88,189,94,143,23,95,124,145,38,169,19,13,177,62,159,15,122,189,126,251,29,177,50,153,236,203,202,202,74,220,186,117,43,166,9,32,81,16,219,222,
222,78,211,101,79,237,136,37,191,246,225,112,24,151,46,93,18,37,170,19,9,177,100,32,156,224,139,123,107,136,181,88,44,212,208,165,132,216,230,230,102,81,118,106,83,136,149,201,100,232,236,236,196,202,
202,138,228,16,235,247,251,163,207,32,199,135,88,131,193,128,229,229,101,218,126,154,12,136,21,116,84,125,25,55,17,219,214,214,134,149,149,21,122,97,76,6,196,90,173,214,205,59,98,83,83,83,177,188,188,
140,112,56,44,217,176,174,225,225,51,24,29,29,199,225,195,125,244,166,163,188,188,92,248,228,225,171,142,88,134,97,62,104,107,107,195,245,235,215,37,31,214,21,158,190,136,209,209,175,102,111,181,181,181,
9,19,140,95,85,19,24,141,198,39,228,33,145,240,203,53,209,16,59,55,183,136,151,95,254,55,252,232,71,63,194,247,190,247,61,252,203,191,252,11,190,243,157,239,224,232,209,163,228,213,250,156,66,172,205,
102,195,232,232,40,70,70,70,36,133,216,239,190,120,23,15,223,124,51,230,237,95,91,91,19,30,39,255,7,196,114,28,247,249,129,3,7,48,58,58,42,218,130,36,26,98,191,251,226,93,60,124,248,48,102,81,225,112,
152,132,117,158,136,170,9,242,242,242,104,135,190,148,16,251,189,239,189,130,171,115,215,48,50,50,130,145,145,17,244,247,247,11,221,243,3,81,71,172,74,165,194,129,3,7,112,250,244,233,164,65,172,215,235,
141,110,211,21,67,172,76,38,67,110,110,174,40,170,43,53,196,214,214,214,138,50,86,113,33,86,169,84,194,229,114,209,7,57,82,66,108,156,193,242,241,33,86,38,147,33,63,63,31,231,207,159,151,188,35,214,237,
118,71,135,153,55,135,88,189,94,143,147,39,79,210,158,88,169,32,150,12,168,220,54,196,150,151,151,99,120,120,88,178,97,93,13,13,13,194,22,221,167,67,172,92,46,95,39,111,161,112,183,152,72,136,173,171,
171,19,18,209,246,134,117,165,167,167,211,214,146,68,67,108,67,67,131,232,8,193,182,33,86,171,213,126,104,181,90,81,91,91,75,67,167,123,133,88,34,195,164,1,110,199,137,88,150,101,191,109,48,24,62,37,222,
153,149,149,133,170,170,42,218,2,176,211,142,216,138,138,10,58,138,74,32,195,159,238,58,17,203,113,28,117,78,173,86,43,114,206,173,22,84,91,91,27,3,177,27,79,212,193,48,204,135,12,195,124,123,207,195,
186,8,108,232,245,122,168,84,42,236,223,191,31,13,13,13,104,108,108,164,11,242,120,60,112,58,157,200,202,202,162,89,225,40,21,94,151,4,98,201,56,32,189,94,15,181,90,141,253,251,247,163,166,166,6,149,149,
149,176,88,44,241,26,36,146,155,136,37,241,92,165,82,9,165,82,25,51,172,235,89,66,236,7,28,199,61,217,200,172,124,254,77,71,236,63,237,176,46,149,74,245,196,100,50,253,141,101,217,223,60,51,136,77,73,
73,1,137,92,246,246,246,98,124,124,28,189,189,189,200,207,207,39,215,167,245,164,65,172,94,175,71,110,110,46,60,30,15,134,135,135,177,178,178,130,217,217,89,186,107,245,249,124,168,172,172,68,90,90,154,
240,57,77,98,32,86,161,80,44,106,181,218,63,104,52,154,117,171,213,10,175,215,139,64,32,128,83,167,78,33,20,10,33,18,137,96,106,106,10,67,67,67,162,91,179,174,174,46,28,58,116,136,30,186,80,171,213,194,
103,53,123,75,196,102,102,102,126,210,215,215,135,75,151,46,225,218,181,107,152,157,157,197,248,248,56,70,71,71,113,234,212,41,81,68,55,30,196,182,181,181,193,239,247,163,174,174,14,57,57,57,187,63,120,
33,236,10,58,126,252,56,150,151,151,17,10,133,48,56,56,24,195,140,219,129,216,142,142,14,186,211,240,249,124,40,42,42,34,219,224,157,119,196,154,76,38,220,189,123,55,238,193,251,221,64,44,177,78,242,5,
190,129,255,159,200,229,242,197,109,15,235,234,237,237,197,243,207,63,31,247,21,218,11,196,182,183,183,163,165,165,5,7,15,30,4,207,243,96,89,246,15,219,130,88,150,101,113,247,238,93,76,77,77,73,6,177,
94,175,151,252,102,62,29,98,89,150,69,93,93,29,238,220,185,35,42,111,72,52,196,250,253,126,33,106,108,13,177,38,147,9,75,75,75,180,214,82,42,136,37,195,6,159,10,177,60,207,163,163,163,3,55,111,222,20,
69,189,165,130,88,193,4,149,248,16,203,178,236,122,97,97,33,150,151,151,69,125,9,82,66,172,211,233,20,197,3,98,32,214,108,54,35,20,10,97,110,110,78,186,106,130,40,136,141,238,94,20,65,44,199,113,191,241,
251,253,184,126,253,58,70,70,70,146,6,177,205,205,205,244,48,107,12,196,230,228,228,124,22,10,133,48,51,51,67,35,186,82,65,108,232,220,52,142,30,61,70,47,164,78,167,51,250,32,226,63,32,150,231,249,39,
157,157,157,56,123,246,172,232,171,67,138,97,93,151,46,95,195,68,104,146,14,124,110,104,104,16,62,57,21,87,19,20,20,20,72,158,136,125,110,237,5,252,240,135,255,142,123,247,126,68,139,144,167,166,166,80,
82,82,18,191,154,192,100,50,97,100,100,4,129,64,64,50,136,125,249,229,123,120,235,173,216,203,75,36,18,33,183,102,98,136,229,121,30,78,167,19,35,35,35,146,65,236,43,255,246,239,120,244,232,81,204,162,
166,167,167,201,103,234,51,17,196,166,164,164,32,63,63,159,230,135,165,130,216,159,188,250,83,92,184,112,1,147,147,147,136,68,34,152,158,158,70,78,78,14,121,235,126,19,3,177,54,155,13,3,3,3,8,4,2,73,171,
38,16,62,48,138,11,177,6,131,1,29,29,29,24,30,30,78,74,34,54,106,76,222,250,166,16,91,85,85,133,177,177,49,201,33,182,166,166,38,58,225,177,57,196,58,28,14,204,204,204,136,30,221,75,1,177,81,79,69,183,
134,88,139,197,130,72,36,66,135,66,72,1,177,110,183,59,70,100,182,132,88,173,86,187,222,209,209,17,213,149,144,88,136,117,56,28,34,212,127,42,196,242,60,255,135,236,236,108,212,212,212,80,120,77,36,196,
186,221,110,97,148,105,123,16,203,178,236,162,193,96,248,36,45,45,13,164,253,205,239,247,39,12,98,163,94,165,157,119,196,170,213,106,232,245,122,144,156,122,67,67,3,61,117,182,93,136,109,104,104,128,199,
227,129,219,237,70,65,65,129,80,250,246,54,172,139,97,24,24,12,6,250,202,53,53,53,209,155,203,120,11,114,187,221,40,45,45,69,65,65,1,172,86,43,116,58,29,20,10,133,240,14,249,147,61,39,98,73,174,147,231,
121,24,141,70,88,44,22,212,212,212,196,44,200,229,114,33,43,43,11,58,157,142,158,71,38,63,27,50,188,174,80,40,254,192,243,252,98,66,33,150,227,56,24,141,70,232,245,122,20,20,20,192,229,114,225,192,129,
3,116,114,133,92,46,23,45,36,105,16,203,48,204,186,70,163,129,209,104,132,70,163,129,92,46,23,26,249,179,133,88,150,101,127,195,243,252,103,12,195,60,249,102,88,215,63,61,196,110,4,155,63,208,104,52,191,
83,42,149,191,124,38,16,75,58,56,116,58,29,50,50,50,80,81,81,65,211,138,249,249,249,96,89,86,26,136,101,24,230,168,82,169,124,139,60,202,77,75,75,131,197,98,129,213,106,69,117,117,53,58,59,59,49,56,56,
136,153,153,25,68,34,17,140,143,143,35,24,12,162,171,171,11,101,101,101,194,61,83,226,18,177,6,131,225,221,234,234,234,245,209,209,81,156,61,123,22,67,67,67,24,31,31,199,236,236,44,194,225,48,198,199,
199,113,230,204,25,58,13,74,152,138,37,23,212,180,180,52,122,130,54,33,195,186,234,235,235,177,176,176,128,153,153,25,12,13,13,225,228,201,147,24,26,26,18,197,227,54,115,207,238,238,110,52,53,53,209,135,
144,130,96,196,238,33,214,100,50,97,117,117,21,23,46,92,136,137,87,238,100,122,152,207,231,67,77,77,13,10,11,11,133,89,132,157,67,172,76,38,67,32,16,192,205,155,55,113,242,228,201,61,65,108,103,103,39,
252,126,63,90,90,90,208,216,216,136,194,194,66,178,167,218,254,176,46,134,97,96,54,155,113,247,238,93,156,59,119,46,97,16,75,198,79,145,67,62,25,25,25,224,56,238,93,134,97,142,62,21,98,141,70,35,130,193,
32,86,87,87,69,159,157,68,65,172,223,239,167,219,226,141,221,196,91,91,66,172,82,169,68,107,107,43,22,23,23,49,62,62,46,25,196,30,58,116,72,120,60,115,107,136,117,185,92,88,90,90,66,56,28,166,61,195,82,
64,236,161,67,135,68,163,130,54,133,216,244,244,116,124,235,91,223,194,236,236,172,136,171,165,130,216,218,218,218,173,33,86,173,86,175,71,34,17,44,47,47,139,62,71,82,66,108,67,67,131,240,164,182,24,98,
181,90,237,47,157,78,39,94,122,233,37,209,156,190,100,36,98,53,26,77,124,136,213,235,245,191,107,109,109,197,149,43,87,68,23,198,100,36,98,5,147,84,196,16,43,151,203,63,112,56,28,184,120,241,162,232,74,
45,53,196,182,180,180,68,155,130,120,88,87,126,126,62,109,102,150,18,98,103,103,175,96,120,248,52,93,20,57,101,187,177,175,23,67,172,201,100,162,51,67,165,28,214,181,184,244,28,166,167,103,233,103,202,
237,118,71,247,244,127,5,177,122,189,30,77,77,77,244,225,180,36,137,216,239,222,197,171,175,254,39,94,126,249,21,92,189,122,149,158,14,16,204,198,18,15,235,98,89,22,197,197,197,152,156,156,164,230,153,
104,136,125,237,181,159,198,244,14,223,191,127,31,237,237,237,228,237,19,39,98,9,152,93,186,116,137,110,65,18,13,177,255,251,141,159,197,45,67,30,30,30,38,159,171,223,197,64,172,213,106,197,236,236,172,
164,137,216,31,255,248,39,152,156,156,196,226,226,34,238,220,185,131,127,253,215,127,165,109,0,44,203,254,50,6,98,173,86,43,70,71,71,17,8,4,36,135,88,194,75,2,29,142,15,177,102,179,25,67,67,67,8,6,131,
73,73,196,10,185,122,83,136,85,169,84,116,15,46,37,196,122,60,158,232,2,135,173,33,182,162,162,66,52,61,76,10,136,45,45,45,141,46,27,221,28,98,57,142,123,43,59,59,27,69,69,69,240,249,124,244,198,50,145,
16,91,95,95,143,125,251,246,109,31,98,25,134,57,202,243,252,187,38,147,9,58,157,14,21,21,21,9,135,216,168,135,217,59,27,214,69,212,110,223,190,125,56,120,240,32,253,109,217,11,196,86,87,87,71,199,189,
119,14,177,50,153,12,26,141,6,169,169,169,112,56,28,104,109,109,165,153,130,157,36,98,73,136,57,234,183,109,111,16,203,243,60,50,50,50,144,157,157,77,95,173,205,22,68,66,203,110,183,27,21,21,21,40,41,
41,65,73,73,9,44,22,75,226,58,98,9,196,166,164,164,32,45,45,13,58,157,14,110,183,27,109,109,109,180,146,192,235,245,162,188,188,28,185,185,185,176,88,44,20,98,57,142,35,83,7,232,130,88,150,125,55,225,
137,88,158,231,97,48,24,144,151,151,7,135,195,1,139,197,2,141,70,35,146,224,148,148,20,209,95,201,193,66,157,78,247,86,97,97,225,81,73,32,86,161,80,172,235,245,122,40,149,74,250,7,147,159,232,49,102,73,
133,88,149,74,245,75,133,66,241,187,141,237,198,255,253,6,98,255,191,128,88,242,246,201,229,242,7,146,67,44,203,178,19,4,98,55,128,21,50,153,140,110,250,73,12,220,96,48,160,176,176,16,197,197,197,40,46,
46,94,103,89,246,177,100,137,88,147,201,244,153,199,227,65,125,125,61,92,46,23,189,62,121,189,94,28,57,114,132,134,82,103,103,103,49,54,54,134,115,231,206,161,175,175,15,217,217,217,210,12,235,50,153,
76,235,164,23,246,204,153,51,24,31,31,199,216,216,24,70,70,70,112,246,236,89,218,168,123,234,212,41,28,59,118,12,125,125,125,56,114,228,8,45,126,20,148,243,37,6,98,141,70,35,134,135,135,177,188,188,76,
179,48,219,117,207,158,158,30,180,183,183,211,90,2,193,3,200,221,67,172,76,38,67,119,119,55,214,214,214,112,246,236,217,93,65,44,137,235,214,214,214,210,190,132,141,11,236,206,33,86,46,151,35,43,43,11,
47,190,248,34,166,166,166,68,131,186,118,147,136,245,251,253,168,174,174,70,126,126,190,80,90,182,15,177,100,212,203,185,115,231,176,176,176,32,242,170,189,64,172,223,239,167,103,106,4,199,88,182,87,77,
96,52,26,49,59,59,139,185,185,57,209,144,165,68,64,44,153,139,229,114,185,144,155,155,11,165,82,249,244,68,172,86,171,197,228,228,36,158,123,238,57,12,15,15,211,207,81,162,33,150,44,204,108,54,35,37,37,
229,51,133,66,49,17,23,98,211,210,210,16,8,4,112,235,214,173,152,87,72,10,136,21,14,254,142,11,177,60,207,63,30,27,27,163,67,223,146,49,172,171,181,181,21,102,179,121,115,136,45,42,42,194,15,126,240,3,
90,4,146,12,136,245,249,124,116,223,30,51,172,75,46,151,63,40,40,40,192,149,43,87,68,215,34,169,33,86,184,168,184,213,4,5,5,5,152,159,159,23,205,121,76,6,196,70,221,225,252,31,81,71,108,106,106,42,125,
132,150,204,68,172,160,181,36,22,98,83,83,83,17,8,4,68,199,77,146,1,177,81,7,164,197,16,203,243,60,186,187,187,49,62,62,46,41,196,206,207,47,35,20,154,162,247,138,81,105,88,49,196,202,100,50,184,221,110,
76,77,77,73,218,17,123,243,230,109,132,167,47,136,66,59,113,135,117,109,180,109,161,188,188,28,51,51,51,146,37,98,127,248,195,255,192,253,251,15,112,231,206,75,56,123,246,44,6,7,7,209,220,220,28,191,35,
150,64,172,211,233,196,236,236,172,100,213,4,15,163,58,135,95,125,245,85,44,46,46,10,119,13,177,213,4,69,69,69,136,68,34,18,13,235,250,5,126,254,243,216,127,118,255,254,125,154,73,103,89,246,129,8,98,
201,25,7,178,40,169,32,118,121,121,25,215,174,93,195,247,191,255,125,188,243,206,59,120,225,133,23,232,169,144,184,16,75,30,26,145,76,129,84,16,219,218,218,74,143,220,9,158,54,60,142,11,177,249,249,249,
180,131,67,234,68,108,101,101,165,232,120,112,92,136,149,203,229,19,70,163,241,179,140,140,12,212,213,213,73,154,136,173,172,172,140,22,189,173,19,177,251,246,237,91,207,200,200,64,73,73,137,104,80,87,
162,32,182,178,178,82,116,44,101,219,213,4,60,207,35,61,61,29,105,105,105,168,175,175,79,24,196,214,215,215,139,166,63,237,24,98,201,217,208,212,212,84,120,189,94,122,171,180,91,136,37,71,53,163,190,82,
118,14,177,12,195,192,104,52,34,53,53,21,30,143,135,222,42,237,6,98,75,74,74,162,65,127,239,16,155,158,158,142,170,170,42,209,162,54,91,80,67,67,131,40,47,28,231,131,157,24,136,85,42,149,48,155,205,112,
185,92,232,234,234,162,85,4,110,183,27,69,69,69,112,56,28,200,201,201,129,217,108,70,102,102,38,44,22,11,210,210,210,144,145,145,33,92,80,226,171,9,180,90,45,50,50,50,144,155,155,11,189,94,15,141,70,35,
66,87,133,66,65,85,88,248,179,241,182,125,38,89,34,86,161,80,60,38,36,173,80,40,98,34,185,66,136,37,39,182,85,42,213,125,155,205,54,33,57,196,110,72,221,166,146,247,13,196,254,83,67,108,210,170,9,212,
106,245,47,82,83,83,255,102,52,26,215,149,74,37,61,151,71,70,232,177,44,75,167,210,105,181,90,112,28,183,206,243,252,95,149,74,229,175,36,171,38,32,9,143,177,177,49,156,56,113,98,99,31,31,68,123,123,59,
58,58,58,208,219,219,139,206,206,78,156,56,113,2,71,142,28,193,200,200,8,218,219,219,201,60,191,79,18,10,177,28,199,173,59,28,14,124,231,59,223,17,245,194,14,14,14,34,24,12,226,216,177,99,24,26,26,162,
63,193,96,144,246,163,117,119,119,163,174,174,142,140,157,90,79,8,196,146,243,89,151,47,95,198,149,43,87,98,152,113,59,238,217,220,220,140,253,251,247,139,58,130,118,13,177,12,195,32,45,45,13,183,111,
223,70,36,18,193,201,147,39,119,5,177,100,194,83,94,94,158,48,170,187,115,136,101,24,6,249,249,249,152,156,156,196,245,235,215,69,193,193,221,64,108,91,91,27,234,235,235,97,183,219,105,49,205,142,33,214,
106,181,98,121,121,25,151,47,95,166,103,147,247,10,177,135,14,29,66,117,117,53,114,114,114,132,251,243,167,67,172,82,169,92,47,43,43,195,149,43,87,48,63,63,47,58,227,158,8,136,245,251,253,168,175,175,
71,105,105,41,180,90,45,228,114,249,214,16,203,178,236,39,29,29,29,152,159,159,143,249,12,37,26,98,107,107,107,97,183,219,201,126,61,126,34,118,223,190,125,191,170,170,170,194,157,59,119,48,53,53,37,250,
12,73,1,177,62,159,143,78,251,85,171,213,72,73,73,249,69,12,196,26,141,198,191,206,204,204,224,242,229,203,49,143,59,164,130,88,191,223,143,202,202,74,114,24,232,111,113,19,177,227,227,227,49,50,44,37,
196,182,182,182,162,166,166,134,238,74,99,170,9,140,70,35,134,134,134,68,213,190,82,67,108,107,107,43,197,141,184,16,171,80,40,48,48,48,128,115,231,206,37,21,98,163,38,146,139,33,86,163,209,160,189,189,
29,231,206,157,75,42,196,70,241,162,24,98,83,82,82,208,214,214,134,233,233,233,164,66,108,212,113,77,49,196,170,84,42,180,182,182,34,28,14,39,21,98,227,220,53,127,5,177,28,199,97,112,112,80,114,136,125,
110,237,22,66,231,206,211,155,215,205,200,186,110,227,189,196,240,240,48,198,199,199,37,237,136,189,123,247,7,56,115,118,130,222,77,71,37,207,196,16,155,150,150,134,145,145,17,140,142,142,74,6,177,15,
30,60,196,195,135,111,97,121,121,25,193,96,16,29,29,29,209,139,138,29,214,21,137,68,48,50,50,146,20,136,125,253,245,215,177,186,186,138,150,150,150,205,171,9,210,211,211,215,79,159,62,77,247,222,82,65,
108,188,34,228,13,107,136,77,196,166,165,165,253,237,224,193,131,24,26,26,138,25,239,154,72,136,13,135,195,184,115,231,142,40,182,187,241,74,253,53,6,98,149,74,229,47,200,140,246,214,214,86,58,20,87,10,
136,61,124,248,48,142,31,63,142,193,193,65,170,195,42,149,234,87,155,38,98,211,211,211,145,147,147,131,190,190,62,201,32,182,166,166,6,102,179,89,116,130,123,75,136,77,73,73,89,55,153,76,208,235,245,104,
104,104,16,97,108,34,32,182,186,186,58,218,61,183,55,172,75,38,147,129,88,84,103,103,39,29,169,177,87,136,61,120,240,96,244,208,183,157,65,44,195,48,200,200,200,64,122,122,58,252,126,63,58,59,59,247,4,
177,110,183,59,122,128,224,222,32,54,59,59,27,157,157,157,180,150,96,167,16,91,83,83,35,108,139,79,12,196,90,44,22,148,150,150,226,240,225,195,59,94,144,199,227,137,126,120,157,24,136,85,40,20,235,153,
153,153,200,207,207,71,77,77,13,170,171,171,81,82,82,130,178,178,50,216,237,118,20,20,20,192,110,183,163,176,176,16,101,101,101,40,42,42,130,203,229,66,121,121,57,178,178,178,68,220,152,80,136,149,203,
229,159,164,166,166,82,234,33,17,92,33,184,146,48,42,249,119,10,133,66,52,34,93,18,136,85,171,213,191,226,56,238,175,114,185,124,157,164,95,201,95,133,249,96,146,23,78,73,73,89,103,89,246,111,42,149,234,
23,207,100,88,215,55,16,251,79,3,177,28,199,173,126,109,32,118,227,108,205,227,3,7,14,172,187,221,110,56,28,14,164,167,167,211,175,14,2,98,228,105,213,6,39,126,196,48,204,67,201,32,86,173,86,227,216,177,
99,136,68,34,8,133,66,24,26,26,162,16,210,215,215,71,47,154,78,167,83,116,45,179,217,108,72,77,77,253,59,203,178,111,38,12,98,73,120,176,163,163,3,183,110,221,194,248,248,184,40,209,72,64,36,16,8,224,
248,241,227,20,99,79,157,58,133,96,48,136,96,48,72,30,100,39,14,98,121,158,199,196,196,4,230,230,230,68,189,140,59,113,79,183,219,13,139,197,66,142,248,238,30,98,89,150,133,205,102,67,48,24,164,83,202,
133,58,188,221,5,145,71,189,165,165,165,66,33,222,29,196,238,223,191,31,11,11,11,184,114,229,10,206,156,57,179,231,97,93,94,175,23,54,155,109,119,16,155,154,154,138,222,222,94,204,205,205,225,234,213,
171,56,113,226,132,8,210,118,11,177,109,109,109,244,8,157,224,240,234,214,16,203,113,220,155,38,147,105,125,122,122,26,171,171,171,56,119,238,28,157,103,148,40,136,37,167,255,83,83,83,105,28,124,75,136,
181,219,237,255,179,184,184,72,43,52,133,175,78,34,33,182,185,185,25,149,149,149,200,200,200,32,175,88,124,136,101,24,230,161,221,110,199,205,155,55,113,234,212,169,152,22,221,68,67,108,99,99,35,138,138,
138,96,52,26,201,14,35,126,53,65,81,81,17,86,86,86,98,130,240,82,65,44,201,43,232,116,58,200,229,242,199,155,118,196,46,46,46,38,45,17,219,218,218,138,186,186,58,58,161,78,163,209,172,198,12,235,42,42,
42,194,194,194,66,82,33,182,169,169,137,126,119,198,64,108,74,74,10,202,203,203,49,55,55,151,84,136,245,122,189,194,39,91,98,136,149,201,100,240,120,60,184,114,229,74,82,33,182,174,174,110,115,136,229,
121,30,189,189,189,184,112,225,66,82,33,86,232,232,49,16,43,147,201,16,10,133,232,33,156,100,65,108,97,97,225,230,16,155,153,153,73,37,47,153,16,155,151,151,183,57,196,146,135,142,193,96,80,82,136,125,
241,206,247,112,122,236,44,189,155,22,36,247,99,33,214,225,112,96,121,121,25,3,3,3,146,14,235,250,241,127,252,167,104,88,87,122,122,250,230,16,235,112,56,48,53,53,133,35,71,142,72,10,177,247,239,63,192,
216,216,24,122,122,122,224,245,122,69,18,35,130,88,142,227,86,179,179,179,209,212,212,132,227,199,143,39,5,98,159,127,254,121,28,63,126,92,8,104,177,16,203,113,220,227,172,172,44,184,92,46,244,244,244,
136,54,105,137,130,216,232,86,137,7,15,30,8,43,84,226,87,19,144,201,226,228,204,94,162,33,118,98,98,2,171,171,171,116,92,222,189,123,247,232,3,35,149,74,245,112,83,136,37,67,227,61,30,143,104,4,85,162,
32,150,84,8,95,187,118,13,141,141,141,100,91,252,63,91,66,44,9,5,90,173,86,81,2,35,81,16,91,87,87,7,187,221,46,172,39,88,215,233,116,111,110,11,98,211,210,210,96,179,217,208,213,213,133,174,174,174,132,
64,172,203,229,18,166,170,119,7,177,38,147,9,78,167,19,3,3,3,123,238,136,45,43,43,139,206,12,239,30,98,115,114,114,54,222,186,46,209,130,182,235,158,100,106,88,156,41,80,187,135,88,157,78,7,147,201,68,
135,142,52,53,53,209,158,31,50,71,123,43,136,45,44,44,140,183,160,189,67,172,193,96,88,55,153,76,72,77,77,133,201,100,66,70,70,6,50,51,51,145,153,153,137,188,188,60,228,231,231,227,224,193,131,112,58,
157,112,187,221,104,104,104,64,109,109,45,28,14,71,116,94,248,203,132,65,172,66,161,120,211,96,48,252,143,90,173,134,90,173,134,74,165,138,249,225,56,14,42,149,10,60,207,35,45,45,77,120,95,39,45,196,202,
229,242,135,164,35,86,152,126,37,127,56,25,218,69,22,185,241,239,31,63,139,97,93,155,66,172,197,98,89,253,6,98,191,150,137,88,149,74,245,142,70,163,249,173,92,46,255,240,153,67,44,233,136,245,249,124,
232,235,235,67,119,119,55,14,28,56,0,139,197,2,165,82,25,221,202,252,101,82,134,117,89,44,22,4,131,65,209,20,40,178,133,30,24,24,128,203,229,66,73,73,9,172,86,43,82,83,83,193,113,28,153,139,245,88,46,
151,255,44,161,16,171,80,40,80,88,88,136,80,40,132,229,229,101,81,206,138,84,17,144,187,160,129,129,1,12,13,13,161,175,175,15,193,96,144,222,158,103,103,103,175,43,149,202,247,246,12,177,114,185,28,26,
141,6,117,117,117,88,91,91,67,36,18,17,85,33,70,187,167,208,60,123,123,123,113,248,240,97,244,244,244,160,175,175,79,56,26,118,215,16,251,39,153,76,6,187,221,142,80,40,132,171,87,175,98,122,122,90,36,
50,59,129,216,174,174,46,52,52,52,32,35,35,131,92,108,255,180,99,136,181,217,108,8,4,2,184,124,249,50,174,94,189,138,145,145,17,145,12,239,6,98,219,218,218,80,86,86,22,93,230,240,116,136,77,79,79,127,
207,231,243,33,18,137,208,35,229,66,247,220,235,176,46,175,215,75,61,125,227,107,104,107,136,77,73,73,249,89,89,89,25,238,220,185,131,112,56,28,211,162,155,8,136,109,107,107,131,211,233,36,122,247,116,
136,101,24,230,113,111,111,47,61,185,45,37,196,110,204,90,163,229,70,155,118,196,234,245,122,132,66,33,58,191,93,42,136,37,179,253,108,54,27,245,244,77,33,86,173,86,227,234,213,171,73,171,38,32,137,142,
141,231,53,177,213,4,50,153,236,75,155,205,134,229,229,101,12,13,13,37,5,98,155,154,154,80,94,94,78,110,40,98,135,117,145,105,228,55,111,222,164,191,109,201,128,88,143,199,131,204,204,76,114,124,234,29,
17,196,202,100,50,148,148,148,96,113,113,49,169,16,219,210,210,130,130,130,2,242,185,250,173,8,98,89,150,197,145,35,71,112,245,234,85,90,201,154,12,136,109,105,105,17,166,60,62,20,65,172,86,171,197,220,
220,28,66,161,80,82,33,214,227,241,8,111,82,197,16,107,183,219,177,180,180,132,96,48,152,84,136,141,106,105,22,67,172,211,233,196,234,234,170,164,195,186,226,65,108,84,206,243,43,136,101,89,246,195,190,
190,62,68,34,17,209,183,189,212,16,75,190,7,227,66,172,78,167,251,109,85,85,21,66,161,144,104,79,36,5,196,254,232,223,95,19,65,108,212,1,215,175,32,150,227,184,119,242,242,242,224,247,251,233,160,46,41,
32,246,209,163,95,224,222,189,31,137,134,131,11,143,115,198,29,214,101,179,217,208,222,222,142,190,190,62,201,32,246,165,151,94,194,177,99,199,208,216,216,136,234,234,106,209,77,70,220,106,130,180,180,
52,88,173,86,28,59,118,76,18,136,37,63,223,255,254,247,113,250,244,233,232,254,141,205,135,117,153,205,102,148,149,149,209,206,132,68,65,236,155,111,190,25,243,247,135,15,31,222,94,71,44,203,178,200,206,
206,134,223,239,143,154,141,181,55,136,157,156,156,68,56,28,198,79,126,242,19,250,207,123,123,123,41,130,60,21,98,121,158,135,213,106,165,179,141,19,5,177,62,159,15,39,78,156,192,210,210,18,102,102,102,
168,161,235,116,186,159,61,21,98,25,134,129,197,98,129,199,227,193,192,192,64,194,170,9,234,234,234,144,159,159,15,147,201,68,249,136,101,217,247,118,84,77,144,155,155,139,254,254,126,122,39,178,91,136,
245,120,60,40,43,43,67,90,90,90,116,181,230,142,33,246,79,38,147,9,165,165,165,8,4,2,49,11,218,9,196,230,229,229,9,11,255,133,87,239,221,65,108,86,86,22,28,14,7,74,74,74,80,89,89,9,143,199,131,218,218,
90,180,182,182,194,235,245,110,185,160,170,170,42,81,162,58,97,29,177,58,157,238,61,163,209,8,158,231,161,86,171,193,178,44,120,158,135,94,175,135,217,108,134,195,225,128,219,237,70,77,77,13,90,90,90,
104,79,66,105,105,105,244,87,72,98,33,150,101,217,159,201,100,178,199,228,212,63,193,87,158,231,161,209,104,160,80,40,160,211,233,96,181,90,81,84,84,132,172,172,172,120,111,151,180,195,186,132,51,140,
24,134,65,74,74,10,93,164,74,165,18,198,144,146,63,172,43,30,196,50,12,243,33,203,178,191,101,89,246,157,111,18,177,95,251,106,2,163,209,248,23,189,94,255,197,198,231,235,243,103,10,177,60,207,195,231,
243,97,120,120,24,71,142,28,65,67,67,3,76,38,83,76,229,78,82,32,86,38,147,161,172,172,12,39,79,158,196,242,242,50,38,38,38,232,29,80,119,119,55,252,126,63,74,74,74,200,124,34,225,165,225,113,66,171,9,
200,37,192,98,177,160,171,171,139,30,158,38,165,145,253,253,253,49,222,217,209,209,129,238,238,110,56,157,78,20,22,22,98,223,190,125,80,171,213,235,44,203,254,62,33,137,88,139,197,2,191,223,143,169,169,
41,44,44,44,96,98,98,130,82,117,60,247,236,238,238,22,45,144,108,175,123,122,122,224,112,56,160,86,171,63,221,21,196,202,229,242,15,236,118,59,186,187,187,17,14,135,177,176,176,128,201,201,73,145,200,
108,23,98,201,115,232,142,142,14,212,214,214,66,171,213,130,97,152,15,118,4,177,185,185,185,159,14,14,14,34,18,137,208,91,249,232,174,132,221,64,44,49,79,193,236,181,237,65,44,199,113,191,239,233,233,
193,218,218,26,253,16,39,106,88,23,49,207,226,226,98,225,3,202,167,87,19,24,141,198,245,197,197,69,132,195,225,24,247,76,212,176,46,143,199,179,125,136,149,201,100,143,45,22,75,76,63,108,162,33,214,239,
247,71,63,95,222,122,88,87,81,81,17,110,223,190,29,147,25,78,116,34,182,182,182,22,58,157,142,46,108,83,136,85,42,149,232,238,238,198,181,107,215,36,135,88,159,207,135,162,162,34,225,211,135,248,16,171,
209,104,16,137,68,112,254,252,249,164,64,108,93,93,157,240,206,102,115,136,189,126,253,58,78,157,58,149,180,142,216,138,138,10,138,252,49,137,88,153,76,134,150,150,22,172,172,172,96,112,112,48,105,16,
75,144,127,227,213,18,39,98,77,38,19,46,93,186,132,112,56,156,212,68,108,75,75,11,213,60,134,97,254,66,33,150,97,152,207,115,115,115,241,220,115,207,37,189,35,182,185,185,89,72,140,95,136,32,54,24,12,
226,218,181,107,56,122,244,104,82,33,150,124,237,196,64,172,217,108,254,98,104,104,136,242,98,50,33,214,229,114,9,111,82,63,167,16,155,154,154,250,151,154,154,26,218,164,155,204,68,108,84,95,130,56,17,
155,151,151,135,246,246,118,145,121,74,13,177,110,183,59,122,251,44,78,196,170,213,106,228,228,228,208,240,169,84,195,186,94,121,229,135,212,27,162,120,113,115,136,173,171,171,195,224,224,160,100,16,123,
254,252,121,10,32,209,147,238,227,66,172,208,164,200,182,54,145,16,251,232,209,35,140,143,143,163,173,173,13,94,175,55,58,97,189,57,196,146,196,226,241,227,199,19,154,136,37,63,111,188,241,6,194,225,48,
14,29,58,36,74,162,109,9,177,228,225,17,97,235,68,64,236,253,251,247,241,218,107,175,137,22,23,10,133,132,97,157,167,67,44,41,175,61,118,236,88,66,58,98,123,123,123,49,52,52,36,106,148,136,68,34,228,88,
221,246,171,9,242,242,242,232,91,152,40,136,13,4,2,152,159,159,199,218,218,26,234,235,235,201,111,220,239,183,13,177,153,153,153,240,122,189,24,24,24,160,11,218,75,34,182,166,166,6,121,121,121,48,153,
76,194,131,135,159,238,8,98,229,114,249,7,54,155,13,78,167,147,146,33,89,204,78,32,150,20,171,241,60,79,246,76,100,65,31,236,10,98,211,211,211,63,53,26,141,80,42,149,224,121,30,89,89,89,40,40,40,128,219,
237,134,219,237,166,19,6,226,45,168,190,190,30,14,135,3,26,141,134,46,36,97,16,203,178,236,239,57,142,91,87,42,149,208,106,181,164,232,17,122,189,30,118,187,29,46,151,11,94,175,87,180,160,202,202,74,145,
153,11,22,243,101,162,59,98,31,147,160,105,74,74,10,120,158,39,104,1,163,209,136,146,146,18,148,150,150,34,55,55,55,94,86,56,185,16,43,84,98,165,82,25,147,21,126,102,16,187,33,120,224,56,238,11,150,101,
255,242,77,53,193,86,16,43,151,203,253,95,171,142,88,189,94,255,196,96,48,128,231,249,39,207,20,98,101,50,25,82,83,83,81,85,85,133,241,241,113,156,58,117,10,29,29,29,176,217,108,194,103,50,201,129,88,
134,97,192,243,60,26,26,26,48,60,60,140,235,215,175,99,114,114,146,238,88,59,59,59,81,83,83,3,187,221,46,204,164,39,126,88,215,198,44,24,24,12,6,248,124,62,144,97,57,225,112,24,195,195,195,34,102,36,231,
108,136,172,20,23,23,131,231,121,114,158,125,239,213,4,41,41,41,239,219,108,54,184,92,46,12,15,15,99,102,102,6,145,72,4,147,147,147,212,27,54,115,207,232,81,160,141,141,141,200,203,203,35,135,161,223,
223,21,196,230,231,231,255,125,112,112,16,23,47,94,196,220,220,28,194,225,48,70,70,70,68,149,154,59,129,88,66,216,132,128,120,158,255,251,142,32,86,161,80,188,31,8,4,176,180,180,68,195,167,209,238,185,
219,68,172,223,239,167,25,98,134,97,222,223,118,53,65,78,78,14,110,221,186,133,115,231,206,197,184,231,94,33,182,163,163,3,77,77,77,132,173,183,7,177,12,195,172,87,84,84,224,91,223,250,86,204,43,148,40,
136,61,116,232,144,240,92,223,246,134,117,157,59,119,46,102,200,82,162,59,98,203,203,203,133,219,154,173,33,54,45,45,45,38,4,47,5,196,54,52,52,8,71,186,108,14,177,12,195,160,177,177,17,171,171,171,34,
29,150,10,98,201,244,148,45,33,86,171,213,226,194,133,11,152,153,153,73,10,196,186,92,46,97,73,72,124,136,173,170,170,194,173,91,183,232,113,1,169,33,182,177,177,17,86,171,85,56,68,64,12,177,25,25,25,
152,153,153,193,236,236,108,210,59,98,5,223,145,98,136,173,172,172,196,237,219,183,69,223,101,201,128,216,134,134,6,100,101,101,197,14,235,50,26,141,79,166,167,167,113,241,226,69,209,87,71,178,33,150,
97,152,39,50,153,172,81,38,151,203,253,25,25,25,152,152,152,192,232,232,168,232,187,44,25,16,219,212,212,132,138,138,10,225,165,193,47,147,201,100,141,122,189,254,73,89,89,153,104,122,88,178,32,182,169,
169,73,120,105,120,34,170,38,200,205,205,165,117,77,201,132,216,170,170,42,225,19,121,49,196,146,27,202,161,161,161,164,38,98,5,223,129,241,135,117,229,229,229,161,191,191,159,30,248,146,26,98,171,170,
170,158,158,136,229,121,30,78,167,19,193,96,80,50,136,157,154,154,162,60,20,245,42,197,135,88,210,57,53,60,60,44,9,196,146,97,93,36,55,44,232,115,217,26,98,141,70,35,29,37,156,104,136,189,113,227,6,250,
250,250,232,72,117,33,128,108,9,177,60,207,163,184,184,24,167,79,159,78,88,71,172,240,103,105,105,9,125,125,125,200,204,204,220,254,176,46,134,97,214,237,118,59,78,159,62,45,170,37,216,11,196,222,184,
113,67,180,176,72,36,34,124,252,177,61,136,37,217,150,254,254,254,152,91,165,221,64,108,95,95,31,150,150,150,68,111,163,90,173,222,89,53,65,74,74,202,251,217,217,217,168,169,169,129,207,231,219,115,34,
214,235,245,130,28,87,88,91,91,67,91,91,27,121,149,222,223,17,196,154,205,230,191,27,141,70,24,12,6,100,101,101,81,144,245,249,124,219,134,88,162,194,37,37,37,200,204,204,132,193,96,16,110,129,255,190,
43,136,149,203,229,239,107,52,26,232,116,58,26,62,37,206,73,16,54,222,130,188,94,47,156,78,39,108,54,27,133,88,225,143,76,38,123,127,207,213,4,12,195,172,147,186,213,125,251,246,65,171,213,34,55,55,23,
85,85,85,180,67,65,120,149,206,204,204,164,219,220,168,197,36,126,88,23,185,37,82,40,20,208,235,245,208,233,116,32,93,48,149,149,149,116,28,122,28,21,78,30,196,170,213,106,236,219,183,79,84,69,16,5,248,
207,14,98,149,74,229,147,141,230,220,39,95,171,142,216,141,29,99,66,32,246,255,13,0,178,128,2,218,17,75,66,59,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00135_png = (const char*) resource_TalComponent_bmp00135_png;
const int TalComponent::bmp00135_pngSize = 53534;
