#include "includes.h"
#include "TalComponent.h"

TalComponent::TalComponent (TalCore* const ownerFilter)
: AudioProcessorEditor (ownerFilter)
{
    const Image internalCachedBackgroundImage = ImageCache::getFromMemory(bmp00128_png, bmp00128_pngSize);
	const Image knobImage = ImageCache::getFromMemory(bmp00129_png, bmp00129_pngSize);
	const Image buttonImage = ImageCache::getFromMemory(bmp00130_png, bmp00130_pngSize);
	const Image sliderImage = ImageCache::getFromMemory(bmp00131_png, bmp00131_pngSize);
	const Image waveform1Image = ImageCache::getFromMemory(bmp00132_png, bmp00132_pngSize);
	const Image waveform2Image = ImageCache::getFromMemory(bmp00133_png, bmp00133_pngSize);
	const Image lfoWaveformImage = ImageCache::getFromMemory(bmp00134_png, bmp00134_pngSize);
	const Image darkKnobImage = ImageCache::getFromMemory(bmp00135_png, bmp00135_pngSize);
	const Image loadButtonImage = ImageCache::getFromMemory(bmp00136_png, bmp00136_pngSize);
	const Image saveButtonImage = ImageCache::getFromMemory(bmp00137_png, bmp00137_pngSize);

    // helpers
	int numberOfImages = knobImage.getHeight() / knobImage.getWidth();

    this->accordeonTabContainer = new AccordeonTabContainer();

    // Synth 1 Tab
    // ------------------------------------------
    this->synth1AccordeonTab = new AccordeonTab(internalCachedBackgroundImage, 0, 275);

    // lfo 1
    lfo1WaveformKnob = addNormalKnob(this->synth1AccordeonTab, 92, 150, ownerFilter, lfoWaveformImage, 6, LFO1WAVEFORM);
	lfo1SyncButton = addNormalButton(this->synth1AccordeonTab, 92, 58, ownerFilter, buttonImage, false, LFO1SYNC);
	lfo1KeyTriggerButton = addNormalButton(this->synth1AccordeonTab, 92, 183, ownerFilter, buttonImage, false, LFO1KEYTRIGGER);
	lfo1RateKnob = addNormalKnob(this->synth1AccordeonTab, 26, 51, ownerFilter, knobImage, numberOfImages, LFO1RATE);
	lfo1AmountKnob = addNormalKnob(this->synth1AccordeonTab, 26, 113, ownerFilter, knobImage, numberOfImages, LFO1AMOUNT);
	lfo1PhaseKnob = addNormalKnob(this->synth1AccordeonTab, 26, 176, ownerFilter, knobImage, numberOfImages, LFO1PHASE);

    lfo1DestinationTalComboBox = addTalComboBox(this->synth1AccordeonTab, 92, 121, 60, ownerFilter, LFO1DESTINATION);
	lfo1DestinationTalComboBox->addItem("Off",1);
	lfo1DestinationTalComboBox->addItem("Filter",2);
	lfo1DestinationTalComboBox->addItem("Osc1",3);
	lfo1DestinationTalComboBox->addItem("Osc2",4);
	lfo1DestinationTalComboBox->addItem("Osc1 & 2",8);
	lfo1DestinationTalComboBox->addItem("PW",5);
	lfo1DestinationTalComboBox->addItem("FM",6);
	lfo1DestinationTalComboBox->addItem("Lfo2 r",7);

    // osc 1
    osc1VolumeKnob = addNormalKnob(this->synth1AccordeonTab, 438, 51, ownerFilter, knobImage, numberOfImages, OSC1VOLUME);
	osc1TuneKnob = addNormalKnob(this->synth1AccordeonTab, 188, 51, ownerFilter, knobImage, numberOfImages, OSC1TUNE);
	osc1PwKnob = addNormalKnob(this->synth1AccordeonTab, 238, 113, ownerFilter, knobImage, numberOfImages, OSC1PW);
	osc1PhaseKnob = addNormalKnob(this->synth1AccordeonTab, 188, 113, ownerFilter, knobImage, numberOfImages, OSC1PHASE);
	osc1FineTuneKnob = addNormalKnob(this->synth1AccordeonTab, 238, 51, ownerFilter, knobImage, numberOfImages, OSC1FINETUNE);

    osc1WaveformTalComboBox = addTalComboBox(this->synth1AccordeonTab, 193, 187, 80, ownerFilter, OSC1WAVEFORM);
	osc1WaveformTalComboBox->addItem("Saw",1);
	osc1WaveformTalComboBox->addItem("Pulse",2);
	osc1WaveformTalComboBox->addItem("Noise",3);

    // osc 2
	osc2VolumeKnob = addNormalKnob(this->synth1AccordeonTab, 488, 51, ownerFilter, knobImage, numberOfImages, OSC2VOLUME);
	osc2TuneKnob = addNormalKnob(this->synth1AccordeonTab, 313, 51, ownerFilter, knobImage, numberOfImages, OSC2TUNE);
	osc2FineTuneKnob = addNormalKnob(this->synth1AccordeonTab, 363, 51, ownerFilter, knobImage, numberOfImages, OSC2FINETUNE);
	osc2FmKnob = addNormalKnob(this->synth1AccordeonTab, 363, 113, ownerFilter, knobImage, numberOfImages, OSC2FM);
	osc2PhaseKnob = addNormalKnob(this->synth1AccordeonTab, 313, 113, ownerFilter, knobImage, numberOfImages, OSC2PHASE);

    osc2WaveformTalComboBox = addTalComboBox(this->synth1AccordeonTab, 319, 187, 80, ownerFilter, OSC2WAVEFORM);
	osc2WaveformTalComboBox->addItem("Saw",1);
	osc2WaveformTalComboBox->addItem("Pulse",2);
	osc2WaveformTalComboBox->addItem("Triangle",3);
	osc2WaveformTalComboBox->addItem("Sine",4);
	osc2WaveformTalComboBox->addItem("Noise",5);

    // master
	osc3VolumeKnob = addNormalKnob(this->synth1AccordeonTab, 538, 51, ownerFilter, knobImage, numberOfImages, OSC3VOLUME);
	portamentoKnob = addNormalKnob(this->synth1AccordeonTab, 538, 113, ownerFilter, knobImage, numberOfImages, PORTAMENTO);
    ringmodulationKnob =  addNormalKnob(this->synth1AccordeonTab, 438, 113, ownerFilter, knobImage, numberOfImages, RINGMODULATION);
	oscMasterTuneKnob = addNormalKnob(this->synth1AccordeonTab, 440, 176, ownerFilter, knobImage, numberOfImages, OSCMASTERTUNE);
	oscSyncButton = addNormalButton(this->synth1AccordeonTab, 490, 125, ownerFilter, buttonImage, false, OSCSYNC);
	transposeKnob = addNormalKnob(this->synth1AccordeonTab, 538, 176, ownerFilter, knobImage, numberOfImages, TRANSPOSE);
	volumeKnob = addNormalKnob(this->synth1AccordeonTab, 613, 51, ownerFilter, knobImage, numberOfImages, VOLUME);

    voicesTalComboBox = addTalComboBox(this->synth1AccordeonTab, 595, 130, 60, ownerFilter, VOICES);
	voicesTalComboBox->addItem("mono",1);
	voicesTalComboBox->addItem("2",2);
	voicesTalComboBox->addItem("3",3);
	voicesTalComboBox->addItem("4",4);
	voicesTalComboBox->addItem("5",5);
	voicesTalComboBox->addItem("6",6);

    portamentoModeTalComboBox = addTalComboBox(this->synth1AccordeonTab, 595, 107, 60, ownerFilter, PORTAMENTOMODE);
	portamentoModeTalComboBox->addItem("Off",1);
	portamentoModeTalComboBox->addItem("Auto",2);
	portamentoModeTalComboBox->addItem("On",3);

    this->accordeonTabContainer->add(this->synth1AccordeonTab);

    // Synth 2 Tab
    // ------------------------------------------
    this->synth2AccordeonTab = new AccordeonTab(internalCachedBackgroundImage, 275, 275);

    // lfo 2
	lfo2RateKnob = addNormalKnob(this->synth2AccordeonTab, 26, 51, ownerFilter, knobImage, numberOfImages, LFO2RATE);
	lfo2AmountKnob = addNormalKnob(this->synth2AccordeonTab, 26, 113, ownerFilter, knobImage, numberOfImages, LFO2AMOUNT);
	lfo2WaveformKnob = addNormalKnob(this->synth2AccordeonTab, 92, 150, ownerFilter, lfoWaveformImage, 6, LFO2WAVEFORM);
	lfo2SyncButton = addNormalButton(this->synth2AccordeonTab, 92, 58, ownerFilter, buttonImage, false, LFO2SYNC);
	lfo2KeyTriggerButton = addNormalButton(this->synth2AccordeonTab, 92, 183, ownerFilter, buttonImage, false, LFO2KEYTRIGGER);
	lfo2PhaseKnob = addNormalKnob(this->synth2AccordeonTab, 26, 176, ownerFilter, knobImage, numberOfImages, LFO2PHASE);

	lfo2DestinationTalComboBox = addTalComboBox(this->synth2AccordeonTab, 92, 121, 60, ownerFilter, LFO2DESTINATION);
	lfo2DestinationTalComboBox->addItem("Off",1);
	lfo2DestinationTalComboBox->addItem("Filter",2);
	lfo2DestinationTalComboBox->addItem("Osc1",3);
	lfo2DestinationTalComboBox->addItem("Osc2",4);
	lfo2DestinationTalComboBox->addItem("Osc1 & 2",8);
	lfo2DestinationTalComboBox->addItem("Lfo1 r",7);
	lfo2DestinationTalComboBox->addItem("Pan",5);
	lfo2DestinationTalComboBox->addItem("Volume",6);

    // filter
	cutoffKnob = addNormalKnob(this->synth2AccordeonTab, 188, 51, ownerFilter, knobImage, numberOfImages, CUTOFF);
	resonanceKnob = addNormalKnob(this->synth2AccordeonTab, 238, 51, ownerFilter, knobImage, numberOfImages, RESONANCE);
	keyfollowKnob = addNormalKnob(this->synth2AccordeonTab, 313, 51, ownerFilter, knobImage, numberOfImages, KEYFOLLOW);
	filterContourKnob = addNormalKnob(this->synth2AccordeonTab, 363, 51, ownerFilter, knobImage, numberOfImages, FILTERCONTOUR);

    filterAttackKnob = addSlider(this->synth2AccordeonTab, 198, 130, ownerFilter, sliderImage, 64, FILTERATTACK);
	filterDecayKnob = addSlider(this->synth2AccordeonTab, 198 + 50, 130, ownerFilter, sliderImage, 64, FILTERDECAY);
	filterSustainKnob = addSlider(this->synth2AccordeonTab, 198 + 100, 130, ownerFilter, sliderImage, 64, FILTERSUSTAIN);
	filterReleaseKnob = addSlider(this->synth2AccordeonTab, 198 + 150, 130, ownerFilter, sliderImage, 64, FILTERRELEASE);

    filtertypeTalComboBox = addTalComboBox(this->synth2AccordeonTab, 292, 15, 80, ownerFilter, FILTERTYPE);
	filtertypeTalComboBox->addItem("LP 24dB",1);
	filtertypeTalComboBox->addItem("LP 18dB",2);
	filtertypeTalComboBox->addItem("LP 12dB",3);
	filtertypeTalComboBox->addItem("LP 6dB",4);
	filtertypeTalComboBox->addItem("HP 12dB",5);
	filtertypeTalComboBox->addItem("BP 12dB",6);
	filtertypeTalComboBox->addItem("Notch",7);
	filtertypeTalComboBox->addItem("LP SV 24dB",8);
	filtertypeTalComboBox->addItem("HP SV 24dB",9);
	filtertypeTalComboBox->addItem("BP SV 24dB",10);

    // adsr
    ampAttackKnob = addSlider(this->synth2AccordeonTab, 448, 130, ownerFilter, sliderImage, 64, AMPATTACK);
	ampDecayKnob = addSlider(this->synth2AccordeonTab, 448 + 50, 130, ownerFilter, sliderImage, 64, AMPDECAY);
	ampSustainKnob = addSlider(this->synth2AccordeonTab, 448 + 100, 130, ownerFilter, sliderImage, 64, AMPSUSTAIN);
	ampReleaseKnob = addSlider(this->synth2AccordeonTab, 448 + 150, 130, ownerFilter, sliderImage, 64, AMPRELEASE);

    freeAdAttackKnob = addNormalKnob(this->synth2AccordeonTab, 440, 51, ownerFilter, knobImage, numberOfImages, FREEADATTACK);
	freeAdDecayKnob = addNormalKnob(this->synth2AccordeonTab, 488, 51, ownerFilter, knobImage, numberOfImages, FREEADDECAY);
	freeAdAmountKnob = addNormalKnob(this->synth2AccordeonTab, 538, 51, ownerFilter, knobImage, numberOfImages, FREEADAMOUNT);

	freeAdDestinationTalComboBox = addTalComboBox(this->synth2AccordeonTab, 595, 55, 60, ownerFilter, FREEADDESTINATION);
	freeAdDestinationTalComboBox->addItem("Off",1);
	freeAdDestinationTalComboBox->addItem("Filter",2);
	freeAdDestinationTalComboBox->addItem("Osc1",3);
	freeAdDestinationTalComboBox->addItem("Osc2",4);
	freeAdDestinationTalComboBox->addItem("PW",5);
	freeAdDestinationTalComboBox->addItem("FM",6);

    this->accordeonTabContainer->add(synth2AccordeonTab);

    // Envelope Tab
    // ------------------------------------------
    this->envelopeEditorAccordeonTab = new AccordeonTab(internalCachedBackgroundImage, 550, 275);
    int width = 540;
    int height = 200;
    this->envelopeEditorView = new EnvelopeEditorView(ownerFilter, width, height);
    this->envelopeEditorView->setBackgroundColor(Colour((const uint8)35, (const uint8)35, (const uint8)35, (const uint8)0));
    this->envelopeEditorView->setSplineColor(Colour((const uint8)160, (const uint8)160, (const uint8)160, (const uint8)255));
	this->envelopeEditorAccordeonTab->addAndMakeVisible(envelopeEditorView);
    this->envelopeEditorView->setBounds(30, this->envelopeEditorAccordeonTab->getTabHeight() + 23, width, height);

    envelopeEditorDest1TalComboBox = addTalComboBox(this->envelopeEditorAccordeonTab, 583, 21, 70, ownerFilter, ENVELOPEEDITORDEST1);
	envelopeEditorDest1TalComboBox->addItem("Off",1);
	envelopeEditorDest1TalComboBox->addItem("Filter",2);
	envelopeEditorDest1TalComboBox->addItem("Osc1",3);
	envelopeEditorDest1TalComboBox->addItem("Osc2",4);
	envelopeEditorDest1TalComboBox->addItem("Osc1 & 2",5);
	envelopeEditorDest1TalComboBox->addItem("FM",6);
	envelopeEditorDest1TalComboBox->addItem("Ringmod",7);
	envelopeEditorDest1TalComboBox->addItem("Volume",8);

	envelopeEditorSpeedTalComboBox = addTalComboBox(this->envelopeEditorAccordeonTab, 583, 44, 70, ownerFilter, ENVELOPEEDITORSPEED);
	envelopeEditorSpeedTalComboBox->addItem("x1",1);
	envelopeEditorSpeedTalComboBox->addItem("x2",2);
	envelopeEditorSpeedTalComboBox->addItem("x4",3);
	envelopeEditorSpeedTalComboBox->addItem("x8",4);
	envelopeEditorSpeedTalComboBox->addItem("x16",5);
	envelopeEditorSpeedTalComboBox->addItem("x32",6);

    this->envelopeOneShotButton = addNormalButton(this->envelopeEditorAccordeonTab, 583, 80, ownerFilter, buttonImage, false, ENVELOPEONESHOT);
    this->envelopeFixTempoButton = addNormalButton(this->envelopeEditorAccordeonTab, 583, 108, ownerFilter, buttonImage, false, ENVELOPEFIXTEMPO);

    this->envelopeEditorAmountKnob = addNormalKnob(this->envelopeEditorAccordeonTab, 600, 143, ownerFilter, darkKnobImage, numberOfImages, ENVELOPEEDITORAMOUNT);
    this->envelopeResetButton = addNormalButton(this->envelopeEditorAccordeonTab, 602, 203, ownerFilter, buttonImage, true, ENVELOPERESET);

    this->accordeonTabContainer->add(this->envelopeEditorAccordeonTab);

    // Control
    // ------------------------------------------
    this->controlAccordeonTab = new AccordeonTab(internalCachedBackgroundImage, 825, 275);

    // velocity control
    velocityVolumeKnob = addSlider(this->controlAccordeonTab, 38, 22, ownerFilter, sliderImage, 50, VELOCITYVOLUME);
    velocityContourKnob = addSlider(this->controlAccordeonTab, 91, 22, ownerFilter, sliderImage, 50, VELOCITYCONTOUR);
    velocityCutoffKnob = addSlider(this->controlAccordeonTab, 146, 22, ownerFilter, sliderImage, 50, VELOCITYCUTOFF);
    pitchwheelCutoffKnob = addSlider(this->controlAccordeonTab, 226, 22, ownerFilter, sliderImage, 50, PITCHWHEELCUTOFF);
    pitchwheelPitchKnob = addSlider(this->controlAccordeonTab, 281, 22, ownerFilter, sliderImage, 50, PITCHWHEELPITCH);
    highpassKnob = addSlider(this->controlAccordeonTab, 359, 22, ownerFilter, sliderImage, 50, HIGHPASS);
    detuneKnob = addSlider(this->controlAccordeonTab, 416, 22, ownerFilter, sliderImage, 50, DETUNE);
    vintageNoiseKnob = addSlider(this->controlAccordeonTab, 473, 22, ownerFilter, sliderImage, 50, VINTAGENOISE);
    filterDriveKnob = addSlider(this->controlAccordeonTab, 532, 22, ownerFilter, sliderImage, 50, FILTERDRIVE);

    // reverb
    reverbWetKnob = addNormalKnob(this->controlAccordeonTab, 188, 122, ownerFilter, darkKnobImage, numberOfImages, REVERBWET);
    reverbDecayKnob = addNormalKnob(this->controlAccordeonTab, 238, 122, ownerFilter, darkKnobImage, numberOfImages, REVERBDECAY);
    reverbPreDelayKnob = addNormalKnob(this->controlAccordeonTab, 288, 122, ownerFilter, darkKnobImage, numberOfImages, REVERBPREDELAY);
	reverbHighCutKnob = addNormalKnob(this->controlAccordeonTab, 338, 122, ownerFilter, darkKnobImage, numberOfImages, REVERBHIGHCUT);
	reverbLowCutKnob = addNormalKnob(this->controlAccordeonTab, 388, 122, ownerFilter, darkKnobImage, numberOfImages, REVERBLOWCUT);

    // delay
    delayWetKnob = addNormalKnob(this->controlAccordeonTab, 188, 179, ownerFilter, darkKnobImage, numberOfImages, DELAYWET);
    delayTimeKnob = addNormalKnob(this->controlAccordeonTab, 238, 179, ownerFilter, darkKnobImage, numberOfImages, DELAYTIME);
    delaySyncButton = addNormalButton(this->controlAccordeonTab, 288, 186, ownerFilter, buttonImage, false, DELAYSYNC);
    delayFactorLButton = addNormalButton(this->controlAccordeonTab, 338, 176, ownerFilter, buttonImage, false, DELAYFACTORL);
    delayFactorRButton = addNormalButton(this->controlAccordeonTab, 338, 196, ownerFilter, buttonImage, false, DELAYFACTORR);
    delayHighShelfKnob = addNormalKnob(this->controlAccordeonTab, 388, 179, ownerFilter, darkKnobImage, numberOfImages, DELAYHIGHSHELF);
    delayLowShelfKnob = addNormalKnob(this->controlAccordeonTab, 439, 179, ownerFilter, darkKnobImage, numberOfImages, DELAYLOWSHELF);
    delayFeedbackKnob = addNormalKnob(this->controlAccordeonTab, 490, 179, ownerFilter, darkKnobImage, numberOfImages, DELAYFEEDBACK);

    // chorus
    chorus1Button = addNormalButton(this->controlAccordeonTab, 38, 129, ownerFilter, buttonImage, false, CHORUS1ENABLE);
	chorus2Button = addNormalButton(this->controlAccordeonTab, 95, 129, ownerFilter, buttonImage, false, CHORUS2ENABLE);

    // others
    oscBitcrusherKnob = addNormalKnob(this->controlAccordeonTab, 65, 179, ownerFilter, darkKnobImage, numberOfImages, OSCBITCRUSHER);

    this->accordeonTabContainer->add(this->controlAccordeonTab);

    // Logo Panel
    // ------------------------------------------
    this->logoPanel = new LogoPanel(internalCachedBackgroundImage, 1100, 60);

	// Version info
	versionLabel = new Label("Version Info", "V 3.22");
	versionLabel->setBounds(235, 15, 200, 20);
	versionLabel->setColour(Label::textColourId, Colour((juce::uint8)100, (juce::uint8)100, (juce::uint8)100, 0.9f));
    this->logoPanel->addAndMakeVisible(versionLabel);

    infoText = new Label("Info Text", "-");
    infoText->setBounds(480, 30, 62, 20);
    infoText->setJustificationType(juce::Justification::centred);
    infoText->setColour(Label::textColourId, Colour::greyLevel(0.8f));
    infoText->setColour(Label::backgroundColourId, Colour((juce::uint8)100, (juce::uint8)100, (juce::uint8)100, (juce::uint8)0));
    this->logoPanel->addAndMakeVisible(infoText);

    loadButton = addNormalButton(this->logoPanel, 544, -13, ownerFilter, loadButtonImage, true, PRESETLOAD);
    saveButton = addNormalButton(this->logoPanel, 544, 8, ownerFilter, saveButtonImage, true, PRESETSAVE);

    panicButton = addNormalButton(this->logoPanel, 620, 5, ownerFilter, buttonImage, true, PANIC);
    this->accordeonTabContainer->add(this->logoPanel);

    this->setBufferedToImage(true);

    this->updateParametersFromFilter();

    ownerFilter->addChangeListener (this);

    this->envelopeEditorAccordeonTab->addListener (this);
    this->synth1AccordeonTab->addListener (this);
    this->synth2AccordeonTab->addListener (this);
    this->controlAccordeonTab->addListener (this);

    this->accordeonTabContainer->resizeComponent();
    this->addAndMakeVisible(this->accordeonTabContainer);

    this->accordeonTabContainer->resizeTabs(true);
    this->resized();
}

TalComponent::~TalComponent()
{
    deleteAllChildren();
    getProcessor()->removeChangeListener (this);
}

FilmStripKnob* TalComponent::addNormalKnob(Component *component, int x, int y, TalCore* const ownerFilter, const Image knobImage, int numOfFrames, int parameter)
{
	FilmStripKnob *filmStripKnob;
	component->addAndMakeVisible(filmStripKnob = new FilmStripKnob(knobImage,
									 numOfFrames,
									 false,
                                     parameter));
    filmStripKnob->setBounds(x, y + this->synth1AccordeonTab->getTabHeight(), knobImage.getWidth(), knobImage.getHeight() / numOfFrames);
	filmStripKnob->addListener (this);
	return filmStripKnob;
}

ImageToggleButton* TalComponent::addNormalButton(Component *component, int x, int y, TalCore* const ownerFilter, const Image buttonImage, bool isKickButton, int parameter)
{
	ImageToggleButton *imageToggleButton;
	component->addAndMakeVisible(imageToggleButton = new ImageToggleButton("Toggle Button", buttonImage, false, isKickButton, parameter));
    imageToggleButton->setBounds(x, y + this->synth1AccordeonTab->getTabHeight(), buttonImage.getWidth(), buttonImage.getHeight() / 2);
	imageToggleButton->addListener(this);
	return imageToggleButton;
}

ImageSlider* TalComponent::addSlider(Component *component, int x, int y, TalCore* const ownerFilter, const Image sliderImage, int height, int parameter)
{
	ImageSlider *imageSlider;
	component->addAndMakeVisible(imageSlider = new ImageSlider(sliderImage, height, parameter));
	imageSlider->setBounds(x, y + this->synth1AccordeonTab->getTabHeight(), /*sliderImage->getWidth()*/ 40, height + sliderImage.getHeight());
	imageSlider->addListener (this);
	return imageSlider;
}

TalComboBox* TalComponent::addTalComboBox(Component *component, int x, int y, int width, TalCore* const ownerFilter, int parameter)
{
	TalComboBox *comboBox;
	component->addAndMakeVisible(comboBox = new TalComboBox(parameter));
	comboBox->setBounds(x, y + this->synth1AccordeonTab->getTabHeight(), width, 20);
	comboBox->setColour(TalComboBox::backgroundColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
	comboBox->setColour(TalComboBox::textColourId, Colour::greyLevel(1.0f));
	comboBox->setColour(TalComboBox::buttonColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
	comboBox->setColour(TalComboBox::arrowColourId, Colour((juce::uint8)8, (juce::uint8)11, (juce::uint8)58, 0.0f));
    comboBox->addListener (this);
	return comboBox;
}

//==============================================================================
void TalComponent::paint (Graphics& g)
{
    // just clear the window
    g.fillAll (Colour((const uint8)44, (const uint8)40, (const uint8)40, (const uint8)255));
    // this->accordeonTabContainer->repaint();
}

void TalComponent::resized()
{
    if (this->accordeonTabContainer->getCurrentHeight() != getHeight()
        || this->accordeonTabContainer->getWidth() != getWidth())
    {
        setSize(this->accordeonTabContainer->getWidth(), 2 * this->synth2AccordeonTab->getTabHeight() + this->logoPanel->getHeight() + this->synth2AccordeonTab->getMaxHeight() + this->synth1AccordeonTab->getMaxHeight());
    }

    repaint();
}

//==============================================================================
void TalComponent::changeListenerCallback (ChangeBroadcaster* source)
{
    // this is the filter telling us that it's changed, so we'll update our
    // display of the time, midi message, etc.
    updateParametersFromFilter();
}

void TalComponent::sliderDragStarted (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->beginParameterChangeGesture(values["index"]);
    }
}

void TalComponent::sliderDragEnded (Slider* slider)
{
    NamedValueSet values = slider->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->endParameterChangeGesture(values["index"]);
    }
}

void TalComponent::sliderValueChanged (Slider* caller)
{
    NamedValueSet values = caller->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->setParameterNotifyingHost(values["index"], (float)caller->getValue());
    }

    this->updateInfo(caller);
}

void TalComponent::updateInfo(Slider* caller)
{
    // FIXME: Refactor
    TalCore* const filter = getProcessor();
    if (caller == osc1TuneKnob || caller == osc2TuneKnob)
    {
        infoText->setText(juce::String(audioUtils.getOscTuneValue((float)caller->getValue()), 0), dontSendNotification);
    }
    else if (caller == osc1FineTuneKnob || caller == osc2FineTuneKnob || caller == oscMasterTuneKnob)
    {
        infoText->setText(juce::String(audioUtils.getOscFineTuneValue((float)caller->getValue()), 4), dontSendNotification);
    }
    else if (caller == filterContourKnob)
    {
        infoText->setText(juce::String(audioUtils.getLogScaledLinearValueCentered((float)caller->getValue()), 2), dontSendNotification);
    }
    else if (caller == lfo1AmountKnob || caller == lfo2AmountKnob || caller == freeAdAmountKnob)
    {
        infoText->setText(juce::String(audioUtils.getLogScaledValueCentered((float)caller->getValue()), 4), dontSendNotification);
    }
    else if (caller == osc1VolumeKnob || caller == osc2VolumeKnob || caller == osc3VolumeKnob)
    {
        infoText->setText(juce::String(audioUtils.getLogScaledValueInDecibel((float)caller->getValue(), 1) , 2) + " dB", dontSendNotification);
    }
    else if (caller == volumeKnob)
    {
        infoText->setText(juce::String(audioUtils.getLogScaledValueInDecibel((float)caller->getValue(), 2) , 2) + " dB", dontSendNotification);
    }
    else if (caller == pitchwheelPitchKnob)
    {
        infoText->setText(juce::String(12.0f * (float)caller->getValue() , 2), dontSendNotification);
    }
    else if (caller == transposeKnob)
    {
        infoText->setText(juce::String(audioUtils.getTranspose((float)caller->getValue()), 2), dontSendNotification);
    }
    else if (caller == osc1PwKnob)
    {
        infoText->setText(juce::String(audioUtils.getLogScaledLinearValueCentered((float)caller->getValue()), 2), dontSendNotification);
    }
    else if (caller == lfo1RateKnob || caller == lfo2RateKnob)
    {
        float isSync = false;
        if ((filter->getParameter(LFO1SYNC) > 0.0f && caller == lfo1RateKnob) || (filter->getParameter(LFO2SYNC) > 0.0f && caller == lfo2RateKnob))
        {
            isSync = true;
        }
        if (isSync)
        {
            float value = (float)caller->getValue();
            const char* text = audioUtils.getSyncedRateAndGetText(&value, filter->getBpm());
            infoText->setText(juce::String(text), dontSendNotification);
        }
        else
        {
            infoText->setText(juce::String(audioUtils.getLogScaledRate((float)caller->getValue()), 2) + " Hz", dontSendNotification);
        }
    }
    else if (caller == delayTimeKnob)
    {
        if (filter->getParameter(DELAYSYNC) > 0.0f)
        {
            float value = (float)caller->getValue();
            const char* text = audioUtils.getDelaySyncTimeAndText(&value);
            infoText->setText(juce::String(text), dontSendNotification);
        }
        else
        {
            infoText->setText(juce::String(audioUtils.getLogScaledValue((float)caller->getValue()) * 4000.0f, 2) + " ms", dontSendNotification);
        }
    }
    else if (caller == delayFeedbackKnob)
    {
        infoText->setText(juce::String(audioUtils.getDelayFeedback(caller->getValue()), 2), dontSendNotification);
    }
    else
    {
        infoText->setText(juce::String(caller->getValue(), 2), dontSendNotification);
    }
}

void TalComponent::buttonClicked (Button* caller)
{
    NamedValueSet values = caller->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->setParameterNotifyingHost((int)values["index"], (float)caller->getToggleState());

        // FIXME: Move following code to separate class
        if ((int)values["index"] == PRESETLOAD)
        {
            FileChooser myChooser ("Please choose a valid NoiseMaker preset.",
                                   File::getSpecialLocation (File::userHomeDirectory),
                                   "*.noisemakerpreset");

            if (myChooser.browseForFileToOpen())
            {
                File selectedFile = myChooser.getResult();

                XmlDocument myDocument(selectedFile.loadFileAsString());
                XmlElement* mainElement = myDocument.getDocumentElement();

                if (mainElement != NULL)
                {
                    filter->setStateInformationFromXml(mainElement);
                }
            }
        }

        if ((int)values["index"] == PRESETSAVE)
        {
            FileChooser myChooser("Save NoiseMaker preset.",
                                   File::getSpecialLocation(File::userHomeDirectory),
                                   "*.noisemakerpreset");

            if (myChooser.browseForFileToSave(true))
            {
                File selectedFile = myChooser.getResult();

                if (!selectedFile.getFileExtension().equalsIgnoreCase(".noisemakerpreset"))
                {
                    selectedFile = selectedFile.getFullPathName() + ".noisemakerpreset";
                }

                XmlElement* const xmlState = filter->getCurrentProgramStateInformationAsXml();
                String myXmlDoc = xmlState->createDocument("");
                selectedFile.replaceWithText(myXmlDoc);
                delete xmlState;
            }
        }
    }

    this->handleClickedTabs(caller);
}

void TalComponent::handleClickedTabs (Button* caller)
{
    TalCore* const filter = getProcessor();

    if (caller == synth1AccordeonTab
        || caller == synth2AccordeonTab
        || caller == envelopeEditorAccordeonTab
        || caller == controlAccordeonTab)
    {
        Array<AccordeonTab*> tabs = this->accordeonTabContainer->getAccordeonArray();

        int openTabCount = 0;
        for (int i = 0; i < tabs.size(); i++)
        {
            if (tabs[i]->isExpanded())
            {
                openTabCount++;
            }
        }

        int shrinkedTabs = 0;
        for (int i = tabs.size() - 1; i >= 0; i--)
        {
            if (tabs[i]->isExpanded() && tabs[i] != caller)
            {
                if (openTabCount - shrinkedTabs > 2)
                {
                    shrinkedTabs++;
                    tabs[i]->setExpanded(false);
                }
                else
                {
                    break;
                }
            }
        }

        this->accordeonTabContainer->resizeTabs(false);
        this->resized();
    }

    if (caller == synth1AccordeonTab) filter->setParameterNotifyingHost(TAB1OPEN, (float)synth1AccordeonTab->isExpanded());
    if (caller == synth2AccordeonTab) filter->setParameterNotifyingHost(TAB2OPEN, (float)synth2AccordeonTab->isExpanded());
    if (caller == envelopeEditorAccordeonTab) filter->setParameterNotifyingHost(TAB3OPEN, (float)envelopeEditorAccordeonTab->isExpanded());
    if (caller == controlAccordeonTab) filter->setParameterNotifyingHost(TAB4OPEN, (float)controlAccordeonTab->isExpanded());
}

void TalComponent::comboBoxChanged(ComboBox* caller)
{
    TalComboBox* talCaller = (TalComboBox*)caller;

    // TalCore* const filter = getProcessor();
    NamedValueSet values = talCaller->getProperties();
    if (values.contains(Identifier("index")))
    {
        TalCore* const filter = getProcessor();
        filter->setParameterNotifyingHost(values["index"], talCaller->getNormalizedSelectedId());
    }
}

//==============================================================================
void TalComponent::updateParametersFromFilter()
{
    TalCore* const filter = getProcessor();

    // we use this lock to make sure the processBlock() method isn't writing to the
    // lastMidiMessage variable while we're trying to read it, but be extra-careful to
    // only hold the lock for a minimum amount of time..
    filter->getCallbackLock().enter();

    // take a local copy of the info we need while we've got the lock..
	float volume = filter->getParameter(VOLUME);
	float cutoff = filter->getParameter(CUTOFF);
	float resonance = filter->getParameter(RESONANCE);
	float filterContour = filter->getParameter(FILTERCONTOUR);
	float keyfollow = filter->getParameter(KEYFOLLOW);

	float filterAttack = filter->getParameter(FILTERATTACK);
	float filterDecay = filter->getParameter(FILTERDECAY);
	float filterSustain = filter->getParameter(FILTERSUSTAIN);
	float filterRelease = filter->getParameter(FILTERRELEASE);

	float ampAttack = filter->getParameter(AMPATTACK);
	float ampDecay = filter->getParameter(AMPDECAY);
	float ampSustain = filter->getParameter(AMPSUSTAIN);
	float ampRelease = filter->getParameter(AMPRELEASE);

	float osc1Volume = filter->getParameter(OSC1VOLUME);
	float osc2Volume = filter->getParameter(OSC2VOLUME);
	float osc3Volume = filter->getParameter(OSC3VOLUME);
	float osc1Waveform = filter->getParameter(OSC1WAVEFORM);
	float osc2Waveform = filter->getParameter(OSC2WAVEFORM);

	float oscMasterTune = filter->getParameter(OSCMASTERTUNE);
	float osc1Tune = filter->getParameter(OSC1TUNE);
	float osc2Tune = filter->getParameter(OSC2TUNE);
	float osc1FineTune = filter->getParameter(OSC1FINETUNE);
	float osc2FineTune = filter->getParameter(OSC2FINETUNE);
	bool oscSync = filter->getParameter(OSCSYNC) > 0.0f;

	float voices = filter->getParameter(VOICES);
	float portamento = filter->getParameter(PORTAMENTO);
	float portamentoMode = filter->getParameter(PORTAMENTOMODE);

	float lfo1Waveform = filter->getParameter(LFO1WAVEFORM);
	float lfo2Waveform = filter->getParameter(LFO2WAVEFORM);

	float lfo1Rate = filter->getParameter(LFO1RATE);
	float lfo2Rate = filter->getParameter(LFO2RATE);

	float lfo1Amount = filter->getParameter(LFO1AMOUNT);
	float lfo2Amount = filter->getParameter(LFO2AMOUNT);

	float lfo1Phase = filter->getParameter(LFO1PHASE);
	float lfo2Phase = filter->getParameter(LFO2PHASE);

	float lfo1Destination = filter->getParameter(LFO1DESTINATION);
	float lfo2Destination = filter->getParameter(LFO2DESTINATION);

	float osc1Pw = filter->getParameter(OSC1PW);
	float osc2Fm = filter->getParameter(OSC2FM);
	float osc1Phase = filter->getParameter(OSC1PHASE);
	float osc2Phase = filter->getParameter(OSC2PHASE);
	float transpose  = filter->getParameter(TRANSPOSE);

	float freeAdAttack  = filter->getParameter(FREEADATTACK);
	float freeAdDecay  = filter->getParameter(FREEADDECAY);
	float freeAdAmount  = filter->getParameter(FREEADAMOUNT);
	float freeAdDestination  = filter->getParameter(FREEADDESTINATION);

	float lfo1Sync  = filter->getParameter(LFO1SYNC);
	float lfo1KeyTrigger  = filter->getParameter(LFO1KEYTRIGGER);
	float lfo2Sync  = filter->getParameter(LFO2SYNC);
	float lfo2KeyTrigger  = filter->getParameter(LFO2KEYTRIGGER);

    float velocityVolume = filter->getParameter(VELOCITYVOLUME);
    float velocityContour = filter->getParameter(VELOCITYCONTOUR);
    float velocityCutoff = filter->getParameter(VELOCITYCUTOFF);
    float pitchwheelCutoff = filter->getParameter(PITCHWHEELCUTOFF);
    float pitchwheelPitch = filter->getParameter(PITCHWHEELPITCH);
    float highpass = filter->getParameter(HIGHPASS);
    float detune = filter->getParameter(DETUNE);
    float vintageNoise = filter->getParameter(VINTAGENOISE);
    float ringmodulation = filter->getParameter(RINGMODULATION);

    float chorus1 = filter->getParameter(CHORUS1ENABLE);
    float chorus2 = filter->getParameter(CHORUS2ENABLE);

    float reverbWet = filter->getParameter(REVERBWET);
    float reverbDecay = filter->getParameter(REVERBDECAY);
    float reverbPreDelay = filter->getParameter(REVERBPREDELAY);
    float reverbHighCut = filter->getParameter(REVERBHIGHCUT);
    float reverbLowCut = filter->getParameter(REVERBLOWCUT);

    float oscBitcrusher = filter->getParameter(OSCBITCRUSHER);
    float filterDrive = filter->getParameter(FILTERDRIVE);
    float filtertype = filter->getParameter(FILTERTYPE);

    float envelopeOneShot = filter->getParameter(ENVELOPEONESHOT);
    float envelopeFixTempo = filter->getParameter(ENVELOPEFIXTEMPO);
    float envelopeEditorAmount = filter->getParameter(ENVELOPEEDITORAMOUNT);
    float envelopeEditorSpeed = filter->getParameter(ENVELOPEEDITORSPEED);
    float envelopeEditorDest1 = filter->getParameter(ENVELOPEEDITORDEST1);

    float tab1Open = filter->getParameter(TAB1OPEN);
    float tab2Open = filter->getParameter(TAB2OPEN);
    float tab3Open = filter->getParameter(TAB3OPEN);
    float tab4Open = filter->getParameter(TAB4OPEN);

    float delayWet = filter->getParameter(DELAYWET);
    float delayTime = filter->getParameter(DELAYTIME);
    float delaySync= filter->getParameter(DELAYSYNC);
    float delayFactorL = filter->getParameter(DELAYFACTORL);
    float delayFactorR = filter->getParameter(DELAYFACTORR);
    float delayHighShelf = filter->getParameter(DELAYHIGHSHELF);
    float delayLowShelf = filter->getParameter(DELAYLOWSHELF);
    float delayFeedback = filter->getParameter(DELAYFEEDBACK);

    // ..release the lock ASAP
    filter->getCallbackLock().exit();

	volumeKnob->setValue(volume, dontSendNotification);
	cutoffKnob->setValue(cutoff, dontSendNotification);
	resonanceKnob->setValue(resonance, dontSendNotification);
	filterContourKnob->setValue(filterContour, dontSendNotification);
	keyfollowKnob->setValue(keyfollow, dontSendNotification);
	filterAttackKnob->setValue(filterAttack, dontSendNotification);
	filterDecayKnob->setValue(filterDecay, dontSendNotification);
	filterSustainKnob->setValue(filterSustain, dontSendNotification);
	filterReleaseKnob->setValue(filterRelease, dontSendNotification);

	ampAttackKnob->setValue(ampAttack, dontSendNotification);
	ampDecayKnob->setValue(ampDecay, dontSendNotification);
	ampSustainKnob->setValue(ampSustain, dontSendNotification);
	ampReleaseKnob->setValue(ampRelease, dontSendNotification);

	osc1VolumeKnob->setValue(osc1Volume, dontSendNotification);
	osc2VolumeKnob->setValue(osc2Volume, dontSendNotification);
	osc3VolumeKnob->setValue(osc3Volume, dontSendNotification);
	portamentoKnob->setValue(portamento, dontSendNotification);
    ringmodulationKnob->setValue(ringmodulation, dontSendNotification);

	oscMasterTuneKnob->setValue(oscMasterTune, dontSendNotification);
	osc1TuneKnob->setValue(osc1Tune, dontSendNotification);
	osc2TuneKnob->setValue(osc2Tune, dontSendNotification);
	osc1FineTuneKnob->setValue(osc1FineTune, dontSendNotification);
	osc2FineTuneKnob->setValue(osc2FineTune, dontSendNotification);

	lfo1WaveformKnob->setValue(lfo1Waveform, dontSendNotification);
	lfo2WaveformKnob->setValue(lfo2Waveform, dontSendNotification);

	lfo1RateKnob->setValue(lfo1Rate, dontSendNotification);
	lfo2RateKnob->setValue(lfo2Rate, dontSendNotification);
	lfo1AmountKnob->setValue(lfo1Amount, dontSendNotification);
	lfo2AmountKnob->setValue(lfo2Amount, dontSendNotification);

	lfo1PhaseKnob->setValue(lfo1Phase, dontSendNotification);
	lfo2PhaseKnob->setValue(lfo2Phase, dontSendNotification);

	osc1PwKnob->setValue(osc1Pw, dontSendNotification);
	osc2FmKnob->setValue(osc2Fm, dontSendNotification);
	osc1PhaseKnob->setValue(osc1Phase, dontSendNotification);
	osc2PhaseKnob->setValue(osc2Phase, dontSendNotification);
	transposeKnob->setValue(transpose, dontSendNotification);

	freeAdAttackKnob->setValue(freeAdAttack, dontSendNotification);
	freeAdDecayKnob->setValue(freeAdDecay, dontSendNotification);
	freeAdAmountKnob->setValue(freeAdAmount, dontSendNotification);

    velocityVolumeKnob->setValue(velocityVolume, dontSendNotification);
    velocityContourKnob->setValue(velocityContour, dontSendNotification);
    velocityCutoffKnob->setValue(velocityCutoff, dontSendNotification);
    pitchwheelCutoffKnob->setValue(pitchwheelCutoff, dontSendNotification);
    pitchwheelPitchKnob->setValue(pitchwheelPitch, dontSendNotification);
    highpassKnob->setValue(highpass, dontSendNotification);
    detuneKnob->setValue(detune, dontSendNotification);
    vintageNoiseKnob->setValue(vintageNoise, dontSendNotification);

    reverbWetKnob->setValue(reverbWet, dontSendNotification);
    reverbDecayKnob->setValue(reverbDecay, dontSendNotification);
    reverbPreDelayKnob->setValue(reverbPreDelay, dontSendNotification);
    reverbHighCutKnob->setValue(reverbHighCut, dontSendNotification);
    reverbLowCutKnob->setValue(reverbLowCut, dontSendNotification);

    oscBitcrusherKnob->setValue(oscBitcrusher, dontSendNotification);
    envelopeEditorAmountKnob->setValue(envelopeEditorAmount, dontSendNotification);

    filterDriveKnob->setValue(filterDrive, dontSendNotification);

    delayWetKnob->setValue(delayWet, dontSendNotification);
    delayTimeKnob->setValue(delayTime, dontSendNotification);
    delayHighShelfKnob->setValue(delayHighShelf, dontSendNotification);
    delayLowShelfKnob->setValue(delayLowShelf, dontSendNotification);
    delayFeedbackKnob->setValue(delayFeedback, dontSendNotification);

	// Buttons
	oscSyncButton->setToggleState(oscSync, dontSendNotification);
	lfo1SyncButton->setToggleState(lfo1Sync > 0, dontSendNotification);
	lfo1KeyTriggerButton->setToggleState(lfo1KeyTrigger > 0, dontSendNotification);
	lfo2SyncButton->setToggleState(lfo2Sync > 0, dontSendNotification);
	lfo2KeyTriggerButton->setToggleState(lfo2KeyTrigger > 0, dontSendNotification);
    chorus1Button->setToggleState(chorus1 > 0, dontSendNotification);
    chorus2Button->setToggleState(chorus2 > 0, dontSendNotification);

    delaySyncButton->setToggleState(delaySync > 0, dontSendNotification);
    delayFactorLButton->setToggleState(delayFactorL > 0, dontSendNotification);
    delayFactorRButton->setToggleState(delayFactorR > 0, dontSendNotification);
    envelopeOneShotButton->setToggleState(envelopeOneShot > 0, dontSendNotification);
    envelopeFixTempoButton->setToggleState(envelopeFixTempo > 0, dontSendNotification);

	// TalComboBox
	voicesTalComboBox->setNormalizedSelectedId(voices);
	portamentoModeTalComboBox->setNormalizedSelectedId(portamentoMode);
	lfo1DestinationTalComboBox->setNormalizedSelectedId(lfo1Destination);
	lfo2DestinationTalComboBox->setNormalizedSelectedId(lfo2Destination);
	freeAdDestinationTalComboBox->setNormalizedSelectedId(freeAdDestination);
	filtertypeTalComboBox->setNormalizedSelectedId(filtertype);

	osc1WaveformTalComboBox->setNormalizedSelectedId(osc1Waveform);
	osc2WaveformTalComboBox->setNormalizedSelectedId(osc2Waveform);

    envelopeEditorSpeedTalComboBox->setNormalizedSelectedId(envelopeEditorSpeed);
    envelopeEditorDest1TalComboBox->setNormalizedSelectedId(envelopeEditorDest1);

    // Tabs
    synth1AccordeonTab->setExpanded(tab1Open > 0);
    synth2AccordeonTab->setExpanded(tab2Open > 0);
    envelopeEditorAccordeonTab->setExpanded(tab3Open > 0);
    controlAccordeonTab->setExpanded(tab4Open > 0);

    this->resized();
    this->accordeonTabContainer->resizeTabs(false);

    if (this->envelopeEditorView->envelopeIsDirty())
    {
        this->envelopeEditorView->repaint();
    }
}

//==============================================================================
// Binary resources - be careful not to edit any of these sections!

// JUCER_RESOURCE: bmp00128_png, 223033, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00128.png"
static const unsigned char resource_TalComponent_bmp00128_png[] = {137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,2,163,0,0,4,139,8,6,0,0,0,161,176,180,9,0,0,0,9,112,72,89,
  115,0,0,15,97,0,0,15,97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,3,204,101,73,68,65,84,120,218,236,
  125,119,128,92,85,245,255,231,190,233,101,123,223,77,217,146,190,233,189,119,66,232,34,8,162,40,69,65,253,34,42,54,252,169,40,86,196,46,88,0,
  69,68,81,58,33,212,132,144,132,18,72,175,155,221,148,237,189,204,238,204,236,238,244,242,238,239,143,247,230,205,123,83,182,101,251,222,79,24,102,118,
  230,181,115,207,185,247,158,123,206,185,231,144,147,39,78,80,48,48,12,54,8,153,24,100,50,78,143,11,217,98,124,28,123,253,154,176,118,100,242,62,
  194,237,75,88,59,12,90,155,145,117,171,87,79,120,101,148,140,97,197,105,212,62,251,8,62,23,25,135,237,63,228,247,25,35,10,222,80,181,195,144,
  92,119,144,174,57,152,207,70,198,9,77,151,124,254,8,183,195,152,164,127,4,158,121,184,158,115,160,237,57,154,199,255,225,212,13,250,122,175,158,142,
  227,216,82,159,129,129,129,129,129,129,129,129,97,164,192,148,81,6,6,6,6,6,6,6,6,6,166,140,50,48,48,48,48,48,48,48,48,48,101,148,
  129,129,129,129,129,129,129,129,129,129,41,163,12,12,12,12,12,12,12,12,12,227,31,234,225,190,97,126,126,62,230,20,207,133,201,100,194,156,226,226,
  152,199,148,149,150,2,0,74,75,207,162,182,166,6,78,167,19,0,96,50,153,112,229,85,87,199,60,167,180,244,172,116,30,0,108,216,180,9,153,25,
  153,81,199,57,157,78,188,245,230,27,200,200,204,196,198,141,155,132,47,101,27,188,92,78,39,222,122,243,77,197,57,55,222,116,83,220,231,44,43,45,
  197,134,141,155,144,145,153,49,224,54,121,233,133,23,0,0,115,138,139,227,182,137,252,184,200,251,17,25,1,22,75,27,222,127,239,189,1,63,203,156,
  226,98,204,153,163,124,134,247,223,219,15,139,197,194,122,11,3,3,3,3,3,3,195,216,85,70,151,45,95,142,219,238,184,3,25,49,20,196,88,10,
  17,0,220,8,65,9,124,235,205,55,240,244,83,79,193,233,116,194,104,50,198,84,72,55,88,54,226,171,95,249,138,164,180,254,223,61,95,141,121,237,
  223,254,250,97,0,64,70,70,70,92,37,51,35,35,19,79,255,235,169,176,50,250,169,216,199,189,132,23,68,101,116,99,143,74,100,127,148,209,120,247,
  82,42,163,241,239,87,86,86,218,47,101,116,106,126,62,242,243,243,49,123,142,160,8,103,100,100,196,188,38,83,70,25,24,24,24,24,24,24,198,172,
  50,106,50,153,240,237,239,222,31,243,55,167,211,137,191,253,229,207,112,58,157,88,182,124,121,76,69,243,202,171,174,70,77,77,13,222,223,191,31,79,
  63,245,20,230,20,207,69,126,126,126,148,2,185,108,249,114,28,61,114,4,27,66,22,207,8,188,245,230,27,56,122,228,72,175,207,123,197,85,87,9,
  247,123,111,127,159,232,171,173,169,81,62,75,102,102,76,165,78,82,238,100,22,220,145,192,3,15,62,136,140,140,158,159,145,129,129,129,129,129,129,129,
  97,220,40,163,83,35,20,71,57,92,46,39,202,74,75,37,87,124,60,55,188,220,229,254,183,63,63,138,135,127,251,187,168,99,54,108,220,132,163,71,
  142,224,202,171,175,138,250,205,98,105,147,44,139,125,193,109,183,223,142,218,154,106,212,68,40,154,177,32,183,162,2,130,91,191,39,11,231,79,31,252,
  241,136,51,222,98,105,131,197,210,6,147,209,212,35,127,24,24,24,24,24,24,24,24,198,188,50,26,138,251,52,153,76,81,191,101,100,100,226,209,191,
  254,13,199,142,30,65,105,105,41,190,250,127,95,129,165,173,173,199,235,213,212,212,224,233,167,158,194,109,119,220,161,248,62,100,89,141,21,10,240,219,
  135,31,150,20,222,190,192,104,50,225,91,223,189,31,223,251,206,183,199,29,211,127,246,224,131,17,60,200,192,35,127,249,43,235,13,12,12,12,12,12,
  12,12,227,83,25,117,58,157,248,233,143,127,132,111,223,127,127,76,69,209,100,50,97,195,198,77,146,123,189,166,166,6,101,165,103,241,254,254,253,113,
  45,147,111,189,249,6,150,45,95,30,21,59,25,43,14,244,165,23,94,232,147,133,51,90,81,206,192,183,190,243,221,81,195,172,231,94,124,105,72,174,
  203,226,65,25,24,24,24,24,24,24,198,181,50,26,82,48,191,250,149,175,32,63,63,31,203,150,175,192,210,229,203,163,226,62,67,200,23,55,213,92,
  121,213,213,176,88,218,240,214,27,111,226,173,55,223,136,58,238,175,127,249,51,30,254,205,111,21,22,215,72,235,107,77,77,13,94,124,225,249,1,63,
  247,165,108,76,98,96,96,96,96,96,96,96,96,232,25,195,158,103,52,164,28,222,255,237,111,225,206,219,62,143,223,254,250,97,49,117,80,108,215,124,
  70,70,38,110,187,227,14,252,248,39,63,141,250,205,210,214,134,191,253,229,207,113,239,229,116,58,165,221,243,253,193,219,17,169,157,70,11,126,250,224,
  143,241,211,7,127,28,181,97,138,129,129,129,129,129,129,129,97,172,98,216,44,163,207,191,244,114,204,239,203,74,75,241,147,31,255,8,128,96,133,252,
  246,119,239,143,25,91,58,167,184,24,249,249,249,81,238,246,158,118,199,215,214,212,244,26,127,26,11,71,143,30,129,209,100,194,134,141,27,71,21,179,
  66,187,240,251,19,251,202,192,192,192,192,192,192,192,48,154,49,108,150,209,190,40,80,101,165,165,61,90,253,140,49,148,212,161,194,191,255,245,20,139,
  165,100,96,96,96,96,96,96,96,24,47,202,104,188,156,157,115,138,139,113,219,29,119,224,83,55,221,140,255,251,234,87,227,198,104,58,157,206,97,117,
  79,135,242,159,50,48,48,48,48,48,48,48,48,12,29,134,205,77,255,244,83,79,161,166,166,6,87,94,117,117,212,198,165,120,185,69,67,74,225,177,
  163,71,240,226,11,47,12,187,123,186,172,180,20,47,189,248,66,143,57,67,25,24,24,24,24,24,24,24,24,6,14,178,110,245,106,58,18,55,158,83,
  92,12,147,201,132,252,252,130,152,10,104,77,77,53,44,22,75,159,98,62,227,89,83,93,78,103,220,148,78,38,83,56,217,59,33,68,241,91,40,47,
  106,111,215,183,180,89,98,110,188,202,200,200,236,177,86,125,188,10,76,125,61,47,63,63,95,17,178,32,127,254,129,90,144,123,202,26,80,83,83,3,
  87,127,23,2,17,109,58,172,66,61,156,247,26,38,58,135,252,62,67,112,125,50,134,218,97,72,174,59,72,215,28,204,103,35,227,132,166,75,62,127,
  132,219,97,76,210,63,2,207,60,92,207,57,208,246,28,205,227,63,25,198,57,184,175,247,234,233,184,17,83,70,71,149,70,62,130,138,211,184,125,118,
  166,140,50,101,148,41,163,76,25,101,202,40,83,70,153,50,202,148,209,62,28,55,236,169,157,24,24,24,24,24,24,24,24,24,24,152,50,202,192,192,
  192,192,192,192,192,192,192,148,81,6,6,6,6,6,6,6,6,134,137,11,114,242,196,9,202,154,129,97,240,37,139,76,12,50,25,167,199,133,108,49,
  62,142,189,126,77,88,59,50,121,31,225,246,37,172,29,6,173,205,212,26,173,150,13,176,12,12,12,12,12,12,12,12,12,67,138,120,214,79,114,195,
  245,215,51,203,40,3,3,3,3,3,3,3,3,195,136,128,197,140,50,48,48,48,48,48,48,48,48,48,101,148,129,129,129,129,129,129,129,129,129,41,
  163,12,12,12,12,12,12,12,12,12,12,76,25,101,96,96,96,96,96,96,96,96,96,202,40,3,3,3,3,3,3,3,3,3,3,83,70,25,24,24,
  24,24,24,24,24,24,152,50,202,192,192,192,192,192,192,192,192,192,192,148,81,6,6,6,6,6,6,6,6,6,166,140,50,48,48,48,48,48,48,48,
  48,48,12,24,234,225,188,89,241,220,185,172,197,199,57,74,207,158,29,245,207,152,153,153,137,140,204,76,198,44,198,115,6,198,115,6,6,134,81,208,
  207,135,84,25,93,190,124,57,150,175,88,129,226,226,98,54,40,76,32,88,218,218,80,90,90,138,35,135,15,227,200,145,35,163,98,82,90,38,147,69,
  6,198,115,6,198,115,6,6,134,209,211,207,7,189,54,189,201,100,194,85,87,95,141,171,174,190,26,38,147,9,0,112,174,180,21,101,165,109,104,183,
  56,96,105,115,2,84,184,37,137,245,64,125,252,174,47,7,13,230,245,73,95,27,84,254,7,237,199,245,73,223,239,75,6,194,104,12,238,245,99,29,
  147,150,105,70,106,134,9,69,197,89,152,86,156,5,0,112,58,157,120,243,141,55,240,230,27,111,192,233,116,14,107,39,42,158,59,23,55,221,124,179,
  52,49,181,118,186,113,186,222,134,150,78,15,78,215,89,5,22,81,128,136,140,162,50,126,81,25,3,9,149,88,9,197,129,226,247,68,252,131,70,240,
  62,244,55,17,143,231,101,114,31,213,233,164,75,82,133,220,240,138,191,101,247,136,122,38,249,59,141,126,46,170,60,52,244,76,52,234,254,17,247,138,
  124,80,170,164,51,39,197,132,156,84,19,150,76,207,196,146,105,153,140,231,140,231,35,206,243,70,103,43,142,180,150,160,209,217,138,195,109,37,151,56,
  67,98,88,206,167,253,189,12,193,152,1,37,125,159,239,134,149,55,131,169,72,145,193,122,28,50,108,50,215,219,37,114,141,89,200,51,102,98,121,218,
  60,44,75,155,55,228,253,124,80,149,209,229,203,151,227,158,123,239,133,201,100,194,185,210,86,188,255,94,21,142,29,105,128,203,233,139,80,210,46,93,
  25,237,75,231,28,204,235,15,182,50,74,250,208,57,135,90,153,30,234,246,89,190,169,8,203,54,22,98,90,113,22,156,78,39,254,242,232,163,195,98,
  65,49,153,76,184,231,222,123,177,124,249,114,56,188,1,188,83,210,132,119,206,54,163,178,173,43,98,178,165,202,73,58,82,49,145,201,41,79,195,52,
  82,73,49,161,61,78,218,210,245,226,41,38,178,243,229,207,69,41,13,223,75,254,119,204,235,71,40,73,52,226,25,251,165,152,80,89,27,40,207,163,
  113,104,4,85,42,79,215,172,40,192,53,43,10,177,100,90,38,227,57,227,249,176,241,188,203,231,196,43,213,123,240,74,213,187,56,103,171,2,23,111,
  1,48,8,171,117,218,159,217,190,31,135,210,94,46,67,134,88,17,163,67,164,220,197,186,46,55,10,20,174,193,188,46,55,88,207,67,200,136,209,208,
  155,62,252,137,201,91,113,253,228,45,88,150,54,111,72,250,249,160,40,163,38,147,9,119,220,121,39,54,110,218,4,139,197,137,199,254,124,16,231,74,
  91,123,80,210,152,50,58,17,148,209,16,166,21,103,225,150,175,174,70,106,134,9,239,237,223,143,167,254,249,207,33,179,158,20,207,157,139,239,222,127,
  63,76,38,19,118,28,175,195,191,63,170,130,195,19,136,51,73,51,197,100,176,21,147,208,253,150,78,203,194,131,183,174,68,110,42,227,57,227,249,208,
  242,252,95,23,94,197,163,37,255,69,151,47,124,47,166,140,50,101,148,41,163,67,99,156,93,158,54,15,191,88,120,31,242,140,153,131,218,207,85,115,
  102,207,126,240,82,21,209,159,252,244,167,88,184,104,17,222,127,175,10,191,251,245,7,104,106,236,234,147,178,50,238,149,209,254,92,127,28,43,163,86,
  139,19,71,223,171,66,66,138,1,107,54,45,198,162,69,139,240,209,71,31,193,239,247,15,234,56,178,105,211,38,124,247,123,223,131,143,114,248,254,75,
  39,241,230,233,70,248,131,124,191,219,38,214,223,244,18,250,56,137,152,108,6,194,47,50,200,227,16,25,196,241,44,214,36,218,212,225,196,235,135,171,
  145,150,168,199,229,107,25,207,25,207,7,159,231,30,248,241,133,247,126,132,103,203,223,130,55,232,239,89,121,27,68,101,20,67,164,140,246,118,153,161,
  86,70,135,83,193,35,227,76,25,37,19,72,25,109,116,183,225,213,134,119,145,161,75,197,246,5,155,7,173,159,95,146,50,26,82,68,243,11,10,240,
  216,95,14,226,229,23,74,224,247,7,251,60,176,50,101,116,98,40,163,0,16,240,7,113,246,72,61,108,22,39,214,109,155,55,232,19,213,166,77,155,
  112,207,189,247,162,178,173,27,95,249,247,97,212,91,93,3,230,25,83,76,46,93,49,1,0,95,32,136,253,103,26,208,108,117,226,250,13,140,231,140,
  231,131,199,243,115,182,42,92,183,235,171,168,234,106,232,155,242,198,148,81,166,140,50,101,116,80,148,81,0,240,241,126,236,109,57,132,70,119,27,110,
  152,125,197,160,244,243,75,218,77,127,199,157,119,74,138,232,251,251,171,250,116,78,209,244,148,184,110,122,155,213,3,91,135,59,234,28,131,65,141,220,
  73,9,253,98,64,85,185,45,230,97,122,241,90,41,105,6,164,164,26,34,238,239,134,189,195,141,230,134,110,120,220,129,65,147,239,236,73,9,48,24,
  194,77,237,113,7,208,210,208,141,137,136,163,251,43,1,0,183,220,179,10,119,220,121,39,254,252,232,163,151,124,205,252,130,2,220,126,231,157,168,108,
  235,198,119,158,63,33,115,209,246,29,70,173,10,83,82,12,200,48,107,145,145,160,85,184,63,45,14,47,44,221,62,212,118,184,225,242,5,123,189,86,
  70,130,14,25,102,45,166,166,25,97,212,170,162,54,199,148,53,118,195,233,13,160,166,189,111,174,141,204,68,29,50,18,116,40,72,55,194,168,83,118,
  217,182,46,15,218,186,188,168,105,115,194,225,29,60,153,205,76,210,35,35,73,175,208,58,74,234,236,3,186,214,107,135,171,0,80,252,228,86,198,243,
  209,198,243,130,76,51,50,18,245,40,204,52,43,84,76,135,39,128,234,54,7,90,59,61,104,179,187,71,21,207,207,217,170,112,235,222,251,21,110,249,
  184,99,175,62,29,217,134,244,94,143,59,101,63,175,248,123,154,121,10,204,106,99,15,179,50,65,119,192,133,10,71,93,212,47,139,146,103,245,73,49,
  136,119,190,28,211,197,231,88,148,60,51,234,82,221,1,23,42,156,245,104,241,180,163,197,211,30,117,238,66,249,115,244,19,194,181,133,103,91,148,212,
  191,235,52,123,195,207,51,205,52,5,102,141,177,207,58,215,169,206,243,96,24,59,120,181,254,93,0,192,47,23,126,227,146,251,249,128,149,209,229,203,
  151,99,227,166,77,120,255,189,170,62,43,162,0,240,127,95,95,18,247,183,119,222,170,194,158,55,43,163,190,207,157,148,128,47,125,99,105,191,158,239,
  254,175,238,81,252,189,100,101,46,214,110,156,130,156,222,148,218,80,135,106,232,198,199,239,213,225,196,161,166,1,181,143,222,160,198,236,249,153,88,180,
  50,23,5,211,83,20,191,213,148,219,240,212,31,143,77,88,1,62,186,191,18,211,138,179,176,113,211,166,65,73,23,241,213,175,126,21,84,173,195,111,
  119,157,232,247,228,188,190,40,21,219,231,164,99,106,196,194,36,30,106,59,220,120,187,180,13,31,92,236,136,82,108,174,152,155,137,245,51,210,145,145,
  160,237,249,34,50,81,62,90,109,195,91,103,90,80,218,216,165,188,158,78,141,171,22,100,99,227,236,12,100,36,232,250,244,108,213,22,39,222,60,217,
  132,253,165,109,3,110,203,21,211,211,177,114,122,26,54,207,205,142,250,237,218,135,223,27,240,117,95,59,84,133,37,211,179,112,45,227,249,168,224,249,
  53,75,242,112,205,226,60,100,38,233,123,61,182,173,211,131,157,199,234,241,218,209,134,17,231,121,80,3,220,255,193,239,251,164,136,2,192,21,57,107,
  113,123,193,117,189,30,247,133,163,63,86,40,134,127,90,116,191,160,140,246,168,192,94,192,215,78,61,28,165,252,254,105,225,119,251,244,108,142,128,11,
  87,126,244,213,152,10,244,157,83,175,195,186,244,69,189,62,131,252,90,7,58,78,226,169,218,157,146,34,248,167,249,223,29,112,91,159,234,188,128,175,
  157,121,120,64,215,121,170,110,39,158,170,221,9,0,248,90,209,45,88,152,52,179,223,247,63,96,61,137,23,155,246,48,229,116,140,40,164,203,211,230,
  225,19,155,182,92,82,63,31,144,50,26,218,197,104,177,56,241,239,167,142,143,250,198,250,252,221,11,48,103,126,236,60,167,54,171,27,30,87,32,74,
  73,205,153,148,128,27,110,45,198,226,21,185,248,199,159,122,86,28,83,210,12,72,78,213,163,96,122,42,82,210,244,200,206,75,232,179,210,59,97,5,
  248,169,99,40,42,206,194,61,247,222,139,210,47,127,121,192,1,208,55,221,124,51,242,11,10,240,183,253,229,168,108,235,159,181,249,190,77,5,88,50,
  37,49,230,111,22,135,15,46,95,48,74,97,153,154,102,192,151,215,79,197,134,233,105,248,217,27,23,164,239,31,184,122,6,166,166,197,158,56,44,221,
  130,149,45,63,93,176,154,201,177,172,32,5,203,10,82,112,207,127,78,193,210,229,21,172,108,137,58,252,228,250,57,113,21,146,26,139,208,86,249,25,
  38,165,149,43,195,132,175,110,155,142,171,22,229,226,91,255,57,217,115,31,214,171,81,144,97,70,126,166,9,153,137,122,20,100,154,49,119,114,210,144,
  242,252,55,47,29,199,210,233,140,231,35,197,243,16,190,118,197,76,108,22,83,175,133,224,244,6,176,175,164,69,80,84,151,78,82,252,150,153,164,199,
  93,91,166,99,203,188,28,124,255,127,39,225,236,135,215,104,176,121,254,139,227,79,224,156,173,106,208,101,243,138,156,53,120,180,92,80,70,215,166,47,
  238,179,18,24,137,117,233,139,250,124,172,89,109,196,186,244,69,248,176,253,164,66,17,125,106,201,131,113,239,95,225,168,131,35,224,198,194,228,153,81,
  215,218,158,181,6,11,147,102,226,230,35,223,197,88,199,218,212,69,88,155,186,8,15,149,63,137,93,109,31,177,9,115,148,227,161,210,39,176,44,109,
  222,37,245,243,1,149,3,13,229,16,253,219,159,15,194,233,244,141,120,67,28,63,212,132,119,223,170,82,188,66,40,158,159,25,87,17,5,128,71,30,
  58,132,71,127,117,40,238,239,5,211,83,122,60,31,0,22,173,200,197,23,190,190,20,155,175,44,196,162,21,185,76,17,237,3,220,78,31,158,251,243,
  199,82,94,218,129,46,138,174,186,250,106,156,169,183,99,199,137,250,126,157,187,100,74,98,92,165,4,0,190,191,243,2,190,255,106,252,85,249,236,28,
  51,150,230,39,139,202,138,49,174,82,82,219,225,194,215,254,123,6,63,123,237,60,254,186,47,254,36,186,113,86,70,184,127,45,200,233,81,41,249,246,
  179,167,241,227,151,75,227,203,108,134,9,203,167,165,245,72,127,65,134,25,63,187,121,62,190,176,169,8,215,44,201,27,114,69,20,0,186,221,62,252,
  232,63,7,25,207,71,136,231,33,197,50,82,17,5,128,125,37,45,120,114,95,5,254,177,175,2,103,227,132,99,20,102,154,177,101,94,246,136,241,252,
  72,107,9,158,62,255,234,208,40,63,233,139,195,10,101,198,162,1,95,103,157,236,58,114,196,114,163,3,192,218,180,197,81,202,108,60,69,244,165,134,
  61,248,194,241,7,241,245,211,15,227,165,198,61,49,143,201,214,167,75,238,249,127,213,238,84,188,226,61,67,232,249,158,170,221,41,88,53,235,118,226,
  237,214,3,125,162,247,84,231,5,233,156,208,235,148,253,66,159,207,221,240,209,157,248,250,217,95,199,61,230,222,130,91,216,100,57,6,208,237,119,226,
  7,167,254,112,73,253,124,64,150,209,171,174,190,26,101,165,173,40,139,72,223,52,82,56,118,168,9,85,21,177,99,68,123,83,12,61,238,64,175,193,
  186,57,147,18,112,238,76,124,55,216,201,195,77,168,46,23,18,106,19,8,49,162,155,175,44,130,222,160,102,82,218,3,42,74,91,81,89,218,138,171,
  174,190,26,47,60,255,124,191,207,223,184,105,19,76,38,19,254,243,230,201,126,159,219,155,139,182,47,113,130,83,211,140,56,86,99,135,41,194,242,37,
  71,70,130,14,115,114,19,80,214,212,13,75,183,23,101,77,33,75,158,114,11,72,200,66,6,0,249,233,241,173,50,78,241,185,156,190,64,47,202,166,
  9,71,42,58,226,254,94,109,113,224,129,231,207,72,79,145,153,168,195,53,75,242,80,144,105,30,218,190,90,222,138,99,229,140,231,35,193,115,129,207,
  177,221,242,78,175,112,15,147,78,141,130,44,243,168,228,249,163,7,255,59,100,114,153,173,79,199,218,244,197,56,101,63,143,237,217,107,7,116,13,179,
  218,24,101,177,12,161,220,81,143,108,125,122,12,229,117,17,30,146,233,110,9,61,88,100,23,38,207,68,182,62,29,45,158,118,148,59,234,112,170,51,
  182,210,231,8,184,0,64,114,149,135,207,159,21,243,25,0,160,197,219,129,167,234,118,246,155,230,147,157,231,163,238,211,95,244,228,138,31,168,133,154,
  97,248,113,164,163,4,71,59,74,6,220,207,251,173,45,109,18,7,134,247,223,59,195,90,95,132,173,195,45,109,188,34,0,170,203,109,48,24,52,216,
  116,101,33,107,156,222,4,248,189,42,220,114,207,42,108,218,180,9,251,247,239,239,215,185,87,95,125,53,90,187,60,56,83,111,27,81,26,202,154,187,
  97,233,246,197,140,27,52,106,85,120,224,218,89,176,116,123,165,88,193,99,213,182,216,57,39,135,17,78,79,0,103,235,237,138,156,147,109,157,30,252,
  252,211,11,134,252,222,175,29,174,194,210,91,25,207,233,8,60,119,117,155,3,63,124,254,140,236,1,132,28,161,109,157,30,152,116,106,252,252,150,133,
  48,233,226,79,11,135,46,182,143,8,207,27,157,173,56,220,58,184,115,206,41,251,121,197,38,159,117,25,177,173,146,145,199,197,67,79,46,250,151,26,
  247,196,252,61,210,85,255,86,203,71,184,99,106,236,24,215,105,230,41,120,126,197,175,81,225,168,195,129,142,147,120,180,242,217,94,55,64,141,5,44,
  236,97,131,84,139,183,29,12,99,7,59,234,247,98,217,194,121,3,234,231,253,86,70,139,231,206,5,128,126,109,90,26,106,68,110,110,170,42,183,225,
  241,63,29,99,146,49,6,112,116,127,37,110,185,103,21,138,231,206,237,151,240,102,102,102,34,35,51,19,207,28,172,30,21,116,252,126,79,5,62,191,
  106,50,102,231,196,182,196,103,36,232,112,229,252,108,92,57,63,27,46,111,16,71,107,108,56,90,101,197,209,234,129,43,85,47,28,14,187,169,105,68,
  86,242,179,13,157,163,150,231,175,29,170,194,79,111,101,60,31,9,158,59,189,194,34,68,161,140,82,193,34,250,139,91,22,246,104,25,255,227,155,231,
  208,214,233,25,17,158,63,90,50,248,86,209,15,45,39,21,74,102,172,88,209,3,237,39,250,108,157,235,73,97,61,105,63,143,22,79,123,76,203,228,
  218,180,197,146,50,218,226,105,199,247,75,31,197,215,138,110,137,107,197,156,102,158,130,105,230,41,184,125,234,117,104,241,180,227,64,199,73,188,221,250,
  209,136,40,166,119,76,185,14,119,76,81,42,207,95,63,243,107,156,236,195,198,163,133,73,51,241,254,154,127,198,253,253,128,245,228,128,172,181,12,35,
  135,87,235,223,197,47,23,126,163,223,253,124,96,202,104,113,49,142,29,105,96,173,206,48,104,56,123,164,94,170,43,221,31,57,4,128,143,43,44,163,
  130,134,218,14,55,126,246,250,5,76,73,51,98,195,140,52,44,205,79,137,187,195,218,168,83,97,195,204,116,108,152,153,14,75,183,23,47,30,109,196,
  123,231,251,79,199,243,135,234,209,123,53,158,209,137,253,167,235,49,151,241,124,84,240,92,80,68,23,244,170,136,238,21,55,56,141,4,207,223,173,63,
  56,232,252,59,96,57,129,123,167,135,99,18,205,106,163,34,118,20,0,62,108,63,137,43,178,215,244,233,122,241,44,163,161,24,202,22,79,71,159,92,
  245,31,182,159,196,135,237,39,177,46,125,17,214,165,45,238,49,142,52,91,159,142,27,243,46,195,141,121,151,9,241,155,181,59,163,210,84,141,53,84,
  56,235,208,226,237,192,135,29,39,164,244,82,12,99,7,123,91,14,97,126,63,251,249,128,148,209,140,204,76,188,255,94,201,168,34,254,248,161,38,216,
  172,225,21,187,213,234,102,18,49,134,208,84,99,195,220,229,243,251,45,135,0,80,217,230,24,85,180,212,118,184,240,239,131,46,252,251,96,61,210,205,
  58,44,203,79,198,210,252,100,204,201,141,111,61,251,191,205,133,152,147,155,128,191,238,237,159,183,225,230,149,147,165,207,81,86,178,250,78,148,214,143,
  94,235,232,133,70,27,54,45,96,60,31,105,158,155,116,106,252,252,211,61,43,162,127,26,4,69,244,82,121,62,20,59,232,67,249,57,227,89,32,5,
  69,242,124,159,148,209,69,201,179,226,42,140,229,162,197,242,100,231,249,152,49,165,130,18,172,220,85,47,87,74,113,65,200,55,122,101,214,26,44,76,
  158,137,105,230,41,49,239,179,48,105,38,254,52,255,187,120,232,194,147,216,213,58,60,59,208,79,117,94,136,178,130,54,247,209,181,126,170,243,2,190,
  126,246,97,44,76,154,133,63,205,13,103,0,152,102,154,130,105,166,41,88,155,186,8,87,100,174,197,215,207,62,204,38,201,49,132,243,93,85,216,50,
  99,101,255,149,209,254,172,166,67,46,250,218,26,235,168,34,190,167,13,76,12,163,31,141,53,2,239,230,20,23,163,244,236,217,62,91,76,70,147,82,
  242,192,213,51,162,234,119,135,20,149,159,189,118,30,70,157,10,87,204,203,194,141,75,243,98,158,191,113,86,6,94,60,210,8,75,183,183,207,247,188,
  105,197,228,248,63,210,186,209,173,140,54,48,158,143,52,207,11,50,205,248,249,167,231,199,141,17,117,122,3,248,254,255,78,161,186,181,123,68,121,62,
  20,138,104,8,7,44,39,112,227,228,109,49,127,171,112,212,245,184,3,93,142,181,61,196,139,174,75,95,132,233,230,41,200,214,199,207,118,176,78,116,
  213,79,55,79,193,189,69,209,59,200,9,128,183,91,15,224,209,202,103,145,173,79,199,189,211,110,193,218,180,216,247,188,99,234,117,195,166,140,14,245,
  6,166,129,228,40,101,24,97,101,180,179,170,223,253,28,24,224,110,122,167,211,207,90,156,97,208,224,30,96,122,48,231,32,86,28,186,84,196,139,27,
  12,193,229,13,226,197,163,141,88,154,159,18,119,231,116,102,162,14,150,110,47,92,222,224,37,63,207,104,106,155,88,232,118,49,158,143,36,207,11,50,
  205,248,249,205,241,21,209,234,54,7,126,241,202,217,1,199,136,14,38,207,187,125,206,33,227,225,73,251,133,184,202,104,127,220,221,139,146,227,43,77,
  217,250,244,30,173,175,33,133,21,23,32,85,91,234,73,105,107,241,180,227,161,11,79,226,205,213,127,142,123,63,6,134,17,27,219,253,3,235,175,163,
  42,247,208,178,21,57,66,185,80,25,30,187,196,74,69,101,103,218,176,181,135,93,237,122,131,26,169,105,134,94,175,193,48,190,112,188,174,11,159,92,
  144,21,247,119,163,86,133,12,147,166,199,107,28,171,177,75,159,107,59,92,49,243,78,102,36,104,97,212,169,224,242,6,145,145,160,131,73,167,234,85,
  153,40,109,236,194,178,194,148,216,202,75,130,14,38,157,186,215,10,61,71,42,172,140,201,140,231,49,177,121,110,22,190,182,61,190,242,180,175,164,5,
  255,216,91,49,168,229,101,71,43,14,88,78,192,17,112,197,116,177,191,221,220,55,235,98,182,62,61,174,235,28,0,222,110,249,72,218,21,30,111,167,
  124,104,87,253,201,30,114,116,78,51,133,239,209,211,253,66,169,157,134,3,87,100,174,137,89,46,52,84,189,137,129,97,76,42,163,41,105,6,164,164,
  25,6,245,154,77,13,221,120,227,229,11,184,250,134,216,131,239,143,127,179,169,199,243,223,124,249,2,154,27,186,123,205,69,202,48,182,80,107,117,227,
  153,163,77,184,117,89,110,204,223,255,254,217,121,61,158,255,159,67,13,168,237,8,15,250,143,189,95,131,7,174,158,25,85,109,39,35,65,135,39,239,
  88,220,235,243,188,119,222,130,154,118,225,122,111,158,110,198,178,194,20,20,231,69,39,104,207,72,212,225,233,47,45,239,241,90,143,238,46,71,91,151,
  135,49,153,241,60,10,5,153,230,30,21,81,0,216,60,47,27,155,99,36,183,47,169,179,227,251,255,59,57,238,228,226,148,237,60,214,102,44,142,82,
  232,42,28,117,232,203,192,223,91,213,165,23,27,247,72,59,221,63,149,119,89,220,216,210,144,171,254,151,23,158,196,247,103,126,33,234,247,181,233,139,
  240,254,134,127,246,250,60,143,86,62,59,108,109,215,23,171,47,3,195,152,83,70,135,108,245,187,191,14,165,103,44,88,187,113,10,10,167,167,244,154,
  8,191,185,161,27,213,229,54,124,252,94,157,148,63,148,97,252,97,87,153,5,199,234,58,177,125,118,58,102,103,155,123,77,138,94,219,225,198,185,22,
  7,222,62,219,10,75,183,47,234,183,175,253,239,12,214,207,76,199,210,252,100,228,167,69,151,129,140,68,77,187,11,181,237,78,188,119,161,61,170,78,
  249,79,118,148,97,89,97,42,150,21,8,10,74,70,98,207,86,177,210,134,78,28,174,180,226,72,69,135,34,153,58,3,227,185,28,61,229,16,157,168,
  248,176,253,100,148,50,122,160,253,68,159,207,239,45,7,169,60,229,82,133,163,62,110,98,252,144,171,254,237,150,143,112,210,126,1,55,229,93,134,105,
  230,41,61,134,0,72,10,117,231,5,84,56,234,240,98,227,158,62,199,185,50,48,76,104,101,244,91,95,125,87,218,6,26,107,209,25,235,187,202,114,
  27,190,123,207,158,158,143,235,101,5,107,235,112,227,245,151,47,72,135,165,164,25,144,146,170,87,156,106,179,122,20,201,235,47,5,251,222,170,196,190,
  55,43,251,69,39,195,240,195,226,240,225,153,35,141,0,4,177,204,48,107,145,97,214,138,123,81,132,255,183,119,251,208,214,237,147,120,22,111,211,159,
  203,23,196,219,37,173,120,187,164,85,186,222,156,220,4,16,241,120,94,60,173,189,219,139,182,110,111,228,118,232,40,28,173,178,226,72,149,21,148,82,
  16,8,85,122,140,33,101,66,76,235,99,233,242,8,22,177,136,52,63,3,193,217,122,59,174,251,245,251,162,124,210,80,26,74,89,75,48,158,143,117,
  158,159,173,183,227,19,191,253,64,198,84,42,61,82,92,158,83,58,102,249,255,84,245,171,120,170,250,213,158,23,40,205,7,176,171,37,126,249,203,175,
  159,82,186,156,105,196,232,253,131,179,143,198,191,120,196,64,255,181,211,241,221,215,242,54,110,241,180,227,17,153,133,51,65,109,196,116,185,107,158,132,
  21,221,190,186,229,191,126,186,23,215,121,156,73,105,253,135,119,14,184,253,191,118,230,97,208,24,215,229,34,190,219,240,209,157,96,96,202,232,132,68,
  100,213,36,6,134,144,162,210,238,240,41,170,229,92,138,124,148,53,117,203,20,147,75,187,94,141,197,169,152,176,40,29,111,106,34,227,57,227,57,67,
  44,56,2,46,229,134,42,54,105,49,140,51,112,172,9,24,24,24,24,24,24,24,24,24,152,50,202,192,192,192,192,192,192,192,192,48,225,48,32,55,
  125,126,190,50,5,73,191,99,34,251,25,51,218,39,143,4,25,248,181,200,192,46,223,251,113,180,31,215,39,125,191,239,64,60,52,4,131,123,125,50,
  88,109,4,32,47,63,101,64,194,107,210,169,49,127,114,74,148,92,197,47,228,160,140,137,11,157,66,66,209,130,50,126,201,227,7,9,141,112,140,210,
  136,114,140,242,107,203,120,31,250,59,150,203,150,70,63,90,248,42,178,231,224,21,127,203,238,17,245,76,242,119,26,253,92,84,121,104,232,153,104,212,
  253,35,238,69,163,251,110,228,125,249,200,131,162,226,76,105,212,243,206,204,75,102,60,103,60,239,19,18,180,38,172,200,154,31,190,62,233,219,24,19,
  179,205,7,107,224,28,228,243,105,127,47,51,134,220,244,177,98,70,201,96,60,255,40,106,3,66,6,235,113,200,176,201,220,80,93,98,86,82,225,240,
  41,163,159,191,99,9,83,227,25,70,28,69,153,102,252,230,166,69,172,33,24,207,25,198,49,102,167,20,226,63,91,127,197,26,130,129,97,28,99,64,
  202,232,147,127,255,59,170,171,171,89,235,49,12,10,10,10,10,240,133,187,238,234,247,121,213,85,85,120,242,31,255,96,13,200,120,206,192,120,206,192,
  192,48,134,251,249,128,148,209,234,234,106,156,45,41,97,173,206,48,162,112,58,157,76,14,25,207,25,24,207,25,24,24,198,56,6,164,140,82,176,228,
  34,12,131,7,58,66,231,50,48,158,51,48,158,51,48,48,140,124,63,103,187,233,25,24,24,24,24,24,24,24,24,70,12,76,25,101,96,96,96,96,
  96,96,96,96,24,49,12,172,2,19,243,211,51,12,38,152,255,142,241,156,241,156,241,156,241,156,129,97,194,246,115,102,25,101,96,96,96,96,96,96,
  96,96,24,49,12,80,25,165,35,242,154,59,111,46,50,179,50,71,236,254,236,53,148,175,75,89,134,13,239,43,51,43,19,115,231,205,29,243,109,62,
  242,116,48,158,51,158,51,158,51,58,38,22,29,172,159,143,3,203,232,47,30,122,8,91,182,108,97,75,8,134,17,197,150,45,91,240,139,135,30,98,
  116,48,158,51,58,24,207,25,29,140,14,214,86,35,103,25,29,57,176,208,33,6,38,131,140,14,214,86,140,231,172,173,24,29,12,227,167,173,184,75,
  33,118,168,95,95,187,239,62,108,222,186,53,202,240,27,250,251,23,15,61,132,185,243,230,49,163,60,38,166,131,126,184,228,112,238,188,121,248,197,67,
  15,197,149,195,205,91,183,226,107,247,221,55,234,219,121,52,209,193,120,206,120,206,120,206,232,152,8,116,176,126,62,6,45,163,43,87,174,196,45,159,
  249,140,244,247,188,121,243,226,186,229,231,205,155,135,185,243,230,33,51,43,11,0,144,153,149,133,175,223,119,31,76,38,19,91,22,49,92,18,76,38,
  19,190,126,223,125,10,217,154,59,111,30,230,205,155,23,243,248,45,91,182,40,126,187,229,51,159,193,202,149,43,25,29,140,231,140,14,198,115,70,7,
  147,93,214,86,125,128,106,246,172,89,15,246,245,224,204,204,76,108,218,188,25,251,246,238,69,91,91,219,160,63,204,13,159,250,20,174,189,238,58,188,
  187,103,15,230,205,155,143,180,244,116,112,106,53,86,172,92,137,229,43,87,161,170,186,26,9,137,137,88,187,110,29,230,205,159,143,174,174,46,116,119,
  119,131,227,56,108,222,188,25,87,92,113,5,118,237,218,5,167,211,9,158,231,193,7,131,8,6,131,224,131,65,4,2,1,4,99,189,100,199,240,148,
  130,138,47,2,128,16,194,52,179,97,64,86,86,22,182,108,221,138,253,251,247,195,210,71,185,218,180,121,51,0,96,239,222,189,131,254,60,41,169,169,
  248,198,55,190,1,0,104,107,179,96,78,113,49,212,26,13,10,167,77,195,202,149,171,96,52,153,81,93,93,131,213,107,215,98,253,198,13,208,104,117,
  232,104,239,0,33,28,40,165,184,255,123,223,131,207,239,199,225,67,135,64,41,21,100,145,231,37,121,11,248,253,113,229,48,24,12,130,138,199,11,75,
  91,10,16,50,32,89,28,205,116,48,158,51,158,51,158,143,127,158,79,68,58,134,3,227,173,159,3,3,206,51,74,133,215,37,162,160,176,16,119,221,
  125,55,254,248,135,63,160,173,181,21,231,74,75,81,88,88,132,59,191,120,23,42,171,170,240,175,127,255,27,23,47,94,68,125,125,61,218,219,45,80,
  115,28,2,193,32,212,26,45,166,78,157,138,130,130,2,204,153,51,7,11,23,47,65,222,164,201,56,114,228,8,218,219,218,224,113,187,113,243,167,63,
  13,151,203,133,215,95,123,45,182,237,152,200,104,9,125,228,169,244,61,17,255,79,56,2,142,227,192,113,28,83,78,135,10,151,34,75,116,112,162,99,
  110,249,236,103,97,50,153,240,143,39,158,64,119,103,39,202,202,202,144,95,80,136,171,175,189,22,71,143,31,199,171,175,189,142,234,234,106,52,53,55,
  193,231,241,128,242,60,130,148,34,57,57,5,133,133,133,152,49,99,6,230,207,155,135,155,63,125,11,74,203,202,112,226,232,49,120,61,30,164,165,166,
  225,246,47,220,137,231,159,125,22,245,245,245,209,178,24,67,14,121,241,7,158,4,195,135,144,176,28,114,28,55,230,233,8,4,2,160,3,229,29,227,
  57,227,57,227,57,163,99,20,209,49,28,24,239,253,92,61,210,122,200,156,226,98,124,225,174,187,80,81,94,1,171,221,142,31,63,248,32,78,156,58,
  137,250,250,122,4,3,1,164,165,167,163,112,234,84,220,240,201,235,161,211,233,208,208,208,128,138,242,114,84,84,86,226,252,249,115,120,237,181,157,48,
  154,76,152,51,123,54,230,22,23,99,253,198,77,240,122,60,88,183,97,3,206,158,57,19,191,81,104,68,59,199,210,86,9,5,229,133,87,144,4,65,
  8,129,138,227,192,169,84,76,129,28,103,40,40,40,192,210,101,203,208,220,212,12,131,201,132,167,255,243,31,156,56,126,2,165,231,202,96,183,219,97,
  54,153,144,155,147,131,141,235,214,97,230,172,89,232,234,234,66,85,85,37,42,42,42,112,242,212,73,124,244,209,1,128,16,76,157,50,21,197,115,102,
  99,110,241,92,108,218,188,5,243,230,207,67,97,97,161,100,113,239,77,14,133,175,4,217,131,184,54,162,226,255,249,96,16,124,144,7,8,36,57,140,
  92,32,141,21,58,188,30,15,44,109,109,240,122,189,130,39,98,4,22,122,140,231,140,231,140,231,140,142,193,160,131,201,238,165,247,243,17,81,70,231,
  205,155,135,146,146,18,84,85,86,226,229,151,94,2,81,169,113,226,244,41,188,249,230,155,104,104,108,4,199,17,36,39,37,99,122,81,17,86,173,94,
  141,169,83,38,195,231,15,192,102,179,34,33,33,1,51,103,206,68,106,90,26,234,235,235,208,220,220,130,110,135,3,135,143,28,193,217,179,103,49,123,
  246,108,44,91,186,12,103,75,206,226,205,55,222,0,165,20,6,131,1,105,105,105,104,104,104,136,255,80,68,108,82,177,225,169,240,39,8,8,168,216,
  232,160,64,128,6,65,130,65,97,165,52,66,66,201,48,72,157,187,176,16,109,173,173,112,58,157,120,250,95,255,130,74,163,129,55,224,199,11,255,254,
  55,78,157,57,13,167,195,1,147,201,132,153,211,167,99,222,220,185,40,158,59,23,73,73,137,232,234,234,134,90,173,70,110,78,30,146,18,147,144,147,
  221,132,202,202,74,216,187,186,80,91,91,131,218,186,90,156,56,121,18,75,22,47,129,57,49,1,141,13,141,168,171,173,5,8,193,244,233,211,81,94,
  94,222,243,131,73,178,23,238,244,130,8,134,63,7,121,30,65,158,7,71,8,10,139,138,208,209,209,49,166,232,160,148,34,16,8,32,224,247,195,239,
  243,129,112,28,84,42,213,144,91,31,24,207,25,207,199,42,207,167,205,152,1,75,91,27,163,99,148,208,193,49,217,29,212,126,62,160,152,209,189,239,
  190,59,224,152,209,107,175,187,14,223,185,255,126,180,182,182,98,214,236,57,112,123,189,120,241,229,151,240,238,158,119,225,114,185,144,145,145,129,57,179,
  102,97,211,166,77,216,186,117,11,18,19,147,208,217,217,137,246,246,118,184,92,110,128,2,28,71,160,215,235,145,150,150,142,172,172,44,24,13,70,80,
  94,104,128,198,198,70,84,215,212,192,239,247,99,217,242,229,32,32,184,245,115,183,98,195,134,13,120,119,207,158,120,74,191,242,111,66,100,166,231,80,
  219,147,240,210,128,8,13,78,197,152,9,194,113,96,42,233,192,145,57,192,88,50,10,96,223,187,239,14,248,190,127,123,252,113,172,92,181,10,21,21,
  21,184,252,138,43,241,218,27,111,224,217,231,158,195,197,242,139,48,155,76,200,159,58,5,203,150,46,197,21,219,175,192,140,25,51,0,0,86,155,21,
  118,187,29,193,128,96,41,87,171,85,48,155,205,200,205,203,67,122,90,26,130,65,30,132,0,93,221,221,168,174,169,134,205,110,71,74,106,10,166,79,
  159,129,220,220,28,220,126,231,157,112,185,92,168,169,169,137,146,59,26,175,211,75,150,122,249,194,73,92,40,1,248,229,175,126,133,197,139,23,163,182,
  174,14,219,175,188,106,76,208,145,150,158,142,213,107,214,224,224,199,31,163,163,163,3,160,20,188,24,187,20,47,36,134,241,156,241,124,34,243,252,137,
  127,252,3,43,86,173,66,57,163,99,84,208,17,138,179,36,67,24,194,55,22,101,119,32,253,252,146,148,209,214,1,42,163,13,13,13,88,180,104,17,
  230,20,207,197,233,179,37,120,235,205,183,80,91,91,131,148,228,100,172,89,181,10,151,111,219,134,85,171,86,35,53,37,21,86,171,21,237,237,237,232,
  234,238,70,48,16,4,21,255,17,49,54,65,173,82,195,96,208,35,51,51,3,147,38,77,66,114,82,18,84,42,21,60,94,47,234,234,234,224,116,186,
  176,114,229,74,228,23,20,224,149,151,94,70,67,67,35,226,89,161,229,223,135,62,82,89,163,209,8,129,4,8,40,129,20,252,59,154,131,157,71,59,
  178,46,65,25,221,123,9,147,148,205,102,195,134,77,155,144,148,146,130,103,159,125,14,31,126,248,1,40,165,152,55,167,24,155,54,109,196,214,173,91,
  49,107,214,108,120,125,62,116,88,173,176,217,236,112,185,221,160,60,5,79,195,11,17,78,165,130,70,173,70,66,98,34,166,76,158,132,236,172,44,24,
  244,122,240,60,15,75,187,5,22,139,5,233,233,25,88,182,124,57,186,187,186,177,227,149,29,240,251,253,113,100,145,132,229,79,254,61,137,236,249,68,
  250,174,179,179,19,43,87,173,130,57,49,17,207,191,240,34,14,28,56,0,140,114,58,66,3,214,161,131,7,97,237,232,16,250,143,120,12,207,243,146,
  123,71,222,167,24,207,25,207,39,50,207,173,54,27,54,109,217,130,100,70,199,168,160,35,164,11,240,60,15,62,228,142,30,100,29,96,44,202,238,64,
  250,249,176,186,233,51,179,178,224,116,56,224,112,56,176,123,247,59,112,121,61,40,41,41,193,178,101,203,176,237,178,173,72,76,76,132,207,235,131,203,
  237,66,93,93,29,188,94,175,176,203,157,231,1,42,42,125,84,32,4,34,49,156,138,3,225,8,40,207,195,160,211,33,127,234,84,20,22,21,193,233,
  112,162,165,181,25,165,101,165,104,109,107,133,219,227,70,125,125,61,40,229,145,155,155,139,166,166,102,165,94,217,27,8,1,161,20,84,92,19,81,16,
  16,66,1,42,8,38,161,84,218,97,166,82,169,6,93,32,25,6,23,133,133,133,168,170,170,66,233,217,82,188,246,218,235,120,235,237,183,144,155,147,
  131,79,221,120,35,178,179,50,193,113,42,56,28,14,216,108,54,184,220,77,240,251,252,130,236,241,66,215,11,201,97,72,22,57,194,129,168,8,56,80,
  16,74,145,154,146,130,244,244,116,204,227,41,218,218,90,209,208,216,136,119,247,237,133,165,221,130,217,51,103,194,233,116,194,104,52,64,111,48,192,102,
  181,69,200,34,149,86,222,146,47,68,246,137,16,113,225,68,128,41,147,167,160,190,190,30,231,207,159,199,174,221,187,177,111,223,126,100,101,103,225,154,
  171,174,198,164,73,185,208,233,244,163,150,14,42,102,187,16,238,15,161,95,137,253,12,132,72,131,214,96,185,113,25,207,25,207,199,42,207,167,21,77,
  67,101,101,37,206,150,148,224,181,215,94,195,91,111,191,205,232,24,65,58,132,169,95,212,9,68,69,139,242,60,2,76,118,47,169,159,15,155,50,250,
  208,67,15,129,167,84,140,17,85,225,157,119,222,193,218,213,107,176,116,233,82,84,86,86,162,205,98,129,211,33,166,100,18,27,210,104,52,130,16,14,
  169,41,105,0,33,8,4,130,240,250,188,240,121,61,112,185,92,240,249,188,8,4,252,0,0,78,165,2,165,20,126,191,31,42,181,10,147,39,79,65,
  83,83,19,206,150,150,34,43,51,19,171,87,173,198,146,37,139,177,106,245,106,188,244,226,139,56,126,252,184,152,145,64,190,133,137,68,52,111,148,78,
  42,9,36,98,8,38,15,128,6,2,80,169,84,32,28,199,180,190,81,136,45,91,183,226,27,247,221,135,23,95,124,17,60,79,177,119,223,62,92,188,
  120,17,223,254,230,55,113,248,240,17,116,118,218,209,217,213,5,191,207,47,216,225,41,133,90,173,134,65,99,128,193,96,132,209,104,2,79,41,124,62,
  31,188,94,47,188,162,44,250,253,62,80,49,150,56,100,45,15,242,65,164,166,166,193,227,241,160,164,180,20,7,15,29,18,172,192,151,109,197,226,133,
  11,65,56,14,191,126,248,225,240,178,82,20,69,177,207,70,7,232,200,4,113,245,170,85,184,243,11,95,192,174,183,223,6,5,193,129,3,31,161,178,
  170,18,95,254,210,151,112,242,212,41,88,59,108,112,185,221,162,100,143,62,58,120,217,2,142,130,70,117,184,208,145,193,64,0,84,165,18,22,121,140,
  231,140,231,19,140,231,151,111,219,134,239,220,127,63,158,123,246,89,128,112,216,183,111,63,163,99,4,233,136,50,74,137,239,144,27,165,68,69,107,162,
  202,238,64,251,121,255,148,81,209,102,59,144,36,27,79,60,241,4,190,120,215,93,200,202,201,197,191,254,253,52,90,90,90,144,155,155,131,134,198,6,
  52,183,52,11,187,214,101,214,207,162,162,34,204,45,158,11,167,219,15,189,86,15,183,219,13,167,203,133,110,71,55,28,14,7,212,157,118,184,156,78,
  120,60,46,248,125,94,4,2,62,209,252,11,168,85,42,80,0,9,9,137,240,120,60,56,113,242,20,204,38,51,182,109,221,138,211,167,78,227,108,201,
  89,105,5,65,9,13,199,129,16,26,126,15,105,158,178,119,42,15,143,8,9,162,124,133,36,10,106,32,24,132,138,2,156,138,41,164,125,18,171,8,
  249,26,74,28,58,120,16,7,87,174,196,220,185,243,240,239,103,254,131,247,62,120,31,235,215,174,69,115,75,11,106,107,107,165,193,36,212,201,141,70,
  35,150,46,93,10,147,49,1,60,21,248,233,114,186,208,237,20,172,252,14,71,55,56,85,39,220,110,23,124,30,55,2,1,175,176,242,131,176,26,133,
  10,208,233,116,80,169,56,52,54,53,225,163,143,15,98,251,101,151,193,23,8,96,215,91,111,11,49,71,226,144,38,133,39,17,34,45,148,40,21,172,
  240,194,187,184,6,162,20,39,79,156,196,241,5,199,48,109,250,12,188,250,218,78,28,60,124,8,43,150,45,67,107,91,43,26,234,235,193,139,109,73,
  8,129,74,140,25,26,77,116,240,60,143,64,32,32,228,246,149,165,84,147,6,43,89,191,226,165,129,141,241,124,162,241,60,20,2,53,81,121,126,224,
  192,1,172,90,179,6,11,22,46,98,116,140,2,58,20,178,24,207,40,37,179,252,145,9,40,187,3,233,231,42,181,26,234,1,229,125,163,144,148,185,
  62,55,240,199,7,49,101,106,62,78,158,62,133,19,39,78,160,168,160,0,105,233,105,56,127,238,188,66,17,77,77,77,69,65,65,1,150,44,89,130,
  162,194,34,180,91,29,104,109,105,69,107,75,179,20,103,192,113,28,52,26,45,52,218,32,130,65,10,158,2,65,62,8,62,24,80,104,202,121,185,121,
  80,115,39,208,222,209,142,83,167,79,35,55,39,7,245,53,53,112,185,92,17,22,79,34,197,162,70,189,147,80,3,10,230,102,144,16,55,168,180,3,
  159,18,34,15,37,5,40,16,164,130,192,112,204,66,218,15,109,20,253,203,79,54,0,57,116,116,59,240,236,127,255,135,37,203,150,225,253,247,223,71,
  187,197,130,21,203,151,163,188,188,28,193,96,80,178,42,105,52,26,228,229,229,137,217,25,150,2,68,131,54,139,21,150,54,11,188,30,175,36,139,42,
  149,26,90,173,78,144,195,32,69,144,242,0,207,3,178,231,74,72,72,68,82,98,18,154,154,154,113,225,226,5,228,102,103,35,224,247,163,244,236,89,
  73,214,194,114,40,186,65,20,50,72,194,178,39,190,187,156,78,188,254,218,235,152,59,127,62,14,31,62,12,107,71,7,22,44,88,136,170,170,42,25,
  29,128,86,171,65,110,78,14,22,44,92,136,149,43,86,140,26,58,40,207,75,97,56,148,242,138,101,158,114,209,23,254,78,112,57,49,158,79,36,158,
  59,156,78,24,244,250,9,203,115,71,87,55,254,243,244,191,177,122,237,90,70,199,40,160,67,49,247,147,168,112,82,217,59,69,128,31,152,66,58,214,
  101,119,64,99,187,223,63,80,55,125,223,7,134,149,171,86,97,213,170,85,56,122,244,40,108,118,59,222,124,243,77,248,253,126,204,154,57,19,221,93,
  14,116,119,59,36,69,116,234,148,41,216,188,121,51,242,11,10,160,215,233,64,8,96,54,233,97,215,233,160,82,171,65,196,120,131,96,48,8,191,207,
  143,64,192,143,32,31,4,165,28,56,78,7,158,7,40,13,128,82,33,160,216,108,54,33,55,39,27,117,141,141,168,173,175,195,129,143,63,194,198,117,
  235,145,152,152,136,153,179,103,225,229,23,94,4,8,194,171,31,74,148,22,81,89,243,133,20,82,41,157,129,120,156,116,188,76,227,15,237,182,23,42,
  20,80,112,28,203,75,58,88,242,116,41,231,253,240,71,15,224,224,199,7,145,145,149,133,151,94,121,5,231,206,157,67,81,97,33,204,102,51,106,75,
  203,164,213,166,193,96,192,226,69,139,176,118,237,90,36,137,155,226,252,1,30,70,131,30,26,141,22,68,21,118,115,4,2,1,248,68,89,20,22,129,
  26,112,28,65,144,122,0,81,14,53,90,13,210,82,83,145,144,96,70,87,119,55,14,29,57,140,180,180,84,20,207,157,139,252,252,124,148,95,188,136,
  139,23,47,8,242,39,5,231,200,186,173,92,30,9,193,23,191,248,69,148,148,148,32,45,35,3,123,222,125,23,21,149,149,152,50,121,50,140,6,3,
  42,42,26,37,107,146,193,160,199,252,249,243,177,106,213,42,36,39,39,67,175,215,131,167,100,84,208,193,243,84,216,148,200,83,177,251,80,197,170,25,
  17,43,104,233,59,209,26,208,215,1,158,241,124,108,243,156,231,121,56,93,174,9,201,243,159,253,226,23,56,240,225,135,200,205,203,195,43,59,119,50,
  58,70,152,14,201,115,23,154,251,99,200,107,228,123,48,24,236,151,66,58,30,100,119,32,253,156,18,114,41,181,233,105,159,94,243,230,207,195,150,203,
  182,226,230,91,110,193,241,227,199,209,208,216,136,244,180,52,44,89,178,4,54,155,85,138,17,77,77,77,197,230,205,155,49,109,218,52,24,244,122,137,
  121,58,157,10,6,131,81,72,52,47,210,207,7,121,4,130,65,4,2,65,4,3,162,6,14,2,149,74,7,194,169,195,86,20,66,48,117,234,84,0,
  128,215,235,69,93,125,61,218,44,109,88,191,113,3,82,83,83,145,158,145,33,54,24,47,105,241,33,69,54,244,57,100,181,165,148,7,168,252,51,47,
  78,140,124,104,138,148,190,131,248,29,196,235,242,124,176,207,237,53,49,95,151,170,200,246,254,42,44,44,64,97,97,33,190,120,247,93,208,104,52,56,
  119,174,12,110,183,27,107,214,172,17,114,162,5,2,146,162,179,96,254,124,172,95,191,30,169,169,169,225,120,22,21,7,189,94,11,173,78,43,184,54,
  32,88,232,132,242,104,188,32,135,193,160,216,39,57,168,84,58,80,194,9,233,191,40,69,114,114,50,146,147,146,65,41,69,123,71,7,46,94,44,71,
  97,81,17,102,205,158,133,188,73,121,130,252,73,241,210,74,121,148,191,114,115,115,144,157,147,131,235,174,255,4,56,142,67,69,101,5,188,110,15,150,
  44,90,132,64,192,7,191,88,253,130,231,131,152,51,123,54,86,173,92,137,228,164,36,16,66,224,112,56,64,64,71,5,29,148,242,162,187,73,236,63,
  20,82,176,125,216,10,42,174,199,169,50,148,67,88,128,242,140,231,19,129,231,226,152,27,12,6,38,20,207,139,138,138,48,109,250,52,220,115,239,87,
  161,211,235,25,29,163,64,118,165,249,94,250,172,156,243,133,247,176,44,138,82,61,225,100,119,96,253,124,24,54,48,253,253,241,199,241,209,129,3,184,
  238,250,79,226,196,169,147,224,56,130,169,83,166,32,39,39,7,167,207,156,145,98,68,11,10,10,144,95,80,0,141,70,163,56,159,35,4,230,4,35,
  52,106,173,100,25,13,136,181,230,3,129,0,130,65,94,145,239,83,171,50,32,16,32,240,120,4,87,124,90,90,58,140,122,61,92,30,15,186,186,186,
  112,177,188,2,217,89,89,216,179,123,119,120,187,18,17,55,50,81,132,227,69,21,155,153,100,155,155,68,11,170,240,27,149,210,63,133,181,124,217,98,
  65,252,24,170,225,202,92,246,35,135,170,170,42,220,121,251,237,248,210,87,190,130,178,115,231,96,105,111,71,106,106,10,102,206,152,33,164,194,16,59,
  57,207,243,88,177,114,37,146,147,147,163,54,175,233,116,90,24,13,6,97,97,36,38,64,246,7,130,8,4,252,240,7,2,178,180,21,156,84,20,193,
  203,119,131,15,6,97,48,26,145,156,148,140,134,166,38,4,131,65,84,85,87,163,190,190,30,39,143,31,71,77,77,181,108,229,40,139,89,142,225,215,
  104,108,104,192,79,127,242,32,62,121,195,141,66,18,99,187,29,73,73,137,152,58,117,42,92,46,183,48,88,136,39,46,92,180,8,9,9,9,82,231,
  231,121,30,78,167,3,106,141,126,196,233,16,250,113,0,60,79,165,88,111,74,137,98,21,29,238,87,33,223,132,92,33,229,161,234,37,191,31,227,249,
  216,231,57,149,149,158,238,139,149,105,188,240,188,162,252,34,110,190,241,70,124,227,91,223,98,116,140,2,58,34,247,140,135,61,165,242,189,35,16,170,
  54,34,236,33,13,89,12,39,146,236,14,108,108,167,24,22,237,40,33,49,17,149,85,85,168,175,175,71,74,114,42,22,45,92,4,151,219,13,143,199,
  35,109,86,90,178,100,9,244,58,93,204,243,77,70,65,219,39,32,224,197,236,254,126,191,95,8,146,21,55,54,112,98,169,78,181,90,3,141,70,47,
  144,71,41,12,70,3,50,50,50,4,235,168,207,135,166,230,38,33,7,87,80,168,164,16,74,135,32,223,197,207,243,178,151,100,13,13,197,181,82,41,
  223,105,104,17,68,21,90,190,76,251,151,189,135,18,228,50,140,96,48,0,165,8,4,121,148,87,148,195,229,114,97,201,226,165,48,153,76,232,238,238,
  150,58,249,246,237,219,145,149,153,25,243,124,141,70,5,131,65,7,149,184,65,46,24,20,171,76,248,125,224,131,60,120,62,148,3,151,64,165,86,67,
  163,214,66,173,18,22,87,106,181,26,169,105,169,208,137,50,222,110,237,64,83,115,51,140,70,163,40,27,50,25,148,228,143,87,164,232,224,67,242,199,
  243,240,7,2,168,173,171,133,199,227,193,156,57,115,160,215,235,209,237,112,72,86,195,13,235,55,32,45,53,85,146,59,233,26,60,15,159,207,3,189,
  94,59,178,116,240,60,130,129,128,50,5,73,40,26,41,148,248,55,212,175,66,95,82,89,112,134,168,144,82,198,243,113,205,115,105,236,21,191,10,6,
  39,22,207,41,165,140,142,209,66,7,13,207,251,84,254,119,72,86,17,237,201,145,251,254,38,140,236,14,164,159,99,128,150,209,190,58,87,239,254,210,
  151,144,153,149,5,183,219,141,151,94,217,1,62,24,196,156,217,179,176,104,241,34,52,212,213,33,16,8,192,104,52,98,110,241,92,20,21,22,197,221,
  52,169,81,115,72,74,74,68,83,147,10,129,128,31,94,175,7,62,175,79,72,13,32,90,28,53,106,53,84,106,21,84,42,53,84,42,19,92,174,46,
  4,125,62,112,28,135,188,188,60,88,44,22,184,61,30,216,59,59,209,216,220,132,53,235,214,33,57,41,9,7,15,30,132,213,218,33,172,112,136,60,
  25,107,248,69,33,196,51,68,5,223,202,19,226,82,26,46,217,20,177,42,146,175,24,88,114,252,193,139,24,237,235,185,133,133,133,184,251,75,95,66,
  85,85,21,108,157,93,168,170,170,70,122,106,42,54,109,218,8,191,223,143,238,174,46,240,60,143,204,140,76,108,222,180,25,28,23,167,66,4,71,96,
  54,27,160,215,233,64,197,212,25,30,143,23,126,159,95,44,17,199,65,165,86,41,100,17,224,225,243,121,65,64,144,146,146,130,212,164,36,248,188,94,
  248,124,62,212,214,213,98,198,230,45,216,154,157,13,135,211,129,131,31,127,172,144,63,34,174,25,67,175,156,236,108,92,117,245,213,104,107,107,67,87,
  87,23,26,26,27,145,156,152,136,37,75,150,192,239,243,193,209,221,13,158,231,145,158,150,142,149,43,87,130,16,46,60,104,242,188,184,209,78,88,201,
  115,132,142,24,29,33,111,65,32,16,144,38,170,200,21,190,184,93,80,17,99,164,212,70,133,15,124,48,40,88,1,24,207,199,37,207,165,227,40,149,
  100,96,188,243,124,198,244,233,248,250,125,247,161,178,178,18,93,221,14,70,199,8,211,17,245,84,210,60,47,27,140,136,60,118,82,169,7,144,9,36,
  187,3,237,231,132,98,104,45,163,38,147,9,133,133,133,72,77,75,199,197,139,23,145,150,150,134,85,171,86,193,239,247,195,106,183,131,231,121,104,212,
  90,120,125,20,237,86,7,220,158,128,148,158,36,18,169,201,9,80,171,84,240,251,2,66,227,138,249,69,213,42,21,180,90,45,116,122,3,116,58,3,
  12,6,61,12,6,29,178,178,114,132,149,10,79,145,146,146,130,172,172,76,0,4,62,177,82,65,90,122,26,252,126,159,164,245,11,49,16,97,107,168,
  98,53,36,126,14,175,212,161,88,29,41,82,94,197,44,89,64,149,10,41,195,176,194,100,54,11,155,233,86,175,70,107,107,43,58,58,58,176,106,213,
  42,164,165,165,193,222,217,9,143,215,11,80,192,104,52,163,165,173,19,93,221,94,248,3,177,87,177,38,163,30,70,163,1,148,231,225,241,248,224,243,
  121,37,151,177,90,163,134,94,175,23,101,81,15,131,65,143,228,228,100,152,204,102,80,74,161,213,106,145,155,151,11,189,78,15,0,232,176,217,192,243,
  60,204,9,102,97,163,73,104,165,41,198,230,132,95,130,28,235,180,58,240,65,30,51,103,206,66,71,71,7,58,59,59,49,119,238,92,36,38,36,162,
  179,171,91,160,131,167,208,235,141,104,181,116,162,171,219,3,175,47,24,101,221,23,98,177,213,80,171,184,17,161,35,180,170,22,60,27,145,125,10,138,
  24,163,152,253,138,74,54,8,69,89,94,198,243,241,199,243,72,203,104,136,231,252,56,230,121,136,142,117,27,54,48,58,70,1,29,225,125,35,145,125,
  138,42,167,248,72,217,149,191,79,16,217,29,248,216,78,135,54,102,244,15,191,255,61,0,224,7,63,250,17,234,235,235,49,107,198,12,76,159,54,13,
  45,205,205,240,138,141,155,152,152,4,181,74,141,214,150,86,216,117,58,24,12,70,152,19,140,48,25,181,208,168,57,133,249,89,167,211,193,231,243,194,
  35,38,118,214,104,117,208,105,181,208,104,181,208,233,52,80,169,84,80,171,213,80,169,212,8,4,252,82,58,16,131,193,128,204,140,76,180,52,183,32,
  16,8,160,171,171,11,39,142,29,71,85,85,165,164,229,43,44,162,60,1,8,39,124,228,194,26,59,149,34,215,194,171,31,249,28,41,165,54,80,196,
  144,202,170,52,8,33,37,82,141,86,134,225,65,201,153,51,248,222,153,51,88,191,97,3,172,246,78,112,132,96,233,210,37,240,122,60,232,236,236,148,
  210,101,76,154,52,25,150,54,11,186,116,90,232,116,122,24,77,70,36,152,245,48,232,53,138,77,149,6,131,30,148,82,120,220,46,248,124,62,168,213,
  66,234,12,157,86,8,39,209,104,212,80,113,42,168,53,106,248,253,126,16,34,20,100,224,8,135,244,180,12,36,38,52,192,227,113,163,187,91,200,153,
  187,127,239,94,4,2,1,209,106,46,4,151,135,42,203,9,98,34,164,199,168,174,174,198,63,255,249,36,230,205,159,15,167,203,5,14,4,51,103,206,
  132,215,227,65,119,119,23,248,64,16,132,227,144,149,149,133,142,246,14,56,29,14,232,13,70,152,205,38,36,36,24,97,50,18,49,47,28,5,199,9,
  53,139,131,193,224,176,211,1,112,160,84,92,61,139,174,28,2,26,211,237,16,51,191,95,200,71,33,171,232,161,138,40,197,199,120,62,62,120,78,35,
  22,33,33,166,80,158,42,227,214,198,17,207,79,28,59,142,123,78,156,192,182,237,219,225,246,250,24,29,35,76,7,192,245,234,130,139,178,248,69,122,
  63,39,136,236,14,184,159,15,212,77,15,153,102,219,23,248,124,126,88,173,29,184,233,83,55,74,53,84,67,171,13,16,2,183,219,141,214,150,102,168,
  212,106,177,134,170,208,96,73,73,137,72,77,78,128,70,35,52,84,66,130,9,14,135,3,42,149,10,137,137,73,208,27,244,208,106,53,66,156,168,90,
  13,142,19,202,131,114,28,7,131,94,143,170,138,243,66,98,87,142,67,74,106,42,50,179,179,80,87,223,0,167,211,137,236,140,12,240,65,62,74,25,
  37,156,152,203,148,0,224,4,197,148,39,68,216,157,198,241,97,99,52,79,100,46,119,185,75,191,55,97,164,160,188,120,28,115,215,71,89,142,135,242,
  220,96,32,136,110,71,55,114,243,114,145,154,154,134,142,142,14,120,61,194,162,136,167,60,84,106,45,44,150,86,112,28,39,132,123,168,213,208,106,181,
  48,155,76,72,74,50,195,108,210,67,173,86,35,41,41,1,62,159,23,254,64,0,9,102,51,140,38,51,244,122,45,52,26,65,22,85,50,57,164,148,
  162,171,211,6,171,213,34,36,40,54,25,145,155,151,11,123,119,55,92,98,33,7,163,201,8,187,213,38,40,13,28,1,71,66,121,236,56,240,84,144,
  73,78,202,107,71,16,240,7,224,116,186,144,158,158,6,179,217,140,206,206,78,120,189,62,209,244,30,4,225,212,232,104,111,131,90,163,17,242,241,106,
  52,208,233,244,48,39,152,145,156,148,0,179,217,0,173,86,131,132,68,51,92,78,7,252,126,255,176,211,193,139,155,16,165,114,191,132,198,28,240,165,
  4,203,145,170,40,149,215,141,134,180,242,103,60,31,95,60,167,161,164,247,52,188,141,52,84,158,145,23,55,133,140,87,158,51,58,70,23,29,161,223,
  136,76,63,141,172,214,40,87,174,162,12,81,19,68,118,7,210,207,9,29,66,203,104,97,81,17,238,254,210,151,0,0,150,14,43,12,122,61,116,90,
  45,236,54,27,124,62,31,40,132,220,162,129,64,16,78,151,43,108,153,20,119,182,19,16,52,53,169,160,86,9,22,209,132,4,19,2,193,0,114,114,
  178,161,213,106,161,86,107,36,101,144,200,146,206,19,113,119,124,168,194,128,160,153,19,104,52,26,232,117,122,169,100,168,201,104,194,138,21,43,224,112,
  116,163,180,180,84,212,244,9,230,207,95,136,180,180,52,168,213,26,168,213,106,168,53,234,240,103,117,216,242,170,82,171,5,75,172,74,5,78,44,105,
  165,82,113,224,56,21,158,254,231,63,16,81,53,52,234,157,15,213,177,103,24,114,220,253,229,47,163,176,176,16,14,167,11,207,62,247,28,18,204,102,
  56,29,14,56,29,14,69,0,182,215,235,131,223,31,16,100,73,148,65,16,8,113,54,98,56,136,94,175,67,114,114,34,76,102,19,166,79,43,130,70,
  163,21,23,65,156,40,183,225,32,163,144,69,74,167,211,74,174,29,142,227,160,215,27,160,226,56,4,131,65,248,188,94,204,158,61,7,60,31,68,89,
  105,153,16,27,196,17,81,30,69,11,58,47,88,232,55,111,222,136,220,220,92,4,130,60,62,254,248,99,232,245,122,184,92,46,184,92,46,240,193,160,
  148,136,217,235,245,33,24,20,228,139,19,101,146,227,56,168,45,130,60,107,181,90,24,13,122,164,164,36,195,96,52,160,160,32,95,72,159,54,76,116,
  112,8,187,114,194,161,46,225,33,42,106,29,39,139,47,138,103,29,13,185,116,67,11,60,198,243,241,193,115,42,223,188,70,16,93,21,111,28,242,252,
  91,223,254,38,102,204,156,9,183,199,131,231,95,120,145,209,49,194,116,112,178,29,73,194,206,249,176,115,52,122,31,73,12,131,212,4,146,221,1,143,
  237,132,12,79,109,122,66,8,252,129,0,42,42,42,80,84,52,45,28,235,69,41,188,62,47,186,29,221,82,218,38,136,110,108,97,215,188,31,126,95,
  0,62,159,87,176,102,230,100,225,198,27,63,9,167,211,129,238,46,167,184,35,223,11,159,223,31,222,189,21,20,206,245,251,125,98,60,153,160,244,250,
  188,94,116,119,119,133,99,55,105,40,7,152,16,255,32,152,155,9,154,155,154,208,101,183,11,10,166,168,112,170,56,65,209,84,73,138,103,120,176,231,
  84,42,112,28,17,254,38,4,132,227,148,147,167,84,75,44,186,93,152,187,126,248,193,243,60,90,90,90,208,102,177,132,51,33,136,175,110,71,55,84,
  42,85,120,39,178,24,63,28,74,35,230,247,249,224,241,120,224,118,187,241,169,79,125,18,25,25,233,112,56,156,112,56,92,112,185,220,240,122,125,66,
  74,139,96,80,146,197,64,48,8,181,90,35,221,35,24,12,162,187,171,11,129,64,64,170,96,33,84,172,16,242,209,134,74,175,133,44,243,60,47,166,
  4,19,243,235,134,142,11,242,65,88,173,86,216,172,54,112,28,7,158,242,82,40,185,195,233,128,70,173,17,23,118,156,98,129,198,243,60,2,254,128,
  16,216,238,245,224,147,215,95,135,140,140,116,4,131,20,78,151,123,88,232,8,29,27,206,134,65,197,193,93,168,171,71,105,236,21,116,56,198,72,172,
  136,22,145,70,141,167,84,112,215,51,158,143,27,158,135,221,244,52,236,121,138,224,109,172,69,253,88,230,185,188,45,24,29,163,128,14,153,50,26,182,
  242,73,214,175,24,214,209,240,254,38,50,209,100,119,160,253,28,67,152,103,180,170,178,18,223,251,238,119,1,0,247,255,224,7,224,121,138,234,154,26,
  100,102,101,65,173,82,195,27,244,2,148,194,231,245,192,225,112,128,19,181,240,112,66,251,0,188,94,15,60,30,47,60,110,143,16,243,160,209,194,233,
  116,96,202,228,92,80,10,184,221,62,116,118,59,224,116,186,225,114,10,177,17,126,191,31,124,48,136,54,75,171,16,10,0,129,81,54,155,13,237,29,
  86,209,188,173,66,71,123,59,42,43,202,101,77,194,131,231,9,90,91,90,4,37,147,19,20,75,142,136,230,236,136,247,144,37,53,108,158,150,89,105,
  65,34,148,207,104,43,78,84,60,20,195,144,225,137,199,30,3,0,172,93,183,30,32,64,123,71,7,90,91,91,145,153,145,41,176,133,23,70,206,174,
  174,46,232,116,58,169,35,5,131,161,234,21,94,120,189,62,120,220,30,248,253,62,164,167,167,163,171,171,27,69,133,83,145,157,149,14,158,167,176,117,
  58,225,232,118,194,233,116,195,227,245,32,224,23,42,94,8,41,57,194,187,10,93,46,23,90,90,91,225,245,249,160,81,171,65,64,112,234,212,73,56,
  28,14,72,25,215,120,0,28,15,194,115,224,56,49,209,50,207,225,189,253,251,64,56,14,179,103,207,17,92,43,93,93,176,218,172,72,77,73,21,35,
  103,120,112,132,192,225,232,134,78,244,2,132,114,53,6,41,21,83,124,4,224,243,250,16,8,248,145,150,158,134,174,174,110,20,20,76,65,82,82,34,
  52,26,237,176,208,65,57,97,165,31,174,210,33,62,167,56,208,199,10,95,81,186,233,149,195,58,145,29,196,243,66,108,36,227,249,248,224,57,141,99,
  129,146,255,29,90,212,143,23,158,255,254,55,191,6,225,56,92,126,197,21,140,142,81,64,7,229,120,128,114,225,188,183,0,8,9,103,246,136,149,101,
  71,218,95,34,115,213,79,4,217,29,104,63,39,151,162,140,246,85,141,42,44,42,130,94,175,199,212,169,83,81,83,91,131,244,180,52,204,154,61,27,
  196,39,92,195,229,114,65,221,105,135,70,163,21,74,124,138,138,168,223,239,23,6,208,96,0,0,144,152,152,8,181,90,131,238,46,167,164,211,25,141,
  90,24,140,169,8,6,121,120,188,126,120,61,62,120,60,62,120,188,62,92,44,63,39,237,184,237,234,234,66,109,109,29,124,126,191,232,174,215,33,40,
  174,6,2,1,191,48,189,209,80,144,63,47,148,108,165,28,56,158,130,231,40,56,41,238,33,220,168,16,133,81,250,19,225,154,173,10,59,120,31,44,
  117,132,89,71,49,212,114,104,54,155,145,155,151,7,111,32,0,142,83,161,180,244,44,18,86,172,132,70,171,21,87,137,128,213,106,129,217,156,140,64,
  192,143,128,63,128,64,48,8,127,64,148,67,191,96,121,215,104,52,48,26,141,112,185,60,240,251,5,217,228,56,130,180,20,51,82,147,205,240,250,2,
  194,160,32,202,98,69,101,57,60,30,183,20,15,84,91,91,139,78,49,77,135,209,156,32,88,211,197,141,112,132,132,243,212,9,74,149,88,62,88,236,
  240,60,37,48,104,181,72,76,78,66,128,231,65,8,135,218,154,26,24,141,70,104,181,90,97,149,12,192,110,183,34,193,156,36,203,3,71,197,65,203,
  47,228,186,19,119,77,26,13,70,56,69,58,252,254,0,180,90,237,176,209,193,139,149,65,248,80,101,51,18,94,65,135,23,105,68,177,65,32,58,91,
  5,13,215,232,13,77,2,148,130,66,197,120,62,78,120,46,85,112,9,13,173,36,186,207,83,177,52,227,120,226,121,146,217,140,73,147,39,67,213,218,
  198,232,24,5,116,112,161,12,30,52,180,113,153,42,172,163,113,247,51,200,92,245,19,69,118,7,212,207,233,16,39,189,127,234,233,167,241,231,191,252,
  5,9,9,9,40,40,40,128,205,102,71,83,83,19,156,78,39,84,106,13,120,74,225,245,10,46,120,151,203,13,183,199,13,183,91,120,121,60,30,65,
  81,164,20,106,181,6,122,131,30,32,4,46,183,27,110,183,79,97,21,81,171,56,152,141,58,36,39,155,145,158,158,132,204,140,84,52,55,55,74,202,
  158,181,163,3,29,54,43,0,33,21,148,65,167,199,148,201,147,177,121,243,102,228,230,230,73,245,83,67,238,251,158,210,57,208,136,56,38,121,241,47,
  69,170,7,32,102,106,135,72,5,138,103,214,209,33,199,214,203,46,195,11,47,189,132,107,174,187,22,233,25,25,48,154,140,104,110,110,70,123,71,135,
  36,69,148,82,56,28,221,146,252,133,100,209,227,246,192,231,243,129,231,131,224,8,7,157,78,7,141,86,11,159,207,7,135,211,131,96,144,87,116,42,
  189,78,141,196,4,35,82,83,19,145,145,145,2,202,251,209,221,221,5,74,41,220,110,23,218,218,218,224,245,9,242,107,50,26,161,86,171,177,106,245,
  106,172,95,191,94,166,103,9,19,48,31,33,135,51,102,204,192,23,239,186,11,243,231,207,135,217,108,134,86,167,133,213,106,69,103,103,167,40,139,4,
  148,167,112,57,157,112,187,61,112,123,132,126,228,241,120,224,245,120,224,247,9,94,3,142,112,66,220,181,70,3,191,63,68,71,16,126,191,127,88,232,
  16,74,238,10,85,58,164,250,197,178,218,243,242,60,162,210,139,42,251,86,212,142,38,153,162,74,121,158,241,124,156,240,92,153,214,137,42,23,36,114,
  153,24,71,60,191,250,218,107,176,239,131,15,240,169,155,111,102,116,140,2,58,66,86,67,41,225,189,252,93,238,194,167,242,121,63,218,108,50,17,100,
  247,82,198,246,33,141,25,253,239,51,207,0,0,178,178,178,48,103,206,28,188,254,198,235,104,106,105,70,99,99,35,102,205,154,5,143,199,141,96,208,
  15,175,199,5,62,40,212,156,14,6,130,82,28,39,0,168,84,156,144,142,64,171,1,33,4,30,143,23,157,221,14,24,140,169,81,11,18,21,71,160,
  210,170,225,232,238,68,93,93,173,84,151,181,181,181,21,1,49,53,130,86,171,133,217,108,198,185,115,231,144,144,96,134,213,106,149,45,98,68,235,39,
  194,141,75,104,216,106,74,136,240,89,158,196,54,158,233,57,214,202,8,113,15,163,108,103,253,16,162,228,204,25,252,247,153,103,112,226,248,113,172,89,
  191,1,153,153,153,168,173,169,65,101,85,37,178,178,178,192,113,28,2,193,0,252,94,55,220,46,23,130,60,47,134,140,4,37,171,18,225,8,212,26,
  53,116,58,29,212,106,53,130,193,32,58,59,29,72,77,77,130,65,165,141,136,145,22,10,53,80,62,0,139,165,21,157,157,66,78,93,187,221,14,167,
  203,37,241,59,37,57,9,14,135,3,141,245,245,225,152,72,185,200,72,3,161,32,127,141,141,141,56,116,240,16,154,154,26,49,107,78,49,146,18,19,
  209,110,177,160,185,185,25,169,169,105,66,168,11,207,35,224,243,194,227,241,40,98,179,169,184,162,231,84,66,122,31,97,19,160,26,193,32,143,206,46,
  129,14,141,70,176,148,13,53,29,160,68,172,130,22,148,13,242,66,248,74,40,70,137,244,228,89,160,50,103,61,137,114,214,131,82,202,120,62,174,120,
  222,251,130,125,60,241,252,216,145,163,120,226,111,143,225,240,225,195,184,234,186,235,24,29,35,76,7,196,247,144,14,16,242,144,18,161,150,101,56,46,
  50,92,35,52,156,250,177,15,62,210,241,36,187,151,210,207,135,212,50,250,238,158,61,120,119,207,30,156,61,123,22,197,115,230,192,96,48,192,102,183,
  195,98,105,19,226,50,9,17,202,220,249,188,240,250,220,72,79,79,71,90,122,42,116,58,173,20,143,169,214,104,160,17,119,207,19,66,224,243,251,225,
  116,186,21,218,190,28,126,191,31,7,14,28,64,87,87,167,144,103,203,227,65,187,168,112,114,28,39,212,109,229,8,90,91,91,81,81,81,9,183,203,
  21,189,130,145,175,134,164,100,247,144,44,168,209,73,184,17,195,34,26,111,14,165,49,133,145,97,232,208,218,218,138,255,62,243,12,206,157,59,135,96,
  32,128,41,147,39,195,31,8,160,185,185,25,62,159,15,42,181,74,168,239,29,12,194,227,233,134,90,165,66,110,110,54,204,9,102,168,84,66,108,176,
  74,165,130,70,163,129,70,35,228,179,229,41,133,203,229,134,199,227,143,27,126,209,220,220,140,51,103,78,195,227,241,192,239,247,195,102,183,75,43,78,
  141,70,131,132,132,4,180,91,44,104,108,106,66,83,99,35,20,37,211,228,73,221,67,193,235,93,93,56,114,228,8,154,154,154,16,12,4,144,150,150,
  134,64,80,216,212,18,8,248,193,113,68,44,222,16,132,215,235,132,74,197,33,35,51,29,38,147,184,107,154,64,202,45,167,209,168,193,169,56,240,60,
  15,183,203,13,175,87,160,35,16,8,12,57,29,16,251,87,48,24,20,7,114,170,56,46,124,124,31,130,51,226,28,211,210,210,194,120,62,14,120,46,
  125,23,195,195,52,94,121,222,212,216,136,199,254,246,55,156,62,117,18,94,143,135,209,49,194,116,32,162,208,77,200,8,165,180,244,245,48,32,77,32,
  217,29,112,63,31,106,203,40,0,204,159,63,31,91,182,108,65,74,106,26,230,22,23,227,208,225,195,168,169,174,65,254,212,26,100,102,102,194,233,116,
  34,16,240,161,120,110,49,46,191,124,59,212,106,21,246,237,123,31,103,207,158,67,48,24,20,18,181,234,132,60,162,132,0,193,64,0,46,167,11,30,
  175,31,102,99,116,45,251,67,135,14,97,255,254,247,164,221,102,13,245,245,240,137,174,40,189,78,135,220,172,108,152,77,102,76,158,50,25,205,77,205,
  194,142,177,208,110,55,170,172,234,37,175,63,75,66,206,248,144,101,20,68,220,49,79,20,121,180,148,134,154,120,97,203,81,203,139,168,210,161,12,131,
  11,147,201,132,203,182,109,195,204,153,51,80,87,95,143,196,196,68,180,91,218,81,86,86,134,197,139,23,75,46,65,175,199,133,207,223,118,27,178,178,
  50,209,220,212,140,93,187,222,69,123,123,7,52,26,13,180,162,133,94,197,113,0,165,240,120,61,232,234,118,33,57,217,20,181,214,179,90,173,216,189,
  123,55,106,107,106,65,41,69,103,103,39,58,172,86,201,226,159,157,145,129,228,196,36,232,180,26,97,16,176,217,196,93,152,242,173,152,97,171,124,72,
  14,117,90,45,138,139,231,32,43,43,11,54,187,29,70,163,17,93,157,93,168,173,169,193,140,153,51,1,42,196,16,250,125,30,108,187,254,122,100,101,
  102,160,163,189,3,31,126,116,16,118,91,151,160,148,104,133,124,115,42,194,129,130,135,215,235,65,119,183,11,41,41,9,224,131,188,98,84,24,42,58,
  66,149,72,132,77,134,162,248,71,230,63,11,245,55,210,123,220,112,212,38,23,42,84,128,97,60,31,227,60,167,52,238,48,26,101,117,26,71,60,79,
  76,48,227,218,79,124,2,179,102,205,66,83,115,51,163,99,132,233,32,146,75,153,40,189,168,132,138,219,71,72,148,9,52,110,138,167,113,46,187,3,
  29,219,7,102,25,141,174,239,20,247,117,221,39,62,129,173,151,93,6,167,195,129,249,243,230,9,3,169,195,1,187,221,14,147,201,4,74,41,230,207,
  95,128,203,47,223,134,201,147,242,144,157,149,133,73,147,243,96,48,26,196,218,169,194,43,148,2,137,167,60,124,62,31,188,30,31,130,60,141,82,68,
  119,239,222,141,246,142,118,169,113,170,107,170,37,171,104,130,217,140,204,140,116,168,84,28,166,79,159,14,131,94,31,222,14,39,91,241,0,52,134,182,
  79,21,171,34,229,59,13,7,140,40,194,33,100,229,66,229,199,196,248,142,246,163,77,199,229,107,192,59,152,250,118,253,172,172,44,220,253,165,47,33,
  53,53,21,153,25,25,200,205,201,65,144,82,180,182,182,194,104,48,72,37,95,111,253,252,231,81,60,103,22,178,51,51,144,147,155,131,204,172,12,112,
  42,149,144,99,86,76,243,69,56,193,221,16,12,4,224,118,187,17,8,40,173,244,29,29,29,120,235,237,183,112,244,232,81,41,63,156,205,102,131,221,
  110,23,66,69,52,26,100,101,100,64,167,213,34,35,35,3,147,39,79,2,20,241,71,84,102,93,87,202,97,130,217,140,213,107,214,194,96,52,32,49,
  33,1,41,201,201,8,82,10,171,205,38,213,35,166,148,226,202,171,174,70,81,65,62,210,82,83,144,145,153,129,244,244,52,112,98,122,50,21,39,188,
  192,65,74,229,225,118,123,224,247,7,165,74,31,67,77,135,48,96,41,143,149,86,208,17,118,132,40,17,137,40,197,23,17,129,36,118,49,198,243,241,
  197,115,249,24,29,253,10,221,103,188,240,60,39,39,23,223,250,206,119,144,154,154,130,140,244,52,70,199,8,211,1,153,178,21,250,45,28,215,140,184,
  114,41,215,5,38,138,236,14,180,159,15,185,101,244,137,199,30,195,239,127,247,59,248,188,94,108,189,124,59,138,231,204,193,233,211,167,209,214,214,6,
  0,88,184,104,17,182,109,219,134,201,147,38,65,165,226,96,179,119,162,178,178,6,30,183,7,42,149,26,26,141,144,104,158,112,226,142,174,32,15,191,
  223,15,143,199,135,64,32,8,149,86,40,103,21,82,68,107,107,235,16,240,11,9,87,221,46,23,156,46,97,151,152,65,111,192,164,156,92,184,157,46,
  28,60,119,30,9,9,9,112,116,119,75,49,31,68,81,162,42,244,63,170,92,156,211,158,99,63,47,5,44,110,116,104,81,85,89,137,175,254,223,255,
  161,162,188,28,203,87,172,196,242,101,203,208,220,210,2,139,197,34,100,116,208,104,112,243,45,183,96,201,226,197,208,106,52,240,249,253,104,109,109,67,
  125,93,163,84,102,86,45,186,57,137,152,158,43,24,12,192,231,245,194,237,246,67,147,32,88,233,27,27,27,177,103,207,30,124,244,241,199,232,118,56,
  132,34,11,62,31,92,110,55,124,126,63,8,33,200,201,202,70,90,106,42,206,159,43,67,89,41,133,70,163,142,181,84,150,125,65,165,128,241,246,246,
  118,188,240,220,115,232,232,232,192,180,25,211,49,173,104,26,108,246,78,116,118,118,193,227,241,128,83,169,112,249,246,43,48,123,214,76,104,52,66,220,
  80,71,135,21,77,77,45,130,187,86,45,188,194,116,64,160,195,231,133,199,227,135,78,60,167,165,165,101,72,233,8,133,194,132,21,203,136,248,42,217,
  10,186,95,29,78,118,120,101,69,5,227,249,120,224,121,63,22,171,227,133,231,23,206,159,199,103,110,186,9,23,47,92,196,166,173,91,24,29,35,76,
  135,100,221,163,241,92,49,151,142,241,34,187,3,237,231,67,158,83,168,181,181,21,78,135,3,46,151,11,94,183,27,43,87,172,68,74,74,10,106,235,
  235,97,183,219,49,99,250,12,100,103,101,67,165,82,193,222,217,137,29,175,238,192,249,178,18,120,189,110,161,210,145,74,5,149,74,40,245,9,34,236,
  62,231,131,65,120,188,62,248,124,65,65,195,127,235,45,188,252,242,43,168,169,173,133,223,239,151,180,239,134,134,6,33,22,65,173,70,90,114,50,114,
  178,179,208,209,222,14,128,162,187,187,187,247,216,120,121,240,39,141,218,23,7,26,39,62,132,97,244,42,164,148,82,156,63,87,134,149,43,86,98,198,
  244,233,240,250,124,56,114,236,40,10,10,10,48,103,246,28,232,180,90,248,253,126,156,59,127,30,175,238,120,5,118,91,187,16,119,167,82,75,47,161,
  106,5,164,228,190,46,183,23,148,82,148,151,151,227,137,191,255,29,31,125,244,145,152,187,141,74,185,33,187,186,132,130,11,38,131,17,121,217,217,8,
  248,253,240,249,132,220,143,110,183,167,231,176,35,170,252,208,222,222,46,197,100,77,159,62,29,121,185,185,194,51,95,56,143,220,156,28,20,228,231,67,
  171,209,192,239,15,160,178,170,10,239,188,179,27,93,157,54,161,82,7,167,150,138,55,132,20,19,26,228,17,240,7,225,118,123,193,83,138,11,23,46,
  14,61,29,17,206,132,126,236,85,81,28,70,17,231,26,148,241,124,60,240,156,246,87,48,198,17,207,47,156,191,0,74,41,206,156,58,197,232,24,5,
  116,244,225,192,222,99,220,39,130,236,14,112,108,31,150,10,76,148,82,124,238,243,159,199,150,173,91,113,250,76,9,58,59,59,177,111,223,94,248,124,
  62,184,221,46,52,53,53,33,49,41,9,143,63,254,24,170,171,171,165,220,91,46,87,23,38,79,158,130,174,46,63,12,122,61,120,158,135,223,239,67,
  155,165,21,229,21,101,104,110,106,68,109,93,29,186,186,58,195,21,9,68,101,208,239,247,227,236,185,115,0,128,212,228,100,20,76,153,130,228,196,36,
  36,46,152,143,179,37,37,112,56,156,76,59,155,128,152,50,117,42,190,113,223,125,224,56,14,54,155,13,110,183,27,29,22,11,50,50,51,113,254,92,
  25,230,206,157,135,83,103,78,227,233,127,253,11,148,231,65,1,184,61,14,240,193,20,16,146,12,2,10,147,209,8,158,231,225,112,58,208,97,237,192,
  145,195,239,163,178,170,6,77,77,13,224,131,188,34,27,3,207,243,104,110,109,69,155,197,2,173,70,131,252,201,147,145,145,158,142,148,148,100,112,132,
  67,133,84,120,161,127,227,89,106,90,26,174,188,242,74,232,244,58,120,60,66,106,143,46,123,39,146,83,82,80,91,83,13,189,126,54,42,170,42,241,
  250,107,175,9,33,204,132,192,231,117,131,242,201,80,113,66,98,120,137,14,151,19,182,78,27,78,28,255,8,213,181,117,104,105,105,10,87,211,24,82,
  58,134,120,225,38,46,188,25,207,199,54,207,233,4,238,231,133,69,69,248,209,131,63,134,90,163,97,116,140,2,58,216,28,53,116,253,92,61,208,49,
  190,63,183,226,41,197,212,252,124,156,58,121,18,59,118,188,138,53,235,214,194,222,105,199,255,158,123,14,239,189,255,62,86,172,88,129,164,196,68,201,
  100,28,58,39,232,243,161,178,178,28,148,167,168,170,56,31,110,192,80,60,2,16,149,15,212,229,116,162,182,182,22,21,149,149,0,0,189,94,143,130,
  41,83,160,226,56,92,184,112,1,179,231,204,134,90,173,142,105,100,142,252,76,227,208,77,122,56,38,114,183,5,233,165,173,228,191,243,19,212,85,79,
  47,225,188,254,158,107,48,24,96,52,26,241,216,95,255,134,172,156,28,108,191,252,114,188,178,99,7,238,186,235,110,108,218,180,17,243,231,205,135,213,
  218,33,5,113,3,66,34,224,118,139,5,150,182,54,69,144,182,124,241,35,151,75,74,41,252,126,63,172,29,29,40,59,119,14,157,93,93,80,171,213,
  200,72,79,199,164,220,92,84,87,85,129,207,159,10,115,130,57,110,91,68,202,24,37,128,57,33,1,60,207,195,231,245,65,171,213,64,171,211,226,189,
  253,251,145,150,158,129,165,75,151,224,227,131,7,241,240,195,191,198,146,165,75,48,115,250,12,49,119,28,132,248,33,10,4,131,65,216,172,29,176,89,
  59,164,155,132,195,159,105,88,22,121,10,162,226,6,149,142,148,228,20,216,237,118,33,243,137,66,250,7,172,99,42,189,61,81,59,8,67,223,16,198,
  243,113,198,243,88,99,167,212,102,148,130,35,99,155,231,114,24,77,38,152,204,102,252,254,183,191,197,228,41,83,25,29,35,76,7,141,33,139,241,230,
  248,158,142,25,255,178,219,255,126,62,108,150,209,159,253,244,167,82,176,111,98,98,2,174,186,226,74,180,182,180,226,252,249,243,216,241,202,14,4,130,
  1,100,101,102,162,104,218,52,164,167,167,75,155,150,40,31,118,137,71,42,162,188,152,107,203,229,114,161,190,174,14,141,141,141,112,136,57,184,0,192,
  160,215,163,40,191,0,57,89,217,40,61,123,22,193,64,0,7,62,248,80,185,211,157,244,162,41,198,42,59,40,125,98,49,158,99,74,233,165,20,101,
  101,101,184,235,139,95,148,130,173,191,252,127,247,192,231,245,225,95,255,126,26,135,15,29,198,7,31,124,0,189,78,135,169,249,249,200,207,207,135,209,
  104,148,74,188,42,83,83,40,59,185,96,181,247,195,210,214,134,250,250,122,180,181,183,195,239,15,128,130,66,163,86,35,51,61,3,69,249,249,112,116,
  119,195,209,221,141,210,146,179,125,175,212,69,128,228,164,100,44,92,180,8,101,101,165,240,122,189,104,106,106,198,63,159,124,18,132,16,84,87,87,227,
  234,107,175,3,207,243,216,253,206,59,40,61,91,138,147,39,79,66,171,213,32,47,47,15,147,242,38,195,96,52,74,101,110,67,185,113,21,41,54,196,
  234,69,60,165,8,248,3,104,111,105,71,195,0,232,48,24,12,112,139,213,60,66,200,202,202,194,212,169,83,113,244,216,209,240,205,132,10,186,209,212,
  19,198,115,198,243,136,73,155,144,152,7,197,11,217,27,203,60,151,127,160,160,56,115,250,52,62,245,201,79,130,16,130,195,7,15,225,219,255,239,123,
  140,142,17,162,35,190,192,146,190,94,104,98,200,238,0,199,118,245,240,104,1,242,21,146,17,159,188,225,70,36,39,39,193,235,245,98,223,254,125,56,
  114,244,40,44,22,11,218,44,22,180,182,89,160,81,171,145,157,157,133,220,188,60,164,164,164,64,171,213,74,177,78,60,21,26,180,75,76,67,208,214,
  214,6,171,213,6,175,223,39,209,170,86,171,145,158,154,138,169,147,167,32,55,43,11,41,41,201,152,49,99,58,206,137,110,251,80,171,144,56,173,68,
  68,6,16,185,194,42,127,197,20,71,194,50,51,141,49,172,93,183,30,203,150,45,69,70,134,176,27,113,247,59,187,81,81,81,1,151,219,141,242,242,
  114,148,95,44,135,217,108,66,126,126,62,82,82,83,97,54,155,5,171,186,104,85,10,6,131,240,122,189,232,232,232,64,123,123,59,90,90,90,224,112,
  58,165,129,134,112,28,18,77,102,228,100,103,99,114,222,36,164,165,36,67,167,211,161,171,171,83,172,243,27,167,95,71,12,0,185,57,185,72,74,74,
  66,123,123,123,88,118,101,178,54,167,184,24,51,103,206,64,122,90,26,52,26,45,142,28,57,130,250,198,6,120,60,94,212,214,212,162,182,166,14,70,
  163,17,147,38,79,66,114,114,50,76,102,179,84,115,152,18,72,150,55,171,181,3,29,29,29,104,109,107,27,48,29,185,121,121,168,172,172,144,158,49,
  63,191,0,62,159,15,46,151,75,236,62,225,133,28,33,156,72,8,137,238,83,164,15,35,152,84,35,178,135,227,40,227,249,80,243,60,47,47,15,21,
  67,200,243,190,13,171,100,204,243,92,209,86,36,218,209,182,105,203,22,172,88,190,28,89,153,89,140,142,145,160,131,40,68,183,111,130,73,38,160,236,
  14,176,159,171,102,207,154,245,96,95,39,239,204,204,76,108,218,188,25,239,238,217,131,214,214,214,190,187,233,197,4,245,70,147,9,223,255,193,15,48,
  101,202,20,252,251,233,127,225,232,145,35,216,180,105,19,140,6,35,60,30,15,108,157,157,240,249,188,224,249,32,156,46,23,90,91,90,16,12,6,16,
  12,4,160,82,169,16,228,121,56,29,14,180,180,180,160,178,162,2,117,13,13,232,236,238,70,144,23,82,147,112,28,7,147,209,136,236,204,44,204,156,
  62,13,58,173,6,21,23,203,161,211,235,144,147,155,11,189,193,128,118,75,123,12,163,39,81,52,174,164,140,114,4,28,39,52,172,240,34,66,157,214,
  208,119,28,17,191,19,126,11,189,16,250,12,40,254,14,41,173,82,74,50,66,194,172,146,58,237,196,211,104,179,178,178,112,217,182,109,216,191,111,159,
  148,101,161,55,108,222,188,25,148,82,188,187,103,79,191,172,100,33,179,208,250,245,235,241,197,187,238,70,109,93,29,126,253,171,95,97,230,244,25,152,
  61,123,54,124,62,63,236,157,157,112,185,220,240,7,252,240,7,2,176,118,88,209,221,213,133,96,80,200,210,192,113,28,252,62,31,58,172,86,52,52,
  52,160,252,226,69,180,88,44,194,78,68,145,135,26,141,6,41,201,201,40,202,47,64,94,78,14,44,173,173,104,109,107,69,110,110,46,114,114,115,209,
  97,181,194,231,243,71,200,34,137,234,232,132,0,14,135,19,86,155,21,201,73,73,240,202,42,237,16,194,161,120,110,49,174,184,226,74,116,180,183,227,
  249,231,159,195,212,41,83,81,80,88,128,96,48,136,238,110,7,220,94,47,2,193,0,2,193,0,236,54,27,156,14,7,120,62,8,74,1,78,197,73,
  249,227,26,27,27,81,81,81,129,182,246,118,41,47,111,127,232,48,155,205,208,105,181,72,78,78,134,215,235,133,90,173,66,32,16,128,221,222,137,238,
  238,110,164,165,165,161,163,163,67,232,51,28,135,164,164,36,20,207,45,70,69,121,5,28,78,71,68,31,11,245,39,241,111,177,159,93,182,109,27,64,
  41,118,239,218,37,246,147,80,31,11,207,118,225,126,38,124,199,120,62,116,60,55,24,12,48,232,245,72,18,121,174,211,233,224,243,249,6,149,231,171,
  86,175,6,0,124,116,224,128,196,103,197,216,25,49,174,142,229,126,46,205,41,92,168,93,4,122,56,194,97,203,101,91,241,181,111,124,3,213,213,213,
  248,201,143,126,132,89,51,103,162,184,184,152,209,49,172,116,200,116,0,185,220,114,68,49,255,71,233,2,33,253,98,130,200,238,64,250,57,8,25,38,
  101,148,15,194,104,52,226,135,15,60,128,41,83,166,224,239,143,63,142,3,7,62,132,211,233,68,249,197,139,88,48,127,62,102,207,153,3,143,215,139,
  128,63,0,95,32,0,143,87,216,253,229,236,118,162,165,165,5,129,64,0,93,93,93,168,175,175,71,77,109,45,186,186,187,37,119,124,168,178,82,90,
  74,10,166,229,23,96,90,65,1,186,236,157,104,168,171,3,229,121,88,44,237,48,24,244,200,205,205,133,193,96,64,187,197,18,165,161,147,136,207,97,
  65,19,220,92,156,172,33,185,40,161,131,100,38,39,242,245,129,66,32,33,9,38,83,70,71,72,25,21,227,139,215,175,95,143,187,191,244,37,212,214,
  214,224,87,191,248,37,220,78,23,42,202,203,1,74,177,105,243,38,97,82,245,251,16,8,4,224,245,137,239,94,31,108,86,43,186,58,59,225,247,249,
  208,218,218,138,234,234,106,52,183,180,192,43,159,200,181,90,36,154,19,144,155,147,131,89,211,166,35,41,33,1,53,85,85,112,58,157,240,249,4,75,
  84,110,110,46,114,115,115,97,237,176,194,47,86,187,128,100,141,7,180,58,29,140,70,35,244,122,157,20,23,196,17,14,105,233,105,240,122,188,112,139,
  138,73,113,241,28,108,223,190,29,237,237,29,120,225,249,23,224,245,122,209,216,80,15,2,130,229,203,151,67,167,211,33,16,8,32,16,12,194,239,15,
  32,16,8,8,202,130,205,142,238,174,110,113,51,160,5,181,181,181,104,105,109,149,20,18,12,128,142,4,179,89,42,181,203,243,60,84,28,7,183,219,
  45,209,148,145,153,137,142,142,14,112,68,168,188,150,148,148,132,185,115,231,162,188,162,28,142,238,110,97,80,226,228,131,59,39,13,84,161,87,76,101,
  84,214,151,98,42,163,194,8,207,120,62,4,60,215,235,116,48,24,12,18,207,117,90,173,104,77,25,60,158,135,149,209,15,99,140,165,242,9,29,202,
  228,226,24,253,60,87,126,38,98,85,66,161,173,56,46,108,101,186,236,242,109,248,198,55,191,137,234,170,42,252,248,135,63,132,211,233,194,197,11,231,
  133,154,230,91,183,50,58,134,137,14,185,18,26,91,15,136,161,140,202,244,137,137,34,187,3,233,231,32,228,18,220,244,253,76,97,20,18,201,191,63,
  254,56,14,124,248,129,100,142,246,122,189,120,103,215,110,228,229,229,225,202,109,151,163,165,181,21,165,231,202,80,87,223,0,171,181,3,30,175,23,28,
  56,212,213,55,192,231,243,34,200,243,32,132,64,171,213,66,165,82,67,167,213,194,104,52,96,82,78,14,242,114,115,209,221,213,133,138,242,114,120,100,
  49,76,4,192,185,178,115,32,132,131,163,219,17,195,193,30,253,156,82,67,201,27,63,34,14,66,25,201,68,227,95,83,172,194,64,66,9,75,67,69,
  181,41,141,142,102,102,169,161,134,80,14,195,171,206,186,186,58,60,244,139,95,192,229,114,34,20,150,94,81,94,129,186,186,58,204,95,176,0,159,253,
  244,45,40,59,87,134,11,229,229,104,110,110,134,203,229,18,44,79,14,39,172,54,155,152,66,12,98,9,54,45,52,106,53,244,6,61,210,82,82,48,
  41,55,15,137,102,51,154,155,154,80,213,214,38,221,147,0,112,116,59,112,242,248,9,204,152,57,3,30,183,59,102,31,209,233,180,72,72,72,0,33,
  4,126,191,31,126,49,247,155,52,16,74,139,38,130,246,246,118,188,244,226,139,240,122,189,146,212,213,215,215,161,181,181,5,51,103,205,198,181,87,93,
  141,202,170,74,84,213,212,192,98,177,192,237,246,32,24,12,194,225,116,194,222,105,151,232,224,56,14,58,157,78,160,67,175,71,90,106,106,191,232,112,
  136,169,210,18,18,19,209,210,220,28,229,133,17,210,10,65,90,188,113,98,249,58,34,5,24,197,119,235,197,99,37,33,209,145,87,180,7,25,97,60,
  31,123,60,15,133,103,41,195,50,136,162,223,75,147,57,165,99,170,159,71,186,61,229,109,21,233,220,172,174,174,198,3,223,255,62,156,98,232,3,32,
  228,240,172,173,173,197,226,197,75,24,29,195,65,7,145,235,7,145,99,81,140,235,200,199,168,9,36,187,3,234,231,0,200,245,215,93,215,231,217,124,
  238,220,185,248,233,207,127,142,251,191,243,29,156,57,125,186,207,42,64,32,16,80,4,210,134,106,181,70,86,55,162,98,197,134,130,162,34,100,100,100,
  160,181,173,13,150,246,118,116,118,10,241,11,126,191,31,193,96,0,132,112,208,233,180,208,105,180,48,25,13,0,5,58,218,219,209,222,110,129,71,204,
  135,69,37,198,246,28,108,171,220,203,36,118,29,142,128,227,84,82,240,63,199,137,22,82,81,179,143,54,205,115,49,45,165,114,119,125,212,106,40,226,
  93,26,192,57,110,194,233,147,243,23,44,192,195,191,249,13,126,244,195,31,226,236,217,179,125,58,231,103,63,255,57,130,60,143,251,191,253,237,126,88,
  232,133,218,190,242,221,196,225,202,26,178,160,111,158,66,163,213,96,250,244,25,200,47,200,135,63,16,68,107,107,43,172,54,43,186,186,186,225,113,11,
  238,17,74,133,142,174,215,233,160,215,106,145,152,152,8,155,213,138,142,246,118,88,173,29,17,226,215,147,44,146,168,50,178,138,21,167,248,154,49,115,
  6,236,54,187,112,109,209,101,27,118,255,144,40,57,6,132,250,194,249,133,5,152,60,121,10,130,60,143,246,142,14,216,59,59,225,116,58,224,241,120,
  16,12,4,164,1,75,167,211,193,160,213,33,49,33,1,118,187,29,214,142,142,126,211,145,150,150,6,107,71,71,20,29,243,23,44,64,89,105,169,212,
  167,38,79,153,130,155,111,185,5,175,189,186,19,77,77,77,226,234,185,103,215,215,195,191,253,45,40,207,227,155,223,248,134,162,95,41,44,12,17,241,
  149,2,63,121,198,243,49,202,243,111,138,253,251,87,191,252,101,212,152,25,107,92,29,15,60,151,218,69,154,236,57,49,94,49,194,179,38,219,32,164,
  211,233,48,115,214,108,20,22,21,33,16,100,116,12,5,29,28,225,20,50,27,203,194,167,152,247,229,115,255,4,146,221,129,244,115,114,73,150,209,254,
  196,233,33,34,13,128,184,18,8,27,4,195,191,54,53,53,162,177,177,65,44,143,149,141,132,164,68,36,154,76,152,148,147,3,66,8,2,129,0,218,
  219,45,240,122,60,232,236,236,68,83,67,61,220,46,183,164,224,134,46,21,98,57,37,189,132,12,71,124,136,27,3,42,117,30,18,225,222,39,225,249,
  80,122,145,136,233,65,185,36,32,151,18,235,204,112,105,178,24,177,50,141,52,77,135,22,39,126,159,15,165,103,75,80,114,230,12,76,38,19,146,147,
  147,145,144,152,0,99,70,6,18,19,19,225,247,251,225,118,187,208,222,222,14,159,215,7,75,107,43,46,158,23,92,78,242,203,18,169,102,2,137,201,
  96,18,175,211,139,43,76,249,2,199,102,181,193,235,245,200,150,163,232,65,54,5,248,253,126,148,95,184,128,139,231,47,192,104,50,34,41,41,9,230,
  132,68,152,244,105,72,72,72,68,48,24,128,219,237,129,205,218,1,159,207,15,75,91,43,202,47,92,80,182,81,63,232,136,86,74,4,58,218,45,22,
  101,72,11,225,160,86,137,149,213,34,99,62,229,174,8,217,206,129,232,20,42,177,119,181,18,198,243,241,195,115,185,101,52,170,237,72,124,111,201,24,
  230,185,52,239,208,176,201,141,196,8,241,146,155,84,124,62,31,74,78,159,198,153,211,167,96,54,155,145,156,146,138,196,196,68,100,165,167,35,49,49,
  81,72,124,238,114,161,163,221,2,159,207,135,182,214,22,92,56,119,110,192,60,159,136,116,68,202,44,137,147,202,72,225,49,37,100,2,202,110,255,251,
  57,48,12,187,233,227,197,64,74,77,75,194,138,168,188,104,20,165,64,75,75,51,154,155,155,196,88,63,94,168,119,42,190,243,148,7,229,133,207,132,
  8,117,88,9,13,53,104,184,106,18,233,45,110,32,50,198,147,11,5,226,198,11,72,14,237,156,143,222,61,79,16,103,163,88,111,10,49,195,176,32,
  148,218,38,42,52,66,182,128,161,49,92,16,78,167,3,14,177,90,133,32,119,74,249,19,100,146,151,22,46,66,209,6,49,94,81,148,69,210,91,178,
  217,8,75,61,145,197,43,135,58,184,213,102,149,226,152,67,71,71,43,37,68,17,58,66,73,152,64,151,203,5,167,211,9,208,38,80,202,131,138,217,
  41,40,207,139,101,220,194,116,132,234,10,15,22,29,22,75,155,20,200,14,49,6,75,165,82,129,19,255,14,245,41,249,64,79,34,188,60,3,233,55,
  140,231,99,155,231,68,28,207,39,18,207,195,139,141,8,239,25,81,90,214,40,228,159,169,164,88,56,186,29,66,133,65,49,109,87,232,61,52,143,134,
  210,252,12,5,207,199,59,29,114,153,13,27,163,100,74,87,140,1,107,162,201,238,128,198,246,225,176,140,42,90,140,198,79,23,27,149,20,54,68,177,
  20,114,33,16,17,122,231,64,192,19,14,28,199,3,32,224,121,14,20,188,216,160,202,70,238,169,117,165,182,80,4,214,114,202,93,243,145,43,61,162,
  20,68,165,105,84,185,42,34,253,89,21,49,12,143,40,246,106,165,87,198,7,9,98,75,35,38,71,2,16,14,132,227,1,62,36,59,60,120,112,128,
  56,24,200,7,218,222,100,81,145,166,141,132,194,67,226,5,123,43,22,169,97,229,132,144,216,215,13,245,161,208,96,70,194,139,191,16,29,210,130,80,
  164,131,82,78,114,7,13,13,29,28,84,106,149,180,232,147,47,242,226,89,27,194,234,150,104,37,37,225,164,247,36,234,217,8,227,249,120,224,57,33,
  189,123,151,122,49,120,140,89,158,203,207,34,178,48,50,133,139,91,118,12,165,160,178,57,39,170,56,4,1,132,10,224,252,48,242,124,252,208,161,216,
  71,18,199,24,21,243,218,19,78,118,251,223,207,201,128,45,163,116,16,52,129,208,242,135,42,87,71,209,199,135,180,246,72,107,0,7,66,120,0,28,
  4,163,1,21,107,177,82,201,74,42,41,165,61,42,250,36,42,101,19,199,69,196,129,202,18,140,197,18,68,133,53,52,42,65,115,79,73,201,152,82,
  122,73,242,116,201,123,189,34,22,70,225,114,62,33,209,140,90,24,133,172,249,161,184,244,80,135,231,56,30,60,31,126,7,39,172,70,21,46,144,158,
  23,156,130,108,197,148,65,46,106,112,167,10,11,131,50,27,68,84,35,145,80,255,145,7,211,139,202,28,149,141,6,242,231,15,213,26,25,34,58,56,
  142,64,173,86,11,241,216,114,175,67,156,216,235,216,29,142,241,124,188,243,156,68,238,28,25,208,176,48,150,121,78,101,247,39,209,41,118,228,52,75,
  26,76,216,74,24,123,10,18,21,56,158,3,5,5,63,132,60,31,79,116,72,186,7,33,209,174,102,89,166,205,72,218,38,154,236,14,116,108,31,120,
  57,208,254,110,98,142,243,30,86,24,123,81,27,37,229,157,132,13,166,210,104,204,129,227,40,120,30,0,23,218,32,21,98,24,85,172,174,16,161,132,
  70,189,184,24,74,168,220,212,220,131,32,42,6,124,16,201,16,76,168,88,42,139,70,174,120,160,168,98,72,7,69,193,154,56,186,40,48,208,205,244,
  68,182,137,46,66,30,123,90,196,136,99,1,165,36,134,60,202,23,71,161,13,121,68,10,56,39,84,17,38,30,109,61,151,201,160,188,19,19,142,147,
  109,132,147,185,188,40,141,8,12,33,49,214,123,36,118,237,182,208,159,52,98,124,147,209,17,174,214,51,52,116,16,142,131,90,173,142,176,52,244,50,
  24,70,9,12,13,135,249,196,74,147,162,144,17,198,243,177,200,243,80,204,40,165,177,172,69,177,203,44,142,175,126,46,173,64,162,44,105,74,158,135,
  45,223,114,207,129,188,191,16,69,110,117,145,14,66,192,81,70,71,159,232,136,83,9,12,17,94,27,128,72,115,126,72,15,152,72,178,59,160,177,125,
  184,220,244,82,28,132,152,222,136,198,72,111,64,227,153,177,99,185,251,37,75,61,39,92,71,150,111,52,86,25,172,200,107,42,227,23,122,120,113,145,
  43,35,196,109,88,249,206,94,18,113,159,216,110,167,248,180,49,12,93,168,72,204,26,193,114,185,140,88,192,196,147,103,9,28,7,240,60,56,0,52,
  66,254,250,42,139,49,229,145,139,145,207,142,11,167,249,161,145,131,68,140,1,30,178,1,62,170,182,55,33,209,238,34,57,29,98,59,12,21,29,28,
  225,164,213,179,50,14,59,122,103,42,98,249,26,72,223,10,161,16,194,129,32,200,120,62,70,121,78,122,88,100,196,27,51,9,199,129,4,199,7,207,
  249,208,241,125,229,185,76,86,66,41,5,99,33,68,7,1,24,29,125,165,131,244,156,220,190,71,255,231,4,146,221,129,142,237,3,84,70,251,111,203,
  138,84,2,190,250,181,175,161,176,176,80,170,47,239,116,58,241,225,7,31,224,221,119,222,81,152,234,111,253,252,231,80,83,93,141,15,63,248,0,0,
  112,239,125,247,9,73,157,3,1,4,252,126,233,243,211,79,61,133,226,185,115,49,123,246,108,4,197,107,242,188,16,180,187,115,199,142,8,254,71,54,
  108,164,165,148,139,221,112,242,206,163,56,55,222,164,72,99,238,118,149,31,172,215,235,112,245,181,215,97,214,172,89,224,56,14,245,117,117,216,189,107,
  23,236,118,59,190,244,149,47,99,231,142,87,81,83,83,35,157,118,219,237,183,227,252,249,243,56,124,232,16,110,187,253,118,216,237,118,236,124,245,85,
  197,239,239,191,247,158,116,206,134,141,27,177,112,225,66,36,167,164,192,110,179,97,215,174,93,184,112,254,60,243,211,19,224,107,95,251,58,10,167,77,
  147,82,62,133,100,241,177,191,254,21,60,207,99,211,230,205,120,242,239,127,151,78,153,50,101,10,86,173,89,131,103,255,251,95,76,154,52,9,215,93,
  127,61,254,245,207,127,162,171,171,19,160,64,118,86,54,230,205,155,143,221,187,118,33,51,51,19,235,55,110,84,200,33,229,121,124,116,224,128,88,226,
  81,86,233,43,34,6,57,246,194,72,22,248,29,17,64,40,151,233,181,235,214,97,237,186,117,225,214,161,20,117,181,181,120,246,127,255,195,77,183,220,
  130,143,15,28,64,93,93,29,110,184,241,70,216,237,118,236,121,231,29,169,45,175,188,230,26,28,63,118,12,205,205,77,216,182,237,114,164,167,167,35,
  16,8,32,24,12,34,24,12,162,173,181,85,236,135,84,185,121,117,0,116,112,28,7,149,74,45,245,181,216,30,7,68,165,111,138,57,152,43,92,248,
  132,241,124,28,241,156,16,229,204,209,147,247,147,196,249,114,254,188,249,184,252,138,43,144,147,155,11,167,195,129,146,51,103,240,234,142,87,132,44,44,
  4,88,188,120,9,214,111,216,128,132,132,4,56,157,78,28,58,120,16,31,188,255,190,116,137,194,162,66,44,89,178,20,73,73,73,112,185,92,40,57,
  115,6,39,142,31,135,82,127,10,111,60,41,44,42,130,90,173,150,198,217,75,225,57,1,149,62,47,92,188,24,215,92,123,45,114,39,77,130,179,187,
  27,167,78,157,194,43,47,189,4,183,219,5,10,96,201,210,101,216,180,121,51,18,18,18,224,112,56,112,240,227,143,241,193,251,239,73,238,227,130,105,
  211,176,100,233,82,36,139,116,156,57,125,26,39,79,156,136,162,35,41,41,9,203,86,172,128,65,175,71,123,123,59,142,28,62,44,230,189,29,28,58,
  34,221,243,148,0,215,127,242,6,44,93,186,20,26,173,22,29,237,237,120,109,231,78,156,43,45,5,8,193,247,127,244,35,252,226,167,63,145,232,160,
  4,248,191,123,238,197,95,30,121,4,0,176,253,202,43,145,145,145,33,233,2,173,45,45,56,124,248,16,156,14,167,36,231,151,42,187,177,94,32,49,
  100,79,158,147,148,196,151,79,2,32,61,35,29,63,255,229,67,248,242,93,119,41,126,155,52,121,18,238,250,242,87,240,163,31,124,31,70,189,1,159,
  184,225,6,20,23,23,67,167,211,161,161,190,30,187,119,237,194,217,146,18,128,0,5,5,133,216,176,113,35,158,122,242,201,168,225,238,235,247,221,135,
  63,254,254,247,98,232,141,96,141,189,253,246,59,165,99,111,190,229,22,116,119,117,225,205,55,222,144,100,119,209,226,69,152,52,121,10,62,22,199,172,
  181,235,214,33,35,51,83,236,187,42,168,84,42,116,116,116,96,255,222,189,131,219,207,113,73,150,209,190,43,2,225,29,98,161,88,38,97,7,252,95,
  255,242,103,84,92,44,7,5,69,74,114,10,238,253,198,55,112,252,216,81,216,172,86,128,2,201,41,41,240,251,252,152,57,107,54,14,124,248,33,64,
  129,63,63,242,39,80,158,98,233,242,101,224,121,30,71,15,31,17,76,204,98,156,194,197,139,23,112,250,212,105,153,0,82,168,84,92,204,153,73,158,
  175,47,230,174,121,18,95,16,137,44,141,147,82,16,169,44,214,41,108,240,38,50,151,162,124,239,224,214,109,219,224,245,184,241,139,159,254,4,132,227,
  176,98,229,74,220,244,233,79,227,145,63,254,1,59,119,236,192,182,237,87,224,137,199,254,6,0,152,57,107,22,0,224,240,161,131,18,21,11,22,46,
  196,233,83,39,21,10,107,200,150,127,249,246,43,144,156,156,140,167,255,245,20,236,118,59,242,243,243,113,211,167,111,193,19,143,253,13,118,187,125,194,
  57,249,195,155,57,5,171,209,163,127,250,35,40,128,105,211,166,97,219,229,219,241,151,71,31,145,114,185,21,20,20,40,119,127,66,216,228,166,10,37,
  242,22,229,109,219,229,151,227,229,151,94,20,23,160,42,104,52,26,97,55,161,90,133,206,78,187,176,184,138,136,191,80,202,35,81,12,142,81,121,234,
  226,188,40,31,97,33,12,149,220,227,8,62,254,232,0,62,252,224,195,168,246,81,201,22,80,156,74,133,185,243,230,225,252,185,50,52,212,55,8,43,
  83,181,10,42,21,39,149,138,123,111,223,62,52,55,55,203,114,221,65,172,168,66,226,247,167,190,210,193,17,168,213,42,89,224,123,172,254,69,162,173,
  121,189,88,69,35,45,10,140,231,99,155,231,68,145,48,155,42,163,11,73,204,189,197,10,158,79,155,49,3,183,126,254,243,120,226,241,199,80,113,241,
  34,244,6,3,174,189,238,19,184,253,206,59,241,183,191,252,5,169,201,41,184,234,234,171,241,143,39,158,64,83,115,19,146,146,146,112,195,141,159,130,
  221,110,199,153,211,167,48,109,218,116,44,95,177,2,251,247,237,67,67,67,61,204,102,51,182,108,189,12,14,167,3,229,23,46,8,74,40,168,104,81,
  38,152,49,115,6,230,47,88,136,178,179,103,7,133,231,132,23,218,126,230,236,217,248,194,93,119,225,207,143,60,130,139,23,206,195,96,52,226,134,27,
  111,196,23,239,190,27,143,254,233,79,72,77,77,193,181,215,93,135,199,255,246,87,52,53,53,33,57,57,25,55,222,116,51,58,59,237,56,125,234,20,
  166,77,159,142,229,43,87,97,255,222,189,18,29,91,47,219,6,167,203,25,69,199,250,141,27,113,246,204,25,212,212,84,99,254,130,133,88,188,100,9,
  142,29,61,50,40,116,72,115,160,44,16,113,229,202,149,152,49,99,6,126,241,179,159,193,237,118,161,176,176,8,159,253,220,231,240,232,159,254,8,155,
  205,6,149,104,129,147,243,95,163,209,8,202,15,21,62,239,125,247,93,52,55,53,130,82,96,238,188,185,88,182,108,185,184,160,160,131,35,187,17,85,
  149,148,165,136,149,99,146,82,73,165,138,140,65,114,217,109,183,88,208,110,177,96,201,210,37,56,113,236,152,244,44,43,87,173,70,201,233,83,0,128,
  187,190,244,37,180,183,183,227,231,63,249,9,92,110,23,138,138,138,240,249,219,239,128,203,237,66,85,69,133,164,252,197,178,121,9,174,114,229,3,105,
  180,90,112,156,16,58,160,211,233,224,21,139,109,8,149,211,40,242,242,38,129,15,6,161,82,171,161,82,9,69,49,14,125,252,49,44,150,54,69,91,
  245,84,2,117,64,99,59,17,172,220,195,224,30,69,12,211,175,114,96,181,217,109,176,219,237,72,77,77,149,30,116,213,234,213,40,57,115,6,245,245,
  117,152,49,115,86,88,113,20,39,8,85,40,151,149,152,136,94,173,86,67,173,214,132,147,212,115,225,132,172,68,250,46,222,139,132,83,58,137,215,227,
  226,88,74,21,155,153,34,221,95,164,151,221,160,17,199,232,116,122,180,182,180,74,223,29,62,116,8,175,189,42,88,114,107,106,106,208,210,210,140,13,
  27,55,66,175,215,227,218,235,62,129,157,175,238,144,197,194,81,188,255,222,126,92,123,221,39,20,223,81,0,122,189,30,203,87,172,192,206,87,119,72,
  138,103,77,77,13,118,190,186,3,122,189,126,2,123,234,229,166,108,25,255,56,229,138,77,8,208,230,20,85,68,56,194,73,3,35,199,113,104,105,105,
  134,74,165,66,113,241,92,225,55,21,7,181,70,3,34,174,34,181,26,109,148,204,197,146,67,18,249,27,225,162,114,221,246,230,34,10,47,168,194,245,
  160,149,131,44,64,84,225,29,142,42,149,10,71,143,28,198,229,87,92,25,78,58,44,246,29,142,227,4,5,75,165,18,203,224,113,131,78,7,199,133,
  219,42,62,77,136,40,163,43,75,96,36,150,250,12,197,84,81,121,5,51,197,23,140,231,99,153,231,10,23,97,111,86,209,24,253,252,198,79,221,132,
  255,62,243,31,84,150,151,3,132,192,237,118,227,249,103,159,133,209,96,196,220,185,243,160,55,24,133,114,169,118,59,8,128,78,123,39,94,126,241,69,
  212,215,213,129,16,130,181,235,215,227,200,145,195,104,106,106,4,33,28,156,14,39,14,124,248,1,130,129,128,172,84,180,64,123,78,110,14,18,147,146,
  80,126,241,130,68,231,165,242,60,244,251,103,110,189,21,79,61,249,15,92,188,120,1,132,16,120,220,110,252,247,63,255,129,209,104,196,252,5,243,97,
  52,26,225,13,209,65,8,58,237,157,120,233,133,23,80,87,43,208,177,110,253,6,28,57,124,8,77,77,141,224,8,17,232,248,32,54,29,231,207,157,
  67,93,93,45,56,142,67,91,107,43,82,82,82,6,141,142,104,197,15,48,26,141,176,217,108,240,120,220,32,132,160,170,186,10,79,254,253,9,120,196,
  10,99,42,181,58,108,37,151,228,86,45,149,231,212,104,4,197,41,68,71,119,119,183,184,64,28,252,62,168,204,45,46,235,135,242,113,42,110,136,94,
  180,172,238,222,245,54,214,173,91,175,56,118,237,186,117,216,191,119,31,230,47,88,8,163,209,136,255,61,243,31,184,197,138,146,149,21,149,120,225,185,
  231,112,221,117,159,144,158,133,227,148,21,218,136,172,141,228,222,94,142,112,208,134,250,159,88,125,173,165,165,21,249,5,249,224,56,14,105,105,233,82,
  46,119,181,74,40,250,163,213,106,197,157,241,67,60,182,3,195,23,51,170,216,177,35,62,192,138,149,43,81,52,109,58,40,229,145,147,147,11,142,227,
  80,85,85,37,53,236,156,185,115,177,235,237,183,96,179,219,176,126,253,6,92,188,112,65,114,223,11,214,10,42,14,194,66,204,131,90,173,198,244,25,
  51,48,105,210,36,49,31,25,143,142,142,14,28,62,124,40,198,254,118,18,149,34,69,241,89,158,91,140,32,198,36,128,168,74,4,253,21,68,0,56,
  240,193,251,184,241,166,155,49,127,225,66,52,212,215,163,182,182,70,225,70,127,103,215,46,220,251,245,111,96,234,212,124,28,57,124,40,202,162,89,83,
  83,131,164,164,100,108,216,184,17,239,191,247,158,244,125,86,118,54,106,107,106,224,17,107,90,135,48,250,92,244,195,173,141,34,118,240,120,200,26,78,
  197,248,28,105,197,73,164,250,32,132,16,177,196,161,168,164,168,212,216,253,246,219,248,204,173,159,67,85,85,101,216,74,70,56,112,156,10,217,57,57,
  184,230,154,107,5,89,20,21,164,183,223,126,43,198,36,26,71,6,17,123,39,34,64,164,112,105,162,220,237,1,142,227,176,96,225,66,20,20,22,74,
  202,217,191,159,126,26,32,16,115,191,137,138,9,199,161,177,177,17,73,73,201,88,189,118,13,62,254,232,35,49,206,71,37,14,240,26,172,219,176,30,
  30,183,71,114,55,95,188,120,17,23,47,94,232,65,25,232,59,29,156,24,228,30,85,96,2,209,49,130,36,78,48,86,116,221,165,56,202,9,227,249,
  152,229,57,33,36,210,73,31,63,28,35,50,227,2,5,38,79,158,172,168,22,24,154,134,74,75,207,34,111,82,30,74,207,150,224,220,185,115,248,206,
  247,190,135,202,138,10,84,85,86,160,172,172,12,118,171,160,212,229,230,230,162,178,162,66,82,64,40,5,172,86,43,58,58,58,192,17,14,148,136,249,
  173,41,133,165,205,130,182,182,54,44,92,184,72,218,195,112,169,60,23,50,112,81,76,157,58,21,167,78,158,140,152,187,128,146,146,18,76,154,52,25,
  37,37,37,40,43,45,197,247,126,240,67,84,148,151,163,178,162,2,101,101,165,176,91,109,2,29,121,121,18,29,161,75,88,173,86,116,88,163,233,168,
  169,174,150,242,218,46,89,182,20,21,229,229,178,60,183,151,70,135,210,211,33,28,123,228,240,33,204,95,176,0,63,254,201,79,113,241,226,69,84,148,
  95,196,217,146,18,120,61,30,73,110,35,227,46,67,138,22,8,160,86,107,176,126,195,70,184,221,110,240,60,143,172,172,44,188,252,210,75,194,30,18,
  208,65,145,93,101,157,249,8,203,94,132,135,148,244,195,24,117,252,216,49,220,245,165,47,195,104,50,193,229,114,97,241,146,37,176,219,237,232,232,232,
  192,138,85,171,112,166,228,76,180,236,158,45,193,109,119,220,33,179,82,170,36,203,49,165,225,10,116,82,27,133,67,113,161,209,106,37,26,181,58,29,
  218,45,109,152,54,125,6,206,159,59,143,188,73,147,96,181,90,145,154,154,42,41,160,90,157,22,171,215,172,133,223,239,151,140,118,23,47,94,196,197,
  11,23,6,183,159,15,151,50,42,119,213,135,75,179,135,87,25,115,231,45,64,71,123,59,30,255,219,95,165,52,79,75,150,46,134,207,235,197,166,45,
  91,64,121,138,130,194,66,164,164,36,195,102,179,139,147,134,160,140,134,76,206,16,87,80,109,109,109,56,87,86,38,185,233,189,62,175,144,124,53,166,
  9,189,119,133,52,202,140,15,68,184,247,7,46,136,0,208,214,218,138,191,254,249,81,100,101,101,163,160,176,0,43,86,172,196,250,13,27,241,247,199,
  31,3,0,120,60,30,188,179,123,23,214,111,216,136,247,159,126,47,102,219,190,179,91,80,88,79,159,58,197,54,41,245,69,14,21,233,13,8,194,118,
  238,112,160,184,188,148,107,40,152,158,112,68,178,146,17,142,64,173,82,161,171,171,27,167,79,157,194,230,45,91,80,82,82,34,12,0,162,229,222,227,
  241,224,212,169,83,225,36,198,160,97,89,140,33,135,82,44,114,148,60,202,100,80,145,175,77,76,92,44,151,59,142,67,93,93,29,78,157,58,41,165,
  254,8,185,106,4,171,31,39,89,201,56,194,225,189,253,251,113,247,151,191,140,178,210,50,168,196,160,243,80,106,142,138,242,114,88,218,218,196,0,119,
  160,187,187,43,60,193,82,92,18,29,210,51,112,17,3,85,212,64,31,89,205,12,125,222,232,39,15,186,103,60,31,155,60,239,139,85,180,39,158,19,
  89,92,113,104,3,72,40,76,44,244,253,27,175,191,134,119,247,188,131,252,130,2,228,231,231,227,43,247,124,21,59,94,126,25,231,207,149,137,70,15,
  18,181,235,65,224,1,149,188,175,132,134,247,127,171,53,154,48,223,7,133,231,113,54,206,18,229,252,243,250,206,87,177,231,157,221,40,44,40,68,126,
  65,1,238,185,247,107,120,229,165,23,113,174,76,78,135,188,33,169,240,220,113,232,216,176,113,19,60,30,47,170,42,171,34,20,235,129,211,161,76,73,
  38,192,237,241,224,209,63,253,17,169,105,105,40,42,154,134,162,105,211,113,205,117,159,192,31,127,247,91,88,173,182,136,103,23,149,81,141,70,72,178,
  46,42,166,229,23,47,162,173,173,21,148,82,76,205,47,192,156,226,98,124,116,224,195,240,200,113,137,178,43,183,136,70,91,67,163,99,216,227,45,166,
  35,101,215,229,114,225,192,7,31,96,237,250,245,216,179,123,55,214,174,91,143,125,123,247,42,75,165,70,200,46,21,21,190,80,159,225,56,162,88,64,
  135,206,140,37,187,161,133,51,37,20,90,173,22,45,45,173,184,226,170,171,177,247,221,61,40,154,54,13,37,37,103,144,149,157,45,246,83,14,26,141,
  22,85,85,149,176,118,116,72,58,144,163,187,91,146,237,65,27,219,9,193,240,22,66,87,112,133,224,232,225,195,216,245,246,91,248,235,163,143,96,250,
  244,233,146,139,158,16,96,233,178,229,168,169,169,145,180,235,186,186,58,44,92,188,88,70,44,39,197,58,73,185,173,84,42,184,221,46,180,180,52,163,
  165,165,5,173,173,45,176,89,173,10,247,123,200,194,193,113,202,58,243,146,22,31,153,99,52,194,82,26,89,30,52,214,238,214,190,10,34,1,240,153,
  207,125,14,122,189,30,173,173,45,56,124,232,16,254,253,244,191,132,149,112,126,190,116,156,221,110,135,221,110,139,17,45,73,195,10,235,174,93,184,230,
  186,235,164,239,106,107,106,48,101,234,212,40,151,252,204,153,179,176,126,195,198,9,238,166,143,88,56,68,236,238,11,13,60,38,147,73,193,79,131,193,
  8,78,165,10,187,111,197,206,126,232,224,199,152,154,95,128,169,83,243,161,14,89,201,84,28,120,74,209,210,210,140,214,214,22,180,182,180,160,165,165,
  37,174,28,134,93,196,36,134,60,146,24,57,218,194,193,225,156,172,116,34,71,8,186,186,58,81,91,83,131,218,218,90,212,214,214,72,199,169,34,92,
  182,132,35,240,251,188,120,111,255,126,92,117,245,213,146,107,38,228,226,177,89,173,104,110,110,70,115,115,11,90,90,154,225,116,58,149,207,117,9,116,
  112,156,88,50,142,112,177,23,124,10,247,188,194,20,40,232,147,178,244,78,36,162,212,90,212,78,93,198,243,49,203,243,72,171,74,172,197,72,172,221,
  202,33,158,151,95,188,136,5,11,23,70,41,113,243,231,47,64,83,83,19,150,45,91,142,245,235,55,192,227,241,224,124,89,25,222,126,235,45,236,220,
  177,3,171,215,172,1,64,208,208,208,128,89,179,103,43,232,72,78,78,198,117,159,188,62,220,6,161,57,68,182,147,56,180,64,185,100,158,139,199,94,
  56,127,30,139,150,44,145,185,170,133,169,123,225,194,69,104,108,108,196,138,149,43,177,97,227,38,120,221,30,148,149,149,225,173,55,223,192,171,175,188,
  140,213,107,215,130,144,48,29,156,236,94,61,209,177,245,178,109,80,171,213,248,240,131,247,7,149,14,249,102,225,16,29,159,186,233,102,204,152,57,11,
  54,171,21,199,142,30,193,243,207,61,139,227,199,142,97,233,242,229,32,162,117,223,104,52,74,231,235,13,6,69,251,170,213,106,88,173,29,104,110,106,
  70,115,115,51,14,126,252,49,150,44,93,42,241,99,48,100,55,74,17,141,154,251,123,240,138,246,34,187,199,143,31,195,186,117,235,97,52,26,177,120,
  201,18,156,60,33,108,142,43,47,191,136,5,11,23,68,201,238,188,121,243,209,220,212,36,41,212,28,225,34,82,47,9,191,168,213,106,24,12,6,233,
  249,13,6,131,20,107,203,137,158,16,191,223,15,75,91,27,230,20,23,35,191,160,0,117,181,181,208,104,52,210,56,167,209,104,96,183,219,209,210,210,
  34,140,9,205,205,112,56,28,67,50,182,115,35,165,4,132,51,243,11,43,163,103,159,253,31,62,243,217,91,65,0,228,230,229,193,104,50,226,173,55,
  222,192,190,125,123,241,222,254,253,216,249,234,14,172,94,179,86,70,176,74,114,159,73,43,127,181,16,71,42,207,129,21,43,62,84,18,74,217,231,104,
  37,84,25,47,26,185,225,73,81,253,36,214,174,222,88,131,100,12,139,78,87,103,39,182,94,182,13,201,41,41,0,128,169,249,249,208,235,245,232,236,
  231,6,163,211,98,192,243,212,169,97,37,246,200,225,67,184,230,186,235,144,156,156,44,93,251,154,235,174,67,109,109,205,132,183,142,42,151,7,202,212,
  21,32,64,85,69,37,82,211,210,176,108,217,114,128,16,164,166,164,98,235,182,203,112,225,252,121,105,51,75,88,254,8,118,191,253,54,214,111,216,16,
  182,146,113,42,104,228,174,10,142,196,151,67,89,5,139,216,29,60,34,14,80,33,175,202,65,145,147,226,121,72,84,146,233,208,125,33,186,158,67,229,
  218,74,207,150,64,165,82,161,160,176,16,156,138,11,187,89,68,247,15,23,49,240,12,14,29,68,86,165,35,210,243,64,162,189,13,61,249,225,251,96,
  41,101,60,31,155,60,143,89,129,170,135,88,209,72,158,191,249,198,235,184,237,182,219,49,99,198,12,128,16,24,13,6,124,246,214,207,193,227,245,226,
  108,73,9,42,43,43,177,109,251,118,204,157,55,79,92,124,24,80,60,119,46,218,90,91,65,8,112,244,200,97,108,220,180,25,249,226,230,182,228,148,
  20,220,240,169,155,4,75,17,71,98,182,149,90,173,146,82,219,92,50,207,197,57,104,231,171,59,240,133,47,222,133,153,179,102,131,128,192,104,50,226,
  182,219,239,128,199,235,193,217,211,103,80,81,81,129,237,87,94,137,185,243,231,131,16,192,32,198,196,182,137,251,17,142,28,62,132,77,155,183,32,191,
  160,16,132,16,164,164,36,199,165,227,242,237,87,64,173,86,227,189,253,251,6,79,118,229,123,47,34,148,145,198,134,122,220,120,211,77,152,52,121,50,
  64,8,210,210,210,48,99,198,12,52,55,53,3,132,224,252,185,115,184,250,154,107,165,197,225,21,87,94,41,133,28,8,27,7,69,185,229,66,158,214,
  121,240,184,221,67,211,7,73,28,69,180,39,175,104,47,178,123,226,248,113,24,77,70,124,230,214,207,225,192,135,31,194,229,114,129,16,193,29,238,118,
  123,240,249,219,110,135,193,104,2,1,48,125,250,116,124,246,115,159,195,206,87,95,149,140,99,138,208,34,217,51,150,151,95,196,118,49,62,92,111,48,
  96,235,182,203,81,93,93,45,209,169,21,55,51,213,214,214,98,233,210,101,168,175,171,19,172,161,90,173,24,59,46,40,172,42,78,53,244,253,28,195,
  88,129,41,218,84,175,220,144,112,246,76,9,150,47,95,129,245,27,55,33,39,39,71,72,157,17,114,27,16,10,175,199,139,186,218,90,44,90,178,24,
  39,143,159,16,172,21,114,119,139,104,150,94,189,102,45,214,173,95,47,213,178,165,148,226,133,231,158,67,125,125,93,236,245,10,137,222,89,7,197,202,
  92,254,188,113,74,154,197,141,96,235,125,146,220,243,206,30,92,182,109,27,238,186,251,75,208,235,245,104,105,105,193,238,221,187,149,177,161,52,78,155,
  71,124,255,218,171,59,113,239,215,191,46,125,247,206,238,221,88,191,97,3,190,40,94,219,110,183,227,181,157,59,81,91,51,202,148,209,97,78,244,79,
  164,189,213,61,43,52,143,253,245,175,184,249,211,159,198,77,159,254,52,156,78,39,142,30,57,130,131,31,127,44,173,232,67,171,71,74,129,250,250,58,
  148,149,149,33,41,41,81,114,143,22,20,22,226,59,223,251,158,34,143,219,199,31,125,132,131,31,127,20,45,139,36,250,179,66,246,16,35,45,10,33,
  210,62,29,233,24,142,147,146,163,135,118,114,134,82,165,113,28,39,201,43,167,10,87,206,160,160,120,243,205,55,112,207,189,95,19,221,113,194,0,255,
  217,207,125,78,170,255,28,162,225,183,191,126,184,199,93,138,125,165,35,212,126,92,100,80,123,132,27,71,254,46,197,113,246,166,140,196,72,78,205,120,
  62,70,121,78,34,147,123,147,168,161,35,78,209,101,16,8,19,250,191,159,126,26,55,222,116,51,242,38,77,130,211,233,196,233,147,39,241,228,223,159,
  0,0,216,172,86,60,255,220,115,184,252,242,237,184,237,246,59,224,116,58,113,182,164,4,251,246,237,5,8,193,133,243,23,0,16,92,255,201,79,34,
  49,49,17,46,151,11,135,14,30,196,199,31,125,36,163,143,202,158,131,66,165,82,199,156,243,6,204,115,78,120,142,127,254,227,31,248,244,103,62,131,
  73,147,39,195,233,112,224,228,137,19,120,226,111,143,1,132,192,102,181,226,217,255,254,23,219,175,188,18,119,124,225,11,112,58,157,40,57,115,6,251,
  246,238,5,1,193,249,115,231,65,64,112,253,13,55,244,72,71,70,86,38,22,44,92,8,74,169,244,222,80,95,143,23,158,127,110,80,232,160,188,210,
  93,79,65,113,232,224,33,164,166,165,227,238,47,127,5,41,41,41,104,183,88,112,232,208,33,148,158,61,11,2,224,165,151,94,196,141,159,186,9,191,
  252,213,195,224,121,30,165,103,207,98,199,203,47,75,50,162,86,171,113,235,231,63,47,201,108,75,75,11,222,120,253,117,209,50,71,7,69,118,163,188,
  51,225,109,143,49,198,19,210,227,61,99,201,238,135,31,124,136,235,63,249,73,60,242,199,63,40,198,171,199,255,246,55,220,248,169,79,225,167,191,248,
  5,244,122,61,26,234,234,240,191,255,62,131,138,138,114,73,198,166,77,159,142,223,252,238,247,82,58,185,61,239,236,198,187,123,246,96,231,142,87,113,
  221,245,215,227,193,159,254,20,60,207,227,92,89,25,94,219,185,83,26,67,52,26,13,8,33,104,106,108,196,214,203,46,195,249,243,231,5,107,168,90,
  45,133,38,169,53,26,220,242,217,207,42,114,139,114,132,224,225,135,126,57,168,253,28,132,128,92,127,221,117,125,86,5,138,231,206,197,207,126,254,115,
  124,231,91,223,194,153,211,103,250,173,4,240,98,61,84,201,201,44,43,199,68,35,190,139,247,46,252,23,251,93,114,93,211,104,87,118,44,133,36,188,
  112,137,248,28,227,61,150,194,74,98,228,252,139,245,93,164,208,71,42,194,138,103,152,128,152,191,96,62,126,243,187,223,225,129,31,254,16,165,103,207,
  246,233,156,159,254,252,231,224,131,65,124,231,91,223,30,128,238,75,193,7,249,240,223,52,82,38,99,124,71,105,108,25,149,125,31,37,147,50,69,187,
  47,114,168,252,91,38,95,145,127,203,190,231,121,62,202,116,72,98,14,193,225,231,137,76,151,44,167,131,64,217,143,134,138,142,153,51,103,225,254,255,
  247,61,60,242,199,63,162,188,188,34,134,181,33,250,253,23,191,250,21,248,96,16,223,190,239,190,30,43,46,129,68,46,20,25,207,71,5,207,103,205,
  194,253,223,235,31,207,127,240,192,3,224,121,30,15,252,224,135,81,99,104,212,184,74,198,49,207,67,169,17,17,237,125,163,145,60,143,156,255,104,196,
  108,40,171,134,192,232,24,24,29,138,82,181,113,230,253,120,159,199,187,236,14,100,108,7,33,80,211,254,212,83,164,178,193,141,14,36,241,61,1,165,
  188,130,100,249,101,100,109,27,33,164,225,242,96,114,9,12,5,145,135,203,93,81,69,185,80,169,253,40,226,76,86,74,205,144,68,9,5,137,96,71,
  132,202,24,42,167,39,43,16,33,124,14,237,106,163,82,67,203,243,14,74,171,125,10,5,117,19,22,178,180,60,253,149,171,129,200,97,136,179,124,132,
  240,209,88,130,72,163,159,53,178,164,171,16,255,47,151,73,89,76,35,145,173,128,35,159,149,196,144,169,136,221,166,49,75,115,203,75,60,18,18,181,
  99,148,70,125,14,255,95,222,51,66,54,29,137,14,233,129,135,158,14,41,134,145,244,92,119,156,198,152,168,66,137,221,195,165,245,148,25,173,67,86,
  66,26,195,166,198,120,62,130,60,199,0,120,46,75,124,79,35,234,39,75,37,151,165,252,213,227,153,231,144,54,224,81,217,181,34,249,30,173,221,17,
  89,17,243,72,127,49,163,99,160,116,132,255,166,242,155,137,217,46,168,98,206,15,21,19,152,40,178,59,144,177,157,80,58,124,187,233,229,238,122,26,
  171,20,40,98,148,13,13,89,13,101,59,201,20,185,6,35,101,33,178,14,115,172,53,87,188,178,93,17,187,226,72,15,223,69,90,100,72,84,89,179,
  30,2,150,89,201,207,81,129,168,178,107,146,204,65,41,127,160,81,59,174,67,178,38,117,104,130,88,43,146,190,116,134,152,178,72,98,110,190,137,45,
  147,28,33,130,165,76,126,61,217,253,133,207,156,66,121,11,237,38,142,164,67,222,255,134,154,14,41,101,82,200,141,76,16,123,3,96,172,146,129,232,
  173,180,46,97,60,31,47,60,143,83,19,156,244,242,247,184,227,57,8,120,202,3,132,139,193,107,121,137,72,153,49,134,200,100,65,50,127,209,97,229,
  249,120,165,35,90,25,139,237,169,137,117,167,241,46,187,3,233,231,195,86,155,190,207,13,28,67,33,69,204,70,142,209,144,177,20,213,120,141,222,91,
  3,199,19,188,24,105,28,72,220,36,247,232,147,227,157,41,167,35,171,144,130,231,123,238,236,178,1,50,100,133,139,217,241,227,44,158,7,210,209,123,
  149,191,136,129,143,136,9,40,105,132,76,81,249,231,88,131,126,4,29,242,123,13,57,29,161,13,132,178,141,129,61,46,242,122,232,103,241,22,126,140,
  231,99,159,231,66,138,166,75,91,208,143,55,158,43,158,69,22,83,43,143,175,13,205,157,161,228,180,52,162,125,25,29,151,78,71,79,159,227,141,83,
  19,66,118,7,56,182,95,66,109,250,129,187,149,9,71,0,158,134,205,210,33,235,167,156,126,74,195,105,1,33,126,150,6,86,200,148,210,56,214,243,
  240,200,28,215,120,18,191,204,32,85,230,237,146,26,51,214,103,26,209,208,52,188,218,137,136,17,32,132,0,19,217,29,223,163,60,13,255,185,132,200,
  93,24,49,100,49,234,115,180,21,62,134,167,38,166,43,34,174,44,198,218,105,25,249,29,137,252,76,101,95,82,112,4,224,35,60,3,136,244,20,132,
  226,3,73,108,47,147,184,155,96,216,232,144,242,119,114,164,95,138,104,200,41,23,25,202,19,90,249,247,214,199,24,207,199,22,207,195,73,115,104,220,
  49,116,162,241,28,49,188,128,202,67,169,44,4,67,246,57,180,0,97,116,12,10,29,242,57,127,160,243,255,120,148,221,129,142,237,195,110,25,149,86,
  62,161,132,187,82,3,43,45,164,97,173,95,100,109,40,32,55,98,181,31,233,158,167,17,13,132,88,49,133,49,115,129,133,91,148,196,92,21,68,164,
  104,232,97,178,36,49,52,221,88,43,34,102,21,29,5,214,81,66,164,88,227,152,178,24,74,130,46,43,125,19,94,4,137,114,73,100,241,79,241,100,
  49,222,128,19,211,146,30,153,14,164,7,185,140,177,114,165,17,131,35,149,92,100,17,21,136,20,229,124,200,176,210,193,201,226,7,123,180,46,144,158,
  147,249,144,190,186,32,24,207,199,36,207,201,32,109,238,28,87,60,167,20,224,136,108,3,90,88,241,161,145,150,42,185,117,145,12,63,207,199,59,29,
  177,255,38,24,204,253,200,99,77,118,7,212,207,71,74,25,237,177,129,67,43,162,184,141,28,169,148,134,181,81,133,134,31,97,213,236,77,221,239,173,
  145,251,108,134,103,138,232,216,83,70,105,140,32,242,24,225,35,114,185,132,20,168,46,235,240,178,191,17,41,139,113,217,29,189,236,140,187,56,234,65,
  22,57,66,16,114,230,16,170,92,210,147,200,58,237,145,116,144,176,197,113,216,232,224,136,148,15,184,207,46,175,136,1,147,12,176,79,49,158,143,29,
  158,43,235,164,12,124,28,29,95,60,151,27,108,16,197,119,41,84,35,114,174,196,8,240,124,156,211,209,87,69,116,66,201,238,64,198,246,97,175,192,
  20,139,17,177,86,190,145,43,34,241,239,63,60,242,8,182,95,121,133,162,28,151,178,196,84,248,59,18,81,87,54,118,69,19,72,101,1,33,79,110,
  171,248,78,246,188,189,8,97,95,21,81,134,209,167,144,198,148,197,200,18,124,226,223,27,55,109,194,143,127,250,147,136,178,133,145,114,216,87,89,84,
  202,33,33,209,178,40,63,136,68,200,162,34,127,155,172,132,165,34,196,62,178,218,80,136,142,141,27,241,224,8,210,33,79,214,28,111,224,138,189,232,
  19,171,46,201,42,47,245,59,22,139,241,124,108,240,92,81,219,250,210,198,214,254,242,124,251,21,87,224,15,143,62,50,42,121,46,205,57,92,244,78,
  103,78,70,39,34,42,7,142,101,58,22,45,90,132,59,239,252,194,168,163,99,40,21,209,129,202,46,145,181,219,88,25,219,213,35,169,4,112,28,39,
  229,204,35,84,138,12,137,107,37,93,184,112,33,78,159,60,169,244,207,81,121,156,103,100,74,236,62,48,57,242,175,30,202,10,246,199,26,218,147,192,
  49,5,117,116,42,163,49,87,159,145,43,78,0,89,89,89,152,55,111,158,148,54,67,178,161,203,86,158,33,149,105,56,101,49,84,41,140,202,210,133,
  32,94,127,2,144,149,157,141,121,243,231,143,8,29,132,227,164,18,129,232,109,19,96,188,176,154,75,232,71,140,231,99,131,231,132,12,94,6,230,254,
  242,60,59,39,7,11,23,46,28,149,60,87,164,3,148,109,246,5,81,62,163,20,75,24,162,109,12,211,145,146,154,138,194,162,194,81,199,143,129,232,
  0,67,45,187,138,190,63,70,198,246,17,179,140,198,210,248,35,181,126,179,217,140,236,156,156,232,28,87,226,107,218,180,105,97,95,78,148,198,31,174,
  218,18,251,69,20,22,81,68,148,156,139,213,144,76,17,157,0,214,209,56,214,250,130,194,66,101,41,187,208,202,143,16,100,102,102,138,53,205,21,203,
  204,62,202,97,124,89,36,61,201,98,140,85,121,168,68,91,196,129,138,227,242,101,116,132,6,141,145,160,35,180,122,38,232,185,207,197,223,128,72,46,
  185,79,49,158,143,17,158,15,98,152,83,79,60,55,155,205,88,176,104,81,180,2,44,182,85,78,118,182,108,206,25,69,60,143,180,136,197,178,138,246,
  112,204,104,166,35,39,55,71,122,54,46,100,45,21,143,73,73,73,129,94,175,31,21,116,12,135,14,208,219,120,21,197,223,17,226,249,64,251,249,168,
  80,70,229,141,108,78,48,75,141,188,243,141,55,240,201,27,111,140,54,69,139,3,199,223,255,249,79,220,126,199,29,82,99,155,205,230,136,22,36,61,
  188,160,84,62,99,185,182,34,149,208,75,20,66,134,209,13,161,44,162,0,147,217,36,173,220,111,249,204,103,240,135,71,30,17,39,109,113,80,148,29,
  123,213,53,215,224,233,103,158,145,228,208,100,50,69,10,214,128,100,17,125,24,12,99,45,136,66,207,70,8,129,217,108,150,232,184,249,211,159,198,239,
  254,240,7,152,76,38,169,172,219,72,210,17,138,43,234,235,194,111,48,21,81,198,243,49,194,243,33,136,183,151,211,159,157,147,45,181,213,13,55,222,
  136,223,255,241,143,200,206,206,142,10,21,3,128,219,238,188,19,127,255,231,63,165,182,202,206,206,30,53,60,87,184,78,229,231,112,61,40,40,138,50,
  187,163,131,14,131,193,32,93,103,203,150,45,184,231,171,95,133,94,175,23,232,8,45,164,68,172,94,189,26,63,124,224,1,137,14,131,94,63,162,116,
  196,211,105,134,74,118,67,22,239,126,41,165,195,196,243,254,246,115,50,80,101,84,172,48,53,104,175,208,35,254,228,103,63,195,239,254,240,7,233,250,
  31,127,252,49,38,79,153,130,77,91,182,98,253,198,77,168,174,169,65,82,82,50,54,110,218,140,235,111,184,1,86,155,13,199,142,29,7,165,192,154,
  117,235,240,250,91,111,97,218,180,233,10,125,158,82,128,242,66,105,45,62,16,20,94,65,94,241,10,6,121,4,121,10,158,167,66,185,82,41,115,21,
  17,163,211,196,106,9,178,103,141,215,6,210,61,251,249,219,132,126,13,176,99,14,182,28,10,9,26,56,20,20,20,226,249,23,95,196,138,149,171,64,
  1,156,58,117,26,246,206,78,108,222,122,25,214,172,91,143,196,164,100,212,215,55,96,245,154,181,88,187,110,61,166,77,159,142,227,199,142,9,242,35,
  202,241,119,191,247,61,229,218,82,148,67,202,83,65,222,98,188,66,178,200,83,128,151,203,161,92,22,67,178,212,139,44,18,194,33,63,191,0,207,252,
  239,89,44,95,177,18,20,192,233,211,103,208,217,217,133,45,151,141,2,58,64,226,12,240,209,11,191,72,215,124,56,177,28,97,60,31,231,60,151,87,
  176,27,204,49,148,16,14,133,69,211,240,204,179,207,98,219,229,219,65,1,28,61,122,12,86,155,13,91,46,219,134,141,155,55,195,148,144,128,11,23,
  47,98,195,198,77,216,180,101,11,178,115,114,113,224,192,1,169,173,254,240,200,35,248,217,47,126,25,179,173,248,32,143,96,32,136,160,56,223,4,252,
  1,4,252,129,216,115,206,37,242,60,122,60,13,219,178,34,191,147,243,37,244,61,141,101,11,147,63,83,196,115,40,230,215,200,249,241,18,232,200,206,
  206,193,3,15,60,128,217,179,231,128,82,160,188,162,2,129,96,16,235,55,110,196,154,117,235,145,157,147,11,183,199,43,201,110,97,81,17,42,202,203,
  37,126,220,241,133,47,224,51,159,249,236,136,246,193,225,152,255,163,120,46,239,45,49,228,96,216,121,62,192,177,125,68,99,70,1,160,104,90,17,42,
  43,42,65,8,65,69,121,5,22,44,90,132,155,111,185,5,233,25,25,184,88,94,129,214,214,86,116,59,157,232,238,238,194,111,127,243,91,240,148,135,
  74,173,6,79,8,254,247,236,115,216,180,117,43,102,205,158,141,197,139,23,193,110,183,163,185,185,9,193,96,16,105,233,233,112,58,28,112,58,157,125,
  138,127,80,196,161,201,19,183,202,92,3,125,54,161,51,247,252,152,131,217,108,134,201,108,70,107,75,11,44,22,11,58,59,59,177,237,242,203,145,55,
  105,18,82,211,210,240,210,75,47,195,210,209,142,206,206,78,236,219,191,31,123,247,237,5,5,96,48,24,177,231,221,189,200,204,204,196,109,119,222,137,
  238,206,78,100,101,103,227,200,225,195,224,121,30,148,231,145,55,105,18,234,235,234,250,234,42,0,196,234,58,114,107,61,23,138,195,233,201,98,40,210,
  97,54,155,209,218,218,10,139,197,130,238,174,46,108,189,236,50,137,142,87,118,236,64,123,71,199,200,211,65,162,93,57,49,173,96,17,253,82,97,149,
  96,60,159,16,60,39,131,56,134,102,101,103,195,108,54,161,178,162,18,85,149,149,176,217,108,88,179,110,29,166,207,156,9,10,224,127,207,62,135,118,
  139,5,118,187,29,251,246,237,199,222,189,251,64,1,24,77,70,236,218,189,27,89,89,89,248,230,119,191,139,46,187,29,70,163,17,39,79,28,7,31,
  12,130,167,20,139,151,44,193,177,163,71,123,94,64,7,131,177,93,149,17,243,13,233,231,156,19,89,22,153,68,164,80,162,177,188,10,49,210,30,210,
  30,44,210,241,230,206,30,173,216,189,92,203,96,48,192,96,48,192,102,179,161,179,179,19,78,151,11,27,55,111,194,156,185,115,145,154,150,134,227,39,
  79,161,203,233,68,103,103,39,90,90,91,113,232,240,33,73,118,75,202,206,73,178,219,105,181,34,51,51,19,231,202,202,36,217,77,207,200,64,91,107,
  235,176,244,193,222,228,115,176,231,255,200,74,150,10,11,232,72,243,124,32,253,124,164,149,209,109,151,95,142,239,220,255,93,252,245,47,127,65,201,153,
  51,224,84,42,188,249,198,155,104,108,110,198,123,31,126,128,170,170,106,116,116,116,0,148,98,210,164,60,24,141,70,216,108,54,180,183,183,195,31,8,
  34,33,49,1,153,153,153,152,50,121,50,220,94,47,206,157,191,128,57,197,197,56,87,90,134,239,127,255,251,0,40,238,251,218,215,149,13,30,105,162,
  150,24,66,148,201,162,41,21,202,150,129,32,72,8,56,142,128,35,74,55,65,31,121,8,166,135,142,126,60,246,196,227,0,128,251,190,241,13,204,155,
  55,31,31,31,58,140,218,218,90,156,191,112,1,229,23,47,162,205,98,129,219,237,130,201,100,66,78,86,54,124,62,47,44,237,237,232,238,238,134,90,
  163,65,106,74,42,178,179,179,48,107,214,44,188,252,242,43,112,59,157,200,206,206,198,140,25,51,240,169,155,62,133,167,159,126,26,135,14,30,140,35,
  136,178,175,168,82,112,40,21,138,67,208,32,5,130,65,16,41,109,6,23,157,223,150,0,127,254,235,95,0,66,240,221,111,125,27,197,243,230,225,224,
  225,195,168,171,175,199,133,139,23,80,94,94,14,139,165,125,84,208,225,245,122,225,113,187,65,41,237,243,96,37,206,186,131,182,161,133,241,124,244,243,
  156,12,242,24,250,187,223,255,78,224,249,215,191,129,89,179,103,227,213,157,175,225,252,249,243,56,121,250,20,106,106,106,224,232,118,128,227,8,166,78,
  153,130,217,179,103,195,237,118,163,182,182,22,13,13,13,112,186,221,48,24,12,72,78,74,194,156,57,197,168,174,173,65,90,74,42,10,139,138,80,88,
  88,128,91,63,255,121,60,241,216,99,248,224,253,15,226,79,230,178,201,135,202,146,62,18,74,193,7,131,32,132,128,151,118,54,115,225,20,57,81,243,
  73,116,168,74,72,57,137,78,205,43,108,19,146,42,105,70,245,43,26,155,135,125,214,76,226,121,49,226,170,48,210,237,239,187,239,62,0,192,227,143,
  63,142,133,139,23,163,203,233,66,67,83,243,128,250,32,57,83,2,173,78,143,236,236,108,100,102,102,96,227,198,77,216,181,107,23,202,202,74,135,188,
  15,246,70,231,96,35,146,231,177,149,210,145,225,185,20,15,28,145,103,180,183,177,93,53,107,230,204,7,251,218,0,153,153,153,216,188,101,11,222,217,
  189,27,173,45,45,151,220,160,149,149,21,88,190,124,57,166,76,157,138,140,172,108,124,124,240,32,222,124,251,109,236,127,111,63,78,158,60,137,174,206,
  78,76,158,60,25,11,23,206,199,178,101,203,144,151,55,9,38,147,9,90,173,6,46,151,11,109,109,173,168,172,168,196,249,11,23,80,89,89,137,150,
  214,86,228,229,77,194,150,203,182,34,59,59,27,79,253,243,41,52,55,55,245,222,186,68,153,199,80,18,46,233,103,97,149,193,83,30,148,242,138,184,
  144,222,21,81,166,137,246,134,236,236,108,108,187,252,114,236,223,183,15,109,109,109,125,58,103,243,150,45,160,148,226,157,221,187,6,229,25,90,90,90,
  176,97,211,38,76,154,50,5,22,75,59,118,189,243,14,222,125,119,15,14,29,62,132,186,186,58,24,244,122,76,159,62,29,139,23,46,192,172,217,179,
  144,156,156,2,131,94,15,66,8,58,187,58,81,87,91,139,138,202,74,84,86,86,162,174,174,22,60,229,177,104,209,98,44,94,178,4,237,237,22,188,
  240,220,115,189,244,243,72,89,34,81,21,64,66,202,24,165,84,172,77,78,197,85,104,248,220,182,182,54,172,223,176,1,147,38,79,129,165,189,29,239,
  188,251,46,222,221,183,23,135,15,31,70,253,40,162,35,61,45,13,43,86,172,192,129,3,7,208,222,209,161,136,179,34,113,6,171,48,207,119,51,158,
  79,16,158,111,218,188,25,4,192,158,119,222,25,52,158,175,94,187,22,147,166,76,198,233,146,179,216,177,243,85,124,120,224,0,42,42,42,224,118,185,
  80,88,80,128,13,235,215,99,197,242,229,200,204,202,130,86,171,133,209,96,64,66,66,2,120,158,71,107,115,51,154,91,91,81,87,87,139,243,162,178,
  84,60,119,46,86,174,92,133,142,118,11,254,243,244,211,125,155,209,73,12,69,155,40,115,63,82,136,60,167,124,120,83,72,15,97,212,189,206,53,161,
  77,194,4,146,163,54,222,49,189,190,226,157,18,17,95,24,111,78,12,61,170,205,102,197,202,85,171,49,111,209,66,116,116,116,12,90,31,44,46,158,
  11,139,197,130,125,123,223,29,150,62,56,18,243,127,248,210,116,84,241,60,43,59,11,91,183,94,134,247,223,127,15,150,246,246,152,231,197,210,210,135,
  221,50,90,52,109,26,190,114,207,61,120,240,129,7,144,152,152,136,83,167,207,224,116,201,25,188,255,254,251,56,119,238,28,220,110,55,82,83,83,176,
  126,237,90,172,92,177,28,233,233,25,160,148,135,189,179,11,46,167,19,70,131,17,83,167,76,69,90,90,58,186,58,59,209,97,181,162,169,177,9,213,
  213,53,168,174,169,193,241,227,199,81,84,52,13,87,93,113,5,178,115,114,0,10,172,94,187,6,51,102,206,196,83,79,62,169,116,242,145,240,178,129,
  70,212,104,6,168,248,179,184,130,165,84,74,67,17,114,3,112,42,149,50,160,152,41,162,99,10,223,185,255,126,156,62,117,10,239,236,222,141,140,204,
  76,188,252,202,14,188,254,198,235,56,124,248,48,172,54,27,84,28,65,65,126,1,214,172,89,141,169,147,167,64,111,208,195,235,245,161,179,179,19,160,
  64,122,122,58,18,18,18,144,151,155,11,155,221,142,142,142,14,84,86,86,225,212,169,211,56,127,254,60,14,31,57,138,181,107,214,96,238,220,98,164,
  165,167,195,229,116,226,11,119,221,133,87,94,126,25,141,141,141,202,177,14,242,224,89,2,34,212,127,83,212,32,6,194,114,74,32,196,66,5,105,16,
  223,250,246,183,81,82,82,130,119,247,236,65,122,70,38,118,188,250,42,222,124,235,45,28,62,114,4,182,209,74,7,13,197,203,9,22,33,158,231,193,
  113,28,212,42,85,204,184,34,198,243,137,201,115,21,199,33,56,8,60,7,128,223,60,252,48,166,230,23,224,181,215,223,136,226,121,97,65,1,214,175,
  93,135,169,83,167,194,96,48,192,235,243,194,110,239,68,48,16,132,217,108,134,90,173,134,217,108,70,78,78,14,218,219,219,81,94,81,129,198,134,70,
  116,116,116,160,174,174,14,173,173,173,88,48,111,62,38,79,153,2,74,41,238,185,247,94,252,229,209,71,165,16,135,112,1,149,208,156,67,164,242,150,
  52,84,101,82,198,111,105,254,17,173,116,65,158,7,79,169,88,102,145,235,65,201,19,173,122,148,246,73,147,33,50,143,195,37,106,68,125,182,230,221,
  116,211,205,168,170,170,196,177,99,199,48,109,198,76,156,41,45,29,146,62,56,107,214,76,164,165,167,163,187,187,27,87,93,117,53,62,252,224,3,88,
  218,45,131,214,7,67,58,192,72,102,206,25,173,60,231,56,210,175,148,124,132,144,225,87,70,179,178,178,176,96,193,2,252,252,151,191,68,201,217,82,
  236,219,191,31,239,125,240,62,218,219,44,152,86,84,132,249,243,231,163,160,32,31,137,9,137,32,28,65,183,195,1,135,195,1,159,215,39,40,129,160,
  80,107,52,48,155,205,208,235,245,72,73,73,65,94,110,46,166,117,118,162,165,185,25,117,245,245,56,83,114,6,110,183,11,91,55,111,193,221,95,249,
  10,22,47,94,132,250,250,122,200,157,242,84,86,168,85,25,99,17,174,53,75,197,193,131,208,24,171,86,2,4,101,3,170,92,41,101,74,232,216,64,
  81,81,17,182,93,126,57,230,204,157,139,242,242,10,252,227,159,79,162,180,180,20,9,38,19,54,111,220,136,57,115,102,35,43,43,27,90,173,6,30,
  183,7,54,155,29,110,143,71,216,136,192,7,193,113,28,116,58,29,184,228,100,152,204,102,100,100,100,32,55,47,23,54,155,29,141,13,13,104,106,106,
  196,222,253,251,96,179,219,176,97,243,22,76,153,52,9,83,166,76,65,106,74,10,26,27,26,226,172,105,165,66,196,225,193,38,84,139,89,145,59,46,
  236,134,41,44,42,194,182,237,219,49,187,184,24,21,149,85,248,231,191,254,133,178,210,82,36,152,204,163,150,14,0,224,195,145,253,82,165,53,127,32,
  0,142,227,164,114,114,131,221,167,24,207,199,24,207,7,129,239,69,69,69,40,44,42,66,98,82,18,14,29,58,140,151,118,188,18,147,231,58,173,22,
  94,159,15,86,155,13,110,183,91,104,43,202,131,112,28,116,122,61,56,149,10,38,179,25,153,25,25,152,52,41,15,86,171,13,245,245,245,104,108,108,
  196,59,123,246,192,106,181,98,253,250,245,152,57,125,58,146,147,146,145,154,154,38,41,163,20,178,137,35,84,231,156,66,81,83,92,169,0,145,200,122,
  175,160,160,8,6,131,32,148,70,185,239,227,125,166,125,85,56,134,212,130,167,188,118,110,110,14,150,44,93,138,229,171,86,161,174,174,126,72,251,224,
  186,141,27,161,215,104,145,154,154,10,115,130,25,22,75,219,160,245,65,30,0,13,4,132,112,10,21,55,36,227,213,64,218,120,164,121,46,197,220,246,
  65,9,149,63,251,176,43,163,31,127,244,17,30,127,236,49,204,153,59,23,111,237,122,27,21,229,21,88,188,112,33,102,206,152,129,244,180,116,168,212,
  42,248,252,126,56,28,78,184,61,110,248,124,62,4,131,65,129,249,148,151,26,90,173,82,129,35,28,212,106,53,116,122,61,140,70,35,50,210,210,48,
  103,206,28,216,236,118,84,86,86,224,181,55,223,128,199,235,65,122,70,6,126,247,155,223,244,129,73,97,229,83,82,68,35,43,64,83,165,130,26,90,
  229,83,113,128,232,201,82,202,48,186,240,237,111,126,19,191,124,248,97,52,183,180,226,63,207,60,131,196,196,4,220,118,235,173,200,159,154,15,141,86,
  3,0,112,185,61,232,232,176,194,231,243,9,178,24,8,138,241,68,226,196,65,56,104,52,26,168,84,106,104,181,90,232,245,122,36,39,37,99,114,94,
  30,60,62,31,234,235,235,81,86,86,6,143,199,139,13,235,215,163,244,236,89,156,57,115,38,238,224,64,66,19,150,172,23,75,213,218,228,247,165,225,
  242,120,223,190,239,62,60,244,219,223,162,181,173,13,255,253,223,255,144,152,96,198,231,63,119,43,10,166,22,140,94,58,40,5,165,188,244,29,149,74,
  215,17,133,231,65,213,131,229,129,241,156,241,188,175,60,255,209,79,126,130,247,63,60,128,71,30,121,4,11,23,204,199,79,127,252,35,228,231,23,162,
  187,171,19,86,155,13,86,171,13,237,238,118,4,249,160,232,26,135,180,111,32,20,227,170,213,104,160,86,169,161,211,233,96,48,24,144,145,158,129,249,
  243,230,129,112,28,74,75,207,226,216,241,99,104,104,108,194,246,109,219,160,215,106,113,234,228,137,24,211,52,141,225,40,142,230,113,232,72,162,80,128,
  4,107,42,229,121,197,124,211,215,10,63,116,160,150,176,65,82,66,67,120,252,241,199,241,253,7,30,128,213,102,31,150,62,184,110,237,26,212,156,168,
  70,85,101,229,160,246,65,233,51,229,17,12,80,112,170,248,252,24,169,246,30,9,158,147,62,230,138,38,35,229,166,255,252,109,183,193,225,112,96,255,
  254,253,40,154,54,29,111,188,241,38,18,18,18,240,165,187,239,134,223,239,131,207,239,71,87,103,23,124,94,47,124,1,63,252,126,191,56,64,81,113,
  69,205,135,77,209,148,130,202,86,48,28,33,208,106,181,208,106,181,160,0,180,26,45,18,204,102,148,149,150,98,215,238,221,72,78,76,194,218,117,235,
  113,236,200,97,92,127,195,13,216,241,242,203,176,90,109,136,187,56,18,133,147,128,132,87,67,148,138,41,11,194,131,66,104,229,68,197,145,149,23,119,
  227,49,133,116,244,34,43,59,27,255,119,207,61,248,235,95,254,130,213,107,214,224,200,209,163,120,233,229,151,241,173,111,126,19,46,151,11,148,242,232,
  118,56,96,179,217,37,185,12,6,130,194,102,54,42,240,54,200,7,133,116,33,84,150,100,72,172,254,165,86,169,160,210,235,1,131,1,134,96,16,90,
  141,22,45,173,45,168,174,174,6,40,197,101,91,183,34,45,61,29,179,103,205,130,222,96,192,187,123,246,40,173,31,4,178,154,36,68,242,225,69,78,
  88,89,57,57,184,231,222,123,241,215,63,255,25,107,214,173,195,177,99,199,241,242,43,59,240,205,251,190,1,143,219,51,234,233,160,84,176,242,80,177,
  47,19,34,44,244,66,202,9,133,48,233,6,129,75,86,78,24,207,39,54,207,55,111,217,138,247,223,255,0,47,60,255,60,118,191,253,22,234,235,27,
  208,220,220,132,147,167,78,194,237,114,75,155,85,120,81,201,147,226,53,67,198,15,113,178,17,82,8,9,109,167,86,169,0,149,10,129,64,0,126,127,
  0,249,83,11,96,52,154,112,254,252,121,188,179,231,29,220,114,243,167,145,158,158,129,57,197,115,96,48,24,176,235,237,183,227,26,167,132,182,32,17,
  109,73,195,115,145,56,247,40,188,116,84,72,87,196,131,239,81,1,146,239,186,30,144,245,108,144,20,208,148,148,20,92,115,237,117,120,253,181,157,216,
  184,121,11,46,150,151,15,123,31,76,61,119,14,185,121,121,208,105,117,56,126,252,24,22,44,90,130,148,148,20,89,12,107,104,7,61,145,54,42,133,
  18,243,19,194,193,102,181,226,232,145,67,138,121,95,190,88,232,11,63,134,75,33,28,113,158,247,146,17,34,214,57,195,166,140,174,94,179,6,83,243,
  243,81,60,111,62,118,188,182,19,239,236,222,141,59,110,187,13,90,173,22,229,229,229,225,65,10,225,64,225,204,140,76,112,42,53,166,21,77,135,74,
  173,133,215,235,67,183,163,27,93,93,93,176,90,45,112,58,186,225,243,134,119,103,242,226,249,58,157,22,26,173,6,65,62,136,170,234,106,188,250,250,
  107,184,234,138,43,241,197,187,239,70,70,70,6,114,114,115,209,209,97,133,24,154,35,42,153,144,220,246,144,86,60,242,193,84,210,84,195,214,83,113,
  160,144,98,120,104,120,64,83,169,84,76,243,27,165,174,249,213,107,214,96,202,212,169,120,119,239,62,252,253,201,39,193,1,152,61,123,54,118,188,186,
  3,126,159,95,234,184,161,9,202,104,52,194,108,78,64,90,90,6,210,211,51,193,243,128,195,233,64,87,103,39,58,187,108,176,90,173,240,184,28,160,
  126,95,120,82,165,20,28,71,96,78,48,67,107,211,194,106,179,162,185,165,5,38,179,9,91,183,109,67,241,156,57,104,107,109,197,158,221,239,200,131,
  120,100,78,13,185,50,2,201,159,23,146,197,162,105,211,176,118,221,58,20,20,20,96,239,254,253,120,242,169,127,66,69,8,138,231,20,143,9,58,168,
  172,175,72,113,91,98,157,191,80,173,117,42,104,0,8,92,98,127,98,60,159,184,60,159,60,101,10,246,238,219,143,151,118,188,2,147,201,8,191,63,
  128,131,135,14,246,185,173,156,46,39,186,187,187,209,213,101,135,213,218,1,183,203,129,160,223,23,14,57,0,160,214,168,64,41,224,247,251,209,237,232,
  134,165,163,3,31,126,244,17,174,249,196,39,48,99,250,52,180,182,182,226,237,183,222,10,123,42,101,115,78,200,19,28,154,195,104,164,81,36,194,50,
  42,223,245,68,69,95,113,144,6,123,244,202,197,218,121,61,24,86,180,254,40,92,185,185,185,40,46,46,70,126,65,1,170,106,106,70,164,15,110,216,
  180,25,73,137,137,176,219,108,56,118,244,40,154,155,26,97,183,118,40,75,207,70,124,150,191,60,30,143,66,55,16,68,87,190,99,156,130,138,252,24,
  108,235,254,64,21,196,17,225,249,0,147,254,171,102,206,152,241,96,95,31,36,51,43,11,155,183,108,193,238,93,187,208,210,207,221,244,111,188,254,58,
  178,114,114,80,118,174,12,207,62,247,28,244,122,61,174,189,230,26,52,54,54,194,225,112,132,156,224,210,128,181,125,251,118,124,246,179,159,69,65,209,
  108,104,181,6,201,92,31,8,4,196,6,228,160,82,105,65,136,6,20,60,2,1,191,34,24,153,16,2,141,86,139,138,202,74,33,230,212,231,71,78,
  78,14,94,126,233,69,92,188,112,65,116,185,203,221,239,84,214,25,32,179,190,134,223,65,101,198,123,185,16,82,170,48,235,135,44,184,44,118,180,239,
  86,140,203,183,111,199,190,125,251,250,156,23,110,243,150,45,160,60,143,221,187,250,183,155,190,190,174,14,39,79,156,64,86,78,14,254,243,204,127,112,
  228,200,17,220,121,231,157,112,187,93,104,109,105,141,26,16,179,50,179,240,197,47,126,17,43,87,174,67,122,122,14,56,194,193,235,245,194,239,247,139,
  124,22,86,193,42,149,30,32,4,65,62,0,42,229,171,19,58,165,74,173,70,103,87,23,90,219,90,209,217,217,137,180,180,52,116,119,117,227,233,127,
  61,37,201,60,165,97,249,138,41,139,82,15,17,222,235,234,234,112,226,248,113,228,78,154,132,103,254,247,63,28,61,122,116,76,209,145,154,150,134,149,
  171,86,225,200,145,35,176,118,116,72,46,29,66,32,219,213,33,27,52,121,30,91,182,110,5,165,148,241,124,2,241,124,221,250,245,224,56,110,192,60,
  207,204,206,198,127,254,251,204,128,121,238,243,251,68,143,151,96,29,83,169,244,0,8,2,65,191,180,135,33,84,234,91,165,86,163,179,179,11,45,173,
  66,91,37,39,37,195,235,241,226,137,199,30,147,205,51,136,248,76,163,219,42,206,187,100,189,147,79,54,160,10,165,162,47,115,206,112,187,110,1,33,
  227,67,117,117,21,102,206,158,141,255,61,251,191,17,233,131,153,153,153,240,121,189,120,227,245,215,65,65,225,232,238,134,205,102,131,221,102,131,221,110,
  131,205,102,133,221,38,188,219,108,54,216,172,86,88,109,86,216,172,29,176,90,173,232,238,234,148,248,33,175,64,68,228,234,1,66,86,116,190,223,10,
  251,80,98,56,121,158,149,149,37,100,199,217,191,95,202,142,211,151,118,24,152,101,180,31,116,153,205,102,201,197,225,241,120,177,255,189,247,208,216,208,
  128,181,171,87,67,171,209,8,59,228,100,10,220,244,105,211,176,121,235,86,76,157,60,25,28,71,96,48,232,160,209,168,197,68,192,2,147,131,193,32,
  130,193,0,130,193,0,120,74,65,56,45,56,85,0,52,224,149,134,110,158,82,164,36,167,64,167,214,192,227,245,162,188,162,2,167,78,159,66,222,164,
  201,168,174,172,66,114,74,50,236,118,187,24,183,68,36,175,146,34,101,24,137,229,82,9,69,145,138,110,123,132,221,248,144,108,24,194,123,48,16,232,
  113,183,29,67,255,229,233,82,144,157,157,141,150,150,22,228,77,154,140,15,62,248,16,39,78,158,132,10,64,81,65,1,46,92,188,168,24,16,77,70,
  35,86,172,92,137,213,107,214,32,193,100,130,199,27,128,78,167,129,74,173,18,138,32,0,162,53,159,7,21,227,153,9,81,65,165,210,131,15,186,65,
  105,64,34,77,175,211,195,108,52,65,205,169,208,218,214,134,211,167,207,32,59,51,11,70,163,17,129,64,16,4,128,199,227,1,21,45,239,36,82,22,
  21,86,123,32,59,55,7,173,45,173,152,58,53,31,7,62,250,24,39,199,32,29,66,149,154,160,98,144,12,13,244,144,185,166,105,68,156,22,227,249,
  196,226,185,199,235,5,231,247,247,207,42,58,109,26,42,203,43,144,145,149,133,215,94,127,3,135,15,31,70,130,201,132,194,252,124,156,59,127,94,106,
  43,78,197,33,59,43,11,171,86,173,66,241,188,121,48,234,245,240,250,2,48,24,116,80,107,52,66,91,81,161,173,2,129,128,248,242,131,130,3,167,
  210,131,231,189,160,65,191,164,80,234,245,122,36,39,38,65,167,213,162,165,181,21,31,125,252,49,146,18,19,161,215,27,160,211,105,225,241,120,132,182,
  146,230,28,217,134,166,80,200,162,108,239,82,104,38,129,100,229,10,135,54,8,222,57,101,184,88,40,214,149,227,184,94,83,43,133,66,17,134,3,41,
  41,41,176,217,108,152,62,99,38,14,29,62,60,162,125,240,178,173,91,161,215,235,17,8,4,64,64,224,243,249,250,220,7,195,6,40,132,249,129,248,
  225,123,18,63,84,35,31,182,55,220,60,15,47,52,251,62,110,171,135,90,129,248,252,109,183,225,147,55,222,136,195,135,14,227,149,157,175,226,131,15,
  63,68,74,74,10,150,44,89,130,230,230,102,120,61,130,2,169,209,104,176,124,217,50,172,95,191,30,201,201,201,97,101,214,164,131,193,96,132,74,173,
  9,167,185,8,6,225,247,9,113,165,129,64,0,193,32,15,66,180,32,42,128,6,125,224,121,97,192,211,104,212,152,49,115,58,74,74,203,96,179,219,
  112,248,200,17,92,119,245,53,248,202,61,247,32,61,35,29,191,126,232,87,226,64,26,202,184,65,163,234,168,82,26,145,212,54,228,198,151,185,207,20,
  174,20,153,4,83,0,124,112,228,99,72,152,34,43,44,138,158,121,246,89,212,215,215,227,196,201,83,248,229,175,30,66,107,107,43,46,219,178,5,94,
  175,23,142,110,135,52,32,230,229,230,98,195,134,13,88,176,96,129,116,190,70,163,130,209,168,135,86,171,7,81,113,194,177,60,69,192,31,128,207,39,
  46,140,120,10,64,5,149,218,0,4,189,8,6,5,171,10,225,8,50,50,210,145,154,150,138,54,139,5,37,101,165,200,206,206,194,101,219,46,199,156,
  226,57,56,114,248,176,224,242,148,203,96,196,78,218,80,108,84,98,130,25,175,236,220,137,186,186,58,156,60,121,10,191,124,248,87,99,146,14,42,179,
  18,201,173,13,52,50,157,10,165,202,239,40,149,250,20,227,249,4,224,57,165,8,248,253,125,230,121,118,118,54,30,255,251,223,21,60,183,218,108,184,
  108,203,22,248,124,62,69,140,104,118,86,86,84,91,169,213,42,232,245,90,104,52,90,161,173,196,88,192,96,32,8,191,95,48,130,8,10,53,1,199,
  105,69,79,158,95,48,138,16,130,148,148,100,36,39,37,161,213,98,65,101,117,21,46,92,188,128,237,87,94,137,226,226,57,56,115,234,52,118,190,250,
  170,108,30,137,126,15,205,53,84,225,210,15,197,38,134,67,32,34,231,153,112,58,76,33,5,20,215,135,16,7,249,230,220,161,178,156,25,12,6,252,
  224,135,15,160,163,163,3,78,143,103,84,244,193,53,107,215,33,51,35,3,231,207,159,199,241,99,199,250,36,187,68,150,111,52,220,230,17,122,64,60,
  126,96,244,236,35,25,14,158,19,16,168,213,234,184,41,200,226,97,128,181,233,251,254,239,192,129,3,216,189,107,23,12,70,35,78,28,63,1,167,195,
  129,156,236,28,204,155,55,79,176,76,138,171,202,188,188,60,172,93,187,86,161,136,10,131,3,7,163,201,8,149,90,45,176,91,172,49,27,8,6,17,
  8,4,133,129,33,20,167,201,105,161,82,235,64,56,149,240,29,128,73,121,147,160,82,169,16,12,6,97,105,111,199,197,242,114,184,92,46,188,245,198,
  27,224,121,94,8,82,15,189,75,65,235,124,216,5,37,163,151,151,118,131,82,69,124,107,204,127,210,239,188,16,120,77,41,216,191,248,255,6,174,139,
  246,253,223,95,255,252,103,216,109,118,212,215,215,163,190,190,30,42,142,195,202,21,43,224,243,251,20,46,162,85,171,86,41,6,68,0,80,113,4,58,
  189,22,122,189,14,28,225,36,87,76,32,24,20,44,245,129,160,228,182,35,28,7,141,70,15,181,70,47,88,244,1,36,36,38,34,57,41,25,132,16,
  184,221,110,84,84,86,162,203,209,141,179,103,74,112,190,172,76,148,97,94,182,131,55,188,97,47,220,70,194,251,31,127,255,123,216,108,54,212,55,52,
  140,93,58,120,42,110,72,224,195,253,76,182,89,36,166,187,82,4,31,234,83,140,231,227,158,231,84,150,108,188,47,60,111,110,105,198,175,127,245,43,
  88,173,214,161,229,185,104,165,80,171,117,224,84,90,105,114,79,72,76,68,114,114,184,173,202,43,42,208,221,221,141,146,51,37,216,183,111,47,120,42,
  204,53,161,185,135,151,183,91,228,252,17,57,215,136,243,9,31,10,103,139,152,103,40,132,239,120,74,251,220,63,36,254,196,73,112,222,239,241,59,70,
  178,244,157,59,95,5,79,233,168,233,131,254,64,0,213,85,85,168,171,173,233,187,236,202,248,17,210,3,248,190,242,131,231,251,205,143,97,153,115,135,
  144,231,3,193,144,111,96,58,125,234,20,108,54,27,62,117,243,167,81,122,174,12,38,179,25,51,103,78,135,78,175,135,195,233,4,165,20,70,163,17,
  179,103,207,70,82,82,82,76,83,111,130,89,15,173,86,11,16,193,253,30,8,4,224,15,132,173,162,161,85,41,225,8,180,42,3,84,132,192,41,6,
  153,39,36,36,34,57,49,17,29,98,238,184,170,154,106,204,108,152,142,195,135,14,65,89,101,73,185,190,145,2,129,40,141,176,138,202,205,166,130,121,
  94,97,29,141,176,146,134,86,181,65,49,63,26,179,144,142,12,28,14,7,94,126,233,37,104,180,58,156,63,127,30,93,93,93,152,86,88,136,236,236,
  108,56,28,78,69,172,82,241,188,121,49,175,161,211,106,96,48,26,192,169,132,13,11,129,96,16,126,127,0,129,128,79,88,24,137,153,31,56,66,192,
  113,42,168,212,106,208,160,31,60,245,67,175,211,35,37,37,25,26,141,6,62,159,15,13,141,141,104,110,110,134,219,225,68,77,77,109,120,213,250,255,
  217,123,239,248,184,174,243,206,251,123,238,244,138,222,1,146,32,192,222,139,68,170,81,84,151,45,91,182,99,91,110,113,156,226,148,77,226,56,201,187,
  217,205,155,100,157,56,111,226,150,77,214,113,188,155,77,239,174,113,73,108,89,86,165,168,198,34,137,189,3,32,65,244,58,131,233,253,222,247,143,123,
  231,206,189,83,0,144,34,41,80,154,195,207,16,131,193,180,123,126,207,57,207,115,126,79,203,187,223,52,249,19,138,153,252,13,135,35,124,227,107,95,
  195,233,114,221,212,215,33,43,50,217,108,214,84,119,82,49,196,95,139,98,150,172,136,13,207,95,231,124,140,67,21,243,155,31,115,163,97,128,194,130,
  152,3,60,249,227,31,179,106,205,218,27,63,87,178,140,195,238,160,182,214,60,87,163,99,163,36,162,49,230,130,115,166,16,48,243,92,137,202,135,114,
  97,80,71,250,252,104,175,41,227,174,191,146,185,154,215,199,90,196,164,93,205,72,36,18,188,248,194,11,116,116,118,45,25,217,157,154,158,34,28,156,
  99,66,139,85,93,140,236,150,226,97,14,209,187,238,120,220,40,191,250,53,192,252,106,199,117,157,153,158,222,94,122,122,123,217,125,219,109,156,61,119,
  142,185,185,57,90,91,90,216,182,109,59,179,179,179,164,211,105,132,16,108,220,184,145,45,91,182,84,204,158,116,58,173,56,157,78,36,33,169,46,250,
  76,134,76,58,163,109,108,50,2,129,197,162,214,28,181,90,44,216,157,110,36,205,165,111,179,219,104,105,105,193,34,73,100,115,57,102,3,1,82,169,
  20,53,181,181,180,183,183,99,183,217,53,215,134,172,151,141,80,76,101,164,48,151,148,202,219,167,74,254,196,100,60,205,83,244,179,52,177,169,58,110,
  252,240,122,189,108,217,186,149,198,198,70,218,59,59,185,52,168,150,251,216,189,251,54,132,36,17,137,68,144,101,25,155,213,198,251,223,255,126,220,78,
  103,249,147,155,69,194,229,178,99,179,90,85,247,97,54,171,150,33,73,103,116,182,68,18,106,241,110,139,213,138,205,106,195,106,115,168,11,205,34,81,
  95,95,143,219,229,66,8,65,52,22,99,108,124,130,186,186,58,26,26,26,104,104,108,208,101,80,47,99,38,23,18,68,20,69,193,227,245,178,125,199,
  118,154,154,155,233,90,190,226,166,189,14,245,49,153,92,54,91,96,201,138,24,50,227,122,195,24,87,136,97,141,205,211,117,164,138,249,91,3,115,227,
  115,48,188,87,37,23,253,29,119,222,201,242,21,43,80,128,115,23,206,99,179,90,121,224,129,7,0,152,155,155,211,179,180,63,254,83,31,175,60,87,
  86,9,175,199,137,195,174,186,225,51,153,180,90,114,48,149,214,25,46,139,100,193,106,181,98,179,219,112,56,28,216,29,110,125,174,154,154,154,240,123,
  189,72,66,109,218,114,249,242,16,53,181,181,180,181,183,209,221,221,173,207,207,188,115,101,188,214,34,189,163,24,23,65,165,88,165,5,230,234,70,12,
  151,203,69,79,79,15,117,117,117,75,110,13,54,54,53,226,243,251,240,251,253,139,146,221,188,174,47,200,163,114,211,225,177,212,199,117,101,70,255,251,
  111,255,54,61,189,189,156,61,119,142,63,248,236,31,226,245,122,185,109,215,46,90,91,91,185,56,48,128,44,203,248,188,62,154,26,91,176,219,221,100,
  178,50,86,75,105,255,93,73,8,26,234,107,24,30,178,144,201,100,72,38,147,106,224,177,44,171,66,104,181,96,179,217,176,90,213,13,194,98,177,34,
  231,82,4,3,179,160,64,91,91,43,211,211,211,204,204,206,18,139,197,24,155,24,231,157,143,188,11,175,215,203,147,63,254,49,131,151,46,105,25,157,
  121,110,180,208,157,169,92,79,86,165,240,95,33,166,68,165,64,139,130,128,138,132,145,155,224,116,244,22,28,91,182,110,229,15,255,232,143,8,4,131,
  124,237,235,223,96,120,100,132,77,27,55,178,101,203,102,162,145,8,177,168,26,183,212,212,212,76,67,99,43,169,116,22,171,213,130,69,42,106,103,38,
  192,231,117,227,118,187,144,21,133,84,42,69,50,153,36,151,85,131,230,11,114,104,85,95,111,177,98,179,89,200,229,50,36,18,113,181,157,96,107,11,
  201,68,146,100,58,197,232,232,8,183,221,122,43,91,182,110,37,16,8,240,239,223,250,166,198,8,21,228,47,47,145,0,219,119,236,224,127,254,217,159,
  221,244,215,161,186,218,84,165,162,187,60,139,15,232,194,16,123,85,146,192,84,112,79,200,178,140,69,146,74,234,217,85,49,127,107,96,110,84,224,249,
  191,229,247,80,81,6,243,255,246,219,191,173,207,213,212,212,20,107,215,172,97,229,202,149,196,98,49,146,9,53,94,180,182,166,14,151,203,95,121,174,
  0,183,219,137,211,233,64,81,20,210,233,12,169,116,74,237,126,4,72,150,210,185,178,72,130,76,38,73,34,17,87,11,226,55,53,18,141,198,72,166,
  83,76,78,79,113,215,29,119,176,109,235,86,98,177,24,3,253,253,26,163,166,37,206,138,188,214,17,38,188,20,165,208,180,50,159,182,96,212,53,102,
  143,28,37,181,175,231,155,171,27,49,86,246,244,240,137,159,254,105,178,185,28,175,190,126,100,73,201,238,142,173,91,184,227,206,59,137,70,34,188,176,
  127,255,130,178,107,198,196,92,227,212,140,135,94,253,113,201,225,241,182,54,70,255,247,87,191,74,111,111,47,59,119,237,226,210,165,75,44,235,236,100,
  203,150,45,132,67,33,149,162,151,21,92,46,55,178,34,49,53,29,192,237,114,226,116,218,213,58,161,54,179,80,122,61,170,171,62,147,78,147,76,36,
  85,80,181,77,193,102,179,97,183,219,177,90,44,88,109,154,49,42,215,18,152,157,81,227,120,124,126,90,154,155,153,155,155,35,147,201,16,156,155,163,
  191,191,159,177,145,97,2,129,128,22,223,161,110,14,162,108,220,131,185,115,134,48,252,87,232,34,194,162,54,135,106,217,167,27,63,6,250,251,249,204,
  239,253,30,187,111,187,157,153,233,105,146,137,4,247,220,125,55,14,187,157,177,217,89,50,153,44,40,208,208,208,196,212,84,0,151,203,161,202,161,211,
  142,195,110,195,106,149,116,113,176,219,172,184,93,46,80,20,146,9,245,80,132,16,216,172,154,28,58,236,58,67,111,181,89,201,218,173,196,98,17,226,
  241,24,86,139,149,150,150,86,166,38,167,73,103,210,4,195,97,194,225,48,23,206,159,35,24,12,34,203,138,126,40,82,101,198,172,160,206,159,59,199,
  127,253,205,223,100,207,222,189,55,245,117,20,92,182,57,189,87,185,30,42,99,40,241,163,171,98,67,71,26,12,134,138,49,107,181,56,97,163,138,249,
  91,4,243,98,54,201,128,121,177,39,45,95,114,112,169,97,62,23,82,231,234,232,235,175,19,8,4,180,185,170,60,79,70,227,39,175,113,242,101,160,
  204,70,169,49,225,182,60,1,82,105,174,110,196,24,27,27,227,159,254,241,31,185,227,206,187,150,156,236,38,18,73,6,250,251,136,70,162,139,146,93,
  51,38,162,80,0,162,72,247,23,140,131,165,135,199,219,218,24,61,126,236,24,199,143,29,99,243,214,109,140,141,143,113,255,61,247,80,87,95,199,229,
  193,203,100,178,25,205,5,239,66,150,101,166,167,166,177,217,236,216,29,118,220,46,23,46,151,3,175,215,133,199,237,68,8,176,90,173,56,157,14,141,
  21,77,97,183,219,113,56,28,56,28,14,205,24,181,97,145,44,72,22,9,139,197,170,198,115,104,18,99,179,217,168,111,104,192,231,243,17,138,68,136,
  70,163,204,5,131,92,188,120,73,239,190,80,232,22,96,188,73,134,13,82,148,33,224,21,115,246,227,18,223,28,222,174,99,98,98,130,137,137,9,108,
  118,59,115,115,115,120,60,30,122,123,123,136,197,98,196,99,113,61,0,190,182,174,158,217,153,105,172,54,27,54,155,26,52,239,114,187,240,122,156,106,
  70,167,205,138,16,224,246,184,200,229,114,36,146,9,80,212,3,149,211,233,192,110,87,155,45,216,44,86,77,14,213,196,57,187,221,174,139,131,223,239,
  167,190,190,142,72,44,74,60,22,35,22,139,49,116,249,50,225,112,88,235,156,33,12,50,105,14,164,25,27,29,101,124,108,12,167,211,69,52,145,184,
  105,175,3,212,50,63,217,108,6,69,235,176,162,26,32,134,14,51,198,85,180,128,107,75,209,222,79,24,14,175,85,204,223,26,152,155,220,249,162,40,
  83,89,150,75,50,118,143,31,59,70,93,125,253,210,194,60,30,35,22,143,51,59,59,75,36,18,38,223,233,167,68,229,204,227,48,51,234,25,12,109,
  84,77,93,128,42,16,32,149,230,234,122,143,96,32,64,48,16,160,73,35,130,150,146,236,198,227,113,166,38,39,137,199,227,139,146,221,74,7,5,69,
  99,83,111,6,60,222,182,198,168,215,235,165,167,183,23,64,173,207,150,201,176,178,167,135,120,44,78,36,26,45,244,154,7,146,201,20,217,108,0,139,
  197,130,36,73,122,220,135,199,235,193,239,115,227,112,168,108,169,215,235,33,153,76,226,243,249,240,120,60,216,53,67,84,117,205,91,180,54,94,2,73,
  146,112,56,180,12,71,212,120,18,175,215,75,93,93,45,193,80,136,68,34,137,197,98,161,161,161,158,108,54,75,40,20,214,133,208,227,117,99,179,217,
  145,36,161,209,233,82,161,7,176,36,144,132,249,103,190,109,88,254,179,133,16,76,79,77,149,21,70,19,75,80,117,215,223,176,209,211,219,139,215,235,
  165,185,185,133,201,233,25,154,155,155,177,219,29,76,79,79,147,209,154,37,40,90,162,218,220,92,80,197,219,98,65,178,88,176,90,109,184,92,78,188,
  30,55,30,183,19,135,83,149,195,116,38,141,205,106,197,83,95,143,203,237,86,55,68,141,149,55,202,97,190,254,96,254,51,28,118,7,245,13,13,140,
  79,78,18,79,36,72,36,18,212,214,213,97,181,218,8,135,195,100,179,25,243,129,72,150,116,69,181,102,205,90,252,126,63,45,173,173,204,245,245,221,
  180,215,145,151,255,92,206,192,146,137,242,161,86,70,247,109,137,177,98,88,83,178,34,35,41,22,253,161,42,230,111,13,204,231,139,179,147,21,5,139,
  65,14,30,122,248,97,90,91,91,241,215,214,242,236,115,251,232,236,236,196,106,181,49,53,53,165,23,75,87,51,217,97,102,118,70,205,184,182,72,72,
  66,157,51,135,93,53,132,220,46,23,110,143,11,191,223,79,42,149,196,229,116,82,95,87,135,203,229,198,102,183,106,110,121,73,175,231,153,247,114,5,
  2,211,122,41,40,135,195,65,115,75,51,19,83,83,68,99,49,18,241,56,221,221,221,164,210,41,134,46,15,145,72,196,75,137,15,121,17,70,144,209,
  43,183,24,2,164,194,92,221,136,209,222,222,142,211,229,162,166,166,118,73,174,65,143,199,139,197,98,33,30,143,147,203,229,230,149,221,114,30,210,155,
  13,143,183,173,49,218,211,219,203,159,125,249,203,0,60,183,239,121,108,86,43,115,193,32,118,155,141,108,38,171,7,11,167,82,41,66,225,144,169,159,
  106,62,1,40,31,244,174,198,237,164,240,122,61,124,224,3,239,35,28,142,16,143,171,84,125,46,151,211,179,51,213,172,53,53,176,88,47,219,164,61,
  174,23,125,213,234,148,250,252,62,118,223,118,59,129,192,44,175,188,244,50,72,170,16,118,118,118,105,27,172,85,191,89,44,90,44,170,241,190,197,162,
  26,192,22,139,186,49,25,126,126,251,27,95,47,43,140,166,223,12,167,253,234,184,190,227,87,126,245,87,217,178,117,43,253,3,3,156,58,115,134,76,
  58,205,204,244,52,137,120,92,143,95,147,101,181,55,114,76,138,155,138,3,43,69,9,108,185,92,142,84,58,201,230,205,155,216,182,117,11,177,152,186,
  177,101,115,90,153,49,12,29,105,20,181,12,136,211,233,212,131,226,133,33,51,92,45,81,146,101,227,198,77,8,33,120,229,229,151,153,157,157,213,15,
  70,234,89,69,211,80,10,252,63,255,237,183,216,121,203,45,244,15,12,112,225,207,191,114,211,94,135,250,125,100,205,64,144,205,157,211,16,165,14,133,
  10,91,122,241,230,111,12,127,169,98,254,214,192,188,80,94,175,104,239,44,131,249,67,15,63,204,150,173,91,233,235,235,231,217,231,246,17,139,70,9,
  204,206,146,78,165,10,165,147,20,133,88,60,166,26,146,139,153,171,45,155,216,190,125,219,162,230,202,97,183,23,230,10,65,46,155,211,223,51,39,231,
  88,214,213,133,16,130,169,201,41,98,177,168,70,100,128,144,64,202,207,149,193,32,21,198,42,46,134,121,161,44,59,90,20,171,88,102,174,100,69,209,
  11,199,223,136,241,238,71,31,101,101,79,15,57,89,166,111,96,96,201,201,238,138,21,43,64,8,206,158,62,77,56,18,153,87,118,5,69,29,9,174,
  1,30,213,112,189,107,97,140,42,44,88,108,124,160,175,159,223,252,244,175,3,240,232,79,252,4,137,100,146,254,254,126,124,62,63,146,36,233,11,53,
  149,74,17,141,70,17,154,251,90,173,69,39,147,203,101,73,167,211,36,147,105,146,137,56,217,108,142,222,222,149,52,53,53,210,179,114,57,153,76,150,
  104,44,73,40,20,37,30,79,144,76,169,1,205,185,156,154,173,25,141,69,77,53,234,162,209,40,211,51,51,250,201,41,24,8,114,234,196,73,181,160,
  178,162,32,100,5,33,36,46,15,14,50,54,58,170,179,161,198,155,48,222,23,66,255,41,36,73,235,16,37,52,163,186,98,212,8,166,130,248,178,218,
  71,247,109,63,148,235,251,218,255,253,23,95,197,235,245,178,102,221,58,114,185,28,99,227,227,76,207,204,224,118,185,245,77,1,69,33,18,137,96,179,
  217,212,14,31,178,162,215,179,203,103,110,38,19,73,146,201,4,46,151,139,237,219,182,177,102,117,15,146,36,145,78,103,9,204,69,136,68,162,36,19,
  73,50,217,52,178,86,7,47,149,74,33,132,164,31,138,210,153,52,83,83,83,164,82,41,181,127,49,130,19,199,142,17,137,70,9,135,195,106,75,59,
  109,87,148,145,212,51,146,36,131,44,241,39,95,248,34,126,191,159,205,91,183,222,212,215,65,190,147,154,161,102,163,222,5,141,210,154,42,37,169,75,
  198,13,221,36,3,50,138,172,174,201,42,230,111,13,204,11,25,204,230,189,179,240,119,69,45,203,131,208,245,205,221,247,220,179,36,231,234,197,253,251,
  9,6,131,134,78,62,178,238,19,86,242,115,165,25,164,66,50,148,122,146,11,132,157,144,140,138,197,56,23,121,67,73,179,126,202,184,134,213,88,107,
  229,134,149,238,249,193,127,252,39,78,151,139,205,91,182,44,73,60,46,14,12,144,72,36,136,199,227,11,202,174,34,25,4,180,24,15,131,212,86,196,
  99,1,217,173,234,243,235,200,140,70,163,81,142,31,59,6,192,251,63,244,24,126,127,13,99,227,227,180,119,76,82,95,87,79,58,147,6,69,33,149,
  74,18,141,70,176,88,172,90,221,208,28,153,108,142,108,38,67,50,153,34,157,78,145,205,100,241,120,84,247,121,52,26,163,181,165,81,11,5,240,80,
  95,95,67,50,153,33,28,137,147,72,36,72,167,82,100,179,89,102,3,179,58,211,154,74,165,152,153,153,33,26,143,35,73,18,54,171,149,64,32,192,
  236,204,140,97,254,4,40,50,177,88,76,53,44,243,6,102,222,224,52,26,159,243,220,200,255,44,82,152,58,131,99,150,198,234,233,232,6,140,129,254,
  126,0,234,27,26,112,185,93,100,50,25,134,134,134,232,213,194,72,242,114,18,14,207,225,114,121,84,57,204,230,244,122,118,233,84,138,84,58,69,38,
  173,182,254,171,175,175,39,22,75,32,73,18,94,175,27,69,1,175,215,77,58,211,64,36,154,36,22,139,147,74,38,201,100,50,76,79,79,145,205,21,
  92,82,193,96,144,192,220,28,217,92,14,167,86,202,100,100,100,132,172,150,25,106,92,207,146,44,35,75,146,166,163,20,206,159,61,139,144,36,154,91,
  91,111,234,235,16,218,65,44,151,203,22,74,215,136,124,143,111,81,185,104,114,81,121,39,81,214,253,37,99,17,150,42,230,111,17,204,77,134,106,5,
  208,139,227,133,37,73,90,146,115,21,137,68,10,251,190,225,114,138,231,202,248,93,213,136,5,51,21,167,136,34,163,221,88,139,183,178,103,184,236,92,
  93,207,49,54,54,6,192,242,21,43,150,36,30,51,51,51,154,123,126,97,217,213,19,238,132,40,107,120,41,98,1,60,22,41,187,85,102,244,58,141,
  79,252,204,79,211,211,219,139,203,229,166,163,163,131,190,11,231,105,105,110,166,161,161,65,173,25,42,231,72,38,85,99,212,102,115,144,201,168,194,152,
  201,169,117,68,51,233,180,150,233,38,112,186,92,72,146,68,60,158,212,25,69,139,69,194,101,81,51,239,60,30,39,233,116,134,68,82,53,98,95,127,
  253,128,46,240,225,112,152,145,145,17,100,89,198,110,183,99,183,219,105,111,107,195,231,245,50,57,57,73,96,118,182,80,62,70,23,68,25,69,146,10,
  89,158,90,24,193,124,134,163,177,44,73,33,194,89,41,13,206,55,233,215,170,49,122,189,199,150,173,91,185,227,174,59,233,93,181,138,31,60,254,56,
  66,146,24,232,239,167,163,163,3,151,211,133,172,133,122,132,66,115,40,138,208,10,42,103,245,154,182,233,84,90,85,162,128,195,174,38,205,165,82,41,
  50,217,172,14,175,221,110,197,102,183,226,116,216,169,173,241,144,74,101,72,36,83,4,2,51,164,146,9,64,141,157,30,30,30,38,166,53,123,112,57,
  157,88,36,137,205,91,182,16,9,135,233,235,235,43,200,4,165,29,105,118,220,122,43,247,222,127,31,235,54,108,228,71,63,126,226,166,189,14,53,225,
  72,205,172,86,221,104,104,173,33,133,201,64,81,22,90,22,197,97,46,249,178,44,138,194,214,109,219,170,152,191,21,48,95,160,54,163,209,157,191,117,
  219,54,30,122,199,195,180,181,183,243,196,143,159,68,146,44,156,60,121,130,182,214,86,188,94,159,202,154,201,10,179,179,51,212,212,200,90,189,202,156,
  214,123,62,77,58,165,182,145,68,8,156,78,23,78,167,75,37,56,210,230,185,178,219,173,120,61,78,100,165,134,84,42,71,60,145,34,24,156,33,17,
  143,1,144,201,100,232,235,235,35,18,137,168,181,98,221,110,172,22,11,59,118,238,36,48,59,171,207,149,209,40,53,206,85,177,241,83,82,6,203,168,
  91,192,212,20,162,172,205,83,166,162,203,141,208,57,43,123,122,216,176,97,3,107,214,173,227,245,99,71,151,148,236,10,96,197,202,110,18,241,4,99,
  163,163,11,202,46,74,233,33,193,0,72,65,239,47,6,143,50,178,91,181,1,174,115,209,251,45,91,183,178,101,235,86,82,201,36,155,54,109,34,18,
  139,49,60,50,194,212,212,20,118,135,67,115,211,39,137,70,163,196,98,113,226,241,152,122,139,197,73,37,83,228,114,50,160,96,119,56,112,187,93,8,
  73,34,26,141,17,12,197,204,23,33,9,236,54,11,94,143,147,166,6,31,157,237,13,92,184,112,65,119,209,143,143,143,19,210,78,165,54,171,21,159,
  219,141,197,98,97,249,242,229,122,76,146,105,115,48,182,250,204,159,226,139,30,43,180,11,51,70,45,40,165,133,238,23,193,102,43,84,11,225,94,239,
  241,254,15,124,0,69,81,232,236,232,164,177,161,145,201,169,41,6,7,7,145,21,25,132,132,172,40,68,35,33,162,209,184,154,237,25,143,17,143,197,
  72,196,19,122,242,131,213,98,197,229,118,97,119,216,73,103,50,204,6,34,106,121,18,3,75,97,181,74,184,156,54,106,107,220,180,181,212,145,205,36,
  152,157,157,69,150,101,194,225,48,211,51,51,100,52,38,169,214,231,71,0,117,117,117,212,213,215,155,147,116,148,210,6,12,138,162,240,177,143,127,28,
  69,150,111,238,235,160,16,71,39,27,226,7,141,44,88,62,254,219,116,51,42,135,121,252,66,249,103,84,49,127,11,96,174,24,219,162,26,169,40,165,
  8,113,245,247,59,238,188,147,112,56,76,75,75,11,13,13,13,68,34,81,134,134,135,201,201,57,16,106,175,249,100,34,70,34,145,34,145,72,146,74,
  37,72,37,19,164,146,105,77,223,128,213,162,86,110,177,217,173,100,115,57,2,115,230,185,210,245,142,16,184,156,86,26,234,60,100,210,9,173,196,149,
  76,36,18,33,18,141,146,147,213,247,243,123,188,160,40,88,44,150,146,185,98,158,185,2,179,158,49,25,151,37,70,145,241,177,162,185,42,254,188,27,
  166,111,20,238,184,235,78,82,169,228,146,147,93,57,151,195,231,245,225,245,122,23,45,187,24,108,3,22,196,131,202,120,84,144,221,42,51,122,149,66,
  182,152,9,252,141,79,127,26,128,251,31,124,144,205,155,54,33,203,50,67,195,195,180,182,182,208,213,217,69,44,22,69,201,229,72,39,19,200,57,5,
  89,65,61,45,201,178,142,151,197,106,197,97,183,99,179,217,16,66,144,74,165,137,70,98,212,215,122,43,230,254,244,247,247,51,54,54,162,149,18,201,
  50,62,54,174,159,40,157,78,39,53,181,53,28,58,116,136,108,54,107,168,107,167,138,181,200,11,159,44,33,75,10,146,214,234,11,69,209,90,132,41,
  152,95,164,20,86,132,30,169,108,174,73,170,20,197,66,41,21,41,213,183,235,120,99,189,233,23,26,199,142,29,229,222,187,239,70,81,20,126,227,183,
  254,27,109,109,109,28,57,122,132,115,231,207,177,106,213,42,172,22,11,153,76,154,92,38,77,34,30,65,65,210,229,48,223,106,78,237,123,172,214,179,
  85,155,42,228,136,68,162,164,51,13,216,237,214,138,161,42,125,125,23,8,135,67,200,57,153,217,153,89,18,137,164,238,74,108,104,168,103,102,102,134,
  67,135,212,214,180,194,228,238,49,180,145,67,149,195,87,15,31,98,243,250,245,8,33,248,195,207,127,225,166,189,14,69,171,164,145,203,229,76,46,91,
  99,59,189,197,172,135,66,153,27,180,134,19,234,39,42,138,82,197,252,45,130,185,249,80,111,200,10,49,238,164,218,62,124,236,216,81,30,125,228,145,
  37,137,249,196,228,4,151,7,7,43,207,149,76,65,223,228,139,168,43,133,190,148,106,219,233,249,22,131,48,173,129,138,115,165,186,14,84,54,250,122,
  135,71,13,12,240,223,127,235,183,150,36,30,193,96,144,75,151,46,46,90,118,85,60,20,195,212,94,201,252,137,5,101,247,70,224,177,212,245,249,13,
  169,45,52,57,62,206,154,213,171,105,106,106,34,151,203,17,141,198,200,100,51,88,45,22,45,83,46,77,38,147,100,203,150,141,108,216,184,150,218,186,
  90,44,22,9,201,34,97,183,219,176,218,109,88,44,170,224,229,114,89,18,137,20,153,108,174,236,103,197,227,113,254,253,59,223,65,206,169,167,239,104,
  36,66,68,115,159,88,44,22,60,110,55,169,100,74,43,67,162,148,112,42,102,35,39,127,74,210,30,81,138,95,97,216,42,245,0,123,165,204,201,104,
  126,152,170,45,194,110,204,16,66,144,77,167,232,238,94,129,195,225,32,17,83,227,140,45,86,11,104,44,70,42,21,197,231,243,114,219,237,183,210,181,
  172,19,167,211,129,144,132,222,250,207,106,179,169,165,67,80,72,167,210,196,19,233,178,159,149,203,229,184,208,215,199,161,195,135,80,100,133,84,58,69,
  56,18,38,155,83,79,231,46,151,11,167,211,73,40,20,50,155,213,138,217,208,202,51,36,198,246,115,0,201,120,252,166,190,14,69,81,212,44,88,83,
  43,93,197,192,144,45,194,69,63,207,218,50,186,57,171,152,223,220,152,43,149,152,167,10,251,168,207,231,67,206,102,217,176,126,61,30,143,135,208,92,
  136,169,169,41,36,139,164,207,85,50,25,97,237,218,85,188,251,209,119,176,99,199,86,124,126,175,154,79,160,21,80,183,217,109,88,164,60,147,154,36,
  18,77,150,253,204,68,34,193,161,67,135,56,120,240,32,178,44,19,139,199,8,4,3,250,92,213,248,253,120,61,94,210,169,212,252,115,85,172,111,244,
  234,48,70,198,110,145,106,255,10,230,234,70,237,187,177,72,100,73,201,238,92,48,120,197,107,176,24,143,69,185,61,151,32,30,111,49,102,116,241,227,
  191,255,191,255,47,15,61,252,48,103,207,158,99,215,173,183,242,248,227,143,211,215,223,79,123,91,27,107,214,172,97,46,149,194,231,245,114,207,189,247,
  114,215,93,123,176,89,173,60,241,228,83,28,56,240,42,153,116,6,155,205,161,118,99,176,168,53,4,115,185,28,241,68,156,68,34,133,205,234,54,177,
  163,179,179,179,252,224,7,63,224,226,192,128,78,173,95,232,187,160,47,8,175,219,77,119,215,50,154,154,26,185,231,222,123,121,237,213,87,213,184,30,
  189,165,23,133,178,12,228,147,139,242,2,41,180,251,197,177,163,165,17,202,166,28,37,202,196,47,151,173,135,95,141,27,185,158,163,183,183,151,63,251,
  243,63,71,150,101,2,115,115,52,55,55,51,58,60,204,225,195,135,121,224,254,251,1,181,244,201,178,142,14,126,241,23,127,9,159,215,203,196,228,36,
  255,254,157,239,51,116,121,164,208,233,203,170,214,24,84,20,133,84,58,73,56,28,165,177,193,103,42,153,146,78,167,57,125,230,52,223,253,206,119,213,
  182,181,138,194,204,204,12,51,179,179,250,233,124,121,71,7,141,245,245,52,174,90,197,248,216,24,167,78,157,170,72,151,23,186,118,41,172,89,187,142,
  191,255,231,127,70,81,20,34,177,232,77,123,29,249,16,154,66,137,51,227,2,212,218,241,45,138,33,173,148,25,160,208,187,106,85,21,243,155,28,243,
  202,74,186,20,119,69,81,88,165,97,174,40,10,51,129,89,234,235,235,25,25,30,230,236,217,179,116,118,116,20,230,170,179,147,15,188,255,189,250,92,
  5,130,65,134,18,35,122,57,63,155,197,170,215,171,204,100,211,196,98,113,100,165,166,100,174,206,157,63,199,190,125,207,171,117,51,21,133,185,185,57,
  230,52,35,93,146,36,218,154,155,169,173,241,211,184,178,155,134,134,6,109,174,138,148,13,198,82,63,6,125,99,170,17,84,96,134,243,61,2,10,57,
  10,230,247,89,236,92,221,8,125,211,214,222,206,47,252,226,47,226,116,185,136,37,226,75,66,118,107,124,62,26,107,235,240,7,106,184,124,121,112,113,
  178,75,41,30,121,121,53,226,161,59,105,150,40,30,75,219,24,189,18,139,92,81,174,200,218,7,232,239,235,35,26,137,112,238,236,89,182,108,222,204,
  235,71,142,16,152,157,37,18,137,106,46,243,90,246,222,115,15,119,222,121,39,46,167,131,68,50,133,197,98,197,106,177,34,91,101,44,22,9,139,100,
  209,51,206,100,69,214,50,237,211,184,221,78,108,86,73,85,54,129,0,63,122,226,71,188,252,202,43,68,180,158,183,217,108,150,81,205,69,111,181,90,
  169,175,171,199,237,114,113,225,220,121,154,90,154,73,36,18,134,133,92,228,234,200,151,199,80,242,129,205,249,13,66,207,146,88,200,249,206,188,85,157,
  171,7,161,242,242,181,88,121,84,148,43,146,195,137,241,113,142,31,61,202,139,47,190,72,115,75,43,171,87,173,98,102,102,70,173,125,107,183,131,16,
  44,91,182,140,79,126,242,231,169,171,173,33,151,83,147,45,108,54,59,22,171,5,171,213,130,213,162,118,248,202,227,39,231,114,164,146,73,82,169,28,
  46,167,85,103,74,206,157,63,199,247,191,247,125,6,6,6,116,5,28,10,133,244,108,90,159,215,75,83,67,35,83,19,147,196,162,106,89,146,252,145,
  92,21,65,109,51,204,187,47,181,235,84,20,133,209,145,81,94,59,124,152,231,158,125,150,101,43,186,111,218,235,80,223,175,80,27,80,81,20,109,109,
  229,25,135,130,139,178,60,246,162,232,168,87,186,200,170,152,191,5,48,87,138,98,50,202,29,62,12,34,144,199,252,137,31,253,136,214,246,142,37,51,
  87,161,96,144,108,54,103,238,60,165,25,164,198,185,50,26,64,249,154,217,66,20,230,71,125,29,11,196,126,137,69,205,21,55,128,141,11,6,2,92,
  28,24,224,244,169,83,172,223,184,105,73,224,49,61,57,133,213,106,81,179,244,23,41,187,66,41,34,165,110,82,60,150,172,62,191,17,204,232,119,190,
  253,109,64,205,104,123,247,123,223,199,173,59,119,242,212,211,79,51,50,58,66,50,153,100,247,238,219,184,253,246,59,112,187,212,210,15,23,250,46,112,
  244,200,49,162,209,40,46,183,27,155,205,138,213,102,213,203,42,41,57,153,108,86,53,70,51,153,28,138,156,101,124,124,156,39,159,124,146,87,95,125,
  85,55,68,21,69,33,24,8,168,133,126,37,137,26,159,143,229,157,157,76,77,78,50,51,61,205,216,216,104,65,48,202,81,153,5,146,30,49,63,207,
  201,181,8,250,172,158,140,174,239,136,70,163,252,143,223,253,93,100,89,166,107,217,50,238,184,253,118,6,47,95,102,98,108,156,161,225,33,58,218,59,
  248,153,159,249,89,26,27,234,145,101,153,224,92,144,163,199,142,209,215,215,143,195,225,80,15,72,54,181,217,65,190,116,87,62,235,51,158,72,225,114,
  90,137,70,163,28,58,116,136,125,251,158,87,55,68,141,93,79,196,227,196,19,9,20,212,214,180,203,58,58,112,57,157,156,29,24,208,219,252,205,119,
  114,86,12,82,24,137,132,249,245,95,253,20,0,61,171,86,221,180,215,81,112,133,153,215,88,225,126,62,102,174,92,118,240,66,204,131,134,121,164,138,
  249,91,2,115,69,89,244,30,27,137,68,150,36,230,121,239,220,66,115,85,174,230,100,185,166,62,243,207,198,226,230,234,70,232,156,100,34,193,191,252,
  211,63,33,203,50,147,147,147,111,58,30,14,187,157,193,171,145,93,227,163,138,184,105,241,120,91,187,233,243,147,220,220,220,204,166,77,27,177,218,108,
  244,15,12,48,60,60,204,92,104,142,186,186,90,20,89,237,204,113,230,220,57,254,253,219,223,214,250,212,59,176,217,109,8,225,212,131,153,1,114,178,
  76,42,153,100,102,102,154,120,60,204,92,112,134,23,94,120,129,19,39,78,144,147,115,166,44,212,139,151,46,1,224,176,219,105,110,104,164,177,190,158,
  108,58,245,6,196,165,194,6,241,6,133,176,58,110,228,97,77,97,195,134,13,108,219,185,147,179,231,206,241,92,48,200,145,35,71,185,235,206,187,244,
  141,46,56,55,199,243,251,247,243,212,143,159,196,106,179,97,145,4,138,195,65,62,126,72,8,129,140,202,188,71,163,81,198,70,71,9,206,90,204,46,
  34,67,229,133,64,48,200,92,48,136,16,130,250,154,26,154,27,155,112,185,156,184,92,106,233,24,33,174,92,78,20,69,97,211,230,77,220,114,211,94,
  135,242,134,200,0,61,121,201,148,180,81,242,238,106,66,66,21,243,155,24,115,229,170,215,249,173,187,118,177,121,203,86,38,38,38,249,209,143,159,224,
  245,215,143,176,247,238,189,196,162,81,210,181,181,140,142,141,242,163,31,253,136,99,199,142,97,145,44,200,57,55,86,171,132,156,179,233,245,39,101,173,
  49,203,244,244,20,193,224,52,153,116,146,190,190,11,28,60,120,80,119,205,231,111,147,211,211,76,79,79,99,177,88,104,107,106,166,189,165,21,183,203,
  133,207,231,211,216,185,183,175,62,80,20,133,166,166,38,86,175,90,197,246,109,219,222,52,217,69,81,203,59,166,211,233,171,90,131,213,113,147,27,163,
  0,159,253,163,63,162,174,174,142,241,177,113,126,226,189,239,227,71,79,60,193,201,147,167,176,219,190,199,192,192,69,252,53,126,14,28,56,64,48,16,
  64,1,50,217,12,137,68,140,76,186,6,89,134,96,192,139,195,97,199,106,181,161,40,50,201,100,130,104,36,204,92,104,142,100,50,169,211,242,153,76,
  134,185,185,57,102,102,102,24,29,31,199,106,181,210,213,209,65,71,91,27,178,156,99,213,234,213,100,50,89,38,198,199,150,228,130,173,178,163,215,119,
  220,115,239,189,252,204,207,253,28,231,206,158,229,190,123,238,69,145,21,78,156,56,206,211,207,60,195,236,236,44,155,54,109,230,210,224,37,142,28,57,
  162,150,162,73,229,72,165,146,36,19,17,34,97,39,86,171,141,25,183,27,135,221,142,16,18,217,92,134,100,60,74,32,56,167,102,111,106,221,101,114,
  185,28,137,120,156,169,233,105,70,70,71,137,198,227,212,248,253,244,172,232,198,34,4,181,181,181,108,223,177,131,151,95,122,233,170,174,227,161,119,60,
  204,47,255,234,175,114,250,212,105,238,221,123,207,77,122,29,202,117,94,79,234,129,177,138,249,219,9,115,117,15,125,228,221,239,230,177,15,125,136,215,
  95,123,157,237,219,182,17,8,4,56,113,226,56,63,126,242,73,166,166,166,244,185,58,122,244,104,65,119,100,67,228,178,41,226,177,8,86,171,141,64,
  96,250,138,231,42,149,78,227,247,249,232,108,111,71,145,101,60,94,15,219,119,238,96,255,190,231,223,214,250,102,211,230,205,60,240,224,131,140,142,142,
  190,105,107,80,206,102,241,248,124,244,174,234,229,204,233,51,85,253,255,118,49,70,155,155,155,185,253,142,59,249,254,247,190,139,162,40,252,195,223,253,
  29,147,147,147,92,186,120,145,15,125,244,99,220,189,103,15,255,254,221,239,240,189,239,127,159,131,135,14,98,183,217,176,217,109,212,213,213,227,118,187,
  181,94,178,130,192,172,218,73,41,16,152,54,157,66,243,224,201,57,153,116,38,77,40,20,34,28,10,17,139,199,213,110,11,193,32,54,171,149,166,198,
  70,150,119,118,50,59,61,195,133,115,231,232,90,182,140,233,169,169,121,191,123,93,125,61,14,187,131,201,169,201,138,207,89,156,204,84,13,203,165,54,
  14,29,60,136,203,229,226,135,63,248,1,203,87,172,224,190,251,31,32,153,76,242,228,83,79,113,238,236,57,26,26,159,34,147,78,227,175,241,83,83,
  83,139,205,102,3,32,22,143,17,141,69,245,204,215,124,207,227,124,54,176,44,171,101,107,98,177,24,161,185,57,34,209,168,218,130,118,122,154,116,54,
  139,223,235,165,179,189,29,143,203,69,127,95,31,99,163,163,56,93,206,43,250,238,189,171,86,51,59,51,67,56,28,226,229,23,95,196,227,245,242,189,
  239,124,151,222,85,171,120,231,163,143,46,185,235,176,88,44,172,236,233,165,79,75,34,44,93,27,98,222,117,116,85,251,178,57,171,227,166,199,188,112,
  89,202,77,129,249,194,251,225,149,98,46,174,138,81,252,225,127,254,39,209,72,132,125,207,61,71,115,115,11,247,60,112,255,13,159,171,243,231,206,113,
  249,242,101,92,78,231,27,218,179,76,115,114,147,234,157,11,231,207,99,183,219,121,245,240,225,55,101,13,186,157,78,6,250,85,215,191,221,110,191,193,
  87,95,181,3,222,52,99,244,246,59,238,164,103,213,42,82,169,164,110,52,30,62,116,72,183,252,103,103,166,185,243,142,59,152,154,158,230,192,129,3,
  156,59,119,158,84,42,73,109,109,13,117,117,245,120,189,94,234,106,107,169,111,104,192,102,181,233,239,97,52,68,179,153,44,241,68,156,208,92,136,224,
  92,144,217,217,89,34,145,8,105,173,215,188,221,110,167,185,177,145,158,229,43,104,168,171,103,124,68,237,178,48,60,52,164,47,112,167,211,137,211,229,
  82,169,124,77,121,182,119,116,224,245,122,153,155,11,206,35,84,162,42,108,55,225,80,20,133,88,44,198,227,63,248,1,0,211,83,83,44,95,214,197,
  131,15,60,64,32,48,203,249,243,231,57,223,119,1,135,221,70,67,67,3,53,53,181,248,188,94,26,155,154,112,185,92,88,36,75,201,134,168,104,110,
  188,104,52,74,96,54,192,108,96,150,80,40,68,60,30,39,43,203,88,44,22,149,41,105,107,103,121,103,167,90,87,47,151,83,11,99,71,34,21,98,
  151,204,191,187,93,110,186,87,174,196,235,243,233,45,108,163,209,24,223,251,206,119,16,8,38,38,198,233,234,236,124,83,175,163,181,173,149,201,137,194,
  225,205,239,247,211,210,210,90,121,227,23,133,24,58,99,84,150,40,90,99,162,146,189,89,33,110,180,184,246,168,162,112,83,98,94,110,44,53,204,175,
  248,58,174,22,115,113,229,225,12,0,207,63,247,28,0,46,151,147,237,219,183,107,108,231,141,155,43,80,99,38,147,137,196,85,99,94,233,57,226,38,
  219,119,147,201,36,175,29,62,12,168,73,102,14,155,245,134,201,110,71,107,43,177,72,4,89,150,73,36,18,90,120,201,213,235,241,114,135,131,170,5,
  176,68,141,209,80,40,196,191,252,227,63,240,216,71,62,82,242,183,187,247,238,229,103,63,249,73,250,206,159,103,121,103,39,246,187,239,230,149,3,175,
  48,56,56,72,40,28,33,20,10,99,179,218,168,175,175,167,190,126,134,214,54,181,149,155,213,98,69,65,209,5,106,122,106,138,153,217,89,130,193,32,
  209,88,76,239,174,32,132,192,229,112,208,209,222,78,87,71,39,169,120,12,135,211,193,142,91,118,242,250,171,175,17,141,70,116,41,178,88,44,216,109,
  54,93,146,44,22,11,193,64,128,185,96,16,175,207,135,48,252,171,220,62,89,152,78,253,111,72,48,23,23,136,90,29,215,96,120,60,30,254,199,239,
  255,1,157,203,186,56,125,242,20,15,62,240,32,110,183,155,35,199,142,17,158,11,49,57,57,197,228,228,52,30,183,155,150,96,80,149,199,134,122,156,
  78,23,146,214,201,37,157,78,19,141,70,25,31,27,83,227,147,230,66,36,83,73,157,199,177,90,44,52,214,215,211,209,222,142,207,227,37,147,73,211,
  217,217,137,69,146,56,125,234,116,217,13,177,28,9,98,181,90,185,120,241,34,237,237,237,5,241,208,126,122,188,94,62,255,197,47,177,188,123,5,39,
  79,28,231,129,251,31,120,83,174,195,233,116,149,204,241,165,139,23,89,190,98,133,97,133,152,255,73,146,40,53,78,174,64,252,197,162,150,148,114,83,
  98,110,218,123,88,154,152,95,233,117,92,21,230,37,93,67,196,21,97,190,98,197,10,126,239,247,255,0,89,206,113,246,212,73,62,242,225,15,115,224,
  224,65,94,120,241,69,198,199,199,25,29,159,96,124,124,18,151,211,137,223,239,163,190,190,158,214,182,54,124,62,31,22,139,5,20,72,36,213,238,74,
  131,131,131,204,133,195,36,147,73,114,217,172,62,87,54,155,141,142,214,86,186,58,58,177,91,173,100,179,153,107,131,185,73,231,136,34,45,179,8,131,
  125,49,186,228,6,235,28,135,211,201,71,62,242,17,188,94,47,151,7,47,95,247,53,232,117,123,136,70,34,52,52,52,128,16,12,93,190,252,6,241,
  16,111,41,60,110,122,99,116,161,140,253,83,39,79,234,52,133,162,160,39,32,237,189,231,30,254,203,175,252,10,3,3,23,249,226,231,62,79,52,22,
  229,238,189,247,240,137,143,127,156,35,71,142,178,255,197,23,153,154,154,36,145,74,50,54,57,193,236,236,12,211,83,211,88,109,86,186,186,186,112,56,
  28,4,131,65,198,39,38,152,11,133,72,167,211,200,134,47,98,183,217,105,111,109,101,121,87,23,181,126,63,23,7,6,8,135,195,132,195,17,118,222,
  178,147,157,183,222,194,107,175,190,70,38,147,161,173,181,21,139,85,205,210,243,248,188,204,5,231,152,11,6,73,167,211,248,124,190,50,155,164,89,8,
  149,162,211,189,169,231,146,98,104,200,164,148,230,224,43,6,217,44,215,213,237,237,199,88,94,253,235,174,228,181,178,214,19,219,227,241,240,251,127,248,
  135,116,117,45,227,255,124,245,171,236,223,183,143,230,214,22,222,253,200,187,216,178,121,51,207,237,123,158,243,23,46,16,143,199,200,68,178,196,226,49,
  38,39,39,113,187,221,212,212,212,208,210,210,66,42,149,98,98,98,130,217,64,128,104,44,70,54,87,104,194,32,73,18,126,175,143,21,203,150,209,213,
  209,65,36,20,98,120,112,144,108,46,167,179,239,66,8,78,157,58,61,111,62,167,158,77,29,141,32,73,146,182,73,9,131,113,229,229,75,127,250,63,
  89,190,162,155,255,245,167,255,147,231,158,121,150,246,246,118,222,245,158,247,220,176,235,112,185,92,204,204,204,226,116,58,89,209,221,77,50,153,100,98,
  124,156,112,56,172,111,227,122,70,170,225,235,11,33,212,107,202,107,220,162,141,126,254,253,184,56,235,85,148,237,196,84,240,168,220,124,152,155,181,220,
  210,194,252,106,175,227,106,48,55,21,197,49,86,125,18,134,125,182,248,249,26,230,221,221,221,252,254,103,63,75,78,150,249,255,62,251,135,92,190,116,
  137,161,161,33,246,220,189,151,182,214,54,158,219,183,79,159,171,104,34,78,42,157,34,30,143,19,8,4,22,63,87,62,31,43,186,212,185,10,6,2,
  244,107,73,179,146,36,45,48,87,98,81,152,151,212,109,49,40,17,81,70,109,20,235,157,5,231,234,6,236,239,249,125,215,233,112,242,145,143,126,148,
  198,166,38,30,255,225,15,57,117,226,228,117,93,131,225,185,57,134,7,7,201,201,50,146,16,52,52,212,35,128,203,67,67,87,189,6,203,226,81,198,
  160,93,202,120,44,85,125,126,77,152,209,60,3,58,61,57,201,62,205,53,82,108,87,53,55,55,243,171,191,246,107,12,12,12,240,255,253,193,31,16,
  139,199,1,193,243,251,246,209,209,217,193,174,221,183,209,218,218,202,233,211,167,57,119,225,60,147,83,83,36,51,25,166,102,103,113,216,108,36,147,41,
  114,185,44,201,84,138,100,42,165,6,44,107,27,156,221,102,163,173,165,133,142,246,118,234,107,235,8,135,66,156,62,121,82,111,247,25,141,68,120,253,
  213,215,216,121,235,45,108,219,190,141,23,247,191,192,224,224,32,30,143,7,143,215,203,212,212,148,65,230,132,46,216,42,35,42,10,39,21,81,206,181,
  244,70,104,250,42,11,250,102,140,159,249,185,159,163,187,187,155,175,254,249,87,216,255,252,243,0,76,78,76,242,173,175,125,141,45,219,183,241,129,247,
  255,4,103,78,159,225,204,185,179,92,28,28,36,18,14,19,138,68,137,39,18,196,98,106,112,124,58,157,38,145,76,146,205,102,245,3,145,36,73,212,
  213,212,208,222,218,70,91,107,11,14,155,157,161,75,151,244,78,53,2,56,115,234,52,66,8,58,58,59,9,6,231,24,27,29,45,145,137,186,250,122,
  234,235,235,17,66,48,54,54,70,42,153,84,13,59,145,103,149,84,185,249,229,95,253,21,122,122,123,249,159,95,250,18,207,61,243,44,0,227,99,99,
  252,235,63,254,35,59,110,185,229,198,92,71,71,7,227,227,19,8,33,24,212,20,177,217,127,37,74,152,6,180,235,200,151,107,43,102,200,68,201,33,
  176,12,255,32,222,90,152,87,98,101,10,243,183,132,48,191,218,235,184,26,204,133,224,74,58,37,26,159,250,216,135,63,12,66,240,135,191,255,7,90,
  43,78,13,243,175,127,157,93,183,223,198,187,31,121,132,245,235,214,114,230,236,27,153,171,86,44,66,232,115,149,191,12,227,92,141,143,141,19,8,4,
  230,103,226,132,40,139,121,121,95,155,88,132,255,120,113,115,117,35,53,208,253,15,62,64,115,75,11,143,255,224,7,156,58,121,234,186,173,65,155,197,
  90,34,187,67,67,67,32,4,13,141,13,68,99,170,107,255,106,215,96,233,94,116,109,102,241,237,158,219,127,77,140,209,111,125,253,235,11,62,103,106,
  106,138,47,125,254,11,156,58,117,138,120,44,102,2,126,116,100,148,239,124,235,91,172,90,179,134,91,119,238,100,251,246,109,140,142,142,210,215,215,207,
  228,244,20,145,112,132,72,44,74,54,147,69,86,100,108,54,27,30,143,7,175,199,67,125,109,45,126,159,159,218,26,63,177,104,140,190,115,231,72,166,
  82,37,230,121,36,18,225,181,87,95,91,208,36,20,168,253,112,245,77,83,19,68,221,181,36,68,197,140,55,81,78,56,133,184,166,2,91,29,111,124,
  252,195,223,253,61,135,14,30,226,240,161,67,166,199,147,169,20,135,94,57,128,219,115,130,237,59,119,242,174,119,62,194,204,236,44,3,3,3,140,142,
  143,17,8,4,136,197,98,132,195,17,114,185,44,66,146,112,187,221,120,189,94,252,30,47,205,77,77,120,180,140,207,201,137,9,38,39,39,203,30,19,
  79,159,60,197,92,32,88,170,204,53,89,11,6,2,4,181,56,102,157,77,18,2,73,24,12,82,69,225,47,255,247,255,225,149,151,95,225,149,87,94,
  65,2,20,77,198,146,201,36,47,191,248,34,199,143,29,227,214,93,187,110,200,117,116,119,119,151,92,135,81,238,245,239,109,184,6,189,118,160,48,175,
  47,211,138,18,243,250,194,222,50,152,207,55,87,44,81,204,175,244,58,174,6,115,99,223,240,197,30,239,243,223,250,171,95,249,11,154,154,154,24,212,
  12,81,227,119,61,244,202,1,44,86,43,235,214,175,191,234,185,82,100,153,80,32,48,239,92,141,143,142,149,24,162,229,244,131,174,99,202,96,175,63,
  207,168,123,68,17,81,114,19,80,33,207,60,245,52,231,207,157,231,194,133,11,215,101,13,218,109,54,166,38,39,43,226,49,116,249,50,177,104,25,67,
  244,10,214,160,184,18,60,174,16,155,183,187,133,112,67,74,59,229,55,136,195,135,14,85,164,165,133,16,244,157,63,207,249,179,103,169,171,175,167,107,
  217,50,182,109,221,130,221,238,80,221,39,177,56,233,116,154,156,118,34,178,72,18,233,84,138,72,56,76,36,28,102,116,104,136,92,62,110,52,191,73,
  23,27,164,225,176,233,247,88,60,78,60,30,47,221,24,12,6,64,89,65,52,62,95,44,38,246,132,178,155,79,117,188,57,35,22,139,241,170,102,148,
  24,101,37,47,143,177,88,140,23,246,237,83,11,37,175,88,65,103,123,59,43,187,87,96,183,59,136,198,212,174,51,153,76,6,89,86,93,68,22,201,
  66,120,110,142,88,44,198,200,208,144,126,216,170,36,135,0,163,70,101,46,68,121,238,195,32,119,70,86,41,255,154,88,44,198,129,87,94,49,48,43,
  102,25,139,69,34,60,247,204,51,216,236,118,186,187,187,233,104,107,165,123,133,218,31,186,210,117,132,230,230,136,199,98,12,95,190,76,66,91,27,139,
  185,142,75,151,46,45,250,58,242,138,215,98,177,154,99,8,133,209,101,43,76,118,201,21,111,218,226,173,129,185,241,32,188,212,48,191,154,235,184,98,
  204,203,48,163,226,10,214,121,44,26,173,136,121,54,155,229,196,241,227,28,63,118,140,85,171,86,177,106,229,74,182,109,217,66,109,93,29,137,120,156,
  100,82,157,171,76,38,163,182,161,142,69,213,132,217,64,96,209,115,101,50,68,139,231,42,255,187,16,136,10,228,135,110,24,81,154,157,176,152,36,251,
  165,164,103,146,169,20,125,154,33,122,45,214,160,64,16,13,135,175,104,13,206,106,173,65,175,122,13,206,131,71,213,224,124,83,140,81,133,197,68,55,
  124,235,235,95,51,59,79,212,128,158,194,79,227,99,218,115,132,16,4,131,1,2,129,217,66,246,156,162,208,222,222,78,78,206,33,231,114,36,18,73,
  2,179,179,200,138,140,34,43,200,138,108,120,223,252,219,42,243,158,168,203,85,16,201,11,93,52,26,33,30,139,33,164,188,32,22,72,116,227,1,72,
  160,232,194,91,144,72,197,160,28,22,239,95,18,8,74,251,138,189,93,134,194,245,148,195,242,207,47,247,83,49,201,104,38,147,166,255,194,121,83,37,
  7,175,215,135,199,235,65,206,169,37,69,102,103,103,72,37,83,186,44,42,37,114,136,222,161,163,50,246,6,7,141,33,105,65,18,66,149,65,161,202,
  221,165,75,23,181,240,145,121,152,125,97,94,111,66,129,76,58,205,133,243,231,180,37,167,126,71,159,207,135,215,235,67,150,115,228,100,153,217,25,245,
  58,20,69,214,98,188,175,205,117,12,244,247,171,223,217,112,29,8,16,82,129,37,203,135,32,20,24,137,202,7,59,227,111,230,172,122,227,65,79,84,
  144,175,155,15,243,124,164,16,202,205,131,121,197,235,184,10,204,133,80,91,46,86,108,253,42,202,68,222,25,245,203,2,152,11,237,247,190,11,231,181,
  249,42,224,238,241,120,241,120,60,140,141,142,154,116,205,213,96,94,18,113,168,233,143,130,199,131,18,204,133,40,98,141,75,172,30,165,136,129,83,10,
  243,33,174,96,174,174,187,206,185,246,251,238,244,244,20,153,116,230,134,174,193,18,60,138,4,86,24,240,16,37,118,192,82,194,99,105,234,243,27,83,
  244,222,112,74,49,65,99,8,196,86,12,192,148,107,170,50,58,58,170,9,166,42,124,122,220,145,22,4,175,234,36,69,127,207,74,233,69,229,206,38,
  249,236,58,161,9,161,36,164,178,46,166,66,188,158,81,75,136,210,147,124,133,173,168,122,42,122,115,71,254,68,171,148,240,242,165,39,117,85,106,68,
  201,30,18,137,68,8,135,67,134,62,219,178,202,90,34,33,75,50,200,5,43,169,112,48,95,164,44,138,10,39,115,147,242,54,84,118,40,27,206,92,
  20,70,175,95,130,186,74,242,237,22,213,235,8,155,54,124,245,253,37,36,41,223,63,92,185,110,215,33,9,9,171,213,130,144,36,221,101,43,68,17,
  215,32,132,33,105,75,204,75,47,136,121,86,87,21,243,155,25,115,195,254,42,22,239,93,42,193,220,200,194,21,49,114,148,193,28,32,26,141,16,137,
  132,11,186,70,2,73,46,131,57,229,116,78,145,241,89,97,174,140,186,166,88,207,8,81,76,126,96,78,172,49,49,227,75,215,19,247,150,217,119,139,
  201,40,49,31,30,69,121,247,98,233,224,241,22,99,70,175,161,160,230,133,15,197,212,1,190,132,11,49,22,178,150,0,25,211,6,161,160,32,35,235,
  76,137,98,146,63,81,241,195,5,21,12,81,169,84,32,43,11,162,160,108,229,60,49,79,233,146,106,12,233,82,16,60,195,65,169,72,18,53,121,43,
  223,49,78,20,200,135,50,242,72,158,168,47,145,67,230,87,78,250,102,87,70,6,141,39,117,211,145,78,152,140,18,97,234,147,172,157,207,245,141,222,
  88,222,65,44,202,135,116,221,174,67,18,88,172,86,61,14,182,192,144,137,18,229,43,68,169,189,117,85,187,121,21,243,155,14,115,73,128,44,22,231,
  8,21,229,140,74,81,238,26,174,12,115,69,152,203,80,233,6,144,2,138,92,228,133,155,111,190,138,245,77,9,206,194,52,31,70,194,195,104,164,155,
  103,163,50,249,177,56,79,92,117,223,93,236,26,44,79,70,113,115,227,241,118,52,70,43,157,84,243,155,165,186,95,10,3,107,174,48,239,206,41,64,
  228,37,80,82,223,82,82,36,149,178,47,114,223,41,243,184,251,10,130,38,85,216,28,48,111,8,149,54,6,227,241,167,18,137,83,61,21,45,5,122,
  116,94,150,222,40,143,101,15,70,134,23,153,220,60,134,67,145,34,105,12,190,34,116,166,105,254,243,185,48,49,69,230,155,84,134,53,49,176,237,162,
  248,36,46,76,223,43,239,10,52,149,132,49,132,83,153,236,19,109,57,233,207,83,196,117,187,14,73,146,176,90,173,166,88,57,19,67,182,136,200,56,
  129,82,246,192,87,162,136,170,152,223,188,152,139,66,44,105,165,253,244,218,172,115,129,137,13,153,7,115,85,229,24,113,55,212,152,19,243,59,134,203,
  207,149,100,74,78,52,25,232,148,99,69,13,25,222,38,189,83,84,161,96,41,145,31,111,129,53,88,158,140,50,219,2,55,13,30,111,21,99,84,121,
  35,194,88,225,228,170,80,82,140,171,178,213,166,24,255,172,89,162,218,77,104,155,131,34,212,35,173,146,103,92,149,98,217,206,187,224,43,9,99,37,
  202,190,168,244,152,241,212,94,78,16,203,92,132,82,70,173,190,157,45,84,229,77,251,156,242,7,163,194,230,88,250,202,194,137,189,12,91,99,18,69,
  73,43,132,170,190,183,98,220,94,77,161,74,133,80,147,114,97,33,66,104,244,144,225,16,100,106,144,88,38,137,165,156,1,39,180,111,80,236,137,80,
  138,67,72,36,81,136,225,22,202,117,187,14,33,84,195,68,18,18,101,125,145,130,82,154,76,20,175,31,193,194,84,95,133,13,167,138,249,77,129,185,
  184,154,12,229,146,228,149,197,96,174,20,133,251,11,3,240,243,97,110,52,226,21,148,10,213,34,133,209,240,41,51,95,104,115,37,74,44,158,226,208,
  133,50,178,46,230,153,135,69,236,137,162,186,239,46,106,13,206,139,135,184,121,240,88,170,250,220,170,92,229,135,44,230,117,219,119,236,224,244,169,83,
  36,18,9,189,43,198,154,181,107,241,122,125,188,250,234,97,237,244,174,224,245,249,232,237,93,197,248,216,24,227,227,227,186,228,116,118,116,82,83,83,
  75,42,149,100,100,100,132,88,190,36,148,128,142,206,78,102,103,103,136,199,226,250,99,173,205,173,4,231,130,36,19,73,154,155,154,72,38,147,132,195,
  33,253,11,55,54,53,17,14,135,113,58,157,212,212,212,20,50,148,53,247,124,40,28,34,52,23,50,49,162,66,72,165,155,100,81,189,199,114,130,104,
  60,249,229,21,136,177,114,192,91,177,216,237,181,16,98,229,58,11,126,123,71,7,237,237,29,122,91,185,84,50,201,185,115,231,73,167,146,32,96,219,
  182,237,28,121,253,136,118,152,81,95,179,105,243,102,78,158,56,97,250,188,158,158,30,46,95,30,36,157,78,155,246,31,175,207,71,67,125,3,178,44,
  51,55,23,100,110,110,14,69,17,56,29,118,26,27,155,74,246,228,153,233,25,210,233,84,17,75,95,126,115,52,177,73,101,187,210,104,49,130,162,16,
  185,223,212,212,68,83,83,19,249,36,150,116,58,205,229,203,151,73,167,211,172,94,179,150,243,231,207,233,111,87,95,87,79,109,109,45,163,99,163,36,
  19,9,245,123,59,29,248,253,126,38,39,39,80,20,67,96,126,137,158,22,230,216,235,5,174,67,146,212,204,106,53,73,80,152,234,168,138,133,226,68,
  243,83,32,48,241,130,84,240,236,181,183,191,189,48,7,88,177,98,57,110,183,91,47,0,175,40,50,211,211,211,204,76,79,211,217,217,133,211,233,228,
  194,133,243,122,91,2,167,203,73,107,107,155,94,86,12,20,213,221,249,38,99,174,38,48,137,146,20,144,123,238,185,151,166,150,102,253,169,241,88,140,
  87,15,191,202,244,244,20,0,183,223,121,39,231,207,158,101,122,102,134,213,107,215,208,80,223,192,43,175,188,172,191,111,79,207,42,64,161,175,175,79,
  127,211,245,235,55,208,220,210,162,118,107,58,115,150,137,137,113,253,162,119,237,222,77,104,110,142,179,103,206,234,84,114,75,107,51,205,205,205,12,244,
  15,168,157,253,20,173,230,117,91,27,110,183,155,120,60,206,248,248,24,153,76,70,159,171,141,155,54,33,105,165,3,37,201,194,212,212,36,19,19,19,
  229,115,20,138,153,98,1,118,135,131,229,203,150,227,243,251,200,100,178,244,247,247,145,78,167,233,238,238,214,106,253,14,234,115,101,119,56,216,178,117,
  43,39,142,29,195,225,116,178,110,253,122,14,29,60,168,191,151,223,239,103,217,178,101,156,62,125,90,159,199,230,230,102,122,123,87,1,112,234,244,41,
  194,90,173,206,55,58,188,126,63,107,215,172,45,244,149,151,21,206,157,63,167,86,184,209,128,181,219,29,116,45,235,194,227,246,16,137,68,24,26,186,
  172,175,53,151,219,195,178,101,203,144,101,89,187,229,152,155,155,99,114,98,2,128,238,238,149,133,191,41,10,138,44,163,40,10,195,195,106,251,111,139,
  197,66,99,99,19,147,147,19,87,37,187,149,240,48,30,150,20,99,254,138,16,120,125,62,86,174,92,201,137,19,39,76,178,235,116,56,216,186,109,59,
  135,14,169,88,56,28,78,122,123,123,240,251,107,136,70,35,244,247,247,147,74,165,0,240,215,212,208,213,213,197,233,83,167,244,185,244,215,212,208,219,
  219,139,211,225,100,120,120,136,225,225,97,253,111,183,223,126,71,201,220,27,159,211,219,187,138,230,230,230,107,142,239,181,208,231,210,245,250,34,77,205,
  205,108,223,190,131,13,27,54,154,30,95,179,102,45,59,119,238,164,163,189,93,223,208,87,173,94,205,182,109,219,104,109,107,3,1,14,187,157,119,62,
  242,46,214,174,93,7,66,237,58,114,255,3,15,178,124,249,10,117,3,147,4,203,150,45,195,237,246,168,134,164,164,102,36,182,119,116,224,118,187,145,
  36,65,91,123,59,107,215,174,197,102,179,171,148,187,36,209,170,109,18,146,36,33,89,36,36,139,133,149,43,87,34,73,22,245,119,97,204,250,149,180,
  19,188,153,57,53,21,190,23,197,84,188,40,113,159,45,196,219,84,199,141,244,20,169,50,210,222,209,161,175,150,134,134,70,222,253,232,163,58,142,219,
  182,109,47,9,104,223,188,121,139,97,3,19,212,214,214,209,187,170,151,229,203,151,155,100,163,165,165,149,157,59,111,209,149,205,218,117,235,233,237,93,
  133,36,73,184,92,110,218,218,219,145,44,22,36,139,5,139,197,130,69,178,104,229,154,36,85,134,165,188,123,168,146,219,174,92,252,114,222,69,100,56,
  64,25,188,71,205,45,205,218,230,163,62,80,91,91,199,93,119,237,65,32,104,106,106,162,177,177,81,127,239,206,174,46,58,187,186,104,104,104,208,214,
  140,160,173,189,13,183,199,173,215,222,205,127,95,33,105,55,97,126,124,209,215,97,112,217,98,138,199,42,172,45,81,54,30,187,152,253,51,239,124,138,
  225,223,219,21,115,33,96,197,138,110,220,30,175,238,253,65,20,10,120,47,91,182,140,53,107,214,208,216,80,192,190,163,163,131,222,222,94,234,27,26,
  180,239,37,45,9,204,141,229,117,140,123,239,222,123,239,161,169,169,217,96,124,119,243,251,127,248,89,29,243,59,238,188,139,134,166,38,4,176,102,205,
  26,62,241,51,63,67,87,87,151,254,30,171,86,175,162,119,213,42,4,224,114,185,249,244,111,252,38,247,61,240,128,218,78,218,229,230,167,126,250,167,
  217,178,117,171,254,249,119,220,113,39,15,62,244,112,193,133,43,9,118,236,188,133,237,59,118,226,243,251,17,146,68,67,67,3,143,190,247,189,180,181,
  181,97,177,90,105,109,107,227,221,143,190,135,250,250,6,125,62,183,109,223,129,197,32,15,247,222,119,63,119,222,181,167,188,225,83,140,185,16,60,244,
  208,195,44,91,190,12,80,59,10,125,232,195,31,193,231,243,49,59,59,203,237,183,223,129,195,225,208,231,106,203,150,173,56,236,118,82,233,52,62,191,
  159,93,187,118,179,117,219,54,125,53,249,253,126,214,27,244,243,67,15,191,131,71,223,243,94,20,20,236,14,59,31,255,248,79,209,219,219,123,77,246,
  93,191,207,199,218,181,107,245,53,104,119,56,120,247,187,31,213,187,29,214,55,52,240,193,199,30,163,181,85,181,1,90,90,91,120,239,123,223,71,93,
  93,29,32,240,122,61,172,88,177,194,228,70,223,176,97,35,157,93,93,186,108,74,146,68,125,67,3,13,245,245,154,126,47,200,104,107,107,27,157,93,
  93,120,189,190,171,147,93,157,217,54,227,65,177,183,194,64,236,70,35,17,118,236,188,5,191,207,103,146,221,149,61,61,52,54,54,234,100,193,207,252,
  236,207,210,209,217,5,64,103,87,23,159,252,249,95,160,73,51,24,107,252,126,54,172,223,160,191,118,195,134,141,124,252,227,63,133,223,239,71,65,225,
  161,135,223,193,222,123,238,209,255,190,251,182,219,76,123,160,98,48,7,247,222,115,15,183,105,127,191,150,248,190,169,110,250,197,140,237,219,119,240,252,
  190,231,184,237,182,219,121,253,245,215,180,210,11,234,226,26,28,28,100,69,247,74,70,199,198,64,8,86,172,232,102,120,120,72,223,116,182,108,221,198,
  220,220,28,175,188,252,146,94,154,100,112,240,18,187,111,187,141,153,153,105,98,177,152,186,160,139,234,46,90,173,86,44,146,5,33,36,108,86,43,66,
  8,186,87,174,100,160,191,31,0,187,205,134,197,34,49,23,10,105,61,234,5,107,214,172,97,104,232,178,217,141,98,18,62,179,33,106,44,78,108,76,
  84,18,229,188,237,198,184,146,10,84,189,49,112,89,84,227,72,174,191,65,138,96,108,108,148,215,94,61,172,47,211,247,191,255,3,180,183,183,51,54,
  58,170,42,91,12,153,200,24,235,123,170,76,209,234,213,171,57,117,234,20,107,214,172,101,96,96,64,127,102,247,202,149,156,63,119,142,201,201,73,20,
  69,97,108,108,148,13,27,55,105,140,144,5,57,151,51,23,11,215,160,151,242,202,25,179,225,96,44,14,94,44,147,70,70,80,136,242,242,164,0,146,
  144,152,153,157,229,236,217,51,186,39,242,190,251,238,163,169,185,137,201,201,73,154,154,154,152,157,153,213,24,196,118,78,159,62,69,107,107,27,227,227,
  227,8,69,208,216,216,196,133,243,231,181,131,153,58,31,37,97,113,134,68,128,197,94,135,36,4,86,171,165,144,204,98,216,228,197,124,113,88,37,191,
  138,42,230,197,33,12,138,130,36,73,12,13,93,102,122,114,10,61,114,94,203,158,151,44,18,211,211,211,180,181,183,171,117,48,5,180,181,181,19,12,
  6,180,210,75,146,206,138,190,217,152,151,171,239,156,255,220,253,251,246,113,250,244,41,125,14,190,248,167,127,202,250,13,27,56,123,230,76,201,252,140,
  140,140,240,137,159,249,89,254,232,15,63,171,237,225,249,236,23,193,93,119,239,97,98,98,156,111,126,227,27,170,107,87,81,56,120,224,0,159,252,133,
  95,224,194,249,243,36,18,9,172,54,27,3,3,3,244,174,234,165,191,191,31,135,221,65,107,107,43,211,83,83,88,52,34,227,174,187,239,230,228,137,
  227,12,12,12,232,148,80,48,24,224,174,187,247,240,248,15,126,0,8,28,14,7,39,79,156,212,231,234,236,217,51,124,244,99,63,201,233,83,167,204,
  153,251,69,33,100,32,104,109,109,69,8,193,190,231,246,233,175,15,6,231,104,111,111,167,239,194,5,78,158,60,201,61,247,222,203,147,63,126,130,198,
  198,70,214,174,93,203,183,190,249,13,125,30,194,225,48,219,182,110,227,210,192,69,194,145,176,233,144,183,97,195,70,106,252,126,254,246,111,254,90,127,
  254,64,127,63,123,247,222,67,191,166,63,223,232,80,27,207,20,214,160,162,40,172,94,179,150,215,95,123,149,187,239,222,203,225,195,135,181,58,164,90,
  59,215,149,221,220,118,219,237,252,232,71,143,35,132,32,153,76,114,222,80,126,43,28,14,179,98,197,10,70,71,71,25,28,188,4,90,11,88,69,81,
  24,25,30,209,194,246,36,132,162,122,196,46,15,14,178,108,217,50,206,95,56,127,197,178,91,98,11,20,37,41,155,237,128,2,25,117,121,112,144,53,
  107,215,241,218,107,175,234,207,91,183,110,189,186,31,3,15,60,240,32,47,236,223,175,255,46,132,96,100,195,8,15,63,244,48,255,242,47,255,108,154,
  63,191,191,134,189,247,220,195,223,254,205,95,235,204,233,145,215,95,231,147,63,255,11,28,120,229,21,253,177,3,175,188,82,209,38,251,223,95,253,11,
  253,121,3,253,253,244,244,246,94,51,124,223,232,184,46,204,168,195,225,160,169,169,137,254,254,126,166,166,167,212,19,169,97,107,24,27,27,163,189,189,
  93,117,37,216,237,248,125,62,2,179,1,157,54,95,181,106,21,39,79,156,48,213,159,155,155,11,49,59,51,75,71,71,167,186,177,89,44,26,35,42,
  233,193,198,86,171,85,59,245,8,172,54,27,195,195,195,180,181,181,225,247,251,245,199,36,201,162,191,70,146,4,54,155,173,224,70,42,9,90,150,74,
  132,175,132,149,40,49,72,171,172,232,210,102,71,205,27,135,221,238,192,225,112,144,78,165,117,229,215,214,214,78,91,91,27,173,173,109,180,182,181,170,
  76,142,134,185,221,110,167,190,190,158,137,177,113,34,225,48,77,205,205,186,140,132,230,230,88,179,118,45,173,173,173,216,237,118,114,57,153,227,199,143,
  169,178,102,145,112,186,92,212,55,212,235,183,134,250,2,3,89,96,229,165,121,78,230,69,110,77,81,84,150,76,136,18,207,118,33,17,66,125,178,221,
  110,199,225,112,144,205,102,153,157,157,161,185,185,5,132,160,169,177,137,116,58,205,244,212,52,29,29,29,250,90,168,175,175,39,150,175,185,171,121,11,
  242,108,130,126,203,175,193,43,185,142,50,201,44,166,18,42,80,180,190,230,97,69,203,44,48,243,33,239,237,133,57,90,18,102,125,93,157,202,140,183,
  52,83,91,91,167,43,91,73,178,16,8,4,244,235,176,219,237,248,124,62,98,177,184,106,140,74,5,246,239,205,198,92,148,221,79,75,15,244,43,186,
  87,208,212,212,196,244,244,180,170,216,132,100,170,1,125,252,216,49,102,103,103,185,247,190,251,205,134,30,176,107,215,110,158,121,234,105,83,166,123,40,
  52,199,255,252,210,151,72,37,83,186,110,25,27,27,101,217,114,149,157,235,234,234,98,102,122,26,155,221,142,197,98,161,169,169,9,159,215,203,165,139,
  151,76,243,53,120,233,18,94,175,143,198,198,38,132,164,26,163,198,185,202,100,50,12,93,190,204,234,213,171,203,27,162,134,164,181,96,32,136,207,231,
  99,219,246,109,212,55,52,0,112,250,212,41,189,163,209,201,147,39,240,251,253,116,175,92,201,189,247,221,207,75,47,189,72,42,149,54,25,163,199,142,
  29,99,207,222,187,245,247,212,67,80,122,123,117,195,190,224,222,29,46,49,136,222,8,59,106,74,188,210,152,217,116,58,77,67,99,35,14,135,131,254,
  190,62,83,8,244,165,139,23,113,56,29,212,215,213,23,66,234,12,114,223,209,209,65,42,149,50,233,110,139,198,56,27,101,183,190,161,158,100,50,73,
  48,24,160,182,174,14,187,238,45,189,242,53,104,78,34,52,26,164,229,179,245,79,156,56,206,218,117,235,12,6,165,159,198,198,70,206,158,61,75,83,
  83,19,14,167,83,55,68,243,227,244,233,83,154,13,213,108,114,117,247,246,246,210,223,223,167,27,147,0,169,84,202,100,96,230,217,85,227,205,225,112,
  232,120,62,244,240,59,232,212,236,177,225,225,97,158,223,183,239,38,103,70,23,8,6,232,233,233,85,217,72,5,78,159,58,205,134,13,27,25,26,188,
  172,191,100,118,102,6,223,206,157,248,124,126,218,218,84,22,198,160,66,176,59,28,90,231,12,227,54,164,144,72,36,212,137,21,2,139,84,48,66,243,
  175,44,176,165,234,166,39,203,50,3,253,253,108,218,188,153,131,7,14,104,204,169,100,114,251,232,1,245,152,217,7,97,42,160,93,250,191,241,251,234,
  213,117,133,48,149,80,17,74,209,138,47,138,235,42,27,11,246,118,13,22,189,65,175,21,66,98,245,154,213,180,105,49,132,94,175,151,209,145,17,102,
  103,102,116,89,219,182,125,187,86,207,78,70,145,101,149,201,65,173,135,215,213,213,165,197,53,11,134,134,134,232,94,209,205,204,244,12,2,232,235,235,
  35,145,72,208,181,108,25,155,183,108,33,30,143,211,215,215,199,212,196,4,22,201,130,79,139,141,54,26,13,175,189,250,106,105,119,143,162,238,52,198,
  254,200,250,255,101,146,52,138,203,43,231,141,209,238,238,149,180,52,183,160,0,110,183,155,169,201,73,66,115,33,20,192,235,245,234,174,236,169,169,41,
  178,217,44,161,80,136,182,182,54,98,177,152,218,111,91,87,138,74,217,178,107,37,242,188,136,235,144,132,164,149,249,41,58,240,21,187,106,141,110,232,
  114,184,139,138,209,162,234,231,41,111,63,204,243,114,222,189,178,135,142,84,10,69,81,8,6,131,28,59,58,135,34,192,34,73,68,194,97,124,94,181,
  168,123,125,125,61,161,80,8,139,36,161,40,10,146,144,52,126,120,9,96,46,4,104,251,115,190,38,117,254,171,252,254,31,254,161,254,217,177,88,140,
  127,250,135,127,96,102,74,53,70,45,86,107,97,95,214,248,228,111,127,227,27,252,198,127,253,175,28,63,122,20,99,21,200,134,198,70,130,129,128,254,
  188,251,30,120,80,143,63,60,242,250,235,204,5,131,216,108,54,70,71,70,121,239,79,236,226,249,231,158,99,245,234,53,76,76,76,208,173,133,121,57,
  93,46,221,48,42,150,199,116,42,133,203,229,66,18,18,14,135,163,68,223,100,179,89,220,110,183,57,153,198,164,93,212,223,50,233,52,63,248,207,255,
  100,195,198,141,236,217,115,55,126,191,159,161,161,33,14,30,56,160,198,86,42,176,239,217,103,121,236,195,31,230,210,165,75,92,26,184,88,208,57,154,
  216,28,59,122,148,117,235,215,179,114,101,15,153,116,90,75,246,81,67,227,66,161,240,117,211,69,2,65,99,67,3,239,121,239,251,80,20,5,187,205,
  134,162,40,92,56,127,158,250,134,6,162,90,167,44,93,231,107,201,96,177,88,28,187,195,129,34,203,180,180,182,242,216,135,63,76,46,151,67,150,101,
  166,38,39,57,120,224,128,233,224,41,89,44,186,12,231,101,183,171,179,139,209,209,81,132,144,8,6,2,180,180,182,170,30,145,171,89,131,37,118,128,
  208,147,171,242,250,222,168,247,103,181,125,162,163,189,131,177,209,81,214,174,91,207,197,139,23,245,24,217,112,40,100,72,156,43,216,7,161,80,72,181,
  117,20,10,24,57,28,132,23,192,72,81,20,110,219,125,155,233,177,231,159,127,158,233,169,41,254,243,251,223,103,253,134,13,220,182,251,54,58,63,248,
  24,3,253,253,28,56,112,128,233,169,169,37,161,207,175,139,155,126,251,246,237,52,53,55,179,251,182,194,164,236,123,246,89,18,201,164,254,251,224,165,
  75,180,183,171,108,196,229,203,151,241,122,60,186,193,150,205,100,240,250,124,68,35,17,61,241,71,160,38,10,4,3,1,117,99,179,88,76,245,234,0,
  157,25,53,178,164,51,51,51,116,45,91,70,247,202,149,234,99,154,193,170,191,70,99,70,141,167,55,35,139,50,223,223,40,166,233,185,57,218,178,189,
  157,135,36,73,140,141,142,114,238,236,89,20,32,18,10,17,214,228,76,104,110,204,39,30,127,92,175,213,168,0,63,245,211,63,173,255,125,237,186,117,
  212,214,212,176,97,227,70,93,97,157,56,126,156,116,58,77,83,83,19,163,163,163,140,140,140,168,113,80,245,245,236,216,185,147,23,95,120,1,73,146,
  72,36,18,28,121,253,117,211,6,104,148,197,146,205,176,140,236,25,195,82,202,101,94,234,242,169,135,197,8,102,166,167,185,120,241,34,138,162,16,139,
  70,85,166,83,51,108,194,145,8,117,117,117,180,180,180,232,70,210,212,212,20,77,205,205,184,99,49,230,230,230,144,52,35,69,207,234,46,106,179,87,
  226,70,93,204,117,72,2,107,62,153,197,100,148,84,114,207,139,82,86,180,92,197,239,50,17,166,111,55,204,81,20,36,33,56,118,244,168,169,79,119,
  30,243,124,236,231,228,228,36,77,205,205,52,52,52,48,53,53,133,203,229,210,11,225,11,45,185,244,205,198,188,196,77,111,184,255,217,207,124,134,51,
  167,79,115,247,61,247,176,247,158,123,216,191,111,159,254,92,139,197,98,250,110,10,106,43,200,231,158,125,150,15,126,248,195,12,95,190,108,98,151,234,
  27,26,8,204,206,162,24,88,176,29,59,119,50,120,233,18,115,115,115,88,173,86,166,166,166,176,90,173,212,214,213,209,189,114,37,7,15,28,96,213,
  234,213,88,44,22,146,137,4,13,141,141,38,108,243,163,161,177,145,76,54,171,179,208,197,115,228,246,120,8,107,135,190,249,244,142,215,231,67,8,193,
  225,67,135,116,210,230,246,59,238,224,246,59,238,224,249,125,251,16,192,204,236,44,99,99,99,156,56,126,188,98,171,234,103,158,122,138,71,222,253,110,
  94,122,241,69,19,107,234,247,251,75,190,123,83,115,51,225,80,200,196,188,93,45,51,26,137,68,120,237,176,234,166,79,37,147,204,104,135,193,76,42,
  133,207,231,43,42,249,164,202,179,223,239,39,147,201,96,179,217,152,153,153,225,217,167,159,198,102,179,113,219,237,183,115,241,226,69,178,218,188,230,135,
  69,51,70,243,243,102,179,90,89,182,98,5,203,150,47,215,229,39,18,14,51,161,19,96,87,182,6,203,218,1,101,126,26,215,229,241,227,42,59,58,
  54,54,198,186,117,235,120,252,241,199,213,67,74,58,141,191,166,166,236,124,249,107,106,72,165,82,58,171,153,55,80,205,94,230,2,219,154,199,16,224,
  219,223,250,86,89,111,181,191,166,134,163,71,142,112,244,200,17,208,14,224,143,190,231,61,252,221,223,252,205,91,211,77,223,217,213,69,42,149,226,127,
  253,233,159,234,183,131,7,14,176,125,231,78,83,64,250,248,216,24,221,43,87,178,98,229,74,38,180,216,209,188,72,157,57,115,134,109,219,182,153,4,
  161,190,190,158,54,195,137,38,153,72,208,220,210,162,43,220,188,43,45,153,76,34,73,146,30,36,46,132,224,252,185,115,244,174,90,165,185,235,205,116,
  188,85,139,45,45,126,188,56,187,177,172,224,85,136,23,49,62,86,124,127,161,205,188,58,174,191,49,26,141,70,25,29,29,101,108,116,148,72,52,106,
  238,154,81,6,235,60,75,214,218,214,70,46,155,229,235,95,255,58,223,250,198,55,248,246,55,191,201,185,179,103,89,189,102,13,66,8,110,185,245,86,
  213,245,169,201,82,48,24,36,147,201,224,241,122,245,26,139,146,41,41,164,180,239,124,229,76,206,50,217,158,70,185,42,202,188,54,254,140,199,227,76,
  77,77,49,61,53,69,60,30,55,189,207,236,204,12,221,43,87,226,116,58,9,133,213,152,181,137,137,9,58,187,186,104,105,105,81,15,127,37,89,209,
  210,27,190,142,124,55,30,169,136,13,43,102,198,16,165,5,161,175,180,55,253,219,17,115,201,224,1,42,222,203,44,218,119,153,157,157,165,179,179,147,
  206,174,46,2,179,179,250,190,89,62,19,254,205,193,188,82,171,80,227,216,191,111,31,177,88,140,15,126,232,67,38,204,243,134,161,62,7,192,115,207,
  60,67,99,99,35,91,183,111,215,31,123,245,240,97,222,241,200,35,250,231,63,243,244,211,60,247,204,51,100,50,25,253,26,44,154,158,24,25,30,230,
  238,189,123,177,218,108,132,35,17,108,54,155,78,122,196,227,113,54,110,218,100,186,246,141,155,54,233,173,87,133,16,216,52,99,52,47,47,46,151,139,
  237,59,118,208,119,225,66,89,204,141,114,217,214,214,198,158,187,239,214,127,79,167,211,92,30,28,212,13,57,138,152,190,74,100,200,244,244,52,151,46,
  94,228,54,3,89,52,60,60,204,134,13,27,74,12,209,15,62,246,216,181,97,70,37,137,76,38,163,175,193,217,217,89,253,59,205,6,2,68,163,81,
  117,77,25,214,96,239,170,85,100,51,25,130,193,160,73,206,50,153,12,39,142,31,103,215,238,221,186,113,159,159,83,75,145,12,247,244,246,114,105,96,
  128,39,127,252,99,245,246,196,19,40,138,66,99,83,211,85,173,193,133,12,208,114,181,196,47,93,186,196,202,149,43,89,185,114,37,0,51,90,40,201,
  204,244,52,145,112,152,117,235,215,155,230,106,253,134,13,164,146,201,18,198,114,100,120,152,158,222,94,147,129,234,112,56,248,224,135,62,84,209,168,53,
  61,239,177,199,76,175,61,115,250,116,217,3,200,77,229,166,47,206,210,50,77,228,250,245,156,62,125,202,244,247,83,167,78,241,193,199,30,227,229,151,
  95,210,95,127,105,240,18,183,223,113,7,225,112,152,112,52,162,177,218,66,155,164,83,60,252,206,119,242,240,59,222,193,248,248,56,54,171,149,238,158,
  30,94,61,124,136,88,60,6,66,117,143,237,189,231,30,236,118,59,233,116,154,101,203,151,51,57,57,73,60,145,0,129,230,178,87,5,40,145,76,112,
  121,240,18,155,54,109,214,55,129,188,196,228,141,209,178,117,106,197,60,101,252,12,133,203,20,65,161,188,10,74,73,123,211,226,30,180,66,175,0,88,
  29,24,28,129,55,226,181,138,162,32,89,164,50,175,212,126,207,39,173,104,189,146,21,61,115,82,141,171,186,120,241,162,73,64,6,7,47,113,207,189,
  247,113,250,244,41,78,156,56,193,29,119,222,201,168,86,134,172,161,177,129,156,44,19,152,157,165,161,177,17,159,207,103,138,31,18,66,48,51,51,173,
  51,4,166,88,57,83,255,199,34,246,175,208,68,217,16,22,162,202,159,208,60,215,249,50,137,194,212,183,218,224,69,209,40,136,177,177,81,238,185,247,
  62,38,39,39,116,131,32,17,143,35,73,170,75,49,172,149,94,49,186,144,138,103,78,148,212,215,91,248,58,212,68,26,139,182,30,49,36,15,148,219,
  232,49,181,245,83,12,159,37,230,97,69,243,223,243,237,134,57,26,11,105,124,157,17,243,188,145,54,54,54,198,182,109,219,136,197,98,196,53,204,21,
  20,141,73,84,150,4,230,5,99,91,49,237,169,74,209,207,255,243,213,191,224,171,127,249,127,121,254,249,125,186,18,87,231,64,49,212,178,86,95,241,
  79,255,240,15,252,238,103,62,195,177,163,71,81,4,60,241,248,227,124,242,23,127,129,255,246,219,191,205,137,227,199,145,21,133,245,235,215,107,165,186,
  230,116,175,27,64,95,223,5,222,243,222,247,113,244,200,17,132,48,120,227,36,193,254,231,159,231,35,31,251,24,43,186,187,9,135,66,248,107,106,88,
  181,106,21,95,255,218,191,233,242,100,183,219,217,179,119,175,126,93,219,182,111,231,228,137,19,204,133,66,134,174,75,101,157,1,244,245,247,177,113,243,
  38,222,247,254,247,115,249,242,101,156,14,7,107,214,174,229,201,39,159,164,184,189,75,177,142,86,138,214,196,193,131,7,232,93,181,138,100,42,133,130,
  194,233,211,167,232,236,234,228,103,63,249,73,206,156,62,141,195,225,96,253,134,13,60,191,239,57,146,169,228,53,217,229,231,91,131,7,14,188,194,35,
  239,122,55,173,109,109,68,194,97,60,94,47,93,93,93,60,241,163,199,205,6,182,54,71,115,161,57,250,251,250,216,182,125,59,135,15,29,82,229,85,
  1,97,81,19,150,132,182,78,86,116,119,243,210,139,47,148,180,22,95,177,98,5,129,192,236,85,173,193,124,185,70,163,222,207,251,44,148,50,221,164,
  34,225,16,163,163,163,220,121,215,93,28,59,118,212,180,198,246,239,127,158,247,127,224,131,116,117,117,17,137,68,240,251,253,244,244,246,242,237,111,127,
  75,155,153,194,255,161,112,136,231,247,61,167,99,148,74,165,88,191,97,131,86,186,105,72,223,235,140,30,105,149,49,13,113,250,244,105,78,159,58,197,
  199,62,254,113,6,180,210,81,235,55,108,224,192,43,175,92,115,91,228,106,223,207,178,102,245,234,63,88,236,147,155,91,90,184,239,190,251,248,241,19,
  79,48,49,49,81,254,57,205,205,156,62,125,154,92,46,103,114,131,128,90,11,46,149,74,50,173,157,34,115,185,28,151,52,55,72,158,198,143,68,35,
  228,114,50,231,206,157,35,157,206,224,118,187,72,36,19,188,246,234,97,213,229,164,109,94,201,100,146,225,225,33,92,46,55,22,139,133,203,151,7,57,
  127,254,188,238,246,145,36,65,40,20,34,147,201,106,140,197,156,198,6,204,144,205,100,117,87,145,36,36,166,181,83,171,40,42,180,44,12,29,151,10,
  108,168,241,244,35,74,50,230,69,145,219,112,49,172,232,219,157,25,109,109,107,227,29,239,124,39,207,61,251,44,83,139,140,95,185,239,190,251,144,101,
  153,31,63,241,196,149,6,47,17,9,135,85,22,208,236,231,212,239,143,143,141,21,152,39,237,231,196,248,4,245,13,13,12,244,247,35,203,57,253,111,
  233,76,6,129,154,233,57,59,27,96,100,100,24,187,221,129,197,106,97,124,108,156,51,167,79,233,229,200,74,130,228,37,65,34,158,32,145,136,23,18,
  141,164,226,122,131,102,185,51,42,44,73,20,50,171,69,57,70,94,219,188,99,209,24,209,188,107,222,216,185,69,99,25,236,118,7,131,151,46,145,76,
  38,117,153,79,36,226,68,162,81,125,109,86,74,168,41,95,155,111,225,235,104,106,110,226,238,189,123,57,116,232,144,22,122,83,201,40,41,196,102,221,
  255,192,3,200,178,204,19,79,252,200,108,20,137,249,140,164,183,39,230,80,96,105,139,49,23,66,48,55,55,71,42,149,68,86,100,198,199,198,9,71,
  34,234,193,61,30,39,22,143,45,25,204,239,222,187,23,69,81,120,230,153,103,138,55,76,6,7,7,137,107,181,167,51,153,12,253,125,125,52,55,55,
  171,213,11,4,92,30,28,84,67,82,128,153,153,25,102,102,103,0,65,40,28,34,48,59,203,200,240,176,170,15,178,89,14,31,60,200,229,203,131,52,
  183,180,2,10,251,158,125,150,167,159,122,146,68,50,169,199,187,94,186,56,64,44,26,69,8,137,211,167,78,17,139,69,145,132,196,228,228,4,169,84,
  138,120,60,198,177,163,199,176,217,109,56,93,46,166,167,167,120,234,201,31,19,14,133,11,250,166,136,97,222,255,252,62,142,31,59,54,47,230,70,61,
  115,238,220,57,18,137,4,30,143,155,68,34,201,43,175,188,76,96,118,182,168,105,149,96,102,118,134,180,166,119,243,88,164,210,41,157,149,203,229,114,
  76,79,79,19,14,135,244,164,175,129,254,126,166,167,167,169,241,215,144,203,229,120,246,153,167,245,240,147,107,49,210,233,12,83,211,83,101,215,96,34,
  145,224,236,217,51,88,44,22,236,14,7,129,217,89,94,121,229,101,162,209,152,62,7,153,76,134,64,48,168,203,200,244,212,20,245,245,245,204,205,5,
  145,101,89,151,215,68,34,78,60,30,199,227,241,144,205,102,153,154,154,50,201,106,50,145,192,95,227,103,70,215,251,87,182,6,43,234,253,34,187,192,
  136,71,48,24,36,39,231,232,235,235,51,197,22,199,227,113,78,158,60,129,213,106,195,233,116,50,53,61,197,179,207,60,163,187,220,243,246,83,30,163,
  233,233,105,206,159,63,71,125,189,154,192,118,228,104,193,237,94,46,68,9,32,169,189,126,112,112,144,145,225,17,106,52,22,245,192,129,3,122,157,233,
  55,91,159,3,136,71,223,245,174,69,155,177,27,55,109,226,143,63,247,57,62,253,169,79,113,236,232,209,43,183,152,149,66,235,52,197,112,100,51,89,
  210,138,177,175,172,98,138,227,42,126,172,240,163,244,190,225,237,202,24,126,165,22,138,16,101,238,27,51,57,5,165,217,242,101,74,55,137,170,33,122,
  197,99,235,182,109,252,249,95,252,5,191,251,59,191,195,169,147,39,23,245,154,63,254,220,231,200,102,179,124,250,83,159,186,170,207,148,101,89,151,201,
  18,217,171,116,255,138,100,178,136,169,80,42,197,82,205,35,139,139,144,63,69,46,48,158,66,201,179,71,5,166,204,244,225,66,152,191,183,40,254,254,
  249,199,174,255,117,172,91,191,158,223,251,204,103,248,242,159,253,25,23,250,250,22,136,189,82,95,244,133,47,126,145,108,46,199,175,253,202,175,44,200,
  138,150,243,233,86,49,191,249,48,255,204,239,255,62,185,92,142,255,254,91,191,117,85,251,104,94,231,188,17,204,11,15,45,140,121,185,249,18,21,202,
  62,148,109,235,186,128,28,44,20,2,182,216,121,121,179,116,206,91,101,13,46,26,143,183,161,13,112,53,250,252,170,221,244,87,29,55,98,12,138,87,
  20,67,75,90,131,219,58,255,55,141,7,215,105,112,173,215,188,90,243,78,49,4,243,151,254,111,20,196,74,24,151,60,94,148,148,84,122,95,148,169,
  173,119,101,134,104,117,44,157,145,151,69,83,0,133,73,46,203,220,207,151,243,80,202,201,36,101,210,189,23,81,3,115,129,131,209,124,50,39,52,183,
  109,254,58,208,50,72,11,155,167,162,119,3,209,183,232,124,162,139,84,104,207,171,175,41,221,5,124,3,174,67,43,195,146,175,241,89,169,52,154,137,
  133,40,118,19,95,97,182,96,21,243,155,15,243,146,118,160,87,169,204,223,8,230,170,167,223,136,127,229,185,170,164,115,222,48,230,139,53,124,150,176,
  33,250,150,90,131,85,67,244,154,15,235,155,37,140,166,254,193,243,25,164,84,216,32,52,86,64,55,74,139,100,112,65,145,20,229,251,195,87,50,72,
  23,22,188,162,55,92,226,39,212,170,49,90,230,96,180,224,198,168,157,198,203,110,136,138,161,231,119,65,250,148,249,237,163,242,237,140,203,132,124,84,
  186,175,149,238,214,135,164,136,2,59,134,64,42,90,47,38,67,165,104,29,229,235,147,220,136,235,16,146,164,39,24,10,67,239,249,114,236,88,225,93,
  132,153,1,85,138,62,94,204,255,45,111,102,204,27,26,26,16,8,102,3,179,58,230,245,13,13,216,237,118,38,199,198,117,204,235,27,26,113,216,108,
  140,107,97,84,70,204,189,94,31,145,104,68,127,220,227,243,225,243,250,244,125,55,16,8,168,29,111,52,111,149,26,22,5,173,45,45,250,215,76,167,
  211,90,187,80,104,105,105,33,22,139,22,218,52,95,15,204,133,40,218,86,197,117,91,231,249,53,177,16,230,165,180,199,34,116,206,66,250,102,129,117,
  62,159,46,186,18,195,167,186,239,94,155,125,247,173,130,199,82,26,210,155,33,140,101,79,114,249,78,8,21,217,73,115,22,187,41,203,13,115,111,217,
  178,241,76,69,221,62,74,159,139,57,62,42,47,76,198,58,99,21,140,210,114,49,34,85,163,243,230,48,72,75,100,173,156,236,21,199,220,153,238,23,
  117,239,160,52,123,185,180,157,35,166,162,214,229,50,166,77,197,175,245,86,53,152,10,121,151,173,234,32,9,115,188,125,190,248,183,241,185,198,117,99,
  188,198,27,122,29,66,175,249,187,56,163,100,30,198,115,126,202,227,166,198,220,225,176,179,231,238,187,241,249,124,120,253,62,246,238,189,7,187,195,193,
  206,157,183,224,243,122,113,216,29,236,186,253,54,4,106,43,192,198,134,6,16,18,123,246,220,141,221,110,215,63,99,195,198,77,220,186,123,183,9,243,
  213,171,87,155,218,114,214,215,231,139,242,171,197,217,183,108,217,138,64,205,72,54,126,255,250,134,122,110,191,227,14,132,36,104,105,109,99,231,45,183,
  94,55,204,203,117,96,186,98,86,116,145,152,179,72,204,205,237,87,43,233,156,242,113,182,102,125,99,120,63,81,60,111,148,89,243,230,250,151,197,134,
  143,207,239,231,214,93,187,216,186,117,155,41,107,58,159,173,221,217,217,169,207,69,103,87,151,158,73,237,247,251,111,104,86,245,91,101,223,189,18,67,
  244,106,101,183,202,140,190,105,167,35,10,174,249,146,19,107,129,25,53,179,58,69,167,35,99,220,232,130,45,3,69,153,31,98,254,223,75,140,233,171,
  55,68,171,130,184,68,78,233,197,167,113,195,137,188,132,173,47,57,173,231,229,210,204,154,152,100,113,81,173,43,69,153,211,250,226,100,81,207,122,22,
  102,150,193,88,10,194,196,138,150,253,238,152,214,211,245,190,14,99,89,149,146,164,128,114,134,232,34,153,142,183,26,230,203,87,116,115,244,232,81,162,
  145,8,2,8,6,2,172,232,238,198,233,114,226,116,58,25,31,27,39,16,12,168,133,219,103,3,244,245,245,105,44,231,172,126,180,247,249,124,4,2,
  1,205,56,45,124,119,33,36,173,14,169,250,157,91,90,91,9,206,5,153,156,154,196,110,179,211,218,218,134,16,232,165,193,242,99,243,150,173,28,60,
  144,111,57,56,137,36,73,106,93,104,99,201,172,107,132,185,169,188,248,85,238,151,215,28,115,10,30,185,202,58,231,10,48,175,96,200,148,255,123,249,
  142,91,0,183,222,186,139,103,159,121,26,191,223,207,214,109,219,56,124,232,16,126,191,159,142,142,78,0,34,225,8,157,157,93,140,140,12,211,217,217,
  9,157,157,28,60,112,128,245,27,54,112,240,192,129,234,190,123,133,251,238,66,120,84,117,253,77,98,140,2,229,139,42,11,179,242,196,232,74,44,54,
  66,141,66,169,251,73,74,169,250,133,117,87,57,133,183,248,77,163,42,132,55,177,107,64,146,244,160,250,114,135,35,161,24,186,208,228,79,231,74,169,
  114,82,12,153,161,230,205,112,126,89,20,11,30,142,22,119,32,18,146,64,86,100,3,67,42,153,147,1,132,64,2,67,89,164,194,193,78,143,27,188,
  145,215,161,149,95,51,245,126,94,192,16,45,12,179,19,78,17,87,182,214,110,38,204,125,126,31,253,125,23,244,107,139,68,34,172,241,249,121,253,181,
  215,104,107,111,103,227,166,77,216,29,118,250,206,95,32,26,139,234,177,120,233,116,90,131,86,16,141,70,137,70,163,180,181,181,21,222,95,203,30,222,
  190,109,59,10,104,93,240,20,122,123,87,177,124,217,114,60,94,47,47,191,244,162,214,42,181,77,171,13,10,19,147,19,72,146,217,125,30,143,197,240,
  121,125,196,180,204,231,107,137,185,177,222,232,27,57,208,95,43,204,49,148,32,20,38,235,231,10,49,47,119,192,90,72,191,44,96,248,56,29,14,238,
  218,179,135,177,209,81,14,31,58,4,192,202,149,61,28,62,116,144,173,91,183,113,238,236,89,238,222,187,151,112,56,68,42,153,194,225,84,217,83,35,
  139,90,221,119,175,108,223,125,35,134,104,213,54,40,50,70,21,229,10,106,66,229,159,91,156,157,120,149,163,228,61,140,25,114,69,159,169,160,118,118,
  26,31,31,215,203,176,40,101,94,91,172,174,22,249,77,10,197,195,160,144,12,165,213,80,41,236,57,133,211,90,62,139,79,143,86,49,100,164,26,5,
  179,210,60,137,121,254,246,182,27,111,64,174,174,213,28,154,179,59,203,200,97,254,251,45,40,175,165,210,167,204,47,121,166,202,152,229,127,43,200,167,
  82,78,14,13,13,27,245,206,127,134,57,53,63,215,124,29,226,77,186,14,76,157,136,140,204,134,130,48,220,71,49,111,218,90,49,14,109,207,87,12,
  47,23,111,76,118,150,40,230,151,7,7,217,184,105,51,167,78,158,208,88,201,45,12,14,94,226,142,59,239,228,185,103,159,101,240,226,69,182,239,216,
  201,216,248,24,123,247,222,195,216,216,24,40,10,59,118,238,228,194,133,11,68,34,17,253,58,132,177,131,147,54,135,175,27,202,194,180,181,182,49,208,
  223,207,248,248,24,219,182,109,199,110,119,144,74,167,153,154,154,228,216,241,99,250,243,26,26,26,104,104,104,32,26,137,210,187,106,21,13,13,13,28,
  58,120,160,194,53,190,65,204,69,229,189,82,188,81,204,203,225,91,230,49,143,219,195,138,149,221,92,186,120,209,28,31,203,213,234,156,114,250,38,191,
  86,231,95,231,66,193,244,156,252,250,118,56,28,60,243,244,83,56,156,78,58,58,59,185,107,207,30,94,216,191,159,142,206,78,28,14,7,205,205,205,
  42,51,174,168,237,186,251,251,251,233,234,234,164,179,179,147,112,40,244,166,233,162,43,89,131,205,45,45,52,55,53,107,25,218,75,100,223,85,10,7,
  38,211,190,187,128,13,240,150,214,255,87,169,207,165,55,243,59,151,156,12,140,44,103,209,105,81,0,95,253,203,191,228,157,239,122,87,161,107,7,69,
  76,124,161,201,177,225,31,101,110,134,223,68,33,46,4,204,105,18,101,79,170,11,156,142,230,59,33,85,79,68,75,115,44,74,14,13,39,116,81,230,
  121,101,229,208,36,139,204,47,139,11,201,225,60,44,125,254,91,8,33,149,189,182,210,235,51,199,139,190,89,215,161,55,160,88,160,52,205,219,25,243,
  217,217,89,162,209,8,59,111,185,133,157,183,220,74,36,18,97,102,102,134,19,199,143,115,235,173,187,184,117,215,110,102,181,226,221,39,79,157,100,231,
  206,91,184,227,206,59,9,4,2,5,67,84,187,14,221,141,174,189,119,52,26,51,93,71,58,147,38,157,73,131,16,28,61,118,148,229,43,86,32,16,
  248,253,126,182,111,223,193,246,237,59,88,189,106,53,103,78,159,166,161,190,129,141,155,54,226,247,251,180,24,83,231,117,193,92,8,113,205,90,41,151,
  255,156,133,49,239,238,89,201,31,127,238,115,172,92,185,242,42,117,206,21,232,155,133,188,113,21,188,5,78,135,131,91,119,239,38,28,14,115,113,96,
  128,84,42,69,83,83,19,23,7,250,57,120,240,0,79,61,245,36,235,55,108,32,20,14,209,212,212,68,56,28,162,191,191,159,187,247,238,101,120,120,
  228,166,216,119,239,187,239,62,62,247,133,207,47,185,125,183,28,30,213,106,58,87,193,140,190,233,70,64,241,201,180,56,110,180,18,168,229,232,249,188,
  97,46,42,19,242,11,210,245,11,184,79,22,218,20,170,66,120,115,14,73,8,100,69,41,221,24,139,79,235,37,97,35,149,229,208,44,139,87,40,135,
  69,15,46,86,22,37,10,215,81,18,127,86,236,10,43,56,238,111,248,117,232,61,218,175,160,101,174,241,77,148,107,112,184,187,89,48,31,28,28,100,
  112,112,208,244,248,236,204,44,179,179,179,38,38,98,118,102,134,192,204,76,81,44,113,225,58,46,92,56,175,29,88,20,253,119,12,97,79,1,237,253,
  242,159,123,250,244,41,16,240,194,11,47,148,24,7,125,253,125,165,152,94,79,204,175,209,129,222,228,30,94,36,230,66,123,157,110,184,40,165,78,224,
  197,234,156,249,244,205,213,232,156,252,175,225,72,132,227,199,142,177,107,215,110,82,169,20,199,142,30,165,169,169,137,51,103,206,32,132,32,28,14,51,
  61,165,21,185,159,82,11,168,167,82,41,142,188,126,132,233,233,169,155,98,223,21,146,164,151,5,91,106,251,110,149,140,186,6,178,240,166,179,82,146,
  68,217,154,192,66,240,233,223,248,13,222,249,200,35,165,127,210,110,95,248,210,151,216,190,125,123,201,169,168,244,159,249,68,100,250,139,152,231,116,84,
  174,198,221,27,52,68,171,130,184,100,233,81,61,46,110,177,167,245,178,172,89,217,211,121,101,198,164,162,28,10,243,115,203,177,242,162,232,244,110,188,
  142,178,178,104,96,153,244,78,61,111,214,117,228,147,89,88,124,45,201,114,186,229,13,173,167,42,230,75,30,243,43,146,141,43,48,128,68,69,204,5,
  155,55,111,230,55,255,235,127,197,227,245,22,178,173,243,223,27,184,239,254,251,248,228,47,252,252,2,58,167,252,191,249,244,205,21,97,94,230,249,225,
  112,152,67,135,14,114,236,216,81,82,169,20,35,35,35,166,185,26,25,25,38,28,14,51,50,50,172,63,118,230,204,233,37,187,239,110,222,178,149,207,
  252,193,31,152,112,207,31,10,4,112,255,253,247,243,243,191,240,11,75,99,13,22,191,166,170,255,111,62,99,212,104,144,110,219,190,157,199,62,244,33,
  253,241,61,123,246,176,231,238,187,205,37,159,52,176,87,173,89,195,158,187,239,166,173,189,29,1,180,181,181,241,115,63,255,243,248,124,190,50,156,124,
  169,192,25,119,98,177,192,134,112,45,132,176,42,136,55,135,65,42,42,41,42,163,107,187,156,129,34,68,177,88,45,32,135,98,241,114,88,110,51,156,
  231,64,100,100,16,22,127,141,55,248,58,12,74,126,49,235,228,186,173,157,42,230,75,31,243,107,141,189,48,39,96,121,189,94,62,254,83,63,69,75,
  91,43,8,240,250,124,60,240,224,131,244,172,92,89,48,152,180,249,69,8,54,109,217,194,123,222,247,62,131,113,122,63,247,61,112,63,243,248,132,175,
  11,230,111,25,15,156,38,23,94,175,151,159,250,196,39,104,105,109,5,160,165,181,133,59,238,188,147,158,158,94,253,0,163,199,28,11,193,238,219,111,
  103,247,237,183,235,243,248,158,247,188,135,77,155,54,189,105,107,176,156,221,80,29,139,31,214,165,35,143,18,143,125,232,67,108,219,190,157,253,207,63,
  79,103,87,23,151,135,134,177,217,29,124,246,115,127,140,197,98,229,185,125,207,211,221,187,138,207,255,201,159,160,40,10,231,206,95,160,163,179,139,229,
  43,186,185,255,129,251,249,217,79,126,146,23,247,239,39,26,137,160,200,90,27,56,242,45,72,203,134,70,23,24,16,97,174,85,42,76,39,54,81,145,
  241,159,63,235,183,228,83,170,99,169,239,139,134,110,94,74,57,176,21,97,114,209,41,197,74,78,123,112,225,122,14,243,208,126,243,201,89,37,134,176,
  228,153,149,175,195,220,239,230,205,185,14,129,49,153,101,126,131,179,226,227,215,104,77,85,49,95,194,152,23,23,189,191,86,152,235,205,82,20,188,30,
  47,63,245,137,159,166,165,181,149,127,252,251,191,167,107,217,114,250,251,7,120,223,7,62,192,251,222,255,1,50,217,44,47,189,252,10,239,126,207,123,
  121,223,251,223,143,16,130,35,71,142,176,251,182,219,57,113,252,56,63,255,139,191,200,165,129,139,60,247,204,51,160,128,92,220,130,84,41,175,121,76,
  93,166,40,212,187,148,140,70,205,2,152,27,135,223,231,199,231,247,171,125,232,83,41,102,102,102,74,106,136,78,79,79,147,74,165,244,199,243,191,47,
  133,125,183,181,181,141,159,250,196,39,0,120,126,223,243,212,214,213,209,215,215,207,79,60,246,65,220,110,55,217,76,150,23,95,122,153,223,253,204,239,
  99,181,90,144,36,11,99,99,99,236,190,253,118,6,250,7,248,232,199,63,206,201,19,39,56,117,242,164,150,232,104,198,161,98,50,141,161,26,135,185,
  54,169,84,232,73,191,200,53,88,181,1,222,12,99,244,26,36,130,245,174,90,197,207,252,220,207,242,249,63,250,99,162,209,40,135,14,30,162,107,249,
  114,126,243,183,254,27,3,23,47,242,79,255,242,47,92,184,112,129,225,225,97,102,102,166,177,74,18,217,92,14,171,205,206,242,229,203,233,238,238,102,
  253,250,245,252,196,99,143,209,179,114,37,175,189,246,26,151,7,47,147,201,100,121,228,93,143,224,118,123,248,246,55,191,185,224,30,170,247,188,85,10,
  249,157,66,8,100,10,1,247,146,36,149,118,124,41,218,196,23,100,68,171,201,243,215,126,92,135,57,85,51,29,13,49,130,243,236,96,133,44,216,114,
  207,19,111,232,154,42,197,43,21,127,88,9,75,162,84,190,142,82,229,255,230,93,135,201,101,187,208,218,169,180,177,43,85,204,223,202,152,11,195,251,
  94,203,125,244,131,31,122,12,175,215,203,223,255,237,223,49,57,49,193,235,175,191,78,71,103,23,159,248,217,159,227,220,249,243,124,238,11,95,96,112,
  112,144,137,137,9,82,201,4,40,10,57,69,161,198,95,67,91,91,27,27,54,108,96,253,250,245,220,123,255,253,76,78,76,242,242,75,47,145,205,100,
  113,185,92,124,248,163,31,225,233,167,158,102,104,104,104,222,25,213,143,56,249,22,173,26,22,185,60,11,152,15,15,168,20,99,91,52,15,235,214,175,
  103,100,100,4,20,232,236,236,162,198,95,67,83,115,51,35,195,195,166,215,172,95,183,30,127,77,13,35,195,195,236,218,125,27,103,79,159,102,122,122,
  250,77,217,190,127,230,231,126,142,104,52,194,183,191,249,45,38,198,199,185,112,161,143,109,59,118,210,220,218,198,235,71,142,240,199,159,255,188,142,67,
  58,153,68,81,100,29,135,21,221,221,244,246,244,176,97,253,122,30,122,248,29,76,76,76,114,224,165,151,117,28,126,246,147,159,228,63,190,255,253,18,
  28,74,176,80,140,241,213,133,142,117,32,151,226,176,136,53,88,181,1,110,66,102,212,231,243,114,215,158,61,228,126,59,199,196,228,36,46,143,135,191,
  250,171,191,230,249,23,246,115,254,252,121,61,27,116,89,71,7,31,255,216,199,168,173,171,227,242,229,203,156,58,121,146,179,103,207,112,244,216,81,254,
  227,63,255,131,218,218,90,118,221,122,43,183,238,188,133,79,254,226,47,114,230,244,105,62,254,137,79,112,230,244,233,5,177,23,198,82,12,134,211,120,
  254,81,69,150,17,128,44,203,186,81,42,93,1,29,95,117,205,223,164,12,105,153,146,94,74,229,39,151,40,247,43,221,119,174,164,139,80,197,205,112,
  129,235,144,22,250,94,55,240,58,242,29,78,174,166,6,223,245,98,24,170,152,47,65,204,141,29,140,174,225,216,186,109,27,119,237,217,67,60,158,96,
  245,218,181,28,61,126,156,151,94,126,153,195,175,30,102,98,98,130,92,54,139,223,231,99,203,230,205,108,217,178,133,108,54,199,133,11,231,57,123,246,
  44,199,142,29,229,181,215,95,195,102,179,209,214,214,198,61,123,247,178,121,227,38,62,242,147,63,137,215,227,97,231,173,183,112,228,245,35,12,151,49,
  130,76,54,144,177,49,133,94,38,16,189,28,144,172,40,8,89,134,188,206,169,20,134,97,24,163,90,156,104,58,157,86,11,219,131,106,160,26,70,99,
  115,51,47,60,255,188,254,183,27,217,125,169,120,108,219,190,141,173,219,182,17,139,197,185,101,215,46,94,62,112,128,253,251,247,115,232,213,195,140,143,
  143,35,231,114,58,14,187,118,237,34,26,139,113,225,188,138,195,203,47,191,196,203,47,191,132,205,102,99,205,234,53,236,216,177,93,199,193,231,243,177,
  121,235,22,158,126,242,201,69,237,22,74,57,155,64,195,33,167,225,144,15,49,145,42,132,154,84,109,128,155,208,24,237,93,181,138,254,190,62,142,188,
  126,132,111,124,253,27,216,157,14,14,189,246,42,143,63,254,56,35,163,163,72,146,160,182,166,150,45,27,55,114,219,237,183,179,124,89,23,233,76,150,
  96,80,237,157,188,99,199,14,150,45,95,206,240,240,16,227,227,19,68,162,81,158,248,241,143,121,225,133,23,88,183,110,29,183,236,188,133,227,199,79,
  240,173,175,127,29,0,183,219,141,199,237,46,156,254,132,40,123,82,53,214,97,52,109,12,121,1,85,20,100,212,190,208,146,36,149,184,156,170,66,248,
  22,52,72,13,10,99,65,3,165,72,25,95,47,244,197,149,26,48,197,241,207,75,226,58,68,197,175,63,191,33,90,197,252,237,132,185,36,4,138,100,
  185,38,159,238,245,122,1,136,70,163,124,249,79,255,12,143,207,71,14,133,255,243,127,255,146,23,94,120,129,88,44,134,199,227,97,121,87,23,59,182,
  111,103,253,250,245,248,253,126,18,137,4,211,211,211,116,119,119,211,220,220,204,236,236,44,231,207,159,39,20,137,48,62,54,198,191,125,237,107,60,211,
  220,204,150,45,91,217,177,125,27,39,78,156,228,200,145,35,8,33,232,236,234,100,100,158,210,73,70,50,68,49,20,119,207,135,139,229,189,118,178,166,
  127,76,93,172,202,204,225,109,183,223,78,99,83,19,51,211,211,28,60,112,128,221,183,221,198,238,219,110,211,159,115,240,192,1,210,69,110,249,112,56,
  124,67,247,213,214,182,54,162,145,8,209,104,148,255,245,63,255,148,95,251,205,223,196,237,243,242,185,207,127,158,99,39,142,19,139,70,241,120,60,172,
  93,181,138,141,27,54,176,110,195,122,252,62,63,145,72,132,76,38,75,119,247,74,154,91,90,24,31,31,227,210,197,75,132,34,17,206,157,59,203,185,
  243,231,116,28,246,220,121,39,207,63,255,60,103,207,157,187,42,28,140,143,233,77,122,0,100,153,156,86,164,191,18,14,85,27,224,134,27,163,87,199,
  55,127,240,67,31,226,83,159,254,52,159,251,163,63,194,237,241,18,12,133,120,252,27,63,226,200,235,71,200,102,51,116,180,183,211,213,217,193,230,205,
  91,216,184,113,3,185,156,76,104,110,142,185,80,136,116,58,141,64,237,155,220,220,212,68,77,77,13,43,86,116,51,62,54,206,240,200,8,209,88,148,
  19,39,78,48,52,60,76,40,52,199,157,123,247,98,177,90,248,232,199,62,70,99,99,35,191,242,75,191,164,27,155,133,43,48,196,123,22,21,177,55,
  109,12,122,97,123,245,132,36,203,57,100,57,87,209,40,85,95,83,229,228,175,255,184,190,115,44,68,113,119,176,69,42,246,27,98,140,40,139,123,157,
  177,227,217,18,185,14,129,114,197,53,70,69,21,243,183,29,230,133,82,76,111,252,27,127,229,127,127,21,128,47,125,241,139,188,235,221,143,242,47,255,
  246,111,60,241,196,19,140,12,143,80,87,87,203,242,101,93,244,244,244,176,113,227,70,154,27,155,72,36,18,204,204,204,16,139,199,201,102,178,88,173,
  54,60,30,15,118,187,157,186,186,58,130,193,32,151,7,47,51,27,12,16,12,6,217,191,255,121,6,7,47,241,142,135,223,193,123,223,255,126,2,51,
  211,124,228,163,31,229,135,63,248,1,143,255,240,135,101,13,125,197,216,243,222,208,138,66,65,41,232,35,3,33,162,32,200,229,228,18,163,52,255,51,
  223,166,245,129,7,31,2,45,95,162,184,33,129,221,110,199,225,176,147,74,165,232,236,236,196,231,247,115,246,204,153,27,38,15,127,255,79,255,200,196,
  248,56,95,250,226,23,121,247,123,222,203,191,126,253,107,60,241,196,19,140,143,143,227,247,249,88,222,213,201,170,85,171,216,177,99,7,62,175,151,68,
  34,193,236,236,44,145,72,132,92,78,198,102,181,226,113,187,89,214,181,140,150,230,22,130,193,32,3,3,3,132,194,17,29,135,96,48,192,237,183,221,
  206,222,123,239,69,206,101,249,200,71,63,202,183,191,245,45,158,123,246,217,178,178,89,220,237,49,175,243,17,230,213,178,16,14,165,107,176,106,3,44,
  89,102,244,137,199,31,231,161,135,31,230,163,63,249,113,254,253,187,223,225,137,239,127,159,225,145,97,218,90,91,216,168,197,225,52,53,55,147,73,103,
  152,153,153,37,26,141,146,74,165,212,132,36,237,159,100,177,96,183,219,177,88,172,56,29,14,26,234,235,233,233,237,97,106,114,146,145,209,81,130,161,
  16,207,239,223,207,220,92,136,135,30,120,128,134,134,70,254,237,95,254,85,219,152,139,131,145,21,83,27,91,245,116,90,232,190,132,33,41,64,239,198,
  96,216,36,100,89,70,49,156,146,170,117,70,223,154,12,169,89,177,191,113,215,230,21,25,35,87,41,83,162,66,73,156,37,113,29,198,44,238,37,200,
  46,84,49,127,235,97,254,237,111,126,147,79,255,63,255,15,31,255,233,159,230,135,63,124,156,167,159,122,10,1,108,223,178,133,91,119,221,74,79,79,
  15,14,135,147,112,56,196,196,212,36,137,120,130,76,54,171,119,146,17,2,172,86,43,146,36,97,181,88,112,187,221,180,183,183,17,12,206,49,62,54,
  198,196,212,20,51,51,51,60,183,239,57,118,239,218,197,238,93,187,185,116,241,18,207,61,243,108,137,183,13,3,17,98,236,238,163,104,6,76,190,211,
  146,34,180,191,153,186,254,169,198,106,46,167,146,33,22,141,12,9,135,67,250,181,30,58,120,128,149,61,61,132,195,33,118,237,46,48,163,103,207,156,
  230,208,193,3,108,221,182,221,244,220,27,57,254,226,203,95,214,113,248,135,127,248,71,14,191,122,24,1,220,126,235,46,54,108,220,64,79,79,15,78,
  167,139,185,208,28,83,211,51,196,98,49,50,153,76,33,9,89,18,88,173,54,132,36,97,179,217,112,187,221,180,180,182,16,14,133,117,28,46,94,186,
  68,50,153,212,113,24,188,52,200,129,151,95,41,234,144,36,74,250,215,155,140,83,13,7,37,223,146,118,1,28,242,164,84,149,13,93,226,198,104,107,
  91,27,19,227,227,68,163,81,246,61,183,15,151,215,195,129,3,7,168,171,171,101,215,173,183,104,221,45,4,145,104,132,241,241,113,226,177,56,153,108,
  70,61,21,107,66,104,204,142,19,66,96,177,72,106,79,110,89,198,97,87,147,154,86,173,90,69,56,28,225,252,133,115,28,57,122,132,76,38,77,54,
  155,165,175,239,2,138,2,13,141,245,204,206,206,22,31,120,10,167,32,67,167,54,147,64,26,55,9,163,235,62,31,223,163,9,164,176,88,170,82,245,
  22,54,74,75,50,50,141,44,79,153,108,205,197,40,249,74,229,120,222,144,33,176,64,130,200,146,184,142,69,196,94,191,217,27,123,21,243,155,31,243,
  124,88,216,225,195,135,121,241,133,23,121,234,153,103,232,235,187,192,237,183,221,198,214,45,155,105,108,108,38,52,23,36,28,137,48,49,57,73,42,153,
  34,155,205,146,147,115,104,4,165,70,131,20,216,90,172,54,36,20,114,153,12,181,181,181,52,54,54,178,9,152,152,156,164,175,175,143,151,94,121,133,
  108,46,71,71,91,27,241,68,28,1,212,213,55,16,8,204,26,227,193,244,57,212,153,57,205,104,53,234,27,197,240,184,170,123,20,132,34,16,146,22,
  211,152,203,33,73,22,19,187,25,14,135,231,117,191,223,104,3,212,104,3,156,58,117,74,199,97,124,98,66,199,161,174,174,158,104,36,98,194,33,39,
  231,80,100,213,240,203,41,57,67,118,188,130,69,146,212,10,13,22,117,30,141,56,12,94,190,204,240,240,48,47,189,172,226,224,117,186,116,28,234,53,
  28,76,181,242,69,222,240,188,114,28,16,104,118,138,74,148,85,199,18,55,70,63,247,133,47,32,203,50,223,250,214,183,144,133,224,95,255,237,223,184,
  243,246,59,216,185,115,39,3,3,3,140,142,141,17,139,198,144,101,25,89,81,217,70,183,219,141,16,18,245,117,13,32,4,217,108,142,84,58,69,58,
  149,36,30,143,147,78,167,144,179,25,4,234,137,85,81,20,146,169,20,22,171,133,141,27,55,17,139,197,120,241,37,53,192,121,239,125,247,51,59,61,
  197,123,222,251,94,254,230,175,254,138,19,199,143,99,140,97,82,244,142,180,165,130,167,104,49,34,166,147,170,225,126,222,245,36,43,10,74,54,135,100,
  145,170,167,163,183,11,75,186,128,18,46,121,196,152,180,112,29,191,223,117,191,142,107,244,93,197,18,53,66,171,152,191,53,48,191,107,207,30,254,248,
  11,95,224,27,95,251,26,177,68,146,239,124,247,187,156,56,121,130,223,252,245,95,39,20,10,19,10,205,113,250,204,105,82,201,148,78,120,228,107,94,
  186,92,110,28,14,39,178,162,144,74,167,73,167,82,164,82,73,146,201,36,169,84,18,37,79,64,104,30,178,156,156,163,174,182,142,101,93,93,156,62,
  115,134,39,159,122,154,59,111,191,157,135,223,249,78,214,173,93,139,219,227,225,207,254,228,79,72,36,18,58,249,161,27,64,121,221,195,34,116,79,190,
  86,166,129,169,147,229,28,130,133,243,24,222,172,145,199,225,175,254,242,47,177,218,29,58,14,127,253,127,255,47,135,14,29,38,20,154,99,124,226,44,
  153,116,70,199,193,106,181,226,116,58,113,185,220,184,221,30,114,178,66,50,149,36,165,205,127,34,30,39,147,73,163,228,114,88,180,196,162,60,14,237,
  109,106,253,241,60,14,143,190,235,93,220,115,223,125,108,221,178,5,73,146,248,211,63,249,19,244,46,244,111,0,7,213,97,47,244,164,65,57,151,83,
  147,156,150,40,14,111,57,99,244,106,92,58,63,250,209,143,248,224,99,143,177,126,227,70,254,254,31,255,145,137,137,9,218,219,219,24,25,29,97,124,
  98,92,175,11,154,55,68,123,122,122,216,184,97,35,177,68,6,167,221,73,34,145,32,22,143,19,137,170,129,207,214,208,28,241,88,140,100,50,78,38,
  157,34,155,77,23,92,41,22,11,10,224,243,249,73,36,147,28,57,118,12,175,215,203,67,247,63,192,217,51,103,24,30,30,86,217,122,145,47,231,100,
  58,159,150,158,140,242,238,125,81,196,146,82,8,50,47,208,251,10,114,78,70,88,164,242,157,37,170,227,154,200,211,27,121,221,53,99,204,242,134,198,
  149,189,240,122,125,161,171,50,30,174,250,58,174,193,247,189,26,131,164,138,121,21,243,43,25,71,143,28,225,133,23,94,224,246,59,239,226,47,255,234,
  255,242,252,11,251,217,115,231,157,32,4,151,47,95,38,151,203,233,251,118,158,0,217,184,113,35,13,245,77,200,138,106,80,36,226,9,130,161,57,162,
  209,40,81,77,255,72,209,8,233,100,130,92,46,77,54,171,190,135,69,178,32,9,5,135,195,129,100,177,48,54,62,198,107,175,191,206,242,247,189,15,
  155,221,206,243,251,246,17,143,199,49,170,27,93,119,20,232,209,18,61,84,208,61,2,69,18,102,87,178,102,60,43,168,149,95,20,5,36,73,44,57,
  50,164,175,175,143,163,71,142,112,247,189,247,242,79,255,252,207,58,14,227,19,19,58,14,198,186,224,110,183,155,157,59,119,226,113,251,116,28,226,177,
  56,145,88,148,72,36,66,56,28,70,178,204,145,76,196,73,39,19,100,179,41,100,89,209,113,80,36,76,56,28,56,116,144,135,238,127,0,187,221,193,
  75,47,190,104,142,165,86,204,39,48,221,53,95,17,7,131,113,170,25,157,249,51,130,9,7,139,244,182,172,44,122,181,235,213,122,69,27,146,94,196,
  247,202,247,177,111,127,227,155,248,124,126,78,156,62,197,145,35,71,232,233,238,166,161,177,129,115,103,207,153,12,209,250,250,122,186,187,187,217,177,99,
  7,61,43,123,152,9,68,153,156,152,100,114,98,188,80,20,88,146,176,217,236,216,236,57,114,57,5,89,129,156,156,67,206,101,77,51,209,209,222,129,
  85,58,194,204,204,12,199,142,29,167,189,181,141,190,243,231,153,153,49,187,233,21,131,16,26,41,1,93,224,132,113,231,48,246,125,54,148,231,48,177,
  31,10,74,78,70,17,162,122,66,186,18,233,85,174,80,176,148,27,175,79,169,192,31,41,111,210,23,49,214,175,83,110,166,235,48,213,141,84,76,247,
  171,152,87,49,191,86,152,71,34,81,254,244,139,95,226,231,127,249,151,217,191,127,63,51,211,211,236,186,245,86,18,137,68,193,16,85,20,108,54,27,
  29,29,29,172,91,183,142,45,91,182,96,183,187,153,154,14,48,61,53,77,54,155,195,106,181,98,177,88,176,90,109,216,108,118,236,118,23,178,172,160,
  100,4,228,18,250,23,20,66,224,243,249,169,241,251,25,31,31,103,104,100,152,147,167,78,33,9,193,11,207,239,55,27,61,134,237,207,212,95,69,161,
  72,167,168,196,199,230,173,91,176,59,28,8,33,200,102,50,12,13,13,177,106,245,106,50,153,12,14,135,131,190,11,23,8,4,2,244,244,174,166,191,
  175,15,167,211,201,174,221,187,121,97,191,250,185,123,238,190,155,67,7,15,146,74,165,104,106,106,2,84,119,254,214,109,219,0,213,120,59,116,240,160,
  94,234,41,95,125,102,157,86,85,32,63,206,158,57,115,197,25,248,227,99,227,124,245,43,127,193,158,123,238,49,225,208,215,215,103,58,16,24,113,184,
  101,231,78,16,54,29,135,84,50,165,235,127,187,221,142,211,233,68,206,129,44,43,228,80,64,73,129,92,224,55,141,56,92,184,112,129,182,150,86,82,
  201,164,62,31,70,28,242,238,121,161,53,40,16,21,112,48,81,87,66,32,100,69,39,167,138,83,211,114,217,220,162,50,238,171,250,252,13,48,163,87,
  50,238,218,179,135,109,219,183,113,234,228,41,20,33,120,241,197,151,72,38,147,44,91,182,140,100,50,69,44,22,215,79,68,141,141,141,220,182,123,55,
  27,55,110,164,166,166,6,33,192,229,178,99,179,217,145,44,22,61,238,73,150,101,53,166,39,151,213,98,53,36,132,100,87,5,67,201,233,167,43,143,
  199,77,125,125,29,83,51,51,140,142,143,113,236,196,113,118,223,114,43,53,53,126,122,122,122,248,206,183,255,221,180,45,43,166,83,144,97,95,192,80,
  118,195,96,152,27,75,113,24,13,215,66,198,157,26,75,82,53,72,223,250,99,94,23,232,245,48,133,132,184,233,175,67,79,250,187,73,55,235,42,230,
  75,27,243,95,251,245,79,211,119,161,15,143,207,199,107,175,189,70,255,192,0,45,205,205,248,253,126,166,52,67,43,79,130,172,94,181,138,59,238,184,
  131,174,174,46,236,118,59,233,76,14,167,195,174,39,44,233,196,133,22,179,152,147,101,20,69,128,176,34,89,236,228,114,105,208,188,122,54,155,21,191,
  207,143,203,229,34,158,72,112,234,204,105,58,59,59,216,190,99,7,126,191,159,137,137,9,206,158,61,107,102,170,141,9,78,101,152,100,5,56,126,236,
  24,173,109,109,0,76,78,78,210,214,214,198,101,173,32,188,16,130,221,183,221,198,193,3,7,240,121,189,200,178,204,218,181,107,117,195,51,111,88,110,
  221,182,141,67,7,15,226,112,56,0,216,181,123,183,110,160,58,28,14,86,246,244,16,41,50,52,253,126,63,135,14,30,188,106,28,126,231,247,126,143,
  163,71,142,176,162,167,135,167,159,125,214,132,195,25,109,30,20,69,193,110,183,179,170,183,183,34,14,66,195,33,31,39,155,203,201,40,200,32,44,72,
  194,134,36,41,228,20,21,7,185,8,135,72,52,106,194,161,185,165,153,193,75,131,38,28,138,109,1,83,82,115,81,168,141,174,227,141,122,223,112,192,
  202,223,207,235,255,106,216,222,34,246,134,235,253,1,219,182,111,227,131,31,250,16,63,253,115,63,199,243,207,63,207,153,179,103,104,105,110,230,174,59,
  239,100,106,114,146,108,54,139,172,200,212,213,213,241,238,119,189,139,221,187,119,83,87,87,167,27,112,110,151,90,78,195,98,181,170,153,135,178,122,226,
  200,100,178,100,50,89,45,208,92,6,36,44,22,39,66,178,234,6,171,144,36,214,172,94,13,64,44,22,227,204,217,179,140,141,143,243,224,131,15,209,
  187,106,21,53,53,53,166,164,40,211,173,194,230,172,119,107,50,254,44,67,79,23,14,7,138,86,154,164,58,222,46,70,233,245,80,182,194,240,222,55,
  162,90,195,245,190,142,124,246,169,168,98,94,197,252,58,140,214,182,54,222,241,200,35,252,218,111,254,6,173,109,109,188,240,226,139,132,67,33,30,124,
  240,65,210,233,52,161,185,16,178,162,230,39,172,234,237,229,209,71,31,165,167,167,7,187,221,14,128,205,106,193,237,118,226,112,58,177,88,44,122,248,
  85,54,155,37,155,201,33,107,110,101,20,129,36,217,176,88,157,32,36,181,30,40,208,216,212,72,67,67,3,138,162,48,52,60,204,137,147,39,233,89,
  181,138,219,110,191,157,213,107,86,151,173,100,160,24,110,149,116,141,158,192,163,253,236,233,237,101,219,54,181,112,252,236,236,172,233,253,156,46,23,71,
  143,28,97,101,79,15,160,182,8,189,56,48,160,255,158,127,44,111,136,54,105,245,73,203,141,93,187,119,235,183,43,197,225,174,187,247,240,169,95,255,
  52,22,139,133,87,94,121,217,132,67,42,153,210,113,216,190,109,219,188,56,72,146,100,194,33,147,201,145,205,100,145,115,57,77,255,219,177,218,92,32,
  89,116,221,93,14,135,222,213,171,217,185,243,22,150,45,91,166,207,163,145,161,47,167,219,43,234,126,195,207,18,150,95,187,159,175,184,83,29,215,133,
  25,85,88,172,131,232,43,95,254,50,143,255,240,135,252,252,47,253,23,142,28,59,138,36,9,150,47,91,70,91,91,27,199,79,156,208,99,68,187,187,
  187,89,209,221,141,205,102,51,91,203,66,224,245,185,177,89,237,250,73,35,155,203,169,155,66,54,171,158,142,52,99,79,72,18,118,139,139,108,86,144,
  76,170,177,57,13,13,141,184,157,78,226,201,36,225,112,152,11,125,125,212,250,253,60,249,196,19,133,88,29,195,89,72,96,8,254,208,91,133,106,149,
  223,242,52,126,133,159,197,167,35,140,217,246,178,82,101,72,231,149,167,235,43,135,111,142,145,50,207,55,175,176,57,205,111,8,40,111,161,235,168,98,
  94,197,252,250,97,62,49,62,198,59,30,120,128,95,250,149,95,229,66,127,63,3,23,7,168,175,175,99,205,234,213,196,19,9,82,233,148,78,20,220,
  123,255,253,212,214,214,150,92,191,195,97,199,237,114,169,68,136,2,57,89,38,147,205,145,201,101,200,100,179,134,178,126,22,44,194,130,213,106,37,147,
  138,145,201,102,240,184,61,212,214,212,48,49,49,73,58,147,230,210,165,65,70,71,71,121,253,213,87,233,239,187,144,167,220,242,110,183,188,197,110,10,
  62,44,164,199,228,117,12,40,138,76,254,47,138,162,208,223,119,129,96,32,200,214,237,219,233,59,127,78,211,55,50,94,175,7,143,215,195,250,13,27,
  104,104,104,208,231,112,122,122,138,142,206,14,194,97,69,235,73,95,48,129,21,20,214,174,91,199,197,139,3,37,102,242,213,50,163,121,28,126,241,151,
  127,153,190,129,139,76,207,204,152,112,200,95,135,44,203,236,218,189,123,94,28,36,139,197,132,67,54,107,198,65,8,9,171,36,97,177,88,72,41,17,
  114,160,225,80,203,232,216,24,185,92,142,75,151,6,25,25,25,225,165,253,251,25,25,25,214,14,75,90,93,215,162,112,59,61,71,172,28,14,249,176,
  190,162,18,192,197,165,184,244,198,162,178,154,100,245,246,40,251,120,117,235,252,134,88,71,237,29,29,12,92,188,200,240,240,48,117,181,245,108,219,186,
  141,120,34,65,50,153,212,147,149,118,236,216,129,83,115,29,20,15,143,219,142,221,97,71,160,102,172,171,167,162,140,202,170,202,50,138,102,180,90,36,
  73,139,233,113,106,130,164,224,114,187,244,248,152,84,58,205,216,248,184,230,22,145,245,133,128,161,92,68,49,67,74,241,253,178,167,34,204,101,34,202,
  20,150,86,140,39,176,234,168,50,168,6,22,234,122,49,82,213,235,168,206,213,219,249,58,242,46,248,190,254,62,102,102,102,216,177,125,39,30,143,71,
  235,228,163,150,13,124,248,225,135,89,222,213,85,246,245,54,155,5,151,203,129,205,106,67,1,114,185,44,217,76,134,76,58,131,156,147,145,229,124,71,
  36,129,197,106,197,102,181,99,179,169,58,204,106,179,210,208,208,64,141,223,7,192,76,96,150,241,137,9,28,78,103,73,178,174,162,255,44,163,115,80,
  147,108,49,36,247,20,216,55,245,111,233,116,138,254,190,11,172,219,176,65,87,63,171,87,175,97,255,190,125,188,254,218,171,188,254,250,107,116,175,92,
  169,95,215,177,163,71,217,186,85,141,19,13,135,195,236,218,189,155,198,166,38,252,126,63,233,180,218,157,169,169,169,137,142,206,78,58,180,150,162,249,
  251,29,157,157,186,139,255,74,112,112,185,61,244,245,247,17,143,199,77,56,228,13,209,135,31,126,152,150,230,230,121,113,176,104,137,201,58,14,153,116,
  69,28,172,22,155,1,135,122,253,59,231,113,240,249,125,165,56,200,70,221,143,25,147,34,28,242,10,223,204,162,150,105,40,97,208,249,85,15,233,117,
  97,70,23,55,126,237,215,127,29,175,215,75,54,39,243,226,203,47,147,203,102,233,94,177,156,181,235,214,50,51,61,77,54,155,197,225,112,208,219,211,
  203,178,174,174,138,27,154,213,42,225,245,122,144,36,137,108,54,75,58,157,38,147,206,168,25,120,40,72,66,194,106,181,96,177,90,176,88,44,56,132,
  147,120,44,68,46,155,65,146,36,154,155,154,152,156,156,36,149,78,19,142,132,153,11,133,184,243,174,61,52,183,52,243,226,254,253,76,79,79,171,153,
  241,198,163,76,254,196,154,175,136,175,152,75,62,20,24,81,12,71,36,189,60,126,225,192,100,56,105,201,178,252,54,58,29,85,71,117,84,71,117,220,
  248,177,106,213,42,62,245,235,191,206,193,131,7,177,218,236,140,140,140,98,183,217,216,185,115,7,217,108,150,88,44,70,46,167,150,97,218,189,107,55,
  86,107,121,53,104,145,4,46,151,3,151,203,137,162,40,100,50,25,210,233,52,89,205,144,205,135,30,88,109,106,114,147,122,147,136,39,98,8,4,126,
  191,159,186,186,58,2,193,32,153,76,134,241,137,113,182,111,217,202,189,247,223,143,36,9,158,126,234,169,162,224,196,242,250,71,37,236,84,102,46,24,
  12,234,121,10,129,64,64,215,58,129,217,89,210,233,52,40,10,125,125,23,16,160,254,14,140,141,142,226,247,251,153,157,153,209,175,237,153,167,159,34,
  149,74,49,58,50,130,195,225,160,177,169,137,209,145,17,206,134,195,37,198,230,217,51,103,240,93,69,255,250,60,14,7,94,121,133,116,38,195,200,200,
  40,53,62,159,9,7,89,150,117,28,44,21,106,116,230,113,176,219,237,69,56,100,117,28,44,22,9,139,181,128,131,44,103,72,165,83,58,14,126,175,
  151,84,50,169,227,176,247,174,61,180,182,182,145,201,100,120,97,255,243,249,32,80,12,93,8,40,100,55,149,226,160,104,30,213,74,30,82,202,176,163,
  213,28,146,55,145,25,85,227,69,238,102,217,138,21,28,57,122,148,198,198,70,238,189,247,94,172,86,43,147,83,83,200,178,140,211,225,66,72,14,66,
  145,52,137,100,86,143,185,49,157,196,129,166,134,90,213,13,146,206,144,72,36,72,103,212,133,103,181,88,176,59,236,56,93,110,156,78,183,218,135,222,
  235,166,165,181,93,63,237,52,183,180,208,214,214,10,8,146,201,36,147,83,83,116,45,83,141,95,143,199,171,63,79,54,156,80,145,11,39,214,194,169,
  20,195,233,136,242,167,163,114,241,39,134,33,87,217,209,234,168,142,234,168,142,235,58,188,62,31,31,124,236,67,12,13,13,49,62,62,206,250,117,235,
  88,177,98,5,193,185,57,226,241,56,138,172,80,83,83,71,56,154,33,24,74,144,201,202,101,157,139,126,159,155,218,90,31,138,44,147,72,164,72,38,
  147,100,178,89,36,73,96,181,89,113,186,28,184,92,110,92,46,23,30,143,155,154,218,26,188,94,85,167,184,92,46,58,58,218,113,57,93,40,138,194,
  248,228,36,201,84,138,149,61,43,113,185,220,102,157,35,155,25,82,138,24,58,157,5,77,165,73,167,211,40,160,214,61,77,167,117,37,19,141,68,81,
  128,104,36,162,179,142,249,17,14,135,9,133,66,166,223,83,90,143,250,188,81,154,207,144,207,255,158,191,133,195,97,211,239,169,162,222,246,243,233,127,
  175,207,199,7,62,84,192,97,207,158,61,38,28,80,208,113,8,71,82,243,226,224,245,184,77,56,100,115,57,3,14,46,19,14,245,245,13,38,28,150,
  45,235,50,225,32,203,50,157,93,157,184,92,174,50,56,20,123,67,139,112,200,43,246,98,207,168,145,16,173,192,142,86,61,164,111,18,51,250,59,255,
  253,191,3,240,197,63,251,51,134,135,135,89,187,122,53,171,122,123,153,24,31,87,5,90,1,191,191,6,171,197,202,228,196,36,115,14,117,97,123,125,
  110,60,110,59,54,107,193,86,182,217,44,56,28,14,210,233,20,201,68,66,125,204,238,192,97,183,99,179,219,113,56,108,90,233,13,43,22,139,149,108,
  54,131,162,149,121,112,185,92,52,55,53,51,49,62,65,54,155,37,20,14,115,232,224,33,46,156,63,167,21,97,22,40,66,171,27,106,58,161,22,106,
  141,229,127,85,133,175,210,233,168,112,12,170,196,142,162,40,40,90,114,85,117,84,71,117,84,71,117,92,219,209,215,215,199,207,254,212,79,241,192,67,
  15,17,75,166,144,132,96,243,166,77,164,146,73,66,161,144,94,74,168,179,179,139,233,169,105,108,54,171,90,88,221,227,198,231,117,226,116,90,245,26,
  209,249,152,69,69,81,212,154,150,233,52,146,36,225,112,56,113,216,237,88,237,54,28,118,27,22,201,130,213,102,213,92,198,106,2,141,36,36,252,254,
  26,252,62,47,201,100,130,136,102,36,126,253,223,254,141,208,92,72,35,222,132,169,235,95,9,11,99,168,61,90,168,232,101,208,57,228,9,17,81,240,
  201,85,104,178,160,200,242,13,237,16,248,226,11,47,240,226,11,47,152,112,216,185,115,71,69,28,194,14,59,14,135,83,199,193,229,180,25,59,201,234,
  12,117,30,7,171,213,138,93,179,1,236,14,59,54,155,85,199,33,147,201,152,112,104,108,104,194,239,27,49,225,240,196,227,143,147,205,100,202,224,96,
  14,6,45,144,163,249,88,82,69,179,5,242,56,228,95,191,48,59,250,102,224,80,53,70,13,35,147,206,48,53,61,197,251,222,251,30,100,69,33,56,
  55,167,247,154,23,146,68,42,157,98,118,102,26,139,85,53,40,237,118,39,78,151,131,250,250,58,154,27,107,176,88,4,86,171,26,55,18,143,199,201,
  100,179,120,61,94,92,110,23,14,135,90,250,201,166,81,244,66,210,234,144,90,173,92,212,150,169,36,73,212,212,214,210,208,216,200,216,248,56,241,120,
  140,150,134,6,213,88,21,20,12,81,5,92,30,55,54,155,21,33,164,66,25,18,237,61,133,16,72,66,109,65,170,198,73,21,254,158,239,194,32,132,
  32,17,143,19,79,196,43,82,245,121,163,180,26,91,87,29,213,81,29,213,113,125,70,46,39,19,142,132,105,108,106,164,123,229,74,66,225,48,233,84,
  90,47,15,100,181,57,8,6,102,180,30,231,106,249,32,135,195,133,219,227,162,165,185,1,159,215,137,16,224,116,58,200,102,85,143,156,205,106,197,229,
  118,227,241,184,177,219,237,106,142,130,213,90,208,17,8,220,46,55,65,77,247,216,237,118,154,91,154,9,134,195,36,147,73,98,241,24,146,36,33,43,
  178,222,74,178,64,132,136,18,91,84,39,64,10,119,10,137,52,121,114,163,40,169,198,100,216,22,25,166,111,134,155,216,136,67,99,99,19,129,64,64,
  199,65,86,100,29,7,73,171,227,154,199,193,95,227,163,161,190,70,199,193,235,117,155,112,240,120,125,186,251,222,102,179,99,53,232,127,69,81,8,5,
  11,56,184,220,46,19,14,241,120,28,159,207,171,182,6,95,0,135,114,100,212,124,56,44,72,72,81,117,215,223,80,99,52,31,47,2,16,12,133,241,
  121,60,120,61,30,166,167,166,212,196,37,212,192,229,76,38,75,36,18,37,42,98,90,9,19,116,182,82,178,88,176,90,84,70,212,231,243,128,128,150,
  150,102,186,186,58,177,90,109,106,145,90,253,150,23,154,66,175,88,189,189,27,2,135,195,129,199,227,65,86,20,210,233,52,94,175,151,59,239,186,139,
  112,56,172,182,6,213,100,176,189,163,131,218,218,90,53,246,196,106,197,106,177,104,139,164,16,143,34,229,227,131,36,9,201,98,209,75,150,228,111,23,
  206,157,227,220,185,179,243,179,163,80,53,70,171,163,58,170,163,58,174,241,120,199,35,143,240,142,119,190,147,100,42,197,215,191,241,77,26,234,235,65,
  81,8,135,195,228,114,57,221,37,155,76,166,72,167,51,234,182,172,233,28,4,72,66,226,194,121,139,86,88,221,129,215,235,193,229,118,177,110,221,90,
  181,230,181,36,33,12,181,35,11,236,157,122,199,233,114,234,159,161,182,22,245,97,181,88,212,240,178,84,154,245,27,54,162,40,50,39,79,156,32,52,
  23,66,72,134,124,133,50,246,137,40,186,163,198,45,106,140,168,129,29,53,85,200,172,64,132,228,19,123,111,76,157,215,95,167,119,213,42,226,137,36,
  223,252,214,183,104,168,175,39,18,14,19,214,88,209,197,224,160,146,83,42,14,181,181,254,138,56,8,195,252,228,235,182,78,27,112,176,88,44,38,28,
  82,169,20,27,55,109,70,206,229,56,113,226,56,161,80,136,197,244,51,51,126,142,238,75,45,199,82,47,16,59,122,35,113,168,50,163,70,0,133,32,
  147,205,210,223,223,79,79,79,175,182,160,84,67,49,149,78,17,137,70,244,178,77,40,104,253,233,21,181,116,67,58,75,58,157,34,22,139,209,218,214,
  194,7,62,240,19,196,98,81,34,225,152,150,145,159,34,157,201,144,211,234,149,42,57,245,181,153,76,218,20,11,154,78,165,136,68,10,197,124,213,147,
  41,234,162,208,130,160,133,36,24,30,26,98,98,124,76,99,66,243,6,166,40,48,165,250,253,50,236,168,16,72,66,232,177,48,197,253,131,139,197,174,
  42,140,213,81,29,213,81,29,215,111,200,178,204,196,196,4,195,35,35,248,188,62,83,60,96,36,26,209,146,93,100,131,174,80,11,170,103,50,25,50,
  233,52,201,100,146,100,42,201,7,222,255,62,154,154,26,137,70,99,196,227,73,162,209,24,169,84,154,92,46,91,48,172,114,50,57,89,198,106,181,233,
  159,33,203,50,145,112,152,108,54,171,181,237,84,195,180,100,237,166,230,39,24,148,131,76,133,76,142,130,41,35,42,25,67,139,13,19,187,193,68,136,
  48,224,48,53,61,173,51,211,11,225,144,47,223,152,199,33,145,72,240,193,15,254,132,142,67,52,26,39,30,79,144,74,165,201,230,212,122,163,121,28,
  178,185,156,9,135,92,46,87,22,7,85,255,43,186,151,212,116,155,23,135,188,59,159,82,118,212,168,227,203,242,172,85,66,234,134,26,163,125,125,125,
  252,218,175,252,10,0,127,252,165,47,33,203,10,151,6,7,105,110,105,193,106,177,146,202,165,64,81,72,167,146,106,175,95,73,82,5,35,87,168,35,
  154,74,37,73,38,83,36,19,73,53,62,196,102,39,22,139,178,172,171,29,69,129,68,34,77,40,18,37,22,75,16,143,169,113,36,153,76,6,57,151,
  99,106,122,82,15,5,200,102,179,4,131,65,102,102,3,72,90,29,178,96,32,200,233,83,39,245,230,94,249,77,33,30,143,21,220,241,6,35,179,112,
  147,76,108,108,190,32,117,225,116,102,184,95,225,164,85,61,25,85,71,117,84,71,117,92,159,241,196,227,143,243,196,227,143,115,223,3,15,128,128,153,
  217,89,70,70,70,88,187,102,173,222,56,37,207,148,58,28,14,173,155,143,218,209,39,95,54,48,153,76,146,76,36,201,100,210,120,189,62,194,225,8,
  61,43,151,211,218,210,136,44,43,4,67,49,162,145,24,137,68,138,120,34,78,54,147,33,155,205,144,74,38,11,137,175,168,201,64,19,90,37,23,155,
  214,205,233,196,137,19,204,5,3,24,219,177,10,73,32,229,173,31,165,96,238,20,88,81,197,108,2,41,148,101,71,85,86,174,162,234,185,161,68,200,
  87,190,252,101,0,19,14,147,147,147,52,55,53,151,197,33,95,170,49,143,67,58,157,34,149,74,235,56,52,54,54,86,196,33,22,75,144,76,37,117,
  28,212,4,174,2,14,241,120,220,132,131,64,112,236,216,49,51,14,154,121,95,17,7,97,122,16,83,116,105,177,193,47,202,224,80,38,150,183,106,3,
  220,64,102,180,181,173,13,187,221,78,75,75,11,19,19,19,140,140,140,208,221,221,13,90,66,94,50,153,36,26,141,96,181,218,200,100,212,22,159,153,
  108,150,76,58,75,42,149,34,155,201,160,40,10,94,175,7,139,197,74,44,150,208,49,117,187,237,184,92,245,100,178,57,98,177,20,137,68,82,163,252,
  211,92,28,236,207,231,189,233,130,152,206,100,52,163,214,70,42,153,44,9,88,214,79,169,146,158,51,103,136,211,17,11,8,35,101,75,116,204,23,55,
  82,21,198,234,168,142,234,168,142,235,51,26,26,27,201,40,106,206,192,240,240,48,221,221,221,166,214,158,161,80,0,175,183,70,101,66,211,25,178,58,
  35,170,150,5,202,229,114,88,45,86,92,46,39,137,68,138,108,86,77,184,145,36,65,67,157,151,186,90,15,201,84,142,104,52,78,50,145,36,149,74,
  19,8,6,244,90,157,114,78,102,46,56,71,36,26,69,150,101,28,110,15,22,201,130,221,102,83,19,172,133,82,32,67,100,5,89,146,85,67,72,201,
  155,63,249,36,89,12,137,76,121,87,177,82,196,142,154,20,13,75,197,85,15,106,141,210,60,14,23,47,94,164,166,166,166,44,14,153,76,150,92,54,
  171,227,144,74,165,201,100,210,6,28,92,21,113,136,199,51,196,19,73,29,135,68,34,97,194,97,114,114,210,132,131,16,104,7,7,3,14,138,106,3,
  40,82,65,247,171,122,95,49,215,17,47,74,110,202,31,8,116,88,22,129,67,85,255,151,142,43,138,160,189,210,130,4,127,255,207,255,204,183,191,251,
  93,106,106,106,88,183,110,29,227,19,19,92,186,120,145,88,44,134,213,102,67,86,20,18,137,4,225,112,152,104,52,70,44,22,85,127,70,99,196,227,
  49,50,233,20,138,34,99,181,217,112,123,60,8,73,34,26,141,145,72,164,11,194,32,192,110,179,80,87,235,166,165,165,142,142,142,38,58,58,90,25,
  30,26,84,5,81,150,153,154,156,100,108,124,28,64,13,124,118,185,89,177,98,5,239,122,247,187,233,234,236,210,42,47,40,186,112,168,133,112,13,37,
  29,180,83,147,185,32,110,190,204,19,250,29,99,131,55,101,161,9,83,10,159,87,29,87,46,95,213,89,171,98,94,29,85,204,203,141,109,219,183,243,
  226,129,3,124,236,227,31,167,173,173,29,183,199,195,208,240,48,83,83,83,186,87,75,150,21,66,161,57,205,221,27,37,26,139,18,141,70,137,197,98,
  36,18,9,114,217,44,146,144,112,186,156,56,93,46,82,169,20,145,104,130,92,174,80,184,92,18,2,183,211,74,83,131,159,182,182,70,58,58,90,240,
  251,220,132,66,115,40,138,66,60,30,99,116,108,148,68,50,9,128,223,235,197,229,112,112,203,173,183,242,174,119,191,219,80,18,200,172,123,204,250,199,
  160,111,76,255,140,201,176,152,244,143,178,200,185,187,222,186,231,174,61,123,120,241,192,1,222,255,193,15,234,56,92,188,56,80,17,135,88,204,140,67,
  50,105,198,193,238,112,84,196,193,235,177,155,112,80,148,140,9,135,161,161,33,19,14,54,171,141,123,239,187,143,251,238,191,191,4,7,185,168,9,129,
  73,239,235,207,51,118,174,42,143,195,124,50,172,220,64,28,110,134,117,14,96,85,174,242,67,22,243,186,111,127,243,155,120,189,94,124,254,26,214,175,
  95,207,15,126,248,3,198,38,198,25,29,29,101,237,218,181,170,176,229,50,164,146,113,228,156,66,78,206,145,203,170,238,146,124,183,2,139,69,82,75,
  55,216,109,8,33,72,38,83,132,34,81,92,238,250,146,83,134,69,18,88,236,86,162,145,16,67,67,151,213,62,246,185,28,147,147,147,100,181,50,18,
  118,187,29,159,215,203,249,243,231,112,58,93,204,228,251,249,154,138,222,26,58,46,104,146,170,103,52,138,114,211,44,204,147,83,225,100,84,105,200,111,
  243,211,145,242,6,4,184,106,160,84,49,175,142,42,230,198,49,62,62,206,223,255,221,223,49,50,60,204,198,45,91,105,110,110,230,210,197,139,12,92,
  28,160,165,165,69,109,158,146,203,146,73,37,72,196,227,122,124,162,172,185,234,243,110,115,171,205,138,195,225,192,102,179,145,203,229,8,133,162,212,215,
  215,224,178,216,205,187,191,0,155,85,66,145,179,204,5,103,8,133,230,144,101,153,185,185,57,130,193,57,157,253,170,171,173,33,155,205,112,234,212,73,
  178,217,172,126,97,70,246,211,212,221,39,207,134,106,44,168,80,10,58,200,100,191,20,5,139,26,186,89,47,192,135,40,122,9,171,235,49,250,250,250,
  248,251,191,251,59,206,156,62,205,237,119,237,161,185,185,153,203,131,131,21,113,200,229,99,56,43,224,96,181,90,23,141,195,244,244,164,9,135,88,60,
  110,194,33,18,14,51,54,58,74,54,155,153,7,7,181,228,99,161,60,129,162,86,43,64,80,62,48,180,188,75,244,205,198,225,102,89,231,215,213,77,
  255,163,199,31,7,96,231,45,183,112,231,221,123,113,185,92,4,231,230,152,158,158,98,253,250,245,72,66,104,46,146,20,178,2,173,173,237,100,179,25,
  66,115,97,146,201,20,8,176,218,108,216,180,18,26,66,8,210,153,12,177,152,122,50,178,90,74,137,221,76,38,195,75,47,189,68,56,28,210,50,245,
  146,204,4,2,234,9,74,146,112,187,92,32,4,67,67,67,6,83,210,44,89,122,151,5,69,32,68,94,40,53,167,124,217,130,112,80,193,26,93,28,
  120,85,170,190,58,170,163,58,170,227,154,140,241,241,113,254,254,111,255,22,128,186,250,6,150,117,117,241,234,171,175,50,62,62,78,58,157,198,229,114,
  65,6,228,92,142,100,50,130,144,236,116,116,180,51,55,23,34,26,137,146,205,230,176,88,44,216,108,54,108,54,181,212,144,172,40,196,227,9,146,201,
  12,14,187,13,73,50,239,215,178,44,51,62,62,206,11,47,188,160,22,198,207,100,8,206,169,46,122,0,155,205,134,207,231,99,118,102,150,201,201,9,
  179,230,49,16,33,106,161,117,85,239,24,245,79,161,178,83,254,239,149,244,77,161,139,80,197,18,79,134,251,215,83,247,24,113,216,121,235,46,150,117,
  117,241,202,43,175,152,112,80,82,249,140,250,8,46,151,159,150,150,38,29,135,92,78,54,225,96,177,88,76,56,184,156,246,82,98,71,195,225,196,137,
  227,38,28,82,90,55,170,60,14,83,83,83,76,76,140,47,128,67,129,144,202,31,4,20,81,108,185,106,83,89,66,56,45,2,135,170,13,112,13,140,
  209,197,82,163,192,170,213,171,120,232,161,135,168,107,104,164,123,197,10,78,159,62,205,228,196,36,211,83,83,184,221,110,114,217,44,185,92,134,174,101,
  203,216,187,119,15,8,120,245,213,215,185,56,48,136,44,203,170,32,218,213,18,75,66,168,217,239,201,68,146,116,58,135,213,85,106,140,158,63,127,158,
  3,7,14,234,201,75,211,211,83,164,51,234,233,199,110,179,209,80,87,135,215,227,161,163,163,131,169,201,41,50,153,140,241,224,163,31,101,20,69,81,
  11,218,230,91,129,41,90,224,184,80,239,191,81,19,180,228,249,111,103,186,71,121,147,94,91,29,85,204,171,227,45,139,249,59,31,121,132,59,239,186,
  147,112,52,66,83,83,19,179,211,211,12,15,13,177,122,205,26,44,146,68,38,151,35,157,78,114,247,222,59,89,187,118,53,163,163,163,28,60,248,26,
  115,193,57,172,86,171,102,4,89,213,22,206,40,106,85,151,120,18,175,215,137,36,153,139,150,71,34,17,14,31,126,149,83,167,78,161,40,10,177,88,
  140,80,56,76,54,167,50,160,245,53,53,120,220,30,156,53,181,100,50,105,2,179,1,76,201,71,154,33,164,23,82,55,18,34,69,198,79,113,222,66,
  177,62,81,116,3,169,194,196,136,27,183,134,188,62,47,123,246,220,205,234,213,171,232,27,232,199,227,118,19,154,11,233,56,228,179,202,211,233,36,247,
  63,240,32,93,93,157,58,14,161,185,144,9,7,169,8,135,186,90,79,201,231,229,113,24,26,26,50,225,144,247,180,214,215,212,226,245,120,112,215,215,
  19,143,197,180,174,83,11,224,80,124,48,200,207,178,9,146,69,226,96,250,253,202,137,171,183,242,58,191,238,85,87,127,246,147,159,228,29,143,60,130,
  0,110,219,189,27,167,203,197,228,244,52,99,99,99,248,124,62,100,69,97,211,166,205,124,240,131,239,103,251,182,45,108,219,178,153,181,107,87,227,241,
  122,176,218,172,122,81,225,124,223,90,89,206,105,101,30,146,228,100,243,85,31,60,120,144,111,124,227,27,140,140,142,32,43,170,171,255,220,185,115,58,
  43,90,227,247,211,213,209,129,197,34,177,113,211,38,245,132,172,157,86,148,10,140,101,113,123,79,20,115,140,135,57,118,71,185,106,44,170,177,163,213,
  81,29,213,81,29,215,102,180,181,181,241,59,255,227,247,104,104,104,96,89,215,50,54,172,95,79,42,157,166,127,96,0,151,211,137,221,102,67,150,101,
  62,246,147,63,201,123,30,125,39,27,215,175,101,203,150,205,116,118,182,99,177,90,177,217,172,216,172,86,172,22,43,146,69,210,123,162,171,37,157,50,
  166,207,154,157,157,229,63,254,243,63,120,234,169,39,201,100,213,132,219,233,233,105,102,180,94,240,118,155,141,174,142,14,60,46,39,173,109,173,172,93,
  183,110,94,37,94,104,67,73,105,155,233,197,43,148,69,25,11,215,91,239,228,113,104,108,108,100,221,154,181,116,117,45,35,105,192,33,95,254,234,99,
  63,249,147,60,112,255,61,243,227,32,9,19,14,153,172,92,17,135,92,46,87,1,135,118,220,78,23,205,45,45,244,172,234,157,31,135,252,252,27,77,
  0,227,124,41,139,32,230,138,231,87,121,115,112,184,25,198,117,207,166,255,202,255,250,50,95,249,95,95,38,145,72,240,158,15,188,159,13,199,142,113,
  252,248,113,166,166,166,0,216,186,109,27,15,62,248,32,93,157,157,88,44,18,193,185,16,3,3,131,36,19,73,44,22,43,54,155,90,112,62,223,229,
  72,201,201,90,233,141,180,234,78,177,171,173,191,14,30,60,200,147,79,62,201,229,203,67,100,51,89,20,69,33,17,143,19,139,171,173,67,93,78,23,
  157,109,237,196,34,81,78,157,56,73,125,125,189,222,139,183,156,0,229,79,74,69,165,132,185,94,103,154,42,77,95,29,213,81,29,213,113,109,198,248,
  248,56,159,250,229,95,225,200,235,175,115,231,158,61,108,223,182,141,195,175,190,202,244,244,52,241,120,28,139,197,194,199,63,241,9,118,108,223,142,195,
  110,39,157,201,48,57,57,197,240,208,40,22,73,194,106,181,98,181,169,134,168,208,138,151,231,114,89,210,169,20,137,100,6,175,199,9,168,149,90,126,
  240,131,31,240,242,43,175,16,137,70,85,99,41,157,38,158,72,144,206,100,16,66,208,214,210,74,67,125,61,253,23,250,56,121,252,4,54,155,173,172,
  174,89,136,108,18,11,228,30,148,117,1,47,244,154,235,172,123,250,46,244,233,56,188,231,125,239,227,214,91,118,50,62,49,174,227,96,181,217,248,208,
  71,62,194,142,237,219,177,219,108,102,28,180,102,51,21,113,72,100,176,249,28,0,140,142,142,242,244,211,79,47,10,135,211,39,79,114,214,98,41,242,
  76,46,48,119,243,100,196,87,109,128,107,51,164,27,177,41,140,143,143,51,51,51,195,204,228,20,187,119,237,166,174,174,142,203,195,195,204,205,205,177,
  122,213,106,90,91,90,177,88,44,204,133,66,124,239,251,223,227,220,153,147,164,82,9,44,150,124,215,35,141,162,23,106,178,143,156,203,145,76,165,73,
  167,115,204,206,206,242,163,31,253,136,239,124,231,187,12,94,190,172,186,221,181,35,199,200,200,8,10,96,181,90,105,168,171,165,173,181,133,233,169,41,
  80,20,2,90,28,233,226,78,146,74,233,115,42,176,169,213,81,29,213,81,29,213,241,230,143,163,71,142,160,40,10,135,14,28,96,243,166,77,172,94,
  181,138,84,58,205,225,215,94,197,233,116,178,126,221,122,28,118,53,238,240,236,185,115,124,255,123,223,101,46,56,131,130,162,233,30,171,174,123,20,69,
  237,41,158,205,102,73,38,83,40,138,66,95,95,31,127,254,149,175,240,242,203,47,107,117,45,213,172,235,112,56,172,19,29,14,187,157,142,214,86,228,
  92,142,112,56,76,38,147,33,158,72,44,108,152,148,33,210,110,118,28,246,239,219,199,238,93,187,77,56,116,119,119,235,56,100,50,25,19,14,66,96,
  194,65,237,172,84,192,33,158,40,224,240,215,127,243,55,243,226,224,113,185,233,104,109,37,147,74,147,74,165,180,150,221,137,178,115,190,8,171,241,106,
  44,205,234,130,92,96,220,144,14,76,138,162,240,243,191,240,11,220,255,224,3,28,60,116,152,203,151,7,57,124,248,176,86,164,62,205,236,236,44,94,
  159,151,239,126,239,123,28,121,253,117,82,201,148,154,105,151,77,227,116,180,18,143,71,113,187,92,40,90,214,99,112,46,192,212,244,56,225,240,28,227,
  99,163,12,13,13,17,156,11,234,113,162,249,231,13,92,186,4,168,165,28,218,91,90,241,121,189,108,221,190,141,227,71,143,233,129,229,213,81,29,213,
  81,29,213,241,214,28,221,221,221,124,234,211,159,198,231,247,113,247,93,123,152,157,13,48,59,51,131,195,225,96,120,232,50,142,222,85,68,227,49,254,
  253,219,223,102,124,124,28,73,72,164,51,41,172,22,176,88,212,106,46,78,167,3,89,145,137,39,18,196,70,135,185,116,233,2,63,156,154,228,242,229,
  75,140,12,143,144,205,101,77,37,128,102,2,1,130,193,160,70,130,212,83,87,83,67,99,83,35,114,46,199,192,192,128,94,39,115,81,6,204,141,96,
  202,110,192,231,116,119,119,243,219,191,243,59,164,51,105,246,220,181,135,217,217,89,102,103,102,168,111,104,208,113,184,208,223,103,194,33,155,77,35,200,
  233,56,120,60,110,19,14,23,7,206,50,61,51,195,229,203,151,24,30,26,38,39,231,76,157,175,140,56,180,181,52,83,87,91,75,99,99,3,66,192,
  249,115,231,171,139,227,173,97,140,94,121,252,74,247,202,110,142,29,61,202,15,190,247,61,222,241,240,195,32,4,95,251,198,55,120,126,255,126,118,237,
  218,69,141,223,175,211,235,0,185,108,142,116,58,77,127,127,8,69,86,184,128,161,214,167,172,232,236,167,34,231,235,178,169,89,121,241,88,140,203,151,
  47,211,63,48,64,86,150,113,58,157,172,90,185,18,175,219,205,249,115,231,217,184,113,35,86,155,141,197,59,218,231,115,208,207,83,226,169,196,193,95,
  206,225,95,244,222,55,106,243,89,114,163,122,106,172,98,94,29,85,204,175,145,225,179,114,37,151,46,94,212,247,212,150,214,22,190,246,47,255,138,215,
  237,230,39,222,247,62,190,251,189,239,242,249,47,124,145,151,94,126,153,205,155,54,19,12,6,200,104,165,150,114,74,142,108,46,203,228,196,24,19,227,
  163,186,62,50,214,158,204,19,44,70,242,35,147,201,16,152,157,101,96,96,128,169,153,25,44,22,11,45,77,77,244,174,232,102,124,124,92,45,41,84,
  95,79,246,220,249,50,123,124,65,15,56,157,78,100,57,167,149,126,18,243,232,156,114,9,77,21,244,202,18,24,30,143,27,143,215,195,223,254,217,95,
  209,209,181,76,195,225,123,252,210,47,253,23,238,185,103,47,155,55,109,38,16,152,213,75,48,230,148,28,217,100,150,68,34,174,227,96,174,245,61,63,
  14,103,206,158,37,20,14,35,105,56,244,172,232,102,100,104,8,1,248,124,254,69,232,122,163,190,94,132,238,174,136,197,210,58,20,44,229,117,110,189,
  17,31,168,40,10,191,243,219,191,173,223,111,109,111,227,145,119,188,147,201,137,73,206,157,59,199,247,190,251,61,178,185,44,45,205,205,244,244,246,210,
  216,216,136,197,98,81,221,35,114,62,41,168,212,16,149,181,186,100,241,120,156,225,161,33,70,71,71,137,106,245,202,0,92,78,39,61,43,186,105,107,
  105,225,248,209,99,100,179,89,246,61,247,156,65,86,12,130,51,95,27,37,81,70,190,196,124,205,62,89,192,16,45,126,206,82,221,66,170,10,174,58,
  170,152,87,199,205,131,249,150,173,91,185,243,174,61,252,239,191,248,10,0,23,47,14,240,145,199,30,211,107,71,254,151,95,253,20,239,122,231,35,252,
  224,241,31,114,232,224,33,94,120,225,5,80,20,214,172,93,203,138,21,43,112,187,221,90,59,103,81,48,120,40,53,128,100,89,205,93,8,135,195,244,
  247,245,49,53,51,67,38,147,69,65,193,102,181,210,220,216,68,207,138,21,100,211,105,166,167,166,212,240,48,125,215,87,176,105,9,84,133,226,237,10,
  30,143,151,101,203,151,51,60,52,68,46,155,157,71,199,20,207,135,40,210,105,98,30,66,164,84,239,92,239,244,122,69,81,56,121,226,4,31,253,208,
  135,64,1,231,137,19,252,151,95,253,20,233,84,154,127,252,231,127,210,113,112,58,28,44,95,177,162,50,14,74,101,28,166,167,166,24,30,30,174,136,
  67,56,20,34,16,8,152,194,243,68,69,195,126,1,45,44,230,147,73,81,222,14,16,111,62,14,75,125,88,111,244,7,122,60,30,30,125,207,123,168,
  173,173,101,118,246,81,172,86,43,71,143,31,35,157,204,48,59,59,203,204,204,44,86,171,149,101,203,186,88,217,211,131,219,237,46,57,1,161,168,109,
  68,3,193,0,163,35,163,76,78,78,146,72,37,77,34,224,245,120,88,185,98,5,221,93,93,212,215,215,179,118,221,90,78,157,60,85,198,144,20,101,
  229,169,96,127,22,119,153,159,167,235,252,27,176,36,69,213,12,173,142,234,168,142,234,184,234,113,223,253,15,16,137,132,153,153,153,214,117,134,81,193,
  223,115,239,189,236,221,123,55,93,93,157,72,22,137,127,249,215,127,37,20,141,96,149,36,46,14,12,48,208,63,128,223,239,99,101,79,15,173,173,173,
  122,162,81,222,251,150,191,31,139,199,24,31,27,99,112,240,50,161,112,8,217,240,41,22,139,133,214,150,22,122,86,172,160,189,181,13,155,213,66,48,
  24,32,18,142,152,116,68,77,77,45,241,120,156,120,60,142,0,234,27,26,112,57,93,68,34,17,12,141,208,223,184,126,89,130,106,101,247,238,221,236,
  186,245,22,26,234,235,9,206,205,241,163,39,126,68,40,26,65,145,101,19,14,235,214,173,163,174,190,190,128,131,98,198,33,28,9,51,53,57,89,22,
  7,187,221,174,49,162,43,104,105,106,194,225,112,48,55,23,44,193,161,116,170,68,249,9,20,87,169,165,171,106,125,105,25,163,121,38,211,227,241,240,
  135,127,244,71,44,91,190,130,191,248,243,47,51,49,49,201,7,223,255,126,122,123,122,120,245,181,87,25,184,116,137,84,42,137,37,35,49,112,241,34,
  131,131,131,44,91,190,140,250,186,122,106,107,107,145,44,22,146,137,4,51,179,179,140,12,15,19,152,155,211,221,43,160,150,111,242,184,92,52,214,55,
  208,211,189,2,139,36,113,226,248,113,214,174,91,71,71,103,39,8,193,169,147,39,203,26,162,162,146,69,42,74,172,87,19,147,46,74,94,43,202,191,
  87,117,84,71,117,84,71,117,92,183,241,236,51,79,3,208,222,209,89,242,183,123,239,187,143,95,254,212,167,24,24,232,231,143,62,251,89,30,124,232,
  97,126,249,151,126,137,231,158,127,158,179,103,207,50,59,27,64,150,115,100,114,89,194,71,143,81,91,91,67,107,91,43,181,181,117,184,221,110,20,89,
  102,46,20,98,122,122,154,201,137,9,130,90,98,140,0,132,16,216,172,86,106,252,126,218,154,91,88,190,172,139,209,225,17,166,39,38,216,121,203,45,
  220,114,235,173,188,122,248,85,34,145,48,118,173,128,187,205,110,199,41,203,32,4,233,84,82,171,59,10,45,173,45,90,63,116,244,159,32,202,120,111,
  69,5,189,34,150,168,202,81,205,68,35,14,127,246,39,95,226,190,7,30,100,197,47,253,18,207,237,123,158,147,167,78,18,10,133,117,28,14,29,58,
  108,194,193,227,241,32,231,114,58,14,195,67,67,196,180,22,159,197,56,172,232,90,70,99,67,61,147,227,19,101,113,16,139,152,47,33,40,224,64,185,
  51,130,40,186,85,117,254,146,55,70,65,101,42,255,232,243,159,103,217,242,229,124,229,203,95,102,255,190,125,40,192,248,216,40,15,60,244,16,203,150,
  117,241,236,190,125,92,186,120,137,96,104,142,100,50,137,213,98,97,232,242,48,67,151,135,232,236,236,196,102,179,17,12,6,153,158,153,33,149,78,235,
  44,168,36,73,184,156,78,252,62,31,93,237,29,116,117,116,48,57,49,193,208,229,65,0,78,107,6,104,103,103,39,146,16,156,60,113,194,44,44,162,
  140,0,25,133,208,240,247,74,33,29,226,10,143,69,85,38,180,58,170,163,58,170,227,234,71,99,163,202,120,1,140,142,142,84,124,222,125,247,223,207,
  167,62,253,105,250,251,251,249,236,255,248,12,177,88,140,239,125,231,59,108,217,182,141,247,190,251,81,86,247,174,226,133,151,94,100,114,114,146,104,44,
  70,60,149,36,23,144,9,133,66,248,124,62,234,235,235,201,100,50,76,77,77,17,142,70,245,48,48,33,4,22,139,5,159,199,75,99,99,3,43,58,
  187,112,57,157,244,93,56,79,76,75,144,125,237,213,87,185,229,214,91,217,181,123,23,135,15,29,34,151,205,226,118,123,176,217,108,106,182,184,213,66,
  54,147,81,221,245,186,225,41,42,24,162,121,235,168,136,40,161,36,130,108,73,178,117,247,221,127,63,191,250,107,191,198,192,192,0,159,253,31,159,33,
  26,139,242,205,175,253,27,187,110,187,157,247,62,250,40,173,173,45,28,59,126,92,199,33,153,78,49,27,8,234,56,52,53,53,145,76,38,203,226,96,
  179,217,240,184,220,52,54,52,176,162,107,97,28,34,225,48,21,253,157,250,212,139,50,7,3,74,49,48,96,81,14,7,81,105,238,171,38,192,155,99,
  140,122,188,94,154,91,90,248,139,63,255,243,66,220,38,48,55,55,199,183,190,241,77,54,111,217,194,195,15,60,200,212,212,20,167,207,158,97,104,120,
  152,64,32,64,50,149,66,66,98,104,120,132,116,58,69,78,150,17,66,96,183,219,177,88,172,56,236,118,220,110,23,157,109,109,116,180,183,19,154,155,
  227,228,137,19,36,19,9,19,218,167,78,158,68,8,169,244,20,35,202,11,162,16,170,192,9,237,120,36,74,12,209,10,39,34,49,223,226,175,74,95,
  117,84,71,117,84,199,53,49,70,155,26,241,249,253,101,140,209,210,230,35,151,46,93,226,51,191,251,187,196,226,49,253,177,99,71,143,112,246,236,25,
  238,185,247,62,126,242,35,31,225,244,153,51,156,239,235,99,106,106,138,72,36,66,46,151,35,18,141,49,23,10,145,74,165,116,55,188,213,102,195,110,
  181,225,116,57,241,122,60,244,172,232,166,174,182,134,137,177,113,46,156,61,107,8,15,128,72,56,204,107,135,15,179,109,199,14,4,130,120,60,65,60,
  158,160,73,8,205,77,31,51,233,5,73,18,8,169,216,16,21,149,25,60,65,5,98,229,202,12,208,235,89,227,50,143,67,247,202,149,12,150,224,32,
  56,120,224,21,206,157,61,203,246,157,59,217,240,225,245,156,57,123,134,243,23,250,24,159,24,39,30,143,235,56,4,130,65,173,116,35,90,155,80,59,
  86,171,21,151,203,73,67,93,29,157,237,29,248,60,30,70,70,134,57,63,49,161,125,112,121,28,22,156,179,178,134,168,78,149,150,231,165,223,168,161,
  41,222,222,121,35,214,171,17,170,43,232,6,170,9,58,76,78,78,242,145,15,125,200,84,159,211,152,186,115,226,216,49,142,29,59,202,198,13,27,217,
  178,113,35,247,220,125,55,23,46,244,49,54,49,65,32,48,75,56,18,33,151,201,34,203,57,44,54,27,30,183,27,175,219,67,93,77,13,22,139,133,
  72,40,196,233,147,39,73,84,172,225,150,103,68,203,95,151,40,35,136,66,80,102,19,40,21,67,83,56,178,82,48,104,149,162,199,22,151,141,247,246,
  27,111,164,166,94,53,149,165,138,121,117,188,61,49,63,123,246,108,69,125,83,232,94,164,240,204,51,207,240,204,211,207,0,134,110,58,218,59,38,19,
  9,126,244,195,31,224,175,169,97,205,154,53,60,250,206,71,136,39,226,244,245,15,48,53,61,205,92,104,142,100,34,73,54,147,65,72,2,187,221,129,
  203,229,164,206,95,67,83,99,35,241,88,140,153,169,41,46,245,247,145,201,100,202,124,81,65,56,28,97,255,190,231,77,143,198,226,113,50,153,116,137,
  22,16,66,42,16,32,139,241,252,42,160,104,253,212,205,186,166,188,14,42,126,206,141,1,94,197,225,111,254,250,175,243,192,148,224,16,12,6,120,230,
  169,39,241,215,212,176,121,243,22,30,125,228,17,2,193,32,195,35,35,76,77,79,49,23,10,145,140,39,200,102,179,26,14,118,188,30,47,126,143,135,
  166,166,70,130,129,32,225,96,144,190,115,218,97,96,145,56,228,231,161,4,7,163,1,90,198,14,40,155,51,111,108,150,163,148,231,169,42,225,160,183,
  183,215,90,147,190,29,215,185,85,185,218,2,174,139,124,157,250,254,230,162,241,162,108,139,51,5,161,192,169,83,39,57,117,242,4,178,162,208,222,222,
  65,107,75,11,109,77,77,212,214,214,146,201,100,200,102,51,100,179,89,198,199,198,136,69,99,140,92,86,131,151,43,181,217,90,228,129,196,36,132,198,
  19,145,70,144,206,187,27,8,131,132,10,109,99,200,247,184,53,39,52,170,29,36,242,46,0,211,245,191,157,19,233,140,237,85,175,101,219,187,234,168,
  98,94,29,111,59,204,207,157,61,99,122,126,137,190,81,74,223,51,60,55,199,225,131,135,56,116,224,32,62,191,143,214,214,54,150,181,183,177,121,253,
  122,36,73,210,116,79,150,80,104,142,72,56,76,104,46,196,177,203,151,73,167,211,87,165,116,226,177,152,41,20,76,85,9,130,88,44,70,38,147,209,
  117,142,81,7,149,248,226,52,29,35,76,22,85,94,255,40,102,29,99,212,65,20,140,53,81,244,251,245,129,93,185,34,28,94,220,191,31,69,81,240,
  250,188,180,181,181,179,172,189,157,205,235,55,32,36,65,54,147,37,155,205,16,154,11,169,201,106,211,51,140,14,13,93,53,14,232,198,161,25,7,81,
  68,72,137,10,158,121,81,142,209,50,224,32,140,56,24,231,186,120,222,141,217,246,55,251,254,118,149,235,252,186,187,233,231,163,254,205,197,14,132,186,
  176,116,251,77,48,54,58,202,232,200,136,169,164,131,146,175,247,102,168,47,42,212,243,132,225,68,147,23,250,249,45,208,146,136,17,97,16,196,188,155,
  30,97,120,12,211,173,36,102,167,28,147,42,230,201,190,175,142,234,168,142,234,168,142,107,58,206,85,96,76,11,70,1,6,23,174,98,250,85,160,186,
  116,195,161,80,145,222,41,148,22,148,13,58,72,237,10,84,96,38,175,76,247,148,26,64,177,104,84,189,47,229,117,16,101,205,159,194,195,102,23,126,
  249,56,197,55,87,251,20,230,104,145,56,104,15,69,35,17,46,132,207,85,192,192,108,11,20,236,110,213,26,200,243,65,87,139,131,249,86,14,3,81,
  86,247,151,201,119,174,142,165,98,140,150,51,62,203,62,34,10,68,98,254,151,121,147,130,12,39,71,36,237,144,33,23,200,243,252,198,80,78,30,203,
  102,211,27,133,79,50,8,161,16,21,5,177,240,103,81,214,200,173,184,33,20,109,14,213,158,244,213,81,29,213,81,29,215,205,26,42,120,229,140,4,
  136,174,115,52,79,152,82,160,52,42,191,151,129,137,52,245,43,49,234,30,131,49,180,128,222,209,117,143,36,76,121,10,197,198,144,137,0,17,6,253,
  83,129,88,153,71,115,46,73,28,140,12,161,208,24,195,74,22,128,122,189,197,117,192,85,35,192,24,175,123,197,56,72,162,130,49,90,74,68,85,38,
  163,42,188,247,82,194,161,106,140,206,115,66,53,185,19,180,83,82,25,97,52,189,76,167,204,37,141,23,149,81,164,124,4,129,57,162,67,176,208,233,
  168,216,8,157,95,8,203,11,162,184,57,54,132,234,168,142,234,248,255,217,59,239,248,184,174,50,239,127,239,244,209,204,168,87,203,69,238,150,171,220,
  228,18,215,84,167,87,18,122,18,202,238,11,33,176,148,44,11,203,178,192,178,244,150,64,150,101,89,66,18,96,33,13,2,36,182,83,29,199,78,220,
  226,222,45,75,150,100,245,174,233,245,222,247,143,123,231,106,70,154,145,37,217,114,61,191,124,38,30,205,204,157,121,206,125,158,115,158,223,121,206,115,
  158,35,32,24,106,218,64,136,36,157,33,156,22,135,22,4,65,171,91,175,232,203,174,67,91,13,235,243,47,113,18,106,72,90,145,139,239,172,31,16,
  0,33,145,148,246,11,112,140,112,69,238,194,5,68,18,162,162,146,148,60,33,72,27,144,74,161,7,57,97,98,112,22,122,144,36,67,242,132,32,97,
  101,116,104,193,168,129,59,242,5,46,70,50,154,114,102,164,82,201,68,99,132,132,101,123,18,103,159,90,200,63,225,122,131,62,164,104,207,226,163,138,
  50,120,190,130,148,88,70,67,91,14,73,63,43,26,104,144,103,50,196,145,12,8,2,2,2,2,2,163,72,61,207,16,8,145,244,205,53,253,203,250,
  13,2,3,72,138,164,31,89,153,184,65,39,45,241,139,251,27,164,129,169,97,73,149,92,250,7,65,250,147,82,105,144,3,90,184,232,86,228,134,163,
  135,254,209,209,65,197,77,116,255,138,116,102,61,36,236,9,33,141,30,250,133,163,83,4,164,82,7,163,164,180,69,75,47,30,61,92,86,100,116,88,
  41,182,253,114,70,18,141,49,30,162,87,226,127,107,164,148,68,99,148,6,254,114,92,225,9,41,39,234,170,188,146,240,135,146,110,87,80,255,196,100,
  41,129,152,146,214,24,21,221,120,181,54,197,127,58,190,131,78,39,215,253,151,33,6,55,68,177,29,99,228,16,247,78,232,92,64,232,252,140,215,37,
  248,151,68,47,18,127,37,209,255,244,109,38,33,137,152,166,174,6,164,126,86,65,209,46,75,220,152,114,38,223,211,207,223,244,11,142,232,17,15,237,
  161,36,108,174,85,18,136,144,238,127,164,100,127,115,49,29,60,121,182,122,208,222,28,162,30,52,223,31,39,187,105,22,234,211,234,161,127,68,180,159,
  46,148,132,128,84,159,94,46,13,61,92,150,100,116,184,179,161,143,127,242,31,244,4,100,89,150,105,110,110,226,205,215,95,39,20,14,51,119,94,5,
  138,162,112,96,255,62,213,16,20,133,242,89,179,81,20,133,35,135,15,105,54,37,49,117,218,52,114,115,115,217,182,237,93,45,154,170,106,189,184,168,
  152,113,227,198,99,54,155,9,135,195,52,53,54,114,250,244,105,36,9,166,79,159,142,211,229,74,146,197,227,245,114,252,216,177,36,99,76,90,174,143,
  207,150,18,140,82,175,115,155,144,48,46,73,201,70,249,145,143,124,148,223,255,254,119,250,223,43,87,174,34,47,47,143,13,27,214,51,123,246,28,22,
  46,90,164,125,39,72,6,3,0,255,243,171,255,230,134,117,235,232,236,236,228,189,93,187,116,57,43,43,151,144,159,159,207,250,245,47,235,175,221,113,
  231,93,148,150,150,14,184,191,47,254,229,207,250,251,137,232,232,232,96,253,250,151,241,184,221,148,150,150,178,184,114,137,254,89,171,213,202,138,149,43,
  153,56,113,18,86,171,21,143,199,205,206,157,59,211,39,255,11,8,8,8,92,82,225,80,41,201,217,127,249,43,95,101,234,244,233,40,178,76,44,22,
  211,206,134,143,241,211,31,255,24,69,145,185,254,134,117,60,254,243,199,244,188,195,137,147,38,178,122,245,26,126,251,196,111,144,144,248,231,47,255,11,
  47,189,244,119,14,29,60,168,7,30,62,249,143,255,192,255,252,234,191,145,20,248,208,71,238,87,191,87,251,110,69,81,56,124,232,16,71,142,28,78,
  77,132,18,119,108,167,244,55,9,5,240,147,42,187,72,41,150,128,83,228,40,14,18,4,25,46,254,252,215,191,165,124,253,174,219,111,99,246,236,57,
  124,235,63,255,51,233,245,83,167,78,241,253,239,252,39,109,109,109,186,30,166,151,151,243,217,127,250,60,95,250,167,207,225,243,251,117,95,255,203,95,
  255,154,39,159,120,130,109,219,222,213,150,233,37,126,244,147,159,240,165,47,124,30,144,152,191,96,1,171,86,175,198,233,116,226,243,121,217,177,125,59,
  91,183,108,1,96,113,229,18,230,47,92,72,52,26,37,22,137,224,15,4,216,185,99,7,213,39,171,80,20,9,139,217,76,197,252,249,140,25,51,6,
  179,217,76,111,111,47,71,14,31,166,177,177,81,247,131,87,173,92,73,89,89,25,38,147,137,246,182,54,182,109,123,151,206,142,78,36,73,226,145,47,
  127,25,147,217,140,201,100,210,31,155,223,122,139,77,111,190,65,234,218,175,195,212,131,136,138,158,95,50,26,239,104,255,251,235,255,33,62,221,92,188,
  100,9,149,75,150,168,70,149,120,226,129,102,140,6,73,66,142,119,121,73,157,243,140,41,45,197,235,241,144,145,225,208,138,5,131,205,102,103,250,244,
  25,236,217,189,27,143,215,131,213,98,101,70,121,57,145,72,132,214,182,86,76,102,51,39,171,170,112,107,199,183,197,97,48,36,22,176,77,14,215,15,
  24,8,146,72,168,148,188,60,159,152,59,154,96,88,43,87,174,36,55,47,151,13,27,214,171,101,39,36,216,179,103,55,123,118,239,30,96,168,111,109,
  218,196,61,239,187,151,19,199,143,227,241,120,112,101,102,50,163,124,6,207,252,233,79,41,73,103,127,98,25,127,173,177,177,49,233,181,21,43,87,82,
  89,89,201,27,175,191,62,64,37,55,222,116,51,30,143,155,103,158,249,147,78,86,175,185,246,90,60,110,183,222,81,5,4,4,4,46,117,66,26,95,
  171,253,254,247,190,11,138,194,180,25,51,184,245,214,219,248,225,15,190,175,5,49,21,166,76,153,170,22,155,215,73,172,186,83,222,96,52,106,171,87,
  10,38,147,137,235,174,187,158,147,85,39,9,6,213,122,214,22,139,5,131,22,29,179,90,173,60,241,191,255,75,255,229,97,245,176,149,190,65,95,74,
  140,144,246,247,57,105,253,15,73,203,246,137,132,52,185,28,230,153,15,163,30,73,46,227,93,183,223,6,192,125,31,248,0,0,207,252,241,143,73,239,
  31,62,116,136,127,251,215,175,234,127,127,236,19,159,224,190,15,124,128,159,63,250,168,254,218,170,85,171,217,179,123,55,11,22,46,98,203,150,183,19,
  238,143,196,251,238,187,143,253,251,246,169,126,93,2,131,209,8,72,228,230,102,115,235,109,183,241,235,95,253,138,230,230,38,50,51,179,184,235,158,123,
  232,237,237,229,224,129,3,24,141,70,246,237,221,195,142,237,219,65,81,112,56,157,220,126,199,157,180,181,181,226,245,120,184,250,154,107,240,120,60,108,
  216,176,158,72,56,66,97,81,33,107,215,94,205,230,183,222,162,181,181,149,165,203,150,33,203,49,158,122,242,183,72,72,204,171,168,224,182,219,239,224,
  201,223,62,161,18,36,179,153,239,127,247,59,3,244,146,60,161,56,11,61,8,244,113,178,11,53,64,156,174,171,215,35,150,122,109,207,196,222,170,39,
  116,171,175,141,25,51,134,238,174,46,26,27,27,153,60,121,146,110,20,22,139,25,5,8,133,67,72,146,68,56,28,230,248,177,99,120,60,110,12,218,
  89,181,6,163,65,221,156,100,144,48,104,15,73,82,95,51,232,27,151,12,3,55,49,245,35,162,169,102,166,82,146,200,234,223,43,86,174,34,55,55,
  143,141,235,215,235,245,207,164,65,102,76,161,80,136,247,118,237,226,234,107,174,5,224,154,107,174,101,231,206,157,132,66,161,179,186,205,141,141,141,184,
  92,153,3,94,207,207,207,199,106,181,242,198,235,175,107,199,162,169,159,125,227,245,215,177,104,199,235,9,8,8,8,92,242,92,148,116,7,236,12,28,
  187,13,6,67,66,238,159,234,27,140,26,41,146,36,9,163,201,196,158,61,187,185,238,186,235,18,252,143,69,247,37,86,171,85,247,37,241,147,148,18,
  31,134,184,191,137,191,159,202,231,72,210,32,129,16,250,21,196,151,82,70,215,206,117,84,116,36,56,116,240,16,133,133,69,250,223,14,167,147,9,19,
  38,240,127,191,255,29,215,175,91,151,112,134,140,218,214,77,111,190,193,251,238,187,87,191,255,6,131,234,251,237,246,12,194,225,48,61,189,61,168,133,
  235,123,249,243,11,207,211,112,250,180,126,28,171,209,104,234,43,141,229,243,225,241,184,201,202,202,162,108,226,36,156,78,39,219,183,111,211,139,229,183,
  183,183,179,99,199,14,174,90,177,2,73,146,112,56,157,184,123,221,186,30,14,28,216,207,235,175,191,166,71,168,77,38,211,224,122,32,5,33,189,136,
  244,32,34,163,41,58,62,64,73,201,24,226,139,38,149,149,75,168,171,171,213,140,17,38,77,158,66,97,81,145,190,140,159,145,145,65,77,117,181,62,
  75,45,43,155,200,145,35,71,232,233,233,102,246,156,57,28,62,172,46,125,120,189,94,58,59,58,88,181,122,53,93,157,93,116,117,117,210,214,214,70,
  32,16,64,50,24,48,153,205,76,159,62,131,104,52,170,203,211,210,210,76,75,75,75,95,186,119,194,145,95,169,9,104,242,0,144,122,70,164,26,239,
  138,149,43,153,58,117,42,207,63,247,44,225,72,36,233,62,76,159,54,157,210,49,165,250,53,157,93,157,188,251,206,59,0,28,63,126,140,153,179,102,
  233,75,237,35,89,46,207,204,116,81,89,185,68,255,123,238,188,121,188,241,198,192,168,232,164,73,147,57,85,83,147,146,188,10,160,219,39,128,197,106,
  161,177,177,145,83,53,53,184,50,51,185,230,154,107,121,241,47,127,102,70,121,57,141,141,141,120,220,110,42,43,151,224,246,184,57,118,244,104,210,235,
  113,168,17,103,143,254,125,187,118,238,100,222,188,10,118,238,220,161,255,214,206,157,59,244,127,65,93,62,90,92,89,73,56,20,198,149,233,98,231,206,
  157,148,207,40,79,146,241,232,177,163,73,191,3,240,209,251,239,231,153,63,253,137,80,40,164,127,223,140,242,114,50,181,73,73,252,247,243,243,243,1,
  112,101,102,146,233,202,196,98,181,224,113,123,216,191,127,31,119,220,121,23,77,154,45,196,219,117,169,33,63,63,159,25,229,229,250,253,139,175,14,36,
  234,42,126,127,18,245,154,168,143,56,74,75,75,153,81,94,142,199,237,193,98,181,208,209,209,193,177,163,71,7,220,167,83,53,53,172,88,185,82,255,
  220,177,163,71,233,232,232,208,191,199,149,153,73,249,140,114,198,148,150,210,212,216,136,219,227,214,245,162,190,239,98,255,190,125,9,231,173,55,14,144,
  237,76,250,117,101,186,216,186,101,75,146,125,245,183,65,32,233,189,116,246,150,169,5,11,74,75,199,18,10,133,216,191,127,95,82,223,56,147,45,94,
  108,145,81,18,198,235,120,154,84,31,33,2,131,100,208,247,118,75,146,90,164,60,78,138,20,69,194,100,54,179,117,203,22,30,250,204,195,76,40,43,
  163,190,182,14,179,197,162,251,55,171,213,202,93,119,223,147,84,19,243,157,173,91,233,76,176,129,164,146,132,82,178,127,76,23,0,73,187,26,39,245,
  219,180,52,74,81,209,161,160,160,176,80,143,154,2,220,114,235,109,201,81,209,213,171,217,186,229,109,252,126,63,29,29,29,140,159,48,129,250,186,58,
  117,19,178,193,192,75,127,255,59,223,254,206,119,153,54,109,26,39,78,156,80,39,6,72,52,55,55,113,244,232,17,190,252,149,175,114,178,170,138,154,
  234,106,142,28,62,76,119,79,23,18,42,25,205,206,204,100,252,248,9,40,138,66,126,126,62,99,198,148,210,213,245,10,115,231,206,163,177,169,17,131,
  148,28,115,59,93,95,207,173,183,221,134,100,144,216,187,103,55,183,220,122,27,243,42,42,104,106,108,164,161,177,129,147,85,85,122,170,132,217,108,230,
  99,159,248,4,6,163,17,131,193,128,209,104,228,165,191,253,141,166,166,198,126,245,156,6,39,157,231,75,15,87,28,25,77,204,193,28,114,8,214,96,
  96,225,162,133,106,177,122,20,78,158,172,226,240,161,67,122,39,108,107,107,165,170,234,164,126,116,216,196,178,137,250,236,53,195,225,164,168,168,136,222,
  222,94,74,198,140,65,142,197,24,63,126,2,245,245,117,72,72,28,63,126,140,234,234,147,228,228,228,146,147,147,195,242,229,87,113,232,208,65,218,218,
  218,48,153,140,180,181,181,226,241,120,117,89,130,193,96,82,199,79,28,28,146,7,131,161,19,209,248,103,115,243,242,216,185,115,7,215,92,119,61,127,
  125,241,47,125,179,115,9,154,155,155,56,113,226,132,254,217,144,126,106,132,138,55,222,120,157,143,126,244,126,158,126,250,169,179,82,170,197,106,97,198,
  140,114,222,120,227,245,148,164,51,21,241,90,92,89,169,59,192,196,165,254,243,129,145,216,19,128,211,233,28,53,153,226,78,61,20,10,233,121,186,243,
  230,205,227,216,177,163,76,156,52,137,198,198,70,74,75,75,57,166,57,224,56,25,200,116,101,114,204,157,76,222,60,110,143,238,252,75,75,75,117,34,
  56,232,239,187,92,132,67,225,164,235,82,145,136,254,168,169,169,97,198,140,114,157,56,88,173,86,50,93,153,73,36,119,222,188,10,253,44,239,252,252,
  124,246,239,223,175,167,106,0,52,53,54,158,241,119,46,118,157,151,150,142,229,84,77,141,78,232,92,153,153,132,67,33,44,86,11,165,153,165,73,4,
  123,210,196,73,28,59,118,148,252,252,124,66,225,48,110,207,64,82,117,236,232,81,125,178,182,98,229,202,148,247,105,197,202,149,186,205,128,122,22,119,
  34,25,245,184,221,3,38,29,253,39,32,137,186,233,111,115,167,106,106,134,164,95,87,138,123,155,104,131,195,177,183,56,66,225,16,174,204,204,36,194,
  57,92,27,185,16,253,92,146,164,126,37,131,18,163,162,201,165,132,36,67,124,172,87,80,20,9,131,100,192,104,52,232,167,20,153,76,106,4,238,229,
  151,94,226,142,59,238,228,231,143,62,138,217,108,214,125,133,197,98,209,242,254,227,203,244,138,90,196,222,144,138,156,164,38,163,169,55,215,50,200,114,
  112,50,177,25,73,52,238,92,238,230,118,56,28,172,189,250,26,126,254,232,163,236,220,177,93,127,253,182,219,239,224,157,173,91,185,235,238,187,145,36,
  137,235,215,221,200,255,254,234,191,147,244,241,244,83,79,114,255,131,15,242,175,95,249,10,70,163,81,159,4,188,244,183,191,241,250,171,175,50,113,210,
  36,38,148,77,228,161,135,31,230,207,207,63,207,209,35,71,48,24,141,76,154,52,137,236,236,108,100,89,166,167,167,155,23,255,242,103,34,225,8,38,
  147,73,77,244,75,113,255,45,86,43,146,36,209,217,209,201,83,79,254,150,194,194,34,38,76,152,192,146,37,75,89,187,246,106,254,247,215,191,70,146,
  192,100,50,241,214,166,77,73,209,235,238,238,174,139,94,15,23,18,35,237,231,195,58,14,180,166,186,26,128,169,83,167,242,246,91,111,13,235,135,140,
  70,35,47,255,253,239,73,59,233,0,125,214,234,243,250,104,109,105,214,95,43,44,44,212,149,51,113,210,68,58,58,59,212,165,20,20,186,186,186,40,
  43,43,227,116,125,61,227,198,143,195,100,50,83,83,125,146,246,246,54,218,219,218,232,233,233,102,226,196,73,180,183,181,99,52,154,240,251,252,244,246,
  244,164,94,166,209,73,40,3,150,63,250,151,202,136,239,147,31,112,196,151,162,237,115,52,24,216,184,94,205,77,201,207,207,103,233,210,101,108,223,246,
  174,110,136,30,143,135,166,198,198,212,70,39,73,120,122,123,213,207,105,255,166,69,138,99,212,80,192,237,246,232,3,64,56,20,162,124,70,57,167,52,
  157,37,94,211,222,222,78,229,146,74,253,179,59,119,108,103,231,142,237,148,150,142,101,241,146,37,231,253,56,178,169,83,167,234,246,53,84,123,172,169,
  174,230,246,59,238,24,53,89,107,170,171,89,177,98,37,86,171,85,117,252,138,66,56,20,226,216,145,35,84,46,89,194,169,234,106,242,231,206,83,237,
  172,189,157,130,130,124,172,22,11,22,139,101,128,76,46,151,139,74,141,236,91,44,86,182,110,121,91,117,246,74,194,158,82,69,73,248,23,58,218,219,
  41,45,45,229,166,155,110,38,20,10,177,115,231,14,74,75,21,253,123,84,189,13,36,3,225,80,72,191,22,20,242,243,242,105,108,104,208,191,55,20,
  12,146,120,12,221,177,163,71,153,55,119,30,153,153,153,52,54,54,208,216,208,192,152,210,82,253,119,26,181,83,208,46,53,157,31,59,122,132,121,21,
  21,204,155,87,65,40,20,226,141,215,95,99,70,121,57,251,247,237,163,180,180,20,151,203,133,219,221,75,126,126,62,249,249,249,188,241,250,107,172,88,
  185,10,143,199,77,77,77,205,128,190,53,163,188,156,137,147,38,97,181,244,217,67,255,251,20,14,133,180,251,171,66,239,123,169,58,112,130,238,43,43,
  43,113,185,50,9,133,66,236,218,185,131,252,252,2,189,175,246,183,185,193,244,91,58,118,172,158,59,158,252,27,201,54,232,118,123,212,35,51,19,144,
  202,222,250,162,182,138,46,131,106,115,241,213,173,193,109,241,162,233,231,138,50,96,188,148,18,203,12,42,234,78,120,135,195,73,226,254,110,123,70,6,
  6,131,81,247,5,38,163,17,9,168,171,171,229,244,233,211,172,89,187,86,35,163,128,162,46,217,55,53,54,12,216,72,47,145,250,80,148,164,243,231,
  211,145,27,82,23,76,143,159,250,41,37,249,160,228,72,176,212,47,42,156,242,24,234,97,223,203,20,190,7,133,246,182,86,158,249,191,255,211,125,249,
  213,215,92,205,206,237,219,0,152,61,103,14,193,96,16,191,223,135,4,212,215,213,177,98,229,74,50,50,50,240,251,252,122,148,186,234,216,49,170,78,
  84,113,203,173,183,106,145,81,88,178,100,9,54,187,141,205,155,54,113,228,240,97,142,28,58,68,109,77,13,203,87,172,224,232,209,35,24,141,70,14,
  31,62,204,230,77,155,146,38,1,146,4,77,77,141,172,189,250,26,118,110,223,158,212,132,41,83,167,210,219,211,141,65,146,248,192,135,62,204,95,94,
  120,158,142,246,54,58,218,219,217,189,251,61,254,223,167,62,205,196,137,19,169,171,61,133,209,100,162,246,84,77,202,192,211,5,213,195,69,140,145,244,
  115,24,102,206,168,215,235,197,231,243,49,117,218,180,97,11,104,140,47,139,12,200,111,145,6,118,186,132,93,134,18,48,99,250,12,118,237,220,201,209,
  35,135,57,122,228,40,7,15,236,199,106,181,146,157,147,77,103,103,39,229,229,229,148,140,25,131,132,154,67,58,166,100,12,126,159,79,205,249,48,26,
  181,252,156,228,135,33,177,168,253,128,93,244,36,189,151,180,193,42,149,65,210,103,144,17,109,179,210,246,109,219,40,41,41,97,218,244,233,73,157,56,
  221,220,71,34,85,39,103,8,206,44,245,107,59,119,236,192,229,114,49,113,210,164,1,239,159,170,169,38,20,12,113,205,181,215,226,202,204,212,151,16,
  103,148,207,184,32,29,100,234,180,105,248,124,62,188,94,239,144,175,105,109,109,213,175,29,141,37,222,176,70,96,214,191,252,18,19,39,78,212,9,76,
  101,229,18,242,243,242,181,165,84,133,210,210,49,156,170,169,166,166,186,154,197,149,149,73,81,173,164,136,216,142,29,236,220,177,131,173,90,226,190,197,
  98,233,155,165,199,159,39,220,250,25,229,229,28,59,122,148,245,47,191,196,214,45,111,171,75,244,10,250,247,164,117,254,10,52,54,54,144,159,151,143,
  197,98,161,163,163,61,41,202,101,181,90,19,202,206,40,148,142,41,101,235,150,183,89,255,242,75,122,68,173,169,161,65,255,141,209,32,162,231,67,231,
  51,102,148,179,115,199,14,214,191,252,18,53,53,213,148,150,150,82,90,90,74,249,12,117,73,187,124,134,154,78,49,99,198,12,58,218,219,245,155,103,
  181,88,83,44,55,43,28,59,122,132,173,111,191,77,40,20,212,223,31,112,159,20,244,37,118,53,170,89,49,184,67,215,158,247,217,133,162,69,85,53,
  221,164,178,185,65,244,219,168,201,83,58,166,116,64,189,152,68,27,236,79,68,211,218,155,94,176,89,253,204,81,45,13,37,254,242,25,109,241,34,233,
  231,82,138,122,207,82,191,67,74,170,142,31,39,191,160,128,229,203,175,2,32,55,47,159,155,110,190,153,35,135,15,233,227,123,60,50,42,33,241,234,
  198,13,44,95,177,2,179,217,172,167,104,197,151,236,13,131,213,170,78,83,87,84,66,74,157,250,69,154,224,136,196,32,84,245,194,110,142,121,230,143,
  255,71,65,97,33,149,75,151,2,234,146,253,51,127,250,35,127,121,225,5,254,252,194,11,188,248,194,243,236,217,189,155,21,171,86,171,155,149,250,24,
  57,207,61,243,39,174,185,246,58,157,140,158,60,121,146,27,111,190,133,57,115,231,33,161,230,144,206,158,59,135,182,214,86,109,153,62,30,189,102,192,
  233,85,141,13,13,196,98,49,110,185,237,54,236,118,27,146,36,49,161,172,140,219,238,184,131,183,54,189,165,31,191,122,195,141,55,146,157,147,163,166,
  11,78,154,132,211,229,162,183,183,7,180,156,209,1,126,191,159,30,164,139,84,15,23,140,140,142,160,159,195,8,150,233,15,28,56,192,130,5,11,70,
  60,32,196,21,164,36,79,19,244,28,157,248,178,137,100,80,207,241,154,52,121,10,29,237,237,68,34,145,190,37,20,36,78,215,215,51,109,234,52,118,
  189,183,139,125,123,247,82,94,62,147,37,75,150,18,14,135,105,110,110,162,234,72,149,154,52,110,52,114,213,138,149,3,228,249,251,223,255,150,108,50,
  82,138,147,148,82,69,79,83,18,81,237,185,150,0,15,16,14,135,217,178,121,51,215,221,112,3,93,157,157,32,193,162,69,139,89,188,184,50,233,250,
  191,253,245,175,106,254,201,40,224,141,215,95,227,166,155,111,214,243,218,18,177,97,253,203,204,155,87,193,157,119,222,137,203,149,169,71,132,182,110,121,
  249,188,27,239,130,5,11,56,112,224,192,176,174,57,168,125,126,193,130,5,84,105,169,15,231,10,29,29,29,84,86,46,161,180,116,172,30,245,42,45,
  29,171,231,29,186,50,51,147,150,234,227,215,148,150,142,101,215,206,157,67,254,141,248,114,111,124,249,87,141,180,169,249,120,53,53,213,122,14,159,197,
  106,97,255,254,253,148,207,40,79,202,215,27,44,79,239,216,177,163,220,247,254,247,179,117,203,22,220,30,119,82,14,108,98,206,104,40,28,210,223,139,
  231,20,38,202,49,90,57,163,163,173,243,198,198,6,86,172,92,153,116,255,26,27,27,245,182,84,86,46,193,227,118,147,159,95,192,254,253,251,117,157,
  100,166,216,240,151,136,196,124,223,254,247,105,255,254,125,73,121,151,251,247,237,27,114,219,66,161,16,167,106,106,152,56,105,18,97,109,153,63,149,205,
  13,69,191,199,142,30,101,210,164,201,73,178,185,250,229,147,247,95,98,63,85,83,51,192,222,50,251,45,183,121,220,110,172,19,251,100,24,170,45,94,
  52,253,92,63,84,37,177,126,116,223,145,126,63,253,225,15,248,232,131,31,227,35,15,60,128,207,235,101,219,187,239,170,43,128,218,117,70,147,73,247,
  65,193,80,136,87,54,110,224,174,187,239,209,151,241,205,102,51,255,254,173,255,72,202,25,173,171,173,229,119,79,61,149,38,50,58,52,127,147,214,7,
  37,176,29,169,255,23,159,105,83,83,10,191,124,46,241,139,159,61,202,151,255,245,95,105,111,109,99,214,156,57,124,239,59,255,153,36,219,214,183,55,
  243,185,47,124,145,87,55,110,72,72,69,128,128,63,192,243,207,62,195,71,31,120,16,9,137,174,174,78,254,239,247,191,231,198,155,111,230,99,159,252,
  36,62,159,143,3,251,247,241,230,235,175,171,121,190,6,35,138,162,36,180,163,239,56,80,5,248,203,11,47,112,221,13,55,240,240,63,125,30,155,205,
  70,107,75,11,27,94,126,137,211,167,235,145,36,137,215,95,123,149,117,55,221,196,167,31,126,152,140,12,7,173,45,45,108,92,191,158,222,158,30,61,
  26,254,147,71,31,195,96,48,232,143,234,147,39,121,236,167,63,77,158,16,92,164,122,184,16,24,73,63,7,144,110,90,183,110,88,161,176,107,175,187,
  142,207,127,225,11,252,203,35,143,240,246,230,205,195,92,45,81,250,194,182,41,150,235,149,193,158,107,255,83,18,162,135,131,190,158,42,10,145,98,186,
  34,165,26,20,82,45,159,164,29,12,82,231,136,72,87,168,33,14,23,171,86,175,230,123,63,252,33,63,253,201,79,120,253,181,215,134,117,237,19,79,
  62,137,187,183,151,251,63,252,97,4,132,206,5,132,206,211,65,150,229,36,63,148,232,31,116,111,147,228,75,18,95,63,131,63,74,229,123,6,57,249,
  103,64,236,44,49,186,150,234,121,98,68,78,34,133,239,145,6,250,180,51,248,162,11,229,123,206,137,30,250,189,223,255,249,249,210,131,116,6,34,122,
  49,235,225,98,236,231,195,46,237,180,237,221,119,241,249,124,73,59,231,70,18,29,77,85,252,85,74,251,60,161,220,83,252,90,41,249,72,206,254,203,
  31,201,75,34,137,7,89,164,255,44,82,242,114,125,242,239,161,127,38,37,17,101,240,98,182,98,247,92,106,220,247,129,15,224,243,249,216,246,238,187,
  35,136,252,190,206,212,105,211,88,176,112,161,184,145,66,231,2,66,231,67,242,61,253,107,148,75,201,12,175,111,249,53,105,185,188,223,243,68,191,195,
  96,71,72,39,248,157,116,239,235,190,39,245,115,93,126,41,225,48,106,41,213,146,253,197,239,103,206,137,30,164,244,207,207,167,30,134,67,68,69,63,
  31,5,50,26,255,161,249,11,22,140,104,112,72,50,70,105,96,193,131,212,132,180,159,65,14,66,74,7,14,16,164,48,202,190,207,232,215,36,206,126,
  244,239,236,255,156,244,68,84,74,63,203,17,68,52,53,22,44,92,200,252,5,11,244,9,206,112,17,159,121,125,252,147,159,20,55,83,232,92,64,232,
  124,200,129,144,33,17,161,148,193,144,84,4,104,40,190,135,190,253,9,73,126,103,16,242,147,80,3,53,149,175,73,36,73,233,252,204,197,22,141,59,
  59,61,112,6,61,72,41,244,192,121,210,3,151,148,30,46,198,126,62,162,162,247,255,247,135,63,224,243,249,248,220,231,63,127,118,198,56,192,56,83,
  71,69,7,12,12,233,72,169,212,223,0,57,195,44,137,132,227,215,250,190,175,239,172,224,129,179,208,148,209,81,105,228,134,117,37,47,209,127,238,243,
  159,199,231,243,241,127,127,248,195,136,174,111,109,109,229,175,47,190,168,31,23,39,32,116,46,32,116,126,78,136,80,218,96,200,64,50,148,232,167,134,
  236,123,18,125,201,96,228,39,141,175,57,91,2,116,177,68,71,135,167,7,105,4,122,144,132,30,46,145,126,110,24,233,224,240,135,223,255,158,169,211,
  166,141,104,182,154,50,114,152,252,129,228,205,130,113,146,152,52,48,244,39,165,3,35,164,67,121,164,50,200,184,145,37,47,141,164,50,66,233,140,196,
  82,68,69,83,227,227,159,252,36,83,167,77,227,15,191,255,189,190,99,118,36,136,95,255,181,175,127,125,196,245,205,4,132,206,5,46,127,157,75,131,
  165,134,37,16,161,65,131,33,253,201,80,154,200,220,153,253,142,52,32,24,146,142,252,164,14,136,156,29,1,186,144,65,16,161,135,203,47,24,117,46,
  250,249,136,143,3,253,235,139,47,114,240,192,1,62,254,201,79,114,211,45,183,156,189,49,166,52,208,100,3,76,14,219,147,68,66,19,167,67,73,57,
  163,233,254,147,164,126,70,219,103,96,3,72,232,25,140,112,164,68,244,74,141,138,222,116,203,45,124,252,147,159,228,224,129,3,252,245,197,23,207,234,
  187,124,62,31,63,253,201,79,112,186,92,60,254,203,95,10,114,34,116,46,32,116,126,86,68,40,117,48,36,21,25,74,142,224,13,201,247,164,240,59,
  201,193,143,68,63,211,63,0,194,192,149,184,75,52,18,55,114,61,144,154,148,158,3,61,72,105,245,32,93,182,122,184,152,250,185,113,234,148,41,223,
  24,233,197,219,182,109,99,225,162,69,220,126,199,29,52,55,55,159,117,233,13,105,152,3,195,128,153,81,202,247,82,63,36,146,141,111,192,172,40,145,
  132,246,143,134,14,200,44,16,68,116,56,134,251,181,175,127,157,154,154,26,190,254,181,175,105,37,187,206,14,109,173,173,180,181,181,113,243,45,183,176,
  116,217,50,94,127,237,53,194,253,78,183,18,16,58,23,16,58,151,82,21,27,239,239,119,82,6,67,82,191,150,210,239,12,230,123,72,231,123,72,65,
  126,82,71,229,206,150,0,93,12,190,103,228,122,144,210,147,210,179,212,3,105,245,192,101,171,135,139,169,159,159,21,25,141,68,34,188,189,121,179,78,
  72,1,246,238,217,51,114,99,28,230,192,64,130,225,13,58,59,74,241,32,165,241,245,27,8,6,24,225,208,8,231,149,98,136,195,197,199,63,249,73,
  254,233,11,95,160,166,166,134,127,249,231,127,30,81,146,115,58,212,212,212,36,57,170,195,135,14,169,245,93,5,132,206,5,132,206,135,66,132,210,6,
  67,206,16,16,25,134,223,25,154,239,73,79,136,146,2,32,151,56,1,146,82,11,152,34,106,74,242,126,145,36,82,42,244,112,185,244,243,179,34,163,
  137,132,116,198,140,25,172,187,233,38,22,44,92,72,213,137,19,67,26,32,134,71,72,7,33,165,41,12,83,26,66,71,24,140,128,14,102,132,164,49,
  66,65,68,83,99,234,180,105,252,199,127,254,39,55,221,114,11,7,15,28,224,235,95,251,218,57,117,80,253,29,213,154,181,107,185,233,230,155,177,88,
  173,67,158,28,9,8,157,11,92,89,58,79,75,134,6,241,61,164,34,61,12,221,247,244,247,59,233,125,79,26,191,115,22,1,144,139,206,247,156,97,
  229,48,149,30,164,97,232,1,161,135,75,170,159,15,187,232,253,96,248,208,135,63,204,7,63,244,33,0,94,126,233,37,158,249,211,159,134,180,116,175,
  104,133,112,83,10,146,170,144,125,210,11,74,250,107,71,60,48,165,49,190,65,58,209,149,96,132,35,49,218,251,222,255,126,110,214,114,138,255,239,15,
  127,224,15,191,255,253,168,255,238,164,73,147,248,252,23,191,200,164,73,147,104,110,110,230,55,191,254,53,155,55,111,198,235,241,8,198,32,116,46,32,
  116,174,185,22,101,224,225,43,67,246,61,202,64,87,52,218,190,39,93,105,160,75,220,247,8,61,136,126,126,206,201,40,64,81,81,17,31,252,208,135,
  184,246,186,235,0,104,110,110,230,237,205,155,169,58,113,130,230,230,230,180,100,84,78,36,164,3,78,179,80,250,140,150,84,39,44,41,169,79,95,24,
  178,1,74,41,143,246,74,52,176,254,134,56,176,214,104,138,191,251,25,167,148,106,9,226,50,67,73,73,9,83,167,77,99,213,234,213,148,148,148,0,
  106,157,192,255,251,195,31,206,106,55,237,72,112,237,117,215,241,193,15,125,136,162,162,34,0,221,14,247,136,200,153,208,185,128,208,185,162,32,203,178,
  122,50,96,194,107,74,42,34,148,138,44,41,41,78,12,58,27,223,147,162,32,124,223,81,153,169,235,137,74,233,252,78,191,107,12,105,86,242,46,18,
  54,58,36,61,40,41,95,79,240,248,231,68,15,82,202,19,26,175,8,61,92,224,126,46,221,116,195,13,202,104,52,162,168,168,136,165,203,151,179,108,
  217,50,230,204,157,123,198,207,135,67,33,205,32,251,204,41,241,140,223,193,254,78,36,170,253,137,235,144,102,39,9,167,43,72,131,60,31,240,119,220,
  140,37,146,222,27,112,126,189,246,121,163,209,120,197,56,171,131,7,14,176,109,219,54,182,191,251,238,121,119,78,253,177,108,249,114,150,106,118,24,119,
  88,2,66,231,2,66,231,138,44,19,10,133,116,215,161,144,218,223,164,243,57,253,253,206,176,124,79,10,63,49,168,191,73,124,45,225,200,234,164,32,
  71,138,239,50,26,141,23,125,16,68,142,197,244,205,104,23,131,30,206,196,3,46,87,61,92,200,126,62,106,100,180,63,38,77,158,140,195,225,24,244,
  51,177,88,172,223,185,243,169,34,161,131,156,7,220,207,24,135,118,7,210,212,60,29,194,153,243,169,34,160,233,118,214,75,6,3,87,2,124,62,31,
  53,213,213,23,173,124,14,167,147,73,147,38,9,22,33,116,46,32,116,174,19,210,164,51,211,251,251,153,254,190,38,237,178,241,57,240,61,103,90,109,
  27,176,201,42,181,159,74,124,237,82,33,64,3,86,72,211,232,97,80,255,47,244,112,201,246,243,243,70,70,135,60,67,74,24,20,56,147,209,165,49,
  192,225,47,211,167,54,202,193,72,234,160,51,173,33,190,46,32,32,32,32,112,17,16,161,196,232,90,58,242,115,1,252,78,58,130,115,185,250,157,148,
  122,16,254,255,138,128,233,98,19,200,96,48,12,36,164,241,89,199,96,70,153,34,223,230,156,176,245,161,12,22,162,158,168,128,128,128,192,37,139,248,
  56,173,164,171,125,153,206,247,140,166,223,57,11,223,115,169,250,157,148,122,72,140,58,166,155,28,8,61,8,50,58,90,132,52,229,12,169,159,81,14,
  119,22,116,206,103,172,162,132,147,128,128,128,192,101,67,72,37,73,74,189,58,215,47,32,50,170,190,231,44,119,100,95,234,126,231,76,122,72,244,255,
  66,15,130,140,94,184,153,234,32,196,116,184,198,57,216,174,55,65,66,5,4,4,4,174,60,164,92,157,75,225,35,132,239,17,122,16,28,224,10,32,
  163,103,156,33,13,98,64,210,40,202,115,46,62,35,32,32,32,32,112,113,19,161,180,121,164,194,247,8,61,8,14,112,101,145,209,254,6,169,78,128,
  206,255,126,43,97,128,2,2,2,2,87,22,226,193,144,33,145,161,11,228,119,174,4,223,35,244,32,200,232,69,103,144,253,49,154,134,57,28,195,18,
  70,40,32,32,32,112,249,147,33,225,119,132,30,132,255,191,194,201,104,127,197,43,138,114,206,8,234,72,141,73,24,161,128,128,128,192,149,67,134,210,
  65,248,29,161,7,129,43,140,140,166,34,165,231,219,64,132,17,10,8,8,8,92,217,100,40,209,247,8,191,35,244,32,112,133,146,209,1,6,169,90,
  229,104,254,16,194,4,5,4,4,4,4,206,155,239,17,126,71,232,225,74,209,227,129,253,251,149,203,173,81,231,178,65,194,0,5,4,4,4,4,206,
  167,223,17,190,71,232,225,74,131,41,22,139,137,187,32,48,42,51,201,43,162,153,66,211,151,133,109,9,61,94,122,253,90,186,148,199,130,11,124,31,
  47,103,123,151,206,227,253,149,68,127,61,103,247,76,90,185,124,185,194,21,142,75,57,7,228,162,149,253,2,202,37,93,134,247,127,212,127,231,18,33,
  120,163,117,31,70,229,123,207,209,119,158,75,217,164,203,164,77,103,125,253,5,190,15,151,100,251,47,128,204,231,75,206,139,125,19,147,116,30,219,52,
  154,242,13,246,57,131,152,234,11,8,8,8,8,8,8,8,8,92,40,8,50,42,32,32,32,32,32,32,32,32,32,200,168,128,128,128,128,128,128,128,
  128,128,32,163,2,2,2,2,2,2,2,2,2,2,231,13,23,180,206,232,204,89,179,82,190,126,228,240,97,161,153,139,68,31,181,181,181,248,125,62,
  113,115,4,4,4,4,4,4,4,46,125,50,186,122,237,90,22,47,174,100,230,172,89,56,28,142,51,126,254,200,225,195,212,214,158,98,253,203,47,211,
  222,214,134,195,225,224,251,63,250,17,5,5,133,3,62,187,249,173,77,252,215,47,126,161,255,253,139,95,254,50,229,231,118,237,220,201,143,126,240,125,
  102,206,154,197,191,127,243,91,3,222,247,251,124,124,235,27,255,78,109,109,173,254,218,159,158,123,62,165,124,207,63,247,44,207,63,251,44,95,255,198,
  55,211,18,235,161,224,253,239,187,7,128,123,238,189,151,123,222,119,239,25,63,55,216,239,29,57,114,152,255,248,198,55,134,244,187,5,5,5,220,253,
  190,123,153,57,107,22,5,5,5,131,235,226,200,97,222,126,235,45,118,237,218,37,200,169,128,128,128,128,128,128,192,57,195,121,91,166,255,254,143,126,
  204,167,31,250,12,139,43,43,7,16,209,218,218,90,142,28,62,140,175,31,201,153,57,107,22,55,221,124,11,223,255,225,143,40,40,44,196,231,243,241,
  163,239,127,63,53,209,93,179,150,130,194,66,253,186,84,68,180,189,189,141,95,62,254,139,65,229,204,112,56,248,226,63,127,121,72,100,249,82,198,234,
  53,107,120,236,241,255,98,245,154,53,73,68,116,195,250,151,121,225,185,231,6,16,206,153,51,103,241,255,62,253,16,223,251,193,15,153,80,86,38,122,
  142,128,128,128,128,128,128,192,57,193,121,137,140,150,149,149,81,150,134,192,212,214,214,242,229,47,125,81,39,145,169,162,149,14,135,131,53,107,214,242,
  220,179,207,80,91,91,203,243,207,62,203,61,247,14,140,32,198,63,179,102,237,218,148,191,245,95,191,248,197,0,194,155,10,5,5,5,124,241,145,127,
  230,91,223,248,247,33,181,239,233,39,127,75,70,2,121,93,189,102,45,171,215,172,73,251,249,161,126,239,104,226,163,247,63,48,224,181,35,71,14,243,
  244,147,79,234,127,223,253,190,247,165,188,55,247,188,239,94,126,252,195,31,136,222,35,32,32,32,32,32,32,112,105,144,209,51,17,213,247,221,123,31,
  111,189,181,137,246,246,118,158,127,246,217,148,159,59,124,248,144,254,252,185,103,159,97,81,101,229,0,130,187,122,237,26,222,122,107,19,171,215,12,36,
  163,235,95,126,105,88,185,168,51,103,205,226,254,7,30,228,169,39,127,123,198,207,38,46,233,199,175,29,12,23,67,78,108,198,32,145,223,12,135,131,
  85,131,144,105,1,1,1,1,1,1,1,129,75,138,140,214,214,214,178,57,13,73,4,45,87,242,222,123,105,111,111,227,200,225,195,236,218,185,51,229,
  178,125,34,126,244,131,239,243,253,31,254,40,105,57,189,160,160,144,79,63,244,153,148,191,255,212,111,127,59,108,185,111,188,249,230,1,68,243,66,34,
  30,13,142,167,35,156,13,62,112,239,251,210,18,209,175,255,251,55,6,205,33,125,254,185,103,69,207,17,16,16,16,16,16,16,184,116,200,40,168,75,
  228,135,15,31,102,241,226,74,22,87,86,166,252,76,65,65,33,171,215,20,234,164,117,215,206,157,236,218,181,147,205,155,54,13,248,108,123,91,27,207,
  63,251,44,247,63,248,96,210,235,169,162,146,191,252,197,207,71,44,247,253,15,60,112,241,144,209,65,54,55,157,11,76,40,43,227,83,159,126,40,109,
  78,168,223,231,227,233,167,158,164,238,34,34,232,2,2,2,2,2,2,2,130,140,14,25,155,55,109,210,137,229,226,74,117,87,253,204,89,179,211,230,
  147,46,174,84,137,235,253,15,60,200,47,31,255,5,187,118,238,76,122,127,253,203,47,49,115,214,172,180,228,22,224,169,223,254,118,216,209,205,246,246,
  118,61,50,152,113,17,109,100,138,47,239,151,149,149,157,115,185,102,206,154,197,23,191,244,72,218,239,245,251,124,124,235,155,223,16,68,84,64,64,64,
  64,64,64,224,210,36,163,239,187,247,190,20,4,199,175,231,114,206,156,53,139,53,107,214,166,140,108,58,28,14,62,245,208,103,56,114,248,83,3,150,
  238,127,249,248,47,88,92,249,116,202,223,172,173,173,101,253,203,47,13,91,214,167,127,251,91,62,245,208,67,23,21,17,133,190,141,79,103,91,74,170,
  63,110,188,233,102,62,58,72,4,248,200,145,195,252,248,135,63,20,37,157,4,4,4,4,4,4,4,46,93,50,154,106,247,59,168,209,190,205,155,54,
  177,185,173,141,205,155,54,241,253,31,253,56,101,164,212,225,112,48,161,172,108,192,230,159,193,242,74,71,74,158,124,126,31,63,250,225,15,248,250,55,
  190,121,89,43,63,195,225,224,163,247,63,48,232,206,255,23,158,123,78,228,136,10,8,8,8,8,8,8,140,26,206,91,157,209,116,75,229,5,133,5,
  250,38,164,130,194,66,28,142,140,115,78,46,71,130,35,135,15,179,225,229,151,47,91,197,79,40,43,227,235,255,254,141,180,68,212,239,243,241,31,223,
  252,134,32,162,2,2,2,2,2,2,2,163,138,243,22,25,253,229,47,126,206,151,190,252,229,1,197,232,11,10,10,121,226,169,167,207,120,253,250,151,
  95,58,239,59,219,159,127,238,89,102,206,154,117,89,22,121,255,250,191,127,227,140,229,157,254,237,223,191,145,242,189,116,59,241,5,4,4,4,4,4,
  4,4,46,90,50,90,91,91,203,103,62,245,41,22,87,86,82,86,54,145,153,179,102,145,225,112,164,92,146,247,249,124,212,213,214,210,222,222,70,237,
  169,90,118,237,218,73,123,91,91,218,239,78,87,183,179,182,246,84,218,107,252,62,95,218,235,226,17,88,159,207,199,47,31,255,5,31,125,224,193,148,
  159,107,111,107,79,251,250,72,106,137,14,245,186,1,155,136,164,65,222,75,167,143,186,90,97,253,2,2,2,2,2,2,2,23,28,210,202,229,203,149,
  43,254,38,72,146,144,253,220,11,118,225,126,250,50,188,255,163,254,59,163,240,253,210,37,116,31,70,229,123,207,209,119,158,75,217,164,203,164,77,103,
  125,253,5,190,15,151,100,251,47,128,204,231,75,206,145,222,207,139,121,252,63,159,220,96,168,191,53,216,231,206,91,206,168,128,128,128,128,128,128,128,
  128,128,128,32,163,2,2,2,2,2,2,2,2,2,130,140,10,8,8,8,8,8,8,8,8,8,8,50,42,32,32,32,32,32,32,32,32,112,193,96,
  122,240,227,31,191,226,111,130,116,201,55,64,18,150,124,161,116,122,62,147,196,133,29,141,222,125,184,200,219,46,93,134,125,252,66,183,73,186,132,237,
  229,82,180,7,73,244,177,203,86,190,115,33,155,164,40,202,21,191,155,94,64,64,64,64,64,64,64,64,224,2,17,218,27,174,187,78,144,81,1,1,
  1,1,1,1,1,1,129,11,2,145,51,42,32,32,32,32,32,32,32,32,32,200,168,128,128,128,128,128,128,128,128,128,32,163,2,2,2,2,2,2,
  2,2,2,2,130,140,10,8,8,8,8,8,8,8,8,8,50,42,32,32,32,32,32,32,32,32,32,48,106,48,33,42,59,9,8,8,8,8,8,8,
  8,8,92,32,136,200,168,128,128,128,128,128,128,128,128,128,32,163,2,2,2,2,2,2,2,2,2,130,140,10,8,8,8,8,8,8,8,8,8,8,
  50,42,32,32,32,32,32,32,32,32,112,249,195,52,154,95,62,121,242,100,150,93,117,21,115,231,206,101,242,228,201,56,156,78,113,199,47,115,248,188,
  94,170,171,171,57,112,224,0,219,222,121,135,234,234,234,11,46,147,211,233,100,217,242,229,204,157,55,143,201,147,39,51,105,242,100,161,40,1,1,209,
  207,5,4,4,46,146,126,46,221,112,237,181,231,124,59,253,117,215,95,207,135,63,250,81,138,138,138,0,168,171,237,166,174,182,155,142,54,159,246,137,
  193,127,82,26,138,224,103,211,232,75,232,251,47,53,89,115,10,157,148,150,229,80,90,150,3,64,107,107,43,191,127,250,105,94,123,245,213,243,222,145,
  138,138,138,248,240,71,63,202,117,215,95,15,64,204,237,33,112,228,40,222,29,59,211,94,115,174,59,195,104,212,170,24,237,250,23,202,5,252,142,243,
  89,219,67,25,240,107,210,136,165,60,231,114,75,163,127,63,206,230,187,237,165,165,56,103,150,227,42,47,191,248,250,121,208,67,176,245,24,222,186,157,
  23,198,131,75,92,250,144,46,194,31,17,247,245,188,203,100,206,28,131,189,96,6,246,194,25,163,222,207,207,41,25,157,59,111,30,95,124,228,17,138,
  138,138,104,111,247,177,241,165,99,188,183,235,52,237,109,190,228,246,106,229,164,164,33,222,151,33,189,38,157,195,239,74,241,218,136,200,163,50,140,239,
  63,199,242,143,230,189,24,202,253,201,45,116,50,103,241,88,86,221,82,78,110,129,131,214,214,86,126,252,195,31,114,96,255,254,243,18,33,249,240,71,
  62,194,29,119,221,5,64,215,243,127,161,231,181,55,232,125,245,245,17,145,20,101,20,73,138,50,194,129,74,57,71,159,25,202,181,231,147,88,42,231,
  80,238,225,235,90,26,145,125,112,174,101,150,206,205,125,228,60,216,136,125,108,41,5,215,93,203,248,7,30,192,62,182,244,130,246,243,238,253,47,210,
  123,252,13,220,199,223,184,112,228,225,124,144,141,115,240,27,202,5,110,131,36,73,23,92,23,138,114,241,235,78,26,101,153,148,33,202,100,201,44,37,
  107,202,213,228,47,252,8,150,204,209,233,231,231,140,140,126,248,163,31,229,195,31,249,8,126,95,152,231,159,61,200,198,151,143,13,66,210,4,25,189,
  220,201,104,34,42,215,78,230,142,7,22,98,119,88,248,253,239,126,199,239,159,126,122,212,58,215,228,201,147,249,226,35,143,48,105,242,100,188,219,119,
  82,247,200,191,16,110,104,58,11,130,34,200,168,32,163,130,140,14,197,70,198,220,125,23,51,190,246,85,76,153,153,231,181,159,251,234,118,113,250,111,
  95,37,220,211,120,225,137,159,32,163,130,140,94,134,100,52,17,185,179,238,96,204,218,127,193,104,117,157,211,126,110,156,50,105,210,55,206,246,75,190,
  248,200,35,220,121,215,93,28,57,220,202,191,125,245,85,142,30,110,29,18,89,185,236,201,232,112,190,255,50,38,163,141,181,221,188,251,90,21,19,166,
  229,179,230,218,229,20,21,23,179,237,221,119,71,197,65,253,224,71,63,34,63,195,65,195,215,190,65,195,127,124,135,152,219,115,14,70,153,243,180,182,
  114,57,44,67,93,210,144,132,24,103,1,207,209,163,156,254,227,159,200,158,95,193,146,155,110,26,253,126,158,237,160,113,253,55,105,122,245,123,196,130,
  158,139,67,23,210,101,96,31,87,8,25,189,20,116,39,93,132,50,5,58,142,209,121,224,25,50,198,204,99,209,202,27,207,89,63,63,107,50,250,197,
  71,30,225,186,235,175,103,243,91,53,252,228,7,111,19,137,196,134,76,86,4,25,189,50,200,40,64,52,18,99,215,91,53,228,21,58,89,121,205,162,
  115,238,168,226,14,202,38,43,156,252,192,71,241,108,222,114,14,71,25,65,70,5,25,21,100,116,40,144,67,33,154,94,248,51,246,177,165,84,220,122,
  203,232,245,115,147,66,205,211,247,227,169,222,122,113,233,66,144,81,65,70,47,115,50,138,4,74,44,76,247,225,23,177,100,150,50,123,217,205,231,164,
  159,159,21,25,253,199,79,125,138,155,111,185,133,205,111,213,240,223,191,216,54,108,178,114,33,201,104,78,158,157,49,99,93,228,228,217,201,205,179,147,
  147,103,7,73,34,24,136,158,51,50,154,157,103,167,68,251,141,156,126,191,113,165,145,209,56,14,237,60,77,174,70,72,29,78,39,187,223,123,239,172,
  251,82,81,81,17,63,123,236,49,157,136,6,142,28,61,199,163,140,32,163,130,140,10,50,58,28,180,189,246,186,78,72,207,121,63,215,136,104,160,245,
  216,69,233,168,207,6,150,146,10,140,174,226,164,135,100,117,34,7,186,4,25,21,100,244,162,179,113,119,245,155,58,33,61,219,126,110,26,105,206,208,
  220,121,243,184,243,174,187,120,111,103,195,0,34,58,24,126,252,139,107,211,190,247,234,250,26,94,123,121,96,233,128,201,83,115,248,199,127,90,52,44,
  249,190,252,153,215,146,254,206,201,179,179,98,205,120,38,77,205,161,100,172,107,208,107,155,27,60,156,170,234,230,221,183,234,233,238,12,12,235,119,203,
  231,22,50,127,105,9,229,115,11,7,253,92,75,131,135,99,7,218,217,187,189,137,158,97,254,198,165,142,63,254,226,93,236,25,102,238,188,235,46,222,
  125,247,221,179,78,130,254,226,35,143,224,112,58,57,126,243,157,41,136,104,106,184,62,241,32,206,143,63,48,228,223,8,237,217,71,199,67,159,3,160,
  116,219,230,180,159,243,252,230,183,184,255,247,183,0,100,126,226,65,92,31,127,48,237,103,27,150,173,2,32,235,19,15,146,57,200,231,206,232,248,63,
  243,57,130,123,246,98,91,48,159,194,95,60,58,172,107,91,181,107,1,38,188,251,246,144,175,147,189,94,194,85,39,145,61,30,124,111,111,197,187,126,
  131,224,178,2,201,19,207,127,254,23,76,153,153,231,188,159,87,253,207,93,4,90,142,13,203,137,90,203,86,144,117,221,127,12,233,179,193,19,27,113,
  111,254,254,128,215,11,63,177,105,200,191,167,132,189,68,59,79,34,135,189,68,187,170,9,158,216,72,204,219,50,224,115,182,169,235,176,78,184,10,75,
  73,5,146,101,240,210,135,49,111,11,209,206,147,132,234,223,33,88,181,49,217,191,221,244,83,204,197,21,67,150,47,220,178,79,29,215,234,222,33,80,
  181,17,37,236,21,6,43,48,34,156,126,229,95,49,90,93,103,221,207,71,84,244,222,233,116,242,165,71,30,193,239,11,243,203,199,183,93,244,55,107,
  197,218,241,124,249,155,43,184,106,237,248,51,18,81,128,146,177,46,150,175,29,207,151,190,185,130,107,110,26,122,173,186,187,63,50,139,15,254,195,188,
  1,68,180,182,170,123,0,225,44,30,235,98,205,77,147,248,212,87,150,82,177,116,204,21,103,192,127,124,124,27,1,95,152,47,61,242,8,206,179,168,
  63,123,231,93,119,49,119,222,60,90,30,253,197,144,137,168,192,217,195,224,116,98,155,95,65,198,170,149,20,124,237,43,148,62,245,4,6,81,71,88,
  32,5,33,141,186,221,231,172,159,183,110,126,92,37,162,195,132,117,194,85,195,34,174,103,29,108,178,56,49,151,84,96,157,176,2,199,252,251,201,187,
  239,143,56,22,60,144,244,126,238,29,191,38,115,213,151,177,78,88,113,70,34,10,96,116,22,99,157,176,130,204,149,95,38,231,166,159,158,149,124,150,
  226,10,44,197,21,184,150,60,68,193,189,127,196,148,59,69,24,171,192,89,17,210,88,200,115,86,253,220,48,210,129,161,168,168,136,167,158,220,141,207,
  23,190,224,55,162,169,193,67,77,85,119,210,35,142,49,99,93,220,114,247,244,180,215,62,246,189,237,124,235,145,244,51,222,171,111,154,52,36,2,91,
  62,183,144,249,75,6,146,202,63,255,238,48,79,252,236,61,254,235,187,219,83,94,103,179,155,184,243,35,179,200,206,181,95,81,198,27,240,133,121,241,
  201,221,20,21,21,113,167,86,154,101,36,147,162,15,127,228,35,132,27,26,105,249,217,47,206,169,124,225,61,251,8,239,217,71,72,123,68,170,78,142,
  218,189,136,54,183,232,191,19,218,123,230,223,138,84,157,36,180,119,159,254,144,61,67,219,188,17,174,58,73,112,239,190,164,199,80,175,245,174,223,64,
  243,103,62,135,255,237,45,169,157,219,212,41,228,253,211,195,98,84,22,72,182,109,183,155,99,223,254,206,185,233,231,61,141,180,110,126,124,68,223,145,
  142,96,166,138,8,74,22,231,176,9,105,176,106,35,61,47,127,158,80,93,250,28,86,199,252,251,49,58,139,53,114,188,2,83,94,106,2,24,170,219,
  74,251,239,110,165,237,55,107,211,126,159,185,184,98,200,4,50,218,117,146,158,13,159,199,183,247,169,180,196,57,251,218,255,16,198,42,48,98,196,66,
  30,154,222,250,222,89,245,243,17,157,192,116,221,245,215,211,222,238,99,243,166,154,139,226,70,252,253,249,227,212,156,236,78,249,222,204,51,44,151,55,
  55,120,206,184,218,51,115,110,33,45,13,131,59,237,116,132,53,30,17,181,101,152,7,189,62,59,207,70,79,215,149,181,92,191,107,83,53,55,220,59,
  151,235,174,191,158,223,141,160,60,196,178,229,203,113,58,157,212,127,243,63,207,185,108,29,15,125,182,207,97,141,114,50,145,239,229,13,248,94,222,160,
  47,59,90,207,176,220,222,253,232,207,245,165,245,225,96,164,215,197,9,115,252,218,140,85,43,83,15,38,37,37,98,84,22,24,24,44,120,225,207,76,
  254,236,195,103,221,207,79,255,245,59,35,115,114,121,83,210,70,30,163,157,39,49,151,84,12,36,175,19,174,34,84,59,244,205,81,49,79,43,225,230,
  125,32,169,68,51,29,140,174,98,98,222,22,140,174,226,65,191,47,78,146,163,93,213,105,191,207,96,29,90,4,74,9,123,9,55,239,35,220,188,15,
  199,252,251,83,203,229,44,22,134,42,112,86,232,58,252,34,69,203,62,61,226,126,62,108,50,186,124,249,114,138,138,138,46,137,229,249,243,137,83,85,
  93,188,185,94,155,105,38,18,128,174,32,54,187,137,15,254,195,188,244,179,234,64,148,218,170,238,43,242,190,189,242,236,1,62,240,208,50,150,47,95,
  206,187,195,220,141,119,231,93,119,17,110,104,164,235,249,191,8,3,60,79,24,108,41,126,168,81,86,129,43,15,213,143,253,156,217,63,248,222,200,251,
  121,79,35,221,251,95,28,209,111,219,166,221,144,246,61,255,161,23,200,74,69,70,203,86,64,138,188,209,115,133,72,243,62,72,67,12,173,19,86,144,
  119,223,31,9,213,109,37,92,247,14,109,123,158,76,254,192,8,231,198,131,165,2,136,156,81,129,115,129,214,109,255,197,184,27,254,115,68,253,124,248,
  100,244,42,53,247,230,189,157,13,23,205,13,184,245,158,233,250,46,120,80,151,237,255,254,194,241,243,76,70,187,57,85,213,61,160,232,189,205,110,226,
  227,255,180,136,226,65,150,250,55,60,127,252,138,53,222,67,59,79,195,67,203,88,126,213,85,195,50,222,162,162,34,38,79,158,76,251,19,163,83,88,
  59,255,241,199,226,42,84,157,214,250,13,248,95,222,120,73,223,235,156,207,61,140,236,77,118,58,173,9,17,224,193,224,188,233,70,108,11,230,99,155,
  95,145,242,253,104,115,11,221,191,249,173,24,141,5,82,162,237,181,215,117,255,49,146,126,222,177,99,228,253,220,82,146,198,102,59,79,18,243,180,164,
  37,110,214,178,21,131,46,187,159,13,194,205,251,240,31,126,158,140,89,247,164,124,223,232,44,38,99,214,61,100,204,186,7,37,236,37,84,183,149,80,
  221,59,35,146,199,148,59,133,156,155,126,154,118,89,95,9,123,241,236,120,92,24,169,192,89,163,247,228,27,42,25,29,102,63,31,17,25,157,59,111,
  30,71,14,183,94,20,185,162,113,140,25,66,78,231,133,128,205,110,226,99,103,32,162,127,249,221,97,246,109,111,58,63,37,28,46,66,4,124,97,170,
  15,183,50,119,222,188,97,219,33,64,239,107,175,143,138,92,150,5,201,14,44,188,119,223,37,127,175,45,83,71,190,73,193,84,82,140,169,100,224,82,
  94,112,239,62,194,39,170,232,254,205,111,7,16,93,1,1,157,248,185,221,116,239,216,57,226,126,238,62,254,230,136,126,215,232,42,78,155,155,25,110,
  222,71,180,51,125,126,182,117,194,85,163,70,70,1,188,219,31,39,112,232,5,236,179,239,198,58,97,69,218,165,114,201,226,196,54,117,29,182,169,235,
  80,194,94,252,135,95,192,127,248,249,33,71,51,37,139,51,229,78,251,104,151,74,198,61,59,30,79,185,211,95,64,96,184,136,133,60,120,27,118,13,
  187,159,143,136,140,22,21,21,177,101,243,193,139,234,6,52,53,120,6,68,70,47,10,34,250,185,69,131,110,126,138,19,209,43,29,213,135,91,185,254,
  222,185,195,186,166,184,184,88,27,208,119,142,78,228,98,143,74,62,227,145,209,104,115,243,37,127,159,195,85,39,71,76,24,189,235,55,224,89,191,145,
  172,123,239,73,202,25,181,205,175,192,54,191,2,231,205,55,210,254,237,239,166,221,224,36,32,208,181,99,7,147,151,84,142,172,159,215,142,172,159,15,
  150,191,25,237,172,214,254,61,153,146,176,90,203,86,192,219,223,31,213,123,18,243,182,224,221,254,56,222,237,143,99,202,155,130,165,184,2,219,212,27,
  210,18,104,201,226,196,49,255,126,50,102,221,77,247,250,207,19,237,58,243,198,202,104,215,73,188,59,30,199,92,92,145,148,51,106,202,157,130,41,119,
  10,214,9,43,240,237,125,10,239,222,39,133,145,10,156,53,124,167,119,81,180,108,241,232,146,209,56,219,109,111,191,184,34,32,131,109,96,186,16,40,
  25,235,226,131,255,48,47,237,14,249,96,32,202,31,127,181,239,138,205,19,29,224,164,218,125,186,125,13,181,70,217,220,185,115,9,55,52,142,154,76,
  231,115,3,211,249,194,104,110,96,50,56,157,100,221,247,62,65,70,5,210,34,160,245,215,97,247,243,179,56,115,222,92,146,62,66,99,41,153,167,22,
  149,79,147,75,41,89,156,88,39,140,206,82,189,209,89,140,109,218,186,212,19,191,29,143,19,233,60,169,150,133,90,112,127,202,136,169,100,113,146,185,
  234,203,116,189,248,201,51,254,214,80,54,48,57,230,223,47,200,168,192,185,9,122,184,135,223,207,135,77,70,227,104,111,243,137,59,158,110,38,63,214,
  197,199,62,183,8,155,61,245,173,109,105,240,240,151,223,29,62,227,238,252,43,138,140,182,141,108,114,51,154,100,84,64,64,224,220,34,56,194,254,26,
  233,29,249,234,209,96,37,154,210,145,193,228,235,71,103,169,222,232,42,78,75,12,125,168,41,4,193,170,141,68,154,247,145,119,223,31,83,59,111,81,
  27,84,224,34,38,163,195,133,73,81,134,113,6,147,162,140,106,35,38,79,205,129,155,39,39,197,161,94,77,113,34,211,112,208,60,4,210,151,142,56,
  14,231,59,0,22,44,29,195,77,119,79,79,251,125,71,247,183,241,151,223,29,78,121,28,168,128,106,95,202,40,219,216,197,132,204,79,36,159,186,20,
  220,179,143,208,222,189,163,246,123,142,155,214,13,216,128,20,109,110,17,167,39,9,92,150,253,252,76,181,66,35,205,251,0,53,202,152,110,89,92,93,
  230,31,250,82,189,101,136,197,245,211,109,156,82,229,113,232,207,101,177,203,93,224,10,233,231,166,139,73,246,201,83,115,84,66,122,14,201,232,225,3,
  109,28,57,208,150,182,222,232,103,255,101,41,182,140,244,183,225,136,118,253,153,200,99,249,220,66,238,250,240,172,65,63,99,207,48,243,193,127,172,24,
  240,250,222,237,77,236,223,33,114,71,175,52,12,60,2,244,183,163,74,70,157,55,221,56,224,181,224,222,125,130,140,10,92,150,56,211,169,75,221,47,
  125,94,37,144,37,21,100,223,242,211,52,196,112,104,75,245,142,5,247,227,88,112,255,224,190,89,91,46,7,53,87,52,88,181,17,219,212,129,209,217,
  140,89,247,96,214,162,158,233,72,50,128,255,240,243,66,201,2,151,13,76,87,66,35,159,254,159,253,44,92,58,134,21,107,6,30,7,154,110,131,81,
  115,131,135,119,223,170,103,207,16,55,24,13,229,148,166,178,126,68,59,14,145,59,42,32,32,32,112,110,97,78,83,210,9,250,162,162,128,78,16,211,
  146,218,179,92,170,143,147,208,254,39,32,185,223,254,62,225,230,253,100,204,186,123,0,233,28,76,246,104,215,73,252,135,95,24,112,62,189,128,128,32,
  163,195,192,127,61,186,91,95,238,79,21,109,76,245,90,83,131,135,95,253,236,189,180,223,57,148,101,244,221,219,155,216,189,189,9,187,221,196,152,177,
  46,114,242,236,228,244,219,96,212,221,21,160,167,51,64,115,191,221,249,67,193,222,29,77,156,170,234,26,80,103,52,93,155,18,209,211,21,20,150,120,
  158,225,127,121,3,161,17,110,230,233,120,232,115,233,29,69,194,174,123,223,203,27,8,238,25,220,209,37,46,98,196,180,107,35,39,170,104,251,76,250,
  223,136,156,168,74,249,122,56,225,186,161,46,142,36,22,170,111,77,248,77,165,159,140,241,118,133,79,84,209,156,70,54,81,244,94,224,98,130,103,144,
  162,245,253,151,191,123,180,40,105,42,135,20,255,108,207,203,159,63,243,143,246,27,236,35,157,39,7,45,193,20,172,218,168,147,74,75,73,69,218,148,
  129,168,246,61,241,83,158,6,180,117,251,227,105,79,100,146,67,125,191,223,179,225,243,195,26,31,4,4,46,91,50,90,93,213,61,108,50,26,8,68,
  213,235,6,251,220,16,147,48,131,129,168,122,118,125,194,247,13,85,142,193,208,221,25,160,187,51,48,100,50,218,255,53,73,36,145,158,87,196,154,91,
  136,53,183,164,33,136,131,15,213,161,180,4,83,25,240,27,209,230,150,33,147,209,184,81,200,94,239,136,136,178,236,245,38,237,150,31,174,195,73,119,
  173,50,200,111,8,8,92,172,56,83,196,115,72,159,149,134,249,125,210,217,203,59,146,40,236,80,74,60,37,254,134,32,163,2,23,27,12,226,22,8,
  8,8,8,8,8,8,8,8,8,50,42,32,32,32,32,32,32,32,32,32,200,168,128,128,128,128,128,128,128,128,128,192,249,194,136,114,70,191,254,205,
  107,197,157,19,184,224,112,46,173,164,226,212,49,113,35,4,4,46,99,56,38,44,102,238,215,143,136,27,33,32,32,200,104,50,94,217,184,145,214,150,
  22,113,247,4,206,9,138,138,139,185,97,221,186,97,95,215,210,210,194,171,27,69,121,19,1,1,209,207,5,4,4,46,229,126,110,98,4,39,48,189,
  178,113,35,251,247,237,19,119,93,224,156,96,94,69,133,106,188,138,50,172,83,190,90,91,90,120,234,201,39,197,13,20,16,16,253,92,64,64,224,18,
  238,231,34,103,84,64,64,64,64,64,64,64,64,224,130,65,144,81,1,1,1,1,1,1,1,1,1,65,70,5,4,4,4,4,4,4,4,4,174,60,
  140,236,4,166,97,230,2,8,8,156,209,158,46,196,181,2,2,2,162,159,11,8,8,92,240,126,46,34,163,2,2,2,2,2,2,2,2,2,23,12,
  35,62,155,254,66,204,83,43,42,42,104,105,105,161,69,148,149,18,184,128,118,88,92,92,76,113,113,49,251,46,241,138,18,151,75,59,196,189,186,252,
  219,113,33,227,162,14,135,131,178,137,19,169,61,117,10,159,207,119,201,233,188,176,176,144,130,194,66,14,31,58,116,73,219,238,229,210,14,129,212,184,
  164,34,163,63,121,244,81,110,184,241,70,161,53,129,11,138,27,110,188,145,159,60,250,168,104,135,208,185,104,199,21,128,178,137,19,249,214,183,191,77,
  217,196,137,151,164,252,107,175,190,154,111,125,251,219,151,188,30,46,151,118,8,92,6,100,84,64,64,64,64,64,64,96,232,80,68,59,4,4,25,61,
  59,220,255,224,131,92,181,98,69,218,247,31,122,248,97,42,42,42,132,22,5,70,21,21,21,21,60,244,240,195,105,223,191,106,197,10,238,127,240,65,
  209,14,161,115,209,142,203,0,179,102,207,230,193,143,125,12,135,195,209,71,132,18,54,101,172,189,250,106,238,125,255,251,47,122,249,147,136,92,130,252,
  149,75,150,92,212,242,95,110,237,16,184,4,201,104,69,69,69,18,249,188,97,221,58,214,165,89,150,159,50,101,10,119,223,115,15,69,197,197,0,56,
  157,78,238,190,231,30,156,78,167,208,170,192,89,161,191,45,21,21,23,115,247,61,247,48,101,202,148,148,159,95,119,227,141,73,199,159,93,181,98,197,
  69,49,73,186,92,218,33,238,149,208,249,104,194,225,112,112,243,45,183,80,80,88,216,247,247,173,183,166,93,150,159,57,107,22,247,222,119,159,254,247,
  226,202,74,102,205,158,125,193,229,143,147,231,130,194,194,100,249,251,237,110,94,179,118,45,107,214,174,189,104,228,191,220,218,33,48,50,152,148,97,108,
  195,87,18,254,85,70,161,212,198,93,247,220,67,197,252,249,124,224,222,123,25,59,110,28,205,205,45,216,51,28,60,252,249,207,83,58,166,148,29,59,
  118,50,99,230,44,126,240,227,159,208,210,210,76,109,109,29,57,185,121,228,229,229,113,203,173,183,114,255,131,15,178,111,223,62,60,85,85,0,200,178,
  12,138,130,172,201,170,164,41,73,37,73,18,146,36,1,18,146,65,74,248,91,224,124,96,36,118,165,140,162,29,22,21,23,243,208,195,15,227,116,58,
  121,233,165,151,176,218,108,212,214,214,113,219,157,119,146,153,153,133,213,102,99,199,142,157,124,239,135,63,162,187,187,11,123,134,131,230,230,22,42,230,
  47,160,234,196,113,190,252,149,175,176,111,239,94,246,238,221,171,219,93,226,3,64,145,229,20,191,44,97,48,72,113,163,212,237,112,164,182,120,185,180,
  227,124,64,232,124,244,219,113,177,245,243,56,50,50,50,120,224,99,31,163,160,176,144,191,190,248,34,185,185,185,212,215,159,230,186,27,214,177,238,198,
  155,176,88,173,236,222,189,135,219,239,188,139,213,107,215,146,147,155,199,193,3,7,153,53,123,54,53,213,213,60,248,177,143,225,243,249,248,210,23,190,
  208,119,207,100,89,109,175,118,175,20,69,73,185,204,108,48,24,116,31,4,32,25,12,195,214,125,126,65,1,15,124,236,99,100,56,28,188,254,218,107,
  88,204,22,93,254,76,151,75,151,255,171,255,246,117,122,122,186,177,218,237,180,182,180,50,107,246,108,78,213,212,240,208,195,15,115,248,208,33,14,29,
  60,152,86,126,57,205,253,79,148,95,146,36,93,247,35,193,229,210,14,225,207,71,198,19,77,23,82,232,98,109,224,252,254,119,191,139,215,235,229,173,
  77,155,24,63,97,2,95,249,218,191,113,178,186,154,95,255,230,55,156,56,113,130,211,167,79,211,209,209,142,201,96,32,26,139,97,50,91,152,48,97,
  2,19,39,78,100,230,204,153,220,247,161,15,51,125,218,52,246,238,217,203,177,163,71,145,101,89,143,168,110,92,191,97,112,33,164,62,146,42,1,200,
  218,139,82,159,145,74,146,164,27,171,192,229,137,123,222,247,62,0,158,127,238,57,78,86,85,113,244,232,81,22,86,86,82,80,84,204,174,221,187,249,
  242,87,190,194,169,83,167,104,106,110,34,28,12,162,200,50,49,69,33,59,59,135,73,147,38,49,109,218,52,230,206,153,195,61,247,222,75,107,107,27,
  27,55,108,36,18,137,96,183,219,249,127,159,250,20,191,123,234,105,218,219,219,7,181,67,80,136,201,10,82,255,55,164,190,65,210,112,6,103,117,185,
  180,67,232,252,242,211,185,114,17,213,10,173,172,172,164,108,226,68,158,125,230,25,218,218,218,168,58,113,130,201,83,166,112,239,125,239,231,68,85,21,
  223,250,143,255,160,174,174,142,150,214,86,194,193,32,40,10,49,69,33,51,43,139,146,146,18,102,150,151,51,179,188,156,235,111,88,135,199,235,227,205,
  215,95,35,26,137,96,179,217,184,251,125,239,99,253,75,47,209,217,217,121,70,57,98,177,24,36,232,86,138,197,250,72,145,193,144,86,247,55,223,114,
  11,0,47,191,244,18,181,167,78,81,117,226,4,115,231,205,35,55,55,143,189,251,247,39,201,31,9,5,81,228,62,249,203,38,76,96,242,228,201,204,
  44,47,231,230,91,110,165,173,173,157,77,111,190,73,36,28,198,110,179,241,225,251,239,231,133,231,158,27,153,252,241,127,19,252,230,96,182,123,185,180,
  67,224,28,68,70,47,40,25,45,41,97,197,202,149,200,178,76,117,117,13,185,249,249,252,242,191,127,197,91,111,111,230,248,241,227,120,60,30,0,198,
  151,150,242,145,15,125,136,236,156,28,234,234,234,56,116,240,32,71,143,30,97,239,190,189,252,245,111,127,37,59,59,155,37,149,149,84,46,90,204,131,
  31,255,56,199,143,29,227,67,31,254,48,71,143,28,97,195,250,245,103,164,241,146,254,84,210,7,86,73,27,55,101,109,70,21,139,197,48,12,50,56,
  8,92,218,184,106,229,74,42,42,42,240,7,2,148,207,156,197,91,155,223,102,203,150,45,108,219,177,157,230,230,102,98,177,24,89,153,153,44,168,168,
  96,217,178,101,120,125,62,142,30,61,202,145,195,135,217,252,246,102,54,191,189,25,179,217,204,204,242,114,150,44,174,164,162,162,130,91,93,183,83,86,
  54,129,249,11,22,176,126,253,122,218,218,219,134,104,135,137,19,34,213,22,21,5,20,73,65,150,101,125,112,52,104,51,248,203,177,29,66,231,151,159,
  206,3,129,0,29,237,237,132,195,97,20,69,185,160,227,232,154,171,175,166,178,178,146,174,174,46,102,207,157,199,214,119,222,101,251,142,29,188,183,123,
  55,109,109,173,68,163,81,50,93,46,230,206,158,205,156,185,115,137,70,163,84,85,85,113,252,216,49,246,239,219,203,238,221,239,97,54,155,41,41,25,
  195,154,149,43,153,61,123,54,119,221,243,62,76,70,3,139,43,43,217,191,119,47,157,29,29,103,8,132,104,237,79,32,233,138,246,186,162,40,72,177,
  152,174,251,254,132,168,114,201,18,102,205,154,69,48,16,100,242,180,105,73,242,183,182,182,16,139,197,116,249,23,45,94,140,207,231,211,229,127,119,219,
  187,188,187,237,93,204,102,51,211,166,77,99,97,197,124,102,207,158,205,181,215,223,192,248,241,227,152,61,103,14,155,222,120,227,236,228,151,101,36,205,
  127,38,202,222,95,231,151,75,59,4,206,30,198,73,19,39,126,99,56,145,204,235,111,184,129,87,54,108,164,165,121,228,181,62,139,139,139,241,122,189,
  180,52,183,80,88,84,136,195,229,226,68,213,73,30,253,249,99,108,216,184,129,211,167,79,19,14,135,25,91,82,194,7,63,248,65,238,190,231,110,10,
  11,10,9,133,66,40,138,66,94,110,46,249,249,121,32,43,120,189,94,60,94,47,85,39,79,178,111,255,62,122,221,110,102,148,151,147,233,202,228,127,
  127,245,43,220,189,189,216,237,25,152,77,102,162,225,72,194,186,79,191,71,124,56,85,208,136,168,182,108,146,240,190,30,246,151,149,164,165,21,129,179,
  141,144,151,176,238,198,27,121,245,149,87,104,29,98,13,217,235,215,173,3,5,94,217,176,97,196,191,235,116,58,177,88,44,132,195,97,170,78,156,96,
  254,130,133,68,162,81,126,255,135,63,240,212,211,79,179,123,207,110,220,110,55,46,167,147,171,150,47,231,190,123,239,101,217,178,101,24,77,70,66,161,
  48,25,118,59,37,37,37,56,157,14,220,61,61,4,130,65,90,91,91,57,112,232,32,199,142,29,35,39,55,135,73,19,39,113,236,232,17,94,219,248,
  10,40,144,151,151,79,192,231,63,179,29,246,179,61,253,111,36,80,64,145,53,91,84,192,229,116,97,181,88,47,249,118,72,218,127,163,9,161,243,11,
  215,142,130,130,66,86,174,90,197,91,155,54,209,218,218,218,23,1,28,100,28,61,23,253,60,14,135,195,129,197,108,33,18,142,112,252,216,49,230,47,
  92,64,48,28,102,227,198,141,60,249,244,83,236,223,127,0,143,199,131,217,100,102,249,210,165,220,126,251,237,44,90,180,16,135,35,131,64,32,136,221,
  102,163,176,168,136,236,156,108,122,58,187,9,132,66,244,246,246,114,228,216,49,170,170,170,176,89,173,148,142,29,75,79,119,55,47,253,253,37,20,20,
  74,74,75,113,187,221,253,110,149,212,119,203,82,144,34,41,5,73,82,0,187,221,142,217,100,34,18,137,114,170,166,134,153,179,102,19,138,70,120,254,
  249,231,121,230,185,103,117,249,157,142,12,150,44,94,172,203,111,52,24,9,105,209,194,194,162,34,28,142,12,60,189,110,2,161,16,237,237,237,186,252,
  89,89,89,140,31,63,158,147,85,85,108,126,107,51,10,10,57,185,185,248,3,129,179,151,95,81,180,180,57,112,58,212,62,24,9,71,56,85,125,233,
  182,227,124,140,87,87,138,63,7,144,174,93,187,118,200,107,39,243,42,42,248,209,79,126,194,231,63,251,89,246,237,221,55,34,65,87,172,92,201,127,
  124,231,63,249,222,119,190,3,72,100,229,230,242,242,134,245,236,217,189,135,104,52,66,86,86,22,227,198,150,50,119,238,60,102,207,158,69,44,38,211,
  219,211,67,79,111,175,58,163,150,21,162,209,8,225,112,152,80,56,140,215,235,165,185,169,153,211,13,13,120,125,94,130,193,32,217,57,57,92,123,245,
  213,84,46,90,204,219,155,54,241,192,131,15,162,160,240,173,127,255,70,138,229,165,129,161,249,36,99,76,122,158,48,184,106,239,197,163,165,2,35,71,
  197,252,10,126,250,216,99,124,233,11,95,96,255,16,139,113,255,232,167,63,69,145,101,62,255,217,207,141,248,119,127,253,196,19,0,252,248,135,63,100,
  237,53,215,176,109,215,78,54,108,216,64,115,115,51,153,46,23,5,249,121,76,157,58,149,202,197,149,100,102,102,18,8,4,232,117,247,226,241,120,145,
  99,50,177,88,140,104,52,66,40,20,34,20,14,211,221,213,197,137,170,147,244,186,123,241,7,2,72,146,196,252,138,10,86,173,88,65,36,20,38,224,
  247,243,137,79,126,130,95,252,252,231,236,219,179,55,173,45,74,233,6,198,52,207,159,248,221,211,72,72,252,232,7,63,224,234,107,175,189,100,219,17,
  95,22,27,205,213,135,203,69,231,63,254,217,79,49,26,77,252,207,175,126,197,53,215,93,119,73,180,99,230,172,89,252,251,55,191,201,127,254,199,183,
  56,122,228,136,254,190,65,50,96,48,166,214,249,185,232,231,113,252,240,199,63,2,224,167,63,249,9,107,175,190,134,61,251,247,241,250,27,111,210,220,
  212,72,86,86,54,121,185,106,218,194,204,153,229,228,231,231,19,12,6,113,123,60,4,252,1,162,145,168,122,207,98,81,34,225,8,193,80,144,222,158,
  30,234,235,79,211,217,221,77,48,24,68,146,96,252,248,241,220,112,221,117,68,35,81,66,193,0,31,248,224,7,121,245,149,87,120,237,149,87,83,220,
  179,126,47,72,201,62,168,191,63,250,153,86,31,246,191,30,127,156,21,171,86,179,123,239,30,94,127,227,77,218,90,91,112,58,93,186,252,243,231,87,
  224,114,185,116,249,125,94,95,159,206,99,81,194,161,48,161,112,136,222,158,30,106,78,157,162,215,237,209,229,159,61,123,54,203,150,44,73,146,255,201,
  39,158,224,240,161,195,103,37,127,226,243,127,255,198,55,48,24,141,252,225,119,191,231,170,149,43,47,217,118,232,182,43,86,75,207,218,159,159,69,100,
  116,3,45,45,205,35,18,180,171,171,147,69,149,149,204,171,152,207,177,170,19,172,127,121,61,117,117,181,228,100,103,115,213,178,101,220,112,253,245,44,
  91,182,156,220,156,92,186,186,186,232,232,232,192,237,241,16,139,198,80,180,255,226,225,114,147,209,132,221,110,163,176,176,128,177,99,199,146,157,149,133,
  209,104,36,24,10,81,95,95,143,47,224,231,154,107,175,101,210,196,73,252,245,197,23,57,93,127,58,133,17,246,155,31,73,9,175,13,120,206,128,231,
  10,138,182,228,116,65,86,26,47,143,153,84,201,72,35,163,202,89,69,76,34,225,48,215,222,112,61,249,133,5,60,255,194,11,108,217,242,54,138,162,
  48,103,230,44,214,174,93,195,181,215,94,203,140,25,229,132,194,97,58,187,186,232,238,238,81,103,215,178,130,172,200,125,228,201,104,196,108,50,225,202,
  204,100,252,184,177,20,23,21,97,183,217,144,101,153,246,142,118,218,218,219,25,83,90,202,138,21,43,113,247,244,242,247,191,253,141,72,36,50,136,45,
  146,108,111,73,118,25,31,47,251,236,48,18,14,115,221,186,117,228,23,22,94,210,237,136,175,76,40,113,153,70,161,67,93,46,58,15,135,195,172,88,
  181,138,236,220,28,94,252,219,223,216,178,117,43,92,228,237,40,40,40,96,229,234,213,108,121,251,109,218,59,58,116,146,170,160,168,171,77,210,64,157,
  159,139,126,174,235,62,18,97,197,170,85,20,20,21,177,109,251,118,222,124,243,77,20,69,166,108,252,4,214,172,89,205,170,85,43,153,50,101,26,146,
  36,209,217,213,69,79,79,47,129,64,16,57,38,39,217,164,193,104,192,100,50,225,112,58,25,51,118,12,5,249,249,216,172,86,100,89,198,227,241,210,
  217,213,73,113,81,49,139,43,43,113,247,186,249,203,159,255,76,36,18,237,139,200,73,253,163,113,67,32,64,146,68,56,18,97,213,154,53,228,230,231,
  243,247,191,191,196,246,237,219,81,20,153,242,233,51,184,234,170,229,186,252,225,112,152,238,158,110,93,126,69,86,146,229,55,24,48,155,205,170,252,99,
  198,80,88,88,160,203,223,217,213,69,71,103,159,252,30,183,135,215,95,123,237,156,200,31,55,133,72,36,66,101,101,37,153,89,89,108,216,184,145,237,
  59,119,192,37,216,14,221,118,21,45,206,42,8,192,136,253,57,156,199,156,209,226,226,98,90,90,90,240,122,189,172,127,249,101,114,243,243,217,182,109,
  27,57,57,217,44,169,92,204,164,73,147,144,144,240,120,61,52,55,55,227,247,249,137,68,35,122,72,60,174,244,248,67,146,36,140,70,3,146,65,66,
  150,101,172,22,117,83,211,212,169,83,113,187,61,28,63,113,140,61,123,246,16,9,135,137,69,99,156,172,170,66,81,20,114,243,114,233,234,236,74,21,
  36,6,73,209,151,197,20,9,125,83,147,34,73,218,115,245,117,73,81,52,67,84,159,43,168,57,165,170,76,70,97,141,23,253,50,130,106,139,187,118,
  238,98,203,219,91,120,245,245,215,105,110,105,97,249,178,101,84,204,155,75,78,78,46,94,143,7,183,199,67,75,107,43,161,96,136,152,28,211,211,51,
  100,69,238,219,93,172,40,24,13,6,36,36,12,42,147,34,59,59,155,130,130,2,20,160,246,212,41,234,234,235,217,252,246,219,68,163,81,156,246,12,
  252,62,31,32,13,176,69,169,223,146,145,20,167,34,73,118,169,229,225,73,18,37,197,197,180,52,55,179,115,199,14,182,188,253,54,175,190,254,198,37,
  217,14,125,243,160,54,184,75,138,218,167,101,89,198,104,52,158,147,65,254,114,209,121,65,126,62,29,237,237,236,219,187,151,109,219,182,241,246,214,173,
  52,54,53,178,120,193,2,22,44,88,64,65,65,193,197,219,14,69,65,142,197,244,223,86,36,41,97,44,85,144,99,231,86,231,137,203,243,62,159,143,
  109,219,182,81,54,121,50,59,119,189,71,77,77,13,139,23,45,98,246,172,153,228,228,230,227,238,237,193,235,245,209,214,214,65,56,28,34,150,144,142,
  149,84,77,64,81,212,188,93,131,17,131,193,160,231,227,230,230,229,49,11,104,107,109,165,186,186,154,119,183,111,39,26,139,226,204,200,32,224,247,3,
  146,186,188,223,221,147,56,239,74,113,255,36,45,200,161,17,32,69,161,160,168,136,182,182,54,246,237,217,195,246,237,219,217,252,246,22,90,219,219,116,
  249,51,179,114,240,121,61,125,242,135,66,196,148,68,249,211,232,92,91,209,75,148,255,116,125,61,13,13,13,231,84,254,120,31,207,207,203,163,179,171,
  139,195,135,14,241,222,238,221,108,219,177,147,230,214,86,230,205,153,203,188,185,115,200,205,203,191,36,218,17,231,4,73,182,43,131,172,196,84,153,4,
  41,29,17,70,20,25,221,184,97,195,176,207,135,255,217,99,143,177,246,234,171,9,134,194,76,154,50,133,63,255,229,47,204,156,81,206,61,119,223,141,
  36,73,116,116,118,210,216,216,68,111,111,47,161,80,136,104,44,138,217,100,198,104,52,225,116,186,176,218,236,88,44,22,76,38,19,6,131,17,5,181,
  76,67,162,97,2,42,129,85,20,138,139,75,232,233,238,230,208,225,195,184,50,93,44,89,178,148,172,172,76,30,248,216,199,104,109,109,213,115,150,226,
  198,168,164,120,78,156,136,38,204,154,146,94,147,164,190,207,106,207,245,196,124,97,144,231,37,50,186,113,152,17,147,21,43,87,242,131,31,255,24,191,
  223,207,242,21,43,217,177,115,39,111,110,218,196,47,30,123,20,151,211,133,223,239,167,190,254,52,29,29,157,120,125,62,53,10,36,129,201,104,194,102,
  179,145,145,225,192,106,181,97,54,171,182,41,25,12,200,178,130,130,58,80,26,80,43,48,200,178,76,36,18,193,233,116,97,50,25,169,175,175,167,165,
  181,149,121,243,230,146,149,153,197,29,119,221,73,229,146,37,108,125,103,235,64,59,148,18,6,73,169,47,238,148,104,127,171,86,175,230,39,143,62,138,
  207,239,103,229,234,53,236,216,181,235,146,108,135,30,67,147,146,151,191,226,253,77,78,32,170,35,197,229,162,243,133,139,22,241,47,95,253,42,129,64,
  144,69,149,149,236,217,187,143,119,223,121,135,239,124,251,219,56,29,78,60,94,15,167,79,55,208,213,221,125,81,182,35,63,63,159,21,43,87,242,206,
  214,173,116,36,68,70,147,35,78,125,187,238,37,73,26,113,63,143,99,214,172,89,252,244,209,71,81,100,153,138,5,11,57,112,240,32,91,183,190,195,
  71,62,244,33,38,76,152,128,223,239,167,169,169,137,142,206,46,252,126,31,209,104,4,69,81,212,123,102,181,98,179,103,96,177,88,49,153,204,26,73,
  54,160,40,125,164,94,82,20,245,62,198,98,68,34,17,44,22,11,102,179,153,134,198,6,218,218,218,40,25,51,134,113,227,198,113,205,181,215,178,116,
  217,50,246,237,221,75,52,18,77,47,112,95,40,14,9,88,178,116,25,255,254,205,111,18,8,4,88,188,100,41,123,246,238,101,235,86,85,231,14,167,
  3,191,223,79,67,83,35,93,93,221,186,252,72,96,28,68,126,89,86,144,181,224,142,65,139,144,199,229,119,56,156,152,76,38,93,254,153,51,103,146,
  233,202,228,134,117,55,80,49,127,62,239,237,218,53,48,201,82,74,47,127,252,223,57,115,231,242,233,135,30,34,20,12,50,103,238,60,14,30,60,196,
  174,93,187,248,234,151,191,140,195,225,192,235,245,209,216,212,68,79,79,239,69,221,142,254,81,210,196,228,20,117,179,158,210,111,28,19,254,252,162,139,
  140,110,221,186,149,187,239,185,135,185,243,43,120,226,201,39,169,175,171,99,221,13,215,115,178,186,154,218,218,90,45,4,175,232,157,124,242,228,201,204,
  158,53,27,95,32,130,205,98,35,16,8,224,243,251,241,120,61,120,189,94,122,123,123,240,251,124,4,131,126,34,225,16,209,104,223,14,77,139,217,140,
  2,100,56,28,120,188,94,222,124,235,45,162,209,40,55,92,119,29,173,173,173,116,116,116,32,203,74,90,123,209,38,64,9,179,35,73,179,180,20,179,
  164,132,8,105,124,23,100,44,22,195,112,142,103,247,2,231,6,45,45,45,52,55,55,179,238,230,155,121,234,233,167,121,225,207,127,102,213,138,21,52,
  53,55,115,240,224,65,98,90,228,38,30,137,207,200,200,96,209,162,69,56,50,92,200,138,58,225,241,251,252,120,124,94,188,94,47,94,175,135,222,222,
  94,2,1,63,225,96,128,168,18,82,163,63,40,122,148,220,98,177,160,160,112,162,170,138,13,175,188,194,141,215,223,64,97,97,17,91,183,188,173,47,
  79,38,142,130,74,124,105,54,49,222,20,127,170,217,95,83,83,51,77,77,77,220,124,219,109,151,116,59,20,36,36,53,236,167,175,58,40,146,218,239,
  212,23,180,157,172,138,130,97,132,171,14,151,139,206,219,219,218,105,105,105,97,229,154,213,252,229,197,23,217,248,202,43,44,89,188,152,230,150,22,142,
  28,57,162,215,80,148,36,9,147,217,132,195,225,184,168,218,33,107,99,99,255,200,104,255,40,83,60,50,158,174,46,244,112,208,214,222,206,225,67,135,
  185,106,229,42,158,250,221,211,188,244,242,122,150,86,86,98,48,26,56,114,228,8,177,152,156,20,117,179,219,237,204,152,49,131,156,236,60,253,158,133,
  66,33,220,94,15,126,159,31,175,207,139,223,231,197,231,243,17,9,5,136,17,33,22,85,151,126,141,6,35,70,131,17,179,217,140,162,64,205,169,90,
  98,49,153,219,110,185,133,188,252,124,118,237,216,73,32,160,70,231,36,37,153,252,12,140,198,169,47,182,105,129,147,107,174,189,142,103,159,127,78,151,
  191,181,173,149,163,71,143,170,242,39,216,174,221,110,103,222,188,121,100,216,157,125,58,15,4,240,249,125,186,252,30,183,155,96,48,64,36,20,32,170,
  132,53,157,171,242,3,73,242,191,177,105,19,87,175,89,171,203,175,46,147,247,147,95,235,170,169,228,143,255,219,217,217,73,123,123,59,139,151,44,101,
  195,198,13,188,185,105,19,11,230,207,167,181,173,149,19,39,142,35,235,28,78,194,100,50,146,145,145,113,81,182,131,126,54,171,143,85,241,255,43,103,
  63,94,93,169,56,111,100,244,201,39,158,192,231,243,115,242,84,13,123,246,236,97,242,196,137,228,229,231,113,236,232,177,36,34,154,155,155,203,196,137,
  19,89,184,112,33,147,39,77,166,163,203,75,107,75,43,173,45,205,73,37,46,204,102,11,102,75,140,88,76,65,86,32,38,199,144,99,209,164,217,78,
  233,152,82,76,134,61,116,116,116,176,111,223,126,198,148,148,112,244,224,33,154,154,154,250,70,128,84,9,204,241,161,33,78,74,227,92,52,213,224,153,
  138,144,2,114,44,166,111,198,16,184,120,112,178,170,138,111,252,219,215,185,254,198,27,217,188,121,51,29,237,237,44,169,172,164,170,170,74,173,53,135,
  154,18,98,54,155,41,45,45,165,188,188,156,197,139,22,129,100,166,173,189,139,246,182,118,66,193,144,110,139,70,163,9,139,197,170,218,97,76,33,166,
  200,104,107,54,250,111,186,92,153,100,101,102,209,212,212,204,177,227,199,41,41,42,198,221,219,203,27,175,189,74,124,167,116,114,238,157,148,60,16,38,
  76,138,226,255,86,157,56,206,191,126,229,171,220,114,251,237,151,116,59,36,73,65,81,226,203,185,169,9,105,60,101,38,94,94,109,184,147,188,203,69,
  231,117,117,181,60,250,211,159,177,114,245,106,118,236,216,65,87,103,39,21,21,21,212,212,212,232,117,18,21,20,44,38,51,37,37,37,204,155,55,143,
  165,75,150,92,52,237,80,20,133,88,52,170,18,77,24,232,212,251,17,82,5,229,172,207,35,111,111,107,227,103,63,253,9,247,126,224,3,188,187,109,
  27,93,157,29,204,159,63,159,96,48,64,44,26,211,221,128,201,100,162,184,184,152,169,83,167,50,107,230,76,204,22,59,29,157,189,116,118,116,18,141,
  70,245,184,152,209,96,196,100,50,99,54,219,212,180,2,69,2,41,166,23,85,7,117,183,120,166,203,69,107,107,43,141,141,141,28,57,114,132,72,40,
  196,91,155,222,212,108,87,163,236,9,247,174,127,125,214,248,61,171,169,169,230,251,223,253,46,87,95,119,125,146,252,53,53,213,186,252,50,10,230,4,
  249,43,230,205,3,201,164,203,31,10,133,146,228,55,155,205,154,206,101,98,178,162,233,60,181,252,39,79,86,83,84,80,136,215,227,230,221,45,91,134,
  45,127,252,223,198,134,6,158,248,223,223,176,176,178,146,61,123,246,208,219,221,195,172,153,51,169,171,171,83,219,161,233,221,108,54,83,88,88,200,172,
  153,179,88,176,96,254,69,215,14,137,4,155,37,110,187,241,124,238,56,15,80,63,61,210,241,234,138,37,163,195,58,129,73,73,206,161,25,10,42,230,
  207,103,254,252,249,28,58,116,8,187,195,193,150,45,91,9,6,131,140,31,63,158,96,48,132,207,231,215,163,18,249,249,249,44,91,186,148,217,179,103,
  147,149,149,133,36,129,221,110,193,108,182,232,145,198,120,105,133,104,52,74,44,22,213,114,74,13,72,6,11,146,172,160,40,49,125,150,232,112,100,144,
  155,155,67,91,71,7,141,205,77,236,219,191,159,89,51,202,49,155,205,44,88,184,128,63,191,240,66,223,228,93,146,244,242,50,138,68,66,145,102,205,
  97,166,154,17,105,131,39,137,57,164,137,247,43,166,45,127,9,66,122,206,237,138,196,83,181,134,136,135,63,251,89,182,108,217,194,210,101,203,121,245,
  141,55,56,89,93,77,81,97,33,153,153,153,28,57,122,84,255,62,139,197,194,212,41,83,184,234,170,171,24,55,110,156,90,10,40,18,195,102,85,211,
  68,36,131,65,219,104,163,104,187,146,85,91,84,205,192,140,193,160,16,83,194,160,200,200,138,130,217,108,34,51,51,19,187,221,142,199,227,225,192,161,
  131,140,27,59,150,177,227,198,49,101,202,20,90,154,91,56,122,244,72,223,240,39,37,216,157,110,155,125,196,237,11,95,250,2,155,223,218,204,138,85,
  171,120,125,211,166,75,182,29,234,235,137,81,208,228,8,131,62,200,211,55,217,139,105,253,105,168,3,252,229,162,243,247,127,224,253,236,219,183,151,138,
  249,11,216,182,99,7,117,117,117,228,231,229,225,200,200,224,100,117,181,30,69,52,91,204,76,154,56,145,37,75,151,50,110,252,120,108,54,27,49,153,
  139,162,29,241,157,208,122,137,170,193,198,212,248,104,26,207,215,148,229,97,47,125,222,255,224,131,28,62,116,136,252,130,2,246,237,219,79,109,109,45,
  5,249,249,184,92,78,58,59,58,245,73,136,172,40,76,156,56,133,202,197,139,25,83,90,138,197,108,38,18,137,97,181,152,49,153,140,72,6,131,238,
  119,98,209,24,209,104,76,191,103,72,70,36,201,140,34,133,117,89,77,38,19,46,151,19,155,205,70,32,24,228,232,241,99,140,41,41,97,202,212,105,
  184,92,78,220,110,55,213,85,39,147,8,123,124,151,90,159,239,129,79,254,227,63,178,125,219,54,22,47,89,202,150,173,91,147,228,175,58,209,156,52,
  137,154,88,86,150,94,254,4,191,57,80,126,19,146,36,235,242,203,253,228,247,250,124,186,252,197,37,99,40,43,43,163,189,189,109,72,242,199,117,127,
  235,109,183,114,228,200,97,102,206,154,197,123,187,247,208,208,208,64,110,110,14,25,25,25,212,214,214,233,27,128,204,102,51,19,198,143,103,193,130,5,
  140,41,45,197,106,181,34,203,92,52,237,208,199,171,132,201,178,106,179,232,175,199,55,235,197,109,87,142,251,255,43,136,144,142,200,159,115,30,206,166,
  95,185,114,37,15,124,236,99,124,254,139,95,228,189,221,187,57,81,117,130,220,236,28,230,205,157,75,87,167,58,235,84,80,112,56,28,44,95,190,156,
  133,11,23,146,147,147,163,19,56,155,213,140,205,110,195,104,52,169,250,149,213,65,45,170,27,99,76,43,76,111,192,96,176,32,25,140,250,224,101,48,
  24,152,48,97,188,26,226,247,251,57,121,178,154,104,52,202,189,239,127,63,83,166,76,33,51,43,171,47,57,93,139,206,246,223,40,5,138,106,236,241,
  227,220,6,252,139,254,119,18,73,138,15,21,241,186,100,2,23,62,143,229,166,155,248,222,15,126,128,195,229,98,207,158,61,120,220,110,42,43,43,137,
  68,34,132,67,97,61,58,63,121,242,100,86,175,94,205,228,201,147,177,88,44,234,172,205,100,192,102,179,98,177,88,84,231,68,191,65,49,26,211,54,
  61,128,36,153,48,24,204,122,52,8,73,34,39,59,155,108,205,222,26,155,154,56,126,226,4,171,215,174,101,197,138,149,140,27,63,110,192,177,139,125,
  201,250,201,27,40,74,74,74,184,249,214,91,249,201,163,63,195,149,149,117,201,182,35,222,55,208,222,75,236,103,125,107,102,201,125,43,222,223,244,37,
  220,43,68,231,185,121,185,44,93,182,148,207,124,246,179,88,109,54,14,31,57,140,215,231,99,238,220,185,132,195,97,194,161,176,186,225,75,145,153,48,
  97,2,203,150,45,163,108,194,4,76,70,35,94,175,23,73,82,46,138,118,200,138,58,110,171,169,88,131,141,165,253,198,84,80,229,27,134,99,115,56,
  28,172,94,189,154,207,124,246,179,148,142,29,199,129,131,7,240,122,189,84,84,204,39,18,81,75,2,198,199,230,66,45,8,50,97,194,4,204,38,147,
  26,64,48,26,176,90,205,152,205,102,205,23,169,237,82,9,124,140,88,52,170,221,51,25,73,50,33,105,247,76,214,8,73,86,86,38,89,89,46,64,
  161,185,165,133,234,154,26,202,103,205,98,217,242,171,152,62,99,6,178,162,168,249,186,73,247,171,239,81,88,88,200,53,215,94,203,191,125,227,27,56,
  156,78,14,28,58,152,36,127,252,224,0,89,150,41,27,63,126,80,249,37,131,52,136,252,74,146,252,202,32,242,47,89,182,148,69,139,23,83,50,102,
  140,186,95,99,16,249,227,143,236,236,108,230,207,159,207,253,15,60,128,217,108,225,248,137,227,248,124,126,102,204,40,39,18,14,19,14,133,244,205,138,
  165,165,99,89,184,96,1,99,75,75,49,26,12,248,124,62,64,185,40,218,161,244,31,175,18,109,87,255,59,197,152,165,40,195,182,93,177,76,63,74,
  248,249,99,143,177,103,207,30,110,190,245,54,222,222,242,54,178,28,99,202,148,201,76,159,62,157,253,7,14,232,57,162,211,167,79,103,225,194,133,216,
  109,182,100,1,141,18,89,89,46,204,102,11,146,164,230,18,69,162,81,34,145,8,145,72,68,29,220,52,178,103,48,154,48,153,76,68,163,65,130,65,
  191,182,84,63,150,195,135,143,224,15,6,233,232,236,96,255,161,131,100,101,101,242,218,198,87,8,134,130,106,216,95,66,15,194,39,135,55,181,61,161,
  241,224,103,194,66,84,223,108,158,132,229,69,82,236,182,71,31,252,69,132,244,2,230,138,54,55,115,223,61,247,240,15,255,239,83,236,217,183,143,211,
  13,167,181,74,14,149,248,252,126,221,14,101,89,230,250,235,175,103,76,73,73,242,172,77,146,176,219,173,56,28,14,53,159,78,129,88,76,38,172,217,
  98,56,18,209,119,10,75,146,1,147,201,138,209,100,38,20,244,32,203,50,174,204,76,242,243,242,105,105,107,35,28,14,115,248,232,17,38,150,149,177,
  123,231,78,14,29,58,148,20,89,146,250,213,190,85,18,82,70,154,154,26,185,253,230,91,120,248,159,62,199,190,3,7,47,217,118,12,236,95,169,35,
  15,125,177,209,196,236,58,73,239,79,131,69,72,47,23,157,119,118,116,240,149,47,255,11,119,222,125,55,39,170,170,104,109,109,37,203,229,98,206,156,
  57,248,245,118,168,247,102,213,170,85,140,41,41,209,29,167,44,203,248,125,62,44,22,219,5,111,135,162,40,68,99,209,132,201,72,114,116,41,97,0,
  237,75,207,72,88,9,25,138,206,227,240,249,124,60,252,208,67,92,119,195,13,52,54,55,115,244,216,49,178,51,51,89,48,191,130,128,63,72,32,24,
  64,86,84,18,116,195,186,117,140,29,59,182,159,238,193,102,179,96,183,219,49,25,77,40,10,196,162,50,145,104,132,104,52,66,36,26,69,142,197,180,
  189,7,18,38,131,5,140,38,194,97,191,90,121,192,225,34,39,59,151,174,174,110,194,145,8,199,171,78,48,110,220,88,246,237,217,195,169,83,53,186,
  223,209,27,169,168,167,173,196,173,190,165,165,133,143,221,255,0,15,124,252,99,28,62,122,148,230,230,230,36,249,19,109,119,245,154,53,20,21,22,166,
  149,223,96,48,166,149,63,158,102,22,151,63,20,242,33,203,178,46,127,123,123,7,145,4,249,15,238,63,192,137,19,199,19,42,203,164,150,63,174,253,
  238,238,46,126,240,253,239,113,253,186,27,169,174,169,161,189,173,29,151,203,73,121,249,12,252,129,128,182,1,73,189,126,233,210,37,20,21,21,37,217,
  110,32,224,199,108,182,94,240,118,36,253,63,62,94,197,109,55,105,104,74,200,127,31,161,237,94,169,56,47,236,72,146,12,84,215,212,112,250,244,105,
  114,178,115,153,95,49,31,127,32,64,48,24,212,55,43,45,92,184,16,155,213,154,122,150,155,97,193,98,181,32,105,51,207,168,54,136,70,19,242,143,
  12,146,132,209,96,208,243,121,36,109,86,98,207,176,83,80,80,0,64,40,28,166,169,169,153,80,40,140,223,239,215,107,238,233,51,163,196,191,101,37,
  97,137,168,111,150,63,32,130,147,98,74,175,244,133,70,251,94,211,190,71,224,194,193,227,241,16,147,99,84,157,172,194,239,247,179,112,193,34,28,14,
  7,30,143,71,31,216,215,173,91,55,96,96,143,195,108,54,98,183,91,49,26,141,90,57,175,40,209,72,132,72,36,172,149,165,137,215,192,149,48,154,
  76,152,77,22,76,70,179,22,101,51,145,155,151,139,85,179,241,142,206,78,154,154,155,136,234,203,150,9,182,38,247,139,214,247,179,65,183,219,77,76,
  86,46,249,118,160,164,238,87,218,159,3,87,28,250,245,45,181,143,94,25,58,247,249,188,68,162,81,234,234,235,9,6,131,204,156,57,19,155,213,134,
  199,235,213,235,96,174,94,181,154,188,220,220,164,205,63,241,239,138,70,195,216,108,150,11,218,14,53,26,27,77,136,108,43,9,58,86,250,86,147,72,
  26,64,7,46,1,14,113,24,245,122,189,88,172,86,78,157,58,69,119,119,55,115,230,206,197,158,145,129,199,235,33,18,81,203,6,174,93,187,150,210,
  49,99,82,71,106,76,70,245,158,105,209,198,104,44,70,52,162,250,158,196,123,38,25,36,45,16,98,198,100,82,239,145,209,100,34,39,39,27,167,211,
  9,64,87,119,55,173,173,173,152,204,166,36,63,163,147,74,18,250,130,230,123,188,94,15,178,172,112,234,212,41,2,129,64,146,252,113,219,93,187,118,
  45,249,121,121,131,203,175,165,25,12,69,126,163,193,148,36,191,197,98,77,146,63,158,102,209,95,126,133,228,126,157,104,123,126,191,159,72,36,66,67,
  99,3,193,80,136,105,83,167,97,181,90,241,106,69,236,21,89,102,233,146,165,100,103,103,107,17,126,117,101,51,30,49,141,70,195,88,173,230,11,222,
  14,72,195,3,20,165,159,213,166,78,33,27,142,237,138,200,232,144,147,2,24,210,77,125,248,115,159,85,151,218,237,25,188,189,117,43,177,104,148,137,
  101,19,152,81,62,131,142,246,118,162,209,40,86,171,149,41,147,167,48,126,220,184,180,179,6,147,201,128,211,233,192,96,48,16,141,70,9,135,195,68,
  194,17,213,160,80,48,72,6,76,38,35,70,147,17,163,209,136,85,178,225,247,245,18,139,70,48,24,12,20,22,20,208,218,218,74,40,28,198,227,241,
  208,211,211,195,85,43,87,144,157,149,205,206,29,59,232,104,111,87,167,239,137,59,231,146,3,49,125,51,35,173,192,125,223,158,209,129,9,206,137,95,
  20,143,12,160,149,3,49,98,16,101,159,82,217,211,40,94,59,101,234,84,30,252,216,199,56,120,240,32,161,112,132,134,134,70,178,92,46,22,45,90,
  72,52,26,197,231,83,103,209,57,217,57,44,93,178,52,109,173,88,163,65,141,148,89,44,22,20,69,81,163,74,225,48,209,72,84,63,187,216,104,52,
  96,52,153,48,26,85,91,148,229,8,161,176,154,120,159,153,153,73,166,211,73,40,24,36,18,137,208,220,220,204,154,149,171,200,205,201,197,96,48,240,
  246,230,205,122,94,148,164,217,99,226,243,233,211,167,243,143,159,254,20,7,246,239,39,18,141,94,178,237,72,17,107,72,234,87,241,157,172,105,122,83,
  194,95,233,87,28,46,23,157,151,142,25,195,13,55,222,72,93,93,29,161,80,136,214,182,86,28,25,25,148,151,151,19,141,70,240,251,212,200,104,150,
  43,139,121,243,230,105,165,135,250,206,179,143,59,83,228,24,146,164,92,88,157,203,106,32,65,73,72,205,208,235,203,42,244,91,113,210,162,165,73,65,
  39,213,46,100,100,12,131,196,81,202,202,202,184,231,222,247,113,252,196,9,12,70,19,205,205,45,152,77,38,230,206,157,75,44,26,37,224,247,35,203,
  50,89,153,89,204,175,152,175,18,244,20,203,168,241,232,162,205,106,213,8,189,26,69,142,7,65,84,2,164,249,30,163,17,131,86,123,52,24,244,33,
  161,30,61,155,157,149,69,79,111,47,145,72,132,214,214,54,230,204,154,197,194,69,139,48,24,36,118,238,220,169,70,228,18,183,116,43,48,105,202,36,
  62,248,145,143,112,244,232,81,194,209,40,205,205,45,56,29,142,180,242,27,52,146,150,78,126,117,87,121,106,249,13,6,131,238,55,13,6,35,177,88,
  148,112,36,164,203,239,116,100,16,10,135,116,249,151,47,89,66,86,86,214,0,249,245,152,120,188,41,18,20,21,21,177,106,245,26,154,26,27,9,133,
  66,116,180,119,224,200,176,51,117,234,20,162,145,8,129,128,218,14,151,203,197,204,153,51,213,254,46,203,200,160,229,54,247,217,46,146,114,193,218,145,
  28,88,83,163,162,241,8,41,74,95,5,9,37,97,163,115,127,30,49,84,219,189,146,253,185,129,179,98,163,131,63,166,76,157,202,61,247,222,203,132,
  137,19,57,118,252,56,89,89,89,44,90,184,16,179,217,76,87,87,151,86,172,222,134,209,100,197,31,136,17,10,199,244,242,36,73,6,0,228,100,57,
  49,26,141,68,163,81,66,161,16,145,104,68,175,47,106,54,155,176,88,173,88,45,86,108,54,27,246,12,27,57,185,249,9,57,87,121,228,229,229,1,
  18,193,112,136,142,206,78,198,143,31,79,65,65,1,14,135,67,171,87,42,107,231,62,199,115,71,228,1,57,36,40,178,54,115,143,207,154,100,253,185,
  130,210,247,94,226,103,232,187,14,226,249,163,138,120,12,60,216,154,209,178,197,226,146,98,42,22,204,231,125,247,221,75,115,75,11,237,29,237,84,84,
  84,80,90,90,138,199,163,30,33,135,162,238,28,14,134,21,124,254,48,209,152,156,82,42,71,134,141,12,155,21,69,150,9,133,194,132,66,97,98,178,
  140,100,144,48,154,140,170,29,106,15,187,221,70,102,102,22,118,187,29,69,81,176,89,109,20,21,21,97,181,168,206,173,189,83,221,68,81,62,179,156,
  172,172,172,52,54,152,144,47,58,166,132,133,139,22,241,193,15,127,248,146,110,135,162,200,122,223,233,43,223,211,175,207,36,246,27,37,89,159,74,226,
  115,37,126,58,206,229,169,243,172,236,44,202,202,202,88,177,106,37,237,237,237,244,116,247,48,109,234,84,10,242,243,241,120,188,4,67,65,20,89,38,
  35,195,65,48,36,227,247,135,137,68,99,90,78,162,156,180,137,192,108,54,96,50,26,46,152,206,213,40,87,76,221,140,20,215,98,98,164,105,128,35,
  83,82,250,56,221,134,210,244,119,5,133,241,19,38,112,227,77,55,209,214,214,70,103,87,39,99,199,170,167,75,121,188,94,130,161,16,178,162,224,112,
  184,8,133,21,188,254,16,209,104,234,124,65,155,205,138,195,161,158,72,21,10,133,9,135,67,68,163,49,253,36,38,179,217,140,197,162,78,86,108,54,
  11,118,173,150,171,162,40,88,45,86,114,115,115,177,152,45,122,84,46,18,137,48,121,202,100,114,243,242,84,226,165,168,145,189,248,233,66,178,162,80,
  80,88,200,156,185,115,185,253,206,59,117,249,103,207,156,153,36,191,162,160,203,239,15,132,7,149,223,170,157,74,20,151,63,22,147,251,228,183,88,146,
  228,119,186,92,73,242,23,20,228,39,201,175,0,147,167,76,198,229,114,245,201,175,244,201,223,87,158,81,198,229,202,100,236,216,82,22,46,94,76,103,
  87,23,189,189,189,76,156,56,145,220,156,92,188,62,31,161,80,8,69,150,177,219,50,8,133,98,248,3,97,34,145,88,95,116,52,193,134,205,70,131,
  94,3,247,124,183,163,255,61,77,140,144,38,218,110,60,240,153,20,217,87,148,97,217,238,149,236,207,71,53,103,244,115,159,249,12,0,223,253,209,143,
  56,126,252,184,58,43,92,184,144,230,166,38,188,62,31,40,144,149,149,141,65,50,209,208,208,164,13,130,25,56,93,25,56,50,44,152,77,125,92,217,
  110,87,137,102,40,24,196,239,243,169,6,106,177,98,181,88,48,91,44,106,24,223,104,196,100,50,97,52,154,144,19,78,29,113,185,92,140,45,45,165,
  171,179,147,72,56,76,119,119,55,59,182,111,167,234,248,9,109,214,174,22,169,239,203,121,26,60,114,169,167,250,40,146,246,60,158,111,26,223,93,79,
  210,174,186,228,82,102,170,193,171,201,216,2,231,3,91,223,126,155,155,223,126,155,107,175,191,30,111,32,136,28,141,113,245,213,107,145,99,49,58,187,
  186,244,227,13,199,143,47,163,177,161,25,171,213,130,213,106,35,195,145,129,203,105,195,110,51,235,193,108,147,201,128,195,233,80,151,159,124,62,66,161,
  32,70,163,17,139,102,139,22,171,5,179,217,164,150,128,49,155,136,68,34,184,221,61,40,138,90,187,177,116,76,41,237,109,109,132,195,33,186,187,187,
  233,117,187,89,255,210,75,68,181,2,229,146,36,105,182,20,223,209,217,231,138,55,189,241,6,111,189,249,38,55,221,122,43,17,89,185,100,219,145,116,
  142,121,191,154,190,72,244,203,27,37,121,205,65,15,142,42,250,151,42,178,130,100,148,46,75,157,31,62,116,136,175,127,237,95,153,191,112,33,161,136,
  154,31,183,96,193,2,162,145,40,61,61,221,68,35,17,36,201,64,73,201,24,154,155,91,177,217,108,216,236,118,156,78,7,153,174,12,50,50,36,61,
  98,105,50,26,176,152,77,196,98,177,11,162,115,69,81,171,160,196,137,134,148,96,4,125,165,241,146,245,159,98,157,30,36,53,93,203,152,102,133,169,
  174,182,150,207,62,244,16,11,22,45,34,34,203,200,209,24,11,42,43,80,98,49,186,187,123,136,104,69,231,75,75,199,210,220,220,138,217,108,194,106,
  179,145,145,97,199,225,176,97,181,152,136,15,207,38,163,132,221,110,3,69,33,16,240,19,10,133,145,36,176,88,108,88,45,102,140,102,19,22,179,89,
  187,103,70,100,89,193,221,107,65,81,188,24,141,70,114,115,115,201,116,57,137,68,194,244,186,123,113,123,220,252,237,197,191,226,241,120,180,18,91,241,
  253,5,146,246,31,188,251,206,59,108,123,247,93,214,94,123,29,17,57,134,28,141,113,213,138,171,210,202,111,209,116,23,151,223,102,53,233,254,198,100,
  148,112,100,216,147,228,55,26,13,152,205,86,117,151,186,197,140,217,100,210,229,143,68,162,120,61,110,93,254,226,162,18,58,58,58,117,249,61,30,15,
  111,189,249,166,90,238,42,81,126,77,247,137,209,173,19,39,142,243,147,31,159,96,214,236,217,4,195,234,38,187,57,115,230,16,139,70,112,247,186,181,
  239,48,80,84,84,68,107,107,7,54,155,5,171,205,78,134,35,3,167,195,78,134,221,172,229,137,42,24,141,6,44,102,35,114,44,54,106,237,200,201,
  205,211,202,49,74,72,146,1,131,22,249,238,232,104,79,145,122,216,23,10,77,220,73,159,52,118,245,149,74,30,178,237,138,101,250,81,70,36,28,161,
  173,189,141,59,239,184,29,89,81,232,238,233,81,103,32,168,167,87,132,194,33,58,59,218,49,106,167,43,88,44,54,108,118,43,185,185,57,20,230,103,
  97,52,170,133,112,237,118,171,154,127,18,141,226,116,56,177,103,216,177,90,213,210,79,102,109,121,73,50,104,117,72,77,38,106,52,71,102,48,24,200,
  202,206,38,47,63,159,166,230,102,124,126,31,69,249,249,122,136,63,78,38,165,20,36,52,209,102,20,165,111,41,191,207,113,162,19,209,148,206,179,159,
  33,38,230,142,136,100,230,243,139,88,76,198,237,113,147,95,144,79,126,126,1,93,93,93,132,67,97,61,133,194,100,182,210,221,213,129,193,104,212,242,
  191,76,88,173,118,50,179,92,228,229,102,225,114,218,144,36,112,58,51,136,70,35,4,2,1,204,38,19,14,167,75,95,202,53,155,45,152,18,236,80,
  81,20,122,187,51,232,214,236,194,158,97,167,176,168,144,110,183,155,96,48,72,192,239,87,75,205,116,118,106,75,69,26,65,147,18,167,49,137,19,36,
  73,109,135,215,115,201,183,35,177,158,175,190,104,155,184,92,155,216,111,206,48,193,75,183,92,127,217,232,60,26,195,231,243,146,149,157,133,203,149,137,
  199,235,37,28,86,87,135,100,41,134,193,104,166,167,187,83,171,191,108,161,167,219,130,205,150,65,86,118,166,218,14,87,6,146,164,144,225,176,19,12,
  248,47,72,59,100,89,86,75,241,200,74,191,90,246,137,199,193,146,80,222,105,32,9,213,39,36,218,247,13,182,41,212,108,182,208,213,209,65,110,94,
  46,227,198,143,199,237,245,18,9,247,149,96,50,154,204,244,244,116,33,73,106,132,211,100,50,97,177,216,176,103,88,201,207,203,193,145,97,213,136,167,
  153,72,36,66,48,24,196,100,52,96,179,103,144,145,97,83,239,151,73,221,52,27,95,42,150,36,176,217,236,244,104,237,50,155,205,228,23,228,227,246,
  122,9,6,131,248,253,1,12,146,212,87,218,74,146,208,15,236,235,23,7,137,197,98,120,188,94,114,243,114,201,206,206,165,183,183,71,151,95,78,144,
  223,96,48,234,114,88,44,54,156,46,7,57,217,46,93,126,123,134,45,73,254,12,135,83,91,190,183,104,167,113,25,117,249,173,86,5,119,111,159,252,
  86,187,45,73,254,64,32,128,35,35,131,158,158,158,148,242,203,82,255,131,140,36,162,209,24,126,159,159,76,151,11,167,195,129,215,231,39,28,209,108,
  151,24,146,193,132,187,183,11,159,207,140,217,98,193,220,109,198,102,179,225,202,116,145,147,157,137,195,97,211,210,253,172,4,131,129,81,107,199,188,249,
  243,245,96,86,252,126,26,77,38,76,38,245,254,30,61,114,132,195,218,134,189,196,88,189,126,100,248,0,219,237,43,241,52,92,219,21,100,244,28,98,
  202,212,169,60,252,185,207,169,97,241,158,94,50,157,78,10,242,243,233,234,236,84,151,200,136,215,186,147,241,251,3,4,131,33,36,131,212,87,216,94,
  50,208,212,216,128,197,98,193,225,112,80,84,148,79,118,118,22,225,112,152,9,227,199,227,112,56,116,99,49,24,212,89,140,62,59,151,36,100,123,223,
  50,160,132,164,27,183,220,212,68,56,28,198,225,112,112,213,138,21,106,237,188,253,251,245,129,96,234,244,105,100,101,231,96,52,25,49,25,77,90,30,
  138,73,139,186,26,49,104,249,84,70,131,150,159,98,52,96,48,24,49,26,12,24,140,6,186,58,59,217,186,229,237,126,167,54,48,192,121,10,50,122,
  254,240,240,231,62,199,148,169,83,241,249,3,60,251,220,179,148,20,23,19,10,6,241,184,221,125,167,239,40,10,145,72,148,94,183,27,131,65,29,93,
  227,118,216,210,162,22,88,182,219,237,184,92,78,198,142,29,67,40,28,34,55,55,23,167,83,173,99,23,207,181,51,104,215,197,109,17,192,98,179,234,
  17,113,163,209,72,86,86,22,102,147,137,64,32,64,40,20,98,246,236,57,196,228,24,7,15,28,160,183,183,55,97,98,148,60,172,255,243,87,254,133,
  233,51,102,16,12,133,120,238,249,231,47,217,118,244,143,144,246,189,147,16,93,208,8,200,80,39,120,241,77,130,113,89,47,23,157,95,119,253,245,148,
  142,45,69,150,21,222,221,190,157,156,236,108,66,161,32,94,175,151,88,44,166,71,104,34,145,40,30,143,87,29,163,76,234,248,100,48,26,105,239,48,
  115,218,162,238,170,206,204,116,49,118,236,24,130,161,32,89,89,89,100,101,101,157,87,157,199,235,67,247,45,83,38,105,62,97,146,159,64,74,83,90,
  75,60,210,154,172,243,56,86,173,94,205,234,53,107,8,4,67,252,237,239,127,163,176,160,0,139,197,66,119,87,151,174,123,25,133,72,36,134,199,227,
  85,163,160,186,239,81,151,131,155,26,27,177,90,109,56,157,25,228,229,229,18,142,132,201,206,202,194,225,116,98,49,155,49,105,37,159,84,123,73,246,
  61,22,171,89,181,88,89,221,207,224,114,185,48,153,76,4,131,65,194,145,48,83,166,78,5,20,142,31,59,142,199,227,81,109,216,16,191,103,6,254,
  223,167,63,205,164,201,147,9,133,35,188,188,254,101,10,11,10,8,135,130,120,61,158,164,211,194,116,249,13,232,182,43,73,6,140,237,70,204,38,147,
  46,127,73,73,81,146,252,86,139,37,201,111,14,208,121,130,252,70,67,63,249,195,97,166,77,159,142,44,199,250,228,55,36,50,105,131,238,248,86,174,
  88,73,81,73,49,177,152,204,238,61,123,200,202,202,34,28,10,227,247,251,250,108,23,137,72,52,138,215,235,199,104,50,168,254,213,160,254,219,217,213,
  73,147,89,45,237,232,114,58,41,41,41,34,20,10,145,233,114,225,202,204,60,231,237,120,111,215,46,12,218,68,76,253,78,3,86,171,149,236,220,92,
  140,6,3,29,157,29,12,60,201,158,4,146,153,108,187,41,163,163,103,176,93,65,70,71,57,143,85,146,36,194,145,8,7,15,30,100,242,228,41,234,
  192,164,229,16,5,130,1,122,122,123,244,178,77,241,58,130,178,150,168,28,9,71,9,135,67,248,124,62,138,75,138,248,252,231,63,139,207,231,197,227,
  246,105,59,242,67,132,35,234,145,108,178,34,163,196,212,107,35,145,112,210,14,249,80,48,72,111,79,143,46,155,172,200,72,10,68,181,58,165,241,221,
  120,49,109,246,174,27,117,76,66,146,212,28,161,152,172,45,121,73,6,100,73,59,151,88,175,161,215,183,243,142,36,231,169,164,140,184,34,140,241,188,
  230,83,199,55,146,197,98,50,245,245,245,52,52,54,234,58,136,239,164,236,233,237,209,54,110,200,9,182,163,110,186,136,70,163,68,194,97,125,86,253,
  190,247,221,69,65,65,62,94,175,15,175,215,143,223,31,32,20,10,19,141,105,37,95,52,91,140,198,98,88,204,22,253,59,99,177,24,61,221,61,68,
  34,17,213,230,21,53,95,41,22,147,245,157,161,3,198,246,184,69,37,214,93,188,132,219,145,196,73,7,36,192,244,149,67,139,159,253,172,247,147,65,
  39,120,125,228,84,186,140,116,174,70,19,101,237,40,205,40,109,109,109,180,183,181,107,249,115,49,205,1,75,244,186,123,49,155,204,32,129,65,210,8,
  129,118,148,170,44,203,68,35,234,198,207,96,40,200,93,119,222,78,65,65,62,49,25,124,190,243,167,243,248,6,166,120,125,205,1,154,79,220,24,154,
  160,99,37,85,116,52,254,146,172,128,113,224,8,27,175,97,30,139,201,52,54,54,82,91,91,139,35,195,161,19,81,20,5,183,167,55,165,238,227,164,
  57,18,142,16,10,5,9,133,67,220,116,211,58,114,115,114,240,251,3,4,2,33,252,126,63,161,176,122,4,101,76,238,43,228,31,147,21,204,38,179,
  254,157,178,44,227,238,237,213,210,41,250,10,161,247,229,67,202,170,135,144,213,168,170,193,144,88,147,82,209,229,111,105,109,237,59,169,240,12,242,199,
  98,106,253,237,184,252,193,96,144,155,111,185,81,151,223,231,83,163,139,161,112,88,171,211,221,39,127,52,22,27,84,254,120,110,101,76,78,144,95,86,
  73,176,100,0,3,178,78,72,99,74,220,46,212,186,160,29,29,29,106,20,93,146,80,228,152,30,145,244,120,220,152,76,102,213,108,36,67,95,234,134,
  118,125,52,22,83,251,97,56,196,141,235,174,39,47,55,151,152,172,224,247,7,207,105,59,186,187,58,181,118,72,26,185,85,39,27,205,205,77,250,235,
  73,254,90,79,213,83,250,165,155,36,205,152,7,70,71,207,96,187,87,44,25,85,70,64,46,213,93,97,131,227,68,85,21,159,213,114,70,255,243,123,
  223,83,75,84,212,214,82,88,84,132,201,104,34,20,11,129,162,168,51,62,175,23,131,193,160,26,148,102,124,234,70,165,32,193,96,136,96,32,168,46,
  65,152,45,248,124,94,198,143,27,131,162,64,32,16,166,215,227,197,231,11,224,247,249,213,93,246,90,205,188,182,246,86,61,21,32,26,141,210,221,221,
  77,71,103,151,186,235,206,104,162,187,187,155,35,122,200,93,219,5,42,195,137,227,199,53,99,52,36,68,106,13,250,172,55,49,122,27,127,168,39,142,
  196,151,43,36,157,128,39,166,183,33,37,135,240,227,145,31,131,32,163,73,118,53,92,123,28,202,113,2,143,61,250,40,0,215,92,123,45,72,106,105,
  154,214,214,86,10,11,10,245,131,20,208,74,38,89,173,86,125,151,102,44,38,107,213,27,66,132,66,97,130,129,32,145,72,152,252,252,124,220,110,15,
  147,39,77,160,184,40,31,89,86,232,238,245,225,245,248,212,129,62,20,36,26,81,235,224,169,37,132,250,28,140,223,239,167,69,171,236,96,54,153,144,
  144,216,183,111,31,61,221,221,125,75,178,40,24,180,12,44,73,137,251,118,133,239,127,247,187,72,146,196,77,183,222,122,73,183,195,32,37,42,58,241,
  15,109,242,22,207,227,30,176,74,159,188,199,62,197,149,234,4,81,146,46,27,157,191,246,218,171,72,146,196,236,57,115,245,178,94,93,221,93,228,230,
  228,246,45,247,73,6,188,94,15,54,171,13,89,91,9,82,20,136,41,178,86,190,41,170,237,156,143,144,151,159,135,219,237,97,226,196,241,228,228,100,
  171,99,225,121,210,185,172,200,125,65,3,37,245,132,164,111,188,236,23,11,85,6,153,204,244,203,47,125,107,243,102,222,218,188,153,101,203,150,131,164,
  110,88,105,110,110,102,242,228,201,170,238,181,243,220,189,94,15,102,179,21,89,142,233,167,67,245,249,158,16,161,160,186,251,218,225,112,224,113,123,25,
  63,174,148,130,252,92,53,125,193,237,199,231,243,19,12,134,9,4,3,68,35,17,117,7,183,86,140,62,126,207,194,225,48,237,29,157,132,195,97,109,
  57,223,192,241,227,199,112,247,186,251,218,172,40,32,201,96,144,144,21,3,255,245,248,227,24,36,3,107,175,185,70,151,191,189,189,93,221,132,155,66,
  254,120,234,67,92,126,181,40,126,68,151,63,39,39,39,173,252,126,191,74,182,227,242,251,180,253,24,137,58,79,146,31,137,163,71,143,12,148,223,32,
  131,108,64,54,104,250,6,222,217,178,21,73,146,152,54,125,58,138,38,111,119,79,15,217,217,217,218,158,68,25,36,131,86,7,215,154,80,29,81,61,
  16,32,62,33,12,135,195,196,162,49,181,29,30,31,227,198,141,33,43,51,19,163,209,116,94,218,161,143,81,74,130,225,73,137,33,81,41,217,41,73,
  253,102,203,195,176,221,43,217,159,143,122,206,168,211,233,196,98,177,80,84,84,68,75,75,11,13,13,13,76,156,56,17,66,234,251,193,96,16,175,215,
  131,201,100,38,18,81,143,166,139,68,163,68,194,234,128,16,141,68,80,20,5,167,211,129,209,104,194,231,11,232,14,42,35,195,130,221,158,75,36,26,
  195,231,11,17,8,168,228,53,28,14,83,83,123,82,175,253,21,31,68,195,145,136,70,106,205,120,61,94,125,128,147,36,37,121,192,83,250,150,210,245,
  156,38,125,58,147,218,121,246,145,218,228,148,17,69,26,124,59,148,136,142,158,63,100,101,101,17,209,114,117,106,106,106,180,210,30,6,93,157,189,189,
  93,56,157,89,170,29,70,163,218,192,30,33,20,10,19,137,168,179,111,147,209,132,221,110,39,16,80,119,213,2,24,12,18,121,57,78,114,178,29,248,
  253,17,252,129,32,193,64,144,80,40,76,32,16,32,28,86,141,93,142,201,180,182,182,170,181,33,101,25,107,134,3,73,98,160,45,42,146,118,140,156,
  58,48,162,13,140,241,207,100,103,103,99,180,88,46,249,118,244,37,96,75,9,185,131,82,82,95,148,36,37,93,24,117,224,66,191,214,111,19,251,211,
  229,162,115,155,221,134,172,181,173,185,185,25,167,211,137,201,104,210,43,121,120,61,189,168,71,203,203,122,117,144,104,76,214,136,165,186,250,99,54,153,
  176,217,236,4,130,106,59,194,225,48,25,25,166,243,214,14,53,226,220,183,204,44,41,36,77,47,146,115,136,165,126,62,60,181,29,196,35,226,169,198,
  80,87,166,139,80,68,45,239,215,212,212,196,184,113,227,52,221,171,223,236,118,247,146,145,225,66,142,197,84,253,203,234,191,209,72,132,112,36,76,44,
  38,99,50,26,177,90,173,4,131,97,162,145,24,138,85,253,173,236,44,7,153,153,25,132,194,81,124,190,32,33,205,239,244,246,246,18,142,132,85,71,
  28,83,163,113,62,159,15,89,81,176,91,44,24,141,6,98,209,104,63,31,66,223,86,108,89,1,131,106,199,174,204,76,12,1,53,29,160,190,190,14,
  151,203,149,82,254,88,52,170,158,70,164,201,31,9,135,137,68,35,186,252,54,91,122,249,3,193,48,129,64,88,151,63,158,74,16,151,191,163,163,35,
  73,126,9,240,121,125,3,229,151,251,136,28,6,89,167,113,72,168,85,26,180,213,199,182,182,86,28,142,12,140,70,147,190,19,221,235,117,171,122,72,
  56,149,42,38,171,181,68,227,39,44,170,185,220,22,130,193,16,209,136,218,71,141,70,211,121,109,135,238,251,147,14,184,233,179,221,1,27,242,82,39,
  23,13,201,118,175,68,140,106,6,237,19,79,62,201,134,87,94,193,229,116,81,86,86,70,75,107,43,13,13,167,85,67,50,169,187,228,84,50,234,197,
  239,15,224,247,171,75,70,1,127,128,64,192,79,36,172,30,21,102,52,25,177,217,237,72,146,68,192,31,208,119,19,198,7,46,139,217,72,118,118,6,
  133,133,217,20,21,229,81,88,152,79,71,91,139,78,246,186,187,186,105,105,105,85,217,183,209,136,205,170,158,67,125,203,45,183,80,92,92,156,52,235,
  238,95,176,25,37,225,245,120,57,7,250,138,136,12,156,17,12,252,127,234,249,193,240,207,86,23,24,25,86,174,90,197,150,119,222,225,163,247,63,64,
  94,94,62,102,139,133,186,186,90,122,123,123,213,8,184,36,33,203,10,30,143,27,191,223,79,32,224,215,108,81,125,30,10,6,137,69,163,72,72,234,
  142,99,139,69,27,240,194,200,9,7,25,24,36,9,167,195,66,126,158,75,181,195,162,124,32,134,223,167,30,61,24,10,135,104,108,108,36,24,84,29,
  188,35,195,142,201,100,226,134,27,110,96,213,170,85,9,36,44,249,108,223,248,227,234,107,174,225,224,209,163,124,242,31,254,241,146,110,71,98,157,73,
  61,226,48,160,50,72,191,34,41,253,251,137,146,28,50,83,250,77,240,46,23,157,79,154,52,153,47,126,233,75,44,91,182,12,135,195,137,209,100,162,
  173,173,21,175,215,171,47,251,201,178,140,215,235,37,16,8,16,8,6,212,127,3,65,130,90,94,103,76,219,49,108,182,170,27,61,226,237,136,105,203,
  209,231,77,231,114,223,217,244,137,115,125,37,69,12,101,64,153,39,165,223,120,59,192,30,250,254,158,48,161,140,255,251,227,159,184,227,142,59,201,201,
  201,193,108,182,208,210,218,130,199,237,86,51,90,53,221,235,247,76,191,111,65,117,131,76,40,174,123,48,91,204,88,44,22,194,17,181,20,150,156,224,
  27,36,192,102,49,145,147,237,32,63,63,155,188,252,92,236,118,43,1,45,42,23,138,132,233,232,236,36,20,82,239,89,134,205,134,217,100,102,241,226,
  74,150,46,93,6,137,69,211,19,252,206,178,229,87,177,241,245,215,249,192,7,63,168,203,223,216,208,56,100,249,117,157,107,242,155,205,230,180,242,103,
  216,44,73,242,163,200,73,242,183,180,180,38,201,111,52,26,89,185,114,37,139,22,45,78,43,127,124,178,49,110,220,56,62,246,241,143,83,49,191,130,
  12,187,93,205,3,237,236,192,239,247,235,220,76,150,101,124,126,63,193,80,144,80,80,125,4,181,127,227,17,81,73,146,212,13,102,102,179,150,54,19,
  214,142,3,141,156,151,118,144,148,130,167,244,59,227,70,73,237,237,251,133,7,135,106,187,34,103,244,108,98,177,131,96,235,219,91,216,250,246,22,156,
  46,23,11,23,46,228,245,55,94,167,174,190,158,162,162,98,102,204,152,65,40,20,66,142,134,9,248,189,68,35,234,204,46,22,141,169,57,72,218,17,
  159,70,163,1,155,213,134,205,102,69,50,24,240,249,3,116,118,187,41,46,202,29,144,34,111,50,26,48,101,88,8,6,60,28,63,113,92,205,81,138,
  69,105,104,56,77,56,170,150,114,177,90,173,100,103,102,81,95,95,143,201,100,194,239,247,245,45,7,198,243,150,18,136,168,146,16,158,215,115,67,20,
  233,204,81,154,225,220,74,37,77,94,169,192,57,177,195,150,166,102,158,123,230,89,234,234,235,153,52,101,10,165,165,165,236,218,185,147,35,71,143,176,
  124,217,114,245,48,133,88,148,80,192,135,207,104,211,242,134,99,106,62,88,76,93,166,148,12,18,102,139,186,161,197,108,54,19,141,70,233,232,236,197,
  149,233,196,110,179,36,207,240,36,9,171,197,136,68,140,182,182,102,58,58,59,144,101,153,206,142,14,220,30,143,154,175,44,73,228,231,230,18,10,4,
  57,222,118,60,105,231,184,94,0,60,94,18,94,179,195,134,134,6,126,247,212,83,212,213,214,49,115,238,220,75,182,29,146,18,63,99,90,74,187,124,
  37,41,210,160,42,151,164,20,186,151,250,6,254,203,69,231,189,189,61,236,218,181,11,143,219,67,241,152,49,228,230,228,82,93,125,146,250,250,122,92,
  174,76,53,189,73,137,17,14,6,240,155,252,253,136,160,122,83,12,70,73,141,138,90,109,90,59,98,116,116,169,237,48,155,76,152,205,230,243,162,115,
  117,41,57,154,20,25,213,43,226,167,84,114,255,5,191,228,220,187,164,165,78,192,160,125,145,223,231,99,243,230,205,212,84,159,164,180,116,44,69,69,
  69,52,55,55,81,117,242,36,11,22,44,68,50,24,80,98,49,194,65,31,126,163,89,95,226,142,251,29,253,104,73,179,9,171,213,138,197,98,38,22,
  141,210,213,237,38,195,145,129,205,102,78,54,59,73,173,225,138,34,225,245,116,211,213,221,133,28,139,209,221,213,69,103,103,151,158,58,145,147,149,141,
  34,203,156,170,169,73,88,21,35,41,85,12,69,166,165,185,153,63,191,240,2,141,13,13,76,157,49,131,162,162,34,14,28,216,159,86,126,89,86,116,
  187,77,37,191,201,60,116,249,59,58,90,146,228,247,248,188,73,242,135,67,161,244,242,235,196,77,125,207,227,241,112,240,192,1,188,94,31,249,5,5,
  100,103,101,83,87,95,71,67,99,35,211,28,78,12,90,238,102,36,20,36,96,180,232,247,160,207,118,193,96,48,104,209,105,11,38,147,90,103,188,187,
  199,141,211,149,161,110,36,54,153,70,189,29,122,25,71,73,27,179,36,53,165,79,13,229,73,9,177,213,20,108,52,241,104,241,33,216,174,32,163,163,
  21,25,253,205,111,0,88,180,104,17,87,173,94,141,221,110,167,187,167,135,246,246,54,102,206,156,137,65,146,180,68,235,16,178,2,197,197,99,136,70,
  35,244,246,184,213,25,184,4,38,179,90,238,193,100,50,235,27,161,124,190,128,182,4,49,48,176,27,137,68,216,186,117,43,110,119,175,30,121,237,232,
  234,210,13,59,195,110,199,100,54,115,240,192,129,126,182,146,92,86,68,81,180,1,52,209,121,74,125,165,103,146,122,64,242,186,98,218,242,36,233,8,
  171,168,59,58,186,168,170,170,162,74,203,33,252,204,103,63,203,248,113,227,120,247,221,119,105,110,110,38,28,14,171,133,189,67,178,102,47,30,236,246,
  76,138,138,10,232,233,233,197,235,241,18,139,201,24,141,234,238,234,120,9,17,89,81,180,42,16,145,1,196,36,62,227,111,110,110,230,192,129,253,4,
  181,83,107,186,123,122,8,133,195,0,152,205,102,92,46,23,173,173,173,52,52,52,164,217,33,30,47,166,172,218,223,177,35,71,249,222,209,239,32,25,
  12,124,117,202,148,75,182,29,241,213,0,41,161,207,12,152,88,72,137,156,68,25,216,175,82,149,255,73,232,104,39,78,156,160,170,170,234,146,215,121,
  123,91,59,111,189,185,9,201,96,224,250,130,2,242,242,114,57,126,34,70,87,87,23,209,104,4,155,205,78,44,26,65,150,20,194,33,31,246,12,23,
  57,185,57,248,188,62,252,126,245,184,229,120,221,80,179,217,132,193,168,22,14,15,248,3,132,66,17,28,25,54,98,177,88,210,9,84,163,165,115,117,
  195,136,220,55,209,215,74,117,233,19,15,41,69,122,198,48,130,70,241,69,209,246,246,118,126,245,203,95,2,112,231,221,119,51,102,204,24,246,31,216,
  79,91,91,27,145,72,24,155,205,70,36,28,65,150,99,132,130,62,36,131,133,226,226,66,220,110,15,62,175,95,187,31,6,76,26,81,55,24,77,200,
  138,76,32,16,36,28,142,96,177,154,6,228,250,43,178,76,91,123,27,59,118,238,80,211,203,162,81,220,110,55,62,45,216,97,54,155,112,58,157,244,
  246,244,208,217,209,65,170,91,22,39,70,39,79,86,81,243,120,53,6,201,192,167,38,76,96,204,152,49,236,222,179,59,73,254,112,40,164,70,253,130,
  62,108,54,39,249,249,185,186,252,178,44,39,203,111,48,38,201,223,159,140,38,202,127,236,216,177,36,249,35,225,72,146,252,157,157,157,180,182,180,164,
  150,95,159,120,168,237,232,234,234,98,231,246,29,96,144,88,126,213,10,178,115,178,169,57,117,138,222,158,30,98,209,8,38,171,141,88,76,221,208,22,
  14,5,176,217,29,100,101,103,227,247,245,29,21,110,48,74,90,153,165,120,59,20,130,193,16,161,80,20,187,77,37,225,134,4,219,29,141,118,232,91,
  145,227,68,84,171,163,75,98,16,73,74,147,90,52,196,40,85,218,141,206,98,153,254,220,161,184,164,132,229,87,93,197,184,177,99,153,88,86,70,36,
  28,161,181,165,149,246,182,54,117,70,174,40,196,98,17,198,148,20,113,245,218,85,92,127,253,53,148,77,28,143,197,98,198,20,119,6,22,147,90,255,
  78,82,107,175,5,3,65,194,225,88,202,223,59,126,252,56,219,182,109,215,55,47,181,183,183,169,53,205,0,139,217,76,94,78,46,70,163,129,210,210,
  210,100,255,167,36,135,209,19,147,159,227,75,243,137,75,130,250,67,81,134,183,212,174,40,67,13,232,9,156,99,220,120,211,77,148,148,148,48,105,210,
  36,28,25,25,244,246,244,114,186,190,62,201,121,134,195,65,174,90,177,148,235,174,187,154,229,203,43,201,204,202,196,96,144,244,129,221,108,54,105,245,
  225,20,181,202,131,63,152,242,183,60,30,15,59,119,238,162,190,190,94,59,91,220,71,175,219,173,71,252,115,179,178,113,58,28,100,231,100,171,196,104,
  144,128,111,162,29,2,220,113,231,29,151,69,59,148,115,189,76,149,226,123,46,23,157,207,154,53,139,204,204,76,10,11,10,177,90,173,248,124,62,218,
  90,219,18,82,136,20,34,145,16,21,243,231,113,213,242,165,204,159,63,15,151,203,161,159,174,100,210,234,104,26,36,117,130,29,14,135,241,251,67,218,
  57,245,177,243,211,14,69,37,15,137,203,157,113,71,172,36,229,18,143,208,30,250,93,182,104,241,34,202,202,202,152,48,126,60,121,121,121,120,60,30,
  154,154,154,212,21,55,131,58,185,137,68,66,204,159,63,143,21,43,150,179,112,65,5,46,151,19,131,65,194,168,233,222,100,50,233,103,162,71,34,97,
  252,129,16,114,108,160,124,94,159,143,253,251,247,115,252,248,113,20,69,33,16,8,224,241,122,136,106,247,54,203,149,73,134,221,78,94,94,30,86,155,
  45,133,217,42,244,119,46,215,221,112,61,133,133,133,140,31,55,142,12,187,61,73,254,184,237,70,34,33,22,45,90,48,168,252,6,45,184,18,151,63,
  21,226,242,55,54,53,38,201,47,43,114,159,252,25,118,92,153,174,51,200,175,244,29,152,166,233,118,234,212,169,184,92,78,242,115,243,176,88,44,4,
  252,1,58,58,58,245,210,139,234,17,159,33,102,205,42,103,225,194,249,204,158,61,3,135,35,67,59,162,214,132,209,108,210,107,136,42,138,106,187,129,
  64,220,118,229,243,214,142,164,245,249,126,41,38,169,108,87,57,11,219,21,100,116,216,61,255,204,143,207,126,238,179,188,239,190,123,177,89,173,204,153,
  61,7,147,217,76,123,71,7,45,45,45,100,100,100,32,43,10,227,198,143,231,234,171,215,48,111,222,108,230,207,155,203,164,73,19,177,219,109,122,225,
  89,179,81,157,209,199,103,237,161,144,186,211,181,255,209,161,85,85,85,188,250,234,171,180,182,182,232,134,94,91,91,167,205,86,36,28,25,25,20,23,
  22,224,200,200,96,246,156,217,100,102,186,82,204,79,18,147,148,18,243,68,250,253,59,140,135,50,224,152,172,20,159,81,174,228,99,66,207,118,157,254,
  204,143,169,83,167,240,213,175,253,43,229,51,203,41,27,63,129,194,194,66,2,161,16,181,181,181,234,153,199,154,195,92,177,98,37,107,86,175,96,86,
  249,12,230,206,153,77,81,97,129,126,178,151,41,161,30,163,162,149,255,241,251,3,68,251,57,39,175,215,203,246,237,219,121,247,221,119,244,186,128,61,
  61,61,244,104,165,197,76,70,35,69,154,29,150,150,150,50,163,124,70,178,237,41,253,108,37,193,14,167,151,207,224,219,223,253,46,179,102,207,186,164,
  219,65,191,99,116,83,245,43,229,140,54,51,72,159,83,228,203,70,231,249,249,249,92,127,195,13,20,23,23,145,159,151,71,86,102,22,225,112,132,150,
  214,22,76,218,185,234,138,162,80,81,49,159,69,11,231,51,121,210,68,166,79,159,74,94,190,90,31,209,104,84,39,243,113,135,142,162,16,139,183,35,
  42,39,145,209,209,108,135,18,223,160,162,127,94,73,211,151,211,123,233,193,70,141,68,123,153,48,97,60,255,244,133,47,48,105,242,36,242,242,114,25,
  55,118,44,193,80,136,134,134,6,173,126,180,186,129,102,209,162,197,172,90,181,156,41,147,39,105,247,44,7,201,96,80,151,128,141,70,189,144,189,162,
  40,68,162,17,2,126,53,186,152,152,11,235,245,122,217,179,103,15,187,223,123,79,175,163,218,219,219,139,219,237,209,82,205,140,228,231,229,98,49,155,
  201,207,207,103,206,156,57,103,76,219,154,52,121,18,95,120,228,17,166,78,85,83,76,242,242,242,146,228,143,219,238,162,69,139,169,92,178,48,73,126,
  195,25,228,239,127,116,104,162,252,170,206,83,203,159,97,179,83,84,88,196,164,73,19,207,40,127,252,191,156,156,92,174,186,234,42,242,242,243,201,201,
  201,193,229,114,18,142,70,105,111,111,199,100,232,179,221,242,153,179,152,61,123,38,19,198,141,101,226,196,50,178,115,178,48,24,13,24,141,6,181,166,
  119,188,29,168,121,206,254,64,64,221,180,21,79,251,24,229,118,36,46,191,247,207,97,87,250,219,238,128,164,209,225,217,238,149,234,207,71,125,55,253,
  115,207,62,203,99,143,62,74,48,16,224,182,187,238,162,234,100,21,251,247,239,167,165,165,133,121,243,230,81,49,127,62,215,95,127,61,227,198,142,197,
  104,52,210,221,211,203,233,211,141,132,195,17,109,118,103,194,164,69,38,36,36,53,87,38,28,198,239,15,146,145,97,197,106,81,143,173,219,190,125,59,
  175,188,242,10,117,117,245,106,121,39,69,38,224,247,211,169,45,209,103,216,51,40,27,55,30,9,137,205,155,222,162,180,180,20,183,219,173,25,130,148,
  112,184,75,95,92,189,255,89,47,103,147,23,42,112,225,151,234,191,243,237,111,179,254,229,151,185,237,142,59,89,185,98,133,90,242,165,165,133,112,40,
  132,201,100,226,190,15,124,128,133,11,22,96,181,88,8,71,34,180,181,119,208,210,210,166,219,161,217,220,119,202,23,90,29,220,96,32,128,223,31,38,
  211,101,5,160,177,177,145,215,94,123,141,119,222,125,87,43,141,163,16,9,135,233,117,187,241,7,2,72,146,68,105,201,24,74,138,138,56,121,252,4,
  71,15,29,198,110,183,167,76,228,136,155,99,98,69,134,99,71,143,242,175,255,242,21,254,250,226,95,248,192,135,62,124,201,182,99,64,141,80,6,86,
  73,57,219,78,119,185,232,188,189,163,157,87,54,110,224,200,145,35,44,90,188,152,242,25,51,240,250,124,116,117,169,103,157,27,140,70,110,88,119,35,
  115,102,207,34,195,110,67,150,99,116,117,117,211,222,222,137,209,104,194,108,54,234,164,90,50,104,164,58,22,33,24,12,16,8,132,177,152,236,250,210,
  252,232,234,92,155,116,39,164,128,14,125,25,115,120,102,80,87,87,199,79,127,252,99,142,28,57,194,244,233,51,88,180,96,33,199,79,156,160,189,173,
  157,112,36,132,209,104,228,142,59,239,98,246,236,217,250,9,75,157,250,61,51,106,167,238,24,49,26,13,154,238,33,22,141,234,247,204,110,87,211,52,
  2,126,63,27,55,108,96,247,238,61,120,181,205,94,145,72,68,221,92,20,12,34,33,81,92,80,72,65,126,1,141,13,13,212,213,214,98,179,217,6,
  180,71,93,38,238,187,41,213,39,171,249,241,15,126,192,235,175,190,202,173,183,223,193,226,133,139,232,233,237,77,146,255,150,91,111,27,146,252,241,218,
  104,137,242,59,157,170,237,182,52,183,240,206,59,91,211,203,47,245,201,95,87,91,75,245,201,147,67,144,191,143,151,116,119,117,177,101,203,22,170,171,
  171,153,53,107,54,147,39,77,198,239,15,208,219,211,67,36,170,86,58,88,181,122,13,211,167,77,195,102,181,16,147,101,122,122,122,233,234,236,81,15,
  147,49,154,180,195,102,180,118,200,32,199,162,132,130,33,2,129,48,38,167,13,89,150,105,107,109,27,213,118,36,229,244,141,178,237,138,200,232,40,97,
  239,158,61,180,52,55,211,209,209,65,123,107,43,75,151,44,37,39,39,135,186,211,167,233,233,233,97,218,212,105,20,23,21,99,52,26,233,233,237,229,
  47,47,254,133,99,71,14,18,10,5,244,147,143,140,70,109,153,76,82,203,131,200,177,24,193,80,152,112,56,70,103,103,39,235,215,175,231,133,23,254,
  76,109,93,29,145,72,68,159,201,52,52,52,160,0,38,147,137,188,156,108,74,138,139,244,37,186,70,173,0,246,80,98,231,138,50,250,33,117,177,171,
  126,244,177,97,253,122,20,69,225,173,77,111,178,116,201,82,166,77,157,74,40,28,102,231,123,187,152,56,113,34,51,203,103,98,181,88,136,68,34,28,
  61,118,140,23,255,242,103,122,186,59,144,36,52,91,84,31,146,182,81,67,145,227,179,116,53,127,171,170,170,138,255,249,245,175,121,231,157,119,116,103,
  30,175,13,25,159,248,56,236,25,148,22,23,19,14,133,113,187,221,4,2,1,186,180,9,211,153,163,191,234,63,127,253,203,159,65,129,87,55,110,184,
  164,219,145,180,165,58,229,26,239,217,173,107,41,138,114,217,232,252,200,225,195,160,40,28,61,114,132,169,83,167,82,58,102,140,42,243,241,99,140,41,
  41,97,98,89,25,22,179,90,30,175,186,166,134,87,95,125,5,119,111,183,90,4,223,160,69,119,245,200,168,90,238,38,26,137,17,8,132,144,21,133,
  227,199,143,159,151,118,40,202,200,7,80,69,25,108,81,36,249,208,17,128,221,239,189,135,207,231,227,192,254,253,76,159,62,157,73,101,101,132,34,17,
  14,28,56,136,213,106,101,202,228,201,88,204,106,121,172,147,213,213,188,178,113,3,238,158,46,80,100,12,6,163,246,48,104,186,151,145,181,26,164,193,
  80,88,219,136,116,138,223,62,249,20,239,189,183,91,39,64,241,8,157,199,235,85,83,195,44,102,138,10,10,48,0,62,175,183,239,240,21,69,233,23,
  69,238,31,87,82,120,245,149,87,80,20,216,178,121,51,21,21,21,73,242,143,29,59,86,151,63,18,137,36,203,15,253,228,39,73,254,64,176,79,254,
  63,253,233,79,131,202,159,97,179,81,84,80,64,52,28,30,134,252,201,175,157,60,121,18,20,168,169,174,102,98,89,25,69,133,133,68,98,49,170,171,
  171,41,44,44,100,108,105,169,190,57,169,190,190,142,45,91,222,198,235,233,81,207,77,136,159,134,164,5,163,100,237,212,198,120,59,100,89,161,166,186,
  250,188,180,67,73,49,56,41,41,135,37,229,172,109,247,74,132,105,68,163,193,48,111,154,162,40,124,224,131,31,228,246,59,238,96,251,142,157,212,213,
  213,178,115,231,78,173,72,125,152,206,206,78,156,46,39,127,254,203,95,216,179,123,55,161,96,72,221,237,26,13,99,179,22,227,247,123,201,176,219,245,
  101,178,238,158,46,218,218,155,113,187,123,104,110,106,164,190,190,158,238,158,110,61,79,52,254,185,234,83,167,0,200,116,58,25,83,84,140,203,233,100,
  206,220,57,28,62,116,24,175,199,51,248,220,69,233,55,197,215,139,134,198,239,129,148,50,138,163,71,83,211,110,184,72,156,117,13,200,94,79,191,41,
  227,114,198,8,237,106,36,185,134,101,19,39,242,240,103,63,139,217,108,102,213,202,85,116,118,118,210,217,209,65,110,94,30,167,235,235,176,78,153,202,
  137,147,85,60,255,220,115,52,55,55,99,144,84,59,148,136,97,52,170,213,29,28,142,12,100,69,198,31,8,224,107,60,77,77,245,81,218,59,58,168,
  171,59,197,233,250,211,234,137,44,9,167,172,116,116,117,209,221,221,141,201,100,162,164,168,144,156,236,108,242,243,243,144,99,49,106,170,171,83,155,96,
  255,130,202,74,114,184,105,242,212,201,252,243,87,190,130,205,102,187,164,219,145,244,111,255,190,151,184,19,117,176,254,145,118,103,160,250,249,203,69,231,
  121,5,249,172,91,183,14,155,61,131,89,51,103,226,241,122,112,247,186,201,204,202,162,181,85,77,123,106,108,106,228,245,215,95,167,179,163,83,61,165,
  41,22,197,32,201,152,77,6,194,38,35,138,195,161,174,26,5,3,4,154,27,168,171,59,65,103,103,39,167,79,215,211,212,212,52,186,237,232,63,176,
  234,250,79,208,177,52,216,92,35,229,15,165,212,121,28,185,57,57,124,232,195,31,38,47,63,159,202,197,149,116,247,244,208,221,221,141,217,98,161,185,
  185,9,139,165,12,95,192,207,250,151,215,211,222,222,142,36,73,68,162,97,12,70,48,26,37,140,6,245,72,72,89,86,8,134,130,4,90,155,57,125,
  250,20,155,186,58,104,108,104,160,185,185,57,233,158,41,138,66,119,79,15,189,110,55,38,163,145,156,172,44,178,92,46,114,243,114,137,70,163,212,106,
  62,105,208,118,37,16,149,9,101,19,248,135,79,253,63,44,86,107,146,252,57,57,57,186,252,167,234,106,147,228,143,198,162,32,197,116,249,51,50,50,
  146,228,175,175,175,166,187,187,139,198,134,6,154,154,155,213,147,139,210,200,95,144,151,175,203,175,40,50,167,235,79,15,45,246,151,148,112,41,145,157,
  155,205,138,21,43,176,90,109,76,157,58,21,159,207,135,215,235,197,233,114,209,209,209,142,221,110,163,165,181,149,119,222,121,135,158,238,110,36,131,81,
  59,97,76,198,100,146,48,26,141,100,216,213,244,130,80,40,72,91,91,11,141,13,167,232,233,237,161,185,169,137,214,182,182,243,210,142,129,99,215,96,
  182,155,226,156,250,97,216,238,149,230,207,77,35,222,60,48,140,235,20,89,102,201,210,165,84,157,56,193,198,151,95,98,237,154,181,120,189,94,94,127,
  243,77,78,159,110,96,225,194,133,100,102,101,82,93,93,77,32,16,0,64,142,198,136,68,163,52,54,214,163,200,208,210,212,128,209,100,64,146,12,26,
  217,140,16,14,133,180,211,47,162,250,17,127,161,96,80,45,174,223,216,72,32,24,196,98,177,48,110,76,41,14,187,157,186,218,90,166,76,157,138,221,
  110,235,35,163,186,77,36,158,154,144,214,187,37,144,80,101,96,137,39,73,63,204,46,233,156,101,148,20,255,66,242,115,129,243,66,72,11,11,10,40,
  44,44,228,247,191,251,29,118,171,149,53,171,87,243,214,230,205,252,244,167,63,99,217,178,165,204,157,51,151,166,230,38,125,163,64,76,137,17,13,69,
  137,118,70,232,233,233,194,32,25,169,211,150,110,1,100,89,61,241,36,16,8,16,10,134,244,99,38,163,209,40,238,222,94,106,235,234,104,239,232,64,
  86,20,242,114,114,24,59,166,148,214,150,22,204,102,19,5,133,5,212,84,159,164,223,214,241,212,19,163,126,47,21,23,23,83,82,92,204,19,191,249,
  13,86,155,253,146,109,199,144,22,178,18,206,125,78,169,111,105,144,96,155,162,92,210,58,55,153,77,24,36,245,116,186,76,151,139,204,172,44,182,111,
  219,134,213,102,99,238,156,57,28,60,120,144,103,158,121,150,185,115,231,48,109,218,116,186,186,58,105,239,232,208,142,47,150,145,195,65,98,61,49,60,
  30,55,70,131,129,134,134,58,109,51,168,164,157,136,20,33,16,8,18,10,133,250,150,240,207,81,59,172,86,171,90,219,113,192,209,136,131,68,186,7,
  76,68,250,57,106,69,25,84,241,253,207,180,207,205,205,101,234,212,169,252,253,111,127,67,2,150,84,86,178,125,199,14,254,246,183,191,209,220,220,204,
  140,233,51,232,117,247,210,214,214,170,255,140,122,236,105,23,158,222,94,36,173,180,144,193,104,68,2,173,4,152,122,207,34,225,176,174,251,88,52,138,
  215,235,165,181,181,149,214,182,118,20,69,33,59,51,147,226,194,34,58,59,59,49,155,77,228,230,230,208,112,250,180,94,219,53,101,190,74,191,123,152,
  95,144,79,126,126,62,207,63,251,28,22,155,85,151,255,55,79,252,150,249,243,43,152,49,125,6,173,237,109,73,242,135,66,106,13,206,184,252,141,141,
  241,52,55,180,218,178,97,130,193,80,74,249,27,26,27,233,234,234,214,229,47,41,42,212,229,207,201,201,29,72,226,82,201,175,249,72,117,243,148,129,
  104,44,134,35,195,129,195,233,228,192,254,253,88,205,22,166,207,152,206,137,227,39,120,249,165,151,153,49,99,58,19,39,78,162,167,167,155,174,238,110,
  237,184,80,153,112,36,68,204,29,195,231,243,96,52,24,105,105,49,171,105,19,90,132,84,142,69,8,134,66,132,67,97,221,118,207,85,59,172,54,43,
  225,184,237,246,63,222,83,107,95,114,2,95,234,73,180,210,159,7,12,195,118,175,52,127,110,58,95,50,125,231,219,223,166,173,181,21,69,81,200,207,
  47,224,190,123,239,227,137,223,254,150,237,59,118,176,101,203,22,162,177,40,69,133,133,76,158,50,133,252,252,124,61,249,218,239,83,107,231,249,3,190,
  190,130,180,154,18,227,103,222,250,253,126,78,215,215,211,216,216,136,215,239,39,166,237,250,180,219,108,76,46,155,200,216,49,99,56,114,248,144,90,39,
  176,163,131,96,32,144,98,180,75,239,12,251,74,46,72,3,163,53,231,82,111,34,183,100,212,177,125,219,54,106,106,106,104,107,105,193,102,207,224,83,
  159,249,12,78,135,147,39,159,126,138,103,159,121,150,223,253,238,119,216,172,86,38,148,149,81,86,86,70,70,134,186,171,83,142,245,29,139,24,31,192,
  19,119,10,203,178,76,36,18,161,189,173,141,211,167,79,211,214,209,65,36,18,69,65,193,108,50,81,92,80,200,228,137,101,248,60,110,90,91,91,104,
  109,109,73,200,183,27,174,157,40,108,221,178,133,234,147,39,105,105,109,197,145,225,224,243,143,60,114,201,181,99,40,125,47,253,37,210,101,175,115,147,
  217,68,229,146,165,156,56,126,140,174,174,110,106,170,107,248,195,239,126,135,219,237,193,102,179,114,203,109,183,227,112,56,120,229,213,87,121,253,245,55,
  88,191,97,3,22,139,153,210,210,82,198,150,142,195,158,145,129,193,96,64,81,194,68,194,97,213,201,251,251,78,76,138,159,189,41,43,10,209,72,148,
  142,206,14,26,206,81,59,10,11,11,41,44,44,226,176,126,228,178,54,146,74,103,82,247,185,29,1,79,156,56,193,35,95,252,34,254,64,0,171,213,
  202,125,239,255,0,25,246,12,222,216,244,38,235,95,94,207,139,47,190,8,218,134,161,210,210,177,216,237,118,117,111,130,44,19,81,194,122,127,139,215,
  239,140,235,62,49,39,177,174,182,142,174,238,46,34,209,24,10,10,38,147,137,252,220,92,198,143,29,139,197,104,162,174,182,150,174,206,78,253,216,217,
  225,224,189,93,239,81,95,251,21,58,58,59,176,219,51,120,240,147,159,32,195,158,193,11,127,249,179,46,191,213,98,161,116,108,105,159,252,82,178,252,
  169,108,55,46,127,87,103,23,205,205,205,105,229,15,248,253,116,117,118,234,242,15,203,118,43,151,112,226,196,113,186,186,186,56,221,112,154,151,254,246,
  119,124,62,31,22,171,133,53,107,214,146,97,183,179,101,235,86,222,121,231,93,222,218,188,25,179,217,76,81,81,17,197,197,197,216,108,118,205,118,35,
  250,121,242,74,64,73,105,187,177,104,140,174,238,46,90,154,91,206,73,59,28,14,7,179,231,204,97,231,142,29,41,204,81,58,111,182,43,150,233,135,
  72,154,134,117,134,184,162,208,218,170,206,220,50,28,14,110,189,253,118,178,179,179,232,236,188,13,147,201,196,222,253,251,8,7,35,116,118,118,210,209,
  209,137,201,100,98,252,248,113,76,154,60,153,140,140,140,190,14,20,39,162,138,122,140,104,87,119,23,141,13,141,180,182,182,18,8,5,147,38,39,78,
  135,131,73,101,101,76,153,56,145,236,172,44,58,58,138,104,106,108,212,34,175,82,250,125,18,82,194,134,38,137,126,51,160,212,185,204,67,113,181,202,
  25,2,56,18,87,110,117,7,229,60,93,27,31,140,227,182,184,100,233,82,42,43,23,147,155,155,75,119,79,15,235,55,172,167,215,235,65,145,101,106,
  170,171,169,62,89,77,102,166,139,242,242,114,114,114,115,245,226,224,73,3,187,172,224,246,184,105,107,109,165,182,182,142,94,119,47,114,130,62,45,22,
  11,69,5,5,76,46,43,163,180,164,68,37,47,237,237,106,206,83,127,91,148,210,44,230,36,217,97,223,187,205,205,45,72,6,137,229,43,86,176,116,
  233,18,242,243,243,47,169,118,244,207,116,81,134,212,39,134,55,224,95,170,58,47,40,200,103,202,148,169,68,35,145,190,248,139,4,189,218,41,60,211,
  166,207,96,198,140,25,228,230,230,226,247,7,216,182,125,27,94,191,58,97,63,93,223,192,233,186,211,56,157,78,166,76,157,74,118,78,54,102,139,133,
  196,26,175,106,59,192,235,245,208,222,214,70,93,253,233,17,181,35,211,229,194,149,153,73,83,83,163,222,142,153,179,102,233,199,136,38,142,161,72,156,
  225,232,67,105,104,58,78,218,241,54,208,46,148,68,221,43,232,171,109,115,231,206,99,206,156,57,250,145,176,47,254,245,175,120,252,62,76,146,129,250,
  186,211,212,215,214,227,116,58,25,55,97,60,5,249,5,152,180,130,234,253,117,31,8,6,104,107,109,165,177,177,73,45,27,148,112,207,140,6,3,5,
  121,121,140,47,45,165,176,64,253,142,142,182,54,124,62,31,161,96,48,57,235,164,95,59,147,226,28,122,29,75,117,3,155,36,25,88,176,112,33,243,
  230,206,37,39,59,135,94,183,155,205,155,55,227,241,251,80,100,37,73,254,73,83,38,147,157,149,157,86,126,175,207,75,103,71,71,74,249,205,102,179,
  74,224,74,75,41,42,44,36,26,141,210,211,213,77,52,26,29,84,254,196,103,249,113,219,141,70,147,62,227,245,121,145,36,137,137,101,19,153,56,105,
  34,153,89,89,4,2,65,246,238,219,139,63,24,64,81,20,90,154,155,105,110,106,198,145,145,193,248,178,9,100,102,101,97,54,153,245,210,96,250,111,
  203,224,243,249,232,234,236,164,169,185,121,196,237,152,94,94,206,241,163,199,116,25,199,141,27,71,86,86,150,58,105,144,250,121,111,41,49,28,32,13,
  56,138,24,6,22,181,151,164,145,217,238,149,232,207,141,19,203,202,190,49,212,15,23,23,23,115,195,186,117,108,88,191,158,150,230,230,33,255,72,188,
  4,147,195,225,224,63,190,253,109,202,202,38,242,191,191,254,31,26,27,78,179,108,217,50,36,131,129,238,238,30,122,61,110,34,209,8,178,150,124,92,
  123,234,148,122,220,167,197,162,46,145,73,16,141,70,233,236,234,228,232,145,35,156,56,121,146,142,174,46,162,177,168,126,3,12,6,3,217,153,153,204,
  157,57,147,252,156,92,142,31,57,74,94,94,62,227,198,143,35,24,8,224,241,120,147,141,72,234,23,253,148,250,254,146,180,99,3,37,131,164,62,55,
  168,71,177,25,244,215,250,30,72,146,118,84,27,9,127,147,242,223,62,67,237,215,153,227,223,117,133,161,164,164,132,155,110,190,153,87,54,110,76,42,
  68,60,24,110,88,183,78,223,160,50,18,114,114,245,53,215,240,208,195,159,229,212,169,83,252,232,7,223,231,170,171,174,98,252,132,241,116,117,117,209,
  217,213,77,48,28,86,207,71,142,70,105,109,105,161,167,167,91,219,153,108,214,102,236,10,126,191,159,134,198,6,246,239,219,71,93,67,3,193,80,40,
  73,183,86,139,133,201,19,38,48,117,210,100,186,58,58,112,247,186,25,55,126,28,249,249,249,180,180,180,32,203,74,10,91,148,250,5,223,53,123,147,
  12,24,116,155,235,139,50,93,191,110,29,95,124,228,17,170,171,107,248,206,183,190,197,202,149,43,153,80,54,225,146,104,71,252,239,254,125,105,64,191,
  74,234,67,82,18,169,73,236,83,3,250,23,201,71,137,94,74,58,87,20,133,250,250,122,28,78,7,193,96,144,64,32,160,183,111,238,220,121,220,118,
  251,237,180,119,180,243,236,51,207,48,111,222,60,74,198,148,224,118,187,233,233,237,37,20,81,11,186,71,163,49,218,219,219,244,188,57,147,217,140,209,
  160,238,89,245,251,3,52,53,53,114,240,224,65,234,27,27,71,220,14,171,213,138,213,106,197,227,241,232,237,112,123,220,180,183,183,83,88,88,68,71,
  123,187,174,243,172,236,108,230,206,157,203,177,99,199,240,184,61,253,244,158,96,7,244,217,249,77,55,223,12,192,203,47,189,212,55,134,38,6,11,210,
  232,61,254,119,188,191,87,46,89,194,7,63,248,33,26,27,27,120,242,137,39,212,77,96,99,75,233,237,233,197,237,118,19,138,132,245,123,214,221,213,
  141,219,221,139,193,104,196,104,50,234,186,15,4,2,180,182,182,82,125,242,36,245,141,141,250,33,0,113,137,44,102,11,69,5,249,76,153,52,137,96,
  32,72,75,83,35,133,69,69,20,22,21,209,211,173,86,62,24,176,178,150,232,127,244,126,173,221,15,125,2,39,177,98,213,74,62,246,137,79,112,250,
  244,105,254,251,191,30,103,209,162,69,140,41,29,67,111,79,47,61,189,61,73,58,239,232,104,199,237,118,167,149,255,232,145,35,52,182,180,164,148,127,
  66,105,41,101,227,199,227,238,233,197,231,245,80,92,82,66,118,78,54,157,29,234,73,92,169,229,151,146,206,101,39,110,187,14,7,193,80,80,173,196,
  160,217,213,180,233,211,89,179,118,45,221,93,221,188,186,113,35,211,167,79,167,176,176,0,175,215,139,219,227,37,28,141,34,203,50,209,88,148,238,174,
  46,188,30,15,6,147,90,192,63,94,222,49,24,12,210,214,218,198,241,227,199,104,110,109,61,171,118,20,20,20,208,209,209,161,203,30,14,133,104,110,
  105,38,63,63,143,246,182,246,36,251,52,72,125,227,22,253,109,87,27,147,84,61,74,186,222,164,254,254,127,24,182,123,37,249,243,243,70,70,21,69,
  193,233,112,240,159,223,249,14,101,19,39,242,139,199,30,227,173,77,111,210,210,220,66,71,71,59,215,94,115,13,115,231,206,33,26,83,143,3,13,71,
  34,4,130,65,100,69,161,167,187,135,186,218,90,130,193,32,221,93,93,84,87,87,115,252,196,9,186,122,122,244,130,194,241,147,149,242,115,115,41,159,
  58,141,138,217,115,8,5,2,156,56,118,140,104,52,74,75,75,51,249,5,5,140,31,63,158,96,48,128,71,171,59,150,216,177,250,59,3,73,146,52,
  167,105,72,34,159,134,65,156,103,220,224,134,66,70,211,25,161,32,163,163,79,70,175,185,230,26,30,254,220,231,56,117,170,134,111,252,219,191,209,211,
  219,203,129,125,123,41,155,80,198,173,183,222,134,195,145,65,52,26,213,118,206,134,84,123,244,7,105,109,105,166,189,189,157,64,32,64,99,99,35,199,
  142,29,163,174,190,158,160,54,24,74,146,132,217,98,33,211,233,98,236,152,82,230,205,154,77,105,201,24,78,86,85,209,219,211,131,199,227,33,16,8,
  50,126,194,120,10,242,243,105,109,105,209,11,137,15,58,41,50,24,116,91,148,12,125,132,108,221,186,27,121,228,203,95,166,186,250,36,95,249,231,71,
  232,233,233,97,207,158,221,76,44,43,227,182,219,239,184,248,219,145,212,151,12,41,250,20,3,251,21,125,3,253,0,50,42,165,137,52,93,130,58,143,
  201,50,6,131,68,126,65,161,74,70,253,106,116,111,206,220,185,220,122,219,173,180,183,183,243,135,167,159,198,231,247,113,178,170,138,49,99,198,176,102,
  245,26,50,50,50,84,66,29,139,17,14,71,8,71,34,186,243,238,236,232,36,24,12,208,212,220,76,85,85,21,167,27,26,244,118,48,204,118,184,221,
  110,202,38,78,36,39,55,23,151,203,69,110,110,46,146,36,225,247,251,181,186,165,18,69,69,69,180,183,183,235,58,207,206,202,102,238,188,121,28,61,
  122,20,143,199,157,236,232,13,253,39,31,103,65,70,181,239,137,23,46,95,178,116,41,31,252,240,135,105,104,104,224,241,159,255,2,143,215,195,201,170,
  19,20,228,229,179,114,229,74,138,139,139,212,186,149,218,61,11,69,194,132,130,65,58,218,59,232,234,234,194,239,247,209,222,214,70,77,77,13,141,205,
  205,248,180,72,171,132,186,177,198,229,116,82,92,84,68,249,212,169,148,150,140,161,169,177,145,238,174,78,34,145,8,61,221,221,20,21,21,171,132,180,
  71,35,164,73,231,222,38,183,71,66,82,251,73,194,164,115,229,234,85,124,226,147,255,64,125,125,29,63,254,193,15,113,123,220,28,61,124,132,177,165,
  99,185,230,154,107,176,219,237,68,99,209,1,58,143,203,31,212,34,185,113,249,67,218,33,48,18,18,38,179,25,151,195,161,203,95,84,80,200,233,250,
  58,188,30,183,26,205,13,133,40,41,25,67,78,110,14,29,113,34,119,6,249,99,178,140,65,146,200,47,44,32,24,8,234,81,244,105,211,166,178,122,
  205,26,186,187,186,121,249,239,127,39,16,8,208,112,186,158,194,194,34,42,43,43,177,219,108,200,49,153,88,76,38,162,245,197,80,40,68,103,167,186,
  137,46,28,10,209,214,214,198,169,218,90,154,91,90,244,118,48,130,118,148,142,29,199,152,49,165,56,50,28,100,231,228,80,80,88,72,71,123,187,106,
  187,18,20,21,151,168,182,27,247,253,6,41,41,56,197,0,146,154,232,191,7,9,74,13,213,118,175,64,50,122,222,114,70,29,78,39,133,69,69,252,
  252,177,199,216,180,233,77,93,19,173,205,205,252,238,201,39,89,92,185,132,59,111,187,141,186,186,122,14,31,61,66,253,233,211,106,71,10,133,48,96,
  160,254,116,3,225,112,136,152,172,158,141,172,70,75,77,88,45,22,50,50,236,148,150,148,48,118,204,24,66,193,16,71,143,28,193,235,245,232,161,145,
  104,36,202,123,59,119,81,185,164,146,220,220,60,26,27,26,7,39,162,154,125,12,32,153,233,206,81,62,211,50,162,36,114,73,46,38,76,156,52,137,
  218,218,90,190,254,181,175,225,211,114,146,1,54,109,122,147,189,123,247,112,245,53,215,50,171,124,38,135,143,30,225,196,137,42,154,91,154,117,39,235,
  241,250,232,210,162,28,10,104,71,70,170,231,38,219,237,118,109,163,199,24,114,178,179,105,105,106,230,248,209,35,73,191,221,212,216,136,36,65,249,204,
  153,216,237,246,164,104,137,148,198,118,18,35,161,137,54,56,101,234,20,106,170,171,249,242,151,190,132,207,231,211,237,247,245,87,95,99,207,123,187,185,
  110,221,186,75,162,29,241,85,201,164,113,88,234,223,151,164,1,221,235,140,81,132,132,77,3,23,187,206,11,10,11,41,44,44,68,146,36,106,107,107,
  9,4,252,125,81,178,120,52,24,137,162,162,98,218,218,218,249,191,63,252,158,144,118,100,178,4,236,222,245,30,39,78,156,96,233,210,101,76,155,58,
  149,147,53,213,156,170,173,213,136,116,144,88,44,134,215,231,163,167,183,71,111,135,65,219,41,110,54,153,176,217,108,228,229,230,14,185,29,40,10,199,
  143,29,195,149,153,73,102,102,38,77,141,141,3,111,191,86,86,72,215,185,65,210,54,2,245,41,185,47,210,159,108,11,3,244,159,184,137,173,95,178,
  148,34,73,131,140,190,10,165,165,165,52,54,52,240,248,99,143,233,17,102,5,216,191,127,63,53,53,53,204,157,55,143,219,111,185,149,170,147,39,57,
  85,91,75,103,87,39,62,159,15,57,38,227,247,251,113,123,60,68,34,97,253,158,89,44,22,204,38,19,86,139,149,140,12,59,19,74,199,146,153,153,
  73,71,123,59,13,245,245,90,94,168,42,137,207,231,227,208,193,131,148,207,154,153,216,186,244,81,69,125,18,214,103,219,227,39,76,160,190,190,158,31,
  126,255,251,154,252,234,235,219,223,121,135,163,135,15,177,236,170,21,76,153,50,133,147,39,171,56,85,91,71,123,123,155,26,204,209,228,239,117,247,106,
  75,230,154,206,45,22,76,70,245,220,250,156,236,44,138,11,10,117,249,107,181,243,218,227,114,182,105,233,45,147,38,79,198,106,181,17,141,250,6,200,
  95,152,194,118,213,54,104,53,90,181,38,231,230,231,211,213,217,197,250,151,95,34,28,14,235,237,56,114,248,16,117,181,181,204,157,55,143,137,101,101,
  212,213,215,211,208,216,72,103,119,23,161,96,136,152,28,35,16,8,82,235,174,87,139,220,39,232,65,61,183,222,70,118,86,22,37,133,67,111,71,188,
  196,99,249,204,153,28,61,122,100,128,30,226,69,246,117,146,169,143,91,36,219,110,114,146,223,192,177,107,192,206,250,225,216,238,149,5,233,234,213,171,
  135,188,196,63,175,162,130,159,252,236,103,60,252,233,135,216,187,119,207,144,127,36,22,141,145,120,114,65,188,214,92,226,191,241,188,150,89,179,103,51,
  99,102,57,70,163,145,19,39,170,104,106,105,166,171,171,11,183,219,67,44,26,69,150,99,24,205,102,28,25,25,56,51,28,228,100,101,97,179,90,113,
  247,246,210,209,222,78,119,119,119,130,13,40,233,147,209,82,185,205,84,81,81,67,114,212,38,241,111,250,69,112,6,204,138,18,156,72,223,42,140,136,
  140,246,199,252,249,11,248,249,127,61,206,23,254,233,159,216,191,111,223,144,174,249,201,207,126,134,44,203,60,252,208,67,67,143,138,202,138,90,254,35,
  193,185,245,217,96,242,243,204,204,44,22,85,86,82,84,84,68,83,115,51,167,27,78,211,214,222,78,79,111,47,65,127,128,104,52,138,100,80,39,69,
  78,135,19,151,195,65,97,65,62,193,64,128,206,142,78,218,90,91,117,167,159,214,14,83,45,211,166,138,38,74,146,230,216,85,251,131,4,27,50,72,
  201,246,213,111,72,203,202,206,102,201,178,165,20,21,151,208,124,145,181,35,41,26,154,184,218,64,98,148,172,95,68,52,33,50,170,183,119,192,82,103,
  223,189,80,80,80,100,249,146,213,121,249,204,114,186,58,187,212,165,68,146,157,99,223,16,146,172,115,167,203,197,220,138,10,10,10,10,104,111,111,167,
  185,165,133,142,174,78,220,110,55,193,64,144,88,44,138,100,48,96,49,91,112,56,28,100,58,28,228,231,231,19,14,133,232,234,28,94,59,50,50,28,
  56,28,14,58,218,219,7,180,163,98,193,2,14,238,223,175,235,124,66,217,4,62,242,209,251,121,238,153,103,56,125,250,244,144,210,51,126,254,248,227,
  40,138,194,167,255,241,31,7,166,54,197,159,247,27,71,227,215,43,178,172,70,242,244,227,156,251,252,130,162,157,63,30,175,239,232,112,56,152,58,109,
  26,57,185,185,4,2,1,106,235,234,85,191,227,245,168,132,40,26,5,73,194,98,54,99,181,90,201,114,185,200,201,201,33,24,12,224,237,117,211,217,
  217,73,36,26,25,90,210,92,127,255,147,96,191,146,161,111,245,45,201,230,13,125,132,168,255,117,0,46,87,38,115,230,205,251,255,236,189,119,156,28,
  197,153,255,255,169,158,184,179,73,90,237,174,210,38,73,171,180,146,80,22,32,4,146,16,65,54,136,40,12,24,99,227,68,48,220,215,246,153,115,184,
  115,192,119,254,157,3,206,103,155,115,58,219,216,38,103,140,1,129,80,206,1,197,213,70,105,115,154,77,179,97,118,98,215,239,143,238,233,233,238,233,
  153,157,217,28,158,55,47,177,51,61,61,61,253,116,87,85,127,234,169,170,231,193,180,204,76,180,180,180,160,177,169,73,58,255,238,110,120,60,30,136,
  193,0,192,4,88,45,22,56,28,73,72,78,114,32,35,35,3,62,175,20,111,179,173,173,45,188,184,106,80,231,31,30,73,92,92,84,132,246,246,118,
  180,58,91,53,101,85,251,87,125,64,14,135,195,129,249,11,23,97,234,212,169,104,107,111,135,211,233,68,135,171,19,61,61,189,240,121,61,178,231,82,
  26,138,79,74,74,66,74,82,18,50,50,166,194,231,147,162,7,36,122,31,242,243,11,80,93,93,29,97,199,170,213,171,113,250,212,41,197,35,42,8,
  170,161,120,213,8,21,211,15,201,235,166,236,169,237,76,164,236,78,182,231,249,32,60,163,60,238,89,170,202,132,121,30,253,80,234,15,207,159,61,139,
  115,103,207,192,106,181,98,230,204,89,200,158,54,13,243,242,11,228,60,188,126,4,2,126,4,3,65,244,246,246,160,183,167,23,77,13,13,104,111,107,
  83,22,54,69,214,26,174,93,9,17,77,136,66,237,21,213,22,38,132,59,173,154,211,142,22,125,36,252,90,31,75,81,119,30,20,213,9,241,181,26,
  67,243,213,80,126,98,232,194,160,169,163,79,132,94,187,92,157,120,127,199,14,112,46,34,51,51,19,211,103,204,192,188,252,2,164,164,164,192,235,243,
  41,229,48,24,12,162,213,233,132,187,183,23,231,207,156,69,175,219,173,57,160,116,155,99,151,195,232,142,61,221,144,116,212,9,245,209,143,229,234,236,
  196,187,111,191,13,112,32,51,43,11,51,102,206,196,220,188,124,164,164,166,194,231,243,34,224,15,32,16,12,64,12,138,146,71,165,215,141,115,103,206,
  192,237,118,107,174,237,240,216,17,253,171,154,69,131,250,16,189,170,38,133,69,169,83,210,103,210,162,199,241,124,207,251,250,60,225,135,107,28,131,48,
  0,208,211,221,133,3,251,246,1,156,99,106,70,6,178,167,103,163,32,39,23,41,41,41,114,27,42,117,234,197,160,136,214,182,86,244,245,186,81,124,
  238,156,228,141,77,240,158,187,221,110,244,185,221,134,118,244,185,221,154,123,46,229,26,55,1,2,139,207,13,196,13,10,132,170,145,13,175,5,49,218,
  102,20,85,70,157,196,132,43,143,49,14,41,13,234,135,39,78,72,161,128,166,78,197,212,140,12,228,231,230,192,225,112,192,231,147,175,89,48,136,160,
  40,162,163,189,13,158,62,15,42,43,202,229,200,44,80,197,155,228,3,108,219,181,94,112,232,34,184,176,126,66,129,117,119,119,225,224,254,125,0,7,
  166,100,76,69,102,102,38,242,102,207,134,35,57,25,62,159,15,193,64,0,65,249,252,59,229,161,251,202,242,114,121,8,157,71,127,94,198,89,230,52,
  158,66,249,127,125,125,30,4,252,1,77,109,142,92,184,167,94,188,195,208,235,118,227,212,135,39,1,14,164,165,167,97,106,70,6,114,103,206,66,146,
  67,154,70,19,148,231,148,138,162,136,206,206,14,120,60,30,84,86,84,194,235,245,168,206,57,126,59,170,171,171,34,70,96,66,211,77,16,33,154,97,
  208,238,198,136,129,163,79,67,150,64,217,157,140,207,115,115,66,81,255,185,58,11,43,143,183,142,169,50,17,135,3,30,43,91,152,78,144,202,239,189,
  94,47,46,93,186,168,9,98,171,172,168,231,82,90,185,80,174,227,232,203,213,121,124,15,60,245,112,161,160,122,24,176,240,98,137,208,64,125,120,155,
  65,97,100,33,219,88,196,41,49,121,59,116,255,215,14,229,79,78,117,170,206,107,157,104,22,10,158,192,53,147,98,43,242,40,37,17,6,13,152,116,
  231,90,91,91,225,116,58,117,229,79,42,131,162,178,77,42,143,12,170,240,179,209,250,71,136,209,152,169,59,204,250,185,147,17,15,170,40,199,140,88,
  162,46,117,136,90,157,78,56,157,45,242,121,169,108,1,143,176,45,244,157,97,181,35,90,152,31,102,40,73,193,25,147,234,17,11,213,32,166,220,63,
  245,235,240,123,132,214,83,140,219,123,94,83,93,173,25,234,52,188,217,17,189,226,176,65,29,29,237,104,111,111,147,179,31,113,101,85,50,7,55,176,
  131,129,15,225,61,175,168,40,215,141,24,73,11,81,180,163,63,44,234,51,92,253,84,230,224,96,92,142,192,160,137,57,202,100,115,185,102,132,137,171,
  122,31,92,243,183,127,167,74,71,123,7,218,219,219,229,123,47,42,30,116,206,69,136,170,235,165,212,19,196,25,251,150,69,58,62,52,205,63,139,225,
  253,87,143,228,247,211,206,117,180,183,163,163,189,221,112,228,145,115,17,16,57,68,72,246,48,149,16,28,220,249,235,71,54,128,154,154,106,8,178,184,
  211,54,235,44,74,251,12,112,30,22,146,93,46,23,186,58,93,114,27,21,110,179,0,249,185,207,69,136,242,125,137,121,206,3,176,163,188,172,76,235,
  245,68,124,211,244,212,101,151,131,201,90,116,128,101,119,146,61,207,205,3,254,53,158,248,217,41,9,11,184,238,172,185,234,113,194,213,239,99,60,168,
  56,11,123,70,32,0,92,212,24,30,30,166,139,163,115,199,161,173,72,208,190,142,38,250,153,62,254,147,82,163,84,207,114,166,114,225,232,247,81,187,
  236,39,179,151,116,196,98,59,169,196,89,168,1,208,148,73,121,59,215,13,244,242,104,101,94,229,9,224,76,211,248,113,117,3,51,160,178,40,132,31,
  226,208,54,244,250,50,30,210,83,90,183,131,174,46,112,85,71,93,165,193,180,229,210,96,0,100,24,237,208,255,167,174,71,140,105,220,39,17,215,158,
  233,28,32,17,251,48,104,83,238,77,180,123,14,189,183,76,127,207,153,34,68,152,94,115,49,93,153,25,33,59,4,198,164,213,221,186,21,243,225,161,
  73,173,232,138,246,131,198,33,133,140,219,4,30,237,67,85,39,141,113,131,91,205,53,141,188,28,140,156,133,159,55,16,1,46,57,121,185,34,56,116,
  215,138,25,95,181,136,5,233,154,149,243,225,69,125,234,235,161,185,23,234,57,215,186,184,131,156,113,168,147,174,232,204,144,174,157,32,219,195,165,215,
  161,50,18,113,175,19,57,127,245,196,111,33,28,13,64,223,65,209,244,159,12,134,163,25,11,61,219,67,98,141,107,79,69,22,182,82,17,22,192,32,
  202,158,71,121,193,218,96,239,67,12,59,164,203,165,106,173,228,255,113,245,180,25,189,42,101,108,192,101,119,178,61,207,205,163,114,178,253,101,37,146,
  43,63,55,240,108,133,61,17,186,30,141,28,190,130,235,212,56,139,235,116,116,115,150,4,33,234,92,38,245,188,16,77,67,193,88,92,81,242,104,41,
  211,216,68,227,201,102,76,201,132,193,117,247,54,42,130,0,132,86,73,43,29,67,158,208,189,55,12,105,164,94,145,169,170,3,154,186,162,154,127,20,
  170,55,76,29,239,78,151,102,142,37,112,61,134,211,14,195,186,161,159,51,165,182,45,202,3,108,210,223,115,69,143,50,213,104,118,2,247,92,182,67,
  59,53,98,24,236,16,152,236,25,21,34,20,39,211,8,22,102,60,167,222,232,77,191,11,218,116,99,84,140,25,76,169,208,186,28,25,116,41,159,149,
  100,123,92,155,0,69,214,164,218,103,48,215,8,187,88,15,129,200,57,135,66,56,228,153,222,11,167,114,139,106,230,70,43,245,156,135,63,227,92,213,
  191,144,223,71,204,43,23,21,135,9,103,146,232,227,92,39,252,6,117,254,80,194,28,69,138,251,200,185,221,218,129,28,174,27,190,15,157,50,87,181,
  17,161,107,31,178,139,43,66,79,25,45,25,98,59,24,51,30,161,48,46,187,81,162,230,38,84,118,39,23,3,159,51,58,32,215,168,218,37,205,85,
  77,4,151,135,223,66,61,80,30,238,184,234,26,122,117,131,171,158,117,194,13,218,159,152,46,98,134,168,19,144,149,17,122,221,63,237,215,185,170,112,
  170,31,20,60,226,47,9,208,49,231,26,13,123,13,148,185,60,186,191,8,7,88,214,119,140,162,182,25,2,83,126,66,154,179,198,148,97,240,88,167,
  201,116,211,65,140,30,236,225,197,109,28,6,115,223,229,215,92,126,248,171,103,99,113,85,150,91,174,56,2,21,135,24,55,10,29,200,212,142,163,97,
  180,131,69,212,47,6,174,234,220,113,141,151,207,192,133,18,249,90,17,106,124,242,220,115,168,239,57,215,14,208,168,238,185,230,82,177,72,59,52,83,
  220,135,193,14,33,52,76,47,36,238,21,229,6,203,243,152,97,106,81,174,115,79,105,167,115,170,157,193,76,89,192,198,228,249,253,154,82,4,237,16,
  151,170,102,112,201,131,41,185,70,197,240,189,215,148,54,30,227,209,99,16,6,80,51,204,173,191,209,208,198,216,213,57,99,212,35,36,234,231,38,135,
  50,26,172,156,119,40,217,68,72,76,43,94,81,30,18,182,124,224,231,47,48,217,115,24,238,84,50,157,29,154,120,170,81,245,161,92,83,229,27,21,
  210,8,202,243,84,47,108,5,169,188,50,197,203,61,60,118,48,77,44,113,117,199,153,133,197,175,170,172,132,231,130,14,164,236,78,190,231,249,136,120,
  70,213,61,123,166,242,60,132,31,4,250,214,34,92,233,181,61,58,77,23,75,30,46,209,206,53,101,161,70,84,158,167,97,52,171,80,179,18,53,202,
  234,93,77,163,10,245,42,121,24,198,57,100,250,99,35,202,147,76,31,11,145,194,62,141,146,43,148,105,230,234,132,31,229,220,192,83,38,63,172,148,
  78,17,139,33,80,88,184,41,12,53,142,74,46,99,110,216,240,170,59,70,96,218,41,35,136,232,32,169,39,187,179,200,48,56,76,45,77,84,231,170,
  202,14,18,238,8,134,187,130,76,95,15,153,92,151,134,209,14,173,16,209,46,220,209,122,69,181,239,89,130,30,59,141,87,113,60,223,115,174,247,134,
  169,238,185,50,39,141,169,58,25,170,76,49,44,236,237,227,209,166,190,13,247,61,23,24,76,38,41,95,185,122,240,57,154,87,212,216,163,196,18,171,
  227,170,97,120,67,239,40,131,246,243,8,239,168,246,166,135,117,40,215,60,131,56,184,74,12,105,142,96,236,20,211,95,47,245,162,89,37,90,139,74,
  15,43,194,137,107,230,48,106,188,217,44,44,123,66,142,155,240,138,5,109,220,221,144,152,150,156,64,92,233,153,198,115,254,76,237,165,141,113,254,76,
  37,70,185,193,179,82,115,253,153,110,58,81,184,87,136,104,79,127,233,157,160,116,214,66,157,167,120,239,67,162,118,104,7,227,117,101,23,198,122,96,
  192,101,151,60,163,35,39,2,30,255,218,215,49,127,254,124,37,191,124,79,79,15,118,237,124,31,255,124,235,173,176,87,130,49,124,246,115,159,195,197,
  202,74,124,176,115,39,0,224,107,255,254,239,8,200,43,66,253,129,128,242,250,119,255,251,20,150,175,88,137,101,151,93,38,173,28,148,87,220,137,162,
  136,151,158,127,222,208,53,175,9,187,164,203,158,160,157,120,31,217,139,103,234,158,217,32,30,156,246,164,36,108,255,216,199,176,116,233,50,8,130,128,
  170,75,151,240,250,107,175,162,163,189,29,95,126,252,223,240,252,179,207,160,178,162,66,217,255,145,71,31,195,185,179,103,177,111,239,30,60,242,232,99,
  104,111,111,199,243,207,62,163,249,124,199,187,239,40,223,185,225,198,173,88,179,110,29,50,50,50,208,222,222,142,215,95,125,21,231,207,157,157,84,133,
  188,96,206,28,220,125,239,189,88,189,102,45,220,189,189,216,187,103,15,254,254,215,167,225,238,237,197,130,69,139,176,237,150,91,241,228,15,127,160,52,
  254,133,11,22,224,198,173,31,193,175,126,241,115,60,249,211,159,201,201,24,2,74,80,244,64,32,128,239,125,247,9,112,206,112,229,85,235,177,112,225,
  34,36,57,28,232,108,111,199,133,11,197,56,113,252,184,236,150,96,184,231,190,143,43,229,48,180,24,98,239,158,221,0,128,77,155,175,5,19,4,152,
  4,65,202,150,98,50,225,239,79,63,173,105,28,245,229,143,169,134,189,66,47,191,249,157,39,176,112,241,98,233,119,228,186,244,222,187,239,226,205,215,
  95,3,99,192,214,143,126,20,215,223,112,99,132,29,223,249,214,55,193,0,108,185,238,58,44,90,44,133,83,107,111,111,199,190,61,123,112,241,98,165,
  114,61,150,95,182,2,249,5,5,176,39,37,161,167,187,27,85,151,46,105,115,142,235,235,148,46,83,82,116,59,212,13,63,12,133,72,162,157,187,88,
  195,241,209,58,195,191,126,234,183,120,244,225,135,20,129,114,247,189,31,199,244,25,51,240,219,167,126,131,43,174,184,18,87,111,220,168,185,247,1,191,
  31,63,253,241,147,216,250,209,155,224,108,105,193,177,163,71,149,3,95,118,217,114,164,167,167,99,223,158,189,97,41,200,49,4,215,74,125,211,37,25,
  178,168,104,49,54,111,190,22,51,102,206,132,199,235,197,225,131,7,177,251,131,15,148,135,243,194,69,139,113,245,198,77,200,206,206,134,187,183,23,37,
  37,37,216,249,222,14,120,60,30,112,206,112,199,246,59,225,234,116,225,253,247,118,40,167,183,245,163,31,197,135,39,79,34,43,43,11,69,75,151,70,
  148,221,11,197,231,81,124,190,120,64,118,104,61,163,136,210,166,234,197,164,182,3,17,235,30,71,95,215,23,101,117,181,106,90,139,122,127,174,233,216,
  113,204,156,49,11,155,183,108,193,11,207,63,7,175,215,171,25,153,11,9,66,30,234,244,168,38,36,179,88,33,5,13,227,238,234,167,56,104,23,217,
  234,102,10,27,123,219,84,29,205,71,255,229,95,16,12,74,25,197,130,193,32,234,235,234,112,228,240,97,180,181,181,169,36,26,83,60,195,92,238,136,
  132,60,238,131,61,255,112,150,52,173,119,31,0,214,172,93,139,53,107,215,42,245,215,231,245,226,236,153,51,56,126,236,40,24,128,85,171,87,99,229,
  234,53,74,120,174,144,78,248,203,159,255,4,48,96,229,202,213,200,203,207,135,32,8,232,233,234,194,153,51,167,209,80,223,128,194,133,11,81,88,88,
  136,96,48,168,148,91,145,115,92,172,172,196,165,80,236,209,1,218,161,148,25,101,45,115,12,143,62,139,240,91,39,84,118,73,140,142,128,103,52,220,
  155,96,248,233,147,63,66,73,137,148,23,54,35,35,3,95,253,198,191,75,149,165,181,13,96,64,198,212,169,240,249,125,40,90,186,20,187,62,216,9,
  128,225,71,223,255,62,0,142,43,174,92,15,206,57,14,29,60,160,244,2,205,102,51,138,207,159,151,66,116,168,198,92,152,156,2,47,178,147,162,43,
  136,6,113,66,245,171,128,153,126,165,35,83,63,48,19,127,112,222,188,109,27,60,125,125,248,250,191,61,14,198,24,174,190,102,35,30,248,244,103,240,
  223,223,251,47,60,255,236,51,184,229,214,219,240,179,159,252,24,0,176,100,233,50,112,112,236,219,187,71,110,10,57,214,172,93,139,227,199,142,42,226,
  83,237,193,184,229,182,219,144,145,145,129,167,126,253,43,116,180,183,99,94,97,33,62,245,233,207,224,103,63,169,71,71,123,251,164,40,224,217,217,217,
  248,218,191,255,7,94,120,246,89,252,240,191,255,27,118,187,29,31,255,196,253,248,248,39,238,199,31,126,247,91,165,124,170,61,101,12,12,130,32,0,
  140,225,171,95,249,87,112,14,108,185,254,58,136,34,199,206,247,222,147,27,108,134,13,87,95,141,185,243,230,225,213,151,95,130,203,229,66,74,106,42,
  110,220,250,17,48,38,72,141,42,3,236,118,59,254,242,167,255,139,8,215,56,125,250,116,184,92,97,33,160,60,200,5,22,181,60,106,227,109,114,205,
  16,211,247,191,247,95,40,57,95,12,206,128,204,204,76,124,235,137,39,112,232,192,126,180,182,181,129,49,134,247,118,188,139,119,254,249,207,112,88,27,
  185,156,94,182,98,37,10,230,206,193,31,127,255,123,120,60,125,152,53,59,7,31,189,233,38,60,247,204,51,232,234,114,97,213,234,213,200,201,205,195,
  206,247,222,67,79,79,55,146,147,147,113,205,198,77,48,155,205,56,115,230,12,34,29,179,76,211,80,247,103,135,102,181,170,145,32,77,176,115,199,12,
  70,30,120,127,30,113,213,177,239,249,196,253,152,57,115,38,126,243,63,191,132,199,227,129,32,8,216,183,119,143,212,25,230,225,216,164,140,49,236,254,
  96,39,62,118,207,189,40,43,45,69,87,151,11,169,169,169,88,188,184,8,175,189,250,170,36,184,120,56,167,188,209,131,39,209,107,21,126,128,2,139,
  22,47,193,166,107,175,197,59,111,255,19,213,23,47,33,61,99,42,62,126,223,39,224,114,117,226,212,137,147,200,159,59,7,183,223,185,29,207,254,253,
  111,168,170,170,130,221,102,195,150,235,111,192,157,119,125,12,207,252,237,175,0,56,204,102,11,150,46,91,134,146,146,11,168,171,173,3,131,148,211,91,
  48,9,56,119,238,44,206,158,61,11,128,227,190,251,63,137,191,63,253,116,88,190,8,194,128,236,144,230,140,154,148,180,138,145,34,52,138,87,84,127,
  199,149,197,87,49,151,73,107,42,28,139,156,172,17,233,157,67,120,200,94,189,192,109,225,162,69,168,173,173,65,65,126,1,202,202,203,0,189,87,61,
  52,228,47,79,50,97,178,55,207,104,150,152,222,139,166,120,64,35,134,234,195,158,65,109,76,89,149,47,222,32,202,144,226,221,103,82,96,248,95,255,
  207,255,40,209,4,138,150,44,193,181,215,93,135,151,94,124,33,156,230,157,133,189,188,161,209,1,197,155,26,227,252,153,206,195,27,237,252,153,190,236,
  34,44,228,78,156,56,142,227,199,142,1,0,82,83,83,177,245,35,31,69,119,79,55,74,47,148,128,49,1,167,78,158,80,62,15,59,178,25,230,206,
  155,135,153,179,102,226,159,255,248,7,188,62,47,178,50,179,176,126,195,85,120,239,221,29,168,40,47,67,69,89,25,56,128,155,110,190,25,111,253,227,
  205,240,119,213,58,96,160,118,232,134,231,35,167,245,234,61,193,131,40,187,147,12,97,48,179,2,226,253,23,241,29,166,46,248,18,109,109,82,56,138,
  140,105,211,148,251,114,245,53,27,241,225,201,147,168,186,116,9,139,151,44,9,55,120,114,188,58,193,100,82,245,98,164,94,183,217,108,14,175,130,19,
  228,21,113,170,127,16,34,115,205,135,222,171,231,56,105,38,44,235,194,209,112,93,246,132,208,195,38,252,151,25,204,88,51,184,38,140,193,102,183,163,
  161,161,65,217,190,119,239,30,60,247,220,179,224,0,42,42,42,80,95,95,143,235,111,188,17,246,164,36,220,125,207,61,120,254,217,103,195,199,224,192,
  142,119,223,193,199,238,185,71,179,141,115,201,227,186,225,234,107,240,220,179,207,74,97,74,228,227,61,255,236,179,176,39,37,37,116,255,134,251,223,96,
  103,167,196,250,119,211,45,183,224,232,145,195,248,224,131,157,224,0,220,110,55,254,254,215,167,81,83,83,173,185,15,154,123,169,30,26,85,134,211,164,
  4,8,161,22,198,158,148,132,77,215,94,139,231,159,123,22,93,93,221,0,99,232,233,238,193,155,175,191,142,43,215,175,135,221,158,36,229,248,182,217,
  194,41,228,84,229,208,100,50,193,106,181,234,242,114,11,49,23,130,132,203,160,46,85,17,211,150,199,214,214,54,180,183,181,99,90,102,150,170,55,47,
  40,182,168,186,246,72,114,56,208,219,211,11,175,207,11,48,134,134,250,122,188,246,234,43,240,249,125,176,217,237,184,226,202,245,120,247,237,183,209,235,
  238,85,226,239,237,220,249,62,214,172,91,7,155,221,166,74,149,55,64,59,84,35,11,161,41,57,220,32,59,137,82,135,228,186,197,245,175,99,148,7,
  245,3,66,179,159,188,93,144,31,82,159,248,212,167,144,155,155,139,223,252,250,87,232,243,120,148,7,152,244,185,174,93,0,131,215,231,195,137,227,199,
  112,195,214,173,96,140,97,203,245,55,224,204,153,51,240,7,252,134,247,124,208,215,138,133,175,207,181,215,93,135,61,187,119,161,234,210,37,112,6,116,
  118,116,224,213,87,94,134,199,227,5,103,192,77,55,111,195,235,175,190,138,234,170,106,0,128,199,235,197,91,111,190,1,135,195,129,5,11,23,41,109,
  232,177,99,71,177,245,35,31,149,126,79,96,176,88,45,16,4,147,38,17,129,221,110,31,18,59,152,16,242,140,10,128,110,69,50,211,142,121,70,78,
  103,226,186,59,200,226,107,19,152,122,236,215,96,18,48,103,250,120,158,90,175,153,205,110,67,86,118,54,142,28,62,140,165,151,93,166,12,143,51,166,
  189,14,161,85,240,130,74,196,8,44,242,159,228,33,134,18,24,30,74,202,105,65,55,20,12,205,123,166,202,208,36,181,87,6,139,127,52,29,55,22,
  14,212,46,215,168,226,243,231,225,234,236,68,126,126,129,46,5,175,160,220,199,120,207,159,245,115,254,122,161,13,221,107,237,232,8,208,221,211,131,93,
  31,236,196,242,229,43,228,78,185,160,60,147,161,202,210,37,48,192,110,183,41,241,119,25,99,104,109,107,197,222,61,123,165,164,20,42,59,172,86,235,
  144,219,161,156,51,15,77,125,130,106,181,61,139,244,138,242,129,151,221,241,250,111,108,15,211,235,134,91,66,21,109,195,134,171,177,112,225,66,112,145,
  99,118,78,14,4,65,64,121,121,185,242,157,101,203,151,227,245,215,94,67,123,123,59,182,108,185,14,23,138,139,149,120,115,130,201,4,166,204,139,145,
  110,176,217,108,198,162,197,139,145,151,159,39,199,131,147,98,43,30,216,191,207,104,156,204,120,184,222,160,151,15,163,109,209,114,207,27,244,146,98,13,
  39,238,124,255,125,220,255,201,79,97,205,154,181,168,170,170,66,101,101,133,102,8,244,141,215,95,195,55,254,227,155,152,59,175,16,251,246,237,69,187,
  202,163,201,1,84,86,84,98,234,212,12,220,112,227,141,216,241,238,187,74,97,152,53,107,54,42,43,43,229,160,198,97,206,77,178,33,250,57,115,230,
  226,249,103,159,213,220,7,183,219,141,119,223,121,39,220,224,179,72,239,90,248,129,41,15,182,10,12,16,5,101,30,214,140,153,179,80,87,87,7,175,
  215,171,241,198,249,252,62,180,180,52,35,123,122,54,234,106,235,96,179,217,112,251,29,119,202,241,77,129,146,11,23,80,86,90,2,147,201,132,153,179,
  102,225,182,59,238,84,178,237,236,223,183,23,78,167,51,236,169,211,13,125,234,61,249,92,117,190,27,55,109,194,226,162,34,112,14,228,230,230,130,49,
  134,210,178,82,233,161,196,4,172,187,226,114,20,206,47,68,48,40,13,121,213,213,213,226,141,215,94,199,233,83,31,98,241,226,197,248,127,95,250,50,
  46,93,188,136,170,75,151,80,86,90,10,159,207,135,217,57,57,112,58,157,240,251,253,170,124,223,82,122,221,246,246,54,100,101,101,163,177,161,94,55,
  95,21,17,43,190,99,217,1,166,159,226,98,16,75,53,218,202,251,88,245,45,138,55,212,112,120,150,49,124,242,211,159,198,186,117,151,227,219,255,241,
  31,114,32,243,80,10,75,1,171,87,173,65,193,156,185,74,160,248,134,134,6,41,145,0,128,11,197,23,176,116,217,50,220,117,247,61,16,197,160,18,
  159,80,19,62,201,120,156,62,225,107,165,54,111,118,78,14,74,138,47,104,60,100,141,141,141,104,106,104,4,24,67,78,78,142,148,230,48,52,135,88,
  94,93,95,81,94,142,153,179,102,162,172,172,20,38,179,25,13,245,245,152,50,101,10,174,186,122,3,14,30,56,0,139,197,2,147,73,22,2,178,247,
  221,102,179,201,243,60,7,103,135,20,218,201,108,48,218,4,195,104,36,12,113,132,137,68,156,43,145,85,222,79,37,38,173,58,154,139,18,199,148,171,
  194,235,50,44,94,92,132,178,178,50,248,124,126,244,246,246,34,51,51,11,173,173,78,249,154,170,203,44,151,231,227,202,107,21,88,140,104,235,76,61,
  183,80,27,212,62,34,123,159,122,53,186,58,248,189,193,164,68,69,119,203,47,4,147,73,87,110,56,58,58,58,144,153,153,137,218,154,106,121,154,40,
  211,205,39,30,202,243,103,134,243,220,1,30,153,97,10,64,91,91,27,178,167,79,87,196,254,162,197,139,49,107,246,108,128,75,49,113,91,157,78,28,
  58,120,0,23,43,43,145,151,95,128,237,119,125,12,141,13,13,104,106,106,66,109,109,141,146,33,45,100,135,197,106,149,127,103,40,237,48,30,197,233,
  175,125,26,84,217,157,44,195,244,137,4,37,213,198,97,75,92,3,43,33,71,52,195,105,12,43,87,173,132,211,233,196,207,127,250,19,165,193,184,252,
  242,43,224,243,122,177,245,35,91,33,138,28,133,243,231,99,234,212,169,82,48,95,6,152,228,80,78,76,85,192,77,102,19,154,154,154,112,238,236,25,
  37,176,183,94,44,104,6,58,152,254,117,100,186,181,200,48,13,114,115,38,55,98,250,32,182,154,76,11,81,230,58,49,213,3,178,177,190,30,63,250,
  254,127,99,214,172,89,40,156,191,0,87,95,115,13,110,184,241,70,252,236,199,210,208,124,159,219,141,55,94,123,13,55,220,120,35,254,87,22,80,250,
  126,212,27,175,189,138,111,124,243,91,210,220,53,189,255,135,143,159,192,101,28,136,59,72,46,143,187,28,114,195,235,160,157,43,166,142,65,201,229,30,
  184,118,96,79,96,2,184,192,21,129,33,200,158,165,80,160,112,166,202,180,101,54,91,20,175,132,213,102,195,241,227,199,21,239,76,87,119,151,212,43,
  55,9,240,122,189,56,113,252,152,210,248,118,119,117,107,86,211,26,133,8,137,62,52,45,53,168,171,214,172,134,179,165,5,79,254,240,7,26,15,75,
  101,101,5,14,29,56,160,4,192,238,235,115,131,1,240,122,188,248,203,159,254,15,233,83,166,32,63,63,31,5,115,230,224,250,27,111,196,31,127,255,
  59,152,152,0,139,197,162,139,249,41,93,19,171,197,42,139,22,35,207,146,190,161,143,109,135,86,132,132,83,70,48,85,252,76,166,174,91,81,4,102,
  196,61,214,119,128,213,43,228,85,211,21,4,65,64,94,94,30,94,121,233,69,60,248,200,195,248,254,247,190,167,12,230,10,140,161,234,210,37,28,59,
  122,68,9,124,222,215,215,167,44,246,1,227,216,241,206,187,120,240,225,135,241,219,167,126,99,120,173,162,5,170,79,248,90,169,66,219,153,76,38,141,
  93,234,182,136,1,114,14,120,104,135,12,101,91,67,87,193,108,54,129,49,1,187,62,248,0,95,120,244,49,20,159,63,175,41,187,33,59,36,239,254,
  16,216,161,11,122,175,78,6,18,145,172,142,243,136,17,38,69,63,170,167,96,133,188,155,234,231,10,215,137,102,30,153,240,0,154,178,192,141,158,112,
  0,128,149,171,87,163,178,162,28,235,46,191,28,38,147,9,203,150,95,134,221,31,124,160,121,246,112,198,213,221,86,227,231,37,16,53,125,173,97,106,
  87,229,57,196,213,33,112,195,215,8,209,23,161,133,190,32,168,134,165,67,43,211,5,249,185,25,30,150,231,170,184,27,67,125,254,186,231,109,216,95,
  29,142,233,169,47,159,38,147,20,207,83,96,104,108,104,192,133,11,197,74,155,229,245,122,0,198,224,243,249,240,238,219,111,35,37,37,25,211,167,207,
  192,172,89,179,112,249,21,87,224,141,215,95,71,79,79,183,98,135,213,98,137,156,166,51,104,59,96,88,118,213,33,192,66,101,87,61,167,120,96,101,
  119,124,147,200,243,124,192,195,244,3,117,221,106,99,220,51,28,216,191,31,111,188,246,42,158,252,209,15,177,104,209,34,76,203,204,84,134,26,174,88,
  191,30,149,149,21,138,107,188,234,210,37,172,89,187,78,233,69,8,38,65,51,76,31,26,118,234,237,237,69,125,125,61,234,235,235,209,80,223,32,207,
  63,101,154,225,124,117,110,121,253,208,188,102,56,193,96,56,199,232,47,215,15,147,168,170,29,12,134,19,213,255,62,255,208,195,176,39,37,161,190,161,
  1,123,247,238,193,83,114,46,230,185,243,10,149,125,218,219,219,209,38,15,181,107,254,201,26,203,221,215,135,215,95,123,21,31,187,231,94,101,91,69,
  69,5,230,206,157,23,49,36,95,180,116,41,174,191,241,198,9,225,218,143,231,184,231,206,158,195,218,203,175,136,40,135,95,250,202,227,200,204,202,2,
  7,144,156,156,172,25,186,117,56,28,138,103,148,135,154,20,205,162,2,134,198,166,70,228,228,228,192,158,148,164,10,241,197,96,79,178,99,94,97,33,
  156,78,167,228,85,178,90,209,32,151,197,250,134,122,116,119,75,67,250,130,96,130,40,138,210,246,250,122,212,213,215,193,235,243,106,114,27,171,135,211,
  161,30,194,150,87,122,170,91,203,189,123,246,224,229,151,95,194,247,191,247,95,88,92,84,132,204,172,172,176,94,16,24,218,219,218,81,94,94,142,138,
  138,114,84,86,86,160,190,190,1,96,192,182,219,110,197,220,121,133,232,236,116,225,244,233,211,120,243,141,215,113,254,220,57,44,95,177,18,45,78,39,
  102,205,158,173,216,24,26,210,179,219,147,48,103,222,60,41,95,186,122,184,15,218,172,101,113,217,161,235,230,105,219,9,109,221,226,81,238,61,226,108,
  123,180,67,244,170,135,138,32,224,167,79,62,137,157,59,119,162,190,174,30,119,222,245,177,112,56,22,65,64,167,203,133,139,23,47,225,210,165,139,184,
  84,85,133,166,166,38,205,61,119,117,185,96,177,90,225,234,234,210,205,57,19,34,226,195,13,197,181,226,0,170,47,93,194,146,101,203,52,109,85,198,
  180,105,120,224,51,159,5,7,80,37,127,206,85,18,31,140,97,201,178,101,104,110,110,86,218,75,38,48,120,189,94,124,240,193,7,184,229,182,219,96,
  182,152,149,249,210,33,59,172,54,219,144,216,193,24,131,217,36,205,25,69,180,78,87,212,128,232,60,74,232,253,216,109,66,56,244,30,15,119,112,12,
  146,170,68,180,67,12,200,205,205,129,223,231,131,207,235,3,99,12,237,237,109,200,201,201,213,92,143,208,162,23,105,93,189,58,188,149,52,228,171,254,
  167,190,102,130,178,96,6,145,161,130,52,35,112,2,52,201,3,52,94,100,205,96,114,88,102,201,135,144,238,163,118,152,60,39,55,23,29,237,237,225,
  223,198,96,207,159,197,60,255,208,2,62,65,183,0,75,51,31,83,190,240,57,57,185,104,115,58,229,33,112,134,158,158,30,212,215,55,160,161,161,30,
  13,13,13,104,107,147,70,5,215,111,216,128,89,179,103,163,167,183,23,149,149,21,56,176,127,31,46,93,172,196,130,5,243,53,118,72,29,233,161,181,
  67,59,52,175,155,19,206,116,137,72,84,195,243,3,41,187,147,109,168,94,24,113,185,172,137,235,37,221,56,183,219,141,63,253,233,255,240,233,207,124,
  22,12,64,78,94,30,146,147,147,241,202,203,47,227,221,183,223,198,187,239,190,131,231,159,123,22,27,55,109,82,230,242,8,130,9,130,73,208,12,249,
  152,76,102,169,129,101,234,57,77,76,153,7,34,168,231,53,9,70,193,236,17,25,35,207,168,17,208,197,130,51,140,11,135,24,73,68,84,251,118,116,
  116,224,230,109,183,32,35,35,3,0,48,111,94,33,146,236,73,104,239,72,108,129,81,104,162,247,188,121,243,148,109,251,246,238,197,221,247,220,139,169,
  170,99,223,125,207,189,168,172,168,156,52,174,255,15,118,190,143,203,175,184,2,215,110,217,2,200,194,243,145,47,60,138,228,148,100,180,58,157,40,185,
  112,1,153,217,217,216,112,205,70,48,72,139,127,110,189,237,118,156,62,245,161,118,218,134,32,72,2,85,46,39,30,143,7,135,14,30,196,67,143,60,
  130,169,83,51,192,24,195,212,140,169,248,244,103,63,135,131,7,246,195,43,63,192,44,202,188,80,109,249,51,153,66,94,71,227,178,136,136,50,9,195,
  116,129,80,135,74,2,224,118,247,225,15,191,251,29,62,255,208,195,170,33,166,208,252,49,93,3,203,24,154,26,27,113,235,237,183,99,246,236,89,0,
  24,50,50,50,80,88,88,8,103,75,11,124,62,47,78,158,56,129,79,126,234,1,76,153,50,21,140,49,76,153,58,5,247,125,242,126,28,61,114,24,
  62,159,175,223,58,21,151,29,70,94,181,136,32,221,232,119,202,75,92,211,133,34,188,53,146,247,211,237,238,3,3,240,210,139,47,96,225,162,69,184,
  98,253,149,202,208,178,160,79,11,168,30,146,150,237,11,61,248,244,246,171,175,207,80,92,171,208,190,251,247,237,195,214,143,126,4,133,243,231,43,247,
  237,51,159,255,60,154,26,27,192,0,188,183,99,7,238,253,248,125,40,156,95,8,128,193,225,72,194,199,238,190,7,62,159,15,23,46,92,144,189,171,
  102,165,92,156,57,125,26,38,147,25,133,133,243,195,115,70,67,98,212,106,29,26,59,132,80,110,122,33,102,123,26,117,88,83,63,143,52,193,103,142,
  206,33,22,57,69,64,151,72,96,245,154,181,56,122,248,48,142,28,57,140,99,71,142,224,216,209,99,168,174,174,198,226,162,162,8,103,134,246,90,160,
  159,127,6,9,84,140,146,171,24,205,171,5,211,8,72,68,74,81,101,15,65,149,188,69,26,129,92,141,169,83,167,226,82,213,37,85,134,161,1,158,
  191,160,63,127,193,240,252,1,173,192,83,66,156,202,191,31,250,60,61,61,29,155,175,189,54,220,238,170,99,174,66,27,125,161,189,173,29,27,174,190,
  26,153,153,89,0,99,72,77,79,71,78,110,30,58,58,58,53,231,102,150,71,117,134,218,14,163,178,98,216,38,177,33,40,187,147,105,152,126,228,52,
  168,193,138,250,208,3,135,3,31,158,60,137,171,174,218,128,235,174,191,1,179,115,114,112,244,240,97,77,236,63,143,199,131,75,151,46,97,221,229,151,
  227,232,209,35,146,87,84,53,44,5,121,232,106,227,166,77,184,118,203,22,77,54,166,167,255,242,103,101,34,127,180,144,95,234,225,70,198,244,141,152,
  193,54,3,33,106,88,40,251,121,255,143,55,223,192,182,109,183,224,203,95,121,28,73,73,73,104,168,175,199,235,175,191,166,91,237,30,101,184,157,107,
  251,33,207,63,247,44,254,253,63,190,169,108,123,227,245,215,112,253,13,55,226,203,255,250,21,36,37,37,73,97,160,158,123,86,242,56,79,18,90,90,
  90,240,131,255,239,255,195,103,62,255,57,252,203,23,191,4,200,15,234,159,254,228,39,202,208,230,247,255,191,239,225,243,15,62,132,207,125,254,243,232,
  233,233,193,190,125,123,177,243,189,247,85,193,177,25,4,129,169,130,197,75,223,219,249,254,123,96,140,225,161,71,30,65,250,148,41,104,107,107,195,137,
  99,199,176,119,207,30,69,64,88,44,22,121,30,147,54,126,158,32,8,114,99,201,34,115,37,199,83,254,192,228,84,132,170,224,241,242,252,193,147,39,
  142,99,195,53,215,224,198,143,108,197,187,239,188,3,198,4,108,187,245,22,220,124,203,45,8,6,131,224,156,35,24,12,226,151,63,255,25,142,31,59,
  142,140,12,201,163,150,62,101,10,218,90,91,113,252,248,49,148,150,150,128,49,134,125,123,247,64,16,4,60,240,153,207,32,61,61,29,157,29,29,56,
  117,234,20,14,236,63,32,207,43,140,86,167,226,183,163,95,33,202,162,207,175,138,158,86,144,25,183,63,234,249,162,161,160,218,74,76,71,134,62,183,
  27,127,253,203,159,241,208,35,143,160,174,174,14,76,16,240,145,143,124,4,91,63,250,81,37,116,150,40,138,120,234,55,191,198,197,202,74,101,192,78,
  51,157,65,53,92,202,117,62,184,193,94,171,208,72,222,185,243,231,32,8,2,238,187,255,126,100,100,76,131,167,175,15,187,119,237,194,59,239,188,3,
  6,134,139,149,21,120,254,185,231,112,251,29,119,98,214,172,89,112,187,221,56,123,230,12,254,246,244,211,202,53,83,58,238,242,124,208,55,94,123,13,
  255,250,248,227,16,4,166,153,139,111,181,90,195,30,207,65,216,193,228,57,163,198,158,81,131,246,180,159,12,82,92,127,159,25,139,235,217,99,156,78,
  94,30,18,151,47,112,250,148,41,200,47,40,192,235,175,189,42,63,167,164,155,89,90,114,1,31,189,233,102,156,57,117,90,51,7,65,155,84,128,161,
  159,76,1,6,215,140,193,96,148,94,227,105,147,202,174,106,59,215,36,137,82,188,75,161,25,34,130,32,224,223,190,246,117,112,81,68,80,20,81,81,
  86,134,183,223,122,75,185,223,234,76,99,195,113,254,76,31,9,67,61,245,137,49,172,191,234,42,92,117,213,6,48,0,174,46,23,206,159,59,143,226,
  226,98,169,131,40,8,184,252,202,43,113,197,250,245,225,172,138,156,227,229,151,94,68,105,105,9,210,210,211,241,209,155,110,66,90,90,26,186,186,186,
  112,225,66,49,106,106,170,85,191,133,112,251,58,196,118,68,8,81,163,160,246,12,67,82,118,39,19,108,211,213,87,199,237,85,93,177,114,37,126,246,
  139,95,224,209,71,30,193,201,19,39,18,31,86,85,167,234,212,167,251,212,205,225,137,120,205,85,51,126,66,129,169,85,243,190,184,193,252,65,206,141,
  102,191,68,9,94,205,152,110,37,101,236,135,166,161,16,29,224,131,211,232,225,57,153,88,181,122,53,126,253,212,83,248,242,23,191,136,83,31,126,24,
  215,119,126,246,203,95,66,20,69,60,250,240,195,3,250,77,49,148,198,145,235,242,116,104,202,89,180,215,6,229,47,202,107,109,89,52,72,191,96,52,
  183,74,231,198,137,37,218,68,46,106,203,158,102,238,178,250,55,153,102,190,92,120,5,148,214,62,245,163,109,36,237,48,252,60,34,109,32,34,83,68,
  38,80,159,38,222,61,215,185,251,180,147,211,52,118,132,55,25,157,239,200,220,243,37,69,75,240,237,239,62,129,31,252,247,127,163,228,194,133,184,58,
  246,191,252,213,175,32,138,34,190,240,208,67,253,122,186,99,181,161,92,20,99,60,99,212,214,133,239,119,196,156,82,110,84,22,140,51,46,37,116,205,
  16,57,204,203,116,147,110,195,115,84,13,20,181,234,126,235,231,78,243,200,19,213,76,87,97,195,113,254,186,122,13,141,232,77,112,16,87,255,76,215,
  107,2,166,183,107,100,236,48,20,162,6,207,254,232,94,211,137,155,252,102,32,207,243,129,123,70,7,56,49,64,89,213,25,122,167,17,164,234,247,186,
  76,40,80,109,87,103,148,8,173,96,84,23,134,136,196,25,70,83,189,153,65,239,94,95,168,34,253,0,74,67,161,78,71,198,117,5,78,253,158,27,
  8,81,30,229,193,57,113,230,45,15,172,60,141,96,57,212,150,197,208,2,25,196,44,139,218,215,234,164,117,170,12,71,186,114,169,121,220,71,93,109,
  0,195,178,24,173,236,1,8,47,236,129,52,63,74,249,13,30,62,147,240,74,110,102,176,80,75,149,58,80,149,66,50,98,153,222,8,218,17,237,181,
  54,93,169,65,0,121,30,229,116,248,36,185,231,186,103,54,211,223,91,85,64,201,232,118,140,204,61,103,42,111,108,162,35,76,28,177,61,77,253,182,
  35,161,185,253,42,23,160,218,67,174,89,208,200,213,89,232,195,222,210,112,142,85,104,107,22,55,40,116,49,34,153,179,168,142,17,163,21,179,76,83,
  164,181,9,28,152,238,89,199,52,30,55,174,239,120,50,213,66,55,245,25,232,206,95,125,188,132,206,159,69,76,158,84,157,63,211,233,102,174,169,239,
  92,93,144,149,186,207,53,105,79,213,223,99,42,123,16,229,62,12,171,29,60,124,241,98,149,93,30,239,240,252,68,122,254,15,208,150,17,207,192,164,
  79,207,167,17,164,154,6,66,253,192,8,165,43,228,225,244,160,242,67,37,156,207,30,202,251,112,37,227,209,111,127,180,33,32,205,252,162,104,30,208,
  232,133,47,81,143,40,49,74,67,2,81,210,68,34,74,89,140,245,90,221,232,168,203,165,166,188,68,43,139,209,26,171,56,202,87,232,181,226,53,209,
  156,3,12,235,152,122,117,169,186,195,7,221,112,223,104,216,145,144,39,1,177,30,50,108,114,221,115,181,215,76,117,78,17,194,196,192,142,200,242,48,
  140,118,168,167,6,36,122,255,251,253,109,214,127,231,51,34,21,104,180,123,172,11,130,31,18,164,106,17,207,117,158,48,77,94,166,216,145,145,34,132,
  144,161,55,89,239,153,99,0,23,181,98,146,233,133,17,84,209,93,160,154,118,16,218,41,28,174,75,219,245,52,112,218,196,142,236,20,101,67,148,209,
  67,93,170,204,144,160,212,166,8,141,86,94,213,191,164,238,24,64,90,220,21,227,62,12,175,29,113,120,65,25,27,146,178,59,89,24,93,49,106,228,
  200,208,11,82,85,195,170,84,78,206,181,15,2,131,7,44,6,208,168,37,42,52,89,44,81,155,192,245,32,198,134,32,101,250,169,35,49,68,72,68,
  57,140,242,126,32,101,177,191,33,105,253,231,250,8,144,202,121,171,231,54,169,237,52,170,67,209,62,27,65,59,226,237,200,13,166,115,55,81,238,57,
  139,117,191,229,109,130,193,61,86,219,161,89,84,50,2,118,132,22,158,42,81,6,250,121,152,179,104,11,154,6,122,239,227,20,164,210,117,130,38,222,
  168,218,75,170,14,132,4,3,1,196,250,63,17,67,101,100,40,66,117,215,84,47,216,194,215,67,39,230,152,222,51,46,132,207,89,63,119,67,151,255,
  157,113,196,145,191,50,246,189,55,62,127,104,198,98,152,230,186,49,77,59,198,52,1,226,153,114,238,44,60,176,19,14,54,143,209,178,35,49,33,74,
  78,169,97,17,163,131,139,181,207,152,106,238,136,146,54,88,237,33,85,123,109,194,61,85,64,59,172,18,174,116,170,225,6,22,191,167,88,83,165,153,
  182,7,6,166,207,84,204,180,110,121,85,35,16,89,208,184,65,1,228,84,218,134,218,175,63,148,229,80,227,17,209,111,211,198,250,211,120,80,66,15,
  114,109,17,129,122,124,153,199,85,14,141,202,162,126,8,136,235,26,58,142,162,162,37,184,80,124,94,57,7,165,172,10,122,143,40,160,170,65,90,91,
  153,54,175,245,104,216,193,140,234,147,246,68,226,170,79,172,159,186,54,17,238,121,104,81,133,200,195,190,59,233,126,67,53,161,9,218,50,16,205,142,
  17,188,231,140,65,142,77,59,176,142,189,102,49,79,132,134,224,113,214,119,131,86,135,233,87,32,133,98,69,234,71,16,140,5,42,244,177,44,57,34,
  202,109,20,183,168,241,148,6,131,33,224,233,51,102,160,185,169,73,99,128,160,10,112,31,233,33,213,46,158,227,106,31,156,226,37,13,127,137,199,213,
  46,39,118,254,250,178,171,220,99,6,64,20,85,34,83,29,9,64,59,93,2,76,85,166,85,222,106,65,127,169,71,195,14,245,47,12,115,217,157,12,
  207,115,243,104,156,170,241,234,70,166,19,164,44,162,241,12,207,215,52,238,197,115,102,208,139,137,187,215,28,205,83,202,34,11,109,188,43,122,251,233,
  9,145,87,116,140,121,71,21,33,194,0,68,78,31,9,119,142,152,174,115,196,162,150,195,161,40,139,70,67,122,12,192,117,55,220,136,197,75,150,98,
  94,225,124,188,249,198,107,202,48,171,102,178,189,218,11,2,131,78,157,46,252,252,104,216,17,79,189,27,74,207,194,120,190,231,17,251,104,157,75,138,
  29,48,240,138,71,12,125,70,76,105,26,94,59,34,67,140,197,16,162,195,20,10,135,193,96,202,130,145,82,141,152,50,22,26,38,86,9,37,173,171,
  34,252,32,102,241,156,117,180,41,14,218,58,144,95,80,128,171,55,109,66,126,126,1,206,156,62,133,55,95,127,77,51,69,131,169,197,28,139,236,80,
  50,213,138,123,237,60,241,225,59,255,168,101,87,227,57,84,214,253,235,202,163,106,20,71,136,212,0,17,101,151,143,178,29,35,88,118,39,188,103,52,
  17,13,203,135,196,31,21,190,129,17,130,84,153,124,111,156,89,133,235,246,83,63,24,194,39,201,19,62,15,189,150,15,103,214,136,44,80,220,224,161,
  201,163,132,24,227,253,8,81,242,149,14,190,92,13,69,57,140,136,236,160,10,111,162,239,28,65,215,57,82,175,88,143,184,223,131,44,139,154,7,134,
  174,44,46,90,178,4,139,151,44,5,99,192,188,249,133,184,113,235,71,228,20,167,220,32,5,55,139,89,127,24,27,61,59,140,38,248,243,40,66,148,
  247,35,68,227,174,83,227,244,158,115,163,112,85,253,221,239,40,118,140,202,61,151,231,140,234,231,11,114,221,131,156,71,173,231,204,248,86,14,244,185,
  163,187,6,145,215,41,252,94,51,28,12,174,117,114,241,65,74,15,22,57,208,159,87,80,128,107,100,17,234,241,120,224,241,120,112,217,242,21,0,164,
  144,125,225,223,102,186,107,160,91,77,47,48,93,71,3,136,24,191,27,226,243,231,6,211,14,244,247,152,49,64,20,185,226,193,135,110,94,187,230,212,
  116,247,65,27,61,96,116,237,208,31,128,179,248,68,239,128,202,238,4,127,158,155,71,235,132,153,209,3,33,134,151,2,0,30,124,232,33,156,56,126,
  28,39,142,31,143,240,148,194,64,44,38,220,155,143,230,9,141,218,219,97,198,243,250,135,200,131,67,140,98,89,52,242,214,235,60,98,26,129,2,104,
  202,34,31,96,25,96,81,202,161,186,92,45,46,42,194,245,55,220,40,229,200,102,64,86,246,116,20,45,93,138,89,57,57,56,124,240,32,42,43,43,
  164,52,184,186,134,92,59,55,91,235,33,25,13,59,34,191,51,240,250,148,232,40,195,120,187,231,209,174,149,250,188,34,163,122,68,177,99,148,238,185,
  218,51,26,143,55,180,191,174,39,27,204,220,81,30,25,150,33,150,151,84,89,36,198,84,94,56,35,65,20,171,167,204,250,191,102,249,249,5,184,122,
  211,70,69,132,238,221,187,27,199,142,28,193,246,143,221,141,188,252,124,44,91,190,28,57,185,185,216,187,103,55,42,202,202,225,245,121,181,222,67,181,
  247,155,33,194,179,171,185,230,177,206,159,199,165,167,226,30,237,48,122,62,235,227,108,106,230,42,235,231,58,235,203,111,188,247,97,36,236,80,239,205,
  141,234,239,208,149,221,9,237,25,29,85,17,32,8,128,40,26,10,82,163,135,194,131,15,61,132,223,3,56,121,252,120,164,167,71,83,193,226,245,153,
  177,254,11,103,2,34,116,184,30,156,196,176,171,209,200,6,93,45,216,212,94,248,104,2,197,240,225,158,136,239,150,197,85,14,23,23,45,81,132,232,
  43,47,190,128,155,111,189,13,245,117,181,40,62,127,30,87,174,191,10,55,108,221,10,81,228,184,88,89,129,139,149,149,184,120,177,18,94,175,215,184,
  113,87,151,197,17,182,99,212,59,118,227,232,158,199,188,86,186,133,156,241,216,49,106,247,156,49,37,139,89,60,66,148,13,111,117,143,12,166,0,253,
  60,82,3,175,41,34,167,142,69,189,102,44,254,103,79,126,126,1,174,222,184,17,249,5,146,8,221,183,103,15,142,30,61,2,175,199,163,156,107,117,
  117,53,206,158,62,141,13,215,92,131,155,110,190,5,98,48,136,242,242,50,84,84,148,163,162,188,28,94,143,39,220,41,17,116,70,234,58,159,253,158,
  63,27,196,61,103,172,223,253,213,81,33,12,111,78,180,155,101,212,185,30,109,59,98,233,93,122,212,143,15,49,218,159,135,52,214,67,65,253,96,69,
  132,199,103,96,165,32,214,67,32,174,135,38,9,209,113,237,29,229,70,226,68,211,57,138,45,80,16,209,208,15,178,28,234,202,211,138,149,171,112,205,
  166,205,104,117,58,241,202,139,47,194,235,243,41,229,169,248,252,121,20,159,63,143,121,133,243,49,103,206,28,204,157,87,136,121,133,243,1,0,78,167,
  19,117,117,181,104,117,58,225,116,182,160,213,233,52,16,47,35,103,199,80,118,236,6,229,33,27,7,247,60,174,182,39,148,255,29,253,219,49,154,247,
  60,148,89,135,197,113,127,217,48,221,115,205,189,231,98,228,118,174,151,148,218,65,109,30,77,205,14,240,154,133,68,104,94,200,19,186,103,15,142,169,
  68,168,62,76,209,217,211,167,113,246,204,105,204,95,176,16,243,230,206,67,225,252,66,20,22,206,7,223,10,180,180,52,163,174,182,6,45,45,78,180,
  180,180,192,233,108,209,13,113,179,136,68,51,67,170,158,88,100,252,91,253,249,235,195,126,50,198,32,114,110,176,210,35,182,87,159,71,155,154,55,74,
  118,24,149,24,74,112,51,206,196,40,244,97,82,84,31,29,61,113,2,207,61,243,12,126,250,147,159,68,196,109,75,77,77,197,7,123,246,224,247,191,
  253,45,126,247,219,223,70,134,35,25,104,119,57,65,17,58,172,222,27,98,108,9,82,64,241,216,71,248,141,116,158,147,161,46,135,215,223,184,21,69,
  69,75,224,116,58,241,202,75,47,40,195,115,208,229,168,190,88,81,142,139,149,21,120,255,189,29,152,61,59,7,243,10,11,145,153,149,133,149,171,86,
  107,26,238,134,134,122,116,185,92,232,234,234,66,93,109,45,188,94,175,246,225,53,76,118,196,93,159,134,89,136,142,135,123,158,208,181,10,165,171,141,
  97,135,58,120,206,104,216,161,205,247,221,143,16,29,129,133,159,130,32,132,179,114,233,175,21,143,24,88,149,255,223,207,84,134,56,175,223,101,203,87,
  224,178,229,203,21,17,186,111,111,12,17,106,112,126,229,101,165,40,47,43,5,127,155,35,39,55,15,243,11,11,145,149,61,29,171,86,175,149,100,28,
  151,162,70,212,214,213,74,245,220,213,133,250,186,90,120,189,30,56,157,206,65,159,127,255,157,136,104,11,122,13,238,3,99,6,63,171,237,8,178,136,
  201,116,67,115,31,134,210,142,145,44,187,36,70,135,83,4,112,142,111,63,241,4,102,206,154,133,135,31,124,80,18,163,71,142,96,225,162,197,40,156,
  63,31,75,150,46,69,75,75,11,114,114,243,112,253,13,55,194,106,179,194,227,245,162,164,164,4,0,176,122,205,26,60,249,147,159,224,225,7,31,68,
  89,105,105,68,193,84,38,213,26,20,210,144,119,214,56,23,242,208,60,52,169,32,142,63,65,26,241,208,86,183,68,28,198,94,179,33,238,152,216,108,
  54,220,188,237,86,204,206,205,69,113,241,121,236,219,179,91,26,118,215,120,16,152,97,140,198,250,250,58,212,215,213,41,123,205,206,201,69,86,86,22,
  178,179,179,145,150,158,142,197,75,150,130,115,142,117,87,92,169,212,145,174,46,23,186,93,93,232,234,146,132,42,0,212,215,215,1,28,232,234,238,66,
  183,188,109,64,215,182,159,250,148,104,199,110,168,234,211,88,187,231,3,189,86,177,236,96,234,172,50,163,101,135,60,103,84,63,76,63,154,15,115,65,
  246,204,25,157,136,145,131,68,115,158,60,166,95,204,144,101,203,151,227,234,107,174,65,122,250,20,120,229,225,120,69,132,198,240,180,113,206,163,140,200,
  49,212,213,212,160,174,182,70,217,150,147,155,139,172,172,108,100,101,103,35,45,53,13,75,138,150,200,71,187,82,126,254,113,116,117,117,161,75,87,207,
  235,228,182,162,187,171,75,217,22,21,163,81,240,88,158,194,254,58,92,44,122,170,80,33,90,89,29,196,125,24,62,59,72,136,142,91,49,58,115,230,
  76,52,54,54,130,49,134,134,134,6,108,222,178,5,31,187,251,30,36,167,166,160,184,164,20,205,205,205,200,45,152,131,250,198,70,124,247,187,255,9,
  145,139,48,153,205,200,76,74,194,31,254,248,127,88,176,104,49,166,101,102,97,243,230,205,8,4,2,104,168,175,7,23,69,100,101,103,163,167,187,27,
  61,61,61,113,247,140,152,170,208,48,38,79,180,23,134,198,27,74,5,113,252,9,82,68,243,152,105,123,34,145,139,63,134,228,1,14,204,206,201,197,
  205,183,220,10,155,221,142,35,135,15,225,200,161,131,81,202,149,241,195,61,180,218,90,178,131,161,190,174,22,13,245,117,154,125,210,210,210,144,154,150,
  142,172,172,44,216,108,54,100,102,102,193,106,179,97,94,225,124,88,109,182,168,185,161,187,187,195,15,44,6,6,103,171,19,62,141,72,6,90,157,78,
  69,56,51,22,123,162,85,107,75,139,102,110,235,72,215,167,177,114,207,99,205,99,143,167,19,108,100,71,196,181,26,37,59,244,195,244,137,118,236,217,
  48,205,29,22,100,177,199,163,124,30,85,148,178,232,162,72,211,161,180,219,177,118,221,229,88,182,252,50,164,167,79,129,203,213,137,157,239,237,192,217,
  211,167,225,241,122,12,239,119,228,124,219,232,34,142,233,4,117,93,109,45,234,106,107,53,215,44,45,53,13,169,105,105,200,202,146,234,119,86,86,54,
  108,54,43,230,21,22,194,102,179,129,115,224,242,8,231,34,151,132,105,119,151,242,139,173,78,85,61,151,207,201,233,140,172,251,225,126,148,182,67,229,
  12,213,115,68,6,151,103,64,68,199,32,194,65,52,136,251,16,71,161,29,88,185,28,173,178,75,98,116,232,88,181,122,53,126,243,191,255,139,175,62,
  254,56,206,157,59,7,127,48,136,55,223,252,7,186,221,110,156,60,115,26,23,47,94,66,91,91,27,192,57,114,114,102,195,225,112,160,163,163,3,173,
  173,173,240,7,130,72,77,75,69,118,118,54,242,114,243,96,119,36,161,172,172,12,121,121,121,56,125,250,52,190,250,181,175,1,0,190,252,165,47,197,
  46,64,6,73,143,57,7,56,68,233,51,81,122,216,50,129,65,144,135,152,250,243,88,80,65,156,16,146,20,161,57,118,134,243,150,99,122,206,180,133,
  43,254,192,225,76,241,134,94,179,105,51,22,47,89,2,159,215,139,87,94,124,1,117,117,181,134,223,99,113,219,33,151,99,221,249,116,201,94,144,122,
  245,241,85,59,164,166,165,33,45,45,13,0,144,41,11,214,240,107,187,178,223,202,149,171,140,31,226,17,43,184,89,100,134,147,40,219,244,245,71,233,
  44,134,98,85,14,49,162,40,34,24,12,106,66,201,113,206,35,231,84,170,61,143,234,135,163,106,191,174,174,46,156,250,240,36,46,86,86,24,122,153,
  88,212,39,105,28,101,165,95,219,85,101,23,17,89,211,135,181,236,198,182,131,69,196,24,77,204,166,97,235,125,198,238,132,232,68,105,212,235,162,59,
  197,244,244,41,216,112,205,53,88,176,96,33,108,118,59,90,154,155,241,214,222,55,112,246,204,233,196,239,47,143,189,159,32,71,8,136,102,65,87,183,
  36,42,67,245,92,191,87,90,68,61,151,234,118,150,170,206,3,192,202,85,171,116,130,51,74,230,45,165,30,171,3,218,171,174,181,126,63,249,181,40,
  138,16,131,65,112,72,94,235,208,76,104,245,186,36,117,29,212,138,83,14,62,64,49,58,80,59,186,187,186,112,242,228,113,84,84,84,160,203,229,26,
  249,178,59,161,196,104,34,115,132,66,21,145,243,168,46,245,68,40,43,45,69,109,109,45,30,248,204,103,112,246,220,57,28,62,124,4,47,189,242,42,
  42,47,86,162,181,181,21,118,155,13,11,22,44,192,130,249,133,88,176,96,1,68,145,163,161,161,1,117,117,181,168,170,170,70,115,83,35,42,202,203,
  113,212,114,20,135,15,231,96,222,220,57,88,126,217,114,108,218,178,5,153,89,89,120,238,217,103,163,206,29,49,234,93,133,23,30,134,210,87,132,39,
  175,243,160,20,67,140,201,43,66,227,205,39,107,148,254,148,136,236,129,135,123,1,60,225,178,56,34,231,166,105,12,135,246,55,67,71,179,217,172,88,
  185,106,53,86,174,92,5,155,221,142,226,243,231,176,103,247,46,205,176,60,83,121,156,120,172,243,236,231,183,226,181,67,26,198,147,26,89,35,65,28,
  139,156,156,220,168,237,177,218,14,6,41,60,213,166,205,215,162,174,182,70,241,234,24,107,6,54,172,157,59,81,20,181,215,82,115,157,184,94,115,70,
  144,154,154,134,162,37,75,96,183,217,176,113,211,102,108,220,180,25,149,149,21,184,88,81,129,226,226,243,6,237,15,143,209,40,69,185,231,9,150,121,
  62,2,101,55,30,59,20,91,228,122,174,182,171,127,155,56,134,191,25,85,39,8,224,241,73,99,131,69,180,11,22,44,196,154,117,151,35,47,63,31,
  128,52,191,243,216,209,35,168,169,174,142,46,81,250,141,149,173,202,178,20,229,154,25,38,31,139,114,209,244,231,47,205,43,117,41,158,213,68,28,138,
  57,185,185,136,237,209,15,11,189,172,236,233,216,116,237,181,168,85,213,115,253,87,196,208,115,64,31,99,60,134,93,220,104,4,103,0,221,27,214,95,
  64,123,249,69,122,218,20,20,45,93,10,155,221,142,77,155,183,96,243,230,45,74,84,131,243,231,206,70,220,148,201,36,1,6,244,60,31,13,207,104,
  106,106,42,190,252,149,175,224,103,63,249,9,2,129,0,246,239,63,128,214,182,54,236,219,191,15,7,14,30,68,123,91,27,28,14,7,22,46,88,128,
  229,203,150,97,225,194,133,72,75,79,131,187,215,13,87,87,23,28,73,73,200,205,201,69,106,74,42,90,156,45,104,105,113,162,181,173,13,23,47,94,
  68,109,93,45,42,42,43,177,116,233,82,120,189,62,184,221,110,112,0,27,55,110,4,0,236,217,179,199,160,64,234,151,79,50,77,150,13,117,10,51,
  197,19,18,12,42,243,159,24,185,230,39,182,127,212,40,91,143,74,9,14,197,195,61,45,45,29,87,92,185,30,243,10,11,97,183,219,80,87,91,139,
  67,135,14,106,30,10,44,90,99,159,192,34,159,225,182,67,143,177,55,215,216,142,218,218,90,108,190,118,11,234,106,107,113,240,192,254,81,175,79,134,
  226,62,142,107,149,147,147,139,37,75,151,226,31,111,190,1,175,215,139,149,171,86,97,222,188,66,20,206,43,84,132,105,101,101,5,42,43,42,226,145,
  70,3,118,172,140,214,61,143,101,71,200,41,202,66,62,250,177,214,60,42,1,213,185,28,235,58,142,107,36,95,103,155,221,142,101,203,46,195,154,117,
  235,148,249,160,199,143,30,197,241,99,71,224,234,236,28,146,242,203,226,40,11,140,177,184,203,170,250,252,25,226,16,177,81,206,63,52,215,52,158,50,
  80,87,87,139,205,91,164,122,126,40,74,61,215,68,0,24,108,89,77,192,142,68,202,114,110,94,30,150,44,91,134,55,94,125,5,94,175,7,171,86,
  175,69,225,130,5,40,156,191,0,155,175,187,14,21,229,229,168,44,47,71,121,121,25,61,68,135,85,140,14,34,5,211,170,85,171,113,211,205,55,35,
  55,47,15,231,139,139,177,107,215,110,236,222,187,7,173,45,78,204,155,55,23,91,111,184,1,115,230,20,32,45,53,13,76,96,232,237,117,163,161,161,
  17,62,175,79,242,90,0,176,218,108,200,152,54,13,105,233,233,200,207,203,71,95,95,31,58,92,46,52,53,54,162,166,182,22,239,238,216,129,139,23,
  47,226,186,45,91,240,192,167,63,141,235,174,187,30,213,213,85,216,179,107,183,241,243,69,147,115,86,219,251,12,9,80,38,123,74,213,243,177,68,46,
  202,105,192,181,162,84,223,32,16,113,187,87,70,246,187,9,52,64,60,170,135,65,157,45,132,199,125,78,105,233,233,152,87,88,136,162,37,75,144,149,
  149,13,0,168,172,168,192,135,39,79,160,174,174,54,252,192,238,207,94,30,227,58,232,196,199,112,216,209,223,211,51,234,8,2,55,22,128,156,115,195,
  96,225,12,35,155,174,100,40,174,149,179,165,5,59,222,121,7,0,228,123,189,84,249,23,186,223,117,117,181,168,172,168,64,87,151,43,161,107,213,239,
  53,31,139,247,28,161,249,248,241,9,108,141,120,27,209,182,148,129,43,49,158,88,204,19,200,158,62,29,107,214,174,195,210,101,151,41,30,198,127,254,
  227,13,148,149,149,133,87,198,15,88,252,104,237,143,231,177,171,172,65,143,243,252,251,19,169,131,87,247,186,211,80,121,204,120,180,243,231,137,223,135,
  145,182,67,61,69,167,185,165,5,111,191,253,22,240,246,91,40,156,191,0,75,151,45,195,178,101,151,97,153,92,38,202,203,203,80,91,83,131,138,178,
  50,184,162,14,229,211,243,124,196,61,163,123,118,239,198,47,127,254,11,172,92,179,26,111,191,243,14,42,202,43,176,106,197,10,44,92,176,0,153,211,
  50,97,50,155,224,243,251,209,211,211,139,62,79,31,124,62,159,60,143,11,16,121,120,8,205,108,50,65,96,2,204,102,51,108,118,59,28,14,7,178,
  166,77,67,81,81,17,58,58,59,81,89,89,129,55,254,241,15,220,120,221,245,40,43,43,195,147,63,252,161,46,229,90,244,158,20,215,199,49,85,175,
  82,85,165,189,11,229,2,23,185,168,205,187,76,76,56,226,153,110,209,159,39,107,118,78,14,230,21,206,151,87,187,102,129,49,134,46,151,11,71,14,
  29,66,241,249,115,154,5,65,67,113,190,195,101,7,231,195,147,77,68,10,249,195,134,229,216,35,117,207,195,161,139,152,230,251,149,21,146,55,212,110,
  183,35,39,39,23,243,230,75,113,96,11,231,207,199,166,205,215,162,91,14,177,213,226,108,65,93,109,45,156,45,45,19,240,158,199,63,197,98,180,219,
  81,37,229,10,139,84,83,54,187,13,243,231,47,192,154,181,235,144,61,125,58,0,224,220,217,51,56,119,230,12,106,106,170,135,76,248,36,218,12,48,
  205,12,222,232,231,63,188,202,62,158,136,240,44,122,86,177,4,238,195,152,176,67,71,69,121,25,42,203,203,97,179,219,145,151,151,135,194,249,11,48,
  127,193,124,204,159,191,0,215,110,185,14,46,151,11,181,53,53,104,105,105,150,254,54,55,211,195,117,164,197,232,61,31,191,23,101,165,101,184,112,225,
  2,242,11,10,240,143,127,188,133,212,212,84,60,244,224,131,240,251,125,240,249,253,232,114,117,193,231,245,194,23,240,195,239,247,75,158,80,81,42,112,
  33,33,26,90,80,16,214,137,210,194,34,171,213,10,171,213,42,121,78,45,86,164,166,164,160,248,252,121,188,243,238,59,72,79,79,199,250,171,174,194,
  193,253,251,177,237,214,91,240,230,235,111,192,221,215,23,46,103,250,148,99,209,222,107,210,194,69,10,84,81,20,73,144,78,112,65,106,244,96,54,34,
  45,93,90,161,158,43,135,89,145,230,85,73,180,180,180,224,200,225,67,168,172,168,24,144,232,232,207,19,197,226,76,188,48,208,249,182,195,85,190,25,
  27,59,66,116,48,215,138,233,196,169,26,175,215,171,12,213,3,239,32,43,59,27,57,57,185,200,205,203,197,188,249,243,81,180,116,169,242,157,186,218,
  90,184,92,46,56,91,90,224,108,105,65,75,75,179,46,172,215,56,188,231,49,194,58,141,149,123,30,225,101,148,197,80,246,244,233,88,179,102,45,230,
  207,95,0,155,221,14,151,203,133,3,251,247,225,220,153,51,112,185,58,7,47,122,6,32,64,99,117,94,245,231,111,236,166,28,172,168,75,252,252,35,
  234,121,63,95,26,143,118,120,61,30,148,151,149,161,188,172,12,111,191,37,149,157,220,188,60,228,230,229,97,254,130,249,88,186,108,153,178,111,109,77,
  13,92,46,23,90,90,154,209,210,220,140,150,150,22,85,188,89,18,163,253,144,248,56,253,231,30,124,16,14,135,3,187,118,237,198,225,163,71,176,103,
  239,94,220,186,237,102,164,166,165,162,180,180,20,158,62,15,124,62,31,184,60,252,205,57,199,148,244,41,96,130,9,179,103,205,134,96,50,195,231,243,
  195,221,231,134,187,183,23,93,221,46,244,185,221,240,251,189,154,212,96,156,139,176,88,204,72,183,166,3,0,170,107,107,177,123,239,30,220,120,221,245,
  248,226,191,126,25,185,185,185,112,182,180,96,231,206,157,170,85,166,170,197,20,92,138,219,24,142,51,7,213,28,82,30,41,66,67,179,198,101,183,171,
  40,6,21,47,41,145,72,121,26,141,239,14,252,121,170,126,176,167,165,167,35,43,59,27,217,217,217,74,140,63,171,106,5,170,179,165,5,39,79,158,
  80,22,231,68,19,20,3,191,2,82,125,100,44,177,161,44,189,29,163,95,10,56,192,184,74,171,140,157,121,46,241,95,43,46,183,23,241,181,145,206,
  150,102,56,91,154,241,225,73,41,197,113,86,118,182,82,150,178,178,167,163,104,233,18,48,182,84,35,102,157,45,45,114,194,2,151,82,158,226,245,176,
  140,238,61,231,49,189,183,99,173,158,3,210,92,208,249,243,231,99,213,154,181,138,23,180,188,172,12,231,207,158,69,121,89,217,224,84,100,2,102,26,
  77,163,136,183,158,40,173,2,143,83,136,13,115,83,30,154,138,195,18,188,183,99,210,14,196,103,71,75,115,19,90,154,155,112,226,216,81,0,146,56,
  205,206,206,70,214,244,233,200,206,158,142,37,203,150,98,9,212,245,220,131,150,102,169,158,75,222,212,234,132,234,249,120,124,158,143,152,103,244,246,109,
  219,240,175,95,253,42,42,42,43,241,183,191,255,29,73,73,73,152,191,96,1,170,170,170,208,229,146,134,39,67,34,84,20,69,108,221,186,21,215,110,
  190,22,77,45,46,56,91,156,112,58,155,209,213,213,5,139,213,6,139,197,6,193,100,133,205,214,135,62,183,27,30,79,55,188,30,183,52,135,147,67,
  17,130,115,230,206,69,73,121,57,62,252,240,67,112,206,113,231,173,183,97,199,187,239,226,248,209,163,170,75,166,10,125,195,33,11,81,30,241,58,180,
  176,41,236,9,101,17,141,5,227,76,9,222,27,242,146,18,19,11,155,221,142,165,203,150,33,59,123,58,210,210,211,49,59,39,71,147,197,165,203,37,
  9,4,167,179,5,181,53,181,154,96,212,195,233,189,27,140,71,105,176,94,179,177,98,199,72,157,99,236,107,165,10,255,54,0,66,94,208,98,213,182,
  180,244,116,164,167,167,35,55,47,15,105,105,233,72,75,79,199,210,203,46,139,184,86,161,7,87,75,75,51,188,30,175,252,215,131,218,154,154,49,115,
  207,199,162,247,59,90,61,191,118,203,117,40,92,48,31,54,155,29,93,46,23,14,238,223,143,115,103,207,192,213,233,26,3,229,112,48,29,170,81,62,
  119,33,122,252,238,113,103,199,0,139,111,75,179,228,5,197,217,240,202,251,244,116,169,110,231,230,229,43,175,151,40,30,212,13,3,170,231,227,137,97,
  23,163,169,169,169,232,238,238,134,40,138,104,111,239,192,174,221,187,81,95,87,135,13,235,215,195,106,177,40,19,122,67,67,240,243,11,11,113,237,117,
  215,33,63,55,23,130,192,144,148,100,131,197,98,150,99,124,134,99,1,6,131,1,4,131,1,41,175,173,96,133,96,10,128,7,188,74,15,82,228,28,
  83,167,76,133,205,108,129,199,235,69,121,69,5,78,158,58,37,137,76,206,145,148,148,132,62,121,168,190,63,33,26,126,205,116,1,74,165,185,88,74,
  195,174,23,164,193,32,4,193,68,97,198,38,16,94,143,7,155,183,92,167,217,38,8,2,184,40,121,244,211,211,167,72,139,147,230,207,199,21,235,7,
  216,200,25,172,52,54,220,166,18,111,143,127,253,27,67,211,167,149,237,24,138,48,64,3,177,99,253,134,171,177,126,195,213,227,162,44,196,186,86,119,
  221,115,207,176,95,43,61,105,202,195,44,47,226,179,138,178,50,188,246,202,203,163,42,74,13,23,121,142,97,150,168,134,82,211,210,211,177,126,195,6,
  172,223,176,97,84,207,233,223,190,49,132,245,92,20,49,116,30,103,214,127,199,67,24,250,122,62,218,118,220,253,241,251,70,188,12,12,164,158,147,24,
  5,112,207,199,63,142,187,239,189,23,123,247,236,193,135,167,207,224,224,161,67,72,78,73,193,252,5,11,224,116,182,194,231,13,15,205,47,94,180,8,
  55,223,124,51,178,178,178,148,239,59,146,172,176,90,109,48,153,205,210,16,185,236,117,12,248,3,8,4,2,8,6,131,16,131,34,24,179,2,2,7,
  68,31,184,28,43,208,98,49,35,47,63,23,229,149,23,209,217,217,137,15,79,125,136,143,110,221,138,207,126,254,243,152,157,147,131,31,126,255,251,240,
  244,121,52,65,161,149,215,33,129,170,10,28,13,213,40,168,218,75,202,185,126,136,52,188,200,73,20,69,8,38,242,144,78,36,14,238,223,31,189,113,
  52,138,193,203,227,11,175,18,87,243,216,79,94,239,33,19,90,19,196,142,17,17,165,227,224,90,213,246,179,176,102,36,68,41,99,12,38,147,105,92,
  140,24,121,61,158,152,245,60,161,123,61,248,11,55,204,101,87,212,109,76,88,175,69,221,129,133,202,214,136,180,87,227,223,142,193,82,59,232,5,116,
  19,88,140,150,149,150,162,188,172,12,185,121,249,248,249,47,255,7,174,206,78,44,191,108,57,174,90,191,30,85,85,85,202,208,252,156,57,115,112,235,
  173,183,34,35,35,67,243,125,171,213,132,212,180,84,152,154,45,210,202,58,145,35,16,8,194,31,8,192,239,15,32,24,8,40,129,170,205,38,27,68,
  65,64,32,224,5,247,251,192,1,20,206,43,196,197,170,106,4,2,1,212,55,54,162,180,172,12,185,179,115,112,244,240,97,112,49,90,240,126,105,13,
  159,20,115,14,0,147,243,127,203,243,73,153,164,82,181,195,246,70,243,73,33,121,76,196,160,168,233,77,17,227,93,140,238,75,168,145,28,106,175,210,
  104,9,174,137,96,7,93,171,248,127,115,56,236,24,79,247,63,145,122,62,81,234,9,181,87,147,175,189,26,51,98,52,145,155,166,222,55,222,239,237,
  222,181,11,213,213,213,248,212,103,62,139,243,23,138,145,156,146,130,133,11,231,195,102,183,163,167,183,23,156,115,56,28,14,44,94,188,24,233,233,233,
  134,125,147,212,20,59,172,86,171,180,64,136,115,4,2,1,248,3,126,217,51,42,74,67,245,76,74,217,105,53,37,193,196,24,122,253,62,0,82,70,
  148,41,105,105,104,235,232,64,159,219,141,139,151,170,80,145,91,129,119,254,249,86,100,239,39,148,240,65,179,38,73,37,68,89,56,157,162,209,58,190,
  232,197,150,131,139,180,168,105,64,30,166,17,106,116,38,139,192,33,59,232,90,141,182,29,84,207,39,190,189,212,94,17,137,214,243,97,85,71,11,22,
  46,4,0,172,90,189,6,23,74,74,208,217,217,137,25,211,167,99,229,202,85,104,107,107,131,207,231,3,99,12,75,151,46,197,242,229,203,97,50,153,
  12,143,99,183,155,97,183,219,33,48,1,193,96,16,126,191,31,126,159,36,70,69,46,130,129,193,100,146,98,142,154,77,38,88,237,14,8,242,240,185,
  197,106,193,244,233,211,97,18,4,4,130,65,180,181,183,1,140,193,145,156,140,180,180,52,216,108,86,229,162,69,132,142,82,255,69,232,47,15,207,17,
  51,200,147,27,18,166,70,105,226,212,11,93,8,130,32,8,130,32,136,97,22,163,223,126,226,9,28,61,121,18,183,221,121,7,206,157,63,15,187,205,
  134,37,139,23,99,214,204,153,104,117,58,33,138,34,146,146,28,200,206,158,129,100,71,42,68,209,120,201,132,192,24,210,211,82,32,8,2,2,129,0,
  124,62,31,252,126,63,184,24,22,162,22,139,5,86,171,5,86,155,13,201,201,41,176,217,147,148,24,165,153,153,153,72,77,78,6,56,135,219,221,135,
  150,150,22,220,251,241,251,240,192,103,62,139,217,179,115,148,225,122,46,139,79,141,48,133,94,144,202,255,211,252,69,164,48,141,236,38,80,79,139,32,
  8,130,32,8,66,199,176,206,25,253,253,111,127,139,5,11,23,98,213,218,181,168,168,168,192,140,25,211,177,122,205,26,184,221,82,158,121,46,114,216,
  109,118,4,2,128,179,205,133,100,135,29,54,155,5,22,179,9,38,147,118,142,101,90,154,3,22,139,5,1,191,31,158,62,15,130,193,32,152,32,192,
  108,50,195,98,53,75,139,156,76,38,201,59,106,54,33,35,35,11,61,61,221,224,34,71,122,122,58,178,167,79,71,119,111,47,252,126,31,58,58,59,
  80,86,86,138,246,182,54,52,55,55,133,5,162,106,172,93,26,178,151,6,229,67,175,89,40,134,32,151,230,141,114,37,63,132,60,199,84,23,113,44,
  34,99,147,74,140,210,92,20,130,32,8,130,32,136,97,22,163,123,118,239,198,158,221,187,241,211,37,75,113,169,234,18,174,219,188,25,179,103,207,66,
  117,85,53,124,62,105,78,103,114,114,10,56,231,168,175,171,135,197,98,133,213,102,133,35,41,9,73,73,54,164,164,36,33,217,97,7,99,128,213,98,
  129,195,145,4,183,219,13,143,167,15,86,171,21,54,155,13,54,155,77,241,138,154,4,19,4,147,0,147,201,12,147,28,110,7,0,236,118,59,102,204,
  152,129,214,214,86,184,186,187,225,114,117,193,34,152,112,254,220,89,37,124,138,97,42,50,189,32,5,147,115,212,235,230,145,42,171,235,161,139,69,106,
  140,118,245,61,65,16,4,65,16,4,137,209,33,39,53,53,85,153,51,26,8,4,16,240,251,49,119,222,60,184,123,221,232,238,233,9,231,154,7,224,
  241,120,17,8,180,43,97,63,76,102,51,44,102,11,146,83,146,145,150,234,128,205,102,133,205,102,69,74,74,50,60,30,15,82,83,83,145,156,156,12,
  171,44,68,205,102,179,252,93,166,100,63,178,217,172,146,240,3,135,192,4,164,164,164,96,234,212,41,232,112,185,208,231,233,131,195,49,27,89,89,89,
  8,248,3,232,232,236,80,92,151,41,41,201,176,88,172,96,2,131,192,4,213,95,65,138,117,42,48,48,38,200,191,195,194,185,168,153,0,198,0,159,
  223,143,142,182,54,249,183,117,194,84,237,41,37,65,74,140,22,186,184,152,60,60,255,68,41,155,186,222,147,42,244,25,51,72,117,169,15,254,204,70,
  38,2,10,217,49,182,236,32,168,158,147,29,84,207,199,154,24,93,176,112,33,158,250,221,239,0,0,31,236,218,13,171,197,2,175,199,3,151,203,133,
  128,63,160,204,201,244,251,253,232,237,237,13,95,96,166,185,180,74,108,58,179,89,128,201,108,194,229,151,175,69,32,16,68,48,40,34,40,135,116,210,
  4,20,147,95,138,162,8,14,46,205,7,5,135,32,8,82,112,114,46,133,134,74,77,75,195,213,215,108,68,107,171,19,123,118,237,86,126,183,160,96,
  14,50,166,77,131,201,108,150,135,252,205,202,235,208,52,0,147,201,164,252,19,66,127,5,1,38,65,64,115,75,51,222,121,235,45,67,239,168,122,32,
  159,196,40,53,238,163,209,152,104,231,70,75,245,39,84,71,148,185,211,202,107,81,179,31,56,192,76,2,4,38,200,245,137,129,9,130,84,79,5,117,
  167,44,92,210,135,171,140,147,29,99,203,14,170,231,84,207,201,14,170,231,35,46,70,67,23,53,22,165,37,37,120,248,243,159,7,0,220,115,223,39,
  224,245,249,80,81,81,129,162,162,37,96,2,67,48,16,4,231,28,94,175,23,221,61,221,96,140,41,241,66,69,145,43,171,230,125,62,31,188,30,15,
  60,94,47,242,243,114,113,219,109,219,192,24,208,231,246,160,215,237,65,79,175,27,94,143,23,62,191,15,98,48,40,253,19,69,244,121,60,154,133,73,
  125,125,110,116,116,116,42,30,205,246,182,54,156,58,121,18,126,191,79,41,8,12,12,149,149,21,168,169,169,81,110,184,226,21,85,121,74,25,147,11,
  132,236,33,101,2,83,10,134,223,231,55,184,54,60,212,76,201,141,28,83,4,51,99,20,238,105,160,139,186,226,41,135,67,249,132,226,162,8,145,135,
  95,115,46,133,26,67,40,141,45,135,244,90,94,136,39,114,17,242,23,148,50,20,46,83,114,57,146,203,79,232,1,21,206,176,195,134,243,130,135,175,
  155,178,176,46,84,78,121,248,181,106,42,117,70,102,22,210,166,76,5,0,244,246,116,195,213,209,174,44,236,3,11,77,95,209,222,75,198,216,240,222,
  159,73,108,135,213,106,131,35,37,5,126,159,15,158,62,183,244,32,227,108,116,237,160,122,78,245,156,236,152,248,118,12,83,61,79,40,206,168,214,194,
  216,223,235,238,238,198,137,227,199,1,0,247,127,250,211,72,78,78,65,229,197,139,152,154,145,129,140,169,25,240,251,253,128,44,18,93,174,78,152,76,
  102,37,163,146,63,16,148,22,42,121,188,240,249,188,8,248,3,72,78,118,192,106,181,193,229,234,66,225,188,124,96,26,16,12,138,240,249,3,240,120,
  252,232,234,118,163,175,175,15,62,175,23,129,64,0,21,21,101,82,35,1,192,227,241,160,169,169,25,237,157,157,16,4,1,22,179,5,93,46,23,156,
  45,45,225,94,172,20,209,9,61,221,61,26,113,25,30,134,103,154,6,132,201,233,73,165,236,40,234,215,208,120,68,165,161,122,38,207,51,13,191,134,
  82,158,104,117,189,182,178,242,132,31,30,35,118,110,186,215,170,64,94,145,222,19,185,81,201,158,57,11,142,148,20,0,128,215,211,135,54,103,139,42,
  152,109,168,241,209,165,125,228,124,248,158,81,3,176,35,45,125,10,210,166,76,69,123,103,23,130,65,17,89,211,166,32,24,8,162,167,171,147,236,24,
  5,59,82,82,211,144,153,61,93,217,214,219,211,141,142,182,86,232,198,94,70,214,14,170,231,84,207,201,142,137,111,199,48,214,243,97,93,192,180,113,
  243,102,44,92,184,16,118,187,29,179,103,207,70,121,89,41,166,103,103,99,218,180,105,82,204,80,49,8,143,199,131,158,158,110,88,44,54,57,163,82,
  16,254,160,20,71,212,239,243,65,20,165,225,108,123,82,18,4,65,128,219,237,129,40,114,8,130,20,210,41,201,100,133,205,106,65,114,178,29,62,159,
  31,125,30,73,196,158,56,113,72,81,233,93,93,93,168,171,171,131,40,138,176,90,173,176,90,173,72,73,77,197,188,121,243,208,216,216,8,183,219,45,
  223,207,80,24,39,64,147,35,84,123,167,195,83,64,245,11,154,160,43,31,74,91,20,61,36,62,13,215,79,92,210,166,76,129,35,37,5,237,157,93,
  240,122,253,152,57,125,26,166,76,157,134,206,246,214,113,101,135,35,57,5,94,159,31,165,21,53,210,251,36,187,148,180,162,11,100,199,40,144,145,153,
  133,94,183,7,149,85,117,152,57,61,19,89,211,166,192,211,215,7,111,159,155,42,29,213,115,170,231,100,199,184,100,88,197,232,131,15,61,132,5,11,
  23,162,180,172,12,133,133,133,248,240,212,135,104,106,110,70,71,71,7,82,83,83,17,112,7,224,243,121,225,118,187,97,181,114,4,2,126,73,140,6,
  36,81,42,13,99,3,22,171,21,118,187,13,140,49,244,245,245,161,187,215,131,244,212,36,229,119,4,129,193,42,152,96,49,11,176,219,173,8,6,146,
  80,91,83,163,12,209,183,182,182,162,187,167,87,50,216,100,130,195,110,71,118,86,22,114,115,114,224,118,187,225,118,187,85,29,12,174,234,88,168,220,
  166,208,10,83,30,218,172,154,167,1,85,150,166,216,61,124,70,98,116,18,224,72,78,65,175,219,163,52,38,201,14,59,108,22,243,184,180,197,235,245,
  41,175,131,193,32,204,130,137,236,24,37,4,65,64,71,103,23,122,221,30,84,213,52,34,107,218,20,88,172,86,18,163,84,207,169,126,144,29,147,75,
  140,234,150,12,69,229,137,239,124,7,101,165,165,248,200,77,55,97,233,146,37,120,238,249,231,80,87,95,143,217,117,181,88,185,114,149,228,145,12,6,
  224,243,244,65,12,2,65,81,132,24,148,22,39,133,220,187,130,201,4,171,213,38,173,112,103,12,30,143,23,221,93,61,72,75,73,138,152,4,206,24,
  131,217,196,208,212,216,128,186,250,90,121,177,82,0,77,141,77,202,144,189,205,102,67,90,90,42,222,127,255,61,88,173,86,41,120,62,194,14,76,57,
  146,19,68,0,2,83,77,222,48,146,152,44,118,18,208,254,146,132,134,16,39,185,32,229,163,244,221,129,254,162,62,183,22,87,189,229,218,23,8,6,
  131,49,14,37,205,251,9,165,157,85,119,116,198,154,29,177,15,69,118,140,150,29,129,80,249,226,99,197,14,170,231,84,63,200,142,137,111,199,208,215,
  213,97,93,61,83,86,90,170,252,93,189,106,21,178,178,178,224,118,187,209,218,218,6,81,20,97,54,153,192,69,17,126,191,7,94,111,47,46,187,108,
  9,138,150,44,196,148,169,233,48,153,4,8,38,41,68,147,213,102,133,197,98,1,24,131,223,239,71,79,79,31,252,1,227,202,239,118,187,241,215,191,
  253,13,125,238,62,112,206,209,221,213,133,182,206,14,73,121,155,205,72,77,73,145,82,137,250,3,112,187,221,210,220,213,136,203,23,154,80,44,175,126,
  83,138,16,87,254,131,190,72,241,248,110,7,143,86,168,8,130,32,8,130,32,38,33,3,19,163,9,104,167,7,31,122,24,127,248,191,255,67,70,70,
  6,150,46,89,2,175,215,139,202,202,74,148,150,148,192,102,179,129,203,189,202,43,174,88,139,237,119,222,134,79,63,112,63,86,44,95,10,187,221,14,
  139,217,12,179,217,10,171,69,202,57,47,48,38,207,51,237,131,215,235,55,252,189,247,223,127,31,53,213,53,242,234,82,142,154,218,26,229,51,187,205,
  134,217,51,102,32,59,59,11,87,95,115,53,146,146,28,145,38,169,211,204,171,87,183,113,149,102,228,6,255,212,75,224,116,127,35,196,42,31,19,93,
  255,177,3,159,188,54,115,178,131,236,152,44,118,80,61,39,59,200,14,170,231,81,48,15,247,15,166,166,166,224,196,241,227,56,123,230,44,46,91,118,
  25,142,29,63,14,159,223,15,143,199,131,148,148,20,116,117,117,225,234,107,54,226,230,155,111,66,178,35,9,129,64,16,89,217,217,72,146,179,45,153,
  76,2,76,130,9,76,96,74,28,46,191,223,15,143,199,7,71,146,13,38,83,120,120,251,173,127,190,133,119,222,121,23,29,157,29,202,124,209,139,23,
  47,1,0,76,130,128,180,212,84,76,157,50,5,213,85,85,200,201,201,69,192,239,215,170,208,240,202,36,41,92,2,12,70,226,163,141,188,135,3,205,
  105,23,64,233,255,66,247,90,249,58,7,3,205,29,29,119,15,56,62,65,236,34,59,200,14,170,231,84,174,200,14,178,99,148,24,246,89,214,63,121,
  242,199,0,36,239,231,23,254,223,255,195,150,205,155,177,227,189,247,80,121,241,34,150,45,91,134,173,91,63,130,107,183,108,65,74,178,3,126,191,31,
  197,37,37,216,187,103,31,92,157,46,36,57,28,176,90,45,176,88,45,16,4,41,166,167,24,12,194,239,247,193,237,246,32,57,57,9,102,19,71,99,
  99,35,222,125,247,93,28,59,118,12,221,61,61,225,133,75,78,39,130,92,10,120,63,117,202,20,44,156,87,136,246,214,54,212,84,87,163,188,172,220,
  240,254,133,87,196,171,230,99,244,147,222,19,253,125,78,16,4,65,16,4,65,12,165,24,141,119,9,83,152,148,148,100,164,167,166,226,242,117,151,163,
  162,178,18,181,181,181,232,116,117,98,234,212,41,242,188,81,73,136,190,244,226,139,112,182,56,97,177,216,96,177,90,192,152,29,162,24,254,173,160,40,
  194,235,241,160,181,213,9,183,187,11,157,29,173,216,187,119,47,206,156,57,131,160,24,84,101,58,224,184,120,73,242,138,218,172,86,100,79,203,196,180,
  140,12,84,182,183,233,186,14,204,160,59,161,78,223,41,191,143,18,187,43,28,206,126,48,221,17,6,26,167,31,59,93,192,156,220,60,220,117,247,61,
  154,109,162,40,42,105,109,253,1,105,206,113,232,175,102,155,223,143,64,192,143,64,32,128,234,218,90,52,181,180,170,206,148,35,51,43,11,215,110,222,
  12,139,28,98,204,98,181,192,108,182,192,98,49,195,100,10,101,247,18,98,6,196,126,241,249,231,80,167,154,126,18,139,180,244,116,164,165,165,107,58,
  78,34,23,33,6,69,233,175,40,74,65,190,69,14,81,20,33,138,65,229,181,180,93,132,217,98,134,55,224,213,216,97,177,88,144,149,149,45,101,32,
  19,4,48,57,11,153,38,57,4,88,204,78,154,215,235,145,98,253,78,34,59,114,114,243,34,58,178,137,218,193,53,139,32,36,59,28,201,14,36,217,
  172,131,178,195,217,210,2,175,215,51,105,234,121,68,153,26,67,229,42,145,50,5,0,89,217,217,176,217,236,99,178,126,116,117,185,208,229,114,81,123,
  53,14,237,24,201,186,106,30,137,230,64,20,57,158,254,251,223,97,179,217,176,227,221,29,216,188,105,51,118,188,183,3,205,205,45,56,125,250,52,2,
  129,32,210,210,211,240,218,107,175,161,177,161,1,28,128,63,224,135,199,219,7,147,32,162,167,187,11,193,128,31,30,79,31,122,186,187,192,185,136,158,
  158,46,180,183,181,162,169,185,25,238,222,94,41,207,61,151,50,55,121,250,250,208,221,211,131,250,198,70,8,130,128,172,105,153,152,53,99,6,24,3,
  86,174,90,133,146,226,11,168,170,170,138,188,95,70,67,240,234,152,178,218,40,78,97,217,170,30,221,87,237,175,25,230,215,197,63,86,255,142,178,203,
  36,119,176,242,81,125,52,169,26,79,151,11,135,14,30,208,61,163,184,28,233,65,250,39,6,131,8,138,162,246,189,188,45,244,58,96,176,48,205,237,
  238,197,249,115,103,165,20,179,38,19,76,102,179,210,176,8,130,180,104,143,49,65,147,86,208,232,252,226,165,104,201,82,92,185,254,170,65,139,234,222,
  62,175,230,184,233,83,210,113,217,210,101,81,68,181,9,38,147,89,105,36,163,25,82,87,91,139,23,159,127,118,82,217,49,20,157,156,11,101,101,17,
  199,205,207,207,71,65,126,193,160,236,72,164,147,51,17,234,185,190,76,141,165,114,149,72,153,2,128,77,155,183,32,39,55,119,76,214,143,67,7,15,
  224,176,174,61,165,246,106,124,216,49,146,146,212,60,144,212,77,156,247,191,0,220,110,183,227,202,171,214,99,215,206,15,0,112,60,253,151,191,160,178,
  162,2,101,165,165,248,248,253,159,196,149,87,92,137,127,188,245,15,156,62,125,6,139,23,45,130,195,145,132,64,48,40,45,92,178,88,164,27,24,20,
  209,208,80,7,46,114,52,212,215,104,188,158,161,5,74,92,228,8,4,3,240,120,60,240,120,60,112,187,221,104,106,108,84,132,104,122,90,26,10,114,
  115,209,210,220,140,178,210,82,204,159,95,136,250,250,250,176,160,52,72,94,144,156,236,128,195,225,64,91,91,155,170,79,194,52,87,152,49,253,69,231,
  96,156,41,193,238,21,129,201,194,2,21,60,188,13,58,49,27,202,198,196,249,228,83,163,234,178,148,104,121,28,142,64,4,46,151,11,135,14,28,136,
  168,90,154,242,167,251,7,57,141,45,7,7,228,52,180,217,179,103,71,28,187,183,183,23,53,23,43,37,193,25,53,195,23,48,84,41,52,206,159,59,
  135,186,154,26,77,239,90,148,123,222,162,200,149,94,120,232,159,242,94,213,139,119,164,164,70,28,183,179,179,19,187,62,120,95,22,210,161,222,121,56,
  117,174,148,42,23,49,231,65,123,188,222,184,239,223,68,177,227,197,231,158,213,151,170,132,237,72,157,154,17,113,220,234,170,42,92,170,172,24,148,29,
  45,45,45,195,26,216,99,172,213,115,125,153,26,75,229,42,145,50,5,0,187,62,216,9,187,205,54,38,235,135,171,171,107,210,213,243,137,98,199,72,
  214,243,97,25,166,159,51,103,46,62,114,211,77,240,120,60,216,133,157,224,156,227,213,151,95,86,206,180,177,161,30,183,110,219,134,96,48,136,67,135,
  14,97,223,254,253,240,122,61,152,50,37,29,83,167,102,32,37,37,5,83,167,76,65,198,180,105,176,152,101,97,42,123,62,67,198,5,252,1,184,251,
  220,112,117,186,208,209,217,129,182,182,54,116,119,119,195,231,247,67,228,28,86,171,21,217,153,153,152,151,95,128,25,217,217,168,190,120,17,1,191,31,
  23,138,47,40,231,105,181,89,97,179,218,208,211,211,173,60,252,51,51,51,145,151,159,143,150,230,230,56,53,1,87,5,200,143,182,138,41,254,155,56,
  57,61,163,227,32,2,33,143,227,117,127,213,36,142,217,45,225,153,33,67,99,87,151,171,19,93,174,78,173,167,95,213,145,211,191,54,18,213,22,91,
  82,132,29,126,159,31,109,45,45,81,69,181,210,28,14,81,121,158,40,118,212,234,61,143,3,176,35,117,74,70,196,49,122,123,221,232,233,234,28,49,
  59,38,66,61,143,40,83,227,184,92,57,91,154,39,132,29,212,94,141,45,59,70,178,174,14,75,156,209,21,171,86,225,153,191,255,205,240,179,235,174,
  191,1,143,60,250,40,76,38,1,133,115,231,98,227,198,141,40,152,83,0,155,221,14,87,87,55,170,170,170,80,90,82,138,178,178,114,148,149,150,162,
  173,189,21,94,159,87,186,105,92,234,93,244,246,246,162,174,174,22,101,101,101,40,41,45,65,229,197,139,112,182,181,193,227,243,129,3,72,178,217,144,
  55,123,54,230,207,157,7,139,201,132,148,212,20,172,187,226,114,197,227,170,136,81,171,13,41,169,41,202,157,115,56,28,176,218,108,168,186,116,81,163,
  10,229,89,24,144,211,207,199,16,140,180,136,137,32,8,130,32,8,34,17,134,101,53,253,171,47,191,36,123,121,212,65,59,129,235,111,184,1,95,254,
  202,87,80,94,94,142,39,190,245,45,4,2,1,92,191,117,43,62,186,245,35,56,126,226,4,78,157,62,5,151,203,133,160,207,139,22,167,19,237,29,
  237,232,233,233,129,35,57,25,211,179,179,97,177,88,208,221,221,141,150,150,22,52,59,157,232,237,237,13,103,32,129,20,212,62,99,202,20,204,154,49,
  3,211,179,178,209,220,216,136,150,230,102,116,117,185,112,217,242,229,184,252,202,43,112,228,208,97,48,198,144,49,109,154,20,80,223,106,195,204,153,12,
  61,61,61,232,238,238,70,95,159,27,169,105,105,138,182,84,220,221,186,41,24,225,245,37,44,82,142,50,253,71,36,82,39,52,220,40,139,6,217,65,
  118,140,164,29,148,56,131,202,21,217,65,118,76,114,49,186,249,218,45,0,128,142,206,14,156,58,121,210,112,159,121,133,133,120,252,171,95,69,121,89,
  25,190,241,181,175,161,183,167,23,28,28,111,190,250,42,230,204,155,135,235,174,189,22,89,153,153,40,45,43,67,93,67,61,92,46,23,188,62,31,26,
  26,27,97,18,76,232,238,234,2,152,36,26,187,186,186,224,15,4,164,129,112,198,96,54,153,48,37,125,10,178,50,167,97,70,118,54,204,130,128,138,
  210,18,120,60,30,128,1,117,117,117,0,24,86,172,92,129,53,107,215,226,232,209,35,232,237,233,134,40,58,192,192,208,221,211,3,159,215,27,78,69,
  15,217,237,29,82,149,113,104,201,248,229,38,9,83,122,166,105,211,183,145,29,100,7,217,65,245,156,236,32,59,38,115,61,31,18,49,122,233,210,69,
  0,144,196,95,20,42,43,42,240,183,167,159,198,43,47,191,140,222,158,30,201,123,200,1,206,24,46,85,86,162,161,174,30,171,215,174,197,29,183,221,
  10,119,95,31,234,235,235,81,94,94,129,102,103,11,186,187,186,209,236,116,34,224,15,64,228,34,44,22,11,50,210,210,144,226,72,198,212,41,83,144,
  158,150,134,41,105,105,240,251,253,168,174,170,66,103,71,71,132,244,171,175,171,69,48,224,135,187,175,15,98,48,136,158,158,94,128,9,48,155,45,232,
  233,238,150,246,85,220,154,12,130,73,80,60,163,76,59,80,175,28,149,105,255,23,189,108,208,179,129,32,8,130,32,8,98,248,196,104,149,28,207,51,
  58,210,66,158,191,62,253,116,84,31,180,199,235,193,129,125,123,177,111,239,30,20,22,206,71,193,220,57,184,102,195,6,112,0,221,61,221,112,247,186,
  225,243,249,16,8,4,192,57,87,66,225,116,117,118,162,173,181,21,231,170,171,36,49,204,163,171,192,166,166,102,205,246,62,183,91,242,136,66,59,15,
  84,96,76,9,178,175,214,159,140,169,231,140,234,5,169,241,111,178,254,132,41,5,203,39,8,130,32,8,98,50,139,209,68,150,248,107,151,236,247,191,
  255,63,223,122,75,10,3,5,24,207,139,80,231,87,229,225,0,242,21,229,229,40,47,47,83,22,45,205,152,57,83,138,229,24,10,129,192,69,248,188,
  62,116,118,118,2,74,168,39,36,60,217,34,20,35,82,177,135,49,48,6,244,186,221,240,249,253,64,132,55,148,41,251,134,4,41,87,194,53,177,112,
  140,81,217,227,171,95,80,175,95,91,63,217,67,221,27,149,175,184,203,35,143,191,28,14,209,217,41,231,199,67,191,203,85,11,228,117,185,130,57,162,
  21,72,30,81,254,13,13,27,54,119,58,217,49,49,236,48,8,233,48,170,118,80,61,167,250,65,118,76,124,59,134,169,158,99,152,211,129,54,53,54,
  198,222,129,177,112,176,79,131,4,238,33,167,97,83,83,83,88,116,170,254,49,166,154,57,193,185,44,0,7,216,106,177,208,64,60,67,48,16,128,24,
  12,202,243,70,35,118,212,122,51,251,245,108,38,178,47,49,118,149,50,217,65,118,144,29,84,207,201,14,178,131,234,249,112,96,30,149,95,237,79,52,
  50,6,38,123,59,153,188,175,70,239,235,68,93,132,32,77,164,43,29,18,156,161,88,93,242,111,50,213,152,188,230,189,238,28,152,254,24,198,50,148,
  166,141,78,218,231,22,79,248,27,100,7,217,65,15,55,42,87,100,7,217,49,30,236,24,127,98,84,37,48,67,226,140,27,125,174,222,79,45,90,101,
  161,199,101,161,105,36,74,195,247,172,31,81,170,137,33,10,69,104,50,149,168,84,139,80,166,223,55,138,40,54,58,126,191,162,156,24,115,164,165,165,
  163,104,233,18,109,145,18,185,60,85,36,136,96,80,68,48,24,128,40,167,3,21,117,105,64,131,65,41,215,112,64,228,240,246,244,66,61,232,146,228,
  72,70,209,146,165,48,153,77,48,155,165,20,110,130,96,146,255,134,230,42,171,230,43,27,80,124,238,60,186,186,92,3,111,4,163,205,183,81,213,79,
  245,107,101,170,141,122,144,152,235,178,218,170,50,54,132,70,45,134,189,49,159,196,118,168,31,78,234,33,191,209,180,99,194,136,79,170,31,100,7,217,
  49,129,197,104,156,34,213,104,187,218,59,26,186,216,82,222,119,30,57,100,174,191,225,49,4,99,132,87,83,255,58,244,93,77,154,70,196,239,21,141,
  33,92,233,209,48,134,197,104,122,26,174,184,114,189,102,155,58,183,112,32,160,207,43,28,144,242,9,71,228,22,174,131,171,167,87,115,156,100,135,3,
  69,75,151,194,106,181,194,106,181,192,98,177,194,108,54,195,108,177,192,100,18,226,206,45,28,175,24,45,90,178,20,69,75,150,168,170,132,136,64,32,
  136,96,48,32,253,13,4,16,8,6,17,12,248,149,237,193,64,16,129,96,0,129,64,0,193,96,16,29,157,46,180,117,106,127,47,125,202,20,44,93,
  178,4,22,139,116,238,102,179,25,102,147,25,38,179,116,254,33,113,205,88,116,59,156,206,22,236,217,181,107,82,217,177,253,99,119,235,30,80,137,219,
  209,210,218,22,113,220,188,130,124,100,78,203,28,148,29,123,118,239,130,179,165,101,210,212,115,125,153,26,75,229,42,145,50,5,0,27,55,111,70,86,
  86,246,152,172,31,197,231,207,163,248,252,57,106,175,198,161,29,19,70,140,94,113,229,149,56,245,225,135,232,235,235,83,122,5,203,150,45,67,90,90,
  26,14,28,56,160,136,205,180,180,52,44,42,42,66,93,109,45,234,235,234,20,239,104,94,65,1,166,78,157,10,175,215,139,234,234,106,244,116,119,43,
  234,63,63,63,31,45,45,45,112,187,221,202,239,205,154,53,11,237,237,237,232,235,235,195,140,233,211,225,241,120,148,69,78,0,48,125,198,12,184,58,
  59,97,183,219,49,101,234,84,48,213,170,121,65,16,208,209,209,129,142,246,118,141,248,100,186,215,106,241,73,222,206,137,73,93,109,45,126,254,147,31,
  71,120,219,69,206,149,30,40,23,69,185,167,26,126,13,57,67,88,232,117,246,204,89,6,13,129,19,47,62,255,28,4,198,192,4,65,242,182,135,34,
  55,132,202,27,149,35,130,32,8,98,18,49,108,98,116,198,140,25,184,226,202,43,1,0,135,14,30,84,132,218,210,101,203,144,147,147,131,154,154,26,
  212,214,72,121,154,23,47,89,130,117,151,95,14,206,57,234,235,234,96,179,217,112,243,182,109,240,200,241,70,83,82,82,112,243,182,109,56,118,244,40,
  42,43,42,192,57,71,193,156,57,112,187,221,112,187,221,202,131,60,55,55,23,30,143,7,30,143,7,57,57,57,176,217,237,56,122,228,8,2,129,128,
  34,86,125,62,31,24,32,137,80,65,192,252,249,243,113,177,178,82,121,31,83,136,70,243,134,198,240,138,70,21,168,36,54,136,17,160,248,252,57,173,
  87,98,136,68,181,171,179,19,149,101,165,35,38,170,39,138,29,47,189,240,252,160,59,57,121,115,231,69,28,183,166,170,26,231,206,156,161,78,206,96,
  202,212,56,46,87,17,158,46,170,231,100,7,137,81,137,43,174,188,18,239,188,253,54,54,109,222,28,22,163,50,21,21,21,40,156,63,31,181,181,181,
  96,156,163,176,176,16,151,42,43,21,207,227,218,181,107,209,222,222,142,93,59,119,42,55,166,162,162,2,27,55,109,66,115,83,19,122,122,122,96,50,
  153,52,98,145,67,74,7,26,242,116,154,45,22,48,0,243,231,207,71,73,73,9,0,192,98,177,192,36,8,104,239,232,128,203,229,2,24,67,81,81,
  17,46,94,188,168,17,159,17,2,52,150,32,237,71,96,26,21,137,168,177,241,233,65,65,16,4,65,16,196,36,67,24,142,131,218,237,118,204,152,49,
  3,37,23,46,160,169,177,17,5,115,230,104,4,91,109,77,13,242,242,242,0,0,54,187,29,233,233,233,112,182,182,42,34,109,113,81,17,78,28,59,
  166,241,60,118,180,183,195,217,210,130,188,252,124,128,49,73,140,202,194,51,244,79,35,70,205,102,92,186,116,9,179,102,207,70,122,122,186,34,80,5,
  147,41,220,171,96,12,22,139,69,179,80,169,95,33,202,34,67,59,69,155,31,26,181,135,66,162,115,66,146,240,90,71,206,201,14,178,99,72,236,224,
  227,192,14,170,231,100,7,217,49,241,237,24,83,98,116,209,162,69,138,55,242,212,135,31,98,197,202,149,154,207,91,90,90,144,150,158,142,244,244,116,
  20,22,22,162,174,174,78,227,109,180,217,237,232,238,238,142,24,10,239,237,237,133,205,102,83,134,217,5,157,23,83,48,153,52,98,52,24,12,162,164,
  164,4,171,87,175,142,16,171,138,128,149,61,168,76,191,130,94,21,206,73,47,72,213,66,51,230,240,188,1,36,67,199,249,131,40,198,138,200,184,27,
  9,30,25,210,152,143,112,195,66,118,144,29,4,149,43,178,131,236,24,43,152,19,211,227,241,197,188,186,252,202,43,49,99,198,12,108,220,180,73,217,
  246,206,63,223,66,95,95,159,44,252,128,138,242,114,228,230,230,34,39,39,7,23,43,43,144,154,154,170,236,235,247,249,144,150,158,134,46,151,75,81,
  112,140,115,164,165,165,161,173,173,21,140,1,130,73,0,19,152,156,63,94,54,198,108,6,19,164,96,249,102,179,36,76,155,155,154,48,103,206,28,204,
  159,63,95,22,163,76,19,54,199,98,54,43,153,151,148,223,130,250,53,215,138,78,205,54,174,250,2,143,20,157,76,27,192,202,104,27,161,47,95,9,
  166,102,25,177,107,201,117,191,25,237,117,188,253,92,117,146,7,125,194,7,178,131,236,24,111,118,80,61,167,114,69,118,80,61,31,104,61,31,134,57,
  163,5,5,115,224,241,120,240,221,239,124,91,217,182,105,243,102,92,113,229,149,216,245,193,7,138,210,171,171,173,69,225,130,249,200,201,201,197,193,67,
  7,81,84,84,164,92,211,211,167,79,97,221,229,151,227,253,247,222,83,86,220,79,203,204,194,236,156,28,124,120,242,36,0,134,62,183,27,51,103,206,
  66,115,83,51,192,0,139,197,138,204,204,76,244,201,11,154,76,42,47,232,185,179,103,176,105,243,181,146,32,148,23,41,133,52,164,217,108,150,244,161,
  70,112,178,112,230,121,22,35,200,189,178,1,131,158,52,76,243,69,9,130,32,8,130,152,140,12,104,152,62,150,71,120,249,138,21,56,245,225,135,154,
  109,31,126,248,33,150,175,88,169,168,55,14,160,188,188,28,121,185,121,232,234,234,146,61,160,97,129,119,234,195,83,200,202,202,198,237,119,220,137,181,
  151,95,142,13,27,174,198,173,183,221,134,147,39,78,160,167,167,27,96,64,113,241,5,44,93,186,20,235,174,184,28,203,87,174,196,182,91,110,65,67,
  67,3,122,123,221,0,24,76,130,73,241,146,186,221,110,84,84,84,32,45,61,29,130,28,80,60,148,252,211,28,242,140,170,242,207,171,100,103,212,255,
  35,198,235,200,253,140,246,143,255,154,78,116,6,106,251,72,94,51,174,126,197,227,221,159,199,60,146,118,19,143,187,79,76,118,76,118,59,140,206,106,
  116,237,160,122,78,245,131,236,152,248,118,12,103,125,53,15,245,15,118,118,118,162,228,194,5,205,231,157,29,157,56,114,232,16,166,164,79,193,217,51,
  103,224,234,116,193,227,241,224,192,129,3,112,117,118,130,115,201,83,26,186,166,94,175,23,207,62,243,12,230,206,157,139,204,204,76,116,119,119,227,197,
  23,94,64,119,87,151,34,232,218,219,218,240,250,107,175,33,47,63,31,22,171,21,199,143,29,67,77,77,141,146,175,190,170,170,10,110,89,152,2,64,
  89,105,41,108,54,27,122,85,113,73,193,24,206,158,61,11,245,204,79,22,53,36,19,83,165,29,133,146,225,137,73,145,248,149,115,103,128,42,251,147,
  74,130,170,246,49,162,191,12,169,196,228,18,241,19,197,46,178,99,120,31,109,137,62,126,198,235,253,160,122,78,118,144,29,19,219,142,33,31,166,223,
  29,37,178,255,225,195,135,1,206,209,121,230,140,210,116,158,56,126,92,17,107,181,181,181,138,42,99,242,149,188,120,241,162,20,118,73,149,18,84,157,
  10,180,167,183,23,197,231,207,71,40,58,6,224,98,101,101,132,184,60,119,246,108,196,182,51,167,79,71,134,105,138,245,222,96,17,83,180,247,202,182,
  126,160,33,122,130,32,8,130,32,38,43,35,155,14,84,37,22,13,243,210,71,219,79,253,94,149,115,94,235,113,52,206,87,111,212,65,96,49,206,79,
  47,66,53,251,71,121,223,175,16,37,177,73,16,4,65,16,4,49,6,196,40,36,47,32,31,168,32,5,20,175,105,84,81,170,168,80,30,91,120,26,
  137,201,120,69,231,16,10,81,242,138,142,61,114,114,114,113,199,93,119,105,182,233,115,211,7,2,114,78,122,191,110,155,63,160,228,166,175,169,171,67,
  139,179,93,211,51,154,150,153,133,141,27,55,194,106,177,194,98,181,194,34,231,23,182,88,44,16,212,185,233,245,241,108,85,188,242,226,139,168,171,171,
  141,203,150,180,180,52,164,165,165,171,78,65,4,23,69,136,162,148,237,67,249,199,57,184,24,132,40,114,249,115,121,31,46,194,108,54,195,23,240,107,
  236,48,155,45,200,202,202,82,194,169,153,76,2,24,19,228,76,102,12,2,19,228,197,127,209,115,36,123,189,30,56,157,206,73,101,71,78,78,174,174,
  179,156,184,29,156,235,195,186,0,73,14,7,236,86,235,160,236,112,58,91,224,245,122,39,77,61,215,151,169,177,84,174,18,41,83,0,144,149,149,5,
  155,205,62,38,235,71,87,151,11,93,93,93,212,94,141,67,59,198,129,24,29,196,132,132,80,104,163,40,46,75,166,254,72,222,151,113,213,175,170,135,
  234,213,162,84,181,223,128,188,145,44,44,33,99,11,83,237,30,177,243,212,147,176,27,222,242,52,60,19,99,186,186,92,56,114,248,144,246,151,228,134,
  34,24,12,34,24,12,42,175,197,96,16,65,49,8,49,168,253,76,12,6,17,48,56,61,183,91,154,90,98,54,153,96,50,155,97,50,153,96,146,27,
  150,208,63,117,140,219,104,231,23,47,139,151,44,193,229,87,92,57,104,81,237,246,180,107,142,155,62,101,10,150,46,41,26,148,168,174,175,171,197,203,
  47,190,56,169,236,24,138,78,78,105,121,69,196,113,243,242,242,145,159,159,55,200,78,206,11,168,171,171,155,52,245,92,95,166,198,82,185,74,164,76,
  1,192,53,155,54,97,182,170,163,51,150,234,199,145,195,135,112,228,208,33,106,175,198,161,29,35,89,95,205,137,4,75,229,74,110,213,193,54,16,178,
  200,4,143,113,148,24,43,201,152,254,37,215,138,63,158,224,169,168,142,175,25,168,215,196,15,101,6,231,102,252,131,204,112,223,232,215,97,210,75,81,
  30,46,95,137,7,239,29,250,235,215,213,229,194,145,67,7,35,206,145,115,81,58,63,229,181,84,246,184,40,157,55,151,62,80,236,152,62,115,118,164,
  24,237,117,163,246,210,69,48,38,24,36,87,8,121,202,217,144,205,236,40,62,127,14,117,181,181,154,58,172,244,192,185,182,183,206,229,109,161,94,59,
  151,183,59,84,49,128,67,184,58,59,177,103,215,46,152,76,2,4,193,164,21,210,74,15,61,100,71,180,30,186,55,238,251,55,81,236,120,249,197,23,
  34,218,212,68,237,72,157,154,17,113,220,154,154,106,84,93,172,28,148,29,78,103,203,176,182,71,99,173,158,235,203,212,88,42,87,137,148,41,0,216,
  179,123,55,108,54,219,152,172,31,82,231,153,83,123,53,14,237,24,201,122,62,228,171,233,19,107,90,36,47,39,143,38,212,34,62,99,154,31,231,81,
  93,144,3,86,163,90,153,201,195,94,79,101,59,215,122,76,185,78,35,107,166,11,240,88,14,98,54,36,215,145,132,236,68,113,244,114,77,89,28,42,
  187,186,92,93,232,114,117,233,174,89,156,162,90,148,94,91,237,73,17,199,245,251,253,104,115,182,24,136,106,132,95,15,225,208,192,68,177,67,47,126,
  6,98,135,145,24,117,187,221,232,113,117,142,152,29,19,161,158,27,149,169,241,90,174,156,45,45,19,194,14,106,175,198,150,29,35,137,121,180,126,56,
  52,119,84,89,52,100,36,74,85,43,235,245,243,75,37,97,24,37,182,214,0,221,74,172,159,99,196,26,146,79,36,232,61,205,19,37,8,130,32,8,
  130,144,16,198,204,153,232,87,178,171,69,161,252,217,217,11,197,248,194,99,143,70,124,22,250,92,253,47,30,225,25,241,29,213,241,140,246,29,10,33,
  74,16,4,65,16,4,65,132,25,196,2,166,193,143,155,72,107,145,184,70,213,177,104,195,246,154,169,156,70,94,84,131,183,253,77,215,52,180,75,191,
  139,58,89,61,143,33,68,121,156,54,211,60,209,254,174,251,200,125,119,96,63,99,156,77,131,71,236,195,35,234,73,248,61,71,56,141,130,102,128,94,
  157,61,129,236,32,59,12,236,48,12,122,207,71,217,14,170,231,84,63,200,142,137,111,199,48,214,213,81,247,140,70,12,89,171,188,156,159,248,228,39,
  177,118,221,58,67,21,203,192,240,133,199,30,195,162,69,139,98,11,206,104,255,98,126,69,181,154,57,198,66,249,88,43,158,227,178,149,32,8,130,32,
  8,98,146,51,38,134,233,67,34,109,225,162,69,216,188,101,139,50,84,254,200,163,143,226,190,251,239,55,252,206,194,197,139,240,200,163,143,98,237,186,
  203,193,192,144,150,150,134,219,110,191,29,105,169,105,113,119,10,180,250,180,127,1,170,254,78,162,195,242,36,68,9,130,32,8,130,32,34,49,143,157,
  83,97,120,228,11,95,192,154,117,235,176,97,231,78,204,153,59,23,101,101,101,152,50,53,3,255,250,213,175,34,63,63,31,71,142,28,197,170,181,235,
  240,31,223,206,132,205,110,67,117,77,13,130,98,16,217,51,166,227,134,27,111,196,87,191,254,117,148,148,220,129,210,11,221,82,208,36,117,104,1,110,
  60,192,5,166,26,58,151,2,153,170,66,236,68,10,209,129,204,13,37,33,58,209,8,151,37,125,169,226,161,105,38,92,179,171,230,53,231,28,140,113,
  57,144,152,81,232,48,178,131,236,72,192,14,140,53,59,168,158,83,253,32,59,38,190,29,67,44,70,19,138,247,166,138,51,58,20,161,54,82,83,83,
  241,200,163,95,192,83,191,254,13,186,187,187,113,232,208,33,44,94,178,4,191,254,223,223,162,189,179,19,239,236,120,15,213,213,213,104,110,110,66,71,
  71,7,76,140,33,40,138,48,89,44,152,53,99,38,14,30,58,140,185,115,231,226,171,255,254,31,152,83,80,128,75,23,47,161,248,220,121,112,0,215,
  95,127,61,122,122,122,112,232,224,193,126,203,69,56,78,169,118,62,103,104,40,94,138,225,37,32,209,240,170,20,190,41,161,98,165,196,232,28,208,119,
  71,232,68,195,161,57,160,137,235,197,213,229,8,60,170,96,208,55,46,92,73,244,160,137,15,54,188,83,228,200,142,137,101,135,209,20,210,209,176,131,
  234,57,213,15,178,99,226,219,49,76,245,124,84,61,163,107,214,173,197,125,247,223,143,228,228,20,156,61,119,22,75,46,91,142,95,252,242,127,176,123,
  239,30,148,150,150,162,187,187,27,0,144,55,123,54,182,109,219,134,41,83,167,162,186,186,26,231,206,158,197,27,199,142,193,227,245,194,100,50,97,202,
  148,41,184,98,221,229,88,187,102,13,190,252,111,255,134,51,167,79,225,115,15,62,136,230,166,102,28,236,79,140,170,251,21,170,204,78,234,56,162,156,
  115,32,24,4,99,44,102,102,3,35,33,74,16,4,65,16,4,65,140,49,49,58,107,246,108,52,212,215,99,215,206,15,176,235,131,15,48,115,246,108,
  84,86,85,225,59,223,125,2,229,229,229,240,249,124,0,128,156,153,51,113,243,182,109,184,236,178,101,0,7,156,173,173,152,58,117,42,86,174,92,137,
  25,51,103,160,162,188,2,181,245,245,104,107,107,195,91,111,255,19,7,14,30,192,186,181,107,113,205,213,87,163,211,229,194,111,127,243,27,48,206,225,
  72,78,6,0,244,246,246,70,58,179,117,162,145,235,211,127,202,177,80,33,139,210,160,44,88,153,156,245,128,132,232,196,35,45,45,13,139,151,44,209,
  150,11,145,67,20,229,116,159,65,49,156,2,84,148,83,130,6,197,240,231,162,252,158,3,254,30,183,230,56,142,100,7,22,23,45,209,164,2,141,76,
  7,42,72,69,46,74,57,186,112,254,124,220,185,158,9,130,32,8,98,130,138,209,129,135,118,218,188,101,11,126,254,203,95,226,91,255,241,31,8,4,
  2,112,117,119,227,47,127,251,27,62,216,181,11,77,77,77,96,156,35,103,214,44,92,115,205,53,184,118,243,38,48,38,160,163,163,29,206,214,54,120,
  60,30,48,48,36,37,37,97,214,204,89,152,150,49,13,11,93,46,92,186,120,9,21,149,149,232,116,185,176,115,215,46,148,148,149,161,173,173,29,31,
  189,101,27,252,129,0,62,255,224,131,0,128,199,255,245,95,85,121,239,67,127,120,164,105,76,159,137,73,222,200,194,187,64,12,74,115,54,4,65,209,
  12,97,17,74,227,242,137,151,167,209,248,174,49,169,105,105,88,119,249,21,154,109,162,40,34,168,201,31,44,231,16,86,189,246,107,114,15,7,80,91,
  87,15,174,18,163,28,64,146,195,129,197,139,139,96,177,90,97,181,88,96,182,88,96,177,152,97,54,91,36,81,106,50,193,20,234,232,68,17,163,117,
  181,181,113,231,167,95,92,180,4,139,139,194,194,90,228,146,29,193,96,80,178,39,24,68,48,16,68,48,24,64,48,16,68,32,40,125,22,8,4,148,
  237,157,93,221,104,239,236,210,216,145,150,158,142,197,139,23,41,185,145,205,102,51,76,38,51,76,102,233,252,5,147,9,76,16,32,32,186,29,78,103,
  11,246,237,217,61,169,236,184,99,251,199,180,229,106,0,118,56,219,59,34,106,64,110,94,30,166,101,172,24,148,29,123,247,236,70,171,179,101,210,212,
  115,125,153,26,75,229,42,145,50,5,0,87,111,220,132,172,172,236,49,89,63,46,20,159,199,133,226,243,212,94,141,67,59,70,178,174,142,184,103,244,
  248,209,163,184,80,92,140,207,126,254,65,188,244,202,203,120,234,169,255,69,109,93,45,178,166,77,195,230,107,174,65,81,81,17,178,178,179,225,247,249,
  209,216,212,140,158,158,30,120,189,94,37,69,22,7,135,96,50,193,110,183,195,98,177,194,225,112,96,230,140,25,88,185,106,37,90,154,155,81,87,95,
  143,14,151,11,47,188,244,34,86,172,88,129,27,111,250,40,50,179,178,240,167,63,254,81,123,145,184,42,157,167,46,134,41,84,41,63,185,126,187,38,
  61,40,7,196,160,20,106,74,24,59,249,3,136,193,81,95,87,139,255,249,249,79,13,234,24,135,200,185,60,223,135,43,41,221,0,14,81,212,231,166,
  7,166,207,156,21,113,136,86,103,43,94,57,255,130,226,253,148,202,77,120,209,28,3,27,222,185,232,19,165,159,68,118,208,69,165,75,64,118,144,29,
  147,221,51,58,0,111,83,106,42,186,187,187,209,221,221,141,103,254,254,119,204,95,180,8,135,14,29,194,212,169,83,112,249,186,181,152,59,119,46,24,
  24,186,123,186,209,216,216,8,119,175,27,254,128,31,162,40,42,139,140,56,15,255,99,140,193,100,18,192,4,6,81,20,97,179,90,145,159,159,143,249,
  243,231,163,171,171,27,165,101,37,56,121,242,36,252,62,31,2,254,0,74,75,75,192,1,100,102,101,161,213,233,84,74,135,54,56,62,11,105,84,149,
  71,84,39,74,117,94,82,69,156,138,34,16,99,232,158,32,70,3,67,175,196,16,136,106,151,203,133,139,229,165,35,38,170,39,138,29,175,188,244,194,
  160,59,57,121,115,231,69,28,162,166,166,6,197,103,207,140,94,39,103,162,212,141,113,90,174,12,61,93,84,207,201,14,18,163,145,252,252,127,254,7,
  156,115,188,252,210,75,88,182,124,5,94,120,241,69,20,45,90,140,205,155,55,161,186,186,26,206,214,86,116,118,116,34,24,12,130,131,67,20,69,88,
  204,22,192,36,13,203,115,0,162,24,84,134,64,253,242,112,40,120,16,12,128,197,108,134,200,57,60,94,15,152,192,176,116,233,50,120,250,60,56,116,
  248,48,178,178,178,176,229,250,27,208,88,95,135,59,182,111,199,31,126,247,123,124,120,242,132,220,97,209,102,88,98,60,154,40,101,202,240,61,147,131,
  42,168,189,164,76,22,164,156,49,8,228,37,37,8,130,32,8,130,24,122,49,58,24,79,243,177,163,199,240,233,207,126,6,157,93,93,248,219,223,255,
  142,146,178,82,108,218,120,13,106,106,107,81,81,81,9,81,20,229,24,91,146,16,157,58,117,42,230,20,204,65,32,200,145,149,53,29,193,160,136,62,
  143,7,221,221,221,232,233,233,70,71,71,59,122,123,123,224,245,244,33,200,125,8,202,223,103,140,193,106,177,72,175,5,134,238,158,30,28,60,116,24,
  201,142,100,108,186,118,11,26,27,27,225,116,182,40,250,50,44,64,101,17,10,0,76,37,74,161,222,47,28,20,159,67,235,37,13,123,78,165,249,133,
  161,144,80,196,240,148,47,62,10,39,23,142,196,17,154,48,18,234,240,170,66,113,112,245,62,81,14,197,17,142,19,199,121,56,28,135,218,56,70,118,
  144,29,198,118,68,61,206,104,218,65,245,156,234,7,217,49,241,237,24,198,186,107,230,3,12,224,150,232,215,158,250,245,175,209,222,209,129,214,142,118,
  188,247,254,251,152,55,103,14,102,231,204,70,201,133,18,69,136,138,92,68,70,70,6,230,204,153,131,213,171,87,99,222,220,121,104,109,239,65,115,83,
  51,154,155,26,225,245,250,32,8,2,76,38,51,236,118,7,68,145,129,193,12,175,175,15,162,215,13,49,24,208,204,247,44,156,55,31,197,197,37,168,
  171,175,195,158,189,123,145,156,156,140,19,71,142,160,182,166,70,115,181,184,188,63,83,109,224,170,247,106,47,40,184,84,8,162,223,121,217,119,202,121,
  212,224,249,68,172,114,53,198,227,15,170,231,20,27,196,134,227,6,25,133,141,170,169,212,144,140,98,188,59,178,131,236,160,122,78,229,138,236,32,59,
  198,72,61,31,246,97,250,133,139,22,97,209,162,69,56,119,238,28,102,206,158,141,87,223,120,3,30,143,7,121,121,121,240,120,188,232,237,117,43,30,
  209,204,204,76,92,121,197,21,88,186,116,41,210,211,211,193,24,144,148,100,133,197,98,149,86,137,49,73,228,137,162,40,173,58,11,6,228,57,165,2,
  152,96,5,19,57,56,15,42,243,75,147,147,29,200,200,152,138,150,214,86,212,55,54,224,212,233,83,88,178,108,25,2,129,0,230,207,159,143,61,187,
  119,203,34,52,84,80,16,126,143,176,168,85,10,82,40,236,147,28,238,41,236,92,229,218,149,107,161,207,229,27,65,130,148,32,8,130,32,8,98,148,
  196,232,39,62,121,63,110,189,237,54,212,215,215,227,255,251,254,15,80,86,94,134,236,204,44,44,191,236,50,180,183,181,33,16,8,128,131,35,57,57,
  25,235,215,175,199,170,149,43,225,112,56,148,239,219,109,22,216,147,236,48,153,204,146,224,23,57,196,160,136,64,32,136,64,32,20,215,81,4,32,64,
  16,172,16,225,131,24,244,131,139,34,4,65,64,126,126,30,90,90,91,225,118,187,81,81,81,137,133,243,231,227,11,143,61,10,155,213,134,51,103,206,
  160,163,189,93,94,36,31,25,95,148,51,201,13,174,136,84,181,8,141,34,72,35,222,147,32,37,8,130,32,8,130,24,14,49,26,159,251,245,91,255,
  254,239,56,122,228,8,110,189,253,14,156,252,240,36,2,126,63,114,114,102,99,222,188,121,40,45,45,85,230,136,230,229,229,97,249,101,151,105,132,40,
  0,152,77,12,41,41,14,152,205,102,48,198,32,114,142,64,48,188,144,73,18,163,210,185,8,130,25,130,73,90,60,228,15,122,192,1,204,152,62,3,
  86,139,21,62,191,15,237,157,29,40,43,47,71,122,106,26,222,125,251,109,116,118,134,227,245,133,156,155,92,158,7,42,101,6,149,5,169,108,174,228,
  18,143,33,72,85,87,70,243,94,57,38,9,210,161,135,143,238,47,242,40,123,40,19,132,244,201,133,85,115,67,88,104,57,156,102,13,29,84,147,152,
  201,14,178,99,220,217,65,245,156,202,21,217,65,245,60,81,70,100,217,183,219,237,70,73,105,41,46,92,184,128,105,211,50,113,229,21,87,194,231,247,
  163,167,183,23,162,40,98,222,188,121,216,176,97,3,82,82,82,12,191,159,150,154,4,155,221,14,6,41,140,147,223,239,135,207,231,131,223,239,71,48,
  40,74,177,71,229,80,79,86,139,13,54,155,67,17,129,41,169,169,152,57,99,58,0,160,175,175,15,149,151,46,193,235,243,74,30,81,85,24,5,192,
  224,181,172,82,67,219,194,101,68,29,175,148,135,203,20,231,81,75,29,31,64,62,102,130,32,8,130,32,136,137,206,176,14,211,127,245,235,95,71,119,
  119,55,102,206,154,133,131,135,143,32,24,8,96,78,65,62,22,45,94,132,86,167,19,129,64,0,54,155,13,133,243,10,145,151,155,27,213,115,104,54,
  11,72,73,73,134,32,8,8,4,2,146,16,245,249,149,48,80,2,19,96,54,155,164,44,4,38,19,108,204,14,119,175,11,193,128,31,130,32,32,59,
  43,11,205,205,205,240,250,124,232,238,234,134,171,211,133,205,91,182,192,158,100,199,137,99,199,209,220,220,172,252,150,228,29,5,164,101,75,242,68,97,
  198,193,185,244,23,156,41,30,82,181,215,20,234,228,75,170,16,80,106,31,105,104,181,63,65,16,4,65,16,4,33,49,172,158,209,181,235,214,225,145,
  71,31,197,188,249,11,80,82,90,138,244,244,116,172,89,189,26,22,139,5,237,237,237,114,176,122,59,76,102,27,220,125,65,120,125,65,136,6,222,67,
  6,96,106,122,10,76,38,19,2,129,0,188,94,47,252,1,63,192,57,76,130,0,139,197,12,171,205,6,155,213,6,187,221,142,36,135,29,83,51,50,
  21,111,100,198,180,105,152,54,109,26,0,6,143,207,139,214,246,54,204,153,59,23,11,22,44,68,74,106,138,38,208,172,214,59,106,252,87,237,69,87,
  249,69,53,255,87,94,233,157,165,162,72,165,142,32,8,130,32,8,66,38,177,56,163,92,251,183,63,238,186,227,14,164,166,166,226,39,191,252,37,170,
  170,170,48,111,238,28,20,21,21,161,173,181,21,110,119,31,192,129,148,148,20,48,152,208,220,210,138,36,187,13,246,36,59,28,73,118,216,237,102,152,
  77,97,173,108,183,91,97,181,90,225,247,249,224,233,235,3,231,92,206,235,109,133,213,106,129,213,106,133,217,100,130,201,108,134,217,108,66,48,232,151,
  134,197,57,144,156,156,140,236,172,108,180,182,182,34,224,247,195,229,234,194,161,131,7,112,238,236,89,89,236,134,23,47,113,245,95,6,197,35,170,255,
  43,121,72,165,216,163,145,115,73,161,157,196,193,245,121,237,165,24,168,196,192,202,149,114,89,135,105,214,195,236,156,28,220,177,253,46,205,182,96,208,
  40,55,189,62,63,189,252,62,16,64,192,31,64,93,67,61,90,90,219,53,199,201,204,204,196,213,87,95,13,171,197,10,139,213,162,202,49,44,229,166,
  55,153,76,16,228,76,94,209,202,200,43,47,189,136,250,186,186,184,108,73,77,75,67,90,90,154,230,90,115,81,132,40,138,82,54,144,208,107,81,202,
  8,34,133,89,147,255,138,210,124,110,179,217,12,127,208,175,57,174,197,98,65,102,86,22,4,65,58,223,208,63,38,167,198,21,164,184,102,17,33,240,
  212,120,189,94,57,27,218,228,177,99,118,78,78,68,217,79,212,14,35,146,28,14,216,172,150,65,217,225,116,58,225,243,122,39,77,61,215,151,169,177,
  84,174,18,41,83,128,148,89,208,102,179,141,201,250,209,213,213,133,238,174,46,106,175,198,161,29,35,86,207,1,152,7,90,209,227,253,94,87,87,55,
  188,30,47,106,235,106,241,209,143,124,30,54,187,29,85,213,213,202,16,187,89,94,92,212,212,80,15,147,252,64,182,90,237,176,39,217,144,145,49,21,
  217,153,233,48,153,24,44,22,51,146,147,147,164,92,245,62,31,82,146,83,144,228,72,130,205,38,133,126,178,152,205,48,153,76,96,130,148,1,201,102,
  181,225,82,101,153,244,27,102,51,178,178,179,144,213,154,133,134,198,70,116,117,119,35,35,61,7,92,12,9,72,30,142,217,165,150,164,154,128,178,92,
  251,87,19,223,139,69,182,160,218,132,247,145,179,150,57,137,209,193,148,171,129,238,31,87,153,117,117,225,200,161,67,186,223,225,8,6,69,136,98,16,
  98,48,136,160,40,106,255,42,159,137,8,138,82,132,135,160,193,252,115,119,175,27,23,206,23,43,83,74,76,38,19,76,130,9,130,73,219,176,40,139,
  232,162,156,95,188,118,47,94,92,132,117,87,92,161,188,23,69,46,45,252,243,251,225,87,139,103,89,64,27,137,236,186,250,122,244,121,218,53,118,164,
  167,167,99,241,162,43,96,177,90,97,181,88,228,142,161,36,170,5,65,128,201,108,14,55,146,81,236,168,175,171,195,43,47,189,52,169,236,184,253,206,
  237,154,247,3,177,163,188,178,50,162,185,201,203,203,67,110,78,238,160,236,120,229,165,151,226,238,228,76,132,122,174,47,83,99,169,92,37,82,166,0,
  224,234,107,54,106,58,58,99,169,126,28,61,124,24,71,14,31,166,246,106,28,218,49,146,245,124,216,230,140,46,90,180,8,95,253,198,55,0,0,221,
  189,189,72,75,73,65,86,102,38,218,219,218,224,241,120,52,15,120,183,187,15,30,143,23,76,96,74,230,34,129,9,104,168,175,131,213,106,69,114,114,
  50,166,79,207,196,148,41,233,240,249,124,200,207,203,67,114,114,50,204,102,179,114,241,5,57,51,82,232,251,98,146,168,196,27,101,96,176,219,237,72,
  77,75,133,216,208,0,159,207,139,212,212,84,108,222,178,5,93,174,78,156,56,126,66,9,227,180,100,233,82,76,205,200,128,73,22,183,33,111,171,34,
  28,66,222,43,147,9,38,65,128,96,210,246,80,206,157,61,139,51,167,78,69,222,17,93,201,160,249,163,99,151,174,174,46,227,198,83,206,45,28,138,
  73,171,44,74,83,166,121,168,94,195,56,183,176,219,237,70,245,165,139,96,130,0,6,40,127,193,216,176,100,237,186,80,92,140,58,181,192,224,128,200,
  69,131,94,122,184,135,46,245,206,195,159,57,82,83,35,142,235,234,116,97,223,158,221,154,242,207,228,84,184,140,9,82,93,238,39,79,178,55,1,47,
  220,68,177,35,226,33,48,0,59,210,166,102,68,28,183,166,186,6,85,149,149,131,178,99,56,189,37,99,145,136,50,53,134,202,149,55,65,15,245,222,
  61,123,52,158,209,177,84,63,226,245,138,82,123,53,246,236,24,73,70,44,55,125,48,24,68,69,69,5,114,114,114,195,55,134,115,120,125,94,116,247,
  116,75,2,82,20,229,97,25,233,161,31,12,6,17,148,243,208,187,92,46,76,203,204,192,227,143,127,25,77,77,205,232,232,112,161,175,175,79,90,81,
  47,123,162,20,49,32,114,37,61,40,23,165,128,250,193,64,0,125,110,183,162,13,67,251,7,130,162,102,149,123,83,83,19,92,46,87,132,200,212,255,
  11,185,196,213,127,25,99,104,105,105,137,25,246,41,150,64,37,136,225,16,214,93,250,135,65,130,162,218,106,79,138,56,174,223,239,135,211,233,28,49,
  81,61,81,236,48,244,60,38,104,135,145,24,237,115,187,209,229,234,28,49,59,38,108,221,24,167,229,202,176,35,65,245,156,236,24,71,12,155,24,45,
  41,41,193,103,62,245,41,0,192,175,127,247,91,248,3,65,20,95,184,128,212,180,52,152,205,102,120,125,94,112,81,132,167,207,45,137,63,65,144,98,
  134,6,69,37,142,168,215,235,129,199,227,133,167,207,3,179,217,140,89,179,102,162,171,203,133,69,11,231,130,115,160,175,207,7,87,119,15,122,123,251,
  224,238,117,43,225,158,196,96,16,45,206,102,69,136,134,110,96,67,99,19,4,65,128,217,108,70,123,123,59,78,157,60,41,157,108,120,9,61,156,45,
  45,154,27,107,116,163,153,102,94,70,228,223,126,93,215,178,64,37,239,40,65,16,4,65,16,147,157,17,241,140,90,173,86,76,159,62,29,77,77,77,
  168,171,171,195,156,57,115,0,217,83,236,241,120,208,211,211,13,179,217,2,191,95,74,241,233,15,4,224,247,73,171,230,3,126,105,33,82,74,74,50,
  76,38,51,122,123,251,20,167,162,195,97,69,82,82,6,252,129,32,122,123,189,232,235,147,196,171,207,231,195,197,170,10,37,95,171,219,237,70,83,115,
  51,124,126,63,204,102,51,172,22,11,218,218,218,52,61,22,64,202,71,15,206,53,81,154,244,233,66,153,42,61,168,34,98,229,109,106,143,168,81,47,
  149,60,161,19,3,174,191,175,49,246,227,209,222,235,203,205,40,148,19,178,131,236,32,168,92,145,29,100,199,88,96,88,67,59,253,223,95,254,130,179,
  23,138,145,156,156,130,130,130,2,52,53,55,163,174,174,22,126,191,31,38,179,9,156,115,89,140,246,192,237,238,131,219,237,134,219,221,135,62,119,31,
  250,250,220,240,251,188,16,69,17,38,179,9,246,164,36,48,198,208,231,238,131,223,31,80,126,131,49,192,106,49,97,202,20,7,178,179,167,96,250,244,
  105,200,206,206,68,107,75,147,124,111,56,58,218,59,208,212,36,197,18,53,155,76,176,219,172,88,186,116,9,182,221,114,11,102,205,154,165,41,24,92,
  125,211,101,183,185,146,183,94,217,141,199,87,168,140,246,139,216,133,2,225,19,4,65,16,4,49,121,25,86,49,90,90,114,1,127,123,250,105,84,148,
  151,97,209,194,133,8,4,2,104,106,110,65,99,99,35,172,54,155,52,47,52,224,135,215,211,7,79,159,7,94,175,23,94,175,23,62,175,87,202,89,
  207,1,65,96,176,90,109,176,217,172,96,130,0,183,199,139,206,174,222,136,223,98,0,204,38,1,201,14,43,236,54,134,170,234,42,121,129,84,16,45,
  45,45,240,5,164,16,9,22,139,5,41,142,100,84,93,170,130,179,165,5,189,242,60,210,8,17,169,23,166,70,226,49,134,216,36,137,73,16,4,65,
  16,4,209,63,230,132,214,222,171,3,72,197,241,189,31,254,247,247,1,0,87,94,117,21,110,248,232,71,241,199,63,253,31,154,154,155,80,91,91,131,
  185,115,231,66,96,12,193,96,16,94,143,27,129,96,16,51,102,204,66,32,224,135,171,179,11,30,143,23,16,0,139,213,10,155,205,6,139,197,10,198,
  24,188,94,47,186,186,122,144,49,53,85,19,135,52,132,223,239,199,123,239,189,7,167,179,5,92,228,232,235,235,67,67,83,163,164,188,5,1,41,201,
  201,176,39,217,113,250,212,41,205,144,60,84,195,239,92,37,112,245,2,147,201,67,243,234,215,81,233,239,115,149,192,157,212,115,71,19,44,87,225,187,
  49,140,65,8,163,157,167,209,185,26,157,119,188,231,165,158,238,145,64,153,33,59,200,142,49,103,7,213,115,42,87,100,7,213,243,1,213,243,17,200,
  77,63,107,246,108,108,184,250,106,228,230,228,96,78,65,1,252,62,63,154,155,154,225,108,105,129,197,98,1,56,71,48,232,199,172,153,211,113,237,230,
  107,112,195,13,91,80,48,39,15,86,171,5,102,147,9,22,139,5,22,171,25,102,147,9,140,73,171,242,61,125,30,248,124,65,195,223,43,45,45,197,
  161,67,135,149,197,75,78,103,11,124,126,201,43,106,181,88,48,109,106,6,76,130,128,25,51,102,40,77,157,225,205,215,189,231,170,33,251,184,11,76,
  63,243,65,136,113,166,153,19,190,175,49,103,253,24,150,19,78,118,144,29,113,217,129,49,103,7,213,115,170,31,100,199,196,183,99,184,24,118,49,250,
  189,239,255,55,62,249,192,167,96,179,90,177,108,233,50,152,45,22,56,91,91,209,212,212,4,135,195,1,145,115,228,230,229,225,218,107,55,97,249,242,
  165,88,185,252,50,204,157,59,7,73,73,118,57,51,141,25,22,147,25,130,236,5,21,69,81,30,206,247,69,164,14,45,47,47,199,142,29,59,208,220,
  220,164,196,24,173,170,170,150,59,12,12,201,14,7,102,100,103,33,57,57,25,107,215,173,69,102,102,102,236,254,184,58,228,66,156,162,146,26,125,130,
  32,8,130,32,136,248,25,246,213,244,79,253,234,215,248,81,247,247,225,116,182,226,182,237,219,113,242,195,15,113,169,234,18,58,58,58,96,54,155,81,
  48,103,14,182,108,217,130,165,75,151,194,106,177,160,187,167,23,174,78,23,130,242,194,37,179,217,4,147,89,78,145,8,6,46,138,8,4,252,240,120,
  188,72,78,182,195,106,49,33,24,12,162,172,172,12,239,188,243,14,206,158,59,135,62,143,71,138,97,234,245,162,189,163,3,0,96,179,217,48,35,59,
  27,98,48,136,221,31,236,194,140,153,51,208,218,218,170,8,200,68,6,201,19,221,63,174,99,82,152,39,130,32,8,130,32,72,140,14,61,199,142,30,
  5,32,121,52,93,29,29,184,254,186,235,240,194,139,47,160,172,162,2,43,86,172,192,173,215,108,196,146,162,37,176,89,173,232,116,185,240,202,171,175,
  226,252,217,243,8,138,12,41,169,83,148,188,221,130,32,0,76,58,78,192,31,128,187,207,139,20,111,0,221,93,157,216,191,127,63,118,237,218,141,214,
  182,86,4,252,1,197,43,90,93,85,5,145,75,233,64,179,167,77,195,156,252,124,148,20,95,128,223,239,71,109,77,173,145,34,28,88,88,4,10,155,
  50,161,72,75,75,195,162,162,162,136,206,66,40,213,103,48,40,37,89,80,167,3,21,69,49,34,93,104,16,128,191,71,187,64,206,225,112,96,241,226,
  34,37,29,168,62,147,151,146,68,33,70,22,141,146,226,98,227,96,221,4,65,16,4,65,98,52,58,219,239,186,11,219,239,218,142,179,231,206,163,186,
  186,10,71,143,30,149,131,212,251,208,214,214,134,148,212,20,188,242,234,171,56,121,226,4,188,30,47,4,65,64,32,224,131,221,54,3,110,119,15,28,
  73,73,82,198,164,64,0,29,157,237,104,113,54,162,171,171,19,141,13,245,168,169,169,65,71,103,135,50,79,52,180,95,229,165,75,146,184,72,73,193,
  172,233,51,144,154,156,130,37,75,151,224,204,233,211,232,115,247,13,206,32,18,160,19,150,212,180,52,172,187,92,159,179,90,84,229,15,86,229,22,246,
  71,223,86,215,80,143,110,3,49,186,104,209,98,88,172,22,88,45,86,152,45,22,101,58,138,146,106,86,80,165,117,51,160,190,174,46,110,49,186,184,
  168,8,139,22,135,133,181,200,69,57,179,89,16,129,96,64,121,29,12,6,228,191,234,237,210,223,206,238,110,116,116,106,127,47,61,61,29,139,23,46,
  130,217,98,86,58,140,38,147,148,27,217,36,132,133,181,148,165,215,216,142,86,167,19,251,246,238,153,84,118,68,228,166,31,128,29,173,242,104,143,154,
  220,188,60,76,155,154,49,40,59,246,237,221,51,169,82,130,234,203,212,88,42,87,137,148,41,64,202,77,159,153,149,53,38,235,71,201,133,98,92,40,
  46,166,246,106,28,218,49,225,196,40,231,28,91,63,250,17,116,180,183,99,255,158,61,216,188,105,51,122,122,122,240,254,7,31,160,182,182,14,171,87,
  175,70,90,122,26,42,43,43,209,215,39,137,68,49,16,132,63,16,64,125,125,13,184,8,52,53,212,193,100,150,242,175,74,98,211,15,159,215,139,62,
  79,159,228,13,229,28,34,23,225,245,120,164,224,250,245,245,232,243,120,96,181,90,145,59,107,54,28,118,59,106,107,106,176,96,209,66,164,165,165,13,
  94,140,18,19,150,250,186,58,252,234,23,63,55,44,199,234,57,196,70,41,220,212,239,179,103,69,230,166,111,109,109,197,171,47,191,20,206,240,165,202,
  234,165,201,236,69,16,253,117,134,9,130,32,200,51,154,24,223,252,198,191,163,177,81,10,177,228,245,121,113,253,117,215,227,111,207,252,29,39,78,158,
  196,133,11,23,192,152,52,60,58,107,246,108,164,167,167,75,10,159,49,184,221,110,128,3,238,190,94,77,174,249,144,183,74,12,138,240,120,61,104,105,
  110,134,179,181,21,189,189,110,184,251,220,232,243,120,96,179,90,145,51,107,54,102,78,159,142,146,226,98,120,60,30,212,55,212,15,141,16,29,6,175,
  40,205,27,29,167,186,160,95,129,192,13,95,70,21,25,67,88,6,46,20,71,122,37,162,137,106,189,176,142,20,213,225,147,119,117,186,80,89,86,22,
  93,84,3,67,90,150,39,138,29,175,190,252,82,220,157,156,104,118,228,205,157,167,181,131,3,181,213,53,40,62,123,118,196,236,152,8,24,149,169,241,
  90,174,140,60,93,84,207,201,14,18,163,6,52,53,53,129,1,72,78,73,193,125,247,125,2,25,211,50,192,57,199,91,255,252,39,62,60,125,10,238,
  222,94,56,236,118,84,84,84,194,108,54,35,47,47,23,115,231,205,131,195,225,8,223,184,144,16,229,82,26,209,246,142,118,212,215,213,163,185,185,25,
  125,94,143,230,182,165,36,39,99,110,65,1,10,231,204,193,148,244,116,180,58,91,80,87,91,23,91,136,14,244,38,82,35,79,76,96,2,254,0,82,
  83,83,145,154,226,64,48,40,194,97,183,195,39,215,55,178,99,116,177,89,45,114,199,60,72,5,149,160,250,65,118,140,91,70,108,152,30,178,16,253,
  241,207,126,134,57,115,231,226,201,31,252,0,157,46,23,238,188,227,14,164,165,167,225,232,209,99,104,109,107,5,56,135,197,98,65,85,117,53,170,171,
  171,177,96,225,2,204,152,49,19,73,114,58,208,64,32,128,142,142,14,84,86,84,160,217,233,132,215,231,3,83,137,80,65,16,144,158,154,138,37,139,
  22,33,197,145,140,179,167,79,99,229,202,149,88,177,114,37,0,134,186,218,200,133,75,137,74,73,22,227,187,36,75,9,163,198,4,192,152,106,76,140,
  122,207,140,49,201,59,175,238,161,3,232,118,117,34,57,37,25,11,231,230,201,245,89,68,79,151,11,66,104,90,193,40,246,202,39,163,29,193,64,0,
  211,51,51,0,0,83,210,82,1,0,94,143,103,76,216,65,245,156,234,57,217,49,241,237,24,183,98,20,0,82,82,82,240,211,95,252,28,115,230,206,
  197,143,190,255,3,188,191,99,7,56,164,249,121,183,221,121,7,10,231,206,195,155,255,124,11,181,181,181,240,249,253,240,249,253,176,91,173,168,44,175,
  68,89,105,25,230,207,159,143,164,164,36,180,180,180,160,190,190,30,110,143,71,17,160,161,191,118,155,29,115,11,10,48,127,238,92,180,183,182,226,220,
  153,51,8,4,2,56,116,240,16,174,188,106,61,86,174,90,9,198,160,89,73,31,145,129,73,85,64,244,255,18,246,128,198,216,159,30,17,227,147,137,
  208,152,12,228,55,59,90,91,97,181,219,33,48,1,158,62,105,81,150,217,98,25,19,247,98,178,217,209,229,234,68,218,148,169,152,53,61,19,156,139,
  232,118,117,66,16,4,41,226,8,65,245,156,234,57,217,65,98,52,134,24,77,77,193,140,153,51,241,228,15,126,136,29,239,190,171,108,111,168,175,199,
  111,126,241,75,92,119,195,13,248,194,131,15,161,166,166,6,231,47,20,163,166,182,22,237,237,237,240,120,189,16,192,80,82,90,6,159,207,139,160,40,
  130,49,6,171,213,10,147,201,12,155,213,10,135,35,9,179,103,206,68,206,172,89,240,184,251,80,81,82,130,182,246,118,229,55,252,126,63,14,29,56,
  136,171,54,92,133,204,204,44,69,140,14,200,35,26,42,80,253,136,77,98,226,62,160,18,105,76,218,156,78,216,108,54,48,65,128,71,94,156,55,30,
  27,147,96,48,136,190,222,222,113,127,15,39,130,29,1,191,31,237,206,22,170,144,84,207,169,126,144,29,36,70,19,165,169,177,9,31,191,235,99,232,
  238,233,9,213,120,205,130,143,247,118,236,192,254,189,123,113,217,202,149,216,112,229,122,88,54,90,80,86,86,142,134,166,70,180,183,183,163,171,171,27,
  193,64,0,162,24,132,201,98,65,178,195,129,20,135,3,83,211,167,32,37,37,5,157,237,237,40,189,80,130,246,182,54,77,206,121,174,18,164,7,246,
  31,128,95,78,13,202,180,173,79,68,99,164,159,32,140,24,251,71,21,174,9,126,135,152,120,112,81,84,30,78,4,65,80,61,39,8,98,20,197,40,
  0,244,132,132,104,20,250,250,250,112,248,224,65,28,62,112,0,83,166,78,197,236,217,179,49,51,43,27,11,231,21,194,235,245,194,239,151,226,56,6,
  131,65,248,188,94,116,116,116,160,169,161,1,173,78,167,34,50,245,66,151,233,60,10,172,31,129,200,162,245,148,213,30,81,245,190,161,80,60,161,99,
  197,121,76,18,167,4,65,16,4,65,16,35,44,70,245,222,80,101,225,81,104,59,99,210,124,28,198,208,217,209,129,142,246,118,77,168,3,205,107,125,
  172,199,88,98,79,255,121,44,33,24,101,158,168,34,56,245,226,51,154,232,36,177,73,16,4,65,16,4,49,198,196,168,90,128,198,179,93,37,78,89,
  72,136,34,60,151,39,36,66,245,239,99,10,211,24,231,133,40,34,212,40,24,185,161,87,52,129,107,64,16,4,65,16,4,65,0,163,187,252,50,202,
  124,76,141,208,211,127,166,26,14,55,10,250,202,6,116,26,44,66,136,170,69,104,60,226,51,98,113,147,193,118,130,32,8,130,32,8,66,203,136,123,
  70,99,13,213,43,33,51,244,2,52,228,29,85,123,70,117,94,210,8,140,142,101,36,66,117,127,53,219,116,226,147,25,137,229,129,44,112,34,113,74,
  16,4,65,16,4,49,74,98,20,186,33,121,157,56,53,20,160,186,225,122,30,154,95,42,31,39,180,141,49,166,188,55,60,174,238,28,96,32,70,141,
  132,104,127,222,205,104,94,81,35,225,73,50,148,32,8,130,32,8,98,132,197,40,99,12,119,222,117,151,38,23,107,93,109,45,142,30,57,2,0,88,
  180,104,17,56,231,40,45,45,85,188,159,243,230,207,7,231,28,149,229,229,138,232,156,45,231,173,47,62,127,62,156,195,155,115,164,166,165,33,55,47,
  15,86,155,13,126,159,15,181,53,53,232,236,236,4,0,204,153,59,23,201,201,201,154,156,223,238,222,94,92,186,116,73,115,126,208,9,207,120,255,170,
  153,53,123,54,26,27,26,148,247,118,155,13,153,217,217,104,149,99,208,165,165,167,107,167,18,48,134,218,154,26,228,230,229,161,165,165,5,94,79,56,
  107,135,221,110,71,246,244,233,168,169,174,86,182,77,159,62,29,54,187,61,226,250,182,52,55,3,0,178,167,79,143,216,238,145,143,105,183,219,145,158,
  158,142,102,121,95,253,49,93,157,157,112,185,92,84,35,134,152,79,62,240,0,42,42,42,112,112,255,254,113,119,238,55,108,221,138,27,182,110,85,222,
  247,246,244,224,149,151,94,194,233,83,167,198,157,45,243,10,11,113,199,246,237,152,62,99,198,184,181,99,94,97,33,30,121,236,49,205,182,202,138,10,
  252,245,207,127,238,55,82,9,49,188,124,247,123,223,195,111,126,245,43,52,55,53,81,61,167,122,78,245,124,172,138,81,0,200,201,205,197,75,207,63,
  15,200,194,114,221,186,117,0,128,163,71,142,32,53,45,45,98,241,81,106,106,170,226,1,13,9,201,165,203,150,65,20,69,148,151,151,195,239,243,129,
  115,14,171,213,138,43,214,175,71,89,105,41,26,27,26,96,179,219,177,124,197,10,156,57,115,6,29,237,237,72,79,79,71,107,107,171,230,38,6,3,
  1,237,156,84,213,235,68,132,41,160,245,138,222,122,219,109,248,223,223,252,6,0,96,179,217,112,251,246,237,104,109,109,197,251,59,118,96,229,234,213,
  88,178,116,41,186,187,186,52,139,165,158,151,197,232,210,101,203,240,206,63,255,169,156,227,150,235,175,135,203,229,210,136,209,101,151,93,134,236,233,211,
  35,132,229,206,247,222,131,205,110,199,157,219,183,107,196,102,94,126,62,158,249,219,223,80,83,93,141,236,233,211,177,225,234,171,241,204,223,254,166,136,
  208,59,238,186,11,0,224,234,236,196,244,233,211,81,86,86,134,183,222,124,147,158,44,67,68,74,74,10,214,111,216,128,245,27,54,140,75,49,58,99,
  198,12,52,55,53,97,199,59,239,72,101,102,198,12,60,241,189,239,225,137,111,126,115,92,53,240,41,41,41,248,241,207,127,142,191,254,249,207,216,241,
  206,59,227,218,142,148,148,20,60,245,171,95,41,219,238,127,224,1,220,177,125,59,158,254,243,159,169,194,141,18,235,55,108,192,244,25,51,112,199,246,
  237,154,123,67,245,156,234,57,213,243,49,38,70,67,139,129,234,234,234,148,109,71,57,199,218,203,47,7,142,28,137,20,134,242,144,123,232,187,156,49,
  100,76,157,138,174,174,46,52,53,54,98,206,156,57,40,43,43,3,3,144,62,101,10,124,62,31,106,170,171,149,208,79,94,143,71,153,255,105,177,88,
  208,231,118,163,75,231,245,99,208,205,55,53,90,56,21,77,136,170,142,17,177,234,94,22,162,119,168,132,104,232,243,226,115,231,112,248,208,161,136,235,
  115,112,255,126,60,240,153,207,32,55,47,15,181,53,53,200,203,207,199,244,233,211,35,132,225,251,239,189,167,136,76,181,176,12,109,107,110,110,214,108,
  91,118,217,101,88,187,110,157,70,208,134,184,227,174,187,176,127,239,94,156,61,115,6,128,228,57,189,99,251,118,44,187,236,50,101,27,49,120,143,195,
  43,47,189,132,229,43,86,96,94,97,33,42,43,42,198,157,13,205,77,77,154,134,188,167,167,7,119,108,223,62,174,26,247,121,133,133,104,110,106,194,
  43,47,189,164,177,107,60,210,211,211,163,189,246,127,254,51,238,127,224,1,170,108,163,200,141,91,183,226,59,223,252,38,190,251,189,239,141,91,27,168,
  158,83,61,159,20,98,212,104,190,229,226,37,75,208,221,221,173,8,181,89,179,103,131,175,93,171,12,227,207,152,49,3,13,242,144,55,3,176,184,168,
  8,53,213,213,104,110,110,198,13,55,222,136,242,178,50,112,198,208,214,218,10,147,201,132,171,174,190,26,141,13,13,104,109,109,69,71,103,167,148,133,
  137,49,152,205,102,228,230,229,33,51,43,75,249,237,246,246,118,41,83,147,94,92,234,189,162,49,68,106,132,93,178,248,181,217,108,184,253,206,59,209,
  234,116,42,226,49,214,117,8,241,193,251,239,227,230,91,110,193,51,127,251,27,110,218,182,109,72,60,148,118,131,33,125,0,88,176,96,1,90,154,155,
  53,162,211,227,241,224,173,55,223,140,250,29,34,113,238,216,190,29,15,127,238,115,74,195,254,228,15,126,48,238,109,58,184,127,255,184,123,232,134,26,
  245,31,255,252,231,56,184,127,63,78,159,58,53,46,135,32,67,94,147,229,43,86,40,239,31,121,236,177,113,233,117,159,40,76,159,49,3,211,101,207,
  226,233,83,167,112,195,214,173,138,135,145,234,57,213,115,170,231,99,77,140,2,16,4,1,95,126,252,113,69,108,86,148,151,227,253,80,142,122,85,104,
  37,69,183,9,130,178,144,200,106,181,98,230,172,89,168,172,172,68,70,70,6,250,250,250,144,61,125,186,52,87,146,49,236,222,181,11,51,103,206,196,
  180,105,211,144,151,159,15,179,217,140,99,71,142,192,229,114,193,100,54,67,48,153,32,8,130,70,116,70,19,158,33,241,107,184,130,94,191,61,180,73,
  117,220,219,239,188,19,89,217,217,216,249,254,251,17,66,116,253,250,245,88,127,213,85,202,239,215,214,212,224,249,103,159,5,0,212,212,212,160,188,172,
  12,31,191,255,126,148,151,150,26,122,51,251,109,24,167,79,199,199,63,241,9,205,123,181,167,52,68,246,140,25,154,225,252,244,244,116,164,79,153,2,
  0,154,121,171,196,192,89,190,98,5,154,154,154,48,175,176,16,189,61,61,88,190,98,5,82,82,82,104,110,223,40,241,240,231,62,135,245,27,54,96,
  249,138,21,248,183,175,127,29,41,41,41,248,206,55,191,57,238,188,213,211,103,204,192,253,15,60,128,148,148,20,76,159,49,3,175,190,244,18,13,209,
  143,34,55,110,221,138,211,167,78,97,249,138,21,168,172,168,152,48,98,148,234,57,213,243,49,45,70,203,203,202,0,0,107,215,173,195,177,163,71,19,
  22,163,63,253,209,143,34,194,45,49,249,95,99,67,3,142,31,59,166,132,100,90,189,102,141,178,216,39,63,63,31,2,99,88,177,98,5,56,231,48,
  155,205,152,87,88,136,150,150,22,100,103,103,3,156,163,169,177,17,141,141,141,0,231,152,87,88,136,197,75,150,224,200,161,67,48,155,76,104,168,175,
  71,107,107,171,246,55,117,94,77,141,192,140,50,159,52,150,88,13,137,209,214,214,86,236,124,255,125,108,191,235,46,252,233,143,127,132,215,235,85,108,
  60,116,240,32,14,29,60,104,120,125,24,128,253,251,246,97,205,186,117,216,191,111,223,128,110,166,203,229,194,254,125,251,96,183,217,176,229,134,27,240,
  242,75,47,69,44,88,10,97,183,217,148,215,121,249,249,88,118,217,101,176,219,237,240,120,60,134,2,118,56,89,43,207,31,14,149,175,120,203,226,221,
  247,222,59,102,43,214,13,91,183,34,37,37,69,51,180,178,126,195,134,113,255,160,10,121,129,198,91,199,32,228,237,9,121,23,238,216,190,29,159,124,
  224,1,124,231,155,223,28,87,182,84,86,84,224,241,47,125,9,0,198,157,231,106,162,214,243,80,167,19,144,134,138,199,99,29,161,122,78,245,124,52,
  235,57,144,96,208,251,238,238,110,244,116,119,99,225,226,69,9,159,160,73,16,162,7,178,143,242,62,228,25,93,181,122,53,222,121,231,29,236,120,247,
  93,188,247,238,187,248,231,91,111,97,198,204,153,72,73,73,1,99,12,151,95,121,37,172,86,171,34,50,5,147,73,249,61,147,201,4,193,40,197,167,
  58,208,61,116,129,239,67,2,211,224,117,52,33,26,58,223,157,239,189,135,86,167,19,71,14,31,198,182,91,110,209,136,216,152,48,38,173,124,231,92,
  89,1,159,40,30,143,7,53,213,213,40,43,43,195,206,29,59,112,221,117,215,25,55,240,165,165,152,191,96,129,50,36,127,246,204,25,60,243,183,191,
  69,78,43,24,33,102,205,158,133,158,238,110,101,218,70,60,52,54,52,2,0,22,13,160,44,142,68,67,190,124,197,10,60,252,185,207,225,241,47,125,
  9,143,127,233,75,248,202,151,190,132,79,78,128,57,63,95,120,236,177,113,41,168,67,94,146,137,196,147,63,248,1,110,223,190,93,17,66,99,157,137,
  86,207,111,144,189,162,161,58,254,248,151,190,132,167,126,245,43,170,231,84,207,169,158,39,88,207,129,1,12,211,159,60,113,82,81,190,9,169,222,208,
  48,121,140,184,162,234,247,2,99,224,156,99,230,172,89,8,4,2,232,13,25,38,239,219,80,87,135,194,194,66,156,62,125,26,141,13,13,216,254,177,
  143,161,185,185,25,22,179,25,102,139,5,187,118,238,4,3,96,50,155,177,114,245,106,248,125,62,141,216,221,245,193,7,90,33,169,250,237,168,239,245,
  251,235,222,171,189,173,31,158,60,137,220,220,92,108,218,188,25,123,118,239,6,32,13,135,231,230,230,106,190,215,210,220,12,159,215,59,228,5,162,172,
  172,12,203,46,187,12,27,174,190,58,194,211,218,220,220,44,77,9,184,239,62,28,59,118,12,46,151,11,233,233,233,88,187,118,109,84,79,234,112,114,
  237,117,215,225,228,137,147,9,150,195,19,0,128,53,235,214,161,228,66,201,152,170,140,55,110,221,138,3,186,249,61,205,77,77,232,145,135,235,199,211,
  60,166,27,182,110,197,101,178,199,97,198,140,25,104,210,45,16,24,15,156,62,117,10,7,246,239,199,171,255,248,7,78,159,58,165,172,86,253,138,236,
  121,24,175,244,244,244,224,201,31,252,0,143,60,246,152,226,69,25,203,76,180,122,30,90,160,168,230,224,254,253,120,228,177,199,198,221,148,28,170,231,
  84,207,71,179,158,15,80,140,158,192,53,155,54,38,60,84,255,204,223,255,174,17,110,106,57,90,114,225,66,196,246,210,210,82,0,82,48,251,247,118,
  236,208,102,104,98,12,167,79,159,70,74,114,50,24,128,19,39,78,224,236,217,179,152,58,117,42,124,94,47,58,58,58,100,29,201,112,250,212,41,88,
  44,22,173,246,133,241,48,189,145,8,141,87,136,130,49,212,171,162,5,0,192,142,119,222,193,141,31,249,8,178,178,179,209,213,213,133,220,188,60,172,
  191,234,42,141,247,119,215,206,157,104,105,105,81,190,83,83,83,211,239,181,244,122,60,74,108,209,88,219,222,250,199,63,112,211,205,55,195,110,183,71,
  124,254,254,123,239,73,195,243,203,150,33,125,202,20,184,58,59,177,127,223,62,148,37,232,90,31,44,107,215,173,67,106,106,170,242,208,73,68,108,247,
  116,119,227,218,45,91,240,183,191,60,61,166,42,227,129,253,251,13,135,184,198,219,80,209,187,239,188,163,17,206,77,77,77,227,118,248,241,169,95,253,
  10,127,253,243,159,49,175,176,16,61,61,61,227,50,178,65,101,69,69,68,232,160,131,251,247,163,183,167,103,204,139,159,137,88,207,255,250,231,63,71,
  116,44,123,122,122,198,133,96,160,122,78,245,124,44,213,115,0,96,151,175,94,195,19,249,66,106,106,42,222,219,245,1,94,127,245,85,124,243,27,255,
  158,240,15,42,241,68,163,165,235,52,216,174,143,65,26,177,143,234,115,62,128,11,168,159,38,16,77,116,70,21,162,113,236,23,243,183,39,113,122,208,
  239,125,255,191,113,235,237,183,227,250,205,215,38,236,214,255,242,87,254,21,119,223,123,47,110,220,114,29,26,234,235,65,16,4,213,115,130,32,198,95,
  61,23,18,253,177,238,238,110,188,245,143,127,224,214,219,111,199,172,217,179,7,126,214,70,243,71,163,108,103,81,230,149,106,132,160,110,94,103,34,255,
  212,223,135,126,187,74,52,50,213,62,208,127,119,160,66,116,18,51,107,246,108,220,122,251,237,120,235,31,255,72,184,224,2,192,115,114,36,130,47,60,
  246,40,181,2,4,65,245,156,32,136,113,90,207,133,129,252,232,31,126,247,59,69,5,39,174,65,141,135,199,251,21,170,6,130,84,179,0,201,64,152,
  198,253,79,45,78,117,219,163,137,75,214,207,249,199,45,68,39,185,87,84,93,158,18,165,177,161,81,233,24,13,100,30,51,65,16,84,207,9,130,24,
  253,122,46,12,180,113,248,195,239,126,143,181,235,214,225,19,159,250,228,160,12,96,137,10,210,40,219,245,194,84,35,80,13,132,32,235,71,128,2,198,
  222,208,254,132,40,75,68,92,78,98,33,250,137,79,125,18,107,215,173,195,31,126,247,123,101,197,236,64,248,249,79,126,138,158,238,110,124,239,251,255,
  141,212,180,84,106,17,8,130,234,57,65,16,227,172,158,155,114,102,205,122,98,32,95,60,121,226,4,54,110,218,132,91,110,187,13,187,118,238,212,196,
  241,76,84,128,246,39,72,99,137,210,104,194,84,35,80,99,132,118,50,20,160,49,68,104,52,33,202,18,16,162,147,221,43,186,104,241,34,252,226,87,
  191,66,121,89,25,190,245,239,255,49,168,99,249,124,62,84,87,87,225,142,237,219,49,103,238,92,188,243,207,127,82,203,64,16,84,207,9,130,24,71,
  245,124,192,98,20,0,14,31,58,132,155,111,190,25,183,222,126,59,14,236,223,151,144,32,53,18,168,209,230,144,162,191,207,162,9,211,120,127,27,113,
  120,72,213,66,215,72,92,146,16,141,187,224,254,223,211,79,195,239,243,225,11,15,63,140,158,238,193,175,12,172,174,170,70,106,106,42,182,221,114,11,
  102,207,158,141,15,118,238,164,22,130,32,168,158,19,4,49,78,234,249,160,196,104,79,119,143,36,72,183,73,130,180,181,181,21,165,37,241,197,130,51,
  244,34,70,19,157,161,207,250,249,60,194,19,138,40,139,150,162,121,74,163,136,212,104,159,147,16,77,140,91,111,191,29,79,254,244,167,96,0,190,240,
  208,195,168,174,170,30,178,99,31,62,116,8,51,103,205,194,182,91,111,197,218,117,235,240,193,206,157,240,121,125,212,90,16,4,213,115,130,32,198,120,
  61,31,148,24,5,128,182,182,54,28,62,116,8,171,215,172,198,93,119,127,12,139,22,47,198,129,125,251,224,243,245,223,64,36,44,72,85,159,199,37,
  231,98,44,88,26,10,225,152,104,88,166,201,26,198,41,53,53,21,79,254,236,167,248,236,231,63,143,154,234,106,124,237,241,127,27,150,120,166,123,119,
  239,1,192,176,237,150,91,240,177,123,238,193,165,139,23,113,233,210,37,106,53,8,130,234,57,65,16,99,184,158,39,28,103,52,214,9,126,238,193,207,
  227,238,123,239,69,119,87,55,254,250,244,95,240,250,43,175,162,62,142,184,112,17,113,68,195,31,196,31,55,116,0,177,70,13,231,155,14,131,168,156,
  172,30,209,217,179,103,227,214,59,110,199,253,159,252,20,82,211,82,241,252,179,207,226,15,191,251,253,128,194,62,36,194,170,213,171,241,173,39,190,131,
  153,51,103,226,216,209,163,248,235,95,158,198,7,239,191,79,173,8,65,80,61,39,8,98,12,214,243,33,19,163,234,6,226,115,15,126,30,171,86,175,
  6,0,148,92,40,193,7,59,223,199,177,35,71,209,80,95,111,40,78,163,138,209,129,136,210,24,34,53,81,209,57,88,17,58,217,132,232,236,217,179,
  49,107,246,108,172,189,124,29,174,221,114,157,146,79,250,228,137,19,248,195,239,126,63,160,172,12,131,233,28,221,125,239,189,184,231,222,123,144,146,154,
  138,238,174,110,165,28,214,215,215,39,148,61,140,32,8,170,231,4,65,245,124,248,234,249,144,139,209,16,11,22,44,192,77,219,110,198,53,155,54,97,
  230,204,153,253,10,70,159,207,7,81,20,193,57,16,146,158,156,115,205,63,253,54,245,123,181,240,236,87,220,42,250,144,105,132,34,83,253,213,191,54,
  220,6,166,124,93,253,89,180,227,153,76,166,73,83,144,27,27,27,177,119,247,110,188,245,230,63,70,60,197,168,254,97,117,205,166,141,216,184,113,19,
  174,217,180,145,90,24,130,160,122,78,16,196,24,171,231,195,38,70,213,204,156,53,19,51,103,206,82,188,165,81,20,41,2,254,128,36,67,67,162,82,
  17,165,242,59,30,222,87,253,26,92,191,175,234,179,232,166,171,68,105,104,11,11,185,63,85,142,76,163,215,44,180,155,34,72,245,175,213,47,24,3,
  4,65,192,100,200,185,116,242,196,9,52,54,54,12,42,166,224,112,178,106,245,106,165,60,18,4,65,245,156,32,136,209,175,231,35,34,70,19,65,227,
  213,84,68,105,66,7,24,192,85,24,222,249,159,140,49,42,213,4,65,16,4,65,16,6,152,199,218,9,49,198,194,130,52,52,204,157,136,40,29,38,
  225,55,208,185,159,36,68,9,130,32,8,130,32,198,145,24,141,16,164,3,21,165,163,40,64,73,136,18,4,65,16,4,65,140,99,49,106,40,72,213,
  162,20,24,80,40,167,132,4,40,137,80,130,32,8,130,32,136,201,43,70,213,162,142,247,19,158,73,47,78,251,19,169,49,179,56,13,209,57,19,4,
  65,16,4,65,16,227,92,140,170,5,94,191,225,154,162,229,140,31,97,225,76,16,4,65,16,4,65,196,143,48,94,78,84,29,231,115,44,158,27,65,
  16,4,65,16,4,145,56,230,241,118,194,49,135,238,73,132,18,4,65,16,4,65,144,24,157,200,162,148,4,40,65,16,4,65,16,4,137,81,67,113,
  56,156,194,148,68,40,65,16,4,65,16,196,48,136,209,207,61,248,224,4,51,105,168,5,41,137,80,130,32,8,130,32,136,225,130,241,177,48,249,146,
  32,8,130,32,8,130,152,156,98,244,234,245,235,39,189,24,29,207,67,240,99,246,220,71,241,188,38,98,72,175,97,255,157,97,56,62,27,71,215,97,
  88,142,59,68,199,28,202,115,99,19,196,166,65,127,127,148,175,195,184,180,127,20,206,121,164,206,147,141,241,4,55,108,4,109,26,206,243,139,181,223,
  184,9,237,68,16,4,65,16,4,65,76,60,72,140,18,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,65,
  98,148,32,8,130,32,8,130,152,248,140,120,208,251,239,124,247,63,19,218,127,247,238,93,216,179,107,87,204,239,254,229,79,255,135,170,170,42,229,253,
  167,62,253,105,20,20,204,137,216,175,170,234,18,254,242,167,63,33,57,57,25,143,127,245,107,81,127,243,248,177,163,248,231,91,111,1,0,138,150,44,
  193,246,187,62,22,117,223,183,223,122,11,199,142,29,197,183,159,248,110,66,118,237,217,189,27,123,118,75,118,125,234,129,79,35,191,160,192,112,191,234,
  170,42,252,229,207,127,138,122,28,135,195,129,181,235,214,97,113,209,18,100,101,103,161,32,191,0,142,228,100,205,62,238,222,94,84,85,87,193,217,226,
  196,133,226,243,56,118,236,24,220,189,189,154,125,54,110,218,132,107,54,109,138,251,252,47,156,47,70,111,111,47,138,139,207,163,90,117,237,1,224,91,
  79,60,129,68,214,65,238,221,189,27,123,246,236,142,185,207,198,141,137,157,31,3,240,95,223,125,2,0,144,95,80,128,79,126,234,129,184,191,219,219,
  219,139,234,234,42,184,123,123,81,92,92,28,97,31,0,124,235,59,79,68,183,103,207,110,236,221,179,7,247,127,234,83,200,207,47,24,112,93,249,222,
  127,126,87,119,175,147,177,102,237,26,44,94,92,132,172,172,44,228,23,20,192,225,112,104,239,181,219,141,234,170,42,56,157,78,92,184,80,140,227,199,
  142,195,237,214,222,235,251,63,249,41,228,231,231,27,95,52,195,50,88,13,183,219,141,226,11,197,184,80,92,76,173,38,65,16,4,49,190,197,104,209,
  146,37,9,237,95,124,254,124,191,223,253,246,119,255,19,255,242,133,71,208,43,11,172,130,130,57,49,127,167,183,183,23,142,228,100,20,20,68,23,10,
  106,49,26,235,88,79,253,250,215,3,179,171,56,108,87,126,65,65,194,223,151,4,228,102,124,234,129,7,34,196,103,132,96,77,78,70,81,209,18,160,
  72,18,157,159,236,237,197,211,127,249,51,246,236,14,11,192,172,172,108,105,159,120,239,163,106,223,226,226,243,248,201,147,79,42,2,55,145,227,0,192,
  5,213,181,136,70,86,118,22,138,138,138,6,84,230,146,29,201,88,156,224,119,215,172,93,171,188,118,58,157,248,233,143,159,212,136,210,88,199,11,9,
  182,252,252,130,132,127,55,26,87,95,179,17,159,252,212,167,34,196,167,81,231,100,113,81,17,22,3,184,102,227,70,220,255,73,55,254,250,244,95,176,
  119,207,158,112,121,203,207,79,232,188,22,47,150,246,189,3,119,194,233,116,226,103,63,253,9,170,171,171,169,245,36,8,130,32,198,167,24,237,79,120,
  170,197,39,0,156,63,127,174,127,177,145,156,140,111,127,247,63,241,181,199,191,18,247,111,237,217,181,11,5,159,254,116,191,130,57,150,176,146,60,80,
  45,241,217,165,19,92,122,59,19,165,160,160,0,143,60,250,104,212,207,63,251,233,7,0,0,127,252,211,159,13,197,233,195,95,120,20,85,85,85,134,
  94,63,35,254,235,187,79,192,225,112,224,145,47,60,26,33,126,139,138,150,96,227,198,77,120,251,159,111,197,33,194,139,35,196,103,241,249,193,123,219,
  94,126,233,197,1,127,247,175,127,249,51,170,171,170,37,79,166,65,7,37,43,43,11,255,250,248,191,225,139,143,61,154,208,113,247,238,217,173,241,36,
  46,46,42,138,41,2,95,121,233,37,195,237,249,5,5,120,248,145,71,162,126,239,243,159,253,44,0,224,247,127,252,163,161,56,125,232,225,71,80,93,
  85,141,234,234,254,239,117,117,117,53,254,250,215,167,81,180,184,8,119,220,121,167,225,181,248,242,191,126,5,95,250,226,255,163,214,147,32,8,130,152,
  152,98,244,197,23,158,31,176,56,251,194,99,143,225,55,191,250,85,92,251,31,59,118,20,159,138,34,70,165,135,110,54,156,206,150,152,222,211,208,48,
  123,255,2,236,60,94,122,225,133,33,189,86,107,214,173,139,249,185,126,24,222,136,181,107,215,197,45,70,67,226,185,170,186,202,80,160,39,247,227,157,
  13,113,161,248,60,94,122,241,197,33,47,59,234,99,38,26,234,183,186,170,26,197,197,231,113,252,248,177,168,211,37,178,178,178,18,62,39,181,55,18,
  0,238,220,126,87,76,49,26,77,80,175,89,179,54,246,189,118,247,127,175,215,172,93,19,151,24,117,187,123,113,161,88,26,142,55,18,163,3,189,22,
  4,65,16,4,49,46,196,104,86,118,86,196,112,117,117,85,149,50,252,222,31,27,55,109,198,249,56,61,142,206,150,22,84,85,85,69,21,155,161,237,
  177,134,192,143,29,61,22,159,93,89,217,131,178,107,44,145,236,72,30,212,247,51,179,178,34,196,108,149,60,63,115,48,232,143,233,118,247,198,45,180,
  137,72,12,231,148,134,234,142,211,73,23,136,32,8,130,152,152,98,116,227,166,205,216,184,105,179,102,219,119,191,243,237,132,134,180,191,240,232,99,113,
  13,157,3,177,135,234,243,231,20,160,55,134,199,41,222,33,122,201,174,77,216,168,91,124,243,159,79,124,103,208,67,245,35,201,183,158,120,2,201,142,
  228,168,158,195,184,175,197,198,77,216,184,81,123,45,254,235,187,223,141,152,198,144,240,249,125,231,59,154,247,23,138,139,149,5,76,68,252,44,94,92,
  132,191,63,243,172,225,103,23,46,20,195,233,116,226,157,183,223,166,11,69,16,4,65,12,25,19,34,180,147,222,195,152,149,149,29,215,247,142,29,59,
  26,245,179,162,162,216,11,151,226,29,162,159,40,20,21,45,49,20,162,213,85,85,120,249,197,23,53,139,161,136,241,139,219,237,86,68,167,30,135,236,
  21,119,187,221,116,161,8,130,32,136,33,99,76,121,70,95,122,225,133,1,205,25,221,179,123,23,214,174,91,23,183,8,13,225,108,105,193,177,163,71,
  177,214,96,254,101,65,65,65,204,185,120,241,14,209,3,192,75,47,190,48,228,115,70,71,26,189,151,177,183,119,96,195,224,47,191,244,226,176,204,25,
  189,247,238,112,248,45,70,245,122,192,84,87,87,225,123,255,245,95,112,56,28,248,253,31,180,11,162,242,243,243,165,149,248,139,139,104,1,19,65,16,
  4,49,100,76,8,207,168,187,215,141,31,255,240,135,3,250,110,52,239,168,35,57,25,249,6,177,74,129,196,134,232,39,10,161,72,7,161,127,52,31,
  115,98,19,203,251,73,11,152,8,130,32,136,161,100,204,47,96,10,9,161,254,168,170,170,194,75,47,188,128,237,31,251,88,66,191,121,252,232,81,224,
  209,196,30,186,177,134,247,141,143,147,61,96,187,146,147,147,13,23,63,85,95,234,95,12,246,23,127,180,170,234,210,136,223,99,163,5,76,0,6,61,
  103,84,127,76,6,105,30,107,188,139,109,148,88,172,99,144,120,86,193,59,250,89,88,86,93,69,113,65,9,130,32,8,18,163,253,98,180,128,9,0,
  238,222,126,103,92,223,127,241,133,231,177,102,221,186,152,225,152,244,244,246,246,226,248,177,163,88,179,118,93,98,2,54,33,187,34,23,48,1,192,61,
  119,109,239,247,187,249,5,5,17,217,157,254,243,137,239,224,216,177,163,216,179,123,183,225,113,1,224,7,63,122,50,230,113,247,236,222,141,227,199,142,
  141,252,61,54,88,192,4,104,135,217,7,130,126,1,19,32,77,9,120,57,142,41,1,223,52,248,110,196,61,31,133,107,165,254,237,189,123,246,224,154,
  141,27,13,63,255,239,31,196,30,21,216,187,103,15,142,31,63,6,130,32,8,130,32,49,58,2,252,248,71,63,196,15,159,252,113,220,113,47,1,105,
  254,103,188,98,212,233,116,106,82,143,142,38,79,253,250,87,216,179,123,23,54,110,218,140,162,37,75,52,158,92,163,197,70,78,167,19,197,231,207,99,
  239,158,221,227,106,37,255,104,17,74,171,41,137,193,221,163,122,46,191,253,223,167,176,119,239,30,92,115,205,70,44,46,42,210,222,107,131,48,76,78,
  167,19,23,138,139,177,119,239,30,74,225,73,16,4,65,140,105,216,213,235,215,243,73,127,17,24,155,48,231,158,95,80,16,33,196,7,186,216,104,144,
  39,54,122,215,100,2,150,29,163,223,201,207,47,128,35,89,151,155,190,215,29,215,176,254,72,220,47,54,142,174,247,176,28,119,136,142,57,148,231,198,
  38,136,77,131,254,254,40,95,135,113,105,255,40,156,243,72,157,231,64,175,231,104,182,255,99,73,215,196,251,91,177,246,155,112,158,209,201,14,45,44,
  154,68,247,186,154,238,53,65,16,4,49,254,17,232,18,16,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,
  49,82,152,63,253,217,207,78,250,139,48,238,51,246,48,202,57,52,106,247,116,36,39,137,83,57,26,190,235,48,198,109,103,19,176,142,143,182,77,108,
  28,151,151,241,88,30,24,213,177,9,123,126,67,113,110,140,115,62,233,87,211,19,4,65,16,4,65,16,163,36,104,215,173,94,77,98,148,32,8,130,
  32,8,130,24,21,104,206,40,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,130,
  32,8,18,163,4,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,18,
  163,4,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,24,127,152,135,243,224,11,22,44,192,198,77,155,48,127,193,2,204,156,53,11,11,22,
  44,160,43,78,16,4,65,16,4,49,198,41,43,43,67,99,67,3,202,203,202,176,103,247,110,148,149,149,13,219,111,13,121,110,250,212,212,84,220,115,
  239,189,184,105,219,54,204,156,57,19,0,208,216,216,136,198,134,6,148,149,149,161,167,187,155,238,48,65,16,4,65,16,196,24,37,37,53,21,11,100,
  71,162,90,203,189,245,230,155,120,238,217,103,209,61,196,90,110,200,196,104,72,132,222,125,239,189,72,77,77,69,89,89,25,222,122,243,77,236,217,179,
  7,141,13,13,116,103,9,130,32,8,130,32,198,25,51,103,205,194,198,141,27,113,211,182,109,88,176,96,1,186,187,187,241,252,179,207,14,169,40,29,
  18,49,186,96,193,2,252,232,39,63,193,204,153,51,113,242,196,9,252,254,119,191,195,201,19,39,232,14,18,4,65,16,4,65,76,16,86,173,94,141,
  207,63,248,32,86,173,94,141,198,198,70,124,245,43,95,25,146,225,251,65,139,209,155,182,109,195,183,191,243,29,116,119,119,227,191,190,251,93,236,217,
  189,155,238,22,65,16,4,65,16,196,4,101,227,166,77,248,214,119,190,131,212,212,84,252,231,119,191,139,183,222,124,115,80,199,51,205,158,53,235,137,
  193,10,209,178,178,50,124,225,225,135,113,238,236,89,186,67,4,65,16,4,65,16,19,152,234,170,42,188,247,222,123,88,181,122,53,238,184,243,78,52,
  54,54,162,124,16,30,210,1,123,70,87,173,94,141,167,126,251,91,73,136,62,244,208,144,79,102,37,8,130,32,8,130,32,198,46,169,169,169,248,205,
  111,127,139,5,11,22,224,145,135,30,26,240,20,205,1,137,209,212,212,84,188,250,198,27,0,128,219,182,109,67,79,79,15,221,17,130,32,8,130,32,
  136,73,70,74,74,10,94,147,135,233,111,191,229,150,1,57,39,7,52,76,255,232,191,252,11,86,175,94,141,47,253,203,191,160,170,170,138,238,4,65,
  16,4,65,16,196,36,196,231,243,225,252,217,179,184,99,251,118,88,109,54,28,62,116,40,225,99,36,156,129,105,230,172,89,184,231,222,123,177,103,247,
  110,156,160,21,243,4,65,16,4,65,16,147,154,19,39,78,96,207,238,221,184,231,222,123,49,115,214,172,225,23,163,27,55,110,4,0,60,247,204,51,
  116,245,9,130,32,8,130,32,8,252,254,183,191,213,232,196,97,21,163,55,109,219,134,178,178,50,242,138,18,4,65,16,4,65,16,0,164,244,161,101,
  101,101,184,105,219,182,225,23,163,11,22,44,192,201,227,199,233,170,19,4,65,16,4,65,16,10,39,143,31,199,130,5,11,134,87,140,174,90,189,90,
  81,191,4,65,16,4,65,16,4,17,34,164,15,67,122,113,88,196,104,8,202,53,79,16,4,65,16,4,65,12,133,62,20,232,210,17,4,65,16,4,
  65,16,163,5,137,81,130,32,8,130,32,8,130,196,40,65,16,4,65,16,4,49,249,48,15,248,155,156,46,30,65,16,4,65,16,4,49,56,6,228,
  25,37,29,74,16,4,65,16,4,65,12,133,62,52,15,252,7,73,146,18,4,65,16,4,65,16,131,131,230,140,18,4,65,16,4,65,16,36,70,9,
  130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,
  130,32,8,18,163,4,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,130,32,136,
  241,199,192,210,129,114,46,253,35,8,130,32,8,130,32,136,144,62,28,0,228,25,37,8,130,32,8,130,32,70,141,129,137,81,114,138,18,4,65,16,
  4,65,16,67,160,15,205,92,76,224,155,178,251,149,115,142,132,190,71,16,4,65,16,4,65,76,108,45,26,26,166,79,80,39,210,48,61,65,16,4,
  65,16,4,49,106,12,104,1,19,7,141,212,19,4,65,16,4,65,16,90,125,56,16,200,51,74,16,4,65,16,4,65,140,26,36,70,9,130,32,8,
  130,32,8,18,163,4,65,16,4,65,16,4,137,81,130,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,144,24,37,8,130,32,8,130,32,8,
  18,163,4,65,16,4,65,16,4,137,81,130,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,144,24,37,8,130,32,8,130,32,136,1,99,30,
  216,215,56,192,41,33,40,65,16,4,65,16,4,161,210,135,3,128,60,163,4,65,16,4,65,16,196,168,65,98,148,32,8,130,32,8,130,32,49,74,
  16,4,65,16,4,65,144,24,37,8,130,32,8,130,32,136,17,195,60,208,47,210,242,37,130,32,8,130,32,8,98,176,144,103,148,32,8,130,32,8,
  130,32,49,74,16,4,65,16,4,65,76,62,204,116,9,8,130,32,8,130,32,128,171,54,108,192,134,171,175,86,222,187,221,110,188,247,238,187,40,41,
  41,49,252,60,196,15,191,255,125,0,64,94,94,30,174,191,241,70,100,102,102,106,190,155,151,151,135,123,239,187,47,226,123,53,53,53,120,246,239,127,
  143,216,126,239,125,247,69,108,119,56,28,184,245,246,219,145,151,151,7,0,216,191,111,31,14,236,223,175,252,174,254,248,59,222,125,23,31,158,60,9,
  0,248,218,55,190,17,241,27,161,239,247,103,147,254,187,37,37,37,120,239,221,119,225,118,187,73,140,18,4,65,16,4,65,12,37,153,89,89,104,109,
  109,197,129,125,251,0,0,211,50,51,241,216,23,191,136,95,253,226,23,40,41,41,137,248,92,47,22,191,250,141,111,224,245,87,95,197,129,125,251,52,
  223,173,169,169,193,235,175,190,10,0,184,231,190,251,112,96,223,62,212,214,212,24,10,186,204,204,76,100,102,102,70,108,255,151,47,126,17,173,173,173,
  120,253,213,87,145,36,11,211,204,172,44,188,254,234,171,112,56,28,72,114,56,240,156,44,96,147,28,14,220,123,223,125,232,115,187,81,82,82,130,133,
  139,22,225,71,178,184,12,209,218,218,106,104,179,30,253,119,175,191,241,70,252,203,23,191,168,136,85,18,163,4,65,16,4,65,16,67,5,231,104,117,
  58,21,79,40,32,121,71,175,187,241,70,105,155,193,231,33,114,243,242,208,218,218,138,247,118,236,80,182,181,201,130,207,45,139,194,208,235,154,154,26,
  148,26,28,35,36,254,78,158,56,97,184,253,127,30,121,68,17,176,181,53,53,88,184,104,145,116,218,186,223,0,36,111,233,194,69,139,148,109,37,81,
  126,47,150,77,33,212,159,149,148,148,224,137,255,250,47,228,229,229,161,166,166,134,196,40,65,16,4,65,16,196,112,242,225,201,147,120,236,255,253,63,
  141,40,188,245,182,219,148,247,173,173,173,56,176,127,191,36,46,57,199,87,191,254,117,124,120,242,36,74,74,74,140,5,94,63,233,212,87,174,90,133,
  231,158,121,38,98,251,142,119,223,197,119,254,243,63,113,112,255,126,148,148,148,160,180,164,4,173,242,48,189,254,184,14,135,3,43,87,173,82,132,49,
  231,92,115,206,0,112,96,255,126,197,59,26,205,166,208,119,35,174,201,137,19,88,185,106,21,137,81,130,32,8,130,32,136,161,132,15,242,243,39,190,
  253,109,172,92,181,10,11,23,45,194,103,62,247,57,56,28,14,252,234,151,191,68,173,74,180,245,119,140,36,135,67,17,137,106,158,123,230,25,28,216,
  191,31,171,86,173,194,173,183,221,134,220,188,60,101,91,72,80,254,241,207,127,6,32,121,73,15,236,223,175,124,54,24,155,134,234,59,36,70,9,130,
  32,8,130,32,6,64,102,102,38,90,157,78,229,125,73,73,9,222,120,237,181,136,253,66,67,230,31,158,60,169,44,28,186,238,134,27,112,203,173,183,
  226,215,255,243,63,42,37,23,93,202,229,230,229,161,182,186,218,240,28,22,46,90,132,3,251,247,43,194,54,55,47,15,255,246,181,175,41,130,179,228,
  194,5,60,249,195,31,26,139,71,206,241,186,193,57,247,103,83,232,187,17,231,51,109,26,74,75,75,71,89,140,114,244,235,102,38,8,130,32,8,130,
  24,87,132,180,141,74,227,220,125,239,189,146,224,227,220,240,115,245,119,63,253,185,207,225,63,191,253,237,240,194,36,131,253,121,232,189,193,49,86,174,
  92,169,12,247,171,113,247,246,226,238,123,239,69,77,117,117,216,203,170,62,54,231,225,227,70,145,109,81,117,91,44,155,12,190,155,155,151,135,21,161,
  169,4,250,239,12,80,26,146,103,148,32,8,130,32,8,66,102,253,134,13,88,32,123,57,51,51,51,209,218,218,138,247,85,139,146,212,159,135,120,254,
  153,103,80,90,82,130,15,79,158,196,47,126,243,27,148,150,148,192,225,112,192,225,112,224,201,31,252,32,238,223,94,177,106,149,161,135,210,237,118,227,
  185,103,158,193,227,95,255,58,218,90,91,225,118,187,145,155,151,135,255,251,195,31,226,62,246,227,95,255,186,230,125,109,77,13,158,151,231,166,70,179,
  41,36,124,213,223,13,77,15,24,202,208,78,108,237,202,85,113,235,216,213,107,86,227,169,223,253,14,15,126,230,179,56,126,236,24,149,88,130,32,8,
  130,32,38,12,211,116,97,149,90,91,91,149,21,241,70,159,171,133,93,72,156,57,28,14,228,230,229,253,255,236,157,119,124,27,245,253,255,95,167,105,
  201,43,177,44,103,120,37,36,182,19,39,33,113,98,103,56,64,22,148,36,36,161,133,36,148,126,217,171,148,194,151,182,140,14,90,202,248,181,253,150,
  213,210,150,210,66,153,165,148,221,146,5,20,66,66,67,246,114,150,19,219,25,222,78,44,201,137,135,36,107,221,231,247,199,13,159,164,147,45,239,245,
  126,242,16,177,78,167,211,189,63,159,247,221,189,62,239,207,120,195,229,114,5,141,21,85,138,57,73,80,170,125,86,213,193,164,32,105,56,128,114,54,
  190,217,108,134,37,57,57,226,119,115,66,132,166,36,112,171,42,43,59,180,41,244,187,37,237,204,186,207,47,40,192,75,175,190,130,239,221,117,23,246,
  239,219,31,117,185,83,100,148,32,8,130,32,8,2,194,82,76,14,149,201,67,209,126,46,137,188,246,4,91,123,98,179,42,138,217,233,106,199,118,185,
  92,112,181,243,221,246,206,167,35,155,218,251,110,79,209,165,116,160,52,90,148,32,8,130,32,8,130,232,9,125,168,235,235,31,36,8,130,32,8,130,
  32,8,9,13,21,1,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,
  18,163,4,65,16,4,65,16,4,209,91,208,58,163,4,65,16,4,65,16,131,144,25,121,121,88,114,249,229,112,185,92,216,185,99,7,138,14,30,4,
  0,60,240,208,67,120,241,133,23,132,76,77,233,233,72,75,79,199,206,29,59,240,192,67,15,5,125,255,217,167,159,38,49,74,16,4,65,16,4,65,
  116,65,136,206,152,129,233,51,102,224,197,23,94,0,0,172,185,238,58,184,92,46,148,150,148,32,201,98,193,138,149,43,241,222,187,239,194,36,102,103,
  2,0,198,24,158,123,230,153,1,103,75,215,186,233,25,173,50,74,16,4,65,16,4,209,95,204,45,44,196,251,239,188,35,100,95,114,185,240,254,59,
  239,96,241,146,37,0,132,172,74,12,64,90,122,186,160,217,68,221,102,50,153,144,157,157,141,236,236,108,225,179,158,166,139,250,80,215,215,63,72,16,
  4,65,16,4,65,116,15,147,201,20,148,223,222,229,114,193,100,50,1,140,129,49,134,13,235,214,225,230,91,111,197,151,95,124,33,235,54,147,217,140,
  236,156,28,89,176,86,71,145,126,180,47,160,110,122,130,32,8,130,32,136,65,70,89,105,41,178,115,114,80,90,82,2,0,200,206,201,65,117,85,149,
  252,185,219,229,194,161,131,7,49,183,176,16,13,14,135,44,64,55,172,91,55,224,108,33,49,74,16,4,65,16,4,49,200,216,252,197,23,184,251,158,
  123,48,125,198,12,0,66,151,252,95,254,252,231,160,125,118,238,216,17,36,70,45,201,201,88,177,106,85,219,231,219,183,195,33,126,70,98,148,32,8,
  130,32,8,130,136,26,183,203,133,223,61,243,140,60,246,83,25,21,125,255,221,119,229,191,223,124,237,53,213,191,1,192,229,118,15,8,91,72,140,18,
  4,65,16,4,65,12,82,148,34,84,109,155,50,242,41,117,233,15,52,104,209,123,130,32,8,130,32,8,130,196,40,65,16,4,65,16,4,49,252,160,
  110,122,130,32,8,130,32,136,65,134,201,108,198,234,181,107,97,177,88,80,86,90,138,141,235,215,3,0,86,175,93,139,15,222,123,79,222,79,249,254,
  170,149,43,145,149,157,13,135,195,129,45,155,55,163,186,170,10,171,215,174,13,91,115,244,247,207,62,219,167,182,116,41,50,74,43,140,18,4,65,16,
  4,65,244,31,115,231,205,67,117,85,21,126,255,236,179,112,185,92,88,36,46,120,159,154,150,22,180,159,244,254,218,181,107,225,114,185,240,251,103,159,
  197,198,245,235,113,237,154,53,48,153,205,248,224,189,247,240,251,103,159,5,99,12,191,127,246,217,110,9,209,174,234,67,93,95,255,32,65,16,4,65,
  16,4,209,61,28,14,7,230,204,155,135,170,170,42,124,185,121,115,208,103,19,179,179,195,52,91,90,90,154,28,33,117,56,28,216,242,229,151,152,51,
  111,30,182,40,190,219,95,218,142,186,233,9,130,32,8,130,32,6,25,135,138,138,224,112,56,48,125,250,116,172,94,187,22,155,54,108,192,161,162,34,
  0,64,118,136,24,85,195,229,114,33,45,36,138,218,95,208,4,38,130,32,8,130,32,136,65,198,181,107,214,192,97,183,99,227,134,13,248,253,179,207,
  98,225,162,69,0,0,198,24,54,110,216,32,191,152,152,190,93,78,23,42,146,149,157,141,42,197,18,80,172,31,211,188,83,100,148,32,8,130,32,8,
  98,144,81,86,90,138,251,127,244,35,28,62,116,8,105,105,105,216,189,107,87,187,251,111,218,176,65,222,223,146,148,36,111,27,8,144,24,37,8,130,
  32,8,130,24,100,28,62,116,8,101,165,165,72,75,79,199,238,157,59,229,197,237,63,124,255,253,160,253,164,247,213,213,213,248,191,95,253,10,89,217,
  217,216,237,112,132,165,1,13,253,30,137,81,130,32,8,130,32,8,162,93,220,110,55,202,74,75,131,182,85,87,87,183,251,62,116,255,72,251,245,37,
  93,28,51,74,115,233,9,130,32,8,130,32,136,238,235,67,93,151,127,139,145,32,37,8,130,32,8,130,32,186,165,69,105,54,61,65,16,4,65,16,
  4,209,127,144,24,37,8,130,32,8,130,32,72,140,18,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,130,196,40,65,16,4,65,16,4,65,
  98,148,32,8,130,32,8,130,32,6,134,24,165,213,156,8,130,32,8,130,32,136,158,212,139,58,214,5,133,201,192,192,72,153,18,4,65,16,4,65,
  16,10,125,216,38,72,163,215,137,212,77,79,16,4,65,16,4,65,244,27,36,70,9,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,137,81,
  130,32,8,130,32,8,130,232,51,116,84,4,4,65,16,4,65,16,131,147,235,111,184,1,123,119,239,198,201,178,50,152,76,38,92,127,195,13,48,153,
  76,112,187,221,248,231,91,111,97,98,86,22,174,92,182,12,0,240,217,39,159,224,200,225,195,242,119,151,46,95,142,169,211,166,193,237,118,195,100,50,
  225,213,151,95,70,67,67,3,126,253,212,83,168,169,174,150,247,123,225,15,127,192,210,229,203,113,217,194,133,248,217,195,15,3,0,146,146,146,240,139,
  199,31,199,11,127,248,3,78,150,149,145,24,37,8,130,32,8,130,24,110,92,185,108,25,82,83,83,177,71,124,63,33,43,11,95,109,221,138,83,101,
  101,152,122,241,197,184,108,225,66,76,204,202,194,51,191,253,45,98,76,38,220,126,231,157,65,98,148,49,134,127,125,244,145,188,127,193,156,57,248,236,
  147,79,80,83,93,141,23,254,240,7,0,192,55,175,185,6,19,178,178,192,24,67,131,195,129,9,89,89,242,254,53,213,213,232,137,181,149,186,44,70,
  105,97,39,130,32,8,130,32,136,254,227,171,173,91,131,116,153,82,104,166,166,166,226,228,201,147,152,152,149,5,6,192,237,118,171,234,183,217,179,103,
  99,226,196,137,152,118,241,197,120,229,229,151,193,0,152,76,38,76,200,202,2,0,76,20,5,46,38,78,196,158,221,187,49,109,218,52,156,44,43,195,
  180,139,47,150,127,175,187,154,144,198,140,18,4,65,16,4,65,12,66,36,129,25,202,236,57,115,208,208,208,16,85,247,121,77,77,13,78,158,60,137,
  61,187,119,99,193,194,133,128,40,70,39,78,156,136,137,19,39,226,95,31,125,132,134,134,6,121,223,212,180,52,36,37,37,161,193,225,232,49,59,168,
  155,158,32,8,130,32,8,98,48,163,88,96,126,233,178,101,104,104,104,192,158,221,187,197,143,218,95,136,190,166,186,26,39,79,158,68,77,117,53,110,
  187,227,14,0,128,195,225,192,167,159,124,162,250,59,53,213,213,184,114,217,50,28,57,124,24,169,105,105,61,146,158,147,196,40,65,16,4,65,16,196,
  16,96,246,236,217,40,152,61,27,13,13,13,40,152,61,27,123,119,239,198,127,183,110,197,131,226,164,163,207,84,4,230,55,175,185,70,158,192,164,246,
  121,40,123,118,239,198,247,239,187,15,255,252,199,63,4,49,218,3,112,249,51,242,162,150,180,179,242,243,241,151,151,95,194,237,183,220,130,189,123,246,
  82,173,19,4,65,16,4,65,16,0,128,130,217,5,120,229,245,215,113,247,157,119,97,255,190,125,81,127,143,198,140,18,4,65,16,4,65,16,253,70,
  215,196,40,77,165,39,8,130,32,8,130,32,122,64,31,118,125,204,40,35,69,74,16,4,65,16,4,65,116,15,234,166,39,8,130,32,8,130,32,72,
  140,18,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,130,196,40,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,72,140,18,4,
  65,16,4,65,16,36,70,9,130,32,8,130,32,8,130,196,40,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,72,140,18,4,65,16,4,
  65,16,131,143,46,165,3,101,160,244,244,4,65,16,4,65,16,68,176,62,236,10,20,25,37,8,130,32,8,130,24,160,228,76,154,132,228,228,228,33,
  109,35,137,81,130,32,8,130,32,136,1,202,195,63,249,49,10,47,153,79,98,148,32,8,130,32,8,130,232,123,24,27,250,3,35,73,140,18,4,65,
  16,4,65,12,16,190,127,223,125,152,223,78,36,244,161,31,255,24,121,51,243,72,140,18,4,65,16,4,65,16,221,103,70,94,30,86,93,125,181,252,
  62,61,35,29,243,230,11,98,212,100,50,193,231,243,33,49,33,17,0,144,51,41,7,217,57,217,48,153,204,0,0,75,114,50,110,189,253,54,152,205,
  230,14,127,135,49,38,191,6,26,58,114,3,130,32,8,130,32,136,254,33,111,102,30,10,231,207,199,254,125,251,144,57,110,28,106,107,235,16,99,50,
  225,254,31,253,16,126,127,0,135,15,31,65,124,98,34,238,189,255,126,24,12,6,84,84,84,34,61,35,3,185,231,207,99,218,244,139,81,56,127,62,
  214,125,188,14,46,151,75,16,154,10,193,25,141,240,228,56,174,237,95,142,147,223,147,24,37,8,130,32,8,130,24,130,164,103,164,227,214,219,110,199,
  11,127,250,19,236,118,59,142,30,57,138,132,196,68,204,187,100,62,170,171,107,240,206,187,239,162,190,190,30,206,150,22,104,181,90,232,117,58,180,122,
  60,240,122,189,48,199,154,145,148,100,193,232,209,163,145,150,150,138,24,147,9,69,7,139,224,108,105,65,192,239,199,138,149,43,193,192,176,97,253,134,
  200,226,51,228,125,168,112,229,4,101,10,78,18,166,125,32,78,187,38,70,69,229,77,16,4,65,16,4,65,116,78,67,165,165,165,226,150,91,111,193,
  201,147,167,224,110,109,197,191,63,254,24,165,101,101,168,175,175,7,31,8,32,49,49,17,99,70,143,198,212,41,83,160,55,24,96,179,217,80,93,93,
  133,154,154,90,212,159,59,135,35,71,143,32,38,38,6,227,50,51,113,209,184,241,88,180,100,9,206,159,111,64,193,156,217,40,57,126,162,77,171,133,
  41,81,78,117,45,80,165,220,100,10,157,199,66,133,105,52,250,176,207,196,40,65,16,4,65,16,4,17,53,133,243,231,99,199,246,237,168,170,172,194,
  127,255,187,13,12,192,153,138,114,236,220,177,19,54,155,13,156,134,67,92,92,28,210,82,199,98,234,212,105,24,53,42,5,126,95,0,77,205,77,48,
  155,204,200,72,207,64,66,66,34,234,235,235,225,112,56,224,114,187,113,252,248,113,156,57,125,6,153,153,153,152,52,105,18,78,159,62,131,173,91,183,
  2,140,97,244,232,209,0,128,179,103,207,182,47,22,21,2,53,76,148,2,224,164,177,166,157,17,165,36,70,9,130,32,8,130,32,6,150,16,189,245,
  246,219,48,49,107,34,106,106,106,209,112,225,60,118,237,218,133,210,210,82,248,253,126,36,39,91,144,146,146,130,9,19,38,96,252,248,241,224,121,30,
  206,150,22,180,180,56,225,243,251,0,0,122,189,1,35,70,140,64,92,108,44,70,143,30,13,135,195,129,250,122,27,220,173,110,156,58,125,10,231,234,
  207,193,233,108,193,220,194,66,84,156,57,131,69,139,23,161,181,181,21,127,252,253,243,97,226,51,8,133,64,101,138,207,184,62,20,165,36,70,9,130,
  32,8,130,32,122,145,29,219,183,99,194,196,9,24,147,154,138,195,199,138,113,98,215,46,112,28,48,227,226,139,49,106,212,104,36,37,141,132,222,96,
  132,207,235,133,195,225,128,199,227,129,223,31,144,39,35,241,140,129,211,112,208,235,245,208,105,181,48,24,140,72,76,76,68,90,90,26,92,78,39,90,
  156,78,52,183,180,224,240,145,35,168,169,173,69,193,172,124,52,54,54,225,223,31,125,20,222,45,31,26,29,85,10,80,165,48,85,8,82,41,122,202,
  137,223,103,16,198,152,246,148,40,37,49,74,16,4,65,16,4,209,11,88,146,147,225,118,185,224,114,185,80,91,91,135,154,186,179,184,112,254,60,46,
  158,54,13,113,113,177,136,141,141,133,207,231,131,167,181,21,45,78,39,188,94,31,120,62,32,206,138,135,44,68,197,55,224,32,68,47,57,13,7,48,
  30,122,157,14,35,70,140,64,82,146,5,238,86,55,26,26,26,80,83,91,139,93,123,118,35,38,38,6,6,131,1,96,12,214,81,41,176,157,171,87,
  23,161,42,145,81,181,168,104,168,32,5,199,201,75,69,105,52,221,91,41,148,196,40,65,16,4,65,16,68,47,240,224,67,15,33,198,20,131,109,219,
  182,161,213,227,197,182,109,219,48,109,218,84,76,202,153,132,154,218,90,92,184,112,1,110,119,43,24,207,131,23,133,93,76,76,12,56,142,67,66,124,
  2,24,56,4,2,60,124,126,47,252,94,47,90,61,30,248,124,94,240,129,0,0,64,163,209,128,49,192,31,240,67,171,209,34,37,37,5,14,135,29,
  103,206,148,99,228,136,145,152,154,155,139,105,23,79,195,164,201,185,248,116,211,38,28,59,122,180,77,136,138,226,86,77,152,170,137,82,46,244,51,81,
  144,2,0,207,243,221,138,144,146,24,37,8,130,32,8,130,232,5,94,123,245,85,172,185,238,58,196,152,76,216,242,223,255,162,161,161,1,22,139,5,
  245,182,122,56,236,118,33,178,8,128,49,30,140,49,164,166,166,98,252,184,241,112,123,252,48,232,13,240,120,60,104,109,245,192,229,118,161,181,213,13,
  77,75,11,90,91,91,225,243,182,194,239,243,182,69,81,1,104,180,66,116,210,108,54,195,235,245,160,236,100,25,76,38,19,242,103,206,196,222,61,123,
  80,86,86,22,28,233,4,130,186,240,131,132,105,104,36,20,8,143,138,134,8,210,238,44,168,79,98,148,32,8,130,32,8,162,23,40,45,41,193,254,
  253,251,81,82,90,138,226,99,199,48,225,162,139,48,34,49,17,167,78,159,14,18,162,214,228,100,100,101,103,35,51,35,3,137,137,137,104,108,110,69,
  131,163,1,13,13,13,208,112,94,112,0,56,112,208,106,117,208,233,244,224,3,12,60,227,192,51,15,248,128,95,254,61,198,24,82,82,70,163,180,164,
  12,231,207,159,199,177,99,199,16,103,54,163,193,225,64,107,107,171,176,76,83,136,8,149,196,169,188,140,147,36,48,21,145,208,72,130,20,33,127,51,
  144,24,37,8,130,32,8,130,232,119,102,228,205,192,244,25,121,56,125,234,20,2,60,143,35,71,143,130,227,56,140,29,51,22,78,167,11,94,143,23,
  12,66,36,113,84,202,40,204,156,153,135,209,99,198,192,160,215,3,0,140,70,61,244,122,189,48,22,147,19,68,30,207,243,8,248,3,8,4,2,224,
  121,94,212,128,58,112,28,3,99,126,57,50,105,48,232,145,100,73,66,189,205,134,11,141,23,80,118,242,36,46,158,54,13,241,241,241,72,77,77,197,
  103,159,126,170,80,161,193,226,148,147,102,203,75,17,78,81,148,42,187,237,131,38,50,73,66,85,41,78,69,129,221,25,40,55,61,65,16,4,65,16,
  68,15,146,150,158,142,121,133,243,176,104,201,18,148,148,148,192,110,179,33,49,33,17,217,217,89,104,110,110,18,197,36,67,124,124,60,242,242,242,48,
  118,236,88,89,136,50,198,160,215,105,160,55,24,132,49,161,0,24,47,138,81,94,18,163,1,48,158,129,49,64,163,209,129,227,180,242,119,1,96,212,
  168,81,0,56,248,124,126,156,171,175,71,99,99,35,166,79,159,142,196,196,68,196,197,197,181,117,169,43,243,213,135,110,19,143,23,180,8,190,114,49,
  124,73,120,42,94,109,98,84,56,223,104,161,200,40,65,16,4,65,16,68,15,178,97,221,122,28,216,127,0,215,172,94,141,178,147,39,193,105,52,24,
  53,42,5,22,139,5,167,78,157,18,162,162,60,195,232,209,99,48,122,204,104,232,116,193,114,76,195,1,102,115,12,180,90,29,56,0,60,99,8,240,
  60,252,98,100,52,192,243,224,25,15,112,0,7,13,116,58,35,2,1,32,16,240,0,0,18,19,18,97,52,24,224,241,122,225,116,185,80,85,83,13,
  179,201,132,61,187,118,5,253,142,28,17,85,102,102,18,163,156,202,238,122,101,87,189,180,15,83,238,35,11,214,224,114,144,150,127,234,8,138,140,18,
  4,65,16,4,65,244,48,233,25,25,56,125,230,12,106,107,107,48,114,196,8,76,191,120,58,90,156,78,180,122,60,96,140,97,194,196,9,152,153,55,
  3,70,131,65,245,251,102,147,1,6,163,1,28,167,1,99,60,252,126,31,252,62,31,2,254,128,28,117,212,112,26,104,181,90,232,116,58,24,12,38,
  161,27,157,1,49,166,24,36,39,39,3,0,124,62,31,206,213,215,67,167,215,203,51,246,149,209,208,160,232,40,218,210,129,202,17,81,229,123,32,44,
  42,26,49,5,104,200,113,219,131,34,163,4,65,16,4,65,16,61,196,221,247,124,15,38,179,25,77,77,205,248,122,199,14,104,53,90,164,166,142,197,
  216,212,177,176,213,215,131,231,121,24,13,70,140,26,53,26,9,137,137,17,197,26,7,192,100,18,150,121,242,251,121,248,124,126,248,253,126,4,248,0,
  192,0,78,195,65,171,21,196,168,70,163,133,70,171,133,203,213,12,230,23,38,52,141,24,145,8,135,221,6,143,215,135,86,119,43,206,55,52,96,222,
  188,121,136,139,143,199,161,162,34,216,234,235,229,223,14,154,160,164,152,41,47,143,15,13,137,144,134,206,164,111,15,249,55,218,217,151,34,163,4,65,
  16,4,65,16,61,132,203,229,134,217,100,194,136,145,35,81,93,85,133,132,196,4,76,201,205,133,223,239,71,115,75,11,24,207,160,211,233,225,243,241,
  104,108,114,195,227,13,128,143,16,60,76,136,51,67,171,213,32,224,247,195,235,245,10,66,20,194,50,78,122,157,14,122,189,17,122,189,1,6,131,30,
  70,131,30,35,71,90,0,113,8,64,124,124,60,70,140,28,9,0,240,249,253,104,106,110,134,49,198,8,183,203,5,94,90,215,84,41,24,149,227,66,
  129,224,72,168,34,66,138,144,237,145,162,163,202,237,29,69,71,41,50,74,16,4,65,16,4,209,67,188,249,250,235,0,128,239,223,255,191,168,173,171,
  67,118,86,22,82,83,83,97,171,175,135,215,235,5,0,152,205,177,224,56,13,108,245,118,232,245,58,232,13,6,196,196,24,17,99,212,195,96,208,65,
  175,19,38,36,233,116,66,234,79,143,199,3,79,171,7,26,78,3,157,65,7,189,78,15,157,94,7,157,78,7,173,70,11,141,86,3,173,70,156,196,
  36,42,91,163,193,8,75,146,5,77,141,77,240,248,188,104,113,182,160,172,180,12,85,149,149,114,180,147,73,209,77,197,184,79,57,242,169,140,144,42,
  198,135,6,165,9,85,8,79,174,3,193,201,243,124,196,76,77,93,18,163,161,179,166,8,130,32,8,130,32,136,54,252,62,63,220,46,39,50,51,51,
  225,243,251,225,116,186,132,25,240,96,0,7,120,189,94,52,53,93,16,186,216,53,26,232,116,122,232,245,58,152,99,99,145,16,103,130,86,167,17,51,
  50,233,225,241,120,160,213,106,229,20,159,130,16,213,66,171,209,130,211,104,192,113,194,248,81,157,40,98,25,132,92,246,230,88,51,226,19,226,225,182,
  217,209,234,241,96,100,66,98,155,0,85,8,79,136,226,84,24,115,170,46,72,149,130,85,18,164,161,34,85,142,181,178,16,145,170,88,24,95,173,187,
  158,34,163,4,65,16,4,65,16,61,64,118,118,54,126,248,192,143,192,243,60,14,29,62,2,163,209,8,189,94,143,230,230,102,248,252,62,89,144,5,
  2,1,180,182,122,192,113,30,65,156,73,235,120,114,28,52,118,97,28,168,94,175,135,217,108,68,32,192,35,201,50,18,122,157,30,90,173,86,222,95,
  158,223,206,41,198,124,50,38,175,95,10,112,208,235,116,48,24,12,194,111,250,3,136,79,72,192,130,133,11,208,216,216,132,162,131,7,195,69,105,4,
  65,10,197,182,118,199,137,118,16,169,36,49,74,16,4,65,16,4,209,139,56,28,14,108,220,176,1,60,207,144,158,145,1,191,223,143,154,154,26,140,
  29,51,86,212,106,130,80,244,250,124,112,187,93,109,93,229,140,201,51,221,133,133,237,253,240,249,124,104,109,109,69,82,210,72,44,184,236,82,180,186,
  221,112,185,90,225,241,122,225,245,122,133,201,76,1,94,72,37,202,243,224,25,224,247,251,196,25,236,194,194,243,94,159,15,110,151,75,62,63,183,203,
  5,123,125,189,60,163,95,22,155,42,130,84,84,143,193,34,21,8,238,174,87,91,226,9,157,23,164,36,70,9,130,32,8,130,32,122,72,140,110,88,
  191,1,0,112,207,189,247,66,163,213,161,186,170,26,113,177,113,48,26,141,240,120,61,0,3,188,30,15,156,78,39,56,141,6,188,152,81,41,192,243,
  194,68,37,159,15,62,175,15,126,191,31,122,189,30,58,157,1,78,167,19,99,70,91,193,24,224,243,7,224,241,120,225,241,248,224,118,123,100,97,202,
  243,1,212,219,206,137,115,134,24,252,62,63,46,156,191,128,243,23,26,161,209,8,209,214,150,150,22,84,150,151,135,117,185,171,69,72,161,20,141,157,
  20,157,242,228,165,8,57,236,73,140,18,4,65,16,4,65,244,18,38,179,9,233,105,233,208,233,117,176,88,44,112,56,28,168,183,157,67,70,70,166,
  48,6,147,241,240,122,61,112,185,157,208,105,245,240,139,235,134,250,3,126,4,252,1,248,124,62,121,29,81,131,65,15,141,134,67,107,171,7,60,207,
  192,113,16,178,51,233,98,96,50,25,97,54,199,192,231,243,195,235,245,193,235,245,163,162,226,148,60,115,221,229,118,193,225,112,32,192,243,208,233,116,
  208,235,116,224,3,1,196,198,198,194,233,116,170,10,210,208,73,72,65,227,73,21,226,18,8,159,196,212,25,66,5,41,137,81,130,32,8,130,32,136,
  30,226,103,143,60,2,139,197,130,202,202,42,140,30,61,26,149,149,149,112,56,26,144,146,50,10,58,157,14,190,0,47,119,193,235,117,60,252,82,100,
  52,16,8,90,208,94,167,19,102,217,115,28,7,143,199,11,175,215,15,163,177,77,182,105,53,28,76,49,122,24,141,58,4,252,70,248,3,1,52,56,
  236,162,216,227,209,220,212,140,166,230,102,113,95,13,140,6,3,70,143,25,131,140,140,12,156,44,43,67,125,125,189,44,52,229,37,152,68,209,169,204,
  69,31,65,77,170,119,213,147,24,37,8,130,32,8,130,232,95,54,110,216,8,183,219,133,140,140,76,100,102,102,98,215,174,157,112,56,26,96,183,219,
  145,158,158,1,175,215,11,22,8,192,231,245,130,241,194,146,71,82,55,189,180,44,147,70,163,129,94,175,135,94,175,7,56,14,94,175,23,78,87,43,
  140,198,184,176,223,211,112,28,52,122,14,110,183,19,231,234,235,133,241,167,60,143,243,231,207,35,32,9,91,189,14,166,152,24,148,149,149,193,108,50,
  161,177,177,81,85,32,202,19,169,20,51,238,1,132,117,229,71,90,226,169,83,40,126,79,215,229,3,48,90,220,137,32,8,130,32,8,66,201,174,29,
  59,0,0,158,86,15,230,204,155,7,115,108,44,154,91,154,209,208,208,128,140,140,12,112,224,192,243,126,248,125,173,8,4,120,164,166,165,33,16,240,
  227,194,249,70,180,182,182,2,224,132,101,155,244,58,97,246,60,132,148,158,78,151,27,241,241,102,232,180,225,18,208,235,245,226,235,175,183,193,233,108,
  1,99,12,110,183,27,231,69,193,169,209,104,96,138,49,129,227,56,92,56,127,30,23,206,159,15,203,51,143,144,25,243,178,248,236,174,224,148,126,66,
  229,56,202,232,40,101,96,34,8,130,32,8,130,232,65,178,178,179,177,116,217,82,36,37,37,33,45,53,13,126,159,31,245,245,54,156,59,119,14,122,
  189,14,12,64,32,192,35,35,35,21,115,231,228,227,178,75,10,145,158,158,10,189,94,47,78,54,210,65,175,213,66,171,209,0,224,16,224,121,120,61,
  94,248,196,84,159,161,148,148,148,160,188,188,66,206,5,223,224,104,144,187,251,245,122,61,70,38,38,194,100,50,33,53,45,13,90,157,54,124,5,38,
  149,28,244,221,13,58,134,101,102,106,231,120,36,70,9,130,32,8,130,32,122,144,69,139,23,35,45,45,29,245,245,245,184,104,252,120,33,13,167,219,
  13,103,139,19,198,152,24,48,198,48,254,162,139,144,159,159,15,107,114,50,70,38,141,68,178,53,25,6,163,1,90,173,6,26,173,6,26,141,22,156,
  134,3,56,65,96,250,253,62,248,188,126,240,33,185,67,143,31,63,142,125,251,246,163,177,169,81,94,214,169,238,108,29,0,97,217,38,83,76,12,70,
  36,38,194,235,245,34,53,53,21,70,131,49,122,49,217,238,14,42,105,67,67,222,117,124,8,97,63,26,51,74,16,4,65,16,4,209,131,124,248,254,
  251,112,185,92,240,7,2,88,176,104,17,50,51,199,225,244,169,83,56,127,225,60,0,134,9,19,39,98,230,204,89,176,38,39,131,227,56,56,157,46,
  212,213,158,133,215,227,21,82,123,138,130,148,3,7,48,128,5,2,240,251,125,240,120,188,136,49,26,160,215,107,225,247,251,5,33,186,127,31,234,207,
  213,195,31,240,131,49,38,164,14,245,8,105,71,99,140,6,88,45,22,180,186,92,56,83,91,11,179,217,12,151,184,238,40,83,89,162,137,139,36,58,
  21,251,6,117,185,171,124,214,21,72,140,18,4,65,16,4,65,244,32,14,135,3,128,48,57,201,231,245,34,127,230,76,156,63,223,128,234,154,90,100,
  101,77,68,254,172,124,36,91,44,208,104,52,104,108,106,194,198,141,27,97,183,219,161,215,27,97,214,199,67,171,213,66,171,21,210,132,10,203,65,9,
  221,250,94,175,31,62,127,0,205,205,141,216,185,115,39,142,28,57,2,158,231,21,89,151,128,218,154,90,48,0,58,173,22,35,19,71,96,148,53,25,
  21,229,21,0,32,11,209,246,8,91,216,94,69,116,118,248,158,196,40,65,16,4,65,16,68,255,194,24,195,141,55,221,132,172,156,108,156,62,125,6,
  245,54,27,14,22,29,4,99,64,75,139,19,54,155,13,177,177,177,216,184,113,35,206,158,173,3,24,224,247,249,224,241,184,97,181,166,192,229,244,195,
  160,55,8,107,144,250,125,104,108,60,143,218,218,74,52,56,236,168,183,213,195,233,116,202,185,238,37,33,234,247,7,80,123,246,44,0,32,46,46,22,
  201,73,73,48,26,141,152,52,41,7,39,79,158,68,115,83,243,128,44,43,18,163,4,65,16,4,65,16,189,33,72,193,176,123,199,78,236,222,189,27,
  133,151,94,138,150,150,22,108,217,178,21,135,15,31,198,164,156,73,48,155,205,112,183,186,101,49,41,9,210,218,218,106,48,158,161,182,166,82,62,14,
  83,140,21,149,38,42,73,175,86,79,43,234,207,157,67,109,157,48,86,212,96,208,99,116,74,10,248,64,0,167,79,158,194,184,139,198,35,224,15,116,
  87,93,183,205,186,87,166,12,5,20,11,232,119,81,140,178,78,45,82,202,183,21,10,45,237,68,16,4,65,16,4,161,10,207,120,252,253,141,55,5,
  213,196,128,178,146,18,92,122,201,37,8,4,2,56,117,234,20,182,111,223,14,127,192,143,132,248,120,88,44,201,136,139,19,82,134,26,99,140,80,74,
  61,73,112,2,144,51,53,121,60,30,184,93,110,52,52,52,160,177,169,17,94,175,15,60,99,208,104,52,72,136,143,197,152,148,81,72,78,178,160,180,
  164,4,129,64,0,135,15,29,14,58,55,185,235,93,126,195,137,41,63,33,46,237,36,101,92,18,255,22,181,159,56,69,30,28,56,48,142,107,219,79,
  236,208,151,230,226,203,231,204,41,4,42,23,44,86,153,188,137,65,199,58,33,99,89,132,191,9,130,32,8,130,32,136,80,221,36,168,165,49,99,198,
  96,213,55,191,9,183,203,133,139,167,77,131,78,167,67,73,105,9,26,27,27,209,220,226,68,115,115,11,180,58,45,146,70,38,33,201,98,65,124,124,
  28,244,58,189,44,224,24,99,8,4,2,112,58,157,104,106,110,194,133,11,141,104,110,110,134,207,239,151,101,171,86,171,69,98,124,60,82,82,82,96,
  25,145,136,184,184,88,100,100,102,224,204,233,51,93,59,121,46,84,1,70,94,232,190,237,83,165,70,100,96,178,180,109,251,127,219,161,57,89,224,82,
  55,61,65,16,4,65,16,68,143,43,209,54,33,122,251,29,119,128,231,25,222,249,231,219,240,120,60,152,153,95,128,216,88,51,74,75,75,81,119,246,
  44,188,94,63,120,198,195,238,112,160,161,161,1,163,70,165,32,46,62,30,113,177,113,208,104,52,240,120,61,104,106,106,130,205,102,71,75,75,11,252,
  1,69,151,59,199,33,198,104,68,98,66,60,198,140,26,13,191,207,135,178,210,50,100,102,102,194,106,181,2,224,112,230,244,105,117,141,201,169,171,208,
  240,185,72,156,176,189,93,221,74,19,152,8,130,32,8,130,32,6,20,35,71,142,196,29,119,222,5,158,241,120,245,149,87,80,39,142,233,252,239,150,
  45,200,159,61,27,151,204,191,4,69,135,138,80,87,119,22,45,78,39,124,62,31,180,26,14,231,234,109,56,119,174,30,86,171,21,90,173,22,45,45,
  45,104,108,106,130,207,231,147,227,143,26,13,7,131,222,0,179,217,12,171,197,130,164,145,35,96,175,183,161,161,161,1,0,112,90,20,160,86,107,50,
  56,14,56,125,234,116,135,114,145,11,18,156,194,178,82,208,64,238,190,15,82,176,194,180,251,182,109,157,213,162,28,137,81,130,32,8,130,32,136,94,
  197,221,218,138,186,186,58,108,92,191,94,22,162,128,144,190,115,199,215,219,144,50,106,20,46,158,50,21,121,211,103,160,246,108,29,236,118,59,236,118,
  7,156,78,39,252,126,31,234,235,235,225,247,251,133,241,160,90,13,98,76,49,48,232,13,48,197,196,192,20,19,131,196,132,4,196,197,198,162,169,177,
  17,167,203,78,34,16,8,4,105,194,51,167,79,131,227,56,52,55,53,131,139,40,6,185,182,127,164,109,28,23,38,24,101,205,201,181,125,208,126,68,
  52,122,117,74,98,148,32,8,130,32,8,162,23,104,117,187,241,183,151,95,82,76,180,17,199,77,138,255,156,59,123,22,231,206,158,69,226,136,145,152,
  48,113,2,198,76,153,2,151,203,141,198,166,38,56,157,78,184,221,110,4,252,126,4,120,30,156,134,131,94,167,135,94,167,67,140,209,8,191,207,135,
  11,231,207,163,174,166,6,129,128,63,226,57,156,62,117,170,29,157,168,140,119,138,255,113,210,59,229,255,219,17,150,92,20,155,56,174,221,61,186,33,
  70,105,10,19,65,16,4,65,16,68,151,225,132,89,236,23,206,159,199,254,189,251,0,48,36,36,38,34,46,46,30,70,163,1,92,76,12,248,64,0,
  1,158,7,227,121,48,198,112,222,97,23,35,167,126,117,145,219,158,68,139,52,96,148,227,212,163,162,138,104,105,167,199,133,114,136,250,27,20,25,37,
  8,130,32,8,130,232,37,161,25,34,23,59,164,241,66,35,26,47,92,0,99,210,114,74,226,50,73,76,177,180,38,147,150,87,98,225,130,180,67,5,
  200,133,157,167,28,5,229,130,227,161,65,81,209,208,46,122,69,4,181,45,242,217,177,244,84,219,67,67,222,66,16,4,65,16,4,209,11,122,180,61,
  5,166,232,252,230,20,59,42,123,180,101,65,168,20,123,156,240,18,130,153,156,74,180,51,36,164,25,182,93,185,73,252,5,49,50,202,201,239,131,119,
  87,138,208,14,149,101,52,162,148,35,49,74,16,4,65,16,4,209,191,18,53,92,137,6,11,213,144,96,35,23,26,193,12,18,164,92,68,193,27,73,
  68,74,49,79,78,62,22,39,175,127,47,31,63,76,48,115,65,167,28,177,235,94,249,121,7,227,69,1,234,166,39,8,130,32,8,130,232,51,25,202,
  66,69,167,184,52,126,91,175,62,7,142,19,178,54,201,139,195,11,41,147,4,105,200,152,164,38,229,69,232,57,38,44,48,47,237,215,222,25,4,137,
  75,174,45,242,42,105,77,142,11,89,83,148,139,32,152,195,102,220,119,173,139,158,196,40,65,16,4,65,16,68,175,169,79,78,20,138,146,18,109,203,
  65,31,38,236,88,72,6,35,78,78,191,4,97,104,40,7,112,109,130,148,137,209,83,233,216,28,19,115,26,49,174,125,49,216,166,58,67,198,125,182,
  9,209,96,189,25,28,162,13,250,142,164,136,1,241,220,186,54,185,157,196,40,65,16,4,65,16,68,15,48,110,252,120,92,255,157,255,193,239,158,125,
  6,110,183,91,222,254,227,159,252,20,23,46,92,192,95,95,124,81,16,140,28,240,141,111,92,137,73,147,39,227,195,15,62,64,77,117,21,24,56,228,
  23,204,194,196,172,108,152,205,102,52,53,54,162,180,180,20,135,138,14,10,137,53,25,112,237,154,181,248,224,253,247,228,108,241,96,28,150,46,95,134,
  79,63,217,4,6,96,201,146,203,81,92,124,12,117,181,109,107,154,46,88,180,8,95,109,217,130,188,153,51,49,50,41,9,26,142,131,70,171,133,70,
  163,129,86,163,193,158,221,187,133,133,242,67,186,225,185,224,190,118,165,30,85,93,127,84,93,248,170,45,84,74,98,148,32,8,130,32,8,162,87,200,
  203,155,137,19,199,143,99,210,228,92,20,29,60,32,204,124,231,228,105,232,72,28,145,136,198,11,23,0,0,233,25,25,104,110,110,150,191,91,48,123,
  54,50,199,141,195,198,13,235,209,220,212,140,184,184,56,44,90,188,24,0,100,65,170,213,233,100,65,39,105,58,189,94,47,79,100,210,27,12,152,156,
  59,5,118,187,3,126,191,15,0,96,48,24,192,105,56,20,21,29,4,192,33,41,41,9,89,217,217,216,179,107,87,91,87,125,104,132,84,250,129,144,
  46,123,73,96,170,141,21,85,23,159,237,234,85,81,248,114,52,129,137,32,8,130,32,8,162,187,196,196,196,96,212,232,209,248,100,211,70,204,157,55,
  79,216,200,32,119,99,159,44,43,195,196,172,44,0,28,82,82,82,224,241,120,100,49,106,48,198,96,246,156,57,88,255,241,199,104,105,110,6,56,160,
  165,165,25,159,126,250,9,242,242,242,96,48,24,193,113,28,180,90,109,208,204,123,112,128,78,167,147,187,220,245,122,61,106,170,171,144,59,101,138,188,
  136,189,36,86,165,201,72,26,141,6,58,173,86,33,146,57,132,117,213,75,111,196,174,255,54,157,201,137,102,113,42,42,147,235,64,152,70,78,25,74,
  98,148,32,8,130,32,8,162,155,204,200,203,67,209,193,3,104,109,109,197,133,243,231,49,122,244,152,54,25,198,113,168,170,170,68,70,70,38,192,1,
  233,233,25,168,174,174,150,63,179,90,173,168,171,171,133,199,235,105,139,60,114,28,124,94,47,234,235,235,145,100,177,0,16,197,40,56,197,228,121,14,
  58,157,78,158,17,175,215,233,80,86,118,18,163,70,141,66,178,53,89,216,166,215,183,9,79,13,7,78,163,105,251,14,56,197,228,121,78,49,102,52,
  100,124,40,215,190,152,140,180,242,83,184,76,37,49,74,16,4,65,16,4,209,43,204,157,87,136,17,35,70,98,225,162,197,226,251,121,65,98,172,178,
  178,18,89,217,217,0,128,172,236,108,212,84,87,41,214,15,109,91,83,84,249,14,28,7,78,211,182,204,147,86,140,104,114,242,75,136,140,74,178,82,
  167,215,131,227,56,28,42,42,66,126,193,108,112,162,24,149,247,7,7,13,199,65,43,125,71,41,66,67,20,101,100,33,26,60,211,94,117,121,167,78,
  68,69,73,140,18,4,65,16,4,65,116,147,113,227,199,163,181,213,141,214,214,86,0,192,217,179,103,49,110,252,120,196,152,76,178,56,243,180,122,80,
  85,89,137,169,211,166,33,61,35,3,39,203,78,202,130,210,110,179,193,154,146,2,163,209,216,166,94,193,193,104,52,34,45,45,29,13,142,6,112,156,
  32,70,13,6,131,44,238,140,70,163,216,77,47,232,63,157,94,15,14,128,195,110,199,249,134,6,76,153,58,85,236,166,111,27,2,170,209,72,221,253,
  161,34,84,49,121,73,90,16,191,3,33,26,141,208,140,38,29,40,137,81,130,32,8,130,32,136,110,48,119,238,60,108,221,178,5,91,183,124,41,191,
  78,28,63,142,188,153,121,65,251,149,149,149,33,63,191,0,167,78,150,41,150,82,2,60,94,15,14,21,21,225,218,213,171,145,144,144,8,0,136,79,
  72,192,213,223,252,38,14,30,216,15,159,207,11,128,67,117,85,149,28,113,53,24,140,40,152,61,7,181,117,181,178,176,148,186,223,57,142,195,161,67,
  69,152,60,121,178,16,5,85,68,83,53,26,141,24,97,141,32,66,149,235,142,118,32,68,67,187,231,213,39,217,183,211,189,47,218,79,98,148,32,8,
  130,32,8,162,139,140,24,49,2,227,198,143,199,137,227,199,131,182,23,29,60,136,185,243,10,229,149,55,25,132,174,250,148,81,163,80,85,89,133,54,
  157,38,8,178,93,187,118,226,244,169,83,184,102,245,181,184,239,127,239,199,183,174,185,6,229,103,202,177,111,239,62,89,44,110,251,239,87,48,26,141,
  184,253,206,59,113,195,141,55,194,104,52,98,231,246,29,178,168,212,42,38,38,249,124,62,20,21,29,148,39,56,201,147,149,56,13,52,90,173,138,8,
  133,34,26,170,144,150,114,186,123,113,54,86,59,217,149,88,167,82,134,42,118,153,57,125,122,212,43,148,206,202,207,199,75,127,251,27,110,185,241,70,
  236,217,189,155,60,144,32,8,130,32,8,162,29,24,131,176,196,83,136,108,11,221,137,5,125,20,250,62,248,13,11,254,31,88,23,214,154,15,157,149,
  207,169,169,199,208,180,159,29,164,249,156,61,103,14,222,252,199,91,184,243,246,219,177,127,239,94,68,19,21,5,104,157,81,130,32,8,130,32,136,94,
  67,78,164,20,34,220,130,182,132,101,106,226,228,37,149,88,208,174,114,130,80,185,143,92,90,202,20,157,202,126,196,33,92,79,114,97,187,112,42,111,
  162,205,55,223,209,88,82,37,36,70,9,130,32,8,130,32,122,85,145,170,105,69,117,65,42,11,80,49,237,39,39,191,15,23,165,0,11,235,86,103,
  29,74,208,14,182,134,165,156,231,84,223,183,119,212,142,150,122,34,49,74,16,4,65,16,4,209,151,90,148,227,0,198,171,11,69,22,46,236,56,169,
  155,158,11,94,120,62,178,40,85,30,161,243,114,84,85,132,134,70,67,67,198,144,182,109,98,145,140,105,191,60,72,140,18,4,65,16,4,65,244,169,
  34,109,235,138,15,21,129,42,27,219,4,169,248,63,53,81,10,68,136,184,70,123,78,106,127,114,42,219,163,155,152,196,117,238,39,73,140,18,4,65,
  16,4,65,244,157,22,21,199,119,162,51,130,84,80,155,145,68,105,152,48,141,40,80,163,17,143,81,138,208,118,143,197,117,44,68,85,242,215,147,24,
  37,8,130,32,8,130,232,11,65,170,225,192,248,206,8,210,118,68,169,40,58,85,187,229,163,14,142,114,17,134,140,70,47,66,131,190,211,209,239,114,
  234,59,116,77,140,50,114,40,130,32,8,130,32,136,206,43,82,200,179,229,35,246,176,119,32,74,229,255,135,206,20,98,81,252,118,187,155,59,151,74,
  41,72,184,114,93,250,233,110,136,81,18,164,4,65,16,4,65,16,93,208,162,28,24,199,68,65,42,8,170,206,138,210,182,255,179,96,89,198,69,123,
  14,209,202,68,116,63,26,138,200,221,243,221,23,163,4,65,16,4,65,16,68,215,5,41,16,57,74,218,161,40,13,86,138,92,176,44,141,94,85,162,
  243,187,247,164,16,37,49,74,16,4,65,16,4,209,191,202,52,120,145,251,46,137,210,110,136,206,174,136,208,40,127,38,218,51,33,49,74,16,4,65,
  16,4,209,231,26,84,17,15,13,153,41,31,69,32,180,103,135,75,70,221,83,207,117,126,40,0,215,241,23,72,140,18,4,65,16,4,65,244,183,32,
  149,20,92,123,99,73,59,18,144,44,170,31,237,130,70,229,186,22,112,229,162,251,18,137,81,130,32,8,130,32,136,129,36,72,17,44,74,163,213,153,
  221,237,161,239,49,17,218,9,33,74,98,148,32,8,130,32,8,98,160,9,210,16,81,42,188,237,132,40,237,174,0,237,130,176,229,208,185,174,121,18,
  163,4,65,16,4,65,16,3,93,144,6,137,66,105,76,169,4,83,249,171,43,226,51,228,29,215,221,99,117,30,18,163,4,65,16,4,65,16,3,68,
  144,10,226,146,69,161,250,56,4,205,127,138,74,150,118,49,161,124,212,130,182,107,7,35,49,74,16,4,65,16,4,49,192,68,41,139,38,222,201,245,
  160,178,236,182,16,237,58,93,18,163,76,252,143,32,8,130,32,8,130,232,69,216,0,62,45,46,248,36,187,170,13,53,84,203,4,65,16,4,65,16,
  3,20,110,128,158,83,15,158,23,117,211,19,4,65,16,4,65,12,6,65,202,6,200,121,244,48,36,70,9,130,32,8,130,32,6,147,40,237,107,97,
  218,203,209,89,18,163,4,65,16,4,65,16,131,85,152,246,254,194,163,189,14,137,81,130,32,8,130,32,136,193,46,74,149,176,30,58,14,137,81,130,
  32,8,130,32,8,98,48,9,203,174,64,179,233,9,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,137,81,130,32,8,130,32,8,130,32,49,
  74,16,4,65,16,4,65,144,24,37,8,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,137,81,130,32,8,130,32,8,130,232,31,49,90,87,
  91,11,0,152,52,121,50,149,28,65,16,4,65,16,4,33,35,233,67,73,47,246,138,24,173,173,173,69,115,115,51,137,81,130,32,8,130,32,8,34,
  76,140,54,55,55,163,182,55,197,40,0,236,223,183,15,151,95,126,57,149,56,65,16,4,65,16,4,33,115,249,229,151,99,255,190,125,157,254,94,167,
  197,232,214,45,91,16,159,144,128,111,93,115,13,149,58,65,16,4,65,16,4,129,111,93,115,13,226,19,18,176,117,203,150,78,127,151,155,57,125,58,
  235,236,151,214,111,218,4,198,24,190,181,106,21,154,155,154,168,6,8,130,32,8,130,32,134,41,241,9,9,248,215,186,117,224,56,14,43,151,47,239,
  244,247,187,52,155,254,165,191,252,5,169,169,169,184,247,190,251,168,6,8,130,32,8,130,32,134,49,247,222,119,31,82,83,83,241,210,95,254,210,165,
  239,119,73,140,174,95,183,14,91,183,108,193,77,183,220,66,221,245,4,65,16,4,65,16,195,148,111,93,115,13,110,186,229,22,108,221,178,5,235,215,
  173,235,210,49,180,99,70,143,126,172,43,95,220,185,99,7,10,231,207,199,218,111,127,27,181,53,53,56,113,252,56,213,8,65,16,4,65,16,196,48,
  18,162,191,254,237,111,81,90,82,130,7,126,248,67,120,189,222,190,21,163,94,175,23,255,249,236,51,89,144,54,55,55,227,80,81,17,213,12,65,16,
  4,65,16,196,16,231,166,91,110,193,99,79,60,129,210,146,18,220,117,199,29,104,110,110,238,242,177,186,52,129,73,73,124,124,60,94,250,219,223,144,
  157,147,131,61,187,119,227,103,63,254,49,106,106,106,168,150,8,130,32,8,130,32,134,24,169,169,169,248,245,111,127,139,217,115,230,244,136,16,237,17,
  49,42,241,221,187,239,198,93,119,223,13,0,248,215,71,31,225,205,215,95,167,174,123,130,32,8,130,32,136,33,192,164,201,147,131,230,10,189,244,151,
  191,224,175,93,156,176,20,38,70,243,46,190,152,245,212,137,142,29,59,22,223,253,222,247,176,114,213,42,0,192,137,227,199,241,197,231,159,99,207,238,
  221,168,169,169,65,77,117,53,213,38,65,16,4,65,16,196,0,39,53,45,13,169,169,169,152,61,103,14,46,191,226,10,57,251,230,250,117,235,240,215,
  23,95,236,116,150,165,62,19,163,74,81,186,112,209,34,172,188,250,106,228,228,228,80,141,18,4,65,16,4,65,12,82,74,74,74,176,254,227,143,177,
  117,203,150,30,21,161,189,42,70,149,196,199,199,35,39,39,7,179,10,10,168,54,9,130,32,8,130,32,6,9,251,247,238,69,73,73,73,183,199,132,
  246,187,24,37,8,130,32,8,130,32,136,72,104,168,8,8,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,137,81,130,32,8,130,32,8,130,
  32,49,74,16,4,65,16,4,65,144,24,37,8,130,32,8,130,32,8,18,163,4,65,16,4,65,16,4,137,81,130,32,8,130,32,8,130,32,49,74,
  16,4,65,16,4,65,144,24,37,8,130,32,8,130,32,136,174,162,251,238,247,190,71,165,64,16,4,65,16,4,65,244,11,28,99,140,210,129,18,4,
  65,16,4,65,16,253,35,70,47,45,44,28,246,98,148,227,56,58,247,158,63,177,254,251,233,33,88,254,189,254,59,189,112,124,110,16,149,67,175,28,
  183,135,142,217,147,231,198,13,17,155,186,253,253,126,46,135,65,105,127,63,156,115,95,157,103,87,203,115,32,223,255,251,82,27,68,251,91,237,237,71,
  99,70,9,130,32,8,130,32,136,126,131,196,40,65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,
  9,130,32,8,130,32,136,161,143,174,63,126,212,154,146,130,220,41,83,144,98,77,65,230,184,113,136,141,141,13,219,167,248,216,49,56,157,78,20,31,
  59,138,242,242,242,118,143,23,27,27,139,220,41,83,48,110,220,120,88,83,172,176,90,83,58,125,60,171,53,5,11,22,45,12,219,238,114,58,177,105,
  227,198,160,109,171,215,174,85,61,143,226,99,199,80,124,236,24,114,167,76,65,238,148,41,93,46,159,175,182,108,133,205,86,31,245,113,56,113,206,161,
  205,86,15,155,205,134,242,242,114,184,156,206,118,191,147,57,110,28,114,115,167,32,54,54,22,147,167,228,134,125,110,171,183,193,110,179,161,188,252,12,
  138,139,139,85,143,183,96,225,66,213,178,22,206,197,134,175,190,218,74,87,24,65,16,4,65,16,3,71,140,142,27,55,14,55,223,122,91,84,2,75,
  185,143,205,86,143,77,27,54,98,211,198,13,97,34,244,230,91,111,197,130,133,139,58,125,188,55,94,123,13,123,247,236,81,8,100,43,86,175,81,23,
  153,86,107,10,222,120,253,181,54,49,26,97,191,15,240,158,44,70,35,237,19,13,197,199,142,201,98,180,171,199,41,46,62,134,55,95,127,29,21,33,
  194,59,191,160,0,55,221,114,43,172,86,107,7,5,22,34,144,183,110,197,155,111,188,30,36,74,47,91,184,16,185,185,83,34,252,126,49,137,81,130,
  32,8,130,32,58,164,207,186,233,173,41,41,248,237,51,207,70,20,162,111,188,246,26,30,255,229,163,176,217,234,85,197,224,205,183,222,138,5,139,130,
  69,231,163,143,63,17,81,136,126,240,222,123,237,30,239,193,135,127,140,130,217,179,163,58,247,101,87,93,21,149,224,149,197,110,189,77,142,146,22,31,
  59,22,38,8,67,169,40,47,15,218,191,163,168,166,108,227,251,239,225,171,173,234,130,47,55,119,10,30,253,229,99,48,43,162,206,249,5,5,120,224,
  161,135,85,133,168,205,102,195,147,143,63,134,15,223,127,95,245,120,11,22,46,196,163,191,124,140,174,24,130,32,8,130,32,122,148,62,139,140,46,236,
  64,204,73,81,79,91,189,45,98,215,111,65,193,108,124,181,101,11,0,33,202,58,110,220,184,136,199,59,118,236,168,16,97,108,231,120,11,22,46,10,
  138,142,182,199,205,183,220,130,138,242,51,29,14,25,0,128,175,182,110,193,87,91,183,180,9,195,41,83,240,232,99,143,71,220,255,141,215,95,67,241,
  177,99,157,46,211,15,222,123,15,28,199,193,154,98,85,141,80,154,99,99,49,110,220,56,249,216,237,9,106,155,173,94,222,239,90,172,81,221,39,115,
  220,56,100,142,27,215,161,184,38,8,130,32,8,130,136,150,65,53,129,73,57,182,212,172,50,206,180,59,199,235,8,115,108,44,30,120,248,199,157,250,
  78,159,149,139,57,242,57,57,21,81,86,115,172,185,79,203,140,32,8,130,32,8,162,35,116,3,237,132,182,110,221,18,49,74,88,175,210,229,222,151,
  88,173,86,60,240,208,195,3,166,172,30,125,236,113,196,198,198,34,83,37,66,236,114,58,241,213,87,91,41,138,73,16,4,65,16,4,137,209,206,32,
  117,195,15,84,186,51,75,190,175,206,101,223,222,189,40,62,118,140,38,16,17,4,65,16,4,65,98,180,179,44,88,180,8,41,17,198,120,214,219,234,
  251,69,172,126,178,113,35,150,93,117,213,128,171,188,111,175,89,141,216,216,88,252,223,211,207,4,77,74,202,47,40,64,126,65,1,86,175,89,131,251,
  238,253,126,212,19,162,8,130,32,8,130,32,134,189,24,93,184,112,81,196,136,95,241,177,99,253,34,70,247,238,221,3,115,108,44,22,44,92,56,224,
  42,208,229,114,193,102,171,87,157,33,31,58,129,137,32,8,130,32,8,98,160,209,103,19,152,122,98,188,167,114,153,38,155,205,214,163,199,235,136,55,
  95,127,173,71,126,179,63,177,213,247,64,153,213,215,211,85,67,16,4,65,16,68,143,209,103,145,209,125,123,246,192,182,182,62,226,50,75,15,62,252,
  99,121,161,119,53,156,78,39,54,109,216,16,36,138,190,218,186,37,226,114,69,203,175,90,129,41,83,166,70,125,188,142,112,58,157,120,241,133,63,181,
  187,68,211,64,231,147,77,27,81,80,80,160,186,18,193,184,204,113,88,189,102,173,106,54,38,137,175,182,110,141,90,144,231,230,230,226,159,239,190,23,
  180,237,201,199,31,71,113,49,69,105,9,130,32,8,130,232,7,49,234,116,58,241,227,7,31,196,234,181,107,81,48,123,118,152,40,141,180,0,189,205,
  86,143,189,123,246,96,211,198,141,97,81,185,63,255,233,79,56,118,236,24,150,95,181,34,108,205,209,130,217,179,85,143,233,116,58,177,111,239,30,188,
  255,222,123,157,142,242,21,31,59,134,15,222,127,175,91,217,149,250,147,138,242,114,252,228,225,135,112,237,154,181,97,162,212,28,27,139,107,215,172,137,
  248,189,79,54,109,140,184,192,62,65,16,4,65,16,68,87,225,46,45,44,100,253,241,195,214,148,20,88,173,86,140,27,55,62,108,237,74,167,211,137,
  242,242,51,176,217,108,81,11,70,105,137,35,107,74,138,234,4,168,99,199,142,194,229,116,170,46,90,207,113,92,196,37,146,36,49,166,92,175,51,82,
  180,213,86,111,83,237,250,111,239,216,106,199,151,203,200,154,2,107,138,181,93,113,204,113,28,50,199,141,139,184,254,103,123,121,234,165,239,169,45,152,
  31,77,158,251,246,126,23,224,194,207,165,162,188,79,38,83,113,125,121,1,113,220,208,248,157,94,56,62,55,136,202,161,87,142,219,67,199,236,201,115,
  227,134,136,77,221,254,126,63,151,195,160,180,191,31,206,185,175,206,179,171,229,57,144,239,255,125,117,110,157,249,173,246,246,235,55,49,58,160,20,57,
  199,209,185,15,2,113,67,98,148,196,40,137,81,18,163,36,70,73,140,146,24,29,122,98,116,80,101,96,34,8,130,32,8,130,32,134,22,36,70,9,
  130,32,8,130,32,8,18,163,4,65,16,4,65,16,196,240,67,119,235,237,183,15,251,66,224,6,189,1,28,121,114,127,213,105,95,142,203,33,63,234,
  189,114,24,224,182,115,67,240,26,239,111,155,184,65,236,47,131,209,31,56,186,198,134,236,249,245,196,185,113,140,177,97,63,129,137,32,8,130,32,8,
  130,232,39,65,155,119,241,197,36,70,9,130,32,8,130,32,136,126,129,198,140,18,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,18,163,4,
  65,16,4,65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,130,32,8,18,163,4,65,16,4,
  65,16,4,65,98,148,32,8,130,32,8,130,32,49,74,16,4,65,16,4,65,16,36,70,9,130,32,8,130,32,136,193,135,174,183,127,32,63,63,31,
  179,10,10,144,159,159,15,0,136,139,143,71,78,78,14,149,60,65,16,4,65,16,196,0,163,164,164,4,45,205,205,0,128,125,251,246,97,255,222,189,
  216,183,111,95,175,254,102,175,228,166,31,59,118,44,190,251,189,239,97,225,162,69,136,143,143,7,0,212,214,214,162,174,182,86,54,142,32,8,130,32,
  8,130,24,88,72,193,195,49,99,199,98,236,216,177,0,128,230,230,102,108,221,178,5,127,125,241,69,212,138,90,110,192,138,209,248,248,120,60,248,240,
  195,88,185,106,21,0,96,235,150,45,216,242,229,151,216,186,101,11,154,69,149,77,16,4,65,16,4,65,12,124,226,227,227,177,112,209,34,44,90,188,
  24,11,23,45,2,0,172,95,183,14,207,60,245,84,143,234,186,30,19,163,11,23,45,194,227,79,62,137,248,248,120,172,95,183,174,215,212,51,65,16,
  4,65,16,4,209,183,72,189,222,43,87,173,66,115,115,51,126,249,139,95,96,235,150,45,61,114,108,237,152,81,163,30,235,238,65,86,174,90,133,255,
  123,234,41,56,28,14,60,240,131,31,224,237,127,252,131,34,161,4,65,16,4,65,16,67,4,169,171,126,255,190,125,40,156,63,31,171,215,172,65,109,
  109,45,74,75,74,250,95,140,174,92,181,10,143,63,249,36,74,74,74,112,203,141,55,162,188,188,156,106,140,32,8,130,32,8,98,8,82,91,91,139,
  245,235,214,245,168,32,237,86,55,189,82,136,222,117,251,237,189,26,13,101,140,201,255,50,198,0,198,192,24,192,32,253,45,188,7,152,188,143,244,158,
  227,184,136,47,40,255,150,10,69,241,119,111,19,100,143,100,103,4,123,32,218,43,149,133,108,135,70,19,110,27,56,112,28,0,209,22,14,109,127,247,
  139,109,42,117,21,169,238,218,108,211,64,163,17,234,72,19,82,87,74,123,184,62,182,139,32,8,130,32,134,59,241,241,241,120,233,149,87,144,147,147,
  131,95,254,226,23,88,191,110,93,223,139,209,156,156,28,188,244,202,43,168,173,173,237,117,33,170,20,54,193,2,167,227,87,226,136,145,208,234,116,112,
  181,180,192,239,243,129,211,8,226,77,163,38,74,69,129,211,103,226,38,68,152,201,231,205,183,111,167,222,96,64,226,200,145,240,121,189,104,110,106,148,
  69,155,36,224,56,77,184,32,237,115,49,26,106,27,207,218,254,109,167,30,37,219,90,221,110,184,156,45,208,200,246,40,236,234,111,219,250,172,252,148,
  130,189,235,98,94,173,225,37,55,81,56,244,143,111,40,254,109,175,209,130,144,107,35,168,17,198,137,141,48,77,155,77,109,215,114,63,217,70,16,4,
  49,12,5,233,216,177,99,113,215,237,183,163,164,139,17,210,46,139,209,151,95,121,5,179,242,243,113,253,154,53,93,254,241,206,139,81,69,100,176,3,
  17,170,209,104,48,33,103,50,76,102,179,124,140,234,138,114,52,158,111,80,23,163,146,184,65,127,68,70,163,23,163,9,35,70,96,220,196,44,249,251,
  173,110,23,206,148,149,129,49,94,93,140,182,105,142,126,138,140,70,47,70,71,90,44,72,31,127,81,4,219,84,196,168,100,219,16,21,27,114,249,117,
  224,235,8,17,243,6,163,1,60,207,195,211,218,218,177,152,71,255,148,159,170,223,119,116,93,243,12,113,241,241,224,52,28,92,45,78,217,46,165,224,
  150,4,170,228,238,67,77,140,118,197,39,36,17,175,145,27,226,3,211,39,148,246,1,172,243,54,114,156,108,155,70,238,45,26,120,54,118,199,255,1,
  22,86,135,3,221,255,195,158,3,81,212,229,96,242,89,66,8,78,254,243,253,247,177,127,223,62,220,121,251,237,93,58,70,151,198,140,174,92,181,10,
  223,185,225,6,252,245,197,23,241,217,167,159,246,131,119,67,140,172,40,94,33,88,172,41,72,74,78,198,241,178,10,156,174,172,69,66,124,44,146,44,
  73,176,215,159,19,34,40,242,213,218,22,73,233,107,167,110,59,125,225,98,148,95,145,140,2,48,62,59,27,94,95,0,7,142,148,162,185,197,133,49,
  163,172,8,248,253,112,59,93,178,13,146,125,156,168,214,250,69,100,43,108,99,146,77,29,216,54,97,210,100,180,184,90,81,116,172,12,173,173,30,164,
  88,45,240,122,60,240,184,221,193,118,133,218,38,71,250,134,144,16,85,150,95,144,163,71,46,191,164,228,100,76,156,52,25,73,201,86,88,172,41,48,
  24,140,104,110,108,12,247,11,169,252,148,189,1,253,96,27,66,253,94,126,133,127,199,96,48,34,103,202,84,88,71,143,65,82,178,21,35,147,147,209,
  212,216,8,158,231,131,34,162,146,129,67,241,33,213,21,159,208,106,181,72,25,51,6,163,211,210,16,151,144,0,175,215,139,128,223,63,224,124,34,178,
  173,81,60,196,180,58,164,141,27,15,235,232,209,136,75,72,128,179,165,5,109,195,179,148,54,74,61,4,104,243,153,126,180,173,51,254,15,0,137,35,
  70,34,125,220,120,36,37,39,131,15,4,224,245,120,194,122,4,6,154,255,171,62,7,58,227,179,241,131,199,103,135,51,14,135,67,208,134,87,95,221,
  229,241,163,93,74,7,250,157,27,110,64,115,115,51,254,241,214,91,170,183,196,222,123,137,173,166,40,132,141,86,171,5,0,148,87,213,161,225,124,19,
  234,109,13,242,54,69,147,77,126,69,186,189,247,230,75,253,6,213,254,93,216,96,48,162,166,206,134,134,243,77,40,175,170,19,42,81,182,139,201,66,
  93,121,195,99,42,55,248,62,179,141,181,21,117,71,182,105,181,90,212,219,26,208,112,190,9,101,103,170,69,123,13,193,199,25,0,182,245,121,249,133,
  186,123,132,242,27,155,145,137,166,22,23,118,31,44,70,77,157,13,35,45,22,196,143,72,108,167,252,208,175,190,209,25,193,49,42,53,21,6,163,17,
  135,143,159,194,225,227,167,96,48,24,48,106,204,24,149,107,25,109,93,251,67,204,55,186,226,19,227,178,178,49,58,53,13,60,167,195,72,75,50,46,
  202,206,129,190,221,107,10,3,160,220,58,39,210,38,136,13,48,111,0,162,141,217,97,247,154,96,17,63,48,234,177,51,254,159,56,98,36,198,103,101,
  67,107,48,66,171,55,34,115,194,68,36,36,142,24,240,254,175,250,28,232,140,207,38,15,22,159,165,215,63,222,122,11,205,205,205,248,206,13,55,116,
  73,208,118,90,140,230,228,228,32,39,39,7,111,139,63,60,96,34,5,33,55,45,121,50,144,114,95,185,11,40,186,27,64,255,182,154,33,139,228,32,
  249,197,88,240,13,77,69,76,99,144,216,134,40,108,99,131,196,182,62,143,22,133,69,155,1,141,70,131,115,162,152,63,94,86,1,198,24,98,98,76,
  131,198,55,208,206,245,108,208,27,224,56,223,136,154,58,27,106,234,108,112,156,111,132,78,111,104,27,119,74,62,161,234,19,177,113,113,40,175,170,195,
  246,61,135,177,251,64,49,52,26,13,226,19,19,7,213,53,21,124,159,15,54,58,54,62,1,49,38,19,142,151,149,99,247,129,98,28,47,43,135,49,
  198,4,115,108,92,144,240,28,12,54,182,231,255,201,41,163,224,114,183,98,251,158,35,216,190,231,8,92,238,86,36,89,83,6,165,255,171,62,7,134,
  152,207,14,71,154,155,155,241,246,91,111,201,26,177,215,197,232,202,171,175,6,0,172,251,248,227,65,123,251,30,76,55,224,78,53,159,7,219,205,104,
  8,218,38,113,199,93,119,97,201,229,151,247,238,195,43,52,44,160,192,231,247,15,74,191,232,139,239,12,101,97,170,230,19,62,127,0,0,208,112,161,
  9,0,194,123,136,6,175,252,150,105,106,118,5,253,59,24,253,163,163,115,117,183,122,224,243,251,225,243,251,225,110,245,12,122,255,103,195,198,103,135,
  15,146,46,148,116,98,103,208,117,246,11,179,242,243,81,82,82,210,191,217,149,228,200,103,240,189,169,173,49,201,20,253,194,106,87,0,3,147,103,245,
  8,112,210,103,125,58,120,46,66,119,84,216,232,48,241,239,8,54,73,101,33,140,155,145,186,179,56,225,59,253,52,142,166,109,16,58,83,105,253,6,
  219,214,54,236,162,29,219,130,236,18,63,147,170,80,156,188,48,144,24,63,254,34,180,180,180,244,76,249,69,30,180,18,85,249,9,126,173,82,126,226,
  184,186,62,47,191,118,204,98,33,238,34,95,207,237,28,136,49,78,113,250,76,121,69,247,249,37,221,103,215,212,80,243,137,72,66,165,93,247,105,207,
  70,133,194,97,92,248,17,56,174,255,156,99,24,249,191,234,115,96,136,250,44,33,172,63,90,82,82,130,89,98,110,251,78,137,81,97,134,94,244,228,
  228,228,224,31,127,127,11,157,253,94,79,57,53,11,17,110,145,102,230,69,106,119,74,254,12,48,128,135,60,18,154,113,178,126,235,211,171,55,10,61,
  26,218,139,17,126,155,150,4,182,116,131,85,220,134,56,198,218,238,197,125,125,157,70,105,27,162,177,77,54,162,237,115,78,122,206,72,55,162,1,38,
  28,192,208,189,235,164,135,202,79,56,23,149,242,3,39,207,66,238,235,242,83,189,102,35,92,215,96,237,244,102,72,51,175,121,0,26,197,90,193,162,
  223,115,80,209,34,131,90,145,14,93,159,104,187,110,148,99,249,219,183,181,61,157,38,125,198,133,117,215,115,109,173,119,214,127,247,135,97,227,255,67,
  220,103,137,96,246,237,217,139,255,185,241,134,78,63,251,58,21,25,205,153,148,35,170,223,154,94,53,102,254,37,151,194,98,77,14,114,196,64,32,128,
  64,192,143,128,63,0,127,192,47,188,247,11,219,252,226,191,129,64,0,126,191,240,47,207,160,58,98,168,176,112,62,116,122,61,116,122,29,116,58,225,
  165,213,106,161,209,106,161,213,104,229,165,36,184,144,200,169,146,29,219,182,193,110,183,119,202,166,228,228,100,20,94,122,105,240,245,197,51,193,22,241,
  156,133,243,111,179,179,205,166,182,109,60,194,47,214,180,244,116,100,79,156,24,100,151,86,171,133,86,171,131,86,171,129,70,163,21,150,59,145,102,92,
  170,80,114,252,4,74,78,28,239,82,125,169,217,198,243,124,155,13,126,209,54,165,93,97,219,218,177,45,107,34,244,122,61,180,98,125,233,180,58,161,
  190,68,219,52,226,146,31,145,234,203,97,179,99,251,215,219,134,244,13,128,181,243,142,108,27,158,12,175,114,27,186,99,126,134,83,61,210,181,62,248,
  145,244,97,206,164,28,148,156,136,126,86,125,167,196,104,124,124,130,32,92,78,244,238,186,162,133,151,92,130,156,201,147,131,90,145,62,159,15,62,175,
  23,62,175,23,94,175,23,94,159,15,94,197,123,159,188,205,3,159,215,135,234,154,26,212,157,59,23,230,215,243,230,207,71,140,41,6,6,99,12,140,
  70,35,140,70,3,244,122,3,244,122,125,144,48,229,218,89,252,190,228,248,241,78,139,81,75,114,50,86,125,243,91,65,219,2,129,128,202,249,123,219,
  236,244,137,182,122,69,219,125,94,236,47,58,20,118,141,166,165,167,97,82,206,36,196,196,8,54,25,140,70,24,12,130,77,122,157,14,90,189,94,176,
  75,204,214,164,198,58,246,175,46,139,209,72,182,121,61,30,185,222,164,58,11,182,87,241,183,215,135,3,135,212,108,75,199,228,73,57,136,137,49,193,
  32,214,151,193,96,132,78,178,77,170,51,141,166,221,250,234,13,49,58,117,218,52,76,155,54,77,245,179,148,148,148,168,91,233,102,179,25,233,25,25,
  97,145,19,158,231,193,7,120,240,124,64,16,247,60,15,158,231,193,164,191,3,194,118,233,165,118,255,78,72,72,64,140,209,32,52,78,52,26,104,196,
  178,146,94,156,74,6,50,213,50,60,113,162,203,229,148,51,105,82,136,109,144,109,10,122,5,164,191,3,193,219,24,15,131,65,15,175,203,27,116,61,
  27,13,70,164,101,100,4,217,38,52,40,131,215,154,108,111,201,151,238,216,53,72,159,240,67,236,33,63,76,4,75,232,106,2,67,125,38,15,27,166,
  245,60,4,144,244,161,164,23,123,69,140,246,21,79,255,223,111,84,174,197,232,179,47,129,49,140,26,155,138,81,99,199,134,93,208,207,62,253,84,216,
  130,247,202,236,52,64,239,172,203,86,114,226,4,238,184,249,166,118,237,138,198,198,233,249,5,97,54,237,218,177,3,235,63,250,40,66,90,80,244,122,
  150,162,222,181,109,59,214,127,244,97,251,182,245,211,56,161,105,211,166,225,219,223,249,78,183,143,147,158,145,129,7,127,242,211,136,13,21,175,207,11,
  159,87,106,168,120,224,243,249,197,70,138,7,94,175,23,126,175,15,94,159,15,7,138,138,194,202,47,119,234,84,76,158,52,9,49,49,70,81,204,27,
  97,48,24,131,26,95,90,173,86,46,223,72,220,121,203,205,93,182,47,212,54,158,231,225,151,26,38,98,3,44,184,17,166,220,38,52,104,78,148,150,
  161,197,85,27,244,112,178,90,173,88,177,98,69,80,35,204,104,52,202,141,48,157,94,47,139,239,72,182,117,199,174,222,36,61,35,3,215,125,231,127,
  130,203,45,16,128,223,239,131,207,231,135,207,239,67,192,231,135,207,231,131,223,239,131,223,239,23,182,75,239,197,125,234,206,213,135,251,196,148,169,24,
  115,249,229,48,26,12,208,233,13,48,24,244,208,233,244,98,79,145,86,232,109,16,27,175,145,238,25,239,190,253,15,84,85,86,118,59,232,80,120,73,
  112,143,74,192,239,135,223,239,151,237,244,251,124,240,137,246,8,54,250,218,182,249,253,112,185,220,112,156,63,31,102,227,194,69,139,49,98,68,162,16,
  104,48,132,216,40,6,28,148,141,149,222,178,81,205,255,25,227,69,123,68,27,21,182,202,54,42,183,5,252,176,217,29,56,107,115,132,248,127,10,102,
  23,20,180,5,84,12,109,65,21,157,86,11,173,78,39,54,204,34,251,127,85,101,37,222,125,251,31,189,231,179,124,64,168,47,177,254,132,191,7,175,
  207,18,189,135,110,168,26,22,105,45,207,161,214,96,28,202,49,142,129,110,219,63,223,126,27,255,124,251,237,110,31,167,170,178,18,207,132,52,192,186,
  18,25,77,73,77,11,43,191,226,163,71,81,180,127,95,183,35,163,221,33,220,182,206,71,70,71,88,146,195,2,67,245,54,27,222,253,231,219,221,138,
  140,14,182,168,159,114,72,93,208,154,153,44,244,174,215,182,196,93,196,239,203,153,142,34,68,226,184,190,24,84,201,212,255,108,51,42,232,154,80,217,
  41,194,154,205,74,187,20,105,102,85,23,124,98,125,91,143,170,149,210,206,210,24,42,203,55,133,174,181,44,47,237,197,148,214,133,12,172,229,162,243,
  177,158,182,149,41,131,19,138,207,25,83,169,135,65,225,179,4,137,81,130,24,64,164,164,164,32,54,46,14,103,78,159,14,251,108,241,146,37,168,175,
  175,199,209,35,71,58,60,142,203,229,82,237,46,238,108,100,57,84,140,2,64,83,83,35,108,103,207,170,71,150,123,177,39,64,73,52,182,117,212,219,
  97,86,233,242,241,120,60,168,174,172,108,179,71,105,163,96,216,160,77,7,42,52,80,254,175,219,62,113,113,104,111,3,128,226,99,71,241,213,230,47,
  250,213,39,0,96,199,215,95,99,199,215,95,135,11,78,165,13,29,248,74,108,92,28,46,202,153,20,118,236,175,182,108,129,203,233,4,199,113,114,79,
  24,250,193,70,161,49,246,127,221,246,255,9,42,54,218,108,54,236,219,185,99,192,248,255,112,240,89,162,247,208,80,17,16,68,215,88,124,249,229,184,
  237,142,59,0,0,223,254,206,119,240,228,175,127,45,127,182,104,201,18,76,141,48,158,148,32,8,130,32,136,94,18,163,41,41,41,3,200,180,65,208,
  233,219,229,101,40,6,65,55,68,95,218,214,79,203,121,4,101,248,82,57,7,214,157,243,26,6,229,215,243,215,248,16,135,124,130,160,231,0,249,236,
  16,69,215,101,7,83,169,248,103,159,127,30,239,190,253,54,54,172,91,55,184,5,41,27,12,137,19,187,112,209,14,248,139,117,16,218,166,188,22,66,
  255,238,243,243,26,78,190,65,15,41,242,137,161,124,159,31,206,254,63,28,124,150,26,205,161,244,104,100,116,207,174,93,184,237,206,59,241,236,243,207,
  99,202,96,236,162,236,15,71,110,47,91,84,119,239,83,253,125,97,14,101,219,196,115,96,106,127,67,153,43,124,0,151,95,127,150,225,80,247,141,193,
  90,110,67,165,236,216,0,23,44,195,201,255,201,103,137,40,232,209,9,76,127,252,253,239,241,229,230,205,184,238,250,235,241,196,175,126,133,45,155,55,
  195,86,223,182,76,195,187,255,252,103,223,58,116,103,26,88,202,12,14,3,121,32,116,103,27,141,202,50,27,232,3,188,187,106,91,63,218,53,101,234,
  84,124,168,232,9,80,254,125,44,138,201,75,253,30,80,24,44,41,244,212,166,19,15,21,191,39,159,160,251,60,249,255,240,184,143,17,125,35,70,165,
  7,240,187,0,158,248,213,175,176,104,201,146,160,207,186,37,70,9,98,128,113,244,232,81,160,29,159,62,122,244,40,21,18,33,19,27,27,11,167,211,
  73,5,65,16,4,209,19,98,52,82,242,135,148,148,20,220,123,255,253,152,50,117,42,142,30,57,130,63,61,255,60,234,235,235,123,182,177,212,137,150,
  115,212,9,43,196,86,149,148,150,190,95,70,249,69,105,23,139,198,46,69,196,176,63,59,48,134,178,109,0,112,244,200,145,136,75,55,141,31,63,30,
  78,167,179,91,231,72,190,161,126,45,163,189,235,120,0,216,182,104,241,98,164,140,26,133,99,71,142,200,13,146,169,83,167,226,222,251,239,199,221,119,
  222,73,215,84,52,54,70,57,230,58,234,245,164,7,162,141,67,212,255,135,163,207,18,157,244,213,158,18,163,145,120,241,229,151,81,95,95,143,223,254,
  250,215,216,179,123,247,32,41,185,224,240,190,36,72,9,162,35,174,187,254,122,76,153,58,21,143,62,242,8,22,45,94,140,113,23,93,132,215,254,246,
  55,0,192,173,119,220,129,99,71,143,82,111,192,48,227,214,59,238,192,138,149,43,133,55,223,254,54,30,125,228,17,204,158,59,23,43,86,174,68,249,
  153,51,84,64,157,121,144,209,88,64,130,24,54,244,168,24,221,176,110,29,222,123,231,157,110,119,69,253,232,161,135,145,157,147,163,184,39,49,248,253,
  254,160,28,238,82,174,118,41,111,123,80,222,115,159,15,213,181,53,56,123,46,60,42,123,255,15,127,132,24,83,12,140,198,24,57,135,187,65,175,135,
  78,124,105,181,90,104,219,201,225,14,0,207,61,253,20,74,75,74,58,101,83,118,78,14,126,244,208,195,65,219,120,158,23,206,219,227,145,83,32,250,
  37,155,124,62,248,188,226,118,143,96,171,207,231,195,129,67,135,195,142,61,103,238,92,228,100,103,195,168,204,77,175,215,67,111,48,136,233,210,116,208,
  104,53,208,112,145,211,194,109,88,183,14,27,214,125,220,165,250,82,179,45,192,243,98,170,202,182,244,142,62,177,110,212,254,246,250,124,56,120,88,197,
  182,121,130,109,49,49,38,24,13,6,232,141,70,49,37,156,88,103,218,182,52,112,145,108,43,45,41,193,115,79,63,213,59,13,25,241,129,153,146,146,
  130,241,227,198,169,126,214,221,135,51,235,137,99,170,68,18,250,187,209,213,99,182,41,26,149,253,109,219,138,21,43,240,218,223,254,134,13,235,215,227,
  199,63,253,41,126,252,211,159,34,54,54,22,239,253,243,159,120,247,157,119,6,86,185,13,64,159,232,45,27,153,226,222,48,228,108,28,64,254,63,156,
  125,150,24,64,98,116,197,202,149,216,179,123,55,142,117,115,172,220,206,29,219,67,196,30,67,32,192,35,16,240,35,224,15,192,31,240,11,121,154,3,
  1,240,226,251,64,32,32,188,252,126,97,95,198,0,165,160,20,61,119,247,206,157,208,233,117,178,72,211,42,242,115,107,52,90,104,52,28,56,78,35,
  6,75,213,221,220,225,112,116,218,38,135,195,17,182,228,21,99,60,252,242,57,75,231,31,16,236,148,254,230,3,240,251,3,224,3,126,248,3,1,64,
  163,13,139,131,87,87,87,163,193,102,147,237,146,108,210,105,69,17,170,209,10,233,17,17,57,111,111,105,201,137,46,215,151,154,109,60,227,195,109,9,
  173,39,158,135,95,97,187,170,109,85,162,109,58,61,116,58,33,223,178,78,43,216,39,217,38,212,25,215,78,125,217,123,61,138,195,58,138,242,180,67,
  90,122,58,214,126,251,250,16,49,31,16,242,87,251,124,240,249,253,8,40,242,88,11,57,158,253,98,206,238,182,28,207,117,54,91,216,143,78,206,205,
  197,152,197,139,96,16,243,86,235,245,134,54,63,209,234,196,20,154,154,14,83,130,118,71,204,135,53,194,24,131,95,202,173,46,231,29,15,206,95,173,
  204,209,237,247,251,97,107,56,143,115,33,185,185,147,173,41,40,200,159,37,218,38,230,229,22,243,210,75,215,64,71,141,176,222,104,164,156,17,35,160,
  103,206,156,193,236,57,115,240,167,63,252,1,91,190,252,178,83,199,48,153,205,72,79,79,15,185,95,4,167,136,101,202,20,177,140,169,166,136,85,115,
  196,132,132,4,196,24,13,208,106,52,66,250,84,173,144,191,188,51,41,98,171,170,170,224,118,185,186,85,78,150,228,100,88,44,150,144,6,58,11,74,
  7,203,66,83,198,134,188,116,122,125,200,213,38,252,155,108,181,130,241,73,208,104,180,208,106,69,187,196,123,69,155,141,154,118,231,188,244,132,141,82,
  99,61,84,79,117,198,70,158,231,97,48,24,224,117,123,130,252,223,96,140,65,106,122,58,180,26,41,29,174,116,63,140,62,29,174,203,229,66,117,85,
  85,143,248,125,199,62,203,131,241,129,65,237,179,196,32,17,163,172,135,186,85,118,110,223,174,122,108,181,52,98,80,73,27,7,198,144,50,102,44,82,
  198,140,145,47,66,78,188,11,236,218,185,3,154,144,212,112,202,84,113,66,163,171,231,219,90,14,187,93,53,242,24,41,77,26,34,164,195,155,54,115,
  22,130,100,23,99,168,169,174,130,93,76,249,24,106,87,95,164,133,235,119,219,250,113,22,101,164,133,239,59,123,45,48,213,13,109,121,180,131,134,208,
  73,121,182,89,112,142,106,48,22,86,126,114,94,110,22,156,179,91,206,11,45,214,67,111,142,149,86,205,176,206,20,231,162,204,53,205,34,100,16,23,
  119,224,66,158,234,74,219,130,139,142,181,253,20,215,63,254,0,8,227,138,59,43,68,1,32,61,61,29,63,12,237,109,8,4,228,222,31,161,23,197,
  7,175,79,236,101,240,250,196,222,33,143,248,175,208,235,80,116,248,112,152,79,76,202,205,197,164,156,108,196,72,189,67,70,3,12,6,35,244,122,29,
  116,58,189,220,56,151,82,47,170,241,187,46,244,14,133,50,175,112,62,174,90,181,42,104,155,223,239,23,237,81,244,118,73,61,97,97,61,96,94,92,
  56,127,1,101,98,58,94,165,141,151,46,88,128,209,163,71,195,104,52,202,189,69,122,131,30,6,157,208,96,209,72,54,182,115,255,232,9,27,1,132,
  213,35,207,243,66,35,51,196,30,175,207,39,246,36,41,123,252,4,187,75,203,78,162,197,125,54,200,255,147,147,147,49,123,217,50,196,196,8,245,104,
  52,24,4,59,165,6,153,216,211,167,105,167,30,75,75,74,240,187,30,106,144,117,228,179,62,159,210,166,16,251,197,109,3,221,103,137,65,34,70,1,
  224,150,219,110,131,43,66,55,253,47,127,241,139,1,211,101,48,208,67,251,161,15,225,78,62,237,201,182,190,81,30,24,55,126,60,30,127,242,73,88,
  83,82,16,27,27,139,199,159,124,18,0,48,110,252,248,168,123,8,170,171,170,84,31,8,237,229,115,86,19,244,211,102,206,10,59,198,241,99,197,216,
  246,229,151,234,98,190,143,114,58,71,99,91,88,3,51,100,159,139,178,179,195,142,97,183,217,176,127,215,206,54,187,6,72,110,250,133,139,22,97,202,
  148,41,152,50,101,10,172,41,41,88,123,221,117,242,103,239,189,251,110,84,199,168,82,241,9,101,148,41,32,70,70,59,138,168,165,140,77,13,59,246,
  137,99,199,112,248,192,254,182,222,32,109,91,132,169,51,81,166,110,7,29,118,108,15,235,145,81,139,140,202,145,52,49,186,198,51,69,100,84,167,199,
  136,228,228,176,99,111,251,106,43,88,128,23,108,147,162,255,93,136,140,246,134,255,135,70,70,149,47,53,27,121,158,199,136,36,139,170,255,127,240,238,
  59,114,61,106,187,24,25,237,41,58,242,89,193,150,192,160,246,89,98,48,69,70,67,22,255,30,80,194,102,176,137,181,161,44,178,135,128,109,245,245,
  245,114,151,108,232,170,17,103,206,156,233,177,149,36,134,139,111,132,61,177,7,227,181,203,24,22,46,90,20,180,109,77,23,196,168,219,229,82,141,226,
  168,53,80,218,235,113,80,123,176,55,53,53,193,126,238,44,16,242,16,239,203,6,10,32,244,168,56,236,118,245,103,72,132,30,175,208,198,138,57,46,
  78,85,140,218,235,109,112,185,156,0,199,181,141,39,239,7,27,165,232,99,167,27,99,33,182,155,227,227,195,142,225,245,120,80,83,85,53,96,26,99,
  195,193,103,137,193,34,70,1,188,254,218,107,221,30,51,218,19,130,52,170,229,33,6,97,4,49,210,218,199,131,77,108,12,5,219,182,110,217,130,173,
  91,182,80,249,245,227,245,60,208,108,91,115,205,53,116,77,245,69,67,54,196,70,12,2,223,24,14,254,79,62,75,12,8,49,74,16,195,137,133,139,
  22,97,225,226,197,145,197,234,151,95,246,155,88,37,250,135,41,83,167,162,252,204,153,176,21,69,172,41,41,88,184,104,17,222,143,50,50,74,16,4,
  49,156,232,209,220,244,107,175,185,166,207,163,162,81,182,163,129,33,181,52,46,11,121,71,182,245,55,102,179,25,185,83,166,244,171,135,15,105,223,232,
  108,58,196,126,226,151,79,60,129,76,113,137,175,247,62,250,72,246,9,171,213,26,212,93,79,62,209,3,54,42,103,174,177,176,15,135,214,147,122,144,
  248,63,249,44,209,85,122,52,50,218,209,205,182,55,163,2,170,99,71,67,111,86,138,153,195,131,242,198,171,180,137,169,216,59,152,31,42,131,208,54,
  101,55,125,238,148,41,248,229,19,79,224,177,254,152,164,55,28,124,67,218,202,212,254,30,64,231,221,206,138,10,172,47,235,99,8,250,68,68,225,162,
  88,57,97,40,216,56,152,253,159,124,150,24,16,98,116,245,218,181,253,38,70,163,108,110,13,204,139,111,40,210,215,15,222,161,54,120,157,202,111,16,
  94,208,228,19,125,239,13,195,197,47,24,249,236,112,124,14,144,24,141,226,194,80,113,178,235,250,113,240,126,228,214,228,224,23,137,67,213,174,33,101,
  155,114,134,40,149,223,176,181,45,116,150,48,212,222,83,185,81,48,128,234,145,124,150,26,78,61,33,70,35,179,96,209,34,84,156,57,131,242,242,114,
  0,66,180,180,162,188,28,123,247,236,25,248,109,200,254,104,89,81,116,116,208,178,96,209,34,44,88,184,16,0,16,27,27,11,0,120,244,241,199,229,
  207,191,218,186,21,95,117,103,2,83,111,151,95,127,70,18,134,176,111,60,250,196,19,170,127,15,138,114,27,68,209,37,54,152,109,28,78,162,137,124,
  150,232,107,49,186,122,237,90,172,94,187,22,111,190,246,154,44,70,11,102,207,198,234,181,107,241,226,11,47,68,253,96,158,51,175,16,150,100,75,144,
  175,241,1,33,53,102,104,58,80,213,52,147,1,30,124,132,33,207,179,231,204,21,179,54,4,167,206,212,104,181,208,114,26,33,181,164,70,3,128,139,
  232,223,187,118,236,64,67,39,83,130,38,89,44,152,91,88,24,124,13,241,44,56,149,105,4,123,252,1,30,188,184,141,87,57,118,106,106,26,38,92,
  52,65,182,171,45,197,169,78,72,19,39,46,130,220,222,162,192,101,37,165,40,43,237,90,118,10,53,219,120,158,111,215,46,101,138,211,64,32,32,164,
  132,139,96,219,196,9,19,196,116,160,58,104,117,82,154,83,113,145,103,177,206,208,206,2,214,14,187,3,187,119,238,232,213,11,201,233,116,162,184,159,
  38,239,117,235,86,63,104,186,99,187,241,16,236,67,251,62,24,32,179,229,7,173,79,244,149,72,27,36,2,102,176,249,63,221,199,136,62,23,163,106,
  142,115,217,194,133,120,255,189,247,176,113,227,70,121,219,195,15,62,136,7,31,126,24,215,174,89,19,245,50,55,115,230,205,67,150,34,227,10,3,224,
  247,41,211,194,249,224,147,83,225,133,164,140,243,9,105,199,106,106,107,113,182,222,134,208,209,69,179,231,204,65,140,41,6,6,131,152,38,206,208,150,
  211,90,167,200,83,223,222,2,186,165,37,37,157,206,79,159,100,177,96,217,85,43,194,4,155,148,38,77,78,151,38,165,247,147,82,250,41,210,253,249,
  124,94,28,60,114,52,168,6,24,128,177,169,169,200,206,202,130,49,198,8,131,49,6,70,131,1,122,131,152,135,92,167,131,78,20,219,237,165,133,99,
  108,3,74,187,33,70,67,109,11,4,2,240,250,188,240,203,105,10,149,233,238,124,10,27,219,222,23,29,85,177,45,77,176,45,40,13,156,222,32,212,
  151,94,15,93,20,105,253,202,74,75,177,171,23,196,40,99,12,199,142,29,67,177,248,234,42,102,179,25,169,105,105,97,199,142,38,195,142,252,10,240,
  97,227,232,24,128,248,248,4,196,24,12,225,153,75,164,134,151,98,65,240,246,40,43,45,237,178,125,89,161,217,147,24,84,179,204,4,217,34,102,107,
  9,136,89,120,244,122,3,60,1,111,144,109,6,163,17,169,105,105,33,25,104,164,140,44,29,55,192,186,107,87,164,251,226,87,91,182,192,102,179,245,
  174,144,138,66,180,49,64,213,39,162,17,46,253,26,183,235,132,32,13,74,135,171,72,36,203,6,186,141,93,168,71,22,82,143,172,163,178,227,184,129,
  51,210,116,168,251,44,49,176,34,163,86,171,85,245,161,188,105,227,70,60,250,216,99,81,31,231,249,231,158,85,117,60,57,115,3,34,103,117,144,198,
  101,89,197,220,244,161,252,233,15,207,203,25,57,34,230,56,239,133,22,86,89,105,41,238,189,251,187,237,218,165,150,109,36,244,53,85,37,229,227,222,
  61,187,241,201,250,117,97,25,56,184,208,12,28,189,212,114,236,85,219,118,239,198,39,235,58,176,173,31,91,196,171,215,172,1,214,172,1,0,84,148,
  151,203,194,180,184,184,56,108,173,201,72,164,166,165,225,254,31,61,160,42,230,165,70,136,50,247,184,207,231,15,202,91,45,189,47,82,137,204,78,202,
  157,140,156,172,108,177,161,34,228,175,214,235,133,6,152,86,175,135,78,171,129,86,211,126,78,103,0,234,245,27,37,161,182,241,60,47,54,188,218,26,
  37,97,13,75,217,102,47,252,94,47,74,79,157,70,75,237,217,160,227,88,146,147,177,116,233,149,48,26,99,196,252,227,98,190,106,157,46,168,113,217,
  94,35,172,59,118,69,242,135,226,99,199,186,45,70,211,210,211,113,237,154,224,9,161,124,32,0,159,223,15,191,223,7,191,207,15,191,95,124,249,124,
  240,5,252,8,248,124,194,231,62,31,252,254,0,252,62,31,234,66,50,28,1,64,206,228,201,88,184,96,33,12,6,3,116,6,189,208,24,215,233,161,
  211,137,189,41,98,90,201,246,132,252,135,239,191,135,234,110,166,87,156,59,111,30,230,204,43,12,241,123,133,77,254,0,2,126,165,77,126,217,110,159,
  248,222,237,118,195,209,216,24,118,236,75,46,91,128,196,196,4,24,244,122,232,165,128,131,78,7,157,86,232,57,210,104,52,208,42,82,228,246,150,141,
  106,254,207,24,47,219,226,243,251,21,54,134,212,173,88,175,126,159,15,246,243,23,112,206,222,16,236,255,214,100,228,207,156,9,131,177,237,154,214,233,
  219,108,212,202,105,80,35,251,127,117,117,21,62,124,239,189,30,241,125,85,159,229,3,240,73,118,41,234,175,237,189,210,118,255,128,247,89,98,144,136,
  209,250,250,122,228,230,230,134,9,210,204,204,204,110,167,70,100,157,110,57,51,149,101,49,152,122,235,42,164,101,213,151,210,166,211,118,177,144,54,49,
  83,89,226,163,159,109,26,14,182,1,226,152,208,173,91,145,59,101,10,50,51,51,145,59,101,10,102,21,20,96,233,242,229,0,128,55,95,127,29,159,
  108,218,212,225,113,170,171,170,195,26,96,202,156,206,140,15,128,23,223,75,145,194,208,72,34,207,7,96,29,155,26,86,126,39,138,139,113,228,224,65,
  89,148,5,229,115,142,50,167,115,119,137,100,91,144,29,29,228,176,30,97,177,132,45,219,102,183,217,240,209,251,239,183,217,38,14,181,225,52,226,208,
  13,141,6,26,72,105,32,251,42,0,196,144,153,153,25,241,243,168,35,232,76,101,105,40,241,255,194,101,193,228,123,26,147,26,126,202,127,209,246,94,
  237,154,98,226,127,210,146,119,109,215,154,120,124,198,192,113,237,92,182,172,103,238,15,225,203,95,41,79,133,137,167,207,218,78,51,232,59,10,27,67,
  150,240,131,50,53,53,107,59,22,131,162,113,12,6,14,189,107,35,218,181,81,57,177,13,114,125,72,231,198,148,247,205,208,200,162,178,30,197,250,82,
  126,31,10,27,219,189,247,178,158,189,225,171,217,42,249,148,92,25,172,237,189,188,73,17,213,30,200,62,75,12,18,49,250,223,173,91,177,76,124,16,
  23,23,23,3,0,114,115,115,177,108,249,242,168,30,202,4,49,24,145,162,161,21,21,21,200,47,40,64,126,65,1,172,86,43,92,46,87,84,223,119,
  187,93,170,221,197,145,242,116,71,138,44,91,85,114,58,55,55,53,195,94,127,174,77,120,74,81,101,69,68,185,183,115,58,71,99,27,208,126,111,135,
  57,62,33,236,24,94,175,7,53,213,213,17,115,85,247,117,110,110,137,155,110,185,37,226,103,215,119,176,252,157,50,98,245,135,223,61,215,109,159,152,
  154,55,51,236,24,37,199,143,99,251,87,91,251,213,39,0,96,247,206,157,216,189,115,167,170,125,65,57,203,67,124,67,105,171,57,54,14,227,179,178,
  194,142,253,245,182,255,194,237,114,129,147,114,211,75,254,208,199,54,2,136,170,30,59,242,255,241,89,217,97,199,112,216,109,56,184,103,247,128,241,255,
  225,224,179,196,32,17,163,31,188,255,62,204,177,177,184,118,205,26,92,171,216,254,201,166,77,248,224,253,247,7,108,33,244,247,66,248,157,143,250,118,
  16,93,28,64,23,229,80,182,205,106,181,98,114,110,46,242,11,10,228,76,59,251,246,238,197,155,175,191,142,226,226,98,184,162,236,166,31,174,229,55,
  84,175,231,55,95,127,29,21,21,21,116,77,145,141,195,210,255,169,62,137,126,23,163,210,141,248,131,247,223,199,56,177,171,170,188,162,162,71,30,202,
  61,238,208,33,78,61,216,50,51,13,214,155,210,80,178,237,178,133,11,113,237,234,213,168,40,47,199,115,79,63,45,247,6,80,249,245,211,245,60,16,
  202,149,49,97,236,240,32,242,133,225,112,191,24,136,54,14,69,255,39,159,37,6,140,24,181,90,173,88,186,124,185,156,159,185,162,188,28,159,110,218,
  212,99,179,75,123,236,2,22,133,232,64,113,230,30,177,107,128,138,235,161,106,91,241,177,99,72,182,90,145,159,159,143,71,126,249,75,236,219,187,23,
  251,247,238,197,190,125,251,122,172,1,70,190,49,184,174,231,15,63,248,160,119,103,210,15,3,159,232,13,27,251,123,124,249,112,123,158,13,87,159,37,
  6,136,24,205,28,55,14,63,127,244,81,152,99,99,113,92,140,12,44,93,190,28,151,45,88,128,255,247,196,19,168,16,215,30,37,136,161,192,241,226,
  98,28,47,46,198,95,1,185,187,126,233,242,229,248,238,61,247,160,162,188,28,127,127,227,13,249,58,32,134,7,31,134,12,71,50,199,198,34,51,51,
  19,118,155,173,79,68,42,65,16,196,176,23,163,215,172,94,141,122,155,13,191,186,239,62,57,50,100,142,141,197,35,143,62,138,107,86,175,198,239,158,
  121,166,231,91,89,145,90,88,33,51,251,152,242,253,0,109,69,70,211,114,100,210,76,68,229,123,132,207,186,29,140,17,128,193,106,27,0,184,92,46,
  216,108,54,152,203,203,97,54,155,145,145,153,137,201,185,185,61,38,70,201,55,194,103,19,51,198,13,184,235,57,115,220,56,220,112,211,77,56,94,92,
  140,109,95,125,133,95,253,246,183,48,155,205,0,128,223,61,251,44,246,239,221,75,215,84,143,218,200,84,102,112,15,236,251,252,80,246,127,242,89,98,
  64,136,209,89,249,249,248,221,179,207,6,117,81,186,156,78,124,244,193,7,248,225,3,15,80,105,19,67,138,201,185,185,184,102,245,106,76,206,205,149,
  183,29,47,46,198,190,125,251,228,168,41,49,188,184,225,166,155,96,181,90,241,81,113,49,174,92,190,28,102,179,25,191,123,246,89,92,182,96,1,174,
  89,189,186,71,197,40,65,16,4,137,209,136,77,31,22,221,182,126,107,154,13,158,86,36,49,240,197,40,0,124,244,193,7,36,62,123,226,210,28,34,
  62,241,187,103,158,193,241,226,98,124,247,123,223,195,241,226,98,236,223,187,23,46,167,19,143,60,250,40,85,114,111,60,95,134,136,51,209,20,38,130,
  196,104,15,113,252,216,49,92,179,122,53,142,23,23,203,107,44,154,205,230,94,139,8,116,216,85,63,216,31,200,157,177,107,40,15,96,31,160,182,125,
  244,193,7,84,126,253,41,56,6,162,109,226,185,91,173,86,36,39,39,227,83,113,125,101,171,213,218,43,247,169,225,224,19,195,78,164,13,229,107,123,
  56,221,199,136,62,16,163,17,252,231,173,55,223,196,207,126,241,11,188,244,202,43,114,148,104,114,110,46,156,78,39,94,122,241,197,168,15,127,239,253,
  247,99,162,98,145,95,198,24,252,62,49,29,162,50,175,185,90,58,65,49,55,125,109,109,29,206,169,76,24,248,222,189,247,33,38,70,204,115,110,48,
  8,185,233,13,82,106,177,224,244,129,145,22,209,253,211,243,191,199,201,178,178,78,21,217,196,172,44,220,123,255,15,130,182,5,2,1,209,46,233,220,
  219,254,246,135,218,36,126,126,72,37,229,99,126,65,1,38,78,204,66,140,209,40,219,37,231,166,215,106,161,211,235,132,188,228,28,7,78,163,81,61,
  191,79,55,109,148,31,156,157,37,146,109,202,122,242,42,235,76,97,79,91,250,71,31,14,171,218,54,27,89,89,19,97,52,198,192,96,52,200,233,44,
  117,122,61,244,186,182,212,126,237,165,124,60,89,86,138,63,61,255,252,144,121,32,117,170,17,198,162,252,173,190,92,183,47,26,219,58,113,35,26,72,
  15,167,125,123,247,226,174,187,239,150,27,227,251,247,238,197,149,203,150,225,154,107,175,197,190,238,54,200,135,178,79,244,184,88,99,3,215,198,33,236,
  255,228,179,68,87,91,143,186,158,116,178,138,242,114,60,242,147,159,96,233,178,101,200,16,215,25,253,116,211,38,124,250,201,39,176,119,98,38,233,158,
  93,187,112,178,180,76,97,155,152,10,209,31,64,32,16,64,32,224,23,255,21,95,210,118,190,237,111,30,12,224,194,133,215,190,61,187,161,211,233,161,
  213,233,4,241,169,211,138,57,124,181,208,104,53,208,112,162,168,225,0,46,194,37,222,224,112,116,58,202,209,224,112,224,211,141,27,131,182,241,140,71,
  32,192,171,218,195,7,217,201,203,118,67,163,13,59,118,109,77,13,26,236,246,54,65,45,217,164,176,75,200,90,193,181,43,216,186,26,185,137,108,155,
  74,29,5,2,224,67,236,245,7,4,123,213,109,171,198,121,187,29,58,189,32,60,117,90,41,23,177,148,2,82,153,179,62,66,125,53,56,6,116,244,
  60,53,45,13,223,188,118,117,112,249,241,188,74,62,231,208,188,228,62,121,187,207,231,199,57,181,156,206,147,38,97,212,101,151,181,229,116,214,137,249,
  171,229,220,213,138,220,237,237,220,200,95,120,254,247,93,182,239,251,33,13,21,198,24,2,210,121,43,108,8,203,205,173,176,217,113,254,2,234,67,115,
  115,91,146,49,115,70,94,91,131,82,47,230,172,214,74,215,128,174,195,134,101,119,236,82,227,165,23,95,196,13,55,221,132,100,171,21,111,189,241,6,
  108,54,27,172,201,201,56,94,92,44,52,200,163,244,67,147,201,132,212,180,180,176,91,46,207,248,144,180,169,66,234,84,213,212,170,60,175,122,236,248,
  248,120,196,24,12,194,253,78,145,191,92,217,168,19,238,21,145,207,175,166,186,26,110,183,187,91,101,149,100,177,32,41,41,41,228,190,161,146,42,86,
  241,106,219,206,192,243,1,104,245,122,213,99,39,39,39,131,231,121,104,52,90,57,111,121,91,26,92,69,182,34,244,174,141,82,99,61,244,89,45,217,
  161,150,222,55,216,70,161,126,13,6,61,124,110,111,208,113,12,6,35,198,166,166,66,163,213,65,163,213,64,43,166,196,213,112,130,141,26,141,70,184,
  39,114,145,197,170,219,237,70,77,117,117,143,248,126,231,124,86,178,51,32,216,201,248,65,225,179,196,64,139,140,182,131,221,102,195,91,111,190,25,124,
  99,176,90,241,173,107,175,197,191,62,252,48,106,49,26,174,125,219,79,35,166,76,31,7,198,144,60,122,52,172,163,71,7,221,2,24,24,246,238,217,
  163,158,62,77,113,99,234,141,180,98,13,14,7,62,221,180,177,93,187,212,236,65,72,10,181,220,25,51,130,154,30,12,12,181,53,53,176,215,215,135,
  229,26,87,222,108,123,51,45,92,191,219,54,36,90,195,44,252,61,107,203,237,44,205,26,110,211,50,76,190,217,7,103,234,102,33,30,175,216,59,40,
  127,55,20,249,158,165,135,21,235,51,219,130,243,76,67,153,164,58,244,177,29,102,75,232,17,153,178,109,204,66,34,46,140,9,198,245,81,248,200,229,
  114,225,165,191,252,37,104,155,116,63,148,102,213,71,219,64,185,231,127,239,15,107,160,200,61,9,161,61,13,62,159,220,139,226,245,249,224,247,10,61,
  73,135,143,29,13,243,137,236,201,147,144,61,49,11,70,163,17,6,163,1,6,131,17,122,73,200,139,189,13,90,81,180,69,42,180,63,255,225,249,78,
  247,14,133,82,48,103,14,174,92,182,60,104,91,32,16,8,233,253,242,194,235,85,233,17,19,109,190,112,161,17,167,202,207,132,184,54,195,188,75,46,
  197,168,148,20,209,70,177,183,72,180,81,106,212,106,180,218,118,239,29,61,97,35,128,176,122,100,140,193,231,19,123,191,34,245,24,201,118,11,159,151,
  157,58,141,22,247,185,32,159,183,36,91,48,235,27,87,32,70,236,53,50,24,4,59,117,98,61,182,245,244,69,174,199,83,101,101,120,225,15,61,211,
  107,212,145,207,182,103,175,215,231,29,20,62,75,12,34,49,170,134,181,147,98,148,32,6,3,223,186,246,90,124,235,218,107,35,126,254,175,15,63,140,
  202,231,107,170,171,241,130,202,48,130,174,137,249,96,74,79,28,199,142,255,126,213,175,98,62,26,219,208,78,227,146,49,134,113,19,39,134,55,124,29,
  118,28,220,171,104,92,42,109,236,131,70,88,180,204,202,207,199,37,151,93,134,89,249,249,184,233,59,223,137,234,59,53,213,213,248,115,136,72,96,114,
  196,147,137,17,165,72,17,53,49,58,26,224,145,60,118,76,216,177,75,142,159,192,209,162,34,81,168,136,61,66,82,148,73,45,199,121,132,243,235,46,
  123,119,237,198,169,16,113,16,26,37,99,237,70,71,121,104,245,122,36,90,44,97,199,222,241,245,54,176,160,200,168,20,21,85,218,168,233,48,146,214,
  19,132,215,35,196,58,138,108,103,232,251,4,21,27,29,118,59,62,254,240,67,33,26,42,70,11,133,8,112,219,181,160,9,9,180,168,69,70,123,138,
  142,125,150,15,243,219,193,230,179,196,32,23,163,4,49,20,57,113,252,184,44,54,147,173,86,92,122,217,101,65,226,243,196,241,227,84,72,195,148,140,
  204,76,92,185,108,25,102,229,231,195,108,54,195,110,179,117,170,49,238,118,187,85,163,56,157,109,160,168,61,216,91,154,155,6,68,111,67,67,131,67,
  24,66,19,98,31,148,246,133,216,138,16,27,205,177,177,170,98,212,97,183,195,229,114,201,130,76,238,253,234,135,30,149,104,234,177,163,198,152,41,62,
  62,236,24,30,175,23,53,53,53,3,166,49,54,28,124,150,24,228,98,84,118,190,238,31,168,147,251,119,240,62,244,216,253,229,204,157,154,81,216,73,
  155,250,59,50,52,132,109,83,46,231,52,105,242,100,92,114,233,165,61,31,253,239,205,242,235,239,155,119,103,109,99,237,188,31,32,190,241,141,165,75,
  113,229,210,165,72,182,90,97,183,217,176,237,171,175,240,141,165,75,241,210,95,254,210,115,141,147,161,236,19,67,249,62,63,12,252,127,216,250,44,209,
  63,98,52,146,111,76,154,60,57,98,148,128,232,225,27,112,148,215,106,191,153,51,132,109,27,40,229,199,168,252,58,124,56,245,245,76,227,255,185,241,
  70,84,86,84,224,23,63,251,25,42,43,42,100,129,74,62,209,11,215,6,163,149,57,7,154,255,147,207,18,93,173,43,93,79,254,224,79,30,121,164,
  157,107,132,245,140,67,117,228,208,29,220,160,84,47,78,149,11,120,208,223,120,7,176,77,67,209,54,214,147,62,222,219,13,153,129,84,126,221,188,158,
  7,90,131,102,255,190,125,152,57,107,22,126,242,200,35,56,176,127,63,246,239,219,215,214,5,73,62,209,115,126,209,149,58,31,136,247,196,33,230,255,
  195,218,103,137,110,209,163,221,244,191,249,213,175,6,191,60,31,128,45,199,238,180,142,251,179,101,220,83,173,226,129,104,27,0,124,243,154,107,240,173,
  107,174,9,218,246,198,91,111,201,127,255,235,163,143,240,239,143,62,234,155,242,139,58,51,205,192,41,191,94,121,152,245,179,109,207,255,238,119,72,182,
  90,49,115,214,44,92,185,116,41,46,185,244,82,0,66,116,212,108,54,227,192,254,253,125,119,77,13,82,159,232,156,141,244,16,31,232,215,54,221,199,
  136,62,23,163,145,198,68,153,205,230,110,119,213,247,138,67,171,56,53,6,160,99,119,183,149,60,40,46,214,65,104,219,137,227,199,241,175,118,196,230,
  96,152,192,212,95,229,215,107,215,243,0,176,205,110,179,225,63,159,126,138,255,124,250,41,50,50,51,113,201,165,151,226,146,203,46,195,204,89,179,112,
  203,13,55,144,79,244,104,227,188,107,226,165,191,239,243,67,217,255,135,155,207,18,3,80,140,190,254,214,91,248,191,95,253,10,39,142,31,199,79,30,
  121,4,111,191,245,22,42,43,42,144,145,153,137,159,60,242,200,160,184,17,19,68,103,196,232,137,227,199,97,54,155,229,140,59,0,228,201,43,4,81,
  89,81,129,183,43,42,240,246,91,111,97,230,172,89,84,32,4,65,16,189,45,70,149,45,184,73,147,38,5,47,242,220,231,131,205,131,103,240,11,43,
  72,12,254,62,157,224,133,207,165,36,0,67,164,117,59,8,109,251,206,13,55,96,230,172,89,120,240,135,63,4,0,204,156,53,11,255,251,131,31,224,
  111,47,189,132,175,183,109,163,242,235,73,219,130,175,238,1,57,127,229,155,33,195,54,66,233,110,55,61,249,68,232,93,94,186,175,179,224,251,254,16,
  235,190,31,44,254,79,62,75,12,12,49,218,67,20,204,158,141,145,73,150,160,75,47,16,224,35,164,200,12,78,15,202,75,219,34,76,38,201,207,47,
  80,164,67,108,203,196,33,231,164,23,211,168,9,168,7,254,247,237,217,141,134,134,134,78,217,148,148,148,132,252,217,115,66,46,186,54,27,120,191,152,
  206,52,160,242,242,43,236,82,57,246,152,177,99,113,209,248,241,66,202,76,173,14,58,69,58,80,193,38,113,33,100,174,157,76,28,39,203,112,234,228,
  201,46,213,87,68,219,252,193,117,226,231,21,182,6,217,232,23,234,55,146,109,23,141,23,83,157,74,118,233,228,20,127,66,74,184,246,109,59,223,224,
  192,222,61,123,122,220,79,51,50,51,241,141,43,175,196,191,255,245,47,121,91,101,69,5,14,236,223,143,59,238,186,11,149,149,149,242,140,106,98,120,
  240,205,111,125,11,46,151,139,234,157,32,8,162,191,196,104,112,36,146,69,252,172,35,242,103,207,198,4,69,198,21,198,0,191,223,167,72,149,166,150,
  62,45,56,77,92,109,221,89,212,171,116,149,206,42,200,135,209,24,35,167,136,147,210,196,73,233,211,218,242,222,70,30,129,114,234,100,89,167,197,232,
  200,164,36,124,99,233,149,65,219,132,84,105,109,233,208,124,94,159,138,157,193,54,30,62,118,76,85,176,77,156,48,65,53,245,157,156,18,78,18,166,
  17,204,250,207,167,172,203,98,84,205,182,64,128,135,207,231,133,223,231,151,109,147,108,240,139,169,238,252,62,33,253,155,80,103,126,28,137,96,91,214,
  196,9,66,157,25,12,208,139,117,166,211,233,160,211,235,131,242,78,71,178,237,212,201,147,189,34,70,103,206,156,137,138,138,138,160,73,74,118,187,29,
  127,248,253,239,241,244,115,207,97,230,204,153,81,137,18,147,201,132,177,169,169,97,215,75,228,44,52,76,53,39,185,26,113,241,241,48,26,12,114,22,
  26,101,78,231,208,5,164,219,163,171,190,1,32,232,90,150,174,103,22,148,87,157,133,101,164,9,205,210,162,55,24,224,109,13,206,205,109,52,24,132,
  220,220,98,70,22,109,104,150,157,14,242,210,119,215,46,53,42,42,42,144,145,145,129,244,140,12,156,56,126,28,7,15,28,192,137,227,199,97,183,219,
  123,52,26,24,84,152,29,237,211,254,77,123,240,79,122,33,27,195,190,51,96,231,61,12,225,250,36,6,144,24,5,128,235,255,231,127,228,241,115,210,
  223,157,201,201,12,0,47,254,233,79,170,66,87,45,3,7,139,144,167,222,58,122,52,146,71,141,14,59,206,95,95,124,177,45,125,24,32,103,230,128,
  226,161,213,27,153,28,78,157,60,137,7,127,240,131,118,237,82,102,29,65,4,187,38,79,159,17,118,140,3,251,246,225,243,79,54,9,15,95,241,252,
  57,101,26,56,197,191,189,65,191,219,54,0,23,61,238,140,248,24,155,154,138,239,221,123,175,170,152,143,212,216,242,41,242,86,75,239,143,136,11,240,
  43,201,206,201,65,214,196,137,138,156,206,6,232,245,6,177,119,64,16,243,90,49,194,220,94,49,170,213,111,180,132,218,198,243,12,126,95,20,13,75,
  175,23,94,191,144,103,253,228,233,211,112,158,173,15,58,78,82,114,50,102,93,113,57,12,70,35,140,6,193,62,189,222,16,156,175,186,131,70,88,119,
  236,82,227,151,63,255,57,204,102,51,38,77,158,140,156,73,147,48,255,210,75,113,251,157,119,194,110,183,227,196,241,227,120,229,229,151,163,246,137,171,
  191,245,173,224,114,11,240,240,251,253,226,203,167,248,91,241,242,73,159,5,224,247,251,112,206,238,8,59,118,86,78,14,46,185,244,18,24,244,66,30,
  115,125,104,46,115,169,215,129,227,34,150,219,199,255,250,23,106,107,106,186,85,86,5,179,103,35,127,246,236,16,191,15,40,236,8,126,5,252,126,248,
  36,187,125,126,4,2,126,184,221,173,104,104,108,12,59,246,188,249,133,72,72,72,144,253,65,176,81,43,250,132,78,110,152,161,151,109,84,243,127,198,
  88,135,54,6,252,126,248,124,126,248,3,126,4,124,126,56,46,92,64,189,227,124,208,113,44,22,11,102,206,152,14,125,80,240,65,234,65,210,201,1,
  136,246,252,191,182,166,6,31,43,122,118,186,131,170,207,242,124,136,157,190,16,91,253,97,159,15,100,159,37,6,137,24,85,206,30,86,254,237,114,58,
  123,116,102,113,79,14,31,25,144,45,171,174,14,6,26,12,217,40,134,144,109,7,246,239,199,213,223,250,22,242,102,206,196,193,3,7,228,237,121,51,
  103,98,210,164,73,248,167,98,153,167,246,168,173,169,9,107,128,41,35,163,225,209,81,245,200,104,242,152,177,97,199,46,45,41,193,177,195,135,131,242,
  115,107,186,16,25,237,14,225,182,117,62,50,154,144,20,158,242,177,193,110,199,186,127,255,91,182,71,171,213,202,81,223,104,35,163,189,129,203,229,194,
  137,227,199,193,24,19,26,230,140,97,210,228,201,152,52,105,82,183,239,85,194,139,5,47,109,196,122,226,46,8,176,176,20,63,189,91,110,172,61,35,
  219,183,186,19,207,1,197,88,82,249,45,235,215,123,63,83,173,179,96,27,163,93,186,74,185,159,116,107,13,63,124,255,61,229,164,225,158,65,43,237,
  138,127,178,65,232,179,196,32,17,163,127,124,254,249,160,89,197,189,126,3,235,206,8,238,1,38,110,88,47,28,143,35,219,122,149,202,202,74,124,189,
  109,27,238,187,255,126,84,86,86,194,237,114,193,100,54,35,35,35,3,95,111,219,134,202,202,202,168,142,227,118,187,85,187,139,85,115,85,135,230,238,
  86,124,174,38,70,91,154,155,225,176,213,7,139,206,16,1,218,219,98,45,26,219,218,235,237,0,99,48,197,169,231,230,174,149,114,115,3,224,228,72,
  87,223,244,8,168,113,253,255,252,15,114,38,77,66,70,70,6,92,46,23,14,30,56,128,175,183,109,195,43,47,191,220,169,104,185,90,3,165,43,62,
  161,214,219,80,86,82,130,93,95,111,235,87,159,0,128,125,123,246,96,95,200,240,153,118,237,82,249,220,28,27,139,204,9,19,195,142,189,115,251,14,
  184,221,46,112,0,52,33,189,95,125,105,163,90,99,172,43,254,175,102,163,195,225,192,161,253,251,6,140,255,15,7,159,37,6,154,24,101,76,85,8,
  254,241,207,127,198,193,3,7,132,215,254,253,189,42,76,59,90,131,141,133,181,17,149,45,51,65,136,70,20,53,253,52,77,177,35,161,205,66,34,5,
  157,110,7,247,227,244,203,94,181,173,31,237,122,229,165,151,112,226,248,113,228,205,156,9,179,217,12,187,205,134,207,63,251,12,219,123,120,38,253,80,
  45,191,206,217,166,246,239,192,186,158,47,191,226,10,184,92,46,28,216,191,31,85,98,99,36,57,57,25,201,151,92,2,0,61,214,37,58,212,125,162,
  115,193,7,53,223,96,3,246,62,63,148,253,159,124,150,232,106,61,244,104,100,244,157,127,252,3,243,47,189,20,183,221,113,7,112,199,29,178,40,61,
  120,224,64,175,71,76,9,162,175,201,153,52,9,201,201,201,216,190,109,91,143,139,79,98,112,82,114,226,4,0,33,209,71,78,55,187,229,137,174,200,
  28,18,36,4,49,24,233,81,49,250,249,127,254,131,207,255,243,31,36,39,39,35,111,230,76,228,76,158,140,219,238,188,19,0,112,240,192,1,252,233,
  249,231,169,196,137,33,131,52,73,101,251,215,95,83,97,16,0,128,167,126,243,27,42,4,130,32,136,254,20,163,18,118,187,93,22,166,243,47,185,4,
  171,190,249,77,204,200,203,235,243,134,114,104,180,120,40,68,241,213,108,26,42,177,128,193,102,27,99,12,38,147,41,98,4,204,97,183,247,232,146,62,
  195,218,55,66,236,99,61,50,113,103,232,51,148,125,34,200,47,88,228,251,62,249,63,249,44,49,12,197,168,217,108,198,140,153,51,133,200,168,152,133,
  169,170,178,18,159,191,253,118,191,24,24,23,107,66,139,211,141,88,179,169,205,179,219,27,95,211,223,87,94,7,4,2,1,36,38,196,201,182,13,170,
  187,74,20,72,245,52,66,180,145,15,4,6,180,137,233,25,25,120,232,39,63,81,253,108,221,191,255,141,117,255,254,119,159,150,95,167,143,63,132,39,
  241,13,139,39,245,16,247,137,94,177,127,128,218,200,200,103,135,166,207,18,125,47,70,191,127,223,125,152,49,115,38,0,160,170,178,18,235,254,253,111,
  28,60,112,0,142,158,136,14,49,38,207,12,148,132,36,131,184,238,36,99,96,28,23,52,75,175,185,241,2,146,172,86,204,186,184,45,106,117,161,161,
  65,88,210,70,60,78,216,154,149,210,69,211,15,142,29,106,155,188,61,196,174,11,13,14,88,172,41,88,48,79,136,52,243,60,143,230,198,198,240,25,
  134,202,227,244,163,93,80,156,7,147,202,185,29,219,146,147,44,65,182,57,91,90,58,182,173,31,111,68,149,21,21,120,247,159,255,84,253,172,167,163,
  162,209,76,230,208,106,181,225,98,190,189,6,88,127,149,95,123,15,168,8,159,197,24,13,208,233,180,242,223,204,239,235,216,182,126,244,251,62,23,48,
  29,60,244,165,178,235,176,81,222,159,215,84,196,73,45,67,76,192,116,193,255,219,179,99,48,249,63,235,45,91,73,144,14,31,49,42,47,199,16,225,
  246,96,73,78,198,59,111,191,141,162,3,7,224,112,56,122,86,208,72,203,55,116,226,226,170,173,172,64,92,66,2,116,58,61,90,221,46,56,155,155,
  161,215,235,7,84,5,112,157,92,126,163,165,177,17,140,231,97,48,26,1,0,77,23,46,128,3,6,156,93,93,177,173,177,161,1,126,159,15,49,38,
  51,252,126,31,90,154,154,6,172,109,210,245,224,114,185,228,73,43,61,125,108,229,205,90,89,130,76,18,231,210,103,162,168,119,187,156,72,27,99,197,
  136,132,56,196,197,154,218,196,188,212,0,131,122,111,0,99,172,79,151,69,9,205,198,198,133,54,84,68,219,148,141,149,11,142,6,140,205,200,144,27,
  151,49,70,3,234,108,231,194,27,148,98,153,73,199,27,74,43,15,134,46,253,131,118,150,5,146,8,4,2,176,140,76,68,139,211,5,75,210,8,0,
  64,171,187,53,178,240,147,202,173,143,125,34,84,172,200,189,208,33,118,2,193,203,1,249,60,30,0,64,102,250,24,160,170,78,248,23,128,215,235,109,
  219,95,249,236,24,0,2,38,232,218,102,225,115,229,213,158,174,94,175,23,35,70,38,97,180,53,73,110,108,54,93,56,47,239,171,12,166,48,41,216,
  50,0,252,63,116,73,174,161,234,179,132,242,250,101,170,247,249,14,197,40,235,74,8,157,169,255,208,227,143,62,58,160,10,197,239,243,225,66,15,139,
  226,129,64,75,83,211,144,117,228,150,166,166,65,99,95,201,137,19,112,216,237,96,189,216,13,213,25,65,111,171,171,67,194,200,145,136,49,153,209,210,
  212,132,166,11,231,1,198,160,211,117,220,230,100,253,48,208,46,40,3,90,7,120,61,173,168,175,171,69,92,124,2,52,90,45,234,235,106,225,113,187,
  161,139,162,161,194,134,216,32,66,169,220,164,158,33,176,144,165,191,21,162,205,81,127,14,41,99,198,98,74,206,69,226,245,213,8,183,203,9,141,86,
  219,230,87,17,124,172,63,203,77,121,62,161,54,50,165,240,98,12,231,237,118,140,76,78,198,136,41,89,0,128,243,14,59,24,207,171,219,24,26,161,
  235,79,223,80,156,19,167,104,132,49,149,122,188,224,112,32,46,46,30,57,19,51,229,103,219,121,135,67,72,244,16,90,86,3,208,255,219,171,207,161,
  226,179,132,242,194,234,124,125,232,168,228,8,162,107,72,195,79,6,202,4,38,158,231,197,198,151,99,72,150,183,171,165,5,174,150,150,97,237,115,29,
  9,15,181,50,171,60,117,18,6,99,12,120,62,0,175,199,3,157,86,59,40,108,140,182,17,214,120,190,1,206,150,102,232,116,122,33,221,164,207,39,
  15,87,25,50,118,50,134,234,242,51,136,49,9,169,181,91,221,194,82,137,131,201,206,161,236,179,68,247,33,49,74,16,93,164,240,146,75,176,242,234,
  171,35,126,190,254,227,143,187,55,129,137,32,122,168,145,34,137,151,161,138,223,39,136,208,161,206,80,175,199,225,228,179,4,137,81,130,232,17,182,127,
  253,117,187,227,69,251,50,42,74,16,4,65,16,36,70,9,98,152,225,176,219,225,176,219,145,51,105,18,92,46,151,156,254,145,32,8,130,32,136,94,
  22,163,61,185,222,174,37,57,25,143,62,246,24,238,191,247,222,160,237,233,25,25,248,254,189,247,226,39,15,63,12,179,217,140,235,174,191,30,51,242,
  242,228,117,75,215,125,252,49,138,14,30,4,32,140,217,91,121,245,213,120,230,183,191,13,59,254,203,175,190,138,59,111,187,45,226,182,151,95,125,21,
  14,187,29,63,121,248,97,249,243,91,111,191,29,133,243,231,227,153,167,158,66,201,137,19,120,240,199,63,70,78,78,78,208,49,74,74,74,84,127,175,
  43,172,186,250,106,20,206,159,15,75,114,50,28,118,59,222,249,231,63,101,219,212,62,223,177,125,59,214,125,252,177,108,251,131,15,63,140,159,60,252,
  112,208,24,70,169,60,94,126,245,85,213,223,12,45,147,222,56,247,7,127,252,99,172,255,248,227,160,232,161,180,45,39,39,71,181,139,123,253,199,31,
  99,221,199,31,35,61,35,3,171,174,190,90,78,150,80,116,240,32,214,125,252,177,44,248,86,93,125,117,216,247,165,58,81,171,47,233,184,61,137,217,
  108,198,131,15,63,140,244,140,12,0,194,114,102,79,60,246,24,149,95,59,132,30,219,97,183,227,133,63,253,9,85,149,149,157,242,219,25,121,121,88,
  117,245,213,72,207,200,128,203,229,194,230,207,63,151,207,111,149,104,151,242,124,213,182,13,53,44,201,201,200,201,201,193,142,237,219,85,183,89,146,147,
  49,127,254,252,160,239,108,223,190,189,103,150,222,235,4,133,243,231,35,57,57,89,126,111,23,239,105,18,57,147,38,133,249,159,84,119,171,34,12,139,
  89,247,241,199,242,216,109,229,245,162,220,150,51,105,18,44,22,75,216,111,73,159,171,149,79,103,203,200,146,156,140,188,188,60,124,241,249,231,97,247,
  138,203,175,184,2,235,62,254,24,133,243,231,163,164,164,4,14,187,29,133,243,231,195,225,112,4,157,179,244,121,178,88,119,161,148,148,148,200,251,207,
  200,203,131,195,225,80,109,8,167,103,100,32,79,188,254,43,43,43,131,158,41,161,117,160,86,15,221,241,67,181,114,12,173,35,181,250,82,59,47,169,
  14,0,180,91,63,161,190,97,183,219,81,116,240,32,165,34,239,99,186,170,13,251,61,50,42,69,151,102,228,229,133,93,44,7,197,247,247,220,123,47,
  28,14,7,126,250,240,195,112,185,92,200,153,52,9,247,220,123,47,254,252,199,63,162,164,164,36,104,137,140,174,22,158,36,6,0,32,59,39,71,232,
  98,85,44,193,241,204,111,127,43,252,86,15,115,221,245,215,195,98,177,224,233,167,158,146,163,108,247,220,123,47,158,120,236,49,56,236,118,172,20,5,
  133,244,185,37,57,25,223,191,247,94,48,81,32,72,231,120,235,109,183,225,153,167,158,146,207,87,218,46,61,188,37,225,177,190,7,31,198,29,157,187,
  106,189,136,219,214,41,196,77,104,131,65,18,121,235,63,254,24,47,252,241,143,0,128,203,175,184,2,15,62,252,176,236,3,76,188,185,169,218,211,139,
  245,21,250,80,77,207,200,192,107,175,188,18,212,136,137,246,134,62,44,203,47,228,216,51,242,242,112,207,189,247,226,167,15,63,28,181,223,74,66,244,
  221,119,222,145,69,196,247,239,189,87,126,152,178,30,188,65,14,38,146,45,150,48,255,83,110,75,182,88,48,111,254,124,236,20,63,55,137,126,242,140,
  232,127,125,41,70,29,14,135,252,155,133,243,231,99,201,21,87,224,73,177,33,151,157,147,131,236,156,28,148,182,227,127,217,162,72,83,238,35,109,83,
  138,27,229,182,236,156,28,172,186,250,106,84,85,86,162,170,170,42,236,243,208,242,233,234,243,108,229,213,87,227,224,193,131,65,101,58,35,47,15,105,
  233,233,109,246,139,207,61,169,33,250,228,99,143,201,162,73,250,92,201,202,171,175,86,189,86,87,93,125,53,236,118,59,254,252,167,63,5,109,191,229,
  182,219,144,51,105,18,138,14,30,132,219,229,146,131,57,175,139,141,188,208,58,232,105,63,140,84,142,161,245,17,90,95,74,65,155,157,147,19,116,140,
  142,234,39,180,140,10,231,207,71,78,78,14,94,139,208,176,37,134,64,100,180,167,249,226,243,207,81,56,127,126,144,24,157,87,88,136,39,31,127,28,
  51,242,242,96,50,155,241,154,36,180,68,231,125,237,149,87,176,246,250,235,229,27,88,228,103,31,107,119,27,99,12,69,7,14,96,70,94,30,54,127,
  254,57,210,211,211,225,114,185,224,238,131,214,148,217,108,198,146,203,47,199,15,238,187,79,190,17,73,182,153,77,38,184,205,102,172,92,181,42,232,115,
  135,221,142,103,158,122,10,191,255,227,31,177,89,108,125,75,23,115,103,132,80,111,159,123,103,230,115,135,214,209,202,85,171,176,227,235,175,131,162,11,
  95,124,254,57,44,22,11,86,174,90,133,119,223,121,103,64,92,60,38,179,25,37,39,78,200,101,46,61,88,168,252,162,167,232,224,65,220,19,210,43,
  210,17,43,175,190,26,235,20,17,99,135,221,142,215,94,121,37,234,178,31,206,56,236,246,224,232,48,99,40,156,63,191,71,27,169,209,176,227,235,175,
  131,26,59,247,220,123,111,208,253,171,164,164,68,245,156,164,115,239,106,227,186,178,178,18,183,220,126,123,196,231,70,88,249,116,129,131,7,14,132,149,
  105,123,247,102,151,203,165,122,93,150,156,56,33,251,248,138,85,171,194,206,75,10,216,228,228,228,4,5,83,114,114,114,144,158,145,17,36,112,215,125,
  252,49,126,243,219,223,34,39,39,71,46,247,208,58,232,85,63,139,214,47,196,50,202,201,201,129,197,98,9,58,70,178,197,210,238,113,153,216,72,87,
  222,99,127,255,199,63,146,24,37,49,218,185,7,210,45,183,221,6,179,217,12,151,203,37,119,61,72,45,71,165,72,237,206,67,44,18,37,37,37,40,
  156,63,31,155,63,255,28,217,147,38,161,244,196,9,185,235,85,98,237,245,215,7,9,212,29,219,183,119,91,248,165,167,167,163,180,164,36,172,27,65,
  30,126,32,70,7,66,63,119,185,92,40,45,41,65,186,216,210,6,128,215,94,125,21,191,248,229,47,85,203,170,55,232,232,220,187,117,236,140,12,213,
  135,76,209,193,131,65,93,203,133,133,133,65,221,88,207,40,26,44,202,250,170,170,172,28,80,2,108,184,151,95,154,226,218,154,55,127,126,187,17,176,
  72,101,23,90,78,85,85,85,114,180,75,205,54,139,197,130,29,59,118,12,249,27,186,201,108,14,178,59,45,228,62,166,22,129,178,247,114,15,66,180,
  34,68,41,216,44,22,75,144,29,46,151,43,168,126,187,243,172,73,79,79,199,146,43,174,144,27,243,237,149,159,244,124,232,12,155,63,255,28,247,220,
  123,175,124,13,90,146,147,145,150,158,30,241,121,177,254,227,143,113,221,183,191,141,29,219,183,119,202,198,194,249,243,241,154,56,204,76,41,126,231,137,
  207,178,208,123,203,79,127,252,227,136,215,33,0,216,123,48,82,26,90,142,85,85,85,61,210,93,222,153,250,153,145,151,215,231,67,80,136,65,46,70,
  93,46,23,118,108,223,142,121,133,133,216,252,197,23,194,191,210,141,162,157,238,119,166,200,56,17,105,113,213,104,34,163,165,37,37,248,222,247,191,47,
  56,240,140,25,216,252,197,23,72,75,79,15,90,124,120,199,246,237,168,86,140,203,177,247,192,98,250,237,157,119,52,118,41,207,207,97,183,99,243,231,
  159,99,173,120,83,99,106,221,187,61,72,71,231,30,122,142,29,109,235,104,159,176,223,100,12,37,37,37,170,93,54,161,245,213,107,99,134,24,67,90,
  90,26,30,120,232,33,225,230,158,150,134,36,139,5,217,217,217,194,3,118,199,142,136,93,74,195,181,252,24,99,40,44,44,132,107,198,12,249,33,37,
  117,49,170,150,73,7,215,111,164,122,9,181,109,222,252,249,253,187,192,121,31,192,68,17,183,98,213,170,160,232,144,84,127,76,244,81,201,95,77,102,
  51,56,160,91,221,210,93,245,1,166,242,12,48,153,76,114,253,101,139,145,177,160,198,134,178,65,164,86,151,29,109,19,255,126,247,157,119,240,192,67,
  15,9,13,26,229,179,64,165,252,0,160,228,233,167,59,101,95,85,85,21,24,32,15,53,40,44,44,12,106,60,133,222,187,93,46,23,222,125,231,29,
  220,124,219,109,248,127,143,63,30,213,53,110,73,78,150,239,251,69,46,23,30,120,232,161,54,241,107,177,4,173,228,49,111,254,124,36,139,101,89,82,
  82,130,210,146,146,176,235,16,0,118,238,216,129,29,61,32,222,212,202,241,189,119,222,17,252,80,165,62,162,189,63,118,84,63,140,49,252,245,111,127,
  147,183,59,28,142,176,225,11,4,137,209,168,90,172,43,86,173,194,206,29,59,48,35,47,15,111,188,246,154,124,241,172,253,246,183,177,126,221,186,176,
  86,79,180,173,72,229,13,217,108,54,171,138,225,210,146,18,204,155,63,31,217,57,57,120,246,233,167,177,228,242,203,131,111,48,149,149,157,142,224,116,
  68,105,73,9,178,115,114,130,206,79,178,45,61,61,93,22,197,161,159,155,205,102,100,231,228,160,186,170,74,30,135,4,0,235,215,173,195,175,127,251,
  219,62,105,13,118,116,238,82,125,133,150,183,73,165,252,213,142,61,35,47,47,172,188,67,235,220,97,183,71,108,21,247,70,125,133,98,119,56,130,206,
  167,51,81,141,225,92,126,239,190,243,78,183,142,45,217,167,124,192,91,146,147,177,246,186,235,240,226,11,47,168,218,150,173,50,17,100,40,82,85,89,
  137,103,21,226,41,59,39,7,43,21,15,111,187,195,33,251,150,187,135,162,141,61,65,232,117,176,115,251,246,176,123,126,79,33,55,220,175,187,14,213,
  33,246,135,150,95,87,217,44,14,61,147,158,43,47,118,32,138,138,14,30,196,188,194,194,176,231,78,36,150,92,126,57,178,115,114,130,196,151,116,77,
  184,92,46,97,200,74,200,53,22,58,206,182,187,215,97,103,252,176,175,142,251,221,59,238,0,32,140,153,117,216,237,3,198,191,137,142,209,12,36,49,
  106,54,153,176,230,186,235,176,115,251,118,249,198,84,90,82,2,183,203,133,155,111,189,85,126,48,103,231,228,224,230,91,111,197,123,81,116,29,22,29,
  60,136,53,215,93,39,223,240,214,92,119,93,196,110,255,37,151,95,222,103,221,220,242,77,235,139,47,112,243,173,183,202,81,0,201,182,18,177,11,119,
  243,23,95,224,71,15,61,36,127,110,177,88,240,163,135,30,194,250,117,235,84,35,86,175,191,250,106,88,203,81,110,105,246,225,185,43,27,24,82,189,
  45,185,252,114,88,44,150,14,111,128,82,116,92,121,99,94,114,249,229,152,87,88,136,13,226,3,170,55,236,233,44,59,183,111,199,179,79,63,29,241,
  213,81,196,105,184,151,95,180,81,22,166,98,223,138,85,171,228,135,171,197,98,193,247,190,255,125,249,193,163,246,157,193,98,111,111,227,22,27,222,165,
  37,37,3,234,65,173,188,247,70,83,87,106,251,216,237,246,176,70,135,60,25,53,228,59,155,191,248,2,150,228,100,76,207,203,235,21,191,40,42,42,
  194,244,25,51,132,213,44,24,139,170,172,223,123,247,93,249,26,239,72,184,207,152,49,3,63,252,223,255,197,119,239,184,3,223,189,227,142,160,0,202,
  206,29,59,176,228,242,203,229,251,134,36,236,165,136,237,64,185,158,123,243,154,124,239,157,119,48,175,176,112,216,52,66,135,2,3,106,157,209,29,59,
  118,96,197,202,149,248,139,24,221,144,120,241,133,23,176,230,186,235,240,171,223,252,6,38,179,25,213,85,85,120,227,213,87,131,30,202,89,217,217,248,
  203,203,47,203,239,55,172,95,143,13,235,214,225,141,215,94,195,205,183,222,42,127,118,168,168,72,142,186,2,109,93,1,165,37,37,178,16,86,110,151,
  254,254,209,131,15,134,157,239,221,119,222,217,35,23,205,138,85,171,240,243,71,31,133,201,108,134,195,225,8,178,77,18,15,146,32,117,56,28,216,185,
  99,135,188,61,244,92,75,75,74,132,101,92,58,184,161,245,212,5,223,222,185,111,254,226,11,88,44,150,160,122,123,78,165,85,27,218,29,227,114,185,
  240,220,211,79,99,197,170,85,114,67,226,80,81,17,158,123,250,233,168,186,140,7,75,126,226,225,88,126,209,12,77,136,166,129,9,198,100,33,239,118,
  185,176,121,243,230,160,107,98,184,18,205,176,164,129,112,142,74,161,144,157,157,13,147,201,132,157,138,49,189,217,217,217,97,141,234,157,219,183,195,209,
  206,240,168,67,69,69,88,114,249,229,114,195,68,58,238,161,162,34,213,253,223,120,245,85,60,242,232,163,65,1,136,36,149,110,96,73,188,119,6,135,
  221,46,63,83,54,127,241,69,84,245,225,176,219,177,249,139,47,228,107,54,210,119,166,207,152,33,7,43,148,231,152,100,177,192,98,177,160,232,224,65,
  100,103,103,227,145,71,31,197,161,131,7,225,114,187,145,158,158,142,236,236,108,188,248,231,63,171,214,129,252,220,236,161,107,40,90,63,11,173,103,101,
  29,171,29,163,189,250,81,238,239,114,185,240,186,248,236,127,228,39,63,33,165,55,8,224,166,79,153,26,245,221,41,191,160,0,127,123,237,85,220,114,
  227,77,216,187,103,15,149,30,65,16,196,0,193,98,177,8,203,225,40,68,157,114,155,218,231,253,193,188,194,194,160,149,15,170,43,43,81,164,16,140,
  210,210,78,161,40,133,138,218,210,78,128,16,53,156,62,99,134,60,187,252,80,81,145,44,218,212,190,51,175,176,16,14,135,3,165,37,37,176,136,75,
  7,133,210,21,49,10,8,19,237,166,231,229,133,137,232,121,133,133,40,45,41,129,195,225,8,250,91,98,197,170,85,97,223,89,177,106,149,44,20,231,
  21,22,162,58,100,194,30,32,204,119,112,185,221,242,185,90,44,22,121,53,154,208,50,14,173,131,158,20,163,237,249,153,178,14,212,234,89,178,59,146,
  47,183,87,63,202,50,82,218,169,86,86,68,239,81,48,123,54,94,255,251,155,184,227,214,219,176,111,239,94,18,163,4,65,16,4,65,16,196,192,23,
  163,26,42,58,130,32,8,130,32,8,162,191,232,226,152,81,54,228,151,73,33,8,130,32,8,130,32,58,169,15,187,0,69,70,9,130,32,8,130,32,
  136,126,67,71,69,64,16,4,65,16,189,143,197,98,145,87,198,80,110,3,16,52,97,201,100,54,203,169,169,67,215,66,149,22,211,15,221,23,8,158,
  156,101,177,88,84,39,41,149,246,81,198,173,208,223,15,181,69,237,252,66,207,95,73,232,74,10,38,179,25,22,139,5,213,85,85,170,19,222,28,118,
  123,187,171,47,16,36,70,9,130,32,8,98,216,49,183,176,16,139,151,44,193,207,127,246,51,57,221,238,220,194,66,0,192,198,245,235,1,0,171,175,
  187,14,243,230,205,67,117,117,53,76,38,19,76,102,51,254,250,231,63,203,66,238,7,15,60,128,123,238,186,75,254,238,154,181,107,85,247,157,91,88,
  136,185,133,133,104,8,17,100,125,37,70,67,127,63,201,98,65,131,195,129,191,252,249,207,112,187,92,97,159,155,76,38,88,44,22,185,108,164,114,169,
  174,170,194,85,43,87,226,215,79,62,25,116,252,171,86,174,4,0,124,240,238,187,248,193,3,15,160,172,180,52,232,243,157,59,118,192,49,12,82,0,
  147,24,37,8,130,32,8,162,83,48,0,55,221,114,11,254,42,174,249,169,100,209,146,37,200,206,206,14,19,171,223,189,231,30,252,250,201,39,229,109,
  18,171,215,174,197,239,158,125,86,22,170,139,151,44,193,85,43,87,202,199,222,181,99,135,44,114,251,131,208,223,191,233,150,91,176,120,201,18,121,155,
  218,231,243,230,205,195,151,155,55,203,219,14,21,21,225,198,91,110,145,215,217,150,152,62,99,70,80,25,254,238,153,103,200,185,6,49,52,102,148,32,
  8,130,32,250,136,45,155,55,35,41,41,9,89,217,217,170,98,244,205,215,95,15,18,157,187,118,236,64,85,101,37,166,79,159,46,136,217,144,197,221,
  149,199,249,114,243,102,85,145,59,80,216,184,126,61,230,204,155,23,241,243,24,147,73,117,250,203,174,29,59,176,104,201,146,32,33,234,114,58,101,17,
  206,104,66,245,160,135,34,163,4,65,16,4,209,135,124,240,222,123,184,241,150,91,240,155,255,247,255,130,182,75,99,32,67,169,169,174,86,29,255,249,
  210,139,47,98,249,138,21,88,189,118,45,202,74,75,113,168,168,8,187,118,238,148,197,236,242,21,43,176,124,197,10,121,255,178,210,82,252,254,217,103,
  251,205,110,105,65,123,137,57,243,230,201,98,218,100,50,1,0,254,190,115,103,184,24,221,185,19,119,125,239,123,248,224,189,247,0,0,23,207,152,129,
  93,33,251,189,240,215,191,6,189,255,253,179,207,134,117,221,19,36,70,9,130,32,8,98,88,35,69,240,36,225,184,104,241,226,176,244,160,38,179,57,
  172,59,62,198,100,146,183,41,247,175,174,170,194,75,47,190,8,64,72,137,189,104,201,18,92,60,125,58,158,127,238,57,48,198,176,113,253,122,108,218,
  176,161,95,109,141,180,157,49,134,178,146,18,236,218,185,19,105,233,233,88,180,120,49,126,243,171,95,69,180,211,237,114,225,226,25,51,80,86,90,138,
  57,115,231,202,194,84,218,247,222,187,239,38,7,27,196,80,55,61,65,16,4,65,244,49,155,54,108,192,162,37,75,144,38,206,132,7,128,221,59,119,
  98,110,72,55,182,201,108,198,244,233,211,113,248,208,161,160,237,105,233,233,120,226,87,191,146,223,151,149,150,226,165,23,95,84,237,254,31,40,204,157,
  55,47,200,14,135,195,129,178,210,82,108,217,188,25,135,14,29,10,138,226,134,178,107,231,78,76,159,62,93,46,139,80,193,78,12,110,40,50,74,16,
  4,65,16,125,140,219,229,194,223,223,120,3,119,221,125,183,220,53,191,105,195,6,252,228,231,63,135,201,100,66,89,105,41,76,102,51,150,175,88,129,
  67,135,14,133,117,223,87,87,85,193,225,112,224,174,239,125,79,238,154,95,180,100,73,144,216,179,88,44,97,226,180,186,186,186,207,132,156,242,247,45,
  22,11,174,93,187,86,142,228,170,137,243,159,62,242,8,14,23,21,169,118,175,239,218,185,19,79,63,247,28,82,211,211,85,163,189,161,118,186,221,110,
  213,33,15,4,137,81,130,32,8,98,136,99,181,90,97,77,73,65,241,177,99,84,24,33,132,174,123,121,184,168,8,187,118,236,144,183,59,28,14,252,
  230,255,253,63,44,90,188,24,203,174,186,10,110,183,27,91,54,111,198,110,197,248,72,229,210,76,47,189,248,34,230,204,155,135,133,139,22,9,199,59,
  116,72,222,215,225,112,96,98,86,22,150,93,117,85,208,111,126,244,254,251,168,238,3,49,26,250,251,13,14,7,94,122,241,69,89,104,134,150,133,36,
  206,47,158,62,29,101,165,165,170,159,111,218,176,1,19,179,178,112,184,168,40,232,179,210,146,146,48,59,107,170,171,73,140,14,34,184,233,83,166,70,
  61,13,45,191,160,0,127,123,237,85,220,114,227,141,216,187,123,143,234,62,82,235,164,172,180,20,203,87,172,192,196,172,44,52,56,28,248,240,253,247,
  225,118,187,169,196,137,65,207,130,133,11,113,217,194,133,120,242,177,199,6,181,29,171,215,172,193,181,107,214,224,250,181,107,7,181,29,191,16,235,97,
  48,215,71,238,148,41,248,197,47,127,137,39,31,127,124,208,139,184,161,226,87,4,65,116,158,130,57,179,241,250,223,255,142,59,110,189,13,251,246,238,
  141,250,123,61,26,25,93,118,213,85,88,190,98,5,54,109,216,128,164,164,36,44,187,234,42,84,87,85,225,98,113,73,138,183,222,124,147,106,106,24,
  179,112,241,98,185,213,123,248,208,33,220,117,247,221,72,178,88,176,245,203,47,241,225,251,239,15,26,59,146,173,86,228,230,230,14,250,250,24,50,139,
  161,12,129,101,93,148,147,58,200,175,8,130,24,110,116,121,2,19,83,121,45,90,188,24,91,54,111,198,166,141,27,113,241,244,233,104,176,219,241,127,
  191,254,53,62,124,255,125,92,60,125,186,234,119,232,53,60,94,19,179,179,113,237,234,213,48,153,76,88,180,120,49,126,242,179,159,161,186,170,10,91,
  54,111,198,194,69,139,48,109,0,251,71,178,213,138,201,185,185,242,123,73,52,68,250,124,32,219,113,217,130,5,17,237,152,156,155,59,40,236,200,24,
  55,14,179,10,10,34,126,62,171,160,0,25,227,198,13,104,27,76,177,177,184,118,205,26,36,91,173,65,117,1,69,93,93,187,102,13,76,177,177,3,
  190,62,46,91,176,0,75,151,47,143,232,87,75,151,47,15,242,59,122,209,139,94,67,247,213,231,98,84,13,147,201,132,195,135,15,3,128,16,1,43,
  43,3,0,56,26,26,228,53,196,136,225,73,86,86,22,170,171,171,241,127,191,254,53,254,46,70,200,255,254,230,155,248,240,131,15,80,93,93,141,180,
  180,180,1,117,190,153,227,198,201,127,47,88,184,16,191,248,229,47,35,238,123,247,61,247,224,238,123,238,81,253,110,127,163,140,224,46,88,184,48,232,
  60,67,185,118,205,26,220,116,243,205,65,118,152,99,99,7,92,125,220,116,243,205,65,231,25,202,77,55,223,140,213,171,87,171,150,65,127,215,133,100,
  71,172,217,140,107,87,175,70,126,65,65,196,125,175,93,189,26,86,171,85,182,63,210,190,125,141,213,106,197,178,229,203,131,234,166,189,250,88,182,124,
  57,46,91,184,48,232,189,100,23,65,16,4,208,195,221,244,14,135,3,115,230,204,65,86,86,22,24,99,56,116,232,16,76,38,19,22,46,90,20,54,
  24,153,24,126,184,196,65,243,110,151,11,140,49,121,12,177,107,128,45,209,113,217,130,5,184,251,123,223,195,255,123,226,9,20,23,23,163,166,166,6,
  231,47,92,192,183,174,189,22,122,189,14,233,233,25,40,175,168,192,205,183,220,130,83,167,79,33,54,46,14,245,245,245,242,247,127,241,232,163,40,46,
  46,198,115,202,244,116,98,164,8,33,107,10,114,202,31,230,56,197,159,92,208,251,238,216,241,220,179,207,98,223,222,189,176,219,237,56,87,95,143,235,
  174,191,30,28,128,164,228,100,156,40,41,193,117,215,95,15,159,207,7,157,94,143,179,231,206,33,198,100,130,70,163,137,108,71,136,13,242,57,171,217,
  194,113,194,246,110,216,114,211,205,55,99,233,178,101,184,255,190,251,96,179,217,80,81,94,142,81,163,71,227,27,75,175,196,152,209,99,228,253,110,190,
  245,86,212,213,213,66,163,213,194,102,179,201,194,233,231,143,62,138,15,63,248,0,31,126,240,65,196,250,232,200,142,158,168,143,31,61,248,32,42,202,
  203,241,228,19,79,32,62,33,1,231,234,235,145,59,117,42,70,140,28,137,196,17,35,112,228,232,81,44,93,190,28,211,103,204,192,216,180,52,156,171,
  175,71,172,216,24,88,182,108,25,242,11,10,240,191,197,197,112,57,157,109,231,172,214,189,175,180,69,121,206,162,13,221,173,143,201,185,185,184,241,166,
  155,96,179,217,112,234,212,41,232,13,6,84,85,87,227,127,110,188,17,23,93,116,17,2,60,143,35,71,143,226,231,143,62,138,51,103,206,160,213,227,
  65,67,67,3,198,166,166,194,231,245,226,198,155,110,130,213,106,197,155,111,188,17,100,71,208,245,161,54,92,65,58,247,30,180,133,32,136,33,40,70,
  55,109,220,136,27,110,188,17,128,176,124,196,225,67,135,112,231,93,119,225,226,139,47,198,91,127,255,59,149,54,49,96,145,34,86,21,229,229,216,183,
  119,47,26,174,187,14,203,87,172,192,204,252,124,184,91,91,241,198,223,255,142,218,154,26,52,54,53,130,241,188,208,13,201,128,132,132,120,252,253,173,
  183,48,110,220,56,220,253,253,239,35,224,243,193,231,243,97,247,174,93,224,121,30,166,152,24,76,157,54,13,123,118,239,86,253,221,160,71,174,66,156,
  50,229,3,184,19,15,222,220,220,92,56,93,46,217,14,199,218,181,88,176,112,33,242,102,206,68,195,249,243,248,231,187,239,70,180,163,164,172,12,227,
  198,141,195,45,183,221,134,128,207,7,143,167,21,91,54,111,150,237,40,152,51,27,95,109,217,26,241,183,149,182,112,74,193,170,34,30,184,14,236,200,
  47,40,64,69,121,57,108,54,27,54,109,218,136,75,46,189,4,55,221,114,11,236,118,59,236,13,13,120,229,213,87,97,183,219,177,125,231,46,104,53,
  26,240,124,0,26,157,14,137,9,137,176,189,250,42,198,142,29,139,59,191,251,93,36,36,36,160,169,185,9,91,183,108,1,207,243,72,26,57,18,99,
  83,83,113,68,236,193,137,214,142,174,212,71,230,184,113,184,108,193,2,252,93,20,93,95,110,254,2,249,5,179,241,163,7,31,196,217,115,231,240,225,
  191,254,133,250,115,231,208,208,208,128,86,183,27,0,3,207,24,140,70,35,172,214,20,164,140,26,133,25,51,103,98,209,146,37,184,104,252,120,236,219,
  183,23,45,205,205,96,140,97,233,210,165,56,114,248,48,106,106,106,218,45,71,73,164,114,74,225,173,172,143,40,253,234,218,213,171,81,81,81,129,125,
  123,247,226,216,209,163,104,104,112,96,241,146,197,152,158,151,135,211,103,206,96,227,39,159,160,174,174,14,142,6,7,56,198,16,8,4,192,3,24,149,
  50,10,155,183,108,65,230,184,113,178,29,181,181,181,248,122,219,54,240,129,0,198,166,142,197,236,57,115,240,233,166,79,58,158,224,42,10,214,32,91,
  66,26,11,28,9,83,130,32,49,186,123,215,46,148,149,149,193,146,148,36,119,209,111,221,178,5,159,108,218,132,234,234,106,42,237,97,78,90,90,26,
  254,247,7,63,144,135,108,252,239,15,126,32,111,63,41,250,75,127,241,139,71,31,69,125,125,61,158,250,237,111,81,56,127,62,246,31,56,128,253,7,
  14,224,244,233,83,176,59,28,112,185,92,48,232,244,200,204,200,192,168,177,99,225,241,120,80,94,81,129,234,202,74,248,121,30,230,88,51,70,36,38,
  34,45,53,13,39,79,159,198,216,212,84,92,116,209,69,152,54,109,26,46,91,184,0,13,14,7,78,158,60,217,225,121,132,198,131,56,41,18,201,113,
  242,103,156,66,16,169,69,223,156,78,39,254,240,252,243,152,59,111,30,14,28,60,216,37,59,82,83,211,80,118,234,36,38,102,103,195,237,118,35,43,
  43,75,176,195,238,192,145,35,71,186,100,7,24,19,196,131,36,42,34,216,97,181,90,241,163,7,30,64,113,113,49,222,250,251,223,145,63,123,54,54,
  127,185,5,199,79,156,192,241,227,199,113,161,241,2,220,110,55,98,12,70,76,154,148,3,139,197,130,11,23,46,224,76,249,25,148,159,62,13,30,194,
  144,161,20,107,10,50,51,50,112,172,184,24,179,242,243,113,244,200,17,172,186,122,21,38,76,156,136,255,247,248,19,104,104,104,232,213,250,184,108,193,
  2,44,91,182,12,77,77,77,208,106,52,208,25,99,240,250,155,111,98,255,129,253,168,173,173,69,32,16,128,193,96,192,184,140,12,20,20,20,64,163,
  209,160,170,186,26,167,79,157,194,193,162,34,4,2,1,104,181,90,36,37,37,97,230,140,25,152,60,57,23,215,92,123,45,46,92,56,143,111,44,189,
  18,49,49,49,168,249,215,191,162,242,239,246,234,35,200,14,77,248,200,45,115,108,44,150,45,95,14,198,24,98,98,98,144,63,123,54,214,109,216,136,
  131,7,15,162,180,172,20,141,141,141,178,208,159,149,151,135,236,236,108,216,108,118,156,60,85,134,147,101,39,113,166,252,12,118,238,218,9,147,201,132,
  236,172,108,76,201,205,197,252,75,47,69,178,213,138,194,194,66,36,37,37,225,171,45,91,163,94,109,69,181,177,160,172,19,73,100,147,48,37,136,65,
  131,118,116,74,202,99,209,238,60,54,53,21,171,190,121,53,254,253,175,127,169,182,200,179,178,178,96,22,133,134,197,98,145,115,208,150,151,151,83,73,
  15,115,146,44,22,36,196,199,3,0,252,62,31,26,20,195,54,154,154,154,80,86,86,134,154,126,108,176,120,189,94,100,79,202,193,232,177,99,112,234,
  212,105,124,254,197,231,56,124,248,8,106,235,234,224,245,120,144,58,118,44,230,205,157,139,236,236,108,36,142,72,132,78,167,131,209,96,128,57,46,22,
  28,199,80,127,174,30,118,135,3,117,103,235,80,81,89,137,86,79,43,114,167,76,193,248,139,198,227,228,201,147,248,106,235,214,136,131,187,163,121,100,
  6,117,79,162,109,130,136,28,21,18,169,173,169,193,148,105,83,17,159,144,136,99,197,197,221,178,163,178,170,10,173,158,86,100,231,100,99,220,248,241,
  56,89,86,134,47,190,248,162,227,115,236,172,29,162,144,144,236,112,185,92,240,251,253,200,153,148,3,173,94,143,67,135,14,99,235,127,191,66,241,177,
  98,156,171,63,7,48,134,236,137,19,49,107,214,76,100,103,101,195,108,54,67,163,209,32,198,24,131,24,147,9,126,159,15,14,135,3,54,187,13,213,
  53,53,168,172,172,68,128,231,49,103,238,28,164,164,164,96,235,150,173,56,214,206,242,73,93,181,67,34,37,37,5,46,151,11,135,15,29,194,156,185,
  115,161,211,235,80,122,242,20,254,241,246,63,112,240,224,65,52,53,53,65,175,211,225,162,241,227,113,229,55,190,129,57,115,230,192,108,54,195,235,245,
  66,171,213,98,228,200,145,136,143,143,131,207,227,69,107,107,43,90,156,78,156,58,125,26,37,37,37,208,234,116,200,205,157,130,64,128,199,63,223,126,
  27,62,191,31,38,147,9,62,191,191,75,182,68,178,131,3,96,21,237,240,249,124,176,217,234,49,102,236,88,120,124,94,252,231,243,207,177,233,147,77,
  40,47,47,71,107,107,43,204,38,19,22,92,122,41,150,44,89,130,9,23,77,128,207,231,131,207,231,69,92,108,28,146,44,22,24,244,122,56,157,78,
  184,91,91,113,246,220,89,156,60,89,134,218,186,179,72,77,75,131,213,106,197,129,3,7,228,6,206,216,177,99,209,220,220,220,61,91,164,40,170,100,
  11,137,82,130,232,51,82,211,82,241,205,107,174,193,186,143,63,70,109,109,109,47,71,70,35,60,85,239,187,255,126,213,237,110,183,27,127,120,254,249,
  126,21,27,68,255,178,123,215,46,236,222,181,43,226,231,125,61,193,205,108,54,227,231,143,254,2,159,126,242,9,246,238,217,139,177,169,169,248,98,243,
  151,216,189,123,55,78,159,57,141,166,166,38,196,24,13,200,201,206,70,118,86,22,198,140,25,3,163,209,0,143,199,11,167,211,133,64,128,71,124,92,
  28,12,6,3,18,226,227,49,34,113,4,26,26,26,112,174,190,30,182,122,27,156,45,78,212,212,212,32,183,122,50,38,78,152,128,212,180,52,4,2,
  1,220,120,243,77,248,251,27,111,162,174,174,174,221,203,137,139,112,201,201,221,223,98,132,209,100,54,227,209,95,254,18,21,21,21,120,243,141,55,144,
  63,123,54,222,251,224,3,108,219,246,53,106,107,107,192,120,30,137,9,9,40,200,207,199,140,233,23,195,146,100,129,203,237,66,83,83,51,92,46,55,
  120,158,71,66,124,2,76,38,19,172,201,201,200,72,207,64,253,185,122,84,86,85,161,254,92,61,108,54,27,14,31,57,130,41,185,185,184,120,234,52,
  76,152,48,1,46,151,11,55,222,116,19,254,254,230,155,168,171,173,109,19,50,93,181,3,64,178,197,130,223,60,245,20,222,122,243,77,148,149,149,97,
  100,82,18,54,127,185,5,91,182,108,65,85,117,21,92,78,39,76,166,24,204,154,153,135,137,19,38,32,57,217,10,173,86,3,151,171,21,46,151,19,
  28,167,65,98,66,34,98,98,98,144,152,144,128,81,41,41,168,171,59,11,71,67,3,106,107,107,113,254,252,121,56,28,14,92,60,237,98,140,25,61,
  26,201,201,201,200,204,204,196,252,75,230,227,165,191,190,132,214,214,214,110,215,7,99,12,171,87,175,70,225,37,243,241,251,231,158,67,193,156,185,216,
  127,240,0,54,127,249,37,142,29,43,134,211,217,2,141,120,179,94,188,104,49,210,211,210,16,224,3,104,106,106,70,99,83,19,252,62,63,180,26,45,
  76,49,49,24,51,122,12,44,73,22,52,54,54,162,178,162,18,103,235,235,113,254,194,5,108,217,186,21,245,231,234,49,103,206,108,92,185,108,25,182,
  110,222,140,27,111,190,25,103,78,159,198,134,245,235,21,39,172,94,39,209,218,145,148,148,132,159,253,252,17,28,61,114,20,155,55,111,70,230,248,139,
  240,230,63,222,194,193,3,130,152,78,72,136,199,184,140,12,140,31,63,30,227,197,137,110,46,151,11,117,103,235,224,118,11,101,105,48,24,144,52,114,
  36,226,227,226,145,153,153,137,243,231,207,163,182,182,14,78,151,19,167,78,157,68,117,117,21,202,203,203,145,55,99,58,38,78,156,136,233,211,167,35,
  191,32,31,47,255,245,37,156,62,125,186,75,182,200,93,248,202,198,78,32,16,49,234,75,16,68,15,211,197,41,245,93,239,166,87,25,96,254,135,223,
  255,94,117,215,107,86,175,198,255,220,112,3,158,250,205,111,168,162,134,121,116,116,206,220,185,112,185,92,216,179,107,23,220,110,55,76,38,147,156,105,
  228,147,141,27,251,244,138,97,0,238,252,238,119,49,99,230,76,236,219,183,31,27,63,217,132,230,166,38,164,36,91,49,247,138,43,48,253,226,139,97,
  48,24,208,210,210,130,230,150,22,92,168,183,129,15,4,0,38,78,62,210,106,17,19,19,3,157,94,143,184,216,56,164,165,165,129,231,121,52,54,54,
  162,180,180,20,103,202,203,81,83,91,139,138,202,74,92,177,228,114,100,102,100,64,175,211,7,77,60,137,230,122,230,84,30,200,156,248,192,117,57,157,
  40,47,47,199,37,151,93,138,17,73,35,177,109,219,215,248,244,211,79,209,220,212,132,209,214,20,204,152,153,23,100,71,189,221,142,150,150,150,32,59,
  160,225,160,215,235,193,105,52,208,235,244,72,72,72,192,69,19,46,146,237,176,55,52,96,255,129,3,104,106,110,198,55,150,92,142,180,212,84,232,245,
  250,182,104,96,59,182,68,101,7,0,167,203,5,155,173,30,55,222,114,51,118,239,222,141,237,219,119,96,199,174,157,112,181,56,145,153,153,129,188,188,
  60,92,116,209,69,208,106,117,104,105,105,70,115,75,11,220,110,55,120,127,0,0,192,51,6,173,94,7,147,86,3,131,193,136,17,137,35,144,150,158,
  6,159,207,7,135,205,129,138,202,10,20,23,23,163,178,170,10,211,167,93,140,249,151,93,138,153,51,102,160,254,92,189,112,254,81,214,7,23,65,32,
  73,245,177,107,231,78,204,42,200,199,218,235,175,199,87,95,253,23,71,142,29,133,179,165,5,25,169,169,200,158,148,131,139,198,141,131,57,46,30,206,
  150,102,156,171,183,161,181,213,13,159,207,47,71,242,120,48,104,180,90,24,141,70,232,117,122,152,76,102,140,26,61,26,173,110,55,108,245,54,212,219,
  234,81,93,83,131,198,207,63,71,238,228,92,172,250,214,183,96,177,88,240,245,182,109,193,231,24,193,158,246,236,80,10,57,135,195,129,125,123,247,98,
  218,244,233,224,180,26,124,246,217,127,112,166,252,12,204,102,19,198,101,164,35,119,202,20,164,164,164,128,227,52,112,185,156,168,183,217,224,245,120,192,
  7,120,185,62,192,113,208,233,116,208,104,52,208,235,117,136,141,141,197,168,81,163,208,212,216,132,122,91,61,46,52,54,226,68,73,9,92,110,23,102,
  229,205,68,230,184,113,216,187,103,47,78,41,132,104,103,109,137,212,216,9,18,165,20,41,37,136,1,71,143,142,25,141,52,238,239,163,15,62,192,125,
  226,248,64,98,248,10,209,135,127,250,83,57,2,122,241,244,233,248,232,131,15,112,199,93,119,33,201,98,193,223,94,122,169,79,207,199,229,116,225,185,
  167,159,193,125,63,184,31,59,118,238,194,23,95,124,129,252,89,179,48,51,111,6,44,73,22,56,157,78,52,156,63,143,230,22,39,124,94,47,120,158,
  7,227,121,249,193,198,131,201,34,70,167,209,128,7,7,142,105,0,248,145,152,152,136,75,47,189,20,94,175,23,85,85,85,168,169,173,197,215,219,191,
  70,128,159,135,170,138,74,33,154,24,22,122,227,218,143,94,73,147,155,148,226,65,124,224,190,241,218,107,24,105,17,34,137,125,97,199,220,185,115,123,
  214,14,0,174,150,22,188,246,202,171,88,122,213,114,108,217,250,21,138,139,139,145,63,107,22,230,205,157,11,83,140,9,78,103,11,26,155,154,208,216,
  216,36,76,142,225,3,96,60,147,133,15,19,237,224,0,232,180,26,240,28,7,142,113,96,90,134,209,163,71,35,35,51,3,45,45,78,156,57,115,6,
  199,138,139,225,243,121,145,52,50,9,159,127,246,153,250,88,197,8,118,68,20,59,226,182,234,170,42,108,221,186,21,30,175,15,21,21,21,24,59,122,
  52,38,92,116,17,198,140,25,11,159,207,43,68,15,107,107,225,241,120,132,186,96,188,220,40,80,214,133,134,227,0,141,6,28,0,158,15,192,104,48,
  32,51,51,19,19,179,38,226,236,217,115,168,171,171,195,209,163,71,17,8,248,225,108,105,193,209,35,71,130,39,40,117,193,14,105,155,57,38,6,238,
  214,86,124,246,233,103,208,25,12,216,178,101,43,60,30,15,10,242,243,49,97,194,4,152,77,102,180,180,52,11,209,117,183,27,126,159,15,60,207,203,
  191,31,90,31,26,78,3,78,195,129,103,12,58,29,67,146,37,9,163,70,141,130,215,231,69,85,85,53,234,234,234,176,199,191,7,1,158,71,229,153,
  51,128,56,54,21,64,80,196,90,94,217,160,189,232,110,4,113,221,22,67,17,26,161,26,18,164,4,49,116,197,104,36,76,38,211,144,200,44,66,116,
  157,217,115,230,192,229,114,225,241,95,252,2,73,22,11,30,250,201,79,112,239,253,247,163,161,161,1,127,251,205,111,250,108,8,199,228,220,92,124,235,
  218,107,241,210,95,254,130,149,171,86,225,179,207,254,131,207,191,248,2,127,248,221,115,168,174,174,129,221,225,64,73,105,41,90,91,61,242,120,70,38,
  61,92,129,224,247,210,195,159,111,251,92,171,209,64,171,209,192,235,245,194,239,243,99,204,152,49,136,137,137,65,217,201,147,224,56,14,121,51,102,96,
  236,216,177,152,50,109,42,0,224,243,207,254,19,22,93,148,39,97,168,60,116,37,241,144,59,101,10,110,188,229,102,252,229,133,63,99,241,21,87,224,
  227,117,235,80,84,116,8,191,127,238,57,148,149,149,225,66,99,35,234,235,79,192,235,245,129,49,97,214,60,199,105,160,213,106,161,143,137,129,70,163,
  1,99,12,129,0,143,64,192,143,64,192,15,191,47,0,134,0,0,6,173,70,3,141,104,71,192,31,192,216,177,99,97,52,26,81,82,90,10,151,203,
  141,57,179,103,35,101,212,40,44,88,176,0,23,26,47,224,63,159,253,39,44,138,197,34,8,8,165,29,203,174,186,10,147,167,228,226,205,215,94,199,
  138,149,43,241,201,103,159,161,177,177,17,79,60,254,24,234,207,213,227,236,185,179,168,172,172,130,207,231,19,203,158,23,162,110,114,80,182,109,69,0,
  105,163,178,62,116,90,45,24,132,49,193,0,195,196,137,19,17,19,99,196,201,83,167,144,184,123,23,46,91,176,0,159,127,246,25,174,92,186,20,37,
  37,37,109,171,30,116,80,31,74,1,100,50,153,240,200,163,143,98,207,158,61,112,216,237,8,4,120,28,56,112,0,87,94,113,5,76,102,51,26,26,
  26,80,89,85,9,175,215,215,22,149,103,128,86,171,129,70,163,23,163,204,0,207,75,117,17,128,223,231,7,184,0,24,207,11,2,91,167,3,207,24,
  90,91,91,17,31,23,7,125,102,6,202,74,75,81,116,232,16,70,140,24,129,153,179,102,65,167,211,97,238,188,185,120,249,175,47,225,66,99,99,196,
  250,224,218,17,114,87,46,93,138,5,11,23,96,227,134,141,152,152,149,133,109,255,221,134,146,146,18,60,248,192,3,168,171,171,67,83,83,19,234,235,
  109,240,121,189,96,96,178,95,233,116,58,232,245,122,104,52,26,240,140,33,224,23,26,12,126,159,15,254,64,64,188,255,11,126,197,24,131,207,239,3,
  31,224,145,154,150,138,0,31,64,69,85,21,24,3,46,95,178,4,49,49,49,184,244,210,75,81,81,81,129,117,235,214,133,69,72,219,243,43,101,99,
  71,77,164,74,117,203,3,109,43,9,16,4,49,180,196,232,196,172,44,85,33,186,116,249,114,26,47,74,160,193,225,128,219,237,150,125,161,193,225,192,
  159,158,127,62,234,89,180,61,65,70,102,38,38,79,158,132,31,255,236,103,216,252,229,151,216,244,233,39,152,55,103,14,154,91,90,80,124,252,184,16,
  225,145,69,39,143,152,24,19,146,147,45,48,197,152,97,142,139,135,78,171,135,223,31,128,203,237,66,75,75,11,156,206,22,184,92,78,180,182,186,17,
  240,123,5,81,33,62,52,117,58,157,156,149,194,233,114,225,208,145,35,8,240,60,230,21,22,34,43,43,11,39,203,74,195,27,105,210,3,82,154,160,
  20,65,60,36,91,173,72,182,90,113,255,3,15,96,243,151,95,98,231,174,93,152,55,103,14,90,156,45,40,175,168,8,177,131,117,193,142,182,9,32,
  90,81,208,129,227,208,234,241,224,228,233,83,136,49,197,96,254,165,151,224,162,9,23,97,223,222,125,109,231,171,98,7,160,88,130,39,196,14,115,172,
  25,249,249,249,72,182,166,96,195,198,13,40,62,94,140,203,46,189,20,54,155,77,174,15,73,200,48,198,16,23,31,15,107,178,21,90,173,14,177,162,
  29,94,175,15,46,183,11,205,205,205,112,58,91,224,118,187,224,245,122,16,240,121,193,131,201,145,96,131,222,0,73,166,58,93,46,236,216,181,11,129,
  64,0,87,173,92,137,148,148,20,28,63,126,188,75,245,193,0,148,149,149,34,111,230,76,28,44,58,136,109,219,182,193,239,247,129,211,106,80,83,83,
  141,198,166,182,9,57,90,173,22,28,199,97,212,232,81,24,145,48,2,156,70,139,184,248,4,4,252,60,90,91,91,225,114,187,224,114,57,209,212,212,
  4,143,199,45,136,62,158,33,32,70,30,57,177,235,27,94,15,120,198,224,104,104,192,225,35,71,48,59,63,31,83,114,167,160,166,186,70,136,38,182,
  83,31,145,236,224,24,195,201,178,50,204,42,200,71,193,156,217,248,120,253,122,236,219,183,15,87,45,91,142,115,231,206,161,162,178,82,142,52,106,181,
  90,177,199,35,9,227,50,51,97,48,152,160,209,232,16,8,8,118,56,93,78,184,221,110,180,180,52,195,237,118,195,227,241,192,239,243,32,16,240,133,
  249,149,209,96,128,207,231,197,137,210,18,196,152,98,112,217,37,151,64,163,213,226,196,241,227,17,163,189,92,136,48,85,109,236,132,246,34,132,8,82,
  181,9,128,4,65,12,1,49,122,111,59,19,152,254,244,252,243,84,218,68,16,255,250,240,195,62,21,162,0,240,217,39,159,160,174,174,22,179,231,206,
  195,238,61,187,209,220,212,132,165,203,150,161,188,188,60,72,192,105,52,28,226,227,71,96,98,86,22,166,228,230,66,167,51,226,92,125,3,154,46,92,
  64,115,75,51,56,241,129,108,48,24,193,243,12,140,105,224,229,52,240,251,60,128,223,47,63,61,57,0,241,113,241,48,26,140,184,208,120,1,165,165,
  165,72,26,57,18,118,155,13,95,109,217,18,244,128,13,125,72,74,15,93,181,7,238,87,91,183,162,174,174,22,139,175,248,70,255,218,81,111,195,127,
  183,110,13,19,112,106,118,72,162,84,105,199,7,239,189,143,6,71,3,198,164,165,97,219,215,95,35,214,108,198,197,211,167,163,186,170,58,72,136,234,
  116,194,152,195,89,249,249,184,104,252,120,120,188,60,26,26,26,209,224,176,195,239,243,203,34,205,104,140,17,245,146,14,30,6,48,191,55,40,138,205,
  129,131,37,41,9,149,162,29,135,14,31,134,53,217,138,61,187,119,227,168,114,237,81,149,250,224,148,118,41,236,112,187,221,120,253,213,215,176,108,197,
  10,28,58,124,8,167,79,159,198,252,194,66,65,148,57,157,178,24,6,128,132,17,137,72,207,200,196,236,217,5,48,197,152,113,190,209,137,134,134,6,
  92,104,104,128,71,195,137,245,33,216,1,104,192,113,58,248,124,30,4,188,173,162,29,194,113,98,140,70,164,88,173,168,171,59,139,242,242,114,24,13,
  70,24,141,49,248,226,211,79,225,243,249,218,173,143,72,118,48,0,39,79,158,196,191,62,252,8,19,178,178,112,228,200,17,248,125,62,204,154,53,11,
  199,79,8,13,3,78,244,43,163,209,128,212,212,52,228,228,100,35,35,35,3,129,0,135,134,134,70,92,184,112,30,30,143,71,30,155,169,213,234,160,
  211,25,32,104,105,38,212,41,130,199,232,38,38,142,128,209,96,132,203,233,70,73,73,41,70,167,140,194,201,146,82,148,150,148,132,219,161,38,168,197,
  37,170,186,36,72,197,9,91,212,109,79,16,67,72,140,70,18,156,253,189,134,36,49,48,24,153,148,132,165,98,26,65,198,24,10,230,204,145,163,233,
  39,203,202,122,221,79,102,229,231,99,255,190,125,24,59,54,21,7,139,138,112,230,244,25,164,88,173,72,26,49,18,69,7,139,196,135,28,3,167,209,
  32,41,41,9,115,230,204,193,216,212,84,232,117,58,248,124,1,232,116,90,112,26,78,126,158,5,2,1,248,253,126,121,12,35,152,6,28,167,7,56,
  6,48,159,108,167,94,167,67,172,201,132,230,230,38,52,53,53,163,172,172,12,25,139,22,35,54,46,14,35,70,140,64,115,83,19,154,154,154,34,10,
  136,80,241,176,96,225,2,236,219,187,15,179,231,206,235,119,59,150,94,185,20,177,113,113,24,53,106,20,108,54,27,154,155,154,130,39,156,132,10,31,
  241,239,216,184,56,228,23,228,227,191,91,191,66,78,110,46,62,253,207,103,168,173,173,69,225,220,185,48,25,141,104,108,108,148,237,208,235,116,72,75,
  77,197,148,169,83,145,158,158,46,116,5,243,126,193,14,142,3,56,136,67,14,66,236,224,180,224,56,29,192,241,0,107,155,88,19,99,140,65,172,201,
  132,11,141,23,224,104,104,192,177,226,99,152,157,95,128,35,135,14,33,119,202,20,28,47,46,14,183,1,8,94,42,72,20,50,147,39,79,6,7,192,
  31,8,192,102,179,225,204,153,114,232,245,122,196,39,36,192,217,210,2,159,215,7,177,105,128,100,139,5,51,102,204,64,106,90,58,116,58,29,56,13,
  7,131,94,7,173,86,7,104,218,126,35,16,8,136,47,191,56,22,83,3,78,163,7,11,120,1,72,67,46,56,152,77,102,196,154,205,104,113,57,81,
  85,93,13,107,114,50,50,199,143,135,219,229,132,209,24,131,211,167,78,201,162,43,204,142,16,33,103,138,137,193,85,43,174,194,230,205,95,34,119,202,
  20,236,63,120,16,23,46,92,64,102,122,6,180,90,13,90,154,91,132,33,17,0,116,58,61,198,143,191,8,57,217,217,24,61,102,52,98,98,98,224,
  241,6,196,201,74,90,185,62,132,97,7,1,121,24,136,212,72,0,199,0,180,77,218,50,24,244,136,143,139,67,83,115,51,154,155,155,113,172,248,24,
  22,47,92,132,209,99,198,32,43,59,27,69,7,15,70,237,87,161,226,58,26,65,10,78,24,207,74,130,148,32,134,136,24,109,79,76,44,88,184,16,
  95,133,70,80,136,97,69,82,82,18,174,92,182,76,126,63,123,206,156,62,107,180,100,100,102,226,254,31,253,16,231,27,206,99,255,129,3,216,252,213,
  86,52,53,55,97,126,225,149,56,119,238,28,60,173,173,178,128,179,36,37,33,47,47,15,153,153,153,242,247,181,90,13,140,6,3,180,58,97,214,185,
  212,141,205,243,60,252,254,0,248,0,47,70,242,0,78,163,3,3,15,198,7,192,11,106,18,73,150,36,52,92,56,15,119,107,43,106,106,107,113,174,
  254,28,150,46,91,134,81,163,71,97,239,238,61,109,99,21,67,186,183,165,165,118,164,237,153,153,153,184,251,158,123,96,183,219,177,115,231,46,236,219,
  183,15,141,141,141,88,241,157,235,81,85,85,9,79,107,43,120,49,146,152,158,154,138,153,179,102,97,244,232,209,109,23,188,78,139,152,24,35,116,122,
  133,29,188,36,228,68,17,196,243,224,25,192,105,244,0,24,120,222,47,219,49,106,244,40,156,111,106,132,203,229,66,101,117,53,78,158,44,195,183,174,
  189,22,9,9,241,56,84,116,8,95,255,247,191,10,189,192,5,11,33,69,4,43,63,63,31,119,223,115,15,174,93,179,6,219,182,125,141,131,7,15,
  130,241,60,198,143,31,15,187,221,33,143,73,212,233,116,72,79,75,67,222,204,153,65,118,232,245,90,161,62,180,58,185,140,4,241,35,8,32,185,62,
  32,10,57,0,188,36,136,52,66,125,212,156,173,3,207,243,56,121,250,52,114,39,231,226,198,91,110,70,66,66,2,120,158,23,186,136,67,162,186,97,
  245,193,24,174,92,122,37,70,140,24,137,202,170,42,124,188,110,29,206,95,56,143,137,23,77,64,124,92,28,220,226,210,89,140,3,226,227,227,49,99,
  70,30,198,101,102,0,26,13,124,94,31,192,0,131,65,7,131,56,214,18,16,198,187,182,137,56,193,6,158,49,112,156,22,26,173,30,124,192,39,76,
  222,98,128,49,198,136,36,75,18,154,157,45,104,108,106,68,85,117,21,230,20,204,70,86,214,68,56,157,78,156,82,38,89,80,218,161,34,228,70,141,
  30,141,233,51,242,144,149,147,131,3,7,14,226,244,153,211,0,3,102,204,152,142,250,250,122,248,197,117,76,117,122,29,198,142,25,141,201,147,114,48,
  98,228,72,33,141,172,86,11,189,78,7,131,81,15,173,216,64,80,250,149,108,7,207,192,192,129,211,232,0,158,129,103,126,89,88,91,44,22,216,196,
  164,12,213,226,10,20,223,88,186,20,177,177,102,52,94,184,128,67,69,69,237,251,85,72,35,161,83,130,84,132,4,41,65,12,17,49,154,154,154,138,
  219,238,188,19,73,22,11,78,150,149,225,213,151,95,134,201,100,194,245,55,220,128,137,89,89,36,70,135,49,159,110,218,132,79,55,109,234,183,223,175,
  172,168,192,203,127,249,43,242,103,207,198,193,131,7,81,91,91,139,24,131,17,151,94,114,9,170,170,171,192,196,84,140,35,226,226,48,103,206,156,32,
  33,10,8,2,38,198,100,132,193,160,7,199,105,0,6,4,2,188,28,137,243,139,81,44,33,223,188,6,90,173,17,60,231,3,99,94,113,221,70,11,
  98,207,158,133,199,227,69,171,199,131,99,197,197,24,149,50,10,187,118,238,194,145,195,135,131,38,91,132,118,167,42,31,184,21,229,229,120,246,169,167,
  177,248,242,37,56,113,226,4,236,14,187,144,114,116,202,20,212,214,213,201,147,74,98,205,102,76,153,58,53,72,192,9,7,5,140,70,131,176,148,147,
  100,135,44,126,132,23,147,198,39,130,3,167,53,136,2,76,176,35,62,62,1,102,147,9,173,238,86,248,124,62,156,58,125,26,153,25,153,216,189,115,
  39,202,203,203,131,150,219,9,141,36,42,237,144,238,5,11,22,45,194,215,219,191,70,83,83,19,70,36,36,32,59,59,27,117,117,117,242,140,236,204,
  212,84,228,23,20,192,106,181,6,153,161,209,10,245,161,211,233,100,241,35,77,198,242,251,165,250,16,103,116,115,90,104,181,82,157,249,196,134,145,5,
  70,131,1,30,175,48,195,253,228,201,147,72,78,74,194,238,93,187,81,114,226,68,91,113,169,116,9,43,237,248,203,159,95,196,220,121,243,16,23,31,
  143,179,103,207,66,171,209,96,244,232,81,48,197,196,224,194,133,70,193,14,198,144,153,145,137,180,180,84,65,88,242,60,120,13,224,243,121,193,105,116,
  48,198,24,132,136,34,3,120,214,38,168,37,81,45,204,184,231,160,225,116,224,116,26,48,222,141,0,227,161,211,233,49,34,49,17,53,90,173,16,153,
  181,219,81,91,87,11,175,199,131,93,59,119,134,101,42,98,145,68,41,199,161,252,204,25,60,251,204,51,184,114,233,82,20,23,23,163,193,225,192,200,
  145,35,144,59,57,23,149,85,149,242,208,143,184,216,56,204,152,49,3,9,241,9,130,159,112,26,184,91,91,97,54,155,97,50,25,97,52,24,101,81,
  237,15,4,224,243,251,225,247,11,117,194,203,227,94,53,208,232,140,208,240,90,120,189,194,48,157,17,137,35,144,16,27,7,151,203,5,143,199,131,99,
  199,139,49,126,220,56,124,248,254,251,65,139,224,171,250,149,74,35,161,83,130,84,241,55,9,82,130,232,31,122,116,21,224,219,238,188,19,110,183,27,
  159,109,218,132,212,84,97,21,254,7,127,252,99,164,166,166,226,213,151,95,166,210,30,198,76,204,202,146,35,161,73,73,73,72,77,77,149,63,251,230,
  53,215,132,69,73,123,131,109,255,253,47,206,158,59,135,211,103,78,195,237,118,99,252,248,241,208,234,116,66,58,67,113,57,153,137,19,179,48,102,236,
  216,224,153,244,226,114,59,70,131,14,6,131,65,152,17,12,38,10,31,63,252,226,204,96,229,67,82,171,213,65,175,19,68,134,52,73,99,100,226,8,
  232,245,66,251,175,238,236,89,56,28,14,212,212,8,19,78,164,223,144,38,196,0,138,44,75,138,117,48,25,99,216,187,119,47,206,158,29,64,118,216,
  237,104,106,106,234,180,29,187,119,239,70,83,115,51,74,197,136,248,212,169,83,193,24,67,139,211,41,78,30,139,193,132,9,19,49,114,228,200,136,118,
  232,13,6,133,24,149,236,240,11,235,93,138,34,67,35,217,161,55,128,227,52,178,29,150,164,36,217,55,206,213,159,131,163,161,1,149,21,21,65,191,
  33,219,33,175,162,16,108,135,219,237,70,83,83,19,170,171,171,113,254,124,3,70,143,26,141,81,163,70,195,213,218,10,159,207,139,0,31,192,196,9,
  19,48,113,226,68,57,234,169,180,131,15,248,161,215,105,196,201,110,76,108,224,248,224,245,250,4,59,120,97,233,39,141,134,19,215,235,52,64,167,23,
  4,159,48,30,56,30,113,113,113,0,128,198,166,38,212,214,157,21,39,63,121,130,38,161,133,218,1,101,244,93,220,230,114,58,225,241,120,80,119,182,
  14,94,175,23,83,167,76,133,199,235,65,139,211,9,158,241,136,141,141,69,110,110,174,16,61,102,188,96,143,248,175,219,237,134,78,203,193,96,212,67,
  35,118,123,7,2,126,4,124,126,217,14,41,10,170,225,52,208,106,180,208,233,132,6,17,99,12,122,189,30,35,147,70,202,245,97,183,59,208,208,208,
  128,184,184,184,144,85,45,218,108,97,145,108,81,218,169,156,232,20,178,178,0,11,41,23,40,252,149,32,136,65,28,25,77,74,74,194,11,127,252,163,
  220,221,122,229,178,101,168,169,169,193,11,127,248,67,159,79,84,33,6,152,24,157,56,17,19,178,178,176,103,247,110,204,158,51,7,19,178,178,240,194,
  31,254,32,68,212,211,210,208,218,139,254,145,108,181,98,210,228,201,240,122,60,112,183,182,194,225,104,128,65,175,71,110,110,46,28,14,7,188,94,33,
  234,151,108,177,32,55,119,50,244,58,245,203,66,171,213,192,20,35,68,227,24,3,252,126,63,124,62,33,234,35,61,192,52,26,13,52,90,141,56,107,
  90,39,44,111,227,247,129,211,112,24,49,114,4,108,118,27,124,62,31,220,238,86,156,61,119,14,86,171,21,26,113,178,71,85,85,149,188,52,141,60,
  251,92,209,5,105,181,90,145,63,187,0,231,27,206,195,237,241,12,28,59,234,207,193,154,146,2,99,76,12,252,62,31,170,42,43,21,33,185,112,59,
  114,115,115,97,77,73,1,0,212,212,214,162,169,169,9,113,177,177,200,207,47,192,133,11,23,224,247,249,192,113,26,36,91,44,200,200,200,144,103,110,
  171,217,97,54,155,160,213,104,192,243,76,182,35,32,45,37,196,113,242,18,85,26,173,48,25,200,239,247,34,16,240,131,211,112,72,73,73,129,205,110,
  135,223,239,71,115,75,11,236,14,59,198,143,31,15,183,219,13,167,211,137,115,231,206,181,91,31,75,151,45,131,173,190,30,137,35,70,224,120,73,9,
  180,90,45,198,142,29,35,140,227,108,105,6,207,11,130,58,35,61,3,102,179,25,60,227,161,101,130,248,18,142,195,137,193,201,0,244,58,29,56,112,
  8,4,68,27,252,1,121,226,19,167,225,132,101,160,180,90,104,52,90,104,56,78,136,40,50,200,25,192,90,90,90,16,8,4,208,216,216,136,248,220,
  92,100,103,103,195,146,108,193,142,237,59,130,163,138,33,209,64,142,177,255,207,222,127,71,73,118,94,231,189,240,239,228,10,29,170,58,167,201,57,97,
  48,51,24,204,12,50,145,9,128,25,4,73,49,152,148,100,89,193,242,181,165,43,221,123,109,175,239,187,235,46,135,207,146,37,235,42,91,178,36,91,
  14,146,40,38,80,4,3,8,130,68,206,97,114,14,157,83,229,124,242,247,199,57,117,186,186,186,170,167,103,56,131,64,214,158,213,107,186,43,158,125,
  222,244,188,207,187,247,179,217,185,107,23,107,214,174,161,82,209,153,153,157,165,144,47,16,14,133,216,178,101,51,233,84,10,211,52,17,5,129,120,60,
  206,200,200,200,194,49,188,224,122,97,197,130,167,88,96,154,6,225,80,200,219,184,184,30,168,54,45,11,199,111,15,65,16,188,62,37,74,136,146,215,
  46,166,169,99,219,22,8,16,139,197,8,249,108,181,97,26,204,207,207,179,102,237,90,6,6,7,201,164,211,92,56,127,126,209,169,129,91,195,148,46,
  97,223,125,102,180,150,5,93,142,17,173,181,37,98,250,45,107,89,203,222,95,204,104,189,165,82,169,22,16,109,217,187,110,183,223,113,59,255,248,159,
  252,2,159,253,194,23,152,154,154,162,84,46,177,122,245,106,134,135,135,72,37,83,216,182,13,174,75,52,210,134,44,107,152,166,237,29,241,214,153,32,
  64,52,26,70,213,84,92,215,241,107,112,155,126,178,140,128,40,73,200,138,167,183,168,40,42,154,166,17,142,68,144,101,5,128,104,212,75,88,242,52,
  35,29,230,230,231,232,237,235,229,214,91,111,101,211,166,77,13,217,191,90,118,241,142,187,238,228,11,95,252,34,159,255,226,23,185,116,233,18,133,98,
  129,109,219,182,177,102,205,106,102,103,231,60,63,28,151,104,180,29,23,153,74,197,194,182,27,251,209,209,17,37,20,210,112,29,7,93,215,49,12,3,
  199,175,80,35,201,50,138,170,160,170,154,231,67,56,76,91,123,59,170,170,129,11,237,237,29,244,246,246,162,170,42,142,235,48,57,61,77,119,79,15,
  251,247,239,247,252,168,178,75,77,253,184,139,95,252,149,95,230,227,143,62,202,212,212,20,142,227,176,125,219,54,250,250,122,201,101,115,56,142,131,36,
  137,68,35,109,8,130,132,101,57,13,139,240,8,2,116,118,68,145,100,25,199,182,3,54,209,117,157,64,126,168,182,61,66,161,16,225,112,52,136,207,
  236,232,232,36,30,139,1,2,186,97,144,76,166,88,189,102,13,251,111,222,207,218,181,107,23,179,113,117,18,67,170,170,178,126,195,122,110,191,243,78,
  52,77,35,149,76,210,221,221,77,127,127,63,150,109,81,46,87,112,28,23,85,209,176,29,129,138,110,98,89,78,112,108,191,192,234,185,200,146,132,36,
  121,208,201,50,173,128,81,69,0,209,103,16,21,85,69,81,84,84,85,37,28,9,17,137,68,113,113,145,36,153,88,44,70,52,18,1,160,88,42,146,
  47,20,216,185,115,39,3,3,131,68,163,209,230,126,212,92,199,246,29,59,216,176,105,35,137,68,2,221,208,89,189,106,53,161,80,136,76,54,139,99,
  59,190,150,168,138,110,216,126,191,242,25,116,151,64,231,213,117,28,20,69,68,86,100,108,171,182,61,92,159,157,150,188,182,80,21,84,213,27,31,109,
  109,237,222,102,195,133,112,56,66,79,79,55,224,129,221,84,58,77,91,91,27,27,54,108,160,179,179,115,241,184,168,103,74,27,48,164,139,192,229,74,
  42,108,181,216,209,150,181,236,39,131,25,173,63,22,169,234,74,182,172,101,181,147,253,146,197,161,94,19,241,26,219,215,191,250,53,78,158,56,201,222,
  125,251,152,159,155,67,22,37,118,108,219,142,44,203,228,11,249,0,120,170,90,136,84,42,135,162,202,104,170,138,166,41,200,138,132,36,46,112,36,154,
  42,163,169,10,174,227,82,41,87,252,196,14,79,86,104,225,71,66,146,188,31,89,18,169,148,75,84,202,101,20,89,166,171,171,139,116,58,141,105,154,
  164,51,25,138,197,18,111,189,241,6,217,76,118,177,38,103,13,203,83,101,130,190,242,119,95,230,153,31,254,136,59,62,112,23,137,68,130,176,22,98,
  251,182,109,184,142,75,169,84,12,252,8,135,35,164,82,25,20,197,59,142,15,135,67,104,170,140,44,47,236,61,101,89,244,65,245,130,31,162,40,120,
  199,192,13,252,144,36,17,73,146,113,93,23,73,146,136,199,226,36,19,73,116,93,39,155,205,82,42,22,249,238,43,175,144,201,100,2,221,198,70,126,
  0,252,201,31,254,33,207,252,232,71,220,113,215,93,76,77,77,209,211,213,229,137,231,167,210,222,145,176,227,162,200,42,225,72,148,185,185,36,138,226,
  131,151,144,74,72,83,124,224,134,239,171,74,72,11,97,154,38,149,114,25,203,178,144,36,209,3,112,138,130,44,203,254,181,75,126,246,61,20,11,121,
  92,199,68,211,52,134,135,134,201,102,115,88,166,73,58,147,102,114,114,146,23,158,187,64,54,147,105,90,19,29,64,215,117,254,232,247,255,128,145,213,
  171,217,184,105,19,165,82,137,213,171,87,209,22,141,82,40,20,48,124,121,165,72,52,74,165,162,147,78,101,209,66,154,15,136,53,52,77,70,22,5,
  28,215,171,6,164,40,50,182,109,99,24,70,32,142,239,181,129,226,11,202,123,101,53,61,166,26,42,149,50,174,227,6,146,91,237,109,237,20,139,94,
  188,101,169,84,226,185,231,158,99,118,102,166,97,181,171,69,255,11,2,199,142,30,229,216,177,99,220,117,247,221,36,147,9,34,90,136,45,91,183,144,
  156,79,80,44,150,0,80,100,153,72,56,66,98,62,133,162,202,132,66,97,34,145,48,145,176,134,170,202,184,162,235,55,181,131,170,40,152,166,73,169,
  88,194,182,237,5,16,234,255,84,219,163,170,185,42,144,14,250,213,208,224,48,201,68,138,138,161,147,201,102,72,167,211,124,239,59,223,89,196,84,46,
  98,71,171,204,117,237,14,197,103,74,3,198,180,118,158,169,198,204,54,99,71,107,250,169,227,186,193,198,165,101,45,107,217,251,8,140,2,252,242,175,
  254,234,162,191,127,183,70,238,233,215,154,232,144,182,236,167,195,66,225,48,27,55,110,164,171,171,43,248,189,250,248,245,182,83,39,79,178,99,231,78,
  18,201,36,107,214,172,102,205,218,53,20,139,69,175,46,184,191,145,82,21,141,100,98,222,63,158,86,80,53,13,77,83,8,133,52,162,145,48,33,77,
  1,1,34,145,48,182,99,83,174,84,124,141,78,149,80,72,243,23,90,9,73,244,164,147,188,227,99,141,98,161,224,39,143,120,89,213,93,93,93,100,
  243,126,125,245,82,137,185,185,249,69,71,130,139,18,47,106,64,157,32,8,204,207,205,209,209,209,65,54,155,101,100,100,152,222,222,94,74,229,18,182,
  189,160,45,42,137,50,185,76,230,178,126,132,67,26,182,99,163,27,6,178,44,47,239,135,162,80,80,180,106,141,35,194,145,48,157,177,78,82,217,12,
  165,114,153,114,185,76,58,157,94,12,160,27,248,81,61,86,61,113,236,24,55,238,217,67,54,155,101,199,142,29,116,116,116,44,210,72,245,190,87,38,
  147,201,248,192,197,139,15,13,133,52,194,225,16,109,209,16,138,236,201,8,181,181,69,168,232,30,160,86,21,207,95,85,245,216,80,89,146,252,35,122,
  193,247,73,38,233,139,173,187,184,180,181,183,17,143,199,152,75,36,40,149,203,8,130,64,58,147,89,164,104,176,200,143,106,101,31,191,61,202,165,18,
  229,114,25,219,177,25,28,28,196,180,44,138,197,18,142,109,251,181,217,21,138,126,44,102,21,232,171,106,136,72,52,76,71,123,27,237,237,17,4,217,
  147,55,2,2,93,210,72,36,130,230,251,225,109,106,100,95,142,203,139,129,181,45,139,228,252,156,23,255,90,147,85,95,40,20,40,149,75,65,193,133,
  42,192,170,7,114,212,180,69,21,176,217,150,69,177,88,164,127,112,128,104,36,74,58,147,193,113,28,47,6,20,112,92,200,231,179,72,146,76,81,41,
  145,203,41,104,154,70,36,18,166,187,171,19,85,85,16,69,8,105,42,229,114,9,221,240,142,237,181,80,200,111,15,5,89,242,64,181,32,122,126,40,
  178,66,82,150,113,117,175,61,194,145,48,61,189,61,140,79,78,82,174,84,22,133,92,52,242,165,190,95,5,213,151,170,128,180,14,128,95,110,195,44,
  52,32,88,90,162,248,45,107,217,251,12,140,126,239,187,223,109,221,209,150,53,163,205,25,30,30,94,180,89,169,253,125,145,224,248,53,182,219,110,191,
  157,158,222,30,134,87,173,66,20,5,6,7,7,193,117,41,22,139,193,130,227,186,14,166,31,59,40,138,2,130,32,34,248,165,61,171,64,45,28,246,
  88,173,112,56,140,105,26,116,118,118,18,14,133,144,107,24,56,81,240,128,143,32,46,196,172,41,170,39,145,228,177,63,50,237,237,237,128,87,162,82,
  215,117,214,173,91,135,32,8,92,56,127,126,17,219,83,101,23,171,217,193,119,222,117,39,189,125,125,116,247,244,190,231,252,48,12,131,181,235,214,33,
  138,162,167,111,89,5,208,13,252,248,228,99,143,121,49,130,93,93,136,162,192,218,181,107,41,22,10,62,11,87,149,204,114,49,45,139,66,62,239,93,
  131,40,46,148,51,149,101,52,173,122,204,27,166,171,59,78,161,80,32,22,143,19,210,52,100,95,187,83,146,36,255,30,44,252,184,170,227,199,160,122,
  167,56,138,162,16,139,117,50,51,55,231,197,220,250,242,82,162,32,112,190,81,123,248,128,116,211,230,205,108,221,182,13,81,20,153,153,157,37,26,142,
  32,137,34,197,98,209,139,173,245,253,183,109,155,114,185,236,93,139,84,241,227,87,37,178,89,137,121,159,241,141,68,194,244,116,119,249,140,161,72,255,
  64,63,33,45,132,40,73,72,162,215,126,162,175,167,90,101,4,13,67,15,142,167,5,65,36,28,14,163,105,42,217,156,131,105,152,244,247,245,209,215,
  223,199,204,244,52,83,147,83,139,24,195,122,209,248,15,127,228,35,72,178,132,227,226,223,143,152,215,55,43,21,4,65,192,193,75,188,50,12,35,96,
  58,5,191,143,84,25,206,185,185,57,66,161,16,209,104,132,129,129,62,50,153,12,253,253,125,68,194,17,36,89,94,228,71,85,27,182,186,111,17,253,
  68,58,28,79,202,171,163,163,3,38,39,131,48,152,61,123,247,34,138,34,111,188,254,250,18,80,234,54,200,158,167,89,140,232,74,217,209,22,24,109,
  89,203,222,231,96,212,63,78,9,135,195,12,249,217,210,83,147,147,173,163,250,150,241,218,171,175,114,174,86,247,176,206,210,169,212,117,251,238,91,111,
  191,157,173,219,182,50,62,54,142,128,64,46,151,35,155,205,97,26,38,174,227,224,248,192,164,202,140,85,23,161,218,5,169,90,239,220,241,229,106,54,
  108,216,192,198,13,80,209,13,76,195,244,100,120,92,103,241,123,253,184,54,89,145,23,101,254,58,246,2,27,107,59,14,107,215,174,5,129,5,93,200,
  154,100,25,106,152,159,59,238,186,139,237,59,118,48,62,54,14,46,36,18,9,186,186,186,22,178,149,125,63,10,197,130,15,162,88,136,235,243,1,133,
  99,59,152,150,137,94,209,41,20,11,220,127,255,189,200,146,68,190,80,162,82,174,96,152,134,151,81,207,226,216,85,199,113,189,24,81,103,33,7,217,
  50,205,37,126,8,130,192,249,115,231,22,142,78,107,64,105,21,4,124,226,177,79,2,112,226,196,73,4,4,146,201,20,209,104,20,199,182,3,73,39,
  219,177,27,183,135,75,77,188,160,139,109,89,152,150,197,253,247,223,67,46,155,71,55,76,44,63,137,169,81,123,56,206,194,17,110,245,241,170,134,166,
  235,122,242,94,94,123,8,11,253,181,190,61,240,142,112,61,22,215,251,95,175,84,72,165,211,132,180,144,255,29,158,52,86,69,215,17,165,146,199,6,
  10,222,230,96,113,191,242,116,106,11,165,2,219,182,110,225,225,135,63,72,161,80,164,82,241,146,120,108,203,243,163,62,94,178,82,46,123,34,244,30,
  77,139,105,26,65,229,37,199,247,3,199,93,164,142,208,12,144,218,142,3,182,224,223,79,155,153,233,25,66,90,200,107,101,199,1,1,44,211,162,80,
  44,32,75,146,127,63,4,4,132,128,97,118,124,57,45,211,52,49,76,131,199,30,123,20,215,117,40,149,117,12,95,206,204,50,45,108,199,94,80,20,
  240,37,197,68,81,192,245,251,149,109,219,100,51,233,0,116,59,142,131,32,138,11,12,41,11,128,220,109,0,72,169,5,143,85,80,90,251,186,122,134,
  120,37,140,105,11,144,182,172,101,239,47,48,10,240,145,143,125,140,59,238,188,115,209,99,207,62,243,12,143,127,253,235,173,187,253,83,108,67,195,195,
  132,195,97,142,29,61,250,142,111,78,254,195,191,251,119,0,124,246,243,159,199,180,44,38,39,167,232,233,237,37,172,133,2,32,138,235,82,42,149,144,
  36,49,16,175,119,108,7,219,241,244,30,77,211,3,112,186,97,16,10,133,216,178,121,19,235,214,173,246,52,33,45,155,116,182,16,104,111,90,182,159,
  209,237,216,126,38,178,88,3,124,76,82,233,12,224,215,41,7,126,240,131,31,4,139,162,91,39,176,94,11,128,254,159,255,239,255,141,0,124,233,231,
  127,30,211,52,73,204,207,211,223,223,79,91,180,109,161,196,162,235,82,42,149,23,249,81,213,172,244,178,205,77,202,165,50,134,225,49,162,133,124,129,
  29,59,182,248,32,217,37,145,202,81,42,150,48,12,15,220,120,85,128,188,88,198,218,101,220,48,12,230,19,9,111,18,145,101,4,224,233,167,159,110,
  232,71,173,30,164,43,8,124,250,209,79,34,0,159,249,220,231,48,12,131,147,39,78,16,239,138,35,203,50,186,105,120,140,162,101,81,40,20,60,214,
  204,245,4,211,189,2,3,86,32,125,164,87,116,42,186,78,95,111,47,178,36,179,107,215,54,112,193,48,44,50,185,34,133,66,17,195,48,124,201,42,
  239,30,84,42,229,0,200,0,84,42,21,38,39,166,130,99,124,199,113,86,212,30,103,207,156,225,220,217,179,116,119,119,179,126,227,70,74,122,133,68,
  34,65,127,95,191,247,62,199,5,1,42,229,178,199,78,87,65,168,235,226,184,120,50,84,182,141,101,122,254,132,195,17,44,203,70,20,97,213,170,65,
  52,85,163,92,49,41,149,202,84,116,131,74,165,130,109,217,190,239,6,169,84,50,0,162,186,174,147,72,36,201,229,10,126,40,130,200,244,212,20,211,
  211,211,139,252,168,15,249,168,94,207,63,124,243,155,8,192,161,91,111,197,197,37,147,201,144,203,229,232,232,236,196,178,44,4,192,241,239,157,44,43,
  193,230,198,117,241,197,236,109,12,195,10,146,249,122,123,122,73,167,210,172,93,187,138,190,94,21,211,178,41,20,43,84,202,21,202,21,221,75,208,242,
  245,121,243,249,92,224,7,174,23,166,48,159,72,6,227,3,224,205,55,222,88,162,202,208,16,144,214,130,71,191,189,22,29,189,215,198,47,215,254,95,
  11,60,27,129,211,101,152,211,150,181,172,101,239,34,24,109,150,108,114,255,131,15,114,199,157,119,242,236,51,207,112,236,232,81,0,118,238,218,197,29,
  119,222,73,185,92,230,201,214,49,254,79,173,197,187,186,248,232,199,62,6,192,209,163,71,189,164,137,163,71,175,171,164,83,189,77,79,79,211,214,214,
  198,220,236,44,19,99,227,172,93,183,118,145,102,100,62,159,243,179,159,125,240,102,219,216,166,199,188,153,166,137,109,89,72,178,68,56,20,162,88,42,
  163,105,106,240,217,109,237,17,108,219,165,82,49,41,149,245,32,153,38,149,74,98,89,62,99,229,184,100,210,105,50,217,44,2,16,14,133,22,22,59,
  150,202,7,81,207,44,250,127,79,77,78,210,222,209,193,252,252,60,211,83,211,75,252,40,22,11,43,242,67,211,52,42,186,17,248,32,74,2,189,61,
  29,216,241,246,203,250,145,203,102,41,150,74,8,120,117,210,27,250,81,11,230,26,248,33,201,18,237,29,29,204,205,206,50,62,54,206,170,213,171,64,
  215,125,134,204,162,88,44,160,40,170,47,254,110,99,57,14,150,97,98,154,22,166,229,49,184,170,162,16,10,135,200,23,74,1,205,165,106,50,189,61,
  29,196,58,219,40,87,12,202,101,29,67,215,49,77,147,98,209,147,64,242,192,183,67,50,145,164,98,232,136,146,132,170,40,94,89,214,70,237,81,3,
  94,22,85,200,194,171,106,165,169,26,153,76,134,182,182,54,218,162,109,1,107,90,169,148,17,68,201,43,97,106,59,62,171,234,109,96,188,178,165,14,
  162,32,162,42,10,182,237,162,235,102,80,18,52,26,213,8,135,85,44,203,161,92,246,66,58,12,211,36,151,205,161,235,21,170,33,13,249,124,158,92,
  46,143,227,216,168,170,134,40,138,148,138,197,69,126,52,140,169,172,3,113,94,114,85,132,84,50,69,34,153,32,22,139,5,159,97,219,22,165,82,9,
  77,11,5,97,20,142,235,49,237,213,234,93,184,46,178,170,160,106,42,5,63,121,201,113,28,20,69,34,214,25,193,105,15,83,169,152,84,116,3,67,
  55,253,36,190,84,208,30,182,99,51,55,55,135,97,154,8,128,234,183,253,34,150,147,165,194,247,110,93,191,18,26,0,201,229,98,71,151,171,202,212,
  98,71,91,214,178,107,131,15,175,27,51,218,232,11,111,218,191,159,239,125,247,187,139,98,71,207,157,59,71,185,92,14,158,107,217,79,167,85,55,40,
  59,119,237,98,255,254,253,124,250,51,159,129,207,124,38,200,228,189,158,140,233,214,109,91,249,200,199,62,70,52,26,101,122,110,142,99,199,142,50,62,
  49,206,192,224,0,154,22,242,234,102,87,51,210,67,11,37,12,23,68,212,61,205,71,65,16,80,21,13,69,145,49,13,3,219,118,16,107,50,237,37,
  73,32,26,85,137,68,84,42,122,24,93,55,201,100,146,232,186,39,6,111,154,6,211,51,51,232,186,14,64,56,20,70,150,36,118,239,222,77,34,153,
  100,114,98,98,49,120,240,23,211,106,34,198,254,155,247,243,208,195,15,51,48,56,200,220,95,254,213,123,206,143,189,251,246,145,76,38,25,189,116,105,
  89,63,30,122,248,97,246,223,188,159,206,120,156,227,39,79,241,114,50,193,232,232,69,86,173,26,65,16,69,79,211,212,180,41,149,138,104,90,77,137,
  79,199,147,11,114,108,79,122,75,20,5,84,45,132,44,73,65,233,205,90,192,160,40,34,138,18,162,45,170,121,146,68,101,157,209,209,243,126,157,116,
  23,221,208,25,245,53,81,37,81,68,211,52,186,186,226,116,180,237,38,153,76,50,49,49,177,40,246,208,173,97,120,163,109,109,220,245,129,15,208,211,
  211,205,232,216,56,225,80,136,84,58,69,172,51,70,36,28,241,65,146,131,174,87,16,37,175,220,167,227,248,18,72,174,29,72,35,9,130,136,168,72,
  200,138,130,227,216,232,186,215,30,2,86,80,90,179,234,135,101,105,24,166,133,169,87,40,20,114,30,104,181,44,210,233,52,133,66,1,0,85,81,144,
  101,137,157,187,118,97,89,22,199,143,29,163,80,40,44,2,164,212,1,179,109,219,183,115,232,150,91,40,149,74,180,183,181,97,217,54,169,84,10,99,
  181,225,95,183,131,229,51,163,85,217,37,207,23,215,63,1,240,54,79,146,232,129,106,73,146,252,83,4,19,81,18,81,69,111,211,38,138,2,145,136,
  74,40,164,96,90,14,166,97,113,250,84,9,203,242,0,184,94,169,48,61,51,19,180,95,56,20,66,211,84,238,184,243,78,102,103,103,131,202,88,75,
  124,169,103,64,151,99,71,151,93,212,22,179,224,245,108,105,75,236,169,101,45,187,126,118,77,117,43,186,186,186,26,198,5,158,59,119,142,174,154,106,
  39,45,251,233,180,84,42,197,179,207,60,195,239,252,199,255,200,191,254,151,255,146,191,253,155,191,1,224,211,159,249,204,146,208,142,107,105,61,61,61,
  244,244,244,80,40,20,24,232,239,71,211,66,148,202,101,230,230,230,60,121,35,63,30,212,208,43,232,186,129,97,232,222,143,110,96,26,126,53,31,64,
  10,234,111,123,50,60,233,76,177,225,247,9,2,132,67,50,177,142,48,150,169,147,207,231,113,93,143,77,44,149,74,65,198,120,71,91,27,150,233,45,
  196,177,206,206,165,155,189,90,205,68,215,37,26,137,2,112,236,232,49,214,172,246,132,212,243,249,60,99,227,99,132,195,161,192,15,189,82,166,92,174,
  80,169,84,208,245,10,122,69,199,52,204,192,15,89,81,208,66,42,146,172,96,219,54,137,100,110,89,63,162,17,153,66,33,75,198,207,174,78,38,18,
  190,4,147,167,231,217,29,143,83,46,149,176,76,115,129,37,245,175,187,161,31,81,79,19,115,236,210,37,214,172,94,141,32,8,164,210,25,46,92,188,
  16,180,135,237,88,232,149,114,224,131,225,251,97,153,86,16,247,41,43,10,161,144,138,36,203,152,166,65,38,91,106,60,201,137,2,225,144,76,56,44,
  51,63,63,71,165,236,1,215,153,169,105,116,195,3,212,138,172,16,107,111,167,82,174,4,50,67,139,216,222,58,63,138,133,2,122,165,194,225,183,15,
  99,91,22,29,157,157,152,166,69,54,155,165,82,77,252,169,22,20,48,188,184,98,211,50,177,76,3,211,180,130,62,32,138,94,70,185,44,75,184,174,
  75,185,172,83,169,24,62,27,105,47,102,14,100,129,72,88,193,182,189,130,7,30,160,54,200,231,243,158,46,169,207,82,171,178,194,248,216,24,229,114,
  57,0,169,245,90,175,110,77,63,211,117,157,99,199,142,114,246,236,89,122,122,122,208,52,141,82,169,204,228,228,36,170,230,105,209,58,142,141,169,235,
  232,149,10,134,110,4,73,107,150,31,211,42,8,94,191,82,85,21,81,146,48,45,147,84,58,183,56,102,181,166,61,52,85,66,20,29,102,102,167,189,
  16,4,219,102,126,126,62,136,123,21,4,129,88,71,7,217,76,150,209,75,151,200,164,211,75,251,85,45,136,172,211,30,109,10,50,127,28,253,208,150,
  246,104,203,90,118,221,236,154,198,140,150,202,101,54,108,220,184,144,136,225,219,134,141,27,73,93,199,4,149,150,189,191,44,28,14,179,97,227,70,54,
  108,216,192,250,141,27,113,253,190,115,189,236,249,231,158,231,249,231,158,7,224,231,126,225,23,136,119,197,185,112,254,60,23,47,94,100,199,142,29,148,
  75,37,47,177,199,208,113,93,193,151,67,82,169,216,21,170,201,58,130,40,162,248,218,149,146,40,97,59,14,185,108,158,142,142,48,138,36,54,88,183,
  92,198,198,199,57,125,250,20,197,98,1,211,52,153,157,157,165,226,179,137,178,44,211,211,211,205,204,204,12,9,63,246,114,209,162,87,155,184,227,51,
  65,63,250,225,15,121,230,71,63,66,0,126,246,159,252,19,58,58,59,73,36,18,76,79,77,179,111,239,62,4,81,192,177,29,44,211,0,60,89,39,
  77,211,168,56,149,224,216,88,16,69,100,201,211,17,149,68,17,219,113,40,22,74,152,241,182,166,126,76,77,77,49,122,233,18,149,74,245,200,62,21,
  196,144,202,146,68,103,103,7,201,100,114,169,31,117,159,83,245,227,239,255,238,203,8,130,64,87,87,23,63,243,133,47,16,10,135,189,146,154,227,19,
  108,88,191,17,4,191,182,185,105,0,158,116,150,172,170,30,107,232,216,94,121,76,105,193,15,65,16,176,109,135,114,169,76,123,91,24,89,18,26,126,
  255,217,179,103,152,157,153,198,48,61,49,246,217,185,217,224,121,85,83,137,182,181,113,241,194,5,74,165,82,243,246,168,2,117,224,169,167,158,10,42,
  74,245,245,246,112,246,220,89,50,217,12,221,221,93,68,163,81,240,171,15,25,122,25,81,86,232,237,237,163,82,46,83,174,84,112,108,23,65,244,245,
  105,21,47,243,223,117,93,42,149,10,229,114,133,112,88,3,203,242,18,134,106,204,52,77,94,126,229,101,10,133,28,182,109,83,40,20,200,251,128,83,
  146,36,162,209,40,165,82,137,177,177,49,198,106,43,97,85,65,92,125,214,57,94,226,220,133,243,231,1,184,239,193,7,105,107,139,146,203,101,201,164,
  211,8,2,62,91,237,96,99,97,154,58,154,22,166,61,218,70,169,92,198,48,44,112,93,68,209,215,163,13,218,195,166,92,170,96,90,54,178,101,33,
  215,85,3,179,109,155,75,23,47,48,59,59,19,196,50,103,50,217,5,86,91,85,9,135,195,140,93,186,180,180,61,154,244,171,250,246,170,101,228,235,
  171,50,213,86,160,106,197,131,182,172,101,63,97,96,244,185,103,159,229,254,7,30,32,28,10,113,236,216,49,192,171,55,125,251,157,119,242,228,247,190,
  215,186,219,63,229,0,116,231,174,93,236,216,185,147,157,187,118,1,158,210,194,147,223,251,30,199,143,30,125,71,54,43,61,61,61,216,150,197,200,208,
  48,147,19,19,148,203,101,50,153,140,199,4,21,139,184,184,8,162,203,240,200,16,195,195,131,140,141,142,51,61,51,139,105,90,158,84,144,226,233,93,
  86,227,212,12,67,71,215,45,148,136,186,228,187,242,249,60,199,142,29,227,226,165,75,30,227,85,42,81,244,25,57,240,244,70,85,69,13,254,110,196,
  194,52,2,14,0,189,125,125,196,58,59,223,83,126,52,141,167,91,198,143,142,142,14,58,218,219,25,25,30,230,236,217,179,228,243,121,202,165,34,138,
  162,160,87,42,184,184,132,66,10,235,214,173,167,173,45,202,212,212,52,19,147,83,184,142,187,216,15,4,28,215,65,55,12,76,211,70,150,228,134,126,
  188,250,234,171,164,51,105,192,243,163,84,169,4,207,183,69,162,40,178,114,121,63,234,146,95,162,209,40,237,237,237,72,178,236,151,1,45,146,205,122,
  201,63,162,127,127,109,219,98,120,213,42,182,110,217,130,174,235,156,56,117,154,124,54,95,83,24,65,66,244,19,156,108,219,66,215,77,175,10,149,15,
  2,107,133,215,79,157,58,197,133,11,23,113,93,175,252,105,62,159,11,24,212,112,40,68,212,215,39,213,52,45,8,163,184,28,136,115,241,42,74,173,
  93,187,134,144,166,209,221,213,205,252,124,130,98,177,72,50,145,36,22,139,81,46,151,113,92,7,85,149,217,180,121,35,221,93,93,76,207,204,48,62,
  62,133,97,24,72,178,136,36,251,82,84,254,231,27,134,129,110,88,168,138,76,125,101,218,68,50,201,241,19,39,40,20,138,128,75,161,80,160,172,47,
  108,72,187,58,59,81,20,133,206,206,78,159,129,181,86,220,175,22,21,89,168,23,180,111,38,225,196,82,93,220,102,27,146,150,181,172,101,239,25,48,
  218,248,184,226,201,239,126,151,112,40,196,237,119,222,201,237,53,199,174,207,61,243,76,43,121,233,167,220,110,191,227,14,238,127,240,65,166,38,39,121,
  252,235,95,231,248,177,99,239,40,91,190,119,239,94,126,245,127,251,103,100,50,89,70,199,198,56,121,234,36,243,115,115,156,59,119,142,155,15,28,160,
  84,44,34,43,10,235,215,173,229,214,219,110,39,18,246,42,204,228,243,5,178,185,124,0,26,36,95,162,199,99,169,12,74,165,10,145,176,226,233,64,
  214,176,87,39,78,158,228,252,249,11,65,124,98,42,189,192,38,10,130,192,64,111,47,33,77,99,203,182,173,204,76,77,115,241,226,197,203,142,182,170,
  188,211,47,255,211,95,33,155,205,113,226,228,201,247,134,31,33,141,141,155,55,209,25,235,228,220,217,115,43,242,227,131,15,63,204,63,250,210,23,73,
  36,146,236,216,190,157,139,23,47,146,201,100,152,153,153,97,253,198,141,84,202,21,20,85,225,134,27,118,178,115,231,46,175,6,123,103,59,243,243,9,
  12,31,84,203,53,58,162,224,98,26,6,149,138,142,166,73,139,252,40,149,74,156,56,121,146,68,34,25,200,10,165,210,11,125,79,20,69,186,187,226,
  104,154,198,13,55,238,230,210,133,139,65,38,250,242,211,160,203,131,15,62,72,36,26,97,102,102,150,88,44,78,190,80,32,155,203,50,96,12,16,14,
  135,176,44,139,145,145,97,118,108,223,74,111,111,31,162,40,50,59,55,79,33,95,244,52,71,253,31,65,244,165,149,108,27,67,247,142,191,101,73,242,
  165,143,60,48,122,230,204,25,94,120,241,165,133,152,87,93,39,233,143,33,81,20,105,111,107,163,45,26,33,222,21,167,171,187,139,215,94,126,5,171,
  238,168,127,9,136,243,109,253,134,13,28,58,116,144,108,54,71,95,95,31,19,147,19,36,19,73,230,230,231,24,30,30,166,92,46,17,10,69,216,188,
  101,11,91,183,108,38,164,105,8,162,64,50,153,246,170,94,137,94,159,170,74,88,121,96,217,68,175,24,68,66,234,34,80,157,76,38,121,251,237,183,
  25,29,29,243,229,169,92,82,169,228,2,219,46,203,190,66,130,196,218,245,235,136,70,163,203,202,194,53,77,74,186,204,107,46,215,182,13,193,103,11,
  148,182,172,101,87,133,15,175,15,24,117,105,26,63,243,248,215,191,206,147,223,253,110,75,103,180,101,139,236,216,209,163,188,246,234,171,215,85,79,116,
  57,59,117,242,36,207,63,247,28,223,251,238,247,216,183,127,63,35,195,35,100,179,89,210,233,52,109,209,40,179,174,23,78,114,219,109,183,211,22,141,
  4,178,72,146,44,5,128,65,242,65,3,2,65,242,70,165,92,65,55,34,132,84,41,96,131,198,198,198,120,237,181,215,72,166,146,190,182,165,67,38,
  147,13,226,225,34,145,8,241,88,140,139,23,46,208,214,222,70,169,80,92,126,145,171,121,238,245,215,94,227,219,79,60,193,119,158,248,54,15,62,252,
  48,91,183,108,161,88,40,50,59,59,75,36,20,66,16,4,110,62,112,128,27,118,237,34,164,105,190,232,122,5,219,175,247,94,101,227,196,58,63,202,
  165,50,149,104,152,112,72,14,50,180,79,158,60,201,11,47,190,72,46,159,15,4,220,211,233,140,231,135,32,16,235,236,164,59,222,197,197,243,23,80,
  20,5,219,178,86,236,199,137,227,199,249,202,151,255,158,87,94,126,153,187,239,187,143,205,155,206,114,234,244,41,178,217,28,33,85,165,173,189,141,221,
  55,222,200,141,187,111,68,81,100,116,195,164,92,174,120,137,50,146,136,44,139,190,31,34,8,94,134,191,105,154,193,17,183,166,74,216,182,67,50,153,
  224,237,183,223,230,216,241,227,148,202,37,112,61,125,212,180,47,175,37,8,2,189,221,61,116,182,119,112,230,212,41,58,99,157,228,178,217,21,77,181,
  2,240,198,155,111,146,152,159,167,173,189,157,254,190,62,210,233,52,134,174,99,152,6,145,104,132,225,225,17,118,236,216,65,60,30,71,16,160,92,169,
  80,44,20,17,5,193,47,239,41,46,36,143,249,82,73,166,101,162,235,38,138,34,35,248,88,242,252,249,243,60,251,220,115,204,204,204,248,9,68,158,
  182,105,169,88,2,65,32,18,14,19,235,140,145,203,100,153,24,27,163,163,195,147,101,186,44,112,242,219,228,194,185,115,148,138,69,198,199,198,216,183,
  127,63,107,86,175,33,149,74,147,76,38,17,68,232,236,140,177,103,239,94,134,135,135,208,84,5,195,191,135,165,82,41,240,35,232,87,0,142,151,92,
  165,87,42,24,70,8,213,215,218,189,116,105,148,215,94,127,141,11,23,46,162,27,186,151,16,167,235,228,243,5,44,203,70,16,69,134,6,6,105,143,
  182,113,228,237,183,137,182,181,161,87,42,151,237,87,139,142,224,175,6,120,214,178,170,151,89,94,133,86,236,104,203,90,118,173,177,232,181,61,166,191,
  255,193,7,151,60,182,193,47,249,88,101,78,91,246,211,105,213,66,8,175,191,250,42,241,174,46,194,225,48,83,147,147,0,124,248,163,31,101,106,106,
  138,215,95,125,245,186,125,127,169,84,226,47,254,252,191,224,250,85,119,14,220,124,51,217,108,150,177,177,81,78,159,62,205,64,127,63,119,220,126,59,
  209,72,4,219,113,152,157,157,227,173,55,223,36,153,76,163,40,154,87,167,93,150,3,230,7,215,23,192,55,77,42,21,3,73,144,153,158,158,230,216,
  177,227,156,191,112,222,99,172,252,132,138,82,169,20,176,62,170,170,50,50,56,136,101,152,158,204,83,109,98,198,50,139,108,213,138,197,34,127,253,95,
  255,27,184,46,207,60,253,52,251,15,30,100,114,114,138,177,177,81,206,158,59,199,64,127,63,59,182,111,247,128,168,227,48,59,55,207,201,19,39,200,
  231,178,40,138,22,128,6,161,206,15,203,178,208,13,19,89,114,23,249,81,5,162,94,114,77,57,0,212,170,162,208,215,211,67,185,84,90,222,135,38,
  126,92,186,116,137,209,209,81,112,93,46,156,59,199,222,189,123,153,158,153,102,118,110,142,217,185,57,54,109,218,196,142,237,219,189,140,127,211,228,226,
  197,11,156,60,113,2,67,175,160,168,161,32,70,177,90,89,169,10,150,77,203,194,48,45,68,193,225,236,217,115,156,56,121,130,209,209,81,47,174,210,
  93,144,66,170,250,17,210,52,134,6,6,72,204,207,147,207,229,200,231,114,151,5,38,181,192,232,226,133,11,8,130,64,62,159,231,192,193,131,172,26,
  30,230,194,197,139,76,77,77,209,219,211,203,166,77,155,136,197,98,136,130,72,62,95,224,153,103,159,33,151,201,161,134,34,11,113,150,190,31,224,137,
  204,219,150,69,69,55,137,132,53,230,83,41,222,122,251,45,206,156,57,75,190,176,208,22,158,214,104,202,59,98,87,20,122,187,123,136,117,116,112,254,
  236,89,47,25,168,50,119,69,96,204,48,77,198,199,198,112,125,2,97,253,134,13,92,184,120,145,108,54,203,153,51,103,217,127,211,77,244,244,244,32,
  73,18,150,109,115,250,244,105,78,157,58,133,105,218,168,90,181,61,188,42,75,30,22,117,125,96,109,99,152,38,165,178,203,232,232,37,94,120,225,69,
  82,233,20,186,97,4,137,71,153,76,6,195,111,143,104,56,204,64,95,47,153,116,26,219,182,151,223,24,52,97,121,91,214,178,150,189,255,236,154,130,
  209,251,30,120,96,217,231,91,96,244,167,215,54,108,220,200,250,13,27,120,253,213,87,217,127,243,205,172,223,176,129,63,253,163,63,2,60,65,252,74,
  77,252,222,245,6,197,187,118,237,98,245,186,181,108,159,220,206,212,212,20,231,206,157,99,231,206,157,148,74,37,20,89,33,145,76,242,242,203,47,51,
  57,57,142,40,74,56,142,141,36,9,88,138,140,162,40,254,58,232,3,130,116,146,108,54,73,161,144,99,116,244,18,151,46,141,98,217,214,162,236,222,
  92,46,135,110,24,72,146,68,172,163,147,206,246,14,194,225,16,145,104,212,211,131,188,26,63,34,17,30,124,232,131,196,226,93,108,159,156,124,215,252,
  104,107,107,163,163,163,195,211,231,188,10,235,234,238,102,239,190,189,184,130,192,232,232,40,71,143,29,101,110,110,142,61,123,246,248,69,8,36,46,93,
  186,196,171,175,190,198,236,236,12,146,40,227,184,158,118,165,101,137,184,238,130,238,101,185,92,194,48,116,230,231,166,41,149,10,156,60,121,130,217,217,
  57,63,233,105,193,143,68,50,137,110,24,94,18,89,188,139,142,182,54,236,72,152,76,58,189,108,156,101,51,118,78,81,20,118,238,220,201,170,85,35,
  56,174,75,190,88,36,159,207,34,203,18,197,82,145,74,165,130,105,153,60,243,204,51,36,18,9,47,33,168,236,224,58,54,138,220,142,97,136,104,161,
  16,174,227,149,114,205,100,210,36,18,179,20,138,121,18,137,4,137,68,194,203,250,175,241,161,92,46,147,72,165,16,69,145,142,246,118,98,29,94,91,
  140,172,90,197,196,196,132,199,82,95,133,245,247,247,115,240,208,33,0,54,109,220,200,177,19,199,73,167,82,72,178,76,54,155,69,18,69,146,169,20,
  199,142,29,163,92,46,33,138,18,182,109,161,40,18,134,1,146,36,67,24,92,215,161,162,151,25,159,184,196,249,243,101,50,233,12,137,100,130,116,58,
  237,85,198,114,221,128,225,77,36,189,35,122,69,81,232,239,233,37,18,142,16,235,140,161,235,250,187,118,146,114,57,226,167,5,128,91,214,178,247,48,
  24,253,141,127,241,47,130,223,127,241,87,126,133,11,231,206,181,18,151,90,182,176,144,55,250,125,185,199,174,131,221,113,231,157,220,113,215,157,188,249,
  250,27,108,220,176,129,177,177,81,166,167,167,121,229,149,87,40,151,203,244,247,245,51,55,63,199,217,179,103,169,150,122,180,44,19,219,54,41,87,74,
  40,138,138,170,42,65,178,139,101,89,24,70,133,76,38,67,165,82,89,84,166,178,92,46,147,207,231,153,79,36,112,93,232,236,104,167,175,187,27,211,
  48,136,197,58,217,188,101,51,111,191,249,214,85,249,241,200,135,30,97,247,141,55,242,228,119,191,247,174,249,81,44,20,136,197,58,89,189,118,45,199,
  142,28,185,42,63,62,249,216,99,108,218,188,153,239,60,241,109,118,237,220,73,50,153,96,102,102,134,151,94,122,153,68,34,65,119,119,15,163,163,163,
  76,79,79,121,140,155,99,96,90,6,174,99,33,136,18,197,66,1,85,149,145,101,37,136,83,172,84,74,228,243,5,42,149,114,80,18,213,182,45,242,
  249,60,133,66,129,116,58,141,235,66,87,60,70,95,111,47,249,124,158,254,193,1,86,175,93,203,217,211,167,175,216,7,69,81,88,183,110,29,243,115,
  243,228,179,89,214,175,91,199,185,115,231,120,227,141,55,153,157,157,99,245,234,213,8,162,192,252,252,188,7,102,29,23,203,245,74,200,26,70,5,65,
  16,73,38,230,16,37,47,214,181,26,59,106,154,6,134,97,130,235,4,26,167,122,165,194,212,244,52,153,108,22,203,178,8,135,195,244,117,247,96,26,
  6,249,92,142,254,193,1,102,166,167,175,26,140,246,245,245,161,170,10,175,189,250,26,189,189,189,108,88,191,129,209,209,75,60,253,244,15,89,191,126,
  61,61,221,221,20,138,5,138,197,162,23,230,225,23,36,200,102,82,158,102,170,36,49,59,43,7,241,161,142,237,96,217,22,134,97,224,216,118,80,102,
  214,178,44,114,217,44,179,115,115,94,17,8,65,36,30,139,209,21,143,51,53,49,193,200,154,213,196,187,226,239,73,48,218,178,150,181,236,58,128,81,
  247,10,64,128,91,163,185,183,146,119,173,244,117,45,251,41,192,162,13,250,132,251,46,244,149,103,126,244,35,38,38,198,57,122,228,40,61,61,61,28,
  188,245,22,158,125,238,121,94,123,237,117,38,39,167,136,132,195,216,142,189,144,37,45,73,224,66,197,46,227,6,177,207,126,173,250,26,77,67,199,175,
  179,173,235,21,138,69,175,20,101,177,84,164,80,40,96,89,22,237,109,237,244,247,246,225,56,54,99,163,147,204,206,204,32,213,167,24,95,46,3,184,
  102,252,125,235,155,255,192,219,111,189,205,217,179,103,233,239,239,231,3,247,222,203,43,175,189,198,27,111,190,201,233,83,167,17,37,17,81,16,136,119,
  117,209,211,211,19,72,235,216,129,31,158,0,59,213,58,230,254,39,59,142,131,105,90,148,74,69,210,233,140,207,134,234,65,152,65,119,60,206,208,192,
  0,182,101,49,51,61,205,252,252,60,90,173,182,232,21,250,241,247,95,254,59,190,245,173,111,49,57,49,193,13,55,220,192,237,183,221,206,139,47,191,
  196,75,47,189,196,219,111,189,229,11,219,107,116,117,117,209,217,217,25,248,81,44,20,112,129,60,153,5,125,73,151,69,126,232,186,65,161,144,39,147,
  201,80,44,150,40,87,188,48,3,81,20,233,142,199,89,183,106,53,169,132,199,214,205,207,207,47,5,112,13,252,112,27,212,62,47,22,10,124,247,187,
  223,13,36,136,110,220,179,135,245,235,215,115,248,232,17,206,157,63,199,232,216,40,174,235,210,222,209,78,87,87,55,33,45,4,130,247,177,182,97,32,
  224,23,158,90,52,6,220,32,222,88,247,143,228,115,185,28,186,110,80,174,120,106,6,225,80,152,254,158,94,194,33,141,241,209,49,166,167,166,150,102,
  209,55,105,11,183,230,177,64,64,94,16,56,122,228,8,167,79,159,198,52,12,210,169,20,183,221,121,39,161,80,136,227,199,143,51,53,53,133,101,91,
  40,178,204,240,240,48,241,120,23,138,226,1,79,195,48,253,138,78,53,151,207,130,88,188,235,120,33,20,169,84,138,249,249,121,242,133,2,182,237,201,
  141,41,126,200,199,200,224,32,185,108,150,100,50,73,166,209,241,252,74,250,85,61,115,237,251,42,212,72,61,193,226,204,121,234,139,51,192,162,2,7,
  85,241,252,218,13,179,219,74,98,106,89,203,150,95,235,253,121,249,93,97,70,91,214,178,229,44,28,10,177,97,227,70,186,186,186,130,223,171,143,191,
  83,236,108,185,84,226,232,17,175,84,173,170,170,236,216,190,131,182,72,148,23,95,121,133,163,71,14,83,44,149,144,37,137,72,36,130,166,106,116,116,
  180,19,239,234,34,18,137,248,90,144,44,28,151,250,3,78,215,43,164,83,41,82,233,12,165,82,49,208,177,172,138,167,119,118,118,178,122,120,152,174,
  120,156,66,62,15,120,66,227,44,119,36,124,25,43,149,203,156,57,115,6,65,16,88,187,118,45,187,119,223,128,162,120,149,126,106,253,40,150,74,204,
  207,205,47,235,7,43,244,163,163,163,131,161,193,129,69,126,216,150,69,233,42,89,56,128,100,34,9,66,10,65,16,8,133,195,236,220,185,211,151,53,
  146,56,127,254,28,165,82,9,85,85,201,229,114,168,138,74,119,119,55,177,120,140,144,159,172,85,15,68,189,164,178,18,185,92,142,68,34,73,185,82,
  198,48,61,22,178,26,47,28,143,197,25,236,239,35,22,139,145,203,102,188,251,121,53,225,18,53,128,164,88,44,6,218,169,59,118,238,192,52,76,36,
  73,226,196,201,147,204,39,230,113,108,135,98,177,200,212,228,52,138,36,49,178,106,21,157,177,78,20,85,197,173,157,197,125,84,84,209,43,228,179,57,
  230,19,9,178,185,236,162,172,120,81,20,105,111,111,103,104,96,128,254,158,30,84,77,99,110,118,142,82,177,184,124,152,193,10,205,240,1,242,174,221,
  187,233,235,237,101,247,13,55,160,235,21,206,157,63,239,125,190,235,50,62,62,193,228,196,36,29,29,29,244,246,245,210,222,209,129,36,74,1,8,173,
  58,99,89,222,198,38,149,74,147,72,36,40,87,202,139,54,157,154,166,209,219,211,195,80,255,0,241,88,39,145,72,152,84,42,213,156,217,93,14,4,
  94,79,128,216,202,162,111,89,203,174,47,51,122,213,3,179,1,226,189,191,38,102,52,30,139,177,126,195,6,238,191,255,254,224,177,214,145,253,79,243,
  118,201,101,112,104,136,95,252,165,95,10,30,170,253,253,216,209,163,239,104,133,147,161,225,33,254,201,47,254,18,142,235,112,250,244,105,110,218,179,7,
  199,79,204,200,229,115,228,243,5,138,98,145,82,169,68,58,157,161,163,179,131,190,190,62,52,77,243,19,102,60,192,144,205,100,73,36,18,228,11,158,
  176,189,227,56,62,19,3,170,170,209,21,139,209,215,211,131,99,217,20,242,121,122,123,123,1,56,127,238,124,211,5,85,104,114,255,220,26,57,166,234,
  107,14,28,56,192,103,62,251,89,138,165,34,111,191,249,38,251,174,179,31,166,110,92,83,63,170,175,59,112,224,102,62,253,217,207,50,62,54,202,165,
  11,23,56,120,224,0,161,80,136,99,199,143,81,46,149,49,12,19,81,20,208,117,157,249,249,121,186,123,186,137,197,98,132,66,225,128,229,45,22,11,
  100,210,25,82,169,148,47,200,110,4,155,6,175,154,84,152,222,238,110,122,122,122,200,101,50,24,157,157,172,223,176,1,23,152,159,155,191,50,63,88,
  26,59,24,143,199,121,224,193,7,81,85,141,215,94,121,149,182,246,118,246,238,217,195,137,83,167,152,154,154,162,162,87,252,91,32,51,54,54,134,58,
  171,208,223,63,64,71,103,39,170,234,105,188,154,134,73,62,151,99,106,106,154,124,161,80,163,67,235,183,187,232,101,255,15,13,12,128,235,114,233,226,
  37,182,108,221,194,246,29,219,57,113,252,132,119,124,222,12,171,45,114,107,177,224,187,80,203,60,186,46,183,223,121,39,155,54,111,226,220,217,115,28,
  57,124,152,173,219,182,17,137,68,56,125,250,12,233,108,6,203,208,145,68,145,76,214,99,207,187,123,186,233,238,238,38,28,241,18,179,192,139,225,157,
  153,158,33,233,199,232,122,247,75,8,190,63,164,105,172,26,26,34,30,143,51,55,51,67,54,147,102,227,198,141,129,47,86,19,64,122,217,246,168,173,
  71,223,132,53,93,148,117,223,136,113,109,192,244,44,186,134,86,70,125,203,90,214,28,31,190,99,96,180,137,213,39,48,197,187,186,216,176,97,67,11,
  140,182,140,215,94,123,141,243,231,207,55,125,254,157,212,28,29,30,30,230,23,127,229,151,113,29,135,63,251,227,63,101,106,106,138,241,209,81,118,237,
  222,77,79,79,15,167,79,159,102,124,98,2,93,215,41,150,203,84,116,29,93,215,201,231,242,116,198,98,180,183,183,97,26,94,69,165,66,177,136,97,
  24,216,53,226,245,130,40,16,235,140,49,208,215,75,123,180,141,249,185,57,178,153,140,191,248,9,244,245,247,35,8,194,178,154,156,203,177,62,1,16,
  61,116,144,207,126,254,243,20,11,69,254,248,15,254,48,168,148,116,227,222,189,215,197,143,153,233,233,128,17,189,22,126,84,125,57,112,232,32,159,253,
  220,231,25,31,31,231,79,255,232,143,41,151,203,76,140,143,115,203,109,183,17,137,68,56,115,246,44,179,179,179,94,156,97,161,128,44,137,152,166,73,
  50,145,164,167,183,151,104,36,76,177,84,102,118,102,198,3,161,62,152,174,154,199,234,182,179,122,120,132,112,40,196,196,248,56,165,98,145,92,54,203,
  246,29,59,216,184,105,19,32,48,63,55,119,121,166,173,201,99,93,241,56,247,63,240,0,154,166,241,252,179,207,114,238,236,89,0,214,172,93,203,77,
  123,247,50,57,48,192,225,163,71,40,20,10,24,150,133,101,219,216,142,205,196,248,4,242,204,12,253,253,253,0,164,83,105,50,217,44,102,3,32,166,
  133,52,214,142,172,98,120,112,144,217,153,153,64,11,245,196,241,19,108,223,185,131,237,59,119,112,226,216,241,165,128,116,185,140,115,65,88,226,211,237,
  119,220,193,166,205,155,57,123,230,12,207,254,232,25,16,4,94,127,237,53,182,108,221,202,189,247,220,195,232,216,24,231,47,92,160,80,200,123,49,173,
  88,204,205,205,51,63,159,160,163,189,221,147,43,43,149,73,167,211,94,214,124,13,0,86,20,133,112,56,68,119,188,139,190,158,30,92,199,97,244,226,
  197,32,12,4,4,54,110,218,24,248,178,8,144,174,180,61,86,0,198,185,210,44,252,22,43,218,178,150,189,7,153,209,38,246,27,191,246,107,173,59,
  218,178,134,150,78,165,72,167,82,108,216,184,145,114,185,28,200,58,1,94,85,166,157,59,121,238,217,103,175,243,134,205,99,71,62,250,241,143,33,0,
  127,242,199,127,194,244,148,151,24,147,72,36,120,250,169,167,216,177,115,39,135,14,30,100,205,212,20,103,206,158,101,62,145,8,192,92,181,30,247,252,
  252,60,150,105,82,209,245,128,65,172,130,158,104,36,66,79,87,23,189,61,61,216,182,205,216,232,40,70,205,209,233,185,115,231,64,16,232,235,235,103,
  122,106,122,49,112,104,176,64,10,53,143,9,117,11,234,7,238,190,135,114,185,204,31,253,193,239,123,247,211,151,24,122,238,153,103,216,182,125,59,15,
  61,248,32,51,179,179,28,62,124,152,137,169,41,74,165,18,249,98,145,138,174,83,44,22,177,124,73,167,218,88,112,4,144,37,153,206,246,118,6,7,
  6,232,237,233,33,155,201,112,233,194,133,69,192,160,234,71,87,87,55,154,54,190,248,120,248,10,253,120,244,147,159,100,114,114,130,63,250,253,223,247,
  52,137,5,129,116,58,205,83,79,62,201,13,187,119,243,137,143,126,148,139,151,46,114,230,204,89,46,142,142,98,24,6,153,92,14,69,150,41,149,74,
  232,134,177,168,142,123,213,15,77,85,233,138,121,49,174,29,29,29,164,147,73,46,156,59,23,248,97,89,22,39,142,31,103,199,206,157,172,91,183,142,
  116,42,181,132,141,19,26,0,26,161,193,115,109,109,109,132,66,26,47,60,255,220,2,56,247,21,2,166,167,166,216,181,123,55,31,126,248,17,166,166,
  166,152,152,154,36,157,74,83,209,117,108,219,194,54,12,70,47,141,98,217,54,142,31,147,234,149,157,21,81,84,149,144,166,49,208,215,71,79,119,55,
  197,98,145,147,39,78,44,186,223,197,98,145,19,199,142,179,115,215,78,162,209,232,146,62,213,20,88,53,1,88,221,221,221,156,59,123,150,103,159,121,
  38,120,141,105,154,28,59,122,148,190,190,62,54,110,218,68,95,111,47,147,83,147,204,205,39,40,20,242,65,210,91,190,80,32,229,203,50,33,120,229,
  78,69,81,68,150,101,84,205,107,143,222,238,110,100,89,102,118,102,134,100,93,9,217,249,185,57,4,96,112,104,104,69,172,104,125,123,8,92,38,211,
  189,5,42,91,214,178,247,164,9,55,108,223,177,98,78,245,166,253,251,249,139,255,250,87,124,225,103,62,203,171,175,188,210,244,53,83,83,83,1,216,
  184,239,129,7,152,154,156,228,184,95,30,180,101,63,157,22,14,135,249,39,191,252,203,12,251,197,16,158,123,246,89,190,255,189,239,241,216,167,63,205,
  206,93,187,248,230,55,190,177,98,48,58,52,52,196,14,191,164,104,185,92,230,249,103,159,93,196,202,87,51,112,227,93,93,193,99,223,255,222,247,2,
  208,21,10,133,232,234,234,98,114,114,114,33,153,131,133,4,6,77,211,216,177,115,39,3,131,131,152,166,233,245,231,233,105,102,231,231,188,132,36,211,
  171,57,46,202,50,161,144,70,52,18,161,59,222,69,60,22,67,150,37,242,185,60,169,100,146,108,85,238,168,193,177,69,35,208,32,52,92,59,125,29,
  77,255,255,218,215,132,35,17,34,145,8,169,100,178,225,66,171,105,26,107,215,173,99,243,150,45,87,236,135,128,167,205,58,61,53,69,37,200,174,113,
  27,178,117,122,69,255,177,252,232,238,238,166,92,46,47,20,199,168,243,163,163,163,131,141,155,54,177,110,253,122,210,153,52,147,19,147,76,207,206,146,
  206,164,201,102,179,56,182,131,227,216,40,170,70,56,28,34,26,137,18,107,111,39,30,143,35,137,34,217,108,150,185,217,217,166,126,200,178,140,36,75,
  11,126,212,133,66,212,251,214,204,143,182,246,54,138,133,230,64,48,26,141,178,126,195,6,6,6,7,189,152,214,100,130,76,54,71,177,88,68,175,84,
  112,28,27,215,245,174,39,20,14,17,214,66,116,180,183,35,138,34,229,82,137,196,252,60,197,98,113,81,34,78,189,31,141,152,196,166,27,131,170,15,
  53,9,60,2,160,106,154,199,84,46,3,220,122,123,123,25,26,30,166,175,191,159,100,42,201,236,220,28,249,124,158,82,169,140,101,154,56,142,141,32,
  74,132,52,141,144,230,245,173,182,182,54,138,133,2,133,188,39,89,181,16,43,235,174,136,149,92,105,123,8,53,108,239,162,77,80,45,11,92,183,57,
  90,242,61,245,190,55,96,244,91,214,178,150,45,181,155,15,28,224,175,255,215,255,228,231,190,248,37,94,127,237,181,119,135,25,189,239,129,7,184,239,
  254,251,249,230,227,143,7,96,116,199,206,157,220,119,255,253,124,249,111,255,246,138,46,172,101,63,89,182,111,255,126,226,241,56,255,245,47,255,146,120,
  87,23,31,254,200,71,88,191,97,3,241,120,156,199,191,241,13,158,191,2,86,244,182,59,238,224,203,127,251,183,1,48,189,237,142,59,2,192,89,219,
  23,191,223,36,44,164,82,46,123,64,180,1,203,226,10,94,92,226,155,111,188,1,174,75,188,171,139,225,145,17,118,108,223,206,190,208,94,92,199,9,
  18,123,108,219,198,113,28,210,169,20,122,165,66,114,126,158,84,13,187,22,100,232,214,101,145,87,217,172,101,23,218,6,108,98,253,181,86,202,101,42,
  62,147,216,200,116,93,231,244,169,83,156,62,117,138,238,238,110,122,251,250,216,180,113,35,219,182,110,33,28,142,120,199,242,254,145,113,185,84,242,234,
  206,23,139,204,206,204,44,0,220,203,248,161,87,244,134,33,4,87,226,199,162,16,141,6,190,228,114,57,222,124,227,13,222,124,227,13,6,7,7,233,
  233,237,229,198,93,187,144,100,25,73,146,48,77,19,219,178,176,29,7,195,48,40,228,114,100,50,25,102,167,167,23,125,118,51,63,44,203,90,82,173,
  72,104,194,202,53,99,119,5,88,22,136,86,219,252,232,145,35,28,61,124,152,206,88,140,254,129,1,86,13,13,5,210,90,213,254,228,58,14,150,101,
  81,40,20,152,153,158,94,34,252,30,100,123,215,129,185,21,249,80,3,68,169,3,103,213,184,202,70,64,52,16,229,247,63,119,62,145,240,164,170,128,
  158,222,94,226,177,24,61,171,187,2,95,28,199,241,149,25,76,138,133,60,249,108,150,201,241,241,69,96,57,200,104,111,224,203,74,192,159,80,3,28,
  27,177,162,66,237,103,92,142,17,93,14,136,182,172,101,45,187,238,118,77,193,232,77,55,221,196,247,159,124,114,17,176,248,189,223,249,29,254,209,151,
  190,196,125,247,223,223,2,163,63,229,204,232,244,212,84,192,144,127,248,35,31,33,30,143,243,159,255,248,143,153,242,143,202,87,98,27,54,108,224,141,
  154,126,52,53,53,197,142,93,187,136,199,227,1,59,90,237,103,213,191,47,156,59,183,108,188,234,114,150,74,38,73,37,147,65,230,124,117,193,12,100,
  157,170,25,233,53,217,245,141,22,81,183,201,34,39,52,93,27,151,130,6,129,186,132,140,250,207,172,99,184,106,89,223,100,50,73,50,153,92,144,85,
  170,61,154,175,247,165,201,194,127,173,253,112,155,149,97,244,147,107,220,234,255,53,159,55,61,61,237,197,74,174,192,15,247,58,251,209,16,224,174,4,
  200,8,2,217,108,150,108,54,27,36,131,46,146,62,171,62,86,149,22,170,191,87,151,241,99,57,31,168,241,65,168,1,113,75,192,91,19,224,91,15,
  192,93,32,49,63,79,162,38,222,182,86,105,162,54,217,181,25,255,217,204,23,97,217,91,40,44,2,162,212,249,86,255,25,245,175,105,193,205,150,181,
  236,39,24,140,198,226,113,206,159,91,154,204,240,220,179,207,242,79,106,50,167,91,246,83,104,117,64,199,117,93,190,252,183,127,123,69,64,20,32,149,
  78,179,126,195,6,168,5,151,174,75,58,149,226,251,79,62,185,232,177,69,127,47,183,176,45,179,80,214,234,11,214,130,130,70,172,78,53,59,185,17,
  160,187,146,197,111,9,104,160,241,17,101,51,31,150,28,59,54,0,102,85,95,170,96,67,168,213,99,244,227,22,175,185,31,117,32,160,41,248,169,3,
  25,1,192,94,129,31,245,58,146,215,170,61,132,102,32,174,89,88,66,245,190,214,15,131,70,126,84,125,172,7,156,190,63,110,19,64,122,37,126,8,
  53,199,214,75,222,87,5,112,245,242,69,203,177,246,245,227,163,78,135,179,122,111,150,212,116,175,211,251,188,98,95,234,238,125,179,49,178,82,86,84,
  184,130,249,107,185,241,215,178,150,181,236,199,51,241,90,126,88,58,157,94,148,61,95,181,161,161,33,210,151,171,95,221,178,159,120,11,133,195,172,223,
  176,193,3,147,120,82,56,213,191,107,227,59,151,237,99,169,20,67,67,67,236,219,191,159,245,27,54,240,216,167,63,125,229,241,200,181,32,161,17,171,
  180,92,92,89,29,251,178,40,70,173,65,204,218,21,95,87,125,28,92,45,40,168,255,220,229,0,70,189,63,77,152,163,70,175,169,189,6,225,26,250,
  209,48,118,239,26,251,81,223,22,92,163,246,104,8,168,235,125,168,99,71,87,148,72,179,156,15,77,238,229,213,248,176,4,144,214,51,237,117,215,34,
  92,14,132,54,2,226,53,247,168,169,47,181,143,95,113,115,44,5,162,75,218,227,106,88,209,43,57,206,111,89,203,90,118,93,236,154,50,163,111,188,
  246,26,183,222,126,59,64,112,44,186,97,195,6,110,189,253,118,94,120,238,185,214,221,254,41,183,161,161,161,69,12,249,135,62,242,145,224,247,167,158,
  124,114,197,76,230,63,60,254,56,241,174,46,226,241,120,16,59,90,174,171,109,255,250,235,175,55,92,204,220,171,209,64,107,196,142,214,176,89,245,44,
  144,80,115,132,222,140,5,106,6,142,23,45,180,181,11,120,179,5,179,1,168,105,196,244,214,46,200,181,215,31,48,111,245,215,91,235,223,53,242,131,
  107,229,7,117,44,104,45,131,88,223,22,181,215,253,227,250,209,12,224,54,58,66,111,2,106,234,217,222,122,240,90,223,159,170,225,10,87,220,175,86,
  210,167,234,0,107,109,88,64,195,123,190,12,72,164,142,237,109,244,121,75,218,164,182,63,94,6,132,54,4,149,77,218,67,184,74,112,217,130,161,45,
  107,217,79,8,24,253,254,147,79,18,10,135,185,247,254,251,185,183,230,241,231,159,123,110,197,64,163,101,63,153,246,250,235,175,47,27,183,121,165,204,
  121,85,42,170,246,239,250,231,235,109,185,163,248,218,163,211,37,64,161,230,232,215,173,123,190,246,152,56,88,92,171,130,226,151,17,211,110,10,96,26,
  45,180,205,88,157,149,0,183,21,0,175,37,62,212,1,135,37,73,51,215,192,15,183,238,158,94,137,31,139,98,53,47,179,57,104,4,74,87,236,71,
  125,108,98,29,216,169,189,47,194,50,109,210,204,175,229,194,14,130,239,175,5,164,87,224,71,163,182,168,62,190,132,105,174,1,149,151,107,7,161,201,
  152,170,111,147,250,48,131,250,247,212,131,210,21,251,209,168,61,234,218,64,88,102,131,179,146,77,67,203,90,214,178,247,49,24,173,178,86,79,61,249,
  100,160,19,55,61,53,181,32,217,242,14,219,189,126,245,167,167,126,130,128,240,250,13,27,184,247,190,251,248,179,63,253,211,69,191,191,215,173,30,60,
  190,91,214,8,100,2,124,252,147,159,228,137,127,248,7,42,149,10,184,46,219,119,236,96,219,246,237,60,243,163,31,121,98,232,130,192,224,224,32,3,
  131,131,200,178,76,44,22,11,234,209,59,182,77,42,149,226,240,219,111,115,232,182,219,130,76,226,106,102,180,227,56,188,217,128,169,93,2,40,150,3,
  62,245,108,79,77,50,85,253,66,188,121,203,22,118,236,218,133,166,170,100,50,25,94,124,225,5,114,153,12,8,2,31,254,232,71,121,252,235,95,199,
  5,30,122,228,145,32,131,187,250,255,133,243,231,185,116,233,18,183,221,122,171,247,120,141,15,83,147,147,94,140,111,35,38,107,5,126,124,244,19,159,
  240,164,189,252,199,231,231,231,249,230,55,190,65,165,92,102,96,96,128,7,31,122,40,40,239,233,56,14,179,51,51,60,247,236,179,232,149,10,189,125,
  125,220,121,215,93,139,238,173,237,56,188,248,252,243,94,98,86,21,52,11,2,187,111,188,145,142,206,78,28,219,198,118,28,138,133,2,103,78,159,166,
  84,42,33,0,155,55,111,38,218,214,182,136,145,43,20,10,156,59,115,134,246,142,14,134,134,135,57,125,250,244,18,38,115,185,141,193,34,48,221,160,
  77,170,191,119,117,119,179,105,243,102,94,126,233,165,224,177,120,87,23,235,55,108,224,245,215,94,35,30,139,113,227,158,61,129,127,142,227,112,254,220,
  185,64,224,222,109,196,226,53,75,4,91,97,159,18,46,199,20,214,246,183,186,113,243,240,35,143,240,230,27,111,48,51,53,181,104,115,177,235,134,27,
  216,177,115,103,208,175,42,149,10,71,14,31,102,116,116,116,201,230,226,74,98,69,89,6,80,55,13,169,169,191,87,77,54,117,110,75,190,169,101,45,
  251,201,1,163,224,105,63,94,184,202,236,229,150,181,236,122,217,114,137,74,2,176,122,245,106,206,156,57,3,130,192,166,77,155,8,133,66,140,140,140,
  120,18,54,174,203,186,245,235,201,229,114,12,12,12,112,233,226,69,146,201,100,0,158,12,93,71,0,206,158,62,141,11,140,140,140,224,186,46,227,99,
  99,222,119,138,226,178,192,97,9,96,104,192,236,44,58,226,173,205,56,175,1,9,107,214,174,229,145,143,124,132,127,120,252,113,42,229,50,171,86,175,
  230,115,95,248,2,127,241,103,127,134,174,235,140,172,90,21,124,214,219,111,190,137,227,131,39,69,150,57,112,240,32,103,78,159,70,0,250,7,6,120,
  246,153,103,2,255,112,93,74,85,25,169,229,142,160,151,241,99,120,100,132,99,71,142,4,178,111,219,119,236,224,115,159,255,60,127,241,103,127,70,56,
  28,38,28,14,243,221,111,127,59,0,137,183,221,113,7,251,111,190,153,231,159,125,150,80,40,68,40,20,226,71,79,63,29,100,106,187,174,75,161,80,
  88,194,200,117,119,119,51,62,62,238,201,58,185,46,93,221,221,220,249,129,15,240,131,239,127,31,211,52,137,117,117,49,59,51,67,174,170,3,139,47,
  139,36,138,168,170,74,60,30,111,222,30,245,192,167,30,168,93,6,196,105,154,70,79,79,207,162,54,172,62,6,158,198,167,22,10,241,198,235,175,7,
  237,178,251,198,27,177,44,139,196,252,124,243,141,213,10,218,162,41,16,173,121,141,219,140,125,111,114,244,45,86,75,187,214,49,211,130,40,114,252,248,
  113,142,30,62,12,64,36,18,225,222,251,239,39,153,72,120,90,169,117,223,115,185,76,251,134,247,125,5,143,179,28,19,122,45,18,155,90,214,178,150,
  189,119,193,104,203,90,246,254,64,167,11,96,225,204,153,51,172,89,179,134,179,103,206,160,106,26,155,54,111,230,239,254,230,111,184,227,174,187,120,251,
  173,183,112,5,129,145,145,17,190,243,237,111,179,102,205,26,210,233,52,179,51,51,75,64,65,21,160,86,107,183,167,252,240,3,225,50,210,63,151,5,
  12,245,44,87,35,134,71,16,88,179,118,45,71,15,31,230,236,153,51,0,140,143,143,147,205,100,208,66,33,116,195,64,20,197,224,253,181,108,219,7,
  238,190,155,179,103,207,50,53,57,25,84,0,74,213,8,147,187,245,215,182,130,69,188,145,207,83,147,147,156,59,119,14,1,47,174,252,223,252,187,127,
  23,60,175,87,42,92,186,116,105,1,160,60,251,44,183,223,126,59,207,251,96,199,48,12,38,252,66,5,213,107,18,234,143,149,5,1,81,146,200,101,
  179,65,117,159,68,34,129,166,105,172,94,187,150,243,231,206,161,200,50,197,98,145,140,95,162,181,246,122,69,73,66,150,229,166,113,138,141,128,79,53,
  166,86,104,192,28,46,97,140,5,193,107,131,90,32,232,63,86,141,13,182,29,103,81,105,210,174,238,110,250,251,251,73,204,207,47,142,39,93,193,230,
  235,74,250,20,44,142,75,93,209,16,242,175,187,126,76,137,162,184,192,16,251,27,153,66,62,79,91,91,27,165,186,98,15,151,243,165,233,209,250,229,
  128,104,147,4,196,203,130,237,150,181,172,101,45,48,218,178,203,155,187,2,70,161,101,87,198,142,142,141,142,178,235,134,27,192,103,72,207,159,59,199,
  196,248,56,157,157,157,104,154,134,174,235,132,194,97,242,249,60,162,40,178,101,235,86,6,135,134,188,163,120,215,101,102,122,154,25,31,156,130,87,26,
  52,136,253,187,18,112,92,183,48,46,183,208,54,138,29,60,114,248,48,63,247,11,191,128,22,10,49,122,233,18,99,163,163,28,63,126,124,49,147,85,
  179,80,187,174,203,222,125,251,16,69,145,35,135,15,7,159,37,201,50,59,110,184,1,252,35,115,128,75,151,46,45,6,18,87,225,71,45,160,216,127,
  243,205,139,0,161,22,14,179,118,237,218,160,95,223,119,255,253,193,113,57,64,103,44,198,161,91,110,241,88,81,255,190,191,250,202,43,139,24,57,215,
  191,247,245,12,238,204,204,12,219,183,111,231,194,185,115,200,178,204,218,117,235,232,235,235,11,158,79,36,18,30,216,243,75,88,46,2,74,43,104,15,
  161,54,246,113,57,96,35,8,116,116,116,112,211,254,253,65,18,82,180,173,13,177,6,88,73,53,215,174,170,42,171,214,172,225,244,201,147,139,18,166,
  132,107,208,167,26,106,116,94,238,115,234,152,94,177,94,119,181,202,214,11,2,237,237,237,12,12,12,0,158,114,70,119,111,175,215,222,117,34,247,194,
  149,248,177,28,160,110,2,68,151,251,140,70,126,95,22,228,183,128,107,203,90,214,2,163,45,107,217,143,135,70,27,48,87,174,75,54,155,37,22,139,
  1,94,204,229,248,196,4,8,2,147,19,19,108,216,184,145,108,54,235,213,177,175,178,88,254,98,39,10,2,14,11,49,108,110,13,195,230,174,32,57,
  163,217,34,119,185,133,182,217,98,157,205,100,248,163,223,255,125,54,111,217,194,154,53,107,184,245,182,219,200,100,50,124,253,107,95,67,175,84,16,170,
  225,2,190,223,235,214,175,103,253,250,245,252,195,227,143,47,74,38,146,101,25,81,16,112,5,193,251,223,113,86,44,45,212,204,15,128,95,250,213,95,
  13,158,155,154,156,228,127,253,207,255,25,128,153,120,60,206,93,119,223,141,166,105,116,116,118,242,202,75,47,121,37,98,235,165,141,234,129,95,141,46,
  170,80,5,220,162,184,232,122,68,81,68,146,101,16,4,175,114,147,40,6,12,37,53,159,93,251,186,43,106,143,102,25,225,13,64,92,192,78,251,247,
  91,244,251,139,32,8,8,162,200,208,240,48,95,252,217,159,197,113,28,244,74,133,243,231,207,115,241,226,197,37,89,233,87,221,167,154,109,110,154,92,
  123,125,194,94,112,13,53,37,94,107,223,35,248,204,232,208,208,16,237,237,237,56,142,67,62,159,231,7,223,255,62,134,105,46,14,11,248,49,253,88,
  9,16,173,143,53,109,177,160,45,107,89,11,140,190,195,20,98,139,55,108,25,13,89,143,134,236,232,216,24,171,215,174,101,245,154,53,60,239,75,145,
  157,63,127,158,213,171,87,147,201,100,188,35,108,188,163,201,51,103,206,120,177,143,77,42,227,4,71,177,87,184,240,93,86,184,187,62,190,175,78,5,
  96,247,158,61,92,186,116,137,35,71,142,112,244,200,17,0,62,250,241,143,179,115,215,46,222,120,237,181,5,38,11,175,140,227,109,119,220,193,119,158,
  120,194,3,9,53,223,47,75,18,199,142,30,109,152,237,253,227,248,241,39,127,248,135,94,97,140,58,16,33,224,177,151,255,245,175,254,10,1,120,236,
  83,159,90,162,229,154,207,231,121,233,197,23,27,11,144,215,72,80,73,146,132,88,119,13,237,237,237,84,42,149,128,249,28,27,27,99,174,230,40,188,
  122,157,130,40,46,61,166,111,6,122,106,30,119,27,37,112,53,0,113,2,94,178,84,80,141,78,16,24,26,28,100,223,254,253,1,184,155,155,155,227,
  137,127,248,135,134,133,12,220,235,220,167,170,247,210,93,46,41,168,118,227,37,138,11,125,170,22,112,11,2,231,207,159,247,18,247,106,196,238,235,227,
  68,133,171,149,94,186,76,123,52,5,162,205,54,11,77,252,108,89,203,90,246,206,152,216,186,5,45,251,169,179,218,50,133,53,192,102,116,116,148,253,
  55,223,140,174,235,65,61,240,115,103,207,178,113,211,38,86,173,90,197,196,196,132,55,104,106,89,179,122,177,120,255,119,209,95,164,5,46,31,15,87,
  95,87,91,104,242,249,77,23,218,154,199,99,177,24,247,63,240,192,18,0,18,28,205,251,172,174,22,10,241,240,135,62,20,100,163,47,18,168,247,217,
  195,250,154,223,203,9,161,47,121,77,85,214,169,73,88,193,229,0,195,55,31,127,156,3,7,14,4,199,188,139,128,84,125,105,206,186,251,80,101,70,
  171,254,168,154,198,158,125,251,184,120,225,130,119,12,46,73,139,152,211,218,235,16,5,193,123,190,222,143,203,0,31,161,9,235,214,168,50,80,245,218,
  2,127,170,128,142,5,22,183,182,45,234,165,139,174,244,167,97,91,172,148,69,188,76,223,21,27,104,201,226,131,212,90,255,106,55,21,87,235,11,245,
  237,209,164,253,151,245,165,81,118,189,255,35,212,205,7,45,107,89,203,222,57,107,29,211,183,236,167,207,26,212,60,7,47,19,254,190,251,238,11,98,
  39,5,215,69,215,245,32,78,52,239,103,95,11,162,200,29,119,222,73,165,82,9,98,23,93,215,229,155,143,63,30,48,116,146,40,46,214,92,188,204,
  245,52,98,126,150,123,190,97,249,79,224,149,151,95,230,177,79,125,138,255,253,55,127,147,153,153,25,58,58,58,200,100,50,188,241,250,235,1,80,3,
  216,187,119,47,145,72,132,27,247,238,101,247,158,61,65,28,102,98,126,158,23,94,120,1,73,146,120,228,67,31,10,124,115,93,151,100,50,201,171,47,
  191,220,220,151,203,197,233,53,1,63,110,3,208,86,169,84,248,135,111,126,147,251,31,120,128,255,254,223,254,27,8,2,253,253,253,252,204,103,63,27,
  196,233,186,174,203,241,99,199,188,152,216,90,86,78,146,184,245,246,219,131,246,233,234,234,226,204,233,211,204,206,206,122,199,244,146,196,193,131,7,49,
  12,99,209,165,125,247,219,223,70,16,4,122,251,250,120,232,145,71,2,0,119,246,204,25,79,101,97,25,63,133,43,72,42,18,27,84,58,18,107,194,
  39,68,73,90,114,239,132,186,144,15,183,22,76,173,164,63,173,176,79,213,178,20,110,131,231,234,191,77,20,69,62,241,232,163,139,106,209,255,193,239,
  255,254,18,16,189,36,222,181,81,200,65,147,146,155,87,218,167,86,12,68,151,25,139,205,159,110,113,167,45,107,217,117,89,150,111,216,190,99,197,91,
  193,155,246,239,231,47,254,235,95,241,133,159,249,44,175,190,242,74,235,238,181,236,125,107,174,159,148,83,223,249,215,172,89,195,236,236,172,167,55,234,
  47,144,29,126,18,211,220,220,28,184,46,61,189,189,168,154,22,60,95,93,136,171,146,69,0,237,109,109,184,64,33,159,191,34,144,188,146,197,88,168,
  91,220,105,0,28,58,99,49,58,99,49,50,153,12,217,76,38,248,140,145,213,171,25,31,27,163,163,163,131,142,142,14,239,218,107,24,34,221,48,72,
  204,207,51,52,52,180,184,202,143,255,92,170,170,233,121,21,126,12,15,15,147,78,165,2,221,225,90,63,66,154,70,44,30,15,50,252,171,207,175,89,
  187,54,72,12,235,235,239,95,236,175,31,235,155,203,229,22,129,179,158,158,30,175,125,252,215,38,18,9,12,93,15,62,183,171,187,27,85,85,151,92,
  246,204,244,52,170,170,210,221,221,189,8,68,21,242,121,242,249,252,242,0,174,38,102,117,57,16,167,105,26,29,29,29,36,230,231,23,61,214,222,209,
  193,252,252,60,154,170,210,222,222,78,34,145,104,30,139,122,37,236,221,213,244,169,186,126,181,228,8,191,1,120,109,152,84,89,91,189,171,209,117,255,
  56,126,44,7,66,175,2,136,174,248,117,45,48,218,178,150,45,107,55,31,56,192,95,255,175,255,201,207,125,241,75,11,225,72,43,176,22,51,218,178,
  159,210,109,88,227,133,126,116,116,116,241,130,42,8,100,253,35,251,234,223,85,189,199,229,216,169,124,161,176,114,54,166,217,37,46,167,95,89,191,40,
  215,149,195,204,102,179,100,235,164,139,0,38,198,198,0,200,229,114,158,206,102,29,152,173,126,242,212,212,212,85,49,71,203,249,81,5,235,141,252,168,
  84,42,204,204,204,44,174,200,36,8,94,123,248,239,31,171,17,76,95,2,192,107,64,82,34,145,88,170,154,80,243,93,169,102,197,23,4,1,195,52,
  153,174,81,69,184,146,246,16,106,88,112,183,14,224,85,175,79,215,117,15,104,178,192,10,235,186,142,62,63,31,48,241,186,174,47,210,47,117,175,65,
  27,92,77,159,90,196,232,215,30,235,215,171,56,212,129,240,250,202,74,194,245,240,163,238,59,154,249,226,214,253,45,180,128,104,203,90,246,158,180,22,
  24,109,217,79,41,22,173,169,253,221,100,129,106,86,173,137,58,208,113,217,69,247,74,23,217,43,97,119,68,113,161,170,76,125,246,126,45,160,168,63,
  242,133,165,97,4,245,90,147,63,134,47,87,236,71,61,216,185,156,31,245,44,100,61,72,170,1,82,239,104,123,52,185,231,139,174,175,89,159,105,244,
  154,107,224,199,85,183,69,3,192,119,57,224,186,168,61,170,31,113,141,250,85,179,227,250,101,1,245,114,239,231,10,143,240,91,214,178,150,93,55,107,
  37,48,93,35,219,188,101,11,143,124,232,67,173,27,241,126,4,166,87,176,136,127,232,35,31,225,207,254,226,47,22,37,76,44,90,208,154,36,253,172,
  56,241,167,193,231,172,8,52,44,247,154,38,175,251,249,95,248,199,236,221,187,119,233,247,253,152,9,51,139,62,163,201,231,46,185,190,203,85,198,105,
  244,154,101,238,127,163,239,95,174,61,174,169,31,205,36,176,150,105,139,70,9,58,43,245,227,170,218,226,199,233,83,203,188,78,192,43,160,176,110,221,
  186,69,160,244,90,248,194,149,182,199,10,198,248,229,128,232,200,170,17,254,249,175,253,11,86,249,149,203,222,207,246,201,79,125,138,79,126,234,83,173,
  9,191,101,45,48,250,147,108,155,54,111,230,225,22,24,125,159,161,80,97,229,245,185,47,3,146,154,45,254,203,1,187,134,175,107,2,176,150,3,53,
  194,114,0,104,153,235,94,191,97,3,241,174,174,166,89,206,205,174,239,178,32,172,217,253,88,225,117,93,17,75,85,127,111,87,218,30,245,175,105,242,
  186,43,245,67,88,225,245,46,231,195,21,245,171,21,252,52,253,140,229,250,84,93,159,104,232,67,147,207,184,251,222,123,89,183,126,125,195,76,247,235,
  226,203,101,218,99,217,177,125,153,190,22,14,71,216,180,121,51,225,72,228,125,63,221,141,140,140,48,50,50,210,154,247,91,246,158,180,171,58,166,15,
  146,30,90,182,248,190,52,57,242,109,217,251,0,148,54,72,62,1,88,181,106,21,35,171,86,241,226,11,47,84,27,57,168,157,14,112,232,214,91,153,
  24,27,99,124,124,124,225,227,154,100,56,187,203,1,221,21,50,53,213,107,168,103,211,150,77,6,241,159,219,176,97,3,83,83,83,65,2,145,88,147,
  241,28,10,133,24,28,26,100,122,106,58,72,222,18,46,39,228,126,149,108,211,114,126,184,87,35,173,115,57,64,178,140,31,151,205,128,191,82,63,252,
  123,218,212,143,149,136,227,55,97,137,133,171,149,29,186,202,182,168,191,222,96,124,92,230,58,68,161,73,124,229,187,225,75,179,26,245,203,124,86,56,
  28,14,198,72,176,217,243,125,15,71,34,148,75,165,247,205,212,214,221,221,77,210,79,58,172,47,237,251,126,243,165,101,239,19,28,116,149,239,107,49,
  163,87,105,143,124,232,67,252,201,127,254,207,77,159,255,197,95,250,37,175,238,118,203,222,251,64,180,230,127,1,184,241,198,27,131,167,239,190,247,94,
  30,251,212,167,154,50,44,31,250,240,135,121,236,211,159,14,254,222,188,101,11,145,104,244,178,172,142,176,28,3,180,146,203,174,94,115,109,242,76,29,
  27,20,14,135,233,234,234,10,22,216,95,250,167,191,194,237,119,220,190,240,186,26,45,200,193,161,33,126,238,31,255,130,199,104,1,161,80,136,88,87,
  215,85,51,88,43,190,253,43,240,227,154,180,113,51,166,237,26,248,33,176,60,203,118,69,126,92,174,36,231,59,216,22,181,237,209,76,252,62,20,14,
  243,239,127,235,183,184,247,190,251,22,22,21,81,90,232,87,131,131,252,219,127,255,239,217,182,125,251,59,230,139,240,99,0,209,135,30,126,152,223,250,
  157,255,72,87,119,119,240,58,65,88,88,38,239,185,231,158,101,231,253,247,146,253,155,127,247,239,248,228,99,143,53,125,254,119,255,211,127,90,246,249,
  150,181,236,157,180,22,24,93,161,237,190,241,70,126,247,63,253,39,79,246,5,72,204,207,99,24,6,55,236,222,205,142,157,59,233,237,235,35,147,201,
  112,219,237,183,51,60,50,66,79,111,47,243,53,21,94,150,157,24,124,89,29,183,201,15,45,49,230,119,12,152,222,125,239,189,252,226,175,252,74,16,
  35,86,44,22,201,23,10,60,242,209,143,240,177,71,31,165,175,191,159,243,231,207,243,177,79,124,130,15,125,244,35,216,182,205,232,232,40,161,80,136,
  238,158,30,126,253,55,126,99,17,56,93,104,98,143,53,119,170,63,174,187,228,103,81,155,95,229,66,91,11,40,126,227,255,252,63,248,216,39,62,14,
  64,185,92,38,149,76,49,188,106,21,123,111,186,137,7,63,248,65,102,102,102,25,30,25,225,224,45,183,176,239,166,155,168,84,202,65,166,249,45,183,
  222,202,255,254,155,191,65,44,30,111,236,75,245,199,113,22,126,26,245,221,229,252,104,2,48,234,253,120,47,167,150,52,59,46,22,234,75,152,114,249,
  226,7,239,166,15,203,130,233,58,48,248,139,191,252,75,65,31,175,148,203,148,203,37,186,122,186,25,28,28,100,235,214,173,20,10,5,250,250,251,24,
  24,28,100,253,198,13,56,174,67,38,157,6,96,239,190,125,252,239,191,249,155,196,27,244,171,166,125,108,133,115,223,229,142,239,27,1,209,225,145,17,
  254,195,111,255,22,55,236,190,1,128,51,167,79,99,24,6,155,54,109,98,223,254,253,108,219,190,157,76,38,195,182,237,219,217,185,107,23,155,54,111,
  14,20,55,0,254,197,175,255,58,191,248,75,191,244,158,152,215,191,240,197,47,46,34,64,46,94,188,72,71,103,39,91,183,111,99,255,205,55,99,153,
  38,150,101,177,255,192,205,236,219,127,19,133,98,129,49,223,151,145,85,171,248,221,255,244,159,216,93,179,17,95,169,31,173,53,170,101,215,194,222,213,
  108,250,193,193,193,101,227,44,167,167,166,120,226,91,223,122,215,174,175,26,39,84,46,149,152,24,31,71,11,133,120,244,177,199,184,116,241,34,189,125,
  125,188,249,214,91,108,218,178,25,189,162,51,59,59,235,137,140,199,58,185,249,192,1,206,95,188,72,172,163,131,15,125,248,195,100,179,89,58,58,59,
  188,227,18,31,148,196,58,59,73,251,19,244,229,204,109,178,56,180,234,44,95,187,141,70,50,153,100,98,124,156,183,223,126,155,135,30,121,132,71,62,
  252,97,108,199,225,210,232,40,95,253,218,215,72,165,82,190,20,146,227,137,174,227,213,82,31,159,152,164,191,191,159,207,124,238,115,132,66,33,178,217,
  44,223,124,252,113,92,215,37,22,139,177,118,221,58,222,122,243,205,43,110,103,183,254,232,119,25,240,86,251,186,155,15,28,224,21,95,3,248,232,145,
  35,236,187,233,38,30,122,248,97,250,7,7,121,235,240,97,178,217,140,231,215,216,24,151,198,198,112,93,23,89,150,25,29,27,35,87,40,112,215,221,
  119,163,235,58,3,253,125,76,77,76,146,246,193,233,134,141,27,25,31,27,67,175,209,235,108,184,104,53,0,21,77,171,231,44,227,71,189,44,147,176,
  76,8,204,178,50,72,215,27,132,94,230,249,122,69,134,70,161,32,194,187,228,199,229,88,194,218,231,99,241,184,215,23,4,129,116,58,205,158,189,123,
  56,115,234,20,189,253,253,92,188,120,9,195,48,217,189,111,47,166,97,242,234,107,175,226,186,112,195,158,61,104,154,198,209,99,199,185,249,224,65,18,
  243,243,108,220,180,145,80,56,68,58,157,198,117,93,58,58,59,200,101,115,151,5,156,110,163,178,187,181,149,207,86,210,30,53,175,233,234,234,34,149,
  74,145,74,38,177,109,135,219,110,191,131,120,87,55,209,182,54,190,255,253,167,16,21,5,203,52,184,112,225,60,231,207,159,3,4,180,144,198,219,135,
  15,19,143,199,249,240,71,63,202,236,204,12,171,86,175,226,197,231,95,8,174,239,90,204,235,193,239,43,176,218,99,248,51,167,79,179,103,239,30,30,
  122,248,97,0,10,197,34,23,46,94,164,173,189,131,82,185,196,15,127,244,67,92,215,69,148,36,34,145,40,79,63,253,67,54,109,221,74,103,44,198,
  224,208,16,138,166,50,225,207,9,33,77,195,245,55,27,173,53,170,101,63,241,96,180,126,1,91,183,126,61,211,211,211,43,30,0,215,219,254,229,191,
  250,87,36,147,73,126,239,119,127,151,222,222,94,94,121,229,21,38,38,38,152,152,156,224,149,215,94,35,147,201,160,235,58,237,237,109,116,116,116,96,
  154,38,201,100,18,189,162,35,171,42,93,93,113,122,186,123,24,28,28,228,233,167,159,198,113,28,34,209,40,59,119,238,228,131,15,61,196,95,253,229,
  95,114,233,226,197,171,184,101,238,98,241,233,218,35,199,150,93,145,117,119,119,243,133,47,126,145,100,34,193,255,252,31,255,131,27,118,239,230,249,231,
  159,231,236,217,179,92,26,27,37,145,72,82,169,84,208,84,133,225,225,97,134,134,70,200,102,179,204,204,76,115,236,216,4,46,94,188,101,71,71,59,
  107,215,172,101,213,217,179,220,126,199,29,156,56,126,156,91,111,189,149,181,235,215,113,241,226,197,128,25,186,42,176,224,215,10,175,2,59,161,73,166,
  240,221,247,222,203,39,62,249,40,165,114,25,219,113,144,36,153,239,126,239,73,206,158,61,195,220,147,223,35,159,47,224,56,14,157,157,29,140,12,15,
  163,40,10,201,84,138,68,98,158,82,177,140,36,203,254,209,126,156,161,193,33,134,6,7,217,177,107,23,51,211,211,60,248,224,131,140,143,143,243,141,
  111,124,99,197,204,158,208,140,241,170,7,96,13,0,131,80,15,64,170,11,91,29,40,21,234,195,44,174,35,152,91,137,12,208,146,132,183,203,72,53,
  53,210,241,188,158,126,92,141,8,252,205,7,110,230,51,63,243,89,126,235,63,252,255,200,231,242,148,138,37,94,123,237,117,92,81,228,232,241,99,60,
  251,220,115,228,11,121,100,89,166,179,179,19,77,85,201,100,179,20,139,69,92,199,37,20,14,211,221,213,69,79,111,15,210,165,75,76,78,77,209,211,
  219,75,54,147,225,177,79,125,138,241,177,113,190,251,157,239,92,217,245,87,215,14,127,92,84,99,89,133,186,121,176,145,191,255,244,159,253,51,34,225,
  48,191,245,31,254,3,171,214,172,225,248,137,19,36,18,9,18,169,36,111,31,57,66,38,155,197,208,117,194,225,16,93,93,93,152,166,73,54,155,165,
  84,42,33,201,10,29,237,237,116,117,117,177,102,205,26,158,121,230,25,138,249,60,241,120,156,142,142,14,190,244,179,63,203,215,191,246,53,222,126,235,
  173,171,158,215,131,190,113,153,121,125,243,230,205,252,243,95,251,53,190,242,229,47,243,250,27,111,208,211,219,203,43,47,191,66,38,151,99,114,114,146,
  68,50,65,42,149,70,112,93,98,241,56,209,72,152,124,161,64,46,151,195,50,45,34,209,40,177,88,140,222,222,94,202,186,78,46,151,101,120,196,155,
  223,62,253,153,207,48,48,56,192,191,255,183,87,23,106,214,90,163,90,246,190,2,163,211,211,211,252,151,63,255,243,224,239,127,251,239,255,61,79,124,
  235,91,92,188,112,225,61,113,115,14,31,62,204,205,55,223,204,207,254,252,207,51,62,49,193,15,159,249,17,23,206,95,32,157,73,227,56,14,93,241,
  46,54,108,88,207,240,208,16,225,72,132,66,161,64,91,91,148,68,34,73,38,147,225,236,217,179,156,61,123,142,246,182,54,122,251,122,217,188,105,51,
  119,220,117,39,35,195,35,76,76,140,115,105,37,126,174,96,240,214,14,252,106,237,235,150,173,204,146,201,36,223,124,252,113,238,188,235,46,238,184,235,
  46,94,122,229,21,206,157,59,71,42,149,162,82,169,16,82,85,214,111,88,207,186,117,107,233,234,234,198,117,29,84,85,69,83,85,162,209,54,18,137,
  4,201,100,138,100,50,201,204,204,44,39,79,157,100,205,234,53,108,217,188,153,112,52,194,179,207,60,179,98,166,68,88,14,212,52,88,176,234,75,46,
  190,244,210,75,28,186,229,22,238,185,239,62,142,159,56,206,243,47,189,200,133,243,23,40,22,139,152,166,73,127,95,31,27,55,110,100,96,176,159,112,
  40,76,161,80,68,85,85,162,145,8,153,76,150,233,233,105,230,230,230,72,36,18,140,141,141,211,219,219,195,230,77,155,185,237,246,219,113,112,121,241,
  197,23,174,232,56,121,73,37,162,26,29,80,183,166,178,211,146,210,145,151,25,15,75,68,215,27,220,43,161,110,163,123,45,117,45,175,4,232,45,73,
  102,170,47,139,121,157,253,184,22,62,156,59,123,142,82,169,200,35,31,254,48,137,68,130,51,103,207,50,58,58,198,248,196,56,165,98,137,144,166,177,
  102,237,26,134,6,7,232,238,238,193,113,93,230,231,230,72,38,18,164,253,10,96,51,51,211,40,170,74,172,51,70,127,95,31,91,182,109,163,179,163,
  131,112,36,194,185,179,103,175,73,159,90,52,62,106,129,79,157,207,71,143,28,225,161,71,30,230,31,253,236,151,152,152,152,228,153,231,158,101,98,98,
  130,68,34,137,101,153,116,197,227,140,140,12,51,60,52,76,44,30,163,80,40,144,72,204,147,72,36,152,159,79,112,233,210,37,206,95,184,192,153,51,
  103,232,237,235,101,213,200,42,14,222,114,11,195,67,67,228,11,121,78,157,56,113,249,35,235,171,153,215,235,198,200,153,51,103,24,29,29,229,230,131,
  7,233,27,24,96,108,124,156,103,159,123,142,233,153,25,114,185,44,146,40,177,122,245,106,134,135,7,25,26,28,6,193,37,153,244,54,159,222,255,73,
  166,166,166,144,100,153,206,142,14,86,173,94,197,134,117,235,249,196,39,63,73,87,79,55,47,190,240,226,202,142,222,175,212,151,154,88,245,150,181,236,
  189,195,140,190,199,236,209,199,30,227,229,151,94,98,98,124,220,99,45,5,56,114,228,40,103,206,158,101,118,118,38,96,211,134,7,135,24,30,30,166,
  175,175,23,215,117,41,149,203,8,64,87,188,139,104,36,74,87,60,206,220,220,60,57,191,148,96,246,92,142,84,42,197,170,85,171,200,100,179,180,69,
  162,40,170,138,40,138,220,247,192,253,124,255,123,79,46,148,160,172,29,231,77,234,53,47,55,232,93,219,110,129,210,203,176,161,255,242,95,255,107,254,
  254,203,95,230,226,133,11,12,143,140,240,196,119,190,205,43,175,188,202,204,204,12,174,227,208,209,222,198,158,221,55,176,111,223,62,63,187,182,68,190,
  80,164,88,44,34,32,208,214,214,134,166,105,244,245,246,81,174,148,73,204,39,24,27,31,103,122,106,58,0,165,55,237,219,199,150,77,155,25,26,28,
  164,183,183,151,251,31,124,128,63,255,207,127,70,166,166,50,82,51,64,226,54,234,3,245,175,113,93,246,221,116,19,247,220,119,31,127,248,7,127,192,
  198,141,27,57,117,230,52,199,143,159,224,196,201,147,36,19,9,244,74,133,174,238,56,155,54,110,102,237,218,53,68,194,97,108,199,166,92,214,177,44,
  11,85,85,233,236,232,36,20,10,17,141,70,201,231,243,36,19,9,242,133,2,227,227,227,164,211,25,38,38,39,216,187,103,15,27,54,108,100,110,110,
  158,251,31,184,31,128,39,191,247,228,138,89,210,250,234,60,181,11,107,181,156,106,61,235,219,52,43,189,86,80,125,133,139,100,61,168,187,22,11,236,
  74,217,198,134,126,212,197,145,94,145,31,43,245,229,106,124,240,223,55,60,60,204,205,7,14,240,245,175,125,13,203,178,56,121,234,20,23,46,94,228,
  196,137,147,140,141,141,145,207,231,137,70,34,172,95,191,158,161,193,1,86,173,90,133,170,170,24,134,73,169,84,34,28,142,208,215,223,79,123,123,7,
  217,206,44,169,84,154,124,62,207,220,220,28,201,100,146,116,58,205,154,53,107,176,109,27,219,117,112,129,237,59,118,0,112,226,248,241,171,235,83,85,
  159,107,128,105,149,125,62,112,240,32,0,175,190,242,10,137,100,146,99,199,142,115,228,232,17,14,31,62,194,236,220,44,182,101,19,139,117,178,118,237,
  86,70,134,71,232,238,233,66,146,36,138,197,18,2,208,209,222,129,166,133,104,111,107,39,153,74,145,207,23,72,167,211,100,207,229,152,153,153,97,106,
  106,138,93,187,118,210,21,143,19,142,68,112,241,146,27,159,125,246,25,102,103,102,175,205,188,238,186,68,35,17,62,249,233,79,243,247,127,255,247,56,
  182,205,165,75,23,201,230,242,28,59,118,140,243,23,46,144,152,159,71,11,105,140,12,143,48,52,52,200,134,13,27,136,70,194,152,166,77,62,159,67,
  83,53,122,123,122,105,107,107,167,173,173,141,108,38,75,54,231,173,77,153,108,134,233,233,105,38,38,39,217,188,105,83,112,255,110,216,189,155,254,129,
  126,190,223,108,188,95,169,47,126,41,102,65,20,91,71,248,45,107,129,209,102,32,229,224,193,131,28,60,120,144,151,94,122,9,73,146,120,234,233,167,
  185,112,225,2,174,227,48,60,52,204,200,200,48,125,189,189,196,98,49,36,73,162,84,42,161,235,58,166,101,225,56,14,162,36,161,105,33,36,89,38,
  18,137,82,42,21,201,230,114,20,10,69,210,233,52,167,78,157,38,157,74,179,117,235,22,62,242,177,143,17,13,135,25,28,26,226,244,201,83,156,57,
  125,122,233,192,173,63,146,93,142,21,170,223,137,218,54,180,118,161,75,172,84,42,145,76,36,248,236,231,62,199,203,175,188,194,171,175,189,198,235,111,
  188,78,33,95,96,221,154,53,236,220,185,131,53,107,214,160,40,42,249,124,142,185,185,57,42,21,29,219,182,1,112,92,7,65,16,145,21,5,81,148,
  80,20,133,246,182,118,86,173,26,33,149,74,51,49,49,65,42,147,230,149,87,95,37,159,47,112,224,208,33,214,175,95,79,42,233,177,173,75,202,21,
  174,96,209,109,186,224,226,197,191,125,233,231,126,150,92,46,207,83,79,63,205,153,51,103,208,43,21,6,7,7,216,186,101,11,221,61,221,168,138,138,
  109,219,20,203,101,244,74,5,203,118,112,29,27,4,1,89,145,17,68,209,243,163,189,157,158,158,110,202,229,50,243,115,243,36,82,73,206,157,59,135,
  227,184,236,216,190,157,143,125,252,227,172,94,189,138,147,203,176,63,110,3,240,212,144,233,173,147,49,170,46,184,181,71,173,141,128,156,208,172,154,212,
  53,4,103,215,2,132,94,14,144,94,181,31,215,208,151,102,12,252,205,7,14,112,215,7,238,34,218,22,69,16,37,206,156,61,203,139,47,189,72,58,
  149,166,189,189,157,145,225,97,134,134,6,233,233,233,33,18,137,96,24,6,249,124,30,195,48,177,109,27,81,20,80,20,21,177,77,34,20,14,209,217,
  209,73,62,159,167,80,44,82,40,228,153,154,158,38,157,201,144,201,102,216,123,227,30,134,6,135,184,105,255,77,204,207,207,115,226,216,177,171,234,83,
  181,227,164,190,95,221,121,215,157,140,172,90,197,134,77,27,153,155,79,240,149,175,125,149,209,209,49,34,225,16,59,182,111,103,245,170,85,196,227,113,
  20,69,193,178,45,202,165,10,134,97,96,219,54,142,235,32,138,18,154,166,209,213,213,77,103,103,12,211,52,40,150,74,20,242,5,230,19,9,206,159,
  63,207,124,98,158,77,27,55,177,119,255,126,250,122,123,88,179,122,13,135,223,126,155,217,233,153,107,54,175,111,218,188,153,131,135,14,209,215,223,207,
  212,212,20,111,188,245,22,71,143,30,37,155,201,16,143,199,249,192,93,119,210,63,48,64,36,28,65,16,160,162,235,36,147,41,44,203,246,99,221,93,
  100,89,33,26,245,252,233,235,235,195,208,13,202,149,50,243,243,9,18,137,121,222,72,188,193,232,232,40,7,110,190,153,71,31,251,36,155,54,110,98,
  118,118,150,144,166,53,36,76,220,102,155,190,101,124,113,125,80,42,180,64,105,203,126,60,48,122,109,50,231,6,7,7,121,232,145,71,22,77,26,15,
  61,252,112,16,51,58,61,61,205,183,223,193,4,166,100,34,193,239,253,238,239,242,165,159,251,57,82,233,52,207,62,251,44,93,93,113,238,189,251,3,
  196,227,93,200,178,132,227,186,232,186,65,161,88,196,50,45,44,219,242,111,135,139,139,11,46,222,68,44,42,72,162,63,17,199,98,126,38,163,77,165,
  82,33,153,74,113,248,200,17,42,149,10,187,111,184,129,167,158,122,138,211,13,128,104,163,93,167,91,175,255,183,140,224,182,11,8,142,131,251,30,136,
  213,217,187,111,31,233,116,250,61,17,130,81,46,149,248,219,191,249,27,62,246,137,79,240,226,139,47,114,230,204,105,110,188,225,6,118,237,220,73,103,
  71,39,197,82,145,124,62,79,177,56,143,105,154,222,36,238,223,111,199,245,127,119,189,251,45,73,98,0,52,92,215,161,183,183,151,225,225,97,76,211,
  228,252,133,243,156,62,115,26,211,52,176,108,155,179,167,79,55,156,204,155,50,162,181,255,87,193,103,29,144,123,253,245,215,233,238,233,70,86,84,94,
  124,233,37,44,203,228,131,15,220,199,218,181,235,112,29,151,76,54,77,38,155,167,144,247,66,75,28,159,149,240,174,119,97,33,151,36,17,81,84,145,
  101,25,77,85,137,132,35,244,245,246,162,106,33,82,233,20,23,46,156,231,185,231,159,231,198,27,110,192,178,204,166,172,104,51,166,164,94,207,211,173,
  103,248,234,74,80,186,174,139,232,51,251,213,251,187,156,118,229,181,40,97,250,227,128,183,149,225,199,247,134,31,43,241,225,235,95,251,26,161,112,8,
  68,145,151,158,127,158,233,233,105,182,110,222,204,192,192,32,225,80,8,65,4,93,55,209,245,10,233,76,6,211,52,113,157,154,12,107,64,16,5,100,
  65,66,146,68,100,89,33,18,141,208,109,219,216,166,137,237,184,100,115,89,230,230,102,121,225,165,23,217,181,115,23,227,227,19,124,235,155,223,188,186,
  62,85,141,31,173,251,187,106,191,255,123,255,47,255,244,127,251,103,228,242,5,126,248,195,31,18,141,68,184,231,3,119,209,215,215,143,162,200,184,46,
  152,166,73,46,159,199,50,45,108,219,174,241,197,251,39,0,146,236,201,86,201,178,140,162,170,180,69,163,244,244,246,96,153,22,179,115,115,92,26,189,
  132,97,26,220,120,195,13,156,61,115,246,154,207,235,111,191,253,54,255,227,191,255,119,110,220,179,135,147,167,78,49,122,233,34,27,215,175,99,120,100,
  132,142,182,118,36,89,10,214,25,195,48,130,249,107,209,220,133,167,49,44,75,50,146,40,161,42,10,161,144,70,91,180,141,53,171,87,83,40,22,72,
  36,18,188,252,234,43,24,230,30,218,218,218,249,234,223,255,61,229,38,115,215,162,62,91,95,74,118,5,107,20,142,211,138,39,253,137,178,171,155,185,
  222,83,204,232,187,5,84,186,187,187,57,120,232,16,79,124,235,91,172,89,187,150,151,94,126,153,103,158,121,134,79,124,252,227,232,186,142,227,216,232,
  21,157,66,193,139,189,179,109,123,17,64,9,38,44,183,166,28,64,117,209,113,65,18,69,68,85,69,85,5,84,85,37,20,10,145,74,165,56,115,230,
  12,184,176,118,205,26,20,89,102,245,154,53,132,66,26,199,143,29,95,249,174,179,54,14,175,25,32,173,15,36,127,23,44,22,143,243,241,71,31,229,
  173,55,222,224,137,111,125,235,178,160,236,122,216,7,238,185,135,131,135,14,241,119,127,243,55,28,188,229,16,223,252,214,183,168,148,203,252,250,175,253,
  26,147,147,83,228,114,57,230,230,230,177,44,43,104,87,81,20,145,36,143,253,20,68,17,199,113,113,108,27,199,177,177,44,11,203,182,171,251,124,100,
  73,10,22,53,219,177,89,183,118,29,170,162,112,254,252,5,42,149,10,135,14,28,164,92,42,113,243,129,3,76,76,140,243,210,11,47,94,62,147,185,
  9,96,248,204,103,63,203,165,75,23,25,189,52,138,170,133,120,225,197,23,89,179,122,53,187,119,223,64,38,147,97,106,106,138,82,169,28,44,170,182,
  99,227,58,181,64,199,13,54,149,213,254,81,237,37,130,40,32,139,178,31,126,82,66,145,100,54,172,223,192,232,232,24,111,31,57,140,32,138,220,184,
  103,15,243,115,179,220,118,219,237,124,227,235,95,71,55,140,230,128,186,58,78,234,1,87,147,26,232,213,126,236,56,78,192,146,94,182,223,54,138,197,
  188,6,160,78,184,198,172,228,187,225,199,149,248,112,231,93,119,113,238,220,89,242,185,60,134,101,243,230,155,175,17,9,135,121,240,129,7,176,44,11,
  211,50,209,43,21,76,211,244,88,80,191,95,5,115,97,237,252,23,204,59,2,130,0,162,40,162,8,2,138,236,129,63,73,20,105,111,111,103,108,108,
  140,35,71,143,32,8,48,48,56,200,204,244,52,251,111,190,153,231,159,123,174,249,9,66,125,159,106,0,64,195,161,16,15,61,242,48,223,126,226,219,
  172,223,176,129,115,231,206,243,210,139,47,242,209,143,126,4,67,55,192,117,41,87,202,20,10,5,108,219,193,178,109,28,127,188,84,125,9,230,245,186,
  24,103,23,119,97,94,71,192,209,28,100,89,38,149,74,49,51,61,205,219,174,203,222,61,123,137,199,227,68,163,81,250,250,250,120,243,141,55,22,251,
  211,40,46,184,201,188,126,232,150,91,136,132,195,188,240,252,243,12,14,13,241,234,171,175,50,57,57,201,7,63,248,65,108,203,10,0,104,161,88,196,
  178,22,131,233,218,83,7,63,211,171,174,63,10,200,146,132,36,121,112,64,150,101,34,145,8,163,151,46,241,198,27,111,32,32,176,103,239,94,94,126,
  241,69,14,221,114,11,111,190,249,6,185,108,174,241,26,213,36,62,250,253,176,70,181,236,125,201,140,94,27,155,158,158,230,47,106,18,152,222,45,251,
  192,61,247,240,129,187,239,230,134,221,187,25,155,24,231,239,191,242,85,66,154,198,234,213,171,121,243,205,55,209,117,239,136,214,182,237,96,114,82,21,
  213,139,183,235,140,161,106,26,142,3,134,97,120,186,123,122,137,74,69,199,52,116,112,108,95,206,197,155,192,68,73,32,20,210,80,20,133,116,38,203,
  137,147,39,81,85,133,71,62,242,17,86,175,90,197,252,252,60,199,106,192,232,18,166,164,217,130,94,195,12,52,27,236,44,199,204,92,103,123,250,169,
  167,168,148,203,220,125,239,189,108,219,177,131,167,159,122,106,161,170,209,59,200,136,118,117,117,241,169,207,124,154,39,191,255,20,175,191,254,58,247,223,
  123,15,153,76,150,137,137,137,37,27,140,80,40,68,95,95,31,146,36,19,137,182,33,73,50,166,105,81,46,151,41,151,203,20,10,57,202,149,50,166,
  174,99,91,38,142,191,120,33,120,11,149,119,203,5,116,195,224,226,197,75,116,119,247,112,232,150,91,232,235,235,227,220,153,51,139,39,234,186,9,188,
  182,29,235,99,227,98,241,56,177,206,78,238,191,255,1,142,159,56,193,139,47,189,72,62,151,163,189,189,141,241,241,9,178,217,44,174,235,34,73,18,
  162,40,6,0,33,22,139,161,133,66,116,119,245,34,136,34,186,110,80,46,151,40,149,74,148,74,69,202,165,34,182,101,46,128,10,215,243,67,82,85,
  140,146,199,130,101,115,121,142,29,63,198,237,183,222,198,158,189,123,188,205,92,79,15,83,147,147,13,65,142,219,4,148,54,2,15,245,127,215,102,207,
  175,184,207,214,191,174,1,168,187,10,4,249,206,79,74,215,218,143,21,248,208,213,213,197,131,15,125,144,144,246,49,222,62,124,152,87,95,123,141,169,
  169,41,110,222,191,159,72,36,194,248,248,120,176,25,95,72,64,131,104,52,138,170,170,116,116,196,64,16,124,96,228,133,131,84,42,101,116,67,199,50,
  77,4,4,191,140,147,55,78,20,85,69,21,84,44,211,100,106,106,10,89,150,185,241,134,221,220,121,231,157,68,163,81,38,39,38,184,216,68,109,196,
  109,6,74,107,230,197,77,155,55,243,129,15,220,205,238,221,55,114,250,236,25,126,248,163,31,81,42,151,89,179,122,13,111,31,126,27,67,215,49,124,
  70,183,202,28,86,231,245,72,36,74,36,218,134,235,122,243,186,174,235,84,244,50,165,82,9,203,52,112,109,43,152,215,171,199,36,213,121,189,88,44,
  114,238,252,5,186,187,186,56,112,232,16,155,54,110,196,52,77,222,168,1,163,139,198,195,10,230,245,67,183,220,194,230,205,155,217,180,101,11,167,78,
  159,230,232,177,99,172,26,25,33,18,142,48,49,49,177,16,82,80,213,251,197,155,191,20,89,165,187,187,27,65,148,176,76,27,221,168,80,169,232,84,
  244,18,149,114,197,27,239,213,113,231,191,79,150,37,100,217,147,53,156,155,79,240,214,219,111,115,215,157,119,240,169,159,249,12,177,88,140,114,165,204,
  203,47,189,188,44,51,218,20,96,191,135,215,168,150,189,187,38,245,247,246,253,223,43,125,241,208,240,48,31,249,232,71,249,250,87,191,202,228,196,228,
  53,187,136,61,123,247,6,76,217,135,63,250,81,14,221,122,43,241,120,188,233,68,116,173,237,196,241,227,184,142,67,91,103,7,207,61,255,60,135,15,
  31,230,209,71,63,129,101,154,204,204,204,46,12,114,92,92,199,101,112,112,144,27,247,220,200,186,117,27,105,111,139,161,170,33,68,73,66,16,253,129,
  38,72,136,254,142,211,117,253,227,17,199,245,167,46,143,229,113,92,151,185,185,121,242,249,60,21,189,66,71,123,7,185,108,150,111,127,251,219,65,108,
  226,229,216,178,21,233,4,214,47,238,87,168,99,119,45,109,124,124,156,87,95,121,133,129,193,65,14,221,122,43,119,223,115,79,240,35,192,117,111,239,
  137,137,9,18,243,243,152,182,205,247,159,250,62,233,116,154,143,127,252,227,204,204,206,80,242,203,226,121,147,177,76,91,123,27,27,55,109,100,255,77,
  251,233,31,24,70,81,66,184,142,227,29,125,89,38,142,237,224,56,254,180,42,136,158,160,189,227,212,176,15,222,173,22,37,137,84,50,69,161,88,164,
  88,44,208,217,209,201,197,11,23,22,88,146,6,245,189,133,102,237,238,191,86,175,84,120,253,245,215,233,234,238,230,173,183,223,226,196,137,147,108,217,
  178,153,117,235,214,145,76,38,23,98,91,29,7,85,243,22,163,187,239,190,155,27,111,220,67,87,119,63,146,40,225,216,182,15,44,28,127,141,16,17,
  4,201,247,197,93,4,204,241,89,45,199,117,73,36,18,148,202,37,4,4,34,145,8,223,121,226,137,64,231,176,17,0,186,156,47,141,116,8,235,37,
  158,26,245,217,43,138,173,252,113,127,126,92,92,121,45,14,180,222,1,31,202,229,50,111,190,241,38,107,214,174,101,106,102,134,103,158,125,150,129,254,
  62,54,111,217,76,54,151,37,151,207,121,71,190,254,199,57,174,195,190,155,246,177,251,134,221,116,118,118,19,137,68,81,100,197,175,88,228,141,11,47,
  115,90,244,53,209,189,49,179,128,81,188,185,176,80,44,146,76,166,40,151,43,128,75,95,111,47,79,125,255,251,140,141,141,93,85,191,170,254,62,59,
  59,203,217,51,103,232,27,24,224,123,223,255,62,71,143,30,229,190,251,239,195,48,42,204,205,206,99,219,142,119,56,224,19,12,177,88,140,7,30,124,
  128,237,219,119,209,213,221,135,166,133,252,143,244,252,16,5,25,81,16,17,69,5,23,1,199,177,252,57,96,33,249,78,20,69,242,197,34,137,68,130,
  116,38,75,44,22,195,48,12,190,250,213,175,174,120,94,111,52,78,94,122,241,69,218,59,58,152,157,155,229,201,239,63,9,130,192,67,15,125,144,217,
  153,89,10,133,66,48,94,171,71,241,251,246,237,227,246,219,110,103,213,234,245,132,35,237,222,169,141,239,139,40,74,136,162,140,40,200,8,162,84,51,
  222,157,154,118,241,230,174,201,201,41,114,249,60,166,105,210,21,239,226,237,183,222,230,205,55,222,92,89,95,191,204,26,213,80,125,226,93,94,163,90,
  246,227,219,240,200,8,31,255,196,39,248,230,55,30,103,106,106,106,197,239,187,166,233,214,177,120,156,67,183,222,186,232,239,143,63,250,40,161,80,168,
  233,123,126,246,31,255,99,62,254,232,163,196,227,113,30,122,228,17,246,236,221,27,176,149,213,223,175,167,133,195,97,0,50,217,44,111,189,245,54,111,
  189,245,54,33,85,225,198,221,55,50,54,62,190,40,110,40,20,10,113,251,29,183,115,239,189,247,178,113,253,6,250,251,250,104,107,111,67,81,20,36,
  81,244,246,200,78,245,72,199,159,140,37,25,81,82,17,196,197,11,105,56,20,38,172,105,56,142,195,212,212,52,39,78,158,8,130,186,21,85,69,81,
  213,32,246,170,182,166,112,109,76,84,240,119,147,234,23,245,117,164,107,255,118,223,133,106,25,161,80,136,91,110,189,149,117,235,215,147,73,167,249,225,
  15,126,16,252,92,111,32,122,240,208,33,194,225,48,55,238,221,203,235,175,191,193,204,204,44,189,221,93,132,180,144,39,102,239,223,19,89,150,25,26,
  28,228,192,129,3,220,176,235,6,52,85,69,81,100,20,89,246,202,28,250,247,208,113,28,108,255,199,139,201,18,17,68,25,161,102,72,57,174,139,166,
  106,132,67,33,36,81,36,157,206,112,234,204,105,186,122,122,80,84,149,205,91,182,160,170,202,226,182,171,137,185,163,190,221,93,151,206,206,78,58,58,
  59,137,68,34,232,134,193,165,139,151,112,29,135,206,206,78,92,199,197,50,205,32,126,85,81,20,54,109,220,200,237,183,223,193,64,95,63,178,44,19,
  14,107,200,178,236,43,45,120,44,132,227,216,65,60,169,7,76,101,68,81,94,180,180,8,162,72,36,20,70,85,20,44,203,98,114,106,138,137,201,73,
  6,6,7,81,21,197,43,59,90,95,137,229,50,190,52,234,163,66,3,134,184,81,159,125,191,44,83,194,143,249,252,59,109,197,98,145,217,217,89,94,
  125,237,53,82,201,36,35,195,195,68,194,17,239,88,214,159,214,92,215,101,112,104,144,79,125,250,211,236,217,125,35,253,125,125,244,246,247,17,10,135,
  61,160,35,8,117,179,149,136,40,72,136,178,134,40,201,75,200,240,206,246,14,68,65,160,88,42,113,254,194,5,70,199,198,188,120,225,218,254,180,130,
  49,82,59,47,118,117,119,131,235,162,106,26,199,142,31,231,194,133,11,72,2,12,246,15,48,63,151,168,169,55,239,205,235,55,238,190,145,15,62,244,
  16,241,206,24,178,44,161,40,178,119,12,239,251,226,109,62,109,127,220,219,128,128,208,96,94,87,20,149,176,230,141,247,76,38,195,249,11,231,41,20,
  10,254,73,137,187,48,175,215,92,127,253,188,94,171,207,25,9,135,137,248,197,87,58,99,49,142,28,57,74,50,145,36,222,217,73,44,22,163,88,42,
  6,99,195,197,101,104,104,136,135,30,126,152,27,118,237,34,28,10,161,106,10,178,36,250,155,77,111,67,96,251,225,8,142,99,227,184,46,130,32,35,
  72,242,146,164,175,182,104,27,138,36,1,48,57,61,205,133,139,23,81,20,197,123,174,173,109,137,31,151,93,163,154,109,196,222,67,107,84,203,222,61,
  187,102,199,244,161,80,136,95,249,213,95,245,118,113,254,241,235,224,224,32,123,246,238,101,96,112,144,191,252,243,63,95,18,39,56,56,56,200,186,117,
  235,248,95,255,227,127,112,241,194,5,126,230,115,159,227,173,55,223,228,107,95,249,10,31,127,244,81,182,109,223,190,226,234,53,87,99,221,221,221,252,
  63,255,230,223,240,214,91,111,82,40,150,248,214,19,79,144,201,100,216,183,119,15,137,196,60,149,114,37,24,228,225,112,152,93,59,119,178,121,227,166,
  133,155,39,121,25,200,146,36,249,199,11,222,228,102,219,182,23,127,84,141,211,115,5,4,65,197,21,12,92,127,231,42,136,2,241,120,140,124,161,128,
  110,24,140,141,143,179,109,203,86,14,29,58,196,234,181,107,120,235,141,55,57,118,236,216,146,163,28,161,201,17,85,195,99,207,218,193,94,123,252,91,
  115,28,242,78,237,64,215,173,95,207,199,63,241,9,98,241,56,63,252,193,15,120,241,133,23,222,177,184,209,131,135,14,241,249,47,124,129,143,125,226,
  227,188,240,226,75,188,245,246,219,184,142,195,237,119,220,193,252,252,28,166,97,6,140,232,150,205,155,217,186,117,43,157,29,157,193,251,85,69,66,213,
  212,0,196,185,126,194,154,109,89,216,150,237,51,231,46,184,34,130,168,128,235,177,38,94,204,169,64,111,95,47,185,124,158,82,185,204,249,11,23,233,
  239,235,227,209,79,126,146,112,56,140,162,40,28,175,105,231,0,136,85,39,249,58,25,163,123,238,185,135,77,91,182,112,238,220,57,78,159,57,77,34,
  145,96,96,96,128,72,36,74,62,159,243,216,23,1,20,85,101,237,218,53,236,216,177,3,65,16,209,117,29,85,213,208,84,21,45,20,66,46,21,61,
  96,225,179,66,118,181,191,6,25,237,18,130,164,224,58,22,174,235,197,110,106,97,141,88,188,147,217,217,121,50,217,12,147,147,19,108,223,182,141,125,
  251,246,33,74,34,127,243,63,254,231,66,127,170,61,162,107,224,203,162,126,89,199,120,46,215,151,107,143,240,4,222,185,138,75,215,26,132,214,191,246,
  221,246,227,179,159,251,28,27,55,110,228,240,225,183,25,159,156,228,212,169,83,116,247,116,19,137,70,201,229,178,94,28,181,15,68,87,175,90,197,205,
  55,223,76,123,36,186,48,255,171,42,154,22,66,86,74,193,92,232,56,46,118,21,196,249,181,112,5,65,241,40,16,219,12,98,50,67,225,16,177,120,
  39,201,100,154,98,177,196,216,248,56,155,54,108,96,251,142,29,116,116,180,243,247,127,247,229,165,125,170,6,236,212,207,95,225,112,152,255,243,95,253,
  43,38,198,199,73,166,82,252,131,63,175,111,218,184,1,203,50,131,42,98,181,243,250,206,29,59,23,205,235,170,162,32,201,242,226,121,221,89,152,215,
  157,186,121,157,154,121,189,189,189,141,104,52,66,46,95,96,108,124,156,129,254,126,14,28,60,200,200,200,8,103,207,158,225,229,23,95,106,56,46,234,
  67,90,92,65,224,224,45,183,240,216,167,62,197,225,35,135,73,167,51,156,60,117,138,80,56,196,240,200,48,233,84,26,203,180,130,53,170,187,187,155,
  91,14,29,90,52,119,133,84,5,185,186,70,177,120,141,178,131,77,168,139,128,132,32,200,184,174,55,222,29,215,59,174,239,235,235,101,114,122,134,98,
  161,200,249,11,231,185,245,224,33,30,252,224,7,25,30,25,230,239,254,230,111,41,20,10,139,11,81,44,183,70,53,136,21,191,146,241,222,178,159,108,
  187,102,204,232,182,237,219,1,248,157,223,250,173,224,177,147,39,78,240,199,127,240,7,12,14,14,6,207,47,2,176,62,43,121,242,196,9,214,173,95,
  79,40,20,10,146,152,50,233,116,240,252,245,178,82,169,196,203,47,189,68,79,111,31,147,83,83,204,204,206,226,88,22,183,221,122,27,179,179,179,139,
  24,209,109,91,183,45,154,176,192,203,154,215,84,21,73,246,142,111,240,153,81,199,113,188,157,180,237,248,193,253,222,20,35,138,74,192,70,225,186,196,
  227,93,40,170,234,31,145,85,24,29,29,165,189,163,131,185,217,57,18,243,243,151,101,154,234,119,160,52,216,129,54,123,252,157,182,117,235,214,145,206,
  100,248,157,223,254,109,158,254,193,15,22,1,209,88,60,206,224,224,224,117,251,238,151,95,122,137,63,251,211,63,229,252,249,11,156,59,119,142,114,185,
  76,71,123,27,219,182,110,37,147,201,6,128,44,18,142,176,113,227,166,69,147,57,120,153,167,154,166,34,43,138,191,32,83,195,140,250,33,28,142,227,
  29,215,249,199,120,30,179,232,221,255,246,182,246,224,116,192,48,12,70,71,199,200,100,50,252,232,71,63,226,216,209,163,75,107,112,55,104,243,170,61,
  247,252,243,156,56,126,156,138,174,51,58,54,134,237,216,12,12,244,211,22,141,80,44,149,252,35,55,151,225,193,33,182,108,222,28,124,150,97,154,24,
  166,142,44,75,158,47,178,183,64,9,174,224,37,101,249,139,146,227,56,1,189,33,137,10,162,164,4,215,161,200,10,157,29,49,68,81,196,50,45,18,
  201,36,233,76,134,185,185,57,158,126,234,7,11,253,178,238,250,3,145,126,255,247,37,71,170,205,250,108,51,134,191,142,33,125,47,45,85,87,123,61,
  239,182,31,231,206,158,197,118,108,186,186,123,56,119,238,28,133,124,158,53,171,86,211,221,213,69,177,88,10,238,251,208,208,16,251,111,218,191,100,140,
  40,138,132,170,170,200,146,140,44,201,8,1,235,238,31,205,215,108,116,36,81,1,81,10,250,137,162,170,196,98,113,92,192,178,109,230,231,231,41,150,
  74,132,67,33,46,94,188,180,184,79,53,97,74,235,111,230,143,158,126,154,206,152,87,154,115,106,106,10,81,16,216,183,119,223,66,98,143,63,230,247,
  237,221,215,112,94,87,53,5,213,215,129,14,230,117,219,99,21,109,203,198,245,143,197,5,65,64,146,52,4,81,14,198,106,180,173,141,246,246,14,4,
  65,160,92,174,112,233,210,40,134,105,50,55,59,203,84,53,188,173,206,151,134,76,169,235,50,62,54,198,75,47,190,72,103,44,198,171,175,189,70,177,
  88,164,175,167,151,237,219,182,49,63,63,143,227,58,56,174,67,127,95,127,195,118,145,101,145,112,36,140,36,201,129,154,67,173,31,142,189,224,139,40,
  42,136,146,10,213,181,12,24,28,28,66,146,36,108,219,38,153,74,115,225,210,37,68,81,228,232,225,195,24,186,126,93,214,168,229,198,123,203,90,204,
  232,101,45,30,143,123,165,60,235,216,174,233,233,105,46,94,188,72,60,30,95,242,158,106,101,154,67,183,222,202,186,117,235,2,96,26,139,199,217,179,
  119,239,117,63,186,45,151,203,252,247,191,254,107,62,250,241,143,115,236,216,49,242,249,60,171,86,173,34,30,143,113,242,228,41,63,99,82,98,112,112,
  144,173,219,182,53,102,132,195,170,119,212,43,121,199,48,150,101,99,90,22,150,237,103,221,187,46,130,47,247,36,10,222,113,190,105,86,176,108,139,112,
  56,76,188,179,131,74,165,130,101,89,156,191,120,129,181,107,214,112,236,232,81,114,217,108,99,166,165,102,71,45,212,72,153,8,254,78,186,161,206,94,
  221,238,179,150,29,125,167,118,159,111,190,249,38,79,255,224,7,139,152,244,109,219,183,179,103,223,62,214,173,91,199,15,127,240,3,166,167,167,175,219,
  247,159,62,125,154,155,14,28,96,116,108,20,128,173,91,183,34,74,18,197,82,49,72,86,90,183,110,29,177,88,103,195,247,171,170,140,170,170,193,34,
  107,219,94,54,189,109,89,222,198,163,70,31,83,20,69,36,81,192,182,189,35,115,79,88,190,131,148,223,223,103,231,231,200,100,179,204,205,206,46,201,
  60,95,196,86,215,85,44,18,4,129,185,217,89,78,158,56,65,71,60,198,196,196,4,61,221,221,244,245,245,249,137,85,21,28,199,161,175,175,159,141,
  155,54,18,14,135,3,133,7,65,240,178,252,101,25,194,225,16,138,162,6,236,144,237,120,153,196,86,149,225,197,247,69,20,144,197,16,182,32,160,235,
  101,68,73,162,189,189,157,142,142,14,175,178,78,46,199,236,236,28,182,105,49,51,51,179,68,186,73,168,99,62,107,251,108,45,251,211,84,91,181,9,
  163,223,168,207,174,72,60,254,61,192,130,174,244,179,222,105,63,94,121,229,21,102,103,103,217,180,117,11,103,206,158,37,28,137,48,50,50,130,235,111,
  218,29,215,161,45,218,198,174,93,187,104,239,232,88,242,126,73,18,9,71,66,200,138,138,43,120,33,42,182,227,248,167,68,94,108,165,139,235,29,123,
  11,16,210,34,88,102,5,195,208,145,68,175,95,181,183,181,145,47,20,72,103,51,76,207,204,96,232,250,34,205,81,151,229,117,120,171,255,151,139,37,
  158,248,214,183,112,129,137,169,41,74,165,18,67,3,3,196,227,49,202,229,202,66,140,104,103,140,213,107,214,52,188,31,138,191,105,19,252,227,117,219,
  118,176,44,27,219,94,24,239,184,158,242,132,32,8,72,162,134,235,216,56,182,229,235,14,71,145,36,9,203,178,152,79,38,72,38,147,140,94,188,200,
  120,93,28,172,80,87,50,182,150,233,117,241,234,205,207,205,207,243,232,99,143,113,234,244,41,66,161,16,35,35,94,41,223,66,161,224,215,145,15,177,
  106,213,42,122,123,123,27,246,167,112,72,243,194,3,240,216,234,234,88,175,230,66,120,115,151,55,180,36,73,69,20,192,48,42,1,115,220,217,222,78,
  202,47,125,61,51,59,195,240,224,32,175,188,252,74,115,63,170,107,84,13,75,186,100,188,55,91,163,154,141,255,22,67,218,2,163,43,181,116,58,205,
  161,193,65,66,161,208,34,64,26,10,133,136,199,98,188,213,160,36,98,38,157,230,173,55,223,228,161,135,31,6,188,227,253,74,165,194,175,255,230,111,
  6,127,95,47,11,251,177,56,133,66,1,65,20,153,79,204,3,46,91,182,110,37,147,201,80,209,189,133,189,167,187,155,27,118,237,66,83,21,220,6,
  75,132,36,9,168,154,28,100,79,219,182,229,233,212,153,213,224,118,63,176,93,16,17,37,17,1,17,219,177,192,178,16,69,145,120,60,78,58,147,161,
  104,91,20,138,37,230,19,9,122,123,123,201,102,179,68,163,81,191,226,79,13,72,169,1,165,181,3,212,93,41,32,125,151,6,123,181,54,251,182,237,
  219,61,16,234,199,3,87,42,21,190,253,196,19,188,85,147,105,122,45,109,211,230,205,108,218,188,137,100,34,201,244,204,12,233,84,154,104,36,194,238,
  221,55,6,199,92,2,2,241,88,156,205,155,55,45,202,62,95,212,206,162,199,142,138,162,136,227,56,158,172,147,175,71,232,84,181,251,68,15,136,138,
  162,232,197,93,154,58,182,173,35,8,2,177,88,12,101,114,18,203,178,168,84,42,164,82,73,86,175,94,77,165,82,33,155,205,50,63,55,183,40,19,
  117,9,40,21,4,182,110,219,70,62,151,163,51,30,103,118,110,14,211,48,232,239,31,160,45,218,70,177,228,73,186,40,178,204,186,117,235,104,111,107,
  15,22,31,65,116,16,28,239,218,44,203,68,16,188,133,22,240,25,18,203,103,72,106,250,171,232,245,87,73,16,145,165,48,186,94,6,23,84,77,37,
  214,217,233,213,183,182,108,178,185,44,171,70,134,233,244,99,216,70,71,71,23,124,240,251,97,163,62,187,100,129,90,65,136,201,74,142,240,132,119,144,
  77,185,158,227,229,157,242,67,16,4,186,186,187,72,37,83,172,94,179,134,169,169,105,138,197,2,253,125,253,116,117,117,81,40,20,48,45,47,19,126,
  231,206,157,244,246,244,32,138,194,210,49,34,248,225,44,170,236,203,114,121,253,202,246,251,85,32,241,84,51,70,20,53,140,97,234,126,38,122,136,88,
  204,211,248,53,12,147,108,54,195,96,127,191,199,52,70,163,20,75,165,197,125,170,102,92,212,206,111,161,72,132,158,238,110,198,199,199,137,117,117,241,
  230,219,111,7,243,186,32,8,148,202,37,28,199,65,150,100,110,185,229,150,166,243,186,40,10,168,106,221,188,110,85,231,117,199,7,214,162,159,208,228,
  105,13,139,146,12,150,133,32,8,180,181,181,19,210,84,138,182,21,104,75,247,118,119,35,43,10,209,104,148,76,38,211,116,156,187,254,255,61,61,221,
  254,28,182,133,177,241,113,82,169,20,3,253,3,172,95,191,129,108,46,135,97,26,8,8,172,89,179,154,117,235,214,34,203,82,67,95,20,197,155,187,
  170,114,105,118,237,220,85,51,230,37,81,242,219,70,197,180,76,92,203,70,150,101,122,251,122,201,228,178,56,142,77,62,159,39,95,40,208,214,222,238,
  87,168,242,228,14,105,180,137,190,194,13,232,213,140,247,150,181,192,232,18,59,121,226,4,15,61,242,8,63,243,249,207,7,160,50,20,10,113,232,214,
  91,9,133,195,94,197,150,6,246,181,175,124,37,0,157,85,102,236,107,95,249,10,211,211,211,1,128,185,30,118,240,208,33,62,241,201,71,201,231,11,
  60,254,248,227,148,74,101,250,122,122,89,189,106,21,169,84,10,199,207,180,236,232,136,209,222,222,137,237,120,187,250,250,177,32,10,66,16,251,231,186,
  46,166,105,98,90,22,182,47,52,44,138,2,162,36,33,139,18,162,44,33,137,34,8,46,150,229,137,17,71,219,218,104,111,107,67,175,232,152,182,205,
  220,252,28,123,118,223,200,174,27,110,32,153,72,240,236,51,207,44,1,41,141,38,46,161,17,32,125,15,13,246,109,219,183,243,208,195,15,19,243,25,
  244,111,63,241,4,131,131,131,196,226,241,235,186,233,56,120,240,32,7,14,29,100,110,110,142,175,127,253,27,20,75,37,110,218,183,151,158,158,30,70,
  71,47,97,59,54,138,172,16,137,182,227,34,99,24,54,178,44,34,138,117,32,71,128,246,182,48,178,44,123,186,179,186,238,101,213,59,14,162,224,183,
  177,172,120,226,241,146,232,79,236,144,203,90,152,182,73,180,45,74,111,79,55,211,51,179,56,142,203,124,34,201,46,63,38,110,102,122,134,249,217,133,
  146,129,205,218,239,238,187,239,198,5,46,92,184,192,201,83,167,208,84,149,254,190,190,128,229,119,29,7,85,211,80,85,13,219,118,17,45,7,65,17,
  16,252,48,17,183,154,88,231,218,132,67,26,130,47,195,99,154,158,94,106,85,11,82,20,165,58,63,68,52,61,68,185,92,68,18,37,58,58,218,9,
  105,26,165,178,39,117,147,203,229,185,245,214,91,61,89,155,130,39,152,125,185,62,187,18,64,122,185,13,212,114,125,86,168,173,85,254,62,0,159,151,
  251,206,107,229,71,35,31,254,143,255,235,255,34,18,137,240,242,203,175,112,226,212,41,20,89,97,181,95,222,51,155,205,226,58,46,154,22,34,30,239,
  66,148,100,28,199,93,58,62,0,89,150,8,135,195,254,166,199,63,33,178,236,32,118,58,232,87,190,118,175,40,138,24,70,25,189,82,70,150,36,58,
  59,58,72,167,82,20,75,101,79,125,162,84,226,131,15,63,132,166,133,248,225,15,126,64,214,79,52,108,184,57,247,251,199,170,85,35,252,250,111,252,
  6,133,66,129,231,159,127,129,116,58,29,204,235,134,97,248,250,162,208,222,222,78,87,87,79,211,121,93,16,240,147,23,21,255,196,203,211,88,181,107,
  129,168,236,207,235,146,55,102,16,28,111,94,55,29,66,161,208,162,121,61,157,73,179,109,203,22,54,61,242,8,229,114,153,239,126,231,59,203,142,17,
  92,151,47,126,233,75,108,222,178,133,169,169,105,254,238,203,95,198,182,29,86,175,90,69,87,87,23,115,179,179,56,142,131,166,134,232,236,140,163,133,
  194,13,219,165,186,70,181,181,69,17,69,17,219,87,209,176,108,203,255,206,133,49,47,203,146,159,105,47,98,219,150,39,77,8,116,118,118,18,9,133,
  40,150,202,84,116,131,100,42,201,173,183,221,70,56,28,230,232,145,35,156,59,123,118,137,104,255,59,61,222,91,214,2,163,129,85,42,21,254,242,207,
  255,156,159,249,220,231,248,153,207,125,110,17,35,214,40,121,169,202,154,14,14,13,5,127,175,91,191,62,120,207,245,4,162,0,71,14,31,38,28,9,
  51,50,178,138,249,68,2,199,113,216,178,117,11,225,112,152,153,153,233,160,106,133,36,171,100,178,69,239,152,86,81,80,84,201,63,114,175,21,87,86,
  124,166,201,197,208,13,44,203,196,5,79,72,88,150,144,100,175,210,133,36,123,147,176,36,139,152,166,129,105,24,40,138,66,60,30,247,228,51,74,101,
  210,153,44,229,114,153,201,137,113,146,137,228,226,93,102,253,128,175,57,18,89,82,193,230,61,54,216,7,7,7,9,133,195,139,146,151,238,190,231,30,
  98,13,194,55,174,165,253,247,191,254,107,190,253,196,19,60,240,208,67,164,210,105,66,154,198,206,93,59,201,102,50,148,202,30,219,39,203,10,225,80,
  152,76,38,143,170,122,149,85,52,77,65,85,100,100,89,12,238,153,23,111,169,97,89,22,134,110,4,229,95,101,73,70,150,101,100,69,70,150,170,96,
  78,242,143,190,68,159,65,151,232,238,238,38,145,76,98,154,22,133,66,129,100,50,201,75,47,190,72,217,151,149,18,26,48,61,181,139,212,95,255,183,
  255,198,192,224,0,131,67,195,20,11,5,98,177,24,29,29,237,148,203,101,12,195,192,117,33,164,133,169,84,76,114,249,34,170,234,85,86,81,85,5,
  89,22,144,92,1,4,215,103,62,189,255,45,211,244,170,231,184,110,176,32,73,178,236,139,96,75,62,32,21,209,245,16,229,82,49,216,124,69,163,81,
  42,122,133,138,174,251,186,138,231,152,158,154,34,151,205,45,238,99,213,141,210,229,0,233,114,155,165,101,54,80,87,10,190,86,2,234,222,139,139,157,
  208,36,185,227,90,92,255,215,191,250,85,134,71,70,144,100,133,84,58,77,60,30,103,211,166,141,148,203,101,116,93,15,66,77,74,101,147,92,174,132,
  170,42,40,138,132,34,123,114,118,66,112,122,32,18,141,104,254,241,180,137,105,152,56,174,87,93,71,150,36,100,89,70,146,100,68,209,43,65,43,10,
  34,29,29,157,204,87,74,136,130,72,36,26,165,179,179,147,82,217,211,194,44,20,10,204,205,205,145,207,230,40,22,139,75,139,39,52,232,87,137,68,
  146,47,255,237,223,177,122,237,26,146,169,20,165,114,153,155,246,237,245,10,141,164,83,30,139,7,116,118,198,73,36,179,126,248,141,130,162,202,40,178,
  7,150,171,254,168,170,76,56,28,2,188,170,123,166,97,250,190,200,190,22,167,183,249,244,54,110,30,43,108,249,243,186,36,75,244,246,246,82,40,22,
  177,139,222,134,173,88,44,50,53,57,225,43,19,92,102,94,7,254,238,111,254,150,27,247,238,97,104,120,132,177,177,49,218,162,81,118,236,220,65,177,
  88,160,80,44,226,58,94,120,145,237,8,36,83,57,84,69,70,211,60,245,15,111,30,90,232,7,237,109,33,175,212,169,101,162,235,158,38,169,32,138,
  254,26,229,197,249,74,146,136,36,75,254,102,1,138,133,92,16,194,212,221,221,67,165,50,137,101,154,100,50,89,146,201,36,211,83,83,36,19,137,37,
  190,252,216,27,208,31,99,188,183,172,5,70,3,102,243,119,126,251,183,3,224,81,41,151,151,141,3,28,28,26,226,103,127,254,231,155,126,86,51,16,
  123,45,44,153,76,242,237,111,61,193,125,15,60,64,50,149,100,112,160,159,245,235,215,83,41,151,49,204,5,225,239,80,40,68,42,153,66,148,68,175,
  84,98,200,207,72,86,21,52,205,155,148,69,81,32,28,14,225,216,54,21,189,226,149,90,83,61,241,100,197,159,128,171,140,128,40,137,56,182,77,169,
  88,12,22,142,142,142,78,98,157,157,148,74,101,10,197,2,165,82,137,83,39,79,45,74,240,88,116,212,89,59,17,55,59,254,188,78,139,251,143,195,
  156,199,226,113,175,192,192,61,247,112,242,196,137,235,158,160,86,219,214,133,124,158,92,62,199,224,224,0,109,109,237,76,140,143,123,236,55,94,220,151,
  40,73,228,114,89,36,127,209,148,21,5,85,85,136,132,195,68,35,26,178,236,129,203,72,36,140,161,235,232,134,129,42,43,190,12,151,226,181,179,236,
  29,235,121,96,79,240,193,156,20,148,137,141,70,219,136,117,118,50,151,72,82,174,148,189,99,67,191,31,52,170,188,83,63,177,231,115,57,84,85,37,
  222,213,141,101,91,172,91,183,14,23,79,142,199,177,253,140,247,80,136,114,169,132,101,89,94,201,66,69,69,211,52,52,77,165,173,45,66,36,172,33,
  72,2,162,36,224,58,142,31,142,98,163,42,170,191,144,41,65,127,245,84,34,64,18,37,12,195,32,35,164,188,76,125,69,33,30,143,81,44,149,208,
  117,221,3,245,142,67,54,147,109,234,7,13,142,87,23,197,250,53,97,244,175,245,6,234,39,137,85,185,86,190,188,252,210,203,12,14,14,178,247,230,
  155,209,245,10,91,183,108,161,187,187,155,169,169,169,64,27,83,11,133,40,151,75,216,182,237,141,17,69,241,164,131,84,15,0,133,52,5,65,32,72,
  144,211,43,58,182,109,33,75,50,138,63,150,36,89,246,54,227,213,16,16,177,10,152,4,16,61,38,178,171,171,139,84,58,141,97,24,148,202,101,198,
  199,198,23,197,35,95,110,163,147,76,36,120,234,169,167,56,120,232,16,150,227,16,10,105,12,14,14,162,87,42,232,21,61,72,94,106,107,107,95,52,
  222,189,13,168,74,88,243,54,162,213,241,174,105,42,142,109,99,24,58,224,129,114,69,81,80,100,197,7,215,158,15,130,40,226,58,14,5,89,9,174,
  47,18,137,210,209,222,78,165,162,163,235,222,207,244,244,12,217,108,118,233,188,222,192,159,241,241,113,198,199,199,249,223,126,237,215,72,165,83,172,95,
  191,142,246,182,54,230,102,231,130,118,169,74,69,101,51,158,47,138,170,162,169,42,170,166,160,169,138,55,222,253,120,80,69,145,49,12,143,44,145,68,
  17,85,213,60,224,90,109,23,95,43,91,18,197,64,138,106,97,141,106,39,149,14,83,42,149,41,149,75,100,50,25,198,70,71,27,174,81,77,199,123,
  179,53,106,25,107,177,163,45,48,122,213,54,48,56,200,244,244,52,161,80,136,61,123,247,178,213,151,103,106,196,116,78,79,77,241,151,255,229,191,52,
  100,76,63,246,137,79,240,129,123,238,225,59,79,60,113,93,28,31,25,25,33,28,9,211,63,48,128,109,217,12,14,12,34,203,50,185,108,46,136,157,
  115,113,177,45,135,66,33,191,16,235,148,151,130,242,144,225,144,70,40,172,161,42,50,161,144,134,105,153,222,194,175,106,40,190,164,134,44,73,190,96,
  242,66,89,67,175,130,147,18,136,232,123,34,235,237,184,211,51,24,186,87,237,163,163,179,19,65,16,200,102,50,151,7,42,181,128,148,5,25,13,183,
  73,130,135,176,204,98,127,189,6,251,244,244,52,95,251,202,87,248,246,183,190,197,182,237,219,57,116,203,45,12,248,25,244,159,249,236,103,121,249,197,
  23,175,75,194,90,53,30,57,22,143,35,10,2,253,253,253,148,252,35,192,234,194,228,56,46,150,105,83,116,10,222,194,34,8,126,86,188,24,44,164,
  222,130,171,17,143,119,82,40,22,188,163,234,80,56,96,18,165,106,172,168,191,176,10,8,168,170,87,193,8,119,65,195,180,163,179,131,185,132,199,142,
  26,134,193,234,213,171,17,68,129,75,151,70,131,137,223,109,208,206,209,104,148,77,155,55,163,133,66,216,142,141,166,170,116,116,116,80,41,149,3,182,
  167,154,41,91,174,84,48,76,3,81,146,60,70,222,103,232,51,89,141,144,166,161,105,90,192,248,32,64,172,51,230,129,106,69,246,23,164,133,254,138,
  224,29,243,133,66,161,32,241,67,64,32,28,137,160,105,26,165,114,9,211,52,104,239,233,97,245,154,53,228,178,89,50,53,125,86,168,43,5,184,104,
  129,170,237,127,53,73,15,212,132,158,172,116,3,213,90,160,174,206,194,225,48,35,171,70,232,239,31,192,182,45,52,69,101,96,96,192,3,80,134,177,
  40,243,188,202,148,86,99,36,115,62,67,168,105,26,209,72,8,69,245,20,26,66,33,13,199,117,136,68,34,1,120,147,21,217,215,32,245,1,168,0,
  162,32,34,73,242,162,126,21,10,133,208,66,33,42,217,28,134,161,19,105,239,160,179,179,3,189,162,123,196,132,223,63,154,109,216,86,173,90,69,56,
  18,33,30,143,51,51,59,75,172,179,51,152,215,45,203,170,241,7,138,133,188,215,215,125,97,126,73,242,20,1,194,33,13,45,164,46,154,215,37,73,
  66,83,53,180,144,167,211,43,85,143,180,107,198,9,46,11,243,186,187,48,175,39,211,25,12,195,3,163,17,127,3,94,44,20,252,82,194,139,199,68,
  173,63,91,182,108,241,42,90,249,153,239,235,215,175,167,82,209,41,85,202,139,218,165,90,125,73,20,60,80,44,249,165,61,85,85,165,45,234,49,162,
  138,234,249,98,24,6,161,80,216,7,172,234,162,185,75,20,69,79,30,91,16,22,37,56,86,25,216,182,182,54,10,197,162,119,10,227,107,27,91,182,
  77,177,80,88,217,120,111,182,70,213,141,247,149,176,163,173,241,222,2,163,203,218,207,254,252,207,179,118,221,58,254,63,255,234,95,241,51,159,251,28,
  107,253,12,249,67,183,220,194,239,254,246,111,47,97,57,43,149,74,211,122,244,3,190,6,233,245,178,143,127,242,81,54,109,218,196,196,228,20,146,40,
  226,184,14,165,98,201,223,1,47,232,47,234,186,142,97,26,77,153,9,209,223,221,75,146,72,79,111,15,241,120,28,215,193,171,217,220,228,24,205,117,
  93,100,69,9,4,242,171,76,26,16,136,168,239,218,181,11,65,16,120,238,217,103,23,125,159,219,96,226,170,93,172,23,13,228,218,50,111,203,45,238,
  13,6,254,245,26,236,149,74,133,183,222,124,147,183,222,124,51,80,77,216,179,103,15,107,215,175,191,46,96,244,131,15,63,4,192,249,11,23,0,129,
  84,42,69,46,155,243,100,183,252,54,118,108,139,98,169,24,212,156,166,170,86,66,85,166,203,91,12,116,67,39,151,207,241,153,79,63,70,50,153,34,
  159,47,162,235,58,150,101,45,42,195,231,201,155,120,201,2,162,40,250,218,178,30,234,50,253,26,238,213,106,41,171,86,175,6,240,196,235,107,216,239,
  218,137,221,21,4,162,109,109,236,190,113,55,150,101,115,242,212,41,108,219,161,144,207,19,142,68,130,170,56,130,32,96,152,6,149,74,217,175,124,227,
  135,147,8,254,209,99,205,108,239,56,54,221,221,93,220,180,111,47,150,229,120,177,206,166,233,39,103,248,47,10,196,167,9,88,37,252,100,20,215,175,
  216,82,213,147,108,111,111,167,167,167,135,113,215,13,84,50,150,108,162,150,73,150,91,174,191,93,9,147,210,178,43,220,148,175,26,225,159,253,243,127,
  142,101,89,124,239,201,239,35,138,34,166,97,82,242,217,246,64,135,214,178,41,151,203,141,219,200,239,103,213,185,80,148,68,182,108,217,132,109,47,84,
  242,106,58,23,58,11,181,223,93,8,54,66,174,159,141,223,219,215,203,218,181,107,25,27,29,13,146,227,26,141,143,42,128,123,236,211,159,98,243,150,
  45,204,206,206,241,141,199,31,71,64,8,230,245,197,0,206,139,237,111,54,175,87,231,118,89,246,230,245,174,174,46,95,55,213,110,46,226,94,157,215,
  221,197,243,186,151,56,228,233,148,174,93,183,14,73,146,56,122,228,8,153,170,98,74,19,128,253,235,191,249,27,0,28,62,114,20,217,151,87,42,21,
  139,158,180,84,77,187,84,42,149,166,99,167,154,152,36,136,32,201,18,155,183,108,10,170,199,45,170,178,214,168,93,124,32,138,235,6,243,73,117,188,
  135,35,97,118,238,218,69,46,151,227,200,225,195,139,250,66,211,241,126,153,53,103,89,118,180,53,84,91,96,116,165,182,103,239,94,214,174,91,199,215,
  191,250,85,98,241,56,107,215,173,227,135,79,63,205,75,47,188,192,175,253,198,111,92,177,128,253,114,85,155,174,133,125,237,239,191,66,56,18,230,192,
  161,91,176,109,155,217,153,25,226,177,120,144,145,88,157,80,170,199,169,193,68,230,212,84,223,241,99,7,13,211,64,16,4,14,30,188,153,190,190,30,
  76,195,194,52,77,10,197,178,127,44,98,7,213,59,170,98,233,162,184,32,209,100,219,54,133,124,1,32,208,181,59,90,29,224,181,218,107,245,128,148,
  186,236,210,247,217,226,94,173,194,148,73,167,175,91,56,198,175,254,242,175,0,240,161,143,124,4,211,52,73,38,19,172,95,191,30,89,150,189,77,134,
  207,38,86,42,101,68,81,90,40,171,103,123,26,162,85,144,166,235,58,134,97,210,221,221,69,50,153,98,104,176,15,6,192,48,109,178,185,98,192,26,
  121,192,212,107,235,74,165,28,0,83,0,93,215,153,157,157,3,60,41,28,193,133,231,107,54,27,181,237,92,191,224,206,204,204,240,95,255,242,175,232,
  234,234,98,237,134,245,148,43,101,47,188,68,211,130,197,3,193,211,49,173,106,10,86,91,185,218,111,45,219,75,40,49,13,19,240,202,31,118,119,197,
  136,70,163,30,83,84,172,80,42,151,49,12,19,203,50,3,193,114,215,117,201,231,179,1,16,117,92,135,124,193,11,39,241,18,83,4,198,70,71,153,
  157,153,105,232,203,34,38,180,17,163,95,219,31,87,176,129,106,198,156,180,216,146,43,183,137,241,9,126,255,247,126,207,83,101,232,232,160,88,44,50,
  61,61,197,234,213,171,189,92,55,31,140,152,150,73,165,82,169,217,172,121,39,10,142,235,109,212,12,195,196,52,13,44,203,162,183,175,151,67,7,111,
  246,18,58,13,19,93,55,41,150,202,94,54,186,109,5,186,188,85,86,175,58,70,170,155,255,42,184,18,16,188,114,151,185,156,39,84,95,39,174,222,
  8,144,254,221,223,252,45,145,72,132,13,155,54,121,243,250,220,44,169,84,18,77,91,96,246,113,92,10,197,2,146,180,48,222,93,199,197,118,189,57,
  221,52,45,76,211,192,48,12,84,85,229,166,155,246,49,50,60,136,32,10,88,166,77,190,80,166,92,246,78,36,130,141,168,99,99,90,166,55,175,251,
  227,196,180,76,210,169,52,134,105,6,167,38,23,206,159,167,92,42,121,49,176,142,179,48,30,234,106,186,187,130,224,107,118,11,220,255,193,15,162,155,
  6,103,207,156,225,134,27,118,123,9,98,182,229,233,7,27,6,165,82,145,106,130,162,235,120,32,190,182,93,12,195,155,187,70,86,13,179,255,166,125,
  68,35,33,108,219,161,92,214,41,150,42,193,188,85,157,187,188,118,209,253,54,242,110,119,185,228,29,205,87,1,118,177,80,228,226,185,243,30,187,91,
  27,203,91,61,138,175,101,64,235,198,102,125,120,206,123,141,48,105,217,251,28,140,198,226,113,46,93,188,200,91,111,190,201,161,91,110,1,188,56,193,
  74,165,194,204,244,116,195,68,149,80,40,212,80,236,188,202,152,93,207,234,75,19,19,19,0,172,93,183,30,53,164,49,63,49,207,204,204,12,131,131,
  131,158,128,185,63,8,11,133,60,178,172,46,148,130,243,53,218,60,105,31,19,211,244,164,60,226,241,56,165,82,153,246,182,104,176,18,199,45,11,203,
  180,169,24,22,149,138,142,161,235,216,182,77,46,151,245,164,116,252,1,149,207,231,73,103,50,0,129,228,78,182,86,103,180,25,32,173,223,125,94,233,
  226,126,25,112,250,78,13,246,170,166,108,51,197,133,107,97,146,44,121,71,102,201,4,83,147,83,12,13,15,129,225,45,172,182,109,83,42,149,80,20,
  213,147,58,178,109,127,179,97,7,49,86,142,227,120,71,144,90,136,98,177,76,117,229,80,85,137,222,158,14,44,187,141,82,217,164,82,214,189,99,51,
  203,162,84,42,98,251,245,158,93,215,37,157,76,81,49,188,35,117,69,86,208,13,163,209,77,95,248,181,190,78,53,96,90,86,112,244,158,78,103,232,
  236,140,121,71,232,174,215,103,203,101,175,2,142,32,136,1,187,97,251,69,24,44,211,194,118,108,79,123,48,28,198,182,29,76,211,66,150,189,227,201,
  104,52,140,235,198,48,76,135,114,217,160,84,42,123,160,212,178,153,152,24,11,250,159,161,27,100,50,89,116,195,244,142,96,101,185,169,31,139,142,225,
  234,43,134,213,45,50,45,118,244,157,183,114,185,204,217,51,103,201,231,11,220,116,224,0,174,191,241,105,107,111,39,18,137,224,216,54,184,96,232,58,
  37,63,209,174,90,58,182,122,124,236,109,216,12,28,199,69,211,52,20,73,193,177,29,226,241,14,255,245,46,134,105,97,24,22,21,221,240,147,129,12,
  28,219,97,122,118,50,232,242,166,105,146,73,167,41,87,244,32,225,73,247,165,207,26,206,133,13,0,233,248,216,152,47,87,213,141,166,105,216,182,205,
  252,124,130,193,193,65,95,35,216,27,143,165,82,1,85,13,7,224,184,234,143,105,154,88,62,200,4,104,107,107,71,215,117,4,81,32,28,210,64,131,
  112,88,195,180,218,41,87,76,42,21,29,211,48,177,45,147,108,221,188,94,44,20,41,20,139,56,182,141,234,23,55,201,101,179,193,125,172,250,19,128,
  179,26,48,7,112,250,212,105,239,116,231,67,143,16,14,71,72,167,211,100,178,25,218,218,218,192,242,230,46,211,52,40,151,23,54,210,193,252,101,219,
  1,168,182,44,27,73,18,145,69,153,74,165,66,151,223,46,161,144,70,91,123,20,211,180,168,232,38,229,114,5,203,52,113,108,155,68,178,82,163,99,
  239,146,203,229,168,84,60,153,58,69,246,74,2,55,108,151,90,31,46,51,222,151,85,118,105,141,247,22,24,189,90,171,148,203,116,198,188,133,113,235,
  182,109,164,211,105,102,252,216,209,254,129,129,134,64,99,96,112,144,47,45,147,192,244,195,26,145,244,107,109,93,221,221,28,56,120,128,225,145,85,92,
  184,116,137,243,231,207,51,61,61,69,87,119,23,138,44,123,71,84,174,75,169,84,66,211,252,26,228,85,177,96,95,63,207,246,19,96,188,236,82,25,
  195,48,23,228,53,4,79,56,89,145,37,84,77,33,26,209,208,117,19,195,176,40,20,114,94,220,13,158,100,72,98,126,222,31,232,16,210,52,68,81,
  96,100,213,42,114,185,220,98,241,251,26,102,160,209,238,243,138,7,244,85,30,155,92,107,115,235,43,170,92,67,187,235,3,31,224,192,193,131,40,154,
  202,185,243,23,56,125,250,52,163,163,23,233,233,237,65,146,101,111,67,97,91,20,138,121,79,136,219,182,3,86,212,182,45,108,171,90,157,68,68,211,
  188,120,49,47,123,221,66,85,23,134,143,44,137,180,183,105,68,35,170,127,236,109,113,241,226,217,160,86,124,185,82,102,108,124,60,184,197,225,80,136,
  72,56,204,158,125,251,72,167,82,92,170,13,81,168,89,160,106,227,172,246,236,219,71,188,171,139,114,165,130,166,133,60,25,156,98,49,56,69,240,152,
  165,138,23,203,38,73,62,139,143,199,146,216,30,27,37,8,2,162,159,176,225,56,14,166,225,177,165,178,236,4,177,128,154,42,161,170,94,226,150,110,
  88,232,134,73,185,84,4,31,216,22,139,69,114,185,156,151,20,33,203,40,178,76,95,95,63,109,109,109,76,140,143,7,229,22,27,2,210,38,155,157,
  134,125,245,42,22,171,22,91,114,229,118,195,238,27,24,30,25,33,20,10,17,137,70,72,36,147,196,98,49,214,172,93,139,128,128,131,131,97,24,1,
  99,89,21,75,183,124,233,38,175,170,145,227,199,92,42,8,130,16,40,52,84,153,52,77,149,81,21,153,112,88,197,52,109,116,221,219,200,159,59,127,
  10,252,184,237,82,169,72,34,145,4,191,52,175,34,75,116,250,149,140,210,169,84,0,16,151,219,176,117,247,244,112,203,173,183,176,102,221,58,78,158,
  58,133,32,136,193,188,174,170,42,56,62,80,44,22,113,93,193,31,239,118,112,210,229,177,183,222,188,174,40,114,48,175,219,190,68,149,32,8,94,156,
  184,36,162,40,50,209,136,134,105,88,24,166,69,222,159,215,241,75,135,86,231,117,111,211,170,32,10,2,125,253,253,20,139,69,230,231,230,46,59,222,
  119,239,221,235,197,141,186,208,211,221,205,133,11,231,73,204,207,211,222,222,142,32,136,56,174,183,89,46,151,203,200,178,140,109,47,128,81,203,178,130,
  118,18,4,1,85,245,164,220,116,221,8,214,40,81,20,80,125,85,132,144,166,18,9,47,248,50,51,51,25,156,170,152,166,73,34,145,244,74,38,139,
  94,27,183,181,181,49,178,106,85,48,7,253,88,227,253,125,64,152,180,236,125,4,70,79,158,60,201,93,119,223,205,255,245,175,255,53,224,149,99,27,
  24,28,228,75,63,247,115,193,243,245,54,51,61,221,48,129,169,202,166,94,215,9,248,134,27,248,224,67,15,145,207,23,56,122,244,40,146,36,145,203,
  231,73,167,211,12,13,13,81,177,188,35,41,67,175,128,32,250,113,54,11,187,206,106,146,147,36,203,65,224,190,109,217,228,139,21,58,218,22,135,24,
  72,162,23,183,35,75,34,225,144,202,216,152,73,201,175,252,83,44,22,200,23,11,216,142,23,204,222,22,137,32,137,18,35,35,35,228,114,57,78,212,
  85,98,10,88,129,6,153,137,75,216,209,6,146,79,52,98,79,89,90,241,69,248,9,25,236,229,114,153,114,185,196,201,147,39,233,235,235,35,20,210,
  200,229,11,76,79,79,179,102,205,26,175,46,189,107,99,26,58,130,32,227,84,171,146,248,63,213,91,40,251,73,107,162,228,149,194,204,228,138,244,118,
  47,173,68,35,10,160,42,222,145,92,38,147,246,38,127,215,33,151,205,98,218,11,49,106,209,72,132,98,161,128,166,105,20,170,147,121,29,16,91,116,
  236,5,172,89,179,134,92,46,135,36,138,180,251,44,111,62,159,163,173,173,13,85,245,216,123,203,242,22,19,209,223,76,225,186,216,182,27,76,231,130,
  40,248,89,246,178,183,217,42,87,208,117,3,69,145,151,180,179,44,11,190,236,149,76,42,149,194,117,189,152,215,108,46,27,244,87,77,85,8,41,42,
  166,97,16,239,234,98,98,124,188,49,211,91,91,139,219,255,123,69,149,151,234,78,0,88,65,159,109,217,149,217,103,63,255,121,194,225,48,111,191,125,
  152,174,120,23,211,211,211,164,51,25,6,13,3,73,148,112,45,79,63,83,55,116,4,196,224,148,200,59,214,245,142,190,69,161,218,175,188,146,185,229,
  178,142,110,88,104,53,27,54,111,28,121,153,218,170,226,129,167,196,252,188,215,175,44,147,92,46,71,89,247,194,117,84,69,65,83,84,226,177,24,189,
  189,189,156,56,126,188,225,198,188,62,49,179,187,187,155,71,62,252,97,18,137,36,83,83,211,68,34,17,210,233,148,167,55,218,215,7,62,184,214,245,
  50,146,172,96,91,206,162,83,175,170,194,134,40,137,190,178,132,55,175,23,75,21,66,33,13,73,18,27,206,235,90,72,193,182,189,121,221,113,93,42,
  149,202,162,121,61,172,133,112,93,151,129,129,1,116,195,88,12,70,155,140,247,123,238,187,151,45,91,182,112,226,196,73,6,7,7,57,125,230,52,233,
  76,134,222,66,129,112,40,236,131,78,19,93,175,96,219,106,64,152,84,137,146,234,216,145,100,25,85,243,218,69,215,13,202,21,131,72,88,93,212,46,
  146,36,16,18,21,79,206,202,246,146,118,171,177,190,217,108,54,104,23,73,18,9,107,33,143,48,25,25,193,178,172,5,48,186,210,241,222,64,194,174,
  225,251,27,188,182,53,222,91,96,244,178,150,73,167,249,147,63,250,35,246,236,217,67,58,147,225,109,255,136,253,212,201,147,188,244,226,139,13,179,233,
  43,149,10,151,46,94,100,192,175,220,84,203,14,13,12,14,178,103,207,30,190,243,237,111,95,23,199,95,121,249,101,142,28,57,66,177,80,224,193,71,
  30,33,28,14,51,63,63,207,212,212,148,47,241,84,241,98,158,44,3,12,201,79,250,112,22,74,193,249,236,164,44,73,40,126,157,111,199,113,40,228,
  139,180,71,181,134,0,78,20,5,138,197,2,137,249,185,64,142,39,157,78,7,199,181,130,32,16,235,140,49,61,53,69,58,157,70,146,164,229,7,107,
  3,125,183,247,170,197,226,113,98,177,88,195,231,174,103,124,240,43,47,191,204,43,47,191,140,166,105,60,250,233,79,211,222,222,78,34,145,96,122,106,
  138,77,155,54,81,20,60,189,62,219,178,48,133,10,138,226,201,56,85,89,110,112,253,68,6,79,2,69,16,4,28,199,161,92,42,99,199,218,145,164,
  198,250,143,179,179,179,204,207,207,6,199,220,233,204,194,66,42,73,18,209,104,148,153,169,41,10,133,66,243,201,181,54,17,13,248,250,215,190,6,130,
  192,150,173,91,233,245,69,251,147,169,20,157,157,157,254,177,187,13,182,133,169,151,145,36,197,203,188,183,29,192,194,245,75,210,86,65,181,36,201,184,
  46,84,116,157,74,197,240,235,213,47,157,14,28,199,225,245,55,222,240,66,75,28,155,82,169,236,179,87,94,124,115,36,26,69,86,213,37,155,77,161,
  209,66,219,108,1,185,198,12,125,139,45,185,50,251,253,223,251,127,153,156,152,224,246,59,239,164,191,191,159,163,199,142,146,76,38,41,22,139,116,197,
  187,188,106,60,142,131,105,84,16,4,185,166,230,252,66,76,161,40,137,129,220,145,32,8,190,156,145,129,170,200,13,5,229,37,73,96,102,102,150,100,
  114,222,99,244,43,58,233,116,38,72,168,9,133,67,168,154,202,107,175,189,182,168,18,93,179,13,73,53,6,113,124,124,156,127,249,127,252,159,36,19,
  9,190,244,11,191,64,111,111,47,163,99,163,92,186,120,145,222,222,94,79,3,213,182,176,76,131,74,185,4,136,65,12,171,227,46,204,211,178,44,163,
  42,170,95,228,194,159,215,219,163,132,37,181,225,188,174,151,117,166,38,39,200,231,243,184,142,39,181,84,169,232,193,188,222,25,235,164,80,40,112,226,
  248,113,52,77,91,254,36,192,191,7,127,252,7,127,72,185,92,102,239,190,125,108,222,188,153,231,158,127,142,153,153,25,218,219,219,217,190,109,187,23,
  14,228,216,24,70,197,7,159,248,190,44,128,64,81,18,81,21,5,85,81,189,4,71,221,160,80,40,18,14,41,75,171,151,9,32,9,2,179,51,115,
  76,79,79,6,101,143,39,38,38,130,118,81,85,149,88,60,198,137,227,39,2,45,238,70,107,148,219,100,195,185,100,108,94,201,120,111,242,218,214,120,
  111,129,209,134,128,244,135,62,35,186,118,221,58,42,149,10,95,255,234,87,151,125,207,199,62,241,9,110,220,179,39,96,74,255,228,143,254,136,15,220,
  125,55,119,221,125,55,167,26,176,169,215,150,49,243,98,255,20,201,171,63,159,76,38,253,50,135,94,125,97,221,215,96,52,141,50,237,237,49,86,173,
  26,102,106,106,134,124,190,128,109,59,158,240,177,162,32,203,74,80,70,178,82,174,80,209,109,194,161,165,183,214,52,77,14,31,57,194,177,99,199,176,
  109,27,93,175,144,245,191,15,32,28,10,19,14,135,201,102,50,65,64,121,211,193,94,43,252,188,204,46,241,114,53,190,223,41,219,179,103,15,119,221,
  125,119,211,231,175,103,91,3,172,94,189,154,206,246,118,6,6,6,184,120,241,34,249,66,129,108,54,235,197,110,234,30,11,46,8,46,235,214,173,246,
  128,226,204,44,179,179,243,129,94,103,85,35,81,16,60,102,197,52,76,76,211,70,146,150,182,115,161,80,224,228,169,83,100,178,89,92,92,202,229,18,
  229,74,57,120,190,61,218,230,29,111,43,202,242,71,81,77,218,217,50,77,186,122,122,104,111,107,167,80,40,80,46,149,176,99,94,121,89,143,25,177,
  104,239,232,96,245,234,213,254,98,50,69,185,84,65,148,4,175,8,131,95,253,198,147,46,243,146,28,44,219,193,182,157,69,204,15,120,241,131,47,190,
  248,162,87,133,198,182,41,22,11,129,186,132,170,168,132,181,144,39,242,237,103,251,94,118,33,169,13,199,104,84,210,246,74,22,164,150,56,246,53,177,
  73,63,126,126,98,124,156,237,59,119,210,209,209,65,161,80,160,84,44,209,221,213,141,36,138,94,5,34,83,71,20,29,54,108,88,143,105,90,76,79,
  207,162,235,6,162,40,120,27,28,101,1,140,154,150,137,94,49,137,132,29,100,89,92,242,157,165,82,137,175,126,237,171,232,186,129,101,91,193,9,17,
  120,58,182,145,80,152,82,209,139,173,172,50,111,43,217,152,151,74,165,160,136,68,38,149,98,96,160,31,69,81,168,248,58,159,209,72,52,120,173,105,
  86,232,232,232,98,213,170,97,230,231,147,164,211,89,44,203,242,165,208,100,68,89,10,20,54,76,203,139,121,13,135,212,134,27,182,169,169,73,206,156,
  57,227,87,225,51,40,149,75,53,167,7,26,170,162,6,49,150,213,48,150,255,63,123,111,30,31,197,117,166,11,63,213,187,186,91,82,11,73,173,141,
  69,45,4,72,172,194,108,2,60,25,16,54,120,137,177,49,142,99,156,120,188,132,204,29,135,201,197,153,56,246,92,152,92,219,227,216,191,47,19,207,
  77,252,221,207,201,220,49,137,237,201,77,112,60,222,61,120,193,102,241,140,1,1,14,8,75,2,97,36,181,16,146,104,181,150,110,169,187,213,123,215,
  247,71,45,93,213,93,213,139,212,218,224,60,249,17,183,170,171,171,234,212,57,239,123,158,243,158,119,73,38,239,35,35,35,160,40,10,151,59,58,176,
  254,47,254,2,5,5,5,24,28,28,132,215,235,69,40,204,60,103,36,24,70,36,28,2,13,10,197,197,37,208,106,52,232,239,31,196,200,136,23,156,
  187,131,66,160,187,24,242,26,66,32,24,129,86,163,140,107,139,223,239,199,177,227,199,248,109,121,175,119,4,30,239,8,107,9,102,124,230,21,160,88,
  119,132,4,186,43,166,143,164,200,105,50,121,39,126,163,132,140,142,26,197,37,37,216,241,157,239,136,44,96,78,167,19,251,255,240,7,201,109,247,226,
  146,18,212,176,214,79,167,195,129,109,219,183,227,209,93,187,80,92,82,130,250,227,199,113,228,240,225,113,109,252,188,121,243,240,157,7,30,0,40,10,
  151,59,59,209,222,222,142,33,167,19,23,47,180,160,122,209,66,134,164,68,34,152,145,95,128,155,110,190,25,217,70,3,90,219,218,113,246,204,57,120,
  60,35,108,194,228,104,137,59,128,102,29,202,253,208,233,84,113,130,116,249,242,101,180,181,182,49,249,238,64,195,57,52,196,39,216,7,128,162,194,2,
  232,180,90,204,159,63,15,61,221,61,176,197,68,39,167,178,74,204,152,117,9,153,221,170,63,123,246,108,194,212,77,78,54,128,107,60,112,235,237,183,
  225,214,219,110,195,240,208,48,22,204,155,143,179,103,206,96,120,120,24,173,151,90,177,98,197,13,240,249,188,208,235,13,88,186,116,41,22,45,90,4,
  149,74,137,142,220,92,120,189,62,12,13,13,179,253,172,224,21,58,216,9,199,235,243,67,195,86,228,226,224,112,56,208,208,208,128,75,95,95,98,44,
  7,52,208,215,223,207,79,64,42,165,18,165,37,197,208,106,181,168,156,87,137,222,222,94,116,94,238,76,185,159,55,111,217,130,252,130,124,216,174,218,
  80,88,88,8,231,144,19,131,131,76,213,28,131,49,27,190,112,8,57,70,35,22,47,94,132,226,226,98,104,212,42,120,60,35,240,249,108,76,224,147,
  66,201,39,27,231,106,136,251,3,1,4,131,33,68,180,98,50,234,112,56,112,252,248,113,198,149,1,76,228,174,131,157,80,41,138,130,209,160,71,78,
  182,17,249,51,102,160,176,176,0,205,77,205,242,19,109,236,152,146,179,6,147,201,105,82,240,157,7,30,192,194,69,11,113,241,226,215,152,51,123,54,
  26,206,157,195,149,43,157,200,201,201,134,49,59,155,241,65,12,133,241,141,141,223,128,197,98,129,70,163,198,135,31,29,196,213,30,27,159,215,82,197,
  235,65,176,139,109,63,130,33,189,36,25,253,252,243,207,153,116,97,96,34,238,157,130,45,120,163,94,143,156,236,108,20,20,228,195,100,202,133,181,221,
  154,250,184,162,105,20,20,22,226,199,63,249,9,180,58,29,62,255,252,115,232,245,122,12,57,157,104,111,109,195,194,69,139,248,243,34,161,48,110,189,
  117,51,178,141,6,56,28,78,252,231,127,29,131,99,208,201,200,187,82,41,104,15,141,112,40,8,159,47,128,156,28,125,220,253,157,78,39,206,158,61,
  203,235,117,207,200,136,40,72,41,207,148,11,163,94,143,153,165,37,169,203,59,75,226,238,184,243,78,220,113,231,86,92,185,210,133,170,5,85,248,252,
  63,63,135,205,102,67,65,126,1,202,102,150,177,129,99,17,44,172,158,135,21,43,86,64,159,165,195,201,147,95,226,235,75,173,108,224,146,10,106,85,
  180,95,232,8,227,255,235,247,7,37,201,104,67,67,3,250,236,125,12,145,102,117,23,7,181,70,141,252,188,60,232,245,122,44,171,89,134,206,203,157,
  209,52,110,18,237,72,184,27,50,14,115,22,1,33,163,204,182,138,78,135,135,191,247,61,56,29,14,188,115,248,48,156,14,7,76,121,121,88,187,118,
  45,30,254,222,247,240,203,23,94,136,75,223,195,109,207,214,31,63,206,91,70,203,45,22,188,243,246,219,252,54,255,120,98,96,112,16,0,141,67,159,
  126,134,34,179,25,133,249,5,112,187,221,24,118,185,144,63,99,6,28,131,131,40,44,52,99,243,230,205,200,201,201,6,77,211,200,202,202,130,222,96,
  192,136,215,199,151,5,85,240,254,109,204,42,217,239,243,35,20,204,130,90,29,85,194,131,142,65,28,63,113,2,118,187,157,207,69,233,24,116,240,121,
  39,213,106,53,102,228,229,193,49,56,8,99,118,54,83,253,38,141,149,167,148,165,105,42,9,121,162,18,175,197,37,37,227,22,192,4,0,39,235,79,
  194,59,226,69,253,137,19,216,114,235,173,152,61,123,14,46,181,182,98,112,112,144,73,76,109,52,98,241,226,37,88,184,176,26,26,141,26,129,64,16,
  158,145,17,4,217,148,44,92,237,102,46,185,53,205,230,12,12,4,2,8,134,116,208,106,148,160,105,26,253,253,253,104,106,106,66,75,75,11,220,30,
  55,64,3,161,80,8,35,35,76,229,26,138,162,144,103,50,33,219,96,196,197,150,22,24,12,6,184,92,174,212,251,153,162,208,221,213,133,214,75,151,
  16,12,6,81,88,88,136,43,93,87,224,15,50,46,5,42,149,10,217,234,108,44,91,182,20,197,69,102,168,85,42,54,90,94,197,228,111,100,235,205,
  115,129,74,92,30,213,80,48,196,7,50,169,213,220,182,164,27,71,142,30,133,213,218,193,167,134,10,248,3,112,13,187,88,171,168,26,217,198,108,68,
  194,97,180,183,181,195,148,103,138,6,46,37,120,254,137,34,148,100,235,46,61,100,101,101,193,218,222,142,43,151,47,163,124,78,57,46,181,182,194,199,
  38,184,207,98,171,122,109,172,171,67,229,220,185,80,179,254,158,133,133,5,176,217,236,209,252,162,92,2,120,214,90,24,12,6,17,240,7,161,81,171,
  68,238,44,167,191,60,141,166,166,102,62,77,84,32,16,128,99,144,209,13,10,133,2,70,163,17,106,181,10,3,125,253,163,26,87,35,44,25,252,236,
  211,79,97,46,46,198,156,217,179,209,226,245,98,200,53,12,173,86,139,64,192,47,210,235,92,170,62,173,86,199,203,187,82,161,128,130,45,229,75,211,
  64,56,28,129,223,231,67,192,31,134,86,171,228,173,136,221,221,221,56,81,95,143,158,238,30,62,119,176,107,216,197,187,223,24,178,178,48,35,47,143,
  169,138,22,12,166,174,215,5,109,57,113,236,24,190,254,250,107,44,89,188,24,205,205,205,112,185,93,24,25,25,129,209,96,64,36,28,198,146,37,75,
  177,112,97,53,219,182,32,83,112,69,165,2,29,161,153,74,106,108,189,121,166,95,104,132,130,65,102,142,210,107,160,82,50,149,150,188,35,94,212,159,
  170,199,249,230,243,124,153,228,112,36,140,94,91,47,191,136,54,101,231,32,219,104,68,119,87,23,138,138,139,196,89,1,198,64,48,51,161,19,136,188,
  19,50,202,163,170,186,26,52,77,227,149,223,254,54,74,58,173,86,180,156,63,143,199,126,252,99,84,85,87,163,225,236,89,201,65,36,252,124,226,248,
  241,9,33,162,0,48,56,48,128,167,255,231,83,160,105,26,55,172,92,137,21,43,86,192,229,241,192,102,179,225,170,205,6,131,193,128,45,91,182,192,
  104,52,32,18,97,114,43,158,63,223,2,187,221,206,214,37,102,182,111,41,182,114,5,179,149,25,66,32,224,135,215,23,0,69,169,225,118,187,208,209,
  209,129,166,198,38,198,210,201,230,212,243,251,253,8,176,53,236,149,74,37,74,139,138,160,164,20,232,238,238,30,139,68,142,109,194,151,34,132,25,18,
  240,13,117,117,40,47,47,199,171,191,251,29,106,150,47,71,113,73,9,62,102,253,129,111,185,245,86,116,116,116,224,232,56,89,194,7,7,6,112,244,
  200,17,208,52,141,243,205,205,88,186,116,9,122,237,118,56,135,134,96,239,235,67,69,69,5,230,207,159,15,141,70,131,64,48,136,142,203,151,113,241,
  226,69,184,92,195,208,106,178,248,109,109,46,247,33,147,135,148,77,157,18,8,129,66,24,189,189,118,156,61,123,22,93,93,93,44,17,101,250,217,237,
  118,35,24,100,220,45,180,106,13,138,11,205,76,250,26,129,155,72,58,253,211,220,212,196,111,217,175,89,187,22,37,37,165,232,238,234,130,189,207,14,
  189,193,128,85,171,86,193,92,88,8,133,66,1,127,192,143,175,190,106,196,229,142,78,40,89,87,18,174,36,173,144,52,112,245,170,35,97,29,156,206,
  33,180,183,183,163,245,210,37,92,238,236,228,23,78,193,32,19,96,66,179,62,180,249,51,102,32,127,70,30,108,61,87,225,241,120,228,45,36,137,38,
  167,76,16,212,216,49,75,38,164,81,97,223,191,254,43,51,33,168,84,184,227,174,187,176,116,201,18,156,61,219,128,171,108,154,167,229,55,220,128,202,
  185,21,80,169,148,8,134,66,184,210,213,133,175,190,106,98,74,71,178,165,36,21,108,249,88,0,172,124,4,225,243,7,160,213,169,17,12,70,208,215,
  103,71,115,115,51,46,93,106,21,21,135,112,187,221,136,128,33,19,6,189,30,5,51,242,49,216,63,192,148,156,236,236,76,123,92,141,120,60,248,217,
  51,207,0,0,44,21,21,168,174,170,70,175,189,15,78,199,32,236,125,125,200,159,145,135,77,155,54,33,59,219,200,235,245,182,118,43,122,122,122,160,
  86,107,248,182,8,245,122,56,18,70,48,196,100,150,208,104,20,240,122,125,248,250,235,139,104,106,108,194,85,155,45,154,76,223,31,128,63,224,231,245,
  122,65,126,62,178,180,90,116,88,59,18,187,177,200,144,179,67,159,126,202,31,191,251,91,37,152,55,111,30,206,54,52,160,191,191,31,195,46,23,230,
  47,88,128,170,170,42,104,52,76,90,186,174,158,110,180,181,89,225,243,250,160,98,3,176,20,220,142,14,152,28,193,161,112,8,1,54,229,83,36,28,
  134,195,225,192,165,75,95,163,185,233,60,227,78,196,245,203,176,11,92,9,140,172,172,44,20,230,231,99,200,233,68,127,127,63,250,5,22,211,148,73,
  165,64,222,145,44,141,155,28,145,29,199,57,138,224,26,33,163,166,188,60,216,174,94,149,172,178,36,151,103,148,3,87,169,73,167,211,65,167,211,241,
  127,79,68,84,61,175,192,220,110,44,94,188,24,87,186,186,112,178,191,15,141,141,141,184,249,166,155,120,39,110,207,200,8,154,154,154,208,218,122,17,
  42,165,154,33,4,108,68,50,179,21,193,108,173,132,67,33,140,120,71,64,15,68,208,31,9,160,181,181,21,205,205,205,140,255,167,32,1,186,219,237,
  70,48,192,36,203,55,234,13,200,205,201,229,235,62,167,170,180,38,18,153,92,121,154,242,242,80,92,92,60,225,109,40,46,46,198,166,77,155,152,154,
  215,87,186,240,213,87,231,208,223,215,135,165,75,150,192,235,29,129,74,165,194,229,206,203,56,123,246,44,250,236,125,80,40,148,0,77,67,165,86,32,
  28,86,128,142,168,1,110,210,9,6,208,223,223,135,129,254,94,140,120,61,232,186,114,5,157,157,157,236,246,86,52,137,247,224,224,32,2,65,38,42,
  121,70,94,30,140,70,35,168,108,35,227,147,151,204,178,144,96,178,42,52,155,97,202,205,69,89,105,41,60,110,55,60,30,15,179,101,23,14,35,20,
  100,202,30,90,59,58,240,245,197,139,80,40,149,136,208,17,40,85,74,168,213,42,132,66,97,104,52,180,104,108,251,252,62,216,108,93,112,56,6,97,
  181,90,225,118,187,69,132,193,239,247,99,208,225,96,83,63,105,145,155,157,3,173,90,3,163,209,32,142,166,205,148,197,131,108,219,77,10,20,10,5,
  227,42,52,111,62,218,219,173,176,217,108,40,45,45,133,209,96,100,172,106,161,16,174,116,119,227,248,23,199,16,12,120,65,171,52,80,40,185,32,37,
  166,164,39,183,200,241,7,252,24,30,30,130,223,63,130,129,1,59,206,158,57,139,193,193,65,68,232,136,72,70,184,133,140,90,165,134,41,59,7,58,
  173,54,206,39,113,52,227,138,102,73,84,229,220,185,184,210,213,133,198,70,15,218,219,219,176,104,225,55,1,118,87,192,237,241,160,169,169,9,103,207,
  156,101,136,27,69,129,86,171,192,57,146,112,122,61,18,14,195,231,243,98,112,96,0,46,151,2,87,58,59,113,226,196,9,126,107,158,83,236,110,183,
  27,110,183,71,164,215,21,10,229,168,199,61,215,86,93,86,22,0,26,243,231,205,195,213,171,87,209,63,208,143,174,174,46,220,176,252,6,208,52,147,
  35,117,96,96,0,199,191,56,134,161,161,33,168,84,106,80,10,138,221,241,143,68,175,199,90,172,221,110,55,194,225,32,60,238,97,52,55,55,163,179,
  179,147,79,164,207,245,75,175,189,151,31,19,166,236,28,100,103,103,35,44,83,181,138,128,96,202,144,81,159,215,203,71,197,11,9,169,78,167,67,113,
  73,137,164,85,148,195,67,143,60,18,37,12,172,31,41,0,116,88,173,120,245,119,191,27,247,151,176,177,174,14,27,235,234,208,211,211,131,197,139,22,
  225,202,149,43,240,184,221,232,235,235,71,75,203,5,20,20,20,226,170,237,42,206,252,249,12,67,68,194,126,4,131,126,68,34,76,165,154,96,48,128,
  96,208,143,128,223,7,151,123,152,177,8,120,61,176,247,245,97,196,227,17,213,64,15,6,153,20,41,14,167,3,161,112,24,89,186,44,20,204,152,1,
  141,74,133,92,83,46,180,58,45,46,156,79,45,152,135,248,208,165,135,117,235,215,163,164,180,4,159,124,242,9,86,175,92,9,5,69,225,82,235,37,
  188,255,193,7,152,63,111,30,10,10,10,209,215,223,199,87,195,98,182,26,3,8,4,152,154,216,10,5,235,31,204,110,223,133,35,76,144,89,192,31,
  224,39,37,154,102,38,108,215,176,11,125,253,125,24,30,30,6,0,228,153,76,40,45,42,194,240,208,16,10,205,133,40,41,45,65,91,107,219,168,218,
  161,86,171,241,151,27,54,176,9,201,105,204,158,61,27,87,174,116,226,170,205,134,198,166,38,12,58,28,76,85,164,206,43,236,184,11,131,142,68,224,
  141,132,17,9,7,65,81,74,120,61,26,12,15,107,160,84,40,216,103,14,194,227,241,192,235,29,97,210,245,112,109,9,6,49,60,236,194,160,211,1,
  159,223,7,141,90,141,60,83,46,84,44,193,157,57,107,22,70,70,188,113,238,6,4,211,15,90,173,22,63,126,252,113,168,52,106,4,2,1,44,88,
  48,31,237,109,109,24,114,58,209,209,97,133,90,173,6,13,224,244,233,211,112,14,14,50,213,227,66,33,248,253,94,40,20,17,248,124,108,222,222,128,
  31,62,239,8,67,72,253,94,184,92,195,232,31,24,96,114,238,130,75,55,22,129,223,239,103,8,171,219,5,149,74,133,220,236,108,228,230,228,64,173,
  86,163,122,97,53,190,190,248,245,152,198,213,202,149,43,113,239,125,223,70,79,79,15,230,85,86,50,207,209,223,143,190,190,126,52,54,53,97,206,236,
  57,232,234,238,194,153,63,159,97,172,255,236,46,65,36,18,68,56,164,99,139,148,4,89,189,206,4,113,121,71,220,24,28,116,96,216,53,36,40,153,
  25,65,40,20,134,215,231,133,131,147,19,141,6,5,51,102,64,173,84,34,59,219,136,37,75,151,160,225,108,195,168,219,242,55,63,120,20,197,37,37,
  56,223,124,158,177,90,55,156,133,107,120,24,29,29,86,68,34,97,100,233,245,184,120,241,34,28,92,191,132,195,8,4,252,80,169,40,120,189,20,83,
  240,34,20,132,207,59,194,250,127,251,49,228,116,192,233,116,194,231,247,69,211,192,69,152,224,76,159,223,135,1,118,241,105,208,27,48,35,47,15,160,
  105,148,150,149,66,165,82,225,74,108,10,55,2,130,169,66,70,27,206,158,197,134,186,58,60,244,200,35,168,63,113,2,78,167,19,38,147,9,181,107,
  215,2,144,142,150,182,93,189,154,144,108,142,87,137,200,88,156,61,115,6,58,157,14,135,15,29,194,204,89,179,176,106,229,74,156,168,63,129,247,63,
  120,31,243,231,207,71,110,78,46,43,216,106,104,52,76,138,140,8,155,56,153,243,13,18,150,201,227,18,107,112,165,67,67,225,16,130,129,0,124,126,
  63,134,135,135,225,112,56,17,8,248,161,211,106,81,152,159,15,189,78,7,171,213,138,156,156,28,40,85,169,175,162,9,17,77,207,178,123,224,63,254,
  3,135,216,242,163,165,165,165,168,93,191,14,67,67,67,184,210,117,5,3,253,3,108,110,65,21,102,228,205,64,118,54,211,23,148,32,114,84,170,143,
  193,150,69,12,133,66,24,118,49,69,10,70,188,94,248,3,1,4,88,159,59,83,78,46,138,10,11,49,48,192,108,61,246,15,244,35,28,26,189,245,
  59,24,12,226,196,177,99,176,247,245,33,20,8,160,102,197,10,148,150,150,193,218,209,129,115,231,206,161,189,189,29,0,83,39,59,59,155,153,220,41,
  138,6,29,162,17,98,199,171,155,66,92,141,109,26,81,127,88,239,200,8,92,110,55,2,129,0,134,135,93,240,249,188,44,17,53,33,223,148,135,62,
  187,29,93,87,252,200,47,200,31,31,34,74,172,162,19,14,159,207,135,51,103,206,224,194,249,243,232,238,238,70,221,205,55,193,231,243,225,226,215,95,
  195,218,209,129,10,75,5,111,101,83,115,105,143,64,3,17,198,21,134,230,35,167,105,158,168,9,101,132,95,220,5,131,240,122,189,232,239,239,135,203,
  197,16,209,28,99,54,204,133,5,112,13,15,163,207,110,71,105,89,233,168,119,14,56,52,55,53,225,211,131,121,248,226,191,190,64,73,73,9,214,175,
  91,143,47,255,252,37,254,227,192,1,20,22,22,32,39,59,7,161,80,16,122,189,30,6,131,145,209,189,17,36,215,235,92,9,97,191,15,35,30,15,
  70,188,62,248,124,62,12,15,15,33,20,14,67,175,203,66,126,254,12,228,102,103,163,163,163,3,134,62,3,178,244,89,163,26,247,220,95,159,126,114,
  16,0,141,243,205,231,177,238,47,110,196,170,149,43,241,231,51,103,240,238,187,239,33,47,143,41,99,77,81,20,114,114,114,161,213,106,65,41,40,81,
  191,12,162,143,207,27,26,167,187,194,33,248,124,62,184,93,46,248,252,1,102,215,198,203,4,232,230,102,231,160,180,184,8,35,30,15,46,119,116,160,
  180,172,20,3,3,3,83,86,199,19,191,81,66,70,225,243,249,240,234,239,126,135,251,238,191,31,119,221,125,55,127,220,233,116,226,213,223,253,78,146,
  88,114,121,70,203,45,22,81,142,209,154,229,203,209,114,225,194,132,145,81,135,195,129,15,15,28,0,0,116,117,118,98,227,166,58,232,245,89,56,253,
  229,151,104,111,183,194,239,247,65,165,82,34,39,59,7,70,99,54,244,250,44,24,12,70,104,180,26,40,40,69,188,194,162,193,111,133,112,130,237,241,
  120,224,243,251,249,106,62,250,44,61,74,138,139,144,159,55,3,131,131,3,136,68,34,227,26,81,62,21,80,92,82,130,135,30,121,4,166,188,60,232,
  116,58,222,34,94,92,82,130,142,142,142,9,153,108,185,49,85,104,54,195,98,177,96,197,138,21,80,40,21,176,90,59,224,245,142,64,163,81,195,237,
  114,67,173,214,192,152,109,68,65,65,1,163,220,41,42,110,82,138,132,35,240,121,189,24,118,185,224,28,114,98,132,13,122,10,179,245,174,213,42,21,
  76,185,12,17,205,51,229,65,163,86,193,233,116,34,224,15,140,142,156,9,20,45,231,91,76,81,20,116,58,29,22,86,87,195,96,48,160,249,252,121,
  116,117,119,33,18,137,64,171,209,64,163,209,66,173,86,35,111,70,30,114,114,114,162,185,68,163,121,240,217,0,141,16,220,108,186,43,183,199,195,144,
  233,64,128,39,24,6,189,30,133,249,249,40,204,207,71,86,150,30,131,236,132,52,208,63,144,250,98,105,130,39,10,50,57,165,255,190,56,61,8,0,
  67,14,7,214,173,93,135,108,163,17,45,45,23,241,231,51,103,16,137,132,97,48,232,145,109,200,70,150,65,143,236,236,108,232,179,244,108,78,228,120,
  226,22,14,135,17,240,251,49,60,236,130,103,196,195,90,223,189,8,134,194,224,82,15,21,230,231,195,92,80,0,125,86,22,122,175,218,16,14,135,209,
  97,237,24,243,184,242,122,189,248,236,224,167,0,69,225,106,79,15,242,243,103,96,249,178,26,184,92,46,116,119,247,160,167,167,135,215,235,122,189,1,
  186,44,29,114,115,114,147,234,117,159,207,11,151,203,13,151,219,5,15,43,43,145,8,83,193,76,175,203,98,178,75,228,231,195,231,243,50,62,169,46,
  87,90,11,54,74,134,88,115,242,30,10,134,176,108,209,98,68,34,17,156,62,253,37,186,186,187,16,12,6,161,211,106,97,202,53,65,155,165,131,209,
  96,128,222,96,128,70,173,137,46,16,132,186,43,194,4,100,121,60,204,220,228,25,97,8,120,40,204,84,154,82,42,149,48,229,228,162,216,108,70,158,
  41,15,237,3,109,160,40,10,87,123,198,223,109,142,74,71,87,16,249,190,102,160,44,42,52,63,157,234,201,165,101,101,184,243,174,187,240,206,91,111,
  161,187,43,62,208,198,237,118,163,254,196,9,180,180,180,160,169,177,17,245,39,78,224,179,131,7,69,137,189,99,113,203,173,183,226,246,59,238,192,197,
  11,23,224,118,187,97,50,153,240,224,35,143,160,114,222,60,52,55,54,70,75,192,141,163,2,230,160,211,233,240,224,195,15,195,92,84,132,238,43,93,
  40,43,43,131,203,229,130,199,59,194,110,93,248,225,118,185,225,243,249,224,247,251,16,10,135,217,21,169,130,23,242,96,48,8,143,203,141,254,254,62,
  244,245,247,97,208,225,128,219,227,225,219,161,82,169,144,109,48,162,160,32,31,198,44,61,180,26,13,10,10,11,152,178,110,35,222,212,149,20,27,185,
  74,177,36,133,98,21,21,4,233,52,132,127,11,115,188,81,194,191,19,8,115,236,247,99,153,216,77,38,19,83,83,153,37,133,194,49,225,118,187,209,
  97,181,38,78,101,149,65,212,44,175,193,157,219,182,97,160,191,31,103,78,159,198,252,5,11,160,160,20,112,143,120,224,101,43,18,249,253,62,38,122,
  220,229,66,36,28,134,130,13,98,226,2,26,2,126,63,250,250,250,112,181,215,134,193,65,7,31,125,207,21,68,208,168,213,48,23,22,194,92,104,198,
  144,211,9,149,82,137,252,130,2,104,181,154,132,139,14,169,137,54,182,159,133,88,183,126,61,42,231,205,195,213,238,110,40,149,74,228,228,100,195,239,
  15,192,231,99,2,228,252,254,0,130,193,0,252,62,166,182,184,130,13,56,225,83,240,132,194,108,33,134,126,244,218,237,24,118,185,224,243,251,249,241,
  202,5,149,148,22,151,192,148,155,11,199,192,32,242,102,228,33,191,32,31,195,195,140,75,74,170,227,21,20,5,133,224,179,112,140,198,141,73,185,241,
  154,96,204,74,141,103,66,70,71,167,11,107,106,106,112,235,237,183,195,227,118,67,175,55,64,173,86,99,104,200,201,142,39,198,165,195,237,113,35,20,
  12,242,254,214,92,192,15,71,66,125,94,31,27,240,50,128,190,254,126,12,15,15,193,231,247,243,37,42,181,26,45,242,243,102,160,216,108,70,56,20,
  194,140,25,121,200,201,205,101,182,154,19,100,215,144,148,145,152,177,68,9,198,146,46,43,11,223,251,235,191,70,105,73,41,90,47,93,194,172,89,179,
  17,240,251,225,30,241,192,239,247,243,122,157,89,76,6,16,10,135,153,212,103,138,104,54,20,46,43,70,95,159,29,253,3,3,24,24,28,128,203,237,
  102,220,115,88,3,3,151,142,42,199,152,13,157,86,139,25,51,102,140,94,175,11,62,11,177,98,229,74,220,185,109,27,134,135,135,49,56,48,128,220,
  60,19,188,94,31,124,62,47,2,92,191,184,221,8,4,252,240,249,124,108,0,83,180,29,145,72,4,1,191,31,174,161,97,216,237,125,232,103,119,107,
  70,188,94,62,137,189,70,173,134,41,39,23,165,197,197,80,0,48,26,13,233,203,59,0,40,20,146,253,34,41,239,2,89,21,29,35,242,62,237,80,
  54,115,38,238,222,190,29,239,191,251,30,122,122,122,38,222,50,42,68,170,65,71,58,157,14,107,214,174,197,231,71,142,240,68,196,233,116,226,181,87,
  94,193,183,119,236,192,95,110,220,136,79,62,250,104,66,94,160,78,167,195,195,143,60,12,115,81,49,222,121,251,109,156,107,104,192,220,202,74,44,93,
  178,4,185,185,185,184,220,217,137,65,199,32,130,161,16,220,30,198,218,233,114,187,225,113,187,97,204,206,70,150,78,135,80,56,12,199,224,32,115,220,
  227,65,80,64,164,41,138,130,62,75,143,60,83,46,242,114,115,17,14,49,149,45,40,0,11,170,22,192,98,177,128,2,37,25,165,56,25,86,166,76,
  95,189,225,236,217,132,126,195,19,133,229,55,44,199,157,119,109,195,213,158,30,252,219,171,175,194,231,243,193,110,183,227,198,111,124,3,115,43,42,112,
  250,207,127,70,119,119,23,252,129,0,220,35,30,248,252,62,120,71,188,232,238,238,134,201,100,194,140,25,51,224,114,185,97,183,247,194,43,145,114,70,
  165,84,97,70,94,30,74,139,139,161,211,106,209,213,213,5,143,219,13,215,240,48,40,138,66,65,65,33,40,80,9,115,174,202,89,67,99,81,187,110,
  29,42,230,206,69,123,107,43,78,156,56,1,0,152,55,127,62,106,150,45,197,149,43,93,184,210,221,141,161,161,33,4,67,97,132,60,30,248,3,126,
  248,125,126,104,180,26,152,114,115,161,80,42,225,26,118,97,216,53,12,175,207,199,250,189,70,161,86,171,97,46,40,132,185,32,31,145,112,4,151,173,
  86,214,15,208,143,5,85,11,176,160,106,1,46,182,92,148,220,78,165,146,140,41,74,226,28,50,141,76,13,212,44,95,142,187,182,221,133,158,158,171,
  248,211,254,215,17,14,135,241,141,13,27,112,195,242,229,104,107,111,199,85,155,141,223,5,24,24,100,252,147,253,254,0,178,244,89,48,26,141,80,41,
  149,112,185,92,24,26,30,134,219,237,230,9,40,111,1,81,40,97,202,205,69,126,94,30,12,6,61,6,250,152,237,122,191,207,135,114,75,57,22,84,
  85,225,98,75,75,106,129,156,28,201,145,25,87,186,172,44,124,255,175,255,26,69,197,197,120,235,205,55,113,246,204,25,148,148,148,96,217,242,229,200,
  205,205,197,197,175,191,198,160,131,169,123,31,241,251,16,232,11,192,229,118,195,101,48,194,96,208,195,104,52,34,20,14,195,53,60,140,97,151,139,33,
  100,49,122,93,163,209,192,148,147,11,115,65,62,64,3,221,87,174,128,162,168,209,233,245,4,178,191,98,229,74,108,255,214,61,232,233,142,234,174,249,
  85,11,176,226,134,27,208,218,214,138,238,238,110,56,135,134,16,9,135,224,28,30,134,203,229,66,32,16,128,86,171,69,78,78,46,212,106,21,188,94,
  47,134,134,134,152,57,106,100,36,174,95,140,70,3,102,152,76,48,26,140,112,15,15,195,233,116,98,96,96,32,169,188,203,89,44,83,150,105,66,34,
  175,123,168,38,243,230,197,37,37,0,128,163,71,142,136,142,119,88,173,104,56,123,150,255,126,34,112,255,119,191,131,226,146,18,188,243,214,219,248,170,
  161,1,0,208,118,233,18,122,186,187,177,100,217,50,20,153,205,232,184,124,25,221,61,61,112,185,93,108,53,154,17,248,188,62,184,221,30,104,52,26,
  4,131,65,184,220,46,145,178,82,40,20,208,106,52,48,26,140,40,200,207,135,81,175,135,189,183,151,15,108,1,128,139,45,23,81,85,85,5,75,133,
  133,207,145,151,142,2,78,105,197,152,224,187,137,82,16,38,147,9,127,185,113,35,170,170,171,249,64,183,150,11,23,240,249,145,35,105,187,40,152,76,
  38,152,242,242,120,247,142,226,226,98,54,226,148,9,166,243,249,124,162,12,14,214,246,118,0,192,218,117,235,208,107,179,225,181,87,94,129,159,221,178,
  119,187,92,248,232,192,1,44,92,180,8,223,184,241,70,216,108,54,180,182,183,161,231,170,13,94,239,8,70,124,94,168,149,42,56,28,78,216,217,132,
  208,116,28,9,85,34,127,198,12,20,155,139,160,215,235,225,28,28,68,87,103,167,104,66,181,90,173,0,5,228,205,200,67,79,79,79,92,254,68,57,
  2,71,73,124,95,81,81,129,185,115,43,96,109,111,199,241,99,199,248,190,186,244,245,215,200,203,203,195,236,57,115,96,54,155,209,217,217,137,94,187,
  29,158,145,17,248,3,65,4,67,97,168,253,62,248,188,140,101,159,179,4,209,92,173,111,138,169,166,99,202,206,65,81,81,17,116,26,13,134,156,14,
  244,245,69,39,211,145,145,17,92,108,185,136,5,85,85,40,43,43,195,165,75,151,82,31,75,49,214,122,201,90,145,100,210,154,52,84,47,172,198,221,
  219,239,70,79,79,15,254,237,149,87,120,183,150,207,14,30,68,85,117,53,150,47,91,6,179,185,8,61,61,221,176,245,246,194,231,243,67,17,1,6,
  6,7,160,30,86,193,165,55,128,162,40,184,61,241,36,84,165,82,193,144,149,133,156,236,28,20,228,231,131,14,135,209,211,213,205,251,100,115,132,205,
  82,97,129,165,194,130,214,75,173,163,35,111,130,49,242,173,111,223,139,146,210,82,188,249,198,27,56,203,166,12,180,93,189,138,129,129,1,172,90,189,
  26,155,111,186,9,157,87,58,113,229,74,23,122,237,118,140,120,189,8,177,153,46,52,78,38,205,83,40,196,164,66,10,135,163,114,207,20,125,48,96,
  134,41,15,121,38,19,180,26,13,6,7,6,68,164,83,168,215,253,1,63,159,163,119,52,139,207,188,25,51,240,173,111,223,139,238,238,110,252,246,229,
  151,225,99,117,199,215,45,23,225,116,56,177,116,217,50,84,45,88,128,75,151,90,209,221,195,144,210,64,40,132,190,254,1,168,149,74,56,157,76,57,
  223,64,48,40,218,193,1,0,181,74,5,163,193,136,98,179,25,70,163,17,225,80,8,221,93,93,124,191,8,229,189,114,94,37,190,58,247,85,70,22,
  159,19,109,16,33,152,218,160,150,84,87,167,156,109,124,213,234,213,248,237,171,175,226,129,29,59,112,178,190,62,35,100,244,111,126,176,11,191,250,231,
  127,134,211,41,206,83,120,203,109,183,193,100,202,195,235,127,252,195,184,190,0,110,59,168,184,184,24,197,37,37,140,194,162,1,97,88,7,13,192,92,
  88,136,5,213,213,80,40,149,24,24,24,68,255,64,63,6,7,7,49,60,60,12,159,207,143,112,40,132,112,36,12,74,161,132,70,173,102,86,163,70,
  35,140,70,35,52,26,53,116,26,45,19,188,210,215,23,179,226,103,238,164,84,42,81,88,88,200,90,136,169,196,194,41,216,190,161,40,142,156,82,172,
  30,19,110,131,176,169,73,162,127,128,91,175,82,49,210,30,61,79,94,29,140,133,7,232,116,58,60,246,227,199,89,2,122,30,62,159,15,58,157,14,
  85,213,11,161,211,233,240,171,127,126,33,101,31,225,91,110,187,13,182,171,54,56,157,14,212,44,191,1,245,39,142,243,126,198,0,227,6,80,85,93,
  45,242,67,230,200,168,142,173,11,237,243,249,36,251,89,163,214,160,162,162,2,179,230,204,65,48,24,68,103,103,39,236,125,125,112,12,57,225,113,123,
  16,10,6,25,255,48,149,10,6,125,22,244,89,76,181,24,83,110,46,64,211,24,30,30,70,191,221,142,80,2,171,142,86,171,129,95,198,111,52,209,
  150,93,236,54,253,220,202,185,104,111,107,139,254,74,88,51,148,6,202,102,150,97,214,236,57,200,210,235,225,116,56,208,213,221,205,186,141,184,225,243,
  249,153,104,124,138,130,90,163,134,78,167,67,174,49,27,121,38,19,116,90,45,179,75,225,112,160,191,191,95,194,66,197,188,53,189,222,0,191,223,47,
  107,193,162,68,51,145,96,140,114,159,249,118,241,35,146,255,94,184,232,130,240,255,169,68,139,42,138,112,215,49,234,66,157,78,135,91,111,187,13,31,
  30,56,0,159,215,39,232,109,70,62,12,6,3,22,47,94,130,92,147,9,206,33,198,106,214,215,215,143,65,135,131,217,42,14,68,73,155,74,165,130,
  86,163,129,78,171,131,41,151,9,172,209,103,233,16,10,134,208,103,183,199,184,111,69,37,177,160,160,48,225,162,92,212,165,10,177,238,19,233,69,26,
  40,41,43,69,94,222,12,156,111,110,150,84,171,122,189,30,213,11,23,33,191,160,0,3,131,3,176,217,24,183,27,151,219,5,159,207,135,96,32,200,
  232,117,74,1,181,154,145,19,189,78,135,220,92,38,5,149,54,163,122,93,90,78,56,44,90,188,8,237,237,237,108,191,112,149,154,162,242,110,169,168,
  192,172,217,179,65,131,89,28,244,245,245,163,127,160,31,222,17,47,124,1,102,142,98,252,65,85,208,104,52,208,235,179,24,223,210,44,61,35,243,52,
  13,123,111,175,132,91,29,205,191,43,128,74,173,95,100,116,87,84,190,5,127,115,115,15,145,247,107,2,107,106,107,241,251,253,251,241,189,135,30,194,
  233,83,167,166,7,25,5,128,199,126,252,56,156,14,7,222,125,251,109,158,144,214,44,191,1,119,221,125,55,222,125,251,109,52,156,61,51,238,10,152,
  253,16,85,135,18,36,133,139,58,206,202,202,194,236,217,115,144,157,147,131,44,131,158,113,96,247,249,153,192,149,112,152,247,135,162,192,36,126,30,25,
  241,96,200,225,196,144,192,18,58,170,85,131,156,194,66,60,49,29,149,160,199,145,209,204,10,122,237,218,117,216,80,87,23,71,58,57,146,250,241,135,
  31,166,212,215,38,83,30,202,45,22,209,185,92,205,123,97,210,252,13,117,117,162,191,211,237,103,208,52,76,121,51,144,159,159,207,91,88,77,38,19,
  130,193,32,83,26,49,28,198,64,127,63,239,63,231,116,12,242,181,180,233,177,246,113,130,5,135,216,250,45,232,24,185,60,95,52,67,32,138,75,74,
  144,147,155,131,172,44,61,148,42,38,5,79,40,204,36,189,230,170,229,140,120,60,112,185,134,225,116,56,225,29,25,1,157,137,241,42,57,49,65,52,
  225,10,23,71,209,73,71,98,241,36,26,179,201,23,79,100,114,26,133,46,164,5,178,16,35,35,52,162,223,171,213,106,148,148,148,194,92,84,132,44,
  189,30,254,64,128,33,163,108,154,179,72,36,2,154,205,59,170,160,40,248,188,94,12,57,157,24,30,30,78,152,67,116,244,227,42,86,47,10,22,102,
  162,177,18,251,37,211,58,149,90,141,242,57,229,48,102,103,195,96,52,32,24,12,193,231,247,49,228,58,20,230,45,188,10,133,2,161,32,147,34,105,
  200,225,100,11,65,140,151,156,196,251,136,83,241,138,59,78,222,115,114,115,49,123,206,108,104,181,58,232,178,178,224,245,122,225,103,253,192,153,133,2,
  211,47,74,133,2,35,30,15,134,135,134,48,28,83,154,122,108,115,84,50,99,9,89,124,18,50,26,15,213,100,63,248,235,127,252,3,30,122,228,123,
  120,236,199,63,22,29,103,124,12,207,76,224,147,80,34,26,65,197,145,10,230,136,119,196,139,150,150,11,172,146,166,145,205,150,238,140,68,216,104,69,
  54,161,179,219,237,225,147,8,131,142,175,201,61,250,237,10,129,85,73,160,100,133,74,42,9,167,76,229,38,25,135,46,75,151,164,40,130,41,165,235,
  152,242,76,113,86,116,142,164,114,164,180,225,204,89,17,73,237,176,90,121,203,104,170,253,12,80,112,56,6,249,188,125,16,36,128,143,146,91,54,65,
  180,32,233,181,244,53,211,157,148,36,22,10,2,229,77,211,128,4,59,131,164,121,148,2,60,35,30,180,181,182,242,204,66,76,46,104,81,81,19,46,
  223,224,104,219,145,136,84,131,138,90,235,229,183,245,162,231,198,141,102,106,66,134,234,245,141,4,157,78,1,160,217,239,131,193,32,58,47,119,224,114,
  71,7,155,92,158,41,88,18,137,48,121,55,153,148,79,52,130,108,254,90,38,123,67,38,199,21,98,28,69,99,71,21,5,80,116,204,202,141,150,189,
  114,40,24,68,43,235,110,66,179,81,254,217,217,217,34,189,78,71,104,12,13,15,9,4,40,243,114,66,137,228,36,42,239,148,180,5,33,190,227,40,
  96,120,120,8,77,95,53,242,250,73,151,149,197,244,13,29,1,216,246,48,145,243,161,113,237,23,42,174,163,36,25,166,204,44,71,112,189,97,210,201,
  168,237,234,85,252,234,159,95,64,85,117,53,76,38,198,2,213,210,114,97,194,42,47,73,18,68,10,98,114,33,180,160,81,209,191,41,80,76,202,14,
  206,154,64,11,172,7,52,179,250,164,41,177,121,33,29,129,151,139,172,142,157,216,83,18,116,58,218,152,216,73,158,158,8,94,74,139,236,45,49,95,
  209,41,191,148,14,171,21,119,109,187,91,224,43,90,2,159,215,11,39,13,28,61,114,88,116,63,209,223,99,234,103,65,61,22,138,142,146,65,154,2,
  87,204,82,170,159,83,89,124,72,206,41,146,219,216,177,46,113,114,36,84,250,147,144,163,70,155,205,17,220,40,33,165,4,229,250,210,105,135,220,120,
  141,95,44,81,162,118,34,150,152,198,222,144,162,227,174,78,75,220,140,34,211,87,230,245,96,236,55,180,4,99,101,229,196,235,245,49,229,109,185,10,
  62,252,24,226,244,160,32,143,216,40,136,15,149,72,70,132,59,6,66,75,40,77,201,44,51,153,191,56,29,78,199,144,87,138,166,16,14,133,224,24,
  116,68,155,45,104,11,163,78,197,239,36,179,122,61,102,23,68,120,241,132,194,34,177,216,166,88,31,122,175,87,252,204,180,176,95,144,182,222,74,169,
  95,226,116,151,184,183,226,92,197,248,206,161,227,204,156,52,33,171,132,140,78,4,124,62,223,228,71,90,143,70,59,210,49,172,133,83,110,188,82,67,
  66,162,34,39,244,210,187,143,49,91,31,49,214,178,164,130,158,84,139,140,63,138,139,75,240,160,160,218,150,240,248,229,4,121,5,99,81,127,226,4,
  54,108,172,99,199,142,23,245,39,78,240,85,187,56,72,89,79,83,239,103,33,115,163,211,236,103,196,89,73,83,31,83,148,216,114,40,185,224,16,152,
  74,232,88,155,34,37,44,98,40,32,210,226,105,88,68,85,69,164,26,124,16,83,212,26,44,221,142,164,213,191,98,219,2,233,137,41,142,68,16,43,
  201,148,51,143,202,139,12,51,110,164,22,108,160,99,199,150,52,33,77,75,15,38,145,17,74,234,135,163,25,87,81,35,35,104,9,57,225,73,236,132,
  233,117,90,162,29,180,72,242,99,251,128,209,85,226,197,38,247,134,100,245,86,138,109,161,228,222,153,136,136,166,98,44,73,67,194,169,148,70,7,1,
  33,163,211,89,239,82,34,242,32,165,104,227,181,20,5,138,166,229,173,102,252,30,35,157,80,224,83,18,41,169,0,15,196,56,132,11,131,148,18,41,
  96,106,242,4,221,233,116,192,102,147,182,118,219,108,87,229,201,163,204,249,177,215,138,93,204,164,190,184,17,247,115,44,199,19,91,71,185,190,136,42,
  119,58,166,159,163,70,104,58,77,243,2,21,147,103,79,108,33,17,249,86,137,204,141,52,228,167,219,152,111,120,43,8,37,154,160,132,214,17,158,235,
  242,132,84,186,45,84,170,227,85,146,152,10,218,156,104,168,82,169,45,158,200,180,52,65,188,148,138,223,61,16,43,193,232,9,60,73,98,9,40,69,
  211,9,229,131,74,81,70,226,137,168,96,84,9,172,237,209,223,36,147,15,97,51,68,75,50,126,23,73,44,39,236,89,227,166,215,33,173,215,105,177,
  80,68,159,53,129,36,8,214,158,156,170,160,33,61,63,9,55,207,98,231,67,106,140,253,34,212,93,114,89,93,226,69,157,72,53,33,163,68,243,38,
  87,194,113,7,163,43,81,222,42,144,76,224,19,145,21,41,71,111,209,241,152,212,78,49,19,123,34,171,104,188,35,248,196,76,236,147,158,103,84,180,
  232,144,239,103,201,153,54,174,243,99,201,28,120,251,4,71,228,248,137,152,6,18,118,116,140,245,64,214,201,95,138,84,202,77,74,66,99,138,148,157,
  75,48,86,69,109,224,215,90,194,113,29,51,102,19,201,142,236,120,133,36,97,144,158,152,164,198,35,69,172,36,147,103,24,77,115,97,30,213,123,144,
  32,113,146,11,157,164,166,196,196,11,156,88,25,137,29,65,146,228,25,136,117,173,150,120,7,241,150,69,154,179,56,82,9,244,122,194,246,36,151,19,
  42,110,69,70,39,93,120,82,201,136,181,192,212,27,219,79,81,206,206,182,133,191,21,61,138,126,145,32,162,177,79,156,138,177,132,44,62,9,25,37,
  74,56,250,167,86,171,67,65,97,33,239,211,232,243,249,216,60,114,140,68,151,20,151,224,106,207,85,100,27,141,208,27,12,209,218,191,116,4,52,205,
  212,56,30,25,241,0,0,244,89,58,232,116,89,76,46,82,151,75,236,247,70,75,201,165,180,178,226,35,229,19,77,236,99,178,138,94,127,253,124,199,
  157,119,225,224,199,31,241,185,251,170,170,170,48,191,170,10,199,190,248,47,12,244,247,131,162,129,162,146,98,20,21,21,67,169,82,33,55,55,23,145,
  72,24,145,112,4,225,72,4,78,135,3,151,47,95,70,117,117,53,34,145,8,255,143,102,83,61,49,193,67,81,63,51,74,206,126,153,136,136,10,250,
  158,6,96,52,102,99,238,220,185,130,45,62,10,173,173,151,224,113,185,153,239,179,141,168,172,156,135,168,47,51,99,241,104,107,107,131,219,237,70,121,
  69,57,250,122,237,240,120,220,49,91,120,204,243,100,231,100,35,75,151,5,123,111,47,0,10,51,103,207,132,195,225,132,219,29,205,149,104,52,24,145,
  107,50,161,167,187,91,218,162,25,59,54,37,44,36,177,19,83,73,73,9,170,23,46,4,192,248,7,119,88,219,161,203,210,97,237,186,245,56,113,252,
  24,124,190,104,110,214,170,234,106,80,0,90,46,180,160,102,249,114,56,157,206,184,178,194,151,59,172,124,14,91,38,149,24,227,155,238,116,58,38,116,
  113,164,211,233,80,179,124,57,116,186,44,52,156,61,35,153,87,183,220,98,225,219,45,133,154,229,203,209,97,181,162,102,249,13,162,227,54,219,85,62,
  173,89,38,160,213,234,144,155,147,3,187,189,151,239,35,173,70,139,236,156,28,244,245,245,9,214,102,226,5,91,81,81,49,104,154,70,111,175,141,95,
  176,169,84,106,38,239,238,144,147,95,0,169,84,42,232,116,89,112,185,92,200,53,229,9,50,92,68,133,114,104,104,40,78,86,82,149,17,41,217,154,
  61,103,14,0,224,202,229,203,34,35,35,29,163,15,180,26,13,102,151,151,195,96,48,192,229,98,101,151,125,238,217,115,202,209,103,183,195,51,226,225,
  239,83,86,90,138,193,193,65,104,117,58,152,76,38,94,247,115,109,26,26,114,242,109,137,107,143,132,156,72,233,117,26,20,150,45,91,134,92,83,46,
  35,203,236,162,186,243,242,101,92,185,210,137,181,107,215,129,166,105,68,248,249,135,70,79,87,23,122,122,122,80,84,82,138,226,162,34,230,187,72,4,
  52,29,65,36,66,163,185,169,9,89,122,3,102,205,154,197,234,42,38,64,171,187,187,155,241,253,165,0,173,78,199,247,41,0,132,67,33,244,246,246,
  34,20,14,141,170,95,82,54,150,144,197,231,117,9,5,225,159,242,71,10,205,102,220,113,231,157,168,173,93,139,213,181,181,184,237,155,223,196,173,183,
  221,198,159,115,219,55,239,0,69,1,69,197,197,88,186,108,25,150,44,93,138,186,77,155,176,104,241,18,44,92,184,16,133,133,76,181,157,101,203,151,
  99,77,237,90,84,206,155,135,154,229,203,177,98,213,42,104,84,106,62,46,158,226,35,41,133,255,19,238,56,137,87,205,252,113,169,201,62,70,218,19,
  167,199,160,82,122,35,215,122,63,131,2,102,206,154,197,127,174,152,59,23,58,173,22,165,165,101,252,139,157,51,167,28,193,96,16,249,249,249,232,233,
  238,198,249,230,102,92,184,112,30,23,91,46,224,202,149,78,248,188,94,180,182,182,162,173,173,13,90,157,14,131,3,3,184,220,97,133,189,183,151,159,
  100,68,125,43,232,66,74,240,129,162,98,250,63,86,153,179,15,105,52,26,81,57,111,30,107,40,167,160,209,106,113,231,182,109,152,93,94,14,10,128,
  49,59,27,149,243,231,67,156,119,54,122,133,185,115,43,97,96,203,179,198,196,196,129,2,133,249,11,170,48,111,193,124,246,153,128,128,223,143,5,85,
  85,162,17,58,127,193,2,4,2,1,196,12,71,193,61,169,248,54,66,108,37,21,142,207,234,133,11,241,200,247,191,207,127,183,109,251,118,220,122,251,
  55,249,156,180,107,215,173,231,207,101,242,97,222,14,219,85,27,79,212,238,186,251,110,232,116,58,17,121,227,210,114,21,23,151,96,247,223,253,24,115,
  202,25,194,183,108,249,114,252,183,31,252,96,194,198,223,131,143,60,130,57,229,22,104,117,58,60,248,200,247,96,50,153,226,201,104,185,5,229,236,243,
  197,130,35,209,166,188,60,44,139,241,143,222,114,235,109,113,62,211,99,145,145,162,34,51,234,110,186,73,244,93,161,217,140,191,220,176,81,82,158,230,
  45,152,143,239,62,240,87,168,89,190,28,171,86,175,198,93,219,238,70,222,140,60,0,20,76,121,38,44,173,169,97,3,127,152,127,57,57,185,252,66,
  98,238,220,185,168,152,59,23,11,23,46,196,194,133,139,96,177,84,192,98,169,136,151,21,225,64,146,178,188,197,18,158,24,35,220,236,57,115,48,123,
  206,236,152,12,118,226,235,23,20,22,224,254,239,126,23,165,165,165,160,40,10,243,42,231,225,155,119,108,229,47,98,169,168,128,193,104,140,62,25,69,
  97,214,236,217,208,235,245,80,80,20,20,10,10,10,133,2,115,43,43,153,178,187,10,5,20,148,66,82,175,83,50,114,2,153,45,237,165,53,53,200,
  205,53,137,229,149,253,239,218,245,235,33,120,189,34,89,47,45,41,65,9,219,30,74,160,99,0,38,229,219,156,242,114,40,20,10,38,143,170,86,131,
  117,235,215,163,168,168,8,20,0,125,150,30,101,101,101,80,178,109,201,201,201,193,234,53,107,160,86,169,71,215,47,137,118,65,136,85,148,88,70,211,
  57,185,155,181,128,84,45,92,152,177,60,163,83,218,106,70,1,253,125,125,120,251,205,127,231,119,45,238,189,239,62,148,150,149,161,167,187,139,173,239,
  77,161,189,173,149,183,126,237,248,206,119,240,249,209,35,252,54,135,201,148,139,162,162,34,28,57,124,24,193,32,147,95,111,249,242,27,80,54,107,22,
  46,119,88,19,63,8,37,183,154,150,32,40,113,249,244,210,95,113,94,87,130,46,232,231,182,214,86,204,156,53,27,109,173,173,208,104,181,152,91,89,
  137,183,223,124,19,235,110,188,17,141,95,125,5,128,70,105,89,25,62,59,120,16,179,102,207,130,211,201,84,98,138,110,201,49,255,29,28,24,224,255,
  118,123,60,112,56,157,92,68,144,252,251,165,146,89,76,16,183,224,224,38,130,145,145,17,182,90,24,211,152,182,214,75,184,233,230,205,232,188,220,193,
  124,239,241,224,92,195,89,126,236,114,59,143,20,0,5,37,200,243,201,109,205,179,87,202,210,235,145,165,211,161,175,175,15,179,102,207,198,149,43,157,
  232,235,235,199,204,89,179,81,110,41,199,229,142,14,204,153,83,14,128,198,64,127,95,204,204,159,250,120,141,157,152,214,174,95,143,119,222,124,19,23,
  46,156,7,0,212,31,59,142,29,223,253,46,0,224,200,225,67,120,244,111,255,22,23,216,76,27,219,238,222,142,250,227,199,69,22,70,159,215,139,13,
  117,117,248,248,195,15,227,94,243,183,239,191,31,127,218,255,199,168,213,241,8,67,16,171,170,171,51,106,85,44,183,88,68,41,204,202,45,22,248,188,
  62,248,124,62,252,105,255,31,5,68,249,134,184,108,15,137,48,167,220,130,115,103,207,66,151,165,131,211,233,16,253,214,102,187,138,53,107,215,102,212,
  210,75,81,84,188,50,164,168,56,217,41,45,45,195,202,149,171,240,222,187,239,194,237,98,114,110,86,85,85,99,229,170,213,248,244,224,39,160,40,134,
  200,80,2,191,82,74,161,128,74,165,2,40,138,173,138,68,163,98,110,37,0,160,189,173,85,60,244,99,182,118,169,84,100,132,61,78,197,102,49,225,
  72,19,77,71,127,39,216,162,186,245,182,219,113,240,147,79,68,150,254,205,183,220,194,200,64,103,39,223,14,97,116,187,82,165,2,165,80,96,112,160,
  31,14,7,227,247,62,127,254,2,92,190,220,17,47,207,163,212,235,92,79,52,126,245,21,46,179,178,45,186,2,69,225,228,137,122,72,57,212,80,20,
  133,222,222,94,124,213,208,32,178,62,83,20,163,3,252,126,63,190,190,120,145,255,93,79,119,55,110,88,177,18,118,187,157,201,169,26,10,137,218,178,
  114,213,106,228,228,230,240,153,6,100,251,69,138,136,74,108,207,39,205,37,154,134,85,148,228,24,157,26,168,98,119,182,186,5,114,148,113,50,202,9,
  105,117,117,245,53,203,63,227,142,8,70,120,78,110,14,180,58,29,220,46,23,192,38,115,230,83,252,176,126,69,10,165,18,81,191,62,10,193,80,8,
  58,157,14,121,38,19,250,250,250,0,0,103,207,158,69,212,15,72,254,169,40,41,65,147,80,190,241,91,31,41,174,56,147,134,68,227,186,232,231,174,
  174,46,44,92,180,24,0,48,115,230,76,88,219,219,209,211,221,141,156,156,28,104,181,90,248,253,62,166,223,221,110,40,20,74,84,206,155,135,162,226,
  98,118,219,139,217,150,180,247,218,121,66,167,84,42,25,178,23,231,79,150,88,173,38,82,228,113,121,55,121,11,7,197,55,108,112,112,144,181,222,22,
  128,162,40,104,117,58,148,148,150,138,18,253,219,174,94,101,8,41,187,144,18,69,222,114,149,92,44,22,116,117,117,161,207,110,199,242,21,43,208,117,
  229,10,64,1,23,91,90,80,187,118,45,220,110,15,42,42,43,81,127,226,132,116,122,166,84,219,18,99,33,233,104,111,199,198,77,92,110,216,118,248,
  252,62,188,242,219,125,12,209,244,249,240,209,129,15,177,109,219,221,56,114,248,48,76,38,19,234,79,28,23,189,197,163,71,14,227,150,91,111,67,195,
  153,51,108,197,27,166,61,197,197,197,0,77,199,109,127,191,246,187,223,101,124,140,149,151,151,227,47,55,108,196,107,175,252,14,91,110,189,21,58,93,
  22,222,123,231,109,209,189,76,38,19,46,182,72,17,96,58,225,117,63,249,232,67,198,114,26,115,218,130,170,42,56,29,206,140,16,80,154,166,19,174,
  141,99,93,172,23,84,85,161,169,169,137,117,223,96,8,96,75,203,5,244,178,238,29,20,69,241,58,145,243,69,86,80,20,67,226,4,190,163,74,165,
  82,134,81,80,50,193,241,226,68,246,178,132,135,13,56,140,181,136,198,14,212,130,194,66,80,20,133,158,158,110,17,79,61,248,241,199,252,51,42,148,
  74,80,148,66,176,184,164,161,146,144,117,181,90,157,64,183,203,232,245,20,228,164,168,168,136,127,251,62,159,159,113,163,96,23,153,51,103,207,18,229,
  12,238,239,179,195,231,247,131,162,40,100,103,27,81,82,82,194,102,116,162,225,15,248,225,24,28,4,69,81,80,42,149,209,246,210,128,107,216,5,154,
  166,145,107,202,5,165,80,32,43,43,11,249,5,133,0,104,100,101,233,145,157,157,205,84,104,162,70,217,47,233,24,75,36,173,162,215,217,164,53,13,
  193,241,195,158,241,36,163,0,240,229,233,211,88,83,91,123,237,189,65,90,154,44,204,154,53,11,187,127,252,56,147,200,57,18,193,133,243,205,112,177,
  213,148,152,9,29,34,66,170,84,40,4,233,116,24,235,213,153,63,255,25,203,150,47,135,94,175,135,221,110,71,79,119,55,58,47,95,78,137,60,201,
  49,78,121,33,79,181,122,133,184,221,209,243,233,4,150,146,107,179,159,135,135,134,144,155,155,11,80,20,230,86,206,99,86,116,20,112,181,167,7,150,
  138,10,12,15,13,161,215,118,149,223,18,23,89,21,133,230,74,118,54,82,170,148,160,20,148,56,97,117,210,254,149,87,228,241,228,45,106,217,136,53,
  73,4,131,65,104,116,90,166,18,75,78,14,110,88,177,82,148,160,255,163,3,7,64,129,134,66,161,96,126,207,223,32,234,251,55,115,214,108,28,57,
  124,8,193,64,0,74,165,2,6,131,1,30,143,7,94,175,23,151,47,95,198,141,127,241,23,104,106,108,132,207,235,149,182,232,166,213,150,232,120,61,
  114,232,16,156,78,7,150,223,176,28,219,182,111,135,211,225,192,145,35,135,89,203,37,133,150,11,23,176,124,249,114,236,184,255,126,252,203,175,127,29,
  211,161,52,124,62,31,62,254,232,67,220,117,247,54,193,247,128,78,151,37,233,163,57,30,56,122,228,8,30,124,216,130,111,239,216,1,147,41,15,175,
  189,34,38,188,53,53,203,97,50,153,210,178,98,154,76,38,81,246,136,242,242,114,60,245,204,63,242,127,119,116,116,136,172,174,99,150,15,26,152,53,
  123,54,126,242,247,127,207,47,184,104,154,70,103,231,101,161,186,227,171,253,92,188,120,81,180,168,161,64,195,225,24,100,183,113,153,173,107,145,238,82,
  80,60,249,228,122,95,169,84,130,102,115,95,38,182,122,37,25,87,49,229,143,163,57,166,98,13,12,226,168,69,173,86,11,151,219,29,29,155,52,29,
  221,73,96,173,167,204,118,54,37,146,23,133,82,9,69,140,172,171,212,234,132,121,111,165,245,122,18,57,161,40,44,173,89,6,175,119,1,0,26,189,
  54,27,14,125,214,203,47,74,197,126,163,17,252,215,231,255,9,127,159,29,20,69,161,180,180,12,70,99,54,239,203,62,48,48,128,211,167,78,2,10,
  174,111,196,187,35,116,36,2,141,134,41,15,154,157,147,131,170,170,42,128,37,217,35,35,35,8,5,67,9,44,161,201,250,37,254,123,89,99,73,172,
  190,166,146,45,218,8,73,157,10,88,83,91,139,47,79,159,78,251,119,105,147,209,195,135,14,225,137,191,255,123,148,205,156,137,238,174,174,107,196,98,
  70,73,36,100,103,148,78,119,87,23,222,248,211,235,0,205,212,20,191,237,155,119,240,109,87,240,214,165,232,30,168,66,169,20,103,209,1,179,218,238,
  233,233,134,90,173,70,97,161,25,213,11,23,194,104,52,226,194,249,230,228,116,148,146,89,85,203,8,185,36,17,69,242,50,106,215,131,28,115,225,0,
  210,214,209,43,152,57,107,22,102,206,154,133,147,39,142,3,0,172,214,118,148,149,149,33,219,104,196,85,214,55,81,161,80,160,173,173,21,87,123,122,
  68,129,242,124,244,42,197,212,126,166,40,5,107,13,74,65,121,82,233,89,196,153,63,21,209,197,144,112,82,213,233,16,8,4,160,213,104,49,56,56,
  128,3,255,241,129,132,121,56,58,9,197,90,139,102,206,154,141,130,130,2,108,255,214,189,60,17,153,191,96,1,95,13,205,218,222,134,229,55,220,0,
  107,123,155,196,250,134,74,161,45,50,227,21,128,165,194,130,134,51,103,208,112,230,12,64,81,40,183,88,176,227,254,239,224,55,47,189,196,147,201,19,
  39,78,64,167,99,170,121,73,141,217,150,11,23,80,83,179,28,181,107,215,241,199,156,78,135,164,143,166,142,13,58,225,173,168,25,194,39,31,125,136,
  255,246,232,15,240,222,59,239,136,42,142,213,212,44,199,154,181,107,227,8,106,114,107,171,69,148,135,183,163,163,3,175,189,242,59,232,116,58,124,123,
  199,253,56,121,226,68,92,101,179,177,236,30,0,192,149,206,78,236,255,195,31,248,201,127,214,172,89,168,93,183,94,156,149,130,2,130,129,0,180,90,
  109,220,162,134,162,105,134,200,81,28,241,228,198,58,13,138,82,64,37,176,200,113,114,149,104,205,44,75,102,18,44,112,98,84,185,128,148,198,95,219,
  53,60,204,47,72,249,7,161,153,128,38,99,182,17,3,253,3,60,25,21,26,27,148,74,37,191,203,192,145,57,181,90,157,100,203,56,125,189,78,81,
  20,62,59,248,41,46,119,116,196,203,15,69,225,223,223,248,19,164,34,98,41,5,133,75,151,46,225,207,167,79,69,191,161,163,254,235,202,152,157,60,
  176,21,155,194,161,32,84,106,53,60,30,15,78,28,63,198,63,233,138,149,171,120,87,157,244,251,69,154,136,82,137,59,158,208,204,105,132,178,153,51,
  81,54,115,38,254,240,127,255,111,218,191,77,59,128,233,240,161,67,0,128,135,30,126,248,90,98,41,137,243,38,178,231,248,253,126,244,116,119,99,230,
  172,153,98,203,168,64,170,148,74,165,56,40,163,186,26,235,214,173,7,64,33,24,12,161,167,167,27,141,95,157,67,113,73,177,200,241,91,250,95,84,
  147,82,114,171,102,153,137,61,118,241,157,72,172,175,27,65,79,48,209,117,93,185,130,27,86,172,68,32,16,192,240,176,11,20,0,107,123,59,230,86,
  86,162,108,214,44,118,203,129,241,119,19,58,235,199,166,53,228,20,60,165,160,226,163,148,164,250,55,81,31,199,70,99,8,11,236,81,84,220,248,43,
  45,43,131,78,167,195,64,63,227,187,170,160,20,210,141,167,192,78,172,136,203,182,178,112,209,34,28,62,116,8,127,250,227,31,241,167,215,95,199,187,
  239,188,141,89,179,103,67,163,213,242,207,205,249,251,165,53,94,147,16,81,46,96,169,220,98,225,219,219,97,181,194,233,112,240,65,72,169,14,218,143,
  63,250,16,181,107,215,242,4,148,35,178,177,193,65,119,110,219,134,170,113,112,57,218,114,235,109,104,105,185,128,53,107,215,74,18,209,116,137,227,130,
  234,42,116,72,248,151,251,124,62,124,242,209,135,184,115,219,54,81,224,214,152,101,68,88,17,72,52,178,163,58,145,251,220,211,211,205,100,117,16,156,
  171,209,104,240,192,131,15,33,59,59,27,161,96,0,57,57,57,162,224,184,236,236,108,132,35,97,209,216,81,42,149,2,98,36,63,182,164,198,21,37,
  39,35,177,207,78,137,189,45,133,212,105,120,216,5,138,162,80,54,115,150,72,40,54,212,213,177,99,18,24,25,241,48,110,47,236,25,90,141,6,5,
  5,5,240,120,70,68,114,46,47,31,41,234,117,9,57,137,186,49,36,52,27,199,207,107,148,130,15,92,138,53,84,40,88,171,181,48,150,223,92,92,
  132,172,172,44,56,135,134,152,32,44,165,82,244,236,225,72,24,70,99,246,40,251,69,134,136,18,99,201,53,3,142,23,114,60,113,92,45,163,61,221,
  221,248,242,244,105,108,187,231,30,252,239,23,95,196,48,187,101,125,77,16,210,184,116,146,20,116,58,29,102,205,154,13,128,134,70,171,197,242,21,43,
  240,193,123,239,70,39,116,136,179,191,241,22,39,154,177,2,180,183,183,225,238,237,247,32,24,10,225,74,103,39,212,106,53,150,44,93,138,182,182,182,
  164,18,38,111,120,162,36,153,166,116,149,200,20,238,113,61,9,186,100,206,88,160,245,82,43,54,212,109,66,115,99,35,111,69,9,248,253,112,187,92,
  80,80,20,220,46,23,31,248,179,110,253,141,240,251,124,162,84,94,31,29,56,192,91,29,20,130,40,90,196,216,42,82,224,197,18,125,28,175,182,41,
  5,133,220,220,28,172,90,189,154,33,0,90,45,170,170,170,112,228,208,33,126,226,205,201,205,197,170,213,107,216,7,96,44,194,61,221,140,149,94,161,
  160,176,96,65,21,235,83,202,156,114,233,210,215,200,205,205,101,83,243,48,247,10,6,130,232,239,239,199,172,89,179,217,49,11,102,178,77,37,202,53,
  197,182,112,31,62,58,240,33,118,124,231,187,104,185,112,1,78,167,3,229,229,22,248,124,190,248,84,71,73,198,171,211,233,196,201,19,39,176,229,214,
  91,249,99,127,218,255,71,60,248,240,35,104,105,185,128,33,167,19,115,202,45,208,233,116,120,239,157,119,50,58,188,54,108,220,8,159,207,139,63,237,
  223,143,59,183,109,195,150,91,111,197,197,11,45,184,115,219,54,212,215,159,64,45,75,80,59,172,29,146,4,115,78,185,5,130,160,117,52,156,61,155,
  208,205,192,102,179,225,100,61,211,214,76,180,133,162,168,132,21,118,196,22,46,26,45,23,46,96,219,221,11,177,233,230,155,113,225,252,121,104,52,26,
  220,176,114,37,90,91,47,49,190,133,0,66,161,16,110,252,198,55,208,122,233,18,12,6,3,86,172,92,137,227,199,142,137,114,228,70,93,158,168,52,
  244,32,36,179,131,80,49,178,206,253,49,103,246,108,224,27,223,0,155,31,9,77,141,141,112,10,210,46,125,248,193,7,216,118,207,61,104,107,109,197,
  240,240,48,74,74,75,160,86,107,112,244,200,97,80,0,206,55,55,227,182,219,191,201,250,145,251,81,49,119,46,122,122,122,48,226,241,68,83,163,129,
  9,106,66,178,109,250,52,245,186,146,223,201,224,118,119,40,81,159,173,91,127,35,43,230,52,155,82,110,8,231,155,207,179,4,187,140,53,104,211,160,
  35,52,104,208,248,250,226,69,80,172,142,168,185,97,57,104,154,177,232,86,84,84,224,248,177,99,172,53,86,129,156,220,92,44,89,186,140,153,255,212,
  26,88,42,42,240,209,135,31,66,50,65,93,26,253,34,58,154,1,99,9,69,162,151,38,29,57,57,57,216,118,207,61,248,242,244,233,180,253,69,1,
  64,89,84,88,248,116,218,132,180,167,7,247,126,251,219,208,106,181,248,175,255,252,207,107,131,163,72,12,102,157,86,139,89,179,103,99,22,187,125,59,
  99,198,12,212,159,56,129,203,29,76,68,227,204,153,179,112,225,194,121,81,2,250,178,178,50,38,58,145,85,222,145,112,4,151,59,58,96,54,155,97,
  169,168,64,110,110,46,172,237,109,104,253,250,146,56,197,143,196,63,209,42,83,184,210,148,176,46,165,70,68,169,148,23,213,215,170,160,203,181,37,28,
  14,195,214,211,131,139,45,23,16,14,135,249,23,218,121,185,3,221,221,221,24,25,25,1,0,12,12,12,224,106,79,15,174,94,189,138,171,87,123,96,
  187,202,84,131,114,187,221,188,165,212,225,112,96,120,120,136,177,254,136,236,2,49,125,43,248,34,113,31,75,76,178,148,48,241,13,224,29,25,193,177,
  47,190,136,18,73,62,77,148,216,154,229,114,185,224,114,185,163,249,0,5,105,159,130,129,0,122,109,54,12,57,135,4,233,152,152,168,252,44,189,158,
  9,122,96,23,92,226,148,85,233,181,37,198,32,204,255,221,63,208,143,166,198,70,232,116,58,168,213,42,92,184,208,130,207,62,253,52,110,188,250,124,
  62,201,173,117,97,20,123,119,119,23,40,10,232,232,176,194,231,243,193,237,118,227,207,95,158,70,182,49,27,42,181,10,151,173,29,56,240,31,31,32,
  20,10,101,108,108,233,116,58,204,156,53,11,135,62,253,148,137,66,238,232,64,73,73,9,124,62,31,6,6,250,17,22,220,203,233,116,74,16,76,42,
  78,30,125,62,63,122,109,54,54,191,49,3,127,76,251,59,216,251,216,108,182,140,180,71,167,211,194,96,48,162,189,189,93,160,11,117,48,24,12,209,
  133,1,219,129,225,112,8,151,46,125,141,188,25,121,168,170,174,70,81,113,49,206,55,55,163,225,108,3,255,91,171,181,29,69,69,69,176,84,84,192,
  152,157,141,175,206,53,224,74,231,21,209,232,102,172,168,65,56,29,142,132,178,34,74,93,20,99,117,147,36,60,49,245,145,133,223,219,237,118,248,217,
  188,181,92,118,138,166,198,70,232,13,6,40,149,74,244,116,247,224,63,63,63,202,234,3,10,94,175,23,237,214,118,232,245,122,40,149,42,180,183,94,
  138,46,94,5,183,145,146,143,49,235,117,10,232,181,217,216,241,77,197,233,21,80,130,18,192,20,179,144,238,239,235,147,174,115,15,96,176,191,31,129,
  96,0,10,69,244,5,249,188,94,124,249,229,105,244,247,245,243,215,84,8,210,65,249,125,62,156,249,243,159,89,242,45,33,243,233,244,75,58,68,244,
  58,155,163,166,43,30,127,226,9,172,169,173,197,79,247,238,29,21,25,165,150,84,87,211,163,185,241,175,254,247,255,70,221,166,77,120,96,199,142,107,
  38,205,19,77,211,9,10,35,209,177,39,139,143,74,22,222,160,37,62,210,137,174,42,191,46,148,17,218,212,132,156,8,250,232,251,25,16,249,153,210,
  114,103,38,239,235,132,118,146,4,253,69,201,158,74,241,5,25,226,174,145,224,24,157,202,73,49,237,79,207,244,60,138,182,164,178,112,34,19,211,20,
  144,145,209,200,71,242,49,149,90,109,122,42,101,61,8,1,73,227,50,4,8,11,139,197,221,60,70,4,164,159,142,26,117,123,50,163,215,101,84,11,
  37,115,44,217,156,20,155,95,36,133,57,138,74,83,230,229,19,188,36,159,163,136,188,79,31,172,169,173,197,239,247,239,199,225,67,135,240,216,15,127,
  56,58,67,209,104,201,104,118,118,54,62,254,236,51,208,52,141,59,111,191,253,154,9,102,26,119,162,146,194,225,132,123,184,201,148,47,33,162,227,223,
  207,163,233,235,116,200,28,228,231,169,88,242,70,115,81,191,82,87,138,198,36,36,32,164,114,83,44,157,129,183,60,241,227,149,76,78,25,148,15,196,
  149,41,151,31,27,233,46,206,83,29,106,212,216,101,36,250,124,82,1,140,50,227,159,78,74,221,18,63,56,157,106,91,210,147,19,158,88,11,155,20,
  35,243,52,149,184,29,105,117,194,88,219,145,210,194,83,70,230,137,188,79,11,148,205,156,137,247,14,28,0,69,81,184,229,166,155,224,114,185,70,117,
  157,81,87,96,114,185,92,120,236,135,63,68,78,78,14,126,253,127,254,15,114,114,114,174,157,183,75,37,88,1,74,204,144,177,219,41,49,31,33,187,
  73,155,112,31,71,250,119,148,204,170,57,62,160,41,51,19,251,53,141,116,250,153,162,36,183,164,147,247,53,149,100,166,77,161,143,169,152,111,41,177,
  191,88,124,192,9,123,174,66,34,7,104,108,181,165,116,199,236,88,219,50,206,227,149,76,76,25,23,145,212,142,74,149,121,165,198,168,7,101,117,33,
  18,202,98,172,140,136,132,61,54,63,166,192,31,37,17,129,162,210,149,145,113,210,235,81,215,27,74,58,63,188,130,146,118,3,66,124,112,235,168,251,
  6,25,238,151,49,4,213,18,121,159,92,8,249,223,99,63,252,225,168,137,232,152,200,40,0,156,62,117,10,63,221,179,7,213,11,23,226,189,3,7,
  248,154,210,215,35,81,65,18,162,66,37,213,84,242,82,47,58,66,37,39,40,100,98,79,163,109,84,186,19,110,114,82,74,165,68,212,18,76,3,148,
  148,34,79,78,222,36,75,238,197,149,18,69,188,159,87,70,198,108,242,182,144,241,122,173,201,7,149,178,124,140,77,15,82,210,227,146,74,99,129,67,
  137,19,221,83,177,227,37,145,156,72,60,238,120,234,117,36,147,19,42,158,88,199,230,36,165,18,88,39,41,42,179,115,212,152,250,69,78,11,16,81,
  158,242,16,242,190,159,238,217,131,211,167,78,141,233,122,163,10,96,18,226,98,75,11,122,186,187,81,119,211,77,216,182,125,59,252,129,0,206,101,176,
  44,221,164,41,225,209,16,21,57,235,25,149,190,234,77,73,192,227,24,69,10,171,77,50,177,167,185,240,144,91,124,100,174,175,147,253,94,28,20,32,
  53,14,99,170,50,81,73,44,248,162,191,50,59,102,19,93,99,60,199,43,33,163,147,33,31,72,95,62,50,165,7,229,198,86,170,54,54,74,38,125,
  144,4,145,163,20,84,74,250,56,83,122,61,49,121,139,181,92,82,50,101,52,169,132,228,58,147,115,84,38,251,133,204,81,211,3,15,62,242,8,254,
  159,95,252,2,58,157,14,63,123,230,25,188,247,238,187,99,87,51,163,245,25,141,197,130,170,42,252,236,249,231,177,160,170,10,23,206,159,199,243,207,
  62,59,173,3,155,68,101,241,232,68,62,55,114,190,54,116,178,51,226,47,65,37,157,19,82,152,29,50,179,218,188,94,4,125,236,253,44,254,110,44,
  194,68,165,204,0,40,153,7,161,146,63,5,45,56,151,26,195,152,77,171,61,100,188,18,249,72,67,15,38,233,251,212,228,132,74,112,19,42,245,231,
  142,29,87,233,180,103,92,229,36,177,55,171,148,92,211,66,57,201,160,222,202,76,191,16,34,58,29,112,211,230,205,248,225,238,221,168,94,184,16,23,
  91,90,240,15,123,246,224,98,75,75,102,228,32,83,100,148,195,163,187,118,225,187,127,245,87,200,206,206,70,119,87,23,62,59,120,16,159,125,250,233,
  180,35,166,113,53,154,71,163,136,37,132,62,29,193,79,30,77,154,121,33,191,222,4,61,189,126,78,191,175,51,104,130,74,218,199,116,170,207,154,172,
  188,115,130,118,164,30,207,48,182,182,144,241,58,29,9,105,234,139,182,241,211,131,137,23,56,116,186,50,157,97,189,158,190,172,36,19,84,106,236,50,
  159,177,182,140,190,95,136,188,79,93,172,169,173,197,77,55,223,140,155,54,111,70,217,204,153,112,185,92,248,191,255,246,111,248,205,75,47,101,244,62,
  25,39,163,0,19,105,127,231,182,109,184,243,174,187,176,128,173,107,11,0,221,93,93,211,38,234,158,190,142,7,223,245,36,234,215,115,63,147,241,74,
  64,100,132,128,200,59,129,20,184,210,158,28,46,182,180,224,189,119,223,197,123,239,188,51,166,64,165,9,37,163,66,148,150,149,97,213,170,85,40,45,
  43,195,130,170,170,105,21,117,79,211,215,151,26,190,158,87,156,215,91,95,147,241,74,64,228,131,128,200,60,129,28,134,135,135,249,152,160,211,163,172,
  170,52,165,200,232,53,171,132,105,58,69,31,168,177,101,111,148,44,186,152,138,63,85,154,66,123,189,11,121,162,126,78,169,223,82,44,106,144,124,133,
  79,165,150,255,53,65,31,75,181,133,74,117,252,73,180,99,252,182,83,201,120,157,246,50,146,138,124,140,97,76,37,212,133,84,102,101,36,245,23,49,
  246,246,100,82,175,203,235,46,36,215,37,99,156,163,70,45,243,163,144,95,34,243,215,54,84,228,21,36,23,0,73,97,231,162,24,19,145,210,88,103,
  24,26,130,10,198,105,136,57,149,134,98,32,66,158,241,126,6,0,42,217,164,27,155,36,144,78,163,235,210,241,151,74,161,143,99,219,66,165,61,102,
  197,237,160,146,78,85,212,184,181,133,140,215,41,46,35,2,249,72,127,76,141,98,153,67,141,143,140,196,141,173,113,107,79,230,245,186,108,91,20,169,
  182,101,140,115,212,56,207,79,68,230,9,25,37,72,70,84,82,157,224,229,148,88,166,158,143,8,249,132,244,115,74,164,116,124,186,56,237,62,166,40,
  42,26,29,47,145,246,37,253,49,155,153,134,141,214,18,74,198,235,148,23,158,105,167,7,227,22,109,18,185,72,167,75,123,146,182,37,109,189,69,77,
  90,191,16,153,39,100,148,96,180,68,37,93,229,149,41,1,39,66,62,41,253,204,191,255,113,238,235,49,247,113,178,223,77,224,152,29,235,132,68,198,
  235,244,211,131,41,145,159,169,190,24,159,224,246,140,69,230,185,190,161,40,106,82,245,22,33,161,4,132,140,78,182,34,22,42,175,113,80,96,153,32,
  160,68,208,51,212,207,227,208,215,153,236,95,97,31,79,198,152,29,175,182,16,76,31,61,40,36,63,83,93,15,94,43,237,81,40,20,169,203,251,120,
  229,21,38,243,19,1,33,163,83,136,168,196,42,176,24,225,151,82,2,84,146,235,144,73,125,138,246,179,84,95,203,244,247,120,246,173,92,63,143,105,
  204,166,218,142,113,106,15,25,175,215,159,30,148,213,133,227,56,182,50,221,158,137,150,149,81,181,133,138,47,37,60,170,182,16,153,39,32,100,116,154,
  16,149,4,66,75,77,240,115,19,164,255,190,232,41,48,217,140,181,143,71,61,102,39,169,29,100,188,94,195,242,49,69,245,96,38,245,250,100,203,201,
  116,108,11,145,247,235,27,10,242,10,70,47,56,211,69,120,166,211,179,78,229,73,119,186,247,49,25,175,4,215,59,137,72,69,70,174,21,57,185,150,
  250,133,224,218,7,177,140,102,64,136,166,106,66,104,34,224,164,159,229,206,157,138,109,33,227,149,200,199,84,25,91,68,222,137,204,19,16,50,58,109,
  45,3,83,69,224,137,128,147,126,158,78,147,45,25,175,68,62,166,170,140,92,43,237,153,106,228,154,200,60,65,28,25,93,181,122,53,121,11,25,2,
  153,220,73,95,79,183,62,158,10,99,150,140,87,34,31,68,70,38,166,61,68,222,9,166,236,98,150,38,69,135,9,8,8,8,8,8,8,8,8,38,
  139,140,62,242,224,131,132,140,78,67,11,1,89,93,94,251,125,61,89,125,60,30,99,150,140,87,34,27,227,54,137,77,194,216,186,150,244,58,153,163,
  8,166,4,25,93,82,93,77,200,232,20,23,124,34,212,215,110,95,79,135,190,77,117,204,146,113,74,112,189,234,193,107,169,61,233,182,133,200,61,65,
  38,64,2,152,38,154,253,19,193,37,125,77,218,65,64,100,131,180,135,180,133,128,128,7,201,51,74,64,64,64,64,64,64,64,64,64,200,40,1,1,
  1,1,1,1,1,1,1,33,163,4,4,4,4,4,4,4,4,4,4,132,140,18,16,16,16,16,16,16,16,16,92,251,32,1,76,4,4,4,4,4,
  211,26,47,252,242,151,176,217,108,120,225,231,63,231,143,109,219,126,15,214,223,184,30,191,121,233,37,180,181,182,226,209,93,187,112,240,147,79,248,207,
  115,43,43,249,115,109,54,27,126,255,218,107,232,181,217,0,64,116,174,16,155,183,220,130,205,183,108,193,239,95,123,13,231,26,26,0,0,70,163,17,
  79,63,251,44,218,90,91,241,155,151,94,226,207,45,42,46,198,163,187,118,225,233,159,254,84,116,13,169,123,255,203,75,47,193,237,118,39,108,35,119,
  111,14,110,183,27,239,188,245,22,255,28,155,183,220,2,0,56,248,201,199,0,128,117,235,111,196,230,91,182,192,104,52,198,181,239,133,95,254,82,116,
  237,115,13,231,240,251,215,94,149,188,78,44,98,219,181,121,203,45,40,42,46,230,127,47,60,231,133,159,255,60,105,187,8,70,55,222,229,250,143,235,
  251,185,149,149,162,99,220,239,132,99,151,59,246,248,143,126,20,247,153,235,219,117,55,174,135,209,104,196,185,134,115,120,231,173,55,199,173,63,137,101,
  148,128,128,128,128,96,90,99,233,178,26,172,91,127,35,140,70,163,96,34,221,130,138,185,149,252,177,185,149,226,207,199,190,56,134,223,191,246,26,75,
  210,122,241,155,127,125,89,242,220,88,34,86,49,183,18,235,214,175,23,221,187,98,110,165,136,96,2,192,221,219,183,163,168,168,24,235,214,223,40,58,
  30,123,111,143,219,141,167,159,125,54,105,27,139,138,139,121,82,249,251,215,94,195,241,47,142,225,169,127,124,22,203,106,106,248,239,139,138,139,1,0,
  203,106,106,240,248,147,79,226,224,199,159,72,182,111,233,178,26,254,58,191,127,237,53,44,171,89,134,7,30,124,40,238,58,82,136,109,215,241,99,95,
  96,243,150,45,162,246,63,186,107,23,206,53,156,35,68,116,28,199,187,92,255,1,192,3,15,62,136,117,235,215,199,141,225,165,203,106,240,55,63,216,
  21,119,76,234,243,227,79,62,137,205,183,108,225,199,208,220,202,185,41,141,211,209,130,88,70,9,8,8,8,8,166,61,142,31,251,2,235,214,223,136,
  131,159,124,140,162,226,98,24,141,70,180,183,181,202,158,223,222,214,202,91,136,206,53,52,192,104,52,224,230,45,183,224,157,183,222,76,233,62,156,21,
  116,221,141,235,113,252,216,23,40,142,33,112,75,151,213,224,233,255,249,83,60,186,107,23,142,31,251,34,225,189,15,30,62,146,82,27,123,109,189,34,
  171,150,219,237,198,182,237,219,69,199,184,123,127,250,201,199,252,125,207,53,52,160,215,102,131,193,104,228,9,98,236,111,30,120,240,65,252,254,181,212,
  136,144,176,93,110,183,27,47,252,211,207,241,248,19,79,226,209,191,254,62,191,40,72,246,30,9,198,6,185,254,155,91,89,137,182,182,86,124,213,112,
  78,114,60,247,246,218,240,192,131,15,197,89,77,99,23,76,203,150,213,224,129,251,119,136,238,247,244,179,207,194,40,24,67,153,4,177,140,18,16,16,
  16,16,76,123,124,213,112,14,75,107,150,1,0,150,45,171,193,185,115,13,105,146,217,99,88,127,227,250,148,8,33,128,168,21,114,89,13,190,106,56,
  39,58,103,243,150,91,240,213,57,134,0,22,21,37,182,52,110,222,114,11,191,125,62,90,2,30,139,131,159,124,140,155,183,220,194,88,183,216,109,244,
  131,159,124,44,123,159,155,183,108,129,45,133,103,144,107,215,185,134,6,244,246,218,240,232,174,93,120,252,201,39,69,238,10,4,227,15,97,255,109,219,
  190,29,199,191,56,134,227,199,190,192,221,219,183,199,157,251,194,207,127,142,205,91,182,36,28,147,235,214,223,24,183,128,2,128,167,127,250,83,178,77,
  79,64,64,64,64,64,32,135,115,231,26,120,98,182,238,198,245,113,4,113,60,238,197,109,77,247,246,138,137,220,230,91,182,160,173,181,13,203,106,106,
  240,213,185,6,222,15,147,195,47,254,215,47,113,240,240,17,28,60,124,4,219,182,111,199,211,255,243,167,25,125,190,94,155,13,127,117,255,14,158,160,
  191,240,191,126,137,23,126,249,75,209,182,45,119,255,131,135,143,192,104,52,226,95,82,32,144,137,218,245,194,207,127,206,90,226,222,138,243,181,37,200,
  60,164,250,207,104,52,98,217,178,26,120,60,110,204,173,172,68,111,175,141,119,227,224,224,118,187,241,246,91,111,225,39,79,62,153,240,250,110,183,103,
  66,219,67,182,233,9,8,8,8,8,166,61,122,109,54,244,218,108,152,91,89,137,165,203,106,24,11,144,32,224,39,25,138,138,138,83,178,14,2,192,
  241,47,142,241,215,142,181,32,113,126,165,155,111,17,95,91,184,45,250,147,191,251,81,220,54,235,104,80,84,92,44,105,237,220,188,229,22,156,59,215,
  128,131,159,124,204,7,34,61,253,236,179,162,109,219,205,117,27,211,190,87,162,118,185,221,110,180,183,181,226,171,115,13,100,48,78,0,164,250,143,91,
  28,60,240,224,131,0,0,131,193,136,155,183,108,137,27,107,239,188,245,38,214,223,184,94,210,170,14,0,109,173,173,146,178,179,110,253,141,240,120,220,
  25,25,187,132,140,18,16,16,16,16,92,147,248,234,92,3,30,221,181,11,189,54,91,90,219,137,70,163,17,15,60,248,32,94,248,167,159,167,124,31,
  206,127,238,237,55,223,18,125,247,192,131,15,226,95,94,122,73,20,141,206,109,151,203,69,168,143,22,92,212,191,20,113,124,244,198,248,72,254,177,96,
  34,219,69,48,250,62,122,252,239,126,36,90,160,252,254,143,251,37,23,45,191,121,233,37,252,226,127,253,82,118,124,63,254,228,147,88,86,83,195,19,
  207,185,149,149,120,252,73,198,47,120,60,64,200,40,1,1,1,1,193,53,129,227,199,142,97,219,246,123,240,251,215,146,71,226,252,205,15,118,193,227,
  97,8,107,197,220,74,124,250,201,199,41,91,124,220,110,55,190,58,215,128,165,203,106,240,248,143,126,196,111,133,26,141,70,172,91,127,99,220,150,247,
  241,47,142,225,238,123,182,143,153,180,109,222,178,5,203,88,191,216,162,162,98,244,246,218,36,3,133,222,121,235,77,60,253,236,179,120,251,253,15,208,
  222,214,154,240,220,100,247,1,128,223,191,246,218,184,182,139,96,236,88,86,83,3,183,219,29,71,58,153,108,7,183,196,5,44,181,181,182,226,157,183,
  222,226,173,168,177,227,251,39,127,247,35,60,253,143,207,194,237,118,195,227,113,163,98,110,37,254,229,165,151,70,237,223,156,12,212,146,234,106,154,116,
  35,1,1,1,1,193,116,158,136,57,34,185,172,166,6,109,173,173,112,187,89,191,57,214,74,26,251,89,232,63,201,157,207,65,120,174,16,92,208,71,
  175,205,198,71,236,183,181,182,194,104,52,242,214,167,162,226,98,73,159,73,238,25,229,174,157,12,69,197,197,162,136,125,27,235,150,32,245,108,177,191,
  137,61,87,248,190,146,221,135,187,23,215,214,68,239,126,180,109,35,24,253,120,79,212,255,220,2,137,27,147,177,191,51,26,141,152,91,89,41,146,157,
  216,235,114,11,173,88,25,33,100,148,128,128,128,128,128,128,128,128,224,154,1,137,166,39,32,32,32,32,32,32,32,32,32,100,148,128,128,128,128,128,
  128,128,128,128,144,81,2,2,2,2,2,2,2,2,2,130,9,3,137,166,159,36,24,12,6,148,91,44,88,180,120,9,127,172,185,169,17,29,86,43,
  60,30,15,121,65,83,24,76,191,45,134,193,16,13,128,176,219,123,209,97,181,162,195,106,37,47,136,128,128,128,128,128,32,13,76,72,0,211,211,63,
  123,14,139,22,47,78,255,119,255,176,23,205,77,77,184,247,190,29,248,214,125,247,37,60,183,185,169,9,79,255,195,94,209,177,68,191,251,214,93,119,
  142,233,249,143,30,62,140,151,254,223,23,19,158,35,245,76,6,131,1,183,223,177,21,183,221,113,7,12,6,67,220,189,60,30,15,62,252,224,3,28,
  248,224,253,56,82,154,169,246,112,88,180,120,49,158,254,217,115,41,157,251,239,175,191,142,55,94,223,47,251,125,178,62,138,125,23,255,254,238,123,163,
  190,223,175,255,245,101,20,154,205,41,253,54,81,27,185,241,5,0,191,248,229,175,80,110,177,196,157,243,147,31,61,198,19,204,85,107,214,224,225,239,
  237,76,120,239,62,187,29,175,252,118,31,78,159,60,153,214,251,1,192,47,68,58,172,86,28,248,224,125,244,217,237,68,67,17,16,16,16,16,92,243,
  152,22,219,244,137,38,127,14,82,68,98,60,177,106,205,154,180,127,99,48,24,240,244,207,158,195,183,238,187,79,146,136,114,231,124,235,190,251,240,244,
  207,158,147,61,103,58,66,216,63,99,105,87,161,217,156,210,120,72,183,95,164,198,79,159,221,206,19,209,13,117,117,120,226,127,236,73,122,239,66,179,
  25,79,252,143,61,216,80,87,55,170,119,180,104,241,98,220,126,199,29,178,228,152,128,128,128,128,128,128,144,209,41,74,70,39,154,184,201,17,152,68,
  216,245,223,119,167,252,155,114,139,5,187,254,251,238,107,102,160,9,251,103,44,36,107,245,40,22,1,163,93,88,112,86,83,0,120,232,123,59,211,238,
  235,177,144,102,110,225,114,45,45,72,8,8,8,8,8,8,166,45,25,53,167,56,169,143,198,21,96,44,216,80,183,41,229,115,23,45,94,156,182,53,
  117,213,154,53,19,222,166,241,68,38,218,34,244,177,205,220,115,73,95,243,212,201,122,254,185,71,67,10,111,191,99,235,152,9,252,104,44,172,4,4,
  4,4,4,4,132,140,102,24,153,222,150,157,12,114,181,106,77,173,228,241,62,187,29,79,255,195,94,89,255,64,185,223,77,71,232,89,66,55,22,66,
  185,106,2,45,163,156,223,167,156,37,215,227,241,224,161,239,220,31,231,23,204,33,85,11,176,208,2,123,45,247,63,1,1,1,1,1,129,20,38,36,
  154,62,54,194,88,142,196,245,217,237,176,11,72,153,199,227,73,203,34,181,104,241,146,132,19,123,166,81,110,177,160,208,108,78,41,208,68,142,152,156,
  58,121,18,205,77,77,56,122,248,176,100,128,75,186,91,218,169,6,36,189,250,219,125,73,251,76,24,64,101,183,247,142,249,125,89,44,21,113,129,61,
  147,77,68,203,45,22,201,49,38,124,78,97,212,188,212,59,146,27,115,169,46,86,158,254,135,189,178,129,86,230,41,186,16,35,32,32,32,32,32,152,
  86,100,52,150,248,200,69,82,31,57,124,8,255,254,250,235,41,77,232,205,77,77,113,223,233,99,72,5,141,241,175,116,186,104,241,98,28,61,124,56,
  233,121,114,164,194,227,113,39,124,86,225,239,82,105,79,170,4,40,25,201,127,245,183,251,210,34,246,169,60,27,103,225,78,133,96,75,93,47,85,139,
  106,58,253,46,247,190,184,45,250,76,141,163,100,215,144,123,215,133,132,140,18,16,16,16,16,92,235,100,148,166,167,86,105,250,216,231,209,235,83,183,
  140,150,91,44,72,181,61,153,106,247,170,53,107,112,228,208,161,164,247,73,68,42,18,61,75,161,217,156,210,179,166,219,158,100,167,211,116,230,222,81,
  108,91,244,41,90,187,99,239,31,75,28,59,172,86,89,98,203,253,54,81,19,104,90,126,27,188,185,177,49,35,239,125,162,174,65,64,64,64,64,64,
  48,109,201,232,84,127,192,242,10,57,127,61,119,220,177,201,216,210,28,143,128,154,107,21,28,113,28,77,48,144,217,108,22,17,207,76,20,7,48,24,
  12,146,150,209,14,171,85,228,46,210,220,216,132,55,16,159,243,180,175,55,177,123,6,41,94,64,64,64,64,64,64,112,13,144,81,57,226,114,244,208,
  97,172,142,177,106,77,198,150,166,193,96,136,123,14,2,49,33,227,250,208,96,48,196,165,196,234,179,219,83,234,183,88,210,223,220,212,136,114,75,197,
  216,22,18,75,146,111,209,115,247,106,110,106,76,72,148,165,144,232,55,4,4,4,4,4,4,4,211,132,140,202,17,14,57,171,83,185,197,50,225,37,
  25,229,72,205,100,224,158,59,197,233,132,126,243,242,62,73,178,231,241,184,101,3,115,50,137,14,171,85,100,125,140,237,79,123,138,100,116,85,173,56,
  120,169,163,221,58,118,50,42,151,210,169,190,62,133,113,105,129,193,96,132,193,96,192,189,59,118,72,158,115,224,253,15,82,126,22,185,107,76,100,64,
  30,1,1,1,1,1,1,33,163,105,16,28,57,76,4,193,138,197,234,53,181,162,109,221,169,67,228,45,146,68,143,171,44,148,200,197,224,153,231,196,
  145,221,79,237,221,59,42,75,95,172,59,197,198,77,117,9,191,79,149,56,54,55,53,98,195,166,77,99,126,63,114,239,38,25,30,222,249,125,217,224,
  39,143,199,131,87,247,237,75,249,125,189,249,222,251,9,198,122,59,209,82,4,4,4,4,4,215,52,166,124,158,81,169,9,223,227,241,160,79,38,213,
  208,100,148,80,44,52,155,167,100,10,158,141,50,100,45,118,27,122,34,23,14,133,230,162,148,23,22,66,34,42,116,215,136,77,1,150,73,100,98,107,
  125,196,227,201,136,191,104,135,213,138,55,246,239,39,90,138,128,128,128,128,128,144,209,169,6,143,199,45,75,70,12,198,201,41,159,56,21,83,240,200,
  85,136,74,20,253,63,222,24,205,98,97,117,204,22,189,117,28,173,133,250,12,148,223,44,52,155,241,196,158,61,99,10,110,123,233,197,23,241,248,99,
  187,73,16,20,1,1,1,1,1,33,163,147,9,185,201,60,145,53,141,68,183,179,4,110,77,173,100,240,87,135,213,58,161,62,181,205,141,98,159,199,
  216,103,138,253,62,149,62,77,229,55,153,30,115,163,193,195,59,119,142,250,183,187,118,239,30,211,239,9,8,8,8,8,8,8,25,157,0,72,85,62,
  50,24,12,164,87,17,111,77,228,48,153,86,209,209,32,54,250,30,24,223,40,117,169,251,73,225,169,189,123,112,207,157,91,241,212,222,189,178,228,62,
  85,43,240,27,175,239,151,28,203,183,223,177,21,27,235,54,145,193,76,64,64,64,64,64,200,232,100,65,46,74,189,163,157,153,252,165,182,234,39,202,
  103,116,42,71,57,27,12,6,217,100,238,167,83,244,23,61,122,248,16,222,120,125,63,255,175,111,12,229,64,229,200,90,42,22,90,169,180,89,153,178,
  236,202,109,129,167,99,29,109,110,106,76,232,215,153,138,47,241,27,251,247,227,231,207,75,151,113,189,125,235,86,162,165,8,8,8,8,8,174,105,76,
  203,104,250,100,126,116,102,179,121,66,162,219,165,74,146,78,5,200,109,209,55,55,53,165,252,94,142,28,58,156,49,11,164,92,127,165,226,15,25,187,
  32,201,228,34,32,54,237,148,240,158,7,62,136,70,184,223,187,99,7,238,189,111,135,228,251,124,106,239,158,132,1,97,133,230,162,148,222,249,104,173,
  171,92,138,41,249,235,182,195,227,241,192,108,54,163,208,92,196,255,45,92,184,148,91,42,226,142,167,139,84,159,99,66,22,177,139,151,192,227,113,143,
  105,209,194,189,151,230,166,70,209,231,209,92,67,174,237,139,22,47,153,208,247,50,209,48,155,205,240,36,8,230,227,242,14,79,197,76,36,4,4,4,
  132,140,138,148,181,20,54,110,170,195,162,37,139,101,173,78,169,18,128,177,147,209,198,41,73,70,87,201,108,209,31,157,164,45,122,198,170,186,88,230,
  120,146,182,196,88,70,251,236,189,252,184,144,115,201,48,155,205,252,68,63,154,254,155,78,69,12,18,165,152,2,162,41,185,158,216,179,23,229,22,11,
  14,124,240,62,94,217,183,79,64,34,43,240,204,115,207,141,58,117,87,186,207,49,17,11,177,39,246,236,1,0,60,120,255,142,81,19,61,238,189,220,
  115,231,86,209,231,209,92,67,174,237,153,120,239,82,4,239,225,157,59,177,161,110,19,60,30,15,142,30,62,36,234,239,213,107,106,241,240,206,157,40,
  52,155,209,220,212,132,87,246,189,28,71,218,87,175,169,197,237,91,183,226,169,189,123,100,175,123,224,131,247,19,238,8,24,12,6,60,177,103,47,30,
  127,108,119,194,69,234,51,207,61,143,167,246,238,33,129,122,4,4,132,140,78,63,108,72,226,71,103,54,155,209,60,1,207,209,220,216,4,220,55,181,
  222,77,162,138,80,19,153,210,73,8,185,133,65,42,11,134,88,194,185,161,110,83,210,254,231,206,121,106,239,222,132,231,113,238,30,114,11,161,233,80,
  65,73,72,24,56,11,110,44,105,42,183,88,80,110,177,224,232,225,67,216,80,183,73,68,78,38,242,57,38,68,55,108,170,195,169,147,245,88,180,120,
  9,54,212,109,18,89,184,199,178,232,28,143,182,140,199,53,31,222,185,19,139,22,47,193,63,61,255,60,12,6,3,30,218,185,19,29,237,86,28,57,
  124,8,102,54,203,195,27,175,239,71,115,99,19,238,221,177,3,79,238,217,139,71,191,191,83,52,86,118,237,222,29,71,80,31,222,185,19,229,150,10,
  254,186,187,118,239,70,95,175,29,71,14,31,146,125,142,3,239,39,127,247,7,222,127,31,15,239,220,137,255,239,197,23,201,108,76,64,112,29,99,74,
  251,140,142,214,255,179,176,104,98,210,44,77,69,178,146,136,136,78,150,245,193,227,246,164,117,124,194,158,203,227,145,12,28,2,166,86,85,173,177,98,
  227,166,77,232,179,219,241,198,254,253,147,86,190,214,96,48,224,246,59,182,226,222,29,59,36,119,52,54,214,109,226,191,43,183,88,248,115,98,115,204,
  10,191,147,91,136,157,174,63,137,211,39,235,227,242,236,38,187,150,217,108,198,189,59,118,196,5,141,25,12,134,184,93,26,238,121,199,146,125,129,123,
  30,238,250,220,59,186,253,142,173,113,139,176,100,239,143,195,170,53,181,120,99,255,126,156,58,89,143,35,135,15,49,11,16,246,61,172,90,83,203,231,
  174,109,110,106,196,63,61,255,28,10,217,247,205,61,207,51,207,61,47,41,19,177,215,61,240,193,251,178,59,48,102,179,25,171,214,212,198,17,85,238,
  157,9,199,223,145,195,135,176,106,77,237,148,204,211,76,64,64,64,200,40,175,128,39,146,196,142,142,144,78,173,64,38,185,128,151,211,245,39,39,237,
  153,198,18,192,52,89,11,138,107,41,69,216,234,53,181,56,117,178,30,118,182,186,148,92,166,133,241,130,217,108,198,175,95,222,135,111,110,221,138,213,
  107,106,241,235,151,247,137,8,201,223,238,222,141,93,187,119,99,209,226,37,248,197,175,94,196,147,123,246,242,4,234,153,231,158,19,149,125,125,120,231,
  247,101,43,111,113,215,60,117,178,30,167,234,79,198,145,205,68,215,42,183,88,240,139,95,189,200,19,38,97,90,45,110,187,157,195,11,191,122,17,15,
  177,22,200,103,158,123,110,212,41,184,184,231,225,174,255,196,158,189,88,93,91,139,123,119,236,192,11,191,122,49,229,247,39,196,79,30,219,29,183,3,
  194,85,57,91,180,100,177,104,188,123,60,30,81,37,54,179,217,140,127,122,254,249,184,223,51,254,192,6,209,111,155,27,155,100,101,100,213,154,218,184,
  64,73,225,59,123,98,207,30,252,237,238,232,246,253,233,147,245,178,1,151,4,4,4,215,7,166,236,54,253,88,8,101,42,37,65,133,138,116,44,193,
  14,169,250,141,142,54,216,41,29,178,107,22,88,57,196,147,145,71,118,59,45,157,247,63,29,182,173,211,133,181,221,138,13,117,82,227,99,49,12,6,
  3,19,128,49,6,11,238,88,178,16,100,74,142,10,205,102,62,165,215,145,67,135,112,239,142,29,0,38,110,91,244,222,29,59,208,97,181,242,91,249,
  28,217,59,117,178,30,102,179,153,119,169,96,100,105,73,92,41,218,84,177,170,118,13,191,3,192,253,119,213,154,218,148,182,234,191,185,117,171,232,25,
  159,121,238,121,201,243,54,214,109,66,161,217,140,31,124,127,39,60,30,15,255,188,7,222,127,127,204,126,234,167,234,235,113,224,131,247,81,110,177,224,
  133,95,189,200,187,138,220,187,99,7,107,201,100,158,233,246,59,182,242,239,47,22,194,103,40,183,88,176,161,110,19,94,101,221,50,12,6,35,58,60,
  214,56,221,192,21,10,225,116,68,236,174,0,167,79,133,59,43,30,143,71,214,88,176,186,182,22,167,234,235,147,190,179,55,246,239,135,221,110,135,181,
  221,138,213,181,181,25,113,169,32,32,32,32,100,52,163,72,68,40,223,120,125,63,79,190,164,252,7,83,33,125,194,9,143,139,138,30,21,25,77,209,
  111,84,174,6,59,103,185,145,35,223,169,214,110,231,44,18,82,144,170,67,159,44,184,71,202,218,51,90,31,55,57,18,203,145,143,68,224,250,58,17,
  49,144,34,240,205,77,141,232,179,247,198,149,31,149,122,55,137,22,44,167,78,214,203,158,195,77,198,137,182,24,39,59,82,248,155,91,183,162,143,181,
  136,2,140,21,234,225,157,59,177,177,110,83,218,11,148,209,98,213,154,90,158,16,1,76,32,221,189,247,237,64,185,197,194,6,154,89,249,49,210,220,
  212,56,170,133,161,217,108,198,234,53,181,120,73,224,123,120,250,100,61,190,185,117,107,74,36,103,85,204,111,143,30,58,36,169,71,86,213,174,193,105,
  129,203,11,51,206,236,41,147,222,68,56,202,246,71,108,251,185,45,114,78,86,58,172,86,126,123,61,209,216,124,230,185,231,209,97,181,78,88,63,11,
  245,175,48,184,105,85,237,26,28,61,124,72,244,206,30,127,108,55,47,27,29,86,43,41,240,64,64,64,200,232,212,68,34,203,40,167,232,184,32,133,
  68,68,97,188,145,170,181,176,185,177,73,114,107,109,67,221,38,20,154,139,100,9,116,58,213,134,228,74,161,46,90,188,56,206,218,148,44,184,103,170,
  32,81,196,238,162,197,75,100,200,104,52,247,103,50,50,154,168,255,22,45,89,140,83,39,235,101,173,155,229,22,11,31,216,33,125,237,201,119,225,88,
  197,166,249,122,243,189,247,227,72,85,58,36,37,54,189,85,58,81,224,177,233,123,184,207,6,131,17,6,163,33,206,151,121,52,190,205,220,66,108,23,
  187,229,31,219,79,201,8,46,103,5,79,182,136,48,24,140,113,215,178,219,237,25,41,67,156,40,5,146,20,89,147,91,176,115,68,180,207,110,199,63,
  61,255,220,164,143,65,41,139,236,84,112,209,33,32,32,32,100,116,212,196,42,117,50,91,49,97,207,154,202,22,252,209,195,204,246,168,20,73,150,251,
  45,151,154,229,90,64,135,213,42,90,96,76,165,201,72,174,255,56,75,148,221,110,151,61,231,246,59,228,173,197,71,39,185,218,21,151,111,246,165,23,
  95,20,145,171,213,181,107,36,131,100,18,142,223,67,135,68,11,163,100,150,245,68,139,67,225,103,143,219,35,25,172,147,46,54,110,218,132,230,166,166,
  184,197,203,19,123,246,96,227,166,212,50,8,164,122,223,76,60,111,186,144,242,229,76,68,68,153,5,131,56,101,82,135,181,61,110,145,95,110,177,36,
  141,122,231,250,90,72,234,83,33,248,137,222,81,186,191,39,32,32,184,182,49,101,3,152,228,182,62,133,214,166,68,150,153,137,44,11,154,138,133,200,
  227,241,136,182,1,83,193,75,47,190,120,205,228,223,203,132,245,107,252,136,114,187,204,130,198,194,143,163,87,246,189,156,54,249,62,50,201,11,137,213,
  181,107,208,103,103,210,239,52,55,53,242,255,56,242,145,78,84,61,67,200,163,215,72,167,255,78,157,172,23,5,77,113,247,109,110,106,196,233,147,245,
  124,234,41,238,157,199,18,38,78,23,200,149,106,229,124,165,143,30,18,183,147,187,190,176,157,114,215,138,125,198,141,155,234,164,219,82,31,127,189,114,
  139,37,173,29,140,116,113,234,100,61,54,8,158,103,209,226,37,248,205,203,251,36,117,220,174,221,187,97,48,24,36,115,119,114,65,71,220,59,224,22,
  43,201,244,23,23,232,36,204,78,192,144,255,198,148,22,158,177,239,140,35,204,92,214,2,66,76,9,8,8,166,172,101,52,217,246,106,50,148,87,76,
  96,68,125,138,126,163,167,78,214,227,165,23,153,168,210,68,100,217,227,241,224,213,125,251,38,45,47,232,120,32,54,241,253,100,7,246,136,38,207,4,
  249,70,87,179,41,106,152,224,150,189,120,98,207,158,164,11,29,97,32,204,100,129,43,9,43,101,89,231,162,234,55,108,218,196,91,18,31,222,185,83,
  68,94,142,30,58,148,49,50,125,224,253,15,120,55,17,143,199,131,13,117,155,120,95,96,187,221,142,3,31,188,143,23,126,245,34,111,125,22,62,199,
  169,147,245,120,104,231,78,20,22,49,62,161,35,18,36,152,139,136,151,146,151,83,245,39,177,161,110,19,239,255,43,119,45,225,51,2,242,254,215,220,
  14,199,51,207,61,143,230,166,70,108,172,219,196,251,40,203,97,172,239,150,123,182,39,246,236,65,135,213,138,219,239,216,42,185,32,224,252,102,251,236,
  118,60,177,103,175,104,177,245,10,171,79,238,181,239,192,47,126,245,34,95,125,236,192,7,239,167,180,176,120,99,255,126,60,177,103,15,159,130,74,111,
  48,200,90,84,155,155,26,97,17,232,223,216,119,198,189,123,174,191,44,21,150,107,50,56,146,128,128,96,138,147,81,185,160,20,161,117,129,179,114,196,
  145,154,94,123,90,215,122,3,251,147,19,165,222,196,65,38,82,207,34,252,77,115,83,163,228,115,72,93,247,200,225,67,56,117,178,30,183,111,221,138,
  69,139,151,136,182,126,185,73,237,192,251,210,19,68,115,99,83,194,246,36,251,62,158,28,38,14,16,74,151,144,75,221,155,187,207,145,67,135,69,91,
  197,92,255,244,217,123,37,159,33,21,75,147,220,24,17,254,86,238,250,194,103,147,235,63,142,44,9,251,249,7,223,103,170,208,172,174,173,21,245,157,
  199,227,97,44,113,245,39,37,137,70,42,125,51,214,190,16,222,195,96,48,224,192,7,239,203,186,10,188,178,111,31,22,45,89,44,251,126,198,18,120,
  21,219,214,230,166,70,60,181,119,47,110,223,122,7,10,205,69,120,117,223,62,209,59,122,101,223,62,116,180,91,81,88,100,198,43,251,94,198,195,59,
  191,207,127,247,210,139,47,242,137,220,185,60,169,177,207,214,215,107,151,221,69,56,117,178,158,111,95,162,107,113,209,234,27,54,213,97,196,227,193,83,
  123,247,96,117,109,109,220,24,242,120,60,248,201,99,187,249,28,163,71,14,31,146,37,101,201,222,237,27,175,239,151,149,67,225,119,194,247,183,104,241,
  18,28,248,224,125,217,123,38,211,67,79,237,221,131,213,107,106,81,88,100,230,243,141,74,245,95,172,238,58,117,178,30,143,63,182,27,171,107,107,225,
  113,123,68,1,73,113,58,238,208,33,60,41,32,195,177,239,140,35,199,220,239,23,45,94,130,159,79,1,223,86,2,2,130,201,3,181,184,170,138,38,
  175,129,128,128,96,50,176,104,241,18,252,237,238,221,120,252,177,221,124,186,160,95,191,188,15,47,189,248,226,53,181,51,112,189,225,153,231,158,151,37,
  187,177,253,127,239,142,29,147,190,147,64,64,64,48,185,80,145,87,64,64,64,48,89,224,182,155,127,253,242,62,222,215,176,207,110,39,68,116,154,131,
  179,112,63,181,55,49,25,189,119,199,142,180,253,177,9,8,8,174,61,16,203,40,1,1,193,164,130,43,227,89,88,100,78,88,239,156,96,122,97,245,
  154,218,132,193,110,92,25,84,178,240,32,32,32,32,100,148,128,128,128,128,128,128,128,128,96,210,160,32,175,128,128,128,128,128,128,128,128,128,128,144,
  81,2,2,2,2,2,2,2,2,2,66,70,9,8,8,8,8,8,8,8,8,8,8,25,37,32,32,32,32,32,32,32,32,32,100,148,128,128,128,128,
  128,128,128,128,128,128,144,81,2,2,2,2,2,2,2,2,2,66,70,9,8,8,8,8,8,8,8,8,8,8,25,37,32,32,32,32,32,32,32,32,
  152,246,248,255,7,0,146,225,72,126,220,8,78,33,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00128_png = (const char*) resource_TalComponent_bmp00128_png;
const int TalComponent::bmp00128_pngSize = 251706;



















































// JUCER_RESOURCE: bmp00129_png, 49914, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00129.png"
static const unsigned char resource_TalComponent_bmp00129_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,37,0,0,7,58,8,6,0,0,0,242,185,174,195,0,0,0,1,115,82,71,66,0,174,206,28,233,
0,0,0,4,103,65,77,65,0,0,177,143,11,252,97,5,0,0,0,9,112,72,89,115,0,0,14,195,0,0,14,195,1,199,111,168,100,0,0,194,143,73,68,65,84,120,94,237,221,9,188,101,85,149,223,113,204,96,98,38,147,152,57,157,116,
18,99,230,78,210,153,204,208,166,211,105,98,230,164,98,230,185,146,144,116,59,116,107,35,32,168,224,108,3,106,43,218,98,11,206,32,131,140,162,2,45,2,162,136,76,130,204,2,66,65,201,80,69,85,81,80,20,32,
20,188,155,245,221,239,172,251,246,61,247,222,247,238,125,239,158,3,126,62,197,231,115,62,175,120,195,189,235,238,97,237,181,126,251,191,215,62,96,48,24,28,48,207,115,192,1,7,188,32,158,23,199,243,227,
241,252,68,60,47,143,231,21,191,241,55,254,198,159,245,181,249,127,223,247,115,191,247,130,121,94,223,239,206,108,80,188,248,139,226,121,105,60,7,198,115,104,60,39,199,115,117,60,15,252,254,223,255,251,
159,254,131,127,240,15,14,94,252,226,23,63,237,255,155,239,251,185,223,243,251,254,238,69,179,26,183,166,81,241,98,47,140,231,101,241,108,142,231,148,120,182,199,243,108,60,131,63,242,71,254,200,224,159,
252,147,127,50,248,224,7,63,56,120,215,187,222,53,248,127,255,239,255,13,254,252,159,255,243,140,139,143,123,128,199,239,249,125,127,231,239,189,206,11,215,50,110,85,163,226,5,94,18,207,166,120,78,138,
103,135,55,122,193,11,94,80,222,244,223,255,251,127,63,120,207,123,222,51,248,226,23,191,56,248,213,95,253,213,193,91,223,250,214,193,47,254,226,47,14,126,254,231,127,126,240,47,255,229,191,28,252,149,
191,242,87,6,47,122,209,139,202,239,55,6,250,123,175,227,245,94,178,154,97,83,141,138,63,252,177,120,14,142,231,150,223,240,27,126,195,18,67,116,209,255,248,31,255,99,112,236,177,199,14,206,56,227,140,
193,175,253,218,175,13,222,253,238,119,15,222,248,198,55,14,14,62,248,224,193,47,253,210,47,13,94,243,154,215,148,22,59,232,160,131,6,255,234,95,253,171,193,223,251,123,127,175,124,136,223,246,219,126,
27,3,151,188,94,243,186,63,54,205,176,137,70,197,31,25,164,239,124,225,11,95,120,159,238,248,167,255,244,159,150,55,255,196,39,62,49,248,220,231,62,87,90,70,119,29,122,232,161,35,6,189,225,13,111,24,188,
254,245,175,47,45,246,115,63,247,115,229,249,191,255,247,255,22,3,255,211,127,250,79,131,191,252,151,255,242,224,15,253,161,63,52,136,73,113,159,215,247,62,147,12,27,51,170,105,161,119,254,181,191,246,
215,30,127,203,91,222,50,248,204,103,62,51,56,253,244,211,75,171,188,247,189,239,45,221,196,152,67,14,57,100,170,65,191,240,11,191,48,240,188,238,117,175,43,45,247,234,87,191,186,24,248,218,215,190,182,
180,244,223,249,59,127,71,235,61,222,24,54,214,98,35,70,53,99,232,224,223,242,91,126,203,125,135,31,126,248,224,220,115,207,29,252,202,175,252,202,224,168,163,142,26,28,118,216,97,67,99,24,164,187,222,
250,214,35,7,199,31,255,241,193,9,39,126,122,112,252,199,78,28,28,251,190,15,14,142,60,234,29,241,179,67,138,65,140,72,163,140,53,45,182,121,243,230,193,166,77,155,74,151,70,119,106,49,67,100,100,140,
13,141,50,43,154,65,120,203,95,252,139,127,113,112,233,165,151,14,62,242,145,143,12,222,244,166,55,141,25,244,254,247,127,96,112,231,247,239,25,220,117,247,253,131,43,175,250,238,224,130,11,47,29,124,
225,140,243,6,159,250,244,41,131,15,127,228,132,193,47,31,253,193,193,17,111,126,251,224,144,67,223,84,12,99,144,150,50,214,60,255,249,63,255,231,193,239,253,189,191,119,16,99,213,36,48,102,55,197,80,
25,206,202,218,40,211,245,36,131,250,31,254,195,127,56,184,236,178,203,198,90,232,237,111,127,251,224,135,63,252,225,224,217,103,159,29,236,219,247,236,224,254,7,118,13,174,253,206,45,131,95,255,234,101,
197,168,79,124,242,228,193,7,63,244,177,193,187,223,243,254,193,155,223,242,174,48,42,102,228,235,15,139,238,91,30,252,249,252,239,255,253,191,7,63,241,19,63,145,179,114,41,90,204,251,190,44,199,87,49,
42,254,227,24,55,155,246,62,129,22,58,229,148,83,70,90,232,99,31,251,216,224,225,135,31,46,6,45,63,131,193,195,187,159,24,220,112,227,29,131,139,190,246,205,98,212,9,39,126,110,240,254,15,252,234,224,
237,239,56,122,240,166,195,223,62,120,195,47,29,62,248,133,95,60,116,240,154,215,254,82,180,212,171,139,81,57,240,127,230,103,126,166,24,213,180,214,142,104,168,205,191,235,119,253,174,226,96,211,40,30,
151,131,27,252,216,143,253,216,224,75,95,250,210,224,125,239,123,223,112,12,105,161,81,131,150,13,219,247,204,96,112,199,157,91,7,151,92,122,69,49,234,215,62,254,153,193,209,199,124,104,240,150,183,134,
155,56,228,45,165,149,94,251,186,55,134,81,7,23,195,180,152,113,245,127,254,207,255,41,126,238,119,255,238,223,157,173,197,184,83,126,211,111,250,77,47,109,26,169,172,101,150,130,237,172,54,117,207,63,
255,252,210,117,6,180,39,187,108,165,149,178,181,158,29,220,119,255,206,193,55,190,121,117,49,234,248,143,125,170,116,157,86,122,253,27,98,60,53,6,249,170,197,24,105,108,233,190,255,254,223,255,123,241,
254,26,162,121,182,255,190,223,247,251,14,252,51,127,230,207,176,167,44,154,214,168,103,117,221,39,63,249,201,226,143,114,218,127,244,163,199,87,93,182,98,76,26,184,103,207,19,131,171,174,190,161,24,117,
220,135,63,30,179,239,189,131,131,223,248,230,193,235,126,225,144,210,66,12,98,140,174,52,198,14,123,211,225,131,255,245,191,254,87,153,133,252,95,227,84,203,146,20,51,242,208,88,71,217,83,28,165,197,
179,172,99,102,29,71,153,211,222,44,51,168,141,161,73,45,245,236,210,96,112,227,77,119,22,163,140,39,173,180,60,142,14,46,134,105,49,93,233,251,6,255,155,14,127,91,49,232,127,254,207,255,89,122,229,101,
47,123,25,103,154,227,235,228,232,45,246,148,240,227,234,63,254,199,255,120,105,161,79,125,234,83,67,79,205,15,153,246,102,153,65,109,12,77,235,66,70,233,58,45,194,32,134,105,49,198,24,99,6,191,159,115,
23,175,127,253,27,138,81,28,233,63,250,71,255,168,172,145,205,128,23,117,176,167,196,67,15,252,236,207,254,236,224,27,223,248,198,200,90,198,49,242,67,166,189,89,102,80,27,67,186,76,11,165,129,123,247,
62,81,90,234,240,35,222,81,186,139,97,140,209,149,12,49,248,181,34,119,193,143,29,118,216,17,197,32,227,202,128,255,179,127,246,207,230,194,45,236,97,207,1,175,248,195,127,248,15,63,125,194,9,39,148,37,
165,94,92,121,106,142,145,31,50,237,205,50,131,218,24,210,101,12,100,16,227,24,165,171,24,198,16,70,24,99,6,191,89,201,93,240,99,28,236,187,223,115,76,49,232,191,253,183,255,54,248,175,255,245,191,14,
254,238,223,253,187,217,125,226,177,87,28,32,98,212,117,31,250,208,135,6,71,31,125,244,112,181,183,184,90,58,188,217,172,207,25,103,126,105,48,203,115,220,113,199,15,13,250,47,255,229,191,12,56,235,156,
133,209,141,34,216,3,94,241,7,254,192,31,120,250,31,252,131,127,80,130,53,225,71,174,246,214,50,159,204,39,244,73,125,98,159,92,11,104,9,45,162,101,116,23,99,180,152,150,211,130,90,82,139,106,89,45,172,
165,181,184,150,255,240,71,62,86,90,136,65,150,156,202,53,44,183,84,142,41,110,223,140,179,214,101,248,97,113,53,6,140,5,70,24,27,12,49,86,210,99,27,67,12,99,84,118,101,25,107,49,230,140,61,6,26,139,198,
164,177,105,140,30,115,236,7,134,6,153,241,150,189,166,165,134,99,170,204,190,24,87,131,87,188,226,21,131,119,190,243,157,37,30,18,122,88,237,205,22,134,152,61,102,17,99,204,42,179,43,7,181,159,51,138,
1,147,102,167,89,107,246,154,197,102,243,161,135,30,86,90,232,63,254,199,255,56,248,75,127,233,47,213,14,116,56,251,134,126,74,16,102,160,107,177,140,135,172,246,203,254,229,237,197,223,24,204,252,79,
122,105,223,215,138,140,210,101,245,172,28,53,112,80,252,221,109,183,221,89,12,226,163,180,82,227,10,138,97,177,204,156,28,173,86,252,212,208,163,139,113,254,241,63,254,199,131,35,143,60,114,24,15,29,
252,198,8,232,194,19,27,55,185,150,49,136,97,90,76,87,26,99,140,50,134,138,187,24,46,218,227,43,192,241,199,31,95,12,210,74,85,164,80,60,250,159,254,211,127,250,208,8,46,139,71,31,174,125,130,252,159,
252,201,159,44,9,129,113,37,64,19,15,189,237,237,239,141,1,127,210,224,196,79,124,182,180,144,71,139,105,37,93,103,240,51,202,160,158,214,133,12,125,242,201,39,139,127,98,148,248,221,18,147,179,46,190,
110,255,27,127,227,111,28,24,81,196,11,198,162,4,49,180,168,210,128,23,198,190,237,109,111,27,252,250,175,255,122,249,244,15,61,180,163,24,166,197,116,165,49,102,240,155,149,140,50,203,12,234,73,158,127,
199,142,29,165,245,179,149,50,116,73,163,126,199,239,248,29,167,188,228,37,47,89,142,18,218,241,148,46,148,129,28,115,204,49,197,160,11,47,188,112,164,59,180,88,122,108,131,223,120,226,46,24,101,218,155,
101,6,117,189,86,50,72,162,145,6,253,135,255,240,31,6,209,85,117,43,237,248,99,127,236,143,109,14,67,87,226,169,198,176,97,228,41,103,251,192,7,62,48,56,231,156,115,70,12,210,98,186,210,24,51,248,117,
29,119,193,143,49,138,31,50,237,205,178,229,69,124,185,203,234,22,98,144,174,179,222,53,173,180,20,33,203,73,127,226,79,252,137,209,200,179,49,106,24,163,191,244,165,47,45,113,245,117,215,93,55,52,234,
130,11,46,40,153,140,120,232,200,35,223,86,86,123,238,130,31,227,96,25,197,49,242,67,166,189,89,102,80,231,24,50,176,25,100,173,147,205,100,183,197,108,187,229,47,252,133,191,176,41,126,62,30,163,55,134,
201,136,15,150,239,197,47,150,164,243,219,223,254,246,224,236,179,207,46,6,137,24,5,104,25,15,89,237,45,174,214,50,75,7,79,205,49,166,31,210,93,117,151,105,161,191,249,55,255,230,224,247,252,158,223,147,
70,221,23,153,211,193,225,35,39,103,51,195,160,125,57,51,126,231,159,252,147,127,242,113,14,85,171,233,78,174,194,39,181,144,102,60,148,225,71,189,184,230,210,145,198,252,139,127,241,47,138,83,54,134,
34,6,31,198,78,241,30,179,229,125,149,97,37,67,150,201,138,163,99,86,148,60,237,119,254,206,223,57,248,171,127,245,175,150,136,81,107,101,248,145,171,125,26,164,171,94,249,202,87,14,254,220,159,251,115,
37,14,231,32,235,39,94,123,190,12,185,213,98,133,37,96,0,191,245,183,254,214,226,87,98,80,150,156,205,242,192,35,235,78,198,120,210,75,107,149,12,115,91,198,44,25,67,209,98,94,119,62,150,80,25,246,146,
104,173,77,242,178,120,145,33,117,137,113,48,136,200,162,180,160,0,77,203,29,120,224,129,3,227,176,138,185,107,226,98,12,237,48,203,98,144,111,106,143,161,54,79,88,147,79,201,92,37,138,49,133,55,75,131,
120,94,75,130,217,163,59,181,220,111,255,237,191,125,160,37,181,138,85,161,90,207,10,159,226,24,35,235,222,108,218,215,179,108,46,234,50,233,151,37,138,242,178,0,101,7,202,58,226,141,135,36,47,90,231,
105,173,23,173,58,36,121,22,87,107,89,140,173,3,121,234,116,140,107,1,179,185,240,98,190,152,188,76,26,212,100,29,67,230,217,68,140,67,230,105,181,183,184,90,203,102,49,164,254,157,53,187,111,172,191,
167,128,216,232,166,159,110,162,70,129,255,115,15,98,99,28,61,101,140,197,0,222,23,51,243,97,65,99,60,207,13,136,101,8,24,43,56,148,230,191,227,29,239,40,172,64,248,99,224,55,19,162,83,16,187,147,17,222,
204,172,51,253,121,118,80,77,106,118,226,137,39,22,163,132,61,28,171,68,228,199,127,252,199,71,34,129,248,251,197,128,216,152,85,183,199,236,89,66,99,254,245,191,254,215,197,89,126,244,163,31,45,220,19,
255,148,154,9,113,176,135,204,132,44,230,126,207,178,20,131,189,56,219,102,1,230,34,54,6,98,195,107,63,34,137,144,168,90,241,79,58,233,164,194,62,133,53,186,11,19,173,147,215,26,196,10,87,44,224,137,19,
35,170,172,141,187,215,50,22,207,124,32,54,252,204,131,222,80,144,167,53,48,170,26,194,206,10,98,19,253,128,101,175,122,213,171,74,0,41,186,13,23,242,96,99,216,108,32,54,126,249,94,131,247,162,139,46,
42,221,52,137,123,206,11,98,19,194,38,48,211,173,49,54,31,17,42,197,179,54,136,213,255,198,133,86,154,68,134,55,2,98,179,75,255,217,63,251,103,185,78,222,190,38,136,53,141,195,107,23,196,104,252,212,168,
122,81,32,86,87,10,248,172,147,34,144,53,65,172,86,18,134,92,124,241,197,101,28,37,209,91,52,136,245,30,130,190,102,86,238,92,21,196,10,69,206,58,235,172,193,199,63,254,241,78,65,172,168,53,162,134,181,
65,44,171,37,159,178,22,142,176,75,16,203,193,102,2,209,236,116,77,6,177,127,253,175,255,245,194,60,63,252,225,15,15,13,234,18,196,10,153,53,68,99,212,56,136,149,97,188,255,253,239,47,155,67,246,101,250,
0,177,60,126,181,97,201,119,141,130,216,127,251,111,255,109,97,158,50,227,116,140,93,131,88,224,140,63,108,6,187,175,43,32,54,82,230,135,109,155,125,250,211,159,30,217,50,235,3,196,138,237,27,215,192,
168,21,16,27,179,96,159,37,197,114,210,55,136,253,119,255,238,223,149,240,167,105,45,113,89,193,139,34,198,146,50,89,104,25,149,171,125,31,32,22,64,147,75,86,93,200,158,3,94,33,114,148,56,10,65,12,242,
190,65,108,34,235,176,101,95,182,84,129,251,127,244,143,254,209,178,138,255,242,47,255,114,175,32,86,2,91,113,79,161,116,129,251,5,196,106,190,164,120,186,175,15,16,203,32,59,29,85,158,56,14,98,117,161,
205,34,139,112,31,32,214,32,175,67,230,137,32,214,2,137,160,112,15,185,49,109,31,248,232,99,62,24,196,238,35,1,203,222,50,100,158,27,5,177,90,233,111,255,237,191,93,15,240,201,32,86,82,32,108,229,26,172,
129,102,162,232,96,203,150,123,6,15,5,34,60,239,75,231,47,12,196,254,243,127,254,207,235,40,129,113,211,65,172,136,208,30,178,208,87,76,190,123,247,238,33,205,187,246,218,235,11,21,94,4,136,53,126,43,
55,48,88,13,196,238,180,121,196,95,125,225,11,95,24,225,157,247,220,115,79,172,141,199,21,163,54,10,98,129,180,48,98,118,16,27,20,101,201,174,248,229,151,95,62,52,234,250,235,175,47,139,245,107,95,251,
186,210,74,27,5,177,134,72,213,74,207,174,9,98,229,121,252,134,120,234,138,43,174,40,113,186,25,153,91,247,27,5,177,194,224,138,121,218,44,154,13,196,198,2,253,136,228,19,239,180,96,194,133,18,137,141,
130,88,100,184,53,227,238,157,11,196,202,203,0,49,242,36,196,78,147,243,248,90,108,86,16,155,48,86,235,112,146,130,72,128,173,233,186,217,242,190,54,136,141,63,150,201,22,152,154,198,9,97,147,16,79,3,
177,4,54,118,59,225,105,45,46,144,251,205,191,249,55,215,158,251,145,120,221,217,51,228,73,32,86,218,197,56,99,33,49,182,93,114,188,32,201,176,40,18,214,134,183,181,48,178,103,249,8,47,93,30,219,252,116,
52,49,134,110,95,56,136,101,156,86,243,240,105,198,156,84,220,114,145,134,164,49,201,64,155,238,218,217,57,136,229,253,49,1,254,198,118,43,3,180,70,10,31,26,67,158,27,16,27,16,246,225,48,74,28,100,0,139,
28,139,164,242,121,1,98,155,8,118,63,136,21,12,78,85,196,198,207,126,216,116,223,15,99,76,109,139,217,117,75,116,233,89,241,189,254,21,177,12,49,221,197,97,116,124,240,14,186,199,185,54,201,64,119,138,
216,112,124,167,69,114,177,147,19,181,247,18,219,110,37,137,180,39,195,195,211,92,137,44,0,89,10,53,144,141,99,181,10,84,25,240,226,20,177,177,14,110,141,236,121,73,136,252,230,55,191,185,208,24,89,52,
8,139,12,99,88,52,50,90,168,206,132,44,220,150,153,159,254,233,159,46,155,75,141,171,216,56,136,181,204,88,239,40,25,129,88,225,139,13,106,1,32,35,164,247,140,77,137,110,13,98,197,248,12,147,154,91,251,
180,46,223,214,192,140,245,129,88,254,134,48,230,43,95,249,74,233,158,73,220,115,26,25,110,43,98,211,48,222,63,246,117,114,65,230,207,172,127,179,131,88,131,88,43,136,167,16,189,141,40,98,83,13,203,56,
177,185,101,169,97,235,90,108,54,16,171,255,13,100,42,70,241,122,219,160,245,130,216,132,176,162,12,147,228,79,253,169,63,69,10,176,117,38,16,235,83,216,85,176,203,78,247,146,220,115,81,32,86,116,161,
213,4,124,205,186,121,218,170,138,88,43,61,232,64,252,110,86,117,5,98,181,218,203,95,254,242,217,64,172,110,51,219,108,125,28,113,196,17,101,134,117,161,136,213,90,54,198,141,221,102,139,119,178,34,150,
230,146,3,164,27,78,131,186,82,196,2,177,142,29,84,89,205,100,16,75,101,198,5,164,248,148,65,93,130,88,209,170,101,42,99,175,17,69,108,4,243,103,153,170,36,36,150,139,68,213,156,98,215,138,216,58,221,
26,1,177,225,208,110,33,226,146,166,215,71,5,186,6,177,92,3,46,86,117,225,10,136,141,38,220,38,17,32,211,173,183,204,186,6,177,146,142,138,226,141,130,88,241,144,156,140,206,179,94,203,250,80,196,90,122,
170,150,18,151,173,128,88,241,208,113,199,29,87,102,93,159,32,22,191,111,54,193,211,184,101,16,171,165,196,61,214,40,192,172,79,16,107,6,194,3,25,185,102,75,21,16,107,106,242,176,188,56,163,236,29,247,
161,136,21,115,85,210,202,109,97,203,40,136,253,91,127,235,111,13,62,251,217,207,22,58,220,23,136,5,79,56,237,166,165,236,196,23,61,250,143,199,83,148,251,226,39,145,101,173,136,125,215,187,143,9,121,
219,7,138,40,112,209,138,216,140,76,115,160,71,216,125,214,152,34,150,197,54,178,205,64,32,86,87,254,250,87,47,42,7,47,46,187,236,242,56,159,112,212,66,65,44,246,80,157,8,153,12,98,165,224,150,26,131,
29,128,189,245,214,91,135,52,111,251,246,135,66,4,248,193,133,129,88,179,174,165,240,152,12,98,113,40,11,229,105,167,157,54,120,226,137,81,13,240,37,151,92,86,176,98,158,238,216,136,34,150,65,45,221,240,
116,16,43,141,250,251,127,255,239,15,101,185,196,163,247,221,119,95,137,28,44,57,14,228,144,124,111,20,196,114,152,45,16,50,93,17,43,191,195,59,5,96,36,0,118,220,229,113,198,23,56,235,132,208,70,65,44,
189,12,198,85,121,241,181,65,108,116,225,86,216,26,129,131,150,133,173,132,205,139,2,177,181,26,150,97,51,131,88,249,30,26,135,63,49,78,238,39,227,37,13,217,136,34,214,135,172,207,95,121,159,185,64,44,
47,239,147,200,211,60,6,166,22,195,9,230,5,177,169,25,134,30,171,110,155,45,239,155,4,98,17,58,138,104,45,38,37,50,38,116,229,52,137,46,3,180,236,191,249,55,255,166,192,88,143,15,68,144,90,69,153,243,
101,200,211,64,172,23,196,57,113,1,122,2,174,67,144,102,204,217,34,19,143,73,50,131,61,148,223,225,243,146,129,34,195,105,80,124,221,186,112,16,203,151,137,22,65,88,131,86,43,248,94,158,57,246,230,140,
145,207,37,166,110,166,255,206,248,189,211,58,85,196,38,132,245,198,249,212,39,59,50,25,136,175,253,41,98,35,48,59,43,90,132,216,89,184,49,36,121,205,4,217,15,98,133,58,19,75,19,196,247,133,177,207,159,
210,4,147,186,79,140,22,131,253,168,248,218,109,105,130,24,200,167,27,184,49,155,28,156,31,78,243,220,153,74,95,102,81,7,101,195,175,45,197,36,32,90,93,124,105,130,152,238,95,136,20,236,113,96,21,87,226,
16,57,71,203,134,101,199,242,35,6,19,20,218,20,135,34,65,17,14,214,2,111,118,134,97,139,3,177,241,98,183,248,196,194,23,240,85,90,79,244,5,198,250,183,175,34,9,70,97,89,96,45,236,152,149,0,64,12,188,179,
57,39,179,24,16,171,75,24,115,222,121,231,149,45,92,161,178,71,28,191,90,105,130,228,158,90,75,87,114,178,77,139,105,181,249,150,153,248,131,114,68,37,158,251,113,35,139,48,94,165,37,218,152,113,86,16,
171,123,1,20,93,41,74,104,8,241,253,205,251,204,14,98,45,23,198,144,96,207,24,217,8,136,245,193,180,152,48,216,193,176,138,180,204,14,98,205,44,210,74,136,81,188,94,115,79,221,182,17,16,107,241,182,159,
108,155,164,57,74,183,118,105,130,48,136,138,176,72,150,212,217,72,89,119,98,198,141,150,38,16,106,11,24,37,161,160,74,24,182,20,189,242,133,85,65,172,248,217,212,254,234,87,191,90,118,24,114,80,47,90,
17,43,236,225,82,132,222,177,142,62,49,85,17,107,55,10,113,57,243,204,51,59,47,77,144,39,147,68,27,134,11,199,60,86,154,32,100,216,59,25,244,245,175,127,189,248,161,62,64,172,193,47,112,108,194,156,81,
16,27,142,237,40,65,155,153,38,109,79,196,216,53,136,69,242,82,63,76,109,61,86,154,0,253,231,32,235,226,13,93,131,88,3,94,248,156,201,196,152,34,150,79,226,185,83,162,203,49,246,1,98,105,223,171,12,103,
180,52,1,226,98,44,241,73,233,169,187,6,177,102,32,29,96,43,237,90,46,77,32,30,146,62,89,92,109,50,230,166,98,31,32,54,247,103,154,165,103,20,196,178,214,78,38,223,212,39,136,21,123,85,45,229,223,43,32,
150,143,18,35,233,194,62,65,172,248,172,50,106,216,82,5,196,250,129,77,104,74,235,62,75,19,200,184,171,144,102,180,52,129,31,232,95,133,65,106,16,251,145,56,103,252,222,95,254,149,78,74,19,96,158,22,232,
234,88,212,184,34,150,81,159,255,252,231,75,212,104,33,182,222,93,127,253,13,131,219,111,191,51,78,101,127,174,19,16,107,39,62,99,252,49,69,108,124,99,137,175,178,105,100,231,225,91,223,250,214,136,164,
242,43,231,255,250,176,170,205,34,74,19,164,52,174,218,29,21,54,91,89,94,204,37,12,75,19,224,232,95,254,242,151,163,230,193,158,17,131,118,238,220,53,248,248,9,159,30,66,216,69,148,38,16,240,145,199,85,
152,113,231,88,105,2,171,180,112,85,75,109,221,186,181,24,245,248,227,143,15,190,246,181,175,21,165,198,193,111,60,124,104,212,34,20,177,206,255,213,123,50,83,21,177,225,213,159,16,223,240,85,140,209,
141,226,41,188,83,169,166,69,128,88,252,202,177,242,214,38,209,116,16,27,129,214,23,40,98,157,57,192,155,44,3,226,235,44,102,181,8,16,43,229,130,46,43,223,180,54,136,149,231,201,56,48,40,240,29,181,203,
71,86,178,17,69,44,159,164,39,106,15,62,23,136,245,135,66,99,198,33,119,164,224,27,85,196,154,72,173,37,101,110,16,123,63,8,6,194,138,121,56,57,145,162,48,118,61,32,214,7,107,141,163,217,242,190,105,138,
88,41,55,195,48,77,160,159,113,211,64,172,44,197,227,67,24,212,214,83,137,104,107,28,205,151,33,79,3,177,210,46,161,43,213,5,199,7,192,122,83,41,185,233,205,9,202,21,197,101,198,141,71,82,224,169,206,
241,45,190,52,129,230,247,102,90,205,22,171,53,75,43,130,174,9,96,253,142,239,121,252,219,99,93,141,120,233,137,248,253,47,116,2,98,131,115,162,188,67,36,205,160,44,64,148,222,57,43,83,54,5,32,129,216,
211,59,47,77,16,159,248,168,186,52,129,200,181,153,85,190,238,87,196,102,181,156,31,109,16,27,93,217,189,34,54,6,248,249,241,236,142,153,181,132,49,153,85,246,96,204,60,179,204,246,135,177,197,39,193,
60,177,71,179,20,227,239,209,152,8,231,198,247,55,199,179,216,26,177,97,192,46,14,49,25,167,140,135,222,211,122,8,37,250,170,52,20,132,228,103,200,159,116,77,106,142,121,134,65,139,5,177,24,146,136,65,
52,74,1,43,97,37,62,245,213,99,135,94,248,204,16,105,63,56,226,251,178,34,134,162,119,106,44,52,168,123,99,165,9,226,211,149,131,23,114,51,242,74,66,138,245,112,79,71,83,22,82,154,32,140,41,11,178,180,
218,49,96,9,197,122,74,19,136,46,196,99,142,141,19,33,114,180,235,6,177,12,130,153,113,79,152,177,62,231,190,158,26,177,226,48,235,166,50,25,213,62,242,124,32,214,250,134,10,83,50,214,220,115,35,32,86,
100,97,43,215,226,93,213,51,91,27,196,198,212,94,50,134,20,112,208,66,6,239,34,21,177,2,69,17,40,8,107,91,87,106,183,102,105,2,169,187,140,230,228,147,79,30,25,67,139,4,177,130,68,41,150,61,30,145,109,
24,181,107,34,136,141,41,123,174,96,204,17,58,240,172,30,67,93,40,98,117,101,98,235,38,204,57,127,12,196,70,0,247,40,17,51,131,248,153,154,123,118,81,35,54,75,144,1,253,220,78,76,172,221,99,53,98,35,120,
91,210,109,114,190,190,64,172,214,202,242,4,97,216,56,136,21,178,90,22,128,253,190,20,177,45,163,198,75,19,24,112,150,11,21,223,250,42,77,128,14,255,212,79,253,212,116,16,171,95,141,37,169,122,159,32,
118,85,69,172,25,160,57,225,197,62,65,44,127,53,9,47,2,159,5,92,73,207,137,227,251,4,177,118,58,166,130,88,11,37,163,68,2,125,130,216,86,247,141,130,88,139,164,1,174,251,18,196,190,227,29,239,142,82,153,
159,235,180,70,108,75,121,54,10,98,133,178,22,94,91,248,118,204,129,178,115,207,253,98,20,119,188,103,240,181,139,47,27,188,243,93,199,46,188,70,172,76,187,46,239,27,221,56,10,98,109,113,113,156,34,131,
75,46,185,100,160,6,103,150,85,125,224,129,7,7,31,61,254,19,157,128,216,90,250,61,6,98,35,1,88,178,232,222,120,227,141,99,53,94,239,186,123,203,224,173,71,190,107,161,53,98,19,196,226,16,205,64,95,26,
171,17,27,200,103,55,171,235,51,237,234,121,18,56,191,251,61,239,27,49,104,17,32,214,98,44,82,0,69,26,163,30,29,3,177,49,208,207,39,104,230,171,28,8,51,224,37,1,52,234,176,34,222,153,69,107,23,1,98,237,
31,55,89,78,49,42,92,210,185,19,107,196,70,159,238,66,83,24,39,230,193,162,242,86,129,69,129,88,45,36,200,171,132,241,37,253,90,181,70,108,204,194,82,233,13,140,173,33,236,162,20,177,173,227,116,12,154,
173,70,172,230,4,187,200,32,245,187,22,91,132,34,22,25,174,170,221,148,110,155,11,196,74,131,68,14,18,8,78,149,81,27,1,177,198,80,219,32,121,229,188,138,216,34,192,1,200,44,154,242,54,93,152,17,227,106,
220,51,107,85,231,180,87,21,192,206,107,107,141,219,24,136,53,75,12,76,107,226,122,200,240,132,69,119,227,32,54,224,198,18,208,143,14,219,201,68,124,197,214,254,95,246,35,243,245,198,82,115,143,239,249,
89,22,175,173,90,104,241,32,22,132,21,54,139,189,18,255,228,87,201,165,239,251,106,29,245,239,86,85,128,93,157,148,38,136,55,58,55,150,163,71,185,13,111,152,0,54,203,18,24,131,140,244,255,141,68,215,239,
237,142,191,57,191,115,16,219,174,17,187,31,196,78,188,95,232,249,172,136,141,177,114,106,19,148,137,22,199,56,122,124,175,191,26,177,116,86,225,28,151,168,136,184,1,206,21,148,77,180,147,48,150,27,64,
84,226,107,169,17,27,31,226,160,133,131,88,43,185,133,90,56,99,243,27,144,245,239,60,229,102,241,6,99,69,0,214,75,15,95,6,154,136,195,195,149,236,137,215,88,76,141,216,120,33,224,244,89,173,33,134,39,
141,19,54,103,106,159,84,79,34,155,0,86,166,157,128,150,96,213,130,110,103,29,234,110,94,111,254,2,219,241,135,63,30,15,1,106,1,177,226,119,71,55,115,231,61,53,86,243,148,100,181,5,167,165,27,223,54,223,
50,19,70,12,21,177,28,36,209,21,68,164,188,125,125,172,124,86,131,242,110,44,11,184,229,167,170,52,49,219,130,28,6,149,242,246,217,66,116,10,170,0,16,160,214,152,113,61,32,86,88,45,210,64,155,77,134,170,
152,195,108,165,9,176,72,177,147,90,9,167,158,122,234,80,55,156,227,104,189,138,88,49,153,51,125,194,152,185,64,108,16,189,37,3,149,104,80,183,73,76,23,9,98,117,163,25,105,88,52,18,165,103,167,130,216,
240,41,7,197,39,216,35,147,113,127,3,173,121,141,25,23,13,98,49,117,81,109,115,200,103,250,101,93,252,9,84,109,211,167,93,69,119,209,151,117,217,24,39,47,49,35,155,42,244,147,75,19,48,74,43,41,2,210,7,
136,181,43,102,70,50,44,102,186,82,80,163,151,117,133,255,88,18,45,114,142,41,98,238,90,17,155,183,97,228,161,251,137,151,117,9,117,109,123,244,9,98,185,137,148,191,69,112,120,106,83,54,127,165,74,165,
181,138,81,121,118,161,107,69,172,150,146,249,84,194,174,81,69,172,152,218,34,202,168,62,65,108,171,228,197,74,141,216,140,135,164,80,212,139,125,130,216,220,113,104,50,158,81,69,44,188,104,15,198,178,
210,39,136,109,21,243,27,85,196,154,1,138,64,138,147,18,196,30,247,225,227,7,39,127,254,140,78,47,235,82,167,161,202,11,71,65,172,60,95,13,107,145,128,19,67,10,251,95,254,173,43,6,247,220,187,53,118,71,
207,239,236,178,174,214,181,65,163,32,86,102,43,162,116,32,250,193,7,31,28,65,140,55,221,116,235,224,29,239,60,102,225,32,22,171,170,43,127,55,49,255,114,105,2,73,0,163,148,80,105,107,60,193,216,175,125,
237,235,81,208,225,61,157,128,216,90,185,63,6,98,37,1,214,60,156,243,233,167,159,46,21,0,156,121,248,248,199,227,126,134,119,70,181,137,5,95,214,133,200,16,133,85,37,84,198,65,108,196,55,59,245,47,15,
139,161,139,173,13,120,145,35,198,153,119,171,45,234,178,46,93,71,175,32,118,107,6,250,244,210,4,12,19,243,112,11,50,148,174,64,172,86,170,107,14,175,122,89,151,141,35,44,10,135,202,155,122,23,169,136,
77,84,109,241,175,180,122,123,34,82,56,104,234,101,93,153,78,25,128,154,151,184,102,145,138,88,186,4,227,55,137,77,188,223,236,32,150,223,2,202,68,159,245,189,198,235,85,196,102,119,181,106,37,204,7,98,
49,38,193,151,120,90,50,153,215,187,174,183,52,1,170,39,197,111,113,207,249,65,172,23,0,250,53,57,201,81,109,208,60,32,22,59,175,198,79,26,54,91,222,55,73,17,43,93,55,35,137,179,242,130,46,6,229,173,97,
89,9,160,190,120,50,175,51,19,238,10,177,171,203,222,210,160,249,50,228,73,138,216,216,76,90,50,22,44,158,12,148,183,105,65,105,18,149,15,65,141,217,148,87,159,129,28,12,179,67,213,186,56,240,217,133,
94,214,21,33,235,30,97,43,13,94,251,225,70,60,9,98,57,68,19,165,42,231,91,246,95,22,14,98,227,13,14,138,193,95,46,235,82,21,55,7,108,134,178,41,211,149,146,51,90,11,101,157,132,231,228,178,174,140,92,
155,175,69,17,27,217,239,169,207,249,101,93,97,8,41,193,254,26,177,226,174,145,26,177,17,211,159,23,238,226,86,53,175,39,117,95,124,175,63,16,107,160,7,140,93,146,175,89,23,45,176,2,68,180,15,16,171,202,
164,152,113,221,213,136,141,23,31,222,223,103,233,145,241,168,60,40,238,130,138,82,106,41,185,204,27,157,132,185,22,117,198,54,196,24,89,57,35,176,207,171,192,185,213,46,133,155,122,1,92,131,25,203,77,
135,62,177,41,110,185,96,140,155,123,220,218,154,233,125,150,34,16,131,229,77,193,160,45,80,162,53,125,207,42,16,33,145,27,19,183,52,180,112,190,155,14,197,237,241,20,16,11,3,218,186,181,156,128,31,206,
255,101,125,216,181,184,167,133,60,19,91,225,143,160,174,97,158,243,45,51,77,11,149,210,4,28,164,13,72,248,89,58,15,156,233,170,186,133,166,21,173,205,203,41,18,196,50,74,124,38,28,106,198,221,108,11,
178,190,110,154,246,94,1,189,24,93,242,32,73,197,206,235,83,220,243,94,214,149,117,98,137,155,171,243,51,107,43,98,13,194,88,183,182,0,29,68,131,184,103,10,114,106,136,182,94,16,171,181,204,90,155,153,
213,44,93,93,17,27,235,219,25,86,122,122,225,188,250,110,209,151,117,33,196,232,157,197,189,9,137,167,131,216,232,182,205,86,114,154,188,44,220,80,27,180,72,16,43,172,177,45,2,170,52,139,246,116,16,107,
117,39,85,202,106,149,53,170,94,36,136,21,193,26,34,18,136,12,111,68,32,99,138,88,158,90,0,199,40,128,163,107,16,203,209,10,6,5,140,205,248,154,92,35,214,22,5,200,81,59,198,46,107,196,50,140,239,18,114,
139,209,38,130,88,70,185,27,43,157,99,215,165,9,120,121,227,75,88,109,108,197,114,116,222,24,136,21,83,243,75,125,130,88,43,69,86,23,12,131,110,141,97,84,80,80,57,70,110,122,26,128,137,172,181,82,31,165,
9,170,35,44,162,10,97,208,74,105,2,235,155,253,24,83,191,79,16,43,244,169,82,176,81,16,107,22,40,191,99,144,91,237,173,252,110,30,63,251,156,243,7,95,56,227,188,153,30,247,108,207,242,184,250,58,175,239,
224,172,173,177,213,35,172,94,46,226,224,144,169,107,242,196,72,188,57,175,126,227,141,55,15,238,188,243,238,208,123,158,95,174,65,119,63,187,139,227,221,104,239,106,116,133,71,15,63,34,56,86,0,53,198,
160,126,123,247,62,81,46,148,191,241,166,59,203,141,247,46,152,119,159,187,235,211,221,86,238,114,112,119,113,103,242,58,77,59,252,242,24,249,15,26,79,140,122,244,209,71,71,120,231,51,207,60,51,248,246,
149,215,116,2,98,205,188,22,243,92,1,177,145,1,223,10,64,92,121,229,149,99,26,207,125,251,246,13,78,255,194,57,157,148,38,48,158,28,135,170,186,110,69,17,27,240,225,60,117,17,45,47,119,223,125,247,224,
187,223,253,110,193,215,10,97,253,234,71,79,136,11,228,127,57,238,105,127,251,194,65,44,29,86,61,158,198,64,172,67,96,78,198,218,117,32,130,48,208,5,104,7,191,241,208,82,241,180,11,16,219,26,228,147,107,
196,98,0,182,37,10,124,141,66,14,240,79,151,32,86,192,87,181,212,244,203,186,200,102,121,118,225,44,163,186,4,177,72,77,26,181,42,136,21,170,90,3,245,183,88,188,11,69,44,212,168,204,69,85,77,119,117,69,
172,116,74,84,168,197,140,175,52,106,81,138,88,6,65,150,85,85,102,81,232,25,174,79,31,178,177,10,146,189,48,12,218,20,143,60,175,84,0,224,71,12,120,6,45,66,17,155,39,214,106,212,24,116,102,75,0,184,87,
213,247,185,143,36,163,117,54,195,48,82,73,235,160,214,218,8,136,21,101,194,140,173,67,247,198,211,252,32,86,202,237,88,93,118,225,122,64,44,53,172,15,215,42,75,192,160,217,242,190,73,32,214,216,82,169,
141,246,151,220,136,111,1,88,107,0,155,139,171,125,157,90,162,107,178,180,74,57,45,6,196,70,136,186,197,248,50,14,124,90,201,133,212,91,8,107,187,196,130,202,133,200,126,25,100,32,27,139,140,175,54,131,
210,152,197,128,216,112,253,175,66,75,164,95,66,86,160,67,6,226,43,227,60,214,47,70,155,226,90,21,49,110,81,225,110,64,108,4,98,155,19,196,114,27,233,244,236,76,104,65,70,48,140,129,213,101,55,69,40,248,
156,131,216,72,253,31,140,150,189,53,158,243,246,131,216,232,146,23,199,131,97,201,60,4,250,162,215,17,58,220,252,220,239,189,96,53,106,55,81,117,59,235,31,196,139,143,93,214,21,99,231,250,152,242,143,
198,120,123,186,25,99,195,202,18,241,255,253,130,88,74,87,56,135,51,117,114,18,199,178,192,114,182,169,85,111,140,236,7,196,154,97,142,84,226,13,206,42,139,86,177,77,134,73,98,57,82,75,10,247,208,42,59,
190,216,210,4,92,0,8,129,12,139,177,0,53,71,131,221,75,195,160,84,17,9,8,109,134,51,208,34,110,31,154,151,231,92,145,65,209,109,180,228,93,241,90,71,196,107,174,31,196,234,6,78,211,210,226,148,228,185,
231,158,91,206,149,38,111,152,84,146,53,57,39,227,82,55,67,69,43,112,196,165,194,184,93,241,186,179,223,9,233,19,52,127,112,191,150,17,29,56,227,142,198,48,10,138,94,15,136,205,146,172,84,139,13,33,158,
109,65,22,186,4,249,56,34,214,180,93,198,7,209,160,107,168,242,46,219,154,89,173,7,196,138,54,172,143,246,2,155,10,225,107,131,88,65,94,108,62,223,5,43,162,47,32,172,110,90,212,101,93,25,44,214,177,185,
160,114,205,203,186,84,234,186,250,234,171,75,93,79,227,96,209,32,54,121,122,117,161,238,218,32,86,252,228,2,93,64,191,43,16,107,131,220,126,180,9,212,8,155,167,131,88,51,77,96,166,8,72,45,64,237,162,
52,1,129,32,157,66,242,244,169,32,150,81,118,21,28,201,172,5,168,93,148,38,16,185,34,61,66,158,85,65,44,7,7,5,153,109,57,237,187,4,177,73,135,27,125,250,100,16,43,192,199,165,136,187,82,8,223,229,101,
93,150,36,116,88,55,54,199,164,78,30,1,177,152,167,228,0,109,225,6,250,168,17,11,89,91,41,52,70,115,231,195,245,77,56,180,12,98,45,158,32,172,11,187,250,84,196,106,173,60,139,28,173,228,24,203,10,136,
149,153,56,24,77,181,111,144,107,173,147,63,127,218,224,171,23,125,163,96,65,120,16,38,132,11,97,67,248,16,70,132,19,97,69,92,212,165,224,112,35,236,8,63,194,144,112,36,44,9,79,126,226,147,39,7,71,61,
37,126,126,204,80,43,3,51,86,149,42,197,101,43,151,117,25,67,92,129,242,59,206,54,56,36,253,253,239,223,85,152,231,249,23,92,220,25,136,21,69,100,233,232,38,6,91,6,177,177,64,62,173,90,210,53,215,92,51,
134,23,31,127,252,137,48,240,202,206,64,172,4,55,51,163,248,186,162,136,13,244,179,135,197,223,252,230,55,199,140,122,50,110,122,194,60,117,199,209,199,124,168,116,205,145,71,189,183,224,70,116,143,154,
81,151,161,195,217,149,229,160,254,210,32,52,163,203,164,248,142,59,183,14,110,184,241,142,193,181,223,185,101,112,229,85,223,141,11,154,62,80,226,44,139,115,115,157,108,26,182,2,98,99,228,95,111,43,194,
254,176,221,116,145,1,222,121,194,9,159,136,122,9,206,182,191,191,19,16,107,245,104,69,167,163,165,9,132,19,12,203,11,224,114,99,250,136,55,191,125,240,230,183,188,171,19,16,171,60,103,213,117,60,187,
11,41,100,73,37,93,58,84,21,45,48,2,208,178,24,219,190,239,26,196,182,100,223,147,65,44,70,224,168,147,197,88,201,139,46,65,44,16,210,34,50,211,65,172,233,233,60,11,108,221,165,34,22,165,169,193,254,170,
32,54,175,136,165,67,208,125,93,129,88,139,113,53,158,214,6,177,12,179,69,161,27,83,13,187,72,16,107,189,147,213,52,70,205,174,136,253,201,159,252,201,82,208,40,91,106,17,32,54,139,142,50,40,113,99,68,
7,119,5,207,218,52,19,136,53,182,236,241,114,13,105,208,70,74,19,232,46,91,254,213,161,67,45,181,43,140,59,34,72,225,136,116,105,76,170,84,231,125,180,156,242,52,49,117,93,27,118,158,74,0,194,94,113,255,
4,1,234,108,121,95,27,196,70,107,237,194,59,53,53,220,44,216,199,17,56,61,227,34,239,198,154,4,98,121,107,60,180,213,50,27,7,177,209,223,71,200,253,29,30,227,195,44,9,137,16,65,86,70,102,75,90,199,60,
166,58,241,23,42,211,186,14,175,200,225,22,170,136,141,23,68,77,202,39,213,21,90,192,118,9,35,133,179,6,175,141,76,19,164,222,81,200,210,172,254,126,225,138,216,73,32,54,203,19,100,43,50,54,79,108,55,
198,60,247,32,54,186,251,233,24,127,123,194,160,235,45,174,251,65,108,19,89,60,63,65,108,24,119,53,110,30,3,248,105,137,71,116,223,83,50,35,223,143,167,95,16,107,106,155,133,192,43,175,239,188,50,222,
96,25,201,19,217,205,76,237,7,196,122,51,142,148,195,4,98,113,7,28,203,226,157,5,143,44,230,140,171,142,161,148,189,153,120,22,91,154,128,49,28,34,47,14,194,74,92,49,135,68,70,41,218,177,86,90,196,109,
136,103,78,199,167,101,245,146,120,157,141,215,136,229,32,157,186,117,224,158,33,90,71,203,228,13,80,211,238,198,18,181,226,165,2,69,81,108,138,3,155,242,191,219,194,199,29,29,6,254,248,76,59,14,22,228,
112,128,71,199,198,243,35,198,9,37,163,26,177,74,213,81,12,201,156,235,203,193,103,81,196,138,197,68,24,73,141,149,161,11,4,244,72,188,23,66,60,219,101,93,177,174,109,131,165,17,97,154,42,134,180,79,113,
175,23,196,154,24,54,40,27,213,226,108,32,86,159,211,82,209,187,48,136,49,139,4,177,169,138,21,159,55,75,209,218,32,214,190,139,84,90,182,220,21,136,21,155,85,5,214,214,6,177,112,95,94,139,215,21,136,
53,1,90,87,163,175,14,98,101,202,144,80,93,111,163,11,16,43,238,170,174,116,1,251,39,43,98,249,34,129,26,109,94,215,32,22,197,19,123,85,105,214,100,69,44,224,174,76,138,61,152,62,64,44,68,32,32,108,6,
188,139,231,14,13,213,27,140,176,114,27,57,204,103,183,33,47,236,234,90,17,203,227,227,157,121,147,235,216,101,93,166,40,143,109,144,247,89,35,214,76,215,90,141,97,163,165,9,240,3,245,55,112,116,155,139,
18,209,147,78,250,252,224,154,107,111,42,176,11,244,2,191,64,48,48,12,20,203,42,150,96,25,230,9,158,129,104,121,7,55,184,134,109,129,109,160,219,7,63,244,177,144,83,158,16,227,245,136,114,135,31,192,111,
249,33,174,104,198,214,138,34,54,98,235,167,108,16,41,153,201,105,222,124,243,205,229,112,244,214,173,63,24,108,185,231,7,33,114,190,170,83,16,155,53,61,99,108,173,128,88,150,90,104,39,149,205,220,187,
247,241,193,21,223,190,182,83,16,91,149,207,212,98,203,32,54,214,186,167,173,73,92,65,118,73,126,221,189,251,145,193,185,95,188,160,51,16,43,48,172,138,67,138,92,11,178,126,185,144,150,207,16,5,88,136,
169,247,117,231,167,62,253,217,193,103,62,123,106,167,53,98,133,69,85,110,56,90,154,64,198,11,66,136,8,200,41,5,107,7,31,124,200,224,151,143,254,96,103,32,182,53,200,117,221,248,101,93,90,11,33,166,178,
238,3,196,18,88,84,173,52,14,98,37,3,210,111,96,66,107,245,1,98,91,53,25,39,131,88,190,194,47,26,91,93,43,98,13,240,150,240,107,58,136,165,69,200,112,183,75,16,219,10,93,200,234,78,153,88,35,182,73,131,
202,65,103,225,138,140,164,43,16,91,121,241,146,126,173,90,35,214,216,178,95,34,20,150,145,116,5,98,43,137,229,108,32,86,240,85,118,27,66,120,154,70,45,10,196,10,238,234,43,23,231,170,17,171,121,101,186,
117,173,132,141,128,88,56,146,11,32,99,170,160,254,182,185,106,196,70,88,252,8,66,172,213,96,68,1,153,236,120,158,26,177,62,20,67,28,157,203,226,106,149,95,122,36,254,45,33,93,61,239,171,65,108,164,217,
71,71,110,182,77,52,202,48,169,183,127,11,202,124,106,25,73,134,31,109,16,203,24,31,130,123,193,73,197,74,140,201,175,97,200,124,103,70,39,149,38,112,120,43,203,174,114,176,154,223,102,143,221,249,220,
76,244,53,47,239,50,81,178,139,24,83,181,78,119,32,22,172,96,156,86,208,45,89,248,209,50,85,111,42,214,75,136,105,223,11,136,133,122,24,151,114,221,236,158,186,101,194,152,231,70,17,27,4,229,41,108,32,
190,138,28,133,180,87,239,7,177,209,10,63,58,151,117,129,250,113,120,98,95,140,175,93,49,166,174,13,227,251,7,177,6,179,129,110,144,115,13,148,68,248,131,180,191,218,181,234,7,196,114,7,66,103,27,224,
182,216,28,192,167,99,80,14,95,212,106,205,204,242,41,232,113,117,89,238,98,65,44,102,37,227,224,143,100,61,120,39,245,80,242,79,53,19,18,51,226,155,226,48,6,219,235,227,213,243,10,52,17,8,40,23,207,250,
107,196,6,13,126,72,188,142,2,123,100,56,50,29,25,15,48,11,170,173,5,98,45,69,214,63,142,213,114,213,108,40,109,109,120,231,236,32,214,31,196,50,177,221,155,171,13,107,231,93,229,46,106,216,186,18,192,
52,131,36,29,153,120,212,103,110,116,43,60,208,116,171,122,196,179,131,216,24,196,91,109,213,146,86,82,83,79,226,158,243,130,88,241,152,8,67,248,227,171,186,29,17,232,61,212,116,229,228,61,228,248,225,
240,216,19,10,34,155,161,163,106,115,207,141,92,214,149,1,163,113,169,197,154,29,137,181,65,172,193,232,240,22,193,160,241,211,133,34,86,171,85,186,169,181,47,235,146,246,56,59,245,213,175,126,117,228,
110,172,69,150,38,0,98,197,100,149,84,119,117,16,43,78,82,188,86,181,202,174,74,19,24,87,40,94,125,185,238,84,16,107,165,55,133,41,172,129,179,228,158,93,40,98,107,16,219,132,58,147,65,44,81,151,189,24,
93,149,173,212,165,34,86,66,154,53,250,141,229,49,16,43,88,227,139,212,110,233,11,196,74,183,220,34,220,160,197,209,203,186,162,63,175,181,137,227,252,130,177,100,29,227,48,63,244,161,227,2,47,62,56,184,
255,129,93,131,135,119,63,49,216,247,204,10,231,172,225,26,14,138,121,226,155,120,231,107,94,123,112,41,139,120,240,27,223,92,100,115,111,121,235,187,139,166,207,207,161,165,215,191,254,13,101,185,194,
60,243,24,121,211,133,43,32,54,210,169,93,246,240,156,216,126,234,169,167,202,81,242,109,219,182,13,92,19,180,253,33,231,212,239,232,12,196,138,48,170,234,243,43,53,98,195,47,237,115,254,229,134,27,110,
24,151,84,62,249,195,248,254,173,157,129,216,214,249,246,149,210,4,44,37,9,161,136,189,243,206,59,75,145,209,71,30,121,164,148,208,124,248,225,71,226,182,167,111,116,10,98,91,215,80,45,131,216,120,126,
40,196,176,107,110,245,119,148,9,224,56,249,228,211,6,167,157,126,78,167,53,98,5,133,50,232,102,176,239,27,130,216,248,71,81,89,203,108,33,160,188,172,235,200,163,222,81,96,60,40,223,149,34,214,49,224,
234,76,178,67,62,69,101,237,56,220,213,22,71,51,65,43,49,170,47,16,171,149,114,111,38,236,16,219,47,215,136,21,228,155,146,98,107,65,93,22,179,18,41,116,169,136,21,252,213,135,93,199,20,177,60,170,224,
11,152,192,60,187,6,177,134,202,76,32,86,144,239,23,101,37,93,131,88,10,181,86,25,196,233,32,86,44,77,189,223,117,141,216,44,48,147,75,204,170,32,86,214,98,6,26,87,93,129,88,94,188,165,102,92,29,196,74,
167,172,73,146,132,186,44,193,34,21,177,14,241,100,11,25,203,16,209,154,165,9,208,58,241,142,150,90,52,136,21,113,214,101,89,103,6,177,17,78,60,228,180,17,230,137,169,91,159,54,82,154,192,76,67,249,24,
212,170,187,177,117,46,16,27,191,188,93,102,203,143,248,100,144,161,93,46,153,239,60,138,88,123,61,214,85,78,210,36,170,102,220,108,121,223,132,210,4,62,73,65,134,140,147,225,202,215,82,13,59,13,196,230,
85,8,114,71,226,139,74,63,149,99,73,190,55,251,153,209,73,32,214,96,228,195,180,156,46,213,181,66,89,221,154,117,19,44,172,214,49,75,149,104,210,135,17,234,122,252,109,115,50,109,241,151,117,153,49,214,
72,111,170,229,44,75,118,54,25,72,238,38,101,178,92,229,37,93,213,161,194,178,255,210,57,136,229,103,178,92,166,127,91,84,159,23,32,22,177,115,166,56,152,192,190,38,198,30,158,109,223,15,98,99,108,252,
232,128,216,102,217,248,97,140,175,237,49,214,182,196,32,63,61,190,119,104,60,7,198,243,210,120,94,52,115,25,132,181,126,49,94,12,34,122,89,60,155,227,41,197,144,185,136,92,187,0,17,179,209,76,68,83,112,
79,254,203,182,111,125,89,87,243,247,94,231,133,107,190,231,170,181,90,151,235,123,110,138,167,212,136,181,9,196,187,219,213,180,31,136,105,2,175,226,47,153,53,217,37,74,236,144,189,101,69,161,173,74,
21,187,56,16,27,254,232,238,8,91,151,108,221,138,219,65,88,114,38,15,14,42,3,146,77,75,245,243,250,14,1,162,86,67,86,44,49,90,50,133,91,241,225,54,6,98,227,5,238,245,102,142,215,49,192,155,123,16,25,0,
36,177,227,52,1,170,208,199,94,160,210,60,233,92,155,110,159,111,191,47,254,40,75,19,60,96,219,140,108,9,144,157,132,25,103,1,177,90,152,97,2,59,72,209,18,213,108,106,75,209,103,7,177,90,200,226,171,37,
166,113,207,121,64,172,184,204,78,170,238,180,62,146,233,54,251,130,179,43,98,53,177,133,149,76,87,183,45,74,17,107,156,101,232,163,245,26,121,248,221,107,150,38,48,221,53,177,164,52,203,178,46,242,178,
174,172,153,96,123,87,104,211,44,218,39,133,97,163,149,186,56,182,198,143,148,34,69,198,129,253,150,186,2,234,162,65,172,238,172,116,158,171,131,88,254,7,167,178,229,145,187,11,93,40,98,25,149,183,219,
55,59,241,147,21,177,34,68,78,80,241,227,250,142,190,46,64,172,93,135,100,233,83,65,172,154,43,82,117,186,60,201,104,31,138,88,204,51,151,43,99,121,4,196,198,82,112,186,92,140,65,92,128,98,50,198,16,70,
117,247,150,31,12,246,237,139,195,206,207,78,230,157,244,158,206,185,99,158,112,17,198,137,119,226,158,175,251,133,67,202,121,247,55,30,18,34,251,248,254,242,209,225,183,13,19,15,33,116,157,186,143,40,
98,227,120,210,22,227,231,59,223,249,206,8,94,124,232,161,135,6,123,30,123,124,176,109,251,238,78,64,44,191,53,237,186,160,151,135,181,219,249,141,73,199,200,213,243,188,231,222,7,58,1,177,222,51,207,
142,54,227,106,244,178,46,63,116,192,2,182,38,106,86,66,243,241,199,31,31,60,182,119,239,224,218,107,111,232,12,196,74,60,170,113,53,90,35,22,216,200,226,50,22,97,236,243,140,51,207,25,124,233,203,95,
45,135,233,103,169,19,59,75,125,88,191,83,215,136,149,35,214,98,249,48,112,5,196,202,231,76,83,66,230,44,58,116,236,251,62,88,138,46,40,190,208,85,141,88,131,189,58,185,61,122,89,151,89,96,21,55,227,24,
37,38,234,3,196,74,104,101,221,77,23,138,106,87,64,172,111,26,87,54,30,83,124,218,135,34,86,247,9,157,189,127,12,246,45,241,117,5,196,250,166,56,91,89,39,113,82,31,138,88,173,84,131,253,136,233,79,31,
41,77,192,163,74,2,114,35,187,107,16,203,32,203,76,173,98,116,156,51,152,107,57,227,32,143,147,6,109,199,7,44,148,150,153,60,55,213,85,141,88,221,86,85,62,213,125,147,65,172,93,118,145,97,106,60,187,82,
196,102,21,249,202,63,173,174,136,85,99,197,126,95,22,173,237,66,17,75,169,95,95,230,44,125,91,85,17,11,37,139,10,243,84,227,162,21,177,104,96,117,233,91,57,197,189,38,136,141,240,225,110,125,45,87,243,
137,100,187,98,235,69,41,98,91,219,104,243,93,214,37,224,227,229,93,195,145,194,83,48,118,189,151,117,37,136,109,93,216,53,119,141,216,7,184,8,96,140,113,198,154,241,85,151,40,176,15,60,141,123,230,221,
88,73,134,77,160,86,43,205,150,247,77,170,17,107,134,152,149,188,174,251,99,76,101,93,57,205,32,45,146,151,127,211,91,225,9,178,22,113,83,197,21,230,203,144,167,129,88,232,80,211,59,136,42,224,7,96,181,
146,152,200,162,154,26,41,89,181,223,201,234,167,32,44,99,124,21,51,133,215,190,59,62,228,252,162,174,202,176,151,68,202,181,41,94,108,120,51,143,55,51,123,82,9,43,105,133,165,147,6,167,17,104,112,99,
196,240,170,141,78,65,172,162,14,198,25,52,164,21,24,144,68,184,218,229,44,211,157,167,238,173,70,108,180,200,233,202,255,134,17,194,13,113,16,35,246,131,216,97,85,147,102,1,183,146,63,127,75,19,132,113,
252,205,72,247,197,0,191,38,6,254,153,241,253,222,64,172,205,158,2,98,179,26,137,193,206,13,40,183,98,15,26,11,21,43,197,114,181,20,99,14,235,4,112,55,199,179,56,16,27,241,251,153,177,220,60,38,139,182,
197,102,77,244,166,54,167,249,42,168,136,0,85,40,141,129,230,105,74,78,150,235,104,156,230,226,64,108,124,186,91,2,166,46,129,103,222,212,227,223,174,40,246,230,194,28,68,152,108,0,157,201,74,132,4,23,
12,246,1,42,111,190,113,69,108,24,116,175,110,113,7,13,26,12,192,230,141,247,90,39,175,128,153,6,98,133,63,200,157,86,21,255,87,58,226,249,150,153,48,100,88,35,214,86,153,112,24,248,144,160,182,49,227,
172,32,86,62,105,205,84,46,172,42,68,51,91,173,160,48,104,120,89,151,37,34,47,197,105,235,61,179,139,220,86,238,78,154,19,78,252,116,84,14,60,113,32,121,149,43,74,205,50,241,16,193,10,171,69,23,182,110,
69,10,38,68,211,106,243,129,88,144,20,243,84,218,34,171,72,36,179,90,239,101,93,12,211,98,186,83,244,208,132,196,179,41,98,53,177,130,31,96,135,204,102,145,32,214,140,204,155,237,237,168,54,178,240,51,
87,5,177,248,182,68,20,22,50,163,210,160,69,131,88,19,128,75,105,140,122,44,220,198,230,104,140,178,211,85,150,143,248,207,214,215,41,12,18,242,18,159,154,234,93,42,98,117,165,232,52,7,254,68,69,108,12,
188,29,24,2,84,237,102,158,90,119,222,21,136,21,40,86,24,232,161,112,63,7,6,129,145,28,47,87,20,12,41,201,210,93,119,221,85,128,153,11,192,49,5,179,238,228,147,63,63,166,104,172,53,158,206,185,171,239,
137,95,169,223,233,60,59,125,39,222,137,123,58,235,238,58,108,250,79,87,196,28,246,166,195,135,39,223,68,173,114,128,38,236,25,5,177,145,32,156,105,191,4,243,124,236,177,199,134,70,104,161,199,226,24,
121,87,32,214,128,175,115,192,17,16,107,85,151,173,24,75,196,167,237,163,228,90,163,43,69,172,181,177,74,223,71,75,19,240,27,80,181,117,13,224,191,234,170,171,6,187,118,61,28,173,180,47,164,186,247,117,
2,98,141,41,7,26,43,163,70,65,172,20,138,11,160,155,178,150,249,122,230,89,231,14,46,253,250,21,33,62,189,188,51,16,91,85,22,204,112,122,165,70,172,68,192,146,96,197,207,109,87,75,199,44,0,54,127,103,
61,32,182,125,30,57,90,109,5,196,114,249,182,37,108,20,245,9,98,5,132,85,247,141,130,88,63,48,216,29,99,234,91,17,91,157,182,29,45,77,32,180,21,89,166,81,125,41,98,177,134,202,87,93,19,141,51,10,98,181,
148,133,56,235,37,116,173,136,181,204,48,42,207,153,198,170,114,230,24,136,197,179,237,245,25,83,125,128,88,152,177,98,232,75,147,64,236,67,142,199,169,208,108,76,117,13,98,147,123,86,151,160,238,8,3,
15,140,208,230,5,35,81,2,41,165,109,16,97,110,70,140,93,93,214,101,89,171,185,195,84,69,108,28,174,121,140,51,203,238,235,74,17,235,144,71,187,50,220,84,16,27,152,231,76,233,144,24,199,73,125,129,216,
162,65,172,173,180,214,221,70,107,131,88,121,30,172,104,245,150,22,217,189,212,149,139,0,177,8,31,134,90,57,203,249,64,44,15,207,48,64,204,212,101,216,70,74,19,192,142,45,205,48,227,230,6,177,229,146,
102,77,45,37,34,175,109,215,138,157,7,196,90,78,90,202,234,217,242,190,105,32,86,171,97,235,102,76,93,47,161,77,134,179,36,171,86,65,131,165,83,114,61,31,168,181,15,51,95,134,60,13,196,218,164,52,30,108,
84,242,248,98,33,222,152,191,81,200,22,251,228,82,76,20,221,238,145,242,103,157,133,28,71,11,175,17,107,2,248,196,30,199,74,140,55,139,105,194,87,211,92,171,130,181,89,154,213,247,60,225,147,30,139,36,
225,204,200,144,55,181,235,12,183,37,120,83,175,72,207,95,140,23,124,161,68,49,107,196,198,194,189,35,94,124,9,103,240,230,9,98,147,6,107,145,86,105,130,135,122,3,177,97,12,82,103,85,31,35,121,241,189,
253,165,9,228,137,63,122,138,216,166,59,251,171,17,27,51,237,139,49,200,119,199,27,63,155,247,249,25,216,169,236,241,53,175,205,147,29,133,59,176,51,241,88,76,130,179,226,247,54,199,179,56,16,27,47,118,
82,208,145,221,150,27,169,23,201,9,16,38,16,244,240,236,210,50,254,11,131,240,115,231,79,65,89,142,180,137,44,23,11,98,99,74,47,121,67,240,195,205,24,80,163,194,89,98,47,255,246,213,3,196,226,89,126,70,
169,38,85,179,245,43,226,0,50,212,135,177,224,199,51,255,46,86,252,145,221,4,194,80,203,65,33,111,96,172,244,43,65,236,172,119,99,105,81,203,143,176,87,247,54,62,108,190,101,38,140,24,130,88,25,142,116,
30,209,147,229,180,75,178,206,2,98,243,50,1,251,129,173,226,237,179,45,200,97,208,16,196,106,33,6,137,217,79,61,245,212,50,150,54,114,89,87,130,88,45,86,85,166,156,29,196,138,157,173,111,50,27,79,205,
61,55,82,154,128,97,196,205,118,45,170,10,243,107,131,216,88,92,151,240,110,244,5,26,90,52,136,21,98,51,204,224,79,33,189,157,215,85,65,44,131,148,184,128,133,76,237,46,64,172,152,76,216,35,228,17,105,
68,67,236,158,8,98,35,10,56,203,14,20,237,249,89,103,157,53,34,209,237,74,17,43,22,163,24,17,230,132,81,95,12,39,251,210,6,12,47,171,130,2,94,237,213,93,96,25,129,224,117,215,93,87,6,185,147,220,93,129,
88,78,55,79,177,69,247,237,30,3,177,177,189,191,228,250,169,219,111,191,125,4,47,238,216,177,163,51,16,203,168,170,108,230,120,105,2,145,36,128,63,249,238,118,138,216,165,201,198,109,64,17,155,5,66,50,
84,30,171,17,75,8,97,15,207,209,1,107,150,194,32,223,255,254,247,139,33,123,30,235,6,196,74,60,90,23,225,140,130,88,219,17,214,42,130,46,158,218,54,218,41,167,156,62,248,238,13,183,197,243,189,206,64,
108,214,75,104,90,107,20,196,242,178,64,153,150,202,77,69,91,102,23,92,120,105,167,151,117,181,140,26,85,196,178,52,235,231,247,9,98,43,62,149,233,252,10,136,213,82,114,57,91,31,125,130,88,9,110,197,22,
198,65,172,152,137,81,125,130,216,170,86,2,227,70,65,172,68,83,236,99,205,235,179,70,44,141,85,213,82,163,53,98,69,134,162,72,30,188,46,213,212,101,105,2,225,117,205,209,39,150,38,176,13,226,188,76,74,
116,187,190,172,203,162,140,234,53,45,53,14,98,99,221,217,45,236,21,204,49,170,107,16,203,32,173,84,213,11,218,59,6,98,35,126,254,162,29,37,134,241,83,93,131,88,225,113,189,173,22,239,127,214,196,26,177,
145,181,236,14,150,93,176,142,152,199,192,239,66,17,235,204,31,16,87,13,240,213,21,177,65,82,202,173,208,194,97,126,75,94,151,48,246,213,175,126,77,217,122,181,237,234,184,137,50,59,42,153,80,246,219,
193,114,77,163,178,229,202,248,44,239,166,62,59,120,242,201,39,203,107,228,238,130,158,104,105,61,103,171,17,235,19,128,94,0,152,22,91,20,136,69,134,69,154,85,11,205,7,98,37,15,12,195,60,229,109,152,250,
70,64,172,65,221,210,12,175,31,196,66,132,6,164,217,152,21,220,230,41,77,96,150,217,71,108,109,203,50,104,182,188,111,18,136,149,209,26,95,84,173,140,209,90,22,238,54,25,150,158,123,82,158,155,99,8,144,
109,149,74,41,45,212,100,224,179,21,239,155,4,98,221,130,40,29,2,98,189,49,224,47,19,177,213,42,72,179,210,123,99,27,78,254,109,246,218,166,19,127,139,104,171,77,161,197,151,38,48,38,178,22,2,222,233,
205,18,188,38,124,149,153,228,83,213,73,208,58,18,131,147,22,14,98,227,141,207,138,53,114,175,147,216,41,203,77,37,172,25,149,45,226,123,141,62,202,213,234,187,195,255,113,204,155,85,36,169,175,196,155,
118,48,123,77,58,156,127,72,177,35,47,139,41,125,160,115,120,130,252,48,196,170,190,31,196,78,108,197,104,153,231,39,136,141,174,59,60,140,91,181,251,154,159,247,163,136,141,117,241,145,136,171,151,196,
214,89,185,36,183,60,12,254,28,228,252,27,130,19,198,61,20,99,241,140,248,186,57,158,197,130,216,120,147,157,222,212,181,47,46,54,145,184,122,240,42,17,43,167,202,185,90,196,237,94,209,70,113,156,156,
40,159,21,147,99,111,188,6,185,238,38,112,110,213,146,8,83,167,229,50,102,4,76,129,83,55,7,20,18,227,160,171,8,85,178,42,105,101,20,34,12,25,161,51,9,99,209,225,252,127,81,130,29,46,226,250,230,245,54,
6,98,117,3,54,174,80,173,84,94,235,212,152,113,22,238,153,119,22,137,161,26,95,54,223,50,19,159,100,8,98,57,68,107,158,100,66,13,189,90,205,152,119,210,204,114,89,151,208,71,215,218,43,244,1,155,173,255,
217,22,100,125,221,116,153,189,221,114,108,18,132,5,209,148,180,168,239,53,154,167,52,65,30,52,51,214,172,133,213,241,185,217,65,172,79,227,147,17,49,51,104,209,32,86,24,83,237,39,175,14,98,99,58,159,
18,211,124,73,134,76,196,108,119,193,96,93,164,34,22,33,22,129,10,135,26,145,253,234,151,117,5,124,223,11,226,35,122,93,130,216,44,24,162,226,64,211,98,59,39,130,216,248,230,25,80,159,154,27,247,222,123,
111,81,49,42,81,192,47,17,195,63,252,240,195,99,36,79,125,79,247,26,185,207,157,4,206,253,236,238,43,82,191,211,121,118,73,6,157,39,189,231,107,94,251,75,145,29,45,95,159,199,167,1,102,178,26,71,169,86,
43,77,240,144,211,105,132,167,181,164,146,113,170,85,182,101,150,249,255,10,140,186,96,158,81,46,142,151,225,56,92,239,160,125,26,228,171,67,248,140,204,67,102,137,173,5,130,226,180,152,88,143,140,129,
216,248,230,18,163,236,84,25,79,79,60,177,124,115,253,90,207,70,20,177,66,106,91,111,41,213,13,175,127,248,200,101,93,124,135,50,39,206,183,219,38,83,205,134,210,154,206,243,153,103,186,3,177,150,165,
84,8,141,129,88,126,73,26,132,157,167,99,60,246,216,247,13,126,240,131,237,157,149,38,48,174,40,60,38,161,32,199,182,31,224,254,45,158,140,202,165,195,81,1,55,210,119,121,89,87,75,82,57,10,98,77,77,62,
68,53,221,62,65,108,235,78,172,81,16,139,228,217,32,202,179,237,184,103,31,138,216,9,242,165,21,16,107,105,49,184,21,40,234,19,196,10,105,170,49,53,10,98,45,190,102,30,104,214,39,136,173,238,134,28,7,
177,240,143,125,153,190,47,235,106,25,53,10,98,245,173,152,41,139,214,246,85,154,160,186,103,109,244,178,46,217,137,205,64,33,74,93,43,161,15,16,91,149,247,29,7,177,33,42,126,4,36,227,246,133,42,125,128,
88,239,149,85,37,100,61,19,21,177,242,127,205,201,95,49,170,171,210,4,41,155,115,153,96,158,100,11,222,112,198,68,16,43,141,130,255,88,239,83,216,56,234,2,196,122,109,121,99,21,125,238,93,243,178,46,191,
140,222,49,168,190,176,107,17,32,214,162,47,184,171,206,250,45,69,11,157,178,102,141,88,65,87,158,101,23,148,45,10,196,34,195,178,234,117,131,216,36,196,106,196,82,248,108,164,70,172,238,50,126,234,82,
246,141,97,243,43,98,53,177,115,126,20,23,88,122,93,43,97,18,247,20,123,231,149,84,117,105,2,251,47,173,107,241,54,6,98,141,47,75,144,55,132,173,231,37,195,194,93,251,211,117,151,45,4,196,210,117,154,
149,28,172,5,84,214,156,95,41,135,8,106,116,179,243,122,146,1,173,36,120,3,107,91,122,225,197,93,214,37,31,140,241,176,87,44,77,116,74,156,165,59,234,175,254,173,187,125,245,59,54,5,90,5,180,119,118,2,
98,227,77,113,166,135,26,25,219,176,38,108,194,88,209,107,178,42,70,55,211,254,145,222,20,177,179,128,216,88,29,14,143,238,60,144,167,142,177,181,184,34,163,109,126,229,164,162,52,40,140,162,219,115,240,
70,140,239,80,151,136,209,87,255,255,19,206,191,168,251,227,116,199,90,69,69,231,22,52,143,253,193,243,21,196,70,75,28,26,225,243,57,241,220,36,19,138,199,157,179,121,20,206,255,247,167,136,141,55,43,
53,98,99,157,90,74,191,101,233,176,78,230,62,94,214,0,170,42,157,42,135,177,248,210,4,241,162,165,90,137,44,218,166,145,147,146,216,21,6,138,119,122,36,27,22,112,169,63,1,132,16,72,0,71,13,107,123,46,
198,216,195,1,203,206,137,215,217,4,206,45,4,196,234,30,36,14,251,100,84,45,177,148,39,102,174,136,246,49,18,136,37,84,21,243,203,184,1,141,136,14,150,194,125,220,209,208,194,177,187,32,135,59,104,147,
44,142,63,26,81,196,2,168,22,213,139,47,190,184,156,81,246,134,179,42,98,51,51,18,105,80,44,54,221,187,126,16,11,60,8,97,136,188,232,61,145,189,186,122,229,44,100,56,195,106,81,172,88,74,87,54,132,120,
126,16,107,71,220,167,115,201,18,153,110,94,99,150,45,180,30,16,171,11,149,38,200,42,225,205,194,140,169,175,126,89,87,172,87,119,96,146,228,37,64,62,65,78,205,61,55,162,136,21,254,100,45,179,42,242,92,
29,196,198,76,57,199,31,185,145,220,248,1,59,22,173,136,21,147,137,36,68,160,13,151,90,29,196,70,43,61,76,254,150,37,52,233,61,93,218,165,38,163,241,52,202,61,7,229,20,183,251,220,47,250,218,55,11,94,
84,10,81,77,79,247,21,65,140,206,179,47,215,246,148,128,188,186,4,138,89,185,185,85,250,98,122,141,88,121,159,106,18,183,220,114,203,8,86,220,178,101,75,92,32,191,103,97,32,86,107,145,5,136,195,50,188,
153,88,154,128,167,54,245,157,254,71,132,85,3,88,139,119,250,249,70,64,44,71,203,25,55,110,98,242,101,93,162,72,12,157,99,116,114,219,22,200,246,237,219,59,83,196,90,170,196,239,220,142,189,193,177,203,
186,172,85,182,39,140,161,244,194,210,247,123,183,222,223,136,105,186,81,196,10,157,147,39,132,97,231,52,33,209,242,117,65,150,17,89,171,82,209,89,68,70,137,148,187,238,190,191,179,203,186,178,252,88,
94,109,29,6,137,58,202,225,232,2,98,133,175,118,51,121,238,172,219,210,7,136,37,238,169,242,193,81,16,171,165,108,119,64,140,125,130,88,231,72,179,58,83,19,151,173,148,38,64,212,172,254,125,151,38,144,
134,53,1,226,168,34,54,6,249,83,6,185,101,69,181,155,62,65,108,85,100,59,35,215,210,82,101,76,145,246,219,35,182,242,247,5,98,91,247,172,141,130,216,88,243,110,18,86,24,79,89,180,182,143,210,4,224,89,
85,43,129,81,43,32,54,146,197,115,232,159,50,120,235,75,17,43,181,175,180,85,163,32,86,118,18,75,204,146,45,84,139,38,130,215,71,105,130,214,190,12,237,195,120,141,88,107,159,3,247,98,235,62,64,172,16,
166,154,121,147,107,196,202,86,240,40,248,198,96,207,146,172,176,162,240,195,14,103,150,217,177,29,107,91,214,246,172,109,90,58,79,219,182,182,111,39,93,211,232,84,73,173,247,20,243,215,178,239,85,47,
235,98,57,195,112,168,46,65,172,96,50,151,151,120,207,135,215,4,177,130,123,33,133,79,182,104,16,171,133,56,76,27,218,217,117,1,213,206,89,19,196,250,101,124,201,31,203,94,50,98,196,61,37,156,71,30,249,
182,82,17,222,77,152,46,29,84,132,91,247,57,164,97,211,210,34,126,219,109,119,150,109,186,172,202,101,250,115,59,96,91,141,26,35,50,184,35,90,108,83,173,213,27,81,147,201,42,154,68,241,94,129,61,137,100,
98,235,141,128,216,188,164,75,15,180,150,148,249,65,172,51,120,144,162,165,167,45,209,157,21,196,50,200,82,82,233,91,210,176,217,242,190,73,138,88,11,181,232,16,200,160,33,183,28,137,175,237,146,35,193,
190,10,121,38,145,97,225,110,181,33,84,183,210,124,25,114,91,17,27,241,243,29,194,85,203,129,177,224,19,75,84,241,79,16,195,24,161,140,101,52,99,13,100,227,103,146,162,58,186,110,113,32,54,140,57,39,150,
131,135,147,103,2,173,12,212,189,140,245,255,206,221,24,51,162,13,207,4,76,221,253,101,93,57,96,115,237,74,106,204,56,198,55,129,27,185,91,127,53,98,167,128,88,68,239,129,152,185,55,69,246,123,142,181,
108,63,136,109,24,214,84,58,220,252,252,197,241,181,59,58,28,47,238,78,26,149,7,85,157,167,116,61,57,6,243,205,49,184,119,69,234,253,116,51,198,134,37,126,253,188,249,189,126,46,235,162,215,52,160,37,
173,132,167,34,10,220,137,140,160,14,218,184,0,3,61,158,238,64,172,214,240,166,226,32,32,86,142,40,251,193,57,133,209,185,70,138,201,248,176,20,206,55,173,184,216,210,4,205,39,46,57,191,146,172,148,67,
8,95,222,197,157,201,70,198,247,89,191,74,88,45,57,224,92,57,217,88,228,151,194,191,109,105,214,215,141,129,88,78,209,218,135,125,146,88,162,124,162,211,105,2,212,20,90,56,146,231,241,123,12,70,141,157,
127,136,238,223,22,134,205,126,39,100,252,242,80,17,43,157,182,240,58,220,138,49,48,170,190,181,103,86,16,11,194,138,203,68,28,148,34,13,193,155,109,65,206,208,37,132,19,219,128,45,53,61,25,131,226,181,
37,186,235,1,177,140,146,193,88,130,26,143,63,155,34,54,88,228,22,227,229,138,43,174,40,154,60,48,223,245,119,181,136,121,189,53,98,77,4,97,182,46,172,102,233,218,165,9,34,12,94,114,174,253,166,155,110,
42,176,204,149,65,224,89,214,216,200,115,238,14,128,57,8,70,26,231,96,24,230,233,160,152,3,99,100,149,14,144,57,72,70,70,41,241,168,43,126,129,26,85,107,173,14,98,163,251,118,16,180,163,121,109,130,119,
205,53,215,148,27,202,87,144,227,250,65,172,16,89,242,91,97,235,233,32,22,205,19,200,169,151,128,123,126,239,123,223,91,21,45,110,68,17,43,17,53,145,18,3,77,5,177,226,35,32,95,173,97,99,8,190,190,252,
242,203,87,53,108,189,32,86,164,106,135,190,170,199,49,25,196,10,216,8,153,25,147,219,29,39,157,116,114,103,32,86,104,77,243,160,27,109,150,143,129,88,75,129,0,159,218,44,65,135,105,127,231,247,239,233,
172,70,172,216,94,216,108,247,65,118,62,166,136,101,148,241,164,220,133,133,150,99,236,3,196,98,158,50,166,230,242,221,155,195,142,21,16,107,241,196,167,248,168,62,21,177,90,203,62,96,179,104,15,111,184,
135,243,126,200,218,115,206,57,167,120,238,62,65,172,1,95,105,61,197,101,43,32,86,210,152,93,215,103,105,2,203,206,196,203,186,226,155,79,91,116,109,197,166,65,194,143,62,46,235,106,49,170,21,69,108,108,
223,239,178,146,11,73,146,12,247,117,89,23,94,145,233,154,76,40,158,229,203,186,194,63,220,236,135,57,235,250,42,77,48,161,235,70,21,177,146,73,27,131,102,158,214,234,67,17,139,53,84,173,52,14,98,109,
109,73,183,205,6,76,170,15,16,107,153,169,140,154,12,98,45,142,208,34,131,12,120,209,98,87,151,117,129,32,222,171,50,106,50,136,245,11,8,177,18,20,54,34,187,84,196,58,251,87,67,180,53,65,44,103,150,153,
73,87,138,88,70,85,133,107,87,47,77,32,157,226,204,156,61,64,240,22,93,35,22,191,194,177,178,154,82,188,223,236,165,9,180,150,153,152,245,18,22,81,35,150,65,20,142,232,96,194,216,88,136,183,204,12,98,
209,58,99,139,239,74,131,54,82,154,0,162,20,59,113,61,213,224,222,22,164,240,224,118,125,207,177,179,198,117,222,103,145,54,166,24,37,196,96,84,93,142,117,150,26,177,226,37,16,191,101,204,250,21,177,145,
113,108,115,120,75,230,33,101,215,165,142,88,230,101,92,140,18,122,76,2,177,121,99,88,107,177,205,86,218,24,136,141,16,121,75,160,30,161,106,81,141,37,62,4,255,249,25,45,1,218,163,196,169,23,246,255,173,
146,4,105,204,226,64,108,125,89,87,150,27,96,156,22,36,11,49,254,172,6,198,32,226,50,1,194,50,170,63,16,107,21,144,112,232,166,166,138,219,112,32,199,135,121,238,65,108,128,139,167,227,217,21,78,241,102,
117,127,246,131,216,24,31,0,235,170,50,221,230,231,253,131,216,120,227,82,89,34,220,198,190,166,152,255,115,7,98,173,143,102,161,136,194,82,148,170,15,121,99,75,135,222,15,136,149,31,18,97,241,244,42,
161,130,177,14,225,67,135,226,175,188,217,201,206,106,117,179,83,113,9,241,80,214,110,2,231,22,162,136,229,143,72,4,120,106,2,66,198,32,125,128,72,38,175,89,22,95,28,230,193,10,36,154,210,114,238,162,
41,234,176,241,26,177,186,67,152,33,2,117,28,88,6,109,27,22,25,94,237,178,46,45,233,17,101,228,54,175,221,117,171,64,180,246,131,209,90,243,131,216,224,224,187,197,82,242,65,26,24,57,33,41,147,248,93,
173,216,89,75,19,44,223,110,240,234,114,146,132,145,62,28,96,22,45,183,187,49,108,12,93,79,220,67,142,101,227,65,240,30,155,194,61,49,134,246,237,97,235,5,177,152,167,172,169,33,196,179,129,216,176,254,
22,235,23,138,151,5,33,221,109,148,134,109,84,17,171,245,229,123,198,104,19,14,175,13,98,163,89,151,196,79,16,99,45,54,221,186,117,107,1,106,139,0,177,66,30,179,51,133,244,107,214,136,21,158,216,230,80,
162,192,229,37,117,145,209,209,131,247,235,7,177,6,190,65,223,156,254,47,238,98,234,101,93,194,84,167,135,178,222,134,80,24,71,223,182,109,219,68,196,184,17,16,203,93,8,113,178,234,252,84,16,203,40,193,
253,215,190,246,181,2,244,141,31,179,109,210,13,80,27,45,77,0,91,103,245,174,102,108,77,6,177,66,95,142,17,174,78,16,251,209,143,30,223,9,136,53,174,200,6,242,174,25,75,215,84,16,203,13,216,62,75,173,
103,151,32,86,50,162,181,4,138,205,248,58,121,68,17,43,164,37,241,174,75,19,244,1,98,181,86,83,166,124,68,147,87,212,139,114,178,172,112,211,39,136,53,134,9,169,155,157,135,81,69,172,20,221,89,6,155,139,
125,131,216,164,195,209,133,251,162,129,150,65,172,253,94,167,255,173,250,117,153,157,62,74,19,16,241,180,110,148,254,105,70,189,34,66,146,125,178,95,134,245,13,98,91,72,104,180,52,129,236,215,234,111,
225,237,75,17,107,246,57,117,91,177,133,209,26,177,2,49,153,45,167,217,23,136,149,81,79,187,42,79,86,114,178,152,219,65,136,36,196,125,128,88,88,168,106,165,113,16,203,163,138,113,32,27,241,118,150,100,
61,248,141,135,150,109,87,199,77,178,204,14,173,167,235,21,93,179,232,186,69,215,46,18,10,186,134,81,241,153,213,14,108,136,90,147,57,180,106,50,78,6,177,18,2,191,40,17,208,133,93,130,88,7,174,91,98,230,
233,32,214,250,39,180,48,224,187,4,177,85,21,221,210,133,171,130,88,216,207,216,146,203,117,117,89,87,122,241,106,60,173,13,98,45,146,178,21,227,170,43,16,91,21,174,157,13,196,218,5,7,195,204,196,52,106,
81,32,22,25,174,207,53,184,104,105,102,16,203,153,26,232,153,183,109,68,17,107,119,193,126,34,23,32,239,171,6,248,131,115,129,216,104,222,221,78,204,42,218,103,70,166,232,148,113,179,86,2,208,42,248,130,
67,66,186,75,150,92,93,186,52,91,222,215,86,196,198,160,127,48,133,166,38,128,127,211,169,248,196,140,3,96,39,41,98,181,140,56,201,118,29,86,218,236,164,15,207,194,199,0,223,24,136,149,7,170,116,202,177,
106,118,134,9,248,113,5,98,45,45,88,75,116,109,151,65,221,217,69,12,210,58,153,78,45,252,178,174,156,194,18,73,209,162,248,154,145,182,76,106,8,91,117,81,221,93,253,128,88,88,40,33,108,214,75,96,120,235,
178,174,231,78,17,219,212,71,16,57,150,179,237,251,65,108,180,194,212,26,177,17,252,255,76,252,188,212,75,136,71,72,244,220,129,216,120,243,31,26,240,177,211,185,47,198,24,105,72,255,165,9,196,94,57,19,
69,142,220,130,154,159,50,34,137,64,226,196,230,247,186,87,196,202,207,32,71,158,94,48,136,238,225,159,140,194,29,132,59,56,193,79,253,212,79,21,37,89,53,19,23,7,98,227,69,111,13,202,91,118,177,108,217,
99,0,130,64,168,8,119,144,1,105,161,58,53,75,149,34,227,172,161,54,176,179,76,38,103,28,207,250,107,196,198,226,185,219,66,234,77,65,15,52,143,33,210,123,194,47,45,179,154,0,85,218,150,123,129,4,136,14,
240,84,88,113,126,16,27,173,179,157,2,214,205,97,208,144,11,114,24,87,223,231,62,139,34,54,43,208,107,61,173,108,105,194,192,34,81,49,214,24,54,27,136,181,96,26,19,180,158,201,61,29,194,215,66,235,45,
77,128,12,103,209,119,27,0,205,73,35,145,194,218,165,9,244,185,165,3,14,114,209,146,67,247,153,157,124,235,91,223,42,202,161,141,40,98,179,197,232,255,154,113,118,107,44,218,155,220,61,57,86,238,34,140,
81,210,217,54,197,179,40,8,78,165,251,0,253,58,101,74,238,185,17,69,172,40,54,51,99,170,197,53,65,172,52,203,0,197,61,17,24,114,56,164,120,60,151,219,24,136,229,219,178,220,74,52,196,234,32,22,167,98,
16,101,117,138,152,209,152,186,27,211,192,141,128,88,179,178,46,37,54,21,196,234,99,173,100,44,165,42,223,192,238,226,178,174,84,91,87,105,214,100,16,75,238,161,112,173,212,58,91,169,43,16,203,168,204,
106,26,58,60,14,98,69,146,252,144,34,14,89,38,165,107,69,172,164,180,213,133,163,32,150,244,227,235,95,255,122,89,195,250,42,77,32,233,104,65,142,149,203,186,194,218,93,140,81,127,3,56,235,19,196,98,98,
162,214,102,225,94,134,102,17,182,254,76,172,73,251,120,107,139,107,189,116,244,113,89,151,116,172,186,55,100,165,70,172,80,132,197,90,235,185,0,177,121,55,86,211,90,203,32,86,228,200,237,231,22,108,159,
165,9,112,207,74,208,188,47,102,162,144,122,185,52,129,8,210,66,105,6,246,9,98,165,246,213,229,74,67,149,245,176,48,136,149,91,197,248,44,201,122,240,193,135,148,179,198,78,119,168,106,227,216,184,202,
54,176,162,202,54,48,35,236,232,231,240,162,163,5,147,208,34,207,175,90,142,211,35,206,40,31,122,232,97,69,36,102,60,181,174,53,27,85,196,242,172,90,203,122,135,226,245,1,98,5,144,141,242,163,208,188,
137,151,117,249,5,44,210,34,220,181,34,86,43,181,110,164,155,174,136,229,204,196,76,121,174,189,43,69,44,68,84,93,128,170,165,166,131,88,177,84,74,192,187,4,177,237,123,251,86,5,177,48,79,110,211,118,
5,98,219,18,93,241,212,170,165,9,164,83,118,194,209,225,148,232,46,242,178,46,139,48,5,100,21,178,60,171,122,229,170,165,9,228,121,201,59,141,167,46,64,108,189,31,51,51,136,141,166,220,45,71,131,7,197,
210,206,250,137,171,55,162,136,213,66,92,64,107,215,106,190,210,4,17,63,111,231,34,164,222,132,167,6,38,78,48,175,34,86,44,110,119,65,202,166,133,42,101,236,108,121,223,164,210,4,50,31,1,160,69,91,250,
109,55,66,139,225,156,210,242,73,32,214,64,150,116,106,105,65,156,208,68,116,41,220,110,162,204,36,195,235,191,172,75,218,229,5,49,78,173,231,141,108,177,10,105,243,242,46,203,6,249,46,63,215,92,39,92,
140,240,56,210,212,16,98,64,23,155,152,159,37,84,45,246,146,120,209,77,181,34,214,139,115,27,90,206,155,235,26,103,95,76,14,206,176,210,24,212,37,156,250,81,196,2,245,140,163,187,147,191,213,172,188,158,
238,60,117,239,53,98,99,188,236,10,163,246,53,41,248,80,82,249,156,131,216,38,48,19,49,238,175,17,43,66,29,43,77,16,223,27,222,181,214,196,215,63,12,183,241,112,140,33,135,7,251,45,77,192,69,24,204,121,
50,86,121,68,174,32,239,81,230,189,125,175,41,154,220,13,136,141,86,248,188,149,220,182,71,238,203,8,113,236,199,200,19,101,65,24,40,40,2,59,202,136,178,40,95,229,193,23,7,98,131,194,60,24,121,217,18,
207,109,253,19,195,139,32,232,62,25,1,59,250,30,145,5,99,50,19,130,18,179,28,15,143,222,28,27,215,194,27,3,177,225,139,238,147,194,75,231,29,221,164,207,83,65,87,107,72,199,106,230,80,107,100,200,7,68,
174,130,196,188,159,214,146,195,176,102,236,205,183,223,23,159,100,88,154,64,166,113,253,245,215,151,123,141,100,41,132,168,90,101,214,162,181,12,203,154,234,185,48,231,185,227,120,159,249,74,19,196,31,
220,107,44,144,232,158,119,222,121,229,118,158,76,157,20,248,203,174,162,68,83,10,81,122,79,214,228,56,231,145,71,25,83,135,12,19,143,172,246,149,219,188,89,234,158,60,42,38,202,125,241,94,179,129,88,
75,134,184,199,93,33,184,231,69,23,93,52,52,106,215,174,93,5,23,209,234,201,225,212,155,82,119,74,21,0,117,168,212,163,146,43,186,71,249,144,67,227,114,165,8,20,51,172,54,38,45,226,30,198,217,93,181,37,
60,19,136,53,195,24,67,159,151,91,29,89,170,96,136,22,55,80,154,128,113,162,11,241,149,19,35,102,119,24,246,178,17,58,220,56,198,205,166,189,133,86,172,164,60,129,241,147,68,15,114,172,187,209,141,247,
235,173,17,155,114,2,251,59,205,226,189,58,136,21,142,232,30,83,62,103,23,33,252,162,47,235,18,98,215,180,101,42,136,21,246,18,193,155,254,78,66,230,44,235,2,196,10,171,69,167,90,170,113,19,227,32,150,
212,223,0,204,163,228,93,43,98,189,151,72,181,142,189,198,20,177,68,16,42,232,34,195,169,204,239,3,196,214,26,170,145,210,4,177,178,223,204,35,243,65,57,176,251,40,77,96,66,77,211,228,189,60,6,247,195,
150,4,235,88,125,118,161,143,26,177,173,43,168,70,21,177,240,162,5,54,133,204,90,174,15,16,75,134,82,109,233,174,128,216,172,57,140,182,112,3,125,150,38,16,73,180,174,123,89,1,177,250,22,189,19,1,244,
9,98,197,93,85,97,200,81,69,172,169,41,193,20,176,245,9,98,25,149,187,14,209,99,15,55,96,120,185,66,179,208,150,91,176,230,233,190,62,106,196,230,61,220,213,45,229,195,194,32,69,17,171,165,52,163,112,
165,47,16,11,87,215,123,51,19,65,172,84,220,58,135,165,119,13,98,181,82,235,62,154,241,203,186,154,244,186,176,40,209,65,215,53,98,213,10,162,143,169,150,153,201,32,86,216,34,184,211,133,34,198,174,64,
44,55,64,34,87,25,180,186,34,214,57,44,135,195,234,147,102,16,163,154,249,138,215,218,105,176,203,96,231,193,46,132,26,251,106,237,219,113,80,123,95,13,126,113,86,132,244,195,104,213,41,37,245,171,114,
96,139,14,106,137,220,170,32,86,4,24,254,170,68,11,102,95,23,32,86,2,81,5,119,90,107,109,16,27,49,213,131,254,200,30,92,74,39,243,208,253,70,107,196,138,203,91,119,64,210,190,204,166,136,141,216,234,62,
174,223,108,100,156,65,169,245,24,85,223,141,245,250,215,191,33,220,199,17,177,97,116,76,148,135,58,62,146,134,143,13,142,57,246,3,195,13,161,250,110,172,4,177,78,171,85,99,105,62,16,43,47,195,59,19,198,
2,177,222,132,81,179,42,98,115,12,241,71,56,104,171,58,201,108,121,223,36,16,107,64,230,241,94,228,46,47,245,102,216,36,16,155,119,98,233,122,68,216,135,209,58,62,92,222,188,26,31,88,190,55,187,84,169,
93,35,182,201,253,203,185,99,81,34,66,12,110,152,157,89,43,1,132,181,221,75,35,101,97,103,60,247,210,212,148,42,31,42,171,113,197,247,30,92,56,136,5,91,101,60,210,111,217,8,67,154,164,114,196,8,6,37,162,
110,48,53,69,236,231,99,209,223,212,46,111,49,247,109,79,36,68,18,197,248,228,155,165,65,60,63,238,100,18,48,16,132,205,22,201,130,124,207,169,34,54,12,189,57,140,18,110,136,131,204,170,231,15,136,13,
99,246,95,214,53,60,60,221,240,134,145,26,177,130,195,120,100,32,99,221,23,93,123,118,140,189,35,226,103,253,212,136,181,94,101,186,109,102,241,63,6,61,169,147,217,8,45,242,83,17,81,150,26,28,241,44,190,
70,108,236,98,157,23,110,224,81,111,42,228,224,10,168,46,108,78,51,64,76,159,236,19,144,165,146,133,144,236,220,87,78,115,113,32,54,124,205,109,225,91,150,188,25,57,46,161,14,33,170,152,75,60,79,180,195,
8,106,71,154,6,41,127,138,45,56,87,45,6,95,55,179,115,227,32,54,94,232,94,11,178,130,163,56,195,221,119,223,93,238,163,161,91,144,180,174,86,154,32,197,22,217,149,4,167,85,5,249,245,131,88,220,28,18,162,
94,124,230,153,103,74,208,230,160,52,74,60,107,105,2,17,172,117,82,28,101,3,211,82,213,108,10,204,182,32,71,203,12,203,219,27,188,198,4,137,46,205,48,225,105,162,197,27,111,188,177,97,231,179,129,88,235,
164,22,115,250,223,222,160,5,122,174,26,177,205,2,92,22,86,20,198,88,50,104,13,228,157,59,87,4,54,15,62,248,208,220,32,54,5,206,185,120,75,28,98,73,186,109,38,16,43,57,52,160,213,24,206,219,195,164,93,
198,83,182,150,11,230,215,83,35,22,132,21,242,228,121,209,64,227,75,177,84,157,183,42,136,53,75,12,82,184,154,238,92,98,106,252,104,169,209,26,177,27,3,177,130,69,45,102,140,197,66,254,232,212,210,4,28,
32,22,105,96,171,126,154,0,173,43,16,107,139,36,165,186,83,65,172,93,41,104,26,167,170,117,231,93,129,88,126,172,214,41,68,188,117,160,219,57,163,183,14,120,113,204,132,35,76,87,27,67,148,249,121,47,150,
110,235,82,17,11,47,86,149,188,70,21,177,209,175,103,219,202,39,100,174,13,234,26,196,26,240,181,210,108,4,196,90,237,177,71,80,191,222,161,234,186,52,129,61,196,169,32,86,248,33,25,48,195,234,195,20,
125,128,216,138,226,89,31,71,65,172,31,42,72,148,219,102,125,129,216,150,150,106,5,196,242,77,6,156,21,254,121,115,89,151,200,145,191,176,214,9,69,250,4,177,222,179,73,193,50,241,88,185,172,203,2,41,163,
181,97,212,39,136,109,105,61,71,75,19,200,213,204,64,11,112,95,138,216,4,32,118,207,154,32,112,92,17,11,152,209,164,103,189,4,220,211,182,171,2,236,32,153,27,158,94,255,134,55,149,10,0,36,186,0,154,239,
3,103,160,217,141,55,221,57,136,189,201,41,85,0,6,165,38,182,107,132,44,198,105,144,238,203,218,46,145,218,159,237,30,229,226,209,227,113,131,197,179,182,66,78,57,229,148,222,46,235,18,95,73,255,155,86,
114,247,195,17,46,118,102,148,181,70,26,180,93,82,96,223,216,64,239,26,196,138,68,249,198,52,200,251,7,161,57,208,77,211,37,175,139,255,228,115,167,40,226,40,202,20,29,116,9,98,69,7,173,10,37,211,65,108,
208,187,71,45,204,246,250,186,2,177,137,24,91,23,45,77,87,196,198,194,124,30,172,99,61,66,112,69,137,53,140,117,203,128,146,23,202,95,24,252,116,232,110,35,112,43,129,129,238,150,2,183,21,136,72,13,106,
81,234,147,79,62,57,114,157,153,192,174,85,227,115,109,16,27,142,236,182,20,1,26,248,98,30,198,45,162,52,129,113,36,213,170,198,209,236,32,86,190,135,53,241,31,196,167,54,10,133,200,27,85,196,182,152,
39,227,230,7,177,12,211,212,140,179,19,33,127,91,15,136,53,142,236,172,183,110,16,155,45,239,155,4,98,53,181,216,221,88,83,74,78,12,191,218,237,97,102,150,71,87,89,66,180,178,199,223,87,221,54,95,134,
60,13,196,122,65,76,32,175,209,99,152,112,54,203,87,128,246,126,150,215,165,107,21,173,44,91,1,101,211,160,240,216,183,45,20,196,122,113,57,161,55,38,210,242,85,43,230,61,88,178,234,44,24,233,171,199,
247,26,50,252,40,122,211,25,136,77,8,235,77,189,161,55,206,98,13,245,204,178,116,241,212,189,40,98,99,1,61,34,226,175,179,197,246,77,8,187,31,196,78,42,43,61,181,52,65,180,218,80,166,27,255,22,130,88,
241,251,189,172,107,181,238,139,159,21,69,108,4,143,175,108,22,252,23,173,86,237,180,254,217,88,75,140,109,73,28,112,192,11,227,69,157,81,222,28,123,46,95,142,65,77,132,186,148,53,90,114,97,205,244,155,
43,48,17,108,205,5,35,93,10,151,64,165,191,120,16,27,47,122,82,76,253,135,133,52,30,153,51,241,159,165,135,191,242,213,94,32,234,71,255,9,136,224,161,178,35,11,112,179,129,189,56,16,27,6,221,18,159,126,
9,159,186,242,202,43,11,217,3,211,96,34,111,110,111,216,181,84,64,172,175,242,198,188,136,73,56,109,5,16,251,55,188,115,49,32,150,255,177,100,184,138,42,15,223,59,40,157,10,162,181,42,1,136,203,196,225,
128,110,37,224,154,111,153,137,150,25,42,98,57,69,11,49,130,231,10,33,103,148,147,228,129,106,186,170,62,206,153,169,89,125,75,176,176,90,116,33,30,23,10,85,75,206,108,11,114,24,52,4,177,60,179,216,71,
87,49,72,151,200,9,211,40,34,212,19,79,252,196,76,138,88,173,101,204,217,12,168,235,7,9,93,226,153,77,17,203,32,235,154,120,253,244,211,79,31,114,79,41,189,186,9,105,216,67,59,118,151,226,106,179,42,98,
147,16,59,235,101,134,54,75,210,218,53,98,173,87,86,123,245,96,213,98,124,239,123,223,59,130,25,179,224,40,16,187,115,215,158,185,47,235,210,149,121,72,204,14,107,44,87,107,151,38,96,16,36,164,133,204,
166,228,158,190,55,42,64,93,63,136,101,152,201,131,169,123,63,215,133,78,5,177,50,25,229,51,193,51,45,148,100,184,11,16,203,85,208,15,115,23,77,164,241,229,48,238,165,77,202,119,192,11,98,218,191,50,154,
114,183,186,8,54,133,56,193,250,194,183,174,64,172,240,186,58,93,107,227,123,5,196,90,163,100,171,124,17,101,126,109,80,151,32,214,192,175,140,26,191,172,203,121,62,83,63,47,234,234,163,70,44,163,234,
76,121,12,196,58,53,109,203,204,178,145,59,84,93,131,216,188,157,174,138,86,87,74,19,136,30,77,79,179,77,247,245,89,154,128,56,177,50,106,20,196,250,1,168,81,87,2,232,67,17,107,198,87,70,141,130,88,89,
137,217,96,6,246,169,136,181,17,222,74,54,86,20,177,126,128,27,88,253,251,4,177,173,3,247,163,138,88,135,45,12,60,70,245,9,98,91,202,179,81,16,43,110,34,8,100,84,159,53,98,21,158,156,52,251,138,34,86,
75,9,113,63,242,145,143,148,240,182,143,210,4,96,108,125,97,215,152,34,54,194,213,37,45,100,160,167,65,2,180,46,107,196,2,33,117,247,185,224,100,4,196,70,86,178,195,2,201,168,148,232,118,89,35,150,65,
4,136,85,177,209,221,97,208,43,71,64,108,184,132,47,163,195,182,187,140,45,173,212,149,34,86,60,5,95,214,123,125,83,75,19,68,23,62,12,247,88,156,243,40,92,23,151,117,41,171,89,69,158,6,250,234,165,9,108,
201,3,92,12,19,243,44,186,70,172,22,106,17,189,181,65,172,60,207,62,13,195,68,14,104,156,129,191,8,16,43,49,173,187,140,43,152,89,17,43,203,192,59,165,222,120,167,240,117,163,165,9,228,126,54,15,90,75,
202,252,32,214,11,232,123,97,141,152,218,192,159,183,52,129,89,70,13,91,213,3,74,195,102,203,251,166,129,88,211,86,139,49,10,29,110,131,88,236,211,195,25,214,59,84,12,50,211,236,35,182,91,40,254,127,227,
138,88,93,169,249,141,11,227,76,203,185,242,133,226,204,74,207,157,8,67,36,155,164,185,178,97,91,117,190,95,233,166,148,38,184,101,161,32,86,32,136,248,154,61,9,92,165,246,30,233,184,135,1,249,239,252,
157,166,149,30,86,0,164,19,16,27,46,99,55,80,145,0,54,47,90,78,53,108,158,226,110,36,186,75,177,208,239,136,223,255,114,140,169,205,42,146,68,151,14,139,244,77,131,104,107,66,179,252,195,104,242,23,201,
203,162,181,94,233,28,158,32,63,90,96,63,136,157,218,130,209,58,63,250,32,54,62,68,183,138,216,24,83,167,135,200,111,79,76,251,37,236,138,83,5,96,237,150,183,7,190,153,167,220,116,24,181,61,200,202,185,
49,6,15,138,127,3,186,27,27,232,13,68,219,20,95,85,175,220,193,79,209,234,17,126,1,173,64,154,92,145,83,21,85,8,123,196,100,254,159,19,149,29,89,158,64,140,48,254,177,230,117,188,222,98,46,235,226,60,
197,239,42,161,186,104,105,207,158,61,69,9,139,59,160,51,12,164,189,98,36,236,8,220,102,201,86,134,53,183,175,46,6,196,242,55,180,3,120,149,75,156,25,149,36,207,169,238,250,130,147,250,218,226,58,206,
103,148,84,202,217,135,13,213,136,141,38,182,96,18,48,12,139,141,106,5,178,147,52,74,139,65,215,237,90,9,109,16,43,172,182,14,234,198,234,106,198,217,22,228,102,12,1,163,0,105,105,114,231,146,85,63,77,
204,72,200,236,58,170,52,236,203,95,62,127,205,210,4,198,88,94,11,106,45,172,138,139,206,14,98,165,240,22,94,168,26,68,171,169,158,177,114,213,85,87,13,141,186,123,203,15,6,23,254,250,215,103,46,77,64,
117,13,242,86,245,22,103,3,177,102,13,205,11,68,109,204,212,122,207,164,122,64,236,222,199,159,28,220,114,235,93,69,143,160,94,2,125,2,157,2,189,2,221,2,253,2,29,3,61,67,134,213,194,30,9,104,83,77,183,
28,16,155,90,35,54,124,202,65,49,126,30,131,169,109,117,152,105,90,165,11,16,11,15,216,125,231,235,154,129,63,185,52,65,248,145,115,253,34,131,240,242,250,194,183,46,64,44,195,28,207,116,66,78,87,198,
44,63,125,4,196,54,75,193,118,131,81,183,217,125,234,3,196,114,176,12,19,114,71,139,237,25,3,177,241,205,37,209,164,41,223,39,136,21,46,227,24,209,48,227,32,214,15,44,17,142,102,214,165,161,187,188,172,
43,11,210,228,49,169,49,16,203,47,9,254,105,135,251,4,177,118,183,170,164,98,20,196,114,104,126,129,81,125,130,88,91,33,149,81,163,32,86,168,193,151,232,190,62,47,235,178,163,85,25,53,10,98,25,165,219,
248,169,62,65,108,11,89,51,112,5,196,202,241,133,34,214,182,62,65,172,60,113,82,75,149,26,177,242,57,62,202,58,215,39,136,109,25,53,10,98,157,179,58,245,212,83,75,160,214,39,136,109,73,191,71,21,177,66,
10,133,138,232,209,251,4,177,181,81,99,32,150,218,194,190,140,193,222,39,136,173,104,204,120,105,130,0,100,123,68,153,130,127,126,170,107,69,172,72,148,74,173,58,78,48,174,136,13,23,127,186,228,64,218,
196,32,173,213,21,136,77,68,228,72,76,178,136,88,230,206,13,249,201,242,118,109,179,103,171,6,222,102,105,84,94,5,132,53,49,174,11,16,43,68,202,26,103,141,59,120,44,248,215,65,129,150,138,114,168,62,19,
58,188,157,0,242,161,189,19,90,212,23,118,45,66,17,139,234,9,238,234,179,202,107,214,136,13,203,197,204,37,198,1,99,197,86,139,40,77,144,236,188,46,44,195,176,185,64,172,102,213,98,78,107,136,217,25,182,
17,69,172,96,174,186,232,52,189,248,250,64,172,68,20,62,52,35,235,90,9,171,9,80,219,220,19,204,39,247,173,150,19,255,158,45,239,155,4,98,197,207,12,75,142,158,198,248,58,237,178,174,186,120,131,22,170,
18,208,97,11,133,81,27,7,177,64,63,208,106,203,158,247,183,35,161,80,12,231,231,171,180,92,87,83,254,72,161,12,104,231,187,218,50,239,133,130,216,24,248,143,233,2,97,179,214,243,53,193,107,126,175,185,
134,106,136,135,154,216,59,91,103,241,151,117,197,27,28,20,49,215,185,209,236,165,52,65,2,87,51,53,159,44,83,144,116,88,157,180,248,217,158,72,223,78,239,26,196,30,184,31,196,206,34,14,141,238,123,254,
130,216,88,60,191,18,51,233,122,81,107,60,99,130,230,248,94,191,53,98,99,154,47,201,120,172,246,212,141,54,148,8,76,83,153,216,174,193,17,6,118,163,136,21,65,200,120,108,18,57,91,10,51,58,183,69,69,228,
177,112,231,25,118,199,14,160,30,143,221,47,85,156,195,129,238,137,150,37,23,223,4,206,173,54,84,86,131,248,244,158,168,158,154,174,78,190,14,69,168,196,95,64,44,154,151,220,83,178,145,183,4,167,84,87,
104,141,216,192,146,150,158,104,81,8,123,99,53,98,227,5,10,102,180,65,100,243,17,140,85,189,210,147,120,241,142,59,238,24,57,61,153,153,16,132,45,1,209,205,152,103,222,38,198,151,69,247,110,157,107,153,
137,95,30,42,98,25,100,247,157,46,88,89,86,10,70,104,154,156,105,251,246,237,197,48,146,93,170,143,105,100,56,195,106,217,183,72,211,170,208,128,178,217,22,100,125,173,203,194,11,111,21,182,88,112,141,
31,106,70,73,69,214,247,52,134,174,187,238,186,97,107,93,244,181,75,86,173,17,155,97,53,128,98,141,108,105,136,103,83,196,218,89,48,22,228,129,90,167,230,158,178,157,95,249,149,104,169,135,150,203,20,
40,238,240,221,27,110,41,103,177,102,169,17,155,69,146,115,153,106,130,202,233,151,117,57,185,141,130,128,248,168,176,67,173,211,64,172,3,96,143,60,178,119,240,131,251,182,13,46,251,198,149,51,131,88,
33,144,110,20,206,228,77,117,171,94,214,21,227,103,143,67,135,30,16,182,230,158,227,138,216,245,95,214,5,206,217,103,22,214,52,217,204,244,26,177,226,114,101,45,250,0,177,226,45,196,37,111,183,159,90,
154,32,111,185,175,81,117,87,151,117,113,17,169,81,111,206,73,76,190,172,75,147,154,105,166,125,31,53,98,25,70,174,100,166,139,209,198,106,196,242,73,194,90,144,35,203,19,116,93,154,32,15,153,101,137,
186,112,172,95,9,227,40,223,150,171,84,138,30,173,89,252,82,95,151,117,105,41,190,48,141,138,89,40,234,96,207,242,197,74,90,202,130,106,145,237,19,196,114,168,213,101,56,163,32,214,15,140,37,117,128,250,
4,177,173,218,195,163,32,22,109,177,175,103,57,233,19,196,10,133,244,82,245,172,128,88,156,72,81,118,238,160,79,16,75,168,83,25,52,170,136,229,250,69,2,90,169,79,16,91,93,212,197,184,21,16,27,206,235,
122,142,12,29,214,74,125,129,88,225,116,43,165,95,1,177,225,234,191,162,25,83,97,221,23,136,213,16,85,233,195,241,203,186,98,246,45,161,43,102,94,222,46,208,181,34,150,78,175,26,79,147,47,235,178,246,
129,20,229,130,174,8,101,187,86,196,182,220,193,228,210,4,154,50,13,171,43,1,192,138,74,177,170,38,225,177,89,173,202,132,11,225,142,62,230,67,131,95,251,248,103,74,144,119,201,165,87,12,238,184,115,235,
192,133,111,237,139,224,148,100,21,183,103,57,77,100,47,47,170,212,90,171,94,214,133,148,216,125,144,125,232,202,46,64,44,131,32,163,10,87,239,89,245,178,46,233,20,164,195,48,62,107,209,165,9,196,81,24,
87,93,222,55,38,218,217,171,94,214,213,228,101,37,235,165,21,103,212,34,64,108,234,61,193,182,74,193,49,63,136,181,111,98,134,240,93,121,205,193,122,106,196,146,96,154,64,53,247,108,82,251,173,97,224,
193,237,250,158,147,206,25,15,243,62,210,73,153,141,80,70,222,182,30,16,235,67,73,223,107,127,212,184,129,217,242,190,54,136,149,201,138,10,197,89,62,173,125,27,251,41,164,2,226,32,241,144,208,35,175,
172,150,16,212,100,152,115,180,179,80,149,221,89,28,136,149,37,11,109,196,210,96,133,4,195,78,170,239,91,9,84,140,128,165,25,197,224,44,94,91,215,108,105,90,231,217,133,129,88,101,48,35,39,68,77,10,238,
161,195,227,58,100,186,140,244,255,102,44,120,47,138,212,58,19,184,249,226,65,108,93,35,150,219,168,99,32,185,155,86,100,112,26,217,36,154,69,40,216,75,105,130,181,64,108,24,116,125,24,249,21,167,59,34,
227,62,208,150,89,238,80,205,132,48,103,249,165,250,119,148,140,139,174,121,113,147,117,8,242,197,248,251,107,196,78,173,17,27,238,226,148,104,177,219,98,22,238,142,238,122,170,25,99,195,18,191,241,255,
253,130,88,103,135,237,201,112,170,50,32,78,149,32,30,23,200,11,4,211,5,24,232,241,116,7,98,189,17,87,96,133,87,104,244,226,139,47,46,231,221,157,53,5,212,132,38,28,38,48,102,147,60,21,251,205,213,83,
59,227,239,41,107,55,197,179,24,16,203,32,187,154,142,3,203,162,225,162,172,90,249,224,131,15,14,47,15,144,116,120,24,40,244,177,60,97,94,144,64,56,219,165,248,80,247,196,36,57,44,94,239,199,230,18,201,
199,31,200,231,237,199,21,16,43,184,183,164,200,11,93,146,227,14,17,249,225,189,247,222,59,44,209,170,198,94,166,102,98,124,145,107,26,39,67,194,186,188,134,181,212,37,152,205,235,207,118,39,164,79,224,
15,98,32,151,195,173,226,31,146,56,116,143,136,57,225,7,238,89,31,190,151,89,79,43,77,144,119,99,1,26,198,94,19,186,72,167,124,240,177,22,27,137,18,244,117,244,255,97,17,74,108,183,150,105,17,7,239,201,
151,178,2,106,34,34,216,241,242,203,87,234,198,94,124,201,101,83,47,235,74,16,171,43,109,76,10,240,26,130,55,155,34,54,12,186,199,39,214,77,84,103,168,94,27,196,190,255,253,31,40,151,117,109,219,190,171,
128,216,39,158,120,34,8,224,213,131,79,127,230,212,85,47,235,98,148,136,194,34,94,5,122,107,43,98,109,201,234,42,154,225,188,176,121,146,34,22,136,85,145,235,251,119,109,45,116,248,220,47,94,56,147,34,
22,136,53,36,44,216,77,72,51,93,17,27,93,167,40,251,78,185,223,37,151,92,82,52,229,93,129,88,200,201,216,202,99,45,241,222,211,65,172,38,213,188,202,19,116,173,136,213,90,92,76,181,237,123,202,68,69,172,
233,111,80,19,50,119,173,136,229,30,28,153,18,139,53,221,56,25,196,90,38,108,10,17,190,247,1,98,133,214,146,19,177,216,84,16,107,144,163,195,89,158,160,15,16,75,195,192,31,54,53,172,78,25,1,177,6,29,70,
165,24,136,36,180,143,203,186,178,82,51,84,222,220,112,112,91,12,161,21,16,107,41,177,174,157,124,242,201,195,187,177,250,168,17,107,98,101,33,135,88,69,118,55,65,99,137,26,127,72,150,164,238,176,77,162,
62,65,172,20,173,57,190,34,253,18,151,149,202,167,192,103,233,58,165,118,158,139,203,186,234,234,112,141,61,7,188,34,6,217,83,6,247,103,63,251,217,145,227,38,46,158,180,135,231,236,130,219,119,32,159,
227,63,246,169,193,113,31,254,120,41,44,234,44,195,225,71,196,229,94,113,83,15,20,4,255,236,221,251,196,224,190,251,119,150,130,163,170,227,124,227,155,87,23,68,228,214,30,103,33,84,203,113,5,140,22,146,
39,182,170,0,172,128,216,128,161,187,173,228,240,98,189,237,234,38,76,23,79,58,76,193,8,44,138,33,71,30,245,222,82,241,148,49,120,21,195,24,197,160,33,151,138,10,168,123,246,44,27,136,89,185,70,72,177,
72,23,91,186,147,134,65,146,214,86,194,186,2,98,197,218,228,71,214,58,179,174,47,16,139,228,201,19,155,208,217,215,21,16,43,9,80,114,222,186,103,205,235,11,196,214,101,9,24,54,166,136,117,198,129,170,
80,32,70,95,208,199,101,93,173,221,134,201,32,150,187,151,165,200,80,250,0,177,51,213,136,213,132,226,28,174,223,248,234,90,17,107,121,201,241,180,42,136,245,75,32,151,74,242,93,41,98,19,196,86,32,109,
231,154,32,22,210,225,76,141,171,69,131,88,6,137,165,234,141,162,153,20,177,90,139,139,48,232,249,45,177,245,34,74,19,72,84,197,80,96,91,70,157,241,245,158,8,95,54,213,135,166,199,178,153,48,168,156,239,
67,235,108,209,235,70,70,109,4,196,34,195,142,230,113,148,245,146,18,174,200,33,87,217,211,72,198,60,21,196,202,251,184,8,14,78,139,225,5,235,1,177,90,71,156,86,29,186,79,49,243,108,121,95,27,196,70,36,
186,93,83,83,92,251,234,197,205,76,134,26,23,150,10,112,118,18,136,213,58,206,196,180,196,167,27,7,177,114,254,88,6,238,241,194,64,172,71,76,45,193,208,189,118,37,64,88,131,55,235,37,88,207,0,144,166,
128,90,123,43,118,113,32,150,64,38,158,157,153,121,200,110,27,73,91,129,175,128,44,70,0,224,11,216,218,117,54,26,95,212,15,136,181,107,160,213,146,12,51,206,255,39,55,143,159,63,247,32,54,140,121,42,150,
170,221,49,205,111,139,231,148,253,32,54,198,200,139,227,193,176,166,210,225,230,231,126,175,255,26,177,209,93,219,195,159,237,107,202,177,10,252,249,31,193,90,191,32,54,222,240,89,97,14,191,5,162,129,
105,18,14,7,159,91,119,135,150,98,200,241,116,11,98,185,3,144,85,4,225,128,162,50,5,223,249,206,119,10,169,145,234,91,43,45,41,148,254,121,177,96,186,132,248,186,88,16,203,71,217,243,181,228,128,32,144,
163,22,186,252,242,203,135,201,2,221,158,4,196,169,17,33,117,214,214,151,83,74,118,161,238,48,108,227,138,88,27,145,98,119,116,143,200,20,221,3,93,197,242,50,234,186,84,129,236,58,185,103,134,60,169,113,
224,233,161,197,166,242,233,174,112,25,199,154,12,147,8,241,196,5,57,154,253,216,104,149,221,168,136,124,80,203,48,198,73,73,255,159,183,149,139,32,156,240,206,180,10,177,201,76,40,79,190,105,57,81,134,
40,214,191,173,147,150,32,254,44,140,154,13,196,10,93,194,233,237,66,127,9,188,116,9,32,219,22,15,74,237,223,246,182,183,71,69,128,171,135,70,125,237,98,197,108,223,17,105,255,33,35,37,200,106,16,107,
1,23,18,85,197,28,102,83,196,218,49,0,240,181,76,155,123,26,204,9,98,239,186,251,254,193,237,119,108,9,253,240,19,145,120,62,22,89,240,101,107,130,88,193,34,216,111,231,162,89,75,215,6,177,42,117,9,232,
84,209,53,221,219,152,49,75,19,36,136,253,206,117,183,70,230,123,99,168,98,191,61,248,236,231,78,95,179,52,129,174,52,190,72,148,102,2,177,209,117,59,196,77,200,48,121,119,151,32,86,52,49,19,136,205,216,
220,70,144,139,154,187,188,172,139,190,83,216,179,38,136,229,165,69,149,212,248,93,95,214,37,106,149,209,24,240,141,112,98,50,136,229,224,200,189,209,225,174,65,44,39,204,243,139,96,155,110,28,189,172,
171,89,60,75,202,78,252,158,37,89,187,6,177,132,21,220,67,67,135,13,252,147,71,64,172,241,100,39,42,47,236,234,11,196,102,169,195,102,108,141,150,38,48,235,110,190,249,230,178,65,212,167,34,150,123,176,
140,53,187,167,34,10,187,247,5,124,62,101,250,147,84,214,245,206,251,168,17,43,11,170,20,103,207,52,246,28,240,211,72,139,197,20,94,172,235,157,31,255,177,19,203,49,129,89,31,136,113,150,199,213,215,117,
174,216,186,165,103,89,17,27,145,227,51,90,202,0,175,235,254,244,1,98,13,246,170,22,227,16,89,191,92,72,107,230,165,111,234,83,17,91,139,5,155,80,186,140,169,162,71,7,31,132,42,194,218,190,64,172,65,94,
221,214,58,10,98,249,41,73,36,252,163,251,12,246,62,106,196,182,138,214,142,131,88,201,128,176,21,177,21,196,49,170,107,69,44,98,216,196,239,190,78,6,177,124,133,95,20,252,231,193,174,162,142,13,177,169,
43,204,82,124,234,171,235,205,0,126,176,223,46,132,89,103,119,1,204,111,11,79,243,255,221,252,100,231,74,183,81,164,181,24,250,100,69,44,171,245,49,138,130,16,119,9,98,85,64,173,90,105,117,69,44,119,159,
32,182,102,158,63,247,115,17,99,135,84,87,139,217,254,208,74,111,127,199,209,101,123,196,190,141,150,178,255,98,187,227,225,221,182,67,86,164,186,36,186,220,77,45,209,109,21,214,94,189,70,172,190,181,
133,42,77,50,224,23,93,35,86,47,160,123,85,105,223,165,153,64,44,72,134,53,9,135,51,85,90,4,136,77,131,228,0,121,174,116,174,210,4,242,51,75,65,77,134,55,82,154,64,52,64,47,42,161,173,96,236,174,185,20,
177,110,43,16,220,235,74,47,198,167,72,143,230,169,17,203,25,203,92,68,152,52,8,245,5,76,182,102,227,145,144,174,46,192,169,65,108,188,192,177,209,215,187,68,134,160,5,222,9,27,106,65,190,76,198,99,122,
79,2,177,140,33,224,225,140,197,225,245,249,247,38,131,153,239,22,149,73,151,117,57,56,15,245,48,44,161,172,129,106,221,210,114,22,213,220,84,4,97,237,158,55,23,48,151,177,83,201,116,23,11,98,165,95,252,
10,7,107,73,226,248,188,49,35,85,10,20,111,171,154,107,19,40,51,148,234,144,52,159,212,15,136,101,164,76,151,97,169,136,205,150,105,12,234,239,178,174,9,138,216,167,180,94,24,246,140,173,13,81,135,93,
206,253,32,54,90,226,249,13,98,155,104,241,41,137,108,116,215,190,152,169,165,251,154,92,210,253,165,221,214,136,141,55,0,84,189,233,80,250,45,6,19,123,17,163,18,19,130,177,54,152,50,235,109,126,191,91,
16,155,238,64,36,1,47,94,120,225,133,229,96,153,59,179,28,23,6,70,68,3,121,12,165,58,153,189,216,203,186,98,218,47,241,63,60,180,224,204,27,163,194,228,185,146,87,21,45,51,144,67,137,5,137,188,189,237,
183,116,164,201,162,162,229,214,15,98,113,207,16,80,236,178,164,8,246,196,67,184,39,61,12,228,88,95,1,67,178,155,70,81,22,73,215,146,117,90,134,28,218,160,105,33,200,104,106,189,204,183,204,88,32,227,
121,103,12,218,221,222,88,146,170,37,148,246,117,31,77,141,136,50,155,118,14,62,141,82,45,53,19,15,161,180,11,43,178,248,145,110,181,38,54,132,216,198,210,218,11,114,24,51,188,172,75,11,209,123,226,158,
109,204,152,183,246,184,36,135,240,235,91,87,92,89,193,216,75,2,182,173,40,64,242,14,220,140,199,44,228,226,42,251,131,225,108,119,197,123,206,6,98,69,3,234,229,105,165,181,64,44,149,15,230,9,196,62,246,
216,222,193,87,206,191,104,240,203,71,127,176,220,163,124,72,24,87,183,86,70,177,186,84,72,52,51,136,53,221,49,35,32,86,117,193,181,64,44,249,209,197,23,95,94,64,236,5,23,94,60,248,200,175,158,184,106,
141,216,220,185,23,163,53,33,205,218,151,117,177,94,134,204,168,250,42,170,69,150,38,208,133,134,71,6,125,34,136,169,151,117,89,245,109,86,155,97,246,93,186,4,177,98,176,250,192,207,212,210,4,118,168,
232,122,207,58,235,172,82,246,36,185,103,23,151,117,121,159,172,187,184,170,34,22,228,48,184,237,197,228,254,75,151,151,117,137,251,25,148,177,215,88,105,2,25,6,4,100,44,89,195,250,184,172,75,23,90,39,
51,83,30,3,177,214,50,6,161,121,90,169,47,16,75,144,83,165,239,43,32,54,90,105,187,214,177,107,149,18,93,128,163,15,69,172,181,180,42,85,183,82,154,32,6,220,51,182,94,45,37,217,109,56,85,95,32,182,66,
66,43,138,88,89,137,53,138,23,127,46,64,44,1,89,213,133,195,210,4,79,241,226,22,222,190,65,172,69,186,186,176,107,136,172,75,185,11,179,64,4,224,233,19,196,218,160,170,234,49,14,225,126,1,177,154,143,
196,209,122,215,23,136,149,222,183,238,90,27,173,17,203,168,44,92,212,23,136,149,85,87,69,65,38,131,88,191,0,251,89,98,186,86,196,106,165,150,248,116,50,136,181,106,27,240,92,3,163,186,44,77,32,2,205,
235,138,155,153,55,29,196,138,18,12,246,174,75,19,56,204,83,185,129,213,65,172,89,200,203,138,165,50,190,22,152,45,18,196,162,122,115,93,214,229,34,56,93,40,73,96,84,23,32,182,181,222,205,118,89,151,252,
78,188,195,40,1,255,162,20,177,41,209,173,138,55,204,94,154,32,18,207,93,210,113,71,42,137,102,132,24,27,85,196,50,72,69,128,150,30,97,246,26,177,13,239,220,205,211,90,23,25,136,210,225,223,9,99,103,173,
17,43,75,38,134,23,101,170,46,88,41,99,103,19,224,76,171,17,203,48,241,180,175,198,155,55,98,220,52,16,107,61,3,58,204,100,127,147,197,71,65,143,38,210,220,213,144,225,217,164,74,147,64,172,180,75,200,
42,238,145,19,218,201,180,136,114,128,245,182,43,17,23,231,107,6,103,166,194,16,25,82,99,208,226,47,235,226,87,188,1,227,76,105,111,78,31,140,8,27,43,190,87,67,88,31,164,130,177,221,131,88,159,92,215,
104,181,154,149,183,136,240,115,3,98,195,56,245,16,158,105,90,104,40,169,220,15,98,99,92,61,127,107,196,54,160,117,228,174,181,200,176,159,82,170,32,118,35,110,115,106,41,126,167,127,16,107,28,217,14,
49,11,133,34,210,52,44,43,149,177,121,89,87,3,112,55,199,215,197,94,214,69,36,159,165,232,25,34,64,227,10,36,26,244,86,104,158,50,173,234,162,73,104,69,149,214,184,252,27,100,37,158,141,43,98,157,115,
143,165,230,158,240,226,75,170,230,114,148,210,48,154,189,227,142,59,174,144,25,255,118,26,23,241,83,162,0,102,116,191,187,120,204,154,201,201,86,193,220,198,47,235,138,133,121,187,60,80,75,224,228,196,
131,140,201,110,170,203,33,82,201,62,245,212,83,197,168,45,91,182,148,208,39,207,178,243,252,54,178,171,29,209,245,129,88,71,84,44,164,192,42,121,110,251,88,121,222,73,51,60,95,26,130,176,132,177,212,
106,210,127,33,181,240,71,24,164,149,117,55,172,216,148,99,157,237,136,74,13,98,109,213,42,80,155,87,115,212,152,177,13,98,165,247,74,19,172,96,235,203,10,112,203,162,181,144,183,208,199,70,165,197,220,
230,101,115,55,150,10,226,170,76,76,62,204,19,63,120,97,60,155,26,248,94,8,110,86,145,104,27,84,43,98,129,88,231,64,149,38,0,98,247,237,219,55,56,239,75,23,196,189,88,227,32,86,192,152,87,130,154,8,205,
194,173,252,197,244,26,177,205,236,16,158,14,46,184,224,130,50,142,86,171,17,171,52,1,16,235,128,170,210,4,23,95,242,205,193,151,191,242,235,51,129,88,227,76,188,214,132,50,39,197,215,151,13,35,148,246,
229,18,34,1,99,128,164,187,174,38,177,104,16,235,61,236,64,52,25,205,206,85,65,172,38,229,107,192,142,46,65,172,110,204,91,121,154,45,222,201,165,9,236,155,240,57,20,214,57,221,187,170,17,107,151,190,
37,194,25,87,196,198,150,196,146,36,65,43,229,21,47,12,234,18,196,26,236,85,66,58,170,136,181,120,242,31,152,103,125,131,88,215,138,88,131,189,42,33,61,170,136,141,89,112,27,38,101,237,226,36,251,186,
172,139,223,202,250,176,77,139,173,128,216,72,161,118,27,67,214,177,218,83,247,1,98,29,197,99,80,19,70,175,128,88,113,144,77,66,211,190,111,69,172,165,167,170,241,50,90,35,214,244,180,211,192,13,244,89,
35,86,90,38,217,157,4,98,127,40,123,181,107,110,76,245,89,35,86,120,3,15,52,70,173,40,98,129,88,105,18,26,226,56,74,159,32,86,54,109,243,187,49,106,88,238,162,128,88,161,173,46,180,147,206,21,104,53,231,
95,236,114,146,77,18,5,170,223,73,32,120,240,27,223,60,148,88,18,13,250,57,161,224,52,57,165,250,158,196,131,214,74,199,90,14,61,244,176,97,229,65,99,42,15,149,197,96,31,22,6,113,78,207,177,183,162,220,
231,56,83,124,42,30,178,237,234,38,76,198,40,44,74,223,249,186,95,56,164,104,62,41,25,125,159,130,145,81,170,73,60,27,197,27,38,235,60,7,17,65,60,59,184,237,182,59,135,6,25,79,245,45,229,78,79,198,26,
200,158,162,91,145,117,60,171,11,21,253,55,174,186,6,177,186,173,85,154,96,252,178,174,48,10,88,47,26,2,93,216,245,101,93,176,82,235,44,224,244,26,177,0,41,39,154,91,247,194,217,46,106,196,86,131,187,
12,242,169,165,9,164,81,2,124,201,65,198,215,93,212,136,181,169,93,87,63,149,126,173,90,154,192,49,55,107,146,173,144,46,64,172,24,189,42,46,83,212,213,107,42,98,163,159,239,177,72,218,131,131,150,69,
136,139,4,177,242,199,234,12,214,108,32,54,252,213,97,177,29,178,157,10,8,82,148,9,107,57,173,182,17,69,108,74,116,171,16,88,43,205,14,98,227,151,157,80,124,64,122,46,184,231,42,76,0,140,83,170,164,245,
102,5,177,121,3,157,128,174,85,118,124,182,188,111,18,136,53,51,100,182,140,179,161,36,103,203,52,105,18,136,229,127,116,59,121,174,200,3,21,38,84,205,27,203,155,229,100,190,226,125,211,64,108,51,109,
203,27,120,35,81,163,56,91,230,171,123,249,55,177,145,36,83,138,230,131,72,159,140,31,144,195,99,41,139,175,247,68,15,204,95,230,176,50,236,37,241,66,155,148,38,48,109,25,134,250,26,111,140,35,15,215,
26,42,80,104,197,230,77,135,70,84,213,0,140,159,157,102,89,0,219,77,237,242,22,237,19,182,83,111,230,201,95,12,35,94,40,81,172,107,196,210,18,155,4,90,36,25,57,131,24,225,107,5,99,251,5,177,241,230,167,
68,74,118,155,163,187,97,196,88,145,209,253,32,54,198,198,143,22,136,13,131,75,141,88,151,120,69,62,247,149,38,36,234,87,17,155,3,26,62,204,50,210,66,18,206,149,222,64,214,221,132,68,139,87,196,186,70,
40,192,199,30,250,96,126,72,216,225,33,7,177,46,226,87,68,168,148,177,16,164,10,169,190,242,230,128,73,179,27,177,24,16,27,159,178,92,214,21,159,122,137,136,130,248,20,141,17,4,66,142,82,50,97,14,40,130,
113,146,14,224,90,194,225,71,31,125,180,160,36,134,203,148,154,179,236,27,7,177,225,119,182,138,24,156,244,167,26,34,70,213,42,142,219,37,76,171,147,87,228,248,246,219,111,31,198,232,164,4,60,190,200,
160,133,175,215,7,98,163,133,238,215,244,62,45,157,240,90,32,86,174,168,213,92,222,165,165,182,109,219,86,90,80,116,145,87,3,145,81,86,135,194,102,43,218,30,134,12,21,177,150,20,47,104,124,0,178,107,129,
88,231,75,207,253,226,151,135,173,116,237,181,215,22,198,149,97,181,117,210,26,105,2,216,49,197,194,98,162,184,222,108,54,69,44,131,196,61,87,93,117,213,196,203,186,38,129,88,41,214,101,151,93,81,106,
198,106,169,47,158,247,149,72,191,222,60,162,136,5,98,69,17,4,130,66,161,166,213,214,46,77,32,60,245,203,242,63,104,104,86,16,235,4,238,105,167,159,51,184,252,91,87,198,233,181,175,15,222,245,238,247,
135,76,55,10,182,197,217,64,137,71,234,27,24,38,22,51,51,45,230,77,72,115,246,170,32,214,106,111,211,218,5,75,138,67,38,247,92,52,136,21,147,217,100,50,43,195,101,168,82,191,57,26,99,229,150,223,104,157,
151,198,115,138,179,47,128,3,162,103,234,231,133,111,6,187,253,150,118,230,43,29,87,194,80,109,69,45,165,54,163,82,136,210,123,217,180,86,146,73,191,230,181,113,93,94,156,13,212,98,181,190,65,107,53,65,
159,8,99,50,136,21,85,242,51,206,184,215,119,99,117,161,136,53,240,25,85,237,114,141,131,216,200,98,150,64,88,254,168,38,195,93,130,88,131,190,98,158,163,32,54,2,182,175,8,111,1,253,218,160,174,65,44,
199,90,157,214,30,5,177,174,11,2,27,44,35,41,98,238,67,17,107,66,181,40,222,104,105,2,130,61,59,86,121,123,88,95,138,216,105,55,220,151,26,177,142,250,90,96,249,165,62,107,196,86,26,79,51,113,20,196,90,
201,1,88,101,119,250,4,177,237,154,158,97,216,80,17,251,67,94,156,143,178,206,245,9,98,1,143,234,222,228,209,203,186,56,49,235,81,93,17,190,235,26,177,26,129,175,170,206,36,175,212,136,13,131,174,22,218,
138,38,157,22,66,135,251,2,177,117,241,218,176,225,250,176,165,84,104,30,130,88,132,69,24,171,251,250,40,77,144,87,192,52,215,47,90,7,221,28,60,10,98,81,17,91,180,124,85,31,32,86,146,33,182,106,214,191,
201,32,86,173,50,155,217,245,78,121,87,165,9,136,6,107,92,45,235,137,225,115,96,132,54,47,40,44,33,163,4,46,95,72,203,168,46,65,44,66,211,98,85,211,65,172,52,138,156,141,243,204,90,9,93,128,88,193,93,
117,96,186,156,81,158,10,98,67,238,120,182,236,69,194,32,100,21,136,45,186,70,108,222,76,144,113,148,72,119,77,16,43,207,211,141,121,203,83,234,135,23,81,154,128,65,173,186,27,179,129,88,217,133,44,67,
236,108,233,201,58,196,140,218,40,136,37,96,173,130,186,117,129,88,121,89,33,116,90,77,20,33,224,95,47,136,173,119,171,154,174,155,45,239,155,6,98,57,55,113,143,173,183,218,168,73,32,214,178,225,145,223,
73,12,248,34,97,118,11,85,207,119,157,245,52,16,75,243,43,188,33,21,16,95,39,132,101,128,55,206,27,200,253,142,46,215,253,30,31,40,229,185,90,104,97,151,117,37,136,53,30,44,158,208,15,35,156,114,131,128,
242,141,165,75,249,52,7,240,135,84,56,12,219,227,74,188,206,64,108,64,217,165,108,5,70,52,215,29,12,37,186,245,116,231,169,123,187,172,43,222,200,141,188,86,117,225,134,56,40,35,199,82,100,116,63,136,
141,86,248,209,4,177,186,47,158,238,107,196,198,24,57,40,98,233,115,13,92,74,215,12,204,204,200,156,113,106,113,200,120,185,5,97,73,172,163,75,193,73,247,196,216,59,61,254,110,115,60,139,85,196,70,172,
245,152,72,81,88,99,161,230,56,179,104,40,204,147,71,58,33,69,76,84,4,11,146,72,114,155,196,96,177,32,214,74,46,123,38,197,165,124,5,92,189,49,6,234,129,17,73,7,164,103,244,157,160,237,158,61,123,6,183,
222,122,107,193,146,194,236,166,101,55,14,98,195,152,123,173,125,90,72,145,6,88,90,139,212,183,246,212,32,86,108,15,67,146,207,37,54,82,123,47,47,143,231,116,27,246,190,126,16,203,247,88,70,180,80,187,
190,103,91,17,155,185,34,66,252,253,239,127,127,104,148,46,212,189,54,38,109,104,87,71,84,102,91,144,195,136,17,16,235,188,31,60,196,160,73,53,98,179,52,1,69,44,16,171,174,222,231,78,90,41,232,112,207,
61,247,20,190,46,138,181,86,218,178,149,198,85,132,120,182,203,186,4,121,89,99,216,167,116,83,207,180,203,186,136,105,82,17,139,228,185,172,235,210,175,175,212,33,190,228,146,75,75,102,148,129,162,237,
17,45,47,106,16,14,53,177,213,108,32,214,78,39,89,55,92,173,59,86,187,172,43,21,177,140,114,5,204,231,63,127,198,224,166,155,111,29,124,239,123,119,132,48,231,35,19,65,172,8,84,192,215,132,51,171,95,214,
37,128,183,129,141,121,42,161,201,32,3,215,167,157,7,196,190,235,221,239,27,188,231,189,31,40,181,207,40,135,240,78,71,241,82,223,144,149,152,4,126,26,32,22,248,199,34,170,56,104,12,196,134,67,60,93,210,
224,205,205,30,211,190,107,69,172,89,137,79,153,145,209,149,231,134,13,47,109,82,190,178,150,29,24,131,122,143,58,9,237,46,235,82,17,107,140,85,249,223,56,136,141,101,97,73,177,236,122,12,117,173,136,
21,243,87,70,45,141,149,38,112,208,80,241,152,62,65,44,163,82,36,216,136,154,71,107,196,10,109,73,116,251,84,196,90,63,91,199,126,71,65,172,93,79,203,136,238,235,179,70,108,43,57,93,81,196,10,103,37,0,
150,10,27,69,125,130,216,212,164,87,225,244,202,101,93,178,15,25,48,233,91,159,32,86,145,182,42,209,240,239,21,16,43,180,192,140,96,235,62,65,108,75,193,56,10,98,69,143,66,148,190,65,44,73,83,213,82,163,
32,214,15,208,90,45,213,87,105,130,188,113,165,50,106,180,52,129,56,71,254,207,168,190,46,235,74,16,155,162,11,185,226,8,136,117,89,23,183,47,92,73,50,220,117,141,216,12,99,154,150,154,12,98,81,18,7,189,
146,12,119,89,154,128,65,198,83,117,115,207,158,49,16,27,221,119,174,117,72,43,153,125,93,130,88,6,193,151,28,118,142,167,136,223,79,119,23,119,137,18,242,132,81,68,155,7,5,81,121,12,189,115,166,88,204,
211,197,101,93,164,149,206,203,180,174,80,95,187,70,44,64,150,173,182,104,16,171,203,170,138,55,179,41,98,197,232,102,34,195,178,160,131,192,127,81,32,182,218,28,42,93,55,87,141,88,127,144,148,216,24,
200,139,72,234,187,177,94,255,122,121,222,17,161,220,63,38,54,5,142,47,215,42,186,197,176,86,230,103,89,95,238,166,165,25,94,63,136,181,80,187,166,133,49,98,236,121,106,196,174,82,103,152,65,179,229,125,
147,64,172,157,1,201,132,195,244,182,47,128,216,181,36,186,6,115,26,228,111,90,181,128,74,11,197,67,10,188,177,210,4,210,119,51,147,84,64,171,233,14,91,173,14,149,217,188,148,100,130,180,30,49,146,174,
18,205,202,86,106,237,193,194,65,172,174,212,98,198,154,44,55,111,222,17,246,36,120,245,189,44,205,234,223,85,133,183,199,58,81,196,74,195,194,3,239,81,32,50,13,73,105,110,214,134,205,141,160,70,25,155,
151,117,157,27,45,118,80,112,171,151,213,87,226,205,117,69,250,164,95,150,40,202,203,34,221,62,112,66,141,216,253,32,118,210,117,66,207,63,16,27,99,230,149,49,149,29,254,1,90,71,74,19,72,64,226,233,255,
178,46,245,16,34,87,91,202,170,37,25,122,164,224,193,204,51,232,205,60,87,78,197,236,219,17,203,214,151,59,1,177,136,140,204,131,194,3,108,149,138,81,162,73,199,228,137,2,68,107,164,4,83,248,131,73,229,
87,167,225,2,96,60,28,175,177,241,210,4,241,34,69,17,27,207,179,50,104,69,71,79,58,233,164,66,231,82,26,39,205,247,111,202,52,217,53,169,156,7,152,117,37,21,232,230,119,44,85,162,219,230,245,188,238,88,
9,242,145,19,219,99,114,254,101,1,133,101,192,114,80,188,52,136,230,13,234,114,154,211,184,167,196,131,140,119,239,222,189,133,125,186,246,204,249,135,42,100,153,111,153,241,9,26,131,202,206,40,173,130,
79,172,218,96,155,200,180,201,112,30,86,108,95,226,165,152,173,53,211,238,170,229,170,113,176,179,45,200,97,196,16,196,90,30,236,26,104,33,55,222,131,104,73,245,38,149,38,72,16,75,180,243,145,143,124,
116,176,107,215,174,33,33,246,26,130,197,60,242,212,42,230,48,155,34,214,39,17,33,146,114,123,218,45,52,73,17,139,121,186,250,250,87,63,250,137,168,10,112,201,208,32,93,135,44,51,202,66,110,17,183,72,
87,232,122,109,16,75,197,136,121,67,140,208,80,93,9,192,191,219,151,117,181,65,172,171,175,207,56,243,220,193,3,15,110,43,134,125,236,215,62,57,162,136,77,1,189,8,162,49,108,117,16,27,253,253,112,206,
26,66,28,51,106,189,138,216,79,124,242,164,184,77,236,163,69,170,219,6,177,220,134,144,199,240,200,125,155,137,138,216,112,130,95,246,41,174,184,226,138,178,183,210,214,12,47,90,17,75,78,41,38,203,43,
100,39,42,98,195,19,239,48,128,117,89,91,230,221,149,34,150,164,146,182,198,158,77,68,32,187,35,222,122,101,248,67,107,235,242,65,86,85,223,100,198,168,75,189,33,212,165,34,86,88,109,70,230,121,191,49,
16,203,90,75,3,192,145,2,212,174,21,177,140,162,105,200,44,121,172,70,44,55,96,76,217,28,234,171,52,129,1,175,174,80,26,213,68,29,69,147,87,234,121,154,5,44,215,125,125,130,88,135,50,42,163,70,65,172,
31,100,75,245,9,98,41,65,42,163,70,21,177,22,75,11,105,223,32,182,125,30,57,12,92,1,177,172,197,209,105,242,250,4,177,60,251,164,150,26,214,136,21,184,217,164,238,179,52,65,235,218,160,81,16,171,96,17,
233,183,101,166,79,16,59,109,27,164,40,98,69,134,121,99,102,159,32,182,222,113,24,3,177,54,3,69,1,2,180,62,65,108,85,234,98,20,196,70,246,241,202,192,125,187,173,218,150,24,99,170,235,210,4,121,104,186,
26,232,59,38,42,98,105,84,140,45,219,168,90,171,203,203,186,172,121,124,84,214,106,12,231,253,229,49,16,219,228,101,101,99,219,248,18,83,27,240,93,40,98,83,55,92,29,188,127,120,213,210,4,210,41,172,147,
110,192,182,136,214,202,115,84,78,8,57,53,36,112,83,151,67,65,16,145,166,173,127,7,196,20,199,18,137,42,0,162,222,134,200,211,253,106,162,143,132,104,88,61,17,70,229,155,102,187,172,203,31,224,80,90,12,
128,197,59,23,1,98,197,253,173,42,186,243,131,88,70,113,110,198,215,70,20,177,112,163,104,160,197,206,215,7,98,245,185,188,79,186,158,247,123,172,7,196,106,161,9,6,205,150,247,77,2,177,96,133,49,32,195,
209,98,147,106,37,152,173,174,230,52,6,61,198,79,142,33,162,27,48,164,26,67,139,1,177,1,82,151,180,24,225,12,141,138,169,156,199,126,65,87,14,208,67,103,46,230,214,85,102,24,176,81,223,62,96,18,45,20,
196,70,86,251,176,79,44,87,243,200,114,61,249,111,65,98,2,216,252,90,157,135,41,66,248,78,64,44,7,23,97,243,142,112,120,75,233,244,242,107,250,156,4,180,210,126,255,142,223,223,221,155,34,86,144,223,196,
212,171,42,98,99,199,234,149,60,117,108,96,151,163,151,179,60,51,253,82,253,66,242,178,152,73,47,14,163,68,23,130,124,241,24,45,129,136,209,87,255,255,19,182,93,99,156,189,216,233,142,89,12,169,127,103,
110,163,226,13,159,127,32,54,140,58,48,198,209,17,49,190,206,94,173,251,196,104,241,244,87,35,54,118,78,151,248,38,225,135,208,131,140,77,50,75,99,215,58,122,82,106,112,196,179,248,210,4,241,162,165,90,
137,253,24,138,11,167,144,84,67,5,210,112,78,121,34,32,194,80,85,232,237,90,165,47,115,172,73,72,20,41,249,163,225,42,206,139,215,217,20,207,72,253,206,153,171,149,196,31,194,140,67,16,203,35,203,104,
113,79,234,216,212,89,49,40,31,249,34,227,112,81,198,130,109,128,25,154,204,195,99,95,77,137,176,141,131,88,91,172,222,144,186,149,102,175,22,126,77,227,158,150,36,191,167,130,192,29,119,220,81,104,142,
163,152,77,137,167,245,131,88,14,80,211,211,7,187,203,221,39,158,116,89,87,125,159,72,102,66,140,242,1,178,152,173,131,248,150,30,67,96,221,32,214,39,18,178,146,196,161,195,114,194,26,17,105,185,73,138,
88,32,86,213,56,31,228,250,235,175,31,178,207,75,47,189,180,4,140,118,241,77,138,198,176,217,20,177,49,221,111,19,67,17,46,163,194,40,140,44,167,38,195,171,129,216,15,127,228,132,216,220,62,110,112,234,
105,103,14,13,122,232,161,135,202,225,125,49,25,244,163,245,155,242,23,214,196,213,65,108,32,198,243,98,235,117,73,87,73,76,109,121,232,174,121,20,177,194,99,97,242,37,151,126,115,104,212,217,103,159,
83,80,128,176,26,33,38,150,168,100,186,171,131,216,200,100,30,85,35,33,175,13,170,49,227,60,138,88,133,254,126,245,163,39,134,24,250,138,193,165,151,126,99,240,182,183,191,103,68,17,155,88,177,210,185,
76,191,172,11,223,86,39,65,234,222,190,206,108,61,32,246,151,14,62,162,20,250,155,84,154,32,15,30,42,75,215,156,64,154,92,154,192,88,114,115,97,91,85,221,5,136,21,193,26,95,222,179,169,117,61,174,136,
21,21,138,38,141,167,190,74,19,112,198,194,101,9,138,37,42,188,254,17,34,144,176,101,229,200,175,13,29,51,46,7,118,31,32,86,108,159,121,96,116,163,90,28,66,162,229,123,219,37,8,98,106,70,245,121,89,151,
65,95,197,241,163,138,88,97,45,117,134,13,162,62,65,172,5,188,218,60,26,5,177,130,125,94,152,179,236,19,196,218,10,177,216,55,207,40,136,53,53,157,107,119,46,166,79,69,108,235,238,153,81,69,44,104,38,
78,18,106,244,9,98,91,204,115,84,17,107,161,84,61,169,239,203,186,90,103,73,71,65,172,253,55,204,51,201,112,31,165,9,76,172,22,133,89,81,196,134,71,61,155,6,202,14,121,150,37,232,163,52,129,24,191,58,
216,51,122,89,151,236,132,238,0,93,97,148,64,173,175,210,4,85,177,209,201,151,117,57,162,98,3,71,23,118,125,89,151,86,18,68,86,238,96,114,105,2,64,66,246,1,1,138,18,186,4,177,9,210,210,168,169,53,98,165,
81,126,137,158,206,30,111,23,138,216,68,213,78,23,137,64,27,163,30,93,19,196,114,164,186,82,75,229,21,194,162,198,69,128,88,203,138,156,176,174,17,20,26,226,243,72,227,198,180,46,97,241,200,61,14,210,
42,41,17,239,14,45,110,20,196,234,46,34,102,189,80,221,38,45,86,191,45,180,52,155,106,173,222,8,224,144,185,54,9,232,189,226,27,47,224,147,153,145,27,189,172,75,102,93,171,170,155,110,155,187,70,236,253,
50,14,138,32,94,158,81,181,68,119,45,1,106,114,79,31,138,15,172,34,129,28,71,235,7,177,132,205,246,108,12,124,145,162,110,96,16,224,234,255,5,105,245,221,88,53,136,149,124,218,58,107,65,143,141,131,216,
120,193,219,4,97,60,175,45,93,147,64,122,228,123,34,70,11,185,177,151,10,89,70,101,57,95,63,111,95,73,181,48,16,27,62,228,188,32,196,143,234,2,190,76,160,239,43,249,109,62,60,51,163,25,44,43,170,37,222,
141,97,221,128,88,122,148,134,55,13,47,147,79,37,172,241,151,134,250,90,101,192,253,149,38,88,11,196,70,203,157,29,236,253,136,216,109,56,112,63,136,109,50,161,137,116,184,73,221,228,112,221,209,225,120,
241,23,197,163,242,160,154,175,69,17,27,206,240,230,232,26,250,205,31,54,131,249,185,81,196,26,232,138,33,11,113,232,59,121,123,17,164,233,159,27,67,149,95,234,30,196,90,26,76,121,112,226,226,139,47,46,
71,164,156,252,79,230,73,234,228,103,210,127,27,76,124,155,212,173,153,137,59,162,53,63,31,175,177,201,114,182,26,240,159,10,247,227,15,71,64,108,86,254,166,175,130,138,160,161,76,239,25,37,48,204,44,
200,255,147,59,137,48,68,177,28,171,61,191,248,64,170,156,60,216,172,175,235,87,196,234,10,6,193,209,206,41,59,201,13,21,229,109,229,171,129,88,6,49,84,218,6,192,33,130,178,151,120,205,251,194,176,217,
15,94,52,45,228,15,238,231,4,69,161,208,96,150,211,132,163,235,82,136,245,77,211,117,38,148,137,135,223,191,228,146,101,221,231,101,151,93,86,56,124,115,208,126,182,5,57,67,151,24,7,247,137,125,140,23,
23,203,227,230,152,120,138,152,215,82,196,2,177,98,124,173,8,83,102,65,7,119,102,89,188,231,6,177,17,67,61,136,79,169,235,137,125,187,192,164,230,158,12,90,11,196,186,79,196,245,29,31,63,225,196,193,206,
157,59,135,70,65,151,162,89,81,135,214,106,220,200,234,32,214,236,8,163,150,174,185,230,154,162,163,210,50,109,204,56,139,34,22,136,125,219,219,127,121,112,221,117,223,29,26,4,95,219,54,17,193,234,5,194,
230,198,168,181,75,19,208,246,66,140,6,242,70,64,236,91,143,124,79,236,54,92,16,101,92,119,15,182,7,178,254,208,135,142,27,150,38,16,131,101,177,144,44,41,61,181,70,172,208,196,88,48,14,234,2,32,235,173,
17,123,248,17,111,143,186,177,159,10,132,253,190,184,125,101,185,170,110,70,176,220,132,0,50,183,121,167,214,136,229,236,220,82,216,22,160,118,1,98,17,60,187,243,244,10,141,115,157,12,98,137,111,28,23,
48,184,115,218,119,169,136,21,199,19,117,53,213,154,199,75,19,88,66,104,12,92,34,136,232,229,180,191,243,251,247,52,98,154,41,151,219,196,165,55,46,191,161,71,119,25,142,75,113,150,75,92,28,92,24,186,
75,115,148,251,245,253,229,203,116,222,54,20,92,100,141,107,57,102,179,129,57,90,154,64,223,186,2,193,180,53,235,250,168,17,155,53,209,178,144,67,12,250,155,155,112,104,89,17,171,165,232,207,129,216,108,
165,62,46,235,226,76,171,75,192,133,65,236,41,219,246,63,132,130,44,37,152,103,159,32,214,154,90,33,235,21,16,203,137,241,27,118,27,108,179,246,9,98,129,142,86,65,191,21,69,172,21,93,4,160,149,250,4,177,
45,58,188,2,98,35,206,121,216,170,238,40,73,26,196,200,174,107,196,114,9,130,193,134,43,248,186,2,98,99,223,237,102,82,35,142,179,79,16,43,74,173,239,239,11,163,70,75,19,32,44,8,177,238,203,205,233,174,
47,235,178,48,87,173,52,10,98,101,39,234,148,33,44,88,20,195,250,0,177,45,129,252,100,16,107,29,178,72,106,41,221,216,117,105,130,74,162,171,197,38,131,88,77,137,170,248,4,25,248,119,117,89,151,180,172,
30,228,171,130,88,244,132,168,70,48,150,104,17,239,84,170,201,73,33,235,154,18,78,214,50,27,216,214,187,19,78,252,92,17,159,94,244,181,111,14,110,184,241,142,114,3,93,132,228,195,0,111,199,142,29,101,
61,173,143,149,183,46,169,92,187,52,129,92,13,232,23,6,47,186,52,1,118,101,214,85,103,27,150,162,119,62,191,38,136,149,86,137,163,109,62,167,158,96,163,32,22,170,134,26,45,250,68,211,149,166,239,193,153,
65,172,92,207,210,35,69,74,131,214,91,35,86,203,72,173,228,124,173,37,229,190,24,46,7,183,47,90,154,116,206,120,120,18,146,139,208,247,102,163,86,91,15,136,21,243,123,157,214,133,111,102,219,108,121,95,
91,17,27,97,234,125,110,63,193,15,12,126,33,6,135,231,141,12,90,43,188,176,118,18,136,245,97,204,226,26,226,87,142,114,62,169,82,187,70,108,144,148,7,35,29,42,149,78,165,69,41,62,181,79,167,59,140,17,
70,214,84,216,96,22,230,102,209,72,198,100,58,181,48,16,27,3,31,45,217,145,101,87,117,133,53,139,145,105,172,248,158,174,193,99,207,165,69,133,187,83,196,182,65,108,125,135,81,26,153,53,20,154,174,202,
210,4,167,196,108,222,252,156,149,38,136,86,124,88,172,189,191,70,108,116,203,152,34,54,90,230,103,226,251,35,50,221,248,127,154,149,254,65,172,160,44,34,139,93,241,236,107,54,168,159,59,16,27,198,148,
139,82,121,124,7,202,44,182,118,81,185,129,214,150,126,63,32,214,154,197,67,51,130,238,211,121,101,0,36,3,67,68,133,131,205,155,17,43,82,188,248,26,177,32,132,244,218,194,154,167,185,157,87,78,89,83,94,
194,43,98,5,198,68,23,182,224,172,121,54,144,240,2,229,166,163,149,111,221,48,136,245,98,34,68,107,159,11,114,96,34,133,179,18,21,77,2,177,50,163,140,90,243,248,20,99,45,61,186,60,156,238,238,117,129,
216,112,134,219,109,34,122,115,226,83,138,33,217,78,125,180,124,53,50,156,32,150,113,30,132,24,59,85,133,87,196,25,221,154,151,85,142,161,235,137,123,200,49,102,118,107,13,149,36,168,169,219,199,202,103,
5,177,121,242,13,196,133,44,1,217,239,126,247,187,37,107,106,226,169,217,20,177,177,116,220,10,196,43,88,139,83,181,49,227,60,32,246,144,67,223,84,102,166,34,14,73,136,85,155,16,85,84,218,245,181,75,19,
196,38,208,18,200,113,242,201,39,143,25,52,107,105,2,32,22,139,58,242,168,247,12,190,252,149,11,71,174,174,50,12,4,139,160,70,51,51,215,6,177,178,25,226,174,246,253,106,243,42,98,37,22,176,245,125,247,
61,48,52,138,150,29,220,101,148,24,157,81,185,149,59,21,196,90,217,37,12,232,112,187,120,195,122,21,177,95,249,202,175,15,238,186,107,75,112,249,175,150,208,58,245,13,38,17,153,110,198,234,83,65,172,95,
208,223,132,237,181,238,124,163,32,246,200,163,222,61,56,244,176,35,203,217,192,44,123,33,98,213,0,213,193,195,201,32,86,63,155,109,117,37,128,46,65,44,108,205,243,55,134,141,94,214,21,221,86,110,35,39,
83,178,253,97,151,170,47,16,43,82,101,84,115,163,235,40,136,101,148,65,104,144,247,9,98,137,154,37,39,205,216,26,85,196,106,37,213,189,157,99,232,83,17,107,1,207,234,167,97,216,174,17,16,203,98,90,116,
51,164,79,16,203,168,170,58,220,190,152,133,2,198,3,126,218,96,179,245,81,75,116,165,235,170,218,128,246,179,62,32,199,44,143,66,127,117,174,216,170,168,251,211,7,196,32,251,153,40,53,183,79,165,36,107,
84,125,220,68,189,115,85,109,28,233,69,86,92,26,120,252,199,62,53,56,238,195,31,47,180,133,231,62,252,136,119,20,10,195,24,75,201,222,189,79,12,238,187,127,103,217,133,184,234,234,27,6,223,248,230,213,
229,210,65,68,198,145,224,11,46,188,180,84,30,212,66,146,89,199,62,27,145,188,52,108,5,196,70,22,187,75,204,99,173,99,84,159,165,9,84,91,154,8,98,197,218,102,128,38,53,3,251,42,77,0,124,180,78,221,142,
130,88,129,191,0,172,174,151,208,53,136,109,85,42,25,7,177,146,1,188,211,153,3,75,76,31,32,182,117,13,213,100,16,107,213,214,90,54,182,187,6,177,0,90,43,243,153,14,98,181,150,253,18,97,111,151,138,88,
61,82,51,244,85,65,172,136,144,112,93,229,146,204,74,186,0,177,14,89,87,70,173,13,98,109,179,9,196,44,55,89,150,96,81,138,88,236,42,85,68,73,102,102,186,172,11,24,147,86,153,137,245,157,179,2,52,241,208,
145,71,190,173,108,187,18,69,56,165,198,193,114,158,28,163,123,29,220,239,112,219,109,119,150,194,53,118,63,219,168,218,134,118,178,207,136,56,111,157,25,196,146,74,102,213,18,198,108,84,17,75,230,173,
116,176,211,68,21,255,220,61,23,136,141,214,218,142,200,153,141,142,190,101,137,85,48,118,82,37,128,73,220,83,54,109,25,49,245,177,79,113,83,69,248,102,203,251,218,32,54,22,234,221,166,109,222,236,228,
83,234,86,172,0,227,20,210,78,2,177,41,128,39,238,66,248,50,65,96,208,186,110,81,153,112,89,215,173,77,97,199,34,72,150,237,240,254,190,130,176,166,118,202,118,25,106,233,176,158,105,153,108,149,42,107,
89,108,105,2,32,54,41,175,137,160,5,24,198,80,187,19,18,88,181,18,116,121,101,196,200,180,239,164,52,193,36,69,172,86,211,42,12,173,138,212,14,17,181,45,179,231,172,52,65,83,57,119,95,19,210,62,127,46,
235,106,66,217,253,165,9,198,20,177,130,195,120,74,101,9,213,155,195,125,236,107,254,191,159,203,186,226,205,54,199,51,34,253,54,243,0,48,220,65,140,143,71,217,36,50,248,155,60,174,31,16,203,45,144,227,
90,50,172,141,64,44,205,21,14,37,148,182,86,230,37,131,89,189,185,89,120,23,11,98,99,55,106,137,95,162,15,182,204,48,64,38,141,57,200,128,234,139,157,69,172,54,195,45,69,28,169,237,56,254,172,81,147,109,
252,178,174,232,158,7,117,135,149,30,159,242,144,50,57,221,150,188,97,26,247,196,57,115,17,103,36,74,44,11,111,210,169,249,246,251,162,169,203,206,104,180,202,110,45,145,220,19,132,53,118,234,251,220,
215,2,177,198,152,39,197,129,90,17,151,210,122,97,92,18,226,217,64,108,24,117,175,68,209,149,233,121,149,89,187,156,230,90,165,9,234,187,32,50,172,38,131,130,23,117,185,13,204,136,26,82,220,60,162,186,
30,210,225,48,100,120,236,201,246,44,216,97,16,111,20,196,38,33,134,43,111,191,253,246,146,69,223,112,195,13,101,157,156,73,17,27,134,41,0,82,142,147,95,125,245,213,195,123,177,214,83,154,0,136,85,209,
203,161,251,15,127,248,35,131,45,91,182,12,217,39,233,110,38,14,97,152,186,28,39,197,106,48,122,64,44,12,225,24,249,161,29,6,34,242,66,115,190,17,69,44,16,139,49,64,139,63,248,193,125,67,131,182,111,223,
94,116,90,220,198,204,32,86,232,161,52,69,222,139,140,232,173,87,17,139,113,190,255,3,31,14,66,124,127,49,234,190,251,238,43,175,107,102,138,189,170,122,9,12,156,92,154,192,238,184,1,158,245,167,83,128,
186,81,16,123,218,233,103,13,190,245,173,43,3,200,29,95,102,99,222,148,216,42,6,57,25,196,50,138,254,60,7,120,215,151,117,9,171,171,43,100,199,65,44,143,43,215,67,243,116,87,31,32,86,234,133,246,84,97,
243,40,136,149,20,240,214,114,53,179,173,47,69,172,19,111,214,198,102,75,100,20,196,202,241,224,106,173,212,39,136,229,225,43,208,177,82,154,32,154,112,159,98,50,90,169,94,58,92,171,40,235,133,5,225,65,
152,16,46,132,13,225,67,24,17,78,52,179,112,81,226,101,184,17,118,132,31,97,72,56,18,150,132,39,97,74,217,180,11,78,50,53,211,133,21,18,90,81,196,114,249,150,13,167,62,234,211,29,125,129,216,150,0,117,
69,17,43,77,207,149,191,79,69,44,127,85,9,154,69,174,229,90,179,162,178,214,90,88,167,216,168,79,16,107,185,169,4,168,163,165,9,164,230,120,182,173,217,190,64,172,244,190,117,1,220,40,136,53,45,169,98,
179,80,81,31,165,9,180,82,45,135,27,171,17,43,58,16,123,243,178,194,221,174,65,44,112,102,28,87,52,111,50,136,245,11,166,39,169,136,46,236,82,17,43,74,168,118,26,86,87,196,74,153,242,204,94,151,32,86,
54,84,181,18,3,79,153,90,35,214,76,48,0,121,246,188,222,181,11,16,75,235,89,25,181,58,136,197,162,204,8,113,79,173,134,93,36,136,109,221,217,247,236,76,32,22,111,2,248,13,246,46,64,108,93,203,101,230,
203,186,34,237,126,16,228,183,130,91,48,141,45,129,217,122,21,177,89,77,66,114,170,80,105,5,98,231,43,77,16,43,247,110,155,147,201,11,24,200,48,145,169,58,177,179,150,38,240,161,36,162,90,71,52,80,105,
63,103,203,251,38,213,136,37,85,146,118,163,117,137,18,125,98,89,115,158,16,106,147,97,36,153,200,134,51,246,119,169,107,105,118,212,13,112,249,222,236,103,70,39,128,216,114,223,140,23,78,206,137,170,
216,140,182,135,147,53,19,180,32,223,3,206,90,71,181,136,192,45,13,105,50,23,55,245,220,18,127,63,127,153,195,202,176,151,196,139,110,146,151,73,191,76,225,212,16,232,10,45,135,4,227,236,186,87,57,130,
140,185,179,58,64,61,237,59,7,177,90,128,71,182,179,233,171,24,191,125,97,151,22,142,231,185,1,177,49,238,246,53,168,103,40,169,220,175,136,141,238,120,126,214,136,13,195,38,94,214,21,179,234,233,72,197,
119,5,232,119,120,176,223,26,177,205,0,46,11,42,239,76,211,192,119,201,136,232,26,172,109,198,152,162,201,6,122,60,221,213,136,53,187,204,52,91,30,60,180,109,51,132,15,123,162,225,203,154,159,150,35,106,
126,217,111,39,138,216,240,71,91,130,196,44,217,243,37,5,145,23,50,4,247,84,128,84,52,33,204,177,128,167,36,5,196,176,20,113,166,254,174,190,233,48,90,141,51,158,223,121,198,31,145,109,191,51,214,190,
109,222,28,133,161,251,4,62,24,1,57,66,133,50,159,213,64,172,208,199,18,36,196,22,14,85,165,121,214,7,98,195,168,251,173,97,87,94,121,101,49,130,208,171,93,189,114,45,16,43,244,201,98,239,16,182,50,7,
54,49,27,227,102,59,162,18,134,12,171,42,89,50,112,79,26,189,73,220,115,94,16,155,219,188,140,51,222,140,201,88,31,183,53,93,185,54,136,53,171,40,44,220,252,212,46,56,58,175,34,214,248,202,176,90,218,
102,252,217,161,128,190,155,146,172,91,98,50,108,138,217,250,194,73,69,177,94,22,198,20,16,43,213,130,150,9,144,215,91,154,32,65,44,196,232,160,189,214,214,234,219,182,45,95,226,69,186,75,255,16,239,183,
54,136,213,74,242,251,111,125,235,91,101,19,104,189,151,117,37,136,117,244,238,200,163,222,21,39,193,191,50,34,215,125,224,129,7,234,251,28,166,215,136,101,16,16,171,149,72,225,106,84,189,30,69,172,86,
122,237,235,162,228,69,85,238,87,43,221,116,211,77,69,2,94,159,132,156,10,98,197,66,170,63,80,230,251,186,40,16,251,249,83,190,80,138,57,48,8,121,246,218,252,88,34,160,6,47,142,131,88,183,16,18,89,57,
183,0,114,44,178,52,193,97,111,58,50,28,237,169,49,192,79,139,236,251,216,225,165,188,92,68,93,45,103,236,178,46,231,170,156,216,118,100,128,63,234,2,196,190,229,173,81,15,166,42,77,192,15,214,231,144,
149,143,138,135,227,62,224,39,28,142,230,119,156,109,79,50,220,7,136,181,108,181,32,199,10,136,13,33,243,46,80,195,82,82,223,243,216,71,105,2,137,70,21,199,175,128,216,0,102,79,203,207,172,115,125,131,
88,139,118,117,9,197,104,141,88,11,102,94,179,216,103,105,2,217,179,195,65,85,107,173,128,88,129,90,174,252,125,131,216,20,52,135,107,120,58,140,91,1,177,34,72,89,45,108,221,39,136,197,209,43,58,60,84,
89,151,10,205,154,79,23,166,84,183,143,26,177,12,194,61,51,206,138,150,26,22,6,225,23,138,114,223,244,52,11,45,194,125,128,88,146,184,102,81,46,99,106,34,136,21,127,51,198,50,208,53,136,213,74,173,157,
134,209,203,186,194,74,21,221,182,243,174,150,26,227,170,235,203,186,44,49,117,37,93,239,63,241,178,46,11,163,77,109,71,10,24,213,37,136,109,233,134,87,7,177,126,217,250,151,242,144,46,46,235,50,203,91,
103,220,87,7,177,33,57,90,202,176,165,43,16,219,58,62,176,54,136,141,197,121,11,150,142,115,155,29,9,99,23,81,154,32,203,250,86,167,138,102,191,172,43,6,224,54,217,140,45,122,222,214,73,32,187,15,27,5,
177,121,18,178,90,82,230,3,177,242,62,11,37,24,139,121,58,9,100,209,38,166,153,85,17,155,154,97,45,206,73,34,195,85,80,55,91,222,55,9,196,250,84,60,46,227,180,156,65,234,205,82,91,208,86,196,250,25,35,
156,133,129,28,57,71,173,110,250,87,131,91,190,183,113,16,203,56,209,41,227,232,160,178,128,173,180,60,33,44,81,13,1,4,227,73,44,83,105,198,152,134,16,47,197,215,45,11,7,177,154,95,94,8,194,146,112,107,
17,110,132,33,205,61,88,67,99,90,48,182,219,26,177,184,19,3,48,242,108,17,173,210,82,81,91,207,250,7,177,97,208,205,193,3,118,133,65,226,160,60,33,36,164,125,254,40,98,155,0,109,191,34,118,85,69,172,238,
139,129,253,84,124,125,32,66,160,155,162,107,207,137,127,247,119,89,151,1,156,74,158,220,149,178,31,99,159,208,94,13,7,11,43,198,212,239,14,196,134,127,58,199,253,125,124,148,25,39,66,149,104,216,48,226,
163,208,25,169,190,51,130,234,198,90,208,133,61,40,177,192,49,207,26,55,168,105,83,124,221,88,141,216,88,106,238,136,104,97,73,250,133,254,98,158,30,26,80,201,43,125,157,239,219,12,151,238,231,81,60,241,
152,244,137,67,173,46,144,216,184,34,54,62,209,189,162,133,235,174,187,174,24,128,234,17,17,162,112,24,104,125,47,205,164,187,177,114,71,53,241,117,149,116,174,31,196,218,161,82,253,148,216,107,18,247,
156,5,196,106,49,204,32,241,53,195,26,5,246,108,11,178,190,142,7,227,22,82,148,69,247,242,203,47,47,116,120,35,138,88,204,211,224,23,186,88,208,45,75,205,4,153,173,52,65,24,4,190,151,26,81,103,159,125,
118,25,59,109,131,102,173,17,171,52,65,13,98,125,200,60,206,98,172,53,69,70,239,152,9,196,58,78,78,117,166,98,101,251,224,253,172,53,98,219,32,54,165,4,186,60,235,16,11,123,154,13,204,115,86,85,196,106,
37,159,198,161,123,92,114,17,32,86,45,180,159,255,249,215,20,215,129,20,42,228,0,53,218,193,104,246,150,31,158,90,154,192,47,216,178,64,134,105,207,179,206,240,70,20,177,64,236,47,190,254,208,112,35,199,
151,43,243,178,194,132,175,138,36,167,70,97,42,136,85,34,197,110,148,20,171,46,124,188,81,69,236,155,223,242,142,208,15,47,75,117,243,81,78,3,72,89,181,52,65,56,184,37,106,88,15,71,184,72,16,75,109,125,
221,117,55,20,131,126,240,131,31,20,28,110,242,8,157,171,218,252,163,138,216,88,163,206,177,100,80,233,215,181,170,23,121,89,215,7,66,98,249,153,207,158,18,151,58,31,95,220,131,113,203,127,73,74,50,187,
25,1,177,129,23,111,226,220,210,73,38,247,116,105,160,83,141,110,130,86,144,111,223,51,147,139,215,210,123,210,121,210,119,146,123,43,90,107,167,225,224,55,190,185,20,249,67,133,21,250,243,115,167,39,
95,255,250,55,20,163,90,23,117,49,110,84,17,43,191,227,189,83,162,171,149,250,0,177,201,20,154,69,123,5,196,138,131,172,250,22,90,7,163,251,44,77,208,58,101,59,10,98,205,60,83,223,32,239,19,196,90,172,
171,140,217,191,87,64,172,31,88,155,20,154,233,19,196,218,87,172,174,124,17,185,174,128,88,169,146,212,92,136,210,39,136,21,115,85,241,214,168,34,86,75,161,180,156,39,163,250,2,177,122,135,84,160,137,
241,111,138,175,192,240,242,165,130,70,63,17,150,98,255,198,84,31,32,150,243,164,109,208,75,140,138,148,254,28,247,40,51,234,197,77,214,241,172,216,155,183,229,167,250,0,177,6,121,117,83,249,100,16,235,
23,210,45,116,13,98,201,80,184,161,106,230,77,6,177,206,57,112,11,226,29,144,172,43,16,155,215,121,214,174,96,170,34,54,56,212,195,250,88,28,37,132,237,74,17,75,46,215,170,239,57,29,196,6,71,63,199,142,
146,13,103,11,52,81,243,162,75,19,32,122,245,34,44,177,93,83,17,27,159,224,14,228,205,248,178,65,104,134,232,202,69,129,216,150,40,126,118,16,43,155,145,10,81,88,128,97,124,137,221,241,246,221,88,86,251,
195,14,59,162,156,238,16,89,42,34,115,204,177,31,136,69,253,176,145,162,163,169,136,213,109,45,126,62,63,136,21,21,66,138,90,205,106,158,18,164,121,65,108,86,147,104,85,248,158,45,239,155,6,98,25,102,
102,106,122,113,23,116,45,30,154,164,136,213,221,214,51,48,214,62,143,214,161,5,109,237,126,206,151,33,79,83,196,154,190,14,64,203,154,117,37,0,43,72,35,65,49,41,236,72,81,250,48,158,48,85,247,123,18,
194,102,237,178,248,255,59,22,10,98,45,69,102,142,55,181,149,161,75,141,187,172,0,160,187,197,220,166,59,99,124,245,189,230,114,146,135,227,111,207,9,54,177,169,93,103,120,230,43,210,243,23,73,136,36,
138,177,62,109,206,210,4,106,233,49,174,54,38,7,112,5,239,251,7,177,193,2,206,9,67,172,234,194,13,113,208,126,16,219,46,26,249,163,165,136,141,46,28,169,17,27,19,227,240,248,158,173,96,151,124,189,104,
181,107,167,234,159,77,189,130,170,114,13,14,77,147,198,109,142,49,127,70,124,125,136,172,201,76,52,184,77,117,179,12,111,50,240,185,4,212,70,150,18,241,153,131,250,143,196,223,44,94,17,27,111,124,74,
156,117,223,43,228,224,56,45,27,124,21,71,233,255,45,218,162,85,202,88,25,54,45,159,4,4,143,16,247,55,51,114,103,24,71,243,183,41,158,141,129,216,120,129,91,92,178,37,161,112,119,136,77,111,218,5,162,
10,132,207,87,144,86,184,35,22,3,99,229,142,121,108,74,40,196,167,53,71,156,22,3,98,237,211,1,175,184,210,36,204,184,22,247,20,93,88,208,69,153,213,134,210,124,203,76,180,204,176,104,187,177,226,5,157,
115,215,66,237,171,168,214,50,40,227,124,93,108,29,84,119,168,210,34,204,182,32,235,235,120,10,136,229,12,109,54,146,196,33,110,27,1,177,137,174,125,64,105,28,81,198,92,53,98,141,33,51,75,115,147,233,
226,85,53,247,220,136,34,22,136,181,15,173,90,133,210,25,13,48,91,187,70,172,240,212,39,49,147,220,225,110,75,99,35,165,9,82,17,155,97,53,195,68,24,34,11,252,66,129,100,107,234,84,16,27,45,180,211,39,
48,165,145,97,179,108,81,32,214,229,20,12,51,240,13,5,100,135,129,122,36,124,220,222,169,32,214,192,182,207,130,12,35,185,121,126,116,163,32,22,68,67,136,223,252,230,183,148,30,112,59,29,142,122,219,109,
183,21,105,184,241,27,99,236,140,232,206,151,114,218,254,179,150,29,24,160,253,17,74,124,71,201,235,22,50,134,54,10,98,17,226,183,30,249,174,168,233,241,173,177,215,50,9,154,8,227,161,200,106,14,12,201,
0,123,14,120,113,244,231,225,66,93,202,231,20,11,46,18,196,26,87,164,186,53,29,198,210,53,128,157,7,70,41,123,49,166,136,133,23,137,79,219,119,244,45,234,178,174,147,78,62,125,112,207,61,91,7,79,60,241,
68,185,31,222,74,128,131,217,184,108,90,202,114,52,94,154,128,94,220,1,138,174,64,236,199,126,237,83,131,79,126,234,164,80,73,190,109,152,120,76,85,196,134,165,15,152,117,78,121,144,85,246,89,154,160,
37,91,90,1,177,226,32,190,201,10,111,246,245,9,98,91,138,179,81,16,43,22,146,199,9,57,250,4,177,45,177,224,40,136,69,211,100,178,160,89,159,32,214,121,210,28,232,122,44,158,21,16,107,29,66,214,120,243,
62,65,172,229,166,50,106,20,196,242,230,100,142,118,176,250,42,77,32,114,133,1,170,157,172,209,210,4,185,215,7,196,102,33,200,174,107,196,50,74,136,157,137,236,152,34,214,241,113,155,206,186,47,201,48,
202,114,240,27,15,45,133,136,108,4,229,233,14,203,134,91,12,109,8,29,121,212,123,75,1,16,165,51,21,12,217,179,103,185,80,200,180,199,54,112,94,159,46,214,7,64,178,251,130,202,28,30,192,14,24,94,94,251,
226,121,72,6,226,244,79,93,59,138,97,140,177,43,197,152,124,84,38,177,67,101,103,74,69,18,70,169,100,98,71,107,154,65,79,62,249,228,72,125,79,130,176,196,213,241,254,143,140,41,98,163,9,207,224,51,232,
15,44,146,93,130,88,179,156,104,176,46,96,52,21,196,70,206,182,87,222,230,19,136,163,186,2,177,94,191,82,115,232,190,157,241,190,155,195,61,148,132,181,174,212,245,50,17,160,251,138,156,50,210,149,196,
53,90,44,163,70,103,170,116,165,49,102,139,95,215,125,240,67,31,43,69,100,116,159,162,52,215,126,231,150,178,105,185,111,223,242,184,210,101,134,67,202,43,205,240,86,85,230,181,65,172,24,93,182,171,197,
168,128,68,135,139,2,177,200,112,125,201,188,1,62,115,105,2,217,12,223,1,142,49,78,176,159,135,7,235,187,177,230,5,177,117,157,132,102,198,205,15,98,253,161,214,98,152,96,223,4,144,174,207,11,98,49,80,
247,54,84,245,97,141,163,217,242,190,73,32,214,11,233,78,208,66,10,158,59,229,211,64,108,125,63,86,162,106,51,173,154,250,12,154,47,67,158,6,98,179,43,189,169,150,242,21,1,182,160,202,227,196,69,169,64,
35,205,181,77,39,133,242,160,203,77,119,45,188,52,193,78,227,76,119,162,196,100,112,90,208,83,227,32,45,82,63,249,243,48,106,111,112,210,83,58,1,177,1,97,31,241,201,243,82,174,172,11,107,237,204,18,115,
9,99,227,43,161,224,67,225,24,207,232,227,178,174,195,5,249,241,134,195,34,163,77,247,236,47,77,80,123,254,31,109,16,27,93,218,189,34,54,6,247,89,49,213,31,179,62,242,91,50,160,172,10,96,76,229,153,42,
51,174,129,99,207,198,87,149,189,191,24,63,223,28,15,160,59,60,4,61,141,22,175,74,135,227,5,64,180,77,241,148,106,37,14,250,80,188,130,104,246,153,229,136,146,12,158,94,96,104,3,169,190,56,85,29,43,43,
0,178,18,179,116,119,243,58,94,111,227,32,54,98,173,37,219,99,234,194,170,94,201,32,161,13,232,42,205,247,255,158,212,202,96,164,254,157,183,28,136,195,45,85,174,162,181,224,199,179,254,210,4,150,3,62,
39,47,112,166,238,144,176,38,179,170,207,151,78,82,196,106,61,145,171,21,160,85,55,97,190,101,38,12,25,130,88,227,132,222,83,119,17,124,213,23,117,205,98,80,29,86,75,16,108,68,54,215,161,207,190,32,55,
99,168,128,88,75,135,245,140,49,48,99,125,206,125,222,203,186,140,57,11,185,67,65,21,83,207,133,217,251,205,86,154,64,236,3,162,161,195,53,247,220,40,136,53,190,224,31,51,116,230,26,177,49,197,151,4,117,
6,180,131,247,93,128,88,33,177,15,45,113,176,62,174,90,35,54,22,219,221,146,5,80,203,56,170,7,245,122,46,235,146,43,46,223,118,255,75,229,150,224,4,177,92,135,22,51,163,155,237,145,201,165,9,194,201,125,
81,94,143,230,213,87,225,109,180,70,172,60,49,65,172,196,35,171,54,99,96,2,69,106,145,102,3,252,172,49,16,27,218,131,221,142,4,152,246,237,2,32,139,0,177,90,204,133,93,38,140,99,7,223,254,246,183,35,155,
222,83,104,177,214,10,131,246,142,128,216,102,141,122,150,195,171,171,232,46,186,70,236,151,190,116,193,224,254,251,87,238,94,147,130,217,247,17,52,134,13,227,32,214,238,57,71,199,168,36,195,139,84,196,
254,202,175,124,52,10,34,223,59,146,210,239,222,189,187,76,168,38,22,27,7,177,22,83,235,22,0,209,5,136,125,215,187,223,23,247,134,92,91,140,250,222,247,110,47,254,207,153,9,78,53,141,106,130,198,162,201,
43,245,60,253,32,75,142,119,5,98,63,126,194,103,7,159,59,233,244,224,170,191,82,146,92,179,176,133,23,71,65,172,80,195,138,238,248,64,159,32,182,186,107,59,117,13,5,47,82,160,150,38,132,21,133,38,125,
130,216,246,121,228,198,158,2,62,127,168,165,140,37,133,28,250,4,177,211,144,117,25,83,164,178,252,20,100,221,39,136,109,221,114,63,10,98,185,4,51,143,115,235,19,196,138,102,39,205,190,162,136,149,94,
171,51,37,130,236,19,196,214,215,154,141,129,216,88,177,151,132,42,150,152,62,65,108,85,1,117,188,70,108,180,212,94,1,24,131,140,169,174,21,177,121,123,143,30,106,186,111,247,24,136,141,52,232,44,161,
175,204,35,51,148,174,20,177,12,34,7,168,199,83,68,10,95,156,90,35,214,249,40,122,4,127,164,197,186,40,77,0,51,122,143,10,162,237,158,10,98,229,101,210,32,137,38,173,157,160,172,11,16,235,245,171,25,183,
180,166,34,86,94,230,19,224,79,196,235,210,35,209,232,34,20,177,194,96,31,182,50,104,62,16,235,15,37,144,164,254,180,42,12,219,72,105,2,136,177,165,25,46,217,204,92,151,117,197,31,220,47,17,101,152,1,
153,226,134,245,128,88,137,136,215,169,91,200,235,199,163,18,192,234,119,131,76,83,196,218,95,182,153,196,160,118,209,90,212,56,43,1,216,12,104,131,88,240,182,58,88,145,134,205,151,33,79,2,177,238,155,
161,251,181,78,57,198,171,43,232,24,132,29,190,199,191,81,148,145,17,168,12,224,231,164,73,146,207,38,212,237,6,196,70,100,186,219,32,21,228,219,159,171,97,171,168,181,254,127,28,52,133,169,213,180,239,
166,52,65,188,241,23,163,213,32,157,82,93,55,245,192,185,179,233,107,202,114,27,42,188,20,191,179,55,28,227,89,125,128,216,67,247,131,216,181,4,162,209,117,207,95,16,27,99,230,204,24,196,215,52,153,16,
126,158,129,127,169,44,33,70,139,167,123,16,27,111,114,74,12,240,29,49,253,151,178,130,55,16,150,117,174,115,215,60,39,128,9,17,207,67,254,46,158,205,241,44,30,196,186,86,65,190,230,194,37,20,70,154,15,
198,34,124,22,111,203,80,86,249,182,139,206,143,49,26,143,138,37,230,177,48,250,204,48,108,83,60,27,7,177,62,49,131,44,202,168,158,114,4,53,247,180,4,73,96,129,87,145,43,99,97,36,79,18,192,230,74,244,
197,128,88,99,5,47,247,198,206,186,51,168,190,62,125,53,1,170,4,36,75,70,193,62,150,171,74,223,57,123,5,28,11,100,179,80,222,207,27,11,198,100,57,95,253,234,87,135,135,199,18,128,204,170,136,205,152,204,
225,160,6,251,172,15,196,250,99,27,140,78,220,106,161,141,130,88,161,143,48,216,248,106,21,115,152,13,196,98,145,154,29,166,33,200,89,20,136,205,137,160,245,171,3,98,171,43,98,163,117,206,212,196,116,
158,196,167,228,112,53,59,55,128,55,90,154,192,236,21,163,11,137,243,142,191,85,65,108,12,196,199,18,228,155,234,139,184,172,107,18,136,133,156,242,46,210,198,183,77,175,17,43,70,50,160,219,186,243,69,
41,98,107,16,43,91,18,127,217,147,110,74,226,79,188,172,235,33,123,48,112,117,123,187,99,81,32,150,36,69,202,150,133,145,181,24,241,169,216,61,186,80,188,181,162,136,109,214,168,103,69,143,120,66,45,64,
253,232,71,143,95,85,15,69,47,69,55,229,192,61,29,21,61,21,93,21,125,21,84,189,92,196,225,176,162,191,82,138,245,216,99,223,87,142,30,152,213,26,128,180,92,139,129,252,99,138,88,78,82,94,239,151,200,149,
186,168,17,251,201,79,157,60,248,222,237,119,20,81,78,106,172,30,125,244,209,178,60,121,255,232,198,51,135,53,98,173,234,250,149,81,246,244,24,213,69,141,216,43,175,188,102,172,213,13,141,138,123,138,
58,70,65,172,108,132,90,191,43,16,251,213,139,46,29,60,253,244,211,81,58,122,123,89,212,179,20,66,85,162,124,20,196,138,169,25,195,37,116,5,98,63,243,217,211,6,167,127,225,220,184,184,107,89,247,39,53,
107,93,40,63,170,136,149,169,0,251,78,125,244,9,98,107,104,214,4,138,192,240,50,136,133,129,212,224,0,220,251,4,177,173,59,252,70,21,177,40,173,229,197,32,239,19,196,146,218,53,45,228,235,10,136,21,107,
231,204,51,174,250,2,177,188,58,143,94,25,181,162,136,141,36,243,76,142,83,84,153,181,18,250,40,77,192,168,90,86,57,6,98,67,136,181,148,106,197,140,24,187,86,196,74,62,170,216,106,28,196,134,149,59,168,
195,0,138,52,170,203,203,186,4,124,226,246,234,4,247,67,19,107,196,178,90,28,141,172,164,50,86,196,104,117,183,15,156,18,93,107,217,27,15,121,75,41,34,115,244,49,31,42,151,6,210,121,186,116,240,142,59,
183,78,44,74,179,99,199,142,17,189,39,85,108,173,27,94,245,178,46,134,137,12,25,163,197,186,0,177,12,178,95,93,93,216,245,216,170,32,86,58,197,237,3,93,54,161,97,235,69,42,98,49,43,76,171,190,103,45,6,
251,153,225,26,70,175,137,109,142,26,13,47,212,53,69,225,64,142,45,133,205,139,2,177,144,120,125,128,122,46,69,172,244,42,177,162,196,51,183,238,215,171,136,133,39,91,154,225,245,129,88,116,78,74,36,18,
213,82,181,65,179,94,214,69,13,235,160,97,75,124,58,123,222,55,9,196,230,29,88,132,166,234,69,213,247,96,217,3,246,248,190,167,13,98,25,100,249,106,21,110,88,140,34,214,160,183,73,73,33,141,176,24,111,
214,44,223,7,97,129,12,254,141,84,132,33,60,53,63,100,149,104,233,133,159,93,216,101,93,88,84,0,142,199,180,154,153,105,230,228,131,51,100,109,4,95,141,27,198,86,121,93,174,109,221,128,216,166,52,1,222,
132,59,213,23,225,14,37,187,105,108,83,49,16,101,113,241,243,41,207,7,16,123,77,116,211,153,177,151,119,104,172,14,7,218,50,203,163,2,107,225,203,145,227,4,179,252,178,223,113,82,49,186,200,65,69,219,
188,130,124,27,227,2,69,17,163,175,251,107,196,142,212,136,141,105,126,122,172,93,91,98,236,184,247,234,185,5,177,97,192,246,152,129,75,114,53,140,193,218,136,162,216,131,33,9,111,213,0,42,197,144,227,
233,14,196,154,113,220,130,168,20,72,35,206,202,227,156,88,39,175,159,197,68,185,5,6,50,180,201,235,220,255,183,184,210,4,92,0,150,4,128,48,228,194,11,47,44,169,88,50,7,113,125,134,209,254,45,27,98,36,
141,3,67,179,18,120,184,133,165,232,238,187,227,245,54,166,136,213,58,130,123,203,9,76,68,247,84,223,218,211,230,158,98,176,124,68,24,136,160,5,93,46,105,137,106,42,115,205,183,223,23,70,36,136,45,250,
23,53,206,157,77,62,253,244,211,139,128,176,38,50,243,128,88,99,143,81,66,151,134,224,73,167,214,222,25,141,95,26,150,38,176,169,141,188,80,117,56,243,231,20,119,141,170,231,189,172,43,207,117,209,16,
27,99,205,34,61,91,141,216,56,138,114,183,214,208,85,212,176,212,133,109,136,182,222,26,177,34,88,11,181,56,109,102,16,107,118,200,237,21,0,197,21,24,183,168,203,186,50,164,198,162,108,231,230,253,163,
38,209,170,32,214,226,137,132,0,103,109,131,22,169,136,21,119,9,119,170,75,149,166,131,88,251,47,102,140,49,212,110,161,141,92,214,213,86,196,138,96,197,92,54,147,114,171,119,106,141,88,83,223,57,228,
246,134,80,87,32,86,220,222,136,154,197,92,227,151,117,233,91,65,26,249,44,100,189,232,210,4,64,172,195,138,135,189,233,240,225,201,55,26,6,221,232,125,167,130,88,73,2,98,219,221,101,93,239,26,188,231,
189,239,47,123,132,8,177,30,161,3,164,16,105,46,182,60,125,4,196,242,220,206,114,2,177,6,121,23,32,246,248,143,125,50,74,92,220,94,152,103,125,2,151,115,246,254,49,190,182,52,49,218,178,34,86,24,139,178,
156,122,234,169,195,194,29,139,174,17,123,201,37,223,24,60,243,204,51,99,132,216,165,59,77,130,42,162,96,207,50,94,196,60,213,2,226,159,186,2,177,23,92,120,241,224,145,71,30,29,60,246,216,99,131,219,111,
191,189,16,98,231,158,171,66,14,163,32,22,212,32,195,214,223,93,129,216,51,207,250,114,236,142,93,20,189,113,70,89,164,185,31,249,164,174,171,158,21,16,203,211,218,2,169,207,191,28,251,190,15,14,62,245,
233,83,202,145,222,19,78,252,92,65,62,78,103,219,238,120,127,20,55,118,228,247,240,35,222,81,182,56,160,32,227,100,239,222,39,202,169,237,27,111,186,179,108,143,56,197,13,17,93,244,181,111,150,35,193,
23,92,120,105,169,60,152,201,107,139,14,175,128,216,152,146,219,241,40,98,230,36,195,194,143,35,143,122,71,188,192,9,229,172,49,35,176,40,134,216,127,201,235,21,49,43,134,49,138,65,195,1,188,52,40,231,
220,25,136,89,221,112,227,29,229,140,242,149,87,125,183,148,66,228,213,33,161,44,197,218,180,212,10,136,141,40,113,139,124,223,52,101,84,95,32,22,94,108,221,248,180,2,98,99,42,158,238,240,41,118,212,167,
34,182,61,158,198,64,44,225,150,89,160,165,248,168,62,106,196,138,22,170,1,62,14,98,173,61,220,61,133,170,248,137,81,93,131,88,0,164,50,106,114,141,88,191,128,166,168,197,145,50,221,46,65,108,93,185,114,
85,16,203,179,90,3,45,150,186,178,43,16,107,144,87,135,239,87,191,172,75,180,32,42,196,212,121,246,69,131,88,110,128,65,245,217,134,153,20,177,186,209,108,52,59,120,222,60,67,133,123,106,185,35,143,124,
91,217,15,86,128,157,31,227,96,249,41,142,145,31,82,232,253,182,219,238,44,114,20,25,76,214,74,48,187,185,158,90,132,26,51,238,238,232,198,77,17,187,143,95,168,219,16,226,97,54,35,84,205,251,252,146,12,
175,87,17,139,238,17,97,88,95,235,219,195,226,195,207,173,136,125,192,139,0,174,102,33,209,233,122,64,172,174,162,119,200,123,105,244,66,85,53,126,237,188,111,18,136,181,80,107,49,205,110,10,219,34,201,
117,75,183,232,30,201,198,36,16,75,192,92,139,79,43,254,57,95,134,220,86,196,198,50,112,183,178,4,64,133,37,33,197,167,198,133,252,141,161,222,92,90,47,167,203,2,50,180,6,213,134,117,250,163,197,129,88,
121,153,244,75,200,154,165,6,56,90,134,202,128,24,106,198,58,221,40,3,230,128,91,101,89,25,213,41,136,21,29,22,16,219,132,176,197,208,52,210,215,106,48,151,59,140,158,115,16,27,6,109,143,148,124,139,69,
126,63,136,141,46,121,113,60,171,210,225,230,231,126,239,5,179,146,231,177,219,200,215,250,195,120,241,177,203,186,98,48,95,27,51,115,87,48,167,125,205,56,26,169,17,27,127,211,143,34,214,192,53,208,1,
53,119,211,224,86,46,164,112,64,195,230,80,214,230,104,124,82,63,32,150,239,177,73,132,12,3,33,226,122,209,170,136,66,210,97,41,66,142,109,42,113,25,149,100,119,241,32,86,11,32,190,121,90,210,209,96,173,
36,197,79,204,152,123,206,98,49,11,119,110,94,107,65,97,138,123,255,162,149,55,86,154,32,186,96,171,133,89,198,33,190,66,245,24,131,125,202,15,167,85,2,128,180,179,176,3,106,156,225,143,40,3,67,141,214,
182,233,52,191,34,54,28,222,118,193,158,79,13,83,163,194,192,89,173,249,156,5,196,90,204,243,17,197,106,61,132,56,90,205,216,92,123,65,142,95,42,161,75,204,166,135,84,22,84,65,151,49,244,158,237,226,13,
235,5,177,214,72,91,255,205,50,180,213,251,121,223,137,21,154,227,7,195,221,118,73,131,155,232,25,214,198,140,27,41,77,144,220,83,86,92,29,220,88,187,70,172,122,27,102,149,83,182,237,251,213,22,161,136,
101,152,160,79,148,49,147,34,214,74,238,124,31,28,148,55,72,119,81,35,150,79,19,81,52,162,211,18,65,76,173,17,107,213,23,31,1,103,237,226,13,139,4,177,124,153,24,76,107,85,21,232,39,42,98,183,139,151,
4,105,56,85,238,78,45,170,70,44,13,113,173,136,5,56,200,75,248,191,4,102,19,21,177,118,200,49,72,10,198,62,64,172,195,99,60,127,211,90,163,151,117,69,183,149,75,112,104,91,84,150,48,158,186,80,196,42,
99,39,53,203,154,104,226,250,172,247,98,208,143,213,136,181,87,98,102,208,161,119,5,98,177,173,227,62,252,107,113,10,224,189,133,87,152,205,220,143,228,164,137,50,174,141,6,90,81,196,130,236,124,19,140,
220,149,34,150,118,248,214,91,111,143,98,14,247,68,133,137,251,7,138,56,60,245,212,83,229,232,122,115,11,240,240,134,251,2,98,165,78,54,138,44,176,93,129,216,75,46,253,230,224,241,199,199,171,88,234,157,
6,158,237,11,91,150,75,19,8,252,45,178,180,188,105,144,80,228,248,143,157,88,36,221,179,62,82,247,213,158,139,47,249,102,220,217,246,72,209,15,115,49,110,168,187,229,150,91,74,207,84,183,211,45,131,216,
224,6,251,20,39,182,172,116,13,98,65,217,47,156,113,118,137,193,140,93,139,116,181,155,181,2,98,99,213,222,69,213,99,209,205,203,75,251,0,177,194,161,22,56,91,1,177,98,109,238,64,10,110,76,245,9,98,91,
234,179,209,26,177,66,215,100,82,125,93,214,101,47,40,147,218,38,177,61,121,236,178,46,233,182,89,152,21,182,186,86,196,210,57,84,70,77,6,177,102,161,110,92,190,103,45,10,173,71,212,216,85,141,88,74,143,
150,152,121,58,136,21,21,58,131,101,208,119,89,154,64,148,80,3,180,85,65,172,214,146,66,153,178,150,128,174,64,172,202,150,85,215,173,13,98,181,22,222,105,108,45,250,178,174,188,249,169,42,157,50,91,105,
2,56,199,32,228,80,211,168,69,40,98,147,12,75,86,83,44,49,179,34,86,190,7,242,235,62,121,155,124,109,163,138,88,27,6,66,96,126,169,202,154,31,154,171,52,65,68,132,219,185,8,190,4,106,22,236,103,213,211,
89,107,196,194,141,92,12,255,103,193,205,122,159,13,103,152,45,239,107,131,216,136,161,31,50,109,117,167,208,70,218,238,211,50,148,129,121,129,233,36,16,43,170,228,94,56,101,70,212,79,12,112,249,222,236,
25,242,164,210,4,52,3,22,77,164,133,113,56,167,44,23,138,54,78,44,79,140,204,186,177,242,58,63,175,42,160,38,166,94,60,136,53,133,189,17,242,162,5,242,110,44,187,19,166,184,22,244,239,90,151,215,121,105,
130,70,17,91,248,148,55,203,130,181,90,207,152,105,148,24,197,255,212,124,234,57,3,177,12,140,201,177,47,190,10,105,175,181,203,185,31,196,70,75,60,127,65,108,24,87,74,252,198,122,249,148,52,41,198,213,
211,241,255,15,54,223,239,175,52,65,188,225,112,199,193,246,135,89,71,64,131,242,137,183,157,233,170,98,238,126,64,172,153,101,150,241,212,50,32,82,75,216,72,98,153,75,18,103,202,135,181,170,197,47,22,
196,42,106,4,68,112,136,150,14,172,65,174,198,24,184,58,175,255,201,164,67,139,229,89,8,45,87,237,132,110,236,178,174,112,128,71,199,139,109,179,180,136,18,29,70,36,32,68,100,108,195,174,6,98,243,246,
176,60,65,98,49,183,25,41,78,107,90,111,190,253,190,232,158,162,136,13,207,252,8,25,28,35,116,17,94,110,236,228,189,70,179,214,136,205,178,4,152,20,35,61,77,137,130,71,194,167,29,237,253,86,189,126,49,
126,97,184,135,204,32,215,249,76,227,158,235,5,177,182,124,69,29,186,52,134,197,182,120,207,217,64,172,157,208,43,175,188,178,40,169,219,220,115,17,32,22,33,22,87,205,92,35,214,186,230,136,155,75,150,
196,233,139,86,196,166,156,32,15,176,186,80,105,77,69,172,41,111,252,144,165,105,165,46,64,172,129,159,231,143,155,214,90,29,196,218,242,208,74,226,243,250,58,179,69,131,88,20,175,170,197,47,36,154,12,
98,181,18,31,195,7,117,13,98,109,60,181,206,32,79,86,196,114,142,206,34,243,67,217,74,139,44,77,208,86,196,58,176,129,75,101,236,53,177,52,1,95,162,149,210,49,46,178,70,44,13,223,36,16,75,219,144,73,233,
24,136,37,33,209,74,150,14,173,212,133,34,22,136,37,218,113,125,71,94,78,1,17,72,223,155,214,26,162,160,151,199,55,30,52,35,28,142,230,6,186,2,177,84,144,31,63,225,51,33,152,62,118,136,49,189,111,165,
96,92,41,77,16,187,150,79,231,214,107,189,184,158,112,226,167,139,252,136,62,147,78,19,26,164,219,164,223,164,227,36,151,76,25,37,46,234,182,30,242,74,45,66,130,73,15,74,23,74,31,74,39,122,218,233,231,
4,227,252,222,224,251,223,191,107,240,221,239,126,119,112,221,117,215,13,174,184,226,138,226,11,155,155,128,221,129,187,12,98,109,18,89,86,172,109,93,130,216,111,95,121,237,96,239,227,143,143,105,135,
157,195,169,142,147,47,131,88,145,35,229,23,195,24,213,85,105,130,111,95,249,157,208,13,239,29,51,138,179,110,196,58,34,215,149,203,186,132,21,124,83,215,165,9,174,191,254,214,193,169,167,125,161,244,
138,199,248,181,214,54,51,80,40,93,84,214,182,29,174,22,231,104,45,251,50,125,129,88,1,99,171,154,238,40,136,101,41,15,75,170,219,23,136,117,148,188,214,88,141,41,98,69,7,126,1,246,203,141,156,46,65,172,
86,106,93,44,63,25,196,106,45,191,104,171,191,235,26,177,66,236,150,152,121,58,136,53,232,180,148,46,236,18,196,218,225,200,229,197,215,85,65,172,172,133,235,151,40,116,117,89,151,131,60,85,21,221,178,
177,189,106,105,2,122,20,123,187,186,48,201,240,34,75,19,224,89,98,244,10,17,205,118,89,151,125,221,172,90,178,104,16,171,149,234,35,4,51,131,216,32,118,219,192,47,83,22,223,148,129,8,107,214,171,136,
205,171,168,228,126,210,171,186,208,218,204,32,54,22,199,163,35,224,123,36,121,39,120,33,166,206,42,0,179,130,88,114,2,221,133,55,228,85,84,21,25,126,68,126,185,102,222,55,73,17,107,7,34,57,39,230,153,
156,83,154,36,53,247,117,18,136,213,202,148,180,226,240,188,9,170,217,81,55,158,182,197,35,17,253,241,73,178,187,169,151,224,52,153,178,68,145,8,171,232,58,81,22,249,154,230,183,60,24,31,90,35,37,186,
186,199,73,52,45,155,69,27,90,152,122,225,151,117,161,38,197,183,120,67,254,12,242,97,156,88,27,132,21,172,213,194,229,218,15,153,246,116,231,157,92,214,21,47,62,228,83,90,140,60,215,242,228,105,181,138,
15,80,248,212,115,1,98,31,140,150,123,186,185,170,69,228,88,138,140,238,7,177,209,10,207,223,26,177,97,220,216,93,107,49,118,158,14,61,223,163,209,117,215,55,63,239,87,17,107,102,201,110,205,64,37,165,
69,171,52,43,190,242,101,141,127,42,210,239,120,186,45,77,192,24,190,74,148,106,187,77,157,79,89,53,73,138,196,35,15,214,195,73,142,18,84,251,52,139,1,177,241,130,71,132,23,191,43,118,173,150,172,133,
150,12,82,16,55,131,57,49,233,220,59,238,64,42,87,167,102,226,48,235,37,163,181,154,214,108,118,180,54,6,98,173,75,177,94,237,146,110,97,85,138,250,185,215,22,47,215,34,12,147,253,76,19,160,98,157,178,
95,235,164,80,133,144,181,58,98,176,62,16,27,70,221,79,120,69,128,170,85,24,80,83,189,89,65,172,152,44,35,13,107,37,153,110,195,165,102,43,218,30,134,12,65,172,186,6,215,94,123,109,81,195,78,18,160,206,
11,98,25,87,47,224,77,13,170,93,177,72,31,225,125,215,84,196,26,204,102,209,197,23,95,60,177,133,54,82,154,0,178,22,85,56,219,39,113,8,247,113,87,116,235,166,152,201,227,199,158,194,16,37,157,29,69,121,
22,154,209,74,6,113,87,32,86,224,215,204,204,181,65,44,131,12,204,171,174,186,106,164,120,195,34,75,19,228,4,168,160,198,234,32,214,88,114,244,215,76,235,18,196,98,158,164,1,25,222,76,5,177,102,133,1,
233,208,61,1,78,10,80,187,40,77,192,40,51,177,138,185,198,65,172,157,42,82,52,46,192,150,89,31,138,88,219,115,25,131,25,203,99,32,214,110,21,127,164,42,115,226,234,174,65,172,112,90,220,95,117,225,201,
195,210,4,17,160,93,207,193,97,158,152,81,23,151,117,189,253,29,71,23,236,88,131,88,177,125,30,102,29,3,177,177,4,236,225,36,149,186,200,162,181,90,105,209,165,9,128,88,199,57,15,59,236,136,161,35,213,
133,90,170,58,136,184,92,154,192,134,52,110,64,97,93,175,101,139,6,177,14,238,159,116,242,23,6,31,58,238,163,195,11,159,117,97,181,115,58,90,154,64,201,9,110,160,75,16,123,193,133,151,12,238,188,243,238,
193,29,119,124,127,88,8,153,131,166,180,174,102,225,74,105,2,254,201,120,210,125,93,129,216,91,110,185,61,10,33,255,112,12,196,194,229,77,158,56,10,98,173,67,250,151,97,93,213,136,189,237,123,119,23,221,
112,93,127,195,191,157,203,105,194,154,61,209,98,43,32,214,122,40,32,75,253,240,178,172,242,144,50,91,204,26,179,71,253,78,123,44,106,85,59,30,160,86,130,2,14,126,78,200,12,246,183,223,208,255,239,123,
102,16,66,230,39,202,125,238,91,127,176,125,240,201,79,126,178,248,66,65,34,131,196,90,77,247,137,237,139,30,221,57,61,73,64,185,233,201,210,162,11,251,168,17,75,140,88,111,24,77,4,177,182,214,240,40,
20,175,143,210,4,173,107,242,150,168,136,34,206,162,163,41,121,220,129,77,214,81,144,143,46,236,26,196,146,199,213,133,70,189,255,196,26,177,186,208,33,104,10,13,70,117,9,98,91,187,162,171,131,88,228,
196,0,204,75,33,186,80,196,170,162,91,249,165,181,65,108,44,144,75,198,85,93,43,97,209,32,214,105,238,202,168,181,65,108,132,15,119,89,36,49,167,44,69,176,168,210,4,242,70,80,191,146,50,205,118,89,87,
240,206,35,28,47,112,218,40,171,191,9,200,54,170,136,77,137,110,171,149,230,46,77,112,191,79,36,69,183,4,24,152,22,79,217,200,60,32,214,223,192,142,198,145,245,181,34,125,179,229,125,211,46,235,18,86,
240,97,217,114,14,119,73,203,167,129,88,93,68,180,204,8,233,190,40,64,230,82,1,125,10,199,141,41,98,133,170,214,38,49,188,83,178,102,168,49,87,147,97,106,88,59,96,32,44,35,180,72,94,73,229,43,50,76,215,
18,198,221,21,235,172,4,116,76,162,52,211,137,109,153,107,188,216,166,44,77,96,198,120,51,124,93,183,106,9,7,164,121,102,198,230,145,203,52,166,98,230,101,218,119,10,98,105,137,233,135,25,231,171,55,79,
70,94,37,2,207,13,136,69,236,162,43,246,132,33,226,32,70,12,207,182,239,7,177,205,2,254,252,87,196,54,221,86,186,47,198,214,131,49,206,110,13,216,127,94,252,127,255,32,214,56,50,227,184,0,39,72,236,86,
165,90,17,127,114,166,185,9,137,22,11,98,195,29,188,42,62,245,25,166,52,207,14,174,162,193,156,162,195,20,82,51,135,202,228,140,18,0,248,145,136,71,216,227,46,153,42,102,90,12,136,13,67,14,14,79,190,37,
222,124,137,94,65,94,8,186,66,211,210,49,114,16,255,86,231,78,180,42,148,206,76,136,177,60,63,135,106,29,109,78,19,109,28,196,134,81,247,138,163,41,97,5,126,12,178,19,79,154,155,165,16,243,20,110,125,
88,49,21,32,186,146,167,207,77,200,74,24,177,126,16,43,23,60,255,252,243,75,246,209,150,87,206,10,98,83,71,44,252,69,117,42,113,224,108,11,178,101,69,151,105,33,131,24,128,189,232,162,139,74,235,180,201,
240,60,32,150,97,194,31,172,211,166,37,227,154,152,106,182,26,177,97,76,185,145,21,26,194,61,93,111,221,54,104,189,32,86,216,147,53,133,236,160,154,177,177,150,110,137,21,225,85,107,130,88,139,43,177,
160,171,94,140,157,26,51,110,244,178,46,81,172,86,19,244,9,248,154,104,226,140,48,108,244,202,132,104,29,53,91,54,155,246,6,35,17,159,74,0,102,86,87,138,88,241,88,117,227,197,234,32,214,20,6,98,169,24,
179,206,112,23,151,117,49,170,190,42,104,42,136,165,220,184,236,178,203,202,142,148,45,144,174,65,44,153,64,149,213,140,131,216,152,13,165,204,5,16,91,75,116,187,84,196,138,223,171,147,218,163,32,54,98,
239,243,204,134,107,174,185,166,236,223,101,11,117,13,98,49,79,65,98,182,214,136,34,54,52,6,183,26,220,159,255,252,231,135,6,245,161,136,205,114,42,85,23,174,40,98,133,27,166,167,11,4,107,79,221,53,136,
229,76,91,76,97,244,178,46,192,44,43,51,119,85,154,0,136,85,232,239,221,239,57,102,152,9,229,41,219,38,253,26,5,177,206,136,90,223,204,186,174,106,196,162,125,231,156,123,65,208,187,79,21,95,104,76,73,
82,91,117,250,87,64,44,237,138,21,221,86,108,87,32,246,219,223,254,206,224,190,251,30,12,253,240,109,165,118,12,14,198,57,87,119,99,141,94,214,37,130,164,0,83,238,34,75,178,46,186,70,236,253,247,111,31,
171,59,172,228,133,53,177,233,190,81,69,108,2,51,155,216,89,212,127,209,32,118,231,174,61,19,139,33,115,67,222,63,12,187,53,190,142,130,88,80,148,55,239,82,17,187,101,203,214,66,134,173,173,234,28,223,
124,243,205,229,196,110,19,239,159,55,86,154,128,81,194,94,116,184,107,16,139,220,192,75,21,29,158,12,98,197,80,66,94,97,75,215,32,214,172,147,124,212,107,223,212,203,186,68,4,12,203,91,158,186,168,17,
75,217,209,42,224,176,58,136,229,204,132,193,73,134,187,0,177,216,85,171,216,232,116,69,108,68,129,103,224,157,14,10,2,98,186,112,209,165,9,100,71,36,115,85,183,173,14,98,37,158,145,109,108,145,197,200,
122,181,26,163,180,214,162,74,19,248,192,149,65,179,129,216,204,102,236,106,229,237,78,212,22,57,91,214,123,89,151,152,92,70,221,146,123,207,15,98,125,34,241,142,131,62,162,8,235,85,10,29,102,189,172,
43,37,186,4,24,173,217,54,91,222,55,13,196,74,32,165,67,198,27,163,180,86,158,16,106,43,98,179,156,133,41,175,251,237,122,122,240,135,10,196,206,151,33,79,42,77,16,45,166,164,64,97,2,198,154,55,100,28,
131,240,2,173,65,195,46,54,2,105,253,14,24,226,105,110,29,24,106,167,98,141,221,18,93,56,255,125,51,149,97,99,32,214,152,128,175,29,105,209,165,90,64,23,91,208,243,97,136,127,231,215,102,215,115,71,24,
123,70,100,199,175,138,177,186,190,43,210,211,48,153,171,68,49,62,245,230,44,77,16,241,207,146,80,71,142,152,111,156,68,184,234,162,254,65,108,72,116,207,11,227,172,234,194,141,177,219,158,246,131,216,
104,149,31,45,69,108,211,141,69,208,28,25,210,169,241,181,91,16,27,99,228,160,120,19,64,117,59,89,83,46,21,25,244,103,149,102,51,19,127,18,90,199,179,20,179,112,103,243,119,0,10,137,221,80,123,55,173,
88,238,84,229,126,85,154,124,83,188,208,73,177,30,238,17,114,88,80,65,144,244,79,156,165,148,76,96,72,251,137,133,74,8,112,45,17,44,135,219,201,101,93,145,109,44,129,174,74,177,130,174,8,176,115,202,212,
30,224,172,147,110,98,48,70,33,53,18,15,134,58,61,98,67,201,206,86,150,50,140,15,184,177,26,177,241,2,247,218,20,18,216,3,31,169,133,73,102,149,7,94,107,209,78,74,82,82,108,33,186,208,194,156,109,181,
1,57,223,50,19,134,228,101,93,22,204,18,83,57,62,169,69,214,163,136,245,247,90,12,246,97,152,177,215,56,216,217,22,228,48,98,8,98,109,149,105,118,213,0,200,137,116,75,182,80,98,234,183,190,245,200,34,
252,162,179,82,232,207,5,39,114,69,169,89,38,30,25,86,67,138,66,23,120,160,34,45,179,131,88,45,196,32,131,21,67,175,185,231,70,74,19,136,201,208,97,173,86,29,99,89,187,70,172,136,192,56,50,139,136,6,165,
92,139,4,177,198,24,60,96,2,56,236,227,46,192,53,75,19,216,188,150,44,170,42,97,128,103,121,139,69,43,98,115,139,68,90,23,139,250,158,240,119,7,69,4,242,162,225,69,149,209,58,47,229,224,108,6,241,55,74,
19,180,91,104,209,165,9,4,138,12,75,184,49,17,196,134,149,59,57,61,133,64,248,161,122,80,119,165,136,21,189,106,169,102,117,24,7,177,178,140,27,111,188,113,88,8,164,15,69,172,52,46,211,118,242,128,17,
69,172,197,211,118,190,18,209,117,137,148,174,65,108,109,148,214,26,43,77,192,40,93,87,223,243,200,15,185,231,131,150,142,166,142,182,110,146,230,142,22,207,129,123,218,60,26,61,23,190,209,236,209,238,
209,240,209,242,77,82,196,50,170,149,225,172,128,216,176,178,92,185,97,11,13,220,232,178,52,65,91,17,91,85,45,49,182,70,65,44,163,180,82,125,220,164,11,69,108,27,196,182,14,72,143,130,88,185,157,117,202,
204,235,18,196,130,177,199,29,119,252,240,244,100,75,13,171,181,86,64,172,129,38,159,179,206,117,5,98,179,24,196,199,79,248,228,240,244,164,216,44,3,198,38,114,93,41,77,144,135,162,237,154,119,165,136,
189,243,206,123,163,214,240,163,129,21,175,41,81,135,53,21,182,174,140,90,169,17,43,166,246,3,50,93,107,94,87,165,9,246,62,254,228,200,236,85,226,215,117,179,85,212,48,94,154,192,153,3,187,14,105,148,
240,227,136,55,103,233,147,183,15,222,120,200,91,74,157,141,215,253,194,33,69,162,107,218,155,242,234,109,24,43,234,114,60,27,215,153,77,114,27,207,198,15,158,121,230,217,177,159,65,214,54,171,52,74,196,
254,167,142,128,88,106,11,1,63,57,37,163,186,2,177,202,180,56,215,117,195,13,55,148,146,209,151,92,114,73,145,7,51,106,162,34,54,55,141,132,178,93,130,88,107,158,201,148,215,198,54,99,106,231,68,16,75,
159,194,87,25,232,93,130,88,148,198,254,113,125,60,120,98,105,130,88,255,14,10,26,178,7,135,2,184,186,82,196,230,45,24,117,69,165,104,169,213,75,19,4,36,93,98,152,131,241,8,113,93,158,224,213,175,126,
77,41,81,175,16,145,186,63,214,58,135,41,108,153,25,232,252,144,59,250,172,149,251,246,45,15,234,39,159,124,178,124,192,188,27,75,11,85,199,157,10,153,153,233,178,46,12,202,38,55,72,38,249,92,20,136,245,
90,94,179,242,75,243,129,88,127,72,161,104,77,52,6,50,85,90,47,136,205,163,154,181,65,242,202,153,75,19,196,47,19,134,222,143,192,73,34,24,70,213,170,59,165,74,243,40,98,19,196,26,14,45,160,63,91,222,
55,13,196,102,119,18,55,48,40,141,106,43,98,179,36,171,173,125,3,154,65,90,200,97,252,54,170,110,62,248,198,74,19,128,20,226,105,169,145,22,19,244,123,67,146,92,43,1,113,189,3,20,126,71,193,6,64,86,34,
98,210,84,6,45,190,52,129,244,203,155,8,204,48,79,107,86,13,93,97,161,164,194,124,144,39,175,15,138,150,217,211,153,34,54,222,120,103,106,131,25,209,150,228,86,199,122,139,86,56,140,41,165,9,98,41,57,
40,210,169,151,213,87,226,173,139,79,213,127,36,81,12,3,94,26,131,254,64,89,71,67,234,74,145,209,38,14,218,175,136,29,161,125,209,42,63,186,32,54,140,239,167,70,108,200,153,206,141,49,245,104,56,212,37,
46,34,207,229,229,21,122,6,125,222,239,103,166,198,204,91,138,177,184,59,126,247,252,48,114,115,60,139,5,177,86,114,190,137,194,3,115,112,250,86,48,40,79,132,119,108,86,138,195,236,57,251,153,236,26,142,
244,179,230,62,63,71,8,28,190,222,20,207,250,246,102,226,15,97,198,82,215,197,21,178,2,64,229,227,200,226,228,134,210,123,226,47,250,24,148,207,247,136,82,115,183,94,82,128,224,136,203,132,66,205,181,
83,27,191,172,203,130,105,1,181,188,32,195,244,85,109,204,56,203,221,88,50,22,245,95,42,29,241,250,65,44,71,104,169,112,237,180,132,130,60,160,70,68,179,24,148,87,156,101,225,218,74,170,59,219,130,172,
175,155,46,187,151,65,82,106,84,207,81,96,18,166,141,128,88,155,150,226,41,18,18,134,53,132,120,62,16,139,134,96,231,186,205,128,173,13,90,175,34,86,44,134,16,91,184,179,150,144,49,187,102,105,2,96,84,
189,49,227,231,140,51,206,24,185,206,108,17,151,117,9,123,180,24,193,132,176,88,101,176,85,65,108,228,126,187,132,190,140,145,255,213,99,104,145,32,54,97,191,174,108,142,145,79,86,196,70,88,113,190,89,
70,62,4,114,212,99,168,11,69,172,0,49,111,251,165,184,14,7,124,174,5,191,217,168,90,62,158,25,94,122,55,33,188,49,212,222,238,232,10,196,234,74,81,105,115,24,232,209,177,203,186,40,98,9,152,107,17,51,
24,219,165,34,86,166,76,60,223,36,19,163,32,214,226,41,65,224,141,109,153,245,165,136,101,84,30,184,103,216,24,136,21,69,50,198,126,76,95,165,9,140,171,214,141,228,163,32,86,188,109,241,100,84,159,32,
182,133,23,71,65,172,240,131,215,117,172,187,79,69,108,235,184,230,40,136,85,159,76,31,219,150,237,19,196,182,232,240,40,136,229,246,133,27,93,131,88,213,121,63,252,145,143,13,65,44,252,84,165,242,163,
138,88,63,112,74,136,75,232,10,196,34,50,215,126,231,166,192,145,31,42,201,172,140,186,165,60,27,5,177,104,136,133,87,100,217,21,136,125,228,145,189,165,78,177,133,94,196,42,90,109,25,53,10,98,165,214,
98,38,134,9,107,23,95,154,96,20,196,110,219,182,173,28,129,161,73,207,238,27,43,77,16,60,106,137,190,64,124,157,6,117,81,35,182,77,142,237,228,55,172,97,92,17,27,221,247,40,71,198,171,215,59,229,139,190,
172,203,236,86,28,41,141,51,177,154,150,218,61,6,98,35,142,58,215,14,56,120,193,53,24,87,93,149,38,48,145,68,33,102,59,249,38,163,194,37,157,31,41,219,114,148,208,132,10,195,19,70,176,162,200,144,175,
234,170,70,172,131,213,144,81,5,67,118,173,90,35,54,60,251,18,235,161,69,240,43,175,13,54,83,22,1,98,133,195,45,238,57,219,101,93,154,51,89,148,214,146,187,45,66,17,107,204,182,46,125,155,15,196,202,54,
0,50,221,137,115,50,110,35,53,98,109,219,181,46,16,52,150,230,7,177,12,243,201,104,160,24,165,181,214,3,98,141,161,170,90,124,206,182,217,242,190,105,32,150,150,83,246,97,214,204,75,134,113,209,186,118,
75,51,253,231,203,144,39,41,98,233,58,65,10,43,186,79,109,176,138,131,28,233,149,145,216,169,247,213,46,152,244,12,152,5,108,169,100,185,153,244,216,112,163,58,195,11,83,196,186,90,10,79,199,3,32,31,144,
85,12,150,224,213,215,20,163,166,60,87,52,91,95,145,222,9,136,141,228,226,252,120,195,221,18,200,166,112,255,240,186,131,44,77,160,85,82,47,204,189,196,7,120,52,62,200,185,225,251,54,119,14,98,5,249,97,
192,126,16,187,90,145,228,231,39,136,141,113,117,84,3,90,167,118,95,243,243,110,21,177,241,38,155,227,57,37,6,237,206,40,4,178,100,79,70,38,194,227,231,210,145,64,44,119,171,76,136,248,155,237,49,254,
78,111,254,126,241,32,22,136,128,23,69,168,10,253,97,156,121,10,87,84,97,247,211,35,235,77,95,198,111,217,77,15,119,242,120,24,251,133,48,110,19,56,55,109,11,100,24,186,76,172,188,93,129,216,120,145,34,
30,132,137,220,46,77,15,83,159,192,77,205,149,77,113,70,50,86,12,78,33,235,171,167,217,200,94,28,136,181,9,237,230,39,247,99,193,141,243,114,79,81,134,238,182,254,53,71,124,231,91,102,162,85,134,138,88,
99,67,121,75,159,52,111,41,207,83,220,179,150,38,72,125,131,8,195,250,89,17,226,217,22,228,26,196,138,16,96,102,226,83,178,56,66,230,141,128,88,81,134,157,82,227,107,93,32,214,26,39,163,33,62,117,182,
83,134,83,107,245,214,11,98,181,86,30,161,171,14,175,174,14,98,195,255,124,129,204,27,228,144,147,33,48,118,20,22,169,136,69,136,209,59,31,60,235,114,172,10,98,35,252,120,130,116,40,75,104,214,152,113,
145,32,214,14,4,195,68,182,77,139,77,6,177,28,156,152,72,41,86,83,185,93,69,119,209,138,88,254,76,220,197,213,52,17,198,228,203,186,228,245,138,101,215,215,153,153,101,93,128,88,17,44,153,0,189,130,45,
186,48,108,231,24,136,205,26,177,186,47,181,158,125,128,88,59,92,18,10,57,96,208,196,163,226,223,234,16,173,212,94,212,82,92,64,14,236,174,21,177,50,113,72,40,215,206,49,16,203,211,138,169,121,237,62,
47,235,226,187,170,157,173,81,16,43,140,149,121,112,148,125,130,88,221,87,145,188,81,16,11,199,112,152,164,186,125,130,216,154,163,135,113,163,32,150,181,42,234,230,33,28,154,57,135,41,8,149,103,125,8,
187,102,121,106,69,172,180,172,106,169,81,16,43,175,75,146,215,165,34,182,13,98,29,91,153,10,98,109,58,146,59,90,86,186,4,177,87,94,245,221,193,49,199,126,96,8,98,43,229,254,136,202,186,92,214,69,106,
196,40,51,175,43,16,75,171,119,207,189,15,198,123,28,86,200,176,153,39,156,174,90,106,20,196,138,183,161,62,131,188,75,16,75,59,108,83,10,194,100,84,117,165,153,5,250,228,161,34,54,82,240,163,144,21,214,
11,103,251,0,177,74,231,123,47,187,249,77,75,141,131,216,8,91,118,130,17,164,216,62,133,71,24,187,72,16,251,204,51,207,140,104,135,31,125,244,209,186,132,202,228,26,177,60,107,94,34,161,27,23,13,98,197,
249,226,180,189,123,151,175,203,187,227,142,59,138,108,78,75,69,152,124,250,84,16,11,43,154,17,188,187,110,92,116,105,2,175,103,187,133,168,89,248,210,192,216,39,86,5,177,210,41,45,6,116,101,23,46,178,
70,172,101,69,220,86,149,39,88,138,22,18,241,142,22,197,106,176,245,11,155,68,209,73,197,50,0,101,50,186,208,179,81,16,43,118,50,94,147,155,167,27,152,249,178,174,248,131,34,43,241,2,214,38,190,107,163,
53,98,81,191,150,102,120,253,32,22,232,55,14,242,102,148,121,65,172,177,41,186,172,100,36,27,7,177,66,85,45,198,152,60,105,150,117,126,242,30,237,73,151,117,165,170,218,130,59,225,90,170,249,50,228,73,
32,54,4,89,101,23,66,90,228,171,44,196,2,238,177,160,2,180,192,189,174,150,215,217,169,112,218,209,116,111,233,133,149,56,88,12,136,141,52,232,11,97,216,19,168,139,71,234,221,126,116,143,125,104,113,153,
109,14,31,160,57,230,155,221,213,77,141,216,200,56,112,166,237,141,210,117,184,225,147,173,145,151,118,49,152,145,141,15,218,249,92,92,214,53,85,17,27,148,229,168,232,210,3,121,234,136,255,203,209,203,
89,158,153,126,169,126,161,88,134,94,32,13,106,110,166,16,246,168,145,239,168,146,51,84,190,250,255,159,176,218,43,192,30,254,237,5,179,24,50,241,18,156,89,255,48,222,240,249,9,98,195,176,67,99,80,159,
21,201,227,45,49,206,220,189,62,86,89,34,190,215,143,34,54,222,168,148,38,176,62,242,81,56,167,240,70,133,102,148,206,64,111,85,101,46,183,210,53,127,183,57,190,46,30,196,90,175,188,57,178,39,116,206,
91,160,50,132,150,230,91,223,120,112,11,58,183,192,113,114,35,49,99,119,134,193,167,53,235,235,98,64,44,100,195,59,75,195,136,29,178,72,26,67,178,20,65,102,65,140,4,99,253,191,15,96,5,16,213,70,116,75,
115,188,53,12,155,191,174,75,252,209,143,199,227,68,200,189,186,131,140,73,200,129,123,74,195,196,67,41,252,106,11,80,37,29,153,120,136,199,100,70,66,32,93,205,219,55,39,139,230,91,102,194,144,4,177,252,
79,41,94,165,155,96,70,132,175,38,50,179,42,98,243,92,151,229,199,18,213,120,122,175,191,246,237,153,241,75,67,69,172,88,10,167,34,28,100,16,185,73,77,134,231,169,17,155,97,181,240,71,8,83,69,12,179,41,
98,35,44,221,170,6,16,157,39,189,103,22,55,94,132,34,86,4,43,173,178,253,139,242,52,82,221,213,65,108,252,226,105,182,45,104,61,201,63,218,85,116,23,161,136,69,136,157,124,108,232,93,57,32,182,42,136,
53,109,13,78,39,183,117,93,151,32,182,185,107,45,23,237,233,53,98,97,62,83,158,16,190,75,16,155,119,251,1,253,201,211,167,214,136,229,228,12,234,182,0,181,43,16,43,48,84,162,172,217,175,25,47,77,160,111,
205,56,158,154,227,235,163,52,1,135,10,105,54,162,230,241,203,186,248,36,53,49,112,116,94,56,43,146,220,249,253,123,154,83,141,83,142,242,198,17,95,71,125,129,53,71,127,29,1,118,20,88,37,0,71,131,29,17,
118,84,216,247,157,158,124,211,225,111,27,17,92,16,53,11,171,155,130,90,103,53,33,209,242,189,237,140,34,128,128,172,251,4,177,146,14,98,11,239,31,6,201,57,75,149,74,65,217,3,44,229,16,141,41,131,188,
175,203,186,24,85,169,60,132,65,203,151,117,137,135,244,43,92,109,73,233,19,196,234,190,234,190,181,81,16,107,141,83,46,218,120,234,83,17,219,170,210,60,10,98,121,89,5,218,45,184,125,130,216,150,206,115,
69,17,27,41,210,54,134,88,239,210,32,225,199,162,107,196,82,196,214,32,22,244,104,233,170,86,20,177,177,14,221,226,124,131,241,148,69,107,187,186,172,75,161,145,26,196,182,14,184,174,128,216,8,37,206,
146,126,219,238,210,125,93,129,88,167,184,111,191,253,174,18,133,102,33,163,186,174,222,196,203,186,172,69,56,148,129,222,85,105,2,69,188,197,101,140,34,168,224,159,154,231,217,137,165,9,128,50,204,211,
31,117,1,98,83,5,171,86,2,73,48,53,90,101,212,244,203,186,240,78,209,161,46,20,198,46,170,70,236,227,143,63,62,66,134,253,191,248,61,141,154,88,154,32,126,40,47,219,193,179,99,234,120,148,110,92,20,136,
165,252,16,18,109,221,186,117,104,156,15,158,16,100,77,16,107,70,200,219,24,181,72,69,172,4,150,198,129,147,86,21,69,0,160,165,98,246,159,182,38,136,101,61,255,1,13,90,156,23,117,89,151,52,13,96,3,222,
18,31,197,215,173,209,51,155,234,67,211,237,227,188,35,55,94,248,36,210,40,227,106,35,32,214,135,51,78,193,180,86,105,205,185,21,177,15,248,68,68,51,226,245,182,68,119,214,26,177,89,35,175,42,248,159,
131,123,182,188,111,146,34,86,88,129,93,226,3,84,23,86,117,139,40,163,164,75,198,201,52,16,107,149,168,54,132,22,87,35,54,6,225,86,91,244,210,109,49,188,217,105,34,216,18,227,62,24,44,29,167,148,101,28,
67,61,90,184,85,252,163,164,83,11,3,177,49,24,209,146,157,121,234,31,112,197,53,25,135,51,248,190,60,206,140,181,234,107,221,9,221,213,13,136,205,26,177,62,113,58,61,51,149,1,140,212,50,12,174,100,72,
133,79,61,231,32,54,102,215,182,232,226,91,226,57,203,90,182,31,196,70,183,96,88,83,233,112,243,115,98,154,238,232,112,188,184,19,35,42,15,186,254,147,210,181,8,154,99,102,62,28,51,113,159,49,22,93,247,
148,204,200,247,155,159,247,163,136,53,112,13,116,131,154,143,66,245,100,63,128,24,103,91,223,4,221,76,136,238,65,172,22,177,100,192,66,216,167,125,97,2,11,97,180,80,71,116,33,169,69,236,82,44,223,204,
84,5,108,23,87,154,32,93,128,69,90,212,0,21,73,50,156,112,75,189,167,136,162,102,157,22,113,70,202,146,0,92,219,179,78,240,134,207,187,61,94,111,99,32,214,167,100,140,248,157,152,130,232,75,235,228,101,
93,147,184,39,217,64,198,249,190,102,120,45,154,37,202,8,227,30,137,215,157,253,78,200,248,229,2,98,93,247,34,65,245,6,168,30,99,4,106,53,145,153,21,196,230,62,180,128,209,186,105,219,54,38,133,26,216,
179,131,216,8,89,30,209,18,186,136,0,21,72,107,215,74,88,15,136,101,148,117,209,34,221,196,83,179,129,216,216,33,184,93,51,99,230,186,73,26,191,168,203,186,140,51,33,144,234,205,213,6,229,218,165,9,84,
34,129,131,116,89,155,123,46,2,196,50,76,140,53,51,136,21,9,24,212,234,195,242,63,93,129,88,147,70,200,83,181,214,234,151,117,81,169,170,185,209,214,12,47,90,17,107,192,107,173,140,213,167,130,88,91,99,
102,27,29,74,215,53,98,109,32,136,189,42,121,192,100,16,43,96,51,227,136,151,251,0,177,178,27,75,83,22,223,30,169,17,219,44,158,69,203,68,97,77,208,220,7,136,181,118,186,235,65,128,40,88,28,83,196,242,
220,98,108,130,102,135,42,250,184,172,43,75,94,0,43,205,216,26,85,196,106,70,247,23,113,5,125,42,98,53,132,201,165,81,162,165,30,142,175,203,32,54,254,231,41,93,199,21,244,93,35,150,96,62,141,10,91,196,
101,165,242,41,61,65,113,1,102,29,223,212,39,136,133,24,91,167,109,217,115,192,43,34,127,219,167,133,224,197,250,106,78,245,206,85,191,85,91,241,132,19,63,55,248,181,143,127,38,164,187,159,26,28,247,225,
143,151,221,5,181,23,15,63,34,246,138,163,214,48,121,46,254,228,64,189,58,196,118,33,174,186,250,134,193,55,190,121,245,224,146,75,175,24,92,244,181,111,150,218,140,109,69,172,141,240,234,60,178,200,117,
185,70,108,236,138,63,44,153,180,225,216,39,136,133,25,91,245,18,70,75,19,96,158,208,34,230,217,165,34,182,6,177,78,29,181,194,231,81,69,172,148,156,170,140,32,180,155,26,177,131,178,241,116,219,109,119,
14,65,108,11,47,142,42,98,163,11,101,29,101,107,141,151,229,167,186,2,177,86,139,60,221,150,26,207,38,134,159,12,98,253,144,107,224,167,178,36,235,34,21,177,251,246,237,43,27,156,98,42,193,94,75,97,61,
29,196,166,98,81,55,46,82,17,155,100,88,180,129,125,218,178,173,165,113,171,130,88,173,37,54,79,108,189,8,16,171,182,240,253,247,223,63,66,135,185,159,74,172,179,115,77,16,155,197,107,25,182,136,203,186,
76,28,126,144,180,32,91,76,52,130,208,68,35,204,94,154,128,139,200,46,92,20,136,229,110,228,140,234,110,168,214,165,87,34,108,185,125,102,16,107,95,215,96,20,236,165,68,119,35,165,9,56,103,31,212,152,
173,14,30,62,50,87,141,88,121,31,23,65,187,153,165,85,165,228,243,42,98,179,176,26,246,128,12,87,116,120,182,188,175,13,98,99,240,61,66,55,32,76,134,18,97,67,155,208,24,39,111,44,1,104,95,214,149,59,84,
186,135,31,18,205,166,33,11,187,172,75,238,175,176,35,195,44,9,77,161,199,114,168,139,132,219,130,170,155,179,204,14,199,136,34,91,249,115,161,173,140,89,28,136,37,144,145,126,25,152,124,11,198,169,229,
26,104,81,14,120,65,218,136,48,231,59,65,188,108,80,247,7,98,107,74,172,53,219,90,225,48,230,185,1,177,13,193,211,66,251,194,40,33,237,213,118,57,247,131,216,104,9,128,117,4,196,134,11,249,217,38,106,
44,50,221,230,231,207,13,136,141,55,127,32,102,226,211,138,136,134,47,122,218,255,235,190,120,250,87,196,154,137,196,165,54,133,132,209,18,88,203,145,28,174,218,147,233,71,17,203,231,120,83,52,216,205,
61,50,106,132,47,35,86,123,131,170,4,138,52,184,134,202,71,237,136,15,178,56,16,27,78,80,9,194,82,231,85,86,235,128,23,101,163,184,8,50,170,49,35,248,170,197,108,90,34,43,0,153,191,109,206,179,111,188,
52,65,56,201,251,116,135,88,218,155,43,24,41,228,208,50,186,107,181,203,186,196,97,117,5,122,158,222,130,220,28,134,190,47,90,109,126,16,27,169,207,227,98,117,247,110,147,48,105,21,2,212,186,164,230,44,
32,54,213,176,12,20,90,107,105,8,40,90,239,241,198,176,31,107,107,225,39,238,33,71,56,113,159,216,199,174,184,152,122,18,247,92,15,136,205,235,64,83,67,28,227,77,139,97,234,35,170,235,161,81,241,131,225,
177,39,169,150,178,22,14,181,182,185,231,70,46,235,202,40,86,36,1,91,55,11,246,218,32,214,160,198,140,46,187,236,178,177,22,90,20,136,149,52,84,87,163,175,125,89,151,79,160,133,242,146,165,196,140,139,
44,77,160,27,73,41,249,186,166,181,86,7,177,192,59,173,103,45,130,239,162,70,44,63,87,93,233,194,184,201,165,9,88,109,234,146,193,165,214,179,171,210,4,34,214,188,61,165,201,144,39,131,88,93,231,24,57,
127,148,221,214,101,141,88,254,175,42,18,50,89,17,107,29,51,235,242,184,64,215,165,9,244,10,217,64,38,165,99,32,86,2,170,133,92,23,212,167,34,150,168,185,58,183,53,10,98,5,251,118,26,234,181,204,45,134,
212,134,84,135,55,220,120,199,224,142,59,183,22,74,183,103,207,19,35,215,2,161,119,180,195,168,158,107,132,144,61,90,225,35,143,122,111,161,125,71,31,243,161,66,254,220,184,210,190,172,203,128,151,37,
53,11,247,74,105,130,72,163,158,118,168,194,146,82,47,29,125,92,214,85,131,216,232,62,241,216,43,14,16,49,234,58,219,104,238,236,171,111,194,236,163,52,129,212,140,179,110,102,32,215,32,130,61,224,21,
177,133,255,180,179,13,245,237,97,248,84,31,32,214,146,83,185,134,229,150,138,255,138,202,154,219,231,147,172,117,125,150,38,48,227,43,177,206,40,136,149,158,75,40,33,155,190,64,172,204,58,117,121,77,
247,141,95,214,37,8,51,208,181,216,226,107,196,142,131,88,173,84,49,244,201,32,86,216,74,71,96,247,161,43,16,75,70,144,199,129,171,72,161,8,190,38,42,98,179,230,148,132,160,11,16,235,230,39,145,103,234,
134,171,122,46,140,154,14,98,197,208,121,149,240,162,65,236,87,191,250,213,178,205,146,87,170,167,27,240,117,85,16,171,11,101,32,180,189,139,170,17,91,151,245,21,98,219,106,65,10,43,163,86,191,172,75,
228,41,103,99,84,166,75,27,45,77,160,133,106,195,156,17,172,14,223,207,6,98,157,42,67,233,204,196,52,104,35,165,9,116,153,22,74,195,24,105,219,69,75,205,92,154,64,190,71,212,44,211,21,140,109,84,17,107,
12,233,50,45,68,25,2,232,54,93,119,223,92,32,54,182,40,30,231,80,101,197,86,113,174,2,144,101,228,172,37,89,25,131,10,115,202,198,144,215,170,66,224,217,242,190,54,136,141,197,250,62,47,66,212,144,226,
83,156,83,196,136,18,183,47,235,74,238,201,24,84,25,93,246,247,205,78,250,240,107,180,210,124,25,242,164,210,4,14,216,19,53,240,43,56,167,65,106,121,224,145,179,134,2,131,248,31,223,211,42,25,230,214,
6,133,49,139,173,17,27,47,136,154,148,96,140,184,148,76,87,11,234,90,45,167,182,98,94,135,62,1,81,247,7,98,205,32,45,103,143,69,75,106,21,6,85,235,217,115,167,136,141,174,122,186,169,156,59,36,121,207,
11,16,219,68,140,11,41,77,240,255,1,129,114,194,37,112,87,93,62,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00129_png = (const char*) resource_TalComponent_bmp00129_png;
const int TalComponent::bmp00129_pngSize = 49914;

// JUCER_RESOURCE: bmp00130_png, 3710, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00130.png"
static const unsigned char resource_TalComponent_bmp00130_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,35,0,0,0,42,8,6,0,0,0,57,214,226,145,0,0,0,9,112,72,89,115,0,0,15,97,0,0,15,
97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,3,169,73,68,65,84,120,218,236,152,205,110,228,84,16,133,191,83,190,157,31,4,72,72,48,108,129,45,154,231,152,87,64,130,37,139,217,
242,12,44,224,69,136,52,175,48,17,60,1,155,233,9,98,69,146,13,139,52,11,64,74,66,210,246,173,154,197,181,221,238,252,186,59,97,18,36,44,57,110,117,220,183,63,87,213,61,167,170,245,106,186,23,60,130,99,
111,58,37,1,252,180,187,203,143,187,47,31,12,228,235,231,207,1,10,204,236,232,136,189,233,244,193,96,78,142,79,0,48,30,209,241,168,96,210,152,155,100,134,236,14,220,17,120,206,227,97,36,93,15,35,33,137,
13,202,198,139,246,100,112,5,208,224,218,189,174,17,126,203,250,203,48,26,247,128,31,224,4,144,163,61,1,7,60,202,18,38,168,128,170,189,154,224,79,140,249,5,232,59,165,233,66,196,11,64,7,69,148,39,7,44,
0,9,69,1,137,127,163,102,150,96,150,128,130,220,66,89,27,97,191,144,198,183,186,155,226,138,47,95,23,102,229,200,232,194,89,149,204,92,122,95,111,5,70,139,250,168,84,10,195,218,56,84,82,95,184,182,6,208,
98,107,155,110,14,7,112,210,102,213,5,49,168,143,46,39,161,178,195,12,152,183,31,203,8,136,81,235,39,70,104,128,218,187,143,175,123,86,141,200,109,220,83,154,34,124,189,138,252,47,123,211,255,48,55,214,
204,231,79,159,242,197,151,95,61,24,196,147,39,31,51,155,29,161,71,211,3,191,158,62,158,52,9,149,52,189,216,217,225,197,206,15,183,43,111,167,172,2,147,122,53,190,100,162,17,120,148,215,221,123,55,29,
223,126,247,253,10,157,158,138,196,75,69,242,205,6,146,175,133,168,21,16,240,16,30,144,35,122,135,143,251,16,189,14,196,4,149,169,52,78,86,78,147,250,214,161,139,66,246,40,125,142,131,66,52,30,24,227,
128,210,152,212,168,3,49,72,6,201,68,178,210,209,117,233,234,210,147,77,52,14,141,130,38,67,152,200,30,40,110,23,241,52,166,93,136,16,103,205,197,46,230,170,215,215,247,208,166,104,215,90,179,33,151,192,
172,93,32,224,217,135,239,151,40,17,76,34,72,225,36,15,20,65,0,110,162,150,81,203,104,84,234,230,231,191,79,248,171,201,88,187,190,223,22,153,171,76,187,223,61,2,111,43,244,217,71,239,145,36,182,195,121,
55,55,188,83,207,217,58,107,72,141,227,38,206,55,18,167,155,137,227,106,194,137,85,212,192,111,167,231,5,198,32,156,27,163,51,34,77,90,26,65,42,130,141,112,54,115,195,246,63,13,219,167,115,228,101,245,
84,23,168,218,42,206,49,154,246,147,234,219,144,88,207,155,212,254,25,70,77,237,238,177,8,146,7,147,121,238,65,0,82,147,153,52,153,20,78,21,131,246,115,100,203,183,178,2,71,183,149,175,248,146,0,66,34,
208,165,255,141,152,225,86,132,105,5,44,99,52,149,49,223,74,120,178,2,103,162,222,76,212,147,138,198,68,30,78,9,113,71,209,235,100,60,98,56,202,6,142,152,155,56,171,18,182,21,184,137,170,45,224,249,164,
226,108,146,56,87,69,131,150,70,224,24,33,1,183,42,112,25,104,75,140,191,249,245,247,245,154,38,13,30,44,214,137,204,192,228,4,108,84,197,147,42,45,148,183,210,98,116,9,58,95,42,158,228,173,37,120,148,
168,220,217,155,122,24,11,146,138,5,164,10,38,166,222,18,108,0,147,189,128,212,94,60,169,206,208,120,11,197,29,237,128,65,221,228,142,206,219,218,9,104,250,25,187,228,193,187,95,39,60,22,16,195,118,98,
156,81,234,250,217,105,224,200,30,208,184,138,41,90,153,42,203,108,22,75,247,184,139,28,165,222,60,212,71,249,218,237,173,21,34,131,218,220,59,132,130,24,252,236,161,193,144,54,172,155,24,68,196,71,54,
182,122,53,221,139,63,102,51,102,179,163,81,67,255,178,136,233,10,248,88,193,207,203,241,201,167,159,113,120,176,79,250,101,239,245,106,234,59,140,216,61,141,153,135,7,251,28,238,31,240,102,0,116,166,
228,212,162,20,173,254,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00130_png = (const char*) resource_TalComponent_bmp00130_png;
const int TalComponent::bmp00130_pngSize = 3710;

// JUCER_RESOURCE: bmp00131_png, 4062, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00131.png"
static const unsigned char resource_TalComponent_bmp00131_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,61,0,0,0,43,8,6,0,0,0,203,237,128,111,0,0,0,9,112,72,89,115,0,0,15,97,0,0,15,
97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,5,9,73,68,65,84,120,218,204,154,193,110,27,55,16,134,255,225,174,45,25,144,0,161,50,130,52,40,114,109,128,30,154,91,47,69,209,107,
94,168,237,91,244,80,20,104,209,83,223,192,233,27,212,111,208,22,54,146,7,112,144,0,53,130,196,174,108,75,203,153,30,180,171,208,212,12,201,149,28,88,11,16,164,236,21,151,31,255,25,114,56,43,250,233,231,
95,158,2,152,160,199,37,34,96,102,242,77,227,152,153,152,153,88,152,132,133,68,152,152,133,68,4,64,87,3,88,86,212,117,177,108,17,104,89,11,17,193,57,18,34,39,228,72,156,171,228,205,171,87,127,253,254,
219,175,111,1,48,0,62,191,188,16,108,113,117,99,169,1,252,8,224,219,62,95,38,34,84,85,133,170,170,240,49,175,79,14,15,159,1,56,238,160,167,163,49,183,109,217,102,2,106,0,120,243,250,53,94,158,252,141,
93,185,166,135,15,240,197,151,79,177,88,204,171,118,140,220,218,138,143,38,96,35,248,26,0,22,139,57,222,158,159,239,12,244,254,254,0,0,176,152,47,106,0,123,29,40,0,215,214,183,224,207,47,47,184,55,116,
85,85,56,24,141,118,6,122,120,112,0,0,104,22,183,160,125,4,223,125,246,211,209,152,250,248,252,7,232,225,193,238,40,61,104,149,110,86,208,94,81,153,130,218,3,64,171,186,20,65,59,231,48,28,14,112,118,118,
182,19,208,159,62,122,180,84,186,105,170,22,58,4,142,97,41,40,126,58,26,251,28,248,10,122,127,48,192,96,56,196,119,223,255,128,207,159,60,185,23,216,63,158,31,225,249,209,17,234,189,61,0,128,111,154,186,
29,163,11,212,118,17,172,95,91,8,51,224,117,176,239,130,153,113,124,252,39,78,79,79,238,5,250,229,139,23,96,102,8,115,11,237,99,243,142,149,37,115,7,72,128,215,0,192,158,49,155,205,48,153,76,112,122,114,
114,175,166,61,153,76,112,51,191,89,66,123,31,154,119,49,112,187,189,137,229,227,53,0,242,222,227,106,54,219,153,133,236,230,122,5,221,153,119,9,104,12,221,93,107,230,95,139,8,188,111,48,187,252,111,103,
160,175,199,87,0,0,102,95,25,208,20,171,10,160,210,218,154,218,173,210,140,235,235,235,157,129,158,223,204,151,18,49,199,208,154,154,98,128,187,174,142,193,107,0,120,240,240,33,190,250,250,155,221,9,78,
134,195,37,205,109,232,18,208,91,176,81,20,247,65,105,239,253,63,251,123,123,238,179,199,143,73,68,8,192,178,22,33,137,218,16,33,9,219,203,26,2,33,8,168,171,151,167,171,206,12,5,34,186,47,18,65,90,30,
233,78,91,32,8,129,228,242,242,66,222,191,123,119,213,9,19,64,161,133,136,85,13,11,43,237,149,111,211,116,52,30,181,29,212,91,150,42,168,195,210,61,156,34,197,36,8,43,125,80,26,165,222,166,172,250,250,
247,226,189,32,48,157,112,80,46,83,170,160,14,75,109,212,149,1,205,81,44,77,198,2,149,27,15,37,138,11,162,55,234,44,165,86,96,83,29,184,204,196,84,145,213,84,65,68,229,20,104,31,236,191,80,252,85,34,83,
117,137,113,57,99,31,95,91,4,107,205,213,140,142,173,207,26,180,6,95,69,208,222,152,8,103,76,112,137,56,57,229,1,101,15,76,221,76,5,166,20,15,44,134,119,193,66,68,65,219,109,160,152,21,172,88,81,27,165,
148,182,58,201,181,17,193,107,174,208,61,175,41,80,165,228,185,232,17,154,162,15,180,40,251,163,214,14,77,52,174,187,210,4,234,178,226,191,90,73,61,23,198,253,89,232,190,15,182,10,27,91,81,232,179,177,
79,251,104,82,196,152,140,20,172,37,146,118,191,169,116,14,200,2,100,229,36,132,64,89,159,88,189,125,102,18,56,51,158,220,223,37,134,150,68,231,98,252,159,141,226,141,32,196,10,78,124,240,189,166,96,18,
172,9,224,30,22,178,130,230,32,102,229,76,241,70,170,38,182,150,240,254,62,17,89,14,152,51,160,69,74,135,192,37,138,198,89,12,235,212,163,69,91,22,116,152,240,107,148,144,180,239,68,172,185,70,124,202,
138,31,78,74,18,46,54,221,38,177,22,196,105,90,151,176,6,13,220,27,177,184,55,238,201,2,107,230,29,30,191,56,1,138,196,62,40,6,180,203,184,128,68,139,26,103,252,188,81,128,83,19,16,46,110,107,208,177,
159,198,217,70,43,99,145,83,184,15,52,23,248,120,110,193,83,149,215,50,39,172,0,151,170,42,202,43,151,92,56,153,235,135,19,166,156,83,60,254,190,153,24,228,2,85,145,9,80,52,232,48,14,71,34,221,19,155,
161,223,2,94,83,95,141,171,209,190,11,210,206,198,185,226,162,68,129,118,66,162,196,4,34,17,178,198,166,206,125,193,99,149,69,110,191,36,199,116,52,214,78,70,46,3,108,65,83,198,159,45,191,182,252,187,
68,245,36,176,10,109,128,87,153,207,125,178,25,200,36,248,54,5,231,18,96,19,58,50,117,103,152,113,137,89,107,254,108,173,252,80,98,121,107,97,75,77,0,167,128,147,208,1,184,75,168,90,234,199,169,61,126,
83,181,45,248,236,107,218,36,180,98,238,46,99,206,206,200,158,80,207,237,175,79,40,204,37,234,246,134,14,84,167,204,130,213,87,229,148,218,156,73,70,132,202,246,250,217,69,49,116,66,249,28,108,42,79,37,
5,106,155,192,155,254,178,104,35,104,69,125,11,30,27,42,141,68,130,66,238,234,119,100,132,59,184,162,73,208,130,146,18,232,181,116,209,182,144,31,21,186,192,37,204,171,175,95,222,5,244,255,3,0,138,105,
124,173,220,213,196,96,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00131_png = (const char*) resource_TalComponent_bmp00131_png;
const int TalComponent::bmp00131_pngSize = 4062;

// JUCER_RESOURCE: bmp00132_png, 6060, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00132.png"
static const unsigned char resource_TalComponent_bmp00132_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,38,0,0,0,69,8,6,0,0,0,38,93,148,90,0,0,0,9,112,72,89,115,0,0,46,35,0,0,46,35,
1,120,165,63,118,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,12,215,73,68,65,84,120,218,204,154,105,140,101,199,85,128,191,83,117,151,215,175,247,113,47,211,179,180,123,50,99,143,99,143,215,
137,193,24,7,153,32,146,8,57,178,34,12,1,188,16,16,196,10,81,64,24,35,132,3,66,1,203,8,89,16,39,33,252,9,34,34,145,144,18,164,4,43,192,159,132,95,78,226,132,216,145,199,177,61,246,56,102,198,179,79,239,
203,91,238,82,85,135,31,247,245,58,221,51,131,81,226,190,173,210,187,173,170,115,238,169,83,167,234,158,175,234,202,157,183,255,180,222,116,203,45,180,218,45,182,211,21,221,116,203,45,60,246,23,159,100,
100,116,116,91,24,52,53,53,201,244,212,20,60,244,192,131,154,149,170,192,182,40,191,242,203,247,233,145,23,95,209,8,64,125,0,96,239,248,174,183,213,91,211,211,83,140,238,25,67,85,137,0,66,80,136,32,233,
174,93,212,88,157,34,145,188,229,135,105,213,103,196,92,190,109,146,215,137,172,93,53,140,160,144,130,77,204,26,131,2,238,76,70,52,81,39,204,150,216,193,248,255,108,84,104,56,40,3,24,193,164,6,169,217,
75,7,124,34,96,45,40,84,150,40,16,129,90,69,173,226,22,10,252,92,193,174,127,191,157,221,95,189,141,238,247,13,225,219,229,74,253,101,139,6,138,55,155,216,125,53,246,126,235,78,118,255,219,97,204,104,
76,240,254,210,114,145,98,34,133,106,0,59,134,197,213,77,113,188,69,223,135,118,177,227,145,3,224,2,238,66,142,25,73,0,69,162,203,123,201,205,20,168,194,216,23,111,166,118,120,144,226,104,3,59,154,32,
221,22,230,203,45,117,104,0,12,4,83,133,205,170,97,14,138,165,146,93,255,124,11,93,183,13,82,252,168,197,155,31,124,150,253,207,255,60,193,121,66,44,72,188,117,172,133,204,147,255,79,147,171,62,58,193,
142,135,38,176,253,49,147,79,188,74,235,59,115,140,127,237,167,80,13,132,88,96,19,29,33,243,80,6,188,247,72,240,107,12,43,20,246,192,254,255,188,11,99,12,231,62,247,58,51,159,127,3,82,193,70,150,96,149,
210,56,130,108,98,88,80,138,243,109,226,241,30,246,255,215,93,196,59,106,180,142,204,114,250,175,142,226,167,50,210,235,123,49,98,40,108,192,139,67,54,204,2,55,87,96,186,45,227,159,189,141,174,214,60,
237,127,205,144,229,161,52,214,192,121,40,143,53,56,241,228,15,201,79,54,137,111,168,147,157,106,96,84,240,40,101,228,80,187,65,105,163,164,152,201,153,248,227,67,12,221,59,142,46,57,78,253,237,75,76,
127,237,36,233,120,55,58,102,9,81,192,168,16,140,167,140,28,198,86,19,64,85,201,78,55,25,120,247,78,38,30,187,9,3,188,242,169,215,144,241,67,171,30,179,70,160,23,158,255,195,103,137,122,99,204,88,74,129,
210,178,138,5,74,81,26,70,137,173,86,78,114,74,235,84,131,209,159,27,227,208,99,55,19,215,99,206,124,253,36,175,61,249,34,201,96,66,116,109,15,45,192,103,129,154,21,172,66,203,10,153,81,172,85,124,230,
201,39,51,174,123,244,70,198,62,48,78,243,245,5,94,124,236,121,22,39,47,16,174,183,8,194,106,40,26,96,184,134,91,227,145,2,16,64,5,114,169,66,177,156,205,161,240,220,250,233,159,97,199,205,87,81,76,103,
124,231,227,223,102,233,84,139,218,120,55,193,8,69,71,222,11,68,120,4,193,225,41,4,194,76,142,169,25,238,252,202,123,72,135,187,56,243,245,147,188,252,233,35,36,67,117,24,76,112,82,61,51,170,20,40,20,
48,84,174,31,170,188,176,148,2,169,23,134,23,193,31,111,48,252,187,251,217,251,241,235,113,18,56,245,247,199,56,251,249,215,232,221,215,203,192,80,15,120,170,178,28,126,165,18,59,75,41,74,127,97,137,94,
109,49,244,161,9,174,254,196,141,100,75,5,71,63,246,93,218,47,205,177,103,172,15,49,194,116,97,168,123,33,176,188,242,139,66,9,137,91,191,136,74,153,224,77,192,76,58,186,106,93,236,253,198,187,177,35,
41,11,207,79,115,246,145,35,136,49,12,29,24,238,4,220,38,75,64,105,176,62,198,227,145,83,158,125,127,115,152,158,247,237,100,233,141,37,78,255,246,247,177,181,136,254,209,1,8,64,128,212,37,152,96,80,89,
158,149,82,41,54,126,189,98,227,132,208,116,244,188,103,152,161,63,59,8,1,206,253,209,75,180,191,57,73,114,117,189,122,85,249,75,172,252,94,48,94,112,51,5,187,158,186,145,228,157,61,204,125,225,36,51,
79,28,35,61,216,115,145,188,113,130,4,179,102,185,16,32,163,10,164,181,87,6,209,222,26,82,219,65,227,233,11,76,127,226,85,162,225,148,100,207,197,195,182,217,37,57,24,99,136,246,212,208,158,136,115,15,
188,64,113,100,145,218,129,190,205,229,115,192,7,88,241,152,17,200,192,23,186,94,241,104,202,241,27,158,33,189,182,135,230,183,103,137,119,215,81,35,23,181,219,242,50,134,214,11,75,156,127,224,69,138,
31,53,9,109,143,25,169,109,41,239,51,5,167,29,195,180,234,21,14,242,102,115,211,236,34,251,65,11,179,195,144,191,149,44,55,129,133,31,76,34,177,84,67,215,44,182,108,154,53,26,184,210,35,34,68,198,24,164,
243,126,90,152,90,216,250,1,141,31,127,62,150,181,61,121,171,93,25,6,208,106,183,168,199,17,123,247,238,121,91,19,197,249,249,121,250,123,251,41,203,146,168,175,175,15,239,28,51,205,114,91,228,252,11,
243,115,204,207,207,35,251,199,175,214,90,119,13,87,186,237,69,73,187,199,247,112,251,187,238,160,213,218,102,248,118,195,245,55,241,251,143,62,202,232,200,206,237,133,111,31,126,240,55,53,43,182,41,190,
133,176,77,241,77,67,216,18,223,126,146,215,69,248,166,161,130,17,27,153,183,55,224,35,1,43,27,40,41,133,114,54,187,248,69,156,152,43,102,74,45,2,126,238,226,245,208,116,91,76,79,116,89,25,95,56,76,100,
54,24,54,15,187,255,227,48,238,66,182,206,168,252,88,147,217,191,126,157,100,95,253,178,70,153,110,195,200,223,221,92,129,238,178,81,93,150,249,47,159,37,127,102,6,123,85,178,165,12,10,197,151,95,197,
181,220,74,66,141,40,96,151,231,133,172,20,191,224,233,187,103,39,131,143,188,131,178,233,8,177,108,89,202,134,227,170,191,188,14,91,143,209,92,65,5,245,160,109,101,231,19,239,36,186,177,23,231,117,107,
153,66,65,2,74,88,205,199,4,129,49,56,254,91,255,189,174,71,197,201,38,55,189,126,15,229,82,177,53,190,45,19,147,148,132,166,35,159,43,120,243,225,231,136,70,107,104,230,137,246,116,113,205,87,126,150,
178,44,113,27,240,109,157,204,71,158,99,105,71,6,15,217,85,124,147,206,22,1,187,162,13,25,168,189,36,190,173,235,68,228,64,5,137,45,97,183,133,145,8,205,5,25,234,232,216,128,111,155,201,248,84,81,116,
213,99,70,5,44,52,237,250,4,110,43,124,219,52,101,177,138,118,28,186,100,149,154,85,188,209,77,241,109,43,153,220,40,165,232,42,190,201,26,92,227,50,248,182,165,199,96,37,62,138,78,240,110,134,111,246,
18,50,8,171,248,166,232,21,225,219,72,97,136,202,75,12,101,105,176,157,46,142,148,134,164,52,235,240,109,192,89,186,75,83,101,203,91,200,44,202,26,124,51,98,46,139,111,81,33,36,13,139,189,68,140,201,82,
132,250,138,31,106,101,66,228,226,21,124,11,38,16,181,45,178,100,145,53,157,219,40,147,155,13,248,166,151,192,55,18,193,212,45,201,72,125,203,69,178,226,25,139,24,193,116,27,140,19,140,175,240,205,138,
33,168,146,12,166,184,82,144,53,155,131,203,50,146,74,37,99,55,226,155,217,28,223,4,67,246,221,57,6,31,158,96,240,99,251,46,191,244,91,97,250,15,142,98,226,8,114,65,188,224,79,228,132,169,156,145,207,
28,90,222,38,220,176,50,43,243,159,61,137,136,133,188,227,156,101,143,73,135,43,47,194,170,254,152,169,63,57,70,52,146,160,229,229,145,77,186,12,237,35,139,68,195,105,71,151,16,90,158,115,15,191,140,169,
89,240,23,235,144,212,80,30,111,162,53,139,159,83,188,243,107,12,179,151,192,183,166,162,211,122,197,47,98,83,55,248,230,250,52,189,56,213,70,253,214,58,36,53,72,168,240,205,47,227,91,181,163,124,5,248,
118,165,87,227,173,203,92,132,111,237,214,246,193,183,222,21,124,235,239,199,121,199,204,37,8,249,39,109,220,194,252,28,114,221,254,3,218,223,55,64,233,182,7,87,174,36,141,113,26,99,99,139,74,216,94,134,
253,194,47,190,151,63,253,243,79,210,219,219,187,141,134,114,158,104,113,113,145,200,90,70,251,187,185,122,98,226,109,55,234,215,239,191,159,95,253,141,7,171,89,217,85,171,179,148,101,156,153,58,251,99,
123,104,88,234,164,204,189,91,191,214,230,102,22,89,108,52,137,163,120,61,190,237,24,25,254,127,61,220,207,84,51,219,94,149,84,144,49,83,84,247,121,160,231,215,170,131,218,198,211,23,182,52,78,35,161,
171,187,134,170,86,111,47,85,237,224,155,172,20,227,2,225,84,27,227,2,54,18,116,58,71,167,243,77,235,108,36,176,80,50,252,248,181,12,63,126,45,58,149,19,239,73,25,255,198,237,196,123,82,88,112,244,222,
59,76,239,189,195,248,211,173,21,61,155,149,101,124,51,132,85,124,211,88,209,88,113,11,57,102,119,194,53,75,239,39,58,88,167,56,211,100,215,211,135,217,245,244,97,138,51,77,162,131,245,149,186,224,61,
193,123,220,124,78,52,150,16,141,37,132,224,145,68,136,15,116,67,80,124,51,39,44,57,138,99,77,118,125,245,54,118,61,125,152,114,38,91,121,222,74,73,116,19,124,139,64,162,234,140,177,254,129,81,146,145,
152,252,216,18,90,6,36,2,119,174,194,58,137,64,203,64,121,182,141,150,1,117,158,244,214,126,186,239,25,197,79,230,152,129,184,58,97,51,224,103,75,146,235,122,72,111,237,35,204,151,85,134,122,46,7,237,
232,241,1,119,62,35,218,89,171,206,51,35,232,100,61,107,206,43,19,8,177,224,138,64,207,123,135,168,191,127,148,176,228,113,193,227,84,87,118,61,124,34,120,9,136,130,151,128,15,74,114,93,55,131,31,25,199,
45,56,20,200,230,114,202,86,1,237,64,116,125,157,193,135,175,174,234,150,201,16,40,219,14,51,158,114,240,252,47,97,247,213,86,208,110,21,223,4,68,5,12,148,90,82,20,25,190,225,177,13,143,116,11,244,25,
100,127,74,40,20,147,26,242,34,195,106,55,162,130,19,37,47,50,92,238,9,243,142,96,160,156,204,25,255,151,119,209,122,101,129,208,240,132,60,16,230,74,130,169,54,199,1,66,161,149,206,4,202,19,77,202,50,
224,164,172,16,209,42,162,96,52,40,134,202,48,23,7,228,154,122,229,199,160,213,225,195,221,195,28,248,210,29,20,231,91,132,204,33,215,212,209,62,32,87,66,82,253,239,141,71,2,168,6,124,230,168,237,235,
65,186,35,112,74,32,0,66,58,156,146,142,212,80,148,226,124,139,3,95,186,131,158,187,135,209,134,199,155,146,50,170,240,110,25,223,76,133,111,213,97,64,57,90,227,208,63,221,69,50,148,162,78,177,169,165,
200,61,238,66,142,55,66,123,42,231,208,23,238,162,247,238,49,194,92,129,31,174,218,155,225,122,181,60,24,193,27,65,179,128,11,138,0,222,8,93,195,53,190,247,123,207,240,189,143,62,67,186,183,27,173,89,
220,133,156,162,237,49,29,180,107,152,10,223,114,169,28,101,86,240,77,160,44,74,202,201,12,117,138,170,210,56,177,68,80,69,68,8,157,191,114,50,163,232,100,34,174,244,148,147,25,174,116,8,172,180,17,192,
175,251,21,24,169,33,123,234,100,147,45,218,147,45,76,108,86,234,150,209,174,58,215,90,19,252,174,3,35,3,206,18,68,145,196,80,78,102,28,251,240,183,224,124,14,169,80,31,72,169,15,164,4,160,11,131,214,
12,61,161,34,172,122,61,198,140,166,164,42,164,42,4,160,22,170,149,168,203,88,92,151,48,234,44,187,123,187,56,247,212,81,166,158,58,138,25,76,232,194,16,186,12,3,33,98,216,25,234,126,21,223,170,5,86,20,
50,136,125,204,114,142,97,18,203,142,190,126,210,40,66,118,196,76,63,249,6,211,79,190,129,25,138,17,103,200,143,47,17,149,17,210,23,209,250,230,52,39,239,127,142,120,164,11,9,6,21,173,38,84,12,44,6,242,
19,13,186,164,70,82,196,244,118,247,208,223,213,75,136,20,113,134,236,248,18,113,17,147,186,132,52,95,197,55,179,30,223,4,68,145,94,139,237,177,136,19,76,26,97,119,38,112,58,135,211,57,166,55,194,79,22,
156,185,239,57,244,124,81,125,29,176,24,112,175,54,72,38,234,196,59,83,226,119,212,137,250,99,164,167,50,250,244,125,223,39,25,170,97,124,69,65,198,9,146,154,53,122,50,172,53,24,191,6,223,130,134,21,124,
147,32,196,19,93,44,252,195,155,184,211,57,81,111,66,241,195,6,11,159,59,137,237,79,9,115,37,118,40,198,118,71,68,3,53,76,100,177,195,49,198,88,146,177,58,103,63,248,124,21,39,243,37,201,141,125,12,252,
206,56,182,30,19,15,214,214,163,97,198,170,158,193,26,162,21,186,173,195,55,65,170,207,91,28,184,153,146,249,127,60,205,194,23,207,225,151,28,118,48,161,253,194,34,205,103,231,177,131,213,190,215,236,
83,39,200,95,88,66,19,139,155,41,153,253,212,9,220,76,89,157,230,237,72,171,48,232,139,81,207,250,186,53,104,168,245,104,69,79,16,169,190,98,232,156,190,173,195,183,229,253,132,214,133,69,26,143,207,98,
122,35,36,17,220,50,7,36,84,247,9,156,123,252,101,164,102,49,93,134,226,66,182,218,62,146,170,215,107,176,111,171,186,141,122,104,174,158,190,173,224,91,32,32,29,195,22,231,22,59,137,209,21,224,214,90,
76,187,84,251,185,43,215,179,30,223,194,246,58,125,91,135,111,222,123,102,91,229,54,201,249,231,152,159,155,71,14,78,236,215,158,129,94,92,185,189,240,237,127,7,0,45,14,106,147,46,200,39,32,0,0,0,0,73,
69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00132_png = (const char*) resource_TalComponent_bmp00132_png;
const int TalComponent::bmp00132_pngSize = 6060;

// JUCER_RESOURCE: bmp00133_png, 6767, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00133.png"
static const unsigned char resource_TalComponent_bmp00133_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,38,0,0,0,92,8,6,0,0,0,2,132,199,9,0,0,0,9,112,72,89,115,0,0,46,35,0,0,46,35,
1,120,165,63,118,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,
8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,15,154,73,68,65,84,120,218,236,154,121,144,156,71,117,192,127,221,253,205,124,51,179,179,51,59,171,61,180,247,202,90,99,89,182,14,
75,62,240,129,13,84,136,33,64,40,19,7,2,72,36,225,8,80,42,72,197,118,112,69,38,24,219,177,83,198,54,9,113,72,138,184,168,112,21,41,142,96,2,4,42,1,82,36,62,49,22,182,132,44,203,146,37,173,246,146,180,
231,236,206,249,29,221,157,63,102,239,157,61,236,4,188,127,208,85,175,106,170,166,223,251,94,119,191,175,191,247,235,215,226,170,203,174,176,219,119,238,164,88,42,178,158,154,179,125,231,78,246,223,126,
7,77,205,205,235,194,161,145,145,97,70,71,70,224,189,123,246,218,114,96,45,176,46,228,247,127,239,70,123,240,208,17,235,0,88,109,0,232,232,108,125,69,103,107,116,116,132,230,246,22,172,181,56,0,198,88,
112,32,90,19,91,210,217,134,22,225,136,151,253,48,91,25,51,66,174,222,55,234,37,112,148,154,115,12,99,193,5,21,149,243,28,50,132,131,101,156,238,4,102,60,64,101,34,47,217,41,147,15,33,48,32,5,210,149,
136,152,90,57,224,163,2,148,2,11,21,79,44,224,128,85,22,171,44,225,164,143,158,240,105,253,254,101,180,125,123,23,53,215,55,160,75,193,236,255,171,138,53,248,167,11,168,77,49,58,30,189,138,182,239,236,
70,54,71,48,90,175,172,231,88,164,99,161,178,128,211,142,69,42,63,252,83,69,82,239,108,165,254,166,30,8,13,225,57,15,217,20,5,44,194,89,125,150,194,49,31,107,161,229,203,59,136,237,206,224,63,159,71,53,
71,17,53,10,178,193,178,54,172,1,36,24,89,9,155,57,199,66,240,115,1,173,95,218,73,124,87,6,255,197,34,167,111,120,130,205,7,94,135,9,53,38,34,16,145,229,99,205,148,53,222,201,2,27,62,210,77,253,123,187,
81,233,8,195,247,28,165,248,248,4,157,15,95,142,181,6,19,17,80,197,134,41,107,8,12,90,107,132,209,243,28,243,45,180,195,230,31,92,131,148,146,51,159,59,206,216,67,39,192,21,40,71,97,148,37,144,33,70,84,
113,204,88,252,179,37,34,157,73,54,255,228,26,34,245,49,138,7,199,25,184,235,121,244,72,25,119,107,45,82,72,124,101,208,34,68,44,122,11,194,9,31,89,163,232,124,112,23,241,98,150,210,55,203,136,153,165,
148,74,194,89,8,142,229,233,189,239,151,120,125,5,34,23,37,40,247,231,145,86,160,177,4,78,136,85,139,140,230,3,252,49,143,238,63,191,152,134,183,117,98,115,33,253,15,28,102,244,225,62,220,206,26,108,139,
194,56,6,105,5,70,106,2,39,68,170,202,11,96,173,165,60,80,160,238,53,27,233,222,191,29,9,28,249,155,23,16,157,23,207,205,152,146,2,106,225,192,159,61,129,83,27,65,182,184,248,88,138,202,162,128,64,88,
242,210,18,81,182,50,73,161,165,216,159,167,249,218,22,46,222,191,131,72,34,194,224,247,250,120,225,190,67,68,51,81,156,87,37,41,2,186,108,136,41,129,178,80,84,130,178,180,40,101,209,101,141,55,92,102,
203,45,219,104,121,107,39,133,227,147,28,218,127,128,169,225,115,152,173,10,129,96,46,20,37,208,24,35,156,55,35,62,32,0,43,192,19,149,80,12,198,61,240,53,151,124,246,74,234,119,108,192,31,45,243,248,71,
31,35,215,95,36,214,89,131,145,2,127,90,95,11,112,208,8,4,33,26,95,128,25,243,144,49,201,85,223,120,61,110,99,156,193,239,245,241,220,103,15,18,109,72,64,38,74,40,42,207,116,42,6,44,248,208,16,44,92,42,
207,87,4,2,92,45,104,156,2,125,42,79,227,7,55,211,241,209,173,132,194,208,255,247,199,24,122,232,5,106,55,213,82,215,144,4,77,69,102,194,47,176,68,66,69,32,44,105,95,225,28,45,210,240,206,110,186,110,
219,70,57,231,243,252,190,39,41,29,158,160,189,37,133,144,130,81,95,146,208,2,195,204,206,47,44,4,16,13,23,110,162,34,136,162,165,65,14,135,196,99,113,58,126,244,26,84,147,203,228,129,81,134,110,58,136,
144,146,134,158,198,233,128,171,178,5,4,18,165,35,104,52,162,95,179,233,222,221,36,175,223,72,238,68,142,129,247,253,28,21,115,72,55,215,129,1,12,184,97,20,105,36,86,204,188,149,162,98,88,234,133,134,
101,40,48,133,144,228,235,27,105,248,196,5,96,224,204,205,135,41,253,120,152,104,87,162,242,169,210,43,236,252,90,32,181,32,28,243,105,253,219,109,68,47,76,50,241,207,125,140,221,115,12,247,130,228,18,
125,25,10,132,145,243,182,11,1,148,169,4,210,252,86,6,167,35,134,136,213,147,255,183,115,140,222,118,20,167,209,37,218,190,116,217,170,53,225,129,148,18,167,61,134,77,58,156,217,243,44,254,193,41,98,61,
169,234,250,30,160,13,204,206,152,20,80,6,237,219,133,134,155,93,78,93,244,8,238,171,146,20,30,27,39,210,150,192,74,177,164,223,178,77,74,138,207,230,56,187,231,16,254,139,5,76,73,35,155,98,203,234,235,
178,133,208,78,59,102,43,163,34,4,175,80,168,154,93,148,127,81,68,214,75,188,151,147,229,70,97,242,23,195,136,136,168,44,93,193,95,182,107,57,159,39,12,52,66,8,28,41,37,98,250,251,52,57,50,185,252,3,242,
191,250,124,172,92,210,120,197,82,197,49,128,98,169,72,34,226,208,209,209,254,138,38,138,217,108,150,116,109,154,32,8,112,82,169,20,58,12,25,43,4,235,34,231,159,204,78,144,205,102,17,155,59,187,108,172,
38,70,24,132,235,139,146,218,58,219,185,236,210,87,83,44,174,51,124,187,104,235,118,62,118,203,45,52,55,109,92,95,248,246,71,123,255,208,150,253,117,138,111,198,172,83,124,179,198,44,139,111,191,206,182,
4,223,172,169,192,136,114,228,43,27,240,142,0,37,22,81,146,11,193,120,121,233,135,56,42,215,204,148,214,55,232,137,165,251,161,172,81,200,164,179,170,142,246,67,164,35,23,57,150,133,182,127,223,77,120,
174,188,192,41,239,88,129,241,191,62,78,116,83,98,85,167,100,141,164,233,51,59,42,160,59,227,84,92,145,253,250,16,222,35,99,168,13,209,101,117,176,224,127,253,40,97,49,156,77,168,17,22,80,51,239,133,152,
21,61,169,73,189,101,35,153,155,206,35,40,132,152,136,88,86,130,124,200,134,59,183,160,18,17,172,103,193,10,172,6,91,178,108,188,231,66,156,109,181,132,218,46,175,227,91,16,6,139,153,203,199,4,2,90,224,
212,31,63,181,96,68,126,95,129,237,199,223,66,144,243,151,199,183,25,98,18,1,166,16,226,77,248,156,254,208,211,56,205,49,108,89,227,180,199,57,255,27,87,19,4,1,225,34,124,91,160,243,39,79,147,171,47,195,
123,213,28,190,137,233,35,2,90,157,69,25,168,90,17,223,22,12,194,9,193,10,68,68,97,218,20,52,57,88,79,32,26,166,109,44,194,183,106,58,218,181,88,236,220,140,73,43,64,65,65,45,76,224,150,195,183,170,41,
139,178,216,233,9,205,41,75,76,89,180,180,85,241,109,57,29,79,90,2,97,231,240,77,204,195,53,86,193,183,101,103,12,102,227,195,159,14,222,106,248,166,86,208,65,48,135,111,22,187,38,124,107,242,37,78,176,
194,82,6,18,53,61,196,166,64,18,13,228,2,124,171,11,21,53,129,172,100,203,203,232,76,137,121,248,38,133,92,21,223,28,95,16,205,43,212,10,49,38,114,14,86,87,248,33,22,68,113,194,200,44,190,25,105,112,74,
10,145,83,136,121,131,91,172,227,201,69,248,102,87,192,55,162,2,153,80,68,155,18,203,110,146,21,158,81,8,41,144,53,18,25,10,164,174,224,155,18,18,99,45,209,140,75,24,8,196,188,195,193,25,29,225,138,138,
142,90,140,111,178,58,190,9,36,229,39,39,200,124,168,155,204,190,77,171,111,253,74,48,250,167,207,35,35,14,120,2,161,5,186,215,195,140,120,52,253,221,197,51,199,132,139,118,102,75,246,193,62,132,80,224,
77,79,206,204,140,137,105,174,92,130,85,233,8,35,183,30,195,105,138,98,131,213,145,77,196,37,165,131,83,56,141,238,180,45,129,41,106,206,124,232,57,100,76,129,94,106,67,184,146,224,84,1,27,83,232,9,139,
14,245,60,199,212,10,248,86,176,216,81,187,230,15,177,76,72,116,97,97,154,238,247,151,176,122,121,27,194,149,8,83,193,55,61,131,111,149,19,229,53,224,219,90,91,254,229,235,44,193,183,82,113,253,224,91,
237,44,190,165,211,132,58,100,108,5,66,254,117,59,55,153,157,64,108,233,233,177,233,84,29,97,176,62,184,114,254,57,226,186,108,206,245,111,122,51,183,125,234,14,146,201,218,245,65,226,147,89,38,179,89,
156,169,201,73,148,114,104,78,215,208,213,221,253,138,199,215,187,222,243,30,222,241,238,189,149,183,50,30,139,147,43,151,25,28,25,122,69,29,155,24,155,98,42,95,32,226,68,152,43,11,58,80,223,212,248,138,
58,102,29,65,188,38,54,15,223,236,12,190,137,5,21,143,96,168,12,218,162,26,93,100,66,189,172,135,233,81,31,83,8,145,233,8,170,110,101,218,82,243,240,77,206,199,55,27,177,21,81,22,175,63,79,231,99,87,210,
117,228,90,156,243,99,232,130,63,247,255,26,197,239,47,80,255,151,231,209,125,244,58,146,239,106,33,24,43,175,172,19,181,179,248,182,160,44,40,166,197,59,158,163,245,107,187,176,218,18,158,44,209,252,
217,109,136,132,192,134,102,182,207,106,162,39,124,106,255,160,133,196,27,154,8,79,150,200,236,235,194,233,142,97,203,225,138,122,211,89,207,60,124,139,130,137,8,202,67,37,154,30,216,74,124,71,29,217,
47,244,209,191,247,105,36,130,142,239,94,73,105,176,128,22,172,136,113,38,34,8,60,141,108,139,208,240,201,45,152,65,143,99,215,252,4,25,10,218,190,184,139,192,211,43,218,152,73,181,231,28,147,80,26,47,
224,94,91,71,250,45,109,76,126,119,136,145,111,244,98,234,4,167,62,240,51,156,152,67,247,191,94,65,254,236,36,129,13,9,68,117,241,3,159,114,169,68,219,23,46,69,148,12,199,222,241,8,238,174,58,142,189,
253,81,34,201,8,109,95,184,132,220,139,227,213,245,101,136,81,22,49,179,148,130,233,66,64,171,67,247,253,187,8,94,156,162,239,129,195,200,30,23,83,39,40,101,139,156,252,248,1,106,54,167,233,188,103,7,
217,83,227,132,74,47,21,25,146,31,201,177,233,243,87,16,79,185,60,119,227,79,81,231,197,208,181,16,24,159,190,191,58,76,205,121,41,154,63,126,1,249,179,83,75,244,3,39,156,197,55,89,193,183,202,17,193,
142,175,190,14,81,54,60,190,231,191,241,219,93,10,78,5,233,188,198,40,3,79,157,101,224,107,39,168,191,170,153,238,219,47,97,120,40,71,65,217,5,114,246,244,20,221,119,239,38,221,147,226,208,199,159,34,
143,161,152,144,179,54,78,253,176,151,241,71,134,105,125,119,15,241,171,155,152,152,242,22,232,207,199,55,57,139,111,27,225,71,191,243,3,158,190,249,113,84,87,13,129,35,241,167,177,202,7,84,123,13,71,
62,119,152,225,39,134,105,121,83,59,153,87,55,147,31,247,102,255,207,157,45,210,181,231,124,154,174,110,230,228,191,156,96,232,169,97,108,38,186,208,70,87,146,3,159,250,25,133,227,147,108,249,139,157,
152,184,164,236,233,217,255,231,227,155,172,160,122,5,70,54,38,226,36,206,132,52,203,8,13,129,92,34,29,29,117,244,223,246,12,185,83,57,46,185,247,50,218,82,9,50,227,154,204,104,72,103,207,6,122,62,178,
133,236,47,199,24,185,255,8,157,77,181,75,244,27,181,162,61,83,203,139,251,159,65,99,184,252,238,43,72,15,5,52,120,130,134,64,146,152,87,125,147,21,74,170,224,155,107,162,196,132,75,52,140,84,21,215,68,
169,107,72,209,123,227,147,248,229,144,205,15,93,142,235,71,113,77,140,77,247,237,164,52,92,98,224,3,207,82,191,105,195,178,54,98,42,70,60,43,56,115,231,243,56,23,212,208,113,243,197,136,19,65,197,126,
184,70,124,171,158,160,75,18,173,53,156,126,243,19,116,126,255,213,52,223,127,17,102,60,196,40,201,224,222,167,136,55,197,17,118,229,170,156,76,69,241,126,60,198,228,195,67,164,110,104,161,124,104,138,
242,255,140,33,245,26,241,109,89,223,132,66,22,13,103,63,120,136,214,111,238,2,9,195,31,60,140,152,178,136,250,10,134,173,214,34,45,9,70,247,29,33,190,187,142,150,207,239,224,197,204,127,64,106,173,248,
182,82,75,56,148,143,20,232,127,227,207,49,227,1,186,160,81,153,200,75,178,161,46,72,50,116,227,47,32,176,136,230,24,122,162,176,118,124,91,173,178,22,244,123,32,65,68,5,225,203,224,6,75,37,129,16,165,
224,87,140,111,255,159,213,183,245,137,111,169,212,58,195,183,233,234,219,214,243,95,101,211,169,52,65,248,27,124,91,27,190,253,214,245,111,228,19,119,220,185,126,240,45,155,37,59,153,197,201,231,115,
40,169,214,33,190,89,136,199,19,191,193,183,151,136,111,118,41,190,45,135,99,217,0,91,214,8,71,34,83,206,130,51,213,151,210,76,81,87,106,78,198,34,147,14,50,233,44,192,55,103,49,190,173,212,252,163,121,
50,55,109,38,126,77,29,186,183,196,196,63,244,18,78,249,168,250,151,118,227,83,15,123,184,59,211,164,247,180,33,234,28,114,95,29,162,240,95,163,88,119,21,124,171,38,254,137,60,173,223,218,77,221,251,59,
176,69,75,252,234,13,116,252,228,74,220,171,234,48,83,193,154,177,206,228,3,220,107,235,217,248,208,118,212,198,24,50,230,208,120,239,133,196,94,91,143,45,134,11,241,141,121,248,86,21,199,114,1,233,125,
93,196,182,212,146,251,225,57,6,246,60,205,169,183,61,129,127,40,79,243,237,23,98,210,18,109,237,170,88,167,5,104,5,205,159,220,130,30,242,232,123,251,207,56,245,250,71,209,35,62,173,15,108,35,140,128,
157,190,60,57,141,111,2,36,213,145,202,6,20,243,37,50,123,187,8,122,75,12,236,63,132,218,85,3,117,146,211,183,62,139,176,134,214,7,182,145,235,205,46,139,116,51,82,26,47,80,183,175,11,39,25,161,239,166,
103,16,155,162,56,151,38,57,253,177,103,152,250,207,179,4,58,192,68,152,195,55,57,157,203,86,67,178,226,68,129,141,251,183,224,38,93,122,239,62,136,220,18,35,84,26,93,11,190,87,102,232,193,227,36,206,
75,209,240,225,205,148,38,11,213,177,78,105,2,2,2,215,208,120,67,23,227,223,233,163,48,52,133,78,130,142,91,130,208,163,255,129,195,216,102,185,168,250,6,85,171,111,38,52,248,142,160,245,119,187,25,124,
248,52,163,131,57,98,173,9,102,203,93,141,81,78,126,255,36,153,223,110,165,245,221,155,232,253,222,105,28,171,43,1,188,56,165,25,43,211,243,225,173,68,16,188,240,79,71,81,45,46,193,204,243,148,128,88,
4,47,91,13,223,230,97,214,28,146,149,184,240,214,109,0,60,255,208,115,208,20,91,210,71,117,36,57,244,233,3,68,210,46,155,62,176,133,252,112,121,73,31,47,52,132,64,231,13,221,244,62,220,75,201,24,194,69,
120,232,207,22,87,231,93,158,156,193,183,6,45,231,237,51,33,170,53,69,195,117,45,12,126,229,56,27,2,69,84,56,75,33,67,73,188,115,1,131,95,62,78,219,222,30,10,223,238,35,204,249,200,132,51,175,82,92,162,
243,206,157,4,194,48,250,143,199,104,171,139,35,244,210,89,29,213,213,46,79,122,16,245,167,247,35,223,224,157,242,217,252,228,21,4,94,200,196,131,167,73,182,39,193,175,190,1,71,211,14,19,15,158,102,195,
141,93,116,126,122,23,189,111,124,156,232,38,7,28,129,201,6,196,182,54,147,124,93,19,195,95,236,165,198,139,160,140,187,180,56,10,184,222,28,190,85,220,22,182,114,43,51,111,48,3,30,250,92,64,231,183,46,
199,214,57,12,190,231,0,110,198,69,218,74,69,173,170,88,129,155,118,25,218,119,16,181,49,74,199,183,174,32,60,94,192,12,120,96,5,173,247,95,132,9,13,147,159,57,129,83,231,46,111,71,87,187,60,57,1,209,
43,147,36,174,203,80,243,214,141,168,166,8,195,239,255,37,102,192,67,53,184,171,34,153,140,56,4,79,231,152,252,226,0,233,247,181,211,242,181,75,153,250,202,32,13,183,245,32,155,35,12,188,225,231,56,153,
56,34,88,225,179,183,164,250,38,43,128,154,184,38,195,134,155,207,103,252,75,125,140,221,117,28,147,211,168,13,209,53,35,153,104,138,49,122,247,9,74,7,166,104,186,243,2,90,30,218,78,208,91,162,255,181,
79,17,140,248,171,226,157,46,87,171,190,1,131,119,61,71,223,173,207,34,163,10,149,137,34,98,130,240,165,222,196,107,132,201,159,158,97,124,219,105,44,149,87,95,53,186,200,184,92,21,239,150,197,183,188,
46,66,28,32,128,169,242,255,45,135,137,207,175,174,249,107,170,202,45,189,60,185,46,241,173,46,69,24,134,140,230,215,7,190,205,94,158,188,248,194,45,54,85,155,34,88,103,151,39,255,119,0,54,22,35,183,31,
97,218,222,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00133_png = (const char*) resource_TalComponent_bmp00133_png;
const int TalComponent::bmp00133_pngSize = 6767;

// JUCER_RESOURCE: bmp00134_png, 8812, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00134.png"
static const unsigned char resource_TalComponent_bmp00134_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,38,0,0,0,138,8,6,0,0,0,198,227,56,59,0,0,0,9,112,72,89,115,0,0,46,35,0,0,46,
35,1,120,165,63,118,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,
21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,188,
247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,
7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,0,96,39,
127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,
0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,
121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,
128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,95,192,
87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,
68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,225,
207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,
128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,8,
23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,
55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,12,
171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,
98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,32,
43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,
175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,
40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,243,
85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,
198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,40,
167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,
5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,
88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,38,
6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,
89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,125,
97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,
227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,23,151,73,68,65,84,120,218,236,156,123,148,93,213,125,223,63,123,239,243,184,247,206,157,151,52,15,205,72,51,26,33,9,9,129,120,72,
96,192,38,24,59,141,141,137,93,74,74,19,199,6,55,177,113,28,151,229,116,21,187,184,129,196,177,49,56,203,193,56,118,169,147,149,80,59,14,206,162,43,142,107,76,236,198,171,181,221,166,5,131,121,200,128,
120,8,36,132,70,163,209,72,154,231,157,185,207,115,206,126,244,143,115,231,125,103,36,72,9,211,181,114,181,246,210,149,206,217,191,251,59,251,236,179,207,239,115,190,191,223,17,187,182,159,237,90,91,90,
73,116,194,90,250,120,255,236,221,87,243,123,159,187,131,124,190,121,77,56,52,93,40,80,152,46,224,149,74,69,148,84,116,183,54,177,121,96,224,77,117,170,80,40,240,235,31,252,32,191,250,129,27,241,112,144,
205,230,40,214,106,28,31,27,121,83,29,155,154,152,97,166,84,198,247,124,60,0,103,44,120,176,174,171,243,77,117,204,121,130,108,83,6,231,92,221,49,231,192,7,229,137,211,118,54,133,4,87,51,8,79,34,91,60,
68,32,95,151,19,182,98,176,37,13,214,33,243,30,50,239,165,191,175,4,56,82,199,112,64,8,206,119,171,26,139,95,42,209,126,203,86,178,87,180,97,6,171,76,253,201,32,122,38,70,173,243,95,147,83,102,52,34,188,
176,149,214,27,54,34,218,60,138,127,53,66,249,127,142,227,66,135,244,36,56,144,115,142,121,32,86,105,241,225,18,189,223,217,75,219,71,250,112,21,71,246,109,235,233,251,201,229,132,111,109,195,206,36,171,
246,93,216,108,41,33,188,114,29,27,238,59,31,181,33,131,204,120,116,126,241,28,50,87,173,195,85,52,182,126,2,230,29,11,192,250,162,97,75,138,9,173,55,111,38,179,179,153,226,15,79,49,124,195,83,28,185,
246,49,226,253,37,186,255,224,28,108,171,196,56,183,98,255,217,102,4,24,5,221,159,217,137,25,137,24,250,149,199,57,242,206,71,48,99,49,189,247,236,70,251,224,172,153,119,76,56,1,18,18,161,151,55,151,80,
41,85,105,191,113,51,201,96,149,225,219,246,163,246,52,65,155,228,232,167,159,65,56,75,239,61,187,41,14,22,26,247,95,208,170,147,101,218,110,222,140,151,247,25,186,229,105,196,150,0,239,226,60,71,127,
231,105,102,254,199,73,18,147,96,125,16,179,167,82,58,1,30,104,101,150,181,202,84,153,13,183,237,36,204,135,12,222,245,44,114,103,6,173,12,166,25,226,168,198,200,189,135,200,157,213,66,199,111,111,165,
58,93,110,104,67,43,67,66,66,18,90,58,175,219,204,228,247,134,40,143,204,96,242,96,178,142,68,71,28,187,231,121,92,183,196,145,206,115,111,110,216,20,148,213,226,201,111,181,37,246,4,189,255,124,128,227,
15,30,101,252,120,145,76,111,174,126,238,129,206,128,87,127,240,42,237,239,234,165,247,3,91,24,252,254,81,60,103,210,9,188,228,83,155,168,177,237,183,119,225,35,120,249,207,95,66,245,132,36,179,191,167,
4,100,124,162,130,35,17,14,129,168,159,202,217,171,110,73,43,158,172,114,206,167,119,3,112,224,190,23,160,43,179,108,31,213,151,103,255,31,237,195,111,13,217,114,211,78,74,163,181,101,251,68,218,162,129,
254,235,6,24,124,112,144,170,181,104,79,46,219,15,64,139,212,31,111,246,31,104,232,48,114,193,58,163,81,189,45,116,188,189,135,227,223,58,196,250,68,17,8,15,204,146,161,80,146,232,84,194,241,251,15,177,
241,198,109,148,191,59,132,46,198,200,156,55,191,204,12,85,233,191,227,66,18,97,25,255,211,131,108,108,203,34,204,242,81,29,55,146,156,17,88,234,11,172,21,14,34,8,226,250,122,20,91,162,35,49,91,127,118,
41,73,164,153,186,247,40,249,77,121,136,27,47,192,65,171,199,212,189,71,89,127,253,102,250,255,104,15,131,87,63,74,176,197,3,79,96,11,9,153,93,221,228,223,209,197,232,55,7,105,138,124,148,13,231,135,104,
193,39,140,2,164,149,212,175,69,210,203,64,3,37,139,29,142,48,167,18,250,191,243,22,92,155,199,241,15,238,35,108,15,145,78,32,13,141,155,19,132,173,33,35,55,63,139,218,16,208,247,157,75,209,135,202,216,
225,8,156,160,247,75,231,98,181,101,250,203,135,241,218,194,149,237,24,129,168,47,100,233,120,11,96,10,130,203,243,228,222,222,78,211,251,54,160,186,124,70,63,242,28,118,56,66,117,132,16,173,190,154,75,
223,35,121,170,200,244,55,135,105,253,240,38,122,30,184,152,153,111,29,167,227,246,109,200,110,159,225,95,122,18,175,61,139,72,86,185,237,69,169,131,136,186,99,66,10,48,144,187,162,157,245,159,220,206,
228,95,14,49,241,249,67,216,162,65,173,15,48,177,59,163,91,141,232,202,48,126,215,97,170,251,102,232,186,99,7,61,247,157,79,50,88,229,216,85,79,144,140,197,168,118,127,85,91,166,230,48,218,44,112,76,165,
195,119,252,243,47,48,244,233,103,144,129,66,181,7,136,140,64,151,95,99,100,219,9,211,127,127,130,201,221,71,113,164,151,190,234,12,145,89,137,46,199,171,118,173,149,74,152,196,32,132,88,48,98,64,201,
84,32,11,144,192,76,237,31,22,195,100,23,124,47,197,80,58,125,151,90,213,16,85,170,243,142,85,42,21,114,190,71,95,223,166,55,61,130,109,110,110,37,73,18,188,150,182,22,180,214,140,151,226,53,18,243,79,
81,40,20,16,187,119,157,227,154,243,45,232,181,70,73,73,148,96,66,131,214,102,109,57,246,238,107,126,153,219,63,251,185,181,131,111,211,5,166,11,5,188,153,233,105,148,242,214,32,190,1,217,76,246,159,240,
237,53,226,27,203,241,205,58,146,145,26,24,151,174,220,57,245,186,126,204,140,199,216,178,70,182,250,168,182,213,105,106,33,190,201,165,248,230,124,135,83,142,232,88,137,254,159,94,206,230,23,175,196,
219,158,193,148,227,249,237,103,216,226,99,101,214,253,254,89,12,188,244,118,242,191,222,67,50,81,91,189,79,112,26,124,139,14,21,233,125,96,15,206,56,244,171,85,186,191,186,27,145,19,56,109,207,24,211,
204,84,76,243,251,123,200,253,82,23,250,213,42,237,55,111,198,27,200,224,106,122,117,188,91,136,111,98,1,190,213,70,170,116,221,179,139,236,5,109,20,190,62,196,177,27,159,66,34,232,251,219,203,169,30,
47,99,4,167,197,180,36,50,200,141,62,29,159,217,137,61,30,113,240,138,159,32,181,96,227,55,247,144,68,102,85,27,14,187,196,49,9,213,201,50,225,149,109,180,190,119,35,211,127,59,194,216,183,7,177,109,130,
35,55,61,142,151,241,24,248,175,151,82,58,57,77,226,86,70,180,56,137,169,85,171,108,252,250,197,136,170,229,224,175,62,76,184,167,141,131,191,242,8,126,222,103,227,215,47,162,248,202,100,227,254,82,99,
149,155,199,55,65,26,143,209,235,49,240,165,61,36,175,204,48,116,207,243,200,109,33,182,77,80,45,84,120,245,214,125,52,109,109,165,255,11,23,80,56,50,217,24,211,164,166,52,86,100,203,159,93,74,182,37,
228,133,235,255,30,117,86,6,211,12,137,141,25,186,243,121,154,206,106,161,251,214,29,148,78,206,44,71,60,79,207,225,91,157,43,129,2,92,240,87,239,64,212,44,143,222,240,191,137,55,133,148,189,20,233,162,
206,128,225,39,78,50,252,192,97,214,189,181,155,129,63,184,136,209,145,34,101,229,22,181,147,71,103,24,184,107,47,173,219,90,216,127,235,19,148,176,84,114,114,206,198,145,31,14,50,249,240,40,189,31,216,
70,246,109,93,76,205,68,139,250,71,178,17,190,109,128,31,93,243,119,60,245,201,71,81,155,155,72,150,224,149,218,212,196,139,95,123,158,209,199,70,233,121,207,38,218,47,235,166,52,25,45,64,189,10,155,111,
216,78,215,219,186,121,245,191,28,102,228,137,81,92,123,176,216,198,230,60,251,62,251,56,229,67,211,236,252,221,11,177,89,73,45,50,243,248,38,230,241,77,46,196,183,13,185,44,185,19,154,110,233,211,145,
200,101,173,175,175,141,99,183,63,77,241,72,145,139,190,120,9,27,91,114,180,79,26,218,199,53,253,219,214,179,237,227,59,41,60,55,193,216,151,94,164,191,171,121,89,255,78,163,216,212,222,204,43,183,61,
141,193,242,150,187,46,165,117,36,161,35,18,116,36,146,92,60,143,111,18,192,9,7,9,132,54,32,35,66,2,237,55,108,161,13,104,235,104,97,240,250,159,17,215,52,91,239,123,11,97,28,16,218,12,91,238,190,144,
234,104,149,225,155,158,97,221,150,245,43,218,200,168,12,217,130,224,196,29,7,240,118,52,209,247,201,243,16,135,147,212,190,158,199,183,116,229,175,143,152,60,147,200,71,72,114,189,77,28,253,229,199,232,
255,193,101,116,127,233,92,236,164,198,42,201,241,27,159,32,219,149,77,31,210,172,98,75,182,4,68,63,158,96,250,193,17,90,174,235,161,182,127,134,218,255,153,104,128,111,18,168,1,145,56,163,219,140,16,
10,89,177,156,252,232,126,122,255,102,15,72,24,253,232,243,136,25,135,88,167,78,139,122,0,126,79,142,241,155,95,36,187,183,141,158,63,187,128,87,218,255,59,180,44,197,55,145,58,118,166,152,6,64,206,163,
246,98,153,99,87,63,137,157,76,48,101,115,90,60,91,118,111,220,145,103,228,250,159,67,226,16,221,25,204,84,185,1,190,105,136,202,229,215,118,135,14,32,57,22,129,4,17,136,211,226,89,195,136,130,52,128,
16,213,100,101,124,155,30,155,126,83,195,158,101,248,86,93,147,248,214,210,130,54,154,137,242,218,192,183,194,44,190,157,189,101,139,203,101,155,214,158,200,181,97,67,47,123,46,185,132,106,181,178,182,
28,219,117,222,121,220,114,235,127,160,171,187,123,77,56,52,54,54,202,248,216,24,124,232,134,27,93,45,113,174,126,229,190,233,237,95,253,203,235,221,179,251,95,116,243,148,4,244,245,247,190,169,163,53,
62,62,70,247,166,158,121,74,178,54,85,149,130,166,204,242,5,80,59,132,39,94,247,143,57,59,119,139,61,253,122,29,229,240,148,154,119,12,235,32,4,181,64,73,115,218,162,143,215,240,6,114,216,201,4,213,238,
191,102,167,108,73,67,98,65,10,100,40,17,153,213,17,208,11,4,40,181,156,146,156,74,209,77,79,199,152,169,152,222,31,92,194,198,239,238,161,233,221,29,152,106,50,183,253,180,205,89,226,163,101,212,150,
12,125,143,188,149,141,223,219,139,236,246,177,198,172,222,207,115,72,207,45,145,5,253,244,75,124,164,66,203,175,245,178,238,150,109,160,45,250,84,132,236,10,0,135,240,78,63,74,122,34,198,57,232,185,255,
2,50,123,219,137,15,148,80,221,1,162,73,65,33,89,209,134,179,105,148,99,235,183,199,121,199,52,196,197,132,222,191,188,144,236,158,118,226,87,42,28,189,238,49,182,238,123,7,86,27,172,47,16,254,202,115,
205,214,12,209,171,101,214,127,124,128,117,31,26,64,181,250,140,126,225,37,42,143,78,209,255,224,91,112,206,98,125,1,13,108,216,154,129,196,98,140,65,212,213,183,212,177,216,193,38,216,250,119,87,32,165,
228,196,215,14,49,113,223,97,8,5,202,83,88,229,82,180,18,13,28,179,142,248,100,21,191,63,207,214,159,92,129,191,46,67,229,217,73,134,63,127,0,51,86,35,220,213,140,20,146,88,89,140,208,136,37,87,129,158,
138,145,77,138,254,123,247,144,173,20,168,254,77,13,49,123,42,165,146,112,18,146,131,37,6,239,126,142,104,168,140,127,110,142,218,177,18,210,9,12,46,69,43,181,196,104,41,33,158,136,24,248,247,231,209,
113,109,63,174,168,57,118,207,243,140,63,56,68,216,223,132,235,81,88,207,34,157,192,202,20,207,164,74,47,0,231,28,181,225,50,109,191,176,129,129,219,206,71,2,47,254,241,203,136,254,243,230,71,76,73,1,
205,176,239,223,61,134,215,236,35,123,66,98,28,21,229,80,64,34,28,37,233,240,235,106,153,213,142,202,177,18,221,87,246,112,222,109,23,224,231,124,142,127,127,136,151,239,222,79,208,30,224,157,157,167,
2,152,154,37,163,4,202,65,69,9,106,210,161,148,195,212,12,209,104,141,157,159,218,77,207,251,250,41,31,154,102,255,109,251,152,25,61,133,221,165,16,212,195,158,185,240,186,51,131,94,168,129,215,209,206,
137,52,234,118,64,50,25,65,108,184,232,171,151,179,238,130,245,196,227,53,30,253,196,79,41,30,171,144,233,111,194,74,49,39,19,25,1,30,6,129,64,99,136,5,216,137,8,153,145,188,245,219,239,36,236,204,114,
252,251,67,188,240,213,103,9,58,114,208,30,44,86,223,140,112,16,67,71,178,248,84,69,177,34,17,16,26,65,231,12,152,35,37,58,63,186,149,190,79,236,66,11,203,177,255,116,144,145,251,94,166,121,75,51,109,
29,249,20,64,22,64,136,77,28,190,86,36,194,209,26,43,188,151,42,116,252,218,0,155,111,223,77,173,24,115,224,230,159,81,125,126,138,77,61,45,8,41,24,143,27,169,111,9,4,122,241,34,42,146,0,35,45,114,84,
147,205,100,233,251,209,47,160,186,66,166,247,141,51,114,203,179,8,41,233,216,86,127,216,167,27,44,1,137,68,25,31,131,65,28,51,108,249,226,94,242,239,222,64,241,112,145,225,15,63,137,202,120,180,118,183,
129,5,44,203,241,141,21,240,77,106,129,45,107,242,239,236,164,227,247,118,128,133,19,159,124,158,234,143,71,9,54,231,210,91,213,42,152,102,141,64,26,129,158,136,233,253,202,110,130,115,242,76,253,197,
16,19,95,56,72,184,35,191,172,191,212,141,212,183,70,248,86,3,175,47,131,200,172,163,244,208,41,198,111,127,9,175,51,36,216,180,252,180,53,196,188,8,164,148,120,155,50,184,188,199,137,27,158,33,126,118,
134,204,182,150,198,253,35,192,216,121,74,66,138,134,248,38,186,67,142,156,251,48,225,217,121,202,63,157,196,223,152,195,73,113,230,136,38,37,149,103,138,156,188,97,63,241,43,101,108,213,32,187,50,43,
246,55,53,7,218,213,29,115,233,81,173,132,111,78,59,106,63,175,32,215,73,162,215,19,229,6,48,253,243,81,132,47,210,83,183,10,91,212,74,37,244,44,190,73,41,207,12,223,74,111,124,60,182,92,125,171,174,29,
124,107,93,136,111,70,107,38,202,107,131,146,230,212,183,173,253,155,93,166,41,131,78,244,218,162,164,141,253,155,184,228,226,203,168,84,214,24,190,157,187,235,124,126,231,83,159,162,187,107,195,218,194,
183,223,184,241,95,187,90,188,70,241,205,218,53,138,111,206,218,21,241,237,31,243,179,12,223,156,157,85,223,228,155,59,225,87,74,158,76,38,151,231,90,136,64,158,49,83,186,216,98,166,150,175,135,178,73,
33,243,222,105,251,152,88,207,169,111,243,142,21,96,227,127,219,139,62,85,91,228,84,116,176,204,228,31,30,34,216,146,59,173,83,178,73,210,245,229,11,82,208,157,117,42,171,40,252,245,8,209,195,19,168,245,
193,138,125,112,16,255,245,75,232,138,94,34,114,169,217,235,66,204,53,51,109,104,121,239,6,218,111,57,139,164,172,87,87,220,74,154,245,119,236,68,229,124,92,228,192,9,156,1,87,117,108,248,194,57,120,187,
155,209,198,173,220,39,118,32,236,156,250,230,205,137,92,61,112,228,55,159,88,116,68,241,80,153,243,15,189,151,164,24,175,140,111,179,196,36,18,108,89,19,77,197,28,253,216,83,120,221,25,92,205,224,109,
202,178,253,219,111,35,73,18,244,18,124,91,212,231,183,158,162,184,174,6,31,82,243,248,38,234,143,8,232,245,150,68,160,106,85,124,91,116,16,158,6,39,16,190,194,110,84,208,229,225,34,129,232,168,219,88,
130,111,141,250,152,208,45,73,158,116,162,97,242,228,74,248,214,48,100,81,46,85,88,128,162,114,100,148,195,72,215,16,223,86,234,179,80,125,243,230,212,55,150,103,231,53,194,183,21,71,12,230,230,71,92,
159,188,141,240,77,173,210,103,161,250,230,57,220,25,225,91,87,44,241,146,85,78,101,34,81,245,67,236,74,36,65,34,23,225,91,155,86,52,37,50,141,150,87,232,51,35,22,224,155,20,242,180,248,230,197,130,160,
164,80,171,204,49,81,244,112,38,229,135,76,18,224,105,127,14,223,172,180,120,85,133,40,42,196,130,131,91,218,39,146,103,168,190,73,45,32,16,200,156,34,232,202,173,184,72,166,60,163,16,82,32,155,36,82,
167,137,150,214,8,148,144,88,231,8,218,67,116,34,22,165,217,207,246,17,161,72,251,168,51,84,223,4,146,218,207,166,104,255,216,0,237,55,111,57,3,213,74,48,254,111,15,32,125,15,34,129,48,2,51,24,97,199,
34,186,254,227,121,179,143,9,151,172,204,142,194,189,67,8,161,26,36,79,174,164,190,181,250,140,125,250,32,94,87,128,75,78,143,108,34,43,169,62,59,131,215,25,214,109,9,108,197,112,226,99,47,32,51,10,204,
114,27,34,148,36,71,202,184,140,194,76,185,51,87,223,92,217,225,198,207,92,234,147,57,137,41,47,14,211,227,99,85,156,89,217,134,8,37,194,242,6,171,111,165,215,223,231,255,19,245,173,181,117,141,169,111,
5,166,11,83,136,115,182,111,119,173,45,109,36,201,26,83,223,116,162,73,146,100,237,57,246,174,247,92,195,103,239,250,67,242,249,252,26,33,241,122,237,219,204,244,52,74,74,186,90,114,107,48,121,242,159,
106,223,78,3,51,139,146,39,197,242,218,55,91,49,216,153,229,23,131,200,168,211,38,64,254,67,62,139,106,223,156,77,193,97,46,121,178,98,8,46,106,166,245,195,125,184,234,124,184,33,124,65,245,137,25,138,
15,12,35,223,32,231,86,77,158,180,21,77,235,111,246,161,90,3,92,146,62,168,197,56,156,146,180,127,98,0,91,58,243,58,183,215,211,102,147,39,231,97,100,182,246,45,16,216,154,193,12,149,25,249,232,51,120,
221,25,76,33,161,237,253,189,100,207,111,197,4,2,229,139,55,100,196,22,38,79,122,96,23,213,190,105,145,128,16,136,80,225,122,20,174,91,225,114,22,215,172,144,8,202,227,69,146,156,157,51,230,175,11,151,
73,211,46,177,56,187,114,52,33,195,198,74,111,138,136,174,142,111,78,164,81,183,4,45,13,177,210,245,84,98,210,239,50,77,138,148,237,62,148,13,91,191,113,105,26,91,1,178,201,227,213,207,60,141,180,2,81,
127,238,97,202,9,94,71,22,153,85,13,157,147,161,162,114,96,138,96,93,166,161,99,78,45,197,55,31,202,158,163,230,57,92,29,108,139,158,35,227,57,88,239,243,210,159,191,136,11,37,185,109,45,152,216,98,34,
67,251,182,28,173,255,162,143,225,239,28,193,207,4,216,196,226,119,101,56,247,203,151,32,3,15,217,64,228,183,198,241,242,151,159,99,242,201,49,188,166,197,161,122,228,57,34,28,18,129,103,89,94,251,182,
20,195,0,188,205,121,14,220,255,50,46,78,183,153,99,37,222,179,255,122,146,233,132,90,100,48,53,131,141,12,178,205,199,11,60,158,189,99,31,181,241,42,210,159,63,109,181,177,42,87,124,227,29,72,79,82,58,
82,68,181,44,126,150,65,104,176,117,103,60,49,171,190,213,107,223,18,45,81,117,10,237,214,18,127,97,141,90,243,124,233,76,84,6,157,149,228,51,30,27,187,154,81,77,30,46,182,248,45,33,182,73,18,142,37,136,
145,120,209,124,146,199,106,184,78,159,45,31,57,155,29,183,156,187,200,39,191,43,203,35,215,126,143,76,197,46,175,125,19,67,22,121,210,97,99,135,8,36,65,28,224,197,141,215,44,225,231,152,250,246,16,29,
55,109,165,243,166,109,11,158,190,9,38,31,24,194,43,73,148,159,155,139,142,1,194,141,33,7,175,252,95,180,191,191,127,209,19,33,0,153,247,16,99,22,79,120,117,124,179,22,39,129,147,208,247,232,94,244,80,
21,175,47,135,45,39,40,45,80,43,8,89,170,37,96,234,79,7,41,255,112,108,238,244,206,62,186,138,7,203,72,95,33,130,37,234,156,16,168,192,167,120,255,177,198,7,155,24,132,84,245,201,47,22,76,50,0,95,98,203,
9,227,183,190,132,106,10,86,77,168,148,210,35,57,92,93,254,255,190,151,46,65,13,146,40,5,2,149,13,26,59,86,147,13,106,223,60,24,190,230,169,249,17,89,31,32,2,177,104,52,26,91,19,13,133,49,135,123,237,
11,236,194,218,55,11,72,41,64,131,105,153,55,102,146,8,254,145,131,218,90,169,146,230,94,204,225,155,74,143,186,48,54,245,166,134,61,213,106,66,84,170,227,155,64,80,169,84,215,30,190,53,183,180,224,156,
99,170,186,86,212,183,233,84,125,235,233,232,116,173,109,107,16,223,186,55,116,211,220,220,76,178,214,100,193,171,126,241,23,249,221,223,255,28,205,205,107,163,246,45,37,241,2,222,204,204,12,158,82,107,
181,246,237,141,199,55,91,172,43,30,205,222,107,192,55,251,255,6,223,204,68,60,119,215,112,177,197,76,196,233,247,200,146,127,127,154,103,91,122,232,212,138,206,45,196,183,180,52,99,1,190,205,54,169,45,
246,88,21,169,45,202,19,184,241,8,55,30,53,220,166,60,1,211,9,157,119,158,77,231,157,103,227,198,34,252,77,33,253,63,186,4,127,83,8,211,154,230,107,59,105,190,182,19,51,92,153,179,211,168,205,215,190,
89,150,213,190,233,233,8,185,49,96,123,241,106,188,29,57,226,227,101,122,31,218,75,239,67,123,137,143,151,241,118,228,230,182,89,99,176,198,160,11,17,94,79,128,215,19,96,173,65,4,2,127,91,19,88,135,41,
71,216,162,38,62,88,166,247,187,123,232,125,104,111,227,58,184,5,248,230,45,195,183,146,38,247,190,110,130,46,159,232,96,17,151,164,245,110,250,68,170,202,9,47,133,141,100,164,154,66,135,54,132,23,181,
210,244,222,110,204,104,132,108,243,211,4,73,9,102,50,33,216,153,39,188,168,5,91,72,82,129,225,68,4,174,110,199,88,244,201,26,222,134,76,154,142,186,180,246,109,225,171,75,116,108,201,191,171,131,220,
213,221,216,162,65,91,131,118,110,78,180,54,129,192,8,139,112,96,132,197,88,71,176,179,137,246,223,234,71,79,107,28,80,155,138,72,42,49,84,45,222,174,28,237,31,219,156,110,155,21,246,128,164,170,145,253,
33,59,78,94,131,218,146,153,83,230,230,241,77,44,120,117,137,75,136,227,26,166,100,80,37,131,104,18,208,34,17,91,67,108,236,144,161,36,138,107,40,215,132,112,2,45,28,81,92,67,71,6,91,72,95,212,146,140,
70,244,63,112,49,149,23,167,177,37,131,141,44,118,42,193,202,52,183,9,72,35,228,173,33,54,128,100,176,76,146,88,180,72,22,215,190,57,155,82,9,18,180,111,17,219,115,233,56,90,151,230,142,93,213,201,182,
111,93,70,124,178,130,173,105,196,246,28,174,5,136,28,54,72,255,109,164,65,88,112,206,98,106,154,204,150,60,162,201,3,237,176,164,184,19,118,134,132,93,25,28,142,248,100,133,109,223,186,140,252,85,157,
184,146,193,200,132,196,91,169,246,205,64,210,157,225,188,111,92,65,208,17,226,180,67,133,138,56,50,232,83,17,70,10,170,99,17,231,253,197,21,52,95,213,131,157,138,49,157,233,254,178,51,151,46,15,82,96,
164,192,213,44,218,186,244,233,130,20,100,59,51,60,254,111,30,230,241,143,63,76,216,215,132,203,40,244,169,136,184,106,144,117,101,174,36,83,245,45,18,233,64,205,215,190,9,72,226,132,100,180,150,70,160,
206,81,26,44,98,157,67,8,129,173,255,73,70,107,196,245,7,201,58,49,36,163,53,116,162,17,48,183,143,0,204,162,191,5,116,101,16,155,114,212,70,43,84,71,43,72,95,206,109,155,85,230,82,48,90,48,249,103,107,
223,218,180,74,17,61,144,36,163,53,14,254,198,35,112,50,130,80,144,107,11,201,181,133,88,32,139,196,101,36,121,155,10,100,185,156,143,236,14,9,157,32,116,2,11,100,108,186,18,101,165,66,103,5,221,90,177,
177,57,203,137,175,28,96,236,43,7,144,237,1,89,36,54,43,105,179,30,157,90,146,51,141,106,223,106,224,27,159,217,8,95,6,138,117,45,173,132,158,135,88,231,51,126,247,97,198,239,62,140,236,240,17,90,18,29,
41,226,37,30,162,197,163,242,227,113,134,62,248,20,126,87,22,97,37,78,184,244,130,242,129,25,75,52,88,34,43,50,4,177,79,115,83,158,214,108,51,214,115,8,45,169,29,41,226,199,105,221,219,178,87,151,204,
171,111,2,132,67,52,43,84,94,33,180,64,134,30,106,67,0,195,17,12,71,200,102,15,51,26,115,252,250,167,112,39,227,52,185,123,198,162,95,42,17,12,228,240,55,132,248,103,229,240,90,125,68,62,117,122,248,250,
39,9,58,50,243,175,40,209,2,17,202,5,118,106,40,37,23,215,190,89,103,231,212,55,97,5,254,64,150,233,63,57,138,30,142,240,154,3,226,231,74,76,127,109,8,213,26,98,167,18,84,135,143,106,242,240,218,50,72,
79,161,58,125,164,84,4,61,57,70,174,219,151,206,147,66,66,176,187,133,182,155,250,81,57,31,191,61,179,24,3,107,204,219,105,207,32,92,189,94,110,161,250,38,16,105,117,130,6,61,145,80,248,207,195,76,223,
127,2,83,212,168,246,128,234,51,51,148,31,43,160,218,211,180,133,201,175,12,18,61,83,196,5,10,61,145,48,249,199,131,232,137,36,77,198,92,23,166,211,160,197,199,25,22,111,91,160,236,185,156,55,103,199,
10,145,38,161,211,224,213,37,179,114,112,229,212,12,165,59,39,145,205,222,226,90,182,128,244,123,0,39,238,124,1,145,81,200,172,36,62,85,155,223,223,91,12,184,174,236,86,220,182,212,14,229,89,124,91,160,
190,89,44,162,238,216,204,212,76,61,48,58,3,181,108,161,202,182,218,254,83,103,110,103,177,250,102,215,86,242,228,34,245,205,24,195,100,37,89,35,49,255,20,133,169,2,98,199,192,86,151,111,107,70,175,49,
124,251,191,3,0,20,226,211,195,166,100,116,240,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00134_png = (const char*) resource_TalComponent_bmp00134_png;
const int TalComponent::bmp00134_pngSize = 8812;

// JUCER_RESOURCE: bmp00135_png, 53534, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00135.png"
static const unsigned char resource_TalComponent_bmp00135_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,37,0,0,7,58,8,6,0,0,0,242,185,174,195,0,0,0,9,112,72,89,115,0,0,14,195,0,0,
14,195,1,199,111,168,100,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,
82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,123,163,107,214,
188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,
34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,98,227,0,80,45,
0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,
0,8,12,0,48,81,136,133,41,0,4,123,0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,30,40,206,73,23,43,20,54,97,2,97,154,64,
46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,
179,26,128,59,6,128,109,254,162,37,238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,202,87,125,254,103,194,
95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,
142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,104,131,
225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,
100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,88,35,142,
8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,
168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,
24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,
82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,
103,40,25,155,24,7,24,103,25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,47,84,169,170,166,170,222,170,11,85,
243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,
188,198,32,11,99,25,179,120,44,33,107,13,171,134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,116,78,9,231,
40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,
206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,
12,88,6,179,12,36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,140,105,198,92,227,36,227,109,198,109,198,163,
38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,
166,89,238,182,188,110,133,90,57,89,165,88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,182,109,182,
125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,
51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,
175,238,54,238,105,238,135,220,159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,215,176,183,165,119,170,247,
97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,
124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,
12,39,133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,
157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,
22,140,37,242,19,119,37,142,10,121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,76,13,76,221,155,58,158,22,
154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,
40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,
227,21,5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,
157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,
13,223,86,180,237,245,246,69,219,47,151,205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,188,246,52,236,
213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,
143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,
83,154,251,91,98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,46,249,220,96,219,162,182,123,231,99,206,
223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,
122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,
237,65,217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,235,158,56,62,57,57,226,63,114,253,233,252,167,
67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,
237,193,119,220,119,29,239,163,223,15,79,228,124,32,127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,131,0,0,249,255,0,
0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,198,73,73,68,65,84,120,218,236,221,251,87,91,231,157,47,254,189,181,165,173,11,18,66,8,132,16,2,33,4,24,99,108,16,66,32,132,16,8,
113,7,115,7,113,191,139,139,140,1,27,99,130,141,49,33,177,137,19,55,113,210,201,164,105,166,105,154,38,105,156,139,227,118,205,156,63,98,206,89,107,126,152,94,214,153,181,206,233,180,171,147,52,205,197,
109,218,243,93,77,115,194,251,251,67,216,79,246,150,192,230,34,201,233,89,249,129,149,222,226,60,149,196,214,179,95,251,253,188,63,212,79,127,250,83,234,32,63,20,69,209,20,69,169,41,138,50,81,20,117,146,
162,168,18,138,162,202,69,34,145,151,162,168,242,157,127,127,114,231,191,87,83,20,69,31,248,159,113,128,197,200,41,138,178,80,20,85,77,81,212,50,69,81,175,81,20,245,175,20,69,189,175,86,171,255,22,23,
23,7,133,66,241,55,138,162,222,223,249,207,95,219,249,223,85,239,252,125,242,176,45,138,162,40,150,162,168,44,138,162,70,41,138,122,131,162,168,15,41,138,250,146,162,40,196,199,199,163,176,176,16,19,19,
19,24,24,24,64,93,93,29,140,70,35,20,10,5,40,138,194,206,255,238,195,157,191,111,116,231,207,97,143,180,40,138,162,180,20,69,181,81,20,245,99,138,162,62,162,40,10,52,77,67,161,80,160,172,172,12,131,131,
131,184,124,249,50,166,167,167,209,211,211,131,230,230,102,212,215,215,195,110,183,195,108,54,131,101,89,208,52,205,45,240,163,157,63,167,141,162,40,237,161,22,69,81,148,145,162,168,243,20,69,253,130,
166,233,109,133,66,129,184,184,56,120,60,30,140,142,142,98,101,101,5,129,64,0,131,131,131,104,107,107,67,107,107,43,90,90,90,208,216,216,136,186,186,58,212,212,212,160,184,184,24,199,143,31,135,66,161,
128,84,42,5,77,211,219,20,69,253,98,231,207,53,30,104,81,59,31,210,77,177,88,252,59,163,209,8,155,205,134,193,193,65,204,205,205,97,113,113,17,211,211,211,24,24,24,64,123,123,187,96,65,45,45,45,56,125,
250,52,154,155,155,81,87,87,135,186,186,58,212,214,214,162,166,166,6,229,229,229,72,79,79,135,70,163,129,72,36,250,29,69,81,155,20,69,153,246,181,168,157,87,104,211,98,177,252,159,238,238,110,44,44,44,
96,121,121,25,129,64,0,67,67,67,232,233,233,65,123,123,251,125,23,196,253,52,53,53,161,177,177,17,13,13,13,168,171,171,67,99,99,35,60,30,15,114,114,114,160,80,40,254,207,206,194,140,247,93,212,206,103,
232,188,68,34,249,93,103,103,39,46,93,186,132,137,137,9,248,124,62,116,116,116,144,197,180,183,183,163,181,181,21,61,61,62,204,204,4,112,102,110,30,51,179,115,24,29,157,128,207,215,143,214,214,118,178,
32,110,81,245,245,245,168,169,169,129,215,235,133,195,225,128,66,161,0,77,211,191,219,121,43,181,187,46,106,231,183,172,141,162,168,95,164,165,165,225,218,181,107,152,154,154,66,103,103,103,200,130,198,
198,198,241,189,23,95,194,139,223,255,33,158,186,121,11,87,55,174,225,226,202,26,230,231,151,224,159,58,131,225,225,9,116,117,247,161,189,189,19,141,141,141,168,175,175,39,111,103,93,93,29,220,110,55,
148,74,37,247,75,240,11,154,166,219,196,98,49,187,219,162,178,40,138,250,49,77,211,219,167,78,157,194,245,235,215,67,94,161,190,190,62,188,243,206,59,184,123,247,46,238,220,185,139,31,190,242,58,158,126,
250,31,176,241,232,117,92,92,89,195,220,217,243,152,152,156,197,224,224,24,186,187,7,208,222,222,131,230,211,29,104,104,104,20,44,170,186,186,26,38,147,137,251,173,220,166,105,250,199,52,77,103,9,22,181,
115,97,28,165,40,234,35,165,82,137,169,169,41,44,45,45,9,22,52,59,59,139,215,95,127,29,119,239,222,221,249,249,41,94,127,227,109,60,251,220,11,216,220,220,194,197,149,53,156,153,91,196,216,248,52,250,
250,135,209,217,217,135,150,150,78,52,55,183,163,177,177,5,117,117,13,130,15,254,201,147,39,193,93,98,40,138,250,72,44,22,143,42,20,10,57,127,81,150,157,11,28,180,90,45,214,214,214,48,58,58,42,120,133,
132,11,250,234,231,206,123,63,197,11,223,251,1,30,191,118,3,23,87,214,16,8,44,96,120,100,18,221,61,131,104,107,239,70,243,233,14,52,54,181,161,177,177,21,141,141,45,104,104,104,68,77,77,13,106,106,106,
80,86,86,134,152,152,24,238,213,2,77,211,111,48,12,99,217,89,15,69,239,124,21,124,72,211,52,202,203,203,177,190,190,14,159,207,71,22,197,189,101,187,253,188,252,195,215,176,181,117,19,23,87,214,48,51,
59,143,193,193,49,116,118,246,225,116,75,231,215,11,106,106,67,115,115,59,154,79,119,144,183,207,227,241,192,104,52,146,69,81,20,245,97,108,108,108,181,193,96,160,169,157,47,205,101,138,162,190,84,42,
149,56,123,246,44,230,230,230,200,130,166,167,103,246,92,208,221,187,119,241,230,155,111,227,230,205,103,113,113,101,13,126,127,0,62,223,16,90,219,186,209,212,220,254,245,130,78,119,160,165,165,19,237,
237,61,232,232,236,132,215,235,133,215,235,133,205,102,227,46,170,160,40,234,75,133,66,177,172,86,171,213,220,133,242,53,138,162,80,88,88,136,107,215,174,97,112,112,144,252,218,127,239,197,151,112,231,
206,87,159,161,221,23,246,83,60,247,220,247,112,113,101,13,99,227,211,232,236,236,219,249,28,181,162,169,185,29,167,91,58,209,214,222,141,206,206,62,116,119,15,160,179,179,23,94,175,23,85,85,85,40,47,
47,135,193,96,128,72,36,226,222,194,215,104,154,54,81,59,219,140,127,77,76,76,196,220,220,28,230,231,231,201,133,177,167,199,135,23,191,255,67,252,240,149,215,241,250,27,111,227,206,123,63,221,243,45,
188,184,178,134,193,193,49,180,180,116,162,177,177,21,205,205,237,104,109,219,89,76,207,32,250,250,135,49,56,56,134,225,225,9,156,62,221,130,170,170,42,120,60,30,20,20,20,240,191,35,255,117,103,61,84,
9,69,81,239,231,231,231,99,107,107,75,240,93,54,51,19,192,83,55,111,225,233,167,255,1,207,62,247,2,94,248,222,15,240,242,15,95,195,155,111,190,45,120,229,110,223,126,27,23,87,214,208,217,213,143,198,166,
54,180,180,116,162,179,179,15,62,223,208,87,11,25,153,196,216,248,52,38,38,103,225,159,58,131,142,142,46,120,60,30,120,60,30,148,149,149,33,37,37,133,91,212,251,59,235,161,202,53,26,205,223,206,156,57,
131,133,133,5,193,87,199,153,185,121,92,221,184,134,141,71,175,99,115,115,11,143,95,187,129,173,173,155,184,121,243,89,60,247,220,247,240,242,15,95,195,237,219,111,227,238,221,187,184,184,178,134,211,
45,157,232,236,234,199,224,224,24,198,198,167,225,247,7,48,51,59,143,64,96,1,103,230,22,49,119,246,60,230,231,151,48,56,56,2,143,199,131,202,202,74,84,84,84,32,39,39,135,123,251,254,70,81,84,57,37,18,
137,188,137,137,137,152,156,156,196,240,240,176,224,187,108,102,118,14,23,87,214,246,253,179,178,207,31,191,127,134,44,200,237,118,227,212,169,83,252,75,131,151,162,40,170,92,173,86,255,45,47,47,15,19,
19,19,130,47,215,209,209,9,204,207,47,97,238,236,121,156,153,91,68,32,176,128,153,217,121,248,253,1,140,141,79,127,245,235,223,213,143,150,150,78,172,172,172,145,183,242,229,31,190,134,231,158,251,30,
110,222,124,22,91,91,55,241,248,181,27,216,220,220,194,198,163,215,113,117,227,26,252,83,179,100,65,110,183,155,127,105,248,219,206,150,250,171,207,148,201,100,66,123,123,59,58,59,59,201,183,189,207,215,
15,255,212,25,76,76,206,98,108,124,26,195,35,147,24,28,28,131,207,55,68,174,216,141,77,109,232,236,234,199,202,202,26,121,43,185,223,202,55,223,252,106,129,47,124,239,7,120,246,185,23,240,244,211,255,
128,167,110,222,194,200,232,56,89,80,97,97,33,196,98,49,183,40,242,153,58,73,81,212,191,106,52,26,228,230,230,162,191,191,159,108,61,90,91,219,49,60,60,129,193,193,49,244,245,15,163,187,103,16,157,157,
125,104,109,235,254,234,215,126,231,67,61,56,56,134,149,149,53,188,252,195,215,118,253,237,188,243,222,87,95,73,63,124,229,117,188,248,253,31,162,189,189,3,110,183,27,46,151,139,255,29,40,248,237,35,215,
169,244,244,116,180,181,181,161,189,189,157,236,135,186,186,185,235,75,31,218,218,187,113,186,165,243,171,11,227,206,85,186,179,179,15,99,227,211,88,89,89,195,115,207,125,239,190,215,179,59,119,238,226,
249,231,191,7,183,219,141,242,242,114,20,22,22,242,183,203,96,24,230,53,177,88,108,18,92,209,21,10,5,172,86,43,124,62,31,217,15,181,182,181,163,189,189,231,171,47,87,238,187,172,169,13,77,59,215,33,159,
111,8,126,127,0,43,43,107,184,121,243,217,157,203,197,222,223,0,51,51,51,40,47,47,223,237,85,250,50,57,57,121,217,98,177,168,67,190,251,50,50,50,48,56,56,136,211,167,79,147,77,90,111,223,16,230,231,207,
97,110,110,145,44,234,244,206,181,104,112,112,12,51,179,243,88,89,89,195,214,214,205,61,223,194,187,119,239,226,237,183,223,134,199,227,65,121,121,57,138,139,139,33,149,74,5,223,125,153,153,153,213,181,
181,181,116,200,46,65,163,209,160,181,181,21,157,157,157,104,104,104,64,111,111,47,54,54,54,112,247,238,93,188,250,234,143,49,55,183,136,230,211,29,104,107,239,70,119,207,32,134,71,38,17,8,44,96,101,101,
13,143,95,187,129,23,190,247,131,93,175,252,63,254,241,143,209,212,212,68,94,37,110,235,194,253,200,100,178,55,84,42,149,101,215,253,148,66,161,192,241,227,199,49,50,50,34,88,16,247,51,63,127,142,92,177,
251,250,135,49,54,62,141,51,115,139,88,89,89,195,230,230,22,158,125,238,5,188,254,134,240,138,255,227,31,255,24,3,3,3,100,65,46,151,11,201,201,201,252,69,125,148,144,144,48,122,242,228,73,249,158,59,79,
179,217,140,241,241,113,172,174,174,10,22,180,177,177,129,222,190,33,180,183,247,160,187,123,0,131,131,99,152,152,156,197,220,217,243,88,89,89,195,198,163,215,241,244,211,255,128,31,190,242,250,206,151,
248,87,111,25,255,21,114,185,92,40,46,46,6,203,178,100,231,25,27,27,251,99,157,78,151,117,223,61,186,94,175,71,125,125,61,158,121,230,25,178,160,171,87,175,162,167,167,7,117,117,117,240,249,122,209,217,
217,139,225,225,9,248,167,206,96,126,126,9,43,43,107,184,186,113,13,79,221,188,133,23,191,255,67,60,255,252,247,48,51,51,67,62,67,220,130,202,202,202,200,215,10,69,81,16,139,197,191,72,77,77,109,115,185,
92,236,125,239,102,196,98,241,239,82,83,83,49,58,58,138,39,159,124,18,171,171,171,232,233,233,65,77,77,13,170,171,171,201,126,232,244,233,22,116,116,116,97,112,112,4,126,255,12,252,83,179,24,25,29,39,
215,161,242,242,242,144,87,40,43,43,11,74,165,146,91,212,239,36,18,201,121,141,70,163,221,215,125,95,82,82,210,255,209,104,52,208,235,245,48,155,205,176,90,173,112,185,92,240,120,60,100,63,196,109,63,
248,95,174,220,149,154,91,140,221,110,71,110,110,46,146,147,147,161,80,40,200,222,137,162,168,253,221,247,5,223,33,139,68,162,223,197,196,196,64,165,82,65,161,80,64,46,151,195,108,54,195,102,179,193,235,
245,238,185,32,151,203,5,171,213,10,163,209,136,152,152,24,208,52,45,248,161,40,234,96,119,200,123,89,2,203,178,144,74,165,136,141,141,133,82,169,132,201,100,66,97,97,33,170,171,171,5,175,78,97,97,33,
146,147,147,201,54,55,104,49,219,98,177,248,23,10,133,226,224,150,192,255,140,137,68,162,54,154,166,5,234,34,145,72,160,86,171,161,82,169,144,146,146,2,155,205,134,252,252,124,164,166,166,242,247,220,
130,175,16,138,162,62,138,141,141,253,113,78,78,78,91,240,103,232,192,62,37,22,139,89,154,166,179,88,150,29,165,105,90,224,83,114,185,28,177,177,177,144,201,100,100,75,203,123,85,136,79,201,100,178,55,
210,210,210,70,117,58,93,22,255,183,236,200,146,167,80,40,228,12,195,88,226,227,227,171,21,10,197,50,77,211,68,242,164,82,233,223,36,18,9,68,34,17,145,60,134,97,94,75,78,78,94,182,90,173,213,42,149,202,
194,93,24,195,202,139,220,143,193,96,160,213,106,181,122,231,174,131,152,231,206,142,145,152,167,88,44,54,89,44,22,117,109,109,109,228,204,243,65,16,75,81,84,197,55,10,98,105,154,254,124,231,203,252,11,
165,82,249,233,67,133,88,14,99,211,211,211,225,243,249,208,223,223,143,154,154,26,100,100,100,112,223,109,17,135,216,143,41,138,2,203,178,144,203,229,72,77,77,133,215,235,197,196,196,4,22,22,22,48,55,
55,135,254,254,126,180,182,182,194,235,245,34,47,47,15,58,157,78,112,73,8,27,196,74,36,146,255,169,82,169,182,181,90,45,74,74,74,80,93,93,141,233,233,105,76,79,79,99,113,113,17,147,147,147,232,237,237,
69,123,123,59,185,19,170,175,175,71,117,117,53,92,46,23,44,22,11,255,187,238,203,35,67,172,201,100,250,99,115,115,51,206,156,57,131,153,153,25,156,59,119,14,129,64,0,227,227,227,240,249,124,232,238,238,
222,211,61,155,154,154,4,156,152,153,153,201,95,220,111,14,5,177,12,195,124,208,214,214,134,141,141,13,76,78,78,162,175,175,79,128,176,251,133,88,142,126,106,107,107,81,90,90,138,227,199,143,67,163,209,
128,166,233,15,14,4,177,20,69,253,38,61,61,29,155,155,155,152,158,158,222,213,61,15,10,177,220,171,198,129,153,197,98,129,92,46,255,227,190,33,150,162,40,212,215,215,99,99,99,99,87,25,62,10,196,114,139,
227,217,212,255,124,32,196,82,20,245,165,193,96,192,218,218,26,2,129,64,68,32,182,182,182,22,197,197,197,220,247,228,131,33,150,162,40,184,221,110,60,246,216,99,232,235,235,139,24,196,186,221,110,254,
131,165,143,239,11,177,41,41,41,120,228,145,71,16,8,4,34,10,177,85,85,85,130,235,216,158,16,75,81,20,26,27,27,177,177,177,129,254,254,254,136,66,172,215,235,229,187,212,222,16,107,177,88,112,237,218,53,
248,253,254,168,64,172,213,106,229,47,42,20,98,149,74,37,198,198,198,176,188,188,140,206,206,206,168,64,172,203,229,226,127,174,66,33,214,233,116,98,107,107,11,35,35,35,81,131,216,138,138,10,164,167,167,
239,74,65,37,9,9,9,159,14,13,13,9,100,56,90,16,155,159,159,207,223,66,127,13,177,58,157,238,139,182,182,54,76,78,78,70,29,98,157,78,39,226,227,227,185,87,234,115,142,23,43,40,138,130,201,100,194,248,248,
56,218,218,218,162,14,177,102,179,153,255,22,86,80,59,251,235,207,141,70,35,122,123,123,209,217,217,25,117,136,229,217,194,23,2,136,213,106,181,40,45,45,197,240,240,112,84,33,182,188,188,156,47,122,159,
10,32,150,162,40,162,120,45,45,45,81,129,216,242,242,114,242,65,223,19,98,141,70,35,6,7,7,209,209,209,17,21,136,117,58,157,130,175,154,61,33,214,227,241,96,104,104,136,236,135,218,219,59,49,60,50,129,
177,241,41,180,183,119,135,13,98,203,203,203,113,236,216,177,7,67,44,203,178,200,204,204,196,228,228,36,26,27,27,209,214,214,134,217,217,89,188,244,210,75,120,245,199,63,198,218,218,122,216,32,182,168,
168,72,112,53,191,47,196,90,44,22,76,77,77,161,167,167,7,231,206,157,195,27,111,188,65,254,208,167,159,190,133,211,59,116,125,84,136,205,200,200,216,55,196,126,156,152,152,136,241,241,113,92,188,120,81,
240,135,190,244,210,75,24,27,243,227,116,75,231,145,33,214,110,183,67,38,147,237,31,98,99,99,99,183,235,234,234,240,196,19,79,144,63,244,214,173,91,24,27,27,67,99,99,19,154,79,119,28,25,98,51,51,51,5,
159,165,7,66,172,68,34,249,159,38,147,9,253,253,253,184,113,227,6,54,54,54,48,56,56,72,118,140,71,133,216,226,226,98,254,173,214,254,33,54,33,33,225,143,90,173,22,122,189,30,249,249,249,112,185,92,36,
22,114,20,136,45,43,43,11,254,141,251,205,129,32,150,166,233,15,98,99,99,17,23,23,7,149,74,133,204,204,76,148,150,150,162,182,182,118,223,16,203,45,168,184,184,24,249,249,249,176,88,44,252,207,210,7,135,
130,216,157,59,89,196,196,196,144,197,229,228,228,16,33,222,107,65,14,135,3,167,78,157,66,86,86,22,180,90,45,20,10,5,24,134,225,95,185,255,120,100,136,229,148,69,169,84,130,99,236,130,130,2,84,87,87,147,
5,113,34,108,54,155,17,27,27,11,137,68,2,154,166,193,48,12,24,134,129,72,36,2,77,211,219,98,177,248,127,134,29,98,41,138,66,92,92,28,226,226,226,96,177,88,144,159,159,143,227,199,143,67,167,211,145,133,
112,139,9,242,207,143,35,14,177,44,203,66,163,209,64,38,147,65,44,22,131,166,105,136,68,34,62,222,63,60,136,149,74,165,159,138,197,226,47,120,59,199,111,14,196,6,155,231,183,16,187,27,196,82,20,245,215,
157,183,239,175,34,145,232,247,10,133,226,23,98,177,248,221,135,6,177,12,195,64,161,80,32,45,45,13,54,155,13,221,221,221,240,122,189,220,205,64,228,32,150,97,152,55,105,154,254,56,38,38,6,82,169,20,73,
73,73,72,79,79,135,205,102,67,109,109,45,206,158,61,139,241,241,113,44,44,44,96,116,116,20,157,157,157,240,120,60,200,204,204,228,239,153,194,7,177,9,9,9,191,117,58,157,219,29,29,29,232,238,238,70,32,
16,192,200,200,8,22,23,23,49,49,49,129,64,32,128,225,225,97,116,119,119,11,238,132,234,234,234,80,94,94,142,188,188,60,168,213,106,238,82,113,116,136,165,40,234,55,165,165,165,184,126,253,58,102,102,102,
48,54,54,134,129,129,1,244,244,244,160,187,187,27,237,237,237,232,232,232,216,211,61,235,234,234,80,81,81,129,226,226,98,36,37,37,241,115,82,135,131,88,138,162,222,55,26,141,88,95,95,199,217,179,103,119,
117,207,253,38,98,185,133,229,231,231,67,173,86,243,115,45,7,131,88,138,162,208,209,209,129,141,141,13,244,245,245,29,9,98,185,109,79,69,69,5,138,138,138,112,252,248,113,110,79,245,155,125,67,172,72,36,
66,122,122,58,214,215,215,49,53,53,21,54,136,229,16,214,227,241,192,102,179,65,175,215,131,101,217,223,238,11,98,149,74,37,166,167,167,177,186,186,42,136,86,134,51,17,91,81,81,129,99,199,142,113,223,155,
111,222,23,98,37,18,9,220,110,55,46,95,190,140,225,225,225,136,65,108,77,77,13,178,179,179,247,7,177,233,233,233,184,126,253,58,206,156,57,131,174,174,174,136,65,108,117,117,53,10,11,11,9,47,238,9,177,
6,131,1,11,11,11,88,90,90,34,11,138,36,196,218,237,246,7,39,98,251,251,251,177,190,190,78,194,167,145,134,88,151,203,5,134,97,200,222,75,0,177,50,153,236,221,162,162,34,60,249,228,147,24,31,31,23,252,
218,71,58,17,203,191,221,18,64,172,90,173,254,197,233,211,167,113,238,220,57,193,133,49,26,137,88,173,86,187,59,196,50,12,243,251,130,130,2,18,100,142,22,196,86,86,86,10,18,66,2,136,165,40,234,175,22,
139,5,19,19,19,130,239,178,104,64,108,126,126,62,127,81,127,21,64,108,90,90,26,252,126,63,186,186,186,162,10,177,78,167,147,31,240,250,26,98,41,138,250,171,66,161,64,125,125,61,134,134,134,162,10,177,
46,151,11,122,189,62,228,149,42,161,40,234,125,134,97,144,157,157,141,225,225,97,98,158,15,1,98,127,31,2,177,89,89,89,88,92,92,36,91,144,104,64,172,203,229,130,193,96,224,22,245,139,93,33,150,123,206,
199,237,135,6,6,71,48,56,56,142,193,193,177,176,67,44,183,51,229,237,247,223,13,129,88,131,193,128,233,233,105,116,116,116,160,169,169,9,195,195,195,216,120,116,19,239,188,243,14,174,95,127,2,29,157,190,
176,66,172,213,106,229,159,8,217,29,98,101,50,25,250,251,251,49,52,52,132,217,217,89,60,255,252,243,228,15,123,245,213,87,49,50,58,17,54,136,13,230,234,61,33,86,165,82,193,235,245,98,121,121,25,111,191,
45,252,127,252,248,227,215,209,222,222,67,190,203,142,2,177,78,167,51,56,55,188,55,196,210,52,253,241,137,19,39,4,41,216,151,95,126,25,11,11,11,232,233,241,161,177,177,5,205,205,237,71,134,216,252,252,
252,96,8,217,27,98,25,134,121,211,100,50,161,166,166,6,107,107,107,120,236,177,199,48,58,58,74,146,24,13,13,141,71,134,216,146,146,18,104,52,154,131,65,172,74,165,250,109,98,98,34,180,90,45,236,118,59,
42,42,42,208,208,208,16,54,136,13,250,174,219,63,196,82,20,245,155,216,216,88,104,52,26,112,79,183,60,30,207,145,33,214,110,183,11,206,95,29,24,98,119,190,177,161,86,171,161,86,171,97,50,153,208,208,208,
64,22,116,16,136,229,100,56,232,97,227,251,135,134,88,154,166,193,178,44,56,194,46,41,41,65,109,109,237,158,11,114,185,92,40,45,45,37,24,123,234,212,41,152,76,38,36,39,39,243,119,153,191,9,11,196,50,12,
131,184,184,56,168,213,106,88,173,86,114,255,198,125,211,23,20,20,224,248,241,227,72,72,72,128,90,173,134,76,38,11,193,216,157,171,246,111,195,14,177,42,149,10,90,173,22,22,139,5,57,57,57,48,153,76,80,
169,84,196,9,24,134,129,68,34,129,88,44,38,24,187,243,235,255,177,74,165,122,51,162,16,203,33,44,247,42,4,57,249,195,129,88,150,101,223,149,72,36,191,160,105,250,247,124,201,251,22,98,255,174,33,150,95,
77,32,22,139,215,35,14,177,52,77,223,166,40,234,67,145,72,180,205,33,236,206,55,60,184,147,185,122,189,30,39,78,156,224,168,103,155,97,152,143,35,2,177,44,203,190,101,177,88,254,79,102,102,38,108,54,27,
78,157,58,5,135,195,1,187,221,142,234,234,106,212,212,212,96,104,104,8,195,195,195,152,156,156,196,204,204,12,250,250,250,80,85,85,5,147,201,196,45,62,188,213,4,122,189,126,123,97,97,1,139,139,139,24,
31,31,135,223,239,199,200,200,8,198,199,199,49,50,50,130,209,209,81,12,13,13,9,2,169,156,121,218,237,118,226,157,97,131,88,173,86,139,197,197,69,172,173,173,161,191,191,31,29,29,29,2,217,123,144,123,86,
85,85,193,102,179,241,95,177,35,65,236,127,209,52,141,134,134,6,92,191,126,29,67,67,67,135,134,216,154,154,26,20,21,21,193,100,50,145,211,33,20,69,253,215,129,33,150,97,24,100,102,102,98,109,109,13,51,
51,51,71,78,196,86,85,85,193,233,116,226,228,201,147,193,25,226,253,39,98,83,82,82,112,249,242,101,44,47,47,11,220,243,168,16,235,116,58,145,159,159,15,179,217,204,29,165,251,197,190,32,86,161,80,96,120,
120,24,43,43,43,36,214,29,46,136,173,169,169,129,199,227,33,177,112,169,84,186,205,48,204,91,247,133,88,141,70,131,225,225,97,60,250,232,163,24,27,27,35,111,91,184,33,214,227,241,192,110,183,35,49,49,
17,44,203,254,127,123,66,172,66,161,64,75,75,11,86,86,86,34,94,77,192,157,76,226,110,34,104,154,190,29,2,177,114,185,252,227,150,150,22,18,62,141,6,196,86,85,85,33,46,46,110,119,136,101,89,118,221,98,
177,96,109,109,13,211,211,211,130,15,117,36,33,182,178,178,146,220,37,239,116,199,8,19,177,199,142,29,195,218,218,154,224,55,45,210,16,235,241,120,144,144,144,176,119,34,54,51,51,19,139,139,139,36,162,
27,45,136,229,49,80,104,53,129,193,96,128,223,239,135,207,231,139,106,34,54,168,242,66,8,177,122,189,30,35,35,35,24,24,24,136,42,196,114,207,103,118,190,122,132,16,75,81,20,234,234,234,48,54,54,22,85,
136,61,113,226,68,240,205,134,16,98,29,14,7,252,126,127,84,33,214,102,179,237,74,214,37,220,45,122,94,94,30,230,231,231,163,154,136,45,41,41,225,111,105,132,213,4,156,14,243,23,213,218,218,142,169,169,
51,24,26,158,136,24,196,58,157,78,254,174,33,52,17,155,157,157,141,165,165,37,52,55,55,163,183,183,23,179,179,179,184,117,235,89,188,240,194,247,48,119,118,49,34,16,91,92,92,76,246,248,33,137,88,134,97,
182,51,51,51,113,238,220,57,248,253,126,220,184,113,67,240,135,174,175,111,124,253,213,17,38,136,117,185,92,252,167,163,72,74,74,90,15,129,216,172,172,44,92,185,114,5,111,190,249,166,224,15,122,237,181,
215,17,56,51,79,22,20,46,136,205,202,202,226,51,227,199,33,16,75,211,244,109,147,201,132,115,231,206,225,7,63,248,1,238,222,189,139,183,222,122,11,155,155,155,152,152,152,64,107,219,215,175,82,56,18,177,
121,121,121,130,103,50,123,66,172,193,96,248,255,18,19,19,49,54,54,134,205,205,77,248,253,126,116,116,116,236,108,208,26,194,6,177,185,185,185,193,15,137,246,134,88,177,88,252,86,108,108,236,118,124,124,
60,10,10,10,224,241,120,72,31,89,184,32,214,110,183,35,54,54,246,96,16,75,81,212,47,98,98,98,160,213,106,161,211,233,80,91,91,75,126,142,10,177,37,37,37,72,76,76,60,60,196,114,213,23,90,173,22,86,171,
149,220,42,29,5,98,179,178,178,130,191,82,14,12,177,255,69,211,52,212,106,53,18,18,18,224,116,58,81,85,85,37,216,53,30,4,98,173,86,107,240,231,232,191,142,148,136,101,89,22,9,9,9,56,126,252,56,154,154,
154,238,187,32,110,17,78,167,19,46,151,11,14,135,3,69,69,69,193,159,163,240,64,172,66,161,128,78,167,67,97,97,33,202,203,203,73,10,214,102,179,33,47,47,15,217,217,217,72,73,73,129,193,96,64,98,98,34,18,
19,19,161,209,104,160,209,104,248,105,179,240,87,19,176,44,139,196,196,68,36,36,36,64,175,215,67,169,84,130,101,89,130,174,18,137,4,44,203,134,252,236,120,232,255,167,84,42,223,138,8,196,74,36,146,143,
85,42,21,33,105,177,88,76,10,136,120,53,96,228,102,96,7,98,111,71,28,98,149,74,229,58,191,154,224,91,136,253,127,18,98,163,146,136,85,169,84,255,77,165,82,221,99,89,118,91,169,84,130,97,24,242,212,129,
101,89,72,36,18,80,20,133,216,216,88,72,165,82,168,213,234,109,165,82,249,39,177,88,124,55,34,213,4,12,195,124,226,116,58,137,113,182,180,180,144,86,156,230,230,102,212,212,212,160,177,177,145,132,45,
134,134,134,48,48,48,128,202,202,74,36,37,37,133,31,98,165,82,233,182,199,227,193,141,27,55,16,8,4,8,184,250,124,62,82,222,208,219,219,75,48,182,171,171,11,62,159,143,116,198,158,60,121,18,114,185,28,
59,212,125,116,136,165,40,10,39,78,156,192,250,250,122,200,177,242,253,186,103,89,89,89,120,170,9,184,47,100,163,209,136,229,229,101,44,45,45,29,170,154,128,107,215,117,187,221,72,79,79,39,71,15,14,5,
177,52,77,195,100,50,225,242,229,203,88,89,89,17,156,115,63,12,196,214,213,213,161,176,176,16,169,169,169,252,231,200,7,131,216,132,132,4,204,204,204,96,105,105,73,224,158,71,129,216,170,170,42,148,150,
150,34,59,59,155,223,103,246,96,136,101,89,118,251,196,137,19,216,220,220,196,202,202,10,134,134,134,194,14,177,37,37,37,176,88,44,80,171,213,96,24,230,193,213,4,121,121,121,56,119,238,28,206,159,63,47,
248,12,133,19,98,189,94,47,156,78,39,78,157,58,5,173,86,11,154,166,63,217,21,98,229,114,249,221,162,162,34,60,241,196,19,88,92,92,20,124,134,34,85,77,192,177,245,206,22,231,191,133,64,172,78,167,251,19,
87,139,194,71,216,72,66,172,215,235,37,65,65,154,166,239,133,36,98,19,19,19,183,207,157,59,135,177,177,177,168,65,108,85,85,21,169,39,160,40,42,20,98,21,10,5,6,6,6,4,201,234,72,67,108,208,162,66,33,86,
171,213,162,191,191,159,160,126,180,32,54,55,55,119,111,136,165,40,10,93,93,93,240,251,253,81,133,216,251,38,98,89,150,69,85,85,21,252,126,127,84,33,214,98,177,236,157,136,149,201,100,168,173,173,197,
153,51,103,162,10,177,65,139,18,66,172,88,44,70,109,109,45,2,129,64,84,33,54,232,237,19,66,108,76,76,12,218,219,219,225,247,251,201,183,125,127,255,32,230,230,22,35,10,177,123,125,166,78,82,20,245,175,
18,137,4,189,189,189,152,156,156,68,119,119,55,38,38,38,112,233,210,101,188,248,253,151,176,249,216,117,244,15,140,70,4,98,249,245,190,33,16,27,23,23,135,177,177,49,204,204,204,224,241,199,31,199,143,
127,252,99,242,135,190,242,202,43,152,158,153,139,8,196,242,163,223,33,16,43,147,201,182,103,103,103,241,220,115,207,133,252,129,47,126,255,37,244,248,6,194,218,17,203,103,70,238,138,30,146,136,85,171,
213,247,242,243,243,5,103,218,159,121,230,25,44,46,46,98,112,112,84,176,160,112,64,44,39,51,41,41,41,220,162,254,20,2,177,49,49,49,255,205,104,52,162,170,170,10,103,207,158,37,79,180,234,235,235,209,208,
208,136,198,198,150,176,66,44,151,236,224,65,236,221,93,33,150,97,152,79,244,122,61,140,70,35,188,94,47,9,157,134,27,98,75,74,74,248,193,248,7,87,19,72,36,146,109,173,86,139,248,248,120,1,194,134,11,98,
131,142,211,237,191,35,150,243,206,228,228,100,56,157,78,52,53,53,133,5,98,237,118,59,191,237,230,224,16,75,211,52,180,90,45,18,18,18,208,222,222,78,22,117,88,136,221,109,65,135,134,88,149,74,5,139,197,
2,183,219,141,186,186,186,67,37,98,29,14,71,240,103,232,232,16,155,148,148,4,189,94,143,150,150,150,67,201,240,46,95,186,71,135,88,134,97,182,117,58,29,142,31,63,14,155,205,6,135,195,65,78,255,231,229,
229,33,61,61,29,22,139,5,153,153,153,200,204,204,68,94,94,30,142,31,63,78,202,107,249,31,234,176,67,108,98,98,34,20,10,5,193,85,254,95,165,82,41,249,43,7,178,65,173,0,159,68,164,154,128,101,217,187,50,
153,236,79,18,137,100,91,36,18,17,128,229,162,184,252,158,132,157,191,110,139,68,162,123,50,153,236,191,69,189,35,246,91,136,253,187,131,88,145,72,244,147,111,4,196,82,20,245,134,92,46,255,216,237,118,
111,31,59,118,12,22,139,5,42,149,10,74,165,146,208,14,135,177,59,155,182,109,174,154,64,36,18,77,70,100,88,151,86,171,133,207,231,195,149,43,87,48,50,50,2,159,207,71,224,149,235,236,44,41,41,129,211,233,
36,129,192,162,162,34,110,186,211,103,97,133,88,238,20,119,111,111,47,86,86,86,66,188,161,181,181,21,93,93,93,4,96,251,251,251,201,95,251,251,251,209,212,212,132,172,172,44,176,44,27,62,136,141,139,139,
195,196,196,4,121,242,206,101,172,14,82,201,106,179,217,160,213,106,185,107,219,225,33,86,36,18,129,59,201,118,245,234,85,1,126,28,4,98,27,27,27,81,85,85,133,188,188,60,254,179,191,255,58,84,53,129,201,
100,194,212,212,20,150,151,151,5,204,120,216,97,93,220,112,18,238,148,200,129,32,86,173,86,163,166,166,6,27,27,27,184,112,225,66,72,110,248,176,16,91,91,91,11,171,213,10,189,94,127,48,136,213,233,116,
219,253,253,253,36,201,200,213,90,132,11,98,171,170,170,80,84,84,4,131,193,192,69,148,190,220,19,98,25,134,153,212,235,245,159,157,61,123,22,171,171,171,24,31,31,23,48,99,184,33,182,176,176,16,9,9,9,36,
64,191,103,34,54,39,39,7,155,155,155,24,30,30,14,105,209,13,55,196,86,84,84,32,55,55,23,90,173,150,59,8,180,123,53,65,78,78,14,86,87,87,5,191,242,145,132,88,167,211,137,188,188,60,104,181,90,136,68,162,
143,67,32,86,44,22,111,31,63,126,28,43,43,43,130,87,41,146,16,203,221,233,112,135,238,119,29,214,101,179,217,176,180,180,20,85,136,229,198,230,237,108,22,127,34,128,88,138,162,144,159,159,79,22,21,45,
136,117,187,221,252,45,179,16,98,89,150,133,211,233,196,210,210,82,84,33,54,168,242,66,8,177,20,69,161,165,165,5,243,243,243,223,156,106,130,152,152,24,244,245,245,97,106,106,42,170,16,27,84,230,39,132,
88,173,86,139,139,23,47,98,100,100,132,124,185,250,253,51,56,191,180,18,81,136,77,75,75,219,27,98,227,227,227,177,177,177,129,225,225,97,140,142,142,98,110,110,14,79,220,184,129,151,254,233,7,184,124,
121,61,98,16,27,52,54,72,8,177,153,153,153,184,114,229,10,30,123,236,49,188,242,202,43,130,223,174,231,158,123,30,253,253,35,17,25,214,21,116,190,253,107,136,21,137,68,63,201,204,204,196,147,79,62,25,
146,241,188,123,247,46,54,55,175,193,231,27,140,8,196,242,147,251,33,16,107,177,88,48,60,60,140,103,158,121,6,119,238,220,193,243,207,63,191,211,38,63,135,254,254,225,176,15,235,226,0,132,87,161,18,10,
177,82,169,244,227,228,228,100,184,221,110,76,76,76,160,191,191,159,132,29,154,79,119,144,183,43,156,16,91,86,86,198,87,189,189,171,9,146,147,147,81,85,85,69,238,80,34,5,177,28,53,114,1,176,251,14,235,
146,201,100,208,235,245,240,122,189,2,239,12,39,196,58,157,78,50,146,120,231,85,250,76,171,213,78,238,9,177,220,237,84,66,66,2,202,202,202,4,99,132,195,1,177,165,165,165,176,88,44,252,153,125,251,135,
216,248,248,120,232,116,58,116,117,117,9,206,80,29,5,98,243,243,243,131,187,18,14,7,177,69,69,69,100,206,199,81,32,214,106,181,66,165,82,29,29,98,185,90,149,210,210,82,52,54,54,10,22,116,16,247,204,207,
207,15,110,99,62,26,196,42,149,74,36,39,39,163,178,178,146,236,22,185,197,84,86,86,146,38,128,221,22,228,116,58,145,147,147,19,28,207,13,15,196,170,213,234,109,141,70,131,180,180,52,36,39,39,131,123,84,
98,48,24,144,154,154,138,236,236,108,2,175,220,99,14,174,72,38,104,112,224,151,97,133,88,165,82,249,153,82,169,132,76,38,219,243,135,131,88,181,90,141,248,248,248,144,106,232,176,67,172,76,38,155,228,
170,9,118,146,174,130,20,44,23,211,149,72,36,144,201,100,144,74,165,164,39,225,161,12,235,218,3,98,127,242,45,196,126,99,33,54,38,38,230,103,74,165,242,151,59,179,136,30,46,196,82,20,245,161,92,46,223,
118,187,221,104,110,110,38,125,231,113,113,113,164,179,58,184,131,35,146,195,186,62,226,58,17,166,166,166,176,190,190,142,137,137,9,116,119,119,147,152,156,199,227,33,19,157,116,58,29,18,18,18,16,23,23,
199,137,241,71,98,177,248,109,154,166,59,194,6,177,82,169,20,249,249,249,88,95,95,199,35,143,60,34,168,66,228,170,8,90,90,90,200,164,96,110,122,70,115,115,51,218,219,219,81,85,85,5,189,94,191,205,178,
236,175,143,12,177,52,77,67,163,209,192,227,241,96,101,101,5,243,243,243,2,72,187,159,123,54,53,53,145,255,174,174,174,142,255,29,120,120,136,229,30,64,250,124,62,44,45,45,97,118,118,86,144,104,60,8,196,
214,213,213,161,172,172,12,58,157,142,251,220,29,28,98,181,90,45,90,90,90,176,182,182,134,139,23,47,98,108,108,76,32,123,135,129,216,154,154,26,156,60,121,146,255,5,253,96,136,165,105,186,67,173,86,255,
218,233,116,226,220,185,115,88,91,91,35,129,156,112,13,235,114,187,221,72,76,76,228,255,150,222,31,98,101,50,217,219,86,171,21,151,47,95,38,163,239,194,61,172,171,182,182,22,121,121,121,80,42,149,220,
150,248,203,7,38,98,253,126,63,41,110,224,47,40,156,16,91,94,94,142,172,172,44,196,196,196,112,95,218,123,67,172,76,38,195,204,204,12,105,171,140,20,196,86,87,87,195,233,116,10,166,21,236,9,177,201,201,
201,152,153,153,17,116,37,68,10,98,61,30,15,201,188,236,124,190,118,239,136,53,26,141,184,114,229,74,212,58,98,185,105,79,74,165,114,247,106,2,238,38,97,107,107,139,252,182,69,3,98,203,203,203,97,48,24,
32,18,137,32,145,72,126,22,2,177,121,121,121,88,91,91,139,42,196,122,60,30,210,46,72,211,244,47,5,16,43,22,139,81,93,93,77,116,56,90,16,203,59,194,194,141,230,248,26,98,53,26,13,198,199,199,49,59,59,27,
85,136,117,185,92,252,43,188,16,98,245,122,61,174,95,191,78,166,61,181,181,181,97,126,254,28,86,87,215,35,10,177,252,148,117,8,196,102,101,101,225,218,181,107,152,152,152,192,220,220,28,46,95,190,140,
231,158,251,46,190,247,189,239,227,210,165,245,136,65,236,94,217,225,18,145,72,244,65,117,117,53,174,93,187,134,159,252,228,39,130,15,241,123,239,189,135,39,159,250,78,68,32,182,188,188,60,216,60,191,
134,88,181,90,253,75,171,213,138,167,158,122,42,244,170,125,231,14,150,47,174,70,164,154,192,229,114,65,46,151,239,14,177,10,133,226,103,102,179,25,19,19,19,248,254,247,191,143,91,183,110,97,99,99,3,11,
11,11,152,158,62,131,222,190,225,136,12,235,114,56,28,130,207,83,8,196,74,165,210,109,131,193,128,190,190,62,210,88,210,216,216,136,214,182,118,180,183,247,68,4,98,131,62,228,187,119,196,170,213,106,28,
59,118,236,43,124,109,110,38,155,180,72,65,236,201,147,39,247,55,172,43,37,37,5,121,121,121,100,81,145,132,216,227,199,143,239,107,88,215,71,44,203,194,104,52,194,225,112,160,189,189,61,34,16,203,13,91,
226,253,230,221,63,17,203,65,108,74,74,10,250,250,250,194,158,136,117,185,92,36,2,192,189,74,74,165,242,237,125,39,98,203,203,203,209,214,214,22,54,136,229,78,172,241,169,145,97,152,95,27,141,198,142,
125,87,19,164,167,167,163,181,181,149,156,54,59,44,196,114,204,24,116,232,254,112,16,27,23,23,135,19,39,78,8,134,66,28,20,98,29,14,7,210,211,211,119,115,207,195,67,108,74,74,10,204,102,51,236,118,59,108,
54,27,172,86,43,156,78,167,0,96,239,183,160,160,42,167,240,64,172,66,161,248,53,87,47,192,178,44,100,50,25,20,10,5,148,74,37,244,122,61,44,22,11,242,242,242,80,86,86,70,190,58,184,11,99,80,196,59,124,
16,203,48,76,135,88,44,126,155,162,168,143,68,34,17,164,82,41,49,77,133,66,1,133,66,1,185,92,14,174,79,193,108,54,35,53,53,53,88,133,35,3,177,193,29,177,188,147,102,68,133,185,5,242,134,221,108,63,148,
97,93,193,16,75,211,244,7,98,177,248,151,98,177,248,103,223,66,236,55,186,154,128,101,217,127,51,24,12,127,98,89,246,111,15,29,98,25,134,217,78,76,76,68,117,117,53,105,4,40,46,46,70,92,92,28,201,168,71,
21,98,21,10,5,114,115,115,49,58,58,138,243,231,207,147,220,39,231,157,220,188,15,157,78,23,92,59,254,81,216,19,177,12,195,64,171,213,162,177,177,17,231,206,157,219,57,14,53,136,206,206,78,194,140,156,
218,113,198,233,245,122,225,118,187,73,154,76,42,149,110,179,44,251,191,24,134,89,61,114,34,86,36,18,193,233,116,226,204,153,51,184,116,233,18,252,126,63,241,134,221,220,147,219,177,242,255,51,78,138,
179,178,178,32,149,74,63,57,52,196,106,181,90,212,212,212,96,101,101,5,87,174,92,193,153,51,103,4,135,239,15,2,177,92,37,107,97,97,33,183,117,217,63,196,74,36,146,85,189,94,255,73,123,123,59,86,87,87,
177,186,186,74,102,217,30,53,17,203,153,103,124,124,252,193,18,177,42,149,234,127,117,116,116,8,16,54,92,195,186,184,205,34,127,111,190,175,68,108,108,108,236,246,205,155,55,49,61,61,29,50,95,45,92,195,
186,202,202,202,248,79,77,31,12,177,41,41,41,184,117,235,86,68,33,214,235,245,34,45,45,141,28,107,185,47,196,82,20,5,135,195,129,245,245,117,65,0,53,18,137,88,187,221,14,149,74,69,46,180,123,66,44,119,
148,124,113,113,81,240,42,69,10,98,119,46,15,247,135,88,173,86,139,137,137,9,76,77,77,69,5,98,57,29,222,201,167,239,14,177,233,233,233,130,193,111,145,134,88,46,82,201,189,141,33,213,4,98,177,24,118,187,
29,27,27,27,164,78,60,26,16,235,116,58,145,158,158,206,189,90,255,38,128,216,152,152,24,44,45,45,97,110,110,46,170,137,216,202,202,74,114,22,153,166,233,207,66,58,98,159,123,238,57,242,144,168,187,187,
27,231,151,150,241,232,14,11,70,10,98,43,42,42,248,77,149,127,19,64,108,123,123,59,214,215,215,177,180,180,132,141,141,13,60,241,196,19,120,241,197,23,241,189,239,125,31,235,87,31,139,24,196,242,171,163,
5,16,171,80,40,254,22,8,4,240,157,239,124,39,228,243,242,214,91,111,227,137,39,158,138,126,71,172,209,104,252,204,237,118,99,107,107,43,20,186,222,121,7,203,23,87,35,150,136,221,101,220,203,87,16,43,145,
72,254,205,100,50,161,167,167,7,235,235,235,88,93,93,197,194,194,2,206,156,153,195,220,220,98,196,134,117,57,28,142,224,221,169,176,154,32,62,62,30,38,147,137,12,128,227,246,67,93,221,220,245,37,252,16,
91,80,80,112,127,136,21,137,68,219,6,131,1,69,69,69,240,249,124,100,63,20,73,136,13,138,125,239,61,172,203,100,50,161,179,179,83,176,173,141,20,196,6,137,204,222,16,171,209,104,144,155,155,11,159,207,
23,209,68,108,89,89,153,0,246,239,11,177,220,136,216,177,177,49,18,114,136,4,196,150,148,148,236,191,154,128,162,168,47,69,34,17,92,46,23,58,59,59,35,2,177,92,189,239,129,19,177,25,25,25,24,24,24,16,236,
173,195,49,172,203,225,112,112,167,34,185,223,184,255,181,239,68,172,70,163,65,85,85,21,154,155,155,5,243,102,15,155,136,229,42,195,130,66,168,159,176,44,187,122,32,136,53,26,141,40,44,44,132,215,235,
21,116,195,30,4,98,237,118,59,82,82,82,194,7,177,26,141,230,19,141,70,3,150,101,17,27,27,139,180,180,52,228,230,230,162,160,160,0,78,167,147,60,198,223,109,65,14,135,99,183,87,230,232,16,203,48,204,42,
203,178,255,75,44,22,111,203,100,50,196,196,196,16,66,84,171,213,72,75,75,35,175,36,183,160,178,178,50,228,231,231,35,49,49,49,120,28,94,248,19,177,156,186,112,237,186,156,16,43,20,10,168,84,42,100,101,
101,193,106,181,34,35,35,67,240,234,240,102,101,69,7,98,185,122,2,238,85,228,215,103,126,99,32,150,97,152,191,177,44,251,25,77,211,255,246,109,53,193,223,69,71,44,77,211,31,196,198,198,254,109,39,114,
244,249,67,131,88,238,131,30,23,23,135,170,170,42,76,79,79,163,175,175,15,46,151,139,156,200,142,42,196,114,197,180,149,149,149,56,127,254,60,22,23,23,49,57,57,137,206,206,78,114,184,158,91,28,239,24,
74,100,32,150,162,40,36,38,38,194,235,245,98,113,113,17,75,75,75,152,154,154,34,100,196,49,35,119,206,166,190,190,158,220,211,113,153,226,157,112,253,209,171,9,148,74,37,108,54,27,134,134,134,176,180,
180,132,243,231,207,99,114,114,146,204,165,217,203,61,185,146,247,134,134,6,156,62,125,154,132,3,119,234,127,127,47,18,137,158,56,16,196,74,36,146,39,12,6,195,31,251,250,250,176,188,188,140,229,229,101,
4,2,1,12,14,14,238,59,162,27,220,17,203,85,0,55,54,54,34,39,39,7,82,169,244,143,7,74,196,170,213,234,223,251,253,126,92,185,114,5,227,227,227,232,238,238,14,57,197,125,88,136,229,74,32,119,120,113,255,
213,4,122,189,30,183,110,221,194,248,248,120,136,123,30,21,98,57,33,150,203,229,220,87,209,131,33,150,97,152,109,155,205,134,173,173,173,136,65,108,117,117,53,191,96,237,193,16,43,149,74,193,141,197,139,
20,196,214,212,212,4,159,63,190,63,196,198,199,199,227,185,231,158,19,244,109,68,2,98,203,202,202,248,141,205,247,135,88,183,219,141,235,215,175,71,5,98,51,50,50,4,55,163,187,66,172,76,38,195,230,230,
38,206,156,57,19,21,136,45,42,42,130,66,161,224,62,240,95,238,10,177,121,121,121,184,121,243,38,185,48,70,26,98,43,42,42,144,158,158,206,31,28,32,132,88,185,92,142,165,165,37,44,44,44,68,53,17,107,179,
217,248,175,150,176,154,32,55,55,151,52,220,12,14,14,98,96,96,0,231,206,45,225,59,79,63,23,81,136,45,47,47,71,106,106,106,104,34,150,101,217,207,167,167,167,241,248,227,143,227,214,173,91,248,238,119,
191,139,231,159,127,30,63,248,193,203,120,233,165,151,177,181,117,51,162,16,203,117,122,210,52,253,53,196,166,166,166,226,252,249,243,187,214,102,222,190,253,22,110,60,249,116,68,33,150,63,229,158,64,
172,90,173,254,91,85,85,21,174,95,191,30,178,168,55,222,248,9,46,93,190,26,49,136,117,185,92,252,98,191,207,9,196,210,52,253,65,70,70,6,6,7,7,225,247,251,49,51,51,131,233,233,105,204,207,47,98,97,241,
66,68,59,98,109,54,27,255,222,80,88,77,160,86,171,81,82,82,130,238,238,110,178,245,104,109,109,199,240,240,68,196,32,54,232,67,190,251,176,174,140,140,12,244,244,244,68,13,98,139,139,139,5,211,166,67,
32,150,162,168,47,85,42,21,28,14,7,186,187,187,163,2,177,65,9,235,221,33,150,162,40,88,44,22,12,14,14,70,37,17,27,20,252,218,27,98,181,90,45,249,222,139,36,196,6,109,93,238,15,177,20,69,225,248,241,227,
232,235,235,11,218,49,134,23,98,131,62,224,15,134,88,179,217,140,241,241,241,32,29,14,47,196,242,100,111,127,16,171,84,42,191,250,144,239,4,79,195,9,177,101,101,101,130,145,139,7,170,38,72,77,77,133,203,
229,18,44,232,40,16,235,116,58,81,92,92,140,156,156,28,62,234,255,254,64,137,216,248,248,248,63,106,52,26,40,149,74,164,165,165,161,168,168,136,44,102,191,16,203,165,169,243,242,242,72,185,26,239,186,
244,71,154,166,159,56,16,196,50,12,243,132,76,38,251,189,76,38,35,140,40,147,201,144,158,158,14,167,211,73,182,180,187,45,136,75,82,91,44,22,50,41,154,255,19,150,106,2,154,166,183,185,218,85,149,74,5,
185,92,14,131,193,0,187,221,78,22,194,45,198,102,179,193,108,54,147,183,136,183,144,200,66,44,195,48,80,169,84,136,137,137,129,94,175,39,197,143,25,25,25,130,135,138,252,175,144,168,65,172,88,44,134,74,
165,34,113,221,160,183,136,248,212,67,129,88,145,72,244,185,76,38,131,72,36,250,219,206,150,246,219,142,216,191,175,97,93,18,137,4,58,157,238,11,133,66,241,9,77,211,255,227,161,64,44,77,211,16,139,197,
80,40,20,176,219,237,24,30,30,198,240,240,48,220,110,55,255,169,85,116,32,150,97,24,168,213,106,152,76,38,120,189,94,204,207,207,99,117,117,21,126,191,31,61,61,61,104,108,108,36,245,41,113,113,113,252,
200,110,120,33,150,97,152,109,238,122,84,85,85,5,159,207,135,185,185,57,226,159,221,221,221,132,25,79,159,62,141,134,134,6,120,189,94,216,237,118,88,44,22,112,35,208,118,94,185,163,65,108,98,98,226,31,
90,90,90,48,53,53,133,169,169,41,76,79,79,99,102,102,6,126,191,31,67,67,67,130,136,238,94,16,235,241,120,80,92,92,140,140,140,12,40,149,74,206,216,127,123,168,68,172,92,46,255,208,239,247,227,234,213,
171,24,27,27,35,163,203,248,77,0,251,45,173,229,118,24,229,229,229,200,205,205,229,222,214,15,15,4,177,52,77,255,214,104,52,226,214,173,91,24,29,29,221,213,61,15,10,177,92,148,178,166,166,6,94,175,23,
153,153,153,96,89,246,15,251,134,88,133,66,129,166,166,38,108,108,108,68,4,98,171,170,170,144,155,155,203,61,145,120,48,196,82,20,245,165,193,96,192,229,203,151,49,51,51,19,17,136,173,173,173,229,231,
166,182,247,5,177,78,167,19,143,62,250,168,96,54,86,184,33,214,233,116,242,163,186,247,135,88,131,193,128,179,103,207,98,126,126,62,226,29,177,252,225,186,123,66,172,76,38,131,199,227,193,218,218,154,
160,9,32,210,16,187,179,213,217,29,98,77,38,19,150,151,151,49,59,59,27,21,136,205,207,207,39,29,253,187,66,172,74,165,66,127,127,63,86,86,86,162,6,177,65,141,130,66,136,165,105,250,127,88,173,86,92,186,
116,9,243,243,243,240,251,253,24,27,27,195,228,164,31,47,189,244,74,68,33,150,67,142,16,136,141,143,143,255,100,114,114,18,79,61,245,20,222,125,247,93,252,228,39,63,193,143,126,244,35,188,242,202,43,120,
245,213,215,240,220,115,47,68,180,35,150,247,120,247,235,142,88,131,193,240,197,196,196,4,158,125,246,217,93,164,228,29,60,251,236,243,17,131,216,160,243,237,95,87,19,72,36,18,88,173,86,172,175,175,227,
123,223,251,30,94,121,229,21,252,228,39,63,193,157,59,119,240,250,235,63,193,230,99,91,17,133,216,160,49,84,95,87,19,88,44,22,76,79,79,99,104,104,8,163,163,163,24,31,31,199,249,243,203,88,94,94,141,104,
71,108,144,14,127,17,50,172,203,229,114,161,175,175,143,124,219,251,124,253,240,79,157,137,40,196,150,150,150,242,83,140,159,8,32,86,44,22,195,98,177,96,116,116,148,44,42,26,16,155,151,151,199,191,79,
252,31,2,136,165,105,26,41,41,41,104,105,105,33,71,148,162,1,177,252,235,212,174,16,27,23,23,135,162,162,34,12,14,14,70,28,98,119,25,142,179,59,196,50,12,3,174,192,54,210,16,91,88,88,24,92,131,184,55,
196,102,100,100,144,210,218,72,66,44,87,48,179,47,136,213,235,245,232,235,235,67,103,103,103,196,32,214,110,183,7,167,25,239,15,177,12,195,108,123,60,30,244,246,246,6,21,215,134,15,98,121,105,88,80,20,
245,229,190,32,54,61,61,29,30,143,39,228,180,89,56,32,214,233,116,10,106,89,247,13,177,106,181,250,15,90,173,22,26,141,6,199,143,31,71,121,121,249,145,33,150,91,80,80,239,198,111,15,4,177,18,137,228,67,
165,82,9,181,90,13,149,74,5,179,217,140,146,146,146,67,85,19,88,173,86,228,229,229,33,35,35,131,255,27,247,225,97,171,9,126,43,145,72,16,19,19,3,181,90,13,165,82,137,204,204,76,65,26,118,183,5,113,163,
16,140,70,35,226,227,227,249,249,41,110,65,127,56,50,196,114,167,107,149,74,37,52,26,13,180,90,45,242,243,243,81,94,94,78,122,19,184,239,49,110,206,177,68,34,1,195,48,130,159,136,12,235,226,156,83,173,
86,67,173,86,35,37,37,5,54,155,13,249,249,249,48,26,141,144,201,100,100,70,22,247,19,213,68,44,203,178,164,46,147,171,34,248,70,64,44,77,211,255,67,42,149,126,194,178,236,23,59,11,249,118,88,215,223,39,
196,114,27,127,150,101,63,140,139,139,251,181,72,36,186,253,80,18,177,20,69,145,70,230,148,148,20,212,214,214,162,167,167,7,30,143,7,90,173,86,48,172,43,98,16,43,151,203,161,82,169,192,178,44,82,83,83,
97,183,219,209,216,216,136,225,225,97,172,172,172,32,16,8,144,225,93,46,151,11,102,179,153,159,138,13,31,196,170,213,234,255,93,92,92,188,205,117,82,249,124,62,44,44,44,96,97,97,129,56,232,216,216,24,
124,62,31,97,70,238,4,9,215,238,166,86,171,137,23,132,37,17,235,243,249,176,190,190,142,169,169,41,140,141,141,97,108,108,140,136,12,167,124,123,185,103,93,93,29,42,43,43,225,112,56,200,197,245,80,7,47,
120,71,84,222,215,235,245,248,238,119,191,11,191,223,191,43,51,238,55,17,91,89,89,9,187,221,46,152,89,180,243,11,114,176,142,88,165,82,137,206,206,206,61,221,243,32,16,91,95,95,143,234,234,106,84,84,84,
160,180,180,20,57,57,57,220,115,193,253,39,98,185,36,199,149,43,87,48,53,53,21,54,136,229,132,216,235,245,162,178,178,146,139,135,255,239,125,65,172,86,171,197,232,232,40,169,101,141,4,196,114,35,206,
118,190,180,239,15,177,34,145,8,141,141,141,88,93,93,21,52,160,134,27,98,171,171,171,249,57,207,251,67,108,106,106,42,158,122,234,41,76,79,79,147,167,11,145,128,216,234,234,106,50,221,126,103,103,177,
59,196,42,20,10,172,172,172,224,226,197,139,130,168,119,164,134,117,113,150,190,39,196,74,165,210,237,166,166,38,92,191,126,29,253,253,253,81,129,216,178,178,50,193,173,150,0,98,197,98,241,237,172,172,
44,92,191,126,29,27,27,27,56,119,238,28,102,103,103,49,62,62,142,239,191,244,114,196,32,214,237,118,11,110,221,5,16,155,156,156,252,235,174,174,46,60,253,244,211,130,127,224,171,175,190,138,55,111,191,
133,31,189,250,70,68,32,182,162,162,98,207,113,65,37,50,153,236,195,202,202,202,93,143,145,191,247,222,123,120,233,159,94,137,8,196,86,86,86,146,187,229,157,207,149,112,88,87,86,86,22,150,151,151,113,
233,210,37,220,188,121,19,223,255,254,247,241,214,91,111,225,246,237,219,120,250,233,103,35,6,177,123,29,184,39,29,177,220,163,87,159,207,135,161,161,33,172,172,172,98,237,202,163,88,137,96,71,172,211,
233,20,132,229,5,16,171,209,104,224,245,122,49,48,48,64,190,92,71,71,39,48,63,191,20,113,136,229,153,167,112,88,151,76,38,67,110,110,46,198,199,199,201,158,40,26,16,91,94,94,142,204,204,76,254,173,124,
137,160,161,57,43,43,11,51,51,51,100,81,209,128,88,167,211,73,198,80,209,52,253,235,144,68,108,74,74,10,86,86,86,200,180,129,72,67,108,121,121,185,0,246,25,134,185,29,2,177,82,169,148,60,200,142,70,34,
214,102,179,9,82,140,123,118,196,86,87,87,163,191,191,159,108,208,34,5,177,78,167,51,248,194,185,59,196,42,20,10,148,150,150,146,140,103,164,32,150,107,145,223,55,196,102,103,103,195,239,247,239,242,232,
62,124,16,155,151,151,39,24,230,252,64,136,53,153,76,112,187,221,36,125,17,110,136,45,41,41,225,15,125,219,31,196,42,20,138,255,173,211,233,160,86,171,145,151,151,7,143,199,131,218,218,218,176,65,108,
208,99,180,131,37,98,21,10,5,52,26,13,146,146,146,72,240,52,28,16,27,52,176,235,192,29,177,239,75,165,82,196,197,197,65,163,209,32,59,59,91,176,107,60,200,176,46,174,73,55,232,85,122,255,176,16,251,27,
174,218,80,171,213,34,33,33,1,78,167,19,181,181,181,123,46,136,251,220,112,131,191,205,102,51,140,70,35,116,58,29,223,21,126,19,22,136,165,105,26,42,149,10,177,177,177,200,205,205,21,44,200,237,118,147,
140,148,94,175,71,108,108,44,33,71,134,97,32,22,139,137,14,51,12,243,191,195,14,177,44,203,66,171,213,146,36,172,197,98,65,124,124,188,64,131,131,22,65,70,109,68,20,98,197,98,241,182,70,163,129,92,46,
39,11,216,69,131,163,15,177,12,195,220,86,40,20,191,166,105,250,195,61,134,117,125,11,177,127,127,16,75,81,212,251,52,77,255,119,137,68,242,78,52,33,246,15,252,14,14,238,209,72,108,108,44,52,26,13,44,
22,11,233,189,214,233,116,219,52,77,127,20,17,136,149,201,100,239,36,37,37,253,57,43,43,11,102,179,25,121,121,121,176,217,108,40,44,44,132,199,227,65,123,123,59,186,187,187,201,105,55,238,52,101,109,109,
45,244,122,61,119,209,12,111,34,214,106,181,110,79,77,77,97,102,102,6,51,51,51,152,154,154,194,200,200,8,38,38,38,224,247,251,49,60,60,140,193,193,65,146,225,107,109,109,197,233,211,167,225,241,120,144,
151,151,199,191,154,135,7,98,99,99,99,177,181,181,133,133,133,133,144,22,128,253,12,235,226,116,37,37,37,133,159,35,62,252,176,46,153,76,134,198,198,70,92,191,126,61,164,223,243,32,16,203,205,104,63,118,
236,24,191,136,230,224,195,186,104,154,38,67,113,130,243,158,135,73,196,86,87,87,195,225,112,32,39,39,7,26,141,134,123,213,14,6,177,70,163,17,203,203,203,88,92,92,20,184,231,81,32,150,155,242,196,29,99,
217,217,18,239,47,17,171,80,40,176,184,184,136,75,151,46,9,0,45,92,195,186,184,221,40,55,84,137,97,152,119,238,11,177,49,49,49,104,110,110,198,214,214,150,32,235,25,110,136,245,122,189,40,44,44,228,22,
245,231,61,33,54,38,38,6,53,53,53,88,91,91,131,223,239,143,104,34,182,186,186,26,229,229,229,228,131,191,43,196,50,12,243,81,75,75,11,86,87,87,49,55,55,39,200,157,71,10,98,61,30,15,159,129,254,16,2,177,
233,233,233,219,47,190,248,34,110,222,188,137,39,159,124,18,43,43,43,152,153,153,193,249,243,75,17,131,216,202,202,74,196,197,197,237,14,177,82,169,244,157,156,156,28,60,253,244,211,184,125,251,54,249,
7,190,243,206,59,184,125,251,173,136,65,44,247,176,114,175,68,236,127,207,206,206,198,218,218,26,222,125,247,221,93,95,141,72,37,98,121,71,200,67,171,9,204,102,51,206,157,59,135,169,169,41,172,172,172,
224,230,205,155,120,253,245,215,113,231,206,29,188,244,210,203,17,75,196,166,165,165,237,218,231,89,78,81,212,95,181,90,45,6,7,7,209,219,219,139,214,214,86,244,246,246,98,229,145,75,184,118,237,6,54,31,
123,34,98,16,27,148,165,18,66,172,70,163,33,207,131,185,239,178,153,217,185,125,15,234,58,44,196,6,159,71,22,64,172,68,34,65,89,89,25,166,167,167,163,10,177,118,187,125,247,142,216,157,247,18,217,217,
217,152,159,159,143,122,34,150,119,218,86,88,77,32,18,137,144,149,149,37,88,84,52,32,214,229,114,241,175,85,255,61,4,98,179,179,179,177,184,184,40,216,15,69,26,98,93,46,23,57,103,202,48,204,59,33,16,155,
153,153,137,139,23,47,162,165,165,37,42,16,235,116,58,249,134,190,43,196,254,193,100,50,97,105,105,73,208,15,27,73,136,181,88,44,130,163,118,187,66,108,122,122,58,6,6,6,200,72,179,72,38,98,115,114,114,
4,238,176,39,196,38,38,38,254,57,43,43,139,188,125,145,130,88,171,213,26,210,12,183,39,196,178,44,251,142,78,167,131,66,161,192,137,19,39,224,245,122,195,14,177,54,155,45,120,182,209,254,18,177,82,169,
20,90,173,22,41,41,41,168,168,168,64,99,99,99,88,32,182,164,164,132,63,148,242,224,16,43,145,72,160,213,106,17,31,31,143,242,242,114,114,178,241,40,16,27,244,61,119,184,97,93,20,69,65,173,86,67,163,209,
224,228,201,147,33,93,177,7,129,216,93,146,213,255,117,36,136,221,57,99,140,156,156,28,65,76,247,126,213,4,165,165,165,112,58,157,112,56,28,56,121,242,100,240,115,152,240,64,172,76,38,67,82,82,18,114,
115,115,209,220,220,12,143,199,67,108,188,160,160,0,22,139,5,233,233,233,208,233,116,208,106,181,208,106,181,136,141,141,37,61,11,252,207,80,88,33,86,42,149,66,167,211,65,167,211,193,96,48,32,62,62,30,
114,185,156,224,171,88,44,134,68,34,1,203,178,164,154,149,251,207,105,154,254,179,82,169,124,39,34,16,43,18,137,62,82,42,149,219,50,153,12,44,203,18,136,229,7,150,131,18,177,127,136,26,196,138,197,226,
119,118,190,213,223,255,22,98,255,159,130,216,168,118,196,202,229,242,159,42,149,202,123,156,204,112,23,67,238,87,94,38,147,145,177,121,90,173,22,10,133,98,91,163,209,252,153,97,152,119,35,146,136,85,
42,149,247,202,203,203,209,219,219,139,161,161,33,52,52,52,160,169,169,9,77,77,77,168,170,170,66,107,107,43,154,155,155,209,213,213,133,161,161,33,244,247,247,99,122,122,154,191,179,12,47,196,202,100,
178,237,230,230,102,172,174,174,34,16,8,192,231,243,161,187,187,27,3,3,3,240,249,124,24,24,24,192,192,192,0,250,250,250,208,223,223,79,74,73,91,90,90,224,118,187,113,226,196,9,50,220,59,44,16,75,81,20,
184,126,207,233,233,233,3,65,108,115,115,51,26,26,26,224,118,187,145,153,153,73,142,27,28,9,98,69,34,17,204,102,51,57,117,27,92,201,122,144,142,216,226,226,226,224,242,246,131,67,44,69,81,48,155,205,88,
90,90,194,133,11,23,4,147,195,142,2,177,153,153,153,252,174,197,253,67,44,77,211,208,233,116,184,120,241,34,46,94,188,40,112,207,163,66,172,195,225,128,217,108,230,211,245,254,58,98,143,29,59,134,169,
169,41,172,173,173,133,29,98,189,94,47,28,14,7,78,156,56,65,130,244,15,76,196,30,59,118,12,87,175,94,197,185,115,231,48,52,52,20,177,97,93,46,151,11,5,5,5,136,143,143,7,195,48,247,118,133,88,150,101,223,
181,88,44,88,95,95,199,35,143,60,34,136,232,70,10,98,157,78,39,76,38,19,183,219,248,105,8,196,166,165,165,253,101,109,109,13,55,111,222,196,91,111,189,133,103,158,121,6,75,75,75,152,152,152,136,24,196,
86,85,85,145,83,108,52,77,223,11,129,88,149,74,181,189,177,177,129,23,94,120,33,228,38,50,82,16,91,85,85,197,63,24,29,90,77,96,48,24,48,59,59,187,199,236,246,187,247,89,216,225,33,150,43,8,217,21,98,41,
138,250,87,149,74,133,145,145,17,248,253,126,76,79,79,227,242,229,203,120,241,197,23,191,122,53,110,71,14,98,131,6,225,8,33,86,169,84,162,165,165,5,227,227,227,104,107,107,67,71,71,7,150,150,150,113,235,
217,231,113,235,217,127,140,24,196,6,133,187,132,16,171,80,40,208,220,220,12,191,223,79,190,58,206,204,205,227,234,198,181,136,14,235,10,90,148,16,98,41,138,34,253,249,209,132,88,158,79,133,66,172,66,
161,128,199,227,33,57,207,104,65,108,80,124,41,20,98,155,155,155,49,51,51,19,85,136,13,106,210,21,66,172,84,42,69,67,67,3,166,166,166,162,218,17,27,228,232,194,142,88,165,82,9,159,207,135,137,137,137,
168,66,44,223,209,119,173,38,24,28,28,196,232,232,168,96,131,22,73,136,229,84,111,79,136,141,141,141,189,103,179,217,208,219,219,43,168,37,136,100,71,172,221,110,231,247,5,253,37,4,98,101,50,217,79,211,
210,210,96,179,217,4,19,123,35,5,177,197,197,197,130,199,106,50,153,236,221,93,33,86,38,147,221,75,78,78,134,197,98,129,215,235,69,67,67,67,196,32,54,104,144,224,253,19,177,44,203,110,39,38,38,66,167,
211,193,227,241,8,146,177,225,130,216,160,172,231,254,135,117,105,181,90,232,116,58,120,189,222,176,65,172,205,102,67,124,124,252,225,33,150,166,105,104,181,90,168,213,106,184,221,110,212,215,215,31,9,
98,237,118,123,112,102,248,240,16,27,27,27,11,189,94,79,94,173,131,86,19,112,13,186,65,143,101,143,6,177,220,237,150,209,104,36,249,225,221,22,20,188,24,110,65,92,51,64,216,134,117,113,150,32,18,137,182,
181,90,45,146,146,146,224,118,187,81,81,81,129,130,130,2,228,231,231,227,216,177,99,200,204,204,132,197,98,129,201,100,34,255,58,59,59,27,39,79,158,132,193,96,224,63,20,10,127,53,1,247,25,227,176,149,
97,24,1,188,6,255,240,122,18,64,81,212,189,136,36,98,89,150,125,87,169,84,254,133,97,152,109,46,150,203,37,97,185,6,75,46,64,200,21,64,50,12,115,111,231,194,252,112,134,117,125,11,177,127,23,16,75,211,
244,35,223,24,136,165,40,234,141,196,196,196,63,230,228,228,108,115,129,63,238,88,184,68,34,129,68,34,33,31,242,157,44,250,54,69,81,127,96,24,230,237,136,64,44,77,211,31,75,36,18,156,57,115,6,75,75,75,
24,25,25,193,200,200,8,218,219,219,225,243,249,72,164,141,155,6,93,89,89,137,210,210,82,148,151,151,115,179,177,254,178,147,103,15,223,176,174,228,228,100,140,142,142,98,99,99,3,131,131,131,232,232,232,
64,91,91,27,218,219,219,5,231,149,57,140,237,239,239,39,255,190,169,169,9,233,233,233,16,139,197,225,129,88,154,166,33,151,203,49,55,55,135,203,151,47,147,87,231,32,238,201,205,44,10,202,119,30,14,98,
185,188,231,196,196,4,214,215,215,5,105,198,131,64,44,151,138,205,205,205,229,143,158,58,56,196,74,36,18,24,12,6,92,184,112,1,143,60,242,8,134,135,135,5,169,216,195,12,235,114,58,157,48,24,12,124,33,222,
63,196,202,229,114,120,189,94,44,44,44,224,145,71,30,9,59,196,166,165,165,241,23,118,127,136,21,137,68,111,104,52,154,237,211,167,79,227,250,245,235,8,4,2,130,74,214,112,64,108,117,117,53,23,160,231,190,
51,239,63,172,43,37,37,229,47,243,243,243,228,40,121,164,32,150,43,12,73,77,77,229,94,177,143,119,133,88,177,88,252,118,94,94,30,190,251,221,239,226,159,254,233,159,240,238,187,239,226,169,167,158,194,
153,51,103,48,49,49,17,118,136,245,120,60,176,90,173,72,72,72,184,111,53,193,31,138,138,138,240,228,147,79,10,254,193,239,190,251,46,238,220,185,19,17,136,117,185,92,56,121,242,36,180,90,45,36,18,201,
31,67,32,86,34,145,108,23,21,21,97,117,117,21,215,175,95,199,219,111,223,255,246,59,92,16,235,114,185,72,84,87,161,80,60,34,128,88,154,166,97,183,219,177,188,188,140,129,129,1,4,2,1,108,109,109,225,206,
157,59,17,133,88,143,199,67,18,66,33,16,43,145,72,96,183,219,177,180,180,68,46,140,163,163,163,120,249,229,87,35,86,77,224,241,120,224,112,56,246,134,88,134,97,80,94,94,142,165,165,37,114,165,158,153,
9,224,169,155,183,34,58,172,43,40,82,41,132,88,150,101,81,93,93,141,249,249,249,168,66,108,144,228,9,33,86,42,149,98,104,104,136,152,103,180,32,118,151,248,210,215,16,43,151,203,17,8,4,48,54,54,22,85,
136,77,79,79,223,27,98,13,6,3,150,151,151,209,219,219,27,85,136,229,205,134,12,133,216,196,196,68,92,187,118,45,234,195,186,130,22,37,132,216,172,172,44,172,175,175,135,28,91,138,52,196,242,53,79,0,177,
52,77,63,98,50,153,208,223,223,47,232,74,136,6,196,242,170,46,66,33,54,46,46,238,143,58,157,14,46,151,11,237,237,237,81,129,88,151,203,197,55,171,63,236,154,136,101,24,6,122,189,30,213,213,213,17,135,
88,167,211,137,83,167,78,17,127,144,201,100,111,239,10,177,52,77,127,28,31,31,15,173,86,11,151,203,133,150,150,150,136,13,235,202,201,201,225,239,62,255,242,192,97,93,44,203,66,175,215,163,174,174,78,
48,176,43,28,16,107,183,219,145,154,154,42,48,43,149,74,245,198,3,33,150,243,206,132,132,4,50,50,56,92,16,27,68,213,7,131,88,78,136,143,31,63,142,206,206,206,35,119,196,158,58,117,42,56,51,124,56,136,
101,24,6,70,163,17,14,135,67,176,99,60,40,196,90,173,214,224,177,120,71,131,88,150,101,97,48,24,80,81,81,129,250,250,250,3,47,232,228,201,147,193,93,64,225,129,88,165,82,185,29,31,31,15,147,201,132,244,
244,116,24,141,70,242,87,179,217,12,179,217,140,227,199,143,163,176,176,16,86,171,149,180,232,22,20,20,4,231,133,195,55,172,75,36,18,189,161,86,171,255,162,84,42,33,147,201,32,147,201,32,149,74,5,127,
149,201,100,96,24,134,252,107,181,90,29,92,160,253,113,68,32,118,199,153,254,176,19,99,35,125,176,28,198,114,163,207,36,18,9,164,82,41,247,107,255,199,135,50,172,107,47,136,213,235,245,143,124,11,177,
223,200,68,172,92,46,127,79,46,151,255,251,206,219,245,249,67,133,88,138,162,62,212,233,116,219,220,117,203,102,179,65,175,215,147,231,120,92,7,16,175,233,52,178,29,177,52,77,35,41,41,9,43,43,43,88,88,
88,192,232,232,40,186,186,186,208,213,213,69,118,20,69,69,69,100,220,162,193,96,64,76,76,12,98,99,99,33,22,139,63,101,89,246,189,176,66,44,69,81,112,58,157,152,155,155,195,202,202,138,32,98,217,218,218,
74,238,21,59,58,58,208,213,213,69,146,177,92,88,213,235,245,34,53,53,117,91,36,18,253,71,88,18,177,26,141,6,46,151,11,143,61,246,24,230,230,230,4,178,247,32,247,228,15,239,58,113,226,4,247,246,30,30,98,
197,98,49,180,90,45,166,167,167,113,241,226,69,4,2,1,65,42,246,32,16,219,208,208,0,187,221,142,216,216,88,238,162,122,112,136,85,169,84,104,108,108,196,217,179,103,113,233,210,37,50,198,236,40,16,235,
245,122,201,142,243,64,137,88,153,76,246,31,54,155,141,64,254,212,212,148,192,61,143,58,172,139,235,50,227,237,60,239,15,177,49,49,49,239,57,157,78,220,186,117,11,143,61,246,24,230,231,231,195,158,136,
173,174,174,38,97,9,174,151,227,190,16,43,147,201,62,245,251,253,164,66,243,173,183,222,194,163,143,62,138,201,201,73,4,2,129,176,66,108,80,245,197,222,29,177,58,157,14,203,203,203,248,135,127,248,7,193,
237,209,75,47,189,132,55,223,124,51,108,16,91,93,93,141,178,178,50,152,205,102,178,189,217,179,35,86,163,209,96,101,101,5,103,206,156,193,205,155,55,247,101,158,71,129,216,162,162,34,36,37,37,113,151,
137,221,135,117,153,205,102,108,108,108,160,183,183,23,195,195,195,184,112,225,2,94,125,245,213,136,65,44,231,233,92,218,58,36,17,75,211,52,210,210,210,240,232,163,143,10,142,12,252,211,15,126,72,238,
225,34,1,177,229,229,229,196,19,24,134,121,79,0,177,20,69,33,39,39,7,87,174,92,33,23,198,158,30,31,94,252,254,15,35,58,172,171,178,178,146,140,182,166,105,250,223,5,16,43,149,74,81,89,89,137,139,23,47,
146,139,99,52,32,214,237,118,243,239,7,133,16,75,81,20,38,38,38,16,8,4,162,10,177,37,37,37,36,108,177,179,47,251,26,98,185,46,207,104,87,19,20,20,20,236,14,177,52,77,127,158,150,150,134,141,141,13,82,
83,31,45,136,229,149,108,135,66,172,211,233,196,229,203,151,209,214,214,22,53,136,13,154,179,38,132,216,152,152,152,127,183,219,237,8,4,2,164,31,54,26,16,235,114,185,248,93,9,66,136,149,201,100,239,89,
44,22,242,229,27,45,136,117,56,28,252,221,66,104,34,86,163,209,108,235,116,58,212,213,213,69,173,154,32,232,185,204,238,195,186,52,26,13,178,178,178,208,221,221,29,21,136,229,69,116,247,238,136,165,105,
26,70,163,17,39,79,158,36,157,9,145,130,88,174,150,117,95,29,177,92,64,194,235,245,70,20,98,157,78,39,249,122,161,40,234,211,7,66,108,108,108,44,146,146,146,4,71,225,194,57,172,171,160,160,0,9,9,9,228,
85,82,42,149,239,237,43,17,171,86,171,81,80,80,128,206,206,206,176,13,235,178,219,237,72,79,79,23,80,35,195,48,255,177,111,136,101,24,6,38,147,137,176,245,81,32,150,27,210,21,27,27,123,116,136,85,40,20,
48,153,76,36,31,117,24,136,117,58,157,200,200,200,8,102,198,163,65,108,74,74,10,172,86,43,108,54,27,138,139,139,225,116,58,81,86,86,70,182,29,247,91,80,81,81,17,255,129,80,248,32,86,34,145,252,135,86,
171,133,92,46,135,84,42,5,203,178,80,169,84,80,42,149,208,235,245,72,79,79,71,126,126,62,108,54,27,202,202,202,200,135,121,183,68,117,88,33,86,42,149,190,199,48,204,167,156,103,114,0,171,80,40,32,151,
203,33,147,201,160,80,40,16,27,27,139,180,180,52,164,165,165,237,198,212,145,31,214,197,251,237,33,167,181,185,197,177,44,203,109,220,182,163,62,172,107,23,136,253,124,39,227,240,239,98,177,248,189,111,
19,177,223,232,106,2,165,82,249,243,216,216,216,79,100,50,217,223,30,58,196,138,197,226,109,133,66,65,6,81,156,62,125,26,57,57,57,72,76,76,20,108,218,162,2,177,220,111,155,213,106,197,194,194,2,46,94,
188,72,82,141,220,142,162,182,182,22,89,89,89,208,235,245,36,44,31,246,97,93,124,136,229,106,235,150,150,150,112,246,236,89,210,243,201,221,108,112,251,123,110,47,214,220,220,76,78,166,165,167,167,67,
173,86,111,75,165,210,95,135,5,98,99,99,99,81,83,83,131,233,233,105,114,112,154,243,134,189,220,179,169,169,137,236,245,219,219,219,209,210,210,130,174,174,46,88,44,22,136,197,226,223,31,26,98,213,106,
53,170,170,170,48,62,62,142,43,87,174,96,122,122,250,64,50,204,237,199,26,26,26,200,190,44,43,43,139,19,188,131,65,108,92,92,220,239,171,171,171,225,247,251,113,229,202,21,4,2,129,144,136,238,97,32,182,
174,174,14,14,135,3,74,165,146,187,226,239,15,98,53,26,205,175,207,158,61,139,27,55,110,96,107,107,11,115,115,115,232,233,233,9,27,196,122,189,222,224,46,179,7,87,19,36,37,37,109,63,243,204,51,120,238,
185,231,200,200,160,11,23,46,144,142,141,112,12,235,42,41,41,225,191,90,95,62,176,154,32,59,59,27,27,27,27,33,130,247,157,239,124,39,232,120,249,225,33,150,59,20,205,111,126,219,19,98,105,154,70,113,113,
49,174,94,189,138,51,103,206,224,31,255,241,31,239,123,99,121,148,68,172,221,110,135,90,173,38,12,116,223,97,93,125,125,125,88,92,92,68,123,123,59,252,126,63,158,120,226,137,136,64,108,69,69,5,142,31,
63,206,239,227,216,29,98,229,114,57,206,157,59,39,104,210,61,119,238,124,196,32,150,203,167,239,28,212,8,133,88,110,148,245,250,250,58,185,22,181,182,182,226,123,47,190,20,177,142,88,110,218,147,70,163,
225,158,220,191,22,2,177,197,197,197,88,95,95,71,119,119,119,212,32,214,237,118,195,100,50,113,135,201,126,46,128,88,150,101,17,8,4,112,246,236,217,168,38,98,43,43,43,113,226,196,137,144,9,247,229,20,
69,253,213,100,50,97,117,117,21,131,131,131,81,133,216,138,138,10,126,214,243,111,2,136,117,58,157,228,173,139,38,196,114,95,59,33,16,171,84,42,255,54,62,62,142,185,185,57,193,151,107,52,32,54,200,168,
190,134,216,248,248,248,79,26,26,26,48,58,58,42,248,182,143,70,34,54,104,168,252,215,16,171,80,40,126,110,50,153,200,111,93,180,32,118,151,183,78,152,136,149,201,100,56,118,236,24,218,218,218,200,14,50,
210,16,91,84,84,36,184,139,14,129,88,134,97,182,211,210,210,80,81,81,17,181,97,93,199,143,31,127,48,196,138,68,34,24,141,70,52,53,53,161,165,165,37,226,16,107,52,26,31,12,177,92,48,34,63,63,31,189,189,
189,17,77,196,150,150,150,10,16,237,129,16,155,158,158,78,62,87,145,130,216,210,210,82,193,248,244,7,66,172,82,169,68,94,94,30,124,62,95,216,59,98,185,179,238,92,155,210,129,170,9,210,211,211,209,214,
214,70,250,18,194,53,172,171,176,176,16,41,41,41,4,99,25,134,249,245,190,33,86,165,82,33,63,63,31,221,221,221,100,65,71,73,196,90,173,86,24,12,134,224,68,227,239,15,12,177,156,14,115,83,121,185,195,170,
7,129,88,155,205,134,212,212,212,224,197,28,30,98,149,74,229,239,181,90,45,148,74,37,180,90,45,210,211,211,145,155,155,75,134,113,113,27,181,221,22,196,77,12,219,133,25,143,14,177,114,185,252,215,50,153,
108,91,161,80,32,38,38,134,240,161,90,173,134,209,104,132,205,102,131,221,110,39,159,29,238,233,84,80,37,65,248,33,150,63,172,139,171,27,80,40,20,224,18,178,42,149,10,105,105,105,200,207,207,71,86,86,
214,94,175,78,244,32,86,36,18,65,46,151,67,169,84,146,206,88,94,29,216,195,135,88,177,88,252,55,177,88,252,9,77,211,63,255,182,154,224,239,166,35,86,171,213,126,177,83,230,255,240,32,118,167,91,3,26,141,
6,109,109,109,36,245,81,92,92,28,156,67,143,14,196,114,207,103,234,234,234,72,212,114,100,100,4,93,93,93,130,201,78,73,73,73,252,120,91,100,32,86,38,147,65,167,211,161,180,180,20,243,243,243,184,120,241,
34,252,126,63,250,250,250,4,204,216,212,212,132,134,134,6,52,52,52,192,233,116,226,196,137,19,208,104,52,92,165,74,120,170,9,212,106,53,78,158,60,73,6,204,159,61,123,22,51,51,51,2,25,222,205,61,185,237,
14,87,77,80,87,87,135,188,188,60,24,141,70,176,44,251,193,161,32,86,175,215,223,171,171,171,195,248,248,56,46,95,190,140,185,185,57,76,76,76,160,179,179,147,132,80,247,3,177,95,237,92,191,54,207,198,198,
70,88,44,22,176,44,123,239,64,16,171,82,169,62,88,88,88,192,19,79,60,129,173,173,45,4,2,129,35,15,235,226,22,229,245,122,113,236,216,49,238,202,191,255,106,130,172,172,44,252,227,63,254,35,41,132,124,
241,197,23,5,11,59,10,196,114,66,204,171,251,125,48,196,178,44,187,93,91,91,139,171,87,175,10,194,166,63,248,193,15,112,238,220,185,176,64,108,85,85,21,146,146,146,72,144,254,129,16,27,19,19,131,165,165,
37,204,207,207,99,113,113,81,80,50,42,60,120,127,180,97,93,121,121,121,228,136,203,125,33,150,139,148,112,125,27,221,221,221,152,155,155,195,143,126,244,163,176,67,44,55,26,157,215,62,191,59,196,82,20,
5,151,203,133,205,205,77,2,250,157,157,157,187,78,128,58,42,196,242,135,42,221,23,98,181,90,45,230,231,231,113,230,204,25,242,91,54,61,61,19,17,136,173,170,170,130,205,102,35,179,102,118,237,136,229,32,
246,137,39,158,32,207,97,34,13,177,110,183,27,105,105,105,100,146,235,174,29,177,147,147,147,130,106,130,104,64,172,205,102,227,127,182,132,213,4,6,131,129,52,220,68,19,98,93,46,23,204,102,51,247,228,
65,152,136,245,249,124,184,116,233,18,186,187,187,163,14,177,156,14,211,52,253,5,129,88,189,94,143,177,177,49,65,105,109,180,32,118,151,74,214,175,32,86,167,211,125,81,83,83,67,106,9,162,9,177,65,36,36,
76,196,166,167,167,163,179,179,19,29,29,29,81,133,216,83,167,78,237,93,77,160,209,104,96,183,219,209,211,211,19,213,106,130,189,70,229,153,40,138,122,141,97,24,152,205,102,129,16,71,26,98,11,11,11,239,
15,177,59,247,249,176,90,173,232,234,234,34,231,167,34,9,177,65,157,140,187,67,172,68,34,129,197,98,193,200,200,136,160,39,54,18,16,235,112,56,130,195,204,123,67,172,86,171,69,77,77,13,58,58,58,34,10,
177,65,143,63,238,15,177,82,169,20,102,179,25,195,195,195,17,27,214,197,93,197,247,221,17,75,81,212,151,105,105,105,240,249,124,65,58,28,94,136,229,181,1,236,15,98,213,106,53,108,54,27,186,187,187,5,51,
103,195,1,177,101,101,101,130,115,13,7,170,38,72,79,79,71,99,99,35,185,111,59,42,196,58,157,78,20,22,22,194,104,52,242,63,224,31,28,8,98,53,26,205,189,248,248,120,112,61,196,92,232,148,195,216,253,64,
108,89,89,25,138,139,139,201,12,100,222,97,67,80,20,117,239,80,16,43,149,74,63,224,130,166,82,169,20,42,149,10,22,139,5,133,133,133,168,169,169,65,101,101,229,158,195,186,108,54,27,76,38,19,201,29,240,
127,194,82,77,192,48,204,182,88,44,134,92,46,39,105,88,157,78,135,252,252,124,56,157,78,193,130,138,138,138,96,50,153,200,91,20,180,152,200,64,44,23,52,85,42,149,136,137,137,129,74,165,66,94,94,158,0,
97,249,195,2,249,207,95,162,2,177,44,203,18,132,229,250,18,118,25,214,245,240,32,118,167,31,225,139,111,59,98,247,3,177,34,145,200,243,141,26,214,165,82,169,16,27,27,251,133,92,46,255,228,161,65,44,247,
219,165,215,235,225,116,58,49,49,49,1,159,207,7,183,219,77,46,148,81,131,88,145,72,4,181,90,141,228,228,100,116,117,117,97,98,98,2,23,47,94,36,202,215,208,208,0,143,199,131,220,220,92,168,213,106,193,
37,33,108,16,75,211,244,47,21,10,197,182,66,161,192,241,227,199,81,85,85,133,145,145,17,204,207,207,99,126,126,30,99,99,99,240,249,124,130,91,51,78,236,60,30,15,210,211,211,17,27,27,75,66,129,71,134,88,
163,209,120,207,233,116,194,231,243,225,204,153,51,8,4,2,152,159,159,199,228,228,36,250,251,251,209,217,217,121,95,247,228,190,138,42,42,42,144,157,157,141,132,132,4,62,43,30,28,98,21,10,197,135,211,211,
211,216,218,218,194,230,230,38,102,103,103,225,243,249,4,243,220,247,11,177,220,150,167,170,170,10,249,249,249,48,26,141,96,24,230,195,3,87,19,228,230,230,226,185,231,158,35,238,121,235,214,45,76,78,78,
30,26,98,185,183,148,219,101,236,156,52,186,119,160,142,216,193,193,65,172,173,173,225,173,183,222,34,219,218,27,55,110,96,116,116,244,200,16,91,85,85,133,140,140,12,238,115,246,203,7,66,44,119,156,252,
137,39,158,192,244,244,52,86,87,87,133,55,150,59,238,121,20,136,173,173,173,37,119,198,52,77,63,120,88,151,68,34,65,77,77,13,73,157,13,14,14,226,234,213,171,187,222,92,30,5,98,157,78,39,169,91,121,32,
196,26,12,6,172,175,175,35,16,8,144,207,143,223,239,23,188,141,225,128,216,202,202,74,65,149,216,158,16,43,18,137,80,83,83,131,181,181,53,242,236,165,189,189,61,98,195,186,130,238,144,119,135,216,148,
148,20,60,249,228,147,152,153,153,137,56,196,122,189,94,114,87,195,237,74,67,32,150,27,130,179,178,178,66,30,8,69,26,98,93,46,87,240,91,40,132,216,212,212,84,92,187,118,13,19,19,19,81,133,216,160,183,
240,107,136,213,104,52,159,76,76,76,96,121,121,89,240,213,17,173,142,88,169,84,202,189,133,95,161,25,195,48,158,132,132,132,47,38,39,39,49,54,54,38,248,234,136,6,196,186,92,46,254,220,144,175,59,98,213,
106,53,236,118,59,38,38,38,30,42,196,238,188,90,95,119,196,102,100,100,144,203,64,52,33,150,155,67,186,243,74,125,65,211,180,135,64,172,94,175,135,215,235,37,103,171,162,5,177,101,101,101,252,225,74,159,
8,32,150,139,188,13,15,15,147,69,69,163,154,32,200,208,133,16,203,237,183,187,187,187,209,209,209,17,21,136,117,58,157,216,73,126,236,13,177,113,113,113,112,56,28,24,28,28,140,120,34,214,229,114,5,79,
164,219,59,17,107,177,88,48,58,58,42,216,164,69,2,98,139,139,139,131,19,30,123,67,172,217,108,38,17,240,72,66,108,240,220,190,251,66,172,82,169,36,143,105,35,5,177,193,17,221,7,66,172,66,161,216,206,203,
203,195,240,240,176,96,183,24,78,136,229,79,204,220,169,66,184,63,196,210,52,253,75,206,59,249,93,9,225,132,88,254,243,152,125,67,108,66,66,194,189,132,132,4,152,76,38,156,58,117,10,213,213,213,100,81,
135,133,88,126,215,240,161,171,9,36,18,201,135,113,113,113,80,171,213,208,106,181,200,204,204,132,219,237,62,84,53,129,213,106,69,110,110,46,244,122,61,63,240,245,225,161,171,9,196,98,49,148,74,37,212,
106,53,98,99,99,97,54,155,81,83,83,131,138,138,138,61,23,84,90,90,138,252,252,124,152,76,38,104,52,26,178,53,225,158,170,243,100,216,116,164,80,151,72,36,66,76,76,12,226,226,226,160,209,104,144,147,147,
3,175,215,75,134,119,185,221,110,148,148,148,192,98,177,112,227,132,5,141,168,59,24,187,45,22,139,127,25,118,136,165,105,154,188,114,241,241,241,176,90,173,176,90,173,72,79,79,135,66,161,224,103,12,248,
21,78,209,129,88,150,101,161,84,42,161,80,40,32,145,72,130,105,250,225,66,172,84,42,253,68,34,145,124,177,243,170,124,115,134,117,237,108,204,190,77,196,238,11,98,119,62,75,127,141,141,141,253,84,38,147,
253,252,161,65,44,247,219,165,211,233,96,177,88,200,28,101,151,203,5,157,78,71,162,223,17,129,88,154,166,95,167,40,234,35,150,101,201,115,25,179,217,140,252,252,124,116,118,118,98,98,98,2,243,243,243,
8,4,2,232,239,239,71,107,107,43,41,229,227,93,193,195,7,177,6,131,225,131,180,180,180,109,143,199,131,218,218,90,12,15,15,99,120,120,24,115,115,115,8,4,2,152,157,157,197,240,240,48,122,122,122,200,237,
217,233,211,167,81,85,85,69,234,120,120,21,210,71,135,88,150,101,127,215,222,222,142,107,215,174,97,117,117,21,115,115,115,24,29,29,69,127,127,63,124,62,159,192,28,118,171,38,104,104,104,32,79,214,185,
195,170,135,122,222,199,63,162,146,147,147,131,91,183,110,225,187,223,253,46,238,222,189,139,173,173,45,12,15,15,31,24,98,185,144,51,119,118,230,208,29,177,50,153,12,227,227,227,88,91,91,19,68,41,87,86,
86,200,91,181,95,136,229,132,152,155,91,116,234,212,41,232,245,122,136,68,162,223,237,27,98,105,154,198,169,83,167,112,237,218,53,76,79,79,99,115,115,147,44,234,245,215,95,199,202,202,202,161,32,150,63,
167,148,251,13,149,74,165,31,236,11,98,101,50,25,166,167,167,177,180,180,68,30,117,112,85,5,95,131,236,209,32,214,237,118,67,175,215,115,16,251,250,125,33,150,101,89,84,84,84,96,99,99,67,240,249,9,126,
27,239,222,189,123,36,136,173,169,169,65,114,114,242,254,32,54,62,62,30,43,43,43,152,155,155,35,191,93,125,125,125,97,31,214,85,93,93,45,208,150,61,33,86,171,213,226,236,217,179,88,93,93,21,116,85,71,
10,98,185,161,223,123,38,98,25,134,217,246,122,189,228,40,121,52,32,182,164,164,68,176,247,10,129,88,139,197,130,155,55,111,98,102,102,38,170,16,203,207,80,9,32,54,54,54,246,231,45,45,45,88,89,89,17,92,
24,35,13,177,21,21,21,123,102,242,74,226,227,227,63,109,104,104,192,236,236,172,224,74,29,13,136,13,26,65,37,76,196,102,100,100,144,71,31,209,132,216,162,162,34,254,190,254,107,136,229,58,135,71,71,71,
209,217,217,25,85,136,117,58,157,193,227,94,190,134,88,189,94,143,230,230,102,244,247,247,71,21,98,203,203,203,249,197,144,194,68,44,87,134,60,49,49,17,85,136,229,71,43,105,154,254,84,0,177,156,226,77,
79,79,147,211,218,209,72,196,6,77,124,250,121,8,196,166,166,166,98,124,124,60,106,16,91,86,86,38,120,54,179,43,196,170,213,106,244,245,245,193,231,243,69,165,154,32,104,30,205,246,174,16,43,147,201,80,
93,93,141,225,225,225,136,119,196,218,237,118,168,84,170,7,67,44,203,178,56,117,234,20,198,199,199,35,58,172,203,233,116,6,119,111,220,31,98,179,178,178,176,176,176,32,56,212,21,110,136,205,207,207,15,
134,144,189,33,150,166,233,215,245,122,253,182,215,235,13,73,195,134,11,98,157,78,39,127,115,183,63,136,213,106,181,31,36,39,39,35,45,45,141,68,39,195,9,177,65,51,32,247,15,177,12,195,252,78,163,209,64,
173,86,35,45,45,13,118,187,157,68,66,142,10,177,73,73,73,71,235,136,21,139,197,224,48,54,51,51,19,229,229,229,100,81,7,153,141,85,86,86,134,210,210,210,224,118,146,195,119,196,238,76,21,135,70,163,129,
82,169,36,67,189,185,124,84,240,130,184,0,124,97,97,33,114,115,115,145,153,153,137,164,164,36,196,197,197,241,207,189,255,46,44,16,203,48,12,228,114,57,17,226,134,134,6,210,149,224,118,187,97,183,219,
145,157,157,13,189,94,79,14,223,115,141,1,220,15,77,211,219,12,195,124,16,118,136,85,40,20,136,143,143,135,193,96,64,78,78,14,178,179,179,185,155,202,144,69,240,81,118,7,98,95,143,24,196,210,52,189,173,
86,171,9,194,242,23,242,208,19,177,44,203,254,92,34,145,124,186,215,176,174,111,19,177,127,55,16,27,252,246,177,44,123,135,166,233,213,168,67,44,247,155,37,22,139,161,80,40,144,146,146,130,236,236,108,
84,85,85,161,176,176,16,90,173,54,114,16,171,84,42,127,22,31,31,255,167,148,148,20,152,76,38,228,228,228,224,196,137,19,40,46,46,70,85,85,21,78,159,62,77,236,115,126,126,30,19,19,19,232,234,234,130,195,
225,224,95,52,195,7,177,34,145,232,87,57,57,57,219,92,29,193,228,228,36,206,156,57,131,241,241,113,76,79,79,99,108,108,12,19,19,19,228,220,114,103,103,39,185,241,168,172,172,68,97,97,33,127,128,196,151,
97,169,38,72,75,75,195,141,27,55,112,243,230,77,124,255,251,223,199,163,143,62,42,0,144,251,185,103,83,83,19,121,43,141,70,35,191,65,254,240,16,203,178,44,122,123,123,113,245,234,85,188,247,222,123,100,
142,251,220,220,220,129,58,98,57,136,53,155,205,144,203,229,220,67,129,131,67,172,88,44,134,195,225,192,230,230,38,2,129,0,110,220,184,65,118,146,207,61,247,220,142,157,239,63,17,91,85,85,5,135,195,1,
171,213,138,164,164,164,131,117,196,114,253,83,122,189,30,43,43,43,152,159,159,71,87,87,23,102,103,103,241,218,107,95,223,20,188,242,202,171,7,134,88,46,74,201,125,121,171,84,42,208,52,253,171,125,65,
172,86,171,197,228,228,36,30,121,228,17,50,61,204,231,243,225,209,71,31,37,139,122,239,189,195,65,44,87,199,202,157,23,149,201,100,219,18,137,228,103,247,133,88,138,162,208,220,220,140,107,215,174,97,
100,100,132,20,55,204,206,206,6,117,196,30,13,98,171,171,171,97,183,219,33,151,203,193,48,204,159,246,132,88,133,66,1,175,215,139,171,87,175,10,218,79,35,5,177,46,151,139,63,25,113,119,136,205,207,207,
199,218,218,26,2,129,128,32,119,30,41,136,173,172,172,20,228,20,4,16,43,18,137,86,205,102,51,174,93,187,134,249,249,121,193,92,172,72,66,108,69,69,5,191,171,74,8,177,114,185,252,142,213,106,37,201,106,
126,16,62,146,16,235,241,120,4,73,179,144,68,108,86,86,86,212,33,214,227,241,236,13,177,20,69,189,111,54,155,49,59,59,27,245,68,44,79,241,66,33,54,53,53,21,211,211,211,130,39,156,209,128,216,160,44,213,
95,5,29,177,74,165,18,157,157,157,223,156,97,93,20,69,253,85,34,145,192,225,112,144,122,130,104,65,172,195,225,216,149,172,75,40,138,122,95,36,18,33,55,55,23,103,207,158,141,42,196,6,101,61,133,213,4,
52,77,35,43,43,11,243,243,243,81,75,196,114,222,192,131,216,208,68,108,86,86,22,150,151,151,163,54,172,139,115,7,174,219,133,97,152,59,33,16,155,145,145,129,165,165,165,168,13,235,114,56,28,130,33,57,
122,189,126,53,4,98,77,38,19,89,84,164,33,150,219,137,62,16,98,13,6,3,121,104,20,73,136,221,37,93,189,55,196,106,52,154,63,153,205,102,180,181,181,69,20,98,243,242,242,130,7,45,237,13,177,114,185,252,
103,58,157,14,122,189,30,249,249,249,168,173,173,13,59,196,218,237,246,224,22,203,125,37,98,127,197,133,0,51,50,50,80,89,89,137,218,218,218,176,64,172,211,233,12,238,26,62,88,53,1,195,48,208,106,181,80,
171,213,40,42,42,34,39,27,143,2,177,65,230,121,56,136,101,24,134,84,177,158,58,117,10,94,175,247,80,16,235,114,185,80,82,82,18,156,172,62,60,196,114,164,168,211,233,112,236,216,177,7,46,136,207,211,78,
167,19,69,69,69,40,42,42,10,158,253,248,155,176,64,44,23,115,227,198,232,113,79,203,185,250,10,174,45,144,27,151,30,27,27,11,165,82,9,185,92,46,248,77,19,139,197,191,10,43,196,178,44,11,173,86,11,189,
94,15,147,201,4,189,94,47,152,131,37,22,139,73,97,36,55,149,71,44,22,115,50,252,39,165,82,249,179,136,65,44,135,176,28,75,139,197,226,221,210,176,209,133,216,152,152,152,85,137,68,114,231,126,146,247,
45,196,254,93,67,44,151,136,165,105,186,54,162,16,171,84,42,255,89,44,22,127,196,178,236,54,215,209,194,253,186,115,183,223,177,177,177,80,171,213,208,104,52,72,73,73,217,150,203,229,247,34,86,77,160,
80,40,62,229,223,57,215,215,215,163,185,185,25,85,85,85,228,43,168,183,183,23,62,159,15,125,125,125,152,158,158,70,103,103,39,255,1,118,120,59,98,99,99,99,183,125,62,31,158,122,234,41,108,110,110,98,101,
101,5,35,35,35,232,235,235,195,192,192,0,250,251,251,49,56,56,136,254,254,126,244,244,244,144,222,198,166,166,38,120,189,94,100,101,101,113,222,25,30,136,165,105,26,78,167,19,87,174,92,33,135,239,223,
122,235,45,178,25,124,144,123,54,52,52,192,225,112,32,45,45,141,31,224,58,60,196,138,197,98,156,58,117,10,143,62,250,40,22,23,23,241,228,147,79,146,141,219,181,107,215,66,202,27,246,130,216,154,154,26,
56,28,142,224,41,154,7,135,88,174,219,115,101,101,5,139,139,139,232,233,233,193,217,179,103,5,33,212,185,185,185,125,67,108,77,77,13,138,139,139,5,253,65,135,130,216,64,32,128,229,229,101,226,158,195,
195,195,184,117,235,22,89,216,171,63,126,3,55,111,62,187,111,136,229,132,56,47,47,143,63,129,252,23,251,130,88,163,209,136,115,231,206,225,210,165,75,24,26,26,18,48,35,87,56,250,222,123,119,241,218,235,
111,30,24,98,107,106,106,200,33,177,248,248,120,72,36,146,7,87,19,24,141,70,204,206,206,98,121,121,89,48,236,109,118,118,54,196,61,143,90,77,96,181,90,185,234,139,79,247,132,88,179,217,140,203,151,47,
99,101,101,5,67,67,67,68,134,35,1,177,220,233,164,172,172,44,110,167,241,207,2,136,165,105,186,54,62,62,254,158,223,239,199,197,139,23,5,105,216,72,66,172,215,235,229,223,255,125,20,146,136,53,155,205,
184,114,229,10,70,71,71,5,11,138,36,196,86,85,85,241,23,21,58,172,43,33,33,1,139,139,139,130,207,81,164,33,54,104,81,161,16,155,152,152,136,193,193,65,12,12,12,124,51,170,9,40,138,122,63,62,62,30,253,
253,253,24,29,29,141,42,196,6,45,74,8,177,20,69,161,177,177,81,208,4,16,13,136,13,146,23,33,196,114,207,101,252,126,127,84,33,54,59,59,123,111,136,165,40,10,149,149,149,8,4,2,81,133,216,32,18,18,66,172,
76,38,131,199,227,65,32,16,136,42,196,6,37,207,66,33,182,174,174,14,129,64,32,170,29,177,54,155,237,254,195,186,234,235,235,49,53,53,21,85,136,229,15,236,10,73,196,74,165,210,237,230,230,102,248,253,126,
193,6,45,146,16,27,28,28,12,73,196,178,44,251,145,219,237,134,223,239,23,108,208,34,57,172,203,110,183,243,179,232,247,44,22,75,173,0,98,21,10,197,63,155,76,38,120,60,30,50,218,32,146,16,155,159,159,47,
120,214,183,39,196,74,165,210,79,117,58,29,204,102,51,57,10,23,9,136,181,217,108,252,157,231,131,171,9,100,50,217,54,183,48,175,215,27,118,136,205,207,207,15,22,189,47,247,213,17,75,211,52,116,58,29,18,
18,18,224,116,58,209,220,220,28,22,136,181,219,237,130,183,236,80,16,171,209,104,72,15,241,81,171,9,130,31,121,28,26,98,41,138,130,90,173,70,98,98,34,156,78,39,26,26,26,14,92,77,192,213,91,240,250,128,
194,3,177,220,43,214,208,208,128,234,234,234,144,5,85,84,84,160,162,162,2,229,229,229,187,202,112,208,35,217,240,65,172,70,163,129,193,96,128,221,110,39,242,123,242,228,73,156,56,113,2,217,217,217,48,
153,76,48,155,205,48,153,76,200,204,204,68,70,70,6,178,178,178,96,50,153,248,185,169,237,176,119,196,198,199,199,67,169,84,34,54,54,150,128,171,88,44,38,0,203,178,44,164,82,41,249,215,220,255,102,103,
65,159,70,172,154,64,38,147,221,219,9,205,16,249,229,38,139,241,79,113,239,68,116,183,105,154,254,72,46,151,255,115,164,33,182,118,175,97,93,223,66,236,255,115,16,27,241,68,44,195,48,183,147,146,146,62,
51,153,76,219,70,163,17,137,137,137,144,201,100,136,137,137,9,249,224,243,70,78,125,40,145,72,238,210,52,61,25,145,142,88,131,193,128,201,201,73,172,173,173,145,92,76,127,127,63,26,26,26,208,212,212,68,
178,118,13,13,13,164,152,166,182,182,22,89,89,89,144,201,100,127,14,251,176,46,141,70,131,64,32,128,173,173,45,188,248,226,139,120,243,205,55,73,209,95,71,71,7,122,122,122,208,221,221,77,26,2,124,62,31,
169,108,173,173,173,229,158,136,134,7,98,69,34,17,12,6,3,150,151,151,177,190,190,142,173,173,45,178,79,218,216,216,16,12,56,217,203,61,107,107,107,201,124,200,35,117,196,114,95,200,122,189,158,148,141,
246,247,247,99,126,126,158,44,234,197,23,95,196,200,200,200,3,33,182,177,177,17,46,151,43,120,52,227,225,33,118,102,102,6,171,171,171,132,25,199,199,199,241,131,31,252,128,44,236,202,149,245,125,65,44,
119,200,222,100,50,241,191,122,246,15,177,10,133,2,118,187,29,143,62,250,40,46,94,188,40,80,61,159,207,135,155,55,111,146,69,125,255,165,151,177,113,128,106,130,194,194,66,232,245,122,126,223,226,254,
32,214,235,245,98,105,105,9,103,207,158,21,4,7,251,250,250,136,234,189,247,222,93,220,126,235,109,252,195,243,47,30,8,98,185,129,74,59,109,186,247,31,214,197,48,204,164,94,175,255,115,32,16,32,33,102,
126,251,105,56,33,214,227,241,160,164,164,4,70,163,145,4,232,119,133,88,153,76,118,183,164,164,4,155,155,155,24,30,30,22,12,124,139,4,196,122,60,30,20,20,20,32,57,57,25,82,169,20,34,145,232,246,174,137,
216,218,218,90,44,45,45,145,3,247,145,134,88,183,219,141,130,130,2,168,213,106,136,197,226,207,66,32,86,44,22,111,59,157,78,204,207,207,71,21,98,93,46,23,247,172,48,20,98,101,50,25,220,110,55,22,22,22,
162,10,177,78,167,147,63,111,77,8,177,20,69,161,162,162,130,188,82,209,130,88,135,195,177,55,196,114,145,202,249,249,249,168,66,108,208,220,25,33,196,178,44,139,186,186,58,44,44,44,68,21,98,131,36,79,
8,177,44,203,162,189,189,157,152,103,180,32,54,136,172,133,16,27,23,23,135,137,137,9,140,143,143,71,21,98,131,14,73,11,33,86,169,84,98,105,105,9,67,67,67,81,133,216,160,69,9,33,54,37,37,5,23,46,92,64,
119,119,119,84,33,54,40,250,45,132,88,147,201,132,205,205,77,116,118,118,70,21,98,249,139,10,129,216,148,148,148,109,238,177,90,52,33,150,167,49,161,213,4,9,9,9,159,233,245,122,120,60,30,180,181,181,69,
5,98,131,186,61,67,19,177,52,77,223,214,104,52,72,76,76,252,106,33,59,65,135,72,65,44,71,141,220,45,153,76,38,187,123,223,142,88,157,78,135,242,242,114,65,243,105,56,33,182,164,164,132,116,156,237,188,
74,127,214,106,181,147,247,29,214,21,31,31,15,173,86,11,183,219,45,168,40,8,7,196,150,150,150,194,104,52,10,206,42,239,27,98,213,106,53,116,58,29,58,59,59,195,86,77,144,159,159,31,220,28,127,48,136,229,
10,178,114,115,115,9,93,31,5,98,11,10,10,248,131,78,143,6,177,6,131,1,217,217,217,104,107,107,19,44,232,32,16,187,11,230,31,13,98,165,82,41,12,6,3,121,251,184,197,84,85,85,237,57,172,43,248,21,10,74,85,
135,15,98,19,18,18,96,54,155,145,147,147,67,208,213,98,177,32,45,45,13,22,139,5,57,57,57,200,205,205,69,113,113,49,108,54,27,74,75,75,97,181,90,67,98,222,97,133,88,181,90,253,103,110,38,187,84,42,133,
76,38,35,240,202,253,103,92,61,1,199,67,188,99,114,145,233,136,149,201,100,147,44,203,222,229,170,9,56,112,229,119,195,114,53,5,156,12,51,12,179,77,211,244,103,50,153,236,246,67,25,214,245,45,196,254,
93,65,172,74,165,250,23,177,88,252,111,15,29,98,185,106,2,165,82,185,221,218,218,74,106,48,53,26,13,84,42,21,73,38,6,119,112,68,116,88,23,203,178,56,121,242,36,97,198,229,229,101,12,12,12,96,96,96,0,205,
205,205,228,12,251,177,99,199,144,158,158,142,244,244,116,104,181,90,40,20,10,40,149,202,207,196,98,241,157,176,66,172,66,161,32,33,212,43,87,174,224,205,55,223,20,184,231,233,211,167,201,148,96,46,170,
219,217,217,137,190,190,62,244,247,247,115,227,206,182,195,2,177,20,69,65,46,151,195,235,245,98,121,121,25,115,115,115,152,155,155,35,123,165,23,94,120,65,112,122,146,223,17,251,213,205,71,43,26,27,27,
201,52,177,157,222,235,223,30,9,98,117,58,29,25,71,53,59,59,139,158,158,30,76,76,76,224,213,87,95,37,145,221,169,169,169,7,66,108,93,93,29,242,243,243,57,246,57,56,196,210,52,253,219,248,248,120,84,86,
86,98,125,125,29,43,43,43,164,216,175,189,189,29,3,3,3,120,230,153,103,200,171,181,185,249,248,3,33,182,190,190,30,21,21,21,200,201,201,9,206,16,239,15,98,13,6,3,250,250,250,112,225,194,5,204,206,206,
10,220,179,189,189,125,231,149,122,109,199,62,223,195,173,103,255,1,43,251,132,88,174,36,153,215,157,126,127,136,229,159,220,190,120,241,34,198,198,198,246,132,216,59,119,238,226,39,63,185,141,151,95,
254,17,174,111,61,117,160,106,130,252,252,124,40,149,74,50,169,238,190,137,88,157,78,247,217,210,210,18,150,150,150,224,243,249,4,238,25,10,177,135,31,214,229,118,187,145,153,153,9,165,82,201,221,60,236,
221,17,171,86,171,113,227,198,141,168,64,108,105,105,41,44,22,11,153,75,186,103,53,1,55,229,158,79,213,145,130,88,78,241,120,25,245,221,135,117,101,102,102,146,6,230,104,64,44,55,237,41,62,62,30,52,77,
135,66,44,69,81,176,88,44,216,220,220,36,31,238,104,64,108,69,69,5,169,168,147,72,36,255,34,128,88,154,166,113,236,216,49,172,175,175,71,21,98,43,43,43,201,162,104,154,254,55,1,196,82,20,69,190,74,162,
9,177,21,21,21,252,97,56,66,136,85,40,20,232,233,233,193,153,51,103,162,10,177,65,221,195,66,136,77,76,76,196,234,234,42,153,202,19,45,136,61,121,242,228,222,16,171,215,235,177,181,181,5,159,207,23,85,
136,205,200,200,216,27,98,185,250,20,254,183,125,52,32,150,55,168,75,8,177,44,203,254,155,195,225,32,231,218,163,5,177,46,151,43,248,14,250,107,136,85,42,149,255,146,145,145,65,18,214,209,130,88,135,195,
193,175,62,12,133,88,133,66,177,157,157,157,141,214,214,214,168,65,108,208,195,162,221,135,117,25,12,6,228,230,230,18,243,140,52,196,6,93,14,118,175,38,16,139,197,224,22,198,111,2,136,20,196,242,155,74,
238,219,17,43,147,201,144,146,146,2,183,219,141,214,214,214,136,64,172,203,229,66,113,113,49,159,171,63,187,111,34,150,162,168,47,165,82,41,82,82,82,224,243,249,194,158,136,45,45,45,69,78,78,142,160,222,
87,169,84,222,121,32,196,114,49,202,162,162,162,176,14,235,42,45,45,133,217,108,230,39,56,14,14,177,169,169,169,112,56,28,104,105,105,17,148,89,29,20,98,157,78,39,25,111,205,151,97,138,162,126,123,96,
136,141,141,141,133,193,96,64,123,123,123,200,25,170,253,66,172,195,225,64,122,122,186,224,122,116,36,136,229,238,255,142,29,59,6,167,211,9,143,199,131,178,178,50,210,19,203,109,61,238,183,32,147,201,
180,219,153,247,163,67,172,78,167,35,7,236,21,10,5,212,106,53,226,226,226,160,211,233,144,145,145,129,227,199,143,195,102,179,145,154,157,221,58,91,194,14,177,44,203,222,145,203,229,159,113,201,87,153,
76,6,153,76,6,185,92,14,133,66,65,32,54,38,38,6,201,201,201,48,153,76,187,185,121,228,135,117,241,255,129,124,124,229,22,185,115,163,185,253,80,134,117,237,38,121,34,145,232,223,88,150,253,151,111,33,
246,27,11,177,52,77,191,161,86,171,127,21,23,23,119,79,36,18,125,254,208,33,86,34,145,108,167,165,165,161,189,189,29,61,61,61,168,174,174,38,201,68,94,214,46,58,16,203,245,34,20,23,23,99,110,110,14,143,
61,246,24,158,124,242,73,44,46,46,162,179,179,19,77,77,77,112,56,28,200,201,201,33,97,64,238,183,147,166,233,143,195,158,136,229,190,164,123,123,123,177,190,190,142,165,165,37,210,90,249,202,43,175,144,
225,1,167,79,159,198,233,211,167,209,212,212,68,182,62,86,171,21,22,139,5,58,157,110,91,38,147,253,39,77,211,23,143,12,177,74,165,18,30,143,7,171,171,171,228,222,112,116,116,20,255,244,79,255,68,42,90,
185,161,20,124,247,228,22,215,210,210,130,142,142,14,120,60,30,24,12,6,72,36,146,15,15,5,177,52,77,191,31,31,31,143,210,210,82,204,207,207,99,125,125,29,211,211,211,4,63,6,6,6,4,135,239,207,156,57,115,
95,136,173,175,175,71,85,85,21,178,178,178,184,173,203,251,251,134,88,134,97,46,106,52,154,15,109,54,27,70,71,71,113,238,220,57,4,2,1,210,4,192,253,116,119,119,227,137,39,190,238,141,125,236,241,235,15,
132,88,238,173,84,169,84,156,224,237,15,98,53,26,205,127,158,62,125,26,171,171,171,152,159,159,199,240,240,112,8,196,142,141,141,227,123,47,190,132,31,189,250,58,222,123,239,61,188,253,246,219,184,113,
227,38,230,23,46,220,23,98,235,234,234,80,89,89,137,140,140,12,254,70,239,193,137,88,179,217,140,245,245,117,204,204,204,144,129,205,123,65,236,15,95,121,29,47,190,248,3,220,122,246,31,112,233,242,198,
190,33,182,180,180,20,106,181,154,76,182,191,47,196,210,52,253,113,118,118,54,30,127,252,241,136,66,44,87,22,194,75,112,236,13,177,18,137,4,149,149,149,184,124,249,114,84,32,86,163,209,128,247,216,119,
119,136,165,40,10,129,64,0,103,207,158,141,10,196,158,56,113,2,50,153,108,239,97,93,20,69,125,169,84,42,177,188,188,140,145,145,145,168,64,108,89,89,25,82,83,83,177,211,166,179,59,196,154,205,102,172,
172,172,144,122,130,104,64,172,205,102,67,124,124,60,215,97,245,134,0,98,25,134,65,126,126,62,46,93,186,4,159,207,23,53,136,229,158,62,236,76,55,248,149,0,98,149,74,37,122,123,123,113,254,252,121,242,
42,69,3,98,43,43,43,249,211,233,238,9,32,86,167,211,225,218,181,107,24,27,27,139,42,196,86,84,84,240,11,253,62,23,64,108,126,126,62,174,93,187,246,80,134,117,5,245,48,126,5,177,98,177,248,243,254,254,
126,44,46,46,10,30,187,70,3,98,247,202,14,151,36,38,38,222,107,108,108,36,188,24,45,136,229,30,72,238,10,177,106,181,250,87,38,147,9,221,221,221,104,107,107,139,26,196,150,148,148,240,159,54,8,33,150,
166,233,55,146,146,146,144,157,157,13,159,207,23,53,136,229,215,18,236,10,177,98,177,120,219,104,52,162,170,170,138,236,183,35,13,177,65,79,27,118,135,88,150,101,145,150,150,134,158,158,158,168,64,236,
190,58,98,185,19,218,54,155,13,221,221,221,17,135,216,248,248,248,253,65,44,69,81,72,79,79,71,87,87,87,196,18,177,28,196,242,32,237,227,7,66,108,76,76,12,242,243,243,209,212,212,20,118,136,229,236,147,
255,160,104,95,137,88,138,162,96,50,153,80,85,85,133,150,150,150,176,65,172,195,225,192,137,19,39,160,211,233,248,135,170,255,115,223,16,171,86,171,97,177,88,208,213,213,69,210,176,71,73,196,218,237,118,
24,12,6,193,87,10,77,211,31,74,36,146,139,251,134,88,154,166,223,55,26,141,40,41,41,65,85,85,21,170,171,171,15,13,177,102,179,153,127,232,158,219,105,190,127,40,136,85,42,149,31,234,116,58,104,181,90,
112,21,24,54,155,13,37,37,37,40,45,45,133,219,237,222,115,65,92,61,202,46,105,216,163,65,44,77,211,23,21,10,197,127,42,149,74,40,20,10,66,135,18,137,4,106,181,26,41,41,41,56,126,252,56,92,46,23,233,75,
40,47,47,71,113,113,49,87,160,22,57,136,221,185,47,251,152,187,243,96,24,134,139,180,33,38,38,6,114,185,156,60,104,202,201,201,9,233,217,136,42,196,114,41,88,78,134,229,114,57,151,36,227,62,59,15,31,98,
105,154,254,156,101,217,123,12,195,252,138,97,152,55,190,133,216,111,124,53,1,203,178,31,26,141,198,47,118,98,222,159,63,52,136,165,40,234,75,150,101,97,54,155,209,221,221,77,170,87,11,11,11,131,103,135,
70,7,98,25,134,129,94,175,71,99,99,35,22,22,22,176,182,182,134,167,159,126,26,143,63,254,56,218,219,219,209,210,210,2,187,221,142,132,132,4,210,224,204,93,18,194,14,177,34,145,8,122,189,30,85,85,85,8,
4,2,152,159,159,199,244,244,52,158,120,226,9,178,69,153,157,157,37,77,167,205,205,205,164,91,95,167,211,65,169,84,66,34,145,132,39,17,27,27,27,139,236,236,108,156,62,125,26,83,83,83,100,49,62,159,15,189,
189,189,130,170,130,243,231,207,147,59,33,110,187,195,237,245,203,203,203,97,181,90,185,230,211,79,24,134,121,242,64,16,203,178,236,147,122,189,254,94,89,89,25,250,251,251,201,43,51,49,49,65,138,25,218,
219,219,209,213,213,133,205,205,77,178,40,254,96,10,62,196,126,181,23,107,64,99,99,35,42,43,43,97,52,26,193,178,236,189,3,37,98,147,147,147,63,25,24,24,192,242,242,50,102,103,103,49,58,58,26,18,30,108,
109,109,69,111,111,159,160,17,96,243,177,107,15,132,88,174,218,144,87,230,176,191,68,108,78,78,14,214,215,215,49,49,49,17,226,158,124,136,125,241,251,63,196,11,47,188,132,183,222,122,27,111,190,121,27,
155,155,215,31,8,177,181,181,181,176,90,173,252,103,130,15,134,88,153,76,182,93,83,83,131,149,149,149,125,65,236,211,207,60,143,167,159,126,14,215,183,158,196,226,226,242,3,33,182,174,174,14,229,229,229,
72,76,76,220,31,196,82,20,245,145,78,167,195,204,204,12,230,230,230,34,10,177,57,57,57,251,131,88,110,111,254,216,99,143,97,120,120,56,162,16,107,183,219,17,19,19,243,96,136,101,89,22,37,37,37,216,216,
216,136,248,176,46,46,250,198,85,227,223,23,98,47,92,184,32,208,225,72,65,108,85,85,21,236,118,59,212,106,53,247,54,126,185,43,196,166,165,165,225,153,103,158,33,149,172,145,134,88,183,219,13,139,197,
194,233,240,238,213,4,103,206,156,33,3,187,162,5,177,92,213,225,206,103,75,88,77,160,211,233,240,221,239,126,23,195,195,195,81,77,196,150,151,151,35,59,59,155,123,122,250,33,31,98,63,239,232,232,192,218,
218,154,160,239,60,90,16,203,75,156,253,95,2,177,106,181,26,231,207,159,135,223,239,23,244,254,68,3,98,119,169,100,253,10,98,141,70,227,255,237,232,232,192,192,192,64,212,33,214,237,118,243,187,24,9,89,
151,176,44,251,97,90,90,26,185,54,69,51,17,27,52,214,76,88,77,96,48,24,80,86,86,38,104,1,136,70,53,65,208,200,23,97,53,129,68,34,129,201,100,66,75,75,11,17,226,72,67,108,80,105,109,40,196,82,20,245,165,
78,167,67,81,81,17,121,181,34,13,177,233,233,233,15,134,88,134,97,144,158,158,142,150,150,22,114,176,43,146,195,186,130,12,125,111,136,213,233,116,112,185,92,232,233,233,137,120,71,236,190,134,117,81,
20,245,145,72,36,34,16,43,52,207,240,66,108,16,87,127,244,64,136,77,77,77,69,77,77,13,218,218,218,34,2,177,14,135,131,223,6,176,189,47,136,85,42,149,200,205,205,69,119,119,119,88,19,177,220,130,114,114,
114,8,23,29,40,17,107,50,153,80,89,89,41,88,208,81,33,246,216,177,99,208,235,245,124,140,253,228,64,137,216,248,248,248,123,137,137,137,72,77,77,197,177,99,199,96,179,217,224,241,120,14,4,177,101,101,
101,176,90,173,200,200,200,128,70,163,17,12,96,162,105,250,30,77,211,79,30,8,98,197,98,241,147,177,177,177,159,168,213,106,168,84,42,40,20,10,196,197,197,193,100,50,161,168,168,136,84,128,237,182,160,
178,178,50,216,108,54,152,76,38,196,196,196,8,206,191,243,102,130,28,45,17,203,48,204,182,76,38,35,206,169,80,40,160,209,104,80,80,80,64,106,192,184,87,167,184,184,24,153,153,153,220,0,230,224,209,84,
95,134,189,35,150,83,23,137,68,2,133,66,1,149,74,5,165,82,137,172,172,44,216,237,118,50,239,131,223,16,240,80,18,177,50,153,12,74,165,146,36,98,131,94,157,47,31,38,196,126,190,51,202,236,255,238,108,105,
191,173,38,248,187,234,136,253,92,169,84,34,57,57,249,11,134,97,62,124,104,16,203,125,208,117,58,29,154,155,155,49,55,55,135,190,190,62,20,22,22,66,171,213,146,187,222,168,64,44,119,57,72,79,79,71,111,
111,47,54,54,54,176,180,180,132,167,158,122,138,28,23,46,47,47,39,199,80,120,39,179,195,11,177,50,153,108,59,62,62,30,38,147,9,78,167,19,195,195,195,88,91,91,195,226,226,34,134,135,135,17,8,4,200,54,229,
241,199,31,71,75,75,11,60,30,15,74,75,75,5,23,210,35,119,196,178,44,251,100,82,82,210,39,69,69,69,104,104,104,192,192,192,0,230,231,231,177,184,184,136,217,217,89,193,8,152,165,165,37,193,188,172,142,
142,14,98,157,30,143,7,78,167,19,121,121,121,72,74,74,226,186,94,14,215,17,155,156,156,124,111,96,96,0,231,207,159,71,32,16,192,228,228,36,6,7,7,5,68,196,221,77,95,188,120,145,44,234,209,71,31,37,55,30,
124,90,172,169,169,33,55,11,59,66,124,111,71,136,247,223,17,91,84,84,116,95,247,108,109,109,69,79,143,15,51,51,1,220,184,241,20,15,99,31,71,123,123,231,174,29,177,220,246,199,229,114,113,207,147,63,217,
55,196,198,197,197,97,104,104,8,231,207,159,127,32,196,126,5,32,207,225,205,55,111,227,246,237,219,88,95,223,188,47,196,114,111,105,106,106,234,254,33,150,162,168,47,45,22,11,86,86,86,48,62,62,254,96,
136,125,250,31,240,216,99,79,224,250,214,147,184,186,241,24,166,166,231,30,8,177,53,53,53,56,118,236,24,247,29,249,224,97,93,34,145,8,205,205,205,88,89,89,17,140,162,10,39,196,214,212,212,160,168,168,
136,79,139,247,135,216,148,148,20,44,46,46,98,122,122,58,162,16,235,118,187,131,15,140,237,14,177,12,195,192,235,245,226,145,71,30,17,140,162,138,84,71,44,215,187,120,95,136,53,24,12,228,18,192,61,127,
137,100,34,214,102,179,9,246,94,33,16,203,178,44,78,159,62,141,149,149,21,18,136,143,6,196,242,198,163,135,66,108,122,122,58,86,86,86,224,247,251,209,217,217,25,53,136,61,113,226,196,238,29,177,44,203,
126,216,215,215,135,229,229,101,18,209,141,22,196,58,157,78,126,85,221,215,213,4,38,147,233,255,46,44,44,96,114,114,146,188,109,209,132,88,30,9,125,157,136,85,169,84,168,175,175,199,208,208,208,67,129,
216,32,129,33,213,4,159,91,44,22,65,105,109,180,32,150,219,123,5,147,117,9,69,81,239,107,181,90,116,117,117,161,171,171,43,170,16,91,82,82,194,23,152,15,5,16,203,133,228,199,199,199,163,6,177,101,101,
101,193,179,214,132,16,203,245,44,246,245,245,69,13,98,237,118,187,128,24,119,133,88,133,66,129,252,252,124,244,247,247,71,60,17,91,86,86,22,172,121,187,67,172,88,44,134,197,98,193,228,228,164,96,147,
22,9,136,181,217,108,193,53,136,123,67,108,74,74,10,186,186,186,34,94,77,144,149,149,181,127,136,213,106,181,112,58,157,232,237,237,13,218,49,134,15,98,185,39,162,251,134,88,169,84,186,109,177,88,48,56,
56,24,178,141,13,23,196,6,125,223,237,175,35,214,100,50,193,235,245,162,190,190,30,181,181,181,97,133,216,210,210,82,126,121,195,254,33,54,62,62,254,19,157,78,7,139,197,2,171,213,10,183,219,125,100,136,
229,203,48,47,143,176,255,142,216,29,239,188,199,213,23,234,116,58,228,230,230,194,225,112,16,140,221,111,34,182,180,180,20,39,79,158,132,201,100,130,86,171,229,39,99,239,81,71,233,136,85,169,84,224,64,
214,98,177,160,180,180,148,44,102,175,5,21,22,22,34,37,37,5,42,149,138,148,143,238,60,39,6,77,211,159,236,200,240,209,58,98,105,154,70,76,76,12,226,226,226,16,27,27,139,156,156,28,148,149,149,9,22,196,
61,115,209,106,181,228,78,133,97,24,136,68,34,110,65,219,17,129,88,145,72,132,152,152,24,168,213,106,104,181,90,228,229,229,161,184,184,24,57,57,57,80,171,213,216,173,166,53,106,16,203,48,12,84,42,21,
98,99,99,67,172,60,56,40,24,117,136,101,24,230,67,153,76,246,127,119,94,161,207,191,237,136,253,187,29,214,197,48,204,231,113,113,113,247,20,10,197,175,118,62,127,209,135,88,145,72,4,133,66,1,173,86,11,
155,205,134,190,190,62,116,119,119,147,100,44,55,172,43,98,16,75,211,244,199,220,215,131,66,161,128,217,108,70,113,113,49,90,90,90,112,246,236,89,92,188,120,17,87,174,92,193,214,214,22,252,126,63,236,
118,59,244,122,125,248,19,177,59,231,220,255,211,98,177,108,159,58,117,10,149,149,149,24,26,26,194,204,204,12,252,126,63,166,167,167,49,51,51,131,145,145,17,76,76,76,224,149,87,94,249,106,140,208,235,
175,163,173,173,141,228,88,120,155,185,163,15,235,138,143,143,255,176,181,181,21,103,207,158,37,221,176,126,191,159,188,77,252,58,196,233,233,105,188,251,238,187,184,123,247,46,94,122,233,37,12,14,14,
146,179,236,220,214,151,247,68,244,55,135,174,38,56,121,242,36,30,125,244,81,76,77,77,133,28,43,231,67,108,75,75,11,206,156,57,67,54,117,207,60,243,12,124,62,31,154,154,154,80,95,95,15,175,215,139,202,
202,74,20,23,23,35,53,53,149,171,99,125,159,58,232,176,174,196,196,68,76,76,76,144,209,28,247,131,216,51,115,243,120,236,241,235,60,182,190,142,246,246,118,1,196,114,80,86,90,90,138,130,130,2,174,14,227,
195,157,150,137,7,67,44,69,81,112,185,92,164,69,226,65,16,123,117,227,26,110,220,184,137,219,183,111,227,206,157,59,88,91,187,138,238,61,58,98,185,145,160,118,187,157,251,226,254,207,125,65,108,108,108,
44,174,94,189,138,185,185,185,251,118,196,114,16,187,241,232,117,92,186,188,129,199,30,223,194,149,245,141,125,65,108,89,89,25,52,26,13,247,125,121,127,136,229,218,42,183,182,182,4,109,18,225,134,88,175,
215,139,164,164,36,114,148,238,190,16,171,213,106,177,181,181,133,64,32,16,81,136,173,174,174,38,83,121,238,219,17,43,147,201,48,50,50,130,245,245,117,193,219,22,41,136,13,34,160,80,136,101,89,118,219,
106,181,98,107,107,11,220,136,151,72,67,172,221,110,23,236,189,4,16,75,211,244,27,169,169,169,88,89,89,17,76,16,139,52,196,150,149,149,241,43,164,133,16,171,209,104,126,117,250,244,105,92,185,114,133,
36,244,163,1,177,110,183,155,244,195,134,64,172,78,167,187,215,221,221,141,153,153,25,193,149,58,26,16,123,252,248,241,224,131,136,95,65,44,195,48,159,231,229,229,97,118,118,54,234,137,216,226,226,98,
126,199,139,176,35,150,51,79,238,64,78,180,32,214,229,114,5,199,42,191,238,136,213,106,181,59,206,233,139,122,34,150,55,69,236,115,1,196,198,196,196,224,196,137,19,24,29,29,141,42,196,150,151,151,35,51,
51,147,127,43,255,53,196,138,68,34,88,44,22,12,15,15,147,246,211,104,36,98,139,139,139,201,161,50,154,166,127,21,2,177,105,105,105,164,198,62,26,16,27,52,23,11,12,195,188,17,2,177,49,49,49,24,28,28,36,
27,180,40,87,19,108,239,10,177,20,69,145,48,68,164,135,117,57,28,142,224,179,128,123,119,196,102,101,101,97,102,102,70,112,134,42,18,16,203,251,112,223,31,98,105,154,254,216,108,54,195,239,247,147,253,
117,36,32,182,168,168,72,208,126,250,64,136,53,26,141,100,72,87,36,32,150,27,214,117,32,136,85,169,84,255,153,156,156,140,244,244,116,216,237,118,50,19,43,92,16,123,234,212,41,193,241,242,125,65,172,72,
36,186,168,80,40,62,140,143,143,135,90,173,134,217,108,134,219,237,70,125,125,125,88,32,150,183,5,62,212,176,174,247,185,60,103,76,76,12,178,178,178,224,241,120,200,254,250,32,195,186,56,208,15,170,29,
127,255,208,16,203,77,214,212,104,52,80,40,20,40,40,40,32,183,73,187,45,136,123,58,101,181,90,145,151,151,7,147,201,68,254,143,241,222,182,15,169,112,12,235,226,50,157,26,141,6,38,147,9,101,101,101,228,
206,151,235,134,61,126,252,56,244,122,61,98,99,99,73,230,147,83,97,30,200,254,167,66,161,184,24,86,136,21,139,197,136,143,143,135,70,163,65,110,110,46,242,242,242,96,52,26,161,80,40,136,2,243,121,154,
183,129,251,56,162,16,203,48,204,182,90,173,70,108,108,44,49,114,145,72,4,154,166,201,95,31,10,196,138,68,162,55,100,50,217,175,88,150,189,71,211,244,231,223,14,235,250,187,135,88,238,237,19,139,197,255,
38,151,203,255,229,161,64,44,247,129,150,201,100,164,70,218,225,112,192,235,245,162,164,164,4,44,203,70,14,98,229,114,249,29,173,86,251,153,82,169,132,94,175,71,102,102,38,50,51,51,145,159,159,143,186,
186,58,116,116,116,96,120,120,24,75,75,75,152,155,155,195,234,234,42,230,230,230,224,114,185,160,213,106,185,231,53,225,77,196,150,148,148,108,143,143,143,99,126,126,30,129,64,0,195,195,195,152,154,154,
66,32,16,128,223,239,199,200,200,8,6,7,7,209,223,223,143,171,87,175,226,238,221,187,248,201,79,126,130,190,190,62,193,83,173,176,64,44,77,211,191,77,78,78,198,230,230,38,22,22,22,48,48,48,128,142,142,
14,116,118,118,18,149,225,223,188,14,13,13,225,133,23,94,32,251,168,201,201,73,84,86,86,226,212,169,83,193,124,125,56,136,165,40,234,191,180,90,45,250,250,250,176,182,182,246,64,136,109,105,105,129,223,
239,199,91,111,189,133,187,119,239,226,71,63,250,17,70,70,70,80,95,95,79,70,3,29,59,118,140,159,216,248,175,3,67,44,69,81,168,175,175,199,234,234,42,38,38,38,30,8,177,51,179,115,184,116,249,10,121,149,
158,126,250,105,116,119,119,147,109,53,119,106,164,164,164,4,41,41,41,92,101,230,111,247,157,136,165,40,10,73,73,73,184,121,243,230,174,195,186,118,131,216,149,149,53,92,191,126,3,239,189,247,30,238,222,
189,139,203,107,235,232,236,236,222,117,88,151,205,102,67,86,86,22,247,170,237,47,17,171,80,40,48,58,58,138,43,87,174,236,27,98,47,174,172,97,121,121,21,79,220,120,10,155,155,215,48,48,56,182,103,71,44,
55,217,62,47,47,143,219,210,220,185,47,196,42,20,10,120,60,30,108,110,110,98,108,108,44,98,16,91,93,93,13,155,205,6,177,88,12,145,72,244,217,158,16,203,178,44,220,110,55,86,86,86,4,89,207,72,65,172,203,
229,122,112,34,182,160,160,0,87,175,94,197,153,51,103,4,179,177,34,5,177,46,151,75,176,27,13,129,88,149,74,181,189,181,181,133,133,133,5,193,231,40,146,16,91,94,94,206,55,79,33,196,170,213,234,127,41,
43,43,195,214,214,150,96,65,145,134,216,202,202,74,254,105,109,33,196,178,44,251,111,39,79,158,20,200,112,180,18,177,123,29,142,46,161,40,234,253,236,236,108,92,184,112,33,234,16,187,215,132,123,210,17,
27,8,4,208,221,221,29,85,136,13,106,0,16,66,172,86,171,69,75,75,11,250,251,251,163,10,177,193,157,158,2,136,85,40,20,112,187,221,152,152,152,136,42,196,150,150,150,242,159,49,11,135,117,81,20,133,172,
172,44,65,35,124,52,32,214,229,114,241,19,177,194,106,2,154,166,145,153,153,137,165,165,37,178,168,104,64,44,191,188,150,63,42,79,208,101,61,55,55,71,6,118,69,26,98,57,102,228,158,194,139,197,226,127,
9,129,216,140,140,12,204,205,205,161,189,189,61,106,195,186,82,82,82,238,15,177,122,189,30,171,171,171,130,39,229,145,132,88,62,87,239,9,177,177,177,177,240,251,253,100,81,145,130,88,167,211,25,108,85,
123,67,172,86,171,253,44,47,47,15,173,173,173,130,118,173,112,67,108,94,94,94,200,25,229,61,33,86,42,149,222,73,78,78,38,41,159,234,234,234,136,64,108,80,68,119,127,137,216,216,216,88,112,83,158,248,251,
235,112,12,235,10,234,221,216,127,34,150,166,233,223,198,196,196,64,171,213,146,30,98,254,54,246,176,16,155,155,155,27,92,60,122,96,136,253,47,46,43,21,31,31,143,236,236,108,84,85,85,29,26,98,249,79,171,
142,52,172,139,15,177,92,150,152,191,168,221,22,196,45,162,164,164,4,54,155,13,14,135,3,5,5,5,193,84,253,219,176,64,172,84,42,133,78,167,67,106,106,42,121,251,42,43,43,137,143,115,19,200,117,58,29,212,
106,53,98,98,98,16,19,19,67,250,23,249,245,22,97,77,196,50,12,3,141,70,3,153,76,134,99,199,142,193,96,48,32,46,46,142,252,131,197,98,49,249,217,57,128,207,87,225,207,148,74,229,157,136,65,44,203,178,219,
220,171,32,22,139,5,36,29,220,158,27,85,136,149,201,100,255,178,243,173,30,34,121,223,66,236,223,53,196,70,173,154,128,97,152,73,153,76,118,151,123,226,192,109,204,248,191,117,74,165,18,114,185,28,58,
157,14,105,105,105,72,78,78,222,86,42,149,159,209,52,125,59,34,137,88,189,94,255,231,242,242,114,180,183,183,147,121,49,92,105,104,93,93,29,57,210,57,57,57,137,225,225,97,204,205,205,33,16,8,224,228,201,
147,220,141,65,248,59,98,231,231,231,113,241,226,69,210,126,58,60,60,140,129,129,1,12,12,12,8,32,214,231,243,97,124,124,28,111,190,249,38,158,127,254,121,248,124,62,126,240,244,203,176,116,196,42,20,10,
148,151,151,227,169,167,158,194,240,240,48,225,161,189,220,179,163,163,3,235,235,235,100,47,181,182,182,70,198,183,240,166,240,28,30,98,185,188,231,197,139,23,67,250,61,119,91,208,233,211,167,49,52,52,
132,23,95,124,145,44,42,16,8,160,174,174,14,118,187,29,70,163,145,127,68,229,112,16,107,48,24,176,181,181,133,185,185,185,93,59,98,131,33,118,116,116,2,139,231,190,46,116,224,50,196,28,196,218,237,118,
100,102,102,242,133,120,255,29,177,92,199,112,32,16,192,165,75,151,246,5,177,23,87,214,48,191,112,1,215,174,61,193,171,190,184,134,246,246,118,178,81,228,160,172,160,160,0,241,241,241,220,222,106,127,
16,107,52,26,49,61,61,141,213,213,85,12,13,13,237,27,98,231,206,158,199,210,210,10,158,251,238,243,248,199,127,124,1,99,227,83,123,14,235,202,205,205,229,182,51,15,238,136,213,104,52,88,89,89,193,202,
202,10,134,134,134,200,231,232,32,16,59,48,56,138,193,161,241,251,118,196,186,221,110,82,131,207,178,236,159,37,18,201,100,8,196,50,12,115,59,57,57,25,179,179,179,88,95,95,23,180,0,68,10,98,221,110,55,
153,216,202,48,204,221,16,136,53,24,12,159,45,46,46,134,188,101,145,132,88,175,215,203,191,255,11,133,216,180,180,180,237,39,158,120,34,100,65,145,132,216,170,170,42,254,162,66,135,117,153,76,38,44,46,
46,70,21,98,171,170,170,130,67,205,194,106,2,131,193,128,249,249,249,168,38,98,61,30,79,240,177,223,127,13,25,214,53,60,60,140,161,161,161,168,66,108,208,205,169,16,98,227,226,226,208,210,210,130,209,
209,209,168,66,44,47,227,25,10,177,18,137,4,53,53,53,152,153,153,137,42,196,6,159,71,22,64,44,203,178,112,56,28,164,158,32,90,16,27,148,96,20,66,172,88,44,70,121,121,121,212,33,214,106,181,238,221,17,
75,81,20,220,110,55,2,129,64,84,59,98,139,139,139,247,174,38,144,74,165,40,41,41,17,44,42,90,16,203,221,210,239,58,172,203,235,245,146,69,69,3,98,185,109,204,125,33,214,102,179,97,122,122,90,80,75,16,
201,97,93,86,171,149,63,185,231,179,16,136,149,74,165,119,51,50,50,208,220,220,140,150,150,150,136,39,98,109,54,155,96,148,144,76,38,187,29,2,177,34,145,104,82,163,209,252,89,171,213,34,39,39,7,94,175,
55,98,195,186,242,243,243,15,86,77,192,245,15,115,175,90,184,33,214,106,181,242,27,111,246,15,177,28,144,113,133,14,252,78,178,163,66,44,239,225,208,193,59,98,119,198,141,33,62,62,30,46,151,139,12,34,
57,44,196,150,148,148,4,103,134,15,15,177,113,113,113,72,72,72,32,166,126,208,97,93,123,244,12,31,13,98,105,154,134,70,163,129,90,173,134,211,233,68,77,77,205,3,23,228,114,185,200,130,156,78,103,72,27,
115,88,58,98,185,252,139,86,171,69,89,89,25,170,171,171,81,82,82,130,162,162,34,88,173,86,100,101,101,33,51,51,19,169,169,169,72,77,77,69,122,122,58,140,70,35,76,38,19,140,70,163,32,123,16,118,136,141,
139,139,3,23,58,85,40,20,4,90,37,18,9,129,215,224,31,94,195,219,159,35,146,136,101,24,230,182,82,169,252,76,38,147,109,115,11,225,21,52,8,170,9,118,146,177,220,176,174,187,70,163,113,242,161,12,235,250,
22,98,191,241,16,75,211,116,237,55,6,98,185,68,108,92,92,220,61,179,217,188,205,181,150,112,223,244,92,224,129,11,199,179,44,11,137,68,178,45,22,139,63,146,74,165,255,28,177,142,216,172,172,44,76,76,76,
96,114,114,146,156,217,106,109,109,69,91,91,27,188,94,47,26,26,26,224,241,120,200,241,22,238,175,39,78,156,0,195,48,159,134,29,98,13,6,3,214,215,215,113,238,220,57,12,14,14,18,251,236,238,238,70,103,103,
39,122,122,122,224,243,249,208,215,215,135,190,190,62,12,14,14,98,121,121,153,160,91,66,66,2,196,98,113,120,134,117,81,20,133,212,212,84,76,79,79,227,210,165,75,130,58,205,189,220,179,185,185,25,83,83,
83,184,125,251,54,201,124,30,63,126,156,191,101,57,26,196,234,245,122,44,47,47,99,121,121,57,68,100,246,90,80,240,16,175,107,215,174,161,166,166,6,217,217,217,80,40,20,220,5,246,224,16,43,22,139,145,150,
150,134,233,233,105,92,190,124,25,253,253,253,2,29,222,11,98,125,190,126,76,77,77,11,144,109,122,122,26,117,117,117,228,200,83,80,153,195,254,32,150,166,105,88,173,86,44,45,45,97,105,105,41,228,21,218,
19,98,231,151,48,61,61,135,141,71,31,23,196,117,125,62,31,193,178,162,162,162,96,186,126,48,196,170,84,170,109,187,221,142,245,245,117,204,207,207,11,154,0,246,3,177,19,147,179,88,89,185,132,87,94,249,
209,87,147,234,2,103,5,16,203,5,232,77,38,19,183,176,251,67,172,66,161,248,148,251,173,153,155,155,19,100,61,15,154,136,157,59,123,14,19,147,211,104,111,239,9,129,88,143,199,3,171,213,138,180,180,52,242,
220,102,215,68,172,82,169,252,103,175,215,139,27,55,110,96,124,124,60,36,51,28,110,136,173,172,172,68,73,73,9,25,33,187,107,34,150,101,217,143,218,219,219,49,63,63,31,242,184,35,82,16,235,241,120,144,
151,151,7,153,76,6,134,97,238,197,198,198,214,10,32,86,42,149,110,55,53,53,225,204,153,51,81,133,216,210,210,82,240,74,108,132,16,43,147,201,80,95,95,47,104,148,136,6,196,58,28,142,189,59,98,105,154,134,
215,235,37,73,179,104,65,108,97,97,225,222,16,43,147,201,80,85,85,133,51,103,206,68,21,98,79,157,58,181,55,196,114,45,37,115,115,115,81,133,216,99,199,142,237,13,177,49,49,49,104,110,110,142,58,196,6,
159,71,22,64,44,69,81,24,30,30,134,223,239,143,42,196,6,5,154,255,26,210,17,59,58,58,138,145,145,145,168,66,108,208,237,188,16,98,19,19,19,113,225,194,5,12,14,14,70,21,98,247,122,12,98,162,40,234,181,
132,132,4,50,49,51,154,16,203,127,226,16,2,177,38,147,9,125,125,125,232,234,234,138,42,196,242,22,37,132,88,154,166,107,213,106,245,189,180,180,52,50,0,46,26,16,235,114,185,248,31,244,143,118,77,196,234,
116,58,36,38,38,146,3,247,145,132,216,210,210,82,28,59,118,140,4,193,100,50,217,63,239,217,17,171,209,104,144,144,144,64,198,7,71,2,98,185,220,48,47,94,249,233,3,33,86,171,213,34,62,62,30,110,183,27,13,
13,13,97,133,88,155,205,70,242,231,7,26,214,69,81,20,212,106,53,18,18,18,80,91,91,139,198,198,198,176,64,172,213,106,13,166,234,131,67,108,66,66,2,204,102,51,154,155,155,143,156,136,45,44,44,12,89,208,
161,32,150,155,167,157,149,149,69,186,200,14,3,177,187,189,66,71,134,88,131,193,0,187,221,142,230,230,230,93,23,228,118,187,247,92,144,221,110,71,92,92,92,248,33,86,46,151,111,235,245,122,20,20,20,32,
55,55,23,199,142,29,35,199,126,211,211,211,97,177,88,96,177,88,144,151,151,135,188,188,60,20,22,22,194,225,112,224,196,137,19,130,233,3,97,135,88,150,101,63,141,139,139,131,84,42,133,84,42,37,208,202,
253,107,153,76,22,130,176,65,5,218,145,233,136,149,201,100,255,76,211,244,71,34,145,104,155,187,232,113,127,229,174,57,28,208,114,81,93,154,166,239,61,172,97,93,123,66,108,126,126,126,237,183,16,251,141,
74,196,138,68,162,85,153,76,118,231,27,3,177,20,69,125,152,147,147,179,205,61,24,50,153,76,100,138,19,63,12,207,143,126,71,116,88,151,66,161,192,201,147,39,177,186,186,138,179,103,207,146,176,105,91,91,
27,186,187,187,73,149,74,126,126,62,185,150,165,164,164,64,167,211,65,161,80,252,137,101,217,159,133,21,98,41,138,130,199,227,193,165,75,151,48,63,63,47,104,1,224,126,90,91,91,209,221,221,141,222,222,
94,248,124,62,12,13,13,193,231,243,97,114,114,18,229,229,229,80,169,84,219,59,21,97,71,135,88,141,70,131,214,214,86,172,173,173,133,156,117,223,203,61,155,155,155,209,221,221,141,181,181,53,188,240,194,
11,152,159,159,71,78,78,14,87,241,116,120,136,21,139,197,208,233,116,24,24,24,192,198,198,6,38,39,39,119,29,214,181,219,130,154,155,155,49,59,59,75,202,108,223,123,239,61,56,28,142,163,65,172,68,34,65,
105,105,41,22,23,23,177,188,188,44,104,174,124,16,196,182,182,182,99,96,96,0,183,110,221,18,8,113,67,67,3,50,50,50,130,51,196,15,134,88,145,72,244,43,179,217,140,206,206,78,92,190,124,153,140,192,227,
67,218,253,32,214,63,117,6,35,163,126,92,88,94,33,11,122,245,213,87,49,54,54,134,218,218,90,56,28,14,232,116,58,174,254,226,193,16,171,84,42,127,150,149,149,181,189,177,177,129,11,23,46,96,98,98,66,240,
118,237,23,98,7,7,199,240,248,181,45,178,168,213,213,85,180,180,180,16,197,203,207,207,231,199,116,239,15,177,137,137,137,127,186,114,229,10,25,27,196,127,117,14,10,177,211,211,115,216,218,186,129,107,
215,182,208,219,55,24,2,177,121,121,121,252,88,201,222,195,186,140,70,35,54,55,55,49,56,56,24,50,206,236,48,16,219,210,218,245,53,51,6,65,44,119,240,112,167,136,123,239,106,2,179,217,140,141,141,141,144,
84,117,164,32,214,225,112,192,108,54,115,251,174,221,135,117,101,102,102,134,44,42,146,16,235,241,120,96,183,219,145,144,144,0,134,97,160,80,40,86,5,16,75,81,20,178,179,179,113,249,242,229,168,66,108,
69,69,5,185,15,220,233,226,16,66,236,137,19,39,112,249,242,229,168,14,235,242,120,60,252,125,188,16,98,69,34,17,202,202,202,176,180,180,20,245,97,93,188,135,71,66,136,101,89,22,3,3,3,81,135,216,146,146,
146,189,33,86,165,82,97,105,105,137,180,0,68,11,98,131,102,207,8,33,214,100,50,97,117,117,53,234,29,177,65,188,40,132,216,140,140,12,108,110,110,70,125,88,87,208,89,82,33,196,90,173,86,172,172,172,8,182,
31,209,24,214,21,100,12,95,67,172,76,38,187,147,149,149,133,169,169,41,82,75,16,173,142,216,61,135,117,137,68,162,85,131,193,128,172,172,44,65,87,66,52,170,9,120,101,163,187,15,235,74,78,78,198,169,83,
167,112,250,244,233,136,15,235,226,220,243,129,213,4,50,153,12,41,41,41,176,217,108,232,232,232,136,40,196,114,144,182,175,97,93,20,69,193,104,52,162,178,178,50,98,137,88,238,88,57,175,253,230,79,15,132,
88,149,74,133,228,228,100,52,52,52,8,22,22,14,136,117,58,157,56,118,236,152,160,35,72,169,84,254,108,95,16,171,209,104,144,147,147,67,182,177,225,128,88,155,205,6,163,209,40,56,68,45,22,139,127,181,111,
136,101,24,6,70,163,17,78,167,19,45,45,45,71,30,214,117,242,228,73,65,170,250,208,16,43,22,139,145,154,154,138,174,174,174,144,5,237,23,98,157,78,39,178,178,178,248,59,129,163,67,44,119,124,183,178,178,
18,30,143,7,46,151,139,244,229,123,60,30,84,84,84,236,185,32,110,187,27,132,30,71,135,88,154,166,127,165,213,106,161,80,40,16,19,19,3,149,74,133,248,248,120,104,181,90,24,12,6,152,76,38,228,228,228,144,
132,44,167,49,14,135,3,6,131,33,68,97,194,6,177,50,153,236,103,114,185,252,79,82,169,20,50,153,140,252,200,229,114,242,35,145,72,16,19,19,131,248,248,120,24,141,70,65,196,155,155,121,28,241,97,93,252,
127,32,77,211,16,139,197,100,161,50,153,140,127,7,252,225,55,6,98,37,18,201,29,189,94,191,250,109,34,246,27,91,77,160,84,42,127,174,82,169,62,165,40,234,175,59,31,230,135,7,177,12,195,108,159,58,117,10,
157,157,157,104,105,105,65,89,89,25,41,9,225,119,199,70,5,98,41,138,66,124,124,60,156,78,39,30,123,236,49,4,2,1,244,247,247,19,243,172,175,175,135,211,233,68,102,102,38,244,122,61,9,210,239,252,38,126,
68,211,244,235,97,133,88,174,249,59,16,8,96,121,121,25,179,179,179,2,102,60,125,250,52,185,11,106,107,107,35,117,81,205,205,205,40,41,41,129,213,106,69,124,124,252,118,76,76,204,7,71,134,88,46,156,236,
118,187,177,184,184,136,149,149,21,65,76,247,65,238,217,210,210,2,159,207,135,137,137,9,244,245,245,33,41,41,9,52,77,255,238,208,16,43,147,201,96,179,217,48,54,54,70,234,52,249,49,221,7,45,168,169,169,
9,189,189,189,120,252,241,175,114,159,215,175,95,199,201,147,39,57,193,59,24,196,178,44,251,59,155,205,134,254,254,126,60,242,200,35,152,159,159,199,208,208,144,160,192,225,65,16,219,212,212,132,182,182,
54,92,184,112,225,235,91,173,59,119,80,89,89,121,112,136,53,26,141,31,116,119,119,227,234,213,171,152,159,159,199,232,232,168,192,61,247,3,177,195,195,19,232,234,238,67,224,204,28,94,123,237,235,27,137,
43,87,174,160,161,161,129,100,18,118,22,118,127,136,165,105,250,117,163,209,184,253,157,239,124,135,204,111,15,102,198,253,66,108,111,223,48,158,121,230,107,178,190,117,235,22,250,251,251,81,91,91,11,
155,205,198,247,244,7,87,19,20,21,21,97,99,99,35,100,65,7,133,216,30,223,32,214,214,174,226,245,215,223,192,171,175,190,138,201,73,63,217,86,87,84,84,144,178,16,174,82,122,79,136,149,74,165,104,106,106,
194,133,11,23,4,159,159,195,66,108,103,87,31,2,129,121,12,14,142,226,116,75,7,129,88,238,132,146,78,167,35,143,121,247,132,88,133,66,1,191,223,31,18,64,141,4,196,114,108,173,86,171,185,139,235,238,16,
171,209,104,112,238,220,185,168,13,235,226,66,93,59,191,141,161,213,4,20,69,33,61,61,29,143,62,250,40,6,6,6,162,2,177,92,199,117,114,114,50,247,219,40,76,196,50,12,131,220,220,92,92,185,114,37,234,16,
203,21,57,136,68,162,159,11,32,150,162,40,244,244,244,96,105,105,137,188,74,209,128,216,202,202,74,254,161,251,79,5,16,107,48,24,176,178,178,130,137,137,137,168,66,172,219,237,230,147,245,215,16,75,211,
52,184,71,32,156,121,70,11,98,139,139,139,131,11,253,190,134,216,230,230,102,172,172,172,144,87,41,90,16,27,164,195,95,67,108,124,124,252,167,205,205,205,152,158,158,22,124,219,71,26,98,203,203,203,5,
147,201,5,16,171,84,42,127,110,177,88,224,247,251,163,10,177,78,167,83,48,191,47,36,17,107,52,26,97,181,90,201,244,176,104,64,172,205,102,19,24,67,72,34,150,97,152,237,212,212,84,212,213,213,161,181,181,
53,42,16,27,20,144,223,29,98,213,106,53,116,58,29,154,155,155,5,169,216,72,65,108,80,83,201,238,16,203,61,163,57,117,234,20,217,248,71,10,98,203,202,202,4,31,242,251,66,172,68,34,129,201,100,66,109,109,
45,161,197,72,64,108,208,144,202,7,87,19,176,44,139,172,172,44,116,117,117,133,189,154,128,143,177,92,34,86,165,82,189,254,64,136,165,105,26,105,105,105,240,122,189,97,133,88,167,211,137,220,220,92,36,
38,38,130,151,233,251,96,223,16,203,178,44,204,102,51,122,123,123,201,130,14,11,177,78,167,19,39,79,158,68,82,82,82,240,87,202,239,14,12,177,169,169,169,40,43,43,35,161,211,195,64,108,81,81,17,82,83,83,
131,7,190,29,30,98,197,98,241,239,244,122,61,226,227,227,97,50,153,144,149,149,5,155,205,134,162,162,34,148,151,151,19,140,221,109,65,101,101,101,56,117,234,148,0,241,195,6,177,113,113,113,31,40,20,138,
237,152,152,24,40,20,10,18,62,85,171,213,72,74,74,66,110,110,46,1,216,224,99,2,188,99,152,228,118,42,156,137,216,215,185,142,88,174,138,64,46,151,19,49,230,102,178,101,103,103,35,59,59,27,58,157,110,55,
166,142,14,196,242,103,24,113,139,228,58,20,184,95,247,111,4,196,74,36,146,79,69,34,209,207,191,173,38,216,13,98,69,34,145,231,27,213,17,171,86,171,63,81,171,213,95,236,60,160,126,120,16,203,13,74,53,
155,205,24,25,25,193,192,192,0,26,27,27,97,177,88,66,186,207,163,2,177,34,145,8,169,169,169,104,108,108,196,218,218,26,9,18,114,27,195,138,138,10,216,108,54,50,25,145,119,105,8,127,71,172,88,44,70,114,
114,50,92,46,23,57,205,61,62,62,78,238,166,185,109,116,83,83,19,57,107,83,85,85,133,147,39,79,194,104,52,114,201,254,109,138,162,126,121,100,136,149,201,100,176,88,44,240,122,189,88,88,88,192,133,11,23,
48,61,61,253,64,136,37,187,214,157,227,83,77,77,77,40,43,43,131,217,108,134,66,161,184,119,40,136,149,203,229,31,58,28,14,116,118,118,98,121,121,25,75,75,75,240,251,253,130,163,229,251,129,88,110,113,
62,159,15,243,243,243,24,30,30,134,94,175,7,77,211,31,30,8,98,83,83,83,239,77,79,79,99,125,125,29,103,207,158,13,57,86,190,95,136,229,22,52,52,52,132,239,124,231,59,132,25,173,86,43,183,159,218,31,196,
210,52,253,75,254,80,249,96,102,60,8,196,182,183,119,98,96,96,0,215,174,93,227,97,200,109,84,86,86,238,31,98,41,138,250,177,82,169,220,158,152,152,192,249,243,231,143,4,177,221,221,3,240,249,6,113,101,
125,67,112,3,177,180,180,132,154,154,26,104,181,218,253,67,172,74,165,218,117,190,218,97,134,117,117,117,247,225,229,151,95,33,127,207,165,75,151,208,209,209,129,154,154,26,28,59,118,140,223,45,180,55,
196,114,51,103,54,54,54,66,202,27,14,155,136,93,95,223,192,139,47,190,132,171,87,31,69,111,111,47,217,86,59,28,14,40,149,74,178,87,223,19,98,69,34,17,42,43,43,177,182,182,38,0,253,163,66,172,207,55,136,
246,14,95,8,196,102,101,101,221,127,88,23,119,216,240,236,217,179,130,38,128,72,66,108,73,73,9,52,26,13,73,91,239,10,177,41,41,41,184,122,245,42,6,7,7,163,6,177,217,217,217,252,137,174,66,136,165,40,10,
29,29,29,88,93,93,141,42,196,58,28,14,232,245,122,238,179,37,76,196,166,164,164,224,226,197,139,152,156,156,140,106,34,150,223,126,42,18,137,62,17,64,172,195,225,192,230,230,38,122,123,123,163,10,177,
21,21,21,252,118,184,47,104,154,246,80,20,69,85,104,52,26,156,57,115,70,16,209,141,22,196,186,221,238,224,70,221,10,138,97,24,143,86,171,253,98,112,112,16,67,67,67,130,47,215,104,64,172,221,110,223,125,
88,87,108,108,236,39,70,163,17,163,163,163,81,133,88,183,219,141,227,199,143,239,157,136,213,235,245,168,172,172,68,71,71,71,84,33,118,207,68,44,69,81,175,41,20,10,152,205,102,65,95,66,164,33,54,168,169,
36,20,98,185,30,79,187,221,142,174,174,174,168,64,108,208,24,170,221,33,150,123,28,210,221,221,29,113,136,221,229,173,219,27,98,117,58,29,138,139,139,209,211,211,19,209,68,172,221,110,223,223,176,46,78,
86,204,102,51,134,135,135,5,13,0,225,134,216,220,220,220,131,117,196,26,12,6,212,212,212,160,173,173,45,34,16,203,165,136,14,84,77,32,149,74,97,177,88,48,56,56,40,24,117,112,84,136,229,22,148,151,151,
71,246,81,98,177,248,151,251,134,88,147,201,68,46,13,220,110,241,40,137,216,210,210,82,228,228,228,32,33,33,129,239,159,247,14,4,177,82,169,244,67,157,78,7,179,217,140,252,252,124,82,177,202,45,106,63,
16,203,125,160,51,50,50,16,27,27,27,124,250,246,195,67,65,172,70,163,185,199,205,184,98,89,22,9,9,9,176,88,44,176,217,108,4,97,247,234,136,45,45,45,69,102,102,38,228,114,185,160,128,244,80,83,84,118,177,
132,95,238,20,59,66,169,84,66,165,82,65,161,80,64,165,82,33,55,55,23,118,187,157,196,118,185,175,142,227,199,143,35,54,54,150,188,42,188,197,132,191,35,150,251,7,72,165,82,200,229,114,168,84,42,40,149,
74,164,165,165,193,106,181,34,47,47,143,32,236,67,131,88,110,64,92,108,108,44,164,82,41,191,164,150,188,50,15,21,98,119,2,129,95,236,108,105,191,57,29,177,59,91,217,111,19,177,247,29,214,181,51,21,241,
139,135,10,177,59,167,56,160,215,235,73,211,105,99,99,35,78,157,58,5,133,66,193,221,199,69,7,98,185,38,240,156,156,28,12,14,14,98,110,110,14,231,206,157,195,192,192,0,58,58,190,178,2,110,200,32,215,222,
28,17,136,101,89,118,91,46,151,195,96,48,160,186,186,26,3,3,3,88,93,93,69,32,16,192,208,208,144,96,176,115,83,83,19,234,235,235,225,245,122,201,161,11,149,74,197,165,201,142,62,172,75,169,84,126,112,234,
212,41,120,60,30,204,206,206,98,118,118,22,75,75,75,152,154,154,34,222,112,63,136,229,190,196,235,235,235,113,242,228,73,164,164,164,112,183,83,135,171,38,48,24,12,247,2,129,0,113,79,191,223,143,190,190,
190,3,65,44,183,157,230,118,23,77,77,77,112,56,28,200,202,202,130,76,38,187,119,224,106,2,183,219,141,173,173,45,50,202,44,216,61,15,2,177,220,182,218,231,243,145,122,215,157,231,201,31,236,187,35,54,
62,62,158,140,50,59,42,196,114,11,155,152,152,192,11,47,188,128,187,119,239,226,217,103,159,229,207,175,221,223,176,174,204,204,76,220,188,121,83,48,23,235,48,16,219,222,222,131,150,214,46,248,253,83,
120,233,165,151,200,158,253,181,215,94,35,55,14,52,77,111,63,16,98,101,50,25,122,123,123,177,188,188,124,100,136,109,105,233,132,207,55,136,151,95,126,89,80,232,48,61,61,77,6,149,236,11,98,117,58,29,174,
93,187,70,230,34,31,181,35,118,108,220,143,151,95,254,225,87,184,246,242,203,240,251,253,104,108,108,132,219,237,22,28,24,219,19,98,185,48,124,240,208,183,163,66,236,242,242,35,184,113,227,41,76,78,206,
160,177,177,145,64,108,80,25,228,238,16,203,178,44,30,121,228,17,242,1,143,52,196,218,108,54,254,190,61,20,98,197,98,49,106,106,106,176,185,185,73,12,61,210,16,203,101,242,120,27,67,33,196,38,36,36,96,
105,105,9,51,51,51,81,237,136,205,201,201,225,239,84,133,16,219,220,220,140,171,87,175,162,175,175,47,170,16,235,112,56,248,208,241,117,53,129,94,175,255,98,116,116,20,51,51,51,130,175,142,104,64,108,
16,9,125,157,136,77,74,74,66,107,107,43,198,198,198,30,10,196,6,5,80,133,213,4,220,55,127,52,33,214,237,118,243,3,205,95,8,170,9,146,146,146,72,19,82,52,33,214,110,183,243,179,121,66,136,141,137,137,65,
122,122,58,198,199,199,163,6,177,187,196,42,133,16,203,29,31,231,190,239,162,1,177,118,187,93,16,135,219,21,98,227,226,226,96,179,217,48,48,48,16,113,136,45,43,43,11,254,154,217,29,98,41,138,66,70,70,
6,70,70,70,132,217,130,8,64,172,195,225,8,174,65,220,27,98,245,122,61,57,179,23,73,136,229,174,228,251,130,88,185,92,14,151,203,133,190,190,62,50,222,53,18,16,155,152,152,184,127,136,149,72,36,219,92,
54,157,143,176,225,132,216,160,153,125,251,27,214,149,150,150,134,238,238,110,52,53,53,69,4,98,249,93,46,251,238,136,85,40,20,31,232,116,58,228,228,228,192,225,112,160,161,161,1,213,213,213,71,130,88,
46,166,91,80,80,192,135,216,131,85,19,168,213,234,123,26,141,134,120,65,78,78,14,153,55,123,144,68,172,195,225,192,201,147,39,97,54,155,161,86,171,249,217,207,123,135,174,38,144,201,100,80,169,84,100,
224,77,90,90,26,156,78,39,89,204,94,16,107,181,90,97,52,26,5,42,188,243,68,29,20,69,125,16,150,97,93,12,195,16,231,84,40,20,200,202,202,66,113,113,49,121,56,237,118,187,225,112,56,200,156,99,110,66,15,
183,144,157,159,237,136,64,44,151,33,80,171,213,80,42,149,56,126,252,56,242,243,243,145,147,147,67,130,15,193,51,178,162,6,177,34,145,8,50,153,12,106,181,154,84,17,236,178,152,135,7,177,50,153,236,139,
29,234,249,118,88,215,223,39,196,238,64,255,223,148,74,229,39,82,169,244,231,15,13,98,185,242,99,174,148,134,203,53,88,173,86,136,197,98,136,68,162,237,136,67,44,77,211,144,201,100,144,74,165,48,155,205,
176,219,237,240,251,253,224,50,124,220,1,216,154,154,26,20,21,21,9,238,126,195,10,177,106,181,250,215,58,157,110,59,55,55,23,21,21,21,104,107,107,131,223,239,199,212,212,20,22,23,23,9,59,118,119,119,147,
91,179,198,198,70,210,173,152,155,155,43,152,116,120,100,136,213,104,52,191,159,154,154,194,165,75,151,48,61,61,141,185,185,57,248,253,126,12,15,15,195,231,243,161,187,187,251,190,16,91,87,87,135,202,
202,74,216,108,54,88,44,22,126,53,207,225,59,98,93,46,23,158,122,234,41,226,158,193,1,212,253,36,98,249,101,239,65,139,59,120,71,172,70,163,193,197,139,23,113,246,236,217,176,64,44,127,203,83,84,84,196,
157,109,255,253,190,33,150,162,40,56,157,78,220,184,113,35,164,112,244,48,16,203,109,171,155,155,155,209,215,215,135,115,231,206,161,167,167,135,171,100,253,245,190,32,54,46,46,14,75,75,75,33,147,159,
14,3,177,205,167,191,58,104,223,217,217,137,179,103,207,226,71,63,250,17,105,215,221,201,38,60,24,98,185,177,214,220,171,116,84,136,109,110,110,135,207,55,128,43,87,214,5,119,55,175,188,242,10,63,48,241,
96,136,93,90,90,34,3,150,194,49,172,139,95,247,123,247,238,93,60,247,220,115,24,30,30,22,220,178,239,9,177,12,195,160,191,191,31,235,235,235,232,239,239,15,27,196,46,45,93,196,235,175,191,65,222,182,254,
254,126,120,189,94,66,64,59,23,216,80,136,21,139,197,219,133,133,133,88,91,91,195,248,248,120,88,19,177,29,157,62,44,46,94,192,185,115,203,24,25,25,21,64,44,127,239,21,2,177,10,133,2,43,43,43,88,88,88,
32,215,163,176,67,108,207,160,0,98,93,46,151,224,28,178,0,98,149,74,229,207,91,91,91,5,233,234,104,64,108,69,69,69,48,114,124,13,177,90,173,246,147,145,145,17,76,79,79,11,174,212,209,128,216,160,28,213,
215,16,27,19,19,243,55,135,195,129,169,169,169,168,67,108,144,190,8,59,98,45,22,11,25,179,24,77,136,117,185,92,72,77,77,221,29,98,141,70,35,249,230,127,88,16,75,211,244,223,4,16,43,145,72,96,181,90,49,
54,54,22,85,136,45,47,47,231,235,240,39,2,136,229,222,66,110,75,27,173,97,93,118,187,157,236,179,104,154,254,121,8,196,154,76,38,140,140,140,144,168,110,164,33,182,180,180,84,48,207,104,87,136,149,201,
100,104,110,110,70,127,127,127,84,18,177,65,79,26,182,119,133,88,185,92,142,194,194,66,140,141,141,69,124,88,23,215,172,244,64,136,165,105,26,153,153,153,152,156,156,20,60,186,143,4,196,6,229,134,239,
15,177,25,25,25,88,88,88,32,241,144,72,64,172,213,106,13,62,227,126,127,136,213,106,181,219,220,182,37,82,16,27,116,124,224,193,16,171,84,42,127,205,113,98,105,105,41,185,35,9,23,196,22,22,22,10,94,165,
131,64,236,239,57,239,204,202,202,66,73,73,137,32,179,114,88,136,13,150,225,67,117,196,202,100,50,104,52,26,168,84,42,100,103,103,195,225,112,144,123,184,131,12,235,42,45,45,133,221,110,71,65,65,1,127,
83,119,248,142,88,138,162,32,149,74,193,189,114,86,171,149,196,117,119,91,16,119,81,228,200,209,96,48,144,147,182,188,183,237,247,97,129,88,46,211,169,209,104,144,146,146,66,10,108,185,102,36,135,195,
129,188,188,60,152,76,38,40,149,74,82,162,198,53,7,236,88,232,54,195,48,191,142,72,34,54,46,46,14,74,165,18,121,121,121,200,207,207,71,70,70,6,148,74,37,55,7,139,44,38,200,63,35,11,177,34,145,104,91,34,
145,32,54,54,150,188,34,252,133,4,159,216,142,42,196,178,44,251,115,150,101,63,17,137,68,127,219,107,88,215,183,16,251,119,3,177,52,77,127,190,227,234,255,206,178,236,123,81,135,88,238,3,205,61,149,74,
74,74,66,90,90,26,60,30,15,106,106,106,144,155,155,11,133,66,17,57,136,85,40,20,239,177,44,251,169,70,163,65,108,108,44,76,38,19,140,70,35,138,139,139,225,118,187,49,56,56,136,241,241,113,156,57,115,134,
244,198,54,52,52,192,106,181,242,187,59,195,7,177,50,153,236,63,234,234,234,182,135,134,134,48,58,58,10,191,223,15,191,223,143,64,32,128,233,233,105,76,79,79,99,116,116,20,253,253,253,130,59,161,198,198,
70,184,92,46,228,229,229,241,7,72,28,29,98,41,138,250,77,78,78,14,158,121,230,25,76,79,79,99,112,112,16,189,189,189,232,233,233,9,105,65,221,205,61,57,86,228,248,154,119,211,121,120,136,85,171,213,152,
159,159,199,210,210,210,174,238,185,223,106,130,138,138,10,194,215,50,153,140,75,96,31,110,88,87,117,117,53,158,120,226,9,12,15,15,31,9,98,235,235,235,81,83,83,131,178,178,50,88,173,86,100,100,100,112,
191,32,191,57,16,196,38,38,38,146,36,117,184,32,182,186,186,154,28,103,225,78,142,48,12,243,31,251,130,88,173,86,139,153,153,25,92,185,114,37,228,224,253,97,33,150,219,82,183,181,181,145,30,98,143,199,
195,61,192,124,239,129,16,91,85,85,133,181,181,53,12,15,15,135,5,98,27,27,91,80,95,223,136,193,193,65,172,173,173,225,246,237,219,184,123,247,46,230,230,230,184,157,196,167,123,66,44,77,211,176,217,108,
184,113,227,6,102,103,103,201,111,216,81,33,182,249,116,59,252,254,25,18,104,230,126,30,127,252,113,146,81,216,19,98,147,147,147,113,254,252,121,44,44,44,8,126,229,143,10,177,221,221,253,36,170,203,253,
108,109,109,225,244,233,211,247,175,38,72,74,74,218,126,228,145,71,240,200,35,143,8,100,56,28,16,219,222,222,131,103,158,121,150,228,135,231,230,230,208,209,209,129,242,242,114,126,55,191,16,98,21,10,
197,123,197,197,197,88,95,95,23,116,85,135,19,98,199,199,167,177,176,184,132,137,201,25,212,212,212,16,243,84,169,84,187,67,108,76,76,204,191,59,28,14,114,145,140,102,34,54,104,224,174,48,17,155,149,149,
133,249,249,249,168,67,44,103,10,59,95,218,95,67,44,77,211,159,27,141,70,248,253,126,116,117,117,69,21,98,131,116,88,8,177,201,201,201,232,235,235,67,127,127,127,84,33,182,168,168,104,239,97,93,20,69,
161,172,172,140,76,16,139,22,196,58,157,78,126,130,255,115,1,196,178,44,139,236,236,108,76,79,79,71,21,98,93,46,23,127,191,245,126,8,196,102,101,101,225,252,249,243,100,81,209,128,88,126,175,39,77,211,
255,46,128,88,154,166,97,52,26,177,184,184,40,152,141,21,73,136,229,116,152,219,231,51,12,243,94,8,196,154,76,38,242,240,49,26,16,91,84,84,196,159,84,190,59,196,198,197,197,145,203,66,164,33,214,233,116,
194,104,52,62,24,98,19,18,18,208,215,215,7,159,207,23,81,136,229,198,121,238,11,98,165,82,233,167,118,187,29,195,195,195,168,175,175,143,24,196,230,231,231,7,247,123,238,13,177,114,185,252,189,212,212,
84,88,44,22,20,23,23,163,185,185,57,236,16,107,183,219,5,95,194,251,130,88,137,68,242,31,26,141,6,113,113,113,200,205,205,69,121,121,57,26,27,27,195,6,177,65,161,248,253,67,44,151,239,140,143,143,135,
82,169,68,89,89,25,188,94,239,145,33,54,63,63,63,216,207,15,14,177,59,83,82,17,23,23,135,140,140,12,18,65,58,40,196,238,86,126,124,100,136,85,40,20,164,150,128,19,226,189,22,196,253,118,149,150,150,162,
176,176,16,249,249,249,56,126,252,120,240,211,207,223,132,13,98,119,14,64,195,235,245,146,137,98,110,183,27,197,197,197,56,113,226,4,204,102,51,18,18,18,72,246,147,27,155,206,5,86,119,174,218,255,17,86,
136,165,105,26,42,149,10,106,181,26,233,233,233,200,200,200,64,124,124,60,105,0,96,24,6,98,177,24,18,137,132,116,201,242,100,248,83,149,74,245,94,196,32,86,169,84,110,171,84,42,193,98,120,227,52,30,30,
196,178,44,251,30,195,48,255,190,179,221,248,252,91,136,253,187,135,216,224,142,88,154,166,31,137,56,196,138,197,226,183,41,138,250,131,88,44,222,166,105,154,252,170,75,36,18,176,44,11,169,84,138,132,
132,4,104,181,90,228,230,230,194,100,50,109,203,229,242,63,70,4,98,69,34,209,27,6,131,225,47,78,167,19,53,53,53,112,185,92,100,52,11,247,125,216,210,210,130,137,137,9,244,246,246,98,126,126,30,19,19,19,
232,236,236,68,118,118,54,247,27,249,113,88,171,9,226,227,227,183,167,167,167,177,178,178,130,201,201,73,140,140,140,96,116,116,20,195,195,195,24,29,29,197,208,208,16,134,135,135,73,50,182,171,171,139,
4,80,237,118,59,210,211,211,57,239,12,15,196,42,20,10,244,244,244,16,175,58,168,123,214,214,214,162,172,172,12,70,163,145,255,64,233,240,16,43,145,72,80,86,86,134,155,55,111,98,114,114,50,100,20,213,126,
33,182,166,166,6,133,133,133,92,105,237,193,190,144,249,91,23,174,32,114,101,101,133,208,205,81,19,177,101,101,101,200,202,202,130,92,46,63,88,71,44,55,8,192,104,52,226,226,197,139,120,228,145,71,4,238,
121,84,136,117,56,28,56,117,234,20,12,6,3,247,213,180,191,106,2,185,92,142,137,137,9,92,190,124,89,0,104,225,128,216,234,234,106,84,86,86,162,184,184,24,201,201,201,96,89,118,155,166,233,55,246,132,88,
154,166,63,54,24,12,152,159,159,199,149,43,87,48,50,50,18,54,136,173,171,107,32,249,134,142,142,14,180,182,182,162,186,186,26,70,163,17,44,203,254,101,79,136,149,72,36,24,29,29,197,218,218,154,32,200,
124,84,136,109,108,108,69,99,99,11,186,187,187,49,49,49,129,235,215,175,227,157,119,222,193,243,207,63,143,186,186,58,174,54,243,237,16,136,213,233,116,127,92,92,92,196,213,171,87,5,175,80,184,134,117,
245,248,6,176,181,117,35,228,207,106,104,104,224,126,27,255,32,128,88,154,166,31,177,88,44,184,118,237,26,9,11,134,187,35,118,105,233,162,224,239,187,125,251,54,174,94,189,138,188,188,60,82,123,17,146,
136,229,170,125,131,159,78,133,11,98,207,157,95,198,75,47,253,0,111,191,253,54,54,55,55,73,204,46,45,45,109,119,136,229,170,9,206,158,61,75,34,186,145,128,216,217,192,60,206,206,159,131,207,215,75,110,
60,246,76,196,82,20,245,190,193,96,128,223,239,135,207,231,139,42,196,6,197,150,222,23,116,196,202,229,114,244,246,246,98,116,116,52,170,16,27,4,29,66,136,149,74,165,240,120,60,196,60,163,5,177,65,97,
65,33,196,178,44,11,151,203,133,185,185,185,168,66,236,201,147,39,119,31,214,69,81,212,251,12,195,160,168,168,8,243,243,243,81,133,216,226,226,226,189,59,98,37,18,9,108,54,27,22,22,22,162,86,77,192,157,
190,229,161,199,191,134,64,172,205,102,195,226,226,98,212,58,98,57,119,224,22,21,146,136,149,72,36,219,86,171,21,243,243,243,130,13,90,36,33,150,227,33,238,237,211,235,245,143,4,67,236,31,114,115,115,
113,238,220,57,193,6,45,146,195,186,242,243,243,249,135,165,255,24,2,177,12,195,188,157,145,145,129,206,206,78,52,52,52,68,60,17,91,80,80,32,40,48,218,19,98,19,18,18,254,146,144,144,128,252,252,124,180,
183,183,71,20,98,131,10,28,62,222,19,98,105,154,126,67,161,80,108,235,116,58,228,230,230,194,237,118,147,115,122,225,130,88,155,205,22,50,212,100,95,213,4,92,161,168,70,163,65,117,117,117,216,32,214,110,
183,11,134,204,31,24,98,25,134,129,90,173,70,66,66,2,28,14,7,57,60,120,20,136,13,74,195,30,14,98,41,138,130,70,163,65,66,66,2,42,43,43,209,208,208,112,40,136,229,102,171,5,129,218,225,33,150,27,103,173,
213,106,97,183,219,9,194,30,68,134,11,10,10,4,51,178,194,6,177,220,91,201,149,56,184,221,110,148,150,150,226,228,201,147,56,117,234,20,210,211,211,73,2,205,96,48,192,104,52,34,57,57,25,201,201,201,252,
187,226,176,87,19,124,204,48,12,52,26,13,180,90,45,9,158,114,225,83,142,131,118,251,217,185,27,254,139,74,165,122,35,34,16,171,84,42,255,200,31,202,21,92,73,192,143,230,210,52,189,77,81,212,31,100,50,
217,219,209,128,216,71,246,26,214,245,45,196,254,63,3,177,81,73,196,178,44,251,174,209,104,252,179,66,161,216,214,106,181,144,203,229,164,21,128,187,249,224,82,178,59,151,129,47,21,10,197,61,137,68,242,
211,136,85,19,100,102,102,34,16,8,96,110,110,14,62,159,143,140,41,110,104,104,64,115,115,51,60,30,143,96,112,234,192,192,0,188,94,47,140,70,35,196,98,241,189,176,66,44,195,48,219,199,143,31,199,234,234,
42,150,151,151,225,243,249,208,222,222,142,238,238,110,116,118,118,146,5,114,89,153,158,158,30,244,245,245,145,41,7,14,135,3,9,9,9,96,24,102,59,44,16,203,31,224,124,254,252,121,65,232,107,191,149,172,
94,175,55,184,55,225,240,16,203,229,61,231,230,230,176,180,180,36,232,135,61,232,176,46,167,211,137,212,212,84,126,192,249,224,16,43,18,137,144,158,158,142,165,165,37,92,190,124,89,112,206,253,48,16,91,
83,83,131,146,146,18,190,169,31,60,17,107,54,155,177,184,184,136,149,149,21,129,123,134,3,98,211,210,210,72,151,208,190,32,86,38,147,109,59,28,14,50,95,45,18,16,107,179,217,96,54,155,161,84,42,31,220,
17,171,84,42,239,213,215,215,99,115,115,19,115,115,115,130,15,117,56,33,150,107,167,60,126,252,56,247,86,238,94,77,160,80,40,126,90,92,92,140,141,141,13,193,40,188,112,66,108,67,195,215,173,205,109,109,
109,168,170,170,130,197,98,225,30,128,191,27,2,177,113,113,113,247,198,199,199,113,254,252,249,144,2,144,112,64,108,115,115,59,218,59,124,24,28,28,196,153,51,103,240,228,147,79,226,205,55,223,196,196,
196,4,183,168,191,236,218,17,219,215,215,39,104,209,13,119,71,236,218,218,85,252,240,135,175,8,254,254,155,55,111,114,89,207,80,136,85,169,84,104,108,108,20,44,42,156,16,59,49,49,141,151,94,250,39,193,
223,251,198,27,111,96,121,121,121,111,136,149,74,165,104,110,110,142,88,71,236,192,224,40,158,186,249,52,238,222,189,139,127,252,199,23,176,180,180,132,193,193,65,56,157,206,189,33,150,162,40,82,57,30,
41,136,13,156,89,196,226,185,101,140,142,78,144,243,129,65,188,40,132,88,133,66,1,175,215,139,64,32,16,85,136,229,205,218,14,133,88,138,162,112,250,244,105,4,2,129,168,66,108,112,12,78,0,177,10,133,2,
237,237,237,152,154,154,138,42,196,238,69,214,37,20,69,189,175,84,42,49,62,62,142,137,137,137,168,66,108,80,242,76,8,177,42,149,10,51,51,51,130,185,88,209,128,216,204,204,204,189,59,98,147,147,147,113,
233,210,37,248,124,190,168,66,44,127,172,89,8,196,42,149,202,237,197,197,69,116,119,119,71,21,98,121,13,168,161,29,177,201,201,201,127,73,79,79,23,108,252,35,13,177,165,165,165,72,78,78,38,173,149,33,
16,43,145,72,222,77,76,76,132,209,104,36,119,40,145,132,216,178,178,50,193,231,73,38,147,253,116,207,106,2,157,78,135,148,148,20,148,149,149,225,244,233,211,17,129,88,187,221,142,148,148,20,62,162,221,
187,111,53,1,195,48,219,90,173,22,241,241,241,168,171,171,139,8,196,6,157,0,217,223,176,46,154,166,17,27,27,11,189,94,15,175,215,139,250,250,250,176,64,172,205,102,67,124,124,252,225,33,150,162,40,36,
36,36,32,41,41,137,204,247,56,10,196,22,20,20,4,103,134,15,7,177,52,77,35,33,33,1,153,153,153,36,220,112,24,136,229,238,144,131,22,116,180,68,172,193,96,128,213,106,221,117,65,149,149,149,164,9,96,183,
5,217,237,118,254,193,138,240,65,44,203,178,219,122,189,30,22,139,5,86,171,21,5,5,5,176,88,44,56,118,236,24,44,22,11,210,211,211,145,157,157,141,172,172,44,228,229,229,161,160,160,0,133,133,133,40,41,
41,9,249,80,135,21,98,165,82,233,189,248,248,120,18,189,229,67,171,84,42,21,252,123,177,88,28,50,39,43,98,213,4,44,203,254,148,101,217,123,92,187,46,47,15,76,254,202,253,236,168,240,182,72,36,250,139,
76,38,123,247,161,12,235,250,22,98,255,174,32,150,101,217,119,104,154,254,239,15,29,98,41,138,122,131,97,152,143,178,179,179,183,185,6,239,212,212,84,210,115,205,125,224,185,95,128,157,92,231,31,34,218,
17,27,19,19,131,234,234,106,108,109,109,97,118,118,150,28,252,233,233,233,33,207,3,185,150,239,252,252,124,28,59,118,12,169,169,169,72,75,75,131,74,165,250,51,195,48,239,132,13,98,41,138,250,50,38,38,
6,181,181,181,184,124,249,50,233,211,227,223,222,183,181,181,161,171,171,11,221,221,221,232,235,235,67,111,111,47,122,123,123,137,0,238,140,68,15,15,196,82,20,5,185,92,142,214,214,86,172,174,174,98,122,
122,90,16,66,221,143,123,214,215,215,163,170,170,10,134,255,159,189,123,125,106,235,188,247,5,174,165,133,150,164,37,201,146,184,72,66,136,139,12,194,220,177,16,23,33,196,69,200,220,49,6,203,88,54,6,140,
13,50,88,96,44,95,48,23,3,193,78,92,183,141,227,221,118,39,219,237,238,110,178,211,115,98,215,185,184,157,190,232,31,113,118,103,250,170,157,233,139,54,51,153,76,115,169,211,166,217,211,61,39,206,132,
239,121,81,158,39,107,73,2,115,209,146,211,51,121,193,100,146,76,198,79,36,177,244,172,207,250,62,223,159,213,42,204,119,238,14,98,25,134,65,86,86,22,166,166,166,112,227,198,13,122,120,108,55,16,219,209,
209,65,171,9,54,62,123,59,135,88,149,74,133,242,242,114,204,206,206,98,113,113,113,207,16,219,217,217,9,175,215,139,156,156,156,221,65,172,86,171,69,79,79,15,230,230,230,112,241,226,197,132,65,44,249,
69,200,202,202,18,30,16,219,26,98,85,42,213,219,37,37,37,88,94,94,198,202,202,10,66,161,144,200,206,19,5,177,53,53,53,72,75,75,163,51,254,182,132,88,171,213,250,223,4,242,131,193,96,66,134,117,109,6,177,
100,22,233,198,231,107,243,142,216,252,252,124,220,184,113,35,38,119,46,5,196,146,196,181,160,18,63,110,53,193,199,118,187,29,55,111,222,140,121,220,145,40,136,237,61,124,148,190,133,126,191,31,62,159,
15,101,101,101,208,233,116,96,24,230,207,113,33,182,176,176,16,75,75,75,34,174,78,36,196,6,2,39,232,49,132,217,217,89,220,188,121,19,11,11,11,112,56,28,241,19,177,50,153,12,69,69,69,88,88,88,160,139,74,
116,71,236,133,217,203,248,183,123,247,240,214,91,95,121,195,131,7,15,208,214,214,70,54,137,111,139,32,150,101,89,20,21,21,97,113,113,17,129,64,64,146,68,236,139,47,190,20,243,223,190,253,246,219,66,247,
252,175,24,136,109,106,106,194,181,107,215,36,131,216,27,55,111,225,209,163,71,120,253,245,159,226,246,237,219,180,10,65,80,81,46,134,88,142,227,208,223,223,143,153,153,25,201,32,246,98,100,14,115,215,
150,48,61,243,143,220,95,75,75,75,244,64,121,49,196,166,166,166,34,20,10,97,98,98,34,169,16,43,68,179,24,136,205,200,200,192,234,234,42,134,135,135,147,10,177,81,41,107,49,196,238,223,191,31,203,203,203,
8,4,2,73,133,88,147,201,20,31,98,25,134,249,47,242,155,215,223,223,159,52,136,109,104,104,136,142,195,125,5,177,28,199,189,93,88,88,136,235,215,175,139,246,68,82,67,108,67,67,131,40,86,25,3,177,86,171,
117,157,164,21,147,5,177,30,143,71,184,183,138,133,88,150,101,255,108,179,217,80,86,86,22,179,107,148,10,98,163,194,204,31,199,173,38,80,169,84,148,125,72,50,86,42,136,141,206,13,111,217,17,171,82,169,
144,149,149,133,206,206,206,168,22,128,196,14,235,170,173,173,21,14,236,250,239,45,33,86,38,147,125,169,84,42,41,43,118,117,117,37,20,98,235,234,234,232,195,108,242,42,105,181,218,183,159,10,177,196,59,
115,115,115,233,198,63,81,16,75,134,205,239,10,98,25,134,161,159,47,50,25,101,47,16,235,116,58,163,51,195,187,131,88,142,227,144,147,147,131,227,199,143,163,189,189,125,87,16,75,70,90,71,133,79,247,6,
177,100,6,86,125,125,61,154,155,155,69,115,176,200,130,154,155,155,55,93,208,254,253,251,163,139,27,18,3,177,102,179,25,90,173,22,164,123,49,61,61,29,38,147,9,102,179,25,121,121,121,56,112,224,0,202,202,
202,224,118,187,225,241,120,208,208,208,0,175,215,43,60,238,75,47,144,9,131,88,142,227,222,214,104,52,255,205,243,60,148,74,37,237,66,80,169,84,52,151,71,126,244,122,61,204,102,179,240,190,46,41,29,177,
31,147,82,7,65,242,149,202,48,89,220,198,103,104,125,227,218,119,255,235,0,177,255,149,146,146,242,246,55,16,251,181,133,88,150,101,31,102,100,100,188,203,113,220,71,207,28,98,101,50,217,71,74,165,114,
189,180,180,20,39,79,158,68,87,87,23,85,20,158,231,163,59,128,190,148,124,88,23,233,120,233,237,237,197,226,226,34,166,167,167,105,0,108,112,112,16,237,237,237,162,193,130,10,133,2,228,50,146,208,142,
88,2,177,114,185,28,118,187,29,211,211,211,88,91,91,195,196,196,132,168,232,143,108,163,251,251,251,209,215,247,143,51,199,193,96,16,237,237,237,180,9,60,39,39,103,157,227,184,63,38,4,98,141,70,35,124,
62,31,110,220,184,17,147,104,140,118,207,232,52,108,79,79,15,157,55,226,243,249,72,51,215,174,33,246,79,90,173,22,69,69,69,24,25,25,193,220,220,92,76,128,112,39,16,219,213,213,5,159,207,7,147,201,68,174,
244,127,218,49,196,90,44,22,12,12,12,32,18,137,32,18,137,224,244,233,211,34,25,222,13,196,18,33,86,42,149,228,75,122,123,16,171,215,235,255,24,8,4,112,227,198,13,204,207,207,99,124,124,60,6,209,118,11,
177,29,29,29,240,122,189,176,90,173,219,135,88,153,76,246,191,138,139,139,113,251,246,109,132,66,161,152,90,223,68,64,108,91,91,27,14,30,60,40,244,206,47,159,58,172,171,165,165,133,194,153,84,16,219,212,
212,132,3,7,14,8,163,224,155,67,172,70,163,65,119,119,55,34,145,136,164,16,75,170,47,178,179,179,133,197,71,241,59,98,141,70,35,102,102,102,98,30,8,73,5,177,78,167,147,132,154,227,119,196,146,209,46,223,
253,238,119,69,213,190,137,132,216,129,129,227,56,26,8,208,125,62,121,27,55,130,243,241,33,54,39,39,7,55,111,222,148,112,88,215,41,12,143,156,65,48,24,196,232,232,40,38,39,39,113,246,236,89,20,20,20,144,
193,150,15,69,16,43,147,201,80,89,89,41,41,196,78,157,191,128,87,94,185,135,215,95,255,169,232,191,31,25,25,33,234,242,174,8,98,85,42,21,186,187,187,113,245,234,85,90,220,145,104,136,125,225,133,219,248,
249,207,127,30,243,63,117,230,204,25,250,185,18,65,108,106,106,42,230,231,231,17,10,133,36,131,216,231,214,158,199,131,7,15,240,240,225,67,220,187,119,15,183,111,223,198,165,75,151,132,196,40,134,216,
172,172,44,220,185,115,135,30,208,145,4,98,23,86,176,178,114,19,87,175,206,99,96,96,0,221,221,221,168,175,175,223,28,98,219,218,218,176,180,180,36,250,114,77,6,196,70,233,240,87,16,171,82,169,62,10,4,
2,52,55,156,44,136,245,122,189,209,45,131,95,65,172,217,108,126,215,225,112,96,116,116,84,180,39,146,26,98,61,30,79,244,196,167,175,32,150,101,217,135,86,171,21,117,117,117,73,77,196,70,127,158,98,32,
150,227,184,245,220,220,92,244,245,245,161,191,191,63,41,16,123,240,224,65,209,77,70,220,142,88,149,74,133,204,204,76,28,63,126,60,41,16,91,88,88,248,244,142,88,50,210,179,186,186,90,52,89,64,42,136,21,
54,87,110,9,177,12,195,32,39,39,7,77,77,77,232,235,235,147,12,98,61,30,143,240,105,195,159,159,10,177,251,246,237,131,205,102,195,145,35,71,18,14,177,194,179,238,228,85,218,86,34,150,196,40,235,235,235,
233,36,177,68,64,108,93,93,29,28,14,135,40,132,202,178,236,31,183,13,177,106,181,154,206,243,35,11,218,45,196,122,60,30,148,148,148,144,169,190,123,130,216,63,165,166,166,162,181,181,149,30,177,220,45,
196,230,231,231,139,32,86,208,26,191,59,136,205,202,202,66,110,110,46,28,14,7,10,11,11,225,116,58,69,125,176,91,45,40,58,81,45,240,207,189,65,44,207,243,127,212,235,245,224,121,30,60,207,211,168,145,78,
167,131,213,106,69,97,97,33,92,46,23,124,62,31,188,94,175,136,169,5,15,172,19,15,177,164,35,54,37,37,133,86,13,168,84,42,240,60,15,141,70,3,149,74,133,125,251,246,193,110,183,163,164,164,4,153,153,153,
209,42,44,253,176,46,225,48,121,146,126,37,139,20,132,182,8,196,126,244,204,33,150,101,217,143,20,10,197,187,44,203,62,252,6,98,191,214,213,4,12,195,252,90,175,215,127,98,50,153,190,216,248,28,61,59,136,
149,201,100,95,106,181,90,184,92,46,156,62,125,154,206,204,38,179,137,4,101,162,201,129,88,165,82,137,188,188,60,156,60,121,18,145,72,4,225,112,88,116,7,212,214,214,134,210,210,82,164,167,167,131,231,
121,225,243,153,196,67,44,199,113,48,155,205,244,180,228,229,203,151,113,250,244,105,12,14,14,82,102,36,207,156,73,174,147,60,188,222,191,127,63,140,70,35,228,114,249,222,19,177,12,195,188,167,86,171,
97,177,88,208,212,212,132,241,241,113,92,190,124,25,147,147,147,180,51,97,51,247,36,197,14,135,15,31,166,219,159,182,182,54,20,21,21,65,169,84,126,188,43,136,85,169,84,31,229,228,228,160,187,187,27,211,
211,211,184,114,229,10,206,159,63,191,165,12,111,5,177,100,23,235,245,122,97,48,24,32,151,203,63,218,17,196,154,76,166,143,73,175,231,149,43,87,16,10,133,98,202,27,118,11,177,62,159,15,25,25,25,228,23,
225,189,109,39,98,123,122,122,112,235,214,45,140,141,141,197,29,103,182,23,136,237,232,232,64,113,113,177,176,200,253,233,16,203,178,236,58,249,173,146,2,98,59,58,58,224,241,120,160,211,233,182,151,136,
149,201,100,127,182,217,108,184,122,245,42,157,32,45,5,196,250,253,126,216,237,118,225,245,108,115,136,149,201,100,40,44,44,196,226,226,98,76,121,67,162,19,177,46,151,11,58,157,238,233,16,155,146,146,
130,198,198,70,204,207,207,139,234,9,164,128,88,114,220,87,80,123,17,31,98,211,210,210,112,246,236,89,58,222,69,106,136,173,169,169,33,125,47,177,195,186,72,247,98,110,110,46,214,214,214,104,155,110,194,
33,118,240,20,2,129,19,244,198,163,185,185,153,246,189,200,229,242,216,106,2,133,66,129,142,142,14,172,172,172,72,6,177,195,195,103,16,10,133,49,60,60,66,91,114,122,122,122,144,149,149,69,94,173,95,139,
32,86,171,213,226,214,173,91,152,156,156,148,44,17,123,97,246,50,94,121,229,30,126,252,227,31,227,181,215,94,195,253,251,247,241,206,59,239,96,124,124,156,252,38,126,34,130,88,167,211,137,219,183,111,
99,112,112,80,50,136,125,225,214,109,188,249,102,44,21,173,172,172,16,60,251,130,66,44,199,113,152,156,156,196,236,236,172,232,187,44,209,16,251,252,11,183,241,198,253,7,120,244,232,17,222,120,227,13,
188,250,234,171,120,249,229,151,209,223,223,47,156,78,247,15,136,181,90,173,95,140,141,141,209,130,98,41,33,246,31,215,180,69,4,131,65,4,2,1,116,118,118,10,159,102,125,5,177,6,131,225,147,178,178,50,156,
56,113,34,169,16,219,212,212,20,13,103,162,68,236,175,115,115,115,225,243,249,232,129,251,100,65,108,84,250,76,92,77,192,243,60,53,169,100,13,235,18,70,42,227,66,236,6,132,194,233,116,210,134,45,169,33,
54,170,181,50,62,196,114,28,135,220,220,220,141,183,238,136,164,16,235,118,187,163,195,204,155,67,108,70,70,6,202,203,203,113,226,196,9,73,19,177,46,151,75,4,104,91,66,44,199,113,200,203,203,67,48,24,
68,79,79,143,100,16,91,81,81,177,189,97,93,4,98,51,50,50,80,95,95,143,99,199,142,37,124,88,23,41,112,16,84,167,108,175,154,64,165,82,193,110,183,99,108,108,140,46,40,81,195,186,60,30,15,74,75,75,105,88,
98,219,137,88,134,97,222,51,153,76,232,233,233,193,225,195,135,233,110,113,47,137,216,234,234,106,216,237,118,232,245,122,225,93,243,199,59,130,88,142,227,62,202,204,204,68,101,101,37,234,234,234,224,
241,120,104,235,233,118,33,150,196,115,109,54,27,120,158,143,238,251,252,104,87,16,171,211,233,62,54,26,141,80,169,84,208,106,181,48,155,205,176,219,237,168,172,172,68,83,83,147,168,158,32,122,65,53,53,
53,200,205,205,141,89,140,224,126,111,111,137,88,134,97,214,213,106,53,180,90,45,180,90,45,52,26,13,12,6,3,242,243,243,225,245,122,233,208,46,210,27,91,92,92,12,131,193,32,202,129,18,133,73,56,196,146,
63,64,169,84,130,231,121,144,217,88,86,171,21,21,21,21,168,172,172,140,78,252,72,95,77,32,132,88,134,97,104,97,173,86,171,165,8,43,124,117,146,62,172,43,26,98,229,114,57,56,142,251,66,46,151,127,34,147,
201,126,253,77,53,193,63,77,71,44,199,113,159,103,101,101,129,231,249,39,50,153,236,131,103,6,177,27,223,236,168,168,168,192,200,200,8,198,199,199,209,217,217,137,162,162,34,225,158,59,57,16,75,22,227,
116,58,49,57,57,137,185,185,57,132,195,97,156,60,121,146,126,37,181,180,180,196,28,67,73,56,196,178,44,187,206,113,28,12,6,3,157,138,177,178,178,130,112,56,140,161,161,33,154,34,34,55,29,157,157,157,244,
236,67,81,81,145,240,73,232,222,134,117,41,20,138,111,235,116,186,15,205,102,51,220,110,55,134,135,135,177,176,176,128,249,249,121,76,77,77,61,21,98,73,251,252,161,67,135,208,214,214,6,143,199,35,108,
61,221,93,34,214,106,181,126,58,58,58,138,249,249,121,250,22,141,140,140,136,32,109,187,16,75,100,184,189,189,29,237,237,237,164,162,224,83,185,92,254,237,29,37,98,71,71,71,241,220,115,207,109,234,158,
187,133,216,67,135,14,161,174,174,142,128,217,135,219,134,88,189,94,143,23,95,124,145,38,169,19,13,177,62,159,15,122,189,126,251,29,177,50,153,236,203,202,202,74,220,186,117,43,166,9,32,81,16,219,222,
222,78,211,101,79,237,136,37,191,246,225,112,24,151,46,93,18,37,170,19,9,177,100,32,156,224,139,123,107,136,181,88,44,212,208,165,132,216,230,230,102,81,118,106,83,136,149,201,100,232,236,236,196,202,
202,138,228,16,235,247,251,163,207,32,199,135,88,131,193,128,229,229,101,218,126,154,12,136,21,116,84,125,25,55,17,219,214,214,134,149,149,21,122,97,76,6,196,90,173,214,205,59,98,83,83,83,177,188,188,
140,112,56,44,217,176,174,225,225,51,24,29,29,199,225,195,125,244,166,163,188,188,92,248,228,225,171,142,88,134,97,62,104,107,107,195,245,235,215,37,31,214,21,158,190,136,209,209,175,102,111,181,181,181,
9,19,140,95,85,19,24,141,198,39,228,33,145,240,203,53,209,16,59,55,183,136,151,95,254,55,252,232,71,63,194,247,190,247,61,252,203,191,252,11,190,243,157,239,224,232,209,163,228,213,250,156,66,172,205,
102,195,232,232,40,70,70,70,36,133,216,239,190,120,23,15,223,124,51,230,237,95,91,91,19,30,39,255,7,196,114,28,247,249,129,3,7,48,58,58,42,218,130,36,26,98,191,251,226,93,60,124,248,48,102,81,225,112,
152,132,117,158,136,170,9,242,242,242,104,135,190,148,16,251,189,239,189,130,171,115,215,48,50,50,130,145,145,17,244,247,247,11,221,243,3,81,71,172,74,165,194,129,3,7,112,250,244,233,164,65,172,215,235,
141,110,211,21,67,172,76,38,67,110,110,174,40,170,43,53,196,214,214,214,138,50,86,113,33,86,169,84,194,229,114,209,7,57,82,66,108,156,193,242,241,33,86,38,147,33,63,63,31,231,207,159,151,188,35,214,237,
118,71,135,153,55,135,88,189,94,143,147,39,79,210,158,88,169,32,150,12,168,220,54,196,150,151,151,99,120,120,88,178,97,93,13,13,13,194,22,221,167,67,172,92,46,95,39,111,161,112,183,152,72,136,173,171,
171,19,18,209,246,134,117,165,167,167,211,214,146,68,67,108,67,67,131,232,8,193,182,33,86,171,213,126,104,181,90,81,91,91,75,67,167,123,133,88,34,195,164,1,110,199,137,88,150,101,191,109,48,24,62,37,222,
153,149,149,133,170,170,42,218,2,176,211,142,216,138,138,10,58,138,74,32,195,159,238,58,17,203,113,28,117,78,173,86,43,114,206,173,22,84,91,91,27,3,177,27,79,212,193,48,204,135,12,195,124,123,207,195,
186,8,108,232,245,122,168,84,42,236,223,191,31,13,13,13,104,108,108,164,11,242,120,60,112,58,157,200,202,202,162,89,225,40,21,94,151,4,98,201,56,32,189,94,15,181,90,141,253,251,247,163,166,166,6,149,149,
149,176,88,44,241,26,36,146,155,136,37,241,92,165,82,9,165,82,25,51,172,235,89,66,236,7,28,199,61,217,200,172,124,254,77,71,236,63,237,176,46,149,74,245,196,100,50,253,141,101,217,223,60,51,136,77,73,
73,1,137,92,246,246,246,98,124,124,28,189,189,189,200,207,207,39,215,167,245,164,65,172,94,175,71,110,110,46,60,30,15,134,135,135,177,178,178,130,217,217,89,186,107,245,249,124,168,172,172,68,90,90,154,
240,57,77,98,32,86,161,80,44,106,181,218,63,104,52,154,117,171,213,10,175,215,139,64,32,128,83,167,78,33,20,10,33,18,137,96,106,106,10,67,67,67,162,91,179,174,174,46,28,58,116,136,30,186,80,171,213,194,
103,53,123,75,196,102,102,102,126,210,215,215,135,75,151,46,225,218,181,107,152,157,157,197,248,248,56,70,71,71,113,234,212,41,81,68,55,30,196,182,181,181,193,239,247,163,174,174,14,57,57,57,187,63,120,
33,236,10,58,126,252,56,150,151,151,17,10,133,48,56,56,24,195,140,219,129,216,142,142,14,186,211,240,249,124,40,42,42,34,219,224,157,119,196,154,76,38,220,189,123,55,238,193,251,221,64,44,177,78,242,5,
190,129,255,159,200,229,242,197,109,15,235,234,237,237,197,243,207,63,31,247,21,218,11,196,182,183,183,163,165,165,5,7,15,30,4,207,243,96,89,246,15,219,130,88,150,101,113,247,238,93,76,77,77,73,6,177,
94,175,151,252,102,62,29,98,89,150,69,93,93,29,238,220,185,35,42,111,72,52,196,250,253,126,33,106,108,13,177,38,147,9,75,75,75,180,214,82,42,136,37,195,6,159,10,177,60,207,163,163,163,3,55,111,222,20,
69,189,165,130,88,193,4,149,248,16,203,178,236,122,97,97,33,150,151,151,69,125,9,82,66,172,211,233,20,197,3,98,32,214,108,54,35,20,10,97,110,110,78,186,106,130,40,136,141,238,94,20,65,44,199,113,191,241,
251,253,184,126,253,58,70,70,70,146,6,177,205,205,205,244,48,107,12,196,230,228,228,124,22,10,133,48,51,51,67,35,186,82,65,108,232,220,52,142,30,61,70,47,164,78,167,51,250,32,226,63,32,150,231,249,39,
157,157,157,56,123,246,172,232,171,67,138,97,93,151,46,95,195,68,104,146,14,124,110,104,104,16,62,57,21,87,19,20,20,20,72,158,136,125,110,237,5,252,240,135,255,142,123,247,126,68,139,144,167,166,166,80,
82,82,18,191,154,192,100,50,97,100,100,4,129,64,64,50,136,125,249,229,123,120,235,173,216,203,75,36,18,33,183,102,98,136,229,121,30,78,167,19,35,35,35,146,65,236,43,255,246,239,120,244,232,81,204,162,
166,167,167,201,103,234,51,17,196,166,164,164,32,63,63,159,230,135,165,130,216,159,188,250,83,92,184,112,1,147,147,147,136,68,34,152,158,158,70,78,78,14,121,235,126,19,3,177,54,155,13,3,3,3,8,4,2,73,171,
38,16,62,48,138,11,177,6,131,1,29,29,29,24,30,30,78,74,34,54,106,76,222,250,166,16,91,85,85,133,177,177,49,201,33,182,166,166,38,58,225,177,57,196,58,28,14,204,204,204,136,30,221,75,1,177,81,79,69,183,
134,88,139,197,130,72,36,66,135,66,72,1,177,110,183,59,70,100,182,132,88,173,86,187,222,209,209,17,213,149,144,88,136,117,56,28,34,212,127,42,196,242,60,255,135,236,236,108,212,212,212,80,120,77,36,196,
186,221,110,97,148,105,123,16,203,178,236,162,193,96,248,36,45,45,13,164,253,205,239,247,39,12,98,163,94,165,157,119,196,170,213,106,232,245,122,144,156,122,67,67,3,61,117,182,93,136,109,104,104,128,199,
227,129,219,237,70,65,65,129,80,250,246,54,172,139,97,24,24,12,6,250,202,53,53,53,209,155,203,120,11,114,187,221,40,45,45,69,65,65,1,172,86,43,116,58,29,20,10,133,240,14,249,147,61,39,98,73,174,147,231,
121,24,141,70,88,44,22,212,212,212,196,44,200,229,114,33,43,43,11,58,157,142,158,71,38,63,27,50,188,174,80,40,254,192,243,252,98,66,33,150,227,56,24,141,70,232,245,122,20,20,20,192,229,114,225,192,129,
3,116,114,133,92,46,23,45,36,105,16,203,48,204,186,70,163,129,209,104,132,70,163,129,92,46,23,26,249,179,133,88,150,101,127,195,243,252,103,12,195,60,249,102,88,215,63,61,196,110,4,155,63,208,104,52,191,
83,42,149,191,124,38,16,75,58,56,116,58,29,50,50,50,80,81,81,65,211,138,249,249,249,96,89,86,26,136,101,24,230,168,82,169,124,139,60,202,77,75,75,131,197,98,129,213,106,69,117,117,53,58,59,59,49,56,56,
136,153,153,25,68,34,17,140,143,143,35,24,12,162,171,171,11,101,101,101,194,61,83,226,18,177,6,131,225,221,234,234,234,245,209,209,81,156,61,123,22,67,67,67,24,31,31,199,236,236,44,194,225,48,198,199,
199,113,230,204,25,58,13,74,152,138,37,23,212,180,180,52,122,130,54,33,195,186,234,235,235,177,176,176,128,153,153,25,12,13,13,225,228,201,147,24,26,26,18,197,227,54,115,207,238,238,110,52,53,53,209,135,
144,130,96,196,238,33,214,100,50,97,117,117,21,23,46,92,136,137,87,238,100,122,152,207,231,67,77,77,13,10,11,11,133,89,132,157,67,172,76,38,67,32,16,192,205,155,55,113,242,228,201,61,65,108,103,103,39,
252,126,63,90,90,90,208,216,216,136,194,194,66,178,167,218,254,176,46,134,97,96,54,155,113,247,238,93,156,59,119,46,97,16,75,198,79,145,67,62,25,25,25,224,56,238,93,134,97,142,62,21,98,141,70,35,130,193,
32,86,87,87,69,159,157,68,65,172,223,239,167,219,226,141,221,196,91,91,66,172,82,169,68,107,107,43,22,23,23,49,62,62,46,25,196,30,58,116,72,120,60,115,107,136,117,185,92,88,90,90,66,56,28,166,61,195,82,
64,236,161,67,135,68,163,130,54,133,216,244,244,116,124,235,91,223,194,236,236,172,136,171,165,130,216,218,218,218,173,33,86,173,86,175,71,34,17,44,47,47,139,62,71,82,66,108,67,67,131,240,164,182,24,98,
181,90,237,47,157,78,39,94,122,233,37,209,156,190,100,36,98,53,26,77,124,136,213,235,245,191,107,109,109,197,149,43,87,68,23,198,100,36,98,5,147,84,196,16,43,151,203,63,112,56,28,184,120,241,162,232,74,
45,53,196,182,180,180,68,155,130,120,88,87,126,126,62,109,102,150,18,98,103,103,175,96,120,248,52,93,20,57,101,187,177,175,23,67,172,201,100,162,51,67,165,28,214,181,184,244,28,166,167,103,233,103,202,
237,118,71,247,244,127,5,177,122,189,30,77,77,77,244,225,180,36,137,216,239,222,197,171,175,254,39,94,126,249,21,92,189,122,149,158,14,16,204,198,18,15,235,98,89,22,197,197,197,152,156,156,164,230,153,
104,136,125,237,181,159,198,244,14,223,191,127,31,237,237,237,228,237,19,39,98,9,152,93,186,116,137,110,65,18,13,177,255,251,141,159,197,45,67,30,30,30,38,159,171,223,197,64,172,213,106,197,236,236,172,
164,137,216,31,255,248,39,152,156,156,196,226,226,34,238,220,185,131,127,253,215,127,165,109,0,44,203,254,50,6,98,173,86,43,70,71,71,17,8,4,36,135,88,194,75,2,29,142,15,177,102,179,25,67,67,67,8,6,131,
73,73,196,10,185,122,83,136,85,169,84,116,15,46,37,196,122,60,158,232,2,135,173,33,182,162,162,66,52,61,76,10,136,45,45,45,141,46,27,221,28,98,57,142,123,43,59,59,27,69,69,69,240,249,124,244,198,50,145,
16,91,95,95,143,125,251,246,109,31,98,25,134,57,202,243,252,187,38,147,9,58,157,14,21,21,21,9,135,216,168,135,217,59,27,214,69,212,110,223,190,125,56,120,240,32,253,109,217,11,196,86,87,87,71,199,189,
119,14,177,50,153,12,26,141,6,169,169,169,112,56,28,104,109,109,165,153,130,157,36,98,73,136,57,234,183,109,111,16,203,243,60,50,50,50,144,157,157,77,95,173,205,22,68,66,203,110,183,27,21,21,21,40,41,
41,65,73,73,9,44,22,75,226,58,98,9,196,166,164,164,32,45,45,13,58,157,14,110,183,27,109,109,109,180,146,192,235,245,162,188,188,28,185,185,185,176,88,44,20,98,57,142,35,83,7,232,130,88,150,125,55,225,
137,88,158,231,97,48,24,144,151,151,7,135,195,1,139,197,2,141,70,35,146,224,148,148,20,209,95,201,193,66,157,78,247,86,97,97,225,81,73,32,86,161,80,172,235,245,122,40,149,74,250,7,147,159,232,49,102,73,
133,88,149,74,245,75,133,66,241,187,141,237,198,255,253,6,98,255,191,128,88,242,246,201,229,242,7,146,67,44,203,178,19,4,98,55,128,21,50,153,140,110,250,73,12,220,96,48,160,176,176,16,197,197,197,40,46,
46,94,103,89,246,177,100,137,88,147,201,244,153,199,227,65,125,125,61,92,46,23,189,62,121,189,94,28,57,114,132,134,82,103,103,103,49,54,54,134,115,231,206,161,175,175,15,217,217,217,210,12,235,50,153,
76,235,164,23,246,204,153,51,24,31,31,199,216,216,24,70,70,70,112,246,236,89,218,168,123,234,212,41,28,59,118,12,125,125,125,56,114,228,8,45,126,20,148,243,37,6,98,141,70,35,134,135,135,177,188,188,76,
179,48,219,117,207,158,158,30,180,183,183,211,90,2,193,3,200,221,67,172,76,38,67,119,119,55,214,214,214,112,246,236,217,93,65,44,137,235,214,214,214,210,190,132,141,11,236,206,33,86,46,151,35,43,43,11,
47,190,248,34,166,166,166,68,131,186,118,147,136,245,251,253,168,174,174,70,126,126,190,80,90,182,15,177,100,212,203,185,115,231,176,176,176,32,242,170,189,64,172,223,239,167,103,106,4,199,88,182,87,77,
96,52,26,49,59,59,139,185,185,57,209,144,165,68,64,44,153,139,229,114,185,144,155,155,11,165,82,249,244,68,172,86,171,197,228,228,36,158,123,238,57,12,15,15,211,207,81,162,33,150,44,204,108,54,35,37,37,
229,51,133,66,49,17,23,98,211,210,210,16,8,4,112,235,214,173,152,87,72,10,136,21,14,254,142,11,177,60,207,63,30,27,27,163,67,223,146,49,172,171,181,181,21,102,179,121,115,136,45,42,42,194,15,126,240,3,
90,4,146,12,136,245,249,124,116,223,30,51,172,75,46,151,63,40,40,40,192,149,43,87,68,215,34,169,33,86,184,168,184,213,4,5,5,5,152,159,159,23,205,121,76,6,196,70,221,225,252,31,81,71,108,106,106,42,125,
132,150,204,68,172,160,181,36,22,98,83,83,83,17,8,4,68,199,77,146,1,177,81,7,164,197,16,203,243,60,186,187,187,49,62,62,46,41,196,206,207,47,35,20,154,162,247,138,81,105,88,49,196,202,100,50,184,221,110,
76,77,77,73,218,17,123,243,230,109,132,167,47,136,66,59,113,135,117,109,180,109,161,188,188,28,51,51,51,146,37,98,127,248,195,255,192,253,251,15,112,231,206,75,56,123,246,44,6,7,7,209,220,220,28,191,35,
150,64,172,211,233,196,236,236,172,100,213,4,15,163,58,135,95,125,245,85,44,46,46,10,119,13,177,213,4,69,69,69,136,68,34,18,13,235,250,5,126,254,243,216,127,118,255,254,125,154,73,103,89,246,129,8,98,
201,25,7,178,40,169,32,118,121,121,25,215,174,93,195,247,191,255,125,188,243,206,59,120,225,133,23,232,169,144,184,16,75,30,26,145,76,129,84,16,219,218,218,74,143,220,9,158,54,60,142,11,177,249,249,249,
180,131,67,234,68,108,101,101,165,232,120,112,92,136,149,203,229,19,70,163,241,179,140,140,12,212,213,213,73,154,136,173,172,172,140,22,189,173,19,177,251,246,237,91,207,200,200,64,73,73,137,104,80,87,
162,32,182,178,178,82,116,44,101,219,213,4,60,207,35,61,61,29,105,105,105,168,175,175,79,24,196,214,215,215,139,166,63,237,24,98,201,217,208,212,212,84,120,189,94,122,171,180,91,136,37,71,53,163,190,82,
118,14,177,12,195,192,104,52,34,53,53,21,30,143,135,222,42,237,6,98,75,74,74,162,65,127,239,16,155,158,158,142,170,170,42,209,162,54,91,80,67,67,131,40,47,28,231,131,157,24,136,85,42,149,48,155,205,112,
185,92,232,234,234,162,85,4,110,183,27,69,69,69,112,56,28,200,201,201,129,217,108,70,102,102,38,44,22,11,210,210,210,144,145,145,33,92,80,226,171,9,180,90,45,50,50,50,144,155,155,11,189,94,15,141,70,35,
66,87,133,66,65,85,88,248,179,241,182,125,38,89,34,86,161,80,60,38,36,173,80,40,98,34,185,66,136,37,39,182,85,42,213,125,155,205,54,33,57,196,110,72,221,166,146,247,13,196,254,83,67,108,210,170,9,212,
106,245,47,82,83,83,255,102,52,26,215,149,74,37,61,151,71,70,232,177,44,75,167,210,105,181,90,112,28,183,206,243,252,95,149,74,229,175,36,171,38,32,9,143,177,177,49,156,56,113,98,99,31,31,68,123,123,59,
58,58,58,208,219,219,139,206,206,78,156,56,113,2,71,142,28,193,200,200,8,218,219,219,201,60,191,79,18,10,177,28,199,173,59,28,14,124,231,59,223,17,245,194,14,14,14,34,24,12,226,216,177,99,24,26,26,162,
63,193,96,144,246,163,117,119,119,163,174,174,142,140,157,90,79,8,196,146,243,89,151,47,95,198,149,43,87,98,152,113,59,238,217,220,220,140,253,251,247,139,58,130,118,13,177,12,195,32,45,45,13,183,111,
223,70,36,18,193,201,147,39,119,5,177,100,194,83,94,94,158,48,170,187,115,136,101,24,6,249,249,249,152,156,156,196,245,235,215,69,193,193,221,64,108,91,91,27,234,235,235,97,183,219,105,49,205,142,33,214,
106,181,98,121,121,25,151,47,95,166,103,147,247,10,177,135,14,29,66,117,117,53,114,114,114,132,251,243,167,67,172,82,169,92,47,43,43,195,149,43,87,48,63,63,47,58,227,158,8,136,245,251,253,168,175,175,
71,105,105,41,180,90,45,228,114,249,214,16,203,178,236,39,29,29,29,152,159,159,143,249,12,37,26,98,107,107,107,97,183,219,201,126,61,126,34,118,223,190,125,191,170,170,170,194,157,59,119,48,53,53,37,250,
12,73,1,177,62,159,143,78,251,85,171,213,72,73,73,249,69,12,196,26,141,198,191,206,204,204,224,242,229,203,49,143,59,164,130,88,191,223,143,202,202,74,114,24,232,111,113,19,177,227,227,227,49,50,44,37,
196,182,182,182,162,166,166,134,238,74,99,170,9,140,70,35,134,134,134,68,213,190,82,67,108,107,107,43,197,141,184,16,171,80,40,48,48,48,128,115,231,206,37,21,98,163,38,146,139,33,86,163,209,160,189,189,
29,231,206,157,75,42,196,70,241,162,24,98,83,82,82,208,214,214,134,233,233,233,164,66,108,212,113,77,49,196,170,84,42,180,182,182,34,28,14,39,21,98,227,220,53,127,5,177,28,199,97,112,112,80,114,136,125,
110,237,22,66,231,206,211,155,215,205,200,186,110,227,189,196,240,240,48,198,199,199,37,237,136,189,123,247,7,56,115,118,130,222,77,71,37,207,196,16,155,150,150,134,145,145,17,140,142,142,74,6,177,15,
30,60,196,195,135,111,97,121,121,25,193,96,16,29,29,29,209,139,138,29,214,21,137,68,48,50,50,146,20,136,125,253,245,215,177,186,186,138,150,150,150,205,171,9,210,211,211,215,79,159,62,77,247,222,82,65,
108,188,34,228,13,107,136,77,196,166,165,165,253,237,224,193,131,24,26,26,138,25,239,154,72,136,13,135,195,184,115,231,142,40,182,187,241,74,253,53,6,98,149,74,229,47,200,140,246,214,214,86,58,20,87,10,
136,61,124,248,48,142,31,63,142,193,193,65,170,195,42,149,234,87,155,38,98,211,211,211,145,147,147,131,190,190,62,201,32,182,166,166,6,102,179,89,116,130,123,75,136,77,73,73,89,55,153,76,208,235,245,104,
104,104,16,97,108,34,32,182,186,186,58,218,61,183,55,172,75,38,147,129,88,84,103,103,39,29,169,177,87,136,61,120,240,96,244,208,183,157,65,44,195,48,200,200,200,64,122,122,58,252,126,63,58,59,59,247,4,
177,110,183,59,122,128,224,222,32,54,59,59,27,157,157,157,180,150,96,167,16,91,83,83,35,108,139,79,12,196,90,44,22,148,150,150,226,240,225,195,59,94,144,199,227,137,126,120,157,24,136,85,40,20,235,153,
153,153,200,207,207,71,77,77,13,170,171,171,81,82,82,130,178,178,50,216,237,118,20,20,20,192,110,183,163,176,176,16,101,101,101,40,42,42,130,203,229,66,121,121,57,178,178,178,68,220,152,80,136,149,203,
229,159,164,166,166,82,234,33,17,92,33,184,146,48,42,249,119,10,133,66,52,34,93,18,136,85,171,213,191,226,56,238,175,114,185,124,157,164,95,201,95,133,249,96,146,23,78,73,73,89,103,89,246,111,42,149,234,
23,207,100,88,215,55,16,251,79,3,177,28,199,173,126,109,32,118,227,108,205,227,3,7,14,172,187,221,110,56,28,14,164,167,167,211,175,14,2,98,228,105,213,6,39,126,196,48,204,67,201,32,86,173,86,227,216,177,
99,136,68,34,8,133,66,24,26,26,162,16,210,215,215,71,47,154,78,167,83,116,45,179,217,108,72,77,77,253,59,203,178,111,38,12,98,73,120,176,163,163,3,183,110,221,194,248,248,184,40,209,72,64,36,16,8,224,
248,241,227,20,99,79,157,58,133,96,48,136,96,48,72,30,100,39,14,98,121,158,199,196,196,4,230,230,230,68,189,140,59,113,79,183,219,13,139,197,66,142,248,238,30,98,89,150,133,205,102,67,48,24,164,83,202,
133,58,188,221,5,145,71,189,165,165,165,66,33,222,29,196,238,223,191,31,11,11,11,184,114,229,10,206,156,57,179,231,97,93,94,175,23,54,155,109,119,16,155,154,154,138,222,222,94,204,205,205,225,234,213,
171,56,113,226,132,8,210,118,11,177,109,109,109,244,8,157,224,240,234,214,16,203,113,220,155,38,147,105,125,122,122,26,171,171,171,56,119,238,28,157,103,148,40,136,37,167,255,83,83,83,105,28,124,75,136,
181,219,237,255,179,184,184,72,43,52,133,175,78,34,33,182,185,185,25,149,149,149,200,200,200,32,175,88,124,136,101,24,230,161,221,110,199,205,155,55,113,234,212,169,152,22,221,68,67,108,99,99,35,138,138,
138,96,52,26,201,14,35,126,53,65,81,81,17,86,86,86,98,130,240,82,65,44,201,43,232,116,58,200,229,242,199,155,118,196,46,46,46,38,45,17,219,218,218,138,186,186,58,58,161,78,163,209,172,198,12,235,42,42,
42,194,194,194,66,82,33,182,169,169,137,126,119,198,64,108,74,74,10,202,203,203,49,55,55,151,84,136,245,122,189,194,39,91,98,136,149,201,100,240,120,60,184,114,229,74,82,33,182,174,174,110,115,136,229,
121,30,189,189,189,184,112,225,66,82,33,86,232,232,49,16,43,147,201,16,10,133,232,33,156,100,65,108,97,97,225,230,16,155,153,153,73,37,47,153,16,155,151,151,183,57,196,146,135,142,193,96,80,82,136,125,
241,206,247,112,122,236,44,189,155,22,36,247,99,33,214,225,112,96,121,121,25,3,3,3,146,14,235,250,241,127,252,167,104,88,87,122,122,250,230,16,235,112,56,48,53,53,133,35,71,142,72,10,177,247,239,63,192,
216,216,24,122,122,122,224,245,122,69,18,35,130,88,142,227,86,179,179,179,209,212,212,132,227,199,143,39,5,98,159,127,254,121,28,63,126,92,8,104,177,16,203,113,220,227,172,172,44,184,92,46,244,244,244,
136,54,105,137,130,216,232,86,137,7,15,30,8,43,84,226,87,19,144,201,226,228,204,94,162,33,118,98,98,2,171,171,171,116,92,222,189,123,247,232,3,35,149,74,245,112,83,136,37,67,227,61,30,143,104,4,85,162,
32,150,84,8,95,187,118,13,141,141,141,100,91,252,63,91,66,44,9,5,90,173,86,81,2,35,81,16,91,87,87,7,187,221,46,172,39,88,215,233,116,111,110,11,98,211,210,210,96,179,217,208,213,213,133,174,174,174,132,
64,172,203,229,18,166,170,119,7,177,38,147,9,78,167,19,3,3,3,123,238,136,45,43,43,139,206,12,239,30,98,115,114,114,54,222,186,46,209,130,182,235,158,100,106,88,156,41,80,187,135,88,157,78,7,147,201,68,
135,142,52,53,53,209,158,31,50,71,123,43,136,45,44,44,140,183,160,189,67,172,193,96,88,55,153,76,72,77,77,133,201,100,66,70,70,6,50,51,51,145,153,153,137,188,188,60,228,231,231,227,224,193,131,112,58,
157,112,187,221,104,104,104,64,109,109,45,28,14,71,116,94,248,203,132,65,172,66,161,120,211,96,48,252,143,90,173,134,90,173,134,74,165,138,249,225,56,14,42,149,10,60,207,35,45,45,77,120,95,39,45,196,202,
229,242,135,164,35,86,152,126,37,127,56,25,218,69,22,185,241,239,31,63,139,97,93,155,66,172,197,98,89,253,6,98,191,150,137,88,149,74,245,142,70,163,249,173,92,46,255,240,153,67,44,233,136,245,249,124,
232,235,235,67,119,119,55,14,28,56,0,139,197,2,165,82,25,221,202,252,101,82,134,117,89,44,22,4,131,65,209,20,40,178,133,30,24,24,128,203,229,66,73,73,9,172,86,43,82,83,83,193,113,28,153,139,245,88,46,
151,255,44,161,16,171,80,40,80,88,88,136,80,40,132,229,229,101,81,206,138,84,17,144,187,160,129,129,1,12,13,13,161,175,175,15,193,96,144,222,158,103,103,103,175,43,149,202,247,246,12,177,114,185,28,26,
141,6,117,117,117,88,91,91,67,36,18,17,85,33,70,187,167,208,60,123,123,123,113,248,240,97,244,244,244,160,175,175,79,56,26,118,215,16,251,39,153,76,6,187,221,142,80,40,132,171,87,175,98,122,122,90,36,
50,59,129,216,174,174,46,52,52,52,32,35,35,131,92,108,255,180,99,136,181,217,108,8,4,2,184,124,249,50,174,94,189,138,145,145,17,145,12,239,6,98,219,218,218,80,86,86,22,93,230,240,116,136,77,79,79,127,
207,231,243,33,18,137,208,35,229,66,247,220,235,176,46,175,215,75,61,125,227,107,104,107,136,77,73,73,249,89,89,89,25,238,220,185,131,112,56,28,211,162,155,8,136,109,107,107,131,211,233,36,122,247,116,
136,101,24,230,113,111,111,47,61,185,45,37,196,110,204,90,163,229,70,155,118,196,234,245,122,132,66,33,58,191,93,42,136,37,179,253,108,54,27,245,244,77,33,86,173,86,227,234,213,171,73,171,38,32,137,142,
141,231,53,177,213,4,50,153,236,75,155,205,134,229,229,101,12,13,13,37,5,98,155,154,154,80,94,94,78,110,40,98,135,117,145,105,228,55,111,222,164,191,109,201,128,88,143,199,131,204,204,76,114,124,234,29,
17,196,202,100,50,148,148,148,96,113,113,49,169,16,219,210,210,130,130,130,2,242,185,250,173,8,98,89,150,197,145,35,71,112,245,234,85,90,201,154,12,136,109,105,105,17,166,60,62,20,65,172,86,171,197,220,
220,28,66,161,80,82,33,214,227,241,8,111,82,197,16,107,183,219,177,180,180,132,96,48,152,84,136,141,106,105,22,67,172,211,233,196,234,234,170,164,195,186,226,65,108,84,206,243,43,136,101,89,246,195,190,
190,62,68,34,17,209,183,189,212,16,75,190,7,227,66,172,78,167,251,109,85,85,21,66,161,144,104,79,36,5,196,254,232,223,95,19,65,108,212,1,215,175,32,150,227,184,119,242,242,242,224,247,251,233,160,46,41,
32,246,209,163,95,224,222,189,31,137,134,131,11,143,115,198,29,214,101,179,217,208,222,222,142,190,190,62,201,32,246,165,151,94,194,177,99,199,208,216,216,136,234,234,106,209,77,70,220,106,130,180,180,
52,88,173,86,28,59,118,76,18,136,37,63,223,255,254,247,113,250,244,233,232,254,141,205,135,117,153,205,102,148,149,149,209,206,132,68,65,236,155,111,190,25,243,247,135,15,31,222,94,71,44,203,178,200,206,
206,134,223,239,143,154,141,181,55,136,157,156,156,68,56,28,198,79,126,242,19,250,207,123,123,123,41,130,60,21,98,121,158,135,213,106,165,179,141,19,5,177,62,159,15,39,78,156,192,210,210,18,102,102,102,
168,161,235,116,186,159,61,21,98,25,134,129,197,98,129,199,227,193,192,192,64,194,170,9,234,234,234,144,159,159,15,147,201,68,249,136,101,217,247,118,84,77,144,155,155,139,254,254,126,122,39,178,91,136,
245,120,60,40,43,43,67,90,90,90,116,181,230,142,33,246,79,38,147,9,165,165,165,8,4,2,49,11,218,9,196,230,229,229,9,11,255,133,87,239,221,65,108,86,86,22,28,14,7,74,74,74,80,89,89,9,143,199,131,218,218,
90,180,182,182,194,235,245,110,185,160,170,170,42,81,162,58,97,29,177,58,157,238,61,163,209,8,158,231,161,86,171,193,178,44,120,158,135,94,175,135,217,108,134,195,225,128,219,237,70,77,77,13,90,90,90,
104,79,66,105,105,105,244,87,72,98,33,150,101,217,159,201,100,178,199,228,212,63,193,87,158,231,161,209,104,160,80,40,160,211,233,96,181,90,81,84,84,132,172,172,172,120,111,151,180,195,186,132,51,140,
24,134,65,74,74,10,93,164,74,165,18,198,144,146,63,172,43,30,196,50,12,243,33,203,178,191,101,89,246,157,111,18,177,95,251,106,2,163,209,248,23,189,94,255,197,198,231,235,243,103,10,177,60,207,195,231,
243,97,120,120,24,71,142,28,65,67,67,3,76,38,83,76,229,78,82,32,86,38,147,161,172,172,12,39,79,158,196,242,242,50,38,38,38,232,29,80,119,119,55,252,126,63,74,74,74,200,124,34,225,165,225,113,66,171,9,
200,37,192,98,177,160,171,171,139,30,158,38,165,145,253,253,253,49,222,217,209,209,129,238,238,110,56,157,78,20,22,22,98,223,190,125,80,171,213,235,44,203,254,62,33,137,88,139,197,2,191,223,143,169,169,
41,44,44,44,96,98,98,130,82,117,60,247,236,238,238,22,45,144,108,175,123,122,122,224,112,56,160,86,171,63,221,21,196,202,229,242,15,236,118,59,186,187,187,17,14,135,177,176,176,128,201,201,73,145,200,
108,23,98,201,115,232,142,142,14,212,214,214,66,171,213,130,97,152,15,118,4,177,185,185,185,159,14,14,14,34,18,137,208,91,249,232,174,132,221,64,44,49,79,193,236,181,237,65,44,199,113,191,239,233,233,
193,218,218,26,253,16,39,106,88,23,49,207,226,226,98,225,3,202,167,87,19,24,141,198,245,197,197,69,132,195,225,24,247,76,212,176,46,143,199,179,125,136,149,201,100,143,45,22,75,76,63,108,162,33,214,239,
247,71,63,95,222,122,88,87,81,81,17,110,223,190,29,147,25,78,116,34,182,182,182,22,58,157,142,46,108,83,136,85,42,149,232,238,238,198,181,107,215,36,135,88,159,207,135,162,162,34,225,211,135,248,16,171,
209,104,16,137,68,112,254,252,249,164,64,108,93,93,157,240,206,102,115,136,189,126,253,58,78,157,58,149,180,142,216,138,138,10,138,252,49,137,88,153,76,134,150,150,22,172,172,172,96,112,112,48,105,16,
75,144,127,227,213,18,39,98,77,38,19,46,93,186,132,112,56,156,212,68,108,75,75,11,213,60,134,97,254,66,33,150,97,152,207,115,115,115,241,220,115,207,37,189,35,182,185,185,89,72,140,95,136,32,54,24,12,
226,218,181,107,56,122,244,104,82,33,150,124,237,196,64,172,217,108,254,98,104,104,136,242,98,50,33,214,229,114,9,111,82,63,167,16,155,154,154,250,151,154,154,26,218,164,155,204,68,108,84,95,130,56,17,
155,151,151,135,246,246,118,145,121,74,13,177,110,183,59,122,251,44,78,196,170,213,106,228,228,228,208,240,169,84,195,186,94,121,229,135,212,27,162,120,113,115,136,173,171,171,195,224,224,160,100,16,123,
254,252,121,10,32,209,147,238,227,66,172,208,164,200,182,54,145,16,251,232,209,35,140,143,143,163,173,173,13,94,175,55,58,97,189,57,196,146,196,226,241,227,199,19,154,136,37,63,111,188,241,6,194,225,48,
14,29,58,36,74,162,109,9,177,228,225,17,97,235,68,64,236,253,251,247,241,218,107,175,137,22,23,10,133,132,97,157,167,67,44,41,175,61,118,236,88,66,58,98,123,123,123,49,52,52,36,106,148,136,68,34,228,88,
221,246,171,9,242,242,242,232,91,152,40,136,13,4,2,152,159,159,199,218,218,26,234,235,235,201,111,220,239,183,13,177,153,153,153,240,122,189,24,24,24,160,11,218,75,34,182,166,166,6,121,121,121,48,153,
76,194,131,135,159,238,8,98,229,114,249,7,54,155,13,78,167,147,146,33,89,204,78,32,150,20,171,241,60,79,246,76,100,65,31,236,10,98,211,211,211,63,53,26,141,80,42,149,224,121,30,89,89,89,40,40,40,128,219,
237,134,219,237,166,19,6,226,45,168,190,190,30,14,135,3,26,141,134,46,36,97,16,203,178,236,239,57,142,91,87,42,149,208,106,181,164,232,17,122,189,30,118,187,29,46,151,11,94,175,87,180,160,202,202,74,145,
153,11,22,243,101,162,59,98,31,147,160,105,74,74,10,120,158,39,104,1,163,209,136,146,146,18,148,150,150,34,55,55,55,94,86,56,185,16,43,84,98,165,82,25,147,21,126,102,16,187,33,120,224,56,238,11,150,101,
255,242,77,53,193,86,16,43,151,203,253,95,171,142,88,189,94,255,196,96,48,128,231,249,39,207,20,98,101,50,25,82,83,83,81,85,85,133,241,241,113,156,58,117,10,29,29,29,176,217,108,194,103,50,201,129,88,
134,97,192,243,60,26,26,26,48,60,60,140,235,215,175,99,114,114,146,238,88,59,59,59,81,83,83,3,187,221,46,204,164,39,126,88,215,198,44,24,24,12,6,248,124,62,144,97,57,225,112,24,195,195,195,34,102,36,231,
108,136,172,20,23,23,131,231,121,114,158,125,239,213,4,41,41,41,239,219,108,54,184,92,46,12,15,15,99,102,102,6,145,72,4,147,147,147,212,27,54,115,207,232,81,160,141,141,141,200,203,203,35,135,161,223,
223,21,196,230,231,231,255,125,112,112,16,23,47,94,196,220,220,28,194,225,48,70,70,70,68,149,154,59,129,88,66,216,132,128,120,158,255,251,142,32,86,161,80,188,31,8,4,176,180,180,68,195,167,209,238,185,
219,68,172,223,239,167,25,98,134,97,222,223,118,53,65,78,78,14,110,221,186,133,115,231,206,197,184,231,94,33,182,163,163,3,77,77,77,132,173,183,7,177,12,195,172,87,84,84,224,91,223,250,86,204,43,148,40,
136,61,116,232,144,240,92,223,246,134,117,157,59,119,46,102,200,82,162,59,98,203,203,203,133,219,154,173,33,54,45,45,45,38,4,47,5,196,54,52,52,8,71,186,108,14,177,12,195,160,177,177,17,171,171,171,34,
29,150,10,98,201,244,148,45,33,86,171,213,226,194,133,11,152,153,153,73,10,196,186,92,46,97,73,72,124,136,173,170,170,194,173,91,183,232,113,1,169,33,182,177,177,17,86,171,85,56,68,64,12,177,25,25,25,
152,153,153,193,236,236,108,210,59,98,5,223,145,98,136,173,172,172,196,237,219,183,69,223,101,201,128,216,134,134,6,100,101,101,197,14,235,50,26,141,79,166,167,167,113,241,226,69,209,87,71,178,33,150,
97,152,39,50,153,172,81,38,151,203,253,25,25,25,152,152,152,192,232,232,168,232,187,44,25,16,219,212,212,132,138,138,10,225,165,193,47,147,201,100,141,122,189,254,73,89,89,153,104,122,88,178,32,182,169,
169,73,120,105,120,34,170,38,200,205,205,165,117,77,201,132,216,170,170,42,225,19,121,49,196,146,27,202,161,161,161,164,38,98,5,223,129,241,135,117,229,229,229,161,191,191,159,30,248,146,26,98,171,170,
170,158,158,136,229,121,30,78,167,19,193,96,80,50,136,157,154,154,162,60,20,245,42,197,135,88,210,57,53,60,60,44,9,196,146,97,93,36,55,44,232,115,217,26,98,141,70,35,29,37,156,104,136,189,113,227,6,250,
250,250,232,72,117,33,128,108,9,177,60,207,163,184,184,24,167,79,159,78,88,71,172,240,103,105,105,9,125,125,125,200,204,204,220,254,176,46,134,97,214,237,118,59,78,159,62,45,170,37,216,11,196,222,184,
113,67,180,176,72,36,34,124,252,177,61,136,37,217,150,254,254,254,152,91,165,221,64,108,95,95,31,150,150,150,68,111,163,90,173,222,89,53,65,74,74,202,251,217,217,217,168,169,169,129,207,231,219,115,34,
214,235,245,130,28,87,88,91,91,67,91,91,27,121,149,222,223,17,196,154,205,230,191,27,141,70,24,12,6,100,101,101,81,144,245,249,124,219,134,88,162,194,37,37,37,200,204,204,132,193,96,16,110,129,255,190,
43,136,149,203,229,239,107,52,26,232,116,58,26,62,37,206,73,16,54,222,130,188,94,47,156,78,39,108,54,27,133,88,225,143,76,38,123,127,207,213,4,12,195,172,147,186,213,125,251,246,65,171,213,34,55,55,23,
85,85,85,180,67,65,120,149,206,204,204,164,219,220,168,197,36,126,88,23,185,37,82,40,20,208,235,245,208,233,116,32,93,48,149,149,149,116,28,122,28,21,78,30,196,170,213,106,236,219,183,79,84,69,16,5,248,
207,14,98,149,74,229,147,141,230,220,39,95,171,142,216,141,29,99,66,32,246,255,13,0,178,128,2,218,17,75,66,59,0,0,0,0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00135_png = (const char*) resource_TalComponent_bmp00135_png;
const int TalComponent::bmp00135_pngSize = 53534;











// JUCER_RESOURCE: bmp00135_png, 53534, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00135.png"
static const unsigned char resource_TalComponent_bmp00136_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,70,0,0,0,36,8,6,0,0,0,52,18,76,213,0,0,0,9,112,72,89,
  115,0,0,15,97,0,0,15,97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,4,92,73,68,65,84,120,218,236,
  152,223,78,27,71,20,198,127,97,237,178,10,44,152,63,134,29,92,168,163,74,109,239,202,35,36,79,0,149,42,85,189,106,243,6,241,19,64,158,128,
  228,9,66,174,16,18,18,201,19,224,60,65,125,223,86,114,105,147,25,255,137,187,150,187,171,37,102,77,47,152,89,141,205,54,34,8,212,155,253,164,
  149,173,111,102,206,156,249,230,156,179,71,123,239,224,240,232,130,28,87,80,208,191,53,160,145,203,145,226,196,8,211,248,241,135,239,235,185,30,151,56,
  56,60,98,42,151,33,27,185,48,185,48,55,43,190,153,144,82,237,2,117,33,252,186,197,109,2,47,128,192,20,110,33,252,134,30,219,214,99,15,132,
  240,3,205,157,88,38,75,192,99,51,223,218,99,75,219,43,1,79,133,240,95,73,169,254,182,94,8,37,224,59,33,252,230,4,143,16,254,163,140,125,
  106,66,248,141,9,206,188,100,126,2,54,245,211,176,109,92,91,152,255,192,11,224,145,16,126,32,165,42,1,199,128,49,188,5,60,7,182,129,253,12,
  231,55,129,61,107,190,125,144,186,101,239,21,208,176,214,109,3,63,3,187,54,111,137,251,80,243,53,109,99,231,35,135,54,151,120,146,37,200,141,132,
  209,14,188,54,209,160,197,121,99,28,211,55,91,211,226,237,79,174,215,183,152,101,122,83,243,155,64,83,115,85,29,77,70,240,90,6,223,20,194,223,
  215,123,239,72,169,246,128,55,66,248,181,137,136,52,251,239,254,31,53,102,91,11,243,68,31,180,250,9,107,191,5,30,90,225,142,78,173,186,30,123,
  106,165,179,225,235,86,84,4,250,246,95,234,189,237,20,170,91,207,237,212,152,140,27,175,75,169,118,164,84,37,43,149,182,132,240,119,165,84,199,192,
  83,203,249,109,224,217,68,196,85,173,218,100,227,165,93,199,140,13,189,95,3,56,145,82,213,117,164,6,147,115,165,84,79,180,127,207,128,134,148,234,
  23,219,231,91,47,190,26,123,82,170,192,202,215,26,112,108,165,196,99,45,144,237,68,93,223,218,179,172,194,248,137,151,17,72,169,158,235,72,220,205,
  136,136,154,78,219,23,82,170,45,205,61,183,68,59,153,184,128,253,235,118,121,23,7,135,71,15,243,23,244,184,38,121,31,147,55,120,185,48,183,218,
  249,110,30,28,30,229,106,100,8,179,151,75,145,167,210,245,35,102,118,102,134,66,161,144,171,97,58,203,126,255,82,152,66,161,64,177,88,204,21,201,
  83,41,23,38,23,230,174,94,215,153,88,41,47,19,70,17,97,24,165,156,235,186,84,214,4,201,40,1,64,169,54,113,28,3,224,121,30,149,53,193,
  175,191,253,78,146,92,142,87,171,27,233,90,103,202,225,237,59,153,206,55,123,120,158,71,50,74,112,166,28,218,157,46,131,193,128,111,190,254,138,248,
  44,78,215,157,254,249,23,195,225,112,140,7,104,54,79,175,236,99,124,178,57,195,151,74,243,184,238,52,238,180,155,218,49,54,110,252,217,1,160,178,
  38,104,254,113,74,146,36,56,142,195,250,122,37,53,60,231,205,210,235,245,240,188,89,130,160,127,197,121,215,117,241,253,149,43,142,168,86,139,48,140,
  82,123,131,193,128,248,44,78,231,121,158,199,66,105,158,118,167,59,198,27,204,204,220,39,142,207,80,170,133,227,56,148,203,203,40,149,125,104,195,87,
  171,27,153,130,220,72,152,153,153,251,12,6,131,52,26,146,36,33,10,163,212,49,199,113,80,173,54,149,53,49,38,140,129,29,41,54,220,105,55,253,
  29,126,24,2,240,89,177,200,74,121,57,21,70,181,90,87,248,15,195,33,65,208,39,142,207,40,151,151,241,253,85,194,48,66,169,214,88,68,26,180,
  59,221,187,249,80,245,49,120,222,44,83,206,20,75,139,11,184,238,52,197,98,145,225,112,120,173,181,174,59,141,227,76,145,36,35,84,171,173,69,31,
  17,70,17,75,139,139,180,59,221,52,157,13,111,254,155,11,106,54,79,113,93,151,57,111,150,165,165,133,52,26,204,220,187,248,80,149,34,12,35,202,
  229,101,156,158,147,166,146,231,121,180,59,93,54,214,63,167,163,111,36,73,70,204,121,30,239,123,189,177,245,197,98,145,145,62,204,100,67,101,215,49,
  128,100,148,16,134,17,113,124,70,245,139,13,162,40,34,73,146,148,183,177,180,184,8,192,251,94,143,56,142,249,210,123,48,230,243,157,8,227,175,174,
  166,133,182,217,60,69,169,54,235,235,149,116,252,237,59,137,227,56,99,78,132,97,68,181,186,145,10,51,89,24,63,5,73,146,240,190,215,99,105,113,
  129,118,167,139,59,237,94,177,23,244,251,84,214,4,222,220,108,42,144,129,61,55,8,250,153,41,158,133,123,7,135,71,23,165,249,249,188,243,181,208,
  233,118,243,62,38,111,240,114,97,110,177,243,61,63,63,207,149,200,18,230,159,48,204,149,152,192,191,3,0,62,13,80,107,146,81,62,232,0,0,0,
  0,73,69,78,68,174,66,96,130,0,0};

const char* TalComponent::bmp00136_png = (const char*) resource_TalComponent_bmp00136_png;
const int TalComponent::bmp00136_pngSize = 3889;



// JUCER_RESOURCE: bmp00135_png, 53534, "E:/Code/tal/tal-elek7ro-juce/resources/bmp00135.png"
static const unsigned char resource_TalComponent_bmp00137_png[] = { 137,80,78,71,13,10,26,10,0,0,0,13,73,72,68,82,0,0,0,70,0,0,0,36,8,6,0,0,0,52,18,76,213,0,0,0,9,112,72,89,
  115,0,0,15,97,0,0,15,97,1,168,63,167,105,0,0,10,79,105,67,67,80,80,104,111,116,111,115,104,111,112,32,73,67,67,32,112,114,111,102,
  105,108,101,0,0,120,218,157,83,103,84,83,233,22,61,247,222,244,66,75,136,128,148,75,111,82,21,8,32,82,66,139,128,20,145,38,42,33,9,16,
  74,136,33,161,217,21,81,193,17,69,69,4,27,200,160,136,3,142,142,128,140,21,81,44,12,138,10,216,7,228,33,162,142,131,163,136,138,202,251,225,
  123,163,107,214,188,247,230,205,254,181,215,62,231,172,243,157,179,207,7,192,8,12,150,72,51,81,53,128,12,169,66,30,17,224,131,199,196,198,225,228,
  46,64,129,10,36,112,0,16,8,179,100,33,115,253,35,1,0,248,126,60,60,43,34,192,7,190,0,1,120,211,11,8,0,192,77,155,192,48,28,135,
  255,15,234,66,153,92,1,128,132,1,192,116,145,56,75,8,128,20,0,64,122,142,66,166,0,64,70,1,128,157,152,38,83,0,160,4,0,96,203,99,
  98,227,0,80,45,0,96,39,127,230,211,0,128,157,248,153,123,1,0,91,148,33,21,1,160,145,0,32,19,101,136,68,0,104,59,0,172,207,86,138,
  69,0,88,48,0,20,102,75,196,57,0,216,45,0,48,73,87,102,72,0,176,183,0,192,206,16,11,178,0,8,12,0,48,81,136,133,41,0,4,123,
  0,96,200,35,35,120,0,132,153,0,20,70,242,87,60,241,43,174,16,231,42,0,0,120,153,178,60,185,36,57,69,129,91,8,45,113,7,87,87,46,
  30,40,206,73,23,43,20,54,97,2,97,154,64,46,194,121,153,25,50,129,52,15,224,243,204,0,0,160,145,21,17,224,131,243,253,120,206,14,174,206,
  206,54,142,182,14,95,45,234,191,6,255,34,98,98,227,254,229,207,171,112,64,0,0,225,116,126,209,254,44,47,179,26,128,59,6,128,109,254,162,37,
  238,4,104,94,11,160,117,247,139,102,178,15,64,181,0,160,233,218,87,243,112,248,126,60,60,69,161,144,185,217,217,229,228,228,216,74,196,66,91,97,
  202,87,125,254,103,194,95,192,87,253,108,249,126,60,252,247,245,224,190,226,36,129,50,93,129,71,4,248,224,194,204,244,76,165,28,207,146,9,132,98,
  220,230,143,71,252,183,11,255,252,29,211,34,196,73,98,185,88,42,20,227,81,18,113,142,68,154,140,243,50,165,34,137,66,146,41,197,37,210,255,100,
  226,223,44,251,3,62,223,53,0,176,106,62,1,123,145,45,168,93,99,3,246,75,39,16,88,116,192,226,247,0,0,242,187,111,193,212,40,8,3,128,
  104,131,225,207,119,255,239,63,253,71,160,37,0,128,102,73,146,113,0,0,94,68,36,46,84,202,179,63,199,8,0,0,68,160,129,42,176,65,27,244,
  193,24,44,192,6,28,193,5,220,193,11,252,96,54,132,66,36,196,194,66,16,66,10,100,128,28,114,96,41,172,130,66,40,134,205,176,29,42,96,47,
  212,64,29,52,192,81,104,134,147,112,14,46,194,85,184,14,61,112,15,250,97,8,158,193,40,188,129,9,4,65,200,8,19,97,33,218,136,1,98,138,
  88,35,142,8,23,153,133,248,33,193,72,4,18,139,36,32,201,136,20,81,34,75,145,53,72,49,82,138,84,32,85,72,29,242,61,114,2,57,135,92,
  70,186,145,59,200,0,50,130,252,134,188,71,49,148,129,178,81,61,212,12,181,67,185,168,55,26,132,70,162,11,208,100,116,49,154,143,22,160,155,208,
  114,180,26,61,140,54,161,231,208,171,104,15,218,143,62,67,199,48,192,232,24,7,51,196,108,48,46,198,195,66,177,56,44,9,147,99,203,177,34,172,
  12,171,198,26,176,86,172,3,187,137,245,99,207,177,119,4,18,129,69,192,9,54,4,119,66,32,97,30,65,72,88,76,88,78,216,72,168,32,28,36,
  52,17,218,9,55,9,3,132,81,194,39,34,147,168,75,180,38,186,17,249,196,24,98,50,49,135,88,72,44,35,214,18,143,19,47,16,123,136,67,196,
  55,36,18,137,67,50,39,185,144,2,73,177,164,84,210,18,210,70,210,110,82,35,233,44,169,155,52,72,26,35,147,201,218,100,107,178,7,57,148,44,
  32,43,200,133,228,157,228,195,228,51,228,27,228,33,242,91,10,157,98,64,113,164,248,83,226,40,82,202,106,74,25,229,16,229,52,229,6,101,152,50,
  65,85,163,154,82,221,168,161,84,17,53,143,90,66,173,161,182,82,175,81,135,168,19,52,117,154,57,205,131,22,73,75,165,173,162,149,211,26,104,23,
  104,247,105,175,232,116,186,17,221,149,30,78,151,208,87,210,203,233,71,232,151,232,3,244,119,12,13,134,21,131,199,136,103,40,25,155,24,7,24,103,
  25,119,24,175,152,76,166,25,211,139,25,199,84,48,55,49,235,152,231,153,15,153,111,85,88,42,182,42,124,21,145,202,10,149,74,149,38,149,27,42,
  47,84,169,170,166,170,222,170,11,85,243,85,203,84,143,169,94,83,125,174,70,85,51,83,227,169,9,212,150,171,85,170,157,80,235,83,27,83,103,169,
  59,168,135,170,103,168,111,84,63,164,126,89,253,137,6,89,195,76,195,79,67,164,81,160,177,95,227,188,198,32,11,99,25,179,120,44,33,107,13,171,
  134,117,129,53,196,38,177,205,217,124,118,42,187,152,253,29,187,139,61,170,169,161,57,67,51,74,51,87,179,82,243,148,102,63,7,227,152,113,248,156,
  116,78,9,231,40,167,151,243,126,138,222,20,239,41,226,41,27,166,52,76,185,49,101,92,107,170,150,151,150,88,171,72,171,81,171,71,235,189,54,174,
  237,167,157,166,189,69,187,89,251,129,14,65,199,74,39,92,39,71,103,143,206,5,157,231,83,217,83,221,167,10,167,22,77,61,58,245,174,46,170,107,
  165,27,161,187,68,119,191,110,167,238,152,158,190,94,128,158,76,111,167,222,121,189,231,250,28,125,47,253,84,253,109,250,167,245,71,12,88,6,179,12,
  36,6,219,12,206,24,60,197,53,113,111,60,29,47,199,219,241,81,67,93,195,64,67,165,97,149,97,151,225,132,145,185,209,60,163,213,70,141,70,15,
  140,105,198,92,227,36,227,109,198,109,198,163,38,6,38,33,38,75,77,234,77,238,154,82,77,185,166,41,166,59,76,59,76,199,205,204,205,162,205,214,
  153,53,155,61,49,215,50,231,155,231,155,215,155,223,183,96,90,120,90,44,182,168,182,184,101,73,178,228,90,166,89,238,182,188,110,133,90,57,89,165,
  88,85,90,93,179,70,173,157,173,37,214,187,173,187,167,17,167,185,78,147,78,171,158,214,103,195,176,241,182,201,182,169,183,25,176,229,216,6,219,174,
  182,109,182,125,97,103,98,23,103,183,197,174,195,238,147,189,147,125,186,125,141,253,61,7,13,135,217,14,171,29,90,29,126,115,180,114,20,58,86,58,
  222,154,206,156,238,63,125,197,244,150,233,47,103,88,207,16,207,216,51,227,182,19,203,41,196,105,157,83,155,211,71,103,23,103,185,115,131,243,136,139,
  137,75,130,203,46,151,62,46,155,27,198,221,200,189,228,74,116,245,113,93,225,122,210,245,157,155,179,155,194,237,168,219,175,238,54,238,105,238,135,220,
  159,204,52,159,41,158,89,51,115,208,195,200,67,224,81,229,209,63,11,159,149,48,107,223,172,126,79,67,79,129,103,181,231,35,47,99,47,145,87,173,
  215,176,183,165,119,170,247,97,239,23,62,246,62,114,159,227,62,227,60,55,222,50,222,89,95,204,55,192,183,200,183,203,79,195,111,158,95,133,223,67,
  127,35,255,100,255,122,255,209,0,167,128,37,1,103,3,137,129,65,129,91,2,251,248,122,124,33,191,142,63,58,219,101,246,178,217,237,65,140,160,185,
  65,21,65,143,130,173,130,229,193,173,33,104,200,236,144,173,33,247,231,152,206,145,206,105,14,133,80,126,232,214,208,7,97,230,97,139,195,126,12,39,
  133,135,133,87,134,63,142,112,136,88,26,209,49,151,53,119,209,220,67,115,223,68,250,68,150,68,222,155,103,49,79,57,175,45,74,53,42,62,170,46,
  106,60,218,55,186,52,186,63,198,46,102,89,204,213,88,157,88,73,108,75,28,57,46,42,174,54,110,108,190,223,252,237,243,135,226,157,226,11,227,123,
  23,152,47,200,93,112,121,161,206,194,244,133,167,22,169,46,18,44,58,150,64,76,136,78,56,148,240,65,16,42,168,22,140,37,242,19,119,37,142,10,
  121,194,29,194,103,34,47,209,54,209,136,216,67,92,42,30,78,242,72,42,77,122,146,236,145,188,53,121,36,197,51,165,44,229,185,132,39,169,144,188,
  76,13,76,221,155,58,158,22,154,118,32,109,50,61,58,189,49,131,146,145,144,113,66,170,33,77,147,182,103,234,103,230,102,118,203,172,101,133,178,254,
  197,110,139,183,47,30,149,7,201,107,179,144,172,5,89,45,10,182,66,166,232,84,90,40,215,42,7,178,103,101,87,102,191,205,137,202,57,150,171,158,
  43,205,237,204,179,202,219,144,55,156,239,159,255,237,18,194,18,225,146,182,165,134,75,87,45,29,88,230,189,172,106,57,178,60,113,121,219,10,227,21,
  5,43,134,86,6,172,60,184,138,182,42,109,213,79,171,237,87,151,174,126,189,38,122,77,107,129,94,193,202,130,193,181,1,107,235,11,85,10,229,133,
  125,235,220,215,237,93,79,88,47,89,223,181,97,250,134,157,27,62,21,137,138,174,20,219,23,151,21,127,216,40,220,120,229,27,135,111,202,191,153,220,
  148,180,169,171,196,185,100,207,102,210,102,233,230,222,45,158,91,14,150,170,151,230,151,14,110,13,217,218,180,13,223,86,180,237,245,246,69,219,47,151,
  205,40,219,187,131,182,67,185,163,191,60,184,188,101,167,201,206,205,59,63,84,164,84,244,84,250,84,54,238,210,221,181,97,215,248,110,209,238,27,123,
  188,246,52,236,213,219,91,188,247,253,62,201,190,219,85,1,85,77,213,102,213,101,251,73,251,179,247,63,174,137,170,233,248,150,251,109,93,173,78,109,
  113,237,199,3,210,3,253,7,35,14,182,215,185,212,213,29,210,61,84,82,143,214,43,235,71,14,199,31,190,254,157,239,119,45,13,54,13,85,141,156,
  198,226,35,112,68,121,228,233,247,9,223,247,30,13,58,218,118,140,123,172,225,7,211,31,118,29,103,29,47,106,66,154,242,154,70,155,83,154,251,91,
  98,91,186,79,204,62,209,214,234,222,122,252,71,219,31,15,156,52,60,89,121,74,243,84,201,105,218,233,130,211,147,103,242,207,140,157,149,157,125,126,
  46,249,220,96,219,162,182,123,231,99,206,223,106,15,111,239,186,16,116,225,210,69,255,139,231,59,188,59,206,92,242,184,116,242,178,219,229,19,87,184,
  87,154,175,58,95,109,234,116,234,60,254,147,211,79,199,187,156,187,154,174,185,92,107,185,238,122,189,181,123,102,247,233,27,158,55,206,221,244,189,121,
  241,22,255,214,213,158,57,61,221,189,243,122,111,247,197,247,245,223,22,221,126,114,39,253,206,203,187,217,119,39,238,173,188,79,188,95,244,64,237,65,
  217,67,221,135,213,63,91,254,220,216,239,220,127,106,192,119,160,243,209,220,71,247,6,133,131,207,254,145,245,143,15,67,5,143,153,143,203,134,13,134,
  235,158,56,62,57,57,226,63,114,253,233,252,167,67,207,100,207,38,158,23,254,162,254,203,174,23,22,47,126,248,213,235,215,206,209,152,209,161,151,242,
  151,147,191,109,124,165,253,234,192,235,25,175,219,198,194,198,30,190,201,120,51,49,94,244,86,251,237,193,119,220,119,29,239,163,223,15,79,228,124,32,
  127,40,255,104,249,177,245,83,208,167,251,147,25,147,147,255,4,3,152,243,252,99,51,45,219,0,0,0,32,99,72,82,77,0,0,122,37,0,0,128,
  131,0,0,249,255,0,0,128,233,0,0,117,48,0,0,234,96,0,0,58,152,0,0,23,111,146,95,197,70,0,0,4,188,73,68,65,84,120,218,236,
  153,93,78,220,86,20,199,127,177,235,225,202,30,51,174,2,212,55,4,50,93,65,233,10,50,89,65,168,84,169,234,83,218,29,36,43,8,89,65,232,
  10,2,79,8,9,169,97,5,165,43,96,178,128,182,147,164,228,90,204,0,30,141,64,6,203,147,62,204,181,227,49,6,102,42,218,151,248,72,163,225,
  158,115,63,254,231,127,62,124,61,220,217,218,222,249,72,37,151,228,11,253,253,232,199,31,190,223,171,232,24,201,214,246,206,71,163,162,161,92,42,98,
  42,98,42,98,110,181,249,94,41,74,5,107,192,99,32,4,54,165,244,55,180,126,21,120,5,124,45,165,31,42,21,236,75,233,127,155,91,247,43,240,
  12,216,7,218,169,94,74,255,81,97,255,147,162,189,112,166,7,236,74,233,175,21,244,164,120,202,48,42,21,180,52,190,142,158,219,6,54,129,151,122,
  79,79,219,50,159,166,34,6,120,156,58,172,84,240,10,72,55,121,12,252,2,172,106,221,174,82,193,170,148,254,107,165,2,15,240,164,244,59,74,5,
  237,34,25,5,201,236,74,5,47,181,67,0,207,164,244,247,180,126,31,88,43,234,39,192,184,41,165,191,86,152,251,72,159,209,42,177,77,69,76,91,
  31,246,187,206,0,82,199,245,56,5,178,1,60,7,94,107,178,54,245,250,166,142,40,64,167,36,58,94,142,140,150,38,27,224,137,214,55,114,25,146,
  215,3,108,72,233,119,202,48,106,121,152,59,123,175,132,208,127,223,99,164,244,127,6,94,232,225,190,38,101,85,19,243,20,88,81,42,104,106,128,77,
  109,127,162,9,66,59,181,167,63,237,146,35,60,77,72,75,103,67,154,250,111,244,252,86,33,227,222,228,246,11,175,193,136,46,149,116,110,231,214,122,
  140,82,193,10,240,92,74,255,59,96,67,169,224,27,96,69,151,209,139,156,227,171,192,58,176,171,201,10,165,244,211,40,135,55,68,170,115,69,74,183,
  165,244,247,148,10,86,148,10,158,74,233,175,231,245,19,96,4,120,59,77,150,76,76,140,148,126,91,169,160,163,84,240,91,174,129,181,181,45,61,112,
  79,219,215,117,150,236,23,210,121,37,183,62,237,17,237,73,1,234,166,187,175,84,144,150,224,75,165,130,48,71,210,179,34,70,77,104,75,151,221,195,
  171,26,255,141,215,223,173,237,157,86,245,128,174,94,9,170,11,94,69,204,127,120,243,125,82,245,153,114,98,126,170,168,168,74,105,242,140,241,26,13,
  44,203,170,216,208,210,237,245,170,140,169,74,169,34,166,34,230,127,121,92,95,41,174,235,114,247,238,151,12,147,33,135,221,30,81,20,1,32,132,192,
  247,23,232,116,222,1,176,120,79,114,240,65,101,235,22,230,231,56,9,251,44,46,202,241,87,105,61,63,149,102,115,57,251,251,236,244,140,195,110,15,
  207,107,224,121,141,76,127,116,116,194,96,48,184,164,15,195,62,97,216,47,197,152,226,75,37,138,206,9,195,254,152,46,191,199,212,196,44,204,207,209,
  121,59,114,166,249,96,153,63,254,252,107,244,35,138,215,32,190,136,113,93,151,193,96,144,145,149,18,231,186,46,135,221,94,41,25,151,126,119,208,246,
  229,165,251,8,33,168,89,22,221,110,143,211,211,51,76,211,164,249,96,153,193,96,48,166,191,9,163,105,26,25,209,197,179,28,199,198,177,237,75,182,
  169,74,41,25,38,184,110,29,195,48,50,82,70,36,204,112,216,237,49,235,214,71,81,61,62,201,162,233,186,110,22,5,211,48,71,64,28,27,33,196,
  196,169,44,102,4,142,99,99,219,118,169,222,113,108,76,211,188,22,163,101,89,217,220,105,175,35,55,102,204,251,247,7,184,110,157,133,249,57,172,154,
  69,167,243,14,215,117,137,47,98,106,53,11,33,4,166,105,234,244,29,165,233,172,91,207,162,97,154,6,142,118,238,34,142,179,140,202,59,154,150,211,
  217,233,25,81,20,49,235,214,17,98,6,171,102,225,216,54,239,222,255,61,22,16,211,28,197,243,226,34,38,73,146,82,140,64,182,94,23,42,113,28,
  223,14,49,66,8,28,219,230,232,248,88,215,231,87,56,142,205,172,91,39,142,99,28,219,38,138,34,92,183,78,24,246,137,162,115,60,175,129,16,34,
  3,113,17,199,215,166,108,116,30,149,150,90,216,239,115,122,122,198,242,210,125,106,53,43,219,47,213,223,132,49,223,179,110,189,249,70,81,196,226,61,
  137,85,179,24,38,9,66,204,208,237,246,176,230,173,172,209,154,166,201,226,61,153,53,177,229,165,69,142,142,78,178,61,106,150,197,194,252,92,54,62,
  9,251,83,69,238,224,131,26,239,109,141,198,88,6,134,97,191,20,163,16,51,216,142,205,2,159,206,158,134,164,59,91,219,59,31,111,122,37,24,149,
  139,145,53,67,195,48,198,156,203,55,93,199,177,137,162,115,146,36,201,198,227,100,127,178,21,215,230,123,195,112,56,204,230,165,25,104,24,6,181,218,
  39,156,73,50,28,123,74,166,24,211,128,9,49,51,182,111,222,86,244,161,248,74,48,201,191,79,198,128,39,73,50,230,88,209,94,124,98,20,199,215,
  237,157,74,17,112,58,39,73,146,43,157,41,238,147,36,201,149,103,151,249,80,93,240,170,155,239,45,220,124,163,243,243,169,26,226,231,67,76,73,157,
  127,238,242,207,0,246,175,172,13,167,16,4,171,0,0,0,0,73,69,78,68,174,66,96,130,0,0 };

const char* TalComponent::bmp00137_png = (const char*) resource_TalComponent_bmp00137_png;
const int TalComponent::bmp00137_pngSize = 3985;
